@@ -349,7 +349,7 @@ void cdfDefVarCompression(const stream_t *streamptr, int ncvarid, bool lchunk)
 static
 void cdfDefVarPacking(const stream_t *streamptr, int ncvarid, nc_type xtype, int vlistID, int varID)
 {
-  /*  if ( xtype == NC_BYTE || xtype == NC_SHORT || xtype == NC_INT ) */
+  //  if ( xtype == NC_BYTE || xtype == NC_SHORT || xtype == NC_INT )
     {
       const double addoffset   = vlistInqVarAddoffset(vlistID, varID);
       const double scalefactor = vlistInqVarScalefactor(vlistID, varID);
@@ -373,85 +373,140 @@ void cdfDefVarPacking(const stream_t *streamptr, int ncvarid, nc_type xtype, int
 }
 
 static
-int cdfDefVar(stream_t *streamptr, int varID)
+void cdfAppendCoordinates(int fileID, int ncvarid, char coordinates[CDI_MAX_NAME])
 {
-  int ncvarid = -1;
-  int xid = CDI_UNDEFID, yid = CDI_UNDEFID;
-  size_t xsize = 0, ysize = 0;
-  int dims[4];
-  size_t chunks[4] = {0,0,0,0};
-  int ndims = 0;
-  int tablenum;
-  int dimorder[3];
-  size_t iax = 0;
-  char axis[5];
+  if (ncvarid != CDI_UNDEFID)
+    {
+      size_t len = strlen(coordinates);
+      if (len) coordinates[len++] = ' ';
+      cdf_inq_varname(fileID, ncvarid, coordinates+len);
+    }
+}
 
+static
+void cdfDefineCoordinates(const stream_t *streamptr, int ncvarid, int nczvarID, int gridtype, int gridID, int gridindex, int xid, int yid, size_t gridsize, char axis[5], size_t iax)
+{
   const int fileID  = streamptr->fileID;
 
-  if ( CDI_Debug )
-    Message("streamID = %d, fileID = %d, varID = %d", streamptr->self, fileID, varID);
+  char coordinates[CDI_MAX_NAME]; coordinates[0] = 0;
 
-  if ( streamptr->vars[varID].ncvarid != CDI_UNDEFID )
-    return streamptr->vars[varID].ncvarid;
+  if (nczvarID != CDI_UNDEFID) cdfAppendCoordinates(fileID, nczvarID, coordinates);
 
+  if ( gridtype != GRID_GENERIC && gridtype != GRID_LONLAT && gridtype != GRID_GAUSSIAN &&
+       gridtype != GRID_PROJECTION && gridtype != GRID_CURVILINEAR && gridtype != GRID_CHARXY )
+    {
+      const size_t len = strlen(gridNamePtr(gridtype));
+      if ( len > 0 ) cdf_put_att_text(fileID, ncvarid, "CDI_grid_type", len, gridNamePtr(gridtype));
+    }
+
+  char gmapvarname[CDI_MAX_NAME]; gmapvarname[0] = 0;
+  cdf_get_gmapvarname(gridID, gmapvarname);
+  if ( gmapvarname[0] ) cdf_put_att_text(fileID, ncvarid, "grid_mapping", strlen(gmapvarname), gmapvarname);
+
+  if ( gridtype == GRID_TRAJECTORY )
+    {
+      cdf_put_att_text(fileID, ncvarid, "coordinates", 9, "tlon tlat" );
+    }
+  else if ( gridtype == GRID_LONLAT && xid == CDI_UNDEFID && yid == CDI_UNDEFID && gridsize == 1 )
+    {
+      const int ncxvarID = streamptr->ncgrid[gridindex].ncIDs[CDF_VARID_X];
+      const int ncyvarID = streamptr->ncgrid[gridindex].ncIDs[CDF_VARID_Y];
+      cdfAppendCoordinates(fileID, ncyvarID, coordinates);
+      cdfAppendCoordinates(fileID, ncxvarID, coordinates);
+    }
+  else if ( gridtype == GRID_GAUSSIAN_REDUCED )
+    {
+      const int ncxvarID = streamptr->ncgrid[gridindex].ncIDs[CDF_VARID_X];
+      const int ncyvarID = streamptr->ncgrid[gridindex].ncIDs[CDF_VARID_Y];
+      cdfAppendCoordinates(fileID, ncyvarID, coordinates);
+      cdfAppendCoordinates(fileID, ncxvarID, coordinates);
+    }
+  else if ( gridtype == GRID_UNSTRUCTURED || gridtype == GRID_CURVILINEAR )
+    {
+      char cellarea[CDI_MAX_NAME] = "area: ";
+      const int ncxvarID = streamptr->ncgrid[gridindex].ncIDs[CDF_VARID_X];
+      const int ncyvarID = streamptr->ncgrid[gridindex].ncIDs[CDF_VARID_Y];
+      const int ncavarID = streamptr->ncgrid[gridindex].ncIDs[CDF_VARID_A];
+      // CMOR order: coordinates = "lat lon"
+      if ( cdiCoordinatesLonLat )
+        {
+          cdfAppendCoordinates(fileID, ncxvarID, coordinates);
+          cdfAppendCoordinates(fileID, ncyvarID, coordinates);
+        }
+      else
+        {
+          cdfAppendCoordinates(fileID, ncyvarID, coordinates);
+          cdfAppendCoordinates(fileID, ncxvarID, coordinates);
+        }
+
+      if ( ncavarID != CDI_UNDEFID )
+        {
+          size_t len = strlen(cellarea);
+          cdf_inq_varname(fileID, ncavarID, cellarea+len);
+          len = strlen(cellarea);
+          cdf_put_att_text(fileID, ncvarid, "cell_measures", len, cellarea);
+        }
+
+      if ( gridtype == GRID_UNSTRUCTURED )
+        {
+          int position = gridInqPosition(gridID);
+          if ( position > 0 )
+            cdf_put_att_int(fileID, ncvarid, "number_of_grid_in_reference", NC_INT, 1, &position);
+        }
+    }
+  else if ( gridtype == GRID_SPECTRAL || gridtype == GRID_FOURIER )
+    {
+      axis[iax++] = '-';
+      axis[iax++] = '-';
+      cdf_put_att_text(fileID, ncvarid, "axis", iax, axis);
+      int gridTruncation = gridInqTrunc(gridID);
+      cdf_put_att_int(fileID, ncvarid, "truncation", NC_INT, 1, &gridTruncation);
+    }
+  else if ( gridtype == GRID_CHARXY )
+    {
+      if ( gridInqXIsc(gridID) )
+        {
+          const int ncxvarID = streamptr->ncgrid[gridindex].ncIDs[CDF_VARID_X];
+          cdfAppendCoordinates(fileID, ncxvarID, coordinates);
+        }
+      else if ( gridInqYIsc(gridID) )
+        {
+          const int ncyvarID = streamptr->ncgrid[gridindex].ncIDs[CDF_VARID_Y];
+          cdfAppendCoordinates(fileID, ncyvarID, coordinates);
+        }
+    }
+
+  const size_t len = strlen(coordinates);
+  if ( len ) cdf_put_att_text(fileID, ncvarid, "coordinates", len, coordinates);
+}
+
+static
+int cdfDefineDimsAndChunks(const stream_t *streamptr, int varID, int xid, int yid, int zid, size_t gridsize, const int dimorder[3], int dims[4], bool lchunk, size_t chunks[4], char axis[5], size_t *piax)
+{
+  const int fileID  = streamptr->fileID;
   const int vlistID = streamptr->vlistID;
-  const int gridID = vlistInqVarGrid(vlistID, varID);
-  const int zaxisID = vlistInqVarZaxis(vlistID, varID);
+
+  size_t iax = *piax;
+  int ndims = 0;
+
+  for (int i = 0; i < 4; ++i) chunks[i] = 0;
+
+  size_t xsize = 0, ysize = 0;
+  if ( xid != CDI_UNDEFID ) cdf_inq_dimlen(fileID, xid, &xsize);
+  if ( yid != CDI_UNDEFID ) cdf_inq_dimlen(fileID, yid, &ysize);
+
   const int timetype = vlistInqVarTimetype(vlistID, varID);
-  const int param = vlistInqVarParam(vlistID, varID);
-  int code = vlistInqVarCode(vlistID, varID);
-  int pnum, pcat, pdis;
-  cdiDecodeParam(param, &pnum, &pcat, &pdis);
-
-  int chunktype = vlistInqVarChunkType(vlistID, varID);
-
-  vlistInqVarDimorder(vlistID, varID, &dimorder);
-
-  const size_t gridsize = gridInqSize(gridID);
-  bool lchunk = (gridsize >= 16);
-  const int gridtype = gridInqType(gridID);
-  const int gridindex = nc_grid_index(streamptr, gridID);
-  if ( gridtype != GRID_TRAJECTORY )
-    {
-      xid = streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_X];
-      yid = streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_Y];
-      if ( xid != CDI_UNDEFID ) cdf_inq_dimlen(fileID, xid, &xsize);
-      if ( yid != CDI_UNDEFID ) cdf_inq_dimlen(fileID, yid, &ysize);
-    }
-
-  const int zaxisindex = vlistZaxisIndex(vlistID, zaxisID);
-  const int zid = streamptr->zaxisID[zaxisindex];
-  bool zaxis_is_scalar = false;
-  if ( zid == CDI_UNDEFID ) zaxis_is_scalar = zaxisInqScalar(zaxisID) > 0;
-
-  if ( dimorder[0] != 3 ) lchunk = false; /* ZYX and ZXY */
-
-  if ( ((dimorder[0]>0)+(dimorder[1]>0)+(dimorder[2]>0)) < ((xid!=CDI_UNDEFID)+(yid!=CDI_UNDEFID)+(zid!=CDI_UNDEFID)) )
-    {
-      printf("zid=%d  yid=%d  xid=%d\n", zid, yid, xid);
-      Error("Internal problem, dimension order missing!");
-    }
-
-  const int tid = streamptr->basetime.ncdimid;
-
   if ( vlistHasTime(vlistID) && timetype != TIME_CONSTANT )
     {
-      if ( tid == CDI_UNDEFID ) Error("Internal problem, time undefined!");
-      chunks[ndims] = 1;
-      dims[ndims++] = tid;
+      const int tid = streamptr->basetime.ncdimid;
+      if (tid == CDI_UNDEFID) Error("Internal problem, time undefined!");
       axis[iax++] = 'T';
+      chunks[ndims] = 1;
+      dims[ndims] = tid;
+      ndims++;
     }
-  /*
-  if ( zid != CDI_UNDEFID ) axis[iax++] = 'Z';
-  if ( zid != CDI_UNDEFID ) chunks[ndims] = 1;
-  if ( zid != CDI_UNDEFID ) dims[ndims++] = zid;
 
-  if ( yid != CDI_UNDEFID ) chunks[ndims] = ysize;
-  if ( yid != CDI_UNDEFID ) dims[ndims++] = yid;
-
-  if ( xid != CDI_UNDEFID ) chunks[ndims] = xsize;
-  if ( xid != CDI_UNDEFID ) dims[ndims++] = xid;
-  */
+  int chunktype = vlistInqVarChunkType(vlistID, varID);
   size_t chunk_size_max = 65536;
   if ( chunktype != CDI_CHUNK_LINES && gridsize > INT32_MAX )
     {
@@ -491,20 +546,50 @@ int cdfDefVar(stream_t *streamptr, int varID)
   if ( CDI_Debug )
     fprintf(stderr, "lchunk %d chunktype %d  chunks %zu %zu %zu %zu\n", lchunk, chunktype, chunks[0], chunks[1], chunks[2], chunks[3]);
 
-  char varname[CDI_MAX_NAME];
-  char name[CDI_MAX_NAME]; name[0] = 0;
-  char longname[CDI_MAX_NAME]; longname[0] = 0;
-  char stdname[CDI_MAX_NAME]; stdname[0] = 0;
-  char units[CDI_MAX_NAME]; units[0] = 0;
-  if ( vlistInqVarNamePtr(vlistID, varID) ) vlistInqVarName(vlistID, varID, name);
-  vlistInqVarLongname(vlistID, varID, longname);
-  vlistInqVarStdname(vlistID, varID, stdname);
-  vlistInqVarUnits(vlistID, varID, units);
+  iax = *piax;
+  return ndims;
+}
 
-  const int tableID  = vlistInqVarTable(vlistID, varID);
-  if ( !name[0] ) tableInqEntry(tableID, code, -1, name, longname, units);
+static
+void cdfDefineAttrLeveltype(int fileID, int ncvarid, int zaxisID, int zaxistype)
+{
+  if ( zaxistype == ZAXIS_CLOUD_BASE          ||
+       zaxistype == ZAXIS_CLOUD_TOP           ||
+       zaxistype == ZAXIS_ISOTHERM_ZERO       ||
+       zaxistype == ZAXIS_TOA                 ||
+       zaxistype == ZAXIS_SEA_BOTTOM          ||
+       zaxistype == ZAXIS_LAKE_BOTTOM         ||
+       zaxistype == ZAXIS_SEDIMENT_BOTTOM     ||
+       zaxistype == ZAXIS_SEDIMENT_BOTTOM_TA  ||
+       zaxistype == ZAXIS_SEDIMENT_BOTTOM_TW  ||
+       zaxistype == ZAXIS_MIX_LAYER           ||
+       zaxistype == ZAXIS_ATMOSPHERE )
+    {
+      char varname[CDI_MAX_NAME];
+      zaxisInqName(zaxisID, varname);
+      cdf_put_att_text(fileID, ncvarid, "level_type", strlen(varname), varname);
+    }
+}
+
+static
+void cdfDefineAttrEnsemble(int fileID, int ncvarid, int vlistID, int varID)
+{
+  int perturbationNumber, numberOfForecastsInEnsemble, typeOfEnsembleForecast;
+  if ( cdiInqKeyInt(vlistID, varID, CDI_KEY_PERTURBATIONNUMBER, &perturbationNumber) == 0 )
+    cdf_put_att_int(fileID, ncvarid, "realization", NC_INT, 1, &perturbationNumber);
+  if ( cdiInqKeyInt(vlistID, varID, CDI_KEY_NUMBEROFFORECASTSINENSEMBLE, &numberOfForecastsInEnsemble) == 0 )
+    cdf_put_att_int(fileID, ncvarid, "ensemble_members", NC_INT, 1, &numberOfForecastsInEnsemble);
+  if ( cdiInqKeyInt(vlistID, varID, CDI_KEY_TYPEOFENSEMBLEFORECAST, &typeOfEnsembleForecast) == 0 )
+    cdf_put_att_int(fileID, ncvarid, "forecast_init_type", NC_INT, 1, &typeOfEnsembleForecast);
+}
+
+static
+void cdfCheckVarname(int fileID, char name[CDI_MAX_NAME])
+{
   if ( name[0] )
     {
+      int ncvarid;
+      char varname[CDI_MAX_NAME];
       sprintf(varname, "%s", name);
       char *varname2 = varname+strlen(varname);
       unsigned iz = 0;
@@ -526,41 +611,105 @@ int cdfDefVar(stream_t *streamptr, int varID)
 
       strcpy(name, varname);
     }
+}
+
+static
+void cdfGenVarname(int fileID, char name[CDI_MAX_NAME], int pnum, int pcat, int *pdis, int *pcode)
+{
+  char varname[CDI_MAX_NAME];
+
+  int code = *pcode;
+  if ( code < 0 ) code = -code;
+  if ( pnum < 0 ) pnum = -pnum;
+
+  if ( *pdis == 255 )
+    sprintf(varname, "var%d", code);
   else
+    sprintf(varname, "param%d.%d.%d", pnum, pcat, *pdis);
+
+  char *varname2 = varname+strlen(varname);
+  int ncvarid;
+  unsigned iz = 0;
+
+  do
     {
-      if ( code < 0 ) code = -code;
-      if ( pnum < 0 ) pnum = -pnum;
+      if ( iz ) sprintf(varname2, "_%u", iz+1);
 
-      if ( pdis == 255 )
-	sprintf(varname, "var%d", code);
-      else
-	sprintf(varname, "param%d.%d.%d", pnum, pcat, pdis);
+      if ( nc_inq_varid(fileID, varname, &ncvarid) != NC_NOERR ) break;
 
-      char *varname2 = varname+strlen(varname);
-      unsigned iz = 0;
+      ++iz;
+    }
+  while ( iz <= 99 );
 
-      do
-        {
-          if ( iz ) sprintf(varname2, "_%u", iz+1);
+  if (iz > 99) Error("Variable name %s already exsist!", name);
 
-          if ( nc_inq_varid(fileID, varname, &ncvarid) != NC_NOERR ) break;
+  strcpy(name, varname);
+  *pcode = 0;
+  *pdis = 255;
+}
 
-          ++iz;
-        }
-      while ( iz <= 99 );
+static
+int cdfDefVar(stream_t *streamptr, int varID)
+{
+  if (streamptr->vars[varID].ncvarid != CDI_UNDEFID)
+    return streamptr->vars[varID].ncvarid;
 
-      if (iz > 99) Error("Variable name %s already exsist!", name);
+  const int fileID  = streamptr->fileID;
+  if (CDI_Debug) Message("streamID = %d, fileID = %d, varID = %d", streamptr->self, fileID, varID);
 
-      strcpy(name, varname);
-      code = 0;
-      pdis = 255;
+  const int vlistID = streamptr->vlistID;
+  const int param = vlistInqVarParam(vlistID, varID);
+  int code = vlistInqVarCode(vlistID, varID);
+  int pnum, pcat, pdis;
+  cdiDecodeParam(param, &pnum, &pcat, &pdis);
+
+  const int gridID = vlistInqVarGrid(vlistID, varID);
+  const size_t gridsize = gridInqSize(gridID);
+  const int gridtype = gridInqType(gridID);
+  const int gridindex = nc_grid_index(streamptr, gridID);
+  const int xid = (gridtype != GRID_TRAJECTORY) ? streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_X] : CDI_UNDEFID;
+  const int yid = (gridtype != GRID_TRAJECTORY) ? streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_Y] : CDI_UNDEFID;
+
+  const int zaxisID = vlistInqVarZaxis(vlistID, varID);
+  const int zaxistype = zaxisInqType(zaxisID);
+  const int zaxisindex = vlistZaxisIndex(vlistID, zaxisID);
+  const int zid = streamptr->zaxisID[zaxisindex];
+
+  int dimorder[3]; // ZYX and ZXY
+  vlistInqVarDimorder(vlistID, varID, &dimorder);
+  const bool lchunk = (dimorder[0] == 3) ? (gridsize >= 16) : false;
+
+  if ( ((dimorder[0]>0)+(dimorder[1]>0)+(dimorder[2]>0)) < ((xid!=CDI_UNDEFID)+(yid!=CDI_UNDEFID)+(zid!=CDI_UNDEFID)) )
+    {
+      printf("zid=%d  yid=%d  xid=%d\n", zid, yid, xid);
+      Error("Internal problem, dimension order missing!");
     }
 
-  /* if ( streamptr->ncmode == 2 ) cdf_redef(fileID); */
+  size_t iax = 0;
+  char axis[5];
+  int dims[4];
+  size_t chunks[4];
+  int ndims = cdfDefineDimsAndChunks(streamptr, varID, xid, yid, zid, gridsize, dimorder, dims, lchunk, chunks, axis, &iax);
+
+  char name[CDI_MAX_NAME]; name[0] = 0;
+  if ( vlistInqVarNamePtr(vlistID, varID) ) vlistInqVarName(vlistID, varID, name);
+
+  char longname[CDI_MAX_NAME]; longname[0] = 0;
+  char stdname[CDI_MAX_NAME]; stdname[0] = 0;
+  char units[CDI_MAX_NAME]; units[0] = 0;
+  vlistInqVarLongname(vlistID, varID, longname);
+  vlistInqVarStdname(vlistID, varID, stdname);
+  vlistInqVarUnits(vlistID, varID, units);
+
+  const int tableID  = vlistInqVarTable(vlistID, varID);
+  if (!name[0]) tableInqEntry(tableID, code, -1, name, longname, units);
+  if (name[0]) cdfCheckVarname(fileID, name);
+  else         cdfGenVarname(fileID, name, pnum, pcat, &pdis, &code);
 
   const int dtype = vlistInqVarDatatype(vlistID, varID);
   const nc_type xtype = cdfDefDatatype(dtype, streamptr);
 
+  int ncvarid = -1;
   cdf_def_var(fileID, name, xtype, ndims, dims, &ncvarid);
 
 #if  defined  (HAVE_NETCDF4)
@@ -586,139 +735,14 @@ int cdfDefVar(stream_t *streamptr, int varID)
 
   if ( tableID != CDI_UNDEFID )
     {
-      tablenum = tableInqNum(tableID);
-      if ( tablenum > 0 )
-        cdf_put_att_int(fileID, ncvarid, "table", NC_INT, 1, &tablenum);
+      int tablenum = tableInqNum(tableID);
+      if (tablenum > 0) cdf_put_att_int(fileID, ncvarid, "table", NC_INT, 1, &tablenum);
     }
 
-  char coordinates[CDI_MAX_NAME]; coordinates[0] = 0;
+  const bool zaxisIsScalar = (zid == CDI_UNDEFID) ? (zaxisInqScalar(zaxisID) > 0) : false;
+  const int nczvarID = (zaxisIsScalar || zaxistype == ZAXIS_CHAR) ? streamptr->nczvarID[zaxisindex] : CDI_UNDEFID;
 
-  if ( zaxis_is_scalar || zaxisInqType(zaxisID) == ZAXIS_CHAR )
-    {
-      int nczvarID = streamptr->nczvarID[zaxisindex];
-      if ( nczvarID != CDI_UNDEFID )
-        {
-          size_t len = strlen(coordinates);
-          if ( len ) coordinates[len++] = ' ';
-          cdf_inq_varname(fileID, nczvarID, coordinates+len);
-        }
-    }
-
-  if ( gridtype != GRID_GENERIC && gridtype != GRID_LONLAT && gridtype != GRID_GAUSSIAN &&
-       gridtype != GRID_PROJECTION && gridtype != GRID_CURVILINEAR && gridtype != GRID_CHARXY )
-    {
-      const size_t len = strlen(gridNamePtr(gridtype));
-      if ( len > 0 ) cdf_put_att_text(fileID, ncvarid, "CDI_grid_type", len, gridNamePtr(gridtype));
-    }
-
-  char gmapvarname[CDI_MAX_NAME]; gmapvarname[0] = 0;
-
-  cdf_get_gmapvarname(gridID, gmapvarname);
-
-  if ( gmapvarname[0] ) cdf_put_att_text(fileID, ncvarid, "grid_mapping", strlen(gmapvarname), gmapvarname);
-
-  if ( gridtype == GRID_TRAJECTORY )
-    {
-      cdf_put_att_text(fileID, ncvarid, "coordinates", 9, "tlon tlat" );
-    }
-  else if ( gridtype == GRID_LONLAT && xid == CDI_UNDEFID && yid == CDI_UNDEFID && gridsize == 1 )
-    {
-      const int ncxvarID = streamptr->ncgrid[gridindex].ncIDs[CDF_VARID_X];
-      const int ncyvarID = streamptr->ncgrid[gridindex].ncIDs[CDF_VARID_Y];
-      if ( ncyvarID != CDI_UNDEFID )
-        {
-          size_t len = strlen(coordinates);
-          if ( len ) coordinates[len++] = ' ';
-          cdf_inq_varname(fileID, ncyvarID, coordinates+len);
-        }
-      if ( ncxvarID != CDI_UNDEFID )
-        {
-          size_t len = strlen(coordinates);
-          if ( len ) coordinates[len++] = ' ';
-          cdf_inq_varname(fileID, ncxvarID, coordinates+len);
-        }
-    }
-  else if ( gridtype == GRID_UNSTRUCTURED || gridtype == GRID_CURVILINEAR )
-    {
-      char cellarea[CDI_MAX_NAME] = "area: ";
-      const int ncxvarID = streamptr->ncgrid[gridindex].ncIDs[CDF_VARID_X];
-      const int ncyvarID = streamptr->ncgrid[gridindex].ncIDs[CDF_VARID_Y];
-      const int ncavarID = streamptr->ncgrid[gridindex].ncIDs[CDF_VARID_A];
-      // CMOR order: coordinates = "lat lon"
-      if ( cdiCoordinatesLonLat )
-        {
-          if ( ncxvarID != CDI_UNDEFID )
-            {
-              size_t len = strlen(coordinates);
-              if ( len ) coordinates[len++] = ' ';
-              cdf_inq_varname(fileID, ncxvarID, coordinates+len);
-            }
-          if ( ncyvarID != CDI_UNDEFID )
-            {
-              size_t len = strlen(coordinates);
-              if ( len ) coordinates[len++] = ' ';
-              cdf_inq_varname(fileID, ncyvarID, coordinates+len);
-            }
-        }
-      else
-        {
-          if ( ncyvarID != CDI_UNDEFID )
-            {
-              size_t len = strlen(coordinates);
-              if ( len ) coordinates[len++] = ' ';
-              cdf_inq_varname(fileID, ncyvarID, coordinates+len);
-            }
-          if ( ncxvarID != CDI_UNDEFID )
-            {
-              size_t len = strlen(coordinates);
-              if ( len ) coordinates[len++] = ' ';
-              cdf_inq_varname(fileID, ncxvarID, coordinates+len);
-            }
-        }
-
-      if ( ncavarID != CDI_UNDEFID )
-        {
-          size_t len = strlen(cellarea);
-          cdf_inq_varname(fileID, ncavarID, cellarea+len);
-          len = strlen(cellarea);
-          cdf_put_att_text(fileID, ncvarid, "cell_measures", len, cellarea);
-        }
-
-      if ( gridtype == GRID_UNSTRUCTURED )
-        {
-          int position = gridInqPosition(gridID);
-          if ( position > 0 )
-            cdf_put_att_int(fileID, ncvarid, "number_of_grid_in_reference", NC_INT, 1, &position);
-        }
-    }
-  else if ( gridtype == GRID_SPECTRAL || gridtype == GRID_FOURIER )
-    {
-      const int gridTruncation = gridInqTrunc(gridID);
-      axis[iax++] = '-';
-      axis[iax++] = '-';
-      cdf_put_att_text(fileID, ncvarid, "axis", iax, axis);
-      cdf_put_att_int(fileID, ncvarid, "truncation", NC_INT, 1, &gridTruncation);
-    }
-  else if ( gridtype == GRID_CHARXY )
-    {
-      if ( gridInqXIsc(gridID) )
-        {
-          const int ncxvarID = streamptr->ncgrid[gridindex].ncIDs[CDF_VARID_X];
-          size_t len = strlen(coordinates);
-          if ( len ) coordinates[len++] = ' ';
-          cdf_inq_varname(fileID, ncxvarID, coordinates+len);
-        }
-      else if ( gridInqYIsc(gridID) )
-        {
-          const int ncyvarID = streamptr->ncgrid[gridindex].ncIDs[CDF_VARID_Y];
-          size_t len = strlen(coordinates);
-          if ( len ) coordinates[len++] = ' ';
-          cdf_inq_varname(fileID, ncyvarID, coordinates+len);
-        }
-    }
-
-  size_t len = strlen(coordinates);
-  if ( len ) cdf_put_att_text(fileID, ncvarid, "coordinates", len, coordinates);
+  cdfDefineCoordinates(streamptr, ncvarid, nczvarID, gridtype, gridID, gridindex, xid, yid, gridsize, axis, iax);
 
   cdfDefVarPacking(streamptr, ncvarid, xtype, vlistID, varID);
 
@@ -734,38 +758,13 @@ int cdfDefVar(stream_t *streamptr, int varID)
   if ( vlistInqVarMissvalUsed(vlistID, varID) )
     cdfDefVarMissval(streamptr, varID, vlistInqVarDatatype(vlistID, varID), 0);
 
-  if ( zid == -1 )
-    {
-      if ( zaxisInqType(zaxisID) == ZAXIS_CLOUD_BASE          ||
-           zaxisInqType(zaxisID) == ZAXIS_CLOUD_TOP           ||
-           zaxisInqType(zaxisID) == ZAXIS_ISOTHERM_ZERO       ||
-           zaxisInqType(zaxisID) == ZAXIS_TOA                 ||
-           zaxisInqType(zaxisID) == ZAXIS_SEA_BOTTOM          ||
-           zaxisInqType(zaxisID) == ZAXIS_LAKE_BOTTOM         ||
-           zaxisInqType(zaxisID) == ZAXIS_SEDIMENT_BOTTOM     ||
-           zaxisInqType(zaxisID) == ZAXIS_SEDIMENT_BOTTOM_TA  ||
-           zaxisInqType(zaxisID) == ZAXIS_SEDIMENT_BOTTOM_TW  ||
-           zaxisInqType(zaxisID) == ZAXIS_MIX_LAYER           ||
-           zaxisInqType(zaxisID) == ZAXIS_ATMOSPHERE )
-        {
-          zaxisInqName(zaxisID, varname);
-          cdf_put_att_text(fileID, ncvarid, "level_type", strlen(varname), varname);
-        }
-    }
+  if (zid == CDI_UNDEFID) cdfDefineAttrLeveltype(fileID, ncvarid, zaxisID, zaxistype);
 
-  int perturbationNumber, numberOfForecastsInEnsemble, typeOfEnsembleForecast;
-  if ( cdiInqKeyInt(vlistID, varID, CDI_KEY_PERTURBATIONNUMBER, &perturbationNumber) == 0 )
-    cdf_put_att_int(fileID, ncvarid, "realization", NC_INT, 1, &perturbationNumber);
-  if ( cdiInqKeyInt(vlistID, varID, CDI_KEY_NUMBEROFFORECASTSINENSEMBLE, &numberOfForecastsInEnsemble) == 0 )
-    cdf_put_att_int(fileID, ncvarid, "ensemble_members", NC_INT, 1, &numberOfForecastsInEnsemble);
-  if ( cdiInqKeyInt(vlistID, varID, CDI_KEY_TYPEOFENSEMBLEFORECAST, &typeOfEnsembleForecast) == 0 )
-    cdf_put_att_int(fileID, ncvarid, "forecast_init_type", NC_INT, 1, &typeOfEnsembleForecast);
+  cdfDefineAttrEnsemble(fileID, ncvarid, vlistID, varID);
 
   // Attributes
   cdfDefineAttributes(vlistID, varID, fileID, ncvarid);
 
-  /* if ( streamptr->ncmode == 2 ) cdf_enddef(fileID); */
-
   return ncvarid;
 }
 
@@ -802,11 +801,11 @@ void cdfWriteGridTraj(stream_t *streamptr, int gridID)
   const int gridindex = nc_grid_index(streamptr, gridID);
   const int lonID = streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_X];
   const int latID = streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_Y];
+  const int tsID = streamptr->curTsID;
+  size_t index = (size_t)tsID;
 
   double xlon = gridInqXval(gridID, 0);
   double xlat = gridInqYval(gridID, 0);
-  int tsID = streamptr->curTsID;
-  size_t index = (size_t)tsID;
 
   cdf_put_var1_double(streamptr->fileID, lonID, &index, &xlon);
   cdf_put_var1_double(streamptr->fileID, latID, &index, &xlat);
@@ -989,19 +988,34 @@ void cdf_write_var_data(int fileID, int vlistID, int varID, int ncvarid, int dty
   if ( sdata_sp ) Free(sdata_sp);
 }
 
-
-void cdf_write_var(stream_t *streamptr, int varID, int memtype, const void *data, size_t nmiss)
+static
+void cdfGetXYZid(stream_t *streamptr, int gridID, int zaxisID, int *xid, int *yid, int *zid)
 {
-  if ( streamptr->accessmode == 0 ) cdfEndDef(streamptr);
+  *xid = CDI_UNDEFID;
+  *yid = CDI_UNDEFID;
 
-  size_t xsize = 0, ysize = 0;
-  size_t size;
-  size_t start[5];
-  size_t count[5];
-  bool swapxy = false;
+  if (gridInqType(gridID) == GRID_TRAJECTORY)
+    {
+      cdfWriteGridTraj(streamptr, gridID);
+    }
+  else
+    {
+      const int gridindex = nc_grid_index(streamptr, gridID);
+      *xid = streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_X];
+      *yid = streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_Y];
+    }
+
+  const int vlistID = streamptr->vlistID;
+  const int zaxisindex = vlistZaxisIndex(vlistID, zaxisID);
+  *zid = streamptr->zaxisID[zaxisindex];
+}
+
+static
+void cdfDefineStartAndCount(stream_t *streamptr, int varID, int xid, int yid, int zid, size_t start[5], size_t count[5], size_t *xsize, size_t *ysize)
+{
   size_t ndims = 0;
-
-  if ( CDI_Debug ) Message("streamID = %d  varID = %d", streamptr->self, varID);
+  *xsize = 0;
+  *ysize = 0;
 
   const int vlistID = streamptr->vlistID;
   const int fileID  = streamptr->fileID;
@@ -1009,27 +1023,8 @@ void cdf_write_var(stream_t *streamptr, int varID, int memtype, const void *data
   const long ntsteps = streamptr->ntsteps;
   if ( CDI_Debug ) Message("ntsteps = %ld", ntsteps);
 
-  const int ncvarid = cdfDefVar(streamptr, varID);
-
-  const int gridID   = vlistInqVarGrid(vlistID, varID);
-  const int zaxisID  = vlistInqVarZaxis(vlistID, varID);
   const int timetype = vlistInqVarTimetype(vlistID, varID);
 
-  int xid = CDI_UNDEFID, yid = CDI_UNDEFID;
-  if ( gridInqType(gridID) == GRID_TRAJECTORY )
-    {
-      cdfWriteGridTraj(streamptr, gridID);
-    }
-  else
-    {
-      const int gridindex = nc_grid_index(streamptr, gridID);
-      xid = streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_X];
-      yid = streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_Y];
-    }
-
-  const int zaxisindex = vlistZaxisIndex(vlistID, zaxisID);
-  const int zid = streamptr->zaxisID[zaxisindex];
-
   if ( vlistHasTime(vlistID) && timetype != TIME_CONSTANT )
     {
       start[ndims] = (size_t)ntsteps - 1;
@@ -1039,6 +1034,7 @@ void cdf_write_var(stream_t *streamptr, int varID, int memtype, const void *data
 
   if ( zid != CDI_UNDEFID )
     {
+      const int zaxisID = vlistInqVarZaxis(vlistID, varID);
       start[ndims] = 0;
       count[ndims] = (size_t)zaxisInqSize(zaxisID);
       ndims++;
@@ -1047,6 +1043,7 @@ void cdf_write_var(stream_t *streamptr, int varID, int memtype, const void *data
   if ( yid != CDI_UNDEFID )
     {
       start[ndims] = 0;
+      size_t size;
       cdf_inq_dimlen(fileID, yid, &size);
       /*      count[ndims] = gridInqYsize(gridID); */
       count[ndims] = size;
@@ -1056,6 +1053,7 @@ void cdf_write_var(stream_t *streamptr, int varID, int memtype, const void *data
   if ( xid != CDI_UNDEFID )
     {
       start[ndims] = 0;
+      size_t size;
       cdf_inq_dimlen(fileID, xid, &size);
       /*      count[ndims] = gridInqXsize(gridID); */
       count[ndims] = size;
@@ -1065,6 +1063,28 @@ void cdf_write_var(stream_t *streamptr, int varID, int memtype, const void *data
   if ( CDI_Debug )
     for (size_t idim = 0; idim < ndims; idim++)
       Message("dim = %d  start = %d  count = %d", idim, start[idim], count[idim]);
+}
+
+void cdf_write_var(stream_t *streamptr, int varID, int memtype, const void *data, size_t nmiss)
+{
+  if ( streamptr->accessmode == 0 ) cdfEndDef(streamptr);
+
+  if ( CDI_Debug ) Message("streamID = %d  varID = %d", streamptr->self, varID);
+
+  const int vlistID = streamptr->vlistID;
+  const int fileID  = streamptr->fileID;
+
+  const int ncvarid = cdfDefVar(streamptr, varID);
+
+  const int gridID   = vlistInqVarGrid(vlistID, varID);
+  const int zaxisID  = vlistInqVarZaxis(vlistID, varID);
+
+  int xid, yid, zid;
+  cdfGetXYZid(streamptr, gridID, zaxisID, &xid, &yid, &zid);
+
+  size_t xsize, ysize;
+  size_t start[5], count[5];
+  cdfDefineStartAndCount(streamptr, varID, xid, yid, zid, start, count, &xsize, &ysize);
 
   if ( streamptr->ncmode == 1 )
     {
@@ -1078,83 +1098,57 @@ void cdf_write_var(stream_t *streamptr, int varID, int memtype, const void *data
 
   const size_t nvals = gridInqSize(gridID) * (size_t)(zaxisInqSize(zaxisID));
 
+  bool swapxy = false;
   cdf_write_var_data(fileID, vlistID, varID, ncvarid, dtype, nvals, xsize, ysize, swapxy, start, count, memtype, data, nmiss);
 }
 
-
-void cdf_write_var_chunk(stream_t *streamptr, int varID, int memtype,
-                         const int rect[][2], const void *data, size_t nmiss)
+static
+void cdfDefineStartAndCountChunk(stream_t *streamptr, const int rect[][2], int varID, int xid, int yid, int zid, size_t start[5], size_t count[5], size_t *xsize, size_t *ysize)
 {
-  if ( streamptr->accessmode == 0 ) cdfEndDef(streamptr);
-
-  int xid = CDI_UNDEFID, yid = CDI_UNDEFID;
-  size_t xsize = 0, ysize = 0;
-  size_t start[5];
-  size_t count[5];
-  bool swapxy = false;
   size_t ndims = 0;
-  const int streamID = streamptr->self;
+  *xsize = 0;
+  *ysize = 0;
 
-  if ( CDI_Debug )
-    Message("streamID = %d  varID = %d", streamID, varID);
-
-  const int vlistID = streamInqVlist(streamID);
-  const int fileID  = streamInqFileID(streamID);
+  const int vlistID = streamptr->vlistID;
+  const int fileID  = streamptr->fileID;
 
   const long ntsteps = streamptr->ntsteps;
   if ( CDI_Debug ) Message("ntsteps = %ld", ntsteps);
 
-  const int ncvarid = cdfDefVar(streamptr, varID);
-
-  const int gridID   = vlistInqVarGrid(vlistID, varID);
-  const int zaxisID  = vlistInqVarZaxis(vlistID, varID);
   const int timetype = vlistInqVarTimetype(vlistID, varID);
 
-  if ( gridInqType(gridID) == GRID_TRAJECTORY )
-    {
-      cdfWriteGridTraj(streamptr, gridID);
-    }
-  else
-    {
-      const int gridindex = nc_grid_index(streamptr, gridID);
-      xid = streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_X];
-      yid = streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_Y];
-    }
-
-  const int zaxisindex = vlistZaxisIndex(vlistID, zaxisID);
-  const int zid = streamptr->zaxisID[zaxisindex];
-
   if ( vlistHasTime(vlistID) && timetype != TIME_CONSTANT )
     {
       start[ndims] = (size_t)ntsteps - 1;
       count[ndims] = 1;
       ndims++;
     }
+
   if ( zid != CDI_UNDEFID )
     {
+      const int zaxisID  = vlistInqVarZaxis(vlistID, varID);
       int size = zaxisInqSize(zaxisID);
-      xassert(rect[2][0] >= 0 && rect[2][0] <= rect[2][1]
-              && rect[2][1] <= size);
+      xassert(rect[2][0] >= 0 && rect[2][0] <= rect[2][1] && rect[2][1] <= size);
       start[ndims] = (size_t)rect[2][0];
       count[ndims] = (size_t)rect[2][1] - (size_t)rect[2][0] + 1;
       ndims++;
     }
+
   if ( yid != CDI_UNDEFID )
     {
       size_t size;
       cdf_inq_dimlen(fileID, yid, &size);
-      xassert(rect[1][0] >= 0 && rect[1][0] <= rect[1][1]
-              && (size_t)rect[1][1] <= size);
+      xassert(rect[1][0] >= 0 && rect[1][0] <= rect[1][1] && (size_t)rect[1][1] <= size);
       start[ndims] = (size_t)rect[1][0];
       count[ndims] = (size_t)rect[1][1] - (size_t)rect[1][0] + 1;
       ndims++;
     }
+
   if ( xid != CDI_UNDEFID )
     {
       size_t size;
       cdf_inq_dimlen(fileID, xid, &size);
-      xassert(rect[0][0] >= 0 && rect[0][0] <= rect[0][1]
-              && (size_t)rect[0][1] <= size);
+      xassert(rect[0][0] >= 0 && rect[0][0] <= rect[0][1] && (size_t)rect[0][1] <= size);
       start[ndims] = (size_t)rect[0][0];
       count[ndims] = (size_t)rect[0][1] - (size_t)rect[0][0] + 1;
       ndims++;
@@ -1163,6 +1157,32 @@ void cdf_write_var_chunk(stream_t *streamptr, int varID, int memtype,
   if ( CDI_Debug )
     for (size_t idim = 0; idim < ndims; idim++)
       Message("dim = %d  start = %d  count = %d", idim, start[idim], count[idim]);
+}
+
+void cdf_write_var_chunk(stream_t *streamptr, int varID, int memtype,
+                         const int rect[][2], const void *data, size_t nmiss)
+{
+  if ( streamptr->accessmode == 0 ) cdfEndDef(streamptr);
+
+  const int streamID = streamptr->self;
+
+  if ( CDI_Debug )
+    Message("streamID = %d  varID = %d", streamID, varID);
+
+  const int vlistID = streamInqVlist(streamID);
+  const int fileID  = streamInqFileID(streamID);
+
+  const int ncvarid = cdfDefVar(streamptr, varID);
+
+  const int gridID   = vlistInqVarGrid(vlistID, varID);
+  const int zaxisID  = vlistInqVarZaxis(vlistID, varID);
+
+  int xid, yid, zid;
+  cdfGetXYZid(streamptr, gridID, zaxisID, &xid, &yid, &zid);
+
+  size_t xsize, ysize;
+  size_t start[5], count[5];
+  cdfDefineStartAndCountChunk(streamptr, rect, varID, xid, yid, zid, start, count, &xsize, &ysize);
 
   if ( streamptr->ncmode == 1 )
     {
@@ -1176,22 +1196,17 @@ void cdf_write_var_chunk(stream_t *streamptr, int varID, int memtype,
 
   const size_t nvals = gridInqSize(gridID) * (size_t)(zaxisInqSize(zaxisID));
 
+  bool swapxy = false;
   cdf_write_var_data(fileID, vlistID, varID, ncvarid, dtype, nvals,
                      xsize, ysize, swapxy, start, count, memtype, data, nmiss);
 }
 
-
-void cdf_write_var_slice(stream_t *streamptr, int varID, int levelID, int memtype, const void *data, size_t nmiss)
+static
+void cdfDefineStartAndCountSlice(stream_t *streamptr, int varID, int levelID, int dimorder[3], int xid, int yid, int zid, size_t start[5], size_t count[5], size_t *xsize, size_t *ysize)
 {
-  if ( streamptr->accessmode == 0 ) cdfEndDef(streamptr);
-
-  size_t xsize = 0, ysize = 0;
-  size_t start[5];
-  size_t count[5];
-  int dimorder[3];
-  int xid = CDI_UNDEFID, yid = CDI_UNDEFID;
-
-  if ( CDI_Debug ) Message("streamID = %d  varID = %d", streamptr->self, varID);
+  size_t ndims = 0;
+  *xsize = 0;
+  *ysize = 0;
 
   const int vlistID = streamptr->vlistID;
   const int fileID  = streamptr->fileID;
@@ -1199,30 +1214,8 @@ void cdf_write_var_slice(stream_t *streamptr, int varID, int levelID, int memtyp
   const long ntsteps = streamptr->ntsteps;
   if ( CDI_Debug ) Message("ntsteps = %ld", ntsteps);
 
-  const int ncvarid = cdfDefVar(streamptr, varID);
-
-  const int gridID   = vlistInqVarGrid(vlistID, varID);
-  const int zaxisID  = vlistInqVarZaxis(vlistID, varID);
   const int timetype = vlistInqVarTimetype(vlistID, varID);
-  vlistInqVarDimorder(vlistID, varID, &dimorder);
 
-  if ( gridInqType(gridID) == GRID_TRAJECTORY )
-    {
-      cdfWriteGridTraj(streamptr, gridID);
-    }
-  else
-    {
-      const int gridindex = nc_grid_index(streamptr, gridID);
-      xid = streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_X];
-      yid = streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_Y];
-    }
-
-  const int zaxisindex = vlistZaxisIndex(vlistID, zaxisID);
-  const int zid = streamptr->zaxisID[zaxisindex];
-
-  const bool swapxy = (dimorder[2] == 2 || dimorder[0] == 1) && xid != CDI_UNDEFID && yid != CDI_UNDEFID;
-
-  size_t ndims = 0;
   if ( vlistHasTime(vlistID) && timetype != TIME_CONSTANT )
     {
       start[ndims] = (size_t)ntsteps - 1;
@@ -1241,15 +1234,15 @@ void cdf_write_var_slice(stream_t *streamptr, int varID, int levelID, int memtyp
       else if ( dimorder[id] == 2 && yid != CDI_UNDEFID )
         {
           start[ndims] = 0;
-          cdf_inq_dimlen(fileID, yid, &ysize);
-          count[ndims] = ysize;
+          cdf_inq_dimlen(fileID, yid, ysize);
+          count[ndims] = *ysize;
           ndims++;
         }
       else if ( dimorder[id] == 1 && xid != CDI_UNDEFID )
         {
           start[ndims] = 0;
-          cdf_inq_dimlen(fileID, xid, &xsize);
-          count[ndims] = xsize;
+          cdf_inq_dimlen(fileID, xid, xsize);
+          count[ndims] = *xsize;
           ndims++;
         }
     }
@@ -1257,6 +1250,32 @@ void cdf_write_var_slice(stream_t *streamptr, int varID, int levelID, int memtyp
   if ( CDI_Debug )
     for (size_t idim = 0; idim < ndims; idim++)
       Message("dim = %d  start = %d  count = %d", idim, start[idim], count[idim]);
+}
+
+void cdf_write_var_slice(stream_t *streamptr, int varID, int levelID, int memtype, const void *data, size_t nmiss)
+{
+  if ( streamptr->accessmode == 0 ) cdfEndDef(streamptr);
+
+  if ( CDI_Debug ) Message("streamID = %d  varID = %d", streamptr->self, varID);
+
+  const int vlistID = streamptr->vlistID;
+  const int fileID  = streamptr->fileID;
+
+  const int ncvarid = cdfDefVar(streamptr, varID);
+
+  const int gridID   = vlistInqVarGrid(vlistID, varID);
+  const int zaxisID  = vlistInqVarZaxis(vlistID, varID);
+
+  int xid, yid, zid;
+  cdfGetXYZid(streamptr, gridID, zaxisID, &xid, &yid, &zid);
+
+  int dimorder[3];
+  vlistInqVarDimorder(vlistID, varID, &dimorder);
+  const bool swapxy = (dimorder[2] == 2 || dimorder[0] == 1) && xid != CDI_UNDEFID && yid != CDI_UNDEFID;
+
+  size_t xsize, ysize;
+  size_t start[5], count[5];
+  cdfDefineStartAndCountSlice(streamptr, varID, levelID, dimorder, xid, yid, zid, start, count, &xsize, &ysize);
 
   const int dtype = vlistInqVarDatatype(vlistID, varID);
 
@@ -1272,7 +1291,6 @@ void cdf_write_record(stream_t *streamptr, int memtype, const void *data, size_t
 {
   const int varID   = streamptr->record->varID;
   const int levelID = streamptr->record->levelID;
-
   cdf_write_var_slice(streamptr, varID, levelID, memtype, data, nmiss);
 }
 
