@@ -15,8 +15,7 @@
 extern "C" {
 #endif
 
-
-#define  CDI_MAX_NAME             256   // max length of a name
+#define  CDI_MAX_NAME             256   // Max length of a name
 
 #define  CDI_UNDEFID               -1
 #define  CDI_GLOBAL                -1   // Global var ID for vlist
@@ -200,35 +199,36 @@ typedef struct  {
 
 
 
-/* TIME types */
+// TIME types
 
-#define  TIME_CONSTANT              0  /* Time constant              */
-#define  TIME_VARYING               1  /* Time varying               */
-#define  TIME_VARIABLE              1  /* obsolate, use TIME_VARYING */
+#define  TIME_CONSTANT            0  // Time constant
+#define  TIME_VARYING             1  // Time varying
+#define  TIME_VARIABLE            1  // obsolate, use TIME_VARYING
 
-/* TSTEP types */
+// TSTEP types
 
-#define  TSTEP_INSTANT              1  /* Instant             */
-#define  TSTEP_AVG                  2  /* Average             */
-#define  TSTEP_ACCUM                3  /* Accumulation        */
-#define  TSTEP_MAX                  4  /* Maximum             */
-#define  TSTEP_MIN                  5  /* Minimum             */
-#define  TSTEP_DIFF                 6  /* Difference          */
-#define  TSTEP_RMS                  7  /* Root mean square    */
-#define  TSTEP_SD                   8  /* Standard deviation  */
-#define  TSTEP_COV                  9  /* Covariance          */
-#define  TSTEP_RATIO               10  /* Ratio               */
-#define  TSTEP_RANGE               11
-#define  TSTEP_INSTANT2            12
-#define  TSTEP_INSTANT3            13
+#define  TSTEP_INSTANT            1  // Instant
+#define  TSTEP_AVG                2  // Average
+#define  TSTEP_ACCUM              3  // Accumulation
+#define  TSTEP_MAX                4  // Maximum
+#define  TSTEP_MIN                5  // Minimum
+#define  TSTEP_DIFF               6  // Difference
+#define  TSTEP_RMS                7  // Root mean square
+#define  TSTEP_SD                 8  // Standard deviation
+#define  TSTEP_COV                9  // Covariance
+#define  TSTEP_RATIO             10  // Ratio
+#define  TSTEP_SUM               11  // Summation
+#define  TSTEP_RANGE             12
+#define  TSTEP_INSTANT2          13
+#define  TSTEP_INSTANT3          14
 
-/* TAXIS types */
+// TAXIS types
 
 #define  TAXIS_ABSOLUTE           1
 #define  TAXIS_RELATIVE           2
 #define  TAXIS_FORECAST           3
 
-/* TUNIT types */
+// TUNIT types
 
 #define  TUNIT_SECOND             1
 #define  TUNIT_MINUTE             2
@@ -242,9 +242,9 @@ typedef struct  {
 #define  TUNIT_MONTH             10
 #define  TUNIT_YEAR              11
 
-/* CALENDAR types */
+// CALENDAR types
 
-#define  CALENDAR_STANDARD        0  /* don't change this value (used also in cgribexlib)! */
+#define  CALENDAR_STANDARD        0  // don't change this value (used also in cgribexlib)!
 #define  CALENDAR_GREGORIAN       1
 #define  CALENDAR_PROLEPTIC       2
 #define  CALENDAR_360DAYS         3
@@ -311,54 +311,56 @@ int     cdiEncodeTime(int hour, int minute, int second);
 
 int     cdiGetFiletype(const char *path, int *byteorder);
 
-/*      streamOpenRead: Open a dataset for reading */
+//      streamOpenRead: Open a dataset for reading
 int     streamOpenRead(const char *path);
 
-/*      streamOpenWrite: Create a new dataset */
+//      streamOpenWrite: Create a new dataset
 int     streamOpenWrite(const char *path, int filetype);
 
 int     streamOpenAppend(const char *path);
 
-/*      streamClose: Close an open dataset */
+//      streamClose: Close an open dataset
 void    streamClose(int streamID);
 
-/*      streamSync: Synchronize an Open Dataset to Disk */
+//      streamSync: Synchronize an Open Dataset to Disk
 void    streamSync(int streamID);
 
-/*      streamDefVlist: Define the Vlist for a stream */
+void    streamDefNumWorker(int streamID, int numWorker);
+
+//      streamDefVlist: Define the Vlist for a stream
 void    streamDefVlist(int streamID, int vlistID);
 
-/*      streamInqVlist: Get the Vlist of a stream */
+//      streamInqVlist: Get the Vlist of a stream
 int     streamInqVlist(int streamID);
 
-/*      streamInqFiletype: Get the filetype */
+//      streamInqFiletype: Get the filetype
 int     streamInqFiletype(int streamID);
 
-/*      streamDefByteorder: Define the byteorder */
+//      streamDefByteorder: Define the byteorder
 void    streamDefByteorder(int streamID, int byteorder);
 
-/*      streamInqByteorder: Get the byteorder */
+//      streamInqByteorder: Get the byteorder
 int     streamInqByteorder(int streamID);
 
-/*      streamDefCompType: Define compression type */
+//      streamDefCompType: Define compression type
 void    streamDefCompType(int streamID, int comptype);
 
-/*      streamInqCompType: Get compression type */
+//      streamInqCompType: Get compression type
 int     streamInqCompType(int streamID);
 
-/*      streamDefCompLevel: Define compression level */
+//      streamDefCompLevel: Define compression level
 void    streamDefCompLevel(int streamID, int complevel);
 
-/*      streamInqCompLevel: Get compression level */
+//      streamInqCompLevel: Get compression level
 int     streamInqCompLevel(int streamID);
 
-/*      streamDefTimestep: Define time step */
+//      streamDefTimestep: Define time step
 int     streamDefTimestep(int streamID, int tsID);
 
-/*      streamInqTimestep: Get time step */
+//      streamInqTimestep: Get time step
 int     streamInqTimestep(int streamID, int tsID);
 
-/*      PIO: query currently set timestep id  */
+//      PIO: query currently set timestep id
 int     streamInqCurTimestepID(int streamID);
 
 const char *streamFilename(int streamID);
@@ -656,9 +658,9 @@ int     vlistFindLevel(int vlistID, int fvarID, int flevelID);
 int     vlistMergedVar(int vlistID, int varID);
 int     vlistMergedLevel(int vlistID, int varID, int levelID);
 
-/* cdiClearAdditionalKeys: Clear the list of additional GRIB keys. */
+/* cdiClearAdditionalKeys: Clear the list of additional GRIB keys */
 void    cdiClearAdditionalKeys(void);
-/* cdiDefAdditionalKey: Register an additional GRIB key which is read when file is opened. */
+/* cdiDefAdditionalKey: Register an additional GRIB key which is read when file is opened */
 void    cdiDefAdditionalKey(const char *string);
 
 /* vlistDefVarIntKey: Set an arbitrary keyword/integer value pair for GRIB API */
@@ -666,39 +668,39 @@ void    vlistDefVarIntKey(int vlistID, int varID, const char *name, int value);
 /* vlistDefVarDblKey: Set an arbitrary keyword/double value pair for GRIB API */
 void    vlistDefVarDblKey(int vlistID, int varID, const char *name, double value);
 
-/* vlistHasVarKey: returns 1 if meta-data key was read, 0 otherwise. */
+/* vlistHasVarKey: returns 1 if meta-data key was read, 0 otherwise */
 int     vlistHasVarKey(int vlistID, int varID, const char *name);
 /* vlistInqVarDblKey: raw access to GRIB meta-data */
 double  vlistInqVarDblKey(int vlistID, int varID, const char *name);
 /* vlistInqVarIntKey: raw access to GRIB meta-data */
 int     vlistInqVarIntKey(int vlistID, int varID, const char *name);
 
-/* CDI attributes */
+// CDI attributes
 
-/*      cdiInqNatts: Get number of attributes assigned to this variable */
+//      cdiInqNatts: Get number of attributes assigned to this variable
 int     cdiInqNatts(int cdiID, int varID, int *nattsp);
-/*      cdiInqAtt: Get information about an attribute */
+//      cdiInqAtt: Get information about an attribute
 int     cdiInqAtt(int cdiID, int varID, int attrnum, char *name, int *typep, int *lenp);
 int     cdiDelAtt(int cdiID, int varID, const char *name);
 
 int     cdiCopyAtts(int cdiID1, int varID1, int cdiID2, int varID2);
 
-/*      cdiDefAttInt: Define an integer attribute */
+//      cdiDefAttInt: Define an integer attribute
 int     cdiDefAttInt(int cdiID, int varID, const char *name, int type, int len, const int ip[]);
-/*      cdiDefAttFlt: Define a floating point attribute */
+//      cdiDefAttFlt: Define a floating point attribute
 int     cdiDefAttFlt(int cdiID, int varID, const char *name, int type, int len, const double dp[]);
-/*      cdiDefAttTxt: Define a text attribute */
+//      cdiDefAttTxt: Define a text attribute
 int     cdiDefAttTxt(int cdiID, int varID, const char *name, int len, const char *tp_cbuf);
 
-/*      cdiInqAttInt: Get the value(s) of an integer attribute */
+//      cdiInqAttInt: Get the value(s) of an integer attribute
 int     cdiInqAttInt(int cdiID, int varID, const char *name, int mlen, int ip[]);
-/*      cdiInqAttFlt: Get the value(s) of a floating point attribute */
+//      cdiInqAttFlt: Get the value(s) of a floating point attribute
 int     cdiInqAttFlt(int cdiID, int varID, const char *name, int mlen, double dp[]);
-/*      cdiInqAttTxt: Get the value(s) of a text attribute */
+//      cdiInqAttTxt: Get the value(s) of a text attribute
 int     cdiInqAttTxt(int cdiID, int varID, const char *name, int mlen, char *tp_cbuf);
 
 
-/* GRID routines */
+// GRID routines
 
 void    gridName(int gridtype, char *gridname);
 const char *gridNamePtr(int gridtype);
@@ -711,74 +713,72 @@ int     gridInqMaskGME(int gridID, int mask[]);
 void    gridDefMask(int gridID, const int mask[]);
 int     gridInqMask(int gridID, int mask[]);
 
-void    gridPrint(int gridID, int opt);
-
-/*      gridCreate: Create a horizontal Grid */
+//      gridCreate: Create a horizontal Grid
 int     gridCreate(int gridtype, size_t size);
 
-/*      gridDestroy: Destroy a horizontal Grid */
+//      gridDestroy: Destroy a horizontal Grid
 void    gridDestroy(int gridID);
 
-/*      gridDuplicate: Duplicate a Grid */
+//      gridDuplicate: Duplicate a Grid
 int     gridDuplicate(int gridID);
 
-/*      gridDefProj: Define the projection ID of a Grid */
+//      gridDefProj: Define the projection ID of a Grid
 void    gridDefProj(int gridID, int projID);
 
-/*      gridInqProj: Get the projection ID of a Grid */
+//      gridInqProj: Get the projection ID of a Grid
 int     gridInqProj(int gridID);
 
-/*      gridInqProjType: Get the projection type */
+//      gridInqProjType: Get the projection type
 int     gridInqProjType(int gridID);
 
-/*      gridInqType: Get the type of a Grid */
+//      gridInqType: Get the type of a Grid
 int     gridInqType(int gridID);
 
-/*      gridInqSize: Get the size of a Grid */
+//      gridInqSize: Get the size of a Grid
 size_t  gridInqSize(int gridID);
 
-/*      gridDefXsize: Define the size of a X-axis */
+//      gridDefXsize: Define the size of a X-axis
 void    gridDefXsize(int gridID, size_t xsize);
 
-/*      gridInqXsize: Get the size of a X-axis */
+//      gridInqXsize: Get the size of a X-axis
 size_t  gridInqXsize(int gridID);
 
-/*      gridDefYsize: Define the size of a Y-axis */
+//      gridDefYsize: Define the size of a Y-axis
 void    gridDefYsize(int gridID, size_t ysize);
 
-/*      gridInqYsize: Get the size of a Y-axis */
+//      gridInqYsize: Get the size of a Y-axis
 size_t  gridInqYsize(int gridID);
 
-/*      gridDefNP: Define the number of parallels between a pole and the equator */
+//      gridDefNP: Define the number of parallels between a pole and the equator
 void    gridDefNP(int gridID, int np);
 
-/*      gridInqNP: Get the number of parallels between a pole and the equator */
+//      gridInqNP: Get the number of parallels between a pole and the equator
 int     gridInqNP(int gridID);
 
-/*      gridDefXvals: Define the values of a X-axis */
+//      gridDefXvals: Define the values of a X-axis
 void    gridDefXvals(int gridID, const double xvals[]);
 
-/*      gridInqXvals: Get all values of a X-axis */
+//      gridInqXvals: Get all values of a X-axis
 size_t  gridInqXvals(int gridID, double xvals[]);
 size_t  gridInqXvalsPart(int gridID, int start, size_t size, double xvals[]);
 
-/*      gridInqXIsc: Find out whether X-coordinate is of type CHAR */
+//      gridInqXIsc: Find out whether X-coordinate is of type CHAR
 int     gridInqXIsc(int gridID);
 
-/*      gridInqXCvals: Get strings from X-axis in case grid is of type GRID_CHARXY */
+//      gridInqXCvals: Get strings from X-axis in case grid is of type GRID_CHARXY
 size_t  gridInqXCvals(int gridID, char *xcvals[]);
 
-/*      gridDefYvals: Define the values of a Y-axis */
+//      gridDefYvals: Define the values of a Y-axis
 void    gridDefYvals(int gridID, const double yvals[]);
 
-/*      gridInqYvals: Get all values of a Y-axis */
+//      gridInqYvals: Get all values of a Y-axis
 size_t  gridInqYvals(int gridID, double yvals[]);
 size_t  gridInqYvalsPart(int gridID, int start, size_t size, double yvals[]);
 
-/*      gridInqYIsc: Find out whether Y-coordinate is of type CHAR */
+//      gridInqYIsc: Find out whether Y-coordinate is of type CHAR
 int     gridInqYIsc(int gridID);
 
-/*      gridInqYCvals: Get strings from Y-axis in case grid is of type GRID_CHARXY */
+//      gridInqYCvals: Get strings from Y-axis in case grid is of type GRID_CHARXY
 size_t  gridInqYCvals(int gridID, char *ycvals[]);
 
 // CDI var keys
@@ -802,6 +802,8 @@ size_t  gridInqYCvals(int gridID, char *ycvals[]);
 #define  CDI_KEY_GRIB2LOCALSECTIONNUMBER       818  // GRIB2 grib2LocalSectionNumber
 #define  CDI_KEY_SECTION2PADDINGLENGTH         819  // GRIB2 length of section2Padding
 #define  CDI_KEY_SECTION2PADDING               820  // GRIB2 section2Padding
+#define  CDI_KEY_UVRELATIVETOGRID              821  // GRIB  uvRelativeToGrid
+#define  CDI_KEY_CONSTITUENTTYPE               822  // GRIB2 constituentType
 
 //      cdiDefKeyInt: Define an integer value from a key of a CDI variable
 int     cdiDefKeyInt(int cdiID, int varID, int key, int value);
@@ -1001,92 +1003,87 @@ void    gridChangeType(int gridID, int gridtype);
 void    gridDefComplexPacking(int gridID, int lpack);
 int     gridInqComplexPacking(int gridID);
 
-void    gridDefUvRelativeToGrid(int gridID, int uvRelativeToGrid);
-int     gridInqUvRelativeToGrid(int gridID);
-
 void    gridDefScanningMode(int gridID, int mode);
 int     gridInqScanningMode(int gridID);
 
-/* ZAXIS routines */
+// ZAXIS routines
 
 void    zaxisName(int zaxistype, char *zaxisname);
 const char *zaxisNamePtr(int leveltype);
 
-/*      zaxisCreate: Create a vertical Z-axis */
+//      zaxisCreate: Create a vertical Z-axis
 int     zaxisCreate(int zaxistype, int size);
 
-/*      zaxisDestroy: Destroy a vertical Z-axis */
+//      zaxisDestroy: Destroy a vertical Z-axis
 void    zaxisDestroy(int zaxisID);
 
-/*      zaxisInqType: Get the type of a Z-axis */
+//      zaxisInqType: Get the type of a Z-axis
 int     zaxisInqType(int zaxisID);
 
-/*      zaxisInqSize: Get the size of a Z-axis */
+//      zaxisInqSize: Get the size of a Z-axis
 int     zaxisInqSize(int zaxisID);
 
-/*      zaxisDuplicate: Duplicate a Z-axis */
+//      zaxisDuplicate: Duplicate a Z-axis
 int     zaxisDuplicate(int zaxisID);
 
-void    zaxisPrint(int zaxisID);
-
-/*      zaxisDefLevels: Define the levels of a Z-axis */
+//      zaxisDefLevels: Define the levels of a Z-axis
 void    zaxisDefLevels(int zaxisID, const double levels[]);
 
-/*      zaxisDefCvals: Define area types of a Z-axis */
+//      zaxisDefCvals: Define area types of a Z-axis
 void    zaxisDefCvals(int zaxisID, const char *cvals[], int clength);
 
-/*      zaxisInqLevels: Get all levels of a Z-axis */
+//      zaxisInqLevels: Get all levels of a Z-axis
 int     zaxisInqLevels(int zaxisID, double levels[]);
 
-/*      zaxisInqCLen: Get maximal string length of character Z-axis */
+//      zaxisInqCLen: Get maximal string length of character Z-axis
 int     zaxisInqCLen(int zaxisID);
 
-/*      zaxisInqCVals: Get all string values of a character Z-axis */
+//      zaxisInqCVals: Get all string values of a character Z-axis
 int     zaxisInqCVals(int zaxisID, char ***clevels);
 
-/*      zaxisDefLevel: Define one level of a Z-axis */
+//      zaxisDefLevel: Define one level of a Z-axis
 void    zaxisDefLevel(int zaxisID, int levelID, double levels);
 
-/*      zaxisInqLevel: Get one level of a Z-axis */
+//      zaxisInqLevel: Get one level of a Z-axis
 double  zaxisInqLevel(int zaxisID, int levelID);
 
-/*      zaxisDefNlevRef: Define the number of half levels of a generalized Z-axis */
+//      zaxisDefNlevRef: Define the number of half levels of a generalized Z-axis
 void    zaxisDefNlevRef(int gridID, int nhlev);
 
-/*      zaxisInqNlevRef: Get the number of half levels of a generalized Z-axis */
+//      zaxisInqNlevRef: Get the number of half levels of a generalized Z-axis
 int     zaxisInqNlevRef(int gridID);
 
 /*      zaxisDefNumber: Define the reference number for a generalized Z-axis */
 void    zaxisDefNumber(int gridID, int number);
 
-/*      zaxisInqNumber: Get the reference number to a generalized Z-axis */
+//      zaxisInqNumber: Get the reference number to a generalized Z-axis
 int     zaxisInqNumber(int gridID);
 
-/*      zaxisDefUUID: Define the UUID of a generalized Z-axis */
+//      zaxisDefUUID: Define the UUID of a generalized Z-axis
 void    zaxisDefUUID(int zaxisID, const unsigned char uuid[CDI_UUID_SIZE]);
 
-/*      zaxisInqUUID: Get the UUID of a generalized Z-axis */
+//      zaxisInqUUID: Get the UUID of a generalized Z-axis
 void    zaxisInqUUID(int zaxisID, unsigned char uuid[CDI_UUID_SIZE]);
 
-/*      zaxisDefName: Define the name of a Z-axis */
+//      zaxisDefName: Define the name of a Z-axis
 void    zaxisDefName(int zaxisID, const char *name_optional);
 
-/*      zaxisInqName: Get the name of a Z-axis */
+//      zaxisInqName: Get the name of a Z-axis
 void    zaxisInqName(int zaxisID, char *name);
 
-/*      zaxisDefLongname: Define the longname of a Z-axis */
+//      zaxisDefLongname: Define the longname of a Z-axis
 void    zaxisDefLongname(int zaxisID, const char *longname_optional);
 
-/*      zaxisInqLongname: Get the longname of a Z-axis */
+//      zaxisInqLongname: Get the longname of a Z-axis
 void    zaxisInqLongname(int zaxisID, char *longname);
 
-/*      zaxisDefUnits: Define the units of a Z-axis */
+//      zaxisDefUnits: Define the units of a Z-axis
 void    zaxisDefUnits(int zaxisID, const char *units_optional);
 
-/*      zaxisInqUnits: Get the units of a Z-axis */
+//      zaxisInqUnits: Get the units of a Z-axis
 void    zaxisInqUnits(int zaxisID, char *units);
 
-/*      zaxisInqStdname: Get the standard name of a Z-axis */
+//      zaxisInqStdname: Get the standard name of a Z-axis
 void    zaxisInqStdname(int zaxisID, char *stdname);
 
 void    zaxisDefDatatype(int zaxisID, int prec);
@@ -1101,6 +1098,9 @@ int     zaxisInqScalar(int zaxisID);
 void    zaxisDefLtype(int zaxisID, int ltype);
 int     zaxisInqLtype(int zaxisID);
 
+void    zaxisDefLtype2(int zaxisID, int ltype2);
+int     zaxisInqLtype2(int zaxisID);
+
 void    zaxisDefVct(int zaxisID, int size, const double vct[]);
 void    zaxisInqVct(int zaxisID, double vct[]);
 int     zaxisInqVctSize(int zaxisID);
@@ -1252,41 +1252,41 @@ int     subtypeCreate(int subtype);
 /*      Gives a textual summary of the variable subtype */
 void    subtypePrint(int subtypeID);
 
-/* Compares two subtype data structures. */
+/* Compares two subtype data structures */
 int     subtypeCompare(int subtypeID1, int subtypeID2);
 
-/*      subtypeInqSize: Get the size of a subtype (e.g. no. of tiles). */
+/*      subtypeInqSize: Get the size of a subtype (e.g. no. of tiles) */
 int     subtypeInqSize(int subtypeID);
 
-/*      subtypeInqActiveIndex: Get the currently active index of a subtype (e.g. current tile index). */
+/*      subtypeInqActiveIndex: Get the currently active index of a subtype (e.g. current tile index) */
 int     subtypeInqActiveIndex(int subtypeID);
 
-/*      subtypeDefActiveIndex: Set the currently active index of a subtype (e.g. current tile index). */
+/*      subtypeDefActiveIndex: Set the currently active index of a subtype (e.g. current tile index) */
 void    subtypeDefActiveIndex(int subtypeID, int index);
 
-/*      Generate a "query object" out of a key-value pair. */
+/*      Generate a "query object" out of a key-value pair */
 subtype_query_t keyValuePair(const char *key, int value);
 
-/*       Generate an AND-combined "query object" out of two previous
-         query objects. */
+/*       Generate an AND-combined "query object" out of two previous query objects */
 subtype_query_t matchAND(subtype_query_t q1, subtype_query_t q2);
 
-/*      subtypeInqSubEntry: Returns subtype entry ID for a given criterion. */
+/*      subtypeInqSubEntry: Returns subtype entry ID for a given criterion */
 int     subtypeInqSubEntry(int subtypeID, subtype_query_t criterion);
 
-/*      subtypeInqTile: Specialized version of subtypeInqSubEntry looking for tile/attribute pair. */
+/*      subtypeInqTile: Specialized version of subtypeInqSubEntry looking for tile/attribute pair */
 int     subtypeInqTile(int subtypeID, int tileindex, int attribute);
 
 /*      subtypeInqAttribute: Inquire the value of a subtype attribute. Returns CDI_EINVAL if the attribute does not exist.*/
 int     subtypeInqAttribute(int subtypeID, int index, const char *key, int *outValue);
 
-/*      vlistInqVarSubtype: Return subtype ID for a given variable. */
+/*      vlistInqVarSubtype: Return subtype ID for a given variable */
 int     vlistInqVarSubtype(int vlistID, int varID);
 
 void gribapiLibraryVersion(int *major_version, int *minor_version, int *revision_version);
 
+void gaussianLatitudes(double *latitudes, double *weights, size_t nlat);
 
-#if defined (__cplusplus)
+#ifdef __cplusplus
 }
 #endif
 
