@@ -1,5 +1,5 @@
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#include "config.h"
 #endif
 
 #ifdef HAVE_LIBGRIB_API
@@ -340,18 +340,22 @@ int gribapiTimeIsFC(grib_handle *gh)
   return sigofrtime != 3;
 }
 
+
 struct cdiGribAPI_ts_str_map_elem cdiGribAPI_ts_str_map[] = {
+  // clang-format off
   [TSTEP_INSTANT] = {  0, "instant" },
-  [TSTEP_AVG] = { 8, "avg" },
-  [TSTEP_ACCUM] = {  8, "accum" },
-  [TSTEP_MAX] = {  8, "max" },
-  [TSTEP_MIN] = {  8, "min" },
-  [TSTEP_DIFF] = {  8, "diff" },
-  [TSTEP_RMS] = {  8, "rms" },
-  [TSTEP_SD] = {  8, "sd" },
-  [TSTEP_COV] = { 8, "cov" },
-  [TSTEP_RATIO] = {  8, "ratio" },
-  { 0, "" }
+  [TSTEP_AVG]     = {  8, "avg" },
+  [TSTEP_ACCUM]   = {  8, "accum" },
+  [TSTEP_MAX]     = {  8, "max" },
+  [TSTEP_MIN]     = {  8, "min" },
+  [TSTEP_DIFF]    = {  8, "diff" },
+  [TSTEP_RMS]     = {  8, "rms" },
+  [TSTEP_SD]      = {  8, "sd" },
+  [TSTEP_COV]     = {  8, "cov" },
+  [TSTEP_RATIO]   = {  8, "ratio" },
+  [TSTEP_SUM]     = {  8, "sum" },
+                    {  0, "" }
+  // clang-format on
 };
 
 
@@ -764,8 +768,9 @@ void gribapiGetGridGeneric(grib_handle *gh, grid_t *grid, size_t numberOfPoints)
 }
 
 //TODO: Simplify by use of the convenience functions (gribGetLong(), gribGetLongDefault(), etc.).
-void gribapiGetGrid(grib_handle *gh, grid_t *grid)
+bool gribapiGetGrid(grib_handle *gh, grid_t *grid)
 {
+  bool uvRelativeToGrid = false;
   const long editionNumber = gribEditionNumber(gh);
   int gridtype = gribapiGetGridType(gh);
   int projtype = (gridtype == GRID_PROJECTION && gribapiGetIsRotated(gh)) ? CDI_PROJ_RLL : CDI_UNDEFID;
@@ -833,7 +838,7 @@ void gribapiGetGrid(grib_handle *gh, grid_t *grid)
       long temp;
       GRIB_CHECK(grib_get_long(gh, "uvRelativeToGrid", &temp), 0);
       assert(temp == 0 || temp == 1);
-      grid->uvRelativeToGrid = (bool)temp;
+      uvRelativeToGrid = (bool)temp;
     }
 
   if ( gridtype == GRID_GAUSSIAN || gridtype == GRID_LONLAT || gridtype == GRID_PROJECTION )
@@ -863,14 +868,16 @@ void gribapiGetGrid(grib_handle *gh, grid_t *grid)
         Message("(param,ltype,level) = (%3d,%3d,%4d); Scanning mode = %02d -> bits:(%1d.%1d.%1d)*32;  uvRelativeToGrid = %02d",\
                 (int)paramId, (int)levelTypeId, (int)levelId,
                 grid->scanningMode,grid->jPointsAreConsecutive,
-                grid->jScansPositively,grid->iScansNegatively,
-                grid->uvRelativeToGrid);
+                grid->jScansPositively, grid->iScansNegatively,
+                uvRelativeToGrid);
       }
 #endif //HIRLAM_EXTENSIONS
     }
 
   grid->type  = gridtype;
   grid->projtype  = projtype;
+
+  return uvRelativeToGrid;
 }
 #endif
 /*
