@@ -30,6 +30,7 @@
 #include "resource_handle.h"
 #include "resource_unpack.h"
 #include "namespace.h"
+#include "async_worker.h"
 
 
 static stream_t *stream_new_entry(int resH);
@@ -231,6 +232,17 @@ int getByteswap(int byteorder)
   return byteswap;
 }
 
+
+void streamDefNumWorker(int streamID, int numWorker)
+{
+  if (numWorker > 0)
+    {
+      stream_t *streamptr = stream_to_pointer(streamID);
+      streamptr->numWorker = numWorker;
+    }
+}
+
+
 /*
 @Function  streamDefByteorder
 @Title     Define the byte order
@@ -439,7 +451,7 @@ int cdiStreamOpenDefaultDelegate(const char *filename, char filemode, int filety
 
   switch (filetype)
     {
-#ifdef  HAVE_LIBGRIB
+#if defined(HAVE_LIBGRIB) && (defined(HAVE_LIBCGRIBEX) || defined(HAVE_LIBGRIB_API))
     case CDI_FILETYPE_GRB:
       {
         fileID = gribOpen(filename, temp);
@@ -660,7 +672,7 @@ int streamOpenA(const char *filename, const char *filemode, int filetype)
 
   switch (filetype)
     {
-#ifdef  HAVE_LIBGRIB
+#if defined(HAVE_LIBGRIB) && (defined(HAVE_LIBCGRIBEX) || defined(HAVE_LIBGRIB_API))
     case CDI_FILETYPE_GRB:
 #ifdef  HAVE_LIBGRIB_API
     case CDI_FILETYPE_GRB2:
@@ -905,6 +917,12 @@ void streamDefaultValue ( stream_t * streamptr )
 #endif
 
   streamptr->gribContainers    = NULL;
+
+  streamptr->numWorker         = 0;
+  streamptr->nextRecID         = 0;
+  streamptr->cachedTsID        = -1;
+  streamptr->jobs              = NULL;
+  streamptr->jobManager        = NULL;
 }
 
 static
@@ -936,7 +954,7 @@ void cdiStreamCloseDefaultDelegate(stream_t *streamptr, int recordBufIsToBeDelet
   else
     switch (filetype)
       {
-#ifdef  HAVE_LIBGRIB
+#if defined(HAVE_LIBGRIB) && (defined(HAVE_LIBCGRIBEX) || defined(HAVE_LIBGRIB_API))
       case CDI_FILETYPE_GRB:
         {
           gribClose(fileID);
@@ -1040,7 +1058,7 @@ void streamClose(int streamID)
   if ( CDI_Debug )
     Message("streamID = %d filename = %s", streamID, streamptr->filename);
 
-  int vlistID  = streamptr->vlistID;
+  const int vlistID  = streamptr->vlistID;
 
   void (*streamCloseDelegate)(stream_t *streamptr, int recordBufIsToBeDeleted)
     = (void (*)(stream_t *, int))
@@ -1062,7 +1080,7 @@ void streamClose(int streamID)
   for ( int index = 0; index < streamptr->nvars; index++ )
     {
       sleveltable_t *pslev = streamptr->vars[index].recordTable;
-      unsigned nsub = streamptr->vars[index].subtypeSize >= 0
+      const unsigned nsub = streamptr->vars[index].subtypeSize >= 0
         ? (unsigned)streamptr->vars[index].subtypeSize : 0U;
       for (size_t isub=0; isub < nsub; isub++)
         {
@@ -1092,6 +1110,9 @@ void streamClose(int streamID)
       cdiVlistDestroy_(vlistID);
     }
 
+  if (streamptr->jobs) free(streamptr->jobs);
+  if (streamptr->jobManager) AsyncWorker_finalize(streamptr->jobManager);
+
   stream_delete_entry(streamptr);
 }
 
