@@ -448,10 +448,9 @@ cdfDefAxisCommon(stream_t *streamptr, int gridID, int gridindex, int ndims,
 {
   int dimID = CDI_UNDEFID;
   int ncvarid = CDI_UNDEFID, ncbvarid = CDI_UNDEFID;
-  int nvdimID = CDI_UNDEFID;
-  int fileID  = streamptr->fileID;
-  size_t dimlen = gridAxisInq->axisSize(gridID);
-  nc_type xtype = (nc_type)cdfDefDatatype(gridInqDatatype(gridID), streamptr);
+  const int fileID  = streamptr->fileID;
+  const size_t dimlen = gridAxisInq->axisSize(gridID);
+  const nc_type xtype = (nc_type)cdfDefDatatype(gridInqDatatype(gridID), streamptr);
 
   ncgrid_t *ncgrid = streamptr->ncgrid;
 
@@ -461,27 +460,25 @@ cdfDefAxisCommon(stream_t *streamptr, int gridID, int gridindex, int ndims,
 
   for ( int index = 0; index < gridindex; ++index )
     {
-      int gridID0 = ncgrid[index].gridID;
+      const int gridID0 = ncgrid[index].gridID;
       assert(gridID0 != CDI_UNDEFID);
-      int gridtype0 = gridInqType(gridID0);
+      const int gridtype0 = gridInqType(gridID0);
       if ( gridtype0 == GRID_GAUSSIAN    ||
            gridtype0 == GRID_LONLAT      ||
            gridtype0 == GRID_PROJECTION  ||
-           gridtype0 == GRID_CURVILINEAR ||
            gridtype0 == GRID_GENERIC )
         {
-          size_t dimlen0 = gridAxisInq->axisSize(gridID0);
+          const size_t dimlen0 = gridAxisInq->axisSize(gridID0);
           char dimname0[CDI_MAX_NAME]; dimname0[0] = 0;
           if ( dimname[0] ) cdiGridInqKeyStr(gridID0, dimKey, CDI_MAX_NAME, dimname0);
-          bool lname = dimname0[0] ? strcmp(dimname, dimname0) == 0 : true;
+          const bool lname = dimname0[0] ? strcmp(dimname, dimname0) == 0 : true;
           if ( dimlen == dimlen0 && lname )
             {
               double (*inqVal)(int gridID, size_t index) = gridAxisInq->axisVal;
               if ( IS_EQUAL(inqVal(gridID0, 0), inqVal(gridID, 0)) &&
                    IS_EQUAL(inqVal(gridID0, dimlen-1), inqVal(gridID, dimlen-1)) )
                 {
-                  dimID = ncgrid[index].ncIDs[dimKey == CDI_KEY_XDIMNAME
-                                              ? CDF_DIMID_X : CDF_DIMID_Y];
+                  dimID = ncgrid[index].ncIDs[dimKey == CDI_KEY_XDIMNAME ? CDF_DIMID_X : CDF_DIMID_Y];
                   break;
                 }
             }
@@ -491,7 +488,7 @@ cdfDefAxisCommon(stream_t *streamptr, int gridID, int gridindex, int ndims,
   if ( dimID == CDI_UNDEFID )
     {
       char axisname[CDI_MAX_NAME]; axisname[0] = 0;
-      int keyname = (axisLetter == 'X') ? CDI_KEY_XNAME : CDI_KEY_YNAME;
+      const int keyname = (axisLetter == 'X') ? CDI_KEY_XNAME : CDI_KEY_YNAME;
       gridAxisInq->axisName(gridID, keyname, CDI_MAX_NAME, axisname);
       if ( axisname[0] == 0 ) Error("axis name undefined!");
 
@@ -509,7 +506,7 @@ cdfDefAxisCommon(stream_t *streamptr, int gridID, int gridindex, int ndims,
         }
 
       bool gen_bounds = false;
-      bool grid_is_cyclic = gridIsCircular(gridID) > 0;
+      const bool grid_is_cyclic = gridIsCircular(gridID) > 0;
       double *pbounds = NULL;
       if ( pvals )
         {
@@ -536,6 +533,8 @@ cdfDefAxisCommon(stream_t *streamptr, int gridID, int gridindex, int ndims,
                 }
               finishCyclicBounds(pbounds, dimlen, pvals);
             }
+
+          int nvdimID = CDI_UNDEFID;
           if ( pbounds )
             {
               if ( nc_inq_dimid(fileID, bndsName, &nvdimID) != NC_NOERR )
@@ -561,13 +560,11 @@ cdfDefAxisCommon(stream_t *streamptr, int gridID, int gridindex, int ndims,
       if ( gen_bounds ) Free(pbounds);
 
       if ( ndims == 0 )
-        ncgrid[gridindex].ncIDs[dimKey == CDI_KEY_XDIMNAME
-                                ? CDF_VARID_X : CDF_VARID_Y] = ncvarid;
+        ncgrid[gridindex].ncIDs[dimKey == CDI_KEY_XDIMNAME ? CDF_VARID_X : CDF_VARID_Y] = ncvarid;
     }
 
   ncgrid[gridindex].gridID = gridID;
-  ncgrid[gridindex].ncIDs[dimKey == CDI_KEY_XDIMNAME
-                          ? CDF_DIMID_X : CDF_DIMID_Y] = dimID;
+  ncgrid[gridindex].ncIDs[dimKey == CDI_KEY_XDIMNAME ? CDF_DIMID_X : CDF_DIMID_Y] = dimID;
 }
 
 static
@@ -613,13 +610,10 @@ void cdfGridCompress(int fileID, int ncvarid, size_t gridsize, int filetype, int
 static
 void cdfDefGridReference(stream_t *streamptr, int gridID)
 {
-  int fileID  = streamptr->fileID;
-  int number = gridInqNumber(gridID);
+  const int fileID = streamptr->fileID;
 
-  if ( number > 0 )
-    {
-      cdf_put_att_int(fileID, NC_GLOBAL, "number_of_grid_used", NC_INT, 1, &number);
-    }
+  const int number = gridInqNumber(gridID);
+  if ( number > 0 ) cdf_put_att_int(fileID, NC_GLOBAL, "number_of_grid_used", NC_INT, 1, &number);
 
   const char *gridfile = gridInqReferencePtr(gridID);
   if ( gridfile && gridfile[0] != 0 )
@@ -805,9 +799,9 @@ void cdfDefCurvilinear(stream_t *streamptr, int gridID, int gridindex)
 {
   ncgrid_t *ncgrid = streamptr->ncgrid;
 
-  size_t dimlen = gridInqSize(gridID);
-  size_t xdimlen = gridInqXsize(gridID);
-  size_t ydimlen = gridInqYsize(gridID);
+  const size_t dimlen = gridInqSize(gridID);
+  const size_t xdimlen = gridInqXsize(gridID);
+  const size_t ydimlen = gridInqYsize(gridID);
 
   int xdimID = CDI_UNDEFID, ydimID = CDI_UNDEFID;
   int ncxvarid = CDI_UNDEFID, ncyvarid = CDI_UNDEFID, ncavarid = CDI_UNDEFID;
@@ -818,10 +812,10 @@ void cdfDefCurvilinear(stream_t *streamptr, int gridID, int gridindex)
         = cdfSearchIDBySize(ofs, (size_t)gridindex, ncgrid, CDF_DIMID_X,
                             GRID_CURVILINEAR, (int)dimlen,
                             gridInqType, gridInqSize);
-      size_t index = search.foundIdx;
+      const size_t index = search.foundIdx;
       if ( index != SIZE_MAX )
         {
-          int gridID0 = ncgrid[index].gridID;
+          const int gridID0 = ncgrid[index].gridID;
           if (    IS_EQUAL(gridInqXval(gridID0, 0), gridInqXval(gridID, 0))
                && IS_EQUAL(gridInqXval(gridID0, dimlen-1),
                            gridInqXval(gridID, dimlen-1))
@@ -870,7 +864,7 @@ void cdfDefUnstructured(stream_t *streamptr, int gridID, int gridindex)
 {
   ncgrid_t *ncgrid = streamptr->ncgrid;
 
-  size_t dimlen = gridInqSize(gridID);
+  const size_t dimlen = gridInqSize(gridID);
 
   int dimID = CDI_UNDEFID;
   int ncxvarid = CDI_UNDEFID, ncyvarid = CDI_UNDEFID, ncavarid = CDI_UNDEFID;
@@ -881,10 +875,10 @@ void cdfDefUnstructured(stream_t *streamptr, int gridID, int gridindex)
         = cdfSearchIDBySize(ofs, (size_t)gridindex, ncgrid, CDF_DIMID_X,
                             GRID_UNSTRUCTURED, (int)dimlen,
                             gridInqType, gridInqSize);
-      size_t index = search.foundIdx;
+      const size_t index = search.foundIdx;
       if ( index != SIZE_MAX )
         {
-          int gridID0 = ncgrid[index].gridID;
+          const int gridID0 = ncgrid[index].gridID;
           if ( gridInqNvertex(gridID0) == gridInqNvertex(gridID) &&
                IS_EQUAL(gridInqXval(gridID0, 0), gridInqXval(gridID, 0)) &&
                IS_EQUAL(gridInqXval(gridID0, dimlen-1),
@@ -908,7 +902,7 @@ void cdfDefUnstructured(stream_t *streamptr, int gridID, int gridindex)
 
   if ( dimID == CDI_UNDEFID )
     {
-      size_t nvertex = (size_t)gridInqNvertex(gridID);
+      const size_t nvertex = (size_t)gridInqNvertex(gridID);
       struct cdfDefIrregularGridCommonIDs createdIDs
         = cdfDefIrregularGridCommon(streamptr, gridID,
                                     dimlen, 1, 2, "ncells",
@@ -1412,6 +1406,7 @@ void cdfDefZaxisUUID(stream_t *streamptr, int zaxisID)
     }
 }
 
+#ifndef USE_MPI
 static
 void cdfDefZaxisChar(stream_t *streamptr, int zaxisID, char *axisname, int *dimID, size_t dimlen, int zaxisindex)
 {
@@ -1472,6 +1467,7 @@ void cdfDefZaxisChar(stream_t *streamptr, int zaxisID, char *axisname, int *dimI
 
   streamptr->ncmode = 2;
 }
+#endif
 
 static
 void cdfDefZaxis(stream_t *streamptr, int zaxisID)
@@ -1547,8 +1543,10 @@ void cdfDefZaxis(stream_t *streamptr, int zaxisID)
           cdfDefineAttributes(zaxisID, CDI_GLOBAL, fileID, ncvarid);
           if ( natts > 0 && streamptr->ncmode == 2 ) cdf_enddef(fileID);
         }
+#ifndef USE_MPI
       else if ( type == ZAXIS_CHAR )
         cdfDefZaxisChar(streamptr, zaxisID, axisname, &dimID, dimlen, zaxisindex);
+#endif
       else
         {
           dimID = checkDimName(fileID, dimlen, dimname);
@@ -1674,7 +1672,7 @@ void cdfDefCharacter(stream_t *streamptr, int gridID, int gridindex, int xory, i
   int dimID, strlenID;
   ncgrid_t *ncgrid = streamptr->ncgrid;
 
-  /* Check for all grids up to gridindex whether it already is defined */
+  // Check for all grids up to gridindex whether it already is defined
 
   for ( int index = 0; index < gridindex; index++ )
     {
@@ -1682,11 +1680,9 @@ void cdfDefCharacter(stream_t *streamptr, int gridID, int gridindex, int xory, i
       int gridtype0 = gridInqType(gridID0);
       if ( gridtype0 == GRID_CHARXY )
         {
-          if ( gridInqXIsc(gridID0) == strlen &&
-               gridInqXsize(gridID0) == dimlen )
+          if ( gridInqXIsc(gridID0) == strlen && gridInqXsize(gridID0) == dimlen )
             return;
-          else if ( gridInqYIsc(gridID0) == strlen &&
-               gridInqYsize(gridID0) == dimlen )
+          else if ( gridInqYIsc(gridID0) == strlen && gridInqYsize(gridID0) == dimlen )
             return;
         }
     }
@@ -1695,7 +1691,7 @@ void cdfDefCharacter(stream_t *streamptr, int gridID, int gridindex, int xory, i
 
   if ( streamptr->ncmode == 2 ) cdf_redef(fileID);
 
-/* Define Dims */
+  // Define Dims
 
   char dimname[CDI_MAX_NAME+3];
   dimname[0] = 0;
@@ -1707,13 +1703,13 @@ void cdfDefCharacter(stream_t *streamptr, int gridID, int gridindex, int xory, i
   dimID = checkDimName(fileID, dimlen, dimname);
   if ( dimID == CDI_UNDEFID ) cdf_def_dim(fileID, dimname, dimlen, &dimID);
 
-/* Define strlength dim */
+  // Define strlength dim
 
   strcpy(dimname, "strlen");
   strlenID = checkDimName(fileID, strlen, dimname);
   if ( strlenID == CDI_UNDEFID ) cdf_def_dim(fileID, dimname, strlen, &strlenID);
 
-/* Define Variable */
+  // Define Variable
 
   int dimIDs[2];
   dimIDs[0] = dimID;
@@ -1747,7 +1743,7 @@ void cdfDefCharacter(stream_t *streamptr, int gridID, int gridindex, int xory, i
     return;
   cdf_enddef(fileID);
 
-/* Write Var */
+  // Write Var
 
   size_t start[2], count[2];
   start[1] = 0;
@@ -1778,10 +1774,11 @@ void cdfDefRgrid(stream_t *streamptr, int gridID, int gridindex)
 {
   ncgrid_t *ncgrid = streamptr->ncgrid;
 
-  size_t dimlen = gridInqSize(gridID);
+  const size_t dimlen = gridInqSize(gridID);
 
   int iz;
   int dimID;
+
   {
     struct idSearch search
       = cdfSearchIDBySize(0, (size_t)gridindex, ncgrid, CDF_DIMID_X,
@@ -1793,7 +1790,7 @@ void cdfDefRgrid(stream_t *streamptr, int gridID, int gridindex)
 
   if ( dimID == CDI_UNDEFID )
     {
-      int fileID  = streamptr->fileID;
+      const int fileID  = streamptr->fileID;
       static bool lwarn = true;
       if ( lwarn )
         {
@@ -1872,8 +1869,8 @@ void cdfDefGrid(stream_t *streamptr, int gridID, int gridindex)
 {
   if ( streamptr->ncgrid[gridindex].ncIDs[CDF_DIMID_X] != CDI_UNDEFID ) return;
 
-  int gridtype = gridInqType(gridID);
-  size_t size = gridInqSize(gridID);
+  const int gridtype = gridInqType(gridID);
+  const size_t size = gridInqSize(gridID);
 
   if ( CDI_Debug )
     Message("gridtype = %d  size = %zu", gridtype, size);
@@ -1900,13 +1897,13 @@ void cdfDefGrid(stream_t *streamptr, int gridID, int gridindex)
           else
             {
               bool lx = false, ly = false;
-              if ( gridInqXsize(gridID) > 0 /*&& gridInqXvals(gridID, NULL) > 0*/ )
+              if ( gridInqXsize(gridID) /*&& gridInqXvals(gridID, NULL) */ )
                 {
                   cdfDefXaxis(streamptr, gridID, gridindex, 1);
                   lx = true;
                 }
 
-              if ( gridInqYsize(gridID) > 0 /*&& gridInqYvals(gridID, NULL) > 0*/ )
+              if ( gridInqYsize(gridID) /*&& gridInqYvals(gridID, NULL) */ )
                 {
                   cdfDefYaxis(streamptr, gridID, gridindex, 1);
                   ly = true;
@@ -1917,10 +1914,10 @@ void cdfDefGrid(stream_t *streamptr, int gridID, int gridindex)
         }
       else
         {
-          int ndims = !(gridtype == GRID_LONLAT && size == 1 && !gridInqHasDims(gridID));
+          const int ndims = !(gridtype == GRID_LONLAT && size == 1 && !gridInqHasDims(gridID));
 
-          if ( gridInqXsize(gridID) > 0 ) cdfDefXaxis(streamptr, gridID, gridindex, ndims);
-          if ( gridInqYsize(gridID) > 0 ) cdfDefYaxis(streamptr, gridID, gridindex, ndims);
+          if ( gridInqXsize(gridID) ) cdfDefXaxis(streamptr, gridID, gridindex, ndims);
+          if ( gridInqYsize(gridID) ) cdfDefYaxis(streamptr, gridID, gridindex, ndims);
 
           cdf_def_mapping(streamptr, gridID);
         }
@@ -1936,6 +1933,7 @@ void cdfDefGrid(stream_t *streamptr, int gridID, int gridindex)
   else if ( gridtype == GRID_GAUSSIAN_REDUCED )
     {
       cdfDefRgrid(streamptr, gridID, gridindex);
+      // if ( gridInqYsize(gridID) ) cdfDefYaxis(streamptr, gridID, gridindex, 1);
     }
   else if ( gridtype == GRID_SPECTRAL )
     {
@@ -1958,11 +1956,11 @@ void cdfDefGrid(stream_t *streamptr, int gridID, int gridindex)
       if ( (strlen = gridInqXIsc(gridID)) )
         cdfDefCharacter(streamptr, gridID, gridindex, 0, strlen);
       else
-        if ( gridInqXsize(gridID) > 0 ) cdfDefXaxis(streamptr, gridID, gridindex, 1);
+        if ( gridInqXsize(gridID) ) cdfDefXaxis(streamptr, gridID, gridindex, 1);
       if ( (strlen = gridInqYIsc(gridID)) )
         cdfDefCharacter(streamptr, gridID, gridindex, 1, strlen);
       else
-        if ( gridInqYsize(gridID) > 0 ) cdfDefYaxis(streamptr, gridID, gridindex, 1);
+        if ( gridInqYsize(gridID) ) cdfDefYaxis(streamptr, gridID, gridindex, 1);
     }
   else
     {
@@ -1980,13 +1978,12 @@ void cdfDefHistory(stream_t *streamptr, int size, const char *history)
 
 void cdfDefVars(stream_t *streamptr)
 {
-  int vlistID = streamptr->vlistID;
-  if ( vlistID == CDI_UNDEFID )
-    Error("Internal problem! vlist undefined for streamptr %p", streamptr);
+  const int vlistID = streamptr->vlistID;
+  if ( vlistID == CDI_UNDEFID ) Error("Internal problem! vlist undefined for streamptr %p", streamptr);
 
   if ( vlistHasTime(vlistID) ) cdfDefTime(streamptr);
 
-  int ngrids = vlistNgrids(vlistID);
+  const int ngrids = vlistNgrids(vlistID);
   if ( 2*ngrids > MAX_GRIDS_PS ) Error("Internal problem! Too many grids per stream (max=%d)\n", MAX_GRIDS_PS);
   for ( int index = 0; index < 2*ngrids; ++index )
     {
@@ -1997,7 +1994,7 @@ void cdfDefVars(stream_t *streamptr)
 
   for ( int index = 0; index < ngrids; ++index )
     {
-      int gridID = vlistGrid(vlistID, index);
+      const int gridID = vlistGrid(vlistID, index);
       cdfDefGrid(streamptr, gridID, index);
     }
   {
@@ -2009,10 +2006,10 @@ void cdfDefVars(stream_t *streamptr)
         if ( projID != CDI_UNDEFID ) cdfDefGrid(streamptr, projID, ++index);
       }
   }
-  int nzaxis = vlistNzaxis(vlistID);
+  const int nzaxis = vlistNzaxis(vlistID);
   for ( int index = 0; index < nzaxis; ++index )
     {
-      int zaxisID = vlistZaxis(vlistID, index);
+      const int zaxisID = vlistZaxis(vlistID, index);
       if ( streamptr->zaxisID[index] == CDI_UNDEFID ) cdfDefZaxis(streamptr, zaxisID);
     }
 
