@@ -11,6 +11,7 @@
 #include "file.h"
 #include "varscan.h"
 #include "datetime.h"
+#include "stream_scan.h"
 #include "stream_grb.h"
 #include "stream_cgribex.h"
 
@@ -239,8 +240,9 @@ int cgribexGetTsteptype(int timerange)
 }
 
 static
-void cgribexGetGrid(stream_t *streamptr, int *isec2, int *isec4, grid_t *grid, int iret)
+bool cgribexGetGrid(stream_t *streamptr, int *isec2, int *isec4, grid_t *grid, int iret)
 {
+  bool uvRelativeToGrid = false;
   bool compyinc = true;
   int gridtype = cgribexGetGridType(isec2);
   int projtype = (gridtype == GRID_PROJECTION && cgribexGetIsRotated(isec2)) ? CDI_PROJ_RLL : CDI_UNDEFID;
@@ -267,8 +269,7 @@ void cgribexGetGrid(stream_t *streamptr, int *isec2, int *isec4, grid_t *grid, i
   if ( gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || projtype == CDI_PROJ_RLL )
     {
       const bool ijDirectionIncrementGiven = gribbyte_get_bit(ISEC2_ResFlag, 1);
-      const bool uvRelativeToGrid = gribbyte_get_bit(ISEC2_ResFlag, 5);
-      if ( uvRelativeToGrid ) grid->uvRelativeToGrid = 1;
+      uvRelativeToGrid = gribbyte_get_bit(ISEC2_ResFlag, 5);
 
       const size_t nvalues = (size_t) ISEC4_NumValues;
       const size_t nlon = (size_t) ISEC2_NumLon;
@@ -351,8 +352,7 @@ void cgribexGetGrid(stream_t *streamptr, int *isec2, int *isec4, grid_t *grid, i
   else if ( gridtype == GRID_GAUSSIAN_REDUCED )
     {
       const bool ijDirectionIncrementGiven = gribbyte_get_bit(ISEC2_ResFlag, 1);
-      const bool uvRelativeToGrid = gribbyte_get_bit(ISEC2_ResFlag, 5);
-      if ( uvRelativeToGrid ) grid->uvRelativeToGrid = 1;
+      uvRelativeToGrid = gribbyte_get_bit(ISEC2_ResFlag, 5);
       grid->np      = ISEC2_NumPar;
       grid->size    = (size_t)ISEC4_NumValues;
       grid->rowlon  = ISEC2_RowLonPtr;
@@ -393,8 +393,7 @@ void cgribexGetGrid(stream_t *streamptr, int *isec2, int *isec4, grid_t *grid, i
     }
   else if ( projtype == CDI_PROJ_LCC )
     {
-      const bool uvRelativeToGrid = gribbyte_get_bit(ISEC2_ResFlag, 5);
-      if ( uvRelativeToGrid ) grid->uvRelativeToGrid = 1;
+      uvRelativeToGrid = gribbyte_get_bit(ISEC2_ResFlag, 5);
 
       const size_t nvalues = (size_t) ISEC4_NumValues;
       const size_t nlon = (size_t) ISEC2_NumLon;
@@ -442,6 +441,8 @@ void cgribexGetGrid(stream_t *streamptr, int *isec2, int *isec4, grid_t *grid, i
 
   grid->type = gridtype;
   grid->projtype = projtype;
+
+  return uvRelativeToGrid;
 }
 
 static
@@ -465,9 +466,8 @@ void cgribexAddRecord(stream_t *streamptr, cgribexrec_t *cgribexp, int param, si
   double *fsec3 = cgribexp->fsec3;
 
   int datatype = (ISEC4_NumBits > 0 && ISEC4_NumBits <= 32) ? ISEC4_NumBits : CDI_DATATYPE_PACK;
-
-  int varID;
-  int levelID = 0;
+  if ( datatype > 32 ) datatype = CDI_DATATYPE_PACK32;
+  if ( datatype <  0 ) datatype = CDI_DATATYPE_PACK;
 
   const int vlistID = streamptr->vlistID;
   const int tsID    = streamptr->curTsID;
@@ -491,7 +491,7 @@ void cgribexAddRecord(stream_t *streamptr, cgribexrec_t *cgribexp, int param, si
   record->tsteptype = (short)tsteptype;
 
   grid_t *gridptr = (grid_t*) Malloc(sizeof(*gridptr));
-  cgribexGetGrid(streamptr, isec2, isec4, gridptr, iret);
+  const bool uvRelativeToGrid = cgribexGetGrid(streamptr, isec2, isec4, gridptr, iret);
 
   struct addIfNewRes gridAdded = cdiVlistAddGridIfNew(vlistID, gridptr, 0);
   int gridID = gridAdded.Id;
@@ -513,9 +513,9 @@ void cgribexAddRecord(stream_t *streamptr, cgribexrec_t *cgribexp, int param, si
         }
       else if ( gridptr->projtype == CDI_PROJ_LCC )
         {
-          double a = 6367470., rf = 0;
-          bool earthIsOblate = gribbyte_get_bit(ISEC2_ResFlag, 2);
-          if ( earthIsOblate ) { a = 6378160.; rf = 297.0; }
+          const bool earthIsOblate = gribbyte_get_bit(ISEC2_ResFlag, 2);
+          const double a = earthIsOblate ? 6378160. : 6367470.;
+          const double rf = earthIsOblate ? 297.0 : 0;
           const double xval_0 = ISEC2_FirstLon * 0.001;
           const double yval_0 = ISEC2_FirstLat * 0.001;
           const double lon_0  = ISEC2_Lambert_Lov * 0.001;
@@ -551,18 +551,18 @@ void cgribexAddRecord(stream_t *streamptr, cgribexrec_t *cgribexp, int param, si
 
   const bool lbounds = cgribexGetZaxisHasBounds(leveltype);
 
-  if ( datatype > 32 ) datatype = CDI_DATATYPE_PACK32;
-  if ( datatype <  0 ) datatype = CDI_DATATYPE_PACK;
-
+  int varID = 0, levelID = 0;
   varAddRecord(recID, param, gridID, zaxistype, lbounds, level1, level2, 0, 0,
 	       datatype, &varID, &levelID, tsteptype, numavg, leveltype, -1,
-               NULL, NULL, NULL, NULL, NULL, NULL, 0);
+               NULL, NULL, NULL, NULL, NULL, NULL, NULL);
 
-  record->varID   = (short)varID;
+  record->varID = (short)varID;
   record->levelID = (short)levelID;
 
   varDefCompType(varID, comptype);
 
+  if ( uvRelativeToGrid ) varDefKeyInt(varID, CDI_KEY_UVRELATIVETOGRID, 1);
+
   if ( ISEC1_LocalFLag )
     {
       if      ( ISEC1_CenterID == 78  && isec1[36] == 253 ) // DWD local extension
@@ -583,9 +583,9 @@ void cgribexAddRecord(stream_t *streamptr, cgribexrec_t *cgribexp, int param, si
 
   if ( varInqInst(varID) == CDI_UNDEFID )
     {
-      const int center    = ISEC1_CenterID;
+      const int center = ISEC1_CenterID;
       const int subcenter = ISEC1_SubCenterID;
-      int instID    = institutInq(center, subcenter, NULL, NULL);
+      int instID = institutInq(center, subcenter, NULL, NULL);
       if ( instID == CDI_UNDEFID )
 	instID = institutDef(center, subcenter, NULL, NULL);
       varDefInst(varID, instID);
@@ -615,25 +615,22 @@ static
 void MCH_get_undef(int *isec1, double *undef_pds, double *undef_eps)
 {
   /* 2010-01-13: Oliver Fuhrer */
-  if ( ISEC1_CenterID == 215 ) {
-    if (isec1[34] != 0 && isec1[34] != 255) {
-      if (isec1[34] & 2) {
-        if (isec1[34] & 1) {
-          *undef_pds = -0.99*pow(10.0,-isec1[35]);
-        } else {
-          *undef_pds = +0.99*pow(10.0,-isec1[35]);
+  if ( ISEC1_CenterID == 215 )
+    {
+      if (isec1[34] != 0 && isec1[34] != 255)
+        {
+          if (isec1[34] & 2)
+            {
+              *undef_pds = ((isec1[34] & 1) ? -0.99 : +0.99) * pow(10.0,-isec1[35]);
+              *undef_eps = pow(10.0,-isec1[35]-1);
+            }
+          else
+            {
+              *undef_pds = ((isec1[34] & 1) ? -0.99 : +0.99) * pow(10.0,+isec1[35]);
+              *undef_eps = pow(10.0,isec1[35]-1);
+            }
         }
-        *undef_eps = pow(10.0,-isec1[35]-1);
-      } else {
-        if (isec1[34] & 1) {
-          *undef_pds = -0.99*pow(10.0,+isec1[35]);
-        } else {
-          *undef_pds = +0.99*pow(10.0,+isec1[35]);
-        }
-        *undef_eps = pow(10.0,isec1[35]-1);
-      }
     }
-  }
 }
 
 static
@@ -702,34 +699,6 @@ int cgribexVarCompare(compvar_t compVar, record_t record, int flag)
 #define gribWarning(text, nrecs, timestep, paramstr, level1, level2) \
             Warning("Record %2d (id=%s lev1=%d lev2=%d) timestep %d: %s", nrecs, paramstr, level1, level2, timestep, text)
 
-static inline
-void cgribexScanTsFixNtsteps(stream_t *streamptr, off_t recpos)
-{
-  if ( streamptr->ntsteps == -1 )
-    {
-      const int tsID = tstepsNewEntry(streamptr);
-      if ( tsID != streamptr->rtsteps )
-	Error("Internal error. tsID = %d", tsID);
-
-      streamptr->tsteps[tsID-1].next   = true;
-      streamptr->tsteps[tsID].position = recpos;
-    }
-}
-
-static inline
-void cgribexScanTsConstAdjust(stream_t *streamptr, taxis_t *taxis)
-{
-  const int vlistID = streamptr->vlistID;
-  if ( streamptr->ntsteps == 1 )
-    {
-      if ( taxis->vdate == 0 && taxis->vtime == 0 )
-	{
-	  streamptr->ntsteps = 0;
-	  for ( int varID = 0; varID < streamptr->nvars; varID++ )
-            vlistDefVarTimetype(vlistID, varID, TIME_CONSTANT);
-	}
-    }
-}
 
 static
 void cgribexSkipRecords(const int fileID)
@@ -753,14 +722,12 @@ int cgribexScanTimestep1(stream_t *streamptr)
   size_t buffersize = 0;
   int leveltype = 0, level1 = 0, level2 = 0, vdate = 0, vtime = 0;
   DateTime datetime, datetime0 = { LONG_MIN, LONG_MIN };
-  unsigned nrecords, recID;
+  unsigned recID;
   int nrecs_scanned = 0;
   bool warn_time = true;
   bool warn_numavg = true;
-  int taxisID = -1;
   int rdate = 0, rtime = 0, tunit = 0;
   bool fcast = false;
-  int vlistID;
   char paramstr[32];
 
   streamptr->curTsID = 0;
@@ -830,9 +797,10 @@ int cgribexScanTimestep1(stream_t *streamptr)
 	  datetime.date = vdate;
 	  datetime.time = vtime;
 	  compvar_t compVar = cgribexVarSet(param, level1, level2, leveltype, ISEC1_TimeRange);
+          record_t *records = streamptr->tsteps[tsID].records;
 	  for ( recID = 0; recID < nrecs; recID++ )
 	    {
-	      if ( cgribexVarCompare(compVar, streamptr->tsteps[0].records[recID], 0) == 0 ) break;
+	      if ( cgribexVarCompare(compVar, records[recID], 0) == 0 ) break;
 	    }
 
 	  if ( CDI_inventory_mode == 1 )
@@ -886,33 +854,22 @@ int cgribexScanTimestep1(stream_t *streamptr)
 
   taxis->unit = tunit;
   taxis->type = fcast ? TAXIS_RELATIVE : TAXIS_ABSOLUTE;
-  taxisID = taxisCreate(taxis->type);
+  int taxisID = taxisCreate(taxis->type);
   taxis->rdate = rdate;
   taxis->rtime = rtime;
-  taxis->vdate = (int)datetime0.date;
+  taxis->vdate = (int64_t)datetime0.date;
   taxis->vtime = (int)datetime0.time;
 
-  vlistID = streamptr->vlistID;
+  const int vlistID = streamptr->vlistID;
   vlistDefTaxis(vlistID, taxisID);
 
-  nrecords = (unsigned)streamptr->tsteps[0].nallrecs;
-  if ( nrecords < (unsigned)streamptr->tsteps[0].recordSize )
-    {
-      streamptr->tsteps[0].recordSize = (int)nrecords;
-      streamptr->tsteps[0].records =
-      (record_t *) Realloc(streamptr->tsteps[0].records, nrecords*sizeof(record_t));
-    }
-
-  streamptr->tsteps[0].recIDs = (int *) Malloc(nrecords*sizeof(int));
-  streamptr->tsteps[0].nrecs = (int)nrecords;
-  for ( recID = 0; recID < nrecords; recID++ )
-    streamptr->tsteps[0].recIDs[recID] = (int)recID;
+  streamScanResizeRecords1(streamptr);
 
   streamptr->record->buffer     = gribbuffer;
   streamptr->record->buffersize = buffersize;
 
-  cgribexScanTsFixNtsteps(streamptr, recpos);
-  cgribexScanTsConstAdjust(streamptr, taxis);
+  streamScanTsFixNtsteps(streamptr, recpos);
+  streamScanTimeConstAdjust(streamptr, taxis);
 
   return 0;
 }
@@ -920,12 +877,11 @@ int cgribexScanTimestep1(stream_t *streamptr)
 
 int cgribexScanTimestep2(stream_t * streamptr)
 {
-  int rstatus = 0;
   int lmv = 0, iret = 0;
   off_t recpos = 0;
   int leveltype = 0, level1 = 0, level2 = 0, vdate = 0, vtime = 0;
   DateTime datetime, datetime0 = { LONG_MIN, LONG_MIN };
-  int nrecs, recID;
+  int recID = 0;
   bool warn_numavg = true;
   char paramstr[32];
 
@@ -950,18 +906,9 @@ int cgribexScanTimestep2(stream_t * streamptr)
   fileSetPos(fileID, streamptr->tsteps[tsID].position, SEEK_SET);
 
   cdi_create_records(streamptr, tsID);
+  record_t *records = streamptr->tsteps[tsID].records;
 
-  const int nrecords = streamptr->tsteps[tsID].nallrecs;
-  if ( nrecords ) streamptr->tsteps[1].recIDs = (int *) Malloc((size_t)nrecords * sizeof(int));
-  streamptr->tsteps[1].nrecs = 0;
-  for ( recID = 0; recID < nrecords; recID++ )
-    streamptr->tsteps[1].recIDs[recID] = -1;
-
-  for ( recID = 0; recID < nrecords; recID++ )
-    {
-      streamptr->tsteps[tsID].records[recID].position =	streamptr->tsteps[0].records[recID].position;
-      streamptr->tsteps[tsID].records[recID].size     =	streamptr->tsteps[0].records[recID].size;
-    }
+  const int nrecords = streamScanInitRecords2(streamptr);
 
   int nrecs_scanned = nrecords;
   int rindex = 0;
@@ -970,7 +917,7 @@ int cgribexScanTimestep2(stream_t * streamptr)
       if ( rindex > nrecords ) break;
 
       size_t recsize = gribGetSize(fileID);
-      recpos  = fileGetPos(fileID);
+      recpos = fileGetPos(fileID);
       if ( recsize == 0 )
 	{
 	  streamptr->ntsteps = 2;
@@ -1037,7 +984,7 @@ int cgribexScanTimestep2(stream_t * streamptr)
 
       for ( recID = 0; recID < nrecords; recID++ )
 	{
-	  if ( cgribexVarCompare(compVar, streamptr->tsteps[tsID].records[recID], 0) == 0 ) break;
+	  if ( cgribexVarCompare(compVar, records[recID], 0) == 0 ) break;
 	}
 
       if ( recID == nrecords )
@@ -1048,19 +995,19 @@ int cgribexScanTimestep2(stream_t * streamptr)
 
       if ( CDI_inventory_mode == 1 )
 	{
-	  if ( streamptr->tsteps[tsID].records[recID].used )
+	  if ( records[recID].used )
 	    {
 	      break;
 	    }
 	  else
 	    {
-	      streamptr->tsteps[tsID].records[recID].used = true;
+	      records[recID].used = true;
 	      streamptr->tsteps[tsID].recIDs[rindex] = recID;
 	    }
 	}
       else
 	{
-	  if ( streamptr->tsteps[tsID].records[recID].used )
+	  if ( records[recID].used )
 	    {
 	      if ( datetimeCmp(datetime, datetime0) != 0 ) break;
 
@@ -1069,7 +1016,7 @@ int cgribexScanTimestep2(stream_t * streamptr)
 	    }
 	  else
 	    {
-	      streamptr->tsteps[tsID].records[recID].used = true;
+	      records[recID].used = true;
 	      streamptr->tsteps[tsID].recIDs[rindex] = recID;
 	    }
 	}
@@ -1077,18 +1024,17 @@ int cgribexScanTimestep2(stream_t * streamptr)
       if ( CDI_Debug )
 	Message("Read record %2d (id=%s lev1=%d lev2=%d) %8d %6d", nrecs_scanned, paramstr, level1, level2, vdate, vtime);
 
-      streamptr->tsteps[tsID].records[recID].size = recsize;
-
       if ( cgribexVarCompare(compVar, streamptr->tsteps[tsID].records[recID], 0) != 0 )
 	{
-	  Message("tsID = %d recID = %d param = %3d new %3d  level = %3d new %3d", tsID, recID,
-		  streamptr->tsteps[tsID].records[recID].param, param,
-		  streamptr->tsteps[tsID].records[recID].ilevel, level1);
+	  Message("tsID = %d recID = %d param = %3d new %3d  level = %3d new %3d",
+                  tsID, recID, records[recID].param, param, records[recID].ilevel, level1);
 	  return CDI_EUFSTRUCT;
 	}
 
-      streamptr->tsteps[1].records[recID].position = recpos;
-      const int varID = streamptr->tsteps[tsID].records[recID].varID;
+      records[recID].position = recpos;
+      records[recID].size = recsize;
+
+      const int varID = records[recID].varID;
       const int gridID = vlistInqVarGrid(vlistID, varID);
       if ( gridInqSize(gridID) == 1 && gridInqType(gridID) == GRID_LONLAT )
 	{
@@ -1103,13 +1049,12 @@ int cgribexScanTimestep2(stream_t * streamptr)
       rindex++;
     }
 
-  nrecs = 0;
+  int nrecs = 0;
   for ( recID = 0; recID < nrecords; recID++ )
     {
-      if ( ! streamptr->tsteps[tsID].records[recID].used )
+      if ( ! records[recID].used )
 	{
-	  const int varID = streamptr->tsteps[tsID].records[recID].varID;
-          vlistDefVarTimetype(vlistID, varID, TIME_CONSTANT);
+          vlistDefVarTimetype(vlistID, records[recID].varID, TIME_CONSTANT);
 	}
       else
 	{
@@ -1120,25 +1065,23 @@ int cgribexScanTimestep2(stream_t * streamptr)
 
   streamptr->rtsteps = 2;
 
-  cgribexScanTsFixNtsteps(streamptr, recpos);
+  streamScanTsFixNtsteps(streamptr, recpos);
 
   streamptr->record->buffer     = gribbuffer;
   streamptr->record->buffersize = buffersize;
 
-  return rstatus;
+  return 0;
 }
 
 
 int cgribexScanTimestep(stream_t * streamptr)
 {
-  int rstatus = 0;
   int lmv = 0, iret = 0;
   off_t recpos = 0;
   int leveltype = 0, level1 = 0, level2 = 0, vdate = 0, vtime = 0;
   DateTime datetime, datetime0 = { LONG_MIN, LONG_MIN };
-  int vrecID, recID;
+  int vrecID, recID = 0;
   bool warn_numavg = true;
-  int taxisID = -1;
   int nrecs = 0;
   char paramstr[32];
 
@@ -1154,15 +1097,11 @@ int cgribexScanTimestep(stream_t * streamptr)
       size_t buffersize = streamptr->record->buffersize;
 
       cdi_create_records(streamptr, tsID);
+      record_t *records = streamptr->tsteps[tsID].records;
 
-      nrecs = streamptr->tsteps[1].nrecs;
+      nrecs = streamScanInitRecords(streamptr, tsID);
 
-      streamptr->tsteps[tsID].nrecs = nrecs;
-      streamptr->tsteps[tsID].recIDs = (int *) Malloc((size_t)nrecs * sizeof (int));
-      for ( recID = 0; recID < nrecs; recID++ )
-	streamptr->tsteps[tsID].recIDs[recID] = streamptr->tsteps[1].recIDs[recID];
-
-      int fileID = streamptr->fileID;
+      const int fileID = streamptr->fileID;
 
       fileSetPos(fileID, streamptr->tsteps[tsID].position, SEEK_SET);
 
@@ -1172,7 +1111,7 @@ int cgribexScanTimestep(stream_t * streamptr)
 	{
 	  if ( rindex > nrecs ) break;
 
-	  size_t recsize = gribGetSize(fileID);
+	  const size_t recsize = gribGetSize(fileID);
 	  recpos  = fileGetPos(fileID);
 	  if ( recsize == 0 )
 	    {
@@ -1209,7 +1148,7 @@ int cgribexScanTimestep(stream_t * streamptr)
 	  if ( rindex == 0 )
 	    {
               const int vlistID = streamptr->vlistID;
-	      taxisID = vlistInqTaxis(vlistID);
+	      int taxisID = vlistInqTaxis(vlistID);
 	      if ( taxisInqType(taxisID) == TAXIS_RELATIVE )
 		{
 		  taxis->type  = TAXIS_RELATIVE;
@@ -1251,8 +1190,8 @@ int cgribexScanTimestep(stream_t * streamptr)
 
 	  for ( vrecID = 0; vrecID < nrecs; vrecID++ )
 	    {
-	      recID   = streamptr->tsteps[1].recIDs[vrecID];
-	      if ( cgribexVarCompare(compVar, streamptr->tsteps[tsID].records[recID], 0) == 0 ) break;
+	      recID = streamptr->tsteps[1].recIDs[vrecID];
+	      if ( cgribexVarCompare(compVar, records[recID], 0) == 0 ) break;
 	    }
 
 	  if ( vrecID == nrecs )
@@ -1267,12 +1206,12 @@ int cgribexScanTimestep(stream_t * streamptr)
 
 	  if ( CDI_inventory_mode == 1 )
 	    {
-	      streamptr->tsteps[tsID].records[recID].used = true;
+	      records[recID].used = true;
 	      streamptr->tsteps[tsID].recIDs[rindex] = recID;
 	    }
 	  else
 	    {
-	      if ( streamptr->tsteps[tsID].records[recID].used )
+	      if ( records[recID].used )
 		{
 		  char paramstr_[32];
 		  cdiParamToString(param, paramstr_, sizeof(paramstr_));
@@ -1286,7 +1225,7 @@ int cgribexScanTimestep(stream_t * streamptr)
 		}
 	      else
 		{
-		  streamptr->tsteps[tsID].records[recID].used = true;
+		  records[recID].used = true;
 		  streamptr->tsteps[tsID].recIDs[rindex] = recID;
 		}
 	    }
@@ -1294,31 +1233,30 @@ int cgribexScanTimestep(stream_t * streamptr)
 	  if ( CDI_Debug )
             Message("Read record %2d (id=%s lev1=%d lev2=%d) %8d %6d", nrecs_scanned, paramstr, level1, level2, vdate, vtime);
 
-	  if ( cgribexVarCompare(compVar, streamptr->tsteps[tsID].records[recID], 0) != 0 )
+	  if ( cgribexVarCompare(compVar, records[recID], 0) != 0 )
 	    {
-	      Message("tsID = %d recID = %d param = %3d new %3d  level = %3d new %3d", tsID, recID,
-		      streamptr->tsteps[tsID].records[recID].param, param,
-		      streamptr->tsteps[tsID].records[recID].ilevel, level1);
+	      Message("tsID = %d recID = %d param = %3d new %3d  level = %3d new %3d",
+                      tsID, recID, records[recID].param, param, records[recID].ilevel, level1);
 	      Error("Invalid, unsupported or inconsistent record structure");
 	    }
 
-	  streamptr->tsteps[tsID].records[recID].position = recpos;
-	  streamptr->tsteps[tsID].records[recID].size = recsize;
+	  records[recID].position = recpos;
+	  records[recID].size = recsize;
 
 	  rindex++;
 	}
 
       for ( vrecID = 0; vrecID < nrecs; vrecID++ )
 	{
-	  recID   = streamptr->tsteps[tsID].recIDs[vrecID];
-	  if ( ! streamptr->tsteps[tsID].records[recID].used ) break;
+	  recID = streamptr->tsteps[tsID].recIDs[vrecID];
+	  if ( ! records[recID].used ) break;
 	}
 
       if ( vrecID < nrecs )
 	{
-	  cdiParamToString(streamptr->tsteps[tsID].records[recID].param, paramstr, sizeof(paramstr));
+	  cdiParamToString(records[recID].param, paramstr, sizeof(paramstr));
 	  gribWarning("Paramameter not found!", nrecs_scanned, tsID+1, paramstr,
-                      streamptr->tsteps[tsID].records[recID].ilevel, streamptr->tsteps[tsID].records[recID].ilevel2);
+                      records[recID].ilevel, records[recID].ilevel2);
 	  return CDI_EUFSTRUCT;
 	}
 
@@ -1347,9 +1285,7 @@ int cgribexScanTimestep(stream_t * streamptr)
       streamptr->ntsteps = tsID;
     }
 
-  rstatus = (int)streamptr->ntsteps;
-
-  return rstatus;
+  return streamptr->ntsteps;
 }
 
 #ifdef gribWarning
@@ -1361,7 +1297,8 @@ int cgribexDecode(int memtype, void *cgribex, void *gribbuffer, size_t gribsize,
 {
   int status = 0;
 
-  cgribexrec_t *cgribexp = (cgribexrec_t *) cgribex;
+  bool lalloc = cgribex == NULL;
+  cgribexrec_t *cgribexp = (cgribexrec_t *) (lalloc ? cgribexNew() : cgribex);
 
   int *isec0 = cgribexp->sec0;
   int *isec1 = cgribexp->sec1;
@@ -1416,6 +1353,8 @@ int cgribexDecode(int memtype, void *cgribex, void *gribbuffer, size_t gribsize,
         }
     }
 
+  if (lalloc) cgribexDelete(cgribexp);
+
   return status;
 }
 
@@ -1645,7 +1584,7 @@ void cgribexDefTime(int *isec1, int vdate, int vtime, int tsteptype, int numavg,
 }
 
 static
-void cgribexDefGridRegular(int *isec2, double *fsec2, int gridID, int gridtype, bool gridIsRotated, bool gridIsCurvilinear)
+void cgribexDefGridRegular(int *isec2, double *fsec2, int gridID, int gridtype, bool gridIsRotated, bool gridIsCurvilinear, int uvRelativeToGrid)
 {
   if ( gridtype == GRID_GAUSSIAN || gridtype == GRID_GAUSSIAN_REDUCED )
     ISEC2_GridType = GRIB1_GTYPE_GAUSSIAN;
@@ -1720,8 +1659,8 @@ void cgribexDefGridRegular(int *isec2, double *fsec2, int gridID, int gridtype,
     if ( ISEC2_LonIncr == 0 && ISEC2_LatIncr != 0 ) ISEC2_LonIncr = ISEC2_LatIncr;
 
   ISEC2_ResFlag = 0;
-  if ( ISEC2_LatIncr && ISEC2_LonIncr )   gribbyte_set_bit(&ISEC2_ResFlag, 1);
-  if ( gridInqUvRelativeToGrid(gridID) )  gribbyte_set_bit(&ISEC2_ResFlag, 5);
+  if ( ISEC2_LatIncr && ISEC2_LonIncr ) gribbyte_set_bit(&ISEC2_ResFlag, 1);
+  if ( uvRelativeToGrid > 0 ) gribbyte_set_bit(&ISEC2_ResFlag, 5);
 
   if ( gridIsRotated )
     {
@@ -1740,7 +1679,7 @@ void cgribexDefGridRegular(int *isec2, double *fsec2, int gridID, int gridtype,
 }
 
 static
-void cgribexDefGridLambert(int *isec2, int gridID)
+void cgribexDefGridLambert(int *isec2, int gridID, int uvRelativeToGrid)
 {
   const int xsize = (int)gridInqXsize(gridID);
   const int ysize = (int)gridInqYsize(gridID);
@@ -1772,8 +1711,8 @@ void cgribexDefGridLambert(int *isec2, int gridID)
   const bool earthIsOblate = (IS_EQUAL(a, 6378160.) && IS_EQUAL(rf, 297.));
   ISEC2_ResFlag = 0;
   if ( ISEC2_Lambert_dx && ISEC2_Lambert_dy ) gribbyte_set_bit(&ISEC2_ResFlag, 1);
-  if ( earthIsOblate )                        gribbyte_set_bit(&ISEC2_ResFlag, 2);
-  if ( gridInqUvRelativeToGrid(gridID) )      gribbyte_set_bit(&ISEC2_ResFlag, 5);
+  if ( earthIsOblate ) gribbyte_set_bit(&ISEC2_ResFlag, 2);
+  if ( uvRelativeToGrid > 0 ) gribbyte_set_bit(&ISEC2_ResFlag, 5);
 
   ISEC2_ScanFlag = 0;
   gribbyte_set_bit(&ISEC2_ScanFlag, 2); // South -> North
@@ -1822,7 +1761,7 @@ void cgribexDefGridGME(int *isec2, int gridID)
 }
 
 static
-void cgribexDefGrid(int *isec1, int *isec2, double *fsec2, int *isec4, int gridID)
+void cgribexDefGrid(int *isec1, int *isec2, double *fsec2, int *isec4, int gridID, int uvRelativeToGrid)
 {
   memset(isec2, 0, 16*sizeof(int));
   ISEC1_Sec2Or3Flag = 128;
@@ -1842,12 +1781,12 @@ void cgribexDefGrid(int *isec1, int *isec2, double *fsec2, int *isec4, int gridI
     case GRID_GAUSSIAN_REDUCED:
     case GRID_TRAJECTORY:
       {
-        cgribexDefGridRegular(isec2, fsec2, gridID, gridtype, gridIsRotated, gridIsCurvilinear);
+        cgribexDefGridRegular(isec2, fsec2, gridID, gridtype, gridIsRotated, gridIsCurvilinear, uvRelativeToGrid);
 	break;
       }
     case CDI_PROJ_LCC:
       {
-        cgribexDefGridLambert(isec2, gridID);
+        cgribexDefGridLambert(isec2, gridID, uvRelativeToGrid);
 	break;
       }
     case GRID_SPECTRAL:
@@ -1875,17 +1814,6 @@ void cgribexDefGrid(int *isec1, int *isec2, double *fsec2, int *isec4, int gridI
 	break;
       }
     }
-
-  if ( cdiGribChangeModeUvRelativeToGrid.active )
-    {
-      // this will overrule/change the UvRelativeToGrid flag;
-      // typically when the wind is rotated with respect to north pole
-      const bool uvRelativeToGrid = gribbyte_get_bit(ISEC2_ResFlag, 5);
-      if      ( uvRelativeToGrid && !cdiGribChangeModeUvRelativeToGrid.mode )
-        gribbyte_clear_bit(&ISEC2_ResFlag, 5);
-      else if ( !uvRelativeToGrid && cdiGribChangeModeUvRelativeToGrid.mode )
-        gribbyte_set_bit(&ISEC2_ResFlag, 5);
-    }
 }
 
 static
@@ -2099,7 +2027,7 @@ size_t cgribexEncode(int memtype, int varID, int levelID, int vlistID, int gridI
 {
   cgribexrec_t *cgribexp = (cgribexrec_t *)cgribexNew();
 
-  size_t sec2len = 1024 + 2*gridInqYsize(gridID); // Gaussian reduced grid
+  const size_t sec2len = 1024 + 2*gridInqYsize(gridID); // Gaussian reduced grid
   if (sec2len > cgribexp->sec2len)
     {
       cgribexp->sec2len = sec2len;
@@ -2132,9 +2060,12 @@ size_t cgribexEncode(int memtype, int varID, int levelID, int vlistID, int gridI
 
   const int datatype = vlistInqVarDatatype(vlistID, varID);
 
+  int uvRelativeToGrid = -1;
+  cdiInqKeyInt(vlistID, varID, CDI_KEY_UVRELATIVETOGRID, &uvRelativeToGrid);
+
   cgribexDefParam(isec1, param);
   cgribexDefTime(isec1, vdate, vtime, tsteptype, numavg, vlistInqTaxis(vlistID));
-  cgribexDefGrid(isec1, isec2, fsec2, isec4, gridID);
+  cgribexDefGrid(isec1, isec2, fsec2, isec4, gridID, uvRelativeToGrid);
   cgribexDefLevel(isec1, isec2, fsec2, zaxisID, levelID);
 
   cgribexDefEnsembleVar(isec1, vlistID, varID);
