@@ -8,6 +8,7 @@
 #include "file.h"
 #include "cdi.h"
 #include "cdi_int.h"
+#include "stream_scan.h"
 #include "stream_ext.h"
 #include "varscan.h"
 #include "datetime.h"
@@ -17,11 +18,6 @@
 
 #ifdef HAVE_LIBEXTRA
 
-typedef struct {
-  int param;
-  int level;
-} extcompvar_t;
-
 static
 int extInqDatatype(int prec, int number)
 {
@@ -141,39 +137,39 @@ void extCopyRecord(stream_t *streamptr2, stream_t *streamptr1)
 
 void extDefRecord(stream_t *streamptr)
 {
+  Record *record = streamptr->record;
+
   int pdis, pcat, pnum;
-  cdiDecodeParam(streamptr->record->param, &pnum, &pcat, &pdis);
+  cdiDecodeParam(record->param, &pnum, &pcat, &pdis);
 
   int header[4];
-  header[0] = streamptr->record->date;
+  header[0] = record->date;
   header[1] = pnum;
-  header[2] = streamptr->record->level;
-  int gridID = streamptr->record->gridID;
+  header[2] = record->level;
+  int gridID = record->gridID;
   cdi_check_gridsize_int_limit("EXTRA", gridInqSize(gridID));
   header[3] = (int) gridInqSize(gridID);
 
-  extrec_t *extp = (extrec_t*) streamptr->record->exsep;
-  extDefDatatype(streamptr->record->prec, &extp->prec, &extp->number);
+  extrec_t *extp = (extrec_t*) record->exsep;
+  extDefDatatype(record->prec, &extp->prec, &extp->number);
   extDefHeader(extp, header);
 }
 
 
 void extWriteRecord(stream_t *streamptr, const double *data)
 {
-  int fileID = streamptr->fileID;
   void *extp = streamptr->record->exsep;
-
   extDefDataDP(extp, data);
-  extWrite(fileID, extp);
+  extWrite(streamptr->fileID, extp);
 }
 
 static
 void extAddRecord(stream_t *streamptr, int param, int level, size_t xysize,
 		  size_t recsize, off_t position, int prec, int number)
 {
-  int vlistID = streamptr->vlistID;
-  int tsID    = streamptr->curTsID;
-  int recID   = recordNewEntry(streamptr, tsID);
+  const int vlistID = streamptr->vlistID;
+  const int tsID    = streamptr->curTsID;
+  const int recID   = recordNewEntry(streamptr, tsID);
   record_t *record = &streamptr->tsteps[tsID].records[recID];
 
   record->size     = recsize;
@@ -187,18 +183,17 @@ void extAddRecord(stream_t *streamptr, int param, int level, size_t xysize,
   grid->x.size = xysize;
   grid->y.size = 0;
   struct addIfNewRes gridAdded = cdiVlistAddGridIfNew(vlistID, grid, 0);
-  int gridID = gridAdded.Id;
+  const int gridID = gridAdded.Id;
   if (!gridAdded.isNew) Free(grid);
-  /*
-  if ( level == 0 ) leveltype = ZAXIS_SURFACE;
-  else              leveltype = ZAXIS_GENERIC;
-  */
-  int leveltype = ZAXIS_GENERIC;
+
+  const int leveltype = ZAXIS_GENERIC;
+  const int datatype = extInqDatatype(prec, number);
 
   int varID, levelID = 0;
   varAddRecord(recID, param, gridID, leveltype, 0, level, 0, 0, 0,
-               extInqDatatype(prec, number), &varID, &levelID, TSTEP_INSTANT, 0, 0, -1,
-               NULL, NULL, NULL, NULL, NULL, NULL, 0);
+               datatype, &varID, &levelID, TSTEP_INSTANT, 0, 0, -1,
+               NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+
   record->varID   = (short)varID;
   record->levelID = (short)levelID;
 
@@ -214,44 +209,37 @@ void extScanTimestep1(stream_t *streamptr)
 {
   int header[4];
   DateTime datetime0 = { LONG_MIN, LONG_MIN };
-  int varID;
   off_t recpos = 0;
-  long recsize;
-  int recID;
-  extcompvar_t compVar, compVar0;
   extrec_t *extp = (extrec_t*) streamptr->record->exsep;
 
   streamptr->curTsID = 0;
 
   int tsID  = tstepsNewEntry(streamptr);
+  if ( tsID != 0 ) Error("Internal problem! tstepsNewEntry returns %d", tsID);
   taxis_t *taxis = &streamptr->tsteps[tsID].taxis;
 
-  if ( tsID != 0 )
-    Error("Internal problem! tstepsNewEntry returns %d", tsID);
-
-  int fileID = streamptr->fileID;
+  const int fileID = streamptr->fileID;
 
   int nrecs = 0;
   while ( true )
     {
       recpos = fileGetPos(fileID);
-      int status = extRead(fileID, extp);
-      if ( status != 0 )
+      if ( extRead(fileID, extp) != 0 )
 	{
 	  streamptr->ntsteps = 1;
 	  break;
 	}
-      recsize = fileGetPos(fileID) - recpos;
+
+      const size_t recsize = (size_t)(fileGetPos(fileID) - recpos);
 
       extInqHeader(extp, header);
 
-      int vdate   = header[0];
-      int vtime   = 0;
-      int rcode   = header[1];
-      int rlevel  = header[2];
-      int rxysize = header[3];
-
-      int param = cdiEncodeParam(rcode, 255, 255);
+      const int vdate   = header[0];
+      const int vtime   = 0;
+      const int rcode   = header[1];
+      const int rlevel  = header[2];
+      const int rxysize = header[3];
+      const int param   = cdiEncodeParam(rcode, 255, 255);
 
       if ( nrecs == 0 )
 	{
@@ -260,16 +248,11 @@ void extScanTimestep1(stream_t *streamptr)
 	}
       else
 	{
-	  compVar.param = param;
-          compVar.level = rlevel;
-	  for ( recID = 0; recID < nrecs; recID++ )
-	    {
-	      compVar0.param  = streamptr->tsteps[0].records[recID].param;
-	      compVar0.level = streamptr->tsteps[0].records[recID].ilevel;
+          record_t *records = streamptr->tsteps[tsID].records;
+	  for ( int recID = 0; recID < nrecs; recID++ )
+            if ( param == records[recID].param && rlevel == records[recID].ilevel )
+              goto tstepScanLoopFinished;
 
-	      if ( memcmp(&compVar0, &compVar, sizeof(extcompvar_t)) == 0 ) break;
-	    }
-	  if ( recID < nrecs ) break;
 	  DateTime datetime = { .date = vdate, .time = vtime};
 	  if ( datetimeCmp(datetime, datetime0) )
 	    Warning("Inconsistent verification time for code %d level %d", rcode, rlevel);
@@ -280,118 +263,74 @@ void extScanTimestep1(stream_t *streamptr)
       if ( CDI_Debug )
 	Message("%4d%8d%4d%8d%8d%6d", nrecs, (int)recpos, rcode, rlevel, vdate, vtime);
 
-      extAddRecord(streamptr, param, rlevel, rxysize, (size_t)recsize, recpos, extp->prec, extp->number);
+      extAddRecord(streamptr, param, rlevel, rxysize, recsize, recpos, extp->prec, extp->number);
     }
 
+  tstepScanLoopFinished:
   streamptr->rtsteps = 1;
 
   cdi_generate_vars(streamptr);
 
-  int taxisID = taxisCreate(TAXIS_ABSOLUTE);
+  const int taxisID = taxisCreate(TAXIS_ABSOLUTE);
   taxis->type  = TAXIS_ABSOLUTE;
   taxis->vdate = (int)datetime0.date;
   taxis->vtime = (int)datetime0.time;
   taxis->rdate = taxis->vdate;
   taxis->rtime = taxis->vtime;
 
-  int vlistID = streamptr->vlistID;
+  const int vlistID = streamptr->vlistID;
   vlistDefTaxis(vlistID, taxisID);
 
   vlist_check_contents(vlistID);
 
-  int nrecords = streamptr->tsteps[0].nallrecs;
-  if ( nrecords < streamptr->tsteps[0].recordSize )
-    {
-      streamptr->tsteps[0].recordSize = nrecords;
-      streamptr->tsteps[0].records =
-        (record_t *) Realloc(streamptr->tsteps[0].records, (size_t)nrecords * sizeof (record_t));
-    }
+  streamScanResizeRecords1(streamptr);
 
-  streamptr->tsteps[0].recIDs = (int *) Malloc((size_t)nrecords * sizeof (int));
-  streamptr->tsteps[0].nrecs = nrecords;
-  for ( recID = 0; recID < nrecords; recID++ )
-    streamptr->tsteps[0].recIDs[recID] = recID;
-
-  if ( streamptr->ntsteps == -1 )
-    {
-      tsID = tstepsNewEntry(streamptr);
-      if ( tsID != streamptr->rtsteps )
-	Error("Internal error. tsID = %d", tsID);
-
-      streamptr->tsteps[tsID-1].next   = true;
-      streamptr->tsteps[tsID].position = recpos;
-    }
-
-  if ( streamptr->ntsteps == 1 )
-    {
-      if ( taxis->vdate == 0 && taxis->vtime == 0 )
-	{
-	  streamptr->ntsteps = 0;
-	  for ( varID = 0; varID < streamptr->nvars; varID++ )
-            vlistDefVarTimetype(vlistID, varID, TIME_CONSTANT);
-	}
-    }
+  streamScanTsFixNtsteps(streamptr, recpos);
+  streamScanTimeConstAdjust(streamptr, taxis);
 }
 
 static
 int extScanTimestep2(stream_t *streamptr)
 {
   int header[4];
-  int varID;
   off_t recpos = 0;
-  extcompvar_t compVar, compVar0;
   void *extp = streamptr->record->exsep;
 
   streamptr->curTsID = 1;
 
-  int fileID  = streamptr->fileID;
-  int vlistID = streamptr->vlistID;
+  const int fileID  = streamptr->fileID;
+  const int vlistID = streamptr->vlistID;
 
   int tsID = streamptr->rtsteps;
-  if ( tsID != 1 )
-    Error("Internal problem! unexpected timestep %d", tsID+1);
+  if ( tsID != 1 ) Error("Internal problem! unexpected timestep %d", tsID+1);
 
   taxis_t *taxis = &streamptr->tsteps[tsID].taxis;
 
   fileSetPos(fileID, streamptr->tsteps[tsID].position, SEEK_SET);
 
   cdi_create_records(streamptr, tsID);
+  record_t *records = streamptr->tsteps[tsID].records;
 
-  int nrecords = streamptr->tsteps[0].nallrecs;
-  streamptr->tsteps[1].recIDs = (int *) Malloc((size_t)nrecords * sizeof (int));
-  streamptr->tsteps[1].nrecs = 0;
-  for ( int recID = 0; recID < nrecords; recID++ )
-    streamptr->tsteps[1].recIDs[recID] = -1;
-
-  for ( int recID = 0; recID < nrecords; recID++ )
-    {
-      varID = streamptr->tsteps[0].records[recID].varID;
-      streamptr->tsteps[tsID].records[recID].position =
-	streamptr->tsteps[0].records[recID].position;
-      streamptr->tsteps[tsID].records[recID].size     =
-	streamptr->tsteps[0].records[recID].size;
-    }
+  const int nrecords = streamScanInitRecords2(streamptr);
 
   for ( int rindex = 0; rindex <= nrecords; rindex++ )
     {
       recpos = fileGetPos(fileID);
-      int status = extRead(fileID, extp);
-      if ( status != 0 )
+      if ( extRead(fileID, extp) != 0 )
 	{
 	  streamptr->ntsteps = 2;
 	  break;
 	}
-      size_t recsize = (size_t)(fileGetPos(fileID) - recpos);
+
+      const size_t recsize = (size_t)(fileGetPos(fileID) - recpos);
 
       extInqHeader(extp, header);
 
-      int vdate  = header[0];
-      int vtime  = 0;
-      int rcode  = header[1];
-      int rlevel = header[2];
-      // rxysize = header[3];
-
-      int param = cdiEncodeParam(rcode, 255, 255);
+      const int vdate  = header[0];
+      const int vtime  = 0;
+      const int rcode  = header[1];
+      const int rlevel = header[2];
+      const int param  = cdiEncodeParam(rcode, 255, 255);
 
       if ( rindex == 0 )
 	{
@@ -400,24 +339,19 @@ int extScanTimestep2(stream_t *streamptr)
 	  taxis->vtime = vtime;
 	}
 
-      compVar.param = param;
-      compVar.level = rlevel;
       bool nextstep = false;
       int recID;
       for ( recID = 0; recID < nrecords; recID++ )
 	{
-	  compVar0.param = streamptr->tsteps[tsID].records[recID].param;
-	  compVar0.level = streamptr->tsteps[tsID].records[recID].ilevel;
-
-	  if ( memcmp(&compVar0, &compVar, sizeof(extcompvar_t)) == 0 )
+	  if ( param == records[recID].param && rlevel == records[recID].ilevel )
 	    {
-	      if ( streamptr->tsteps[tsID].records[recID].used )
+	      if ( records[recID].used )
 		{
 		  nextstep = true;
 		}
 	      else
 		{
-		  streamptr->tsteps[tsID].records[recID].used = true;
+		  records[recID].used = true;
 		  streamptr->tsteps[tsID].recIDs[rindex] = recID;
 		}
 	      break;
@@ -434,30 +368,23 @@ int extScanTimestep2(stream_t *streamptr)
       if ( CDI_Debug )
 	Message("%4d%8d%4d%8d%8d%6d", rindex+1, (int)recpos, rcode, rlevel, vdate, vtime);
 
-      streamptr->tsteps[tsID].records[recID].size = recsize;
-
-      compVar0.param  = streamptr->tsteps[tsID].records[recID].param;
-      compVar0.level = streamptr->tsteps[tsID].records[recID].ilevel;
-
-      if ( memcmp(&compVar0, &compVar, sizeof(extcompvar_t)) != 0 )
+      if ( param != records[recID].param || rlevel != records[recID].ilevel )
 	{
 	  Message("tsID = %d recID = %d param = %3d new %3d  level = %3d new %3d",
-		  tsID, recID,
-		  streamptr->tsteps[tsID].records[recID].param, param,
-		  streamptr->tsteps[tsID].records[recID].ilevel, rlevel);
+		  tsID, recID, records[recID].param, param, records[recID].ilevel, rlevel);
 	  return CDI_EUFSTRUCT;
 	}
 
-      streamptr->tsteps[1].records[recID].position = recpos;
+      records[recID].position = recpos;
+      records[recID].size = recsize;
     }
 
   int nrecs = 0;
   for ( int recID = 0; recID < nrecords; recID++ )
     {
-      if ( ! streamptr->tsteps[tsID].records[recID].used )
+      if ( ! records[recID].used )
 	{
-	  varID = streamptr->tsteps[tsID].records[recID].varID;
-          vlistDefVarTimetype(vlistID, varID, TIME_CONSTANT);
+          vlistDefVarTimetype(vlistID, records[recID].varID, TIME_CONSTANT);
 	}
       else
 	{
@@ -468,15 +395,7 @@ int extScanTimestep2(stream_t *streamptr)
 
   streamptr->rtsteps = 2;
 
-  if ( streamptr->ntsteps == -1 )
-    {
-      tsID = tstepsNewEntry(streamptr);
-      if ( tsID != streamptr->rtsteps )
-	Error("Internal error. tsID = %d", tsID);
-
-      streamptr->tsteps[tsID-1].next   = true;
-      streamptr->tsteps[tsID].position = recpos;
-    }
+  streamScanTsFixNtsteps(streamptr, recpos);
 
   return 0;
 }
@@ -488,11 +407,9 @@ int extInqContents(stream_t *streamptr)
 
   extScanTimestep1(streamptr);
 
-  int status = 0;
-  if ( streamptr->ntsteps == -1 ) status = extScanTimestep2(streamptr);
+  const int status = (streamptr->ntsteps == -1) ? extScanTimestep2(streamptr) : 0;
 
-  int fileID = streamptr->fileID;
-  fileSetPos(fileID, 0, SEEK_SET);
+  fileSetPos(streamptr->fileID, 0, SEEK_SET);
 
   return status;
 }
@@ -502,62 +419,45 @@ long extScanTimestep(stream_t *streamptr)
 {
   int header[4];
   off_t recpos = 0;
-  int recID;
   int nrecs = 0;
-  extcompvar_t compVar, compVar0;
   void *extp = streamptr->record->exsep;
-  /*
-  if ( CDI_Debug )
-    {
-      Message("streamID = %d", streamptr->self);
-      Message("cts = %d", streamptr->curTsID);
-      Message("rts = %d", streamptr->rtsteps);
-      Message("nts = %d", streamptr->ntsteps);
-    }
-  */
 
-  int tsID  = streamptr->rtsteps;
+  int tsID = streamptr->rtsteps;
   taxis_t *taxis = &streamptr->tsteps[tsID].taxis;
 
   if ( streamptr->tsteps[tsID].recordSize == 0 )
     {
       cdi_create_records(streamptr, tsID);
+      record_t *records = streamptr->tsteps[tsID].records;
 
-      nrecs = streamptr->tsteps[1].nrecs;
+      nrecs = streamScanInitRecords(streamptr, tsID);
 
-      streamptr->tsteps[tsID].nrecs = nrecs;
-      streamptr->tsteps[tsID].recIDs = (int *) Malloc((size_t)nrecs * sizeof (int));
-      for ( recID = 0; recID < nrecs; recID++ )
-	streamptr->tsteps[tsID].recIDs[recID] = streamptr->tsteps[1].recIDs[recID];
-
-      int fileID = streamptr->fileID;
+      const int fileID = streamptr->fileID;
 
       fileSetPos(fileID, streamptr->tsteps[tsID].position, SEEK_SET);
 
       for ( int rindex = 0; rindex <= nrecs; rindex++ )
 	{
 	  recpos = fileGetPos(fileID);
-	  int status = extRead(fileID, extp);
-	  if ( status != 0 )
+	  if ( extRead(fileID, extp) != 0 )
 	    {
 	      streamptr->ntsteps = streamptr->rtsteps + 1;
 	      break;
 	    }
-	  size_t recsize = (size_t)(fileGetPos(fileID) - recpos);
+
+	  const size_t recsize = (size_t)(fileGetPos(fileID) - recpos);
 
 	  extInqHeader(extp, header);
 
-	  int vdate  = header[0];
-	  int vtime  = 0;
-	  int rcode  = header[1];
-	  int rlevel = header[2];
-	  // rxysize = header[3];
-
-	  int param = cdiEncodeParam(rcode, 255, 255);
+	  const int vdate  = header[0];
+	  const int vtime  = 0;
+	  const int rcode  = header[1];
+	  const int rlevel = header[2];
+	  const int param  = cdiEncodeParam(rcode, 255, 255);
 
 	  // if ( rindex == nrecs ) break; gcc-4.5 internal compiler error
 	  if ( rindex == nrecs ) continue;
-	  recID = streamptr->tsteps[tsID].recIDs[rindex];
+	  const int recID = streamptr->tsteps[tsID].recIDs[rindex];
 
 	  if ( rindex == 0 )
 	    {
@@ -566,22 +466,15 @@ long extScanTimestep(stream_t *streamptr)
 	      taxis->vtime = vtime;
 	    }
 
-	  compVar.param  = param;
-          compVar.level  = rlevel;
-	  compVar0.param = streamptr->tsteps[tsID].records[recID].param;
-	  compVar0.level = streamptr->tsteps[tsID].records[recID].ilevel;
-
-	  if ( memcmp(&compVar0, &compVar, sizeof(extcompvar_t)) != 0 )
+          if ( param != records[recID].param || rlevel != records[recID].ilevel )
 	    {
 	      Message("tsID = %d recID = %d param = %3d new %3d  level = %3d new %3d",
-		      tsID, recID,
-		      streamptr->tsteps[tsID].records[recID].param, param,
-		      streamptr->tsteps[tsID].records[recID].ilevel, rlevel);
+		      tsID, recID, records[recID].param, param, records[recID].ilevel, rlevel);
 	      Error("Invalid, unsupported or inconsistent record structure!");
 	    }
 
-	  streamptr->tsteps[tsID].records[recID].position = recpos;
-	  streamptr->tsteps[tsID].records[recID].size = recsize;
+	  records[recID].position = recpos;
+	  records[recID].size = recsize;
 
 	  if ( CDI_Debug )
 	    Message("%4d%8d%4d%8d%8d%6d", rindex, (int)recpos, rcode, rlevel, vdate, vtime);
@@ -647,7 +540,7 @@ void extReadVarSliceDP(stream_t *streamptr, int varID, int levID, double *data,
   /* NOTE: tiles are not supported here! */
   double missval = vlistInqVarMissval(vlistID, varID);
   size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID, varID));
-  int tsid     = streamptr->curTsID;
+  int tsid = streamptr->curTsID;
 
   off_t currentfilepos = fileGetPos(fileID);
 
