@@ -13,18 +13,13 @@
 #include "varscan.h"
 #include "datetime.h"
 #include "ieg.h"
+#include "stream_scan.h"
 #include "stream_ieg.h"
 #include "exse.h"
 
 
 #ifdef HAVE_LIBIEG
 
-typedef struct {
-  int param;
-  int level;
-} iegcompvar_t;
-
-
 static
 int iegInqDatatype(int prec)
 {
@@ -57,8 +52,7 @@ void iegReadRecord(stream_t *streamptr, double *data, size_t *nmiss)
 
   void *iegp = streamptr->record->exsep;
   int status = iegRead(fileID, iegp);
-  if ( status != 0 )
-    Error("Could not read IEG record!");
+  if ( status != 0 ) Error("Could not read IEG record!");
 
   iegInqDataDP(iegp, data);
 
@@ -448,7 +442,6 @@ void iegCopyRecord(stream_t *streamptr2, stream_t *streamptr1)
 void iegDefRecord(stream_t *streamptr)
 {
   Record *record = streamptr->record;
-  iegrec_t *iegp = (iegrec_t*) record->exsep;
 
   int vlistID = streamptr->vlistID;
   int byteorder = streamptr->byteorder;
@@ -460,6 +453,7 @@ void iegDefRecord(stream_t *streamptr)
   int gridID  = vlistInqVarGrid(vlistID, varID);
   int zaxisID = vlistInqVarZaxis(vlistID, varID);
 
+  iegrec_t *iegp = (iegrec_t*) record->exsep;
   iegInitMem(iegp);
   for ( int i = 0; i < 37; i++ ) iegp->ipdb[i] = -1;
 
@@ -503,9 +497,9 @@ static
 void iegAddRecord(stream_t *streamptr, int param, int *pdb, int *gdb, double *vct,
 		  size_t recsize, off_t position, int prec)
 {
-  int vlistID = streamptr->vlistID;
-  int tsID    = streamptr->curTsID;
-  int recID   = recordNewEntry(streamptr, tsID);
+  const int vlistID = streamptr->vlistID;
+  const int tsID    = streamptr->curTsID;
+  const int recID   = recordNewEntry(streamptr, tsID);
   record_t *record = &streamptr->tsteps[tsID].records[recID];
 
   int level1, level2;
@@ -535,8 +529,8 @@ void iegAddRecord(stream_t *streamptr, int param, int *pdb, int *gdb, double *vc
   grid_t *grid = (grid_t *)Malloc(sizeof (*grid));
   grid_init(grid);
   cdiGridTypeInit(grid, gridtype, IEG_G_NumLon(gdb)*IEG_G_NumLat(gdb));
-  size_t xsize = (size_t)IEG_G_NumLon(gdb);
-  size_t ysize = (size_t)IEG_G_NumLat(gdb);
+  const size_t xsize = (size_t)IEG_G_NumLon(gdb);
+  const size_t ysize = (size_t)IEG_G_NumLat(gdb);
   grid->x.size = xsize;
   grid->y.size = ysize;
   grid->x.inc  = 0;
@@ -545,7 +539,7 @@ void iegAddRecord(stream_t *streamptr, int param, int *pdb, int *gdb, double *vc
 
   int iresfac = IEG_G_ResFac(gdb);
   if ( iresfac == 0 ) iresfac = 1000;
-  double resfac = 1./(double) iresfac;
+  const double resfac = 1./(double) iresfac;
 
   /* if ( IEG_G_FirstLon != 0 || IEG_G_LastLon != 0 ) */
   {
@@ -559,7 +553,7 @@ void iegAddRecord(stream_t *streamptr, int param, int *pdb, int *gdb, double *vc
 	/* correct xinc if necessary */
 	if ( IEG_G_FirstLon(gdb) == 0 && IEG_G_LastLon(gdb) > 354000 )
 	  {
-	    double xinc = 360. / xsize;
+	    const double xinc = 360. / xsize;
             /* FIXME: why not use grid->x.inc != xinc as condition? */
 	    if ( fabs(grid->x.inc-xinc) > 0.0 )
 	      {
@@ -596,7 +590,7 @@ void iegAddRecord(stream_t *streamptr, int param, int *pdb, int *gdb, double *vc
     }
 
   struct addIfNewRes gridAdded = cdiVlistAddGridIfNew(vlistID, grid, 0);
-  int gridID = gridAdded.Id;
+  const int gridID = gridAdded.Id;
   if ( !gridAdded.isNew ) Free(grid);
   else if ( gridtype == GRID_PROJECTION ) gridDefParamRLL(gridID, xpole, ypole, 0);
 
@@ -612,15 +606,14 @@ void iegAddRecord(stream_t *streamptr, int param, int *pdb, int *gdb, double *vc
       varDefVCT(vctsize, tmpvct);
     }
 
-  int lbounds = IEG_P_LevelType(pdb) == IEG_LTYPE_HYBRID_LAYER ? 1 : 0;
+  const int lbounds = IEG_P_LevelType(pdb) == IEG_LTYPE_HYBRID_LAYER ? 1 : 0;
 
-  int datatype = iegInqDatatype(prec);
+  const int datatype = iegInqDatatype(prec);
 
-  int varID;
-  int levelID = 0;
+  int varID, levelID = 0;
   varAddRecord(recID, param, gridID, leveltype, lbounds, level1, level2, 0, 0,
 	       datatype, &varID, &levelID, TSTEP_INSTANT, 0, 0, -1,
-               NULL, NULL, NULL, NULL, NULL, NULL, 0);
+               NULL, NULL, NULL, NULL, NULL, NULL, NULL);
 
   record->varID   = (short)varID;
   record->levelID = (short)levelID;
@@ -681,35 +674,32 @@ void iegScanTimestep1(stream_t *streamptr)
 {
   DateTime datetime0 = { LONG_MIN, LONG_MIN };
   off_t recpos;
-  iegcompvar_t compVar, compVar0;
   iegrec_t *iegp = (iegrec_t*) streamptr->record->exsep;
 
   streamptr->curTsID = 0;
 
   int tsID = tstepsNewEntry(streamptr);
+  if ( tsID != 0 ) Error("Internal problem! tstepsNewEntry returns %d", tsID);
   taxis_t *taxis = &streamptr->tsteps[tsID].taxis;
 
-  if ( tsID != 0 )
-    Error("Internal problem! tstepsNewEntry returns %d", tsID);
-
-  int fileID = streamptr->fileID;
+  const int fileID = streamptr->fileID;
 
   int nrecs = 0;
   while ( true )
     {
       recpos = fileGetPos(fileID);
-      int status = iegRead(fileID, iegp);
-      if ( status != 0 )
+      if ( iegRead(fileID, iegp) != 0 )
 	{
 	  streamptr->ntsteps = 1;
 	  break;
 	}
-      size_t recsize = (size_t)(fileGetPos(fileID) - recpos);
 
-      int prec   = iegp->dprec;
-      int rcode  = IEG_P_Parameter(iegp->ipdb);
-      int tabnum = IEG_P_CodeTable(iegp->ipdb);
-      int param  = cdiEncodeParam(rcode, tabnum, 255);
+      const size_t recsize = (size_t)(fileGetPos(fileID) - recpos);
+
+      const int prec   = iegp->dprec;
+      const int rcode  = IEG_P_Parameter(iegp->ipdb);
+      const int tabnum = IEG_P_CodeTable(iegp->ipdb);
+      const int param  = cdiEncodeParam(rcode, tabnum, 255);
       int rlevel = (IEG_P_LevelType(iegp->ipdb) == IEG_LTYPE_HYBRID_LAYER) ? IEG_P_Level1(iegp->ipdb) : IEG_P_Level2(iegp->ipdb);
 
       if ( IEG_P_LevelType(iegp->ipdb) == 100 ) rlevel *= 100;
@@ -725,18 +715,12 @@ void iegScanTimestep1(stream_t *streamptr)
 	}
       else
 	{
-	  compVar.param = param;
-          compVar.level = rlevel;
-          int recID = 0;
-	  for ( recID = 0; recID < nrecs; recID++ )
-	    {
-	      compVar0.param = streamptr->tsteps[0].records[recID].param;
-	      compVar0.level = streamptr->tsteps[0].records[recID].ilevel;
+          record_t *records = streamptr->tsteps[tsID].records;
+	  for ( int recID = 0; recID < nrecs; recID++ )
+            if ( param == records[recID].param && rlevel == records[recID].ilevel )
+              goto tstepScanLoopFinished;
 
-	      if ( memcmp(&compVar0, &compVar, sizeof(iegcompvar_t)) == 0 ) break;
-	    }
-	  if ( recID < nrecs ) break;
-	  DateTime datetime = { .date = vdate, .time = vtime};
+          DateTime datetime = { .date = vdate, .time = vtime};
 	  if ( datetimeCmp(datetime, datetime0) )
 	    Warning("Inconsistent verification time for param %d level %d", param, rlevel);
 	}
@@ -749,107 +733,66 @@ void iegScanTimestep1(stream_t *streamptr)
       iegAddRecord(streamptr, param, iegp->ipdb, iegp->igdb, iegp->vct, recsize, recpos, prec);
     }
 
+  tstepScanLoopFinished:
   streamptr->rtsteps = 1;
 
   cdi_generate_vars(streamptr);
 
-  int taxisID = taxisCreate(TAXIS_ABSOLUTE);
+  const int taxisID = taxisCreate(TAXIS_ABSOLUTE);
   taxis->type  = TAXIS_ABSOLUTE;
   taxis->vdate = datetime0.date;
   taxis->vtime = (int)datetime0.time;
   taxis->rdate = taxis->vdate;
   taxis->rtime = taxis->vtime;
 
-  int vlistID = streamptr->vlistID;
+  const int vlistID = streamptr->vlistID;
   vlistDefTaxis(vlistID, taxisID);
 
   vlist_check_contents(vlistID);
 
-  int nrecords = streamptr->tsteps[0].nallrecs;
-  if ( nrecords < streamptr->tsteps[0].recordSize )
-    {
-      streamptr->tsteps[0].recordSize = nrecords;
-      streamptr->tsteps[0].records =
-	(record_t *) Realloc(streamptr->tsteps[0].records,
-                             (size_t)nrecords * sizeof (record_t));
-    }
+  streamScanResizeRecords1(streamptr);
 
-  streamptr->tsteps[0].recIDs = (int *) Malloc((size_t)nrecords * sizeof (int));
-  streamptr->tsteps[0].nrecs = nrecords;
-  for ( int recID = 0; recID < nrecords; recID++ )
-    streamptr->tsteps[0].recIDs[recID] = recID;
-
-  if ( streamptr->ntsteps == -1 )
-    {
-      tsID = tstepsNewEntry(streamptr);
-      if ( tsID != streamptr->rtsteps )
-	Error("Internal error. tsID = %d", tsID);
-
-      streamptr->tsteps[tsID-1].next   = true;
-      streamptr->tsteps[tsID].position = recpos;
-    }
-
-  if ( streamptr->ntsteps == 1 )
-    {
-      if ( taxis->vdate == 0 && taxis->vtime == 0 )
-	{
-	  streamptr->ntsteps = 0;
-	  for ( int varID = 0; varID < streamptr->nvars; varID++ )
-            vlistDefVarTimetype(vlistID, varID, TIME_CONSTANT);
-	}
-    }
+  streamScanTsFixNtsteps(streamptr, recpos);
+  streamScanTimeConstAdjust(streamptr, taxis);
 }
 
 static
 int iegScanTimestep2(stream_t *streamptr)
 {
   off_t recpos = 0;
-  iegcompvar_t compVar, compVar0;
   iegrec_t *iegp = (iegrec_t*) streamptr->record->exsep;
 
   streamptr->curTsID = 1;
 
-  int vlistID = streamptr->vlistID;
-  int fileID  = streamptr->fileID;
+  const int vlistID = streamptr->vlistID;
+  const int fileID  = streamptr->fileID;
 
-  int tsID = streamptr->rtsteps;
-  if ( tsID != 1 )
-    Error("Internal problem! unexpected timestep %d", tsID+1);
+  const int tsID = streamptr->rtsteps;
+  if ( tsID != 1 ) Error("Internal problem! unexpected timestep %d", tsID+1);
 
   taxis_t *taxis = &streamptr->tsteps[tsID].taxis;
 
   fileSetPos(fileID, streamptr->tsteps[tsID].position, SEEK_SET);
 
   cdi_create_records(streamptr, tsID);
+  record_t *records = streamptr->tsteps[tsID].records;
 
-  int nrecords = streamptr->tsteps[0].nallrecs;
-  streamptr->tsteps[1].recIDs = (int *) Malloc((size_t)nrecords * sizeof(int));
-  streamptr->tsteps[1].nrecs = 0;
-  for ( int recID = 0; recID < nrecords; recID++ )
-    streamptr->tsteps[1].recIDs[recID] = -1;
-
-  for ( int recID = 0; recID < nrecords; recID++ )
-    {
-      streamptr->tsteps[tsID].records[recID].position =
-	streamptr->tsteps[0].records[recID].position;
-      streamptr->tsteps[tsID].records[recID].size     =
-	streamptr->tsteps[0].records[recID].size;
-    }
+  const int nrecords = streamScanInitRecords2(streamptr);
 
   for ( int rindex = 0; rindex <= nrecords; rindex++ )
     {
       recpos = fileGetPos(fileID);
-      int status = iegRead(fileID, iegp);
-      if ( status != 0 )
+      if ( iegRead(fileID, iegp) != 0 )
 	{
 	  streamptr->ntsteps = 2;
 	  break;
 	}
-      size_t recsize = (size_t)(fileGetPos(fileID) - recpos);
 
-      int rcode  = IEG_P_Parameter(iegp->ipdb);
-      int tabnum = IEG_P_CodeTable(iegp->ipdb);
-      int param  = cdiEncodeParam(rcode, tabnum, 255);
+      const size_t recsize = (size_t)(fileGetPos(fileID) - recpos);
+
+      const int rcode  = IEG_P_Parameter(iegp->ipdb);
+      const int tabnum = IEG_P_CodeTable(iegp->ipdb);
+      const int param  = cdiEncodeParam(rcode, tabnum, 255);
       int rlevel = (IEG_P_LevelType(iegp->ipdb) == IEG_LTYPE_HYBRID_LAYER) ? IEG_P_Level1(iegp->ipdb) : IEG_P_Level2(iegp->ipdb);
 
       if ( IEG_P_LevelType(iegp->ipdb) == 100 ) rlevel *= 100;
@@ -865,24 +808,19 @@ int iegScanTimestep2(stream_t *streamptr)
 	  taxis->vtime = vtime;
 	}
 
-      compVar.param = param;
-      compVar.level = rlevel;
       bool nextstep = false;
       int recID = 0;
       for ( recID = 0; recID < nrecords; recID++ )
 	{
-	  compVar0.param = streamptr->tsteps[tsID].records[recID].param;
-	  compVar0.level = streamptr->tsteps[tsID].records[recID].ilevel;
-
-	  if ( memcmp(&compVar0, &compVar, sizeof(iegcompvar_t)) == 0 )
+	  if ( param == records[recID].param && rlevel == records[recID].ilevel )
 	    {
-	      if ( streamptr->tsteps[tsID].records[recID].used )
+	      if ( records[recID].used )
 		{
 		  nextstep = true;
 		}
 	      else
 		{
-		  streamptr->tsteps[tsID].records[recID].used = true;
+		  records[recID].used = true;
 		  streamptr->tsteps[tsID].recIDs[rindex] = recID;
 		}
 	      break;
@@ -901,30 +839,23 @@ int iegScanTimestep2(stream_t *streamptr)
       if ( CDI_Debug )
 	Message("%4d%8d%4d%8d%8ld%6d", rindex+1, (int)recpos, param, rlevel, (long)vdate, vtime);
 
-      streamptr->tsteps[tsID].records[recID].size = recsize;
-
-      compVar0.param = streamptr->tsteps[tsID].records[recID].param;
-      compVar0.level = streamptr->tsteps[tsID].records[recID].ilevel;
-
-      if ( memcmp(&compVar0, &compVar, sizeof(iegcompvar_t)) != 0 )
+      if ( param != records[recID].param || rlevel != records[recID].ilevel )
 	{
 	  Message("tsID = %d recID = %d param = %3d new %3d  level = %3d new %3d",
-		  tsID, recID,
-		  streamptr->tsteps[tsID].records[recID].param, param,
-		  streamptr->tsteps[tsID].records[recID].ilevel, rlevel);
+		  tsID, recID, records[recID].param, param, records[recID].ilevel, rlevel);
 	  return CDI_EUFSTRUCT;
 	}
 
-      streamptr->tsteps[1].records[recID].position = recpos;
+      records[recID].position = recpos;
+      records[recID].size = recsize;
     }
 
   int nrecs = 0;
   for ( int recID = 0; recID < nrecords; recID++ )
     {
-      if ( ! streamptr->tsteps[tsID].records[recID].used )
+      if ( ! records[recID].used )
 	{
-	  int varID = streamptr->tsteps[tsID].records[recID].varID;
-          vlistDefVarTimetype(vlistID, varID, TIME_CONSTANT);
+          vlistDefVarTimetype(vlistID, records[recID].varID, TIME_CONSTANT);
 	}
       else
 	{
@@ -935,15 +866,7 @@ int iegScanTimestep2(stream_t *streamptr)
 
   streamptr->rtsteps = 2;
 
-  if ( streamptr->ntsteps == -1 )
-    {
-      tsID = tstepsNewEntry(streamptr);
-      if ( tsID != streamptr->rtsteps )
-	Error("Internal error. tsID = %d", tsID);
-
-      streamptr->tsteps[tsID-1].next   = true;
-      streamptr->tsteps[tsID].position = recpos;
-    }
+  streamScanTsFixNtsteps(streamptr, recpos);
 
   return 0;
 }
@@ -951,16 +874,13 @@ int iegScanTimestep2(stream_t *streamptr)
 
 int iegInqContents(stream_t *streamptr)
 {
-  int fileID = streamptr->fileID;
-
   streamptr->curTsID = 0;
 
   iegScanTimestep1(streamptr);
 
-  int status = 0;
-  if ( streamptr->ntsteps == -1 ) status = iegScanTimestep2(streamptr);
+  const int status = (streamptr->ntsteps == -1) ? iegScanTimestep2(streamptr) : 0;
 
-  fileSetPos(fileID, 0, SEEK_SET);
+  fileSetPos(streamptr->fileID, 0, SEEK_SET);
 
   return status;
 }
@@ -969,19 +889,9 @@ static
 long iegScanTimestep(stream_t *streamptr)
 {
   off_t recpos = 0;
-  iegcompvar_t compVar, compVar0;
   iegrec_t *iegp = (iegrec_t*) streamptr->record->exsep;
 
-  if ( CDI_Debug )
-    {
-      Message("streamID = %d", streamptr->self);
-      Message("cts = %d", streamptr->curTsID);
-      Message("rts = %d", streamptr->rtsteps);
-      Message("nts = %d", streamptr->ntsteps);
-    }
-
-  if ( streamptr->rtsteps == 0 )
-    Error("Internal problem! Missing contents.");
+  if ( streamptr->rtsteps == 0 ) Error("Internal problem! Missing contents.");
 
   int tsID = streamptr->rtsteps;
   taxis_t *taxis = &streamptr->tsteps[tsID].taxis;
@@ -990,32 +900,28 @@ long iegScanTimestep(stream_t *streamptr)
   if ( streamptr->tsteps[tsID].recordSize == 0 )
     {
       cdi_create_records(streamptr, tsID);
+      record_t *records = streamptr->tsteps[tsID].records;
 
-      nrecs = streamptr->tsteps[1].nrecs;
+      nrecs = streamScanInitRecords(streamptr, tsID);
 
-      streamptr->tsteps[tsID].nrecs = nrecs;
-      streamptr->tsteps[tsID].recIDs = (int *) Malloc((size_t)nrecs * sizeof (int));
-      for ( int recID = 0; recID < nrecs; recID++ )
-	streamptr->tsteps[tsID].recIDs[recID] = streamptr->tsteps[1].recIDs[recID];
-
-      int fileID = streamptr->fileID;
+      const int fileID = streamptr->fileID;
 
       fileSetPos(fileID, streamptr->tsteps[tsID].position, SEEK_SET);
 
       for ( int rindex = 0; rindex <= nrecs; rindex++ )
 	{
 	  recpos = fileGetPos(fileID);
-	  int status = iegRead(fileID, iegp);
-	  if ( status != 0 )
+	  if ( iegRead(fileID, iegp) != 0 )
 	    {
 	      streamptr->ntsteps = streamptr->rtsteps + 1;
 	      break;
 	    }
-	  size_t recsize = (size_t)(fileGetPos(fileID) - recpos);
 
-	  int rcode  = IEG_P_Parameter(iegp->ipdb);
-	  int tabnum = IEG_P_CodeTable(iegp->ipdb);
-	  int param  = cdiEncodeParam(rcode, tabnum, 255);
+	  const size_t recsize = (size_t)(fileGetPos(fileID) - recpos);
+
+	  const int rcode  = IEG_P_Parameter(iegp->ipdb);
+	  const int tabnum = IEG_P_CodeTable(iegp->ipdb);
+	  const int param  = cdiEncodeParam(rcode, tabnum, 255);
           int rlevel = (IEG_P_LevelType(iegp->ipdb) == IEG_LTYPE_HYBRID_LAYER) ? IEG_P_Level1(iegp->ipdb) : IEG_P_Level2(iegp->ipdb);
 
 	  if ( IEG_P_LevelType(iegp->ipdb) == 100 ) rlevel *= 100;
@@ -1026,7 +932,7 @@ long iegScanTimestep(stream_t *streamptr)
 
 	  // if ( rindex == nrecs ) break; gcc-4.5 internal compiler error
 	  if ( rindex == nrecs ) continue;
-	  int recID = streamptr->tsteps[tsID].recIDs[rindex];
+	  const int recID = streamptr->tsteps[tsID].recIDs[rindex];
 
 	  if ( rindex == 0 )
 	    {
@@ -1035,22 +941,15 @@ long iegScanTimestep(stream_t *streamptr)
 	      taxis->vtime = vtime;
 	    }
 
-	  compVar.param = param;
-          compVar.level = rlevel;
-	  compVar0.param = streamptr->tsteps[tsID].records[recID].param;
-	  compVar0.level = streamptr->tsteps[tsID].records[recID].ilevel;
-
-	  if ( memcmp(&compVar0, &compVar, sizeof(iegcompvar_t)) != 0 )
+          if ( param != records[recID].param || rlevel != records[recID].ilevel )
 	    {
 	      Message("tsID = %d recID = %d param = %3d new %3d  level = %3d new %3d",
-		      tsID, recID,
-		      streamptr->tsteps[tsID].records[recID].param, param,
-		      streamptr->tsteps[tsID].records[recID].ilevel, rlevel);
+		      tsID, recID, records[recID].param, param, records[recID].ilevel, rlevel);
 	      Error("Invalid, unsupported or inconsistent record structure");
 	    }
 
-	  streamptr->tsteps[tsID].records[recID].position = recpos;
-	  streamptr->tsteps[tsID].records[recID].size = recsize;
+	  records[recID].position = recpos;
+	  records[recID].size = recsize;
 
 	  if ( CDI_Debug )
 	    Message("%4d%8d%4d%8d%8ld%6d", rindex, (int)recpos, param, rlevel, (long)vdate, vtime);
