@@ -1,7 +1,3 @@
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
-
 #include <stddef.h>
 
 #include "cdi.h"
@@ -212,8 +208,7 @@ The function @func{taxisCreate} creates a Time axis.
 @func{taxisCreate} returns an identifier to the Time axis.
 
 @Example
-Here is an example using @func{taxisCreate} to create a relative T-axis
-with a standard calendar.
+Here is an example using @func{taxisCreate} to create a relative T-axis with a standard calendar.
 
 @Source
 #include "cdi.h"
@@ -244,6 +239,7 @@ int taxisCreate(int taxistype)
   return taxisID;
 }
 
+
 void taxisDestroyKernel(taxis_t *taxisptr)
 {
   delete_refcount_string(taxisptr->name);
@@ -292,13 +288,13 @@ int taxisDuplicate(int taxisID1)
 }
 
 
-void taxisDefType(int taxisID, int type)
+void taxisDefType(int taxisID, int taxistype)
 {
   taxis_t *taxisptr = (taxis_t *) reshGetVal(taxisID, &taxisOps);
 
-  if ( taxisptr->type != type )
+  if ( taxisptr->type != taxistype )
     {
-      taxisptr->type = type;
+      taxisptr->type = taxistype;
       if ( taxisptr->units )
         {
           delete_refcount_string(taxisptr->units);
@@ -598,6 +594,7 @@ void taxisCopyTimestep(int taxisID2, int taxisID1)
 
   reshLock();
 
+  /* reference date/time and units can't be changed after streamDefVlist().
   if (taxisptr2->units && taxisptr2->rdate != CDI_UNDEFID)
     {
       if (taxisptr2->rdate != taxisptr1->rdate || taxisptr2->rtime != taxisptr1->rtime)
@@ -609,7 +606,7 @@ void taxisCopyTimestep(int taxisID2, int taxisID1)
 
   taxisptr2->rdate = taxisptr1->rdate;
   taxisptr2->rtime = taxisptr1->rtime;
-
+  */
   taxisptr2->vdate = taxisptr1->vdate;
   taxisptr2->vtime = taxisptr1->vtime;
 
