@@ -35,8 +35,7 @@ leveltable_t;
 
 typedef struct
 {
-  int           subtypeIndex; /* corresponding tile in subtype_t structure (subtype->self) */
-
+  int           subtypeIndex; //  corresponding tile in subtype_t structure (subtype->self)
   unsigned      nlevels;
   int           levelTableSize;
   leveltable_t* levelTable;
@@ -49,11 +48,11 @@ typedef struct
   int            param;
   int            prec;
   int            tsteptype;
-  int            perturbationNumber;
+  VarScanKeys    scanKeys;
   int            gridID;
   int            zaxistype;
-  int            ltype1;     /* GRIB first level type */
-  int            ltype2;     /* GRIB second level type */
+  int            ltype1;        // GRIB first level type
+  int            ltype2;        // GRIB second level type
   int            lbounds;
   int            level_sf;
   int            level_unit;
@@ -61,7 +60,7 @@ typedef struct
 
   int            nsubtypes_alloc;
   int            nsubtypes;
-  subtypetable_t *recordTable;    /* ~ two-dimensional record list [nsubtypes_alloc][levelTableSize] */
+  subtypetable_t *recordTable;   // ~ two-dimensional record list [nsubtypes_alloc][levelTableSize]
 
   int            instID;
   int            modelID;
@@ -81,9 +80,9 @@ typedef struct
 
   cdi_keys_t     keys;
 
-  int                 opt_grib_nentries;                       /* current no. key-value pairs            */
-  int                 opt_grib_kvpair_size;                    /* current allocated size                 */
-  opt_key_val_pair_t *opt_grib_kvpair;                         /* (optional) list of keyword/value pairs */
+  int                 opt_grib_nentries;        // current no. key-value pairs
+  int                 opt_grib_kvpair_size;     // current allocated size
+  opt_key_val_pair_t *opt_grib_kvpair;          // (optional) list of keyword/value pairs
 }
 vartable_t;
 
@@ -99,7 +98,7 @@ void paramInitEntry(unsigned varID, int param)
   vartable[varID].param          = param;
   vartable[varID].prec           = 0;
   vartable[varID].tsteptype      = TSTEP_INSTANT;
-  vartable[varID].perturbationNumber = 0;
+  varScanKeysInit(&vartable[varID].scanKeys);
   vartable[varID].timave         = 0;
   vartable[varID].gridID         = CDI_UNDEFID;
   vartable[varID].zaxistype      = 0;
@@ -127,25 +126,22 @@ void paramInitEntry(unsigned varID, int param)
   vartable[varID].tiles          = NULL;
 }
 
-/* Test if a variable specified by the given meta-data has already been registered in "vartable". */
+// Test if a variable specified by the given meta-data has already been registered in "vartable".
 static unsigned
-varGetEntry(int param, int gridID, int zaxistype, int ltype1, int tsteptype, const char *name, const var_tile_t *tiles, int perturbationNumber)
+varGetEntry(int param, int gridID, int zaxistype, int ltype1, int tsteptype, const char *name, const VarScanKeys *scanKeys, const var_tile_t *tiles)
 {
   for ( unsigned varID = 0; varID < varTablesize; varID++ )
     {
-      /* testing for "param" implicitly checks if we are beyond the current vartable size: */
+      // testing for "param" implicitly checks if we are beyond the current vartable size:
       if ( vartable[varID].param == param )
         {
-          int no_of_tiles = -1;
-          if ( tiles ) no_of_tiles = tiles->numberOfTiles;
-          int vt_no_of_tiles = -1;
-          if ( vartable[varID].tiles )
-            vt_no_of_tiles = subtypeGetGlobalDataP(vartable[varID].tiles,
-                                                   SUBTYPE_ATT_NUMBER_OF_TILES);
+          const int no_of_tiles = tiles ? tiles->numberOfTiles : -1;
+          const int vt_no_of_tiles = vartable[varID].tiles ?
+            subtypeGetGlobalDataP(vartable[varID].tiles, SUBTYPE_ATT_NUMBER_OF_TILES) : -1;
           if ( (vartable[varID].zaxistype  == zaxistype)               &&
                (vartable[varID].ltype1     == ltype1   )               &&
                (vartable[varID].tsteptype  == tsteptype)               &&
-               (vartable[varID].perturbationNumber == perturbationNumber)  &&
+               (scanKeys == NULL || varScanKeysIsEqual(&vartable[varID].scanKeys, scanKeys))  &&
                (vartable[varID].gridID     == gridID   )               &&
                (vt_no_of_tiles == no_of_tiles) )
             {
@@ -189,10 +185,11 @@ void varFree(void)
 
       if ( vartable[varID].opt_grib_kvpair )
         {
-          for (int i=0; i<vartable[varID].opt_grib_nentries; i++) {
-            if ( vartable[varID].opt_grib_kvpair[i].keyword )
-              Free(vartable[varID].opt_grib_kvpair[i].keyword);
-          }
+          for (int i=0; i<vartable[varID].opt_grib_nentries; i++)
+            {
+              if ( vartable[varID].opt_grib_kvpair[i].keyword )
+                Free(vartable[varID].opt_grib_kvpair[i].keyword);
+            }
           Free(vartable[varID].opt_grib_kvpair);
         }
       vartable[varID].opt_grib_nentries    = 0;
@@ -200,21 +197,17 @@ void varFree(void)
       vartable[varID].opt_grib_kvpair      = NULL;
     }
 
-  if ( vartable )
-    Free(vartable);
-
+  if ( vartable ) Free(vartable);
   vartable = NULL;
   varTablesize = 0;
   varTableUsed = 0;
 
-  if ( Vct )
-    Free(Vct);
-
+  if ( Vct ) Free(Vct);
   Vct = NULL;
   Vctsize = 0;
 }
 
-/* Search for a tile subtype with subtypeIndex == tile_index. */
+// Search for a tile subtype with subtypeIndex == tile_index.
 static int tileGetEntry(unsigned varID, int tile_index)
 {
   for (int isub=0; isub<vartable[varID].nsubtypes; isub++)
@@ -283,27 +276,20 @@ static int levelNewEntry(unsigned varID, int level1, int level2, int tileID)
   int levelTableSize = vartable[varID].recordTable[tileID].levelTableSize;
   leveltable_t *levelTable = vartable[varID].recordTable[tileID].levelTable;
 
-  /*
-    Look for a free slot in levelTable.
-    (Create the table the first time through).
-  */
+  // Look for a free slot in levelTable. (Create the table the first time through).
   if ( ! levelTableSize )
     {
       levelTableSize = 2;
-      levelTable = (leveltable_t *) Malloc((size_t)levelTableSize
-                                           * sizeof (leveltable_t));
-      for ( int i = 0; i < levelTableSize; i++ )
-        levelTable[i].recID = CDI_UNDEFID;
+      levelTable = (leveltable_t *) Malloc((size_t)levelTableSize * sizeof (leveltable_t));
+      for ( int i = 0; i < levelTableSize; i++ ) levelTable[i].recID = CDI_UNDEFID;
     }
   else
     {
-      while( levelID < levelTableSize
-             && levelTable[levelID].recID != CDI_UNDEFID )
+      while( levelID < levelTableSize && levelTable[levelID].recID != CDI_UNDEFID )
         ++levelID;
     }
-  /*
-    If the table overflows, double its size.
-  */
+
+  // If the table overflows, double its size.
   if ( levelID == levelTableSize )
     {
       levelTable = (leveltable_t *) Realloc(levelTable,
@@ -313,9 +299,9 @@ static int levelNewEntry(unsigned varID, int level1, int level2, int tileID)
         levelTable[i].recID = CDI_UNDEFID;
     }
 
-  levelTable[levelID].level1   = level1;
-  levelTable[levelID].level2   = level2;
-  levelTable[levelID].lindex   = levelID;
+  levelTable[levelID].level1 = level1;
+  levelTable[levelID].level2 = level2;
+  levelTable[levelID].lindex = levelID;
 
   vartable[varID].recordTable[tileID].nlevels        = (unsigned)levelID+1;
   vartable[varID].recordTable[tileID].levelTableSize = levelTableSize;
@@ -331,15 +317,11 @@ paramNewEntry(int param)
 {
   unsigned varID = 0;
 
-  /*
-    Look for a free slot in vartable.
-    (Create the table the first time through).
-  */
+  // Look for a free slot in vartable. (Create the table the first time through).
   if ( ! varTablesize )
     {
       varTablesize = 2;
-      vartable = (vartable_t *) Malloc((size_t)varTablesize
-                                       * sizeof (vartable_t));
+      vartable = (vartable_t *) Malloc((size_t)varTablesize * sizeof (vartable_t));
       if( vartable == NULL )
 	{
           Message("varTablesize = %d", varTablesize);
@@ -362,9 +344,8 @@ paramNewEntry(int param)
 	  varID++;
 	}
     }
-  /*
-    If the table overflows, double its size.
-  */
+
+  // If the table overflows, double its size.
   if ( varID == varTablesize )
     {
       vartable = (vartable_t *) Realloc(vartable, (size_t)(varTablesize *= 2)
@@ -383,30 +364,24 @@ paramNewEntry(int param)
   return varID;
 }
 
-
-/* Append tile set to a subtype. Return index of the new tile (i.e.
- * the "entry->self" value). */
+// Append tile set to a subtype. Return index of the new tile (i.e. the "entry->self" value).
 static
 int varInsertTileSubtype(vartable_t *vptr, const var_tile_t *tiles)
 {
   if ( tiles == NULL ) return -1;
 
-  /* first, generate a subtype based on the info in "tiles". */
-
+  // first, generate a subtype based on the info in "tiles".
   subtype_t *subtype_ptr;
   subtypeAllocate(&subtype_ptr, SUBTYPE_TILES);
   subtypeDefGlobalDataP(subtype_ptr, SUBTYPE_ATT_TOTALNO_OF_TILEATTR_PAIRS, tiles->totalno_of_tileattr_pairs);
   subtypeDefGlobalDataP(subtype_ptr, SUBTYPE_ATT_TILE_CLASSIFICATION      , tiles->tileClassification);
   subtypeDefGlobalDataP(subtype_ptr, SUBTYPE_ATT_NUMBER_OF_TILES          , tiles->numberOfTiles);
 
-  /*
-   * Here, we create a tile set for comparison that contains only one
-   * tile/attribute pair (based on "tiles").
-   */
+  // Here, we create a tile set for comparison that contains only one tile/attribute pair (based on "tiles").
   struct subtype_entry_t *entry = subtypeEntryInsert(subtype_ptr);
-  subtypeDefEntryDataP(entry, SUBTYPE_ATT_NUMBER_OF_ATTR,            tiles->numberOfAttributes);
-  subtypeDefEntryDataP(entry, SUBTYPE_ATT_TILEINDEX,                 tiles->tileindex);
-  subtypeDefEntryDataP(entry, SUBTYPE_ATT_TILEATTRIBUTE,             tiles->attribute);
+  subtypeDefEntryDataP(entry, SUBTYPE_ATT_NUMBER_OF_ATTR, tiles->numberOfAttributes);
+  subtypeDefEntryDataP(entry, SUBTYPE_ATT_TILEINDEX,      tiles->tileindex);
+  subtypeDefEntryDataP(entry, SUBTYPE_ATT_TILEATTRIBUTE,  tiles->attribute);
 
   if (vptr->tiles == NULL) {
     vptr->tiles = subtype_ptr;
@@ -426,10 +401,10 @@ void varAddRecord(int recID, int param, int gridID, int zaxistype, int lbounds,
 		  int level1, int level2, int level_sf, int level_unit, int prec,
 		  int *pvarID, int *plevelID, int tsteptype, int numavg, int ltype1, int ltype2,
 		  const char *name, const char *stdname, const char *longname, const char *units,
-                  const var_tile_t *tiles, int *tile_index, int perturbationNumber)
+                  const VarScanKeys *scanKeys, const var_tile_t *tiles, int *tile_index)
 {
   unsigned varID = (cdiSplitLtype105 != 1 || zaxistype != ZAXIS_HEIGHT) ?
-    varGetEntry(param, gridID, zaxistype, ltype1, tsteptype, name, tiles, perturbationNumber) : (unsigned) CDI_UNDEFID;
+    varGetEntry(param, gridID, zaxistype, ltype1, tsteptype, name, scanKeys, tiles) : (unsigned) CDI_UNDEFID;
 
   if ( varID == (unsigned) CDI_UNDEFID )
     {
@@ -443,7 +418,7 @@ void varAddRecord(int recID, int param, int gridID, int zaxistype, int lbounds,
       vartable[varID].level_sf   = level_sf;
       vartable[varID].level_unit = level_unit;
       vartable[varID].tsteptype  = tsteptype;
-      vartable[varID].perturbationNumber = perturbationNumber;
+      if (scanKeys) vartable[varID].scanKeys = *scanKeys;
 
       if ( numavg ) vartable[varID].timave = 1;
 
@@ -471,8 +446,8 @@ void varAddRecord(int recID, int param, int gridID, int zaxistype, int lbounds,
 
   if ( prec > vartable[varID].prec ) vartable[varID].prec = prec;
 
-  /* append current tile to tile subtype info. */
-  int this_tile = varInsertTileSubtype(&vartable[varID], tiles);
+  // append current tile to tile subtype info.
+  const int this_tile = varInsertTileSubtype(&vartable[varID], tiles);
   int tileID = tileGetEntry(varID, this_tile);
   if ( tile_index ) (*tile_index) = this_tile;
   if ( tileID == CDI_UNDEFID )
@@ -482,8 +457,8 @@ void varAddRecord(int recID, int param, int gridID, int zaxistype, int lbounds,
       vartable[varID].nsubtypes++;
     }
 
-  /* append current level to level table info */
-  int levelID = levelNewEntry(varID, level1, level2, tileID);
+  // append current level to level table info
+  const int levelID = levelNewEntry(varID, level1, level2, tileID);
   if (CDI_Debug)
     Message("vartable[%d].recordTable[%d].levelTable[%d].recID = %d; level1,2=%d,%d",
             varID, tileID, levelID, recID, level1, level2);
@@ -560,7 +535,7 @@ int cdi_generate_cmp_varname(const void *s1, const void *s2)
 
 void cdi_generate_vars(stream_t *streamptr)
 {
-  int vlistID = streamptr->vlistID;
+  const int vlistID = streamptr->vlistID;
 
   int *varids = (int *) Malloc(varTableUsed*sizeof(int));
   for ( size_t varID = 0; varID < varTableUsed; varID++ ) varids[varID] = (int)varID;
@@ -594,29 +569,29 @@ void cdi_generate_vars(stream_t *streamptr)
 
   for ( size_t index = 0; index < varTableUsed; index++ )
     {
-      int varid      = varids[index];
+      const int varid      = varids[index];
 
-      int gridID     = vartable[varid].gridID;
-      int param      = vartable[varid].param;
-      int ltype1     = vartable[varid].ltype1;
-      int ltype2     = vartable[varid].ltype2;
+      const int gridID     = vartable[varid].gridID;
+      const int param      = vartable[varid].param;
+      const int ltype1     = vartable[varid].ltype1;
+      const int ltype2     = vartable[varid].ltype2;
       int zaxistype  = vartable[varid].zaxistype;
       if ( ltype1 == 0 && zaxistype == ZAXIS_GENERIC && cdiDefaultLeveltype != -1 )
 	zaxistype = cdiDefaultLeveltype;
-      int lbounds    = vartable[varid].lbounds;
-      int prec       = vartable[varid].prec;
+      const int lbounds    = vartable[varid].lbounds;
+      const int prec       = vartable[varid].prec;
       int instID     = vartable[varid].instID;
       int modelID    = vartable[varid].modelID;
       int tableID    = vartable[varid].tableID;
-      int tsteptype  = vartable[varid].tsteptype;
-      int timave     = vartable[varid].timave;
-      int comptype   = vartable[varid].comptype;
+      const int tsteptype  = vartable[varid].tsteptype;
+      const int timave     = vartable[varid].timave;
+      const int comptype   = vartable[varid].comptype;
 
       double level_sf = 1;
       if ( vartable[varid].level_sf != 0 ) level_sf = 1./vartable[varid].level_sf;
 
       /* consistency check: test if all subtypes have the same levels: */
-      unsigned nlevels = vartable[varid].recordTable[0].nlevels;
+      const unsigned nlevels = vartable[varid].recordTable[0].nlevels;
       for ( int isub = 1; isub < vartable[varid].nsubtypes; isub++ ) {
         if ( vartable[varid].recordTable[isub].nlevels != nlevels )
           {
@@ -627,8 +602,8 @@ void cdi_generate_vars(stream_t *streamptr)
             Error("zaxis size must not change for same parameter!");
           }
 
-        leveltable_t *t1 = vartable[varid].recordTable[isub-1].levelTable;
-        leveltable_t *t2 = vartable[varid].recordTable[isub  ].levelTable;
+        const leveltable_t *t1 = vartable[varid].recordTable[isub-1].levelTable;
+        const leveltable_t *t2 = vartable[varid].recordTable[isub  ].levelTable;
         for ( unsigned ilev = 0; ilev < nlevels; ilev++ )
           if ((t1[ilev].level1 != t2[ilev].level1)  ||
               (t1[ilev].level2 != t2[ilev].level2)  ||
@@ -664,9 +639,9 @@ void cdi_generate_vars(stream_t *streamptr)
           bool linc = true, ldec = true, lsort = false;
           for ( unsigned levelID = 1; levelID < nlevels; levelID++ )
             {
-              /* check increasing of levels */
+              // check increasing of levels
               linc &= (dlevels[levelID] > dlevels[levelID-1]);
-              /* check decreasing of levels */
+              // check decreasing of levels
               ldec &= (dlevels[levelID] < dlevels[levelID-1]);
             }
           /*
@@ -719,12 +694,10 @@ void cdi_generate_vars(stream_t *streamptr)
       const char **cvals = NULL;
       const char *unitptr = cdiUnitNamePtr(vartable[varid].level_unit);
       int zaxisID = varDefZaxis(vlistID, zaxistype, (int)nlevels, dlevels, cvals, 0, lbounds, dlevels1, dlevels2,
-                                (int)Vctsize, Vct, NULL, NULL, unitptr, 0, 0, ltype1);
+                                (int)Vctsize, Vct, NULL, NULL, unitptr, 0, 0, ltype1, ltype2);
 
       if ( CDI_cmor_mode && nlevels == 1 && zaxistype != ZAXIS_HYBRID ) zaxisDefScalar(zaxisID);
 
-      if ( ltype1 != ltype2 && ltype2 != -1 ) zaxisDefLtype2(zaxisID, ltype2);
-
       if ( zaxisInqType(zaxisID) == ZAXIS_REFERENCE )
         {
           if ( numberOfVerticalLevels > 0 ) zaxisDefNlevRef(zaxisID, numberOfVerticalLevels);
@@ -763,7 +736,7 @@ void cdi_generate_vars(stream_t *streamptr)
         {
           resize_opt_grib_entries(&vlistptr->vars[varID], vlistptr->vars[varID].opt_grib_nentries+1);
           vlistptr->vars[varID].opt_grib_nentries += 1;
-          int idx = vlistptr->vars[varID].opt_grib_nentries-1;
+          const int idx = vlistptr->vars[varID].opt_grib_nentries-1;
 
           vlistptr->vars[varID].opt_grib_kvpair[idx] = vartable[varid].opt_grib_kvpair[i];
           vlistptr->vars[varID].opt_grib_kvpair[idx].keyword = NULL;
@@ -851,13 +824,14 @@ void varDefZAxisReference(int nhlev, int nvgrid, unsigned char uuid[CDI_UUID_SIZ
 }
 
 
-bool zaxisCompare(int zaxisID, int zaxistype, int nlevels, bool lbounds, const double *levels, const char *longname, const char *units, int ltype1)
+bool zaxisCompare(int zaxisID, int zaxistype, int nlevels, bool lbounds, const double *levels, const char *longname, const char *units, int ltype1, int ltype2)
 {
   bool differ = true;
 
-  bool ltype_is_equal = (ltype1 == zaxisInqLtype(zaxisID));
+  bool ltype1_is_equal = (ltype1 == zaxisInqLtype(zaxisID));
+  bool ltype2_is_equal = (ltype2 == zaxisInqLtype2(zaxisID));
 
-  if ( ltype_is_equal && (zaxistype == zaxisInqType(zaxisID) || zaxistype == ZAXIS_GENERIC) )
+  if ( ltype1_is_equal && ltype2_is_equal && (zaxistype == zaxisInqType(zaxisID) || zaxistype == ZAXIS_GENERIC) )
     {
       bool zlbounds = (zaxisInqLbounds(zaxisID, NULL) > 0);
       if ( nlevels == zaxisInqSize(zaxisID) && zlbounds == lbounds )
@@ -904,7 +878,8 @@ struct varDefZAxisSearchState
   const double *levels;
   const char *longname;
   const char *units;
-  int ltype;
+  int ltype1;
+  int ltype2;
 };
 
 static enum cdiApplyRet
@@ -913,7 +888,7 @@ varDefZAxisSearch(int id, void *res, void *data)
   struct varDefZAxisSearchState *state = (struct varDefZAxisSearchState *)data;
   (void)res;
   if ( zaxisCompare(id, state->zaxistype, state->nlevels, state->lbounds,
-                    state->levels, state->longname, state->units, state->ltype)
+                    state->levels, state->longname, state->units, state->ltype1, state->ltype2)
       == false)
     {
       state->resIDValue = id;
@@ -926,7 +901,7 @@ varDefZAxisSearch(int id, void *res, void *data)
 
 int varDefZaxis(int vlistID, int zaxistype, int nlevels, const double *levels, const char **cvals, size_t clength, bool lbounds,
                 const double *levels1, const double *levels2, int vctsize, const double *vct, char *name,
-		const char *longname, const char *units, int prec, int mode, int ltype1)
+		const char *longname, const char *units, int prec, int mode, int ltype1, int ltype2)
 {
   /*
     mode: 0 search in vlist and zaxis table
@@ -938,12 +913,14 @@ int varDefZaxis(int vlistID, int zaxistype, int nlevels, const double *levels, c
   vlist_t *vlistptr = vlist_to_pointer(vlistID);
   int nzaxis = vlistptr->nzaxis;
 
+  if ( ltype2 == 255 ) ltype2 = -1;
+
   if ( mode == 0 )
     for ( int index = 0; index < nzaxis; index++ )
       {
 	zaxisID = vlistptr->zaxisIDs[index];
 
-	if ( !zaxisCompare(zaxisID, zaxistype, nlevels, lbounds, levels, longname, units, ltype1) )
+	if ( !zaxisCompare(zaxisID, zaxistype, nlevels, lbounds, levels, longname, units, ltype1, ltype2) )
 	  {
 	    zaxisdefined = true;
 	    break;
@@ -959,7 +936,8 @@ int varDefZaxis(int vlistID, int zaxistype, int nlevels, const double *levels, c
       query.lbounds = lbounds;
       query.longname = longname;
       query.units = units;
-      query.ltype = ltype1;
+      query.ltype1 = ltype1;
+      query.ltype2 = ltype2;
 
       if ((zaxisglobdefined
            = (cdiResHFilterApply(getZaxisOps(), varDefZAxisSearch, &query)
@@ -994,11 +972,11 @@ int varDefZaxis(int vlistID, int zaxistype, int nlevels, const double *levels, c
 
 	  if ( name && name[0] ) zaxisDefName(zaxisID, name);
 	  if ( longname && longname[0] ) zaxisDefLongname(zaxisID, longname);
-          //else                           zaxisDefLongname(zaxisID, "");
 	  if ( units && units[0] ) zaxisDefUnits(zaxisID, units);
-          //else                     zaxisDefUnits(zaxisID, "");
 	  zaxisDefDatatype(zaxisID, prec);
 	  zaxisDefLtype(zaxisID, ltype1);
+          //if ( ltype1 != ltype2 && ltype2 != -1 ) zaxisDefLtype2(zaxisID, ltype2);
+          if ( ltype2 != -1 ) zaxisDefLtype2(zaxisID, ltype2);
 	}
 
       vlistptr->zaxisIDs[nzaxis] = zaxisID;
