@@ -27,7 +27,6 @@
 #include "cdo_options.h"
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
 #include "util_string.h"
 
 /*
@@ -44,11 +43,11 @@
 */
 
 /* compute insitu temperature from potential temperature */
-static double
-adisit_1(double tpot, double sal, double p)
+static inline double
+adisit_1(const double tpot, const double sal, const double p)
 {
-  const double a_a1 = 3.6504E-4, a_a2 = 8.3198E-5, a_a3 = 5.4065E-7, a_a4 = 4.0274E-9, a_b1 = 1.7439E-5, a_b2 = 2.9778E-7,
-               a_c1 = 8.9309E-7, a_c2 = 3.1628E-8, a_c3 = 2.1987E-10, a_d = 4.1057E-9, a_e1 = 1.6056E-10, a_e2 = 5.0484E-12;
+  constexpr double a_a1 = 3.6504E-4, a_a2 = 8.3198E-5, a_a3 = 5.4065E-7, a_a4 = 4.0274E-9, a_b1 = 1.7439E-5, a_b2 = 2.9778E-7,
+                   a_c1 = 8.9309E-7, a_c2 = 3.1628E-8, a_c3 = 2.1987E-10, a_d = 4.1057E-9, a_e1 = 1.6056E-10, a_e2 = 5.0484E-12;
 
   const double qc = p * (a_a1 + p * (a_c1 - a_e1 * p));
   const double qv = p * (a_b1 - a_d * p);
@@ -70,11 +69,11 @@ adisit_1(double tpot, double sal, double p)
 
 /* compute potential temperature from insitu temperature */
 /* Ref: Gill, p. 602, Section A3.5:Potential Temperature */
-static double
-adipot(double t, double s, double p)
+static inline double
+adipot(const double t, const double s, const double p)
 {
-  const double a_a1 = 3.6504E-4, a_a2 = 8.3198E-5, a_a3 = 5.4065E-7, a_a4 = 4.0274E-9, a_b1 = 1.7439E-5, a_b2 = 2.9778E-7,
-               a_c1 = 8.9309E-7, a_c2 = 3.1628E-8, a_c3 = 2.1987E-10, a_d = 4.1057E-9, a_e1 = 1.6056E-10, a_e2 = 5.0484E-12;
+  constexpr double a_a1 = 3.6504E-4, a_a2 = 8.3198E-5, a_a3 = 5.4065E-7, a_a4 = 4.0274E-9, a_b1 = 1.7439E-5, a_b2 = 2.9778E-7,
+                   a_c1 = 8.9309E-7, a_c2 = 3.1628E-8, a_c3 = 2.1987E-10, a_d = 4.1057E-9, a_e1 = 1.6056E-10, a_e2 = 5.0484E-12;
 
   const double s_rel = s - 35.0;
 
@@ -90,56 +89,60 @@ adipot(double t, double s, double p)
 }
 
 static void
-calc_adisit(long gridsize, long nlevel, double *pressure, Field tho, Field sao, Field tis)
+calc_adisit(size_t gridsize, size_t nlevel, const std::vector<double> &pressure, const FieldVector &tho,
+            const FieldVector &sao, FieldVector &tis)
 {
   // pressure units: hPa
   // tho units:      Celsius
   // sao units:      psu
 
-  for (long levelID = 0; levelID < nlevel; ++levelID)
+  for (size_t levelID = 0; levelID < nlevel; ++levelID)
     {
-      const long offset = gridsize * levelID;
-      const double *thoptr = tho.ptr + offset;
-      const double *saoptr = sao.ptr + offset;
-      double *tisptr = tis.ptr + offset;
-
-      for (long i = 0; i < gridsize; ++i)
+      const auto &thovec = tho[levelID].vec;
+      const auto &saovec = sao[levelID].vec;
+      auto &tisvec = tis[levelID].vec;
+      const double tho_missval = tho[levelID].missval;
+      const double sao_missval = sao[levelID].missval;
+      const double tis_missval = tis[levelID].missval;
+      for (size_t i = 0; i < gridsize; ++i)
         {
-          if (DBL_IS_EQUAL(thoptr[i], tho.missval) || DBL_IS_EQUAL(saoptr[i], sao.missval))
+          if (DBL_IS_EQUAL(thovec[i], tho_missval) || DBL_IS_EQUAL(saovec[i], sao_missval))
             {
-              tisptr[i] = tis.missval;
+              tisvec[i] = tis_missval;
             }
           else
             {
-              tisptr[i] = adisit_1(thoptr[i], saoptr[i], pressure[levelID]);
+              tisvec[i] = adisit_1(thovec[i], saovec[i], pressure[levelID]);
             }
         }
     }
 }
 
 static void
-calc_adipot(long gridsize, long nlevel, double *pressure, Field t, Field s, Field tpot)
+calc_adipot(size_t gridsize, size_t nlevel, const std::vector<double> &pressure, const FieldVector &t,
+            const FieldVector &s, FieldVector &tpot)
 {
   // pressure units: hPa
   // t units:        Celsius
   // s units:        psu
 
-  for (long levelID = 0; levelID < nlevel; ++levelID)
+  for (size_t levelID = 0; levelID < nlevel; ++levelID)
     {
-      long offset = gridsize * levelID;
-      double *tptr = t.ptr + offset;
-      double *sptr = s.ptr + offset;
-      double *tpotptr = tpot.ptr + offset;
-
-      for (long i = 0; i < gridsize; ++i)
+      const auto &tvec = t[levelID].vec;
+      const auto &svec = s[levelID].vec;
+      auto &tpotvec = tpot[levelID].vec;
+      const double t_missval = t[levelID].missval;
+      const double s_missval = s[levelID].missval;
+      const double tpot_missval = tpot[levelID].missval;
+      for (size_t i = 0; i < gridsize; ++i)
         {
-          if (DBL_IS_EQUAL(tptr[i], t.missval) || DBL_IS_EQUAL(sptr[i], s.missval))
+          if (DBL_IS_EQUAL(tvec[i], t_missval) || DBL_IS_EQUAL(svec[i], s_missval))
             {
-              tpotptr[i] = tpot.missval;
+              tpotvec[i] = tpot_missval;
             }
           else
             {
-              tpotptr[i] = adipot(tptr[i], sptr[i], pressure[levelID]);
+              tpotvec[i] = adipot(tvec[i], svec[i], pressure[levelID]);
             }
         }
     }
@@ -150,25 +153,19 @@ Adisit(void *process)
 {
   int nrecs;
   int varID, levelID;
-  size_t offset;
   int i;
-  size_t nmiss;
   int thoID = -1, saoID = -1;
   char varname[CDI_MAX_NAME], stdname[CDI_MAX_NAME];
-  double pin = -1;
-  double *single;
 
   cdoInitialize(process);
   const int ADISIT = cdoOperatorAdd("adisit", 1, 1, "");
   const int ADIPOT = cdoOperatorAdd("adipot", 1, 1, "");
 
-  UNUSED(ADIPOT);
-
   const int operatorID = cdoOperatorID();
 
-  if (operatorArgc() == 1) pin = parameter2double(operatorArgv()[0]);
+  const double pin = (operatorArgc() == 1) ? parameter2double(operatorArgv()[0]) : -1;
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
   const int vlistID1 = cdoStreamInqVlist(streamID1);
 
   const int nvars = vlistNvars(vlistID1);
@@ -180,14 +177,12 @@ Adisit(void *process)
         {
           vlistInqVarName(vlistID1, varID, varname);
           vlistInqVarStdname(vlistID1, varID, stdname);
-          strtolower(varname);
+          cstrToLowerCase(varname);
 
-          if (cstrIsEqual(varname, "s"))
+          if (cstrIsEqual(varname, "s") || cstrIsEqual(stdname, "sea_water_salinity"))
             code = 5;
           else if (cstrIsEqual(varname, "t"))
             code = 2;
-          else if (cstrIsEqual(stdname, "sea_water_salinity"))
-            code = 5;
 
           if (operatorID == ADISIT)
             {
@@ -199,14 +194,14 @@ Adisit(void *process)
             }
         }
 
-      if (code == 2)
+      if (code == 2 || (code == 20 && operatorID == ADIPOT))
         thoID = varID;
       else if (code == 5)
         saoID = varID;
     }
 
   if (saoID == -1) cdoAbort("Sea water salinity not found!");
-  if (thoID == -1) cdoAbort("Potential or Insitu temperature not found!");
+  if (thoID == -1) cdoAbort("%s temperature not found!", (operatorID == ADISIT) ? "Potential" : "Insitu");
 
   const int gridID = vlistGrid(vlistID1, 0);
   const size_t gridsize = vlist_check_gridsize(vlistID1);
@@ -217,7 +212,7 @@ Adisit(void *process)
   const int nlevel2 = zaxisInqSize(zaxisID);
 
   if (nlevel1 != nlevel2) cdoAbort("temperature and salinity have different number of levels!");
-  int nlevel = nlevel1;
+  const int nlevel = nlevel1;
 
   std::vector<double> pressure(nlevel);
   cdoZaxisInqLevels(zaxisID, pressure.data());
@@ -233,21 +228,16 @@ Adisit(void *process)
       for (i = 0; i < nlevel; ++i) cdoPrint("%5d  %g", i + 1, pressure[i]);
     }
 
-  Field tho, sao, tis;
-  fieldInit(tho);
-  fieldInit(sao);
-  fieldInit(tis);
-  tho.ptr = (double *) Malloc(gridsize * nlevel * sizeof(double));
-  sao.ptr = (double *) Malloc(gridsize * nlevel * sizeof(double));
-  tis.ptr = (double *) Malloc(gridsize * nlevel * sizeof(double));
-
-  tho.nmiss = 0;
-  sao.nmiss = 0;
-  tis.nmiss = 0;
-
-  tho.missval = vlistInqVarMissval(vlistID1, thoID);
-  sao.missval = vlistInqVarMissval(vlistID1, saoID);
-  tis.missval = tho.missval;
+  FieldVector tho(nlevel), sao(nlevel), tis(nlevel);
+  for (levelID = 0; levelID < nlevel; ++levelID)
+    {
+      tho[levelID].resize(gridsize);
+      sao[levelID].resize(gridsize);
+      tis[levelID].resize(gridsize);
+      tho[levelID].missval = vlistInqVarMissval(vlistID1, thoID);
+      sao[levelID].missval = vlistInqVarMissval(vlistID1, saoID);
+      tis[levelID].missval = tho[levelID].missval;
+    }
 
   int datatype = CDI_DATATYPE_FLT32;
   if (vlistInqVarDatatype(vlistID1, thoID) == CDI_DATATYPE_FLT64 && vlistInqVarDatatype(vlistID1, saoID) == CDI_DATATYPE_FLT64)
@@ -271,58 +261,50 @@ Adisit(void *process)
       vlistDefVarStdname(vlistID2, tisID2, "sea_water_potential_temperature");
     }
   vlistDefVarUnits(vlistID2, tisID2, "K");
-  vlistDefVarMissval(vlistID2, tisID2, tis.missval);
+  vlistDefVarMissval(vlistID2, tisID2, tis[0].missval);
   vlistDefVarDatatype(vlistID2, tisID2, datatype);
 
-  int saoID2 = vlistDefVar(vlistID2, gridID, zaxisID, TIME_VARYING);
+  const int saoID2 = vlistDefVar(vlistID2, gridID, zaxisID, TIME_VARYING);
   vlistDefVarParam(vlistID2, saoID2, cdiEncodeParam(5, 255, 255));
   vlistDefVarName(vlistID2, saoID2, "s");
   vlistDefVarLongname(vlistID2, saoID2, "Sea water salinity");
   vlistDefVarStdname(vlistID2, saoID2, "sea_water_salinity");
   vlistDefVarUnits(vlistID2, saoID2, "psu");
-  vlistDefVarMissval(vlistID2, saoID2, sao.missval);
+  vlistDefVarMissval(vlistID2, saoID2, sao[0].missval);
   vlistDefVarDatatype(vlistID2, saoID2, datatype);
 
   const int taxisID1 = vlistInqTaxis(vlistID1);
   const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; ++recID)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          offset = gridsize * levelID;
-
-          if (varID == thoID) pstreamReadRecord(streamID1, tho.ptr + offset, &tho.nmiss);
-          if (varID == saoID) pstreamReadRecord(streamID1, sao.ptr + offset, &sao.nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          if (varID == thoID) cdoReadRecord(streamID1, tho[levelID].vec.data(), &tho[levelID].nmiss);
+          if (varID == saoID) cdoReadRecord(streamID1, sao[levelID].vec.data(), &sao[levelID].nmiss);
         }
 
       if (operatorID == ADISIT)
-        calc_adisit(gridsize, nlevel, pressure.data(), tho, sao, tis);
+        calc_adisit(gridsize, nlevel, pressure, tho, sao, tis);
       else
-        calc_adipot(gridsize, nlevel, pressure.data(), tho, sao, tis);
+        calc_adipot(gridsize, nlevel, pressure, tho, sao, tis);
 
       for (levelID = 0; levelID < nlevel; ++levelID)
         {
-          offset = gridsize * levelID;
+          cdoDefRecord(streamID2, tisID2, levelID);
+          cdoWriteRecord(streamID2, tis[levelID].vec.data(), fieldNumMiss(tis[levelID]));
 
-          single = tis.ptr + offset;
-          nmiss = arrayNumMV(gridsize, single, tis.missval);
-          pstreamDefRecord(streamID2, tisID2, levelID);
-          pstreamWriteRecord(streamID2, single, nmiss);
-
-          single = sao.ptr + offset;
-          nmiss = arrayNumMV(gridsize, single, sao.missval);
-          pstreamDefRecord(streamID2, saoID2, levelID);
-          pstreamWriteRecord(streamID2, single, nmiss);
+          cdoDefRecord(streamID2, saoID2, levelID);
+          cdoWriteRecord(streamID2, sao[levelID].vec.data(), fieldNumMiss(sao[levelID]));
         }
 
       tsID++;
@@ -333,10 +315,6 @@ Adisit(void *process)
 
   vlistDestroy(vlistID2);
 
-  Free(tis.ptr);
-  Free(tho.ptr);
-  Free(sao.ptr);
-
   cdoFinish();
 
   return nullptr;
