@@ -33,24 +33,22 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "cdo_task.h"
-#include "pstream_int.h"
 
-#define streamOpenWrite cdoStreamOpenWrite
-#define streamDefVlist pstreamDefVlist
-#define streamDefTimestep pstreamDefTimestep
+#define streamOpenWrite cdoOpenWrite
+#define streamDefVlist cdoDefVlist
+#define streamDefTimestep cdoDefTimestep
 
 #include "afterburner.h"
 #include "constants.h"
 #include "compare.h"
 #include "cdo_options.h"
-#include "text.h"
+#include "mpmo_color.h"
+#include "cdi_lockedIO.h"
 
 #ifdef _OPENMP
 #include <omp.h>
 #endif
 
-
 int scan_par_obsolate(char *namelist, const char *name, int def);
 void scan_code(char *namelist, struct Variable *vars, int maxCodes, int *numCodes);
 int scan_par(int verbose, char *namelist, const char *name, int def);
@@ -66,6 +64,8 @@ struct RARG
   struct Control *globs;
 };
 
+void *afterReadTask = nullptr;
+
 static bool lstdout = true;
 
 static int Source = 0;
@@ -89,7 +89,7 @@ static int oVertID = -1;
 static bool Lhybrid2pressure = false;
 
 static int TsID;
-static bool lparallelread = true;
+bool afterReadAsync = true;
 
 #define TIMESTEP_INTERVAL -1
 #define MONTHLY_INTERVAL 0
@@ -279,7 +279,7 @@ after_SwitchFile(struct Control *globs)
       globs->Multi--;
     }
 
-  if (globs->Nfiles > 0) ifile = (char*) ifiles[--globs->Nfiles];
+  if (globs->Nfiles > 0) ifile = (char *) ifiles[--globs->Nfiles];
 
   fprintf(stderr, " Continuation file: %s\n", ifile);
 
@@ -364,8 +364,8 @@ after_setNextDate(struct Control *globs)
         }
       if (nrecs == 0) break;
 
-      int64_t vdate = taxisInqVdate(globs->taxisID);
-      int vtime = taxisInqVtime(globs->taxisID);
+      const int64_t vdate = taxisInqVdate(globs->taxisID);
+      const int vtime = taxisInqVtime(globs->taxisID);
       after_setDateTime(&globs->NextDate, vdate, vtime);
 
       for (int i = 0; i < nrqh; i++)
@@ -404,7 +404,7 @@ after_readTimestep(void *arg)
     {
       streamInqRecord(globs->istreamID, &varID, &levelID);
 
-      int code = vlistInqVarCode(globs->ivlistID, varID);
+      const int code = vlistInqVarCode(globs->ivlistID, varID);
       if (code <= 0 || code >= MaxCodes) continue;
 
       /* Skip records containing unneeded codes */
@@ -413,7 +413,7 @@ after_readTimestep(void *arg)
 
       vlistInqVar(globs->ivlistID, varID, &gridID, &zaxisID, &timeID);
 
-      int leveltype = zaxisInqType(zaxisID);
+      const int leveltype = zaxisInqType(zaxisID);
 
       /* Skip records with unselected levels */
 
@@ -423,7 +423,7 @@ after_readTimestep(void *arg)
       */
       if ((vars[code].ozaxisID != vars[code].izaxisID) && (leveltype == ZAXIS_PRESSURE))
         {
-          int level = (int) zaxisInqLevel(zaxisID, levelID);
+          const int level = (int) zaxisInqLevel(zaxisID, levelID);
           for (int i = 0; i < globs->NumLevelRequest; ++i)
             {
               if (IS_EQUAL(globs->LevelRequest[i], level))
@@ -476,8 +476,8 @@ static void
 after_defineNextTimestep(struct Control *globs)
 {
   static int otsID = 0;
-  int64_t vdate = after_getDate(globs->OldDate);
-  int vtime = after_getTime(globs->OldDate);
+  const int64_t vdate = after_getDate(globs->OldDate);
+  const int vtime = after_getTime(globs->OldDate);
   taxisDefVdate(globs->taxisID2, vdate);
   taxisDefVtime(globs->taxisID2, vtime);
 
@@ -485,8 +485,8 @@ after_defineNextTimestep(struct Control *globs)
     {
       if (otsID == 0)
         {
-          int nvars = vlistNvars(globs->ovlistID);
-          if (nvars == 0) Error("No variable selected!");
+          const int nvars = vlistNvars(globs->ovlistID);
+          if (nvars == 0) afterAbort("No variable selected!");
           vlistDefTaxis(globs->ovlistID, globs->taxisID2);
           streamDefVlist(globs->ostreamID, globs->ovlistID);
         }
@@ -513,7 +513,7 @@ after_setEndOfInterval(struct Control *globs, int nrecs)
       else if (globs->OutputInterval == UNLIM_INTERVAL)
         globs->EndOfInterval = false;
       else
-        Error("output interval %d not implemented!", globs->OutputInterval);
+        afterAbort("output interval %d not implemented!", globs->OutputInterval);
     }
 }
 
@@ -549,7 +549,6 @@ after_moveTimestep(struct Variable *vars)
 static void
 after_check_content(struct Variable *vars, int timestep)
 {
-  extern int labort_after;
   for (int code = 0; code < 272; code++)
     {
       /*  if ( code == GEOPOTENTIAL ) continue; */
@@ -570,7 +569,7 @@ after_check_content(struct Variable *vars, int timestep)
           if (vars[code].needed && !vars[code].selected && vars[code].spectral == nullptr && vars[code].hybrid == nullptr)
             {
               static bool lwarn = true;
-              if (lwarn) Warning("No humidity in data file, set to zero !");
+              if (lwarn) cdoWarning("No humidity in data file, set to zero !");
               lwarn = false;
               vars[code].needed = false;
             }
@@ -579,10 +578,7 @@ after_check_content(struct Variable *vars, int timestep)
         {
           if (vars[code].needed && !vars[code].comp && vars[code].spectral == nullptr && vars[code].hybrid == nullptr)
             {
-              if (labort_after)
-                Error("Code  %3d not found at timestep %d!", code, timestep);
-              else
-                Warning("Code  %3d not found at timestep %d!", code, timestep);
+              afterAbort("Code  %3d not found at timestep %d!", code, timestep);
             }
         }
     }
@@ -596,12 +592,12 @@ after_check_content(struct Variable *vars, int timestep)
   code = HALF_PRESS;
   if ( vars[code].needed && !vars[code].comp &&
        vars[code].spectral == nullptr && vars[code].hybrid == nullptr )
-    Error( "Hybrid model level not found!");
+    afterAbort( "Hybrid model level not found!");
 
   code = FULL_PRESS;
   if ( vars[code].needed && !vars[code].comp &&
        vars[code].spectral == nullptr && vars[code].hybrid == nullptr )
-    Error( "Hybrid model level not found!");
+    afterAbort( "Hybrid model level not found!");
   */
 }
 
@@ -614,14 +610,13 @@ after_control(struct Control *globs, struct Variable *vars)
   int rtime, vtime;
   int code;
   RARG rarg;
-  void *read_task = nullptr;
 
-  if (lparallelread)
+  if (afterReadAsync)
     {
-      read_task = cdo_task_new();
-      if (read_task == nullptr)
+      afterReadTask = cdo_task_new();
+      if (afterReadTask == nullptr)
         {
-          lparallelread = false;
+          afterReadAsync = false;
           cdoWarning("CDO tasks not available!");
         }
     }
@@ -685,11 +680,11 @@ after_control(struct Control *globs, struct Variable *vars)
 
       int status = -1;
 
-      if (tsFirst || !lparallelread)
+      if (tsFirst || !afterReadAsync)
         {
-          if (lparallelread)
+          if (afterReadAsync)
             {
-              cdo_task_start(read_task, after_readTimestep, &rarg);
+              cdo_task_start(afterReadTask, after_readTimestep, &rarg);
             }
           else
             {
@@ -698,17 +693,17 @@ after_control(struct Control *globs, struct Variable *vars)
 
           if (tsFirst && globs->Type > 0) after_legini_setup(globs, vars);
 
-          if (lparallelread)
+          if (afterReadAsync)
             {
-              status = *(int *) cdo_task_wait(read_task);
-              if (status < 0) Error("after_readTimestep error! (status = %d)", status);
+              status = *(int *) cdo_task_wait(afterReadTask);
+              if (status < 0) cdoAbort("after_readTimestep error! (status = %d)", status);
             }
           tsFirst = false;
         }
       else
         {
-          status = *(int *) cdo_task_wait(read_task);
-          if (status < 0) Error("after_readTimestep error! (status = %d)", status);
+          status = *(int *) cdo_task_wait(afterReadTask);
+          if (status < 0) cdoAbort("after_readTimestep error! (status = %d)", status);
         }
 
       nrecs = status;
@@ -718,9 +713,9 @@ after_control(struct Control *globs, struct Variable *vars)
 
       after_moveTimestep(vars);
 
-      if (nrecs && lparallelread)
+      if (nrecs && afterReadAsync)
         {
-          cdo_task_start(read_task, after_readTimestep, &rarg);
+          cdo_task_start(afterReadTask, after_readTimestep, &rarg);
         }
 
       after_setEndOfInterval(globs, nrecs);
@@ -753,7 +748,7 @@ after_control(struct Control *globs, struct Variable *vars)
       globs->OldDate = globs->NewDate;
     }
 
-  if (read_task) cdo_task_delete(read_task);
+  if (afterReadTask) cdo_task_delete(afterReadTask);
 }
 
 static void
@@ -903,12 +898,8 @@ after_setLevel(struct Control *globs)
 
           if (l != globs->NumLevelRequest)
             {
-              extern int labort_after;
               if (globs->Verbose) lprintf(stdout);
-              if (labort_after)
-                Error("Inconsistent or invalid level list!");
-              else
-                Warning("Inconsistent or invalid level list!");
+              afterAbort("Inconsistent or invalid level list!");
             }
 
           globs->NumLevelRequest = l;
@@ -938,7 +929,7 @@ after_defineLevel(struct Control *globs, struct Variable *vars)
           {
             if (oVertID == -1)
               {
-                if (globs->NumLevelRequest > globs->NumLevelFound) Error("Too much level requested");
+                if (globs->NumLevelRequest > globs->NumLevelFound) afterAbort("Too much level requested");
 
                 if (globs->NumLevelFound == globs->NumLevelRequest)
                   {
@@ -970,7 +961,7 @@ after_defineLevel(struct Control *globs, struct Variable *vars)
         else
           {
             zaxisName(zaxisInqType(iVertID), zaxistypename);
-            Error("%s level data unsupported for TYPE %d", zaxistypename, globs->Type);
+            afterAbort("%s level data unsupported for TYPE %d", zaxistypename, globs->Type);
           }
         break;
       }
@@ -1010,7 +1001,7 @@ after_defineLevel(struct Control *globs, struct Variable *vars)
 
         break;
       }
-    default: Error("TYPE %d unsupported", globs->Type);
+    default: afterAbort("TYPE %d unsupported", globs->Type);
     }
 }
 
@@ -1029,8 +1020,8 @@ after_defineGrid(struct Control *globs, struct Variable *vars)
       {
         if (specGridID == -1)
           {
-            if (globs->DimSP == 0) Error("dim spectral undefined");
-            if (globs->Truncation == 0) Error("truncation undefined");
+            if (globs->DimSP == 0) afterAbort("dim spectral undefined");
+            if (globs->Truncation == 0) afterAbort("truncation undefined");
 
             specGridID = gridCreate(GRID_SPECTRAL, globs->DimSP);
             gridDefTrunc(specGridID, globs->Truncation);
@@ -1045,8 +1036,8 @@ after_defineGrid(struct Control *globs, struct Variable *vars)
       {
         if (gaussGridID == -1)
           {
-            if (globs->Longitudes == 0) Error("number of longitudes undefined");
-            if (globs->Latitudes == 0) Error("number of latitudes undefined");
+            if (globs->Longitudes == 0) afterAbort("number of longitudes undefined");
+            if (globs->Latitudes == 0) afterAbort("number of latitudes undefined");
 
             gaussGridID = gridCreate(GRID_GAUSSIAN, globs->Longitudes * globs->Latitudes);
             gridDefXsize(gaussGridID, globs->Longitudes);
@@ -1060,8 +1051,8 @@ after_defineGrid(struct Control *globs, struct Variable *vars)
     case 40:
     case 60:
       {
-        if (globs->Fouriers == 0) Error("number of fourier coefficients undefined");
-        if (globs->Latitudes == 0) Error("number of latitudes undefined");
+        if (globs->Fouriers == 0) afterAbort("number of fourier coefficients undefined");
+        if (globs->Latitudes == 0) afterAbort("number of latitudes undefined");
 
         ogridID = gridCreate(GRID_FOURIER, globs->Fouriers * globs->Latitudes);
         gridDefXsize(ogridID, globs->Latitudes);
@@ -1072,14 +1063,14 @@ after_defineGrid(struct Control *globs, struct Variable *vars)
     case 41:
     case 61:
       {
-        if (globs->Latitudes == 0) Error("Number of latitudes undefined");
+        if (globs->Latitudes == 0) afterAbort("Number of latitudes undefined");
 
         ogridID = gridCreate(GRID_GAUSSIAN, globs->Latitudes);
         gridDefXsize(ogridID, 1);
         gridDefYsize(ogridID, globs->Latitudes);
         break;
       }
-    default: Error("TYPE %d unsupported", globs->Type);
+    default: afterAbort("TYPE %d unsupported", globs->Type);
     }
 
   if (ogridID != -1)
@@ -1091,7 +1082,7 @@ after_defineGrid(struct Control *globs, struct Variable *vars)
           }
       }
 
-  if (ogridID == -1) Error("out grid undefined");
+  if (ogridID == -1) afterAbort("out grid undefined");
 }
 
 static void
@@ -1124,11 +1115,11 @@ after_setCodes(struct Control *globs, struct Variable *vars, int maxCodes, int n
         longname[0] = 0;
         int tableID;
         int table = 0;
-        int varID = vars[code].ivarID;
+        const int varID = vars[code].ivarID;
 
         if (varID == CDI_UNDEFID)
           {
-            int modelID = vlistInqVarModel(globs->ivlistID, 0);
+            const int modelID = vlistInqVarModel(globs->ivlistID, 0);
             table = 128;
             tableID = tableInq(modelID, table, nullptr);
 
@@ -1163,15 +1154,12 @@ after_setCodes(struct Control *globs, struct Variable *vars, int maxCodes, int n
 static void
 after_checkNamelist(struct Control *globs)
 {
-  if (globs->Mean && globs->Type < 20)
-    {
-      Error("Mean is only available for TYPE >= 20!");
-    }
+  if (globs->Mean && globs->Type < 20) afterAbort("Mean is only available for TYPE >= 20!");
 
   if (globs->Extrapolate == false && globs->Type >= 30)
     {
-      if (globs->Type > 30) Error("EXTRAPOLATE = 0 is only available for TYPE = 30!");
-      if (globs->Mean) Error("EXTRAPOLATE = 0 is only available with MEAN = 0!");
+      if (globs->Type > 30) afterAbort("EXTRAPOLATE = 0 is only available for TYPE = 30!");
+      if (globs->Mean) afterAbort("EXTRAPOLATE = 0 is only available with MEAN = 0!");
     }
 }
 
@@ -1189,7 +1177,7 @@ after_parini(struct Control *globs, struct Variable *vars)
   else
     {
       fseek(stdin, 0L, SEEK_END);
-      long length = ftell(stdin);
+      const long length = ftell(stdin);
       if (length == 0L) fprintf(stderr, "\n stdin not connected\n");
       fseek(stdin, 0L, SEEK_SET);
     }
@@ -1233,13 +1221,13 @@ after_parini(struct Control *globs, struct Variable *vars)
   globs->Mean = scan_par(globs->Verbose, namelist, "mean", 0);
   globs->OutputInterval = scan_par(globs->Verbose, namelist, "interval", MONTHLY_INTERVAL);
 
-  if (globs->Mean >= 2) cdoAbort("Namelist parameter MEAN=%d out of bounds (0:1)", globs->Mean);
+  if (globs->Mean >= 2) afterAbort("Namelist parameter MEAN=%d out of bounds (0:1)", globs->Mean);
 
-  int fileFormat = scan_par(globs->Verbose, namelist, "format", -1);
-  int gribFormat = scan_par_obsolate(namelist, "grib", 0);
-  int cdfFormat = scan_par_obsolate(namelist, "netcdf", 0);
+  const int fileFormat = scan_par(globs->Verbose, namelist, "format", -1);
+  const int gribFormat = scan_par_obsolate(namelist, "grib", 0);
+  const int cdfFormat = scan_par_obsolate(namelist, "netcdf", 0);
 
-  if (gribFormat && cdfFormat) Error("GRIB or NetCDF?");
+  if (gribFormat && cdfFormat) afterAbort("GRIB or NetCDF?");
 
   switch (fileFormat)
     {
@@ -1251,13 +1239,13 @@ after_parini(struct Control *globs, struct Variable *vars)
     case 4: ofiletype = CDI_FILETYPE_NC2; break;
     case 5: ofiletype = CDI_FILETYPE_NC5; break;
     case 6: ofiletype = CDI_FILETYPE_NC4; break;
-    default: Error("unknown file format %d", fileFormat);
+    default: afterAbort("unknown file format %d", fileFormat);
     }
 
   if (gribFormat) ofiletype = CDI_FILETYPE_GRB;
   if (cdfFormat) ofiletype = CDI_FILETYPE_NC;
 
-  int precision = scan_par(globs->Verbose, namelist, "precision", 0);
+  const int precision = scan_par(globs->Verbose, namelist, "precision", 0);
   if (precision) switch (precision)
       {
       case 8: DataType = CDI_DATATYPE_PACK8; break;
@@ -1265,14 +1253,14 @@ after_parini(struct Control *globs, struct Variable *vars)
       case 24: DataType = CDI_DATATYPE_PACK24; break;
       case 32: DataType = CDI_DATATYPE_FLT32; break;
       case 64: DataType = CDI_DATATYPE_FLT64; break;
-      default: Error("unsupported data precision %d", precision);
+      default: afterAbort("unsupported data precision %d", precision);
       }
 
   globs->unitsel = scan_par(globs->Verbose, namelist, "unitsel", 0);
   globs->DayIn = scan_par(globs->Verbose, namelist, "dayinc", 30);
   globs->Extrapolate = scan_par(globs->Verbose, namelist, "extrapolate", 1);
   globs->Szip = scan_par(globs->Verbose, namelist, "szip", 0);
-  int mars = scan_par_obsolate(namelist, "mars", 0);
+  const int mars = scan_par_obsolate(namelist, "mars", 0);
 
   if (globs->Multi) --globs->Multi;
 
@@ -1346,31 +1334,30 @@ after_precntl(struct Control *globs, struct Variable *vars)
 {
   int l;
   int code = 0;
-  int gridID, zaxisID, varID, timeID;
-  int i, index, leveltype, gridtype;
-  int datasize, numlevel;
+  int varID;
+  int i, index;
   int vertfound = 0;
   int nhzaxis = 0;
   int FieldDim = 0;
 
-  int nvars = vlistNvars(globs->ivlistID);
-  int ngrids = vlistNgrids(globs->ivlistID);
-  int nverts = vlistNzaxis(globs->ivlistID);
-  int ntsteps = vlistNtsteps(globs->ivlistID);
+  const int nvars = vlistNvars(globs->ivlistID);
+  const int ngrids = vlistNgrids(globs->ivlistID);
+  const int nverts = vlistNzaxis(globs->ivlistID);
+  const int ntsteps = vlistNtsteps(globs->ivlistID);
 
   if (globs->Debug)
     {
-      Message("nvars      = %d", nvars);
-      Message("ngrids     = %d", ngrids);
-      Message("nverts     = %d", nverts);
-      Message("ntsteps    = %d", ntsteps);
+      fprintf(stderr, "nvars      = %d\n", nvars);
+      fprintf(stderr, "ngrids     = %d\n", ngrids);
+      fprintf(stderr, "nverts     = %d\n", nverts);
+      fprintf(stderr, "ntsteps    = %d\n", ntsteps);
     }
 
   for (index = 0; index < ngrids; index++)
     {
-      gridID = vlistGrid(globs->ivlistID, index);
-      gridtype = gridInqType(gridID);
-      datasize = gridInqSize(gridID);
+      const int gridID = vlistGrid(globs->ivlistID, index);
+      const int gridtype = gridInqType(gridID);
+      const int datasize = gridInqSize(gridID);
 
       if (datasize > FieldDim) FieldDim = datasize;
 
@@ -1387,7 +1374,7 @@ after_precntl(struct Control *globs, struct Variable *vars)
         }
     }
 
-  if (globs->Truncation == 0 && globs->Latitudes == 0) Error("Unsupported file structure (no spectral or Gaussian data found)!");
+  if (globs->Truncation == 0 && globs->Latitudes == 0) afterAbort("Unsupported file structure (no spectral or Gaussian data found)!");
 
   if (globs->Truncation == 0)
     {
@@ -1412,9 +1399,9 @@ after_precntl(struct Control *globs, struct Variable *vars)
 
   for (index = 0; index < nverts; index++)
     {
-      zaxisID = vlistZaxis(globs->ivlistID, index);
-      leveltype = zaxisInqType(zaxisID);
-      numlevel = zaxisInqSize(zaxisID);
+      const int zaxisID = vlistZaxis(globs->ivlistID, index);
+      const int leveltype = zaxisInqType(zaxisID);
+      const int numlevel = zaxisInqSize(zaxisID);
       /*
         printf("leveltype : %d %d\n", leveltype, zaxisInqSize(zaxisID));
       */
@@ -1430,11 +1417,11 @@ after_precntl(struct Control *globs, struct Variable *vars)
                     {
                       if (nvct == 0)
                         {
-                          if (numlevel != 191) Warning("VCT missing for hybrid level data with %d levels!", numlevel);
+                          if (numlevel != 191) cdoWarning("VCT missing for hybrid level data with %d levels!", numlevel);
                         }
                       else
                         {
-                          Warning("Skip %d hybrid level data with %d levels!", (nvct / 2 - 1), numlevel);
+                          cdoWarning("Skip %d hybrid level data with %d levels!", (nvct / 2 - 1), numlevel);
                         }
                       continue;
                     }
@@ -1442,7 +1429,7 @@ after_precntl(struct Control *globs, struct Variable *vars)
               else if (leveltype == ZAXIS_HYBRID && globs->nvct == zaxisInqVctSize(zaxisID))
                 continue;
 
-              if (iVertID != -1) Warning("More than %d different vertical grid structure found!", vertfound);
+              if (iVertID != -1) cdoWarning("More than %d different vertical grid structure found!", vertfound);
 
               vertfound++;
 
@@ -1469,12 +1456,12 @@ after_precntl(struct Control *globs, struct Variable *vars)
                         }
                       else
                         {
-                          Error("VCT not defined in inputfile!");
+                          afterAbort("VCT not defined in inputfile!");
                         }
                     }
 
                   if (numlevel != (globs->nvct / 2 - 1))
-                    Error("Number of hybrid levels %d does not match VCT levels %d", numlevel, globs->nvct / 2 - 1);
+                    afterAbort("Number of hybrid levels %d does not match VCT levels %d", numlevel, globs->nvct / 2 - 1);
 
                   if (globs->Debug)
                     for (i = 0; i < globs->nvct / 2; i++)
@@ -1486,7 +1473,7 @@ after_precntl(struct Control *globs, struct Variable *vars)
         }
     }
 
-  if (nhzaxis > 0 && globs->nvct == 0) Error("VCT missing!");
+  if (nhzaxis > 0 && globs->nvct == 0) afterAbort("VCT missing!");
 
   globs->NumLevel = globs->NumLevelFound;
 
@@ -1499,8 +1486,8 @@ after_precntl(struct Control *globs, struct Variable *vars)
 
   if (globs->Verbose)
     {
-      int instID = vlistInqVarInstitut(globs->ivlistID, 0);
-      int modelID = vlistInqVarModel(globs->ivlistID, 0);
+      const int instID = vlistInqVarInstitut(globs->ivlistID, 0);
+      const int modelID = vlistInqVarModel(globs->ivlistID, 0);
 
       lprintf(stdout);
       fprintf(stdout, " Institute : ");
@@ -1531,16 +1518,17 @@ after_precntl(struct Control *globs, struct Variable *vars)
 
   for (varID = 0; varID < nvars; varID++)
     {
+      int gridID, zaxisID, timeID;
       vlistInqVar(globs->ivlistID, varID, &gridID, &zaxisID, &timeID);
-      code = vlistInqVarCode(globs->ivlistID, varID);
+      const int code = vlistInqVarCode(globs->ivlistID, varID);
       if (code <= 0 || code >= MaxCodes)
         {
-          Warning("Code number %d out of range, variable ignored!", code);
+          cdoWarning("Code number %d out of range, variable ignored!", code);
           continue;
         }
-      gridtype = gridInqType(gridID);
-      numlevel = zaxisInqSize(zaxisID);
-      leveltype = zaxisInqType(zaxisID);
+      const int gridtype = gridInqType(gridID);
+      const int numlevel = zaxisInqSize(zaxisID);
+      const int leveltype = zaxisInqType(zaxisID);
 
       vars[code].ivarID = varID;
       vars[code].igridID = gridID;
@@ -1554,7 +1542,7 @@ after_precntl(struct Control *globs, struct Variable *vars)
         fprintf(stderr, "Code %3d  Levels = %3d  LevelType = %3d  GridType = %3d\n", code, numlevel, leveltype, gridtype);
     }
 
-  if (globs->Debug) Message("FieldDim = %d", FieldDim);
+  if (globs->Debug) fprintf(stderr, "FieldDim = %d\n", FieldDim);
 
   globs->Field = (double *) Malloc(FieldDim * sizeof(double));
 
@@ -1574,10 +1562,6 @@ static void
 after_postcntl(struct Control *globs, struct Variable *vars)
 {
   int code = 0;
-  int gridID, zaxisID;
-  int ovarID, ogridID, ozaxisID;
-  int ovarID2;
-  int ivarID, instID, modelID, tableID;
   char name[CDI_MAX_NAME], longname[CDI_MAX_NAME], units[CDI_MAX_NAME];
   int datatype;
 
@@ -1586,8 +1570,8 @@ after_postcntl(struct Control *globs, struct Variable *vars)
     for (code = 0; code < MaxCodes; code++)
       if (vars[code].detected)
         {
-          gridID = vars[code].igridID;
-          zaxisID = vars[code].izaxisID;
+          const int gridID = vars[code].igridID;
+          const int zaxisID = vars[code].izaxisID;
           zaxisName(zaxisInqType(zaxisID), zaxistypename);
           fprintf(stderr, " Detected Code %3d  grid %-8s size %5zu  level %2d %-8s\n", code, gridNamePtr(gridInqType(gridID)),
                   gridInqSize(gridID), zaxisInqSize(zaxisID), zaxistypename);
@@ -1607,28 +1591,28 @@ after_postcntl(struct Control *globs, struct Variable *vars)
         name[0] = 0;
         longname[0] = 0;
         units[0] = 0;
-        ivarID = vars[code].ivarID;
-        ogridID = vars[code].ogridID;
-        ozaxisID = vars[code].ozaxisID;
+        const int ivarID = vars[code].ivarID;
+        const int ogridID = vars[code].ogridID;
+        const int ozaxisID = vars[code].ozaxisID;
 
         if (ogridID == -1)
           {
             /*
-            Warning( "undefined grid for code %d", code);
+            cdoWarning( "undefined grid for code %d", code);
             */
             continue;
           }
         if (ozaxisID == -1)
           {
             /*
-            Warning( "undefined level for code %d", code);
+            cdoWarning( "undefined level for code %d", code);
             */
             continue;
           }
 
-        instID = vlistInqVarInstitut(globs->ivlistID, ivarID);
-        modelID = vlistInqVarModel(globs->ivlistID, ivarID);
-        tableID = vlistInqVarTable(globs->ivlistID, ivarID);
+        const int instID = vlistInqVarInstitut(globs->ivlistID, ivarID);
+        const int modelID = vlistInqVarModel(globs->ivlistID, ivarID);
+        int tableID = vlistInqVarTable(globs->ivlistID, ivarID);
 
         vars[code].missval = vlistInqVarMissval(globs->ivlistID, ivarID);
         vars[code].samp = nullptr;
@@ -1652,7 +1636,7 @@ after_postcntl(struct Control *globs, struct Variable *vars)
         if (globs->Mean != 2)
           {
             vlistDefTaxis(globs->ovlistID, globs->taxisID2);
-            ovarID = vlistDefVar(globs->ovlistID, ogridID, ozaxisID, TIME_VARYING);
+            const int ovarID = vlistDefVar(globs->ovlistID, ogridID, ozaxisID, TIME_VARYING);
             if (globs->Mean) vlistDefVarTsteptype(globs->ovlistID, ovarID, TSTEP_AVG);
             vlistDefVarCode(globs->ovlistID, ovarID, code);
             vars[code].ovarID = ovarID;
@@ -1669,7 +1653,7 @@ after_postcntl(struct Control *globs, struct Variable *vars)
         if (globs->Mean >= 2)
           {
             vlistDefTaxis(globs->ovlistID2, globs->taxisID2);
-            ovarID2 = vlistDefVar(globs->ovlistID2, ogridID, ozaxisID, TIME_VARYING);
+            const int ovarID2 = vlistDefVar(globs->ovlistID2, ogridID, ozaxisID, TIME_VARYING);
             if (globs->Mean) vlistDefVarTsteptype(globs->ovlistID2, ovarID2, TSTEP_AVG);
             vlistDefVarCode(globs->ovlistID2, ovarID2, code);
             vars[code].ovarID2 = ovarID2;
@@ -1689,8 +1673,8 @@ after_postcntl(struct Control *globs, struct Variable *vars)
     for (code = 0; code < MaxCodes; code++)
       if (vars[code].selected)
         {
-          gridID = vars[code].ogridID;
-          zaxisID = vars[code].ozaxisID;
+          const int gridID = vars[code].ogridID;
+          const int zaxisID = vars[code].ozaxisID;
           zaxisName(zaxisInqType(zaxisID), zaxistypename);
           fprintf(stderr, " Selected Code %3d  grid %-8s size %5zu  level %2d %-8s\n", code, gridNamePtr(gridInqType(gridID)),
                   gridInqSize(gridID), zaxisInqSize(zaxisID), zaxistypename);
@@ -1701,11 +1685,10 @@ static void
 after_readVct(struct Control *globs, const char *vctfile)
 {
   char line[1024];
-  int i, n, nlines = 0;
-  double va, vb;
+  int nlines = 0;
 
   FILE *fp = fopen(vctfile, "r");
-  if (fp == nullptr) SysError("Open failed on %s", vctfile);
+  if (fp == nullptr) cdoSysError("Open failed on %s", vctfile);
 
   while (fgets(line, 1023, fp))
     {
@@ -1718,10 +1701,12 @@ after_readVct(struct Control *globs, const char *vctfile)
 
   rewind(fp);
 
-  i = 0;
+  int i = 0;
   while (fgets(line, 1023, fp))
     {
       if (line[0] == '#' || line[0] == '\0') continue;
+      int n;
+      double va, vb;
       sscanf(line, "%d %lg %lg", &n, &va, &vb);
       globs->vct[i] = va;
       globs->vct[i + globs->nvct / 2] = vb;
@@ -1744,8 +1729,8 @@ after_control_init(struct Control *globs)
   globs->Szip = false;
 
   globs->istreamID = CDI_UNDEFID;
-  globs->ostreamID = CDI_UNDEFID;
-  globs->ostreamID2 = CDI_UNDEFID;
+  globs->ostreamID = CDO_STREAM_UNDEF;
+  globs->ostreamID2 = CDO_STREAM_UNDEF;
   globs->ivlistID = CDI_UNDEFID;
   globs->ovlistID = CDI_UNDEFID;
   globs->ovlistID2 = CDI_UNDEFID;
@@ -1771,10 +1756,8 @@ after_variable_init(struct Variable *vars)
 static void
 after_processing(struct Control *globs, struct Variable *vars)
 {
-  int i;
-
   //#if defined(PSTREAM_H)
-  //  globs->istreamID = streamOpenReadLocked(cdoStreamName(0));
+  //  globs->istreamID = streamOpenReadLocked(0);
   //#else
   globs->istreamID = streamOpenReadLocked(ifile);
   //#endif
@@ -1786,9 +1769,9 @@ after_processing(struct Control *globs, struct Variable *vars)
 
   if (globs->Mean != 2)
     {
-      globs->ostreamID = cdoStreamOpenWrite(cdoStreamName(ofileidx), ofiletype);
+      globs->ostreamID = cdoOpenWrite(ofileidx, ofiletype);
 
-      if (globs->Szip) streamDefCompType(globs->ostreamID, CDI_COMPRESS_SZIP);
+      if (globs->Szip) cdoDefCompType(globs->ostreamID, CDI_COMPRESS_SZIP);
 
       globs->ovlistID = vlistCreate();
     }
@@ -1805,10 +1788,10 @@ after_processing(struct Control *globs, struct Variable *vars)
   after_setCodes(globs, vars, MaxCodes, globs->NumCodesRequest);
 
   if (globs->unitsel == 2)
-    for (i = 0; i < globs->NumLevelRequest; i++) globs->LevelRequest[i] = globs->LevelRequest[i] * 1000;
+    for (int i = 0; i < globs->NumLevelRequest; i++) globs->LevelRequest[i] = globs->LevelRequest[i] * 1000;
 
   if (!globs->AnalysisData)
-    for (i = 0; i < globs->NumLevelRequest; i++)
+    for (int i = 0; i < globs->NumLevelRequest; i++)
       {
         if ((globs->LevelRequest[i] >= 65535) && globs->unitsel && ofiletype == CDI_FILETYPE_GRB)
           {
@@ -1843,10 +1826,10 @@ after_processing(struct Control *globs, struct Variable *vars)
   if (globs->Type == 10 || globs->Type == 40 || globs->Type == 60)
     {
       if (ofiletype == CDI_FILETYPE_GRB)
-        Error("Can't write fourier coefficients to GRIB!");
+        afterAbort("Can't write fourier coefficients to GRIB!");
       else if (ofiletype == CDI_FILETYPE_NC || ofiletype == CDI_FILETYPE_NC2 || ofiletype == CDI_FILETYPE_NC4
                || ofiletype == CDI_FILETYPE_NC4C || ofiletype == CDI_FILETYPE_NC5)
-        Error("Can't write fourier coefficients to NetCDF!");
+        afterAbort("Can't write fourier coefficients to NetCDF!");
     }
 
   filename = strrchr(ifile, '/');
@@ -1855,8 +1838,9 @@ after_processing(struct Control *globs, struct Variable *vars)
   else
     filename++;
 
-  if (globs->Type >= 30 && globs->Type < 50 && (vars[DIVERGENCE].selected || vars[VELOPOT].selected || vars[VORTICITY].selected
-                                                || vars[STREAM].selected || globs->AnalysisData))
+  if (globs->Type >= 30 && globs->Type < 50
+      && (vars[DIVERGENCE].selected || vars[VELOPOT].selected || vars[VORTICITY].selected || vars[STREAM].selected
+          || globs->AnalysisData))
     {
       /*
       int newtype = 0;
@@ -1911,10 +1895,9 @@ after_processing(struct Control *globs, struct Variable *vars)
 
   after_control(globs, vars);
 
-  if (globs->ostreamID != CDI_UNDEFID) cdoStreamClose(globs->ostreamID);
+  if (globs->ostreamID != CDO_STREAM_UNDEF) cdoStreamClose(globs->ostreamID);
 
   processDefVarNum(vlistNvars(globs->ivlistID));
-  processSelf().addNvals((int)streamNvals(globs->istreamID));
 
   streamClose(globs->istreamID);
 
@@ -1961,7 +1944,7 @@ Afterburner(void *process)
 
   if (CdoDefault::FileType != CDI_UNDEFID) ofiletype = CdoDefault::FileType;
 
-  int streamCnt = cdoStreamCnt();
+  const int streamCnt = cdoStreamCnt();
   int nfiles = streamCnt - 1;
 
   ofileidx = nfiles;
@@ -1972,7 +1955,7 @@ Afterburner(void *process)
   globs->Nfiles = nfiles - 1;
   if (globs->Nfiles > 0)
     {
-      if (globs->Multi > 0) Error("Namelist parameter MULTI works only with one inputfile");
+      if (globs->Multi > 0) afterAbort("Namelist parameter MULTI works only with one inputfile");
 
       ifiles = (const char **) Malloc(globs->Nfiles * sizeof(char *));
       for (int i = 0; i < globs->Nfiles; ++i) ifiles[i] = cdoGetStreamName(--nfiles);
