@@ -32,8 +32,6 @@
 
 #include "cdo_int.h"
 #include "cdo_options.h"
-#include "pstream_int.h"
-
 
 void *
 Arith(void *process)
@@ -47,7 +45,6 @@ Arith(void *process)
     FILL_FILE
   };
   int filltype = FILL_NONE;
-  size_t nmiss;
   int nrecs;
   int varID, levelID;
   int nlevels2 = 1;
@@ -75,11 +72,11 @@ Arith(void *process)
   const bool opercplx = cdoOperatorF2(operatorID);
   operatorCheckArgc(0);
 
-  int streamID1 = cdoStreamOpenRead(0);
-  int streamID2 = cdoStreamOpenRead(1);
+  CdoStreamID streamID1 = cdoOpenRead(0);
+  CdoStreamID streamID2 = cdoOpenRead(1);
 
-  int streamIDx1 = streamID1;
-  int streamIDx2 = streamID2;
+  CdoStreamID streamIDx1 = streamID1;
+  CdoStreamID streamIDx2 = streamID2;
 
   Field field1, field2;
   Field *fieldx1 = &field1;
@@ -158,10 +155,8 @@ Arith(void *process)
   if (nwpv == 2 && !opercplx) cdoAbort("Fields with complex numbers are not supported by this operator!");
   const size_t gridsizemax = nwpv * vlistGridsizeMax(vlistIDx1);
 
-  fieldInit(field1);
-  fieldInit(field2);
-  field1.ptr = (double *) Malloc(gridsizemax * sizeof(double));
-  field2.ptr = (double *) Malloc(gridsizemax * sizeof(double));
+  field1.resize(gridsizemax);
+  field2.resize(gridsizemax);
   if (filltype == FILL_VAR || filltype == FILL_VARTS)
     {
       vardata2.resize(gridsizemax * nlevels2);
@@ -215,8 +210,8 @@ Arith(void *process)
   const int taxisID3 = taxisDuplicate(taxisIDx1);
   vlistDefTaxis(vlistID3, taxisID3);
 
-  const int streamID3 = cdoStreamOpenWrite(2);
-  pstreamDefVlist(streamID3, vlistID3);
+  CdoStreamID streamID3 = cdoOpenWrite(2);
+  cdoDefVlist(streamID3, vlistID3);
 
   int tsID = 0;
   int tsID2 = 0;
@@ -237,7 +232,7 @@ Arith(void *process)
                 {
                   tsID2 = 0;
                   cdoStreamClose(streamID2);
-                  streamID2 = cdoStreamOpenRead(1);
+                  streamID2 = cdoOpenRead(1);
                   streamIDx2 = streamID2;
 
                   vlistID2 = cdoStreamInqVlist(streamID2);
@@ -254,13 +249,12 @@ Arith(void *process)
         }
 
       taxisCopyTimestep(taxisID3, taxisIDx1);
-      pstreamDefTimestep(streamID3, tsID);
+      cdoDefTimestep(streamID3, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamIDx1, &varID, &levelID);
-          pstreamReadRecord(streamIDx1, fieldx1->ptr, &nmiss);
-          fieldx1->nmiss = nmiss;
+          cdoInqRecord(streamIDx1, &varID, &levelID);
+          cdoReadRecord(streamIDx1, fieldx1->vec.data(), &fieldx1->nmiss);
           int varID2 = varID;
 
           if (tsID == 0 || filltype == FILL_NONE || filltype == FILL_FILE || filltype == FILL_VARTS)
@@ -268,9 +262,8 @@ Arith(void *process)
               const bool lstatus = nlevels2 > 1 ? varID == 0 : recID == 0;
               if (lstatus || (filltype != FILL_VAR && filltype != FILL_VARTS))
                 {
-                  pstreamInqRecord(streamIDx2, &varID2, &levelID2);
-                  pstreamReadRecord(streamIDx2, fieldx2->ptr, &nmiss);
-                  fieldx2->nmiss = nmiss;
+                  cdoInqRecord(streamIDx2, &varID2, &levelID2);
+                  cdoReadRecord(streamIDx2, fieldx2->vec.data(), &fieldx2->nmiss);
                   if (varID != varID2) cdoAbort("Internal error, varIDs of input streams differ!");
                   if (levelID != levelID2) cdoAbort("Internal error, levelIDs of input streams differ!");
                 }
@@ -279,14 +272,14 @@ Arith(void *process)
                 {
                   const size_t gridsize = nwpv * gridInqSize(vlistInqVarGrid(vlistIDx2, varID));
                   const size_t offset = gridsize * levelID;
-                  arrayCopy(gridsize, fieldx2->ptr, &vardata[varID][offset]);
+                  arrayCopy(gridsize, fieldx2->vec.data(), &vardata[varID][offset]);
                   varnmiss[varID][levelID] = fieldx2->nmiss;
                 }
               else if (lstatus && (filltype == FILL_VAR || filltype == FILL_VARTS))
                 {
                   const size_t gridsize = nwpv * gridInqSize(vlistInqVarGrid(vlistIDx2, 0));
                   const size_t offset = gridsize * levelID2;
-                  arrayCopy(gridsize, fieldx2->ptr, &vardata2[offset]);
+                  arrayCopy(gridsize, fieldx2->vec.data(), &vardata2[offset]);
                   varnmiss2[levelID2] = fieldx2->nmiss;
                 }
             }
@@ -294,7 +287,7 @@ Arith(void *process)
             {
               const size_t gridsize = nwpv * gridInqSize(vlistInqVarGrid(vlistIDx2, varID2));
               const size_t offset = gridsize * levelID;
-              arrayCopy(gridsize, &vardata[varID][offset], fieldx2->ptr);
+              arrayCopy(gridsize, &vardata[varID][offset], fieldx2->vec.data());
               fieldx2->nmiss = varnmiss[varID][levelID];
             }
 
@@ -306,7 +299,7 @@ Arith(void *process)
               levelID2 = (nlevels2 > 1) ? levelID : 0;
               const size_t gridsize = nwpv * gridInqSize(vlistInqVarGrid(vlistIDx2, 0));
               const size_t offset = gridsize * levelID2;
-              arrayCopy(gridsize, &vardata2[offset], fieldx2->ptr);
+              arrayCopy(gridsize, &vardata2[offset], fieldx2->vec.data());
               fieldx2->nmiss = varnmiss2[levelID2];
               fieldx2->grid = vlistInqVarGrid(vlistIDx2, 0);
               fieldx2->missval = vlistInqVarMissval(vlistIDx2, 0);
@@ -318,12 +311,12 @@ Arith(void *process)
             }
 
           if (nwpv == 2)
-            farfuncplx(field1, field2, operfunc);
+            vfarfuncplx(field1, field2, operfunc);
           else
-            farfun(field1, field2, operfunc);
+            vfarfun(field1, field2, operfunc);
 
-          pstreamDefRecord(streamID3, varID, levelID);
-          pstreamWriteRecord(streamID3, field1.ptr, field1.nmiss);
+          cdoDefRecord(streamID3, varID, levelID);
+          cdoWriteRecord(streamID3, field1.vec.data(), field1.nmiss);
         }
 
       tsID++;
@@ -336,9 +329,6 @@ Arith(void *process)
 
   vlistDestroy(vlistID3);
 
-  if (field1.ptr) Free(field1.ptr);
-  if (field2.ptr) Free(field2.ptr);
-
   cdoFinish();
 
   return nullptr;
