@@ -29,21 +29,19 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
 
 static void
 fill_vars(int vlistID, std::vector<bool> &vars)
 {
-  int varID;
-  int nvars = vlistNvars(vlistID);
+  const int nvars = vlistNvars(vlistID);
   vars.resize(nvars);
 
   if (Options::cdoNumVarnames)
     {
-      bool lfound = false;
+      bool found = false;
       char varname[CDI_MAX_NAME];
 
-      for (varID = 0; varID < nvars; ++varID)
+      for (int varID = 0; varID < nvars; ++varID)
         {
           vars[varID] = false;
 
@@ -53,16 +51,16 @@ fill_vars(int vlistID, std::vector<bool> &vars)
             if (strcmp(varname, Options::cdoVarnames[i]) == 0)
               {
                 vars[varID] = true;
-                lfound = true;
+                found = true;
                 break;
               }
         }
 
-      if (!lfound) cdoAbort("Variable %s%s not found!", Options::cdoVarnames[0], Options::cdoNumVarnames > 1 ? ",..." : "");
+      if (!found) cdoAbort("Variable %s%s not found!", Options::cdoVarnames[0], Options::cdoNumVarnames > 1 ? ",..." : "");
     }
   else
     {
-      for (varID = 0; varID < nvars; ++varID) vars[varID] = true;
+      for (int varID = 0; varID < nvars; ++varID) vars[varID] = true;
     }
 }
 
@@ -97,7 +95,7 @@ Arithc(void *process)
   rconstcplx[1] = 0;
   if (operatorArgc() == 2) rconstcplx[1] = parameter2double(operatorArgv()[1]);
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -109,47 +107,46 @@ Arithc(void *process)
   const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   const size_t nwpv = (vlistNumber(vlistID1) == CDI_COMP) ? 2 : 1;
   if (nwpv == 2 && !opercplx) cdoAbort("Fields with complex numbers are not supported by this operator!");
   const size_t gridsizemax = nwpv * vlistGridsizeMax(vlistID1);
 
   Field field;
-  fieldInit(field);
-  field.ptr = (double *) Malloc(gridsizemax * sizeof(double));
-  field.weight = nullptr;
+  field.resize(gridsizemax);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, field.ptr, &field.nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
 
           if (vars[varID])
             {
+              const int nwpv = vlistInqNWPV(vlistID1, varID);
               const int gridID = vlistInqVarGrid(vlistID1, varID);
-              const size_t gridsize = nwpv * gridInqSize(gridID);
               field.grid = gridID;
+              field.size = nwpv * gridInqSize(gridID);
               field.missval = vlistInqVarMissval(vlistID1, varID);
 
               if (nwpv == 2)
-                farcfuncplx(field, rconstcplx, operfunc);
+                vfarcfuncplx(field, rconstcplx, operfunc);
               else
-                farcfun(field, rconst, operfunc);
+                vfarcfun(field, rconst, operfunc);
 
               // recalculate number of missing values
-              field.nmiss = arrayNumMV(gridsize, field.ptr, field.missval);
+              field.nmiss = fieldNumMiss(field);
             }
 
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, field.ptr, field.nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, field.vec.data(), field.nmiss);
         }
 
       tsID++;
@@ -158,8 +155,6 @@ Arithc(void *process)
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
-  if (field.ptr) Free(field.ptr);
-
   vlistDestroy(vlistID2);
 
   cdoFinish();
