@@ -30,7 +30,6 @@
 
 #include "cdo_int.h"
 #include "calendar.h"
-#include "pstream_int.h"
 
 static double
 dayofyear(int calendar, int64_t vdate, int vtime)
@@ -67,8 +66,6 @@ Arithdays(void *process)
   int nrecs;
   int varID, levelID;
   int year, month, day;
-  size_t nmiss;
-  double rconst;
 
   cdoInitialize(process);
 
@@ -84,7 +81,7 @@ Arithdays(void *process)
   const int operfunc = cdoOperatorF1(operatorID);
   const int operfunc2 = cdoOperatorF2(operatorID);
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -95,15 +92,13 @@ Arithdays(void *process)
 
   const int calendar = taxisInqCalendar(taxisID1);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   const size_t gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field;
-  fieldInit(field);
-  field.ptr = (double *) Malloc(gridsizemax * sizeof(double));
-  field.weight = nullptr;
+  field.resize(gridsizemax);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
@@ -112,34 +107,30 @@ Arithdays(void *process)
       const int vtime = taxisInqVtime(taxisID1);
 
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       cdiDecodeDate(vdate, &year, &month, &day);
 
+      double rconst;
       if (operatorID == MULDOY)
-        {
-          rconst = dayofyear(calendar, vdate, vtime);
-        }
+        rconst = dayofyear(calendar, vdate, vtime);
       else
-        {
-          rconst = (operfunc2 == func_month) ? days_per_month(calendar, year, month) : days_per_year(calendar, year);
-        }
+        rconst = (operfunc2 == func_month) ? days_per_month(calendar, year, month) : days_per_year(calendar, year);
 
       if (Options::cdoVerbose) cdoPrint("calendar %d  year %d  month %d  result %g", calendar, year, month, rconst);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, field.ptr, &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
 
-          field.nmiss = nmiss;
           field.grid = vlistInqVarGrid(vlistID1, varID);
           field.missval = vlistInqVarMissval(vlistID1, varID);
 
-          farcfun(field, rconst, operfunc);
+          vfarcfun(field, rconst, operfunc);
 
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, field.ptr, field.nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, field.vec.data(), field.nmiss);
         }
 
       tsID++;
@@ -148,8 +139,6 @@ Arithdays(void *process)
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
-  if (field.ptr) Free(field.ptr);
-
   cdoFinish();
 
   return nullptr;
