@@ -26,8 +26,8 @@
 #include "cdo_options.h"
 
 #include "cdo_int.h"
-#include "pstream_int.h"
-#include "grid.h"
+
+#include <mpim_grid.h>
 
 void *
 Arithlat(void *process)
@@ -47,7 +47,7 @@ Arithlat(void *process)
   const int operatorID = cdoOperatorID();
   const int operfunc = cdoOperatorF1(operatorID);
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -56,8 +56,8 @@ Arithlat(void *process)
   const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   const size_t gridsizemax = vlistGridsizeMax(vlistID1);
   std::vector<double> array(gridsizemax);
@@ -66,12 +66,12 @@ Arithlat(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, &array[0], &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, &array[0], &nmiss);
 
           int gridID = vlistInqVarGrid(vlistID1, varID);
           size_t gridsize = 0;
@@ -123,8 +123,8 @@ Arithlat(void *process)
 
           for (size_t i = 0; i < gridsize; ++i) array[i] *= scale[i];
 
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, &array[0], nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, &array[0], nmiss);
         }
 
       tsID++;
