@@ -19,7 +19,6 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
 #include "process_int.h"
 #include "timer.h"
 #include "util_files.h"
@@ -49,7 +48,6 @@ const char *
 datatypestr(int datatype)
 {
   static char str[20];
-
   str[0] = 0;
   snprintf(str, sizeof(str), "%d bit packed", datatype);
 
@@ -94,13 +92,10 @@ CDIread(void *process)
   size_t nmiss;
   int nrecs;
   int filetype = -1, datatype = -1;
-  int nruns = 1;
   char sinfo[64];
   off_t nvalues = 0;
   double file_size = 0, data_size = 0;
-  double tw, tw0, t0, twsum = 0;
-  float *farray = nullptr;
-  double *darray = nullptr;
+  double twsum = 0;
 
   sinfo[0] = 0;
 
@@ -110,8 +105,7 @@ CDIread(void *process)
 
   if (operatorArgc() > 1) cdoAbort("Too many arguments!");
 
-  if (operatorArgc() == 1) nruns = parameter2int(operatorArgv()[0]);
-
+  int nruns = (operatorArgc() == 1) ? parameter2int(operatorArgv()[0]) : 1;
   if (nruns < 0) nruns = 0;
   if (nruns > 99) nruns = 99;
 
@@ -121,23 +115,26 @@ CDIread(void *process)
 
   for (int irun = 0; irun < nruns; ++irun)
     {
-      tw0 = timer_val(timer_read);
+      double tw0 = timer_val(timer_read);
       data_size = 0;
       nvalues = 0;
 
-      int streamID = cdoStreamOpenRead(cdoStreamName(0));
-
+      CdoStreamID streamID = cdoOpenRead(0);
       int vlistID = cdoStreamInqVlist(streamID);
 
-      filetype = pstreamInqFiletype(streamID);
+      filetype = cdoInqFiletype(streamID);
       datatype = vlistInqVarDatatype(vlistID, 0);
 
       size_t gridsize = vlistGridsizeMax(vlistID);
 
-      if (darray == nullptr) darray = (double *) Malloc(gridsize * sizeof(double));
-      if (farray == nullptr && memtype == MEMTYPE_FLOAT) farray = (float *) Malloc(gridsize * sizeof(float));
+      std::vector<float> farray;
+      std::vector<double> darray;
+      if (memtype == MEMTYPE_FLOAT)
+        farray.resize(gridsize);
+      else
+        darray.resize(gridsize);
 
-      t0 = timer_val(timer_read);
+      double t0 = timer_val(timer_read);
 
       int tsID = 0;
       while ((nrecs = cdoStreamInqTimestep(streamID, tsID)))
@@ -145,27 +142,27 @@ CDIread(void *process)
 
           for (int recID = 0; recID < nrecs; recID++)
             {
-              pstreamInqRecord(streamID, &varID, &levelID);
+              cdoInqRecord(streamID, &varID, &levelID);
 
               gridsize = gridInqSize(vlistInqVarGrid(vlistID, varID));
               nvalues += gridsize;
 
               if (memtype == MEMTYPE_FLOAT)
                 {
-                  pstreamReadRecordF(streamID, farray, &nmiss);
+                  cdoReadRecordF(streamID, farray.data(), &nmiss);
                   //  for ( size_t i = 0; i < gridsize; ++i ) darray[i] = farray[i];
                   data_size += gridsize * 4;
                 }
               else
                 {
-                  pstreamReadRecord(streamID, darray, &nmiss);
+                  cdoReadRecord(streamID, darray.data(), &nmiss);
                   data_size += gridsize * 8;
                 }
             }
 
           if (Options::cdoVerbose)
             {
-              tw = timer_val(timer_read) - t0;
+              const double tw = timer_val(timer_read) - t0;
               t0 = timer_val(timer_read);
               cdoPrint("Timestep %d: %.2f seconds", tsID + 1, tw);
             }
@@ -175,7 +172,7 @@ CDIread(void *process)
 
       cdoStreamClose(streamID);
 
-      tw = timer_val(timer_read) - tw0;
+      const double tw = timer_val(timer_read) - tw0;
       twsum += tw;
 
       file_size = (double) fileSize(cdoGetStreamName(0));
@@ -187,9 +184,6 @@ CDIread(void *process)
 
   if (nruns > 1) print_stat("(mean)", memtype, datatype, filetype, nvalues, data_size, file_size, twsum / nruns);
 
-  if (darray) Free(darray);
-  if (farray) Free(farray);
-
   cdoFinish();
 
   return nullptr;
