@@ -24,7 +24,7 @@
 #include "cdo_int.h"
 #include "param_conversion.h"
 #include "cdo_wtime.h"
-#include "pstream_int.h"
+
 #include "process_int.h"
 
 void *
@@ -38,36 +38,36 @@ CDItest(void *process)
   cdoInitialize(process);
 
   bool lcopy = false;
-  // bool lcopy = UNCHANGED_RECORD;
+  // bool lcopy = unchangedRecord();
 
-  int NCOPY = cdoOperatorAdd("ncopy", 0, 0, nullptr);
+  const int NCOPY = cdoOperatorAdd("ncopy", 0, 0, nullptr);
   UNUSED(NCOPY);
 
-  int operatorID = cdoOperatorID();
+  const int operatorID = cdoOperatorID();
   UNUSED(operatorID);
 
   //  operatorInputArg("Number of copies");
   if (operatorArgc() == 1) max_copy = parameter2int(operatorArgv()[0]);
 
-  double startTime = cdo_get_wtime();
+  const double startTime = cdo_get_wtime();
 
   int n = 0;
   while (true)
     {
-      int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+      CdoStreamID streamID1 = cdoOpenRead(0);
 
-      int vlistID1 = cdoStreamInqVlist(streamID1);
-      int taxisID1 = vlistInqTaxis(vlistID1);
+      const int vlistID1 = cdoStreamInqVlist(streamID1);
+      const int taxisID1 = vlistInqTaxis(vlistID1);
 
-      int vlistID2 = vlistDuplicate(vlistID1);
-      int taxisID2 = taxisDuplicate(taxisID1);
+      const int vlistID2 = vlistDuplicate(vlistID1);
+      const int taxisID2 = taxisDuplicate(taxisID1);
       vlistDefTaxis(vlistID2, taxisID2);
 
-      int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-      pstreamDefVlist(streamID2, vlistID2);
+      CdoStreamID streamID2 = cdoOpenWrite(1);
+      cdoDefVlist(streamID2, vlistID2);
 
-      size_t gridsize = vlistGridsizeMax(vlistID1);
-      double *array = (double *) Malloc(gridsize * sizeof(double));
+      const size_t gridsize = vlistGridsizeMax(vlistID1);
+      std::vector<double> array(gridsize);
 
       int tsID1 = 0;
       int tsID2 = 0;
@@ -75,23 +75,24 @@ CDItest(void *process)
         {
           taxisCopyTimestep(taxisID2, taxisID1);
 
-          pstreamDefTimestep(streamID2, tsID2);
+          cdoDefTimestep(streamID2, tsID2);
 
           for (int recID = 0; recID < nrecs; recID++)
             {
-              pstreamInqRecord(streamID1, &varID, &levelID);
-              pstreamDefRecord(streamID2, varID, levelID);
+              cdoInqRecord(streamID1, &varID, &levelID);
+              cdoDefRecord(streamID2, varID, levelID);
 
               if (lcopy)
                 {
-                  pstreamCopyRecord(streamID2, streamID1);
+                  cdoCopyRecord(streamID2, streamID1);
                 }
               else
                 {
-                  pstreamReadRecord(streamID1, array, &nmiss);
-                  pstreamWriteRecord(streamID2, array, nmiss);
+                  cdoReadRecord(streamID1, array.data(), &nmiss);
+                  cdoWriteRecord(streamID2, array.data(), nmiss);
                 }
             }
+
           tsID1++;
           tsID2++;
         }
@@ -102,8 +103,6 @@ CDItest(void *process)
       vlistDestroy(vlistID2);
       taxisDestroy(taxisID2);
 
-      if (array) Free(array);
-
       n++;
 
       cdoPrint("Copy number %d: %.2fs", n, cdo_get_wtime() - startTime);
