@@ -18,9 +18,9 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
+#include "cdo_zaxis.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
-#include "grid.h"
+#include <mpim_grid.h>
 #include "griddes.h"
 #include "timer.h"
 #include "util_files.h"
@@ -98,7 +98,7 @@ CDIwrite(void *process)
   size_t gridsize = gridInqSize(gridID);
 
   if (nlevs == 1)
-    zaxisID = zaxisCreate(ZAXIS_SURFACE, 1);
+    zaxisID = zaxisFromName("surface");
   else
     {
       std::vector<double> levels(nlevs);
@@ -171,11 +171,11 @@ CDIwrite(void *process)
       data_size = 0;
       nvalues = 0;
 
-      int streamID = cdoStreamOpenWrite(cdoStreamName(0));
+      CdoStreamID streamID = cdoOpenWrite(0);
 
-      pstreamDefVlist(streamID, vlistID);
+      cdoDefVlist(streamID, vlistID);
 
-      filetype = pstreamInqFiletype(streamID);
+      filetype = cdoInqFiletype(streamID);
       datatype = vlistInqVarDatatype(vlistID, 0);
       if (datatype == CDI_UNDEFID) datatype = CDI_DATATYPE_FLT32;
 
@@ -189,23 +189,23 @@ CDIwrite(void *process)
           int vtime = 0;
           taxisDefVdate(taxisID, vdate);
           taxisDefVtime(taxisID, vtime);
-          pstreamDefTimestep(streamID, tsID);
+          cdoDefTimestep(streamID, tsID);
 
           for (varID = 0; varID < nvars; varID++)
             {
               for (levelID = 0; levelID < nlevs; levelID++)
                 {
                   nvalues += gridsize;
-                  pstreamDefRecord(streamID, varID, levelID);
+                  cdoDefRecord(streamID, varID, levelID);
                   if (memtype == MEMTYPE_FLOAT)
                     {
                       for (size_t i = 0; i < gridsize; ++i) farray[i] = vars[varID][levelID][i];
-                      pstreamWriteRecordF(streamID, &farray[0], 0);
+                      cdoWriteRecordF(streamID, &farray[0], 0);
                       data_size += gridsize * 4;
                     }
                   else
                     {
-                      pstreamWriteRecord(streamID, &vars[varID][levelID][0], 0);
+                      cdoWriteRecord(streamID, &vars[varID][levelID][0], 0);
                       data_size += gridsize * 8;
                     }
                 }
