@@ -26,7 +26,7 @@
 #include "cdo_int.h"
 #include "param_conversion.h"
 #include "cdo_cmor.h"
-#include "pstream_int.h"
+
 #include "pmlist.h"
 #include "convert_units.h"
 #include "cdo_cdiWrapper.h"
@@ -353,7 +353,7 @@ CMOR_lite(void *process)
 
   if (operatorArgc() > 2) cdoAbort("Too many arguments!");
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   int vlistID2 = vlistDuplicate(vlistID1);
@@ -426,8 +426,8 @@ CMOR_lite(void *process)
   vlistDefTaxis(vlistID2, taxisID2);
 
   /* vlistPrint(vlistID2);*/
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   size_t gridsizemax = vlistGridsizeMax(vlistID1);
   if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
@@ -438,13 +438,13 @@ CMOR_lite(void *process)
     {
       taxisCopyTimestep(taxisID2, taxisID1);
 
-      pstreamDefTimestep(streamID2, tsID1);
+      cdoDefTimestep(streamID2, tsID1);
 
       cmor_check_init(nvars, vars);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           CmorVar &var = vars[varID];
           int varID2 = varID;
@@ -461,15 +461,15 @@ CMOR_lite(void *process)
                 }
             }
 
-          pstreamDefRecord(streamID2, varID2, levelID2);
+          cdoDefRecord(streamID2, varID2, levelID2);
 
-          pstreamReadRecord(streamID1, array.data(), &nmiss);
+          cdoReadRecord(streamID1, array.data(), &nmiss);
 
           const double missval = vlistInqVarMissval(vlistID2, varID2);
           size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID2));
           if (vlistInqVarNumber(vlistID2, varID2) != CDI_REAL) gridsize *= 2;
 
-          if (nmiss > 0 && var.changemissval)
+          if (nmiss && var.changemissval)
             {
               for (size_t i = 0; i < gridsize; ++i)
                 {
@@ -506,7 +506,7 @@ CMOR_lite(void *process)
             }
 #endif
 
-          pstreamWriteRecord(streamID2, array.data(), nmiss);
+          cdoWriteRecord(streamID2, array.data(), nmiss);
 
           cmor_check_prep(var, gridsize, missval, array.data());
         }
