@@ -24,7 +24,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 #include "timer.h"
 #include "util_files.h"
 #include "progress.h"
@@ -36,7 +35,7 @@ Cat(void *process)
   bool lconstvars = true;
   int nrecs;
   int tsID2 = 0, varID, levelID;
-  int streamID2 = CDI_UNDEFID;
+  CdoStreamID streamID2;
   int vlistID2 = CDI_UNDEFID;
   int taxisID2 = CDI_UNDEFID;
   size_t nmiss;
@@ -45,7 +44,7 @@ Cat(void *process)
 
   cdoInitialize(process);
 
-  const bool lcopy = UNCHANGED_RECORD;
+  const bool lcopy = unchangedRecord();
 
   const int timer_cat = timer_new("cat");
   if (Options::Timer) timer_start(timer_cat);
@@ -60,7 +59,7 @@ Cat(void *process)
       if (Options::cdoVerbose) cdoPrint("Process file: %s", cdoGetStreamName(indf));
       if (Options::Timer) tw0 = timer_val(timer_cat);
 
-      const int streamID1 = cdoStreamOpenRead(cdoStreamName(indf));
+      CdoStreamID streamID1 = cdoOpenRead((indf));
       const int vlistID1 = cdoStreamInqVlist(streamID1);
       const int taxisID1 = vlistInqTaxis(vlistID1);
 
@@ -79,7 +78,7 @@ Cat(void *process)
           const bool file_exists = !Options::cdoOverwriteMode ? fileExists(cdoGetStreamName(nfiles)) : false;
           if (file_exists)
             {
-              streamID2 = cdoStreamOpenAppend(cdoStreamName(nfiles));
+              streamID2 = cdoOpenAppend((nfiles));
 
               vlistID2 = cdoStreamInqVlist(streamID2);
               taxisID2 = vlistInqTaxis(vlistID2);
@@ -87,7 +86,7 @@ Cat(void *process)
               vlistCompare(vlistID1, vlistID2, CMP_ALL);
 
               tsID2 = vlistNtsteps(vlistID2);
-              if (tsID2 == 0) tsID2 = 1; // bug fix for time constant data only
+              if (tsID2 == 0) tsID2 = 1;  // bug fix for time constant data only
 
               if (ntsteps == 0) lconstvars = false;
             }
@@ -95,7 +94,7 @@ Cat(void *process)
             {
               if (Options::cdoVerbose) cdoPrint("Output file doesn't exist, creating: %s", cdoGetStreamName(nfiles));
 
-              streamID2 = cdoStreamOpenWrite(cdoStreamName(nfiles));
+              streamID2 = cdoOpenWrite((nfiles));
 
               vlistID2 = vlistDuplicate(vlistID1);
               taxisID2 = taxisDuplicate(taxisID1);
@@ -107,7 +106,7 @@ Cat(void *process)
                   for (varID = 0; varID < nvars; ++varID) vlistDefVarTimetype(vlistID2, varID, TIME_VARYING);
                 }
 
-              pstreamDefVlist(streamID2, vlistID2);
+              cdoDefVlist(streamID2, vlistID2);
             }
 
           if (!lcopy)
@@ -131,25 +130,25 @@ Cat(void *process)
           if (!Options::cdoVerbose) Progress::update(0, 1, fstatus / nfiles);
 
           taxisCopyTimestep(taxisID2, taxisID1);
-          pstreamDefTimestep(streamID2, tsID2);
+          cdoDefTimestep(streamID2, tsID2);
 
           for (int recID = 0; recID < nrecs; recID++)
             {
-              pstreamInqRecord(streamID1, &varID, &levelID);
+              cdoInqRecord(streamID1, &varID, &levelID);
 
               if (lconstvars && tsID2 > 0 && tsID1 == 0)
                 if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
 
-              pstreamDefRecord(streamID2, varID, levelID);
+              cdoDefRecord(streamID2, varID, levelID);
 
               if (lcopy)
                 {
-                  pstreamCopyRecord(streamID2, streamID1);
+                  cdoCopyRecord(streamID2, streamID1);
                 }
               else
                 {
-                  pstreamReadRecord(streamID1, &array[0], &nmiss);
-                  pstreamWriteRecord(streamID2, &array[0], nmiss);
+                  cdoReadRecord(streamID1, &array[0], &nmiss);
+                  cdoWriteRecord(streamID2, &array[0], nmiss);
                 }
             }
 
