@@ -32,20 +32,19 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
 
 int stringToParam(const char *paramstr);
 
 #define MAXARG 16384
 
-static
-void changeName(const int vlistID1, const int vlistID2, const int nch, char *chnames[MAXARG])
+static void
+changeName(const int vlistID1, const int vlistID2, const int nch, char *chnames[MAXARG])
 {
   char varname[CDI_MAX_NAME], varname2[CDI_MAX_NAME];
-  const int npairs = nch/2;
-  std::vector<std::pair<char*,char*>> vpairs(npairs);
-  for (int i = 0; i < npairs; ++i) vpairs[i].first = chnames[i*2];
-  for (int i = 0; i < npairs; ++i) vpairs[i].second = chnames[i*2+1];
+  const int npairs = nch / 2;
+  std::vector<std::pair<char *, char *>> vpairs(npairs);
+  for (int i = 0; i < npairs; ++i) vpairs[i].first = chnames[i * 2];
+  for (int i = 0; i < npairs; ++i) vpairs[i].second = chnames[i * 2 + 1];
 
   const int nvars = vlistNvars(vlistID2);
   std::vector<bool> namefound(npairs, false);
@@ -55,7 +54,7 @@ void changeName(const int vlistID1, const int vlistID2, const int nch, char *chn
       for (int i = 0; i < npairs; ++i)
         if (strcmp(varname, vpairs[i].first) == 0)
           {
-            namefound[i] = true; 
+            namefound[i] = true;
             vlistDefVarName(vlistID2, varID, vpairs[i].second);
             break;
           }
@@ -68,7 +67,7 @@ void changeName(const int vlistID1, const int vlistID2, const int nch, char *chn
         searchForGridName = true;
         break;
       }
-    
+
   if (searchForGridName)
     {
       const int ngrids = vlistNgrids(vlistID1);
@@ -116,7 +115,7 @@ void changeName(const int vlistID1, const int vlistID2, const int nch, char *chn
         searchForZaxisName = true;
         break;
       }
-    
+
   if (searchForZaxisName)
     {
       const int nzaxis = vlistNzaxis(vlistID1);
@@ -145,7 +144,6 @@ void changeName(const int vlistID1, const int vlistID2, const int nch, char *chn
     if (!namefound[i]) cdoWarning("Variable name %s not found!", vpairs[i].first);
 }
 
-
 void *
 Change(void *process)
 {
@@ -219,7 +217,7 @@ Change(void *process)
       for (i = 0; i < nch; i++) chltypes[i] = parameter2int(operatorArgv()[i]);
     }
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -388,26 +386,26 @@ Change(void *process)
   const size_t gridsizemax = vlistGridsizeMax(vlistID2);
   std::vector<double> array(gridsizemax);
 
-  int streamID2 = CDI_UNDEFID;
+  CdoStreamID streamID2 = CDO_STREAM_UNDEF;
 
   int tsID1 = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID1)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      if (streamID2 == CDI_UNDEFID)
+      if (streamID2 == CDO_STREAM_UNDEF)
         {
-          streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-          pstreamDefVlist(streamID2, vlistID2);
+          streamID2 = cdoOpenWrite(1);
+          cdoDefVlist(streamID2, vlistID2);
         }
-      pstreamDefTimestep(streamID2, tsID1);
+      cdoDefTimestep(streamID2, tsID1);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamDefRecord(streamID2, varID, levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoDefRecord(streamID2, varID, levelID);
 
-          pstreamReadRecord(streamID1, &array[0], &nmiss);
-          pstreamWriteRecord(streamID2, &array[0], nmiss);
+          cdoReadRecord(streamID1, &array[0], &nmiss);
+          cdoWriteRecord(streamID2, &array[0], nmiss);
         }
       tsID1++;
     }
