@@ -24,7 +24,7 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
+#include "cdi_lockedIO.h"
 
 void *
 Change_e5slm(void *process)
@@ -36,7 +36,7 @@ Change_e5slm(void *process)
 
   cdoInitialize(process);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   int vlistID1 = cdoStreamInqVlist(streamID1);
   int taxisID1 = vlistInqTaxis(vlistID1);
@@ -45,8 +45,8 @@ Change_e5slm(void *process)
   int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   /* get filename of SLM */
   operatorInputArg("filename of the sea land mask");
@@ -68,10 +68,10 @@ Change_e5slm(void *process)
   streamInqRecord(streamIDslm, &varID, &levelID);
   streamReadRecord(streamIDslm, cland.data(), &nmiss);
 
-  if (nmiss > 0) cdoAbort("SLM with missing values are unsupported!");
+  if (nmiss) cdoAbort("SLM with missing values are unsupported!");
 
   double minval, maxval;
-  arrayMinMaxMask(gridsize, cland.data(), nullptr, &minval, &maxval);
+  arrayMinMax(gridsize, cland.data(), minval, maxval);
   if (minval < 0 || maxval > 1) cdoWarning("Values of SLM out of bounds! (minval=%g, maxval=%g)", minval, maxval);
 
   streamClose(streamIDslm);
@@ -113,12 +113,12 @@ Change_e5slm(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, array.data(), &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, array.data(), &nmiss);
 
           int code = codes[varID];
           if (code == 172)
@@ -145,8 +145,8 @@ Change_e5slm(void *process)
                 if (lsea[i]) array[i] = array[0];
             }
 
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, array.data(), nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, array.data(), nmiss);
         }
 
       tsID++;
