@@ -20,7 +20,7 @@
 #include "cdo_int.h"
 #include "param_conversion.h"
 #include "cdo_math.h"
-#include "pstream_int.h"
+
 #include "util_string.h"
 
 #define SCALESLP (101325.0)
@@ -32,7 +32,7 @@
 static void
 layer_cloud(const double *cc, double *ll, long MaxLev, long MinLev, long dimgp)
 {
-  double ZEPSEC = 1. - 1.0e-12;
+  const double ZEPSEC = 1. - 1.0e-12;
 
   for (long i = 0; i < dimgp; i++) ll[i] = 1. - cc[i + MaxLev * dimgp];
 
@@ -42,8 +42,8 @@ layer_cloud(const double *cc, double *ll, long MaxLev, long MinLev, long dimgp)
     {
       for (long i = 0; i < dimgp; i++)
         {
-          double maxval = cdo::max(cc[i + (k - 1) * dimgp], cc[i + k * dimgp]);
-          double minval = cdo::min(cc[i + (k - 1) * dimgp], ZEPSEC);
+          const double maxval = cdo::max(cc[i + (k - 1) * dimgp], cc[i + k * dimgp]);
+          const double minval = cdo::min(cc[i + (k - 1) * dimgp], ZEPSEC);
           ll[i] *= (1. - maxval) / (1. - minval);
         }
     }
@@ -72,12 +72,12 @@ hl_index(int *kmax, int *kmin, double pmax, double pmin, long nhlevs, std::vecto
   for (k = 0; k < nhlevs; k++)
     if (pph[k] > pmax) break;
 
-  long MaxLev = k - 1;
+  const long MaxLev = k - 1;
 
   for (k = nhlevs - 1; k >= 0; k--)
     if (pph[k] < pmin) break;
 
-  long MinLev = k;
+  const long MinLev = k;
 
   *kmax = MaxLev;
   *kmin = MinLev;
@@ -123,7 +123,6 @@ Cloudlayer(void *process)
   int kmin[NVARS], kmax[NVARS];
   char varname[CDI_MAX_NAME];
   double sfclevel = 0;
-  double *cloud[NVARS];
   double pmin = 0, pmax = 0;
 
   cdoInitialize(process);
@@ -140,15 +139,15 @@ Cloudlayer(void *process)
       nvars2 = NVARS;
     }
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
 
-  size_t gridsize = vlist_check_gridsize(vlistID1);
+  const size_t gridsize = vlist_check_gridsize(vlistID1);
 
-  int aclcac_code = 223;
+  const int aclcac_code = 223;
 
-  int nvars = vlistNvars(vlistID1);
+  const int nvars = vlistNvars(vlistID1);
   for (varID = 0; varID < nvars; ++varID)
     {
       zaxisID = vlistInqVarZaxis(vlistID1, varID);
@@ -157,7 +156,7 @@ Cloudlayer(void *process)
       if (code <= 0)
         {
           vlistInqVarName(vlistID1, varID, varname);
-          strtolower(varname);
+          cstrToLowerCase(varname);
           if (strcmp(varname, "aclcac") == 0) code = 223;
         }
 
@@ -180,15 +179,16 @@ Cloudlayer(void *process)
         cdoAbort("Cloud cover (parameter 223) not found!");
     }
 
-  double missval = vlistInqVarMissval(vlistID1, aclcacID);
+  const double missval = vlistInqVarMissval(vlistID1, aclcacID);
   gridID = vlistInqVarGrid(vlistID1, aclcacID);
   zaxisID = vlistInqVarZaxis(vlistID1, aclcacID);
 
   nlevel = zaxisInqSize(zaxisID);
-  int nhlev = nlevel + 1;
+  const int nhlev = nlevel + 1;
 
   std::vector<double> aclcac(gridsize * nlevel);
-  for (varID = 0; varID < nvars2; ++varID) cloud[varID] = (double *) Malloc(gridsize * sizeof(double));
+  std::vector<double> cloud[NVARS];
+  for (varID = 0; varID < nvars2; ++varID) cloud[varID].resize(gridsize);
 
   if (zaxisInqType(zaxisID) == ZAXIS_PRESSURE)
     {
@@ -196,11 +196,10 @@ Cloudlayer(void *process)
       zaxisInqLevels(zaxisID, plevs.data());
       if (plevs[0] > plevs[nlevel - 1])
         {
-          double ptmp;
           zrev = true;
           for (levelID = 0; levelID < nlevel / 2; ++levelID)
             {
-              ptmp = plevs[levelID];
+              const double ptmp = plevs[levelID];
               plevs[levelID] = plevs[nlevel - 1 - levelID];
               plevs[nlevel - 1 - levelID] = ptmp;
             }
@@ -224,7 +223,7 @@ Cloudlayer(void *process)
     }
   else if (zaxisInqType(zaxisID) == ZAXIS_HYBRID)
     {
-      int nvct = zaxisInqVctSize(zaxisID);
+      const int nvct = zaxisInqVctSize(zaxisID);
       if (nlevel == (nvct / 2 - 1))
         {
           std::vector<double> vct(nvct);
@@ -251,10 +250,10 @@ Cloudlayer(void *process)
   else
     cdoAbort("Unsupported Z-Axis type!");
 
-  int surfaceID = zaxisCreate(ZAXIS_SURFACE, 1);
+  const int surfaceID = zaxisCreate(ZAXIS_SURFACE, 1);
   zaxisDefLevels(surfaceID, &sfclevel);
 
-  int vlistID2 = vlistCreate();
+  const int vlistID2 = vlistCreate();
 
   if (nvars2 == 1)
     {
@@ -285,29 +284,29 @@ Cloudlayer(void *process)
       vlistDefVarMissval(vlistID2, varID, missval);
     }
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
 
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
-          size_t offset = zrev ? (nlevel - 1 - levelID) * gridsize : levelID * gridsize;
+          const size_t offset = zrev ? (nlevel - 1 - levelID) * gridsize : levelID * gridsize;
 
           if (varID == aclcacID)
             {
-              pstreamReadRecord(streamID1, aclcac.data() + offset, &nmiss);
+              cdoReadRecord(streamID1, aclcac.data() + offset, &nmiss);
               if (nmiss != 0) cdoAbort("Missing values unsupported!");
             }
         }
@@ -319,15 +318,16 @@ Cloudlayer(void *process)
 
       for (varID = 0; varID < nvars2; ++varID)
         {
-          if (kmax[varID] != -1 && kmin[varID] != -1) layer_cloud(aclcac.data(), cloud[varID], kmax[varID], kmin[varID], gridsize);
+          if (kmax[varID] != -1 && kmin[varID] != -1)
+            layer_cloud(aclcac.data(), cloud[varID].data(), kmax[varID], kmin[varID], gridsize);
         }
 
       for (varID = 0; varID < nvars2; ++varID)
         {
-          nmiss = arrayNumMV(gridsize, cloud[varID], missval);
+          nmiss = arrayNumMV(gridsize, cloud[varID].data(), missval);
 
-          pstreamDefRecord(streamID2, varID, 0);
-          pstreamWriteRecord(streamID2, cloud[varID], nmiss);
+          cdoDefRecord(streamID2, varID, 0);
+          cdoWriteRecord(streamID2, cloud[varID].data(), nmiss);
         }
 
       tsID++;
@@ -338,9 +338,7 @@ Cloudlayer(void *process)
 
   vlistDestroy(vlistID2);
 
-  for (varID = 0; varID < nvars2; ++varID) Free(cloud[varID]);
-
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
