@@ -19,15 +19,15 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
-#include "grid.h"
+
+#include <mpim_grid.h>
 #include "util_files.h"
 
 static int globalGridType = CDI_UNDEFID;
 
 struct ensfileType
 {
-  int streamID;
+  CdoStreamID streamID;
   int vlistID;
   int gridID;
   size_t nmiss;
@@ -102,8 +102,8 @@ genGrid(int ngrids, int nfiles, std::vector<ensfileType> &ef, bool ginit, int ig
   if (gridtype == GRID_UNSTRUCTURED) lunstructured = true;
   const int nv = lunstructured ? gridInqNvertex(gridID) : 0;
   const bool lcenter = globalGridType == CDI_UNDEFID && gridInqXvals(gridID, nullptr) > 0 && gridInqYvals(gridID, nullptr) > 0;
-  const bool lbounds
-      = lunstructured && globalGridType == CDI_UNDEFID && gridInqXbounds(gridID, nullptr) > 0 && gridInqYbounds(gridID, nullptr) > 0;
+  const bool lbounds = lunstructured && globalGridType == CDI_UNDEFID && gridInqXbounds(gridID, nullptr) > 0
+                       && gridInqYbounds(gridID, nullptr) > 0;
 
   std::vector<long> xsize(nfiles);
   std::vector<long> ysize(nfiles);
@@ -230,15 +230,13 @@ genGrid(int ngrids, int nfiles, std::vector<ensfileType> &ef, bool ginit, int ig
   for (long j = 0; j < ny; ++j) ysize2 += ysize[xyinfo[j * nx].id];
   if (Options::cdoVerbose) cdoPrint("xsize2 %ld  ysize2 %ld", xsize2, ysize2);
 
-  { // verify size of data
+  {  // verify size of data
     const long xs = xsize[xyinfo[0].id];
     for (long j = 1; j < ny; ++j)
-      if (xsize[xyinfo[j * nx].id] != xs)
-        cdoAbort("xsize=%ld differ from first file (xsize=%ld)!", xsize[xyinfo[j * nx].id], xs);
+      if (xsize[xyinfo[j * nx].id] != xs) cdoAbort("xsize=%ld differ from first file (xsize=%ld)!", xsize[xyinfo[j * nx].id], xs);
     const long ys = ysize[xyinfo[0].id];
     for (long i = 1; i < nx; ++i)
-      if (ysize[xyinfo[i].id] != ys)
-        cdoAbort("ysize=%ld differ from first file (ysize=%ld)!", ysize[xyinfo[i].id], ys);
+      if (ysize[xyinfo[i].id] != ys) cdoAbort("ysize=%ld differ from first file (ysize=%ld)!", ysize[xyinfo[i].id], ys);
   }
 
   std::vector<double> xvals2, yvals2;
@@ -372,7 +370,7 @@ Collgrid(void *process)
 
   for (int fileID = 0; fileID < nfiles; fileID++)
     {
-      ef[fileID].streamID = cdoStreamOpenRead(cdoStreamName(fileID));
+      ef[fileID].streamID = cdoOpenRead((fileID));
       ef[fileID].vlistID = cdoStreamInqVlist(ef[fileID].streamID);
     }
 
@@ -530,8 +528,8 @@ Collgrid(void *process)
         }
     }
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(nfiles));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite((nfiles));
+  cdoDefVlist(streamID2, vlistID2);
 
   std::vector<double> array2;
   if (gridsize2 > 0) array2.resize(gridsize2);
@@ -550,17 +548,17 @@ Collgrid(void *process)
 
       taxisCopyTimestep(taxisID2, taxisID1);
 
-      if (nrecs0 > 0) pstreamDefTimestep(streamID2, tsID);
+      if (nrecs0 > 0) cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs0; recID++)
         {
-          pstreamInqRecord(ef[0].streamID, &varID, &levelID);
+          cdoInqRecord(ef[0].streamID, &varID, &levelID);
           if (Options::cdoVerbose && tsID == 0) printf(" tsID, recID, varID, levelID %d %d %d %d\n", tsID, recID, varID, levelID);
 
           for (int fileID = 1; fileID < nfiles; fileID++)
             {
               int varIDx, levelIDx;
-              pstreamInqRecord(ef[fileID].streamID, &varIDx, &levelIDx);
+              cdoInqRecord(ef[fileID].streamID, &varIDx, &levelIDx);
             }
 
           if (vlistInqFlag(vlistID1, varID, levelID) == true)
@@ -577,7 +575,7 @@ Collgrid(void *process)
 #endif
               for (int fileID = 0; fileID < nfiles; fileID++)
                 {
-                  pstreamReadRecord(ef[fileID].streamID, &ef[fileID].array[0], &ef[fileID].nmiss);
+                  cdoReadRecord(ef[fileID].streamID, &ef[fileID].array[0], &ef[fileID].nmiss);
 
                   if (vars[varID2])
                     {
@@ -586,15 +584,15 @@ Collgrid(void *process)
                     }
                 }
 
-              pstreamDefRecord(streamID2, varID2, levelID2);
+              cdoDefRecord(streamID2, varID2, levelID2);
 
               if (vars[varID2])
                 {
                   const size_t nmiss = arrayNumMV(gridsize2, &array2[0], missval);
-                  pstreamWriteRecord(streamID2, &array2[0], nmiss);
+                  cdoWriteRecord(streamID2, &array2[0], nmiss);
                 }
               else
-                pstreamWriteRecord(streamID2, &ef[0].array[0], ef[0].nmiss);
+                cdoWriteRecord(streamID2, &ef[0].array[0], ef[0].nmiss);
             }
         }
 
