@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -29,7 +29,8 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
+
+#include "cdo_cdiWrapper.h"
 
 void *
 Comp(void *process)
@@ -48,31 +49,31 @@ Comp(void *process)
 
   cdoInitialize(process);
 
-  int EQ = cdoOperatorAdd("eq", 0, 0, NULL);
-  int NE = cdoOperatorAdd("ne", 0, 0, NULL);
-  int LE = cdoOperatorAdd("le", 0, 0, NULL);
-  int LT = cdoOperatorAdd("lt", 0, 0, NULL);
-  int GE = cdoOperatorAdd("ge", 0, 0, NULL);
-  int GT = cdoOperatorAdd("gt", 0, 0, NULL);
+  const int EQ = cdoOperatorAdd("eq", 0, 0, nullptr);
+  const int NE = cdoOperatorAdd("ne", 0, 0, nullptr);
+  const int LE = cdoOperatorAdd("le", 0, 0, nullptr);
+  const int LT = cdoOperatorAdd("lt", 0, 0, nullptr);
+  const int GE = cdoOperatorAdd("ge", 0, 0, nullptr);
+  const int GT = cdoOperatorAdd("gt", 0, 0, nullptr);
 
-  int operatorID = cdoOperatorID();
+  const int operatorID = cdoOperatorID();
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-  int streamID2 = cdoStreamOpenRead(cdoStreamName(1));
+  CdoStreamID streamID1 = cdoOpenRead(0);
+  CdoStreamID streamID2 = cdoOpenRead(1);
 
-  int streamIDx1 = streamID1;
-  int streamIDx2 = streamID2;
+  CdoStreamID streamIDx1 = streamID1;
+  CdoStreamID streamIDx2 = streamID2;
 
   double *missvalx1 = &missval1;
   double *missvalx2 = &missval2;
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = cdoStreamInqVlist(streamID2);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = cdoStreamInqVlist(streamID2);
   int vlistIDx1 = vlistID1;
   int vlistIDx2 = vlistID2;
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = vlistInqTaxis(vlistID2);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = vlistInqTaxis(vlistID2);
   int taxisIDx1 = taxisID1;
 
   int ntsteps1 = vlistNtsteps(vlistID1);
@@ -85,13 +86,13 @@ Comp(void *process)
   if (vlistNrecs(vlistID1) != 1 && vlistNrecs(vlistID2) == 1)
     {
       filltype = FILL_REC;
-      cdoPrint("Filling up stream2 >%s< by copying the first record.", cdoGetStreamName(1).c_str());
+      cdoPrint("Filling up stream2 >%s< by copying the first record.", cdoGetStreamName(1));
       if (ntsteps2 != 1) cdoAbort("stream2 has more than 1 timestep!");
     }
   else if (vlistNrecs(vlistID1) == 1 && vlistNrecs(vlistID2) != 1)
     {
       filltype = FILL_REC;
-      cdoPrint("Filling up stream1 >%s< by copying the first record.", cdoGetStreamName(0).c_str());
+      cdoPrint("Filling up stream1 >%s< by copying the first record.", cdoGetStreamName(0));
       if (ntsteps1 != 1) cdoAbort("stream1 has more than 1 timestep!");
       fillstream1 = true;
       streamIDx1 = streamID2;
@@ -106,7 +107,7 @@ Comp(void *process)
   nospec(vlistID1);
   nospec(vlistID2);
 
-  size_t gridsizemax = vlistGridsizeMax(vlistIDx1);
+  const size_t gridsizemax = vlistGridsizeMax(vlistIDx1);
 
   std::vector<double> array1(gridsizemax);
   std::vector<double> array2(gridsizemax);
@@ -115,19 +116,19 @@ Comp(void *process)
   double *arrayx1 = array1.data();
   double *arrayx2 = array2.data();
 
-  if (cdoVerbose) cdoPrint("Number of timesteps: file1 %d, file2 %d", ntsteps1, ntsteps2);
+  if (Options::cdoVerbose) cdoPrint("Number of timesteps: file1 %d, file2 %d", ntsteps1, ntsteps2);
 
   if (filltype == FILL_NONE)
     {
       if (ntsteps1 != 1 && ntsteps2 == 1)
         {
           filltype = FILL_TS;
-          cdoPrint("Filling up stream2 >%s< by copying the first timestep.", cdoGetStreamName(1).c_str());
+          cdoPrint("Filling up stream2 >%s< by copying the first timestep.", cdoGetStreamName(1));
         }
       else if (ntsteps1 == 1 && ntsteps2 != 1)
         {
           filltype = FILL_TS;
-          cdoPrint("Filling up stream1 >%s< by copying the first timestep.", cdoGetStreamName(0).c_str());
+          cdoPrint("Filling up stream1 >%s< by copying the first timestep.", cdoGetStreamName(0));
           fillstream1 = true;
           streamIDx1 = streamID2;
           streamIDx2 = streamID1;
@@ -157,13 +158,17 @@ Comp(void *process)
       missvalx2 = &missval1;
     }
 
-  int vlistID3 = vlistDuplicate(vlistIDx1);
+  VarList varList1, varList2;
+  varListInit(varList1, vlistIDx1);
+  varListInit(varList2, vlistIDx2);
 
-  int taxisID3 = taxisDuplicate(taxisIDx1);
+  const int vlistID3 = vlistDuplicate(vlistIDx1);
+
+  const int taxisID3 = taxisDuplicate(taxisIDx1);
   vlistDefTaxis(vlistID3, taxisID3);
 
-  int streamID3 = cdoStreamOpenWrite(cdoStreamName(2), cdoFiletype());
-  pstreamDefVlist(streamID3, vlistID3);
+  CdoStreamID streamID3 = cdoOpenWrite(2);
+  cdoDefVlist(streamID3, vlistID3);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamIDx1, tsID)))
@@ -175,60 +180,48 @@ Comp(void *process)
         }
 
       taxisCopyTimestep(taxisID3, taxisIDx1);
-      pstreamDefTimestep(streamID3, tsID);
+      cdoDefTimestep(streamID3, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
           size_t nmiss1;
-          pstreamInqRecord(streamIDx1, &varID, &levelID);
-          pstreamReadRecord(streamIDx1, arrayx1, &nmiss1);
+          cdoInqRecord(streamIDx1, &varID, &levelID);
+          cdoReadRecord(streamIDx1, arrayx1, &nmiss1);
 
           if (tsID == 0 || filltype == FILL_NONE)
             {
               if (recID == 0 || filltype != FILL_REC)
                 {
                   size_t nmiss2;
-                  pstreamInqRecord(streamIDx2, &varID, &levelID);
-                  pstreamReadRecord(streamIDx2, arrayx2, &nmiss2);
+                  cdoInqRecord(streamIDx2, &varID, &levelID);
+                  cdoReadRecord(streamIDx2, arrayx2, &nmiss2);
                 }
 
               if (filltype == FILL_TS)
                 {
-                  size_t gridsize = gridInqSize(vlistInqVarGrid(vlistIDx2, varID));
-                  size_t offset = gridsize * levelID;
-                  arrayCopy(gridsize, arrayx2, &vardata[varID][offset]);
+                  const size_t offset = varList2[varID].gridsize * levelID;
+                  arrayCopy(varList2[varID].gridsize, arrayx2, &vardata[varID][offset]);
                 }
             }
           else if (filltype == FILL_TS)
             {
-              size_t gridsize = gridInqSize(vlistInqVarGrid(vlistIDx2, varID));
-              size_t offset = gridsize * levelID;
-              arrayCopy(gridsize, &vardata[varID][offset], arrayx2);
+              const size_t offset = varList2[varID].gridsize * levelID;
+              arrayCopy(varList2[varID].gridsize, &vardata[varID][offset], arrayx2);
             }
 
-          int datatype1 = vlistInqVarDatatype(vlistIDx1, varID);
-          int datatype2 = CDI_UNDEFID;
-          size_t gridsize1 = gridInqSize(vlistInqVarGrid(vlistIDx1, varID));
-          size_t gridsize2;
-          *missvalx1 = vlistInqVarMissval(vlistIDx1, varID);
+          const int datatype1 = varList1[varID].datatype;
+          const size_t gridsize1 = varList1[varID].gridsize;
+          *missvalx1 = varList1[varID].missval;
 
-          if (filltype == FILL_REC)
-            {
-              datatype2 = vlistInqVarDatatype(vlistIDx2, 0);
-              gridsize2 = gridInqSize(vlistInqVarGrid(vlistIDx2, 0));
-              *missvalx2 = vlistInqVarMissval(vlistIDx2, 0);
-            }
-          else
-            {
-              datatype2 = vlistInqVarDatatype(vlistIDx2, varID);
-              gridsize2 = gridInqSize(vlistInqVarGrid(vlistIDx2, varID));
-              *missvalx2 = vlistInqVarMissval(vlistIDx2, varID);
-            }
+          const int xvarID = (filltype == FILL_REC) ? 0 : varID;
+          const int datatype2 = varList2[xvarID].datatype;
+          const size_t gridsize2 = varList2[xvarID].gridsize;
+          *missvalx2 = varList2[xvarID].missval;
 
           if (gridsize1 != gridsize2)
             cdoAbort("Streams have different gridsize (gridsize1 = %zu; gridsize2 = %zu)!", gridsize1, gridsize2);
 
-          size_t gridsize = gridsize1;
+          const size_t gridsize = gridsize1;
 
           if (datatype1 != datatype2)
             {
@@ -287,19 +280,19 @@ Comp(void *process)
               cdoAbort("Operator not implemented!");
             }
 
-          size_t nmiss3 = arrayNumMV(gridsize, array3.data(), missval1);
-          pstreamDefRecord(streamID3, varID, levelID);
-          pstreamWriteRecord(streamID3, array3.data(), nmiss3);
+          const size_t nmiss3 = arrayNumMV(gridsize, array3.data(), missval1);
+          cdoDefRecord(streamID3, varID, levelID);
+          cdoWriteRecord(streamID3, array3.data(), nmiss3);
         }
 
       tsID++;
     }
 
-  pstreamClose(streamID3);
-  pstreamClose(streamID2);
-  pstreamClose(streamID1);
+  cdoStreamClose(streamID3);
+  cdoStreamClose(streamID2);
+  cdoStreamClose(streamID1);
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
