@@ -30,7 +30,6 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
 
 void *
 Compc(void *process)
@@ -41,55 +40,56 @@ Compc(void *process)
 
   cdoInitialize(process);
 
-  int EQC = cdoOperatorAdd("eqc", 0, 0, nullptr);
-  int NEC = cdoOperatorAdd("nec", 0, 0, nullptr);
-  int LEC = cdoOperatorAdd("lec", 0, 0, nullptr);
-  int LTC = cdoOperatorAdd("ltc", 0, 0, nullptr);
-  int GEC = cdoOperatorAdd("gec", 0, 0, nullptr);
-  int GTC = cdoOperatorAdd("gtc", 0, 0, nullptr);
+  const int EQC = cdoOperatorAdd("eqc", 0, 0, nullptr);
+  const int NEC = cdoOperatorAdd("nec", 0, 0, nullptr);
+  const int LEC = cdoOperatorAdd("lec", 0, 0, nullptr);
+  const int LTC = cdoOperatorAdd("ltc", 0, 0, nullptr);
+  const int GEC = cdoOperatorAdd("gec", 0, 0, nullptr);
+  const int GTC = cdoOperatorAdd("gtc", 0, 0, nullptr);
 
-  int operatorID = cdoOperatorID();
+  const int operatorID = cdoOperatorID();
 
   operatorInputArg("constant value");
-  double rc = parameter2double(operatorArgv()[0]);
+  const double rc = parameter2double(operatorArgv()[0]);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
   nospec(vlistID1);
 
-  size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  VarList varList;
+  varListInit(varList, vlistID1);
 
+  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
   std::vector<double> array1(gridsizemax);
   std::vector<double> array2(gridsizemax);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, array1.data(), &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, array1.data(), &nmiss);
 
-          double missval = vlistInqVarMissval(vlistID1, varID);
-          size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
+          const double missval = varList[varID].missval;
+          const size_t gridsize = varList[varID].gridsize;
+          const int datatype = varList[varID].datatype;
+          const double rcv = (datatype == CDI_DATATYPE_FLT32) ? (float) rc : rc;
 
-          bool rc_is_missval = DBL_IS_EQUAL(rc, missval);
-
-          int datatype = vlistInqVarDatatype(vlistID1, varID);
-          double rcv = (datatype == CDI_DATATYPE_FLT32) ? (float) rc : rc;
+          const bool rc_is_missval = DBL_IS_EQUAL(rc, missval);
 
           if (operatorID == EQC)
             {
@@ -127,8 +127,8 @@ Compc(void *process)
             }
 
           nmiss = arrayNumMV(gridsize, array2.data(), missval);
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, array2.data(), nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, array2.data(), nmiss);
         }
 
       tsID++;
