@@ -25,7 +25,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 
 void *
 Cond(void *process)
@@ -37,33 +36,31 @@ Cond(void *process)
     FILL_REC
   };
   int filltype = FILL_NONE;
-  int nrecs, nrecs2, nvars = 0, nlev;
+  int nrecs, nrecs2, nvars = 0;
   int varID, levelID;
-  size_t offset;
   size_t nmiss1 = 0, nmiss2;
   double missval1 = -9.E33;
-  double missval2 = -9.E33;
   std::vector<std::vector<size_t>> varnmiss1;
   std::vector<std::vector<double>> vardata1;
 
   cdoInitialize(process);
 
   // clang-format off
-  int IFTHEN    = cdoOperatorAdd("ifthen",    0, 0, nullptr);
-  int IFNOTTHEN = cdoOperatorAdd("ifnotthen", 0, 0, nullptr);
+  const int IFTHEN    = cdoOperatorAdd("ifthen",    0, 0, nullptr);
+  const int IFNOTTHEN = cdoOperatorAdd("ifnotthen", 0, 0, nullptr);
   // clang-format on
 
-  int operatorID = cdoOperatorID();
+  const int operatorID = cdoOperatorID();
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-  int streamID2 = cdoStreamOpenRead(cdoStreamName(1));
+  CdoStreamID streamID1 = cdoOpenRead(0);
+  CdoStreamID streamID2 = cdoOpenRead(1);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = cdoStreamInqVlist(streamID2);
-  int vlistID3 = vlistDuplicate(vlistID2);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = cdoStreamInqVlist(streamID2);
+  const int vlistID3 = vlistDuplicate(vlistID2);
 
-  int taxisID2 = vlistInqTaxis(vlistID2);
-  int taxisID3 = taxisDuplicate(taxisID2);
+  const int taxisID2 = vlistInqTaxis(vlistID2);
+  const int taxisID3 = taxisDuplicate(taxisID2);
   vlistDefTaxis(vlistID3, taxisID3);
 
   int ntsteps1 = vlistNtsteps(vlistID1);
@@ -82,17 +79,21 @@ Cond(void *process)
   nospec(vlistID1);
   nospec(vlistID2);
 
-  int streamID3 = cdoStreamOpenWrite(cdoStreamName(2));
-  pstreamDefVlist(streamID3, vlistID3);
+  VarList varList1, varList2;
+  varListInit(varList1, vlistID1);
+  varListInit(varList2, vlistID2);
 
-  size_t gridsize = vlistGridsizeMax(vlistID2);
+  CdoStreamID streamID3 = cdoOpenWrite(2);
+  cdoDefVlist(streamID3, vlistID3);
 
-  if (filltype == FILL_REC && gridsize != gridInqSize(vlistGrid(vlistID1, 0)))
+  const size_t gridsizemax = vlistGridsizeMax(vlistID2);
+
+  if (filltype == FILL_REC && gridsizemax != gridInqSize(vlistGrid(vlistID1, 0)))
     cdoAbort("Stream1 >%s< has wrong gridsize!", cdoGetStreamName(0));
 
-  std::vector<double> array1(gridsize);
-  std::vector<double> array2(gridsize);
-  std::vector<double> array3(gridsize);
+  std::vector<double> array1(gridsizemax);
+  std::vector<double> array2(gridsizemax);
+  std::vector<double> array3(gridsizemax);
 
   if (Options::cdoVerbose) cdoPrint("Number of timesteps: file1 %d, file2 %d", ntsteps1, ntsteps2);
 
@@ -108,10 +109,9 @@ Cond(void *process)
           varnmiss1.resize(nvars);
           for (varID = 0; varID < nvars; varID++)
             {
-              gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-              nlev = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-              vardata1[varID].resize(nlev * gridsize);
-              varnmiss1[varID].resize(nlev);
+              const size_t nlevels = varList1[varID].nlevels;
+              vardata1[varID].resize(nlevels * varList1[varID].gridsize);
+              varnmiss1[varID].resize(nlevels);
             }
         }
     }
@@ -126,43 +126,38 @@ Cond(void *process)
         }
 
       taxisCopyTimestep(taxisID3, taxisID2);
-      pstreamDefTimestep(streamID3, tsID);
+      cdoDefTimestep(streamID3, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID2, &varID, &levelID);
-          pstreamReadRecord(streamID2, &array2[0], &nmiss2);
+          cdoInqRecord(streamID2, &varID, &levelID);
+          cdoReadRecord(streamID2, &array2[0], &nmiss2);
 
           if (tsID == 0 || filltype == FILL_NONE)
             {
               if (recID == 0 || filltype != FILL_REC)
                 {
-                  pstreamInqRecord(streamID1, &varID, &levelID);
-                  pstreamReadRecord(streamID1, &array1[0], &nmiss1);
+                  cdoInqRecord(streamID1, &varID, &levelID);
+                  cdoReadRecord(streamID1, &array1[0], &nmiss1);
                 }
 
               if (filltype == FILL_TS)
                 {
-                  gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-                  offset = gridsize * levelID;
-                  arrayCopy(gridsize, &array1[0], &vardata1[varID][offset]);
+                  const size_t offset = varList1[varID].gridsize * levelID;
+                  arrayCopy(varList1[varID].gridsize, &array1[0], &vardata1[varID][offset]);
                   varnmiss1[varID][levelID] = nmiss1;
                 }
             }
           else if (filltype == FILL_TS)
             {
-              gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-              offset = gridsize * levelID;
-              arrayCopy(gridsize, &vardata1[varID][offset], &array1[0]);
+              const size_t offset = varList1[varID].gridsize * levelID;
+              arrayCopy(varList1[varID].gridsize, &vardata1[varID][offset], &array1[0]);
               nmiss1 = varnmiss1[varID][levelID];
             }
 
-          gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
-          missval2 = vlistInqVarMissval(vlistID2, varID);
-          if (recID == 0 || filltype != FILL_REC)
-            {
-              missval1 = vlistInqVarMissval(vlistID1, varID);
-            }
+          const size_t gridsize = varList2[varID].gridsize;
+          const double missval2 = varList2[varID].missval;
+          if (recID == 0 || filltype != FILL_REC) missval1 = varList1[varID].missval;
 
           if (operatorID == IFTHEN)
             {
@@ -179,9 +174,9 @@ Cond(void *process)
               cdoAbort("Operator not implemented!");
             }
 
-          size_t nmiss3 = arrayNumMV(gridsize, &array3[0], missval2);
-          pstreamDefRecord(streamID3, varID, levelID);
-          pstreamWriteRecord(streamID3, &array3[0], nmiss3);
+          const size_t nmiss3 = arrayNumMV(gridsize, &array3[0], missval2);
+          cdoDefRecord(streamID3, varID, levelID);
+          cdoWriteRecord(streamID3, &array3[0], nmiss3);
         }
 
       tsID++;
