@@ -24,7 +24,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 
 void *
 Cond2(void *process)
@@ -47,9 +46,9 @@ Cond2(void *process)
 
   cdoOperatorAdd("ifthenelse", 0, 0, nullptr);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-  int streamID2 = cdoStreamOpenRead(cdoStreamName(1));
-  int streamID3 = cdoStreamOpenRead(cdoStreamName(2));
+  CdoStreamID streamID1 = cdoOpenRead(0);
+  CdoStreamID streamID2 = cdoOpenRead(1);
+  CdoStreamID streamID3 = cdoOpenRead(2);
 
   int vlistID1 = cdoStreamInqVlist(streamID1);
   int vlistID2 = cdoStreamInqVlist(streamID2);
@@ -79,8 +78,8 @@ Cond2(void *process)
   nospec(vlistID2);
   nospec(vlistID3);
 
-  int streamID4 = cdoStreamOpenWrite(cdoStreamName(3));
-  pstreamDefVlist(streamID4, vlistID4);
+  CdoStreamID streamID4 = cdoOpenWrite(3);
+  cdoDefVlist(streamID4, vlistID4);
 
   size_t gridsize = vlistGridsizeMax(vlistID1);
 
@@ -92,7 +91,8 @@ Cond2(void *process)
   std::vector<double> array3(gridsize);
   std::vector<double> array4(gridsize);
 
-  if (Options::cdoVerbose) cdoPrint("Number of timesteps: file1 %d, file2 %d, file3 %d", ntsteps1, ntsteps2, vlistNtsteps(vlistID3));
+  if (Options::cdoVerbose)
+    cdoPrint("Number of timesteps: file1 %d, file2 %d, file3 %d", ntsteps1, ntsteps2, vlistNtsteps(vlistID3));
 
   if (filltype == FILL_NONE)
     {
@@ -127,23 +127,23 @@ Cond2(void *process)
         }
 
       taxisCopyTimestep(taxisID4, taxisID2);
-      pstreamDefTimestep(streamID4, tsID);
+      cdoDefTimestep(streamID4, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
           size_t nmiss;
-          pstreamInqRecord(streamID2, &varID, &levelID);
-          pstreamReadRecord(streamID2, &array2[0], &nmiss);
+          cdoInqRecord(streamID2, &varID, &levelID);
+          cdoReadRecord(streamID2, &array2[0], &nmiss);
 
-          pstreamInqRecord(streamID3, &varID, &levelID);
-          pstreamReadRecord(streamID3, &array3[0], &nmiss);
+          cdoInqRecord(streamID3, &varID, &levelID);
+          cdoReadRecord(streamID3, &array3[0], &nmiss);
 
           if (tsID == 0 || filltype == FILL_NONE)
             {
               if (recID == 0 || filltype != FILL_REC)
                 {
-                  pstreamInqRecord(streamID1, &varID, &levelID);
-                  pstreamReadRecord(streamID1, &array1[0], &nmiss1);
+                  cdoInqRecord(streamID1, &varID, &levelID);
+                  cdoReadRecord(streamID1, &array1[0], &nmiss1);
                 }
 
               if (filltype == FILL_TS)
@@ -170,8 +170,8 @@ Cond2(void *process)
             array4[i] = DBL_IS_EQUAL(array1[i], missval1) ? missval2 : !DBL_IS_EQUAL(array1[i], 0.) ? array2[i] : array3[i];
 
           nmiss = arrayNumMV(gridsize, &array4[0], missval2);
-          pstreamDefRecord(streamID4, varID, levelID);
-          pstreamWriteRecord(streamID4, &array4[0], nmiss);
+          cdoDefRecord(streamID4, varID, levelID);
+          cdoWriteRecord(streamID4, &array4[0], nmiss);
         }
 
       tsID++;
