@@ -25,18 +25,16 @@
 
 #include "cdo_int.h"
 #include "par_io.h"
-#include "pstream_int.h"
+#include "progress.h"
 #include "cdo_options.h"
 
-extern "C" {
-int streamGrbInqDataScanningMode(void);
-}
+extern "C" int streamGrbInqDataScanningMode(void);
 
 void *
 Copy(void *process)
 {
   bool lconstvars = true;
-  int streamID2 = CDI_UNDEFID;
+  CdoStreamID streamID2 = CDO_STREAM_UNDEF;
   int vlistID2 = CDI_UNDEFID;
   int taxisID2 = CDI_UNDEFID;
   int nrecs;
@@ -47,7 +45,7 @@ Copy(void *process)
 
   cdoInitialize(process);
 
-  const bool lcopy = UNCHANGED_RECORD;
+  const bool lcopy = unchangedRecord();
 
   // clang-format off
                       cdoOperatorAdd("copy",   0, 0, nullptr);
@@ -59,7 +57,7 @@ Copy(void *process)
   // KEEP in mind the difference between copy and selall with respect to
   // unpacking and repacking the GRIB information! Especially when setting the
   // DataScanningMode.
-  printf("cdo copy/selall : UNCHANGED_RECORD=%d\n", UNCHANGED_RECORD);
+  printf("cdo copy/selall : UNCHANGED_RECORD=%d\n", unchangedRecord());
   // if (cdiGribDataScanningMode != -1) lcopy = false;
   printf("cdo copy/selall : cdiGribDataScanningMode=%d; lcopy=%d\n", streamGrbInqDataScanningMode(), lcopy);
 #endif  //#ifdef HIRLAM_EXTENSIONS
@@ -74,12 +72,14 @@ Copy(void *process)
   const int streamCnt = cdoStreamCnt();
   const int nfiles = streamCnt - 1;
 
+  Progress::init();
+
   int tsID2 = 0;
   for (int indf = 0; indf < nfiles; ++indf)
     {
       if (Options::cdoVerbose) cdoPrint("Process file: %s", cdoGetStreamName(indf));
 
-      const int streamID1 = cdoStreamOpenRead(cdoStreamName(indf));
+      CdoStreamID streamID1 = cdoOpenRead((indf));
       const int vlistID1 = cdoStreamInqVlist(streamID1);
       const int taxisID1 = vlistInqTaxis(vlistID1);
 
@@ -124,29 +124,34 @@ Copy(void *process)
           vlistCompare(vlistID1, vlistID2, CMP_ALL);
         }
 
-      if (streamID2 == CDI_UNDEFID)
+      if (streamID2 == CDO_STREAM_UNDEF)
         {
-          streamID2 = cdoStreamOpenWrite(cdoStreamName(nfiles));
-          pstreamDefVlist(streamID2, vlistID2);
+          streamID2 = cdoOpenWrite((nfiles));
+          cdoDefVlist(streamID2, vlistID2);
         }
 
+      const int ntsteps = vlistNtsteps(vlistID1);
+
       int tsID1 = 0;
       while ((nrecs = cdoStreamInqTimestep(streamID1, tsID1)))
         {
+          const double fstatus = (ntsteps > 1) ? indf + (tsID1 + 1.) / ntsteps : indf + 1.;
+          if (!Options::cdoVerbose) Progress::update(0, 1, fstatus / nfiles);
+
           taxisCopyTimestep(taxisID2, taxisID1);
-          pstreamDefTimestep(streamID2, tsID2);
+          cdoDefTimestep(streamID2, tsID2);
 
           for (int recID = 0; recID < nrecs; recID++)
             {
               if (lcopy && (operatorID == SELALL || operatorID == SZIP))
                 {
-                  pstreamInqRecord(streamID1, &varID, &levelID);
+                  cdoInqRecord(streamID1, &varID, &levelID);
 
                   if (lconstvars && tsID2 > 0 && tsID1 == 0)
                     if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
 
-                  pstreamDefRecord(streamID2, varID, levelID);
-                  pstreamCopyRecord(streamID2, streamID1);
+                  cdoDefRecord(streamID2, varID, levelID);
+                  cdoCopyRecord(streamID2, streamID1);
                 }
               else
                 {
@@ -162,26 +167,26 @@ Copy(void *process)
                     }
                   else
                     {
-                      pstreamInqRecord(streamID1, &varID, &levelID);
+                      cdoInqRecord(streamID1, &varID, &levelID);
 
                       if (lconstvars && tsID2 > 0 && tsID1 == 0)
                         if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
 
                       if (Options::CDO_Memtype == MEMTYPE_FLOAT)
-                        pstreamReadRecordF(streamID1, (float *) array, &nmiss);
+                        cdoReadRecordF(streamID1, (float *) array, &nmiss);
                       else
-                        pstreamReadRecord(streamID1, (double *) array, &nmiss);
+                        cdoReadRecord(streamID1, (double *) array, &nmiss);
                     }
                   /*
                   if ( Options::cdoParIO )
                     fprintf(stderr, "out1 %d %d %d\n", streamID2,  varID,
                   levelID);
                   */
-                  pstreamDefRecord(streamID2, varID, levelID);
+                  cdoDefRecord(streamID2, varID, levelID);
                   if (Options::CDO_Memtype == MEMTYPE_FLOAT)
-                    pstreamWriteRecordF(streamID2, (float *) array, nmiss);
+                    cdoWriteRecordF(streamID2, (float *) array, nmiss);
                   else
-                    pstreamWriteRecord(streamID2, (double *) array, nmiss);
+                    cdoWriteRecord(streamID2, (double *) array, nmiss);
                   /*
                   if ( Options::cdoParIO )
                     fprintf(stderr, "out2 %d %d %d\n", streamID2,  varID,
