@@ -24,10 +24,8 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 #include "datetime.h"
 
-
 void *
 Deltat(void *process)
 {
@@ -44,7 +42,7 @@ Deltat(void *process)
   const int operatorID = cdoOperatorID();
   const bool ldivdt = cdoOperatorF2(operatorID);
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -55,11 +53,11 @@ Deltat(void *process)
 
   const int calendar = taxisInqCalendar(taxisID1);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   FieldVector2D vars;
-  fieldsFromVlist(vlistID1, vars, FIELD_PTR);
+  fieldsFromVlist(vlistID1, vars, FIELD_VEC);
 
   const size_t gridsizemax = vlistGridsizeMax(vlistID1);
   std::vector<double> array1(gridsizemax), array2(gridsizemax);
@@ -71,8 +69,8 @@ Deltat(void *process)
 
   for (int recID = 0; recID < nrecs; ++recID)
     {
-      pstreamInqRecord(streamID1, &varID, &levelID);
-      pstreamReadRecord(streamID1, vars[varID][levelID].ptr, &nmiss);
+      cdoInqRecord(streamID1, &varID, &levelID);
+      cdoReadRecord(streamID1, vars[varID][levelID].vec.data(), &nmiss);
       vars[varID][levelID].nmiss = nmiss;
     }
 
@@ -81,23 +79,23 @@ Deltat(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       const JulianDate juldate1 = julianDateEncode(calendar, taxisInqVdate(taxisID1), taxisInqVtime(taxisID1));
-      const double idt_in_sec = ldivdt ? 1./julianDateToSeconds(julianDateSub(juldate1, juldate0)) : 1;
+      const double idt_in_sec = ldivdt ? 1. / julianDateToSeconds(julianDateSub(juldate1, juldate0)) : 1;
       juldate0 = juldate1;
 
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID2);
+      cdoDefTimestep(streamID2, tsID2);
 
       for (int recID = 0; recID < nrecs; ++recID)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, array1.data(), &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, array1.data(), &nmiss);
 
           const double missval = vars[varID][levelID].missval;
-          const size_t gridsize = vars[varID][levelID].size;
-          double *array0 = vars[varID][levelID].ptr;
+          const size_t fieldsize = vars[varID][levelID].size;
+          auto &array0 = vars[varID][levelID].vec;
           if (nmiss || vars[varID][levelID].nmiss)
             {
-              for (size_t i = 0; i < gridsize; ++i)
+              for (size_t i = 0; i < fieldsize; ++i)
                 {
                   if (DBL_IS_EQUAL(array0[i], missval) || DBL_IS_EQUAL(array1[i], missval))
                     array2[i] = missval;
@@ -105,25 +103,23 @@ Deltat(void *process)
                     array2[i] = (array1[i] - array0[i]) * idt_in_sec;
                 }
 
-              nmiss = arrayNumMV(gridsize, array2.data(), missval);
+              nmiss = arrayNumMV(fieldsize, array2.data(), missval);
             }
           else
             {
-              for (size_t i = 0; i < gridsize; ++i) array2[i] = (array1[i] - array0[i]) * idt_in_sec;
+              for (size_t i = 0; i < fieldsize; ++i) array2[i] = (array1[i] - array0[i]) * idt_in_sec;
             }
 
-          arrayCopy(gridsize, array1.data(), array0);
+          arrayCopy(fieldsize, array1.data(), array0.data());
 
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, array2.data(), nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, array2.data(), nmiss);
         }
 
       tsID++;
       tsID2++;
     }
 
-  fieldsFree(vlistID1, vars);
-
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
