@@ -18,7 +18,7 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
+
 #include "util_string.h"
 
 void *
@@ -34,7 +34,7 @@ Deltime(void *process)
 
   cdoInitialize(process);
 
-  bool lcopy = UNCHANGED_RECORD;
+  bool lcopy = unchangedRecord();
 
   // clang-format off
   int DELDAY   = cdoOperatorAdd("delday",   0, 0, nullptr);
@@ -64,7 +64,7 @@ Deltime(void *process)
           char smon[32];
           strncpy(smon, sarg, sizeof(smon) - 1);
           smon[sizeof(smon) - 1] = 0;
-          strtolower(smon);
+          cstrToLowerCase(smon);
           int im;
           for (im = 0; im < 12; ++im)
             if (memcmp(smon, cmons[im + 1], 3) == 0) break;
@@ -75,7 +75,7 @@ Deltime(void *process)
 
   if (Options::cdoVerbose) cdoPrint("delete day %d%s", dday, cmons[dmon]);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   int vlistID1 = cdoStreamInqVlist(streamID1);
   int vlistID2 = vlistDuplicate(vlistID1);
@@ -85,8 +85,8 @@ Deltime(void *process)
   taxisDefCalendar(taxisID2, CALENDAR_365DAYS);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   std::vector<double> array;
   if (!lcopy)
@@ -117,20 +117,20 @@ Deltime(void *process)
       if (copytimestep)
         {
           taxisCopyTimestep(taxisID2, taxisID1);
-          pstreamDefTimestep(streamID2, tsID2++);
+          cdoDefTimestep(streamID2, tsID2++);
 
           for (int recID = 0; recID < nrecs; recID++)
             {
-              pstreamInqRecord(streamID1, &varID, &levelID);
-              pstreamDefRecord(streamID2, varID, levelID);
+              cdoInqRecord(streamID1, &varID, &levelID);
+              cdoDefRecord(streamID2, varID, levelID);
               if (lcopy)
                 {
-                  pstreamCopyRecord(streamID2, streamID1);
+                  cdoCopyRecord(streamID2, streamID1);
                 }
               else
                 {
-                  pstreamReadRecord(streamID1, array.data(), &nmiss);
-                  pstreamWriteRecord(streamID2, array.data(), nmiss);
+                  cdoReadRecord(streamID1, array.data(), &nmiss);
+                  cdoWriteRecord(streamID2, array.data(), nmiss);
                 }
             }
         }
