@@ -23,7 +23,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 #include "after_vertint.h"
 #include "stdnametable.h"
 #include "util_string.h"
@@ -66,7 +65,7 @@ Derivepar(void *process)
 
   const int operatorID = cdoOperatorID();
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
 
@@ -150,10 +149,10 @@ Derivepar(void *process)
       if (code <= 0 || code == 255)
         {
           vlistInqVarName(vlistID1, varID, varname);
-          strtolower(varname);
+          cstrToLowerCase(varname);
 
           vlistInqVarStdname(vlistID1, varID, stdname);
-          strtolower(stdname);
+          cstrToLowerCase(stdname);
 
           code = echamcode_from_stdname(stdname);
           if (code < 0)
@@ -291,22 +290,22 @@ Derivepar(void *process)
   const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
           zaxisID = vlistInqVarZaxis(vlistID1, varID);
           nlevel = zaxisInqSize(zaxisID);
           offset = gridsize * levelID;
-          pstreamReadRecord(streamID1, array.data(), &nmiss);
+          cdoReadRecord(streamID1, array.data(), &nmiss);
 
           if (zaxisIDh != -1)
             {
@@ -335,11 +334,11 @@ Derivepar(void *process)
       if (zaxisIDh != -1)
         {
           // check range of ps_prog
-          arrayMinMaxMask(gridsize, ps.data(), nullptr, &minval, &maxval);
+          arrayMinMax(gridsize, ps.data(), minval, maxval);
           if (minval < MIN_PS || maxval > MAX_PS) cdoWarning("Surface pressure out of range (min=%g max=%g)!", minval, maxval);
 
           // check range of surface geopot
-          arrayMinMaxMask(gridsize, sgeopot.data(), nullptr, &minval, &maxval);
+          arrayMinMax(gridsize, sgeopot.data(), minval, maxval);
           if (minval < MIN_FIS || maxval > MAX_FIS) cdoWarning("Orography out of range (min=%g max=%g)!", minval, maxval);
         }
 
@@ -350,7 +349,7 @@ Derivepar(void *process)
           offset = gridsize * levelID;
           single2 = &temp[offset];
 
-          arrayMinMaxMask(gridsize, single2, nullptr, &minval, &maxval);
+          arrayMinMax(gridsize, single2, minval, maxval);
           if (minval < MIN_T || maxval > MAX_T)
             cdoWarning("Input temperature at level %d out of range (min=%g max=%g)!", levelID + 1, minval, maxval);
         }
@@ -366,7 +365,7 @@ Derivepar(void *process)
 
               // corr_hum(gridsize, single2, MIN_Q);
 
-              arrayMinMaxMask(gridsize, single2, nullptr, &minval, &maxval);
+              arrayMinMax(gridsize, single2, minval, maxval);
               if (minval < -0.1 || maxval > MAX_Q)
                 cdoWarning("Input humidity at level %d out of range (min=%g max=%g)!", levelID + 1, minval, maxval);
             }
@@ -383,8 +382,8 @@ Derivepar(void *process)
           nlevel = nhlevf;
           for (levelID = 0; levelID < nlevel; levelID++)
             {
-              pstreamDefRecord(streamID2, varID, levelID);
-              pstreamWriteRecord(streamID2, gheight.data() + levelID * gridsize, nmissout);
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, gheight.data() + levelID * gridsize, nmissout);
             }
         }
       else if (operatorID == SEALEVELPRESSURE)
@@ -394,8 +393,8 @@ Derivepar(void *process)
           extra_P(sealevelpressure.data(), &half_press[gridsize * (nhlevf)], &full_press[gridsize * (nhlevf - 1)], sgeopot.data(),
                   &temp[gridsize * (nhlevf - 1)], gridsize);
 
-          pstreamDefRecord(streamID2, 0, 0);
-          pstreamWriteRecord(streamID2, sealevelpressure.data(), 0);
+          cdoDefRecord(streamID2, 0, 0);
+          cdoWriteRecord(streamID2, sealevelpressure.data(), 0);
         }
       else
         cdoAbort("Internal error");
