@@ -24,37 +24,33 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 #include "cdo_options.h"
 #include "datetime.h"
 #include "cimdOmp.h"
 
+
 #define NALLOC_INC 1024
 
 static void
-detrend(const long nts, const double missval1, const double *array1, double *array2)
+detrend(const long nts, const double missval1, const std::vector<double> &array1, std::vector<double> &array2)
 {
-  double zj;
-  double sumj, sumjj;
-  double sumx, sumjx;
   const double missval2 = missval1;
-
-  sumx = sumjx = 0;
-  sumj = sumjj = 0;
+  double sumj = 0, sumjj = 0;
+  double sumx = 0, sumjx = 0;
   long n = 0;
   for (long j = 0; j < nts; j++)
     if (!DBL_IS_EQUAL(array1[j], missval1))
       {
-        zj = j;
-        sumx += array1[j];
-        sumjx += zj * array1[j];
+        const double zj = j;
         sumj += zj;
         sumjj += zj * zj;
+        sumjx += zj * array1[j];
+        sumx += array1[j];
         n++;
       }
 
-  const double work1 = DIVMN(SUBMN(sumjx, DIVMN(MULMN(sumx, sumj), n)), SUBMN(sumjj, DIVMN(MULMN(sumj, sumj), n)));
-  const double work2 = SUBMN(DIVMN(sumx, n), MULMN(work1, DIVMN(sumj, n)));
+  const double work1 = DIVMN(SUBMN(sumjx, DIVMN(MULMN(sumj, sumx), n)), SUBMN(sumjj, DIVMN(MULMN(sumj, sumj), n)));
+  const double work2 = SUBMN(DIVMN(sumx, n), MULMN(DIVMN(sumj, n), work1));
 
   for (long j = 0; j < nts; j++) array2[j] = SUBMN(array1[j], ADDMN(work2, MULMN(j, work1)));
 }
@@ -64,13 +60,12 @@ Detrend(void *process)
 {
   int nrecs;
   int varID, levelID;
-  int nalloc = 0;
   size_t nmiss;
   DateTimeList dtlist;
 
   cdoInitialize(process);
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -79,8 +74,11 @@ Detrend(void *process)
   const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
+
+  VarList varList;
+  varListInit(varList, vlistID1);
 
   const int nvars = vlistNvars(vlistID1);
   FieldVector3D vars;
@@ -88,11 +86,7 @@ Detrend(void *process)
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
-      if (tsID >= nalloc)
-        {
-          nalloc += NALLOC_INC;
-          vars.resize(nalloc);
-        }
+      if ((size_t)tsID >= vars.size()) vars.resize(vars.size() + NALLOC_INC);
 
       dtlist.taxisInqTimestep(taxisID1, tsID);
 
@@ -100,42 +94,39 @@ Detrend(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          int gridID = vlistInqVarGrid(vlistID1, varID);
-          size_t gridsize = gridInqSize(gridID);
-          vars[tsID][varID][levelID].ptr = (double *) Malloc(gridsize * sizeof(double));
-          pstreamReadRecord(streamID1, vars[tsID][varID][levelID].ptr, &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          vars[tsID][varID][levelID].resize(varList[varID].gridsize);
+          cdoReadRecord(streamID1, vars[tsID][varID][levelID].vec.data(), &nmiss);
           vars[tsID][varID][levelID].nmiss = nmiss;
         }
 
       tsID++;
     }
 
-  int nts = tsID;
-
+  const int nts = tsID;
   VECTOR_2D(double, array1, Threading::ompNumThreads, nts);
   VECTOR_2D(double, array2, Threading::ompNumThreads, nts);
 
   for (varID = 0; varID < nvars; varID++)
     {
-      int gridID = vlistInqVarGrid(vlistID1, varID);
-      double missval = vlistInqVarMissval(vlistID1, varID);
-      size_t gridsize = gridInqSize(gridID);
-      int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-      for (levelID = 0; levelID < nlevel; levelID++)
+      int nsteps = (varList[varID].timetype == TIME_CONSTANT) ? 1 : nts;
+      double missval = varList[varID].missval;
+      size_t gridsize = varList[varID].gridsize;
+      const int nlevels = varList[varID].nlevels;
+      for (levelID = 0; levelID < nlevels; levelID++)
         {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(array1, array2, vars, varID, levelID, gridsize, nts, missval)
+#pragma omp parallel for default(none) shared(gridsize, nsteps, missval, array1, array2, vars, varID, levelID)
 #endif
           for (size_t i = 0; i < gridsize; i++)
             {
               const int ompthID = cdo_omp_get_thread_num();
 
-              for (int tsID = 0; tsID < nts; tsID++) array1[ompthID][tsID] = vars[tsID][varID][levelID].ptr[i];
+              for (int tsID = 0; tsID < nsteps; ++tsID) array1[ompthID][tsID] = vars[tsID][varID][levelID].vec[i];
 
-              detrend(nts, missval, &array1[ompthID][0], &array2[ompthID][0]);
+              detrend(nsteps, missval, array1[ompthID], array2[ompthID]);
 
-              for (int tsID = 0; tsID < nts; tsID++) vars[tsID][varID][levelID].ptr[i] = array2[ompthID][tsID];
+              for (int tsID = 0; tsID < nsteps; ++tsID) vars[tsID][varID][levelID].vec[i] = array2[ompthID][tsID];
             }
         }
     }
@@ -143,25 +134,19 @@ Detrend(void *process)
   for (int tsID = 0; tsID < nts; tsID++)
     {
       dtlist.taxisDefTimestep(taxisID2, tsID);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (varID = 0; varID < nvars; varID++)
         {
-          int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-          for (levelID = 0; levelID < nlevel; levelID++)
+          if (tsID && varList[varID].timetype == TIME_CONSTANT) continue;
+          const int nlevels = varList[varID].nlevels;
+          for (levelID = 0; levelID < nlevels; levelID++)
             {
-              if (vars[tsID][varID][levelID].ptr)
-                {
-                  nmiss = vars[tsID][varID][levelID].nmiss;
-                  pstreamDefRecord(streamID2, varID, levelID);
-                  pstreamWriteRecord(streamID2, vars[tsID][varID][levelID].ptr, nmiss);
-                  Free(vars[tsID][varID][levelID].ptr);
-                  vars[tsID][varID][levelID].ptr = nullptr;
-                }
+              nmiss = vars[tsID][varID][levelID].nmiss;
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, vars[tsID][varID][levelID].vec.data(), nmiss);
             }
         }
-
-      fieldsFree(vlistID1, vars[tsID]);
     }
 
   cdoStreamClose(streamID2);
