@@ -27,8 +27,8 @@
 #include "cdo_int.h"
 #include "param_conversion.h"
 #include "cdo_math.h"
-#include "pstream_int.h"
-#include "text.h"
+
+#include "mpmo_color.h"
 #include "cdo_options.h"
 #include "printinfo.h"
 #include "pmlist.h"
@@ -103,8 +103,8 @@ Diff(void *process)
   if (abslim < -1.e33 || abslim > 1.e+33) cdoAbort("Abs. limit out of range!");
   if (abslim2 < -1.e33 || abslim2 > 1.e+33) cdoAbort("Abs2. limit out of range!");
 
-  const int streamID1 = cdoStreamOpenRead(0);
-  const int streamID2 = cdoStreamOpenRead(1);
+  CdoStreamID streamID1 = cdoOpenRead(0);
+  CdoStreamID streamID2 = cdoOpenRead(1);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = cdoStreamInqVlist(streamID2);
@@ -149,7 +149,7 @@ Diff(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID1, &levelID);
+          cdoInqRecord(streamID1, &varID1, &levelID);
 
           auto it = mapOfVarIDs.find(varID1);
           if (it == mapOfVarIDs.end())
@@ -160,14 +160,14 @@ Diff(void *process)
 
           for (; recID2next < nrecs2; ++recID2next)
             {
-              pstreamInqRecord(streamID2, &varID2, &levelID);
+              cdoInqRecord(streamID2, &varID2, &levelID);
               if (it->second == varID2)
                 {
                   ++recID2next;
                   break;
                 }
             }
-       
+
           if (it->second != varID2 && recID2next == nrecs2)
             cdoAbort("Internal problem: varID2=%d not found in second stream!", it->second);
 
@@ -188,19 +188,19 @@ Diff(void *process)
 
           if (vlistInqVarNumber(vlistID1, varID1) == CDI_COMP)
             {
-              pstreamReadRecord(streamID1, work.data(), &nmiss1);
+              cdoReadRecord(streamID1, work.data(), &nmiss1);
               for (size_t i = 0; i < gridsize; ++i) array1[i] = work[i * 2];
             }
           else
-            pstreamReadRecord(streamID1, array1.data(), &nmiss1);
+            cdoReadRecord(streamID1, array1.data(), &nmiss1);
 
           if (vlistInqVarNumber(vlistID2, varID2) == CDI_COMP)
             {
-              pstreamReadRecord(streamID2, work.data(), &nmiss2);
+              cdoReadRecord(streamID2, work.data(), &nmiss2);
               for (size_t i = 0; i < gridsize; ++i) array2[i] = work[i * 2];
             }
           else
-            pstreamReadRecord(streamID2, array2.data(), &nmiss2);
+            cdoReadRecord(streamID2, array2.data(), &nmiss2);
 
           int ndiff = 0;
           bool dsgn = false;
@@ -267,27 +267,27 @@ Diff(void *process)
                   set_text_color(stdout, BRIGHT, BLACK);
                   fprintf(stdout, "%6d ", indg);
                   reset_text_color(stdout);
-                  set_text_color(stdout, RESET, BLACK);
+                  set_text_color(stdout, BLACK);
                   fprintf(stdout, ":");
                   reset_text_color(stdout);
 
-                  set_text_color(stdout, RESET, MAGENTA);
+                  set_text_color(stdout, MAGENTA);
                   fprintf(stdout, "%s %s ", vdatestr, vtimestr);
                   reset_text_color(stdout);
-                  set_text_color(stdout, RESET, GREEN);
+                  set_text_color(stdout, GREEN);
                   double level = cdoZaxisInqLevel(zaxisID, levelID);
                   fprintf(stdout, "%7g ", level);
                   fprintf(stdout, "%8zu %7zu ", gridsize, cdo::max(nmiss1, nmiss2));
                   fprintf(stdout, "%7d ", ndiff);
                   reset_text_color(stdout);
 
-                  set_text_color(stdout, RESET, BLACK);
+                  set_text_color(stdout, BLACK);
                   fprintf(stdout, ":");
                   reset_text_color(stdout);
                   fprintf(stdout, " %c %c ", dsgn ? 'T' : 'F', zero ? 'T' : 'F');
-                  set_text_color(stdout, RESET, BLUE);
+                  set_text_color(stdout, BLUE);
                   fprintf(stdout, "%#12.5g%#12.5g", absm, relm);
-                  set_text_color(stdout, RESET, BLACK);
+                  set_text_color(stdout, BLACK);
                   fprintf(stdout, " : ");
                   reset_text_color(stdout);
 
