@@ -20,8 +20,8 @@
 #include "cdo_int.h"
 #include "param_conversion.h"
 #include "cdo_math.h"
-#include "grid.h"
-#include "pstream_int.h"
+#include <mpim_grid.h>
+
 #include "util_files.h"
 
 #define MAX_BLOCKS 65536
@@ -248,7 +248,7 @@ Distgrid(void *process)
   if (nxblocks == 0) cdoAbort("nxblocks has to be greater than 0!");
   if (nyblocks == 0) cdoAbort("nyblocks has to be greater than 0!");
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
   const int vlistID1 = cdoStreamInqVlist(streamID1);
 
   const int ngrids = vlistNgrids(vlistID1);
@@ -290,15 +290,15 @@ Distgrid(void *process)
 
   size_t xinc = nx / nxblocks;
   size_t yinc = ny / nyblocks;
-  if (nx % xinc && nx % (xinc + 1) && nxblocks*(xinc + 1) <= nx) xinc++;
-  if (ny % yinc && ny % (yinc + 1) && nyblocks*(yinc + 1) <= ny) yinc++;
+  if (nx % xinc && nx % (xinc + 1) && nxblocks * (xinc + 1) <= nx) xinc++;
+  if (ny % yinc && ny % (yinc + 1) && nyblocks * (yinc + 1) <= ny) yinc++;
 
   const size_t nsplit = nxblocks * nyblocks;
   if (nsplit > MAX_BLOCKS) cdoAbort("Too many blocks (max = %d)!", MAX_BLOCKS);
 
   std::vector<double> array1(gridsize);
   std::vector<int> vlistIDs(nsplit);
-  std::vector<int> streamIDs(nsplit);
+  std::vector<CdoStreamID> streamIDs(nsplit);
 
   std::vector<GridInfo> gridinfo(ngrids);
   for (int i = 0; i < ngrids; i++)
@@ -338,27 +338,27 @@ Distgrid(void *process)
 
   const char *refname = cdoGetObase();
   filesuffix[0] = 0;
-  cdoGenFileSuffix(filesuffix, sizeof(filesuffix), pstreamInqFiletype(streamID1), vlistID1, refname);
+  cdoGenFileSuffix(filesuffix, sizeof(filesuffix), cdoInqFiletype(streamID1), vlistID1, refname);
 
   for (size_t index = 0; index < nsplit; index++)
     {
       sprintf(filename + nchars, "%05ld", (long) index);
       if (filesuffix[0]) sprintf(filename + nchars + 5, "%s", filesuffix);
 
-      streamIDs[index] = cdoStreamOpenWrite(filename);
-      pstreamDefVlist(streamIDs[index], vlistIDs[index]);
+      streamIDs[index] = cdoOpenWrite(filename);
+      cdoDefVlist(streamIDs[index], vlistIDs[index]);
     }
 
   if (ngrids > 1) cdoPrint("Baustelle: number of different grids > 1!");
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
-      for (size_t index = 0; index < nsplit; index++) pstreamDefTimestep(streamIDs[index], tsID);
+      for (size_t index = 0; index < nsplit; index++) cdoDefTimestep(streamIDs[index], tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, &array1[0], &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, &array1[0], &nmiss);
 
           const double missval = vlistInqVarMissval(vlistID1, varID);
 
@@ -366,9 +366,9 @@ Distgrid(void *process)
             {
               int i = 0;
               window_cell(&array1[0], &array2[0], gridinfo[i].gridsize[index], gridinfo[i].gridindex[index].data());
-              pstreamDefRecord(streamIDs[index], varID, levelID);
-              if (nmiss > 0) nmiss = arrayNumMV(gridinfo[i].gridsize[index], &array2[0], missval);
-              pstreamWriteRecord(streamIDs[index], &array2[0], nmiss);
+              cdoDefRecord(streamIDs[index], varID, levelID);
+              if (nmiss) nmiss = arrayNumMV(gridinfo[i].gridsize[index], &array2[0], missval);
+              cdoWriteRecord(streamIDs[index], &array2[0], nmiss);
             }
         }
 
