@@ -19,7 +19,6 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
 
 #define NALLOC_INC 1024
 
@@ -28,30 +27,26 @@ Duplicate(void *process)
 {
   int nrecs;
   int varID, levelID;
-  int nalloc = 0;
   size_t nmiss;
-  int ndup = 2;
 
   cdoInitialize(process);
 
-  if (operatorArgc() > 1)
-    cdoAbort("Too many arguments!");
-  else if (operatorArgc() == 1)
-    ndup = parameter2int(operatorArgv()[0]);
+  if (operatorArgc() > 1) cdoAbort("Too many arguments!");
 
+  const int ndup = (operatorArgc() == 1) ? parameter2int(operatorArgv()[0]) : 2;
   if (Options::cdoVerbose) cdoPrint("ndup = %d", ndup);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
+  const int nvars = vlistNvars(vlistID1);
   int ntsteps = vlistNtsteps(vlistID1);
-  int nvars = vlistNvars(vlistID1);
 
   if (ntsteps == 1)
     {
@@ -66,8 +61,11 @@ Duplicate(void *process)
       for (varID = 0; varID < nvars; ++varID) vlistDefVarTimetype(vlistID2, varID, TIME_VARYING);
     }
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
+
+  VarList varList;
+  varListInit(varList, vlistID1);
 
   FieldVector3D vars;
   std::vector<int64_t> vdate;
@@ -76,13 +74,9 @@ Duplicate(void *process)
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
-      if (tsID >= nalloc)
-        {
-          nalloc += NALLOC_INC;
-          vdate.resize(nalloc);
-          vtime.resize(nalloc);
-          vars.resize(nalloc);
-        }
+      if ((size_t)tsID >= vdate.size()) vdate.resize(vdate.size() + NALLOC_INC);
+      if ((size_t)tsID >= vtime.size()) vtime.resize(vtime.size() + NALLOC_INC);
+      if ((size_t)tsID >= vars.size()) vars.resize(vars.size() + NALLOC_INC);
 
       vdate[tsID] = taxisInqVdate(taxisID1);
       vtime[tsID] = taxisInqVtime(taxisID1);
@@ -91,11 +85,10 @@ Duplicate(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          int gridID = vlistInqVarGrid(vlistID1, varID);
-          size_t gridsize = gridInqSize(gridID);
-          vars[tsID][varID][levelID].ptr = (double *) Malloc(gridsize * sizeof(double));
-          pstreamReadRecord(streamID1, vars[tsID][varID][levelID].ptr, &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          const size_t gridsize = varList[varID].gridsize;
+          vars[tsID][varID][levelID].resize(gridsize);
+          cdoReadRecord(streamID1, vars[tsID][varID][levelID].vec.data(), &nmiss);
           vars[tsID][varID][levelID].nmiss = nmiss;
         }
 
@@ -110,26 +103,24 @@ Duplicate(void *process)
         {
           taxisDefVdate(taxisID2, vdate[tsID]);
           taxisDefVtime(taxisID2, vtime[tsID]);
-          pstreamDefTimestep(streamID2, idup * nts + tsID);
+          cdoDefTimestep(streamID2, idup * nts + tsID);
 
           for (varID = 0; varID < nvars; varID++)
             {
-              int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+              const int nlevel = varList[varID].nlevels;
               for (levelID = 0; levelID < nlevel; levelID++)
                 {
-                  if (vars[tsID][varID][levelID].ptr)
+                  if (!vars[tsID][varID][levelID].vec.empty())
                     {
                       nmiss = vars[tsID][varID][levelID].nmiss;
-                      pstreamDefRecord(streamID2, varID, levelID);
-                      pstreamWriteRecord(streamID2, vars[tsID][varID][levelID].ptr, nmiss);
+                      cdoDefRecord(streamID2, varID, levelID);
+                      cdoWriteRecord(streamID2, vars[tsID][varID][levelID].vec.data(), nmiss);
                     }
                 }
             }
         }
     }
 
-  for (tsID = 0; tsID < nts; tsID++) fieldsFree(vlistID1, vars[tsID]);
-
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
