@@ -36,8 +36,8 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
-#include "grid.h"
+
+#include <mpim_grid.h>
 #include "statistic.h"
 #include "timer.h"
 #include "datetime.h"
@@ -213,7 +213,7 @@ EOFs(void *process)
   enum T_EIGEN_MODE eigen_mode = get_eigenmode();
   enum T_WEIGHT_MODE weight_mode = get_weightmode();
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
   int vlistID1 = cdoStreamInqVlist(streamID1);
   int taxisID1 = vlistInqTaxis(vlistID1);
   int gridID1 = vlistInqVarGrid(vlistID1, 0);
@@ -242,7 +242,7 @@ EOFs(void *process)
 
           cdoStreamClose(streamID1);
 
-          streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+          streamID1 = cdoOpenRead(0);
           vlistID1 = cdoStreamInqVlist(streamID1);
           taxisID1 = vlistInqTaxis(vlistID1);
         }
@@ -351,8 +351,8 @@ EOFs(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, in, &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, in, &nmiss);
 
           size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
           double missval = vlistInqVarMissval(vlistID1, varID);
@@ -431,7 +431,7 @@ EOFs(void *process)
   /* write files with eigenvalues (ID3) and eigenvectors (ID2) */
 
   /* eigenvalues */
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
+  CdoStreamID streamID2 = cdoOpenWrite(1);
 
   int vlistID2 = vlistDuplicate(vlistID1);
   int taxisID2 = taxisDuplicate(taxisID1);
@@ -448,7 +448,7 @@ EOFs(void *process)
   for (int i = 0; i < ngrids; i++) vlistChangeGridIndex(vlistID2, i, gridID2);
 
   /*  eigenvectors */
-  int streamID3 = cdoStreamOpenWrite(cdoStreamName(2));
+  CdoStreamID streamID3 = cdoOpenWrite(2);
 
   int vlistID3 = vlistDuplicate(vlistID1);
   int taxisID3 = taxisDuplicate(taxisID1);
@@ -456,8 +456,8 @@ EOFs(void *process)
   taxisDefRtime(taxisID3, 0);
   vlistDefTaxis(vlistID3, taxisID3);
 
-  pstreamDefVlist(streamID2, vlistID2);
-  pstreamDefVlist(streamID3, vlistID3);
+  cdoDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID3, vlistID3);
 
   int64_t vdate = 10101;
   int vtime = 0;
@@ -476,13 +476,13 @@ EOFs(void *process)
 
       taxisDefVdate(taxisID2, vdate);
       taxisDefVtime(taxisID2, vtime);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       if (tsID < n_eig)
         {
           taxisDefVdate(taxisID3, vdate);
           taxisDefVtime(taxisID3, vtime);
-          pstreamDefTimestep(streamID3, tsID);
+          cdoDefTimestep(streamID3, tsID);
         }
 
       for (varID = 0; varID < nvars; varID++)
@@ -601,14 +601,14 @@ EOFs(void *process)
                     scale_eigvec_time(out, tsID, nts, npack, pack.data(), weight.data(), covar, data, missval, sum_w);
 
                   nmiss = arrayNumMV(gridsize, out, missval);
-                  pstreamDefRecord(streamID3, varID, levelID);
-                  pstreamWriteRecord(streamID3, out, nmiss);
+                  cdoDefRecord(streamID3, varID, levelID);
+                  cdoWriteRecord(streamID3, out, nmiss);
                 }  // loop n_eig
 
               nmiss = (DBL_IS_EQUAL(eig_val[tsID], missval)) ? 1 : 0;
 
-              pstreamDefRecord(streamID2, varID, levelID);
-              pstreamWriteRecord(streamID2, &eig_val[tsID], nmiss);
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, &eig_val[tsID], nmiss);
             }  // loop nlevs
         }      // loop nvars
     }
