@@ -15,49 +15,44 @@
   GNU General Public License for more details.
 */
 
+// clang-format off
 /*
       MODULE      OPERATOR     INDEX    DESCRIPTION
 
       EcaCfd      eca_cfd      CFD      maximum number of consecutive frost days
-      EcaCsu      eca_csu      CSU      maximum number of consecutive summer
-   days EcaCwdi     eca_cwdi     CWDI     cold wave duration index EcaCwfi
-   eca_cwfi     CWFI     number of cold-spell days EcaEtr      eca_etr      ETR
-   intra-period extreme temperature range EcaFd       eca_fd       FD
-   number of frost days EcaGsl      eca_gsl      GSL      growing season length
+      EcaCsu      eca_csu      CSU      maximum number of consecutive summer days
+      EcaCwdi     eca_cwdi     CWDI     cold wave duration index 
+      EcaCwfi     eca_cwfi     CWFI     number of cold-spell days
+      EcaEtr      eca_etr      ETR      intra-period extreme temperature range
+      EcaFd       eca_fd       FD       number of frost days
+      EcaGsl      eca_gsl      GSL      growing season length
       EcaHd       eca_hd       HD       heating degree days
       EcaHwdi     eca_hwdi     HWDI     heat wave duration index
       EcaHwfi     eca_hwfi     HWFI     number of warm-spell days
       EcaId       eca_id       ID       number of ice days
       EcaSu       eca_su       SU       number of summer days
-      EcaTg10p    eca_tg10p    TG10p    percent of time TX < 10th percentile of
-   daily mean temperature EcaTg90p    eca_tg90p    TG90p    percent of time TX >
-   90th percentile of daily mean temperature EcaTn10p    eca_tn10p    TN10p
-   percent of time TX < 10th percentile of daily minimum temperature EcaTn90p
-   eca_tn90p    TN90p    percent of time TX > 90th percentile of daily minimum
-   temperature EcaTr       eca_tr       TR       number of tropical nights
-      EcaTx10p    eca_tx10p    TX10p    percent of time TX < 10th percentile of
-   daily maximum temperature EcaTx90p    eca_tx90p    TX90p    percent of time
-   TX > 90th percentile of daily maximum temperature
+      EcaTg10p    eca_tg10p    TG10p    percent of time TX < 10th percentile of daily mean temperature
+      EcaTg90p    eca_tg90p    TG90p    percent of time TX > 90th percentile of daily mean temperature
+      EcaTn10p    eca_tn10p    TN10p    percent of time TX < 10th percentile of daily minimum temperature
+      EcaTn90p    eca_tn90p    TN90p    percent of time TX > 90th percentile of daily minimum temperature
+      EcaTr       eca_tr       TR       number of tropical nights
+      EcaTx10p    eca_tx10p    TX10p    percent of time TX < 10th percentile of daily maximum temperature
+      EcaTx90p    eca_tx90p    TX90p    percent of time TX > 90th percentile of daily maximum temperature
 
       EcaCdd      eca_cdd      CDD      maximum number of consecutive dry days
       EcaCwd      eca_cwd      CWD      maximum number of consecutive wet days
-      EcaR10mm    eca_r10mm    R10mm    number of days with precipitation >= 10
-   mm EcaR20mm    eca_r20mm    R20mm    number of days with precipitation >= 20
-   mm EcaR75p     eca_r75p     R75p     Percent of time RR > 75th percentile of
-   daily precipitation amount EcaR75ptot  eca_r75ptot  R75pTOT  Percentage of
-   annual total precipitation due to events wit RR > 75th percentile of daily
-   precipitation amount EcaR90p     eca_r90p     R90p     Percent of time RR >
-   90th percentile of daily precipitation amount EcaR90ptot  eca_r90ptot
-   R90pTOT  Percentage of annual total precipitation due to events wit RR > 90th
-   percentile of daily precipitation amount EcaR95p     eca_r95p     R95p
-   Percent of time RR > 95th percentile of daily precipitation amount EcaR95ptot
-   eca_r95ptot  R95pTOT  Percentage of annual total precipitation due to events
-   wit RR > 95th percentile of daily precipitation amount EcaR99p     eca_r99p
-   R99p     Percent of time RR > 75th percentile of daily precipitation amount
-      EcaR99ptot  eca_r99ptot  R99pTOT  Percentage of annual total precipitation
-   due to events wit RR > 99th percentile of daily precipitation amount EcaRr1
-   eca_rr1      RR1      number of wet days EcaSdii     eca_sdii     SDII
-   simple daily intensity index
+      EcaR10mm    eca_r10mm    R10mm    number of days with precipitation >= 10 mm
+      EcaR20mm    eca_r20mm    R20mm    number of days with precipitation >= 20 mm
+      EcaR75p     eca_r75p     R75p     Percent of time RR > 75th percentile of daily precipitation amount
+      EcaR75ptot  eca_r75ptot  R75pTOT  Percentage of annual total precipitation due to events with RR > 75th percentile of daily precipitation amount
+      EcaR90p     eca_r90p     R90p     Percent of time RR > 90th percentile of daily precipitation amount
+      EcaR90ptot  eca_r90ptot  R90pTOT  Percentage of annual total precipitation due to events with RR > 90th percentile of daily precipitation amount
+      EcaR95p     eca_r95p     R95p     Percent of time RR > 95th percentile of daily precipitation amount
+      EcaR95ptot  eca_r95ptot  R95pTOT  Percentage of annual total precipitation due to events with RR > 95th percentile of daily precipitation amount
+      EcaR99p     eca_r99p     R99p     Percent of time RR > 75th percentile of daily precipitation amount
+      EcaR99ptot  eca_r99ptot  R99pTOT  Percentage of annual total precipitation due to events with RR > 99th percentile of daily precipitation amount
+      EcaRr1      eca_rr1      RR1      number of wet days
+      EcaSdii     eca_sdii     SDII     simple daily intensity index
 
       Fdns        fdns                  frost days without surface snow
 
@@ -66,6 +61,7 @@
       Strgal      strgal                number of strong-gale days
       Hurr        hurr                  number of hurricane days
 */
+// clang-format on
 
 #include "cdo_int.h"
 #include "param_conversion.h"
@@ -291,10 +287,7 @@ void *EcaCfd(void *process)
   cdoOperatorAdd("eca_cfd", 0, 31, nullptr);
   
   if ( operatorArgc() > 1 ) cdoAbort("Too many arguments!");
-  else if ( operatorArgc() > 0 ) 
-    {
-      ndays = parameter2int(operatorArgv()[0]);
-    }
+  if ( operatorArgc() > 0 ) ndays = parameter2int(operatorArgv()[0]);
 
   sprintf(cfd_longname2, CFD_LONGNAME2, ndays);
   sprintf(cfd_name2,     CFD_NAME2, ndays);
@@ -302,20 +295,20 @@ void *EcaCfd(void *process)
   request.var1.name     = CFD_NAME;
   request.var1.longname = CFD_LONGNAME;
   request.var1.units    = nullptr;
-  request.var1.f1       = farselltc;
+  request.var1.f1       = vfarselltc;
   request.var1.f1arg    = TO_KELVIN(0.0);
-  request.var1.f2       = farnum2;
-  request.var1.f3       = farmax;
+  request.var1.f2       = vfarnum2;
+  request.var1.f3       = vfarmax;
   request.var1.mulc     = 0.0;
   request.var1.addc     = 0.0;
-  request.var1.epilog   = NONE;
+  request.var1.epilog   = ECA_NONE;
   request.var2.name     = cfd_name2;
   request.var2.longname = cfd_longname2;
   request.var2.units    = CFD_UNITS2;
-  request.var2.h1       = farseleqc;
+  request.var2.h1       = vfarseleqc;
   request.var2.h1arg    = ndays+1;
   request.var2.h2       = nullptr;
-  request.var2.h3       = farnum;
+  request.var2.h3       = vfarnum;
    
   eca1(&request);
   cdoFinish();
@@ -348,20 +341,20 @@ void *EcaCsu(void *process)
   request.var1.name     = CSU_NAME;
   request.var1.longname = CSU_LONGNAME;
   request.var1.units    = nullptr;
-  request.var1.f1       = farselgtc;
+  request.var1.f1       = vfarselgtc;
   request.var1.f1arg    = TO_KELVIN(argT);
-  request.var1.f2       = farnum2;
-  request.var1.f3       = farmax;
+  request.var1.f2       = vfarnum2;
+  request.var1.f3       = vfarmax;
   request.var1.mulc     = 0.0;
   request.var1.addc     = 0.0;
-  request.var1.epilog   = NONE;
+  request.var1.epilog   = ECA_NONE;
   request.var2.name     = csu_name2;
   request.var2.longname = csu_longname2;
   request.var2.units    = CSU_UNITS2;
-  request.var2.h1       = farseleqc;
+  request.var2.h1       = vfarseleqc;
   request.var2.h1arg    = ndays+1;
   request.var2.h2       = nullptr;
-  request.var2.h3       = farnum;
+  request.var2.h3       = vfarnum;
   
   eca1(&request);
   
@@ -391,19 +384,19 @@ void *EcaCwdi(void *process)
   request.var1.longname = longname;
   request.var1.units    = CWDI_UNITS;
   request.var1.f1       = nullptr;
-  request.var1.f2       = farcsub;
+  request.var1.f2       = vfarcsub;
   request.var1.f2arg    = argT;
-  request.var1.f3       = farsellt;
-  request.var1.f4       = farnum2;
-  request.var1.f5       = farnum3;
+  request.var1.f3       = vfarsellt;
+  request.var1.f4       = vfarnum2;
+  request.var1.f5       = vfarnum3;
   request.var1.f5arg    = argN;
-  request.var1.epilog   = NONE;
+  request.var1.epilog   = ECA_NONE;
   request.var2.name     = CWDI_NAME2;
   request.var2.longname = CWDI_LONGNAME2;
   request.var2.units    = CWDI_UNITS2;
-  request.var2.h1       = farseleqc;
+  request.var2.h1       = vfarseleqc;
   request.var2.h1arg    = argN;
-  request.var2.h2       = farnum;
+  request.var2.h2       = vfarnum;
    
   eca2(&request);
   
@@ -416,7 +409,6 @@ void *EcaCwdi(void *process)
 
 void *EcaCwfi(void *process)
 {
-  char *longname;
   int argN = 6;
   ECA_REQUEST_2 request;
   
@@ -425,7 +417,7 @@ void *EcaCwfi(void *process)
 
   if ( operatorArgc() > 0 ) argN = parameter2int(operatorArgv()[0]);
 
-  longname = (char*) Malloc(strlen(CWFI_LONGNAME) + 40);
+  char *longname = (char*) Malloc(strlen(CWFI_LONGNAME) + 40);
   sprintf(longname, CWFI_LONGNAME, argN);
 
   request.var1.name     = CWFI_NAME;
@@ -433,17 +425,17 @@ void *EcaCwfi(void *process)
   request.var1.units    = CWFI_UNITS;
   request.var1.f1       = nullptr;
   request.var1.f2       = nullptr;
-  request.var1.f3       = farsellt;
-  request.var1.f4       = farnum2;
-  request.var1.f5       = farnum3;
+  request.var1.f3       = vfarsellt;
+  request.var1.f4       = vfarnum2;
+  request.var1.f5       = vfarnum3;
   request.var1.f5arg    = argN;
-  request.var1.epilog   = NONE;
+  request.var1.epilog   = ECA_NONE;
   request.var2.name     = CWFI_NAME2;
   request.var2.longname = CWFI_LONGNAME2;
   request.var2.units    = CWFI_UNITS2;
-  request.var2.h1       = farseleqc;
+  request.var2.h1       = vfarseleqc;
   request.var2.h1arg    = argN;
-  request.var2.h2       = farnum;
+  request.var2.h2       = vfarnum;
    
   eca2(&request);
   
@@ -464,9 +456,9 @@ void *EcaEtr(void *process)
   request.name     = ETR_NAME;
   request.longname = ETR_LONGNAME;
   request.units    = nullptr;
-  request.f1       = farmax; 
-  request.f2       = farmin;
-  request.f3       = farsub;
+  request.f1       = vfarmax; 
+  request.f2       = vfarmin;
+  request.f3       = vfarsub;
    
   eca3(&request);
   cdoFinish();
@@ -485,13 +477,13 @@ void *EcaFd(void *process)
   request.var1.name     = FD_NAME;
   request.var1.longname = FD_LONGNAME;
   request.var1.units    = nullptr;
-  request.var1.f1       = farselltc; 
+  request.var1.f1       = vfarselltc; 
   request.var1.f1arg    = TO_KELVIN(0.0);
-  request.var1.f2       = farnum;
+  request.var1.f2       = vfarnum;
   request.var1.f3       = nullptr;
   request.var1.mulc     = 0.0;    
   request.var1.addc     = 0.0;
-  request.var1.epilog   = NONE;    
+  request.var1.epilog   = ECA_NONE;    
   request.var2.h2       = nullptr; 
   request.var2.h3       = nullptr; 
    
@@ -528,18 +520,18 @@ void *EcaGsl(void *process)
 
   char *longname = (char*) Malloc(strlen(GSL_LONGNAME) + 160);
   sprintf(longname, GSL_LONGNAME, argN, argT, argN, argT);
-  
+
   request.name      = GSL_NAME;
   request.longname  = longname;
   request.units     = GSL_UNITS;
   request.name2     = GSL_NAME2;
   request.longname2 = GSL_LONGNAME2;
   request.units2    = GSL_UNITS2;
-  request.s1        = farselgtc; 
+  request.s1        = vfarselgtc; 
   request.s1arg     = TO_KELVIN(argT);
-  request.s2        = farselltc;
+  request.s2        = vfarselltc;
   request.s2arg     = TO_KELVIN(argT);
-  request.s3        = farselgec;
+  request.s3        = vfarselgec;
   request.s3arg     = minLandFraction;
   request.consecutiveDays = argN;    
    
@@ -573,13 +565,13 @@ void *EcaHd(void *process)
   request.var1.name     = HD_NAME;
   request.var1.longname = HD_LONGNAME;
   request.var1.units    = HD_UNITS;
-  request.var1.f1       = farselltc; 
+  request.var1.f1       = vfarselltc; 
   request.var1.f1arg    = TO_KELVIN(argA);
-  request.var1.f2       = farsum;
+  request.var1.f2       = vfarsum;
   request.var1.f3       = nullptr;
   request.var1.mulc     = -1.0;    
   request.var1.addc     = TO_KELVIN(argX);
-  request.var1.epilog   = NONE;    
+  request.var1.epilog   = ECA_NONE;    
   request.var2.h2       = nullptr; 
   request.var2.h3       = nullptr; 
    
@@ -593,7 +585,6 @@ void *EcaHd(void *process)
 
 void *EcaHwdi(void *process)
 {
-  char *longname;
   int argN = 6;
   double argT = 5.0;
   ECA_REQUEST_2 request;
@@ -604,26 +595,26 @@ void *EcaHwdi(void *process)
   if ( operatorArgc() > 0 ) argN = parameter2int(operatorArgv()[0]);
   if ( operatorArgc() > 1 ) argT = parameter2double(operatorArgv()[1]);
   
-  longname = (char*) Malloc(strlen(HWDI_LONGNAME) + 80);
+  char *longname = (char*) Malloc(strlen(HWDI_LONGNAME) + 80);
   sprintf(longname, HWDI_LONGNAME, argN, argT);
   
   request.var1.name     = HWDI_NAME;
   request.var1.longname = longname;
   request.var1.units    = HWDI_UNITS;
   request.var1.f1       = nullptr;
-  request.var1.f2       = farcadd;
+  request.var1.f2       = vfarcadd;
   request.var1.f2arg    = argT;
-  request.var1.f3       = farselgt;
-  request.var1.f4       = farnum2;
-  request.var1.f5       = farnum3;
+  request.var1.f3       = vfarselgt;
+  request.var1.f4       = vfarnum2;
+  request.var1.f5       = vfarnum3;
   request.var1.f5arg    = argN;
-  request.var1.epilog   = NONE;
+  request.var1.epilog   = ECA_NONE;
   request.var2.name     = HWDI_NAME2;
   request.var2.longname = HWDI_LONGNAME2;
   request.var2.units    = HWDI_UNITS2;
-  request.var2.h1       = farseleqc;
+  request.var2.h1       = vfarseleqc;
   request.var2.h1arg    = argN;
-  request.var2.h2       = farnum;
+  request.var2.h2       = vfarnum;
    
   eca2(&request);
   
@@ -636,7 +627,6 @@ void *EcaHwdi(void *process)
 
 void *EcaHwfi(void *process)
 {
-  char *longname;
   int argN = 6;
   ECA_REQUEST_2 request;
   
@@ -645,7 +635,7 @@ void *EcaHwfi(void *process)
 
   if ( operatorArgc() > 0 ) argN = parameter2int(operatorArgv()[0]);
 
-  longname = (char*) Malloc(strlen(HWFI_LONGNAME) + 40);
+  char *longname = (char*) Malloc(strlen(HWFI_LONGNAME) + 40);
   sprintf(longname, HWFI_LONGNAME, argN);
 
   request.var1.name     = HWFI_NAME;
@@ -653,17 +643,17 @@ void *EcaHwfi(void *process)
   request.var1.units    = HWFI_UNITS;
   request.var1.f1       = nullptr;
   request.var1.f2       = nullptr;
-  request.var1.f3       = farselgt;
-  request.var1.f4       = farnum2;
-  request.var1.f5       = farnum3;
+  request.var1.f3       = vfarselgt;
+  request.var1.f4       = vfarnum2;
+  request.var1.f5       = vfarnum3;
   request.var1.f5arg    = argN;
-  request.var1.epilog   = NONE;
+  request.var1.epilog   = ECA_NONE;
   request.var2.name     = HWFI_NAME2;
   request.var2.longname = HWFI_LONGNAME2;
   request.var2.units    = HWFI_UNITS2;
-  request.var2.h1       = farseleqc;
+  request.var2.h1       = vfarseleqc;
   request.var2.h1arg    = argN;
-  request.var2.h2       = farnum;
+  request.var2.h2       = vfarnum;
    
   eca2(&request);
   
@@ -684,13 +674,13 @@ void *EcaId(void *process)
   request.var1.name     = ID_NAME;
   request.var1.longname = ID_LONGNAME;
   request.var1.units    = ID_UNITS;
-  request.var1.f1       = farselltc;
+  request.var1.f1       = vfarselltc;
   request.var1.f1arg    = TO_KELVIN(0.0);
-  request.var1.f2       = farnum;
+  request.var1.f2       = vfarnum;
   request.var1.f3       = nullptr;
   request.var1.mulc     = 0.0;    
   request.var1.addc     = 0.0;   
-  request.var1.epilog   = NONE; 
+  request.var1.epilog   = ECA_NONE; 
   request.var2.h2       = nullptr; 
   request.var2.h3       = nullptr; 
     
@@ -703,7 +693,6 @@ void *EcaId(void *process)
 
 void *EcaSu(void *process)
 {
-  char *longname;
   double argT = 25.0;
   ECA_REQUEST_1 request;
   
@@ -711,19 +700,19 @@ void *EcaSu(void *process)
   cdoOperatorAdd("eca_su", 0, 31, nullptr);
 
   if ( operatorArgc() > 0 ) argT = parameter2double(operatorArgv()[0]);
-  longname = (char*) Malloc(strlen(SU_LONGNAME) + 40);
+  char *longname = (char*) Malloc(strlen(SU_LONGNAME) + 40);
   sprintf(longname, SU_LONGNAME, argT);
 
   request.var1.name     = SU_NAME;
   request.var1.longname = longname;
   request.var1.units    = nullptr;
-  request.var1.f1       = farselgtc;
+  request.var1.f1       = vfarselgtc;
   request.var1.f1arg    = TO_KELVIN(argT);
-  request.var1.f2       = farnum;
+  request.var1.f2       = vfarnum;
   request.var1.f3       = nullptr;
   request.var1.mulc     = 0.0;    
   request.var1.addc     = 0.0;    
-  request.var1.epilog   = NONE; 
+  request.var1.epilog   = ECA_NONE; 
   request.var2.h2       = nullptr; 
   request.var2.h3       = nullptr; 
  
@@ -748,8 +737,8 @@ void *EcaTg10p(void *process)
   request.var1.units    = TG10P_UNITS;
   request.var1.f1       = nullptr;
   request.var1.f2       = nullptr;
-  request.var1.f3       = farsellt;
-  request.var1.f4       = farnum;
+  request.var1.f3       = vfarsellt;
+  request.var1.f4       = vfarnum;
   request.var1.f5       = nullptr;
   request.var1.epilog   = PERCENT_OF_TIME;
   request.var2.h2       = nullptr;
@@ -773,8 +762,8 @@ void *EcaTg90p(void *process)
   request.var1.units    = TG90P_UNITS;
   request.var1.f1       = nullptr;
   request.var1.f2       = nullptr;
-  request.var1.f3       = farselgt;
-  request.var1.f4       = farnum;
+  request.var1.f3       = vfarselgt;
+  request.var1.f4       = vfarnum;
   request.var1.f5       = nullptr;
   request.var1.epilog   = PERCENT_OF_TIME;
   request.var2.h2       = nullptr;
@@ -798,8 +787,8 @@ void *EcaTn10p(void *process)
   request.var1.units    = TN10P_UNITS;
   request.var1.f1       = nullptr;
   request.var1.f2       = nullptr;
-  request.var1.f3       = farsellt;
-  request.var1.f4       = farnum;
+  request.var1.f3       = vfarsellt;
+  request.var1.f4       = vfarnum;
   request.var1.f5       = nullptr;
   request.var1.epilog   = PERCENT_OF_TIME;
   request.var2.h2       = nullptr;
@@ -823,8 +812,8 @@ void *EcaTn90p(void *process)
   request.var1.units    = TN90P_UNITS;
   request.var1.f1       = nullptr;
   request.var1.f2       = nullptr;
-  request.var1.f3       = farselgt;
-  request.var1.f4       = farnum;
+  request.var1.f3       = vfarselgt;
+  request.var1.f4       = vfarnum;
   request.var1.f5       = nullptr;
   request.var1.epilog   = PERCENT_OF_TIME;
   request.var2.h2       = nullptr;
@@ -852,13 +841,13 @@ void *EcaTr(void *process)
   request.var1.name     = TR_NAME;
   request.var1.longname = tr_longname;
   request.var1.units    = TR_UNITS;
-  request.var1.f1       = farselgtc;
+  request.var1.f1       = vfarselgtc;
   request.var1.f1arg    = TO_KELVIN(argT);
-  request.var1.f2       = farnum;
+  request.var1.f2       = vfarnum;
   request.var1.f3       = nullptr;
   request.var1.mulc     = 0.0;    
   request.var1.addc     = 0.0;    
-  request.var1.epilog   = NONE; 
+  request.var1.epilog   = ECA_NONE; 
   request.var2.h2       = nullptr; 
   request.var2.h3       = nullptr; 
    
@@ -882,8 +871,8 @@ void *EcaTx10p(void *process)
   request.var1.units    = TX10P_UNITS;
   request.var1.f1       = nullptr;
   request.var1.f2       = nullptr;
-  request.var1.f3       = farsellt;
-  request.var1.f4       = farnum;
+  request.var1.f3       = vfarsellt;
+  request.var1.f4       = vfarnum;
   request.var1.f5       = nullptr;
   request.var1.epilog   = PERCENT_OF_TIME;
   request.var2.h2       = nullptr;
@@ -907,8 +896,8 @@ void *EcaTx90p(void *process)
   request.var1.units    = TX90P_UNITS;
   request.var1.f1       = nullptr;
   request.var1.f2       = nullptr;
-  request.var1.f3       = farselgt;
-  request.var1.f4       = farnum;
+  request.var1.f3       = vfarselgt;
+  request.var1.f4       = vfarnum;
   request.var1.f5       = nullptr;
   request.var1.epilog   = PERCENT_OF_TIME;
   request.var2.h2       = nullptr;
@@ -949,20 +938,20 @@ void *EcaCdd(void *process)
   request.var1.name     = CDD_NAME;
   request.var1.longname = cdd_longname;
   request.var1.units    = CDD_UNITS;
-  request.var1.f1       = farselltc;
+  request.var1.f1       = vfarselltc;
   request.var1.f1arg    = threshold;
-  request.var1.f2       = farnum2;
-  request.var1.f3       = farmax;
+  request.var1.f2       = vfarnum2;
+  request.var1.f3       = vfarmax;
   request.var1.mulc     = 0.0;
   request.var1.addc     = 0.0;
-  request.var1.epilog   = NONE;
+  request.var1.epilog   = ECA_NONE;
   request.var2.name     = cdd_name2;
   request.var2.longname = cdd_longname2;
   request.var2.units    = CDD_UNITS2;
-  request.var2.h1       = farseleqc;
+  request.var2.h1       = vfarseleqc;
   request.var2.h1arg    = ndays+1;
   request.var2.h2       = nullptr;
-  request.var2.h3       = farnum;
+  request.var2.h3       = vfarnum;
    
   eca1(&request);
   
@@ -998,20 +987,20 @@ void *EcaCwd(void *process)
   request.var1.name     = CWD_NAME;
   request.var1.longname = cwd_longname;
   request.var1.units    = CWD_UNITS;
-  request.var1.f1       = farselgec;
+  request.var1.f1       = vfarselgec;
   request.var1.f1arg    = threshold;
-  request.var1.f2       = farnum2;
-  request.var1.f3       = farmax;
+  request.var1.f2       = vfarnum2;
+  request.var1.f3       = vfarmax;
   request.var1.mulc     = 0.0;
   request.var1.addc     = 0.0;
-  request.var1.epilog   = NONE;
+  request.var1.epilog   = ECA_NONE;
   request.var2.name     = cwd_name2;
   request.var2.longname = cwd_longname2;
   request.var2.units    = CWD_UNITS2;
-  request.var2.h1       = farseleqc;
+  request.var2.h1       = vfarseleqc;
   request.var2.h1arg    = ndays+1;
   request.var2.h2       = nullptr;
-  request.var2.h3       = farnum;
+  request.var2.h3       = vfarnum;
    
   eca1(&request);
   
@@ -1023,8 +1012,6 @@ void *EcaCwd(void *process)
 
 void *EcaPd(void *process)
 {
-  int ECA_PD, ECA_R10MM, ECA_R20MM;
-  int operatorID;
   char lnamebuffer[1024];
   double threshold = 0;
   ECA_REQUEST_1 request;
@@ -1032,12 +1019,12 @@ void *EcaPd(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  ECA_PD      = cdoOperatorAdd("eca_pd",      0, 31, nullptr);
-  ECA_R10MM   = cdoOperatorAdd("eca_r10mm",   0, 31, nullptr);
-  ECA_R20MM   = cdoOperatorAdd("eca_r20mm",   0, 31, nullptr);
+  const int ECA_PD      = cdoOperatorAdd("eca_pd",      0, 31, nullptr);
+  const int ECA_R10MM   = cdoOperatorAdd("eca_r10mm",   0, 31, nullptr);
+  const int ECA_R20MM   = cdoOperatorAdd("eca_r20mm",   0, 31, nullptr);
   // clang-format on
 
-  operatorID = cdoOperatorID();
+  const int operatorID = cdoOperatorID();
 
   if (operatorID == ECA_PD)
     {
@@ -1074,13 +1061,13 @@ void *EcaPd(void *process)
 
   if (Options::cdoVerbose) cdoPrint("threshold = %g", threshold);
 
-  request.var1.f1 = farselgec;
+  request.var1.f1 = vfarselgec;
   request.var1.f1arg = threshold;
-  request.var1.f2 = farnum;
+  request.var1.f2 = vfarnum;
   request.var1.f3 = nullptr;
   request.var1.mulc = 0.0;
   request.var1.addc = 0.0;
-  request.var1.epilog = NONE;
+  request.var1.epilog = ECA_NONE;
   request.var2.h2 = nullptr;
   request.var2.h3 = nullptr;
 
@@ -1101,11 +1088,11 @@ EcaR75p(void *process)
   request.var1.name = R75P_NAME;
   request.var1.longname = R75P_LONGNAME;
   request.var1.units = R75P_UNITS;
-  request.var1.f1 = farselgec;
+  request.var1.f1 = vfarselgec;
   request.var1.f1arg = 1.0;
   request.var1.f2 = nullptr;
-  request.var1.f3 = farselgt;
-  request.var1.f4 = farnum;
+  request.var1.f3 = vfarselgt;
+  request.var1.f4 = vfarnum;
   request.var1.f5 = nullptr;
   request.var1.epilog = PERCENT_OF_TIME;
   request.var2.h2 = nullptr;
@@ -1127,11 +1114,11 @@ EcaR75ptot(void *process)
   request.var1.name = R75PTOT_NAME;
   request.var1.longname = R75PTOT_LONGNAME;
   request.var1.units = R75PTOT_UNITS;
-  request.var1.f1 = farselgec;
+  request.var1.f1 = vfarselgec;
   request.var1.f1arg = 1.0;
   request.var1.f2 = nullptr;
-  request.var1.f3 = farselgt;
-  request.var1.f4 = farsum;
+  request.var1.f3 = vfarselgt;
+  request.var1.f4 = vfarsum;
   request.var1.f5 = nullptr;
   request.var1.epilog = PERCENT_OF_TOTAL_AMOUNT;
   request.var2.h2 = nullptr;
@@ -1153,11 +1140,11 @@ EcaR90p(void *process)
   request.var1.name = R90P_NAME;
   request.var1.longname = R90P_LONGNAME;
   request.var1.units = R90P_UNITS;
-  request.var1.f1 = farselgec;
+  request.var1.f1 = vfarselgec;
   request.var1.f1arg = 1.0;
   request.var1.f2 = nullptr;
-  request.var1.f3 = farselgt;
-  request.var1.f4 = farnum;
+  request.var1.f3 = vfarselgt;
+  request.var1.f4 = vfarnum;
   request.var1.f5 = nullptr;
   request.var1.epilog = PERCENT_OF_TIME;
   request.var2.h2 = nullptr;
@@ -1179,11 +1166,11 @@ EcaR90ptot(void *process)
   request.var1.name = R90PTOT_NAME;
   request.var1.longname = R90PTOT_LONGNAME;
   request.var1.units = R90PTOT_UNITS;
-  request.var1.f1 = farselgec;
+  request.var1.f1 = vfarselgec;
   request.var1.f1arg = 1.0;
   request.var1.f2 = nullptr;
-  request.var1.f3 = farselgt;
-  request.var1.f4 = farsum;
+  request.var1.f3 = vfarselgt;
+  request.var1.f4 = vfarsum;
   request.var1.f5 = nullptr;
   request.var1.epilog = PERCENT_OF_TOTAL_AMOUNT;
   request.var2.h2 = nullptr;
@@ -1205,11 +1192,11 @@ EcaR95p(void *process)
   request.var1.name = R95P_NAME;
   request.var1.longname = R95P_LONGNAME;
   request.var1.units = R95P_UNITS;
-  request.var1.f1 = farselgec;
+  request.var1.f1 = vfarselgec;
   request.var1.f1arg = 1.0;
   request.var1.f2 = nullptr;
-  request.var1.f3 = farselgt;
-  request.var1.f4 = farnum;
+  request.var1.f3 = vfarselgt;
+  request.var1.f4 = vfarnum;
   request.var1.f5 = nullptr;
   request.var1.epilog = PERCENT_OF_TIME;
   request.var2.h2 = nullptr;
@@ -1231,11 +1218,11 @@ EcaR95ptot(void *process)
   request.var1.name = R95PTOT_NAME;
   request.var1.longname = R95PTOT_LONGNAME;
   request.var1.units = R95PTOT_UNITS;
-  request.var1.f1 = farselgec;
+  request.var1.f1 = vfarselgec;
   request.var1.f1arg = 1.0;
   request.var1.f2 = nullptr;
-  request.var1.f3 = farselgt;
-  request.var1.f4 = farsum;
+  request.var1.f3 = vfarselgt;
+  request.var1.f4 = vfarsum;
   request.var1.f5 = nullptr;
   request.var1.epilog = PERCENT_OF_TOTAL_AMOUNT;
   request.var2.h2 = nullptr;
@@ -1257,11 +1244,11 @@ EcaR99p(void *process)
   request.var1.name = R99P_NAME;
   request.var1.longname = R99P_LONGNAME;
   request.var1.units = R99P_UNITS;
-  request.var1.f1 = farselgec;
+  request.var1.f1 = vfarselgec;
   request.var1.f1arg = 1.0;
   request.var1.f2 = nullptr;
-  request.var1.f3 = farselgt;
-  request.var1.f4 = farnum;
+  request.var1.f3 = vfarselgt;
+  request.var1.f4 = vfarnum;
   request.var1.f5 = nullptr;
   request.var1.epilog = PERCENT_OF_TIME;
   request.var2.h2 = nullptr;
@@ -1283,11 +1270,11 @@ EcaR99ptot(void *process)
   request.var1.name = R99PTOT_NAME;
   request.var1.longname = R99PTOT_LONGNAME;
   request.var1.units = nullptr;
-  request.var1.f1 = farselgec;
+  request.var1.f1 = vfarselgec;
   request.var1.f1arg = 1.0;
   request.var1.f2 = nullptr;
-  request.var1.f3 = farselgt;
-  request.var1.f4 = farsum;
+  request.var1.f3 = vfarselgt;
+  request.var1.f4 = vfarsum;
   request.var1.f5 = nullptr;
   request.var1.epilog = PERCENT_OF_TOTAL_AMOUNT;
   request.var2.h2 = nullptr;
@@ -1316,13 +1303,13 @@ EcaRr1(void *process)
   request.var1.name = RR1_NAME;
   request.var1.longname = lnamebuffer;
   request.var1.units = RR1_UNITS;
-  request.var1.f1 = farselgec;
+  request.var1.f1 = vfarselgec;
   request.var1.f1arg = threshold;
-  request.var1.f2 = farnum;
+  request.var1.f2 = vfarnum;
   request.var1.f3 = nullptr;
   request.var1.mulc = 0.0;
   request.var1.addc = 0.0;
-  request.var1.epilog = NONE;
+  request.var1.epilog = ECA_NONE;
   request.var2.h2 = nullptr;
   request.var2.h3 = nullptr;
 
@@ -1354,11 +1341,11 @@ EcaRx1day(void *process)
   request.var1.longname = RX1DAY_LONGNAME;
   request.var1.units = RX1DAY_UNITS;
   request.var1.f1 = nullptr;
-  request.var1.f2 = farmax;
+  request.var1.f2 = vfarmax;
   request.var1.f3 = nullptr;
   request.var1.mulc = 0.0;
   request.var1.addc = 0.0;
-  request.var1.epilog = NONE;
+  request.var1.epilog = ECA_NONE;
   request.var2.h2 = nullptr;
   request.var2.h3 = nullptr;
 
@@ -1371,14 +1358,13 @@ EcaRx1day(void *process)
 void *
 EcaRx5day(void *process)
 {
-  char *longname;
   double argX = 50.0;
   ECA_REQUEST_1 request;
 
   cdoInitialize(process);
   if (operatorArgc() > 0) argX = parameter2double(operatorArgv()[0]);
 
-  longname = (char *) Malloc(strlen(RX5DAY_LONGNAME2) + 40);
+  char *longname = (char *) Malloc(strlen(RX5DAY_LONGNAME2) + 40);
   sprintf(longname, RX5DAY_LONGNAME2, argX);
 
   cdoOperatorAdd("eca_rx5day", 0, 31, nullptr);
@@ -1387,17 +1373,17 @@ EcaRx5day(void *process)
   request.var1.longname = RX5DAY_LONGNAME;
   request.var1.units = RX5DAY_UNITS;
   request.var1.f1 = nullptr;
-  request.var1.f2 = farmax;
+  request.var1.f2 = vfarmax;
   request.var1.f3 = nullptr;
   request.var1.mulc = 0.0;
   request.var1.addc = 0.0;
-  request.var1.epilog = NONE;
+  request.var1.epilog = ECA_NONE;
   request.var2.name = RX5DAY_NAME2;
   request.var2.longname = longname;
   request.var2.units = RX5DAY_UNITS2;
-  request.var2.h1 = farselgec;
+  request.var2.h1 = vfarselgec;
   request.var2.h1arg = argX;
-  request.var2.h2 = farnum;
+  request.var2.h2 = vfarnum;
   request.var2.h3 = nullptr;
 
   eca1(&request);
@@ -1426,9 +1412,9 @@ EcaSdii(void *process)
   request.var1.name = SDII_NAME;
   request.var1.longname = lnamebuffer;
   request.var1.units = SDII_UNITS;
-  request.var1.f1 = farselgec;
+  request.var1.f1 = vfarselgec;
   request.var1.f1arg = threshold;
-  request.var1.f2 = farsum;
+  request.var1.f2 = vfarsum;
   request.var1.f3 = nullptr;
   request.var1.mulc = 0.0;
   request.var1.addc = 0.0;
@@ -1453,15 +1439,15 @@ Fdns(void *process)
   request.var1.name = FDNS_NAME;
   request.var1.longname = FDNS_LONGNAME;
   request.var1.units = FDNS_UNITS;
-  request.var1.f1 = farsellec;
+  request.var1.f1 = vfarsellec;
   request.var1.f1arg = TO_KELVIN(0.0);
-  request.var1.f2 = farsellec;
+  request.var1.f2 = vfarsellec;
   request.var1.f2arg = 0.01;
-  request.var1.f3 = faradd; /* any f with f(a, b) = miss, if a = miss or b =
+  request.var1.f3 = vfaradd; /* any f with f(a, b) = miss, if a = miss or b =
                                miss will do here */
-  request.var1.f4 = farnum;
+  request.var1.f4 = vfarnum;
   request.var1.f5 = nullptr;
-  request.var1.epilog = NONE;
+  request.var1.epilog = ECA_NONE;
   request.var2.h2 = nullptr;
 
   eca2(&request);
@@ -1473,7 +1459,6 @@ Fdns(void *process)
 void *
 Strwin(void *process)
 {
-  char *longname;
   double maxWind = 10.5;
   ECA_REQUEST_1 request;
 
@@ -1482,26 +1467,26 @@ Strwin(void *process)
 
   if (operatorArgc() > 0) maxWind = parameter2double(operatorArgv()[0]);
 
-  longname = (char *) Malloc(strlen(STRWIN_LONGNAME) + 40);
+  char *longname = (char *) Malloc(strlen(STRWIN_LONGNAME) + 40);
   sprintf(longname, STRWIN_LONGNAME, maxWind);
 
   request.var1.name = STRWIN_NAME;
   request.var1.longname = longname;
   request.var1.units = STRWIN_UNITS;
-  request.var1.f1 = farselgec;
+  request.var1.f1 = vfarselgec;
   request.var1.f1arg = maxWind;
-  request.var1.f2 = farnum;
+  request.var1.f2 = vfarnum;
   request.var1.f3 = nullptr;
   request.var1.mulc = 0.0;
   request.var1.addc = 0.0;
-  request.var1.epilog = NONE;
+  request.var1.epilog = ECA_NONE;
   request.var2.name = STRWIN_NAME2;
   request.var2.longname = STRWIN_LONGNAME2;
   request.var2.units = STRWIN_UNITS2;
-  request.var2.h1 = farselgec;
+  request.var2.h1 = vfarselgec;
   request.var2.h1arg = maxWind;
-  request.var2.h2 = farnum2;
-  request.var2.h3 = farmax;
+  request.var2.h2 = vfarnum2;
+  request.var2.h3 = vfarmax;
 
   eca1(&request);
 
@@ -1523,20 +1508,20 @@ Strbre(void *process)
   request.var1.name = STRBRE_NAME;
   request.var1.longname = STRBRE_LONGNAME;
   request.var1.units = STRWIN_UNITS;
-  request.var1.f1 = farselgec;
+  request.var1.f1 = vfarselgec;
   request.var1.f1arg = maxWind;
-  request.var1.f2 = farnum;
+  request.var1.f2 = vfarnum;
   request.var1.f3 = nullptr;
   request.var1.mulc = 0.0;
   request.var1.addc = 0.0;
-  request.var1.epilog = NONE;
+  request.var1.epilog = ECA_NONE;
   request.var2.name = STRBRE_NAME2;
   request.var2.longname = STRBRE_LONGNAME2;
   request.var2.units = STRWIN_UNITS2;
-  request.var2.h1 = farselgec;
+  request.var2.h1 = vfarselgec;
   request.var2.h1arg = maxWind;
-  request.var2.h2 = farnum2;
-  request.var2.h3 = farmax;
+  request.var2.h2 = vfarnum2;
+  request.var2.h3 = vfarmax;
 
   eca1(&request);
   cdoFinish();
@@ -1556,20 +1541,20 @@ Strgal(void *process)
   request.var1.name = STRBRE_NAME;
   request.var1.longname = STRBRE_LONGNAME;
   request.var1.units = STRWIN_UNITS;
-  request.var1.f1 = farselgec;
+  request.var1.f1 = vfarselgec;
   request.var1.f1arg = maxWind;
-  request.var1.f2 = farnum;
+  request.var1.f2 = vfarnum;
   request.var1.f3 = nullptr;
   request.var1.mulc = 0.0;
   request.var1.addc = 0.0;
-  request.var1.epilog = NONE;
+  request.var1.epilog = ECA_NONE;
   request.var2.name = STRBRE_NAME2;
   request.var2.longname = STRBRE_LONGNAME2;
   request.var2.units = STRWIN_UNITS2;
-  request.var2.h1 = farselgec;
+  request.var2.h1 = vfarselgec;
   request.var2.h1arg = maxWind;
-  request.var2.h2 = farnum2;
-  request.var2.h3 = farmax;
+  request.var2.h2 = vfarnum2;
+  request.var2.h3 = vfarmax;
 
   eca1(&request);
   cdoFinish();
@@ -1589,20 +1574,20 @@ Hurr(void *process)
   request.var1.name = HURR_NAME;
   request.var1.longname = HURR_LONGNAME;
   request.var1.units = STRWIN_UNITS;
-  request.var1.f1 = farselgec;
+  request.var1.f1 = vfarselgec;
   request.var1.f1arg = maxWind;
-  request.var1.f2 = farnum;
+  request.var1.f2 = vfarnum;
   request.var1.f3 = nullptr;
   request.var1.mulc = 0.0;
   request.var1.addc = 0.0;
-  request.var1.epilog = NONE;
+  request.var1.epilog = ECA_NONE;
   request.var2.name = HURR_NAME2;
   request.var2.longname = HURR_LONGNAME2;
   request.var2.units = STRWIN_UNITS2;
-  request.var2.h1 = farselgec;
+  request.var2.h1 = vfarselgec;
   request.var2.h1arg = maxWind;
-  request.var2.h2 = farnum2;
-  request.var2.h3 = farmax;
+  request.var2.h2 = vfarnum2;
+  request.var2.h3 = vfarmax;
 
   eca1(&request);
   cdoFinish();
