@@ -22,9 +22,7 @@
 #include <time.h>
 
 #include <cdi.h>
-
 #include "cdo_int.h"
-#include "pstream_int.h"
 #include "commandline.h"
 
 #ifdef HAVE_LIBNETCDF
@@ -1493,7 +1491,7 @@ export_e5res(const char *filename, VAR *vars, int nvars)
 void *
 Echam5ini(void *process)
 {
-  int streamID1, streamID2 = CDI_UNDEFID;
+  CdoStreamID streamID1, streamID2 = CDO_STREAM_UNDEF;
   int nrecs = 0;
   int varID, levelID;
   int vlistID1, vlistID2;
@@ -1562,12 +1560,12 @@ Echam5ini(void *process)
 
       if (CdoDefault::FileType == CDI_UNDEFID) CdoDefault::FileType = CDI_FILETYPE_NC;
 
-      streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
+      streamID2 = cdoOpenWrite(1);
 
-      pstreamDefVlist(streamID2, vlistID2);
+      cdoDefVlist(streamID2, vlistID2);
 
       tsID = 0;
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (varID = 0; varID < nvars; varID++)
         {
@@ -1576,8 +1574,8 @@ Echam5ini(void *process)
 
           for (levelID = 0; levelID < nlev; levelID++)
             {
-              pstreamDefRecord(streamID2, varID, levelID);
-              pstreamWriteRecord(streamID2, vars[varID].ptr + levelID * gridsize, 0);
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, vars[varID].ptr + levelID * gridsize, 0);
             }
         }
 
@@ -1594,7 +1592,7 @@ Echam5ini(void *process)
       int taxisID, vtime;
       int ntr = 0;
 
-      streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+      streamID1 = cdoOpenRead(0);
 
       vlistID1 = cdoStreamInqVlist(streamID1);
       taxisID = vlistInqTaxis(vlistID1);
@@ -1687,12 +1685,12 @@ Echam5ini(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           gridID = vlistInqVarGrid(vlistID1, varID);
           gridsize = gridInqSize(gridID);
 
-          pstreamReadRecord(streamID1, vars[varID].ptr + levelID * gridsize, &nmiss);
+          cdoReadRecord(streamID1, vars[varID].ptr + levelID * gridsize, &nmiss);
         }
 
       cdoStreamClose(streamID1);
