@@ -24,7 +24,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 #include "griddes.h"
 
 void *
@@ -37,33 +36,30 @@ Enlarge(void *process)
 
   operatorCheckArgc(1);
 
-  int gridID2 = cdoDefineGrid(operatorArgv()[0]);
-  size_t xsize2 = gridInqXsize(gridID2);
-  size_t ysize2 = gridInqYsize(gridID2);
+  const int gridID2 = cdoDefineGrid(operatorArgv()[0]);
+  const size_t xsize2 = gridInqXsize(gridID2);
+  const size_t ysize2 = gridInqYsize(gridID2);
 
   if (Options::cdoVerbose) fprintf(stderr, "gridID2 %d, xsize2 %zu, ysize2 %zu\n", gridID2, xsize2, ysize2);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  size_t gridsize2 = gridInqSize(gridID2);
+  const size_t gridsize2 = gridInqSize(gridID2);
   if (gridsize2 < vlistGridsizeMax(vlistID1)) cdoAbort("Gridsize of input stream is greater than new gridsize!");
 
-  int ngrids = vlistNgrids(vlistID1);
-  for (int index = 0; index < ngrids; index++)
-    {
-      vlistChangeGridIndex(vlistID2, index, gridID2);
-    }
+  const int ngrids = vlistNgrids(vlistID1);
+  for (int index = 0; index < ngrids; index++) vlistChangeGridIndex(vlistID2, index, gridID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
+  CdoStreamID streamID2 = cdoOpenWrite(1);
 
-  pstreamDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID2, vlistID2);
 
   std::vector<double> array1(gridsize2);
   std::vector<double> array2(gridsize2);
@@ -72,25 +68,24 @@ Enlarge(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
           int varID, levelID;
           size_t nmiss;
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, array1.data(), &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, array1.data(), &nmiss);
+
+          const double missval = vlistInqVarMissval(vlistID1, varID);
+          const int gridID1 = vlistInqVarGrid(vlistID1, varID);
+          const size_t gridsize1 = gridInqSize(gridID1);
 
-          double missval = vlistInqVarMissval(vlistID1, varID);
-          int gridID1 = vlistInqVarGrid(vlistID1, varID);
           size_t xsize1 = gridInqXsize(gridID1);
           size_t ysize1 = gridInqYsize(gridID1);
-          size_t gridsize1 = gridInqSize(gridID1);
-
           if (xsize1 == 0) xsize1 = 1;
           if (ysize1 == 0) ysize1 = 1;
 
-          /* printf("%d %d %d %d\n", xsize1, ysize1, xsize2, ysize2); */
           if (xsize1 == 1 && ysize1 == ysize2 && xsize1 * ysize1 == gridsize1)
             {
               if (linfo)
@@ -125,8 +120,8 @@ Enlarge(void *process)
               if (nmiss && DBL_IS_EQUAL(array1[gridsize1 - 1], missval)) nmiss += (gridsize2 - gridsize1);
             }
 
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, array2.data(), nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, array2.data(), nmiss);
         }
 
       tsID++;
