@@ -23,8 +23,8 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
-#include "grid.h"
+
+#include <mpim_grid.h>
 #include "griddes.h"
 
 void
@@ -150,7 +150,7 @@ Enlargegrid(void *process)
 
   int gridID2 = cdoDefineGrid(operatorArgv()[0]);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   int vlistID1 = cdoStreamInqVlist(streamID1);
   int taxisID1 = vlistInqTaxis(vlistID1);
@@ -181,23 +181,23 @@ Enlargegrid(void *process)
       vlistChangeGridIndex(vlistID2, index, gridID2);
     }
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
+  CdoStreamID streamID2 = cdoOpenWrite(1);
 
   vlistDefTaxis(vlistID2, taxisID2);
-  pstreamDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
           int varID, levelID;
           size_t nmiss;
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, array1.data(), &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, array1.data(), &nmiss);
 
           double missval1 = vlistInqVarMissval(vlistID1, varID);
 
@@ -206,8 +206,8 @@ Enlargegrid(void *process)
             if (gindex[i] >= 0) array2[gindex[i]] = array1[i];
 
           nmiss = arrayNumMV(gridsize2, array2.data(), missval1);
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, array2.data(), nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, array2.data(), nmiss);
         }
 
       tsID++;
