@@ -36,7 +36,7 @@
 #include "cdo_int.h"
 #include "param_conversion.h"
 #include "cdo_task.h"
-#include "pstream_int.h"
+
 #include "percentiles.h"
 #include "cdo_options.h"
 #include "util_files.h"
@@ -44,7 +44,7 @@
 
 struct ens_file_t
 {
-  int streamID;
+  CdoStreamID streamID;
   int vlistID;
   size_t nmiss[2];
   double missval[2];
@@ -57,7 +57,7 @@ struct ensstat_arg_t
   int varID[2];
   int levelID[2];
   int vlistID1;
-  int streamID2;
+  CdoStreamID streamID2;
   int nfiles;
   ens_file_t *efData;
   double *array2Data;
@@ -104,42 +104,36 @@ ensstat_func(void *ensarg)
       fields[ompthID].nmiss = 0;
       for (int fileID = 0; fileID < nfiles; fileID++)
         {
-          fields[ompthID].ptr[fileID] = ef[fileID].array[t][i];
-          if (lmiss && DBL_IS_EQUAL(fields[ompthID].ptr[fileID], ef[fileID].missval[t]))
+          fields[ompthID].vec[fileID] = ef[fileID].array[t][i];
+          if (lmiss && DBL_IS_EQUAL(fields[ompthID].vec[fileID], ef[fileID].missval[t]))
             {
-              fields[ompthID].ptr[fileID] = missval;
+              fields[ompthID].vec[fileID] = missval;
               fields[ompthID].nmiss++;
             }
         }
 
-      array2[i] = arg->lpctl ? fldpctl(fields[ompthID], arg->pn) : fldfun(fields[ompthID], arg->operfunc);
+      array2[i] = arg->lpctl ? vfldpctl(fields[ompthID], arg->pn) : vfldfun(fields[ompthID], arg->operfunc);
 
       if (DBL_IS_EQUAL(array2[i], fields[ompthID].missval)) nmiss++;
 
       if (withCountData) count2[i] = nfiles - fields[ompthID].nmiss;
     }
 
-  pstreamDefRecord(arg->streamID2, arg->varID[t], arg->levelID[t]);
-  pstreamWriteRecord(arg->streamID2, array2, nmiss);
+  cdoDefRecord(arg->streamID2, arg->varID[t], arg->levelID[t]);
+  cdoWriteRecord(arg->streamID2, array2, nmiss);
 
   if (withCountData)
     {
-      pstreamDefRecord(arg->streamID2, arg->varID[t] + arg->nvars, arg->levelID[t]);
-      pstreamWriteRecord(arg->streamID2, count2, 0);
+      cdoDefRecord(arg->streamID2, arg->varID[t] + arg->nvars, arg->levelID[t]);
+      cdoWriteRecord(arg->streamID2, count2, 0);
     }
 
-  return 0;
+  return nullptr;
 }
 
-void *
-Ensstat(void *process)
+static void
+addOperators(void)
 {
-  void *task = CDO_task ? cdo_task_new() : nullptr;
-  ensstat_arg_t ensstat_arg;
-  int nrecs0;
-
-  cdoInitialize(process);
-
   // clang-format off
   cdoOperatorAdd("ensrange", func_range, 0, nullptr);
   cdoOperatorAdd("ensmin",   func_min,   0, nullptr);
@@ -155,6 +149,18 @@ Ensstat(void *process)
   cdoOperatorAdd("ensskew",  func_skew,  0, nullptr);
   cdoOperatorAdd("enskurt",  func_kurt,  0, nullptr);
   // clang-format on
+}
+
+void *
+Ensstat(void *process)
+{
+  void *task = CDO_task ? cdo_task_new() : nullptr;
+  ensstat_arg_t ensstat_arg;
+  int nrecs0;
+
+  cdoInitialize(process);
+
+  addOperators();
 
   const int operatorID = cdoOperatorID();
   const int operfunc = cdoOperatorF1(operatorID);
@@ -193,17 +199,12 @@ Ensstat(void *process)
 
   std::vector<ens_file_t> ef(nfiles);
 
-  std::vector<Field> fields(Threading::ompNumThreads);
-  for (int i = 0; i < Threading::ompNumThreads; i++)
-    {
-      fieldInit(fields[i]);
-      fields[i].size = nfiles;
-      fields[i].ptr = (double *) Malloc(nfiles * sizeof(double));
-    }
+  FieldVector fields(Threading::ompNumThreads);
+  for (int i = 0; i < Threading::ompNumThreads; i++) fields[i].resize(nfiles);
 
   for (int fileID = 0; fileID < nfiles; fileID++)
     {
-      ef[fileID].streamID = cdoStreamOpenRead(cdoStreamName(fileID));
+      ef[fileID].streamID = cdoOpenRead((fileID));
       ef[fileID].vlistID = cdoStreamInqVlist(ef[fileID].streamID);
     }
 
@@ -246,8 +247,8 @@ Ensstat(void *process)
         }
     }
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(nfiles));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite((nfiles));
+  cdoDefVlist(streamID2, vlistID2);
 
   ensstat_arg.vlistID1 = vlistID1;
   ensstat_arg.streamID2 = streamID2;
@@ -271,7 +272,7 @@ Ensstat(void *process)
       nrecs0 = cdoStreamInqTimestep(ef[0].streamID, tsID);
       for (int fileID = 1; fileID < nfiles; fileID++)
         {
-          const int streamID = ef[fileID].streamID;
+          CdoStreamID streamID = ef[fileID].streamID;
           const int nrecs = cdoStreamInqTimestep(streamID, tsID);
           if (nrecs != nrecs0)
             {
@@ -298,7 +299,7 @@ Ensstat(void *process)
       if (nrecs0 > 0)
         {
           taxisCopyTimestep(taxisID2, taxisID1);
-          pstreamDefTimestep(streamID2, tsID);
+          cdoDefTimestep(streamID2, tsID);
         }
 
       for (int recID = 0; recID < nrecs0; recID++)
@@ -307,13 +308,13 @@ Ensstat(void *process)
 
           for (int fileID = 0; fileID < nfiles; fileID++)
             {
-              pstreamInqRecord(ef[fileID].streamID, &varID, &levelID);
+              cdoInqRecord(ef[fileID].streamID, &varID, &levelID);
               ef[fileID].missval[t] = vlistInqVarMissval(ef[fileID].vlistID, varID);
             }
           //#pragma omp parallel for default(none) shared(ef, t)
           for (int fileID = 0; fileID < nfiles; ++fileID)
             {
-              pstreamReadRecord(ef[fileID].streamID, ef[fileID].array[t], &ef[fileID].nmiss[t]);
+              cdoReadRecord(ef[fileID].streamID, ef[fileID].array[t], &ef[fileID].nmiss[t]);
             }
 
           ensstat_arg.efData = ef.data();
@@ -323,7 +324,7 @@ Ensstat(void *process)
             {
               cdo_task_start(task, ensstat_func, &ensstat_arg);
               cdo_task_wait(task);
-              //t = !t;
+              // t = !t;
             }
           else
             {
@@ -350,9 +351,6 @@ CLEANUP:
       if (ef[fileID].array[1]) Free(ef[fileID].array[1]);
     }
 
-  for (int i = 0; i < Threading::ompNumThreads; i++)
-    if (fields[i].ptr) Free(fields[i].ptr);
-
   if (task) cdo_task_delete(task);
 
   cdoFinish();
