@@ -17,8 +17,8 @@
 
 /*
    This module contains the following operators:
-   Ensstat3       ensrkhist_space  Ensemble ranked histogram averaged over time
-   Ensstat3       ensrkhist_time   Ensemble ranked histogram averaged over space
+   Ensstat3       ensrkhistspace   Ensemble ranked histogram averaged over time
+   Ensstat3       ensrkhisttime    Ensemble ranked histogram averaged over space
    Ensstat3       ensroccurve      Ensamble Receiver Operating Characteristics
 */
 
@@ -26,17 +26,18 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
 #include "cdo_options.h"
 #include "util_files.h"
 #include "cimdOmp.h"
 
+
 // Defines for rank histogram
 enum TDATA_TYPE
 {
   TIME,
   SPACE
 };
+
 #define time_data TIME
 #define space_data SPACE
 
@@ -63,7 +64,28 @@ enum ROC_ENUM_TYPE
 /* TPR = True Positive Rate = TP / ( TP + FN ) */
 /* FNR = False Negtive Rate = FN / ( FP + TN ) */
 
-double roc_curve_integrate(const double **roc, const int n);
+static double
+roc_curve_integrate(const double **roc, const int n)
+{
+  double y1, y0, x1, x0, dx, dy;
+  double step_area;
+  double area = 0;
+
+  for (int i = 1; i <= n; ++i)
+    {
+      x1 = roc[i][FPR];
+      x0 = roc[i - 1][FPR];
+      y1 = roc[i][TPR];
+      y0 = roc[i - 1][TPR];
+      dx = x1 - x0;
+      dy = y1 - y0;
+
+      step_area = -0.5 * dx * dy - dx * y0;
+      area += step_area;
+    }
+
+  return area - 0.5;
+}
 
 void *
 Ensstat3(void *process)
@@ -77,19 +99,19 @@ Ensstat3(void *process)
   int vlistID;
   int gridID, gridID2;
   int have_miss = 0;
-  int streamID = 0, streamID2 = 0;
+  CdoStreamID streamID = 0, streamID2 = 0;
   int **array2 = nullptr;
   int **ctg_tab = nullptr, *hist = nullptr;  // contingency table and histogram
   double missval;
-  double *dat;                              // pointer to ensemble data for ROC
+  double *dat;                                    // pointer to ensemble data for ROC
   double *uThresh = nullptr, *lThresh = nullptr;  // thresholds for histograms
-  double **roc = nullptr;                      // receiver operating characteristics table
+  double **roc = nullptr;                         // receiver operating characteristics table
   double val;
   int ival;
 
   struct ens_file_t
   {
-    int streamID;
+    CdoStreamID streamID;
     int vlistID;
     double *array;
   };
@@ -97,14 +119,14 @@ Ensstat3(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  cdoOperatorAdd("ensroc",          func_roc,  0,          nullptr);
-  cdoOperatorAdd("ensrkhist_space", func_rank, space_data, nullptr);
-  cdoOperatorAdd("ensrkhist_time",  func_rank, time_data,  nullptr);
+  cdoOperatorAdd("ensroc",         func_roc,  0,          nullptr);
+  cdoOperatorAdd("ensrkhistspace", func_rank, space_data, nullptr);
+  cdoOperatorAdd("ensrkhisttime",  func_rank, time_data,  nullptr);
   // clang-format on
 
-  int operatorID = cdoOperatorID();
-  int operfunc = cdoOperatorF1(operatorID);
-  int datafunc = cdoOperatorF2(operatorID);
+  const int operatorID = cdoOperatorID();
+  const int operfunc = cdoOperatorF1(operatorID);
+  const int datafunc = cdoOperatorF2(operatorID);
 
   int nbins = 0;
   if (operfunc == func_roc)
@@ -114,7 +136,7 @@ Ensstat3(void *process)
     }
 
   int nfiles = cdoStreamCnt() - 1;
-  int nens = nfiles - 1;
+  const int nens = nfiles - 1;
 
   if (Options::cdoVerbose) cdoPrint("Ensemble over %d files.", nfiles);
 
@@ -130,22 +152,18 @@ Ensstat3(void *process)
   /* ("first touch strategy")                            */
   /* --> #pragma omp parallel for ...                    */
   /* *************************************************** */
-  Field *field = (Field *) Malloc(Threading::ompNumThreads * sizeof(Field));
+  FieldVector field(Threading::ompNumThreads);
   for (i = 0; i < Threading::ompNumThreads; i++)
     {
-      fieldInit(field[i]);
-      field[i].size = nfiles;
-      field[i].ptr = (double *) Malloc(nfiles * sizeof(double));
-      field[i].weight = (double *) Malloc(nfiles * sizeof(double));
-      for (int fileID = 0; fileID < nfiles; fileID++) field[i].weight[fileID] = 1;
+      field[i].resize(nfiles);
+      field[i].weightv.resize(nfiles);
+      for (int fileID = 0; fileID < nfiles; fileID++) field[i].weightv[fileID] = 1;
     }
 
   for (int fileID = 0; fileID < nfiles; fileID++)
     {
-      streamID = cdoStreamOpenRead(cdoStreamName(fileID));
-
+      streamID = cdoOpenRead((fileID));
       vlistID = cdoStreamInqVlist(streamID);
-
       ef[fileID].streamID = streamID;
       ef[fileID].vlistID = vlistID;
     }
@@ -153,13 +171,13 @@ Ensstat3(void *process)
   /* check for identical contents of all ensemble members */
   for (int fileID = 1; fileID < nfiles; fileID++) vlistCompare(ef[0].vlistID, ef[fileID].vlistID, CMP_ALL);
 
-  int vlistID1 = ef[0].vlistID;
-  int vlistID2 = vlistCreate();
-  int nvars = vlistNvars(vlistID1);
+  const int vlistID1 = ef[0].vlistID;
+  const int vlistID2 = vlistCreate();
+  const int nvars = vlistNvars(vlistID1);
   int *varID2 = (int *) Malloc(nvars * sizeof(int));
 
   double *levs = (double *) Calloc(nfiles, sizeof(double));
-  int zaxisID2 = zaxisCreate(ZAXIS_GENERIC, nfiles);
+  const int zaxisID2 = zaxisCreate(ZAXIS_GENERIC, nfiles);
   for (i = 0; i < nfiles; i++) levs[i] = i;
   zaxisDefLevels(zaxisID2, levs);
   zaxisDefName(zaxisID2, "histogram_binID");
@@ -168,12 +186,10 @@ Ensstat3(void *process)
 
   for (varID = 0; varID < nvars; varID++)
     {
-
       /* **************************************************************** */
       /* nfiles includes the observation, so there are nfiles-1 ensembles */
       /* and exactly nfiles bins, in which the observation could fall     */
       /* **************************************************************** */
-
       if (datafunc == TIME)
         {
           val = 0;
@@ -189,8 +205,8 @@ Ensstat3(void *process)
       varID2[varID] = vlistDefVar(vlistID2, gridID2, zaxisID2, time_mode);
     }
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
   for (varID = 0; varID < nvars; varID++)
@@ -206,8 +222,8 @@ Ensstat3(void *process)
 
   if (operfunc != func_roc)
     {
-      streamID2 = cdoStreamOpenWrite(cdoStreamName(nfiles));
-      pstreamDefVlist(streamID2, vlistID2);
+      streamID2 = cdoOpenWrite((nfiles));
+      cdoDefVlist(streamID2, vlistID2);
     }
 
   size_t gridsize = vlistGridsizeMax(vlistID1);
@@ -215,13 +231,12 @@ Ensstat3(void *process)
   for (int fileID = 0; fileID < nfiles; fileID++) ef[fileID].array = (double *) Malloc(gridsize * sizeof(double));
 
   if (operfunc == func_rank && datafunc == SPACE)
-    { /*need to memorize data for entire grid before writing          */
+    { // need to memorize data for entire grid before writing
       array2 = (int **) Malloc((nfiles + 1) * sizeof(int *));
       for (binID = 0; binID < nfiles; binID++) array2[binID] = (int *) Calloc(gridsize, sizeof(int));
     }
   else if (operfunc == func_rank)
-    { /* can process data separately for each timestep and only need */
-      /* to cumulate values over the grid                            */
+    { // can process data separately for each timestep and only need to cumulate values over the grid
       array2 = (int **) Malloc((nfiles + 1) * sizeof(int *));
       for (binID = 0; binID < nfiles; binID++) array2[binID] = (int *) Calloc(1, sizeof(int));
     }
@@ -266,7 +281,7 @@ Ensstat3(void *process)
       if (operfunc == func_rank && (datafunc == TIME || tsID == 0))
         {
           taxisCopyTimestep(taxisID2, taxisID1);
-          if (nrecs0 > 0) pstreamDefTimestep(streamID2, tsID);
+          if (nrecs0 > 0) cdoDefTimestep(streamID2, tsID);
         }
 
       //      fprintf(stderr,"TIMESTEP %i varID %i rec %i\n",tsID,varID,recID);
@@ -279,8 +294,8 @@ Ensstat3(void *process)
           for (int fileID = 0; fileID < nfiles; fileID++)
             {
               streamID = ef[fileID].streamID;
-              pstreamInqRecord(streamID, &varID, &levelID);
-              pstreamReadRecord(streamID, ef[fileID].array, &nmiss);
+              cdoInqRecord(streamID, &varID, &levelID);
+              cdoReadRecord(streamID, ef[fileID].array, &nmiss);
             }
 
           gridID = vlistInqVarGrid(vlistID1, varID);
@@ -296,15 +311,15 @@ Ensstat3(void *process)
 #endif
           for (size_t i = 0; i < gridsize; i++)
             {
-              int ompthID = cdo_omp_get_thread_num();
+              const int ompthID = cdo_omp_get_thread_num();
 
               field[ompthID].missval = missval;
               field[ompthID].nmiss = 0;
               have_miss = 0;
               for (int fileID = 0; fileID < nfiles; fileID++)
                 {
-                  field[ompthID].ptr[fileID] = ef[fileID].array[i];
-                  if (DBL_IS_EQUAL(field[ompthID].ptr[fileID], missval))
+                  field[ompthID].vec[fileID] = ef[fileID].array[i];
+                  if (DBL_IS_EQUAL(field[ompthID].vec[fileID], missval))
                     {
                       have_miss = 1;
                       break;
@@ -322,11 +337,11 @@ Ensstat3(void *process)
 /* RANK HISTOGRAM */
 /* ************** */
 // for ( j=0; j<nfiles; j++ )
-//   fprintf(stderr,"%5.2g ",field[ompthID].ptr[j]);
+//   fprintf(stderr,"%5.2g ",field[ompthID].vec[j]);
 #ifdef _OPENMP
 #pragma omp critical
 #endif
-                      binID = (int) fldfun(field[ompthID], operfunc);
+                      binID = (int) vfldrank(field[ompthID]);
                       // fprintf(stderr,"-->%i\n",binID);
 
                       if (datafunc == SPACE && !have_miss)
@@ -339,8 +354,8 @@ Ensstat3(void *process)
                       /* ********************************** */
                       /* RECEIVER OPERATING CHARACTERISTICS */
                       /* ********************************** */
-                      dat = &field[ompthID].ptr[1];
-                      ival = field[ompthID].ptr[0] > 0.5 ? 1 : 0;
+                      dat = &field[ompthID].vec[1];
+                      ival = field[ompthID].vec[0] > 0.5 ? 1 : 0;
 
                       for (binID = 0; binID < nbins; binID++) hist[binID] = 0;
 
@@ -395,8 +410,8 @@ Ensstat3(void *process)
                 {
                   val = (double) array2[binID][0];
                   //		fprintf(stderr,"%i ",(int)val);
-                  pstreamDefRecord(streamID2, varID2[varID], binID);
-                  pstreamWriteRecord(streamID2, &val, nmiss);
+                  cdoDefRecord(streamID2, varID2[varID], binID);
+                  cdoWriteRecord(streamID2, &val, nmiss);
                 }
               // fprintf(stderr,"\n");
             }
@@ -404,15 +419,14 @@ Ensstat3(void *process)
             {
               if (DEBUG_ROC)
                 {
-                  fprintf(stderr, "#             :     TP     FP     FN     TN "
-                                  "        TPR        FPR\n");
+                  fprintf(stderr, "#             :     TP     FP     FN     TN         TPR        FPR\n");
 
                   for (binID = 0; binID <= nbins; binID++)
                     {
-                      int p = ctg_tab[binID][TP] + ctg_tab[binID][FN];
-                      int n = ctg_tab[binID][FP] + ctg_tab[binID][TN];
-                      double tpr = ctg_tab[binID][TP] / (double) p;
-                      double fpr = ctg_tab[binID][FP] / (double) n;
+                      const int p = ctg_tab[binID][TP] + ctg_tab[binID][FN];
+                      const int n = ctg_tab[binID][FP] + ctg_tab[binID][TN];
+                      const double tpr = ctg_tab[binID][TP] / (double) p;
+                      const double fpr = ctg_tab[binID][FP] / (double) n;
                       chksum += ctg_tab[binID][0] + ctg_tab[binID][1] + ctg_tab[binID][2] + ctg_tab[binID][3];
 
                       roc[binID][TPR] = tpr;
@@ -432,21 +446,16 @@ Ensstat3(void *process)
 
   if (operfunc == func_rank)
     {
-      double *tmpdoub;
-      int osize = gridsize;
-
-      if (datafunc == TIME) osize = 1;
-      tmpdoub = (double *) Malloc(osize * sizeof(double));
+      const int osize = (datafunc == TIME) ? 1 : gridsize;
+      std::vector<double> tmpdoub(osize);
 
       for (binID = 0; binID < nfiles; binID++)
         {
           for (i = 0; i < osize; i++) tmpdoub[i] = (double) array2[binID][i];
 
-          pstreamDefRecord(streamID2, varID2[varID], binID);
-          pstreamWriteRecord(streamID2, tmpdoub, nmiss);
+          cdoDefRecord(streamID2, varID2[varID], binID);
+          cdoWriteRecord(streamID2, tmpdoub.data(), nmiss);
         }
-
-      Free(tmpdoub);
     }
   else if (operfunc == func_roc)
     {
@@ -454,16 +463,16 @@ Ensstat3(void *process)
 
       for (i = 0; i <= nbins; i++)
         {
-          int p = ctg_tab[i][TP] + ctg_tab[i][FN];
-          int n = ctg_tab[i][FP] + ctg_tab[i][TN];
-          double tpr = ctg_tab[i][TP] / (double) p;
-          double fpr = ctg_tab[i][FP] / (double) n;
+          const int p = ctg_tab[i][TP] + ctg_tab[i][FN];
+          const int n = ctg_tab[i][FP] + ctg_tab[i][TN];
+          const double tpr = ctg_tab[i][TP] / (double) p;
+          const double fpr = ctg_tab[i][FP] / (double) n;
           chksum += ctg_tab[i][0] + ctg_tab[i][1] + ctg_tab[i][2] + ctg_tab[i][3];
 
           roc[i][TPR] = tpr;
           roc[i][FPR] = fpr;
 
-          int sum = ctg_tab[i][TP] + ctg_tab[i][TN] + ctg_tab[i][FP] + ctg_tab[i][FN];
+          const int sum = ctg_tab[i][TP] + ctg_tab[i][TN] + ctg_tab[i][FP] + ctg_tab[i][FN];
 
           fprintf(stdout, "%3i %10.4g: %6i %6i %6i %6i (%6i): %10.4g %10.4g\n", i, i < nbins ? lThresh[i] : 1, ctg_tab[i][0],
                   ctg_tab[i][1], ctg_tab[i][2], ctg_tab[i][3], sum, tpr, fpr);
@@ -484,44 +493,14 @@ Ensstat3(void *process)
     if (ef[fileID].array) Free(ef[fileID].array);
 
   if (ef) Free(ef);
+
   if (array2)
     {
       for (binID = 0; binID < nfiles; binID++) Free(array2[binID]);
       Free(array2);
     }
 
-  for (i = 0; i < Threading::ompNumThreads; i++)
-    {
-      if (field[i].ptr) Free(field[i].ptr);
-      if (field[i].weight) Free(field[i].weight);
-    }
-
-  if (field) Free(field);
-
   cdoFinish();
 
   return nullptr;
 }
-
-double
-roc_curve_integrate(const double **roc, const int n)
-{
-  double y1, y0, x1, x0, dx, dy;
-  double step_area;
-  double area = 0;
-
-  for (int i = 1; i <= n; ++i)
-    {
-      x1 = roc[i][FPR];
-      x0 = roc[i - 1][FPR];
-      y1 = roc[i][TPR];
-      y0 = roc[i - 1][TPR];
-      dx = x1 - x0;
-      dy = y1 - y0;
-
-      step_area = -0.5 * dx * dy - dx * y0;
-      area += step_area;
-    }
-
-  return area - 0.5;
-}
