@@ -30,7 +30,7 @@
 #include "cdo_int.h"
 #include "param_conversion.h"
 #include "cdo_math.h"
-#include "pstream_int.h"
+
 #include "merge_sort2.h"
 #include "util_files.h"
 
@@ -65,7 +65,8 @@ Ensval(void *process)
   int vlistID;
   int gridID = -1;
   int have_miss = 0;
-  int stream, streamID = 0;
+  int stream = 0;
+  CdoStreamID streamID = 0;
   // int xsize,ysize;
   double missval = 0;
   double xval = 0;
@@ -83,7 +84,7 @@ Ensval(void *process)
 
   struct ens_file_t
   {
-    int streamID;
+    CdoStreamID streamID;
     int vlistID;
     std::vector<double> array;
   };
@@ -124,7 +125,7 @@ Ensval(void *process)
   std::vector<double> r(nostreams);
 
   // one stream for each value of the decomposition
-  std::vector<int> streamID2(nostreams);
+  std::vector<CdoStreamID> streamID2(nostreams);
   std::vector<int> vlistID2(nostreams);
   std::vector<int> taxisID2(nostreams);
   std::vector<int> zaxisID2(nostreams);
@@ -149,14 +150,14 @@ Ensval(void *process)
 
   for (fileID = 0; fileID < nfiles; fileID++)
     {
-      streamID = cdoStreamOpenRead(cdoStreamName(fileID));
+      streamID = cdoOpenRead((fileID));
       vlistID = cdoStreamInqVlist(streamID);
 
       ef[fileID].streamID = streamID;
       ef[fileID].vlistID = vlistID;
     }
 
-  int streamID1 = ef[0].streamID;
+  CdoStreamID streamID1 = ef[0].streamID;
 
   if (Options::cdoVerbose) cdoPrint("Opened %i Input Files for Ensemble Operator", nfiles);
 
@@ -180,7 +181,7 @@ Ensval(void *process)
 
   const char *refname = cdoGetObase();
   memset(file_suffix, 0, sizeof(file_suffix));
-  cdoGenFileSuffix(file_suffix, sizeof(file_suffix), pstreamInqFiletype(streamID1), vlistID1, refname);
+  cdoGenFileSuffix(file_suffix, sizeof(file_suffix), cdoInqFiletype(streamID1), vlistID1, refname);
 
   for (stream = 0; stream < nostreams; stream++)
     {
@@ -217,7 +218,7 @@ Ensval(void *process)
       if (!Options::cdoOverwriteMode && fileExists(ofilename) && !userFileOverwrite(ofilename))
         cdoAbort("Outputfile %s already exists!", ofilename);
 
-      streamID2[stream] = cdoStreamOpenWrite(ofilename);
+      streamID2[stream] = cdoOpenWrite(ofilename);
 
       Free(ofilename);
 
@@ -230,7 +231,7 @@ Ensval(void *process)
       for (i = 0; i < ngrids; i++) vlistChangeGridIndex(vlistID2[stream], i, gridID2);
 
       vlistDefTaxis(vlistID2[stream], taxisID2[stream]);
-      pstreamDefVlist(streamID2[stream], vlistID2[stream]);
+      cdoDefVlist(streamID2[stream], vlistID2[stream]);
 
       // vlistCheck = cdoStreamInqVlist(streamID2[stream]);
       // gridsizeCheck = vlistGridsizeMax(vlistCheck);
@@ -257,14 +258,15 @@ Ensval(void *process)
       for (stream = 0; stream < nostreams; stream++)
         {
           taxisCopyTimestep(taxisID2[stream], taxisID1);
-          if (nrecs0 > 0) pstreamDefTimestep(streamID2[stream], tsID);
+          if (nrecs0 > 0) cdoDefTimestep(streamID2[stream], tsID);
         }
 
       for (int recID = 0; recID < nrecs0; recID++)
         {
           for (fileID = 0; fileID < nfiles; fileID++)
             {
-              pstreamInqRecord(fileID, &varID, &levelID);
+              streamID = ef[fileID].streamID;
+              cdoInqRecord(streamID, &varID, &levelID);
 
               if (fileID == 0)
                 {
@@ -277,8 +279,7 @@ Ensval(void *process)
 
               ef[fileID].array.resize(gridsize);
 
-              streamID = ef[fileID].streamID;
-              pstreamReadRecord(streamID, ef[fileID].array.data(), &nmiss);
+              cdoReadRecord(streamID, ef[fileID].array.data(), &nmiss);
             }
 
           // xsize = gridInqXsize(gridID);
@@ -500,13 +501,13 @@ Ensval(void *process)
 
           for (stream = 0; stream < nostreams; stream++)
             {
-              pstreamDefRecord(streamID2[stream], varID, levelID);
+              cdoDefRecord(streamID2[stream], varID, levelID);
               if (DBL_IS_NAN(r[stream]))
                 {
                   r[stream] = missval;
                   have_miss = 1;
                 }
-              pstreamWriteRecord(streamID2[stream], &r[stream], have_miss);
+              cdoWriteRecord(streamID2[stream], &r[stream], have_miss);
             }
 
           switch (operfunc)
