@@ -36,8 +36,8 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
-#include "grid.h"
+
+#include <mpim_grid.h>
 #include "statistic.h"
 #include "timer.h"
 #include "datetime.h"
@@ -100,7 +100,7 @@ EOF3d(void *process)
 
   if (operfunc == EOF3D_SPATIAL) cdoAbort("Operator not Implemented - use eof3d or eof3dtime instead");
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
   int vlistID1 = cdoStreamInqVlist(streamID1);
 
   /* COUNT NUMBER OF TIMESTEPS if EOF3D_ or EOF3D_TIME */
@@ -114,7 +114,7 @@ EOF3d(void *process)
 
       cdoStreamClose(streamID1);
 
-      streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+      streamID1 = cdoOpenRead(0);
       vlistID1 = cdoStreamInqVlist(streamID1);
     }
   else if (Options::cdoVerbose)
@@ -218,12 +218,12 @@ EOF3d(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
           double missval = vlistInqVarMissval(vlistID1, varID);
 
-          pstreamReadRecord(streamID1, in, &nmiss);
+          cdoReadRecord(streamID1, in, &nmiss);
 
           size_t offset = gridsize * levelID;
           for (size_t i = 0; i < gridsize; ++i)
@@ -390,7 +390,7 @@ EOF3d(void *process)
   /* write files with eigenvalues (ID3) and eigenvectors (ID2) */
 
   /*  eigenvalues */
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
+  CdoStreamID streamID2 = cdoOpenWrite(1);
 
   int vlistID2 = vlistDuplicate(vlistID1);
   int taxisID2 = taxisDuplicate(taxisID1);
@@ -418,7 +418,7 @@ EOF3d(void *process)
   for (int i = 0; i < nzaxis; i++) vlistChangeZaxisIndex(vlistID2, i, zaxisID2);
 
   /*  eigenvectors */
-  int streamID3 = cdoStreamOpenWrite(cdoStreamName(2));
+  CdoStreamID streamID3 = cdoOpenWrite(2);
 
   int vlistID3 = vlistDuplicate(vlistID1);
   int taxisID3 = taxisDuplicate(taxisID1);
@@ -426,8 +426,8 @@ EOF3d(void *process)
   taxisDefRtime(taxisID3, 0);
   vlistDefTaxis(vlistID3, taxisID3);
 
-  pstreamDefVlist(streamID2, vlistID2);
-  pstreamDefVlist(streamID3, vlistID3);
+  cdoDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID3, vlistID3);
 
   int64_t vdate = 10101;
   int vtime = 0;
@@ -439,13 +439,13 @@ EOF3d(void *process)
 
       taxisDefVdate(taxisID2, vdate);
       taxisDefVtime(taxisID2, vtime);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       if (tsID < n_eig)
         {
           taxisDefVdate(taxisID3, vdate);
           taxisDefVtime(taxisID3, vtime);
-          pstreamDefTimestep(streamID3, tsID);
+          cdoDefTimestep(streamID3, tsID);
         }
 
       for (varID = 0; varID < nvars; varID++)
@@ -458,15 +458,15 @@ EOF3d(void *process)
               if (tsID < n_eig)
                 {
                   nmiss = arrayNumMV(gridsizemax, &eigenvectors[varID][tsID][offset], missval);
-                  pstreamDefRecord(streamID3, varID, levelID);
-                  pstreamWriteRecord(streamID3, &eigenvectors[varID][tsID][offset], nmiss);
+                  cdoDefRecord(streamID3, varID, levelID);
+                  cdoWriteRecord(streamID3, &eigenvectors[varID][tsID][offset], nmiss);
                 }
             }
 
           nmiss = (DBL_IS_EQUAL(eigenvalues[varID][tsID][0], missval)) ? 1 : 0;
 
-          pstreamDefRecord(streamID2, varID, 0);
-          pstreamWriteRecord(streamID2, eigenvalues[varID][tsID], nmiss);
+          cdoDefRecord(streamID2, varID, 0);
+          cdoWriteRecord(streamID2, eigenvalues[varID][tsID], nmiss);
         }  // for ( varID = 0; ... )
     }      // for ( tsID = 0; ... )
 
