@@ -16,15 +16,14 @@
  */
 
 /*
- This module contains the following operators:
+  This module contains the following operators:
 
- Eofcoeff             eofcoeff             process eof coefficients
+  Eofcoeff             eofcoeff             process eof coefficients
 */
 
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 #include "util_files.h"
 
 // NO MISSING VALUE SUPPORT ADDED SO FAR
@@ -34,8 +33,6 @@ Eofcoeff(void *process)
 {
   char eof_name[16], oname[1024], filesuffix[32];
   double missval1 = -999, missval2;
-  Field in;
-  Field out;
   int i, varID, levelID;
   int nrecs;
   size_t nmiss;
@@ -46,28 +43,28 @@ Eofcoeff(void *process)
 
   cdoOperatorAdd("eofcoeff", 0, 0, nullptr);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-  int streamID2 = cdoStreamOpenRead(cdoStreamName(1));
+  CdoStreamID streamID1 = cdoOpenRead(0);
+  CdoStreamID streamID2 = cdoOpenRead(1);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = cdoStreamInqVlist(streamID2);
-  int vlistID3 = vlistDuplicate(vlistID2);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = cdoStreamInqVlist(streamID2);
+  const int vlistID3 = vlistDuplicate(vlistID2);
 
   // taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = vlistInqTaxis(vlistID2);
-  int taxisID3 = taxisDuplicate(taxisID2);
+  const int taxisID2 = vlistInqTaxis(vlistID2);
+  const int taxisID3 = taxisDuplicate(taxisID2);
 
-  int gridID1 = vlistInqVarGrid(vlistID1, 0);
-  int gridID2 = vlistInqVarGrid(vlistID2, 0);
+  const int gridID1 = vlistInqVarGrid(vlistID1, 0);
+  const int gridID2 = vlistInqVarGrid(vlistID2, 0);
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
+  const size_t gridsize = vlistGridsizeMax(vlistID1);
   if (gridsize != vlistGridsizeMax(vlistID2))
     cdoAbort("Gridsize of input files does not match! %zu and %zu", gridsize, vlistGridsizeMax(vlistID2));
 
   if (vlistNgrids(vlistID2) > 1 || vlistNgrids(vlistID1) > 1) cdoAbort("Too many different grids in input!");
 
-  int nvars = vlistNvars(vlistID1) == vlistNvars(vlistID2) ? vlistNvars(vlistID1) : -1;
-  int nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, 0));
+  const int nvars = vlistNvars(vlistID1) == vlistNvars(vlistID2) ? vlistNvars(vlistID1) : -1;
+  const int nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, 0));
 
   if (gridID1 != gridID2) cdoCompareGrids(gridID1, gridID2);
 
@@ -76,10 +73,10 @@ Eofcoeff(void *process)
 
   const char *refname = cdoGetObase();
   filesuffix[0] = 0;
-  cdoGenFileSuffix(filesuffix, sizeof(filesuffix), pstreamInqFiletype(streamID1), vlistID1, refname);
+  cdoGenFileSuffix(filesuffix, sizeof(filesuffix), cdoInqFiletype(streamID1), vlistID1, refname);
 
-  Field ***eof = (Field ***) Malloc(nvars * sizeof(Field **));
-  for (varID = 0; varID < nvars; varID++) eof[varID] = (Field **) Malloc(nlevs * sizeof(Field *));
+  FieldVector3D eof(nvars);
+  for (varID = 0; varID < nvars; varID++) eof[varID].resize(nlevs);
 
   int eofID = 0;
   while (1)
@@ -89,49 +86,47 @@ Eofcoeff(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
           missval1 = vlistInqVarMissval(vlistID1, varID);
           if (eofID == 0)
-            eof[varID][levelID] = (Field *) Malloc(1 * sizeof(Field));
+            eof[varID][levelID].resize(1);
           else
-            eof[varID][levelID] = (Field *) Realloc(eof[varID][levelID], (eofID + 1) * sizeof(Field));
+            eof[varID][levelID].resize(eofID + 1);
           eof[varID][levelID][eofID].grid = gridID1;
-          eof[varID][levelID][eofID].nmiss = 0;
           eof[varID][levelID][eofID].missval = missval1;
-          eof[varID][levelID][eofID].ptr = (double *) Malloc(gridsize * sizeof(double));
-          arrayFill(gridsize, &eof[varID][levelID][eofID].ptr[0], missval1);
+          eof[varID][levelID][eofID].resize(gridsize);
+          arrayFill(gridsize, eof[varID][levelID][eofID].vec.data(), missval1);
 
-          if (varID >= nvars) cdoAbort("Internal error - too high varID");
-          if (levelID >= nlevs) cdoAbort("Internal error - too high levelID");
+          if (varID >= nvars) cdoAbort("Internal error - varID >= nvars");
+          if (levelID >= nlevs) cdoAbort("Internal error - levelID >= nlevs");
 
-          pstreamReadRecord(streamID1, eof[varID][levelID][eofID].ptr, &nmiss);
+          cdoReadRecord(streamID1, eof[varID][levelID][eofID].vec.data(), &nmiss);
           eof[varID][levelID][eofID].nmiss = nmiss;
         }
       eofID++;
     }
 
-  int neof = eofID;
+  const int neof = eofID;
 
   if (Options::cdoVerbose) cdoPrint("%s contains %i eof's", cdoGetStreamName(0), neof);
   // Create 1x1 Grid for output
-  int gridID3 = gridCreate(GRID_LONLAT, 1);
+  const int gridID3 = gridCreate(GRID_LONLAT, 1);
   gridDefXsize(gridID3, 1);
   gridDefYsize(gridID3, 1);
-  double xvals = 0;
-  double yvals = 0;
+  double xvals = 0, yvals = 0;
   gridDefXvals(gridID3, &xvals);
   gridDefYvals(gridID3, &yvals);
 
   // Create var-list and time-axis for output
 
-  int ngrids = vlistNgrids(vlistID3);
+  const int ngrids = vlistNgrids(vlistID3);
   for (i = 0; i < ngrids; i++) vlistChangeGridIndex(vlistID3, i, gridID3);
 
   vlistDefTaxis(vlistID3, taxisID3);
   for (varID = 0; varID < nvars; varID++) vlistDefVarTimetype(vlistID3, varID, TIME_VARYING);
 
   // open streams for eofcoeff output
-  std::vector<int> streamIDs(neof);
+  std::vector<CdoStreamID> streamIDs(neof);
   for (eofID = 0; eofID < neof; eofID++)
     {
       oname[nchars] = '\0';
@@ -140,19 +135,20 @@ Eofcoeff(void *process)
       strcat(oname, eof_name);
       if (filesuffix[0]) strcat(oname, filesuffix);
 
-      streamIDs[eofID] = cdoStreamOpenWrite(oname);
+      streamIDs[eofID] = cdoOpenWrite(oname);
 
-      if (Options::cdoVerbose) cdoPrint("opened %s ('w')  as stream%i for %i. eof", oname, streamIDs[eofID], eofID + 1);
+      if (Options::cdoVerbose) cdoPrint("opened %s ('w')  as stream%i for %i. eof", oname, streamIDs[eofID]->getID(), eofID + 1);
 
-      pstreamDefVlist(streamIDs[eofID], vlistID3);
+      cdoDefVlist(streamIDs[eofID], vlistID3);
     }
 
   // ALLOCATE temporary fields for data read and write
-  in.ptr = (double *) Malloc(gridsize * sizeof(double));
+  Field in, out;
+  in.resize(gridsize );
   in.grid = gridID1;
   out.missval = missval1;
   out.nmiss = 0;
-  out.ptr = (double *) Malloc(1 * sizeof(double));
+  out.resize(1);
 
   int tsID = 0;
   while (1)
@@ -164,49 +160,44 @@ Eofcoeff(void *process)
       /*for ( eofID=0; eofID<neof; eofID++)
         {
           fprintf(stderr, "defining ts %i\n", tsID);
-          pstreamDefTimestep(streamIDs[eofID],tsID);
+          cdoDefTimestep(streamIDs[eofID],tsID);
         }
       */
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID2, &varID, &levelID);
+          cdoInqRecord(streamID2, &varID, &levelID);
           missval2 = vlistInqVarMissval(vlistID2, varID);
-          pstreamReadRecord(streamID2, in.ptr, &nmiss);
-          in.nmiss = nmiss;
+          cdoReadRecord(streamID2, in.vec.data(), &in.nmiss);
 
           for (eofID = 0; eofID < neof; eofID++)
             {
-              if (recID == 0) pstreamDefTimestep(streamIDs[eofID], tsID);
-              // if ( recID == 0 ) fprintf(stderr, "ts%i rec%i eof%i\n", tsID,
-              // recID, eofID);
-              out.ptr[0] = 0;
+              if (recID == 0) cdoDefTimestep(streamIDs[eofID], tsID);
+              // if ( recID == 0 ) fprintf(stderr, "ts%i rec%i eof%i\n", tsID, recID, eofID);
+              out.vec[0] = 0;
               out.grid = gridID3;
               out.missval = missval2;
               for (size_t i = 0; i < gridsize; i++)
                 {
-                  if (!DBL_IS_EQUAL(in.ptr[i], missval2) && !DBL_IS_EQUAL(eof[varID][levelID][eofID].ptr[i], missval1))
+                  if (!DBL_IS_EQUAL(in.vec[i], missval2) && !DBL_IS_EQUAL(eof[varID][levelID][eofID].vec[i], missval1))
                     {
-                      // double tmp =
-                      // w[i]*in.ptr[i]*eof[varID][levelID][eofID].ptr[i];
-                      double tmp = in.ptr[i] * eof[varID][levelID][eofID].ptr[i];
-                      out.ptr[0] += tmp;
+                      // out.vec[0] += w[i]*in.vec[i]*eof[varID][levelID][eofID].vec[i];
+                      out.vec[0] += in.vec[i] * eof[varID][levelID][eofID].vec[i];
                     }
                 }
-              if (!DBL_IS_EQUAL(out.ptr[0], 0.))
+              if (!DBL_IS_EQUAL(out.vec[0], 0.))
                 nmiss = 0;
               else
                 {
                   nmiss = 1;
-                  out.ptr[0] = missval2;
+                  out.vec[0] = missval2;
                 }
 
-              pstreamDefRecord(streamIDs[eofID], varID, levelID);
-              // fprintf(stderr, "%d %d %d %d %d %g\n", streamIDs[eofID],tsID,
-              // recID, varID, levelID,*out.ptr);
-              pstreamWriteRecord(streamIDs[eofID], out.ptr, nmiss);
+              cdoDefRecord(streamIDs[eofID], varID, levelID);
+              // fprintf(stderr, "%d %d %d %d %d %g\n", streamIDs[eofID],tsID, recID, varID, levelID,*out.vec.data());
+              cdoWriteRecord(streamIDs[eofID], out.vec.data(), nmiss);
             }
-          if (varID >= nvars) cdoAbort("Internal error - too high varID");
-          if (levelID >= nlevs) cdoAbort("Internal error - too high levelID");
+          if (varID >= nvars) cdoAbort("Internal error - varID >= nvars");
+          if (levelID >= nlevs) cdoAbort("Internal error - levelID >= nlevs");
         }
 
       tsID++;
