@@ -16,15 +16,14 @@
  */
 
 /*
- This module contains the following operators:
+  This module contains the following operators:
 
- Eofcoeff             eofcoeff             process eof coefficients
+  Eofcoeff3d             eofcoeff3d             process eof coefficients
 */
 
 #include "cdi.h"
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 #include "util_files.h"
 
 // NO MISSING VALUE SUPPORT ADDED SO FAR
@@ -34,7 +33,6 @@ Eofcoeff3d(void *process)
 {
   char eof_name[16], oname[1024], filesuffix[32];
   double missval1 = -999, missval2 = -999;
-  Field in;
   int i, varID, levelID;
   int nrecs;
   size_t nmiss;
@@ -45,26 +43,26 @@ Eofcoeff3d(void *process)
 
   cdoOperatorAdd("eofcoeff3d", 0, 0, nullptr);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-  int streamID2 = cdoStreamOpenRead(cdoStreamName(1));
+  CdoStreamID streamID1 = cdoOpenRead(0);
+  CdoStreamID streamID2 = cdoOpenRead(1);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = cdoStreamInqVlist(streamID2);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = cdoStreamInqVlist(streamID2);
 
   // taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = vlistInqTaxis(vlistID2);
-  int taxisID3 = taxisDuplicate(taxisID2);
+  const int taxisID2 = vlistInqTaxis(vlistID2);
+  const int taxisID3 = taxisDuplicate(taxisID2);
 
-  int gridID1 = vlistInqVarGrid(vlistID1, 0);
-  int gridID2 = vlistInqVarGrid(vlistID2, 0);
+  const int gridID1 = vlistInqVarGrid(vlistID1, 0);
+  const int gridID2 = vlistInqVarGrid(vlistID2, 0);
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
+  const size_t gridsize = vlistGridsizeMax(vlistID1);
   if (gridsize != vlistGridsizeMax(vlistID2)) cdoAbort("Gridsize of input files does not match!");
 
   if (vlistNgrids(vlistID2) > 1 || vlistNgrids(vlistID1) > 1) cdoAbort("Too many different grids in input");
 
-  int nvars = vlistNvars(vlistID1) == vlistNvars(vlistID2) ? vlistNvars(vlistID1) : -1;
-  int nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, 0));
+  const int nvars = vlistNvars(vlistID1) == vlistNvars(vlistID2) ? vlistNvars(vlistID1) : -1;
+  const int nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, 0));
 
   if (gridID1 != gridID2) cdoCompareGrids(gridID1, gridID2);
 
@@ -73,10 +71,10 @@ Eofcoeff3d(void *process)
 
   const char *refname = cdoGetObase();
   filesuffix[0] = 0;
-  cdoGenFileSuffix(filesuffix, sizeof(filesuffix), pstreamInqFiletype(streamID1), vlistID1, refname);
+  cdoGenFileSuffix(filesuffix, sizeof(filesuffix), cdoInqFiletype(streamID1), vlistID1, refname);
 
-  Field ***eof = (Field ***) Malloc(nvars * sizeof(Field **));
-  for (varID = 0; varID < nvars; varID++) eof[varID] = (Field **) Malloc(nlevs * sizeof(Field *));
+  FieldVector3D eof(nvars);
+  for (varID = 0; varID < nvars; varID++) eof[varID].resize(nlevs);
 
   int eofID = 0;
   while (1)
@@ -86,60 +84,58 @@ Eofcoeff3d(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
           missval1 = vlistInqVarMissval(vlistID1, varID);
           if (eofID == 0)
-            eof[varID][levelID] = (Field *) Malloc(1 * sizeof(Field));
+            eof[varID][levelID].resize(1);
           else
-            eof[varID][levelID] = (Field *) Realloc(eof[varID][levelID], (eofID + 1) * sizeof(Field));
+            eof[varID][levelID].resize(eofID + 1);
           eof[varID][levelID][eofID].grid = gridID1;
-          eof[varID][levelID][eofID].nmiss = 0;
           eof[varID][levelID][eofID].missval = missval1;
-          eof[varID][levelID][eofID].ptr = (double *) Malloc(gridsize * sizeof(double));
-          arrayFill(gridsize, &eof[varID][levelID][eofID].ptr[0], missval1);
+          eof[varID][levelID][eofID].resize(gridsize);
+          arrayFill(gridsize, eof[varID][levelID][eofID].vec.data(), missval1);
 
-          if (varID >= nvars) cdoAbort("Internal error - too high varID");
-          if (levelID >= nlevs) cdoAbort("Internal error - too high levelID");
+          if (varID >= nvars) cdoAbort("Internal error - varID >= nvars");
+          if (levelID >= nlevs) cdoAbort("Internal error - levelID >= nlevs");
 
-          pstreamReadRecord(streamID1, eof[varID][levelID][eofID].ptr, &nmiss);
+          cdoReadRecord(streamID1, eof[varID][levelID][eofID].vec.data(), &nmiss);
           eof[varID][levelID][eofID].nmiss = nmiss;
         }
       eofID++;
     }
 
-  int neof = eofID;
+  const int neof = eofID;
 
   if (Options::cdoVerbose) cdoPrint("%s contains %i eof's", cdoGetStreamName(0), neof);
   // Create 1x1 Grid for output
-  int gridID3 = gridCreate(GRID_LONLAT, 1);
+  const int gridID3 = gridCreate(GRID_LONLAT, 1);
   gridDefXsize(gridID3, 1);
   gridDefYsize(gridID3, 1);
-  double xvals = 0;
-  double yvals = 0;
+  double xvals = 0, yvals = 0;
   gridDefXvals(gridID3, &xvals);
   gridDefYvals(gridID3, &yvals);
 
   double zvals = 0.;
-  int zaxisID3 = zaxisCreate(ZAXIS_GENERIC, 1);
+  const int zaxisID3 = zaxisCreate(ZAXIS_GENERIC, 1);
   zaxisDefLevels(zaxisID3, &zvals);
   zaxisDefName(zaxisID3, "zaxis_Reduced");
   zaxisDefLongname(zaxisID3, "Reduced zaxis from EOF3D - only one coefficient per 3D eigenvector and time step");
 
   // Create var-list and time-axis for output
 
-  int vlistID3 = vlistDuplicate(vlistID2);
+  const int vlistID3 = vlistDuplicate(vlistID2);
 
-  int ngrids = vlistNgrids(vlistID3);
+  const int ngrids = vlistNgrids(vlistID3);
   for (i = 0; i < ngrids; i++) vlistChangeGridIndex(vlistID3, i, gridID3);
 
-  int nzaxis = vlistNzaxis(vlistID3);
+  const int nzaxis = vlistNzaxis(vlistID3);
   for (i = 0; i < nzaxis; i++) vlistChangeZaxisIndex(vlistID3, i, zaxisID3);
 
   vlistDefTaxis(vlistID3, taxisID3);
   for (varID = 0; varID < nvars; varID++) vlistDefVarTimetype(vlistID3, varID, TIME_VARYING);
 
   // open streams for eofcoeff output
-  int *streamIDs = (int *) Malloc(neof * sizeof(int));
+  std::vector<CdoStreamID> streamIDs(neof);
   for (eofID = 0; eofID < neof; eofID++)
     {
       oname[nchars] = '\0';
@@ -148,25 +144,26 @@ Eofcoeff3d(void *process)
       strcat(oname, eof_name);
       if (filesuffix[0]) strcat(oname, filesuffix);
 
-      streamIDs[eofID] = cdoStreamOpenWrite(oname);
+      streamIDs[eofID] = cdoOpenWrite(oname);
 
-      if (Options::cdoVerbose) cdoPrint("opened %s ('w')  as stream%i for %i. eof", oname, streamIDs[eofID], eofID + 1);
+      if (Options::cdoVerbose) cdoPrint("opened %s ('w')  as stream%i for %i. eof", oname, streamIDs[eofID]->getID(), eofID + 1);
 
-      pstreamDefVlist(streamIDs[eofID], vlistID3);
+      cdoDefVlist(streamIDs[eofID], vlistID3);
     }
 
   // ALLOCATE temporary fields for data read and write
-  in.ptr = (double *) Malloc(gridsize * sizeof(double));
+  Field in;
+  in.resize(gridsize);
   in.grid = gridID1;
-  Field **out = (Field **) Malloc(nvars * sizeof(Field *));
+  FieldVector2D out(nvars);
   for (varID = 0; varID < nvars; varID++)
     {
-      out[varID] = (Field *) Malloc(neof * sizeof(Field));
+      out[varID].resize(neof);
       for (eofID = 0; eofID < neof; eofID++)
         {
           out[varID][eofID].missval = missval1;
           out[varID][eofID].nmiss = 0;
-          out[varID][eofID].ptr = (double *) Malloc(1 * sizeof(double));
+          out[varID][eofID].resize(1);
         }
     }
 
@@ -179,7 +176,7 @@ Eofcoeff3d(void *process)
       for (varID = 0; varID < nvars; varID++)
         for (eofID = 0; eofID < neof; eofID++)
           {
-            out[varID][eofID].ptr[0] = 0;
+            out[varID][eofID].vec[0] = 0;
             out[varID][eofID].grid = gridID3;
             out[varID][eofID].missval = missval2;
           }
@@ -188,24 +185,21 @@ Eofcoeff3d(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID2, &varID, &levelID);
-          pstreamReadRecord(streamID2, in.ptr, &nmiss);
+          cdoInqRecord(streamID2, &varID, &levelID);
+          cdoReadRecord(streamID2, in.vec.data(), &in.nmiss);
           missval2 = vlistInqVarMissval(vlistID2, varID);
-          in.nmiss = nmiss;
 
           for (eofID = 0; eofID < neof; eofID++)
             {
-              if (recID == 0) pstreamDefTimestep(streamIDs[eofID], tsID);
+              if (recID == 0) cdoDefTimestep(streamIDs[eofID], tsID);
 
               nmiss = 0;
               for (size_t i = 0; i < gridsize; i++)
                 {
-                  if (!DBL_IS_EQUAL(in.ptr[i], missval2) && !DBL_IS_EQUAL(eof[varID][levelID][eofID].ptr[i], missval1))
+                  if (!DBL_IS_EQUAL(in.vec[i], missval2) && !DBL_IS_EQUAL(eof[varID][levelID][eofID].vec[i], missval1))
                     {
-                      // double tmp =
-                      // w[i]*in.ptr[i]*eof[varID][levelID][eofID].ptr[i];
-                      double tmp = in.ptr[i] * eof[varID][levelID][eofID].ptr[i];
-                      out[varID][eofID].ptr[0] += tmp;
+                      // out[varID][eofID].vec[0] += w[i]*in.vec[i]*eof[varID][levelID][eofID].vec[i];
+                      out[varID][eofID].vec[0] += in.vec[i] * eof[varID][levelID][eofID].vec[i];;
                     }
                   else
                     nmiss += 1;
@@ -213,21 +207,21 @@ Eofcoeff3d(void *process)
               /*
               if ( nmiss ) {
                 out[varID][eofID].nmiss=1;
-                out[varID][eofID].ptr[0]=missval2;
+                out[varID][eofID].vec[0] = missval2;
               }
               */
             }
 
-          if (varID >= nvars) cdoAbort("Internal error - too high varID");
-          if (levelID >= nlevs) cdoAbort("Internal error - too high levelID");
+          if (varID >= nvars) cdoAbort("Internal error - varID >= nvars");
+          if (levelID >= nlevs) cdoAbort("Internal error - levelID >= nlevs");
         }
 
       for (eofID = 0; eofID < neof; eofID++)
         {
           for (varID = 0; varID < nvars; varID++)
             {
-              pstreamDefRecord(streamIDs[eofID], varID, 0);
-              pstreamWriteRecord(streamIDs[eofID], out[varID][eofID].ptr, out[varID][eofID].nmiss);
+              cdoDefRecord(streamIDs[eofID], varID, 0);
+              cdoWriteRecord(streamIDs[eofID], out[varID][eofID].vec.data(), out[varID][eofID].nmiss);
             }
         }
 
