@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 
 void *
 EstFreq(void *process)
@@ -29,20 +28,20 @@ EstFreq(void *process)
 
   cdoInitialize(process);
 
-  bool lcopy = UNCHANGED_RECORD;
+  bool lcopy = unchangedRecord();
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID1 = cdoStreamInqVlist(streamID1);
   int vlistID2 = vlistDuplicate(vlistID1);
 
   int taxisID1 = vlistInqTaxis(vlistID1);
   int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1), cdoFiletype());
+  CdoStreamID streamID2 = cdoOpenWrite(1);
 
-  streamDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID2, vlistID2);
 
   int ntsteps = vlistNtsteps(vlistID1);
 
@@ -54,10 +53,10 @@ EstFreq(void *process)
     }
 
   int tsID = 0;
-  int fyear, lyear, fmonth, lmonth, dummy;
+  int fyear = 0, lyear, fmonth = 0, lmonth, dummy;
   int step_per_year = 0, currentyear, currentmon;
 
-  while ((nrecs = streamInqTimestep(streamID1, tsID)))
+  while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       if (tsID == 0)
         {
@@ -73,21 +72,21 @@ EstFreq(void *process)
           step_per_year++;
         }
       taxisCopyTimestep(taxisID2, taxisID1);
-      streamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          streamInqRecord(streamID1, &varID, &levelID);
-          streamDefRecord(streamID2, varID, levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoDefRecord(streamID2, varID, levelID);
 
           if (lcopy)
             {
-              streamCopyRecord(streamID2, streamID1);
+              cdoCopyRecord(streamID2, streamID1);
             }
           else
             {
-              streamReadRecord(streamID1, array.data(), &nmiss);
-              streamWriteRecord(streamID2, array.data(), nmiss);
+              cdoReadRecord(streamID1, array.data(), &nmiss);
+              cdoWriteRecord(streamID2, array.data(), nmiss);
             }
         }
 
@@ -98,10 +97,10 @@ EstFreq(void *process)
 
   if (ntsteps > 2)
     {
-      streamInqTimestep(streamID2, ntsteps);
+      cdoStreamInqTimestep(streamID2, ntsteps);
       cdiDecodeDate(taxisInqVdate(taxisID2), &lyear, &lmonth, &dummy);
       /* First, estimation by maximal number of time steps divided by covered years between last and first time step */
-      if (cdoVerbose)
+      if (Options::cdoVerbose)
         printf("Frequency is calculated by dividing the number of time steps '%d' included in the time axis by the covered years"
                " of the time axis\ncomputed by the difference of the year of the last time stamp '%d' and the year of the first"
                " time stamp '%d'.\n",
@@ -123,7 +122,7 @@ EstFreq(void *process)
       else
         {
           int covered_months = lmonth - fmonth + 1;
-          if (cdoVerbose)
+          if (Options::cdoVerbose)
             printf("The fraction ntsteps / covered_years = '%f' is neither 1, 12, 365, 365.25, 366 nor a multiple of 365 which "
                    "would correspond to frequencies yearly, monthly, daily or subdaily respectively.\n Next try:\n\nFrequency is "
                    "calculated by dividing the number of time steps '%d' in year '%d' by the covered months in that year '%d'.\n",
@@ -150,11 +149,11 @@ EstFreq(void *process)
   else
     cdoAbort("For %d found timesteps no frequency can be computed - at least 3 timesteps are required.", ntsteps);
 
-  if (cdoVerbose) printf("Your file indicates a frequency of '%s'.\n", frequency);
+  if (Options::cdoVerbose) printf("Your file indicates a frequency of '%s'.\n", frequency);
   cdiDefAttTxt(vlistID2, CDI_GLOBAL, "frequency", 3, frequency);
 
-  streamClose(streamID1);
-  streamClose(streamID2);
+  cdoStreamClose(streamID1);
+  cdoStreamClose(streamID2);
 
   vlistDestroy(vlistID2);
 
