@@ -30,21 +30,23 @@ Functions: min, max, avg, std, var
 Constansts: M_PI, M_E
 */
 
-#include <sys/stat.h>  /* stat */
+#include <sys/stat.h> /* stat */
 #include <cassert>
 
 #include <cdi.h>
 
 #include "cdo_int.h"
 #include "cdo_math.h"
-#include "pstream_int.h"
-#include "grid.h"
+#include <mpim_grid.h>
+#include "gridreference.h"
 #include "expr.h"
 #include "datetime.h"
 
+
 void grid_cell_area(int gridID, double *array);
 int getSurfaceID(int vlistID);
 
+
 static char *
 exprsFromArgument(int exprArgc, const char **exprArgv)
 {
@@ -101,7 +103,7 @@ exprsFromFile(int exprArgc, const char **exprArgv)
   struct stat filestat;
   if (stat(exprf, &filestat) != 0) cdoAbort("Stat failed on %s", exprf);
 
-  size_t fsize = (size_t) filestat.st_size;
+  const size_t fsize = (size_t) filestat.st_size;
   char *exprs = (char *) Malloc(fsize + 1);
 
   int ichar, ipos = 0;
@@ -123,8 +125,8 @@ str_replace(char *target, const char *needle, const char *replacement)
   char buffer[1024] = { 0 };
   char *insert_point = &buffer[0];
   const char *tmp = target;
-  size_t needle_len = strlen(needle);
-  size_t repl_len = strlen(replacement);
+  const size_t needle_len = strlen(needle);
+  const size_t repl_len = strlen(replacement);
 
   while (1)
     {
@@ -158,7 +160,7 @@ str_replace(char *target, const char *needle, const char *replacement)
 static char *
 exprsExpand(char *exprs, int vlistID)
 {
-  size_t len = strlen(exprs);
+  const size_t len = strlen(exprs);
   unsigned nequal = 0, nsemi = 0;
   for (size_t i = 0; i < len; ++i)
     {
@@ -170,7 +172,7 @@ exprsExpand(char *exprs, int vlistID)
   if (nequal == 0 && nsemi == 1 && strstr(exprs, needle))
     {
       char varname[CDI_MAX_NAME];
-      int nvars = vlistNvars(vlistID);
+      const int nvars = vlistNvars(vlistID);
       const size_t bufsize = 1024;
       char *sbuf = (char *) Malloc(bufsize);
       char *buf = (char *) Malloc(nvars * bufsize);
@@ -198,7 +200,7 @@ params_new(int vlistID)
   paramType *params = (paramType *) Malloc(MAX_PARAMS * sizeof(paramType));
   memset(params, 0, MAX_PARAMS * sizeof(paramType));
 
-  int nvars1 = vlistNvars(vlistID);
+  const int nvars1 = vlistNvars(vlistID);
 
   char name[CDI_MAX_NAME];
   char longname[CDI_MAX_NAME];
@@ -206,14 +208,14 @@ params_new(int vlistID)
 
   for (int varID = 0; varID < nvars1; varID++)
     {
-      int gridID = vlistInqVarGrid(vlistID, varID);
-      int zaxisID = vlistInqVarZaxis(vlistID, varID);
-      int datatype = vlistInqVarDatatype(vlistID, varID);
-      int steptype = vlistInqVarTimetype(vlistID, varID);
-      size_t nlat = gridInqYsize(gridID);
-      size_t ngp = gridInqSize(gridID);
-      size_t nlev = zaxisInqSize(zaxisID);
-      double missval = vlistInqVarMissval(vlistID, varID);
+      const int gridID = vlistInqVarGrid(vlistID, varID);
+      const int zaxisID = vlistInqVarZaxis(vlistID, varID);
+      const int datatype = vlistInqVarDatatype(vlistID, varID);
+      const int steptype = vlistInqVarTimetype(vlistID, varID);
+      const size_t nlat = gridInqYsize(gridID);
+      const size_t ngp = gridInqSize(gridID);
+      const size_t nlev = zaxisInqSize(zaxisID);
+      const double missval = vlistInqVarMissval(vlistID, varID);
 
       vlistInqVarName(vlistID, varID, name);
       vlistInqVarLongname(vlistID, varID, longname);
@@ -245,7 +247,7 @@ params_new(int vlistID)
 static void
 params_add_coord(parseParamType *parse_arg, int coord, int cdiID, size_t size, const char *units, const char *longname)
 {
-  int ncoords = parse_arg->ncoords;
+  const int ncoords = parse_arg->ncoords;
   if (ncoords >= parse_arg->maxcoords) cdoAbort("Too many coordinates (limit=%d)", parse_arg->maxcoords);
 
   parse_arg->coords[ncoords].needed = false;
@@ -264,7 +266,7 @@ params_add_coord(parseParamType *parse_arg, int coord, int cdiID, size_t size, c
 int
 params_get_coordID(parseParamType *parse_arg, int coord, int cdiID)
 {
-  int ncoords = parse_arg->ncoords;
+  const int ncoords = parse_arg->ncoords;
   for (int coordID = 0; coordID < ncoords; ++coordID)
     {
       if (parse_arg->coords[coordID].coord == coord && parse_arg->coords[coordID].cdiID == cdiID) return coordID;
@@ -281,11 +283,11 @@ params_add_coordinates(int vlistID, parseParamType *parse_arg)
   char longname[CDI_MAX_NAME];
   char units[CDI_MAX_NAME];
 
-  int ngrids = vlistNgrids(vlistID);
+  const int ngrids = vlistNgrids(vlistID);
   for (int index = 0; index < ngrids; ++index)
     {
-      int gridID = vlistGrid(vlistID, index);
-      size_t size = gridInqSize(gridID);
+      const int gridID = vlistGrid(vlistID, index);
+      const size_t size = gridInqSize(gridID);
       gridInqXunits(gridID, units);
       params_add_coord(parse_arg, 'x', gridID, size, units, "longitude");
       gridInqYunits(gridID, units);
@@ -295,11 +297,11 @@ params_add_coordinates(int vlistID, parseParamType *parse_arg)
       params_add_coord(parse_arg, 'w', gridID, size, nullptr, "grid cell area weights");
     }
 
-  int nzaxis = vlistNzaxis(vlistID);
+  const int nzaxis = vlistNzaxis(vlistID);
   for (int index = 0; index < nzaxis; ++index)
     {
-      int zaxisID = vlistZaxis(vlistID, index);
-      int size = zaxisInqSize(zaxisID);
+      const int zaxisID = vlistZaxis(vlistID, index);
+      const int size = zaxisInqSize(zaxisID);
       zaxisInqUnits(zaxisID, units);
       zaxisInqLongname(zaxisID, longname);
       params_add_coord(parse_arg, 'z', zaxisID, size, units, longname);
@@ -324,6 +326,7 @@ params_add_ts(parseParamType *parse_arg)
       params[varID].nlev = 1;
 
       parse_arg->nparams++;
+      parse_arg->cnparams++;
     }
 
   return varID;
@@ -355,6 +358,7 @@ parseParamInit(parseParamType *parse_arg, int vlistID, int pointID, int zonalID,
 
   parse_arg->maxparams = MAX_PARAMS;
   parse_arg->nparams = nvars;
+  parse_arg->cnparams = nvars;
   parse_arg->nvars1 = nvars;
   parse_arg->init = true;
   parse_arg->debug = Options::cdoVerbose != 0;
@@ -373,11 +377,11 @@ genZonalID(int vlistID)
 {
   int zonalID = -1;
 
-  int ngrids = vlistNgrids(vlistID);
+  const int ngrids = vlistNgrids(vlistID);
   for (int index = 0; index < ngrids; index++)
     {
-      int gridID = vlistGrid(vlistID, index);
-      int gridtype = gridInqType(gridID);
+      const int gridID = vlistGrid(vlistID, index);
+      const int gridtype = gridInqType(gridID);
       if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || gridtype == GRID_GENERIC)
         if (gridInqXsize(gridID) > 1 && gridInqYsize(gridID) >= 1)
           {
@@ -408,9 +412,9 @@ Expr(void *process)
   cdoOperatorAdd("aexprf", 0, 0, "expr script filename");
   // clang-format on
 
-  int operatorID = cdoOperatorID();
-  bool replacesVariables = cdoOperatorF1(operatorID);
-  bool readsCommandLine = cdoOperatorF2(operatorID);
+  const int operatorID = cdoOperatorID();
+  const bool replacesVariables = cdoOperatorF1(operatorID);
+  const bool readsCommandLine = cdoOperatorF2(operatorID);
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
@@ -419,17 +423,17 @@ Expr(void *process)
 
   char *exprs = readsCommandLine ? exprsFromArgument(exprArgc, exprArgv) : exprsFromFile(exprArgc, exprArgv);
 
-  int streamID1 = cdoStreamOpenRead(0);
-  int vlistID1 = cdoStreamInqVlist(streamID1);
+  CdoStreamID streamID1 = cdoOpenRead(0);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
 
   exprs = exprsExpand(exprs, vlistID1);
   if (Options::cdoVerbose) cdoPrint(exprs);
 
-  int nvars1 = vlistNvars(vlistID1);
+  const int nvars1 = vlistNvars(vlistID1);
 
-  int pointID = gridCreate(GRID_GENERIC, 1);
-  int zonalID = genZonalID(vlistID1);
-  int surfaceID = getSurfaceID(vlistID1);
+  const int pointID = gridCreate(GRID_GENERIC, 1);
+  const int zonalID = genZonalID(vlistID1);
+  const int surfaceID = getSurfaceID(vlistID1);
 
   paramType *params = params_new(vlistID1);
 
@@ -438,7 +442,7 @@ Expr(void *process)
   // Set all input variables to 'needed' if replacing is switched off
   for (int varID = 0; varID < nvars1; varID++) parse_arg.needed[varID] = !replacesVariables;
 
-  int vartsID = params_add_ts(&parse_arg);
+  const int vartsID = params_add_ts(&parse_arg);
   parse_arg.tsID = vartsID;
   params_add_coordinates(vlistID1, &parse_arg);
 
@@ -518,25 +522,25 @@ Expr(void *process)
       cdoPrint("vlistNvars(vlistID1)=%d, vlistNvars(vlistID2)=%d", vlistNvars(vlistID1), vlistNvars(vlistID2));
     }
 
-  int nvars2 = vlistNvars(vlistID2);
+  const int nvars2 = vlistNvars(vlistID2);
   if (nvars2 == 0) cdoAbort("No output variable found!");
 
   for (int varID = 0; varID < nvars1; varID++)
     {
       if (parse_arg.needed[varID])
         {
-          size_t ngp = params[varID].ngp;
-          size_t nlev = params[varID].nlev;
-          size_t nItems = cdo::max((size_t)4, ngp * nlev);
+          const size_t ngp = params[varID].ngp;
+          const size_t nlev = params[varID].nlev;
+          const size_t nItems = cdo::max((size_t) 4, ngp * nlev);
           params[varID].data = (double *) Malloc(nItems * sizeof(double));
         }
     }
 
   for (int varID = parse_arg.nvars1; varID < parse_arg.nparams; varID++)
     {
-      size_t ngp = params[varID].ngp;
-      size_t nlev = params[varID].nlev;
-      size_t nItems = cdo::max((size_t)4, ngp * nlev);
+      const size_t ngp = params[varID].ngp;
+      const size_t nlev = params[varID].nlev;
+      const size_t nItems = cdo::max((size_t) 4, ngp * nlev);
       params[varID].data = (double *) Malloc(nItems * sizeof(double));
     }
 
@@ -544,16 +548,16 @@ Expr(void *process)
     {
       if (parse_arg.coords[i].needed)
         {
-          int coord = parse_arg.coords[i].coord;
+          const int coord = parse_arg.coords[i].coord;
           if (coord == 'x' || coord == 'y' || coord == 'a' || coord == 'w')
             {
               int gridID = parse_arg.coords[i].cdiID;
-              size_t ngp = parse_arg.coords[i].size;
+              const size_t ngp = parse_arg.coords[i].size;
               double *data = (double *) Malloc(ngp * sizeof(double));
               parse_arg.coords[i].data = data;
               if (coord == 'x' || coord == 'y')
                 {
-                  int gridtype = gridInqType(gridID);
+                  const int gridtype = gridInqType(gridID);
                   if (gridtype == GRID_GME) gridID = gridToUnstructured(gridID, 0);
                   if (gridtype != GRID_UNSTRUCTURED && gridtype != GRID_CURVILINEAR) gridID = gridToCurvilinear(gridID, 0);
 
@@ -593,8 +597,8 @@ Expr(void *process)
             }
           else if (coord == 'z')
             {
-              int zaxisID = parse_arg.coords[i].cdiID;
-              size_t nlev = parse_arg.coords[i].size;
+              const int zaxisID = parse_arg.coords[i].cdiID;
+              const size_t nlev = parse_arg.coords[i].size;
               double *data = (double *) Malloc(nlev * sizeof(double));
               parse_arg.coords[i].data = data;
               cdoZaxisInqLevels(zaxisID, data);
@@ -606,16 +610,16 @@ Expr(void *process)
 
   for (int varID = parse_arg.nvars1; varID < parse_arg.nparams; varID++)
     {
-      int coord = params[varID].coord;
+      const int coord = params[varID].coord;
       if (coord)
         {
           char *varname = strdup(params[varID].name);
           varname[strlen(varname) - 2] = 0;
           if (coord == 'x' || coord == 'y' || coord == 'a' || coord == 'w')
             {
-              int coordID = params_get_coordID(&parse_arg, coord, params[varID].gridID);
-              int gridID = parse_arg.coords[coordID].cdiID;
-              size_t ngp = parse_arg.coords[coordID].size;
+              const int coordID = params_get_coordID(&parse_arg, coord, params[varID].gridID);
+              const int gridID = parse_arg.coords[coordID].cdiID;
+              const size_t ngp = parse_arg.coords[coordID].size;
               double *data = parse_arg.coords[coordID].data;
               assert(gridID == params[varID].gridID);
               assert(data != nullptr);
@@ -624,9 +628,9 @@ Expr(void *process)
             }
           else if (coord == 'z')
             {
-              int coordID = params_get_coordID(&parse_arg, coord, params[varID].zaxisID);
-              int zaxisID = parse_arg.coords[coordID].cdiID;
-              size_t nlev = parse_arg.coords[coordID].size;
+              const int coordID = params_get_coordID(&parse_arg, coord, params[varID].zaxisID);
+              const int zaxisID = parse_arg.coords[coordID].cdiID;
+              const size_t nlev = parse_arg.coords[coordID].size;
               double *data = parse_arg.coords[coordID].data;
               assert(zaxisID == params[varID].zaxisID);
               assert(data != nullptr);
@@ -642,23 +646,23 @@ Expr(void *process)
 
   if (Options::cdoVerbose) vlistPrint(vlistID2);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(1);
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   int64_t vdate0 = 0;
   int vtime0 = 0;
-  int calendar = taxisInqCalendar(taxisID1);
+  const int calendar = taxisInqCalendar(taxisID1);
 
   int nrecs;
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
-      int64_t vdate = taxisInqVdate(taxisID1);
-      int vtime = taxisInqVtime(taxisID1);
+      const int64_t vdate = taxisInqVdate(taxisID1);
+      const int vtime = taxisInqVtime(taxisID1);
 
       double jdelta = 0;
 
@@ -691,7 +695,7 @@ Expr(void *process)
 
       taxisCopyTimestep(taxisID2, taxisID1);
 
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int varID = 0; varID < nvars1; varID++)
         if (tsID == 0 || params[varID].steptype != TIME_CONSTANT) params[varID].nmiss = 0;
@@ -699,48 +703,49 @@ Expr(void *process)
       for (int recID = 0; recID < nrecs; recID++)
         {
           int varID, levelID;
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
           if (parse_arg.needed[varID])
             {
-              size_t offset = params[varID].ngp * levelID;
+              const size_t offset = params[varID].ngp * levelID;
               double *vardata = params[varID].data + offset;
               size_t nmiss;
-              pstreamReadRecord(streamID1, vardata, &nmiss);
+              cdoReadRecord(streamID1, vardata, &nmiss);
               params[varID].nmiss += nmiss;
             }
         }
 
       for (int varID = 0; varID < nvars2; varID++)
         {
-          int pidx = varIDmap[varID];
+          const int pidx = varIDmap[varID];
           if (pidx < nvars1) continue;
-          size_t ngp = params[pidx].ngp;
-          size_t nlev = params[pidx].nlev;
+          const size_t ngp = params[pidx].ngp;
+          const size_t nlev = params[pidx].nlev;
 
           params[pidx].nmiss = 0;
           arrayFill(ngp * nlev, params[pidx].data, 0.0);
         }
 
+      parse_arg.cnparams = vartsID + 1;
       yy_scan_string(exprs, scanner);
       yyparse(&parse_arg, scanner);
 
       for (int varID = 0; varID < nvars2; varID++)
         {
-          int pidx = varIDmap[varID];
+          const int pidx = varIDmap[varID];
 
           if (tsID > 0 && params[pidx].steptype == TIME_CONSTANT) continue;
 
-          double missval = vlistInqVarMissval(vlistID2, varID);
+          const double missval = vlistInqVarMissval(vlistID2, varID);
 
-          size_t ngp = params[pidx].ngp;
-          int nlev = (int) params[pidx].nlev;
+          const size_t ngp = params[pidx].ngp;
+          const int nlev = (int) params[pidx].nlev;
           for (int levelID = 0; levelID < nlev; levelID++)
             {
-              size_t offset = ngp * levelID;
+              const size_t offset = ngp * levelID;
               double *vardata = params[pidx].data + offset;
-              size_t nmiss = arrayNumMV(ngp, vardata, missval);
-              pstreamDefRecord(streamID2, varID, levelID);
-              pstreamWriteRecord(streamID2, vardata, nmiss);
+              const size_t nmiss = arrayNumMV(ngp, vardata, missval);
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, vardata, nmiss);
             }
         }
 
