@@ -25,17 +25,17 @@
 #include "cdo_int.h"
 #include "param_conversion.h"
 #include "cdo_wtime.h"
-#include "pstream_int.h"
-#include "grid.h"
+#include <mpim_grid.h>
+#include "gridreference.h"
 #include "grid_point_search.h"
 #include "cdo_options.h"
 #include "progress.h"
 #include "cimdOmp.h"
 
 void
-fillmiss(Field *field1, Field *field2, int nfill)
+fillmiss(Field &field1, Field &field2, int nfill)
 {
-  int nx, ny, i, j;
+  int i, j;
   size_t nmiss2 = 0;
   int kr, ku, kl, ko;
   int ir, iu, il, io;
@@ -43,14 +43,14 @@ fillmiss(Field *field1, Field *field2, int nfill)
   double s1, s2;
   double xr, xu, xl, xo;
 
-  int gridID = field1->grid;
-  size_t nmiss1 = field1->nmiss;
-  double missval = field1->missval;
-  double *array1 = field1->ptr;
-  double *array2 = field2->ptr;
+  int gridID = field1.grid;
+  size_t nmiss1 = field1.nmiss;
+  double missval = field1.missval;
+  double *array1 = field1.vec.data();
+  double *array2 = field2.vec.data();
 
-  nx = gridInqXsize(gridID);
-  ny = gridInqYsize(gridID);
+  int nx = gridInqXsize(gridID);
+  int ny = gridInqYsize(gridID);
   bool globgrid = (bool) gridIsCircular(gridID);
 
   int gridtype = gridInqType(gridID);
@@ -206,7 +206,7 @@ fillmiss(Field *field1, Field *field2, int nfill)
 }
 
 void
-fillmiss_one_step(Field *field1, Field *field2, int maxfill)
+fillmiss_one_step(Field &field1, Field &field2, int maxfill)
 {
   int gridID, nx, ny, i, j;
   size_t nmiss2 = 0;
@@ -219,10 +219,10 @@ fillmiss_one_step(Field *field1, Field *field2, int maxfill)
   double *array1, *array2;
   double **matrix1, **matrix2;
 
-  gridID = field1->grid;
-  missval = field1->missval;
-  array1 = field1->ptr;
-  array2 = field2->ptr;
+  gridID = field1.grid;
+  missval = field1.missval;
+  array1 = field1.vec.data();
+  array2 = field2.vec.data();
 
   nx = gridInqXsize(gridID);
   ny = gridInqYsize(gridID);
@@ -377,19 +377,19 @@ fillmiss_one_step(Field *field1, Field *field2, int maxfill)
 }
 
 static void
-setmisstodis(Field *field1, Field *field2, int numNeighbors)
+setmisstodis(Field &field1, Field &field2, int numNeighbors)
 {
-  int gridID = field1->grid;
-  int gridID0 = gridID;
-  double missval = field1->missval;
-  double *array1 = field1->ptr;
-  double *array2 = field2->ptr;
+  int gridID = field1.grid;
+  const int gridID0 = gridID;
+  const double missval = field1.missval;
+  double *array1 = field1.vec.data();
+  double *array2 = field2.vec.data();
 
-  int gridtype = gridInqType(gridID);
-  size_t gridsize = gridInqSize(gridID);
+  const int gridtype = gridInqType(gridID);
+  const size_t gridsize = gridInqSize(gridID);
 
-  size_t nmiss = field1->nmiss;
-  size_t nvals = gridsize - nmiss;
+  size_t nmiss = field1.nmiss;
+  const size_t nvals = gridsize - nmiss;
 
   if (gridtype == GRID_GME) gridID = gridToUnstructured(gridID, 0);
   if (gridtype != GRID_UNSTRUCTURED && gridtype != GRID_CURVILINEAR) gridID = gridToCurvilinear(gridID, 0);
@@ -406,8 +406,7 @@ setmisstodis(Field *field1, Field *field2, int numNeighbors)
         }
     }
 
-  if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
-    cdoAbort("Cell center coordinates missing!");
+  if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0) cdoAbort("Cell center coordinates missing!");
 
   std::vector<double> xvals(gridsize), yvals(gridsize);
   gridInqXvals(gridID, xvals.data());
@@ -454,7 +453,7 @@ setmisstodis(Field *field1, Field *field2, int numNeighbors)
 
   if (nmiss)
     {
-      bool xIsCyclic = false;
+      const bool xIsCyclic = false;
       size_t dims[2] = { nvals, 0 };
       gps = gridPointSearchCreate(xIsCyclic, dims, nvals, &lons[0], &lats[0]);
       gridPointSearchExtrapolate(gps);
@@ -469,8 +468,8 @@ setmisstodis(Field *field1, Field *field2, int numNeighbors)
   double findex = 0;
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(knnWeights, findex, mindex, vindex, array1, array2, xvals, yvals, gps, nmiss, \
-                                              numNeighbors)
+#pragma omp parallel for default(none) \
+  shared(nmiss, knnWeights, findex, mindex, vindex, array1, array2, xvals, yvals, gps, numNeighbors)
 #endif
   for (size_t i = 0; i < nmiss; ++i)
     {
@@ -480,12 +479,12 @@ setmisstodis(Field *field1, Field *field2, int numNeighbors)
       findex++;
       if (cdo_omp_get_thread_num() == 0) Progress::update(0, 1, findex / nmiss);
 
-      int ompthID = cdo_omp_get_thread_num();
+      const int ompthID = cdo_omp_get_thread_num();
 
       gridSearchPoint(gps, xvals[mindex[i]], yvals[mindex[i]], knnWeights[ompthID]);
 
       // Compute weights based on inverse distance if mask is false, eliminate those points
-      size_t nadds = knnWeights[ompthID].computeWeights();
+      const size_t nadds = knnWeights[ompthID].computeWeights();
       if (nadds)
         {
           double result = 0;
@@ -507,20 +506,19 @@ setmisstodis(Field *field1, Field *field2, int numNeighbors)
 void *
 Fillmiss(void *process)
 {
-  size_t nmiss;
   int nrecs, varID, levelID;
-  void (*fill_method)(Field * fin, Field * fout, int) = nullptr;
+  void (*fill_method)(Field &fin, Field &fout, int) = &setmisstodis;
 
   cdoInitialize(process);
 
   // clang-format off
-  int FILLMISS        = cdoOperatorAdd("fillmiss"   ,   0, 0, "nfill");
-  int FILLMISSONESTEP = cdoOperatorAdd("fillmiss2"  ,   0, 0, "nfill");
-  int SETMISSTONN     = cdoOperatorAdd("setmisstonn" ,  0, 0, "");
-  int SETMISSTODIS    = cdoOperatorAdd("setmisstodis" , 0, 0, "number of neighbors");
+  const int FILLMISS        = cdoOperatorAdd("fillmiss"   ,   0, 0, "nfill");
+  const int FILLMISSONESTEP = cdoOperatorAdd("fillmiss2"  ,   0, 0, "nfill");
+  const int SETMISSTONN     = cdoOperatorAdd("setmisstonn" ,  0, 0, "");
+  const int SETMISSTODIS    = cdoOperatorAdd("setmisstodis" , 0, 0, "number of neighbors");
   // clang-format on
 
-  int operatorID = cdoOperatorID();
+  const int operatorID = cdoOperatorID();
 
   int nfill = 1;
   if (operatorID == FILLMISS)
@@ -543,7 +541,7 @@ Fillmiss(void *process)
 
   /* Argument handling */
   {
-    int oargc = operatorArgc();
+    const int oargc = operatorArgc();
     char **oargv = operatorArgv();
 
     if (oargc == 1)
@@ -558,77 +556,71 @@ Fillmiss(void *process)
       cdoAbort("Too many arguments!");
   }
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
+  CdoStreamID streamID2 = cdoOpenWrite(1);
 
-  pstreamDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID2, vlistID2);
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
+  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field1, field2;
-  fieldInit(field1);
-  fieldInit(field2);
-  field1.ptr = (double *) Malloc(gridsize * sizeof(double));
-  field2.ptr = (double *) Malloc(gridsize * sizeof(double));
+  field1.resize(gridsizemax);
+  field2.resize(gridsizemax);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
 
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, field1.ptr, &nmiss);
-          field1.nmiss = nmiss;
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, field1.vec.data(), &field1.nmiss);
 
-          pstreamDefRecord(streamID2, varID, levelID);
+          cdoDefRecord(streamID2, varID, levelID);
 
           if (field1.nmiss == 0)
             {
-              pstreamWriteRecord(streamID2, field1.ptr, 0);
+              cdoWriteRecord(streamID2, field1.vec.data(), 0);
             }
           else
             {
-              int gridID = vlistInqVarGrid(vlistID1, varID);
+              const int gridID = vlistInqVarGrid(vlistID1, varID);
 
               if ((operatorID == FILLMISS || operatorID == FILLMISSONESTEP)
                   && (gridInqType(gridID) == GRID_GME || gridInqType(gridID) == GRID_UNSTRUCTURED))
                 cdoAbort("%s data unsupported!", gridNamePtr(gridInqType(gridID)));
 
               field1.grid = gridID;
+              field1.size = gridInqSize(field1.grid);
               field1.missval = vlistInqVarMissval(vlistID1, varID);
 
               field2.grid = field1.grid;
-              field2.nmiss = 0;
+              field2.size = field1.size;
               field2.missval = field1.missval;
 
-              fill_method(&field1, &field2, nfill);
+              fill_method(field1, field2, nfill);
 
-              size_t gridsize = gridInqSize(field2.grid);
-              size_t nmiss = arrayNumMV(gridsize, field2.ptr, field2.missval);
-              pstreamWriteRecord(streamID2, field2.ptr, nmiss);
+              cdoWriteRecord(streamID2, field2.vec.data(), fieldNumMiss(field2));
             }
         }
+
       tsID++;
     }
 
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
-  if (field2.ptr) Free(field2.ptr);
-  if (field1.ptr) Free(field1.ptr);
-
   cdoFinish();
 
   return nullptr;
