@@ -32,7 +32,7 @@
 #include "cdo_int.h"
 #include "param_conversion.h"
 #include "statistic.h"
-#include "pstream_int.h"
+
 #include "cdo_options.h"
 #include "datetime.h"
 #include "cimdOmp.h"
@@ -47,13 +47,13 @@
 void getTimeInc(double jdelta, int64_t vdate0, int64_t vdate1, int64_t *incperiod, int *incunit);
 
 static void
-create_fmasc(int nts, double fdata, double fmin, double fmax, int *fmasc)
+create_fmasc(int nts, double fdata, double fmin, double fmax, std::vector<int> &fmasc)
 {
-  double dimin = nts * fmin / fdata;
-  double dimax = nts * fmax / fdata;
+  const double dimin = nts * fmin / fdata;
+  const double dimax = nts * fmax / fdata;
 
-  int imin = (dimin < 0) ? 0 : (int) floor(dimin);
-  int imax = (ceil(dimax) > nts / 2) ? nts / 2 : (int) ceil(dimax);
+  const int imin = (dimin < 0) ? 0 : (int) floor(dimin);
+  const int imax = (ceil(dimax) > nts / 2) ? nts / 2 : (int) ceil(dimax);
 
   if (imin < 0 || imin >= nts) cdoAbort("Parameter fmin=%g: timestep %d out of bounds (1-%d)!", fmin, imin + 1, nts);
   if (imax < 0 || imax >= nts) cdoAbort("Parameter fmax=%g: timestep %d out of bounds (1-%d)!", fmax, imax + 1, nts);
@@ -64,7 +64,7 @@ create_fmasc(int nts, double fdata, double fmin, double fmax, int *fmasc)
 
 #ifdef HAVE_LIBFFTW3
 static void
-filter_fftw(int nts, const int *fmasc, fftw_complex *fft_out, fftw_plan *p_T2S, fftw_plan *p_S2T)
+filter_fftw(int nts, const std::vector<int> &fmasc, fftw_complex *fft_out, fftw_plan *p_T2S, fftw_plan *p_S2T)
 {
   fftw_execute(*p_T2S);
 
@@ -82,9 +82,9 @@ filter_fftw(int nts, const int *fmasc, fftw_complex *fft_out, fftw_plan *p_T2S,
 #endif
 
 static void
-filter_intrinsic(int nts, const int *fmasc, double *array1, double *array2)
+filter_intrinsic(int nts, const std::vector<int> &fmasc, double *array1, double *array2)
 {
-  bool lpower2 = ((nts & (nts - 1)) == 0);
+  const bool lpower2 = ((nts & (nts - 1)) == 0);
 
   std::vector<double> work_r;
   std::vector<double> work_i;
@@ -121,7 +121,6 @@ Filter(void *process)
   int iunits[] = { 31536000, 525600, 8760, 365, 12, 1 };
   int nrecs;
   int varID, levelID;
-  int nalloc = 0;
   size_t nmiss;
   int64_t incperiod0 = 0;
   int incunit0 = 0, incunit;
@@ -148,8 +147,8 @@ Filter(void *process)
   cdoOperatorAdd("highpass", HIGHPASS, 0, nullptr);
   cdoOperatorAdd("lowpass", LOWPASS, 0, nullptr);
 
-  int operatorID = cdoOperatorID();
-  int operfunc = cdoOperatorF1(operatorID);
+  const int operatorID = cdoOperatorID();
+  const int operfunc = cdoOperatorF1(operatorID);
 
   if (Options::Use_FFTW)
     {
@@ -163,28 +162,27 @@ Filter(void *process)
 
   if (Options::cdoVerbose && !use_fftw) cdoPrint("Using intrinsic FFT function!");
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int calendar = taxisInqCalendar(taxisID1);
+  const int calendar = taxisInqCalendar(taxisID1);
 
-  int nvars = vlistNvars(vlistID1);
+  VarList varList;
+  varListInit(varList, vlistID1);
+
+  const int nvars = vlistNvars(vlistID1);
   FieldVector3D vars;
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
-      if (tsID >= nalloc)
-        {
-          nalloc += NALLOC_INC;
-          vars.resize(nalloc);
-        }
+      if ((size_t)tsID >= vars.size()) vars.resize(vars.size() + NALLOC_INC);
 
       dtlist.taxisInqTimestep(taxisID1, tsID);
 
@@ -192,31 +190,30 @@ Filter(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          int gridID = vlistInqVarGrid(vlistID1, varID);
-          size_t gridsize = gridInqSize(gridID);
-          vars[tsID][varID][levelID].ptr = (double *) Malloc(gridsize * sizeof(double));
-          pstreamReadRecord(streamID1, vars[tsID][varID][levelID].ptr, &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          const size_t gridsize = varList[varID].gridsize;
+          vars[tsID][varID][levelID].resize(gridsize);
+          cdoReadRecord(streamID1, vars[tsID][varID][levelID].vec.data(), &nmiss);
           vars[tsID][varID][levelID].nmiss = nmiss;
           if (nmiss) cdoAbort("Missing value support for operators in module Filter not added yet!");
         }
 
-      /* get and check time increment */
+      // get and check time increment
       if (tsID > 0)
         {
           int64_t incperiod = 0;
           int year, month, day;
-          int64_t vdate0 = dtlist.getVdate(tsID - 1);
-          int64_t vdate = dtlist.getVdate(tsID);
-          int vtime0 = dtlist.getVtime(tsID - 1);
-          int vtime = dtlist.getVtime(tsID);
+          const int64_t vdate0 = dtlist.getVdate(tsID - 1);
+          const int64_t vdate = dtlist.getVdate(tsID);
+          const int vtime0 = dtlist.getVtime(tsID - 1);
+          const int vtime = dtlist.getVtime(tsID);
 
           cdiDecodeDate(vdate0, &year0, &month0, &day0);
           cdiDecodeDate(vdate, &year, &month, &day);
 
           JulianDate juldate0 = julianDateEncode(calendar, vdate0, vtime0);
           JulianDate juldate = julianDateEncode(calendar, vdate, vtime);
-          double jdelta = julianDateToSeconds(julianDateSub(juldate, juldate0));
+          const double jdelta = julianDateToSeconds(julianDateSub(juldate, juldate0));
 
           if (tsID == 1)
             {
@@ -300,16 +297,15 @@ Filter(void *process)
 
   if (Options::cdoVerbose) cdoPrint("fmin=%g  fmax=%g", fmin, fmax);
 
-  int *fmasc = (int *) Calloc(nts, sizeof(int));
+  std::vector<int> fmasc(nts, 0);
   create_fmasc(nts, fdata, fmin, fmax, fmasc);
 
   for (int varID = 0; varID < nvars; varID++)
     {
-      int gridID = vlistInqVarGrid(vlistID1, varID);
-      size_t gridsize = gridInqSize(gridID);
-      int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+      const size_t gridsize = varList[varID].gridsize;
+      const int nlevels = varList[varID].nlevels;
 
-      for (int levelID = 0; levelID < nlevel; levelID++)
+      for (int levelID = 0; levelID < nlevels; levelID++)
         {
           if (use_fftw)
             {
@@ -319,17 +315,17 @@ Filter(void *process)
 #endif
               for (size_t i = 0; i < gridsize; i++)
                 {
-                  int ompthID = cdo_omp_get_thread_num();
+                  const int ompthID = cdo_omp_get_thread_num();
 
                   for (int tsID = 0; tsID < nts; tsID++)
                     {
-                      ompmem[ompthID].in_fft[tsID][0] = vars[tsID][varID][levelID].ptr[i];
+                      ompmem[ompthID].in_fft[tsID][0] = vars[tsID][varID][levelID].vec[i];
                       ompmem[ompthID].in_fft[tsID][1] = 0;
                     }
 
                   filter_fftw(nts, fmasc, ompmem[ompthID].out_fft, &ompmem[ompthID].p_T2S, &ompmem[ompthID].p_S2T);
 
-                  for (int tsID = 0; tsID < nts; tsID++) vars[tsID][varID][levelID].ptr[i] = ompmem[ompthID].in_fft[tsID][0] / nts;
+                  for (int tsID = 0; tsID < nts; tsID++) vars[tsID][varID][levelID].vec[i] = ompmem[ompthID].in_fft[tsID][0] / nts;
                 }
 #endif
             }
@@ -340,57 +336,52 @@ Filter(void *process)
 #endif
               for (size_t i = 0; i < gridsize; i++)
                 {
-                  int ompthID = cdo_omp_get_thread_num();
+                  const int ompthID = cdo_omp_get_thread_num();
 
-                  for (int tsID = 0; tsID < nts; tsID++) ompmem[ompthID].array1[tsID] = vars[tsID][varID][levelID].ptr[i];
+                  for (int tsID = 0; tsID < nts; tsID++) ompmem[ompthID].array1[tsID] = vars[tsID][varID][levelID].vec[i];
 
                   arrayFill(nts, ompmem[ompthID].array2.data(), 0.0);
 
                   filter_intrinsic(nts, fmasc, ompmem[ompthID].array1.data(), ompmem[ompthID].array2.data());
 
-                  for (int tsID = 0; tsID < nts; tsID++) vars[tsID][varID][levelID].ptr[i] = ompmem[ompthID].array1[tsID];
+                  for (int tsID = 0; tsID < nts; tsID++) vars[tsID][varID][levelID].vec[i] = ompmem[ompthID].array1[tsID];
                 }
             }
         }
     }
 
+#ifdef HAVE_LIBFFTW3
   if (use_fftw)
     {
-#ifdef HAVE_LIBFFTW3
       for (int i = 0; i < Threading::ompNumThreads; i++)
         {
           Free(ompmem[i].in_fft);
           Free(ompmem[i].out_fft);
         }
-#endif
     }
+#endif
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   for (int tsID = 0; tsID < nts; tsID++)
     {
       dtlist.taxisDefTimestep(taxisID2, tsID);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int varID = 0; varID < nvars; varID++)
         {
-          int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-          for (int levelID = 0; levelID < nlevel; levelID++)
+          const int nlevels = varList[varID].nlevels;
+          for (int levelID = 0; levelID < nlevels; levelID++)
             {
-              if (vars[tsID][varID][levelID].ptr)
+              if (!vars[tsID][varID][levelID].empty())
                 {
-                  size_t nmiss = vars[tsID][varID][levelID].nmiss;
-                  pstreamDefRecord(streamID2, varID, levelID);
-                  pstreamWriteRecord(streamID2, vars[tsID][varID][levelID].ptr, nmiss);
-
-                  Free(vars[tsID][varID][levelID].ptr);
-                  vars[tsID][varID][levelID].ptr = nullptr;
+                  const size_t nmiss = vars[tsID][varID][levelID].nmiss;
+                  cdoDefRecord(streamID2, varID, levelID);
+                  cdoWriteRecord(streamID2, vars[tsID][varID][levelID].vec.data(), nmiss);
                 }
             }
         }
-
-      fieldsFree(vlistID1, vars[tsID]);
     }
 
   cdoStreamClose(streamID2);
