@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,8 +23,7 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
-#include "grid.h"
+#include <mpim_grid.h>
 
 void *
 Fldrms(void *process)
@@ -33,22 +32,20 @@ Fldrms(void *process)
   int index;
   int nrecs;
   int varID, levelID;
-  size_t nmiss;
-  double sglval;
 
   cdoInitialize(process);
 
-  bool needWeights = true;
+  const bool needWeights = true;
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-  int streamID2 = cdoStreamOpenRead(cdoStreamName(1));
+  CdoStreamID streamID1 = cdoOpenRead(0);
+  CdoStreamID streamID2 = cdoOpenRead(1);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = cdoStreamInqVlist(streamID2);
-  int vlistID3 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = cdoStreamInqVlist(streamID2);
+  const int vlistID3 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID3 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID3 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID3, taxisID3);
 
   double slon = 0;
@@ -59,14 +56,14 @@ Fldrms(void *process)
   gridDefXvals(gridID3, &slon);
   gridDefYvals(gridID3, &slat);
 
-  int ngrids = vlistNgrids(vlistID1);
+  const int ngrids = vlistNgrids(vlistID1);
   int ndiffgrids = 0;
   for (index = 1; index < ngrids; index++)
     if (vlistGrid(vlistID1, 0) != vlistGrid(vlistID1, index)) ndiffgrids++;
 
   index = 0;
-  int gridID1 = vlistGrid(vlistID1, index);
-  int gridID2 = vlistGrid(vlistID2, index);
+  const int gridID1 = vlistGrid(vlistID1, index);
+  const int gridID2 = vlistGrid(vlistID2, index);
 
   if (gridInqSize(gridID1) != gridInqSize(gridID2)) cdoAbort("Fields have different grid size!");
 
@@ -77,23 +74,18 @@ Fldrms(void *process)
 
   if (ndiffgrids > 0) cdoAbort("Too many different grids!");
 
-  int streamID3 = cdoStreamOpenWrite(cdoStreamName(2), cdoFiletype());
-  pstreamDefVlist(streamID3, vlistID3);
+  CdoStreamID streamID3 = cdoOpenWrite(2);
+  cdoDefVlist(streamID3, vlistID3);
 
-  field_type field1, field2, field3;
-  field_init(&field1);
-  field_init(&field2);
-  field_init(&field3);
+  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
 
-  size_t lim = vlistGridsizeMax(vlistID1);
-  field1.ptr = (double *) Malloc(lim * sizeof(double));
-  field1.weight = NULL;
-  if (needWeights) field1.weight = (double *) Malloc(lim * sizeof(double));
+  Field field1, field2, field3;
+  field1.resize(gridsizemax);
+  if (needWeights) field1.weightv.resize(gridsizemax);
 
-  field2.ptr = (double *) Malloc(lim * sizeof(double));
-  field2.weight = NULL;
+  field2.resize(gridsizemax);
 
-  field3.ptr = &sglval;
+  field3.resize(1);
   field3.grid = gridID3;
 
   int tsID = 0;
@@ -103,16 +95,14 @@ Fldrms(void *process)
 
       taxisCopyTimestep(taxisID3, taxisID1);
 
-      pstreamDefTimestep(streamID3, tsID);
+      cdoDefTimestep(streamID3, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, field1.ptr, &nmiss);
-          field1.nmiss = nmiss;
-          pstreamInqRecord(streamID2, &varID, &levelID);
-          pstreamReadRecord(streamID2, field2.ptr, &nmiss);
-          field2.nmiss = nmiss;
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, field1.vec.data(), &field1.nmiss);
+          cdoInqRecord(streamID2, &varID, &levelID);
+          cdoReadRecord(streamID2, field2.vec.data(), &field2.nmiss);
 
           field1.grid = vlistInqVarGrid(vlistID1, varID);
           field2.grid = vlistInqVarGrid(vlistID2, varID);
@@ -120,18 +110,15 @@ Fldrms(void *process)
           if (needWeights && field1.grid != lastgrid)
             {
               lastgrid = field1.grid;
-              field1.weight[0] = 1;
+              field1.weightv[0] = 1;
               if (field1.size > 1)
                 {
-                  int wstatus = gridWeights(field1.grid, field1.weight);
+                  const int wstatus = gridWeights(field1.grid, field1.weightv.data());
                   if (wstatus != 0 && tsID == 0 && levelID == 0)
                     {
                       char varname[CDI_MAX_NAME];
                       vlistInqVarName(vlistID1, varID, varname);
-                      cdoWarning("Grid cell bounds not available, using "
-                                 "constant grid cell area weights for variable "
-                                 "%s!",
-                                 varname);
+                      cdoWarning("Grid cell bounds not available, using constant grid cell area weights for variable %s!", varname);
                     }
                 }
             }
@@ -140,23 +127,20 @@ Fldrms(void *process)
           field2.missval = vlistInqVarMissval(vlistID1, varID);
           field3.missval = vlistInqVarMissval(vlistID1, varID);
 
-          fldrms(field1, field2, &field3);
+          vfldrms(field1, field2, field3);
 
-          pstreamDefRecord(streamID3, varID, levelID);
-          pstreamWriteRecord(streamID3, &sglval, field3.nmiss);
+          cdoDefRecord(streamID3, varID, levelID);
+          cdoWriteRecord(streamID3, field3.vec.data(), field3.nmiss);
         }
+
       tsID++;
     }
 
-  pstreamClose(streamID3);
-  pstreamClose(streamID2);
-  pstreamClose(streamID1);
-
-  if (field1.ptr) Free(field1.ptr);
-  if (field1.weight) Free(field1.weight);
-  if (field2.ptr) Free(field2.ptr);
+  cdoStreamClose(streamID3);
+  cdoStreamClose(streamID2);
+  cdoStreamClose(streamID1);
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
