@@ -35,14 +35,13 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
-#include "grid.h"
+#include <mpim_grid.h>
 #include "percentiles.h"
 #include "pmlist.h"
 
 static void
-print_location_LL(int operfunc, int vlistID, int varID, int levelID, int gridID, double sglval, double *fieldptr, int64_t vdate,
-                  int vtime)
+print_location_LL(int operfunc, int vlistID, int varID, int levelID, int gridID, double sglval,
+                  const std::vector<double> &fieldvec, int64_t vdate, int vtime)
 {
   static bool showHeader = true;
   const int code = vlistInqVarCode(vlistID, varID);
@@ -61,7 +60,7 @@ print_location_LL(int operfunc, int vlistID, int varID, int levelID, int gridID,
       const size_t nlon = gridInqXsize(gridID);
       for (size_t ij = 0; ij < gridsize; ++ij)
         {
-          if (DBL_IS_EQUAL(fieldptr[ij], sglval))
+          if (DBL_IS_EQUAL(fieldvec[ij], sglval))
             {
               const size_t j = ij / nlon;
               const size_t i = ij - j * nlon;
@@ -111,7 +110,7 @@ fldstatGetParameter(bool *weights)
 }
 
 static void
-fldstatAddOperators(void)
+addOperators(void)
 {
   // clang-format off
   cdoOperatorAdd("fldrange", func_range,  0, nullptr);
@@ -125,6 +124,8 @@ fldstatAddOperators(void)
   cdoOperatorAdd("fldvar",   func_varw,   1, nullptr);
   cdoOperatorAdd("fldvar1",  func_var1w,  1, nullptr);
   cdoOperatorAdd("fldpctl",  func_pctl,   0, nullptr);
+  cdoOperatorAdd("fldskew",  func_skew,   0, nullptr);
+  cdoOperatorAdd("fldkurt",  func_kurt,   0, nullptr);
   // clang-format on
 }
 
@@ -134,11 +135,10 @@ Fldstat(void *process)
   int lastgrid = -1;
   int nrecs;
   int varID, levelID;
-  size_t nmiss;
 
   cdoInitialize(process);
 
-  fldstatAddOperators();
+  addOperators();
 
   const int operatorID = cdoOperatorID();
   const int operfunc = cdoOperatorF1(operatorID);
@@ -155,7 +155,7 @@ Fldstat(void *process)
   bool useweights = true;
   if (needWeights) fldstatGetParameter(&useweights);
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -176,23 +176,21 @@ Fldstat(void *process)
 
   for (int index = 0; index < ngrids; index++) vlistChangeGridIndex(vlistID2, index, gridID2);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
+  CdoStreamID streamID2 = cdoOpenWrite(1);
 
-  pstreamDefVlist(streamID2, vlistID2);
-
-  Field field;
-  fieldInit(field);
+  cdoDefVlist(streamID2, vlistID2);
 
   const size_t gridsizemax = vlistGridsizeMax(vlistID1);
-  field.ptr = (double *) Malloc(gridsizemax * sizeof(double));
-  field.weight = nullptr;
+
+  Field field;
+  field.resize(gridsizemax);
   if (needWeights)
     {
-      field.weight = (double *) Malloc(gridsizemax * sizeof(double));
+      field.weightv.resize(gridsizemax);
       if (!useweights)
         {
           cdoPrint("Using constant grid cell area weights!");
-          for (size_t i = 0; i < gridsizemax; ++i) field.weight[i] = 1;
+          for (size_t i = 0; i < gridsizemax; ++i) field.weightv[i] = 1;
         }
     }
 
@@ -200,7 +198,7 @@ Fldstat(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       // Precompute date + time for later representation in verbose mode
       const int64_t vdate = (Options::cdoVerbose && (operfunc == func_min || operfunc == func_max)) ? taxisInqVdate(taxisID1) : 0;
@@ -208,39 +206,46 @@ Fldstat(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, field.ptr, &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
 
-          field.nmiss = nmiss;
           field.grid = vlistInqVarGrid(vlistID1, varID);
           field.size = gridInqSize(field.grid);
 
           if (needWeights && field.grid != lastgrid)
             {
               lastgrid = field.grid;
-              field.weight[0] = 1;
+              field.weightv[0] = 1;
               if (useweights && field.size > 1)
                 {
-                  const bool wstatus = gridWeights(field.grid, field.weight) != 0;
+                  const bool wstatus = gridWeights(field.grid, field.weightv.data()) != 0;
                   if (wstatus && tsID == 0 && levelID == 0)
                     {
-                      char varname[CDI_MAX_NAME];
-                      vlistInqVarName(vlistID1, varID, varname);
-                      cdoWarning("Grid cell bounds not available, using constant grid cell area weights for variable %s!", varname);
+                      if (ngrids == 1)
+                        {
+                          cdoWarning("Grid cell bounds not available, using constant grid cell area weights!");
+                        }
+                      else
+                        {
+                          char varname[CDI_MAX_NAME];
+                          vlistInqVarName(vlistID1, varID, varname);
+                          cdoWarning("Grid cell bounds not available, using constant grid cell area weights for variable %s!",
+                                     varname);
+                        }
                     }
                 }
             }
 
           field.missval = vlistInqVarMissval(vlistID1, varID);
-          double sglval = (operfunc == func_pctl) ? fldpctl(field, pn) : fldfun(field, operfunc);
+          double sglval = (operfunc == func_pctl) ? vfldpctl(field, pn) : vfldfun(field, operfunc);
 
           if (Options::cdoVerbose && (operfunc == func_min || operfunc == func_max))
-            print_location_LL(operfunc, vlistID1, varID, levelID, field.grid, sglval, field.ptr, vdate, vtime);
+            print_location_LL(operfunc, vlistID1, varID, levelID, field.grid, sglval, field.vec, vdate, vtime);
 
-          nmiss = DBL_IS_EQUAL(sglval, field.missval);
+          const size_t nmiss = DBL_IS_EQUAL(sglval, field.missval);
 
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, &sglval, nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, &sglval, nmiss);
         }
 
       tsID++;
@@ -251,9 +256,6 @@ Fldstat(void *process)
 
   vlistDestroy(vlistID2);
 
-  if (field.ptr) Free(field.ptr);
-  if (field.weight) Free(field.weight);
-
   cdoFinish();
 
   return nullptr;
