@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -24,8 +24,8 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
-#include "grid.h"
+
+#include <mpim_grid.h>
 
 /* routine corr copied from PINGO */
 /* correclation in space */
@@ -94,15 +94,15 @@ Fldstat2(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  cdoOperatorAdd("fldcor",   func_cor,   0, NULL);
-  cdoOperatorAdd("fldcovar", func_covar, 0, NULL);
+  cdoOperatorAdd("fldcor",   func_cor,   0, nullptr);
+  cdoOperatorAdd("fldcovar", func_covar, 0, nullptr);
   // clang-format on
 
   int operatorID = cdoOperatorID();
   int operfunc = cdoOperatorF1(operatorID);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-  int streamID2 = cdoStreamOpenRead(cdoStreamName(1));
+  CdoStreamID streamID1 = cdoOpenRead(0);
+  CdoStreamID streamID2 = cdoOpenRead(1);
 
   int vlistID1 = cdoStreamInqVlist(streamID1);
   int vlistID2 = cdoStreamInqVlist(streamID2);
@@ -126,8 +126,8 @@ Fldstat2(void *process)
 
   for (int index = 0; index < ngrids; index++) vlistChangeGridIndex(vlistID3, index, gridID3);
 
-  int streamID3 = cdoStreamOpenWrite(cdoStreamName(2), cdoFiletype());
-  pstreamDefVlist(streamID3, vlistID3);
+  CdoStreamID streamID3 = cdoOpenWrite(2);
+  cdoDefVlist(streamID3, vlistID3);
 
   size_t gridsizemax = vlistGridsizeMax(vlistID1);
 
@@ -148,14 +148,14 @@ Fldstat2(void *process)
         }
 
       taxisCopyTimestep(taxisID3, taxisID1);
-      pstreamDefTimestep(streamID3, tsID);
+      cdoDefTimestep(streamID3, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamInqRecord(streamID2, &varID, &levelID);
-          pstreamReadRecord(streamID1, array1.data(), &nmiss1);
-          pstreamReadRecord(streamID2, array2.data(), &nmiss2);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID2, &varID, &levelID);
+          cdoReadRecord(streamID1, array1.data(), &nmiss1);
+          cdoReadRecord(streamID2, array2.data(), &nmiss2);
 
           gridID = vlistInqVarGrid(vlistID1, varID);
           size_t gridsize = gridInqSize(gridID);
@@ -184,18 +184,18 @@ Fldstat2(void *process)
 
           size_t nmiss3 = DBL_IS_EQUAL(sglval, missval1) ? 1 : 0;
 
-          pstreamDefRecord(streamID3, varID, levelID);
-          pstreamWriteRecord(streamID3, &sglval, nmiss3);
+          cdoDefRecord(streamID3, varID, levelID);
+          cdoWriteRecord(streamID3, &sglval, nmiss3);
         }
 
       tsID++;
     }
 
-  pstreamClose(streamID3);
-  pstreamClose(streamID2);
-  pstreamClose(streamID1);
+  cdoStreamClose(streamID3);
+  cdoStreamClose(streamID2);
+  cdoStreamClose(streamID1);
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
