@@ -19,7 +19,7 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
+
 #include "statistic.h"
 #include "cdo_options.h"
 #include "cimdOmp.h"
@@ -44,21 +44,24 @@ Fourier(void *process)
   cdoInitialize(process);
 
   operatorInputArg("the sign of the exponent (-1 for normal or 1 for reverse transformation)!");
-  int sign = parameter2int(operatorArgv()[0]);
+  const int sign = parameter2int(operatorArgv()[0]);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
-  int nvars = vlistNvars(vlistID1);
+  VarList varList;
+  varListInit(varList, vlistID1);
+
+  const int nvars = vlistNvars(vlistID1);
   FieldVector3D vars;
   std::vector<int64_t> vdate;
   std::vector<int> vtime;
@@ -81,11 +84,10 @@ Fourier(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          int gridID = vlistInqVarGrid(vlistID1, varID);
-          size_t gridsize = gridInqSize(gridID);
-          vars[tsID][varID][levelID].ptr = (double *) Malloc(2 * gridsize * sizeof(double));
-          pstreamReadRecord(streamID1, vars[tsID][varID][levelID].ptr, &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          const size_t gridsize = varList[varID].gridsize;
+          vars[tsID][varID][levelID].resize(2 * gridsize);
+          cdoReadRecord(streamID1, vars[tsID][varID][levelID].vec.data(), &nmiss);
           vars[tsID][varID][levelID].nmiss = nmiss;
         }
 
@@ -94,44 +96,51 @@ Fourier(void *process)
 
   int nts = tsID;
 
-  bool lpower2 = ((nts & (nts - 1)) == 0);
+  const bool isPower2 = ((nts & (nts - 1)) == 0);
 
   std::vector<FourierMemory> ompmem(Threading::ompNumThreads);
   for (int i = 0; i < Threading::ompNumThreads; i++)
     {
       ompmem[i].real.resize(nts);
       ompmem[i].imag.resize(nts);
-      if (!lpower2) ompmem[i].work_r.resize(nts);
-      if (!lpower2) ompmem[i].work_i.resize(nts);
+      if (!isPower2) ompmem[i].work_r.resize(nts);
+      if (!isPower2) ompmem[i].work_i.resize(nts);
     }
 
   for (varID = 0; varID < nvars; varID++)
     {
-      int gridID = vlistInqVarGrid(vlistID1, varID);
-      double missval = vlistInqVarMissval(vlistID1, varID);
-      size_t gridsize = gridInqSize(gridID);
-      int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-      for (levelID = 0; levelID < nlevel; levelID++)
+      const double missval = varList[varID].missval;
+      const size_t gridsize = varList[varID].gridsize;
+      const int nlevels = varList[varID].nlevels;
+      for (levelID = 0; levelID < nlevels; levelID++)
         {
 #ifdef _OPENMP
 #pragma omp parallel for default(shared) private(tsID)
 #endif
           for (size_t i = 0; i < gridsize; i++)
             {
-              int lmiss = 0;
-              int ompthID = cdo_omp_get_thread_num();
+              bool hasMissvals = false;
+              const int ompthID = cdo_omp_get_thread_num();
 
               for (tsID = 0; tsID < nts; tsID++)
                 {
-                  ompmem[ompthID].real[tsID] = vars[tsID][varID][levelID].ptr[2 * i];
-                  ompmem[ompthID].imag[tsID] = vars[tsID][varID][levelID].ptr[2 * i + 1];
+                  ompmem[ompthID].real[tsID] = vars[tsID][varID][levelID].vec[2 * i];
+                  ompmem[ompthID].imag[tsID] = vars[tsID][varID][levelID].vec[2 * i + 1];
                   if (DBL_IS_EQUAL(ompmem[ompthID].real[tsID], missval) || DBL_IS_EQUAL(ompmem[ompthID].imag[tsID], missval))
-                    lmiss = 1;
+                    hasMissvals = true;
                 }
 
-              if (lmiss == 0)
+              if (hasMissvals)
                 {
-                  if (lpower2) /* nts is a power of 2 */
+                  for (tsID = 0; tsID < nts; tsID++)
+                    {
+                      vars[tsID][varID][levelID].vec[2 * i] = missval;
+                      vars[tsID][varID][levelID].vec[2 * i + 1] = missval;
+                    }
+                }
+              else
+                {
+                  if (isPower2) /* nts is a power of 2 */
                     fft(ompmem[ompthID].real.data(), ompmem[ompthID].imag.data(), nts, sign);
                   else
                     ft_r(ompmem[ompthID].real.data(), ompmem[ompthID].imag.data(), nts, sign, ompmem[ompthID].work_r.data(),
@@ -139,16 +148,8 @@ Fourier(void *process)
 
                   for (tsID = 0; tsID < nts; tsID++)
                     {
-                      vars[tsID][varID][levelID].ptr[2 * i] = ompmem[ompthID].real[tsID];
-                      vars[tsID][varID][levelID].ptr[2 * i + 1] = ompmem[ompthID].imag[tsID];
-                    }
-                }
-              else
-                {
-                  for (tsID = 0; tsID < nts; tsID++)
-                    {
-                      vars[tsID][varID][levelID].ptr[2 * i] = missval;
-                      vars[tsID][varID][levelID].ptr[2 * i + 1] = missval;
+                      vars[tsID][varID][levelID].vec[2 * i] = ompmem[ompthID].real[tsID];
+                      vars[tsID][varID][levelID].vec[2 * i + 1] = ompmem[ompthID].imag[tsID];
                     }
                 }
             }
@@ -159,25 +160,21 @@ Fourier(void *process)
     {
       taxisDefVdate(taxisID2, vdate[tsID]);
       taxisDefVtime(taxisID2, vtime[tsID]);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (varID = 0; varID < nvars; varID++)
         {
-          int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-          for (levelID = 0; levelID < nlevel; levelID++)
+          const int nlevels = varList[varID].nlevels;
+          for (levelID = 0; levelID < nlevels; levelID++)
             {
-              if (vars[tsID][varID][levelID].ptr)
+              if (!vars[tsID][varID][levelID].empty())
                 {
                   nmiss = vars[tsID][varID][levelID].nmiss;
-                  pstreamDefRecord(streamID2, varID, levelID);
-                  pstreamWriteRecord(streamID2, vars[tsID][varID][levelID].ptr, nmiss);
-                  Free(vars[tsID][varID][levelID].ptr);
-                  vars[tsID][varID][levelID].ptr = nullptr;
+                  cdoDefRecord(streamID2, varID, levelID);
+                  cdoWriteRecord(streamID2, vars[tsID][varID][levelID].vec.data(), nmiss);
                 }
             }
         }
-
-      fieldsFree(vlistID1, vars[tsID]);
     }
 
   cdoStreamClose(streamID2);
