@@ -23,7 +23,7 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
+#include "cdo_zaxis.h"
 
 void *
 Gengrid(void *process)
@@ -34,8 +34,8 @@ Gengrid(void *process)
 
   cdoInitialize(process);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-  int streamID2 = cdoStreamOpenRead(cdoStreamName(1));
+  CdoStreamID streamID1 = cdoOpenRead(0);
+  CdoStreamID streamID2 = cdoOpenRead(1);
 
   int vlistID1 = cdoStreamInqVlist(streamID1);
   int vlistID2 = cdoStreamInqVlist(streamID2);
@@ -56,10 +56,10 @@ Gengrid(void *process)
   cdoStreamInqTimestep(streamID1, 0);
   cdoStreamInqTimestep(streamID2, 0);
 
-  pstreamInqRecord(streamID1, &varID, &levelID);
-  pstreamReadRecord(streamID1, array1.data(), &nmiss1);
-  pstreamInqRecord(streamID2, &varID, &levelID);
-  pstreamReadRecord(streamID2, array2.data(), &nmiss2);
+  cdoInqRecord(streamID1, &varID, &levelID);
+  cdoReadRecord(streamID1, array1.data(), &nmiss1);
+  cdoInqRecord(streamID2, &varID, &levelID);
+  cdoReadRecord(streamID2, array2.data(), &nmiss2);
 
   int datatype = vlistInqVarDatatype(vlistID1, 0);
 
@@ -85,8 +85,8 @@ Gengrid(void *process)
 
   double xminval, xmaxval;
   double yminval, ymaxval;
-  arrayMinMax(gridsize, array1.data(), &xminval, &xmaxval);
-  arrayMinMax(gridsize, array2.data(), &yminval, &ymaxval);
+  arrayMinMax(gridsize, array1.data(), xminval, xmaxval);
+  arrayMinMax(gridsize, array2.data(), yminval, ymaxval);
 
   if (Options::cdoVerbose) cdoPrint("xminval = %g, xmaxval = %g, yminval = %g, ymaxval = %g", xminval, xmaxval, yminval, ymaxval);
 
@@ -105,7 +105,7 @@ Gengrid(void *process)
       cdoAbort("Units undefined!");
     }
 
-  int zaxisID3 = zaxisCreate(ZAXIS_SURFACE, 1);
+  int zaxisID3 = zaxisFromName("surface");
 
   int vlistID3 = vlistCreate();
   vlistDefVar(vlistID3, gridID3, zaxisID3, TIME_CONSTANT);
@@ -116,17 +116,17 @@ Gengrid(void *process)
   int taxisID3 = cdoTaxisCreate(TAXIS_ABSOLUTE);
   vlistDefTaxis(vlistID3, taxisID3);
 
-  int streamID3 = cdoStreamOpenWrite(cdoStreamName(2));
+  CdoStreamID streamID3 = cdoOpenWrite(2);
 
-  pstreamDefVlist(streamID3, vlistID3);
+  cdoDefVlist(streamID3, vlistID3);
 
   int tsID = 0;
-  pstreamDefTimestep(streamID3, tsID);
+  cdoDefTimestep(streamID3, tsID);
 
   for (size_t i = 0; i < gridsize; ++i) array3[i] = missval;
 
-  pstreamDefRecord(streamID3, 0, 0);
-  pstreamWriteRecord(streamID3, array3.data(), gridsize);
+  cdoDefRecord(streamID3, 0, 0);
+  cdoWriteRecord(streamID3, array3.data(), gridsize);
 
   cdoStreamClose(streamID3);
 
