@@ -29,8 +29,8 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "grid.h"
-#include "pstream_int.h"
+#include <mpim_grid.h>
+
 #include "util_fileextensions.h"
 #include "util_files.h"
 
@@ -982,8 +982,6 @@ Gradsdes(void *process)
   char *ctlfile = (char *) Malloc(len + 10);
   strcpy(ctlfile, datfile);
 
-  if (cdoGetStreamName(0)[0] == '-') cdoAbort("This operator does not work with pipes!");
-
   if (operatorID == DUMPMAP)
     {
       dumpmap();
@@ -1008,7 +1006,7 @@ Gradsdes(void *process)
   if (map_version == 4 && sizeof(off_t) != 8)
     cdoAbort("GrADS GRIB map version %d requires size of off_t to be 8! The size of off_t is %ld.", map_version, sizeof(off_t));
 
-  int streamID = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID = cdoOpenRead(0);
 
   int vlistID = cdoStreamInqVlist(streamID);
 
@@ -1016,8 +1014,8 @@ Gradsdes(void *process)
   int ntsteps = vlistNtsteps(vlistID);
   int ngrids = vlistNgrids(vlistID);
 
-  int filetype = pstreamInqFiletype(streamID);
-  int byteorder = pstreamInqByteorder(streamID);
+  int filetype = cdoInqFiletype(streamID);
+  int byteorder = cdoInqByteorder(streamID);
 
   if (filetype == CDI_FILETYPE_NC2 || filetype == CDI_FILETYPE_NC4 || filetype == CDI_FILETYPE_NC4C || filetype == CDI_FILETYPE_NC5)
     filetype = CDI_FILETYPE_NC;
@@ -1288,14 +1286,14 @@ Gradsdes(void *process)
 
           for (int recID = 0; recID < nrecs; recID++)
             {
-              pstreamInqRecord(streamID, &varID, &levelID);
+              cdoInqRecord(streamID, &varID, &levelID);
               if (vars[varID] == true)
                 {
-                  pstreamReadRecord(streamID, array.data(), &nmiss);
+                  cdoReadRecord(streamID, array.data(), &nmiss);
 
                   index = (tsID * nrecsout + recoffset[varID] + levelID);
 
-                  pstreamInqGRIBinfo(streamID, &intnum[index], &fltnum[index * 3], &bignum[index * 2]);
+                  cdoInqGRIBinfo(streamID, &intnum[index], &fltnum[index * 3], &bignum[index * 2]);
 
                   if (map_version != 4)
                     {
