@@ -34,8 +34,7 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
-#include "grid.h"
+#include <mpim_grid.h>
 #include "cdo_options.h"
 #include "progress.h"
 #include "cimdOmp.h"
@@ -45,12 +44,12 @@ genBoxGridReg2D(int gridID1, size_t xinc, size_t yinc, int gridID2)
 {
   size_t i, j, i1;
 
-  size_t nlon1 = gridInqXsize(gridID1);
-  size_t nlat1 = gridInqYsize(gridID1);
-  size_t nlon2 = gridInqXsize(gridID2);
-  size_t nlat2 = gridInqYsize(gridID2);
+  const size_t nlon1 = gridInqXsize(gridID1);
+  const size_t nlat1 = gridInqYsize(gridID1);
+  const size_t nlon2 = gridInqXsize(gridID2);
+  const size_t nlat2 = gridInqYsize(gridID2);
 
-  bool gridHasBounds = (gridInqXbounds(gridID1, nullptr) && gridInqYbounds(gridID1, nullptr));
+  const bool gridHasBounds = (gridInqXbounds(gridID1, nullptr) && gridInqYbounds(gridID1, nullptr));
 
   {
     std::vector<double> xvals1(nlon1);
@@ -120,18 +119,18 @@ genBoxGridReg2D(int gridID1, size_t xinc, size_t yinc, int gridID2)
 static void
 genBoxGridCurv2D(int gridID1, size_t xinc, size_t yinc, int gridID2)
 {
-  size_t nlon1 = gridInqXsize(gridID1);
-  size_t nlat1 = gridInqYsize(gridID1);
-  size_t nlon2 = gridInqXsize(gridID2);
-  size_t nlat2 = gridInqYsize(gridID2);
-  size_t gridsize1 = gridInqSize(gridID1);
-  size_t gridsize2 = gridInqSize(gridID2);
+  const size_t nlon1 = gridInqXsize(gridID1);
+  const size_t nlat1 = gridInqYsize(gridID1);
+  const size_t nlon2 = gridInqXsize(gridID2);
+  const size_t nlat2 = gridInqYsize(gridID2);
+  const size_t gridsize1 = gridInqSize(gridID1);
+  const size_t gridsize2 = gridInqSize(gridID2);
 
   size_t x1, y1, x2, y2;
   size_t use_x1, use_y1;
   size_t g1_add, g2_add;
   int corner, add;
-  int circular = gridIsCircular(gridID1);
+  const int circular = gridIsCircular(gridID1);
   double on_up, on_lo, ox_up, ox_lo, an_le, an_ri, ax_le, ax_ri;
   double xvals2_0 = 0;
   double area_norm;
@@ -400,7 +399,7 @@ genBoxGrid(int gridID1, size_t xinc, size_t yinc)
   if (xinc < 1 || yinc < 1) cdoAbort("xinc and yinc must not be smaller than 1!");
 
   int gridID2 = -1;
-  int gridtype = gridInqType(gridID1);
+  const int gridtype = gridInqType(gridID1);
   if (gridtype == GRID_GAUSSIAN || gridtype == GRID_LONLAT || gridtype == GRID_CURVILINEAR || gridtype == GRID_GENERIC)
     {
       const size_t nlon1 = gridInqXsize(gridID1);
@@ -442,26 +441,21 @@ genBoxGrid(int gridID1, size_t xinc, size_t yinc)
 }
 
 static void
-gridboxstat(Field *field1, Field *field2, const size_t xinc, const size_t yinc, const int statfunc)
+gridboxstat(const Field &field1, Field &field2, const size_t xinc, const size_t yinc, const int statfunc)
 {
-  const bool useWeight = (field1->weight != nullptr);
+  const bool useWeight = (!field1.weightv.empty());
 
   const size_t gridsize = xinc * yinc;
-  Field *field = (Field *) Malloc(Threading::ompNumThreads * sizeof(Field));
+  FieldVector field(Threading::ompNumThreads);
   for (int i = 0; i < Threading::ompNumThreads; i++)
     {
-      field[i].size = gridsize;
-      field[i].ptr = (double *) Malloc(gridsize * sizeof(double));
-      field[i].weight = (useWeight) ? (double *) Malloc(gridsize * sizeof(double)) : nullptr;
-      field[i].missval = field1->missval;
-      field[i].nmiss = 0;
+      field[i].resize(gridsize);
+      if (useWeight) field[i].weightv.resize(gridsize);
+      field[i].missval = field1.missval;
     }
 
-  const int gridID1 = field1->grid;
-  const int gridID2 = field2->grid;
-  const double *array1 = field1->ptr;
-  double *array2 = field2->ptr;
-  const double missval = field1->missval;
+  const int gridID1 = field1.grid;
+  const int gridID2 = field2.grid;
 
   const size_t nlon1 = gridInqXsize(gridID1);
   const size_t nlat1 = gridInqYsize(gridID1);
@@ -490,26 +484,18 @@ gridboxstat(Field *field1, Field *field2, const size_t xinc, const size_t yinc,
               const size_t ii = ilon * xinc + i;
               const size_t index = jj * nlon1 + ii;
               if (ii >= nlon1) break;
-              field[ompthID].ptr[isize] = array1[index];
-              if (useWeight) field[ompthID].weight[isize] = field1->weight[index];
-              if (DBL_IS_EQUAL(field[ompthID].ptr[isize], field[ompthID].missval)) field[ompthID].nmiss++;
+              field[ompthID].vec[isize] = field1.vec[index];
+              if (useWeight) field[ompthID].weightv[isize] = field1.weightv[index];
+              if (DBL_IS_EQUAL(field[ompthID].vec[isize], field[ompthID].missval)) field[ompthID].nmiss++;
               isize++;
             }
         }
 
       field[ompthID].size = isize;
-      field2->ptr[ig] = fldfun(field[ompthID], statfunc);
+      field2.vec[ig] = vfldfun(field[ompthID], statfunc);
     }
 
-  field2->nmiss = arrayNumMV(nlat2 * nlon2, array2, missval);
-
-  for (int i = 0; i < Threading::ompNumThreads; i++)
-    {
-      if (field[i].ptr) Free(field[i].ptr);
-      if (field[i].weight) Free(field[i].weight);
-    }
-
-  if (field) Free(field);
+  field2.nmiss = fieldNumMiss(field2);
 }
 
 void *
@@ -545,7 +531,7 @@ Gridboxstat(void *process)
   const int operfunc = cdoOperatorF1(operatorID);
   const bool needWeights = cdoOperatorF2(operatorID) != 0;
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -564,33 +550,28 @@ Gridboxstat(void *process)
   const int gridID2 = genBoxGrid(gridID1, xinc, yinc);
   for (int index = 0; index < ngrids; index++) vlistChangeGridIndex(vlistID2, index, gridID2);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
-
-  Field field1, field2;
-  fieldInit(field1);
-  fieldInit(field2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   const size_t gridsize1 = gridInqSize(gridID1);
-  field1.ptr = (double *) Malloc(gridsize1 * sizeof(double));
-  field1.weight = needWeights ? (double *) Malloc(gridsize1 * sizeof(double)) : nullptr;
+
+  Field field1, field2;
+  field1.resize(gridsize1);
+  if (needWeights) field1.weightv.resize(gridsize1);
 
   const size_t gridsize2 = gridInqSize(gridID2);
-  field2.ptr = (double *) Malloc(gridsize2 * sizeof(double));
-  field2.weight = nullptr;
+  field2.resize(gridsize2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          size_t nmiss;
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, field1.ptr, &nmiss);
-          field1.nmiss = nmiss;
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, field1.vec.data(), &field1.nmiss);
 
           field1.grid = vlistInqVarGrid(vlistID1, varID);
           field1.size = gridInqSize(field1.grid);
@@ -603,7 +584,7 @@ Gridboxstat(void *process)
           if (needWeights && field1.grid != lastgrid)
             {
               lastgrid = field1.grid;
-              wstatus = gridWeights(field1.grid, field1.weight);
+              wstatus = gridWeights(field1.grid, field1.weightv.data());
             }
           if (wstatus != 0 && tsID == 0 && levelID == 0)
             {
@@ -611,10 +592,10 @@ Gridboxstat(void *process)
               cdoWarning("Grid cell bounds not available, using constant grid cell area weights for variable %s!", varname);
             }
 
-          gridboxstat(&field1, &field2, xinc, yinc, operfunc);
+          gridboxstat(field1, field2, xinc, yinc, operfunc);
 
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, field2.ptr, field2.nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, field2.vec.data(), field2.nmiss);
         }
       tsID++;
     }
@@ -622,12 +603,6 @@ Gridboxstat(void *process)
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
-  if (field1.ptr) Free(field1.ptr);
-  if (field1.weight) Free(field1.weight);
-
-  if (field2.ptr) Free(field2.ptr);
-  if (field2.weight) Free(field2.weight);
-
   cdoFinish();
 
   return nullptr;
