@@ -26,8 +26,8 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
-#include "grid.h"
+#include "cdo_zaxis.h"
+#include <mpim_grid.h>
 #include "constants.h"
 
 static inline double
@@ -36,6 +36,32 @@ orthodrome(double px1, double py1, double px2, double py2)
   return acos(sin(py1) * sin(py2) + cos(py1) * cos(py2) * cos(px2 - px1));
 }
 
+double
+gridGetPlanetRadius(int gridID);
+
+double
+getPlanetRadius(int gridID)
+{
+  static bool radiusFromEnv = true;
+  if (radiusFromEnv)
+    {
+      radiusFromEnv = false;
+      char *envstr = getenv("PLANET_RADIUS");
+      if (envstr)
+        {
+          const double fval = atof(envstr);
+          if (fval > 0) PlanetRadius = fval;
+        }
+    }
+
+  double planetRadius = PlanetRadius;
+
+  double gridRadius = gridGetPlanetRadius(gridID);
+  if (gridRadius > 1) planetRadius = gridRadius;
+
+  return planetRadius;
+}
+  
 void
 grid_cell_area(int gridID, double *array)
 {
@@ -44,7 +70,7 @@ grid_cell_area(int gridID, double *array)
 
   if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || projtype == CDI_PROJ_RLL || projtype == CDI_PROJ_LAEA
       || projtype == CDI_PROJ_SINU || projtype == CDI_PROJ_LCC || gridtype == GRID_GME || gridtype == GRID_CURVILINEAR
-      || gridtype == GRID_UNSTRUCTURED)
+      || gridtype == GRID_UNSTRUCTURED || gridtype == GRID_GAUSSIAN_REDUCED)
     {
       if (gridHasArea(gridID))
         {
@@ -59,8 +85,10 @@ grid_cell_area(int gridID, double *array)
           else if (status == 2)
             cdoAbort("%s: Can't compute grid cell area for this grid!", __func__);
 
-          size_t ngp = gridInqSize(gridID);
-          for (size_t i = 0; i < ngp; ++i) array[i] *= PlanetRadius * PlanetRadius;
+          const double planetRadius = getPlanetRadius(gridID);
+          if (Options::cdoVerbose) cdoPrint("Planet radius: %.2f", planetRadius);
+          const size_t ngp = gridInqSize(gridID);
+          for (size_t i = 0; i < ngp; ++i) array[i] *= planetRadius * planetRadius;
         }
     }
   else
@@ -88,20 +116,9 @@ Gridcell(void *process)
 
   const int operatorID = cdoOperatorID();
 
-  const bool needRadius = cdoOperatorF1(operatorID) > 0;
-  if (needRadius)
-    {
-      char *envstr = getenv("PLANET_RADIUS");
-      if (envstr)
-        {
-          const double fval = atof(envstr);
-          if (fval > 0) PlanetRadius = fval;
-        }
-    }
+  //const bool needRadius = cdoOperatorF1(operatorID) > 0;
 
-  if (Options::cdoVerbose) cdoPrint("PlanetRadius: %g", PlanetRadius);
-
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
   const int vlistID1 = cdoStreamInqVlist(streamID1);
 
   const int ngrids = vlistNgrids(vlistID1);
@@ -109,7 +126,7 @@ Gridcell(void *process)
   if (ngrids > 1) cdoWarning("Found more than 1 grid, using the first one!");
 
   int gridID = vlistGrid(vlistID1, 0);
-  const int zaxisID = zaxisCreate(ZAXIS_SURFACE, 1);
+  const int zaxisID = zaxisFromName("surface");
 
   const int vlistID2 = vlistCreate();
   int varID = vlistDefVar(vlistID2, gridID, zaxisID, TIME_CONSTANT);
@@ -189,7 +206,8 @@ Gridcell(void *process)
     {
       const int gridtype = gridInqType(gridID);
       const int projtype = (gridtype == GRID_PROJECTION) ? gridInqProjType(gridID) : -1;
-      if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || projtype == CDI_PROJ_LCC || projtype == CDI_PROJ_STERE || gridtype == GRID_CURVILINEAR)
+      if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || projtype == CDI_PROJ_LCC || projtype == CDI_PROJ_STERE
+          || gridtype == GRID_CURVILINEAR)
         {
           double len1 = 0, len2 = 0;
           char units[CDI_MAX_NAME];
@@ -212,6 +230,9 @@ Gridcell(void *process)
           grid_to_radian(units, gridsize, &xv[0], "grid longitudes");
           grid_to_radian(units, gridsize, &yv[0], "grid latitudes");
 
+          const double planetRadius = getPlanetRadius(gridID);
+          if (Options::cdoVerbose) cdoPrint("Planet radius: %.2f", planetRadius);
+
           if (operatorID == GRIDDX)
             {
               for (size_t j = 0; j < ysize; ++j)
@@ -233,7 +254,7 @@ Gridcell(void *process)
                         len2 = orthodrome(xv[j * xsize + i], yv[j * xsize + i], xv[j * xsize + i + 1], yv[j * xsize + i + 1]);
                       }
 
-                    array[j * xsize + i] = 0.5 * (len1 + len2) * PlanetRadius;
+                    array[j * xsize + i] = 0.5 * (len1 + len2) * planetRadius;
                   }
             }
           else
@@ -257,7 +278,7 @@ Gridcell(void *process)
                         len2 = orthodrome(xv[j * xsize + i], yv[j * xsize + i], xv[(j + 1) * xsize + i], yv[(j + 1) * xsize + i]);
                       }
 
-                    array[j * xsize + i] = 0.5 * (len1 + len2) * PlanetRadius;
+                    array[j * xsize + i] = 0.5 * (len1 + len2) * planetRadius;
                   }
             }
         }
@@ -270,11 +291,11 @@ Gridcell(void *process)
         }
     }
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
-  pstreamDefTimestep(streamID2, 0);
-  pstreamDefRecord(streamID2, 0, 0);
-  pstreamWriteRecord(streamID2, &array[0], 0);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
+  cdoDefTimestep(streamID2, 0);
+  cdoDefRecord(streamID2, 0, 0);
+  cdoWriteRecord(streamID2, &array[0], 0);
 
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
