@@ -17,7 +17,8 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "grid.h"
+#include <mpim_grid.h>
+#include "griddes.h"
 
 struct grid_type
 {
