@@ -24,7 +24,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 
 static const char HI_NAME[] = "hum_index";
 static const char HI_LONGNAME[] = "Humindex describes empirically in units of temperature how the "
@@ -49,45 +48,37 @@ humidityIndex(double t, double e, double r, double missval)
 }
 
 static void
-farexpr(Field *field1, Field field2, Field field3, double (*expression)(double, double, double, double))
+farexpr(Field &field1, const Field &field2, const Field &field3, double (*expression)(double, double, double, double))
 {
-  const int grid1 = field1->grid;
-  const size_t nmiss1 = field1->nmiss;
-  const double missval1 = field1->missval;
-  double *array1 = field1->ptr;
+  const int grid1 = field1.grid;
+  const double missval1 = field1.missval;
   const int grid2 = field2.grid;
-  const size_t nmiss2 = field2.nmiss;
   const double missval2 = field2.missval;
-  const double *array2 = field2.ptr;
   const int grid3 = field3.grid;
-  const size_t nmiss3 = field3.nmiss;
   const double missval3 = field3.missval;
-  const double *array3 = field3.ptr;
-
-  size_t len = gridInqSize(grid1);
 
+  const size_t len = gridInqSize(grid1);
   if (len != gridInqSize(grid2) || len != gridInqSize(grid3)) cdoAbort("Fields have different gridsize (%s)", __func__);
 
-  if (nmiss1 > 0 || nmiss2 > 0 || nmiss3 > 0)
+  if (field1.nmiss || field2.nmiss || field3.nmiss)
     {
       for (size_t i = 0; i < len; i++)
-        if (DBL_IS_EQUAL(array1[i], missval1) || DBL_IS_EQUAL(array2[i], missval2) || DBL_IS_EQUAL(array3[i], missval3))
-          array1[i] = missval1;
+        if (DBL_IS_EQUAL(field1.vec[i], missval1) || DBL_IS_EQUAL(field2.vec[i], missval2) || DBL_IS_EQUAL(field3.vec[i], missval3))
+          field1.vec[i] = missval1;
         else
-          array1[i] = expression(array1[i], array2[i], array3[i], missval1);
+          field1.vec[i] = expression(field1.vec[i], field2.vec[i], field3.vec[i], missval1);
     }
   else
     {
-      for (size_t i = 0; i < len; i++) array1[i] = expression(array1[i], array2[i], array3[i], missval1);
+      for (size_t i = 0; i < len; i++) field1.vec[i] = expression(field1.vec[i], field2.vec[i], field3.vec[i], missval1);
     }
 
-  field1->nmiss = arrayNumMV(len, array1, missval1);
+  field1.nmiss = fieldNumMiss(field1);
 }
 
 void *
 Hi(void *process)
 {
-  size_t nmiss;
   int nrecs;
   int varID1, varID2, varID3;
   int levelID1, levelID2, levelID3;
@@ -95,41 +86,38 @@ Hi(void *process)
   cdoInitialize(process);
   cdoOperatorAdd("hi", 0, 0, nullptr);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-  int streamID2 = cdoStreamOpenRead(cdoStreamName(1));
-  int streamID3 = cdoStreamOpenRead(cdoStreamName(2));
+  CdoStreamID streamID1 = cdoOpenRead(0);
+  CdoStreamID streamID2 = cdoOpenRead(1);
+  CdoStreamID streamID3 = cdoOpenRead(2);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = cdoStreamInqVlist(streamID2);
-  int vlistID3 = cdoStreamInqVlist(streamID3);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = cdoStreamInqVlist(streamID2);
+  const int vlistID3 = cdoStreamInqVlist(streamID3);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
   // taxisID2 = vlistInqTaxis(vlistID2);
   // taxisID3 = vlistInqTaxis(vlistID3);
 
   vlistCompare(vlistID1, vlistID2, CMP_DIM);
   vlistCompare(vlistID1, vlistID3, CMP_DIM);
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
+  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field1, field2, field3;
-  fieldInit(field1);
-  fieldInit(field2);
-  fieldInit(field3);
-  field1.ptr = (double *) Malloc(gridsize * sizeof(double));
-  field2.ptr = (double *) Malloc(gridsize * sizeof(double));
-  field3.ptr = (double *) Malloc(gridsize * sizeof(double));
+  field1.resize(gridsizemax);
+  field2.resize(gridsizemax);
+  field3.resize(gridsizemax);
 
   if (Options::cdoVerbose)
     cdoPrint("Number of timesteps: file1 %d, file2 %d, file3 %d", vlistNtsteps(vlistID1), vlistNtsteps(vlistID2),
              vlistNtsteps(vlistID3));
 
-  int vlistID4 = vlistCreate();
-  int gridID = vlistInqVarGrid(vlistID1, FIRST_VAR);
-  int zaxisID = vlistInqVarZaxis(vlistID1, FIRST_VAR);
-  int varID4 = vlistDefVar(vlistID4, gridID, zaxisID, TIME_VARYING);
+  const int vlistID4 = vlistCreate();
+  const int gridID = vlistInqVarGrid(vlistID1, FIRST_VAR);
+  const int zaxisID = vlistInqVarZaxis(vlistID1, FIRST_VAR);
+  const int varID4 = vlistDefVar(vlistID4, gridID, zaxisID, TIME_VARYING);
 
-  int taxisID4 = cdoTaxisCreate(TAXIS_RELATIVE);
+  const int taxisID4 = cdoTaxisCreate(TAXIS_RELATIVE);
   taxisDefTunit(taxisID4, TUNIT_MINUTE);
   taxisDefCalendar(taxisID4, CALENDAR_STANDARD);
   taxisDefRdate(taxisID4, 19550101);
@@ -140,33 +128,30 @@ Hi(void *process)
   vlistDefVarLongname(vlistID4, varID4, HI_LONGNAME);
   vlistDefVarUnits(vlistID4, varID4, HI_UNITS);
 
-  int streamID4 = cdoStreamOpenWrite(cdoStreamName(3));
+  CdoStreamID streamID4 = cdoOpenWrite(3);
 
-  pstreamDefVlist(streamID4, vlistID4);
+  cdoDefVlist(streamID4, vlistID4);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
-      int nrecs2 = cdoStreamInqTimestep(streamID2, tsID);
-      int nrecs3 = cdoStreamInqTimestep(streamID3, tsID);
+      const int nrecs2 = cdoStreamInqTimestep(streamID2, tsID);
+      const int nrecs3 = cdoStreamInqTimestep(streamID3, tsID);
       if (nrecs2 == 0 || nrecs3 == 0) cdoAbort("Input streams have different number of timesteps!");
 
       taxisCopyTimestep(taxisID4, taxisID1);
-      pstreamDefTimestep(streamID4, tsID);
+      cdoDefTimestep(streamID4, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID1, &levelID1);
-          pstreamReadRecord(streamID1, field1.ptr, &nmiss);
-          field1.nmiss = nmiss;
+          cdoInqRecord(streamID1, &varID1, &levelID1);
+          cdoReadRecord(streamID1, field1.vec.data(), &field1.nmiss);
 
-          pstreamInqRecord(streamID2, &varID2, &levelID2);
-          pstreamReadRecord(streamID2, field2.ptr, &nmiss);
-          field2.nmiss = nmiss;
+          cdoInqRecord(streamID2, &varID2, &levelID2);
+          cdoReadRecord(streamID2, field2.vec.data(), &field2.nmiss);
 
-          pstreamInqRecord(streamID3, &varID3, &levelID3);
-          pstreamReadRecord(streamID3, field3.ptr, &nmiss);
-          field3.nmiss = nmiss;
+          cdoInqRecord(streamID3, &varID3, &levelID3);
+          cdoReadRecord(streamID3, field3.vec.data(), &field3.nmiss);
 
           if (varID1 != varID2 || varID1 != varID3 || levelID1 != levelID2 || levelID1 != levelID3)
             cdoAbort("Input streams have different structure!");
@@ -182,10 +167,10 @@ Hi(void *process)
           field3.grid = vlistInqVarGrid(vlistID3, varID3);
           field3.missval = vlistInqVarMissval(vlistID3, varID3);
 
-          farexpr(&field1, field2, field3, humidityIndex);
+          farexpr(field1, field2, field3, humidityIndex);
 
-          pstreamDefRecord(streamID4, varID4, levelID1);
-          pstreamWriteRecord(streamID4, field1.ptr, field1.nmiss);
+          cdoDefRecord(streamID4, varID4, levelID1);
+          cdoWriteRecord(streamID4, field1.vec.data(), field1.nmiss);
         }
 
       tsID++;
@@ -196,10 +181,6 @@ Hi(void *process)
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
-  if (field1.ptr) Free(field1.ptr);
-  if (field2.ptr) Free(field2.ptr);
-  if (field3.ptr) Free(field3.ptr);
-
   cdoFinish();
 
   return nullptr;
