@@ -23,7 +23,7 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
+
 #include "listarray.h"
 
 void *
@@ -58,7 +58,7 @@ Histogram(void *process)
       for (int i = 0; i < nbins; i++) printf("flt %d = %g\n", i + 1, fltarr[i]);
     }
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   int vlistID1 = cdoStreamInqVlist(streamID1);
   int taxisID1 = vlistInqTaxis(vlistID1);
@@ -91,12 +91,12 @@ Histogram(void *process)
       vlistChangeZaxisIndex(vlistID2, index, zaxisID2);
     }
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
+  CdoStreamID streamID2 = cdoOpenWrite(1);
 
   int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  pstreamDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID2, vlistID2);
 
   int nvars = vlistNvars(vlistID2);
   std::vector<std::vector<double>> vardata(nvars);
@@ -120,8 +120,8 @@ Histogram(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, array.data(), &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, array.data(), &nmiss);
           double missval = vlistInqVarMissval(vlistID1, varID);
 
           size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
@@ -155,7 +155,7 @@ Histogram(void *process)
       tsID1++;
     }
 
-  pstreamDefTimestep(streamID2, 0);
+  cdoDefTimestep(streamID2, 0);
 
   for (varID = 0; varID < nvars; varID++)
     {
@@ -192,12 +192,12 @@ Histogram(void *process)
                 }
             }
 
-          pstreamDefRecord(streamID2, varID, index);
+          cdoDefRecord(streamID2, varID, index);
 
           if (operatorID == HISTCOUNT)
-            pstreamWriteRecord(streamID2, &varcount[varID][offset], nmiss);
+            cdoWriteRecord(streamID2, &varcount[varID][offset], nmiss);
           else
-            pstreamWriteRecord(streamID2, &vardata[varID][offset], nmiss);
+            cdoWriteRecord(streamID2, &vardata[varID][offset], nmiss);
         }
     }
 
