@@ -18,7 +18,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 
 #define NLON 1440
 #define NLAT 720
@@ -45,13 +44,12 @@ init_amsr_day(int vlistID, int gridID, int zaxisID, int nvars)
   */
   const char *name[] = { "hours", "sst", "wind", "vapor", "cloud", "rain" };
   const char *units[] = { "h", "deg Celcius", "m/s", "mm", "mm", "mm/h" };
-  double xscale[] = { 0.1, 0.15, 0.2, 0.3, 0.01, 0.1 };
-  double xminval[] = { 0., -3., 0., 0., 0., 0. };
-  int i, varID;
+  const double xscale[] = { 0.1, 0.15, 0.2, 0.3, 0.01, 0.1 };
+  const double xminval[] = { 0., -3., 0., 0., 0., 0. };
 
-  for (i = 0; i < nvars; ++i)
+  for (int i = 0; i < nvars; ++i)
     {
-      varID = vlistDefVar(vlistID, gridID, zaxisID, TIME_VARYING);
+      const int varID = vlistDefVar(vlistID, gridID, zaxisID, TIME_VARYING);
       vlistDefVarName(vlistID, varID, name[i]);
       vlistDefVarUnits(vlistID, varID, units[i]);
       vlistDefVarDatatype(vlistID, varID, CDI_DATATYPE_INT16);
@@ -90,14 +88,13 @@ init_amsr_averaged(int vlistID, int gridID, int zaxisID, int nvars)
   */
   const char *name[] = { "sst", "wind", "vapor", "cloud", "rain" };
   const char *units[] = { "deg Celcius", "m/s", "mm", "mm", "mm/h" };
-  double xscale[] = { 0.15, 0.2, 0.3, 0.01, 0.1 };
-  double xminval[] = { -3., 0., 0., 0., 0. };
-  int i, varID;
+  const double xscale[] = { 0.15, 0.2, 0.3, 0.01, 0.1 };
+  const double xminval[] = { -3., 0., 0., 0., 0. };
 
-  for (i = 0; i < nvars; ++i)
+  for (int i = 0; i < nvars; ++i)
     {
       /* varID = vlistDefVar(vlistID, gridID, zaxisID, TIME_CONSTANT); */
-      varID = vlistDefVar(vlistID, gridID, zaxisID, TIME_VARYING);
+      const int varID = vlistDefVar(vlistID, gridID, zaxisID, TIME_VARYING);
       vlistDefVarName(vlistID, varID, name[i]);
       vlistDefVarUnits(vlistID, varID, units[i]);
       vlistDefVarDatatype(vlistID, varID, CDI_DATATYPE_INT16);
@@ -108,18 +105,18 @@ init_amsr_averaged(int vlistID, int gridID, int zaxisID, int nvars)
 }
 
 static void
-read_amsr(FILE *fp, int vlistID, int nvars, double *data[], size_t *nmiss)
+read_amsr(FILE *fp, int vlistID, int nvars, std::vector<std::vector<double>> &data, size_t *nmiss)
 {
   int varID, i, gridsize;
-  unsigned char *amsr_data = nullptr;
+  std::vector<unsigned char> amsr_data;
   double xminval, xscale, missval;
   size_t size;
 
   for (varID = 0; varID < nvars; ++varID)
     {
       gridsize = gridInqSize(vlistInqVarGrid(vlistID, varID));
-      amsr_data = (unsigned char *) Realloc(amsr_data, gridsize);
-      size = fread(amsr_data, 1, gridsize, fp);
+      amsr_data.resize(gridsize);
+      size = fread(amsr_data.data(), 1, gridsize, fp);
       if ((int) size != gridsize) cdoAbort("Read error!");
 
       missval = vlistInqVarMissval(vlistID, varID);
@@ -140,17 +137,15 @@ read_amsr(FILE *fp, int vlistID, int nvars, double *data[], size_t *nmiss)
             }
         }
     }
-
-  Free(amsr_data);
 }
 
 static void
-write_data(int streamID, int nvars, double *data[], size_t *nmiss)
+write_data(CdoStreamID streamID, int nvars, std::vector<std::vector<double>> &data, size_t *nmiss)
 {
   for (int varID = 0; varID < nvars; ++varID)
     {
-      pstreamDefRecord(streamID, varID, 0);
-      pstreamWriteRecord(streamID, data[varID], nmiss[varID]);
+      cdoDefRecord(streamID, varID, 0);
+      cdoWriteRecord(streamID, data[varID].data(), nmiss[varID]);
     }
 }
 
@@ -158,7 +153,7 @@ static int
 getDate(const char *name)
 {
   const char *pname = strchr(name, '_');
-  int date = pname ? atoi(pname + 1) : 0;
+  const int date = pname ? atoi(pname + 1) : 0;
   return date;
 }
 
@@ -169,7 +164,7 @@ Importamsr(void *process)
   int nvars;
   int vtime = 0;
   double xvals[NLON], yvals[NLAT];
-  double *data[MAX_VARS];
+  std::vector<std::vector<double>> data(MAX_VARS);
   size_t nmiss[MAX_VARS];
 
   cdoInitialize(process);
@@ -182,20 +177,20 @@ Importamsr(void *process)
     }
 
   fseek(fp, 0L, SEEK_END);
-  size_t fsize = (size_t) ftell(fp);
+  const size_t fsize = (size_t) ftell(fp);
   fseek(fp, 0L, SEEK_SET);
 
   int64_t vdate = getDate(cdoGetStreamName(0));
   if (vdate <= 999999) vdate = vdate * 100 + 1;
 
-  int streamID = cdoStreamOpenWrite(cdoStreamName(1));
+  CdoStreamID streamID = cdoOpenWrite(1);
 
   /*
     Longitude  is 0.25*xdim-0.125    degrees east
     Latitude   is 0.25*ydim-90.125
   */
-  size_t gridsize = NLON * NLAT;
-  int gridID = gridCreate(GRID_LONLAT, gridsize);
+  const size_t gridsize = NLON * NLAT;
+  const int gridID = gridCreate(GRID_LONLAT, gridsize);
   gridDefXsize(gridID, NLON);
   gridDefYsize(gridID, NLAT);
   for (int i = 0; i < NLON; ++i) xvals[i] = 0.25 * (i + 1) - 0.125;
@@ -203,20 +198,20 @@ Importamsr(void *process)
   gridDefXvals(gridID, xvals);
   gridDefYvals(gridID, yvals);
 
-  int zaxisID = zaxisCreate(ZAXIS_SURFACE, 1);
+  const int zaxisID = zaxisCreate(ZAXIS_SURFACE, 1);
 
-  int vlistID = vlistCreate();
-  int taxisID = cdoTaxisCreate(TAXIS_ABSOLUTE);
+  const int vlistID = vlistCreate();
+  const int taxisID = cdoTaxisCreate(TAXIS_ABSOLUTE);
   vlistDefTaxis(vlistID, taxisID);
 
   if (fsize == 12441600)
     {
       nvars = 6;
-      for (int i = 0; i < nvars; ++i) data[i] = (double *) Malloc(gridsize * sizeof(double));
+      for (int i = 0; i < nvars; ++i) data[i].resize(gridsize);
 
       init_amsr_day(vlistID, gridID, zaxisID, nvars);
 
-      pstreamDefVlist(streamID, vlistID);
+      cdoDefVlist(streamID, vlistID);
 
       vtime = 13000; /* 1:30:00 */
       for (tsID = 0; tsID < 2; ++tsID)
@@ -224,35 +219,31 @@ Importamsr(void *process)
           taxisDefVdate(taxisID, vdate);
           taxisDefVtime(taxisID, vtime);
           vtime += 120000; /* 13:30:00 */
-          pstreamDefTimestep(streamID, tsID);
+          cdoDefTimestep(streamID, tsID);
 
           read_amsr(fp, vlistID, nvars, data, nmiss);
 
           write_data(streamID, nvars, data, nmiss);
         }
-
-      for (int i = 0; i < nvars; ++i) Free(data[i]);
     }
   else if (fsize == 5184000)
     {
       nvars = 5;
-      for (int i = 0; i < nvars; ++i) data[i] = (double *) Malloc(gridsize * sizeof(double));
+      for (int i = 0; i < nvars; ++i) data[i].resize(gridsize);
 
       init_amsr_averaged(vlistID, gridID, zaxisID, nvars);
 
       /* vlistDefNtsteps(vlistID, 0);*/
-      pstreamDefVlist(streamID, vlistID);
+      cdoDefVlist(streamID, vlistID);
 
       taxisDefVdate(taxisID, vdate);
       taxisDefVtime(taxisID, vtime);
       tsID = 0;
-      pstreamDefTimestep(streamID, tsID);
+      cdoDefTimestep(streamID, tsID);
 
       read_amsr(fp, vlistID, nvars, data, nmiss);
 
       write_data(streamID, nvars, data, nmiss);
-
-      for (int i = 0; i < nvars; ++i) Free(data[i]);
     }
   else
     cdoAbort("Unexpected file size for AMSR data!");
