@@ -20,10 +20,12 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 #include "printinfo.h"
 
-#include "gradsdeslib.h"
+extern "C"
+{
+#include "lib/gradsdes/gradsdes.h"
+}
 
 static void
 get_dim_vals(dsets_t *pfi, double *vals, int dimlen, int dim)
@@ -81,7 +83,7 @@ y_is_gauss(std::vector<double> &gridyvals, int ysize)
     {
       std::vector<double> yvals(ysize);
       std::vector<double> yw(ysize);
-      gaussaw(yvals.data(), yw.data(), (size_t) ysize);
+      gaussianLatitudes(yvals.data(), yw.data(), (size_t) ysize);
       for (i = 0; i < ysize; i++) yvals[i] = asin(yvals[i]) / M_PI * 180.0;
 
       for (i = 0; i < ysize; i++)
@@ -218,7 +220,7 @@ Importbinary(void *process)
 
   int *var_zaxisID = (int *) Malloc(nvars * sizeof(int));
   int *var_dfrm = (int *) Malloc(nrecs * sizeof(int));
-  std::vector<RecordInfo> recinfo(nrecs);
+  std::vector<RecordInfo> recList(nrecs);
 
   int recID = 0;
   for (ivar = 0; ivar < nvars; ++ivar)
@@ -304,8 +306,8 @@ Importbinary(void *process)
       for (levelID = 0; levelID < nlevels; ++levelID)
         {
           if (recID >= nrecs) cdoAbort("Internal problem with number of records!");
-          recinfo[recID].varID = varID;
-          recinfo[recID].levelID = levelID;
+          recList[recID].varID = varID;
+          recList[recID].levelID = levelID;
           var_dfrm[recID] = pvar->dfrm;
           recID++;
         }
@@ -318,8 +320,8 @@ Importbinary(void *process)
   taxisDefCalendar(taxisID, calendar);
   vlistDefTaxis(vlistID, taxisID);
 
-  int streamID = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID, vlistID);
+  CdoStreamID streamID = cdoOpenWrite(1);
+  cdoDefVlist(streamID, vlistID);
 
   gridsize = pfi.dnum[0] * pfi.dnum[1];
   if (pfi.flt64)
@@ -419,7 +421,7 @@ Importbinary(void *process)
               cdoAbort("Could not open file: %s", ch);
             }
         }
-      if (pfi.tmplat) gree(ch, "312");
+      // if (pfi.tmplat) gree(ch, "312"); //US: ch is a local stack variable, which is not memory allocated by malloc()
 
       /* file header */
       if (pfi.fhdr > 0) fseeko(pfi.infile, pfi.fhdr, SEEK_SET);
@@ -446,7 +448,7 @@ Importbinary(void *process)
 
           taxisDefVdate(taxisID, vdate);
           taxisDefVtime(taxisID, vtime);
-          pstreamDefTimestep(streamID, tsID);
+          cdoDefTimestep(streamID, tsID);
 
           for (int recID = 0; recID < nrecs; ++recID)
             {
@@ -535,10 +537,10 @@ Importbinary(void *process)
                 printf("%3d %4d %3d %6zu %6zu %12.5g %12.5g\n", tsID, recID,
               recoffset, nmiss, n_nan, fmin, fmax);
               */
-              varID = recinfo[recID].varID;
-              levelID = recinfo[recID].levelID;
-              pstreamDefRecord(streamID, varID, levelID);
-              pstreamWriteRecord(streamID, array, nmiss);
+              varID = recList[recID].varID;
+              levelID = recList[recID].levelID;
+              cdoDefRecord(streamID, varID, levelID);
+              cdoWriteRecord(streamID, array, nmiss);
             }
         }
 
