@@ -28,8 +28,7 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "grid.h"
-#include "pstream_int.h"
+#include <mpim_grid.h>
 
 #define MAX_DSETS 1024
 
@@ -1049,9 +1048,7 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
 
       nmiss = 0;
 
-      minval = 1e35;
-      maxval = -1e35;
-      arrayMinMax(gridsize * nt, array, &minval, &maxval);
+      arrayMinMax(gridsize * nt, array, minval, maxval);
 
       if (Options::cdoVerbose)
         cdoPrint("Dataset %s: missval = %g  addoffset = %g  scalefactor = %g", varname, missval, addoffset, scalefactor);
@@ -1100,7 +1097,7 @@ read_dataset(hid_t loc_id, const char *name, void *opdata)
       if (Options::cdoVerbose)
         cdoPrint("Dataset %s: dtype = %d  minval = %g  maxval = %g  missval = %g", varname, dtype, minval, maxval, missval);
 
-      if (nmiss > 0)
+      if (nmiss)
         {
           if (!(missval < minval || missval > maxval))
             {
@@ -1315,7 +1312,7 @@ void *
 Importcmsaf(void *process)
 {
 #ifdef HAVE_LIBHDF5
-  int streamID;
+  CdoStreamID streamID;
   int gridID = -1, zaxisID, taxisID, vlistID;
   int i, offset;
   size_t nmiss;
@@ -1450,9 +1447,9 @@ Importcmsaf(void *process)
   vdate = get_vdate(vlistID);
   if (vdate == 0) vdate = 10101;
 
-  streamID = cdoStreamOpenWrite(cdoStreamName(1));
+  streamID = cdoOpenWrite(1);
 
-  pstreamDefVlist(streamID, vlistID);
+  cdoDefVlist(streamID, vlistID);
 
   for (tsID = 0; tsID < nt; ++tsID)
     {
@@ -1460,7 +1457,7 @@ Importcmsaf(void *process)
       vtime = 0;
       if (vtimes) vtime = vtimes[tsID];
       taxisDefVtime(taxisID, vtime);
-      pstreamDefTimestep(streamID, tsID);
+      cdoDefTimestep(streamID, tsID);
 
       for (ivar = 0; ivar < dsets.nsets; ++ivar)
         {
@@ -1477,9 +1474,7 @@ Importcmsaf(void *process)
               if (nz == 1) offset = gridsize * tsID;
               array = dsets.obj[ivar].array + offset;
 
-              minval = 1e35;
-              maxval = -1e35;
-              arrayMinMaxMV(gridsize, array, missval, &minval, &maxval);
+              arrayMinMaxMV(gridsize, array, missval, minval, maxval);
 
               nmiss = arrayNumMV(gridsize, array, missval);
 
@@ -1491,8 +1486,8 @@ Importcmsaf(void *process)
                 cdoWarning(" Missval is inside of valid values! Name: %s  Range:
                 %g - %g  Missval: %g\n", dsets.obj[ivar].name, minval, maxval, missval);
               */
-              pstreamDefRecord(streamID, varID, levelID);
-              pstreamWriteRecord(streamID, array, nmiss);
+              cdoDefRecord(streamID, varID, levelID);
+              cdoWriteRecord(streamID, array, nmiss);
             }
         }
     }
