@@ -18,22 +18,22 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
-#include "grid.h"
+
+#include <mpim_grid.h>
+#include "griddes.h"
 
 #define MAX_VARS 6
 
 static void
 init_vars(int vlistID, int gridID, int zaxisID, int nvars)
 {
-  int code[] = { 11, 17, 33, 34, 1, 2 /*, 3*/ };
+  const int code[] = { 11, 17, 33, 34, 1, 2 /*, 3*/ };
   const char *name[] = { "temp", "depoint", "u", "v", "height", "pressure" /*, "station"*/ };
   const char *units[] = { "Celsius", "", "m/s", "m/s", "m", "hPa" /*, ""*/ };
-  int varID;
 
   for (int i = 0; i < nvars; ++i)
     {
-      varID = vlistDefVar(vlistID, gridID, zaxisID, TIME_VARYING);
+      int varID = vlistDefVar(vlistID, gridID, zaxisID, TIME_VARYING);
       vlistDefVarParam(vlistID, varID, cdiEncodeParam(code[i], 255, 255));
       vlistDefVarName(vlistID, varID, name[i]);
       vlistDefVarUnits(vlistID, varID, units[i]);
@@ -46,23 +46,22 @@ init_data(int vlistID, int nvars, double *data[])
 {
   for (int varID = 0; varID < nvars; ++varID)
     {
-      size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID, varID));
-      double missval = vlistInqVarMissval(vlistID, varID);
-
+      const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID, varID));
+      const double missval = vlistInqVarMissval(vlistID, varID);
       for (size_t i = 0; i < gridsize; ++i) data[varID][i] = missval;
     }
 }
 
 static void
-write_data(int streamID, int vlistID, int nvars, double *data[])
+write_data(CdoStreamID streamID, int vlistID, int nvars, double *data[])
 {
   for (int varID = 0; varID < nvars; ++varID)
     {
-      size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID, varID));
-      double missval = vlistInqVarMissval(vlistID, varID);
-      size_t nmiss = arrayNumMV(gridsize, data[varID], missval);
-      pstreamDefRecord(streamID, varID, 0);
-      pstreamWriteRecord(streamID, data[varID], nmiss);
+      const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID, varID));
+      const double missval = vlistInqVarMissval(vlistID, varID);
+      const size_t nmiss = arrayNumMV(gridsize, data[varID], missval);
+      cdoDefRecord(streamID, varID, 0);
+      cdoWriteRecord(streamID, data[varID], nmiss);
     }
 }
 
@@ -70,7 +69,7 @@ static int
 getDate(const char *name)
 {
   const char *pname = strchr(name, '_');
-  int date = pname ? atoi(pname + 1) : 0;
+  const int date = pname ? atoi(pname + 1) : 0;
   return date;
 }
 
@@ -96,33 +95,33 @@ Importobs(void *process)
 
   cdoOperatorAdd("import_obs", 0, 0, "grid description file or name");
 
-  int operatorID = cdoOperatorID();
+  const int operatorID = cdoOperatorID();
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
-  int gridID = cdoDefineGrid(operatorArgv()[0]);
+  const int gridID = cdoDefineGrid(operatorArgv()[0]);
 
   if (gridInqType(gridID) != GRID_LONLAT) cdoAbort("Unsupported grid type: %s", gridNamePtr(gridInqType(gridID)));
 
-  size_t gridsize = gridInqSize(gridID);
-  int xsize = gridInqXsize(gridID);
-  int ysize = gridInqYsize(gridID);
+  const size_t gridsize = gridInqSize(gridID);
+  const int xsize = gridInqXsize(gridID);
+  const int ysize = gridInqYsize(gridID);
 
   // printf("gridsize=%d, xsize=%d, ysize=%d\n", gridsize, xsize, ysize);
 
-  double *xvals = (double *) Malloc(gridsize * sizeof(double));
-  double *yvals = (double *) Malloc(gridsize * sizeof(double));
+  std::vector<double> xvals(gridsize);
+  std::vector<double> yvals(gridsize);
 
-  gridInqXvals(gridID, xvals);
-  gridInqYvals(gridID, yvals);
+  gridInqXvals(gridID, xvals.data());
+  gridInqYvals(gridID, yvals.data());
 
   /* Convert lat/lon units if required */
   {
     char units[CDI_MAX_NAME];
     gridInqXunits(gridID, units);
-    grid_to_degree(units, gridsize, xvals, "grid center lon");
+    grid_to_degree(units, gridsize, xvals.data(), "grid center lon");
     gridInqYunits(gridID, units);
-    grid_to_degree(units, gridsize, yvals, "grid center lat");
+    grid_to_degree(units, gridsize, yvals.data(), "grid center lat");
   }
 
   FILE *fp = fopen(cdoGetStreamName(0), "r");
@@ -135,20 +134,20 @@ Importobs(void *process)
   int64_t vdate = getDate(cdoGetStreamName(0));
   if (vdate <= 999999) vdate = vdate * 100 + 1;
 
-  int streamID = cdoStreamOpenWrite(cdoStreamName(1));
+  CdoStreamID streamID = cdoOpenWrite(1);
 
-  int zaxisID = zaxisCreate(ZAXIS_SURFACE, 1);
+  const int zaxisID = zaxisCreate(ZAXIS_SURFACE, 1);
 
-  int vlistID = vlistCreate();
+  const int vlistID = vlistCreate();
 
-  int taxisID = cdoTaxisCreate(TAXIS_ABSOLUTE);
+  const int taxisID = cdoTaxisCreate(TAXIS_ABSOLUTE);
   vlistDefTaxis(vlistID, taxisID);
 
   for (i = 0; i < nvars; ++i) data[i] = (double *) Malloc(gridsize * sizeof(double));
 
   init_vars(vlistID, gridID, zaxisID, nvars);
 
-  pstreamDefVlist(streamID, vlistID);
+  cdoDefVlist(streamID, vlistID);
 
   int64_t vdate0 = 0;
   int vtime0 = 0;
@@ -173,7 +172,7 @@ Importobs(void *process)
           */
           taxisDefVdate(taxisID, vdate);
           taxisDefVtime(taxisID, vtime);
-          pstreamDefTimestep(streamID, tsID);
+          cdoDefTimestep(streamID, tsID);
 
           init_data(vlistID, nvars, data);
 
@@ -234,9 +233,6 @@ Importobs(void *process)
   zaxisDestroy(zaxisID);
   taxisDestroy(taxisID);
 
-  Free(xvals);
-  Free(yvals);
-
   cdoFinish();
 
   return nullptr;
