@@ -27,16 +27,29 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
-#include "text.h"
+#include "mpmo_color.h"
 #include "array.h"
 #include "datetime.h"
 #include "printinfo.h"
 
+static void
+printGridIndex(int nlon, int nlat, int ilon, int ilat, int i)
+{
+  const int index = nlat < 10 ? 2 : nlat < 100 ? 3 : nlat < i ? 4 : 5;
+
+  std::stringstream s;
+  s << std::string(index, ' ');
+  for (ilon = 0; ilon < nlon; ilon++) s << ((ilon + 1) / i) % 10;
+
+  printf("%s", s.str().c_str());
+  putchar('\n');
+  fflush(stdout);
+}
+
 static void
 printMap(const int nlon, const int nlat, const double *array, const double missval, const double min, const double max)
 {
-  /* source code from PINGO */
+  // source code from PINGO
   int ilon, ilat, i;
   double x, a, b;
   double level[10];
@@ -149,26 +162,33 @@ printMap(const int nlon, const int nlat, const double *array, const double missv
                   }
             }
 
-          if (c == '0') set_text_color(stdout, BRIGHT, BLUE);
-          else if (c == '1') set_text_color(stdout, RESET, BLUE);
-          else if (c == '2') set_text_color(stdout, BRIGHT, CYAN);
-          else if (c == '3') set_text_color(stdout, RESET, CYAN);
-          else if (c == '4') set_text_color(stdout, RESET, GREEN);
-          else if (c == '5') set_text_color(stdout, RESET, YELLOW);
-          else if (c == '6') set_text_color(stdout, RESET, RED);
-          else if (c == '7') set_text_color(stdout, BRIGHT, RED);
-          else if (c == '8') set_text_color(stdout, RESET, MAGENTA);
-          else if (c == '9') set_text_color(stdout, BRIGHT, MAGENTA);
-          else if (c == 'M')
+          TextMode mode(MODELESS);
+          TextColor color(BLACK);
+          switch (c)
             {
-              set_text_color(stdout, bmax ? BLINK : RESET, BLACK);
+            /* clang-format off */
+            case '0': mode = BRIGHT   ; color = BLUE    ; break ;
+            case '1': mode = MODELESS ; color = BLUE    ; break ;
+            case '2': mode = BRIGHT   ; color = CYAN    ; break ;
+            case '3': mode = MODELESS ; color = CYAN    ; break ;
+            case '4': mode = MODELESS ; color = GREEN   ; break ;
+            case '5': mode = MODELESS ; color = YELLOW  ; break ;
+            case '6': mode = MODELESS ; color = RED     ; break ;
+            case '7': mode = BRIGHT   ; color = RED     ; break ;
+            case '8': mode = MODELESS ; color = MAGENTA ; break ;
+            case '9': mode = BRIGHT   ; color = MAGENTA ; break ;
+            /* clang-format on */
+            case 'm':
+              bmax == 1 ? mode = BLINK : mode = MODELESS, color = BLACK;
               if (bmax) bmax = 0;
-            }
-          else if (c == 'm')
-            {
-              set_text_color(stdout, bmin ? BLINK : RESET, BLACK);
+              break;
+            case 'M':
+              bmin == 1 ? mode = BLINK : mode = MODELESS, color = BLACK;
               if (bmin) bmin = 0;
+              break;
             }
+
+          set_text_color(stdout, mode, color);
           putchar(c);
           reset_text_color(stdout);
         }
@@ -178,41 +198,23 @@ printMap(const int nlon, const int nlat, const double *array, const double missv
   putchar('\n');
   fflush(stdout);
 
-  if (nlon >= 1000)
+  int current;
+  for (int i = 1; i <= 4; i++)
     {
-      printf("%.*s", nlat < 10 ? 2 : nlat < 100 ? 3 : nlat < 1000 ? 4 : 5, "     ");
-      for (ilon = 0; ilon < nlon; ilon++) printf("%d", ((ilon + 1) / 1000) % 10);
-      putchar('\n');
-      fflush(stdout);
+      current = 10000 / pow(10, i);
+      if (nlon >= current)
+        {
+          printGridIndex(nlon, nlat, ilon, ilat, current);
+        }
     }
-
-  if (nlon >= 100)
-    {
-      printf("%.*s", nlat < 10 ? 2 : nlat < 100 ? 3 : nlat < 1000 ? 4 : 5, "     ");
-      for (ilon = 0; ilon < nlon; ilon++) printf("%d", ((ilon + 1) / 100) % 10);
-      putchar('\n');
-      fflush(stdout);
-    }
-
-  if (nlon >= 10)
-    {
-      printf("%.*s", nlat < 10 ? 2 : nlat < 100 ? 3 : nlat < 1000 ? 4 : 5, "     ");
-      for (ilon = 0; ilon < nlon; ilon++) printf("%d", ((ilon + 1) / 10) % 10);
-      putchar('\n');
-      fflush(stdout);
-    }
-
-  printf("%.*s", nlat < 10 ? 2 : nlat < 100 ? 3 : nlat < 1000 ? 4 : 5, "     ");
-  for (ilon = 0; ilon < nlon; ilon++) printf("%d", (ilon + 1) % 10);
-  putchar('\n');
   fflush(stdout);
   putchar('\n');
   fflush(stdout);
 
   for (i = 0; i < 10; i++)
     {
-      printf("%d=%c%+9.3e,%+9.3e%c%s", i, '[', i == 0 ? min : level[i - 1],
-             i == 9 ? max : level[i], i == 9 || level[i] <= 0 ? ']' : ']', i != 2 && i != 5 && i != 8 ? "  " : "");
+      printf("%d=%c%+9.3e,%+9.3e%c%s", i, '[', i == 0 ? min : level[i - 1], i == 9 ? max : level[i],
+             i == 9 || level[i] <= 0 ? ']' : ']', i != 2 && i != 5 && i != 8 ? "  " : "");
 
       if (i == 2 || i == 5 || i == 8)
         {
@@ -270,6 +272,7 @@ Info(void *process)
   const int INFOP  = cdoOperatorAdd("infop",  E_PARAM,  0, nullptr);
   const int INFON  = cdoOperatorAdd("infon",  E_NAME,   0, nullptr);
   const int INFOC  = cdoOperatorAdd("infoc",  E_CODE,   0, nullptr);
+  const int VINFON = cdoOperatorAdd("vinfon", E_NAME,   0, nullptr);
   const int XINFON = cdoOperatorAdd("xinfon", E_NAME,   0, nullptr);
   const int MAP    = cdoOperatorAdd("map",    E_PARAM,  0, nullptr);
   // clang-format on
@@ -278,17 +281,28 @@ Info(void *process)
   UNUSED(INFOP);
   UNUSED(INFON);
   UNUSED(INFOC);
-  UNUSED(XINFON);
 
   const int operatorID = cdoOperatorID();
-
   const int operfunc = cdoOperatorF1(operatorID);
 
+  const bool lvinfo = (operatorID == VINFON || operatorID == XINFON);
+
   DateTimeList dtlist;
 
+  std::string e = "";
+  if (operfunc == E_NAME)
+    e = "Parameter name";
+  else if (operfunc == E_CODE)
+    e = "Code number";
+  else
+    e = "Parameter ID";
+
+  std::string v = (Options::cdoVerbose) ? " : Extra" : "";
+  int indg = 0;
+
   for (int indf = 0; indf < cdoStreamCnt(); indf++)
     {
-      const int streamID = cdoStreamOpenRead(indf);
+      CdoStreamID streamID = cdoOpenRead(indf);
       const int vlistID = cdoStreamInqVlist(streamID);
       const int taxisID = vlistInqTaxis(vlistID);
 
@@ -306,9 +320,8 @@ Info(void *process)
         arrayf.resize(gridsizemax);
       else
         array.resize(gridsizemax);
-        
 
-      int indg = 0;
+      indg = 0;
       int tsID = 0;
       while ((nrecs = cdoStreamInqTimestep(streamID, tsID)))
         {
@@ -325,30 +338,19 @@ Info(void *process)
             {
               if ((tsID == 0 && recID == 0) || operatorID == MAP)
                 {
-                  set_text_color(stdout, BRIGHT, BLACK);
-                  fprintf(stdout,
-                          "%6d :       Date     Time   %s Gridsize    Miss :     Minimum        Mean     Maximum : ", -(indf + 1),
-                          operatorID == XINFON ? "Nlevs" : "Level");
-
-                  if (operfunc == E_NAME)
-                    fprintf(stdout, "Parameter name");
-                  else if (operfunc == E_CODE)
-                    fprintf(stdout, "Code number");
-                  else
-                    fprintf(stdout, "Parameter ID");
-
-                  if (Options::cdoVerbose) fprintf(stdout, " : Extra");
+                  set_text_color(stdout, BRIGHT);
+                  MpMO::Print("%6d :       Date     Time   %s Gridsize    Miss :     Minimum        Mean     Maximum : %s%s", -(indf + 1),
+                        lvinfo ? "Nlevs" : "Level", e, v);
                   reset_text_color(stdout);
-                  fprintf(stdout, "\n");
                 }
 
-              pstreamInqRecord(streamID, &varID, &levelID);
+              cdoInqRecord(streamID, &varID, &levelID);
               if (Options::CDO_Memtype == MEMTYPE_FLOAT)
-                pstreamReadRecordF(streamID, arrayf.data(), &nmiss);
+                cdoReadRecordF(streamID, arrayf.data(), &nmiss);
               else
-                pstreamReadRecord(streamID, array.data(), &nmiss);
+                cdoReadRecord(streamID, array.data(), &nmiss);
 
-              indg = (operatorID == XINFON) ? varID + 1 : indg + 1;
+              indg = lvinfo ? varID + 1 : indg + 1;
 
               const int param = vlistInqVarParam(vlistID, varID);
               const int code = vlistInqVarCode(vlistID, varID);
@@ -360,7 +362,7 @@ Info(void *process)
               const double level = cdoZaxisInqLevel(zaxisID, levelID);
               const double missval = vlistInqVarMissval(vlistID, varID);
 
-              bool loutput = (operatorID != XINFON);
+              bool loutput = !lvinfo;
 
               if (loutput) infostatInit(infostat[varID]);
 
@@ -376,43 +378,38 @@ Info(void *process)
 
                   if (operfunc == E_NAME) vlistInqVarName(vlistID, varID, varname);
 
-                  set_text_color(stdout, BRIGHT, BLACK);
                   fprintf(stdout, "%6d ", indg);
-                  reset_text_color(stdout);
-                  set_text_color(stdout, RESET, BLACK);
                   fprintf(stdout, ":");
-                  reset_text_color(stdout);
 
-                  set_text_color(stdout, RESET, MAGENTA);
+                  set_text_color(stdout, MAGENTA);
                   fprintf(stdout, "%s %s ", vdatestr, vtimestr);
                   reset_text_color(stdout);
 
-                  set_text_color(stdout, RESET, GREEN);
-                  if (operatorID == XINFON)
+                  set_text_color(stdout, GREEN);
+                  if (lvinfo)
                     fprintf(stdout, "%7zu ", nlevs);
                   else
                     fprintf(stdout, "%7g ", level);
 
                   fprintf(stdout, "%8zu %7zu ", gridsize, infostatr.nmiss);
-
-                  set_text_color(stdout, RESET, BLACK);
-                  fprintf(stdout, ":");
                   reset_text_color(stdout);
 
-                  set_text_color(stdout, RESET, BLUE);
+                  fprintf(stdout, ":");
+
+                  set_text_color(stdout, BLUE);
                 }
 
               if (number == CDI_REAL)
                 {
                   fpeRaised = 0;
 
-                  if (infostatr.nmiss > 0)
+                  if (infostatr.nmiss)
                     {
                       size_t nvals;
                       if (Options::CDO_Memtype == MEMTYPE_FLOAT)
-                        nvals = arrayMinMaxSumMV_f(gridsize, arrayf.data(), missval, &infostatr.min, &infostatr.max, &infostatr.sum);
+                        nvals = arrayMinMaxSumMV_f(gridsize, arrayf.data(), missval, infostatr.min, infostatr.max, infostatr.sum);
                       else
-                        nvals = arrayMinMaxSumMV(gridsize, array.data(), missval, &infostatr.min, &infostatr.max, &infostatr.sum);
+                        nvals = arrayMinMaxSumMV(gridsize, array.data(), missval, infostatr.min, infostatr.max, infostatr.sum);
                       imiss = gridsize - nvals;
                       infostatr.nvals += nvals;
                     }
@@ -428,9 +425,9 @@ Info(void *process)
                   else
                     {
                       if (Options::CDO_Memtype == MEMTYPE_FLOAT)
-                        arrayMinMaxSum_f(gridsize, arrayf.data(), &infostatr.min, &infostatr.max, &infostatr.sum);
+                        arrayMinMaxSum_f(gridsize, arrayf.data(), infostatr.min, infostatr.max, infostatr.sum);
                       else
-                        arrayMinMaxSum(gridsize, array.data(), &infostatr.min, &infostatr.max, &infostatr.sum);
+                        arrayMinMaxSum(gridsize, array.data(), infostatr.min, infostatr.max, infostatr.sum);
                       infostatr.nvals += gridsize;
                     }
 
@@ -481,7 +478,7 @@ Info(void *process)
                             }
                         }
                     }
-                  
+
                   fpeRaised = 0;
 
                   imiss = gridsize - nvals;
@@ -499,18 +496,16 @@ Info(void *process)
                 {
                   reset_text_color(stdout);
 
-                  set_text_color(stdout, RESET, BLACK);
                   fprintf(stdout, " : ");
-                  reset_text_color(stdout);
 
-                  set_text_color(stdout, BRIGHT, GREEN);
+                  // set_text_color(stdout, GREEN);
                   if (operfunc == E_NAME)
                     fprintf(stdout, "%-14s", varname);
                   else if (operfunc == E_CODE)
                     fprintf(stdout, "%4d   ", code);
                   else
                     fprintf(stdout, "%-14s", paramstr);
-                  reset_text_color(stdout);
+                  // reset_text_color(stdout);
 
                   if (Options::cdoVerbose)
                     {
@@ -522,7 +517,7 @@ Info(void *process)
                   fprintf(stdout, "\n");
                 }
 
-              if (imiss != nmiss && nmiss > 0) cdoPrint("Found %zu of %zu missing values!", imiss, nmiss);
+              if (imiss != nmiss && nmiss) cdoPrint("Found %zu of %zu missing values!", imiss, nmiss);
 
               if (fpeRaised > 0) cdoWarning("floating-point exception reported: %s!", fpe_errstr(fpeRaised));
 
@@ -546,6 +541,14 @@ Info(void *process)
       cdoStreamClose(streamID);
     }
 
+  if (indg > 36 && operatorID != MAP)
+    {
+      set_text_color(stdout, BRIGHT);
+      MpMO::Print("       :       Date     Time   %s Gridsize    Miss :     Minimum        Mean     Maximum : %s%s",
+                  lvinfo ? "Nlevs" : "Level", e, v);
+      reset_text_color(stdout);
+    }
+
   cdoFinish();
 
   return nullptr;
