@@ -24,11 +24,13 @@
 */
 
 #include <cdi.h>
+#include <mpim_grid.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
+
 #include "griddes.h"
 #include "cdo_zaxis.h"
+#include "grid_read_pingo.h"
 
 static size_t
 input_iarray(size_t nval, int *array)
@@ -46,15 +48,13 @@ input_iarray(size_t nval, int *array)
   return ival;
 }
 
-size_t input_darray(FILE *gfp, size_t nval, double *array);
-
 void *
 Input(void *process)
 {
   int varID = 0;
   size_t gridsize0 = 0, gridsize = 0;
   int taxisID = 0;
-  int streamID = -1;
+  CdoStreamID streamID = CDO_STREAM_UNDEF;
   int vlistID = -1;
   int code = 0, level = 0, date = 0, time = 0, nlon = 0, nlat = 0;
   int output_filetype = CDI_FILETYPE_GRB;
@@ -212,29 +212,29 @@ Input(void *process)
           taxisID = cdoTaxisCreate(TAXIS_RELATIVE);
           vlistDefTaxis(vlistID, taxisID);
 
-          streamID = cdoStreamOpenWrite(cdoStreamName(0), output_filetype);
-          pstreamDefVlist(streamID, vlistID);
+          streamID = cdoOpenWrite((0), output_filetype);
+          cdoDefVlist(streamID, vlistID);
         }
 
       int64_t vdate = date;
       int vtime = time;
       taxisDefVdate(taxisID, vdate);
       taxisDefVtime(taxisID, vtime);
-      pstreamDefTimestep(streamID, tsID);
+      cdoDefTimestep(streamID, tsID);
 
       for (int levelID = 0; levelID < nlevs; levelID++)
         {
           const size_t offset = gridsize * levelID;
           const size_t nmiss = arrayNumMV(gridsize, &array[offset], missval);
-          pstreamDefRecord(streamID, varID, levelID);
-          pstreamWriteRecord(streamID, &array[offset], nmiss);
+          cdoDefRecord(streamID, varID, levelID);
+          cdoWriteRecord(streamID, &array[offset], nmiss);
         }
 
       nrecs++;
       tsID++;
     }
 
-  if (streamID >= 0)
+  if (streamID)
     {
       cdoStreamClose(streamID);
       vlistDestroy(vlistID);
