@@ -23,7 +23,7 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
+
 #include "interpol.h"
 #include "datetime.h"
 #include "printinfo.h"
@@ -55,7 +55,6 @@ Intgridtraj(void *process)
   int64_t vdate;
   int vtime;
   size_t nmiss = 0;
-  double point;
   double xpos, ypos;
   int calendar = CALENDAR_STANDARD;
 
@@ -71,21 +70,18 @@ Intgridtraj(void *process)
   JulianDate juldate;
   readnextpos(fp, calendar, &juldate, &xpos, &ypos);
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-
+  CdoStreamID streamID1 = cdoOpenRead(0);
   const int vlistID1 = cdoStreamInqVlist(streamID1);
 
-  Field field1, field2;
-  fieldInit(field1);
-  fieldInit(field2);
-
   const int nvars = vlistNvars(vlistID1);
 
   const int maxrecs = vlistNrecs(vlistID1);
-  std::vector<RecordInfo> recinfo(maxrecs);
+  std::vector<RecordInfo> recList(maxrecs);
 
   const size_t gridsizemax = vlistGridsizeMax(vlistID1);
-  std::vector<double> array(gridsizemax);
+  Field field1, field2;
+  field1.resize(gridsizemax);
+  field2.resize(1);
 
   double **vardata1 = (double **) Malloc(nvars * sizeof(double *));
   double **vardata2 = (double **) Malloc(nvars * sizeof(double *));
@@ -121,18 +117,18 @@ Intgridtraj(void *process)
   const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = CDI_UNDEFID;
+  CdoStreamID streamID2 = CDO_STREAM_UNDEF;
 
   int tsID = 0;
   int nrecs = cdoStreamInqTimestep(streamID1, tsID++);
   JulianDate juldate1 = julianDateEncode(calendar, taxisInqVdate(taxisID1), taxisInqVtime(taxisID1));
   for (int recID = 0; recID < nrecs; recID++)
     {
-      pstreamInqRecord(streamID1, &varID, &levelID);
+      cdoInqRecord(streamID1, &varID, &levelID);
       const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
       const size_t offset = gridsize * levelID;
       double *single1 = vardata1[varID] + offset;
-      pstreamReadRecord(streamID1, single1, &nmiss);
+      cdoReadRecord(streamID1, single1, &nmiss);
       if (nmiss) cdoAbort("Missing values unsupported for this operator!");
     }
 
@@ -145,16 +141,16 @@ Intgridtraj(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
-          recinfo[recID].varID = varID;
-          recinfo[recID].levelID = levelID;
-          recinfo[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+          recList[recID].varID = varID;
+          recList[recID].levelID = levelID;
+          recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
 
           const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
           const size_t offset = gridsize * levelID;
           double *single2 = vardata2[varID] + offset;
-          pstreamReadRecord(streamID1, single2, &nmiss);
+          cdoReadRecord(streamID1, single2, &nmiss);
           if (nmiss) cdoAbort("Missing values unsupported for this operator!");
         }
 
@@ -163,16 +159,16 @@ Intgridtraj(void *process)
           if (julianDateToSeconds(juldate) >= julianDateToSeconds(juldate1)
               && julianDateToSeconds(juldate) < julianDateToSeconds(juldate2))
             {
-              if (streamID2 == CDI_UNDEFID)
+              if (streamID2 == CDO_STREAM_UNDEF)
                 {
-                  streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-                  pstreamDefVlist(streamID2, vlistID2);
+                  streamID2 = cdoOpenWrite(1);
+                  cdoDefVlist(streamID2, vlistID2);
                 }
 
               julianDateDecode(calendar, juldate, &vdate, &vtime);
               taxisDefVdate(taxisID2, vdate);
               taxisDefVtime(taxisID2, vtime);
-              pstreamDefTimestep(streamID2, tsIDo++);
+              cdoDefTimestep(streamID2, tsIDo++);
 
               const double fac1
                   = julianDateToSeconds(julianDateSub(juldate2, juldate)) / julianDateToSeconds(julianDateSub(juldate2, juldate1));
@@ -186,8 +182,8 @@ Intgridtraj(void *process)
               */
               for (int recID = 0; recID < nrecs; recID++)
                 {
-                  varID = recinfo[recID].varID;
-                  levelID = recinfo[recID].levelID;
+                  varID = recList[recID].varID;
+                  levelID = recList[recID].levelID;
                   const double missval = vlistInqVarMissval(vlistID1, varID);
                   const int gridID1 = vlistInqVarGrid(vlistID1, varID);
                   const size_t gridsize = gridInqSize(gridID1);
@@ -195,20 +191,18 @@ Intgridtraj(void *process)
                   double *single1 = vardata1[varID] + offset;
                   double *single2 = vardata2[varID] + offset;
 
-                  for (size_t i = 0; i < gridsize; i++) array[i] = single1[i] * fac1 + single2[i] * fac2;
+                  for (size_t i = 0; i < gridsize; i++) field1.vec[i] = single1[i] * fac1 + single2[i] * fac2;
 
                   field1.grid = gridID1;
                   field1.nmiss = nmiss;
                   field1.missval = missval;
-                  field1.ptr = array.data();
                   field2.grid = gridID2;
-                  field2.ptr = &point;
                   field2.nmiss = 0;
 
-                  intgridbil(&field1, &field2);
+                  intgridbil(field1, field2);
 
-                  pstreamDefRecord(streamID2, varID, levelID);
-                  pstreamWriteRecord(streamID2, &point, nmiss);
+                  cdoDefRecord(streamID2, varID, levelID);
+                  cdoWriteRecord(streamID2, field2.vec.data(), field2.nmiss);
                 }
             }
           if (readnextpos(fp, calendar, &juldate, &xpos, &ypos) == EOF) break;
@@ -235,7 +229,7 @@ Intgridtraj(void *process)
     }
 
   fclose(fp);
-  if (streamID2 != CDI_UNDEFID) cdoStreamClose(streamID2);
+  if (streamID2 != CDO_STREAM_UNDEF) cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
   for (varID = 0; varID < nvars; varID++)
