@@ -25,7 +25,7 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
+
 #include "listarray.h"
 
 static double
@@ -64,8 +64,8 @@ vert_interp_lev(size_t gridsize, double missval, double *vardata1, double *varda
     {
       const int idx1 = lev_idx1[ilev];
       const int idx2 = lev_idx2[ilev];
-      const double wgt1 = lev_wgt1[ilev];
-      const double wgt2 = lev_wgt2[ilev];
+      double wgt1 = lev_wgt1[ilev];
+      double wgt2 = lev_wgt2[ilev];
       double *var2 = vardata2 + gridsize * ilev;
 
       // if ( Options::cdoVerbose ) cdoPrint("level %d: idx1=%d idx2=%d wgt1=%g wgt2=%g", ilev, idx1, idx2, wgt1, wgt2);
@@ -74,7 +74,7 @@ vert_interp_lev(size_t gridsize, double missval, double *vardata1, double *varda
       const double *var1L2 = vardata1 + gridsize * idx2;
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(gridsize, var2, var1L1, var1L2, missval)
+#pragma omp parallel for default(none) shared(gridsize, var2, var1L1, var1L2, missval, wgt1, wgt2)
 #endif
       for (size_t i = 0; i < gridsize; ++i)
         {
@@ -92,11 +92,11 @@ vert_interp_lev3d(size_t gridsize, double missval, double *vardata1, double *var
 {
   for (int ilev = 0; ilev < nlev2; ilev++)
     {
-      const size_t offset = ilev * gridsize;
+      size_t offset = ilev * gridsize;
       double *var2 = vardata2 + offset;
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(gridsize, vardata1, var2, lev_idx1, lev_idx2, lev_wgt1, lev_wgt2, missval)
+#pragma omp parallel for default(none) shared(gridsize, offset, vardata1, var2, lev_idx1, lev_idx2, lev_wgt1, lev_wgt2, missval)
 #endif
       for (size_t i = 0; i < gridsize; i++)
         {
@@ -298,7 +298,7 @@ Intlevel(void *process)
   if (Options::cdoVerbose)
     for (int i = 0; i < nlev2; ++i) cdoPrint("lev2 %d: %g", i, lev2[i]);
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -307,6 +307,9 @@ Intlevel(void *process)
   const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
+  VarList varList1, varList2;
+  varListInit(varList1, vlistID1);
+
   const int nvars = vlistNvars(vlistID1);
 
   // Find z-variable
@@ -331,10 +334,9 @@ Intlevel(void *process)
         }
 
       if (zvarID == CDI_UNDEFID) cdoAbort("Variable %s not found!", zvarname);
-      zvarIsVarying = vlistInqVarTimetype(vlistID1, zvarID) == TIME_VARYING;
-      zvarGridsize = gridInqSize(vlistInqVarGrid(vlistID1, zvarID));
-      zaxisID1 = vlistInqVarZaxis(vlistID1, zvarID);
-      nlev1 = zaxisInqSize(zaxisID1);
+      zvarIsVarying = varList1[zvarID].timetype == TIME_VARYING;
+      zvarGridsize = varList1[zvarID].gridsize;
+      nlev1 = varList1[zvarID].nlevels;
 
       lev1.resize(zvarGridsize * (nlev1 + 2));
 
@@ -414,43 +416,42 @@ Intlevel(void *process)
   for (int i = 0; i < nzaxis; i++)
     if (zaxisID1 == vlistZaxis(vlistID1, i)) vlistChangeZaxisIndex(vlistID2, i, zaxisID2);
 
+  varListInit(varList2, vlistID2);
+
   std::vector<int> lev_idx1(wisize);
   std::vector<int> lev_idx2(wisize);
   std::vector<double> lev_wgt1(wisize);
   std::vector<double> lev_wgt2(wisize);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
+  CdoStreamID streamID2 = cdoOpenWrite(1);
 
-  pstreamDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID2, vlistID2);
 
   std::vector<bool> vars(nvars);
   std::vector<bool> varinterp(nvars);
-  std::vector<size_t *> varnmiss(nvars);
-  std::vector<double *> vardata1(nvars);
-  std::vector<double *> vardata2(nvars);
+  std::vector<std::vector<size_t>> varnmiss(nvars);
+  std::vector<std::vector<double>> vardata1(nvars);
+  std::vector<std::vector<double>> vardata2(nvars);
 
   const int maxlev = nlev1 > nlev2 ? nlev1 : nlev2;
 
   for (varID = 0; varID < nvars; varID++)
     {
-      const int gridID = vlistInqVarGrid(vlistID1, varID);
-      const int zaxisID = vlistInqVarZaxis(vlistID1, varID);
-      const size_t gridsize = gridInqSize(gridID);
-      const int nlevel = zaxisInqSize(zaxisID);
+      const size_t gridsize = varList1[varID].gridsize;
+      const int nlevel = varList1[varID].nlevels;
 
-      vardata1[varID] = (double *) Malloc(gridsize * nlevel * sizeof(double));
+      vardata1[varID].resize(gridsize * nlevel);
 
-      if (zaxisID == zaxisID1)
+      varinterp[varID] = varList1[varID].zaxisID == zaxisID1;
+
+      if (varinterp[varID])
         {
-          varinterp[varID] = true;
-          vardata2[varID] = (double *) Malloc(gridsize * nlev2 * sizeof(double));
-          varnmiss[varID] = (size_t *) Calloc(maxlev, sizeof(size_t));
+          vardata2[varID].resize(gridsize * nlev2);
+          varnmiss[varID].resize(maxlev, 0);
         }
       else
         {
-          varinterp[varID] = false;
-          vardata2[varID] = vardata1[varID];
-          varnmiss[varID] = (size_t *) Malloc(nlevel * sizeof(size_t));
+          varnmiss[varID].resize(nlevel);
         }
     }
 
@@ -460,16 +461,14 @@ Intlevel(void *process)
       for (varID = 0; varID < nvars; ++varID) vars[varID] = false;
 
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-          const size_t offset = gridsize * levelID;
-          double *single1 = vardata1[varID] + offset;
-
-          pstreamReadRecord(streamID1, single1, &varnmiss[varID][levelID]);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          const size_t offset = varList1[varID].gridsize * levelID;
+          double *single1 = &vardata1[varID][offset];
+          cdoReadRecord(streamID1, single1, &varnmiss[varID][levelID]);
           vars[varID] = true;
         }
 
@@ -481,7 +480,7 @@ Intlevel(void *process)
                 {
                   const size_t offset1 = zvarGridsize * levelID;
                   const size_t offset2 = zvarGridsize * (levelID + 1);
-                  arrayCopy(zvarGridsize, vardata1[zvarID] + offset1, &lev1[offset2]);
+                  arrayCopy(zvarGridsize, &vardata1[zvarID][offset1], &lev1[offset2]);
                 }
 
               vert_init_level_0_and_N(nlev1, zvarGridsize, lev1.data());
@@ -497,21 +496,20 @@ Intlevel(void *process)
         {
           if (vars[varID] && varinterp[varID])
             {
-              const int gridID = vlistInqVarGrid(vlistID1, varID);
-              const double missval = vlistInqVarMissval(vlistID1, varID);
-              const size_t gridsize = gridInqSize(gridID);
+              const double missval = varList1[varID].missval;
+              const size_t gridsize = varList1[varID].gridsize;
 
               if (zvarname)
-                vert_interp_lev3d(gridsize, missval, vardata1[varID], vardata2[varID], nlev2, lev_idx1.data(), lev_idx2.data(),
+                vert_interp_lev3d(gridsize, missval, &vardata1[varID][0], &vardata2[varID][0], nlev2, lev_idx1.data(), lev_idx2.data(),
                                   lev_wgt1.data(), lev_wgt2.data());
               else
-                vert_interp_lev(gridsize, missval, vardata1[varID], vardata2[varID], nlev2, lev_idx1.data(), lev_idx2.data(),
+                vert_interp_lev(gridsize, missval, &vardata1[varID][0], &vardata2[varID][0], nlev2, lev_idx1.data(), lev_idx2.data(),
                                 lev_wgt1.data(), lev_wgt2.data());
 
               for (levelID = 0; levelID < nlev2; levelID++)
                 {
                   const size_t offset = gridsize * levelID;
-                  double *single2 = vardata2[varID] + offset;
+                  double *single2 = &vardata2[varID][offset];
                   varnmiss[varID][levelID] = arrayNumMV(gridsize, single2, missval);
                 }
             }
@@ -521,14 +519,14 @@ Intlevel(void *process)
         {
           if (vars[varID])
             {
-              const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
-              const int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
+              const size_t gridsize = varList2[varID].gridsize;
+              const int nlevel = varList2[varID].nlevels;
               for (levelID = 0; levelID < nlevel; levelID++)
                 {
                   const size_t offset = gridsize * levelID;
-                  double *single2 = vardata2[varID] + offset;
-                  pstreamDefRecord(streamID2, varID, levelID);
-                  pstreamWriteRecord(streamID2, single2, varnmiss[varID][levelID]);
+                  double *single = varinterp[varID] ? &vardata2[varID][offset] : &vardata1[varID][offset];
+                  cdoDefRecord(streamID2, varID, levelID);
+                  cdoWriteRecord(streamID2, single, varnmiss[varID][levelID]);
                 }
             }
         }
@@ -539,13 +537,6 @@ Intlevel(void *process)
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
-  for (varID = 0; varID < nvars; varID++)
-    {
-      Free(varnmiss[varID]);
-      Free(vardata1[varID]);
-      if (varinterp[varID]) Free(vardata2[varID]);
-    }
-
   cdoFinish();
 
   return nullptr;
