@@ -26,7 +26,8 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
+#include "cdi_lockedIO.h"
+
 #include "after_vertint.h"
 
 void vert_interp_lev3d(size_t gridsize, double missval, double *vardata1, double *vardata2, int nlev2, int *lev_idx1, int *lev_idx2,
@@ -100,9 +101,9 @@ Intlevel3d(void *process)
 
   operatorInputArg("icoordinate");
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));                 /*  input data */
-  const int streamID2 = cdoStreamOpenRead(cdoStreamName(1));                 /*  3d target vertical coordinate */
-  const int streamID3 = cdoStreamOpenWrite(cdoStreamName(2)); /*  output stream */
+  CdoStreamID streamID1 = cdoOpenRead(0);  /*  input data */
+  CdoStreamID streamID2 = cdoOpenRead(1);  /*  3d target vertical coordinate */
+  CdoStreamID streamID3 = cdoOpenWrite(2); /*  output stream */
 
   /*  Read filename from Parameter */
   operatorInputArg("filename for vertical source coordinates variable");
@@ -127,7 +128,7 @@ Intlevel3d(void *process)
   size_t zlevels_in_miss = 0;
 
   {
-    const int streamID0 = streamOpenReadLocked(operatorArgv()[0]); /*  3d vertical input coordinate */
+    int streamID0 = streamOpenReadLocked(operatorArgv()[0]); /*  3d vertical input coordinate */
     const int vlistID0 = streamInqVlist(streamID0);
 
     const int nvars = vlistNvars(vlistID0);
@@ -174,10 +175,10 @@ Intlevel3d(void *process)
 
     for (int recID = 0; recID < nrecs; recID++)
       {
-        pstreamInqRecord(streamID2, &varID, &levelID);
+        cdoInqRecord(streamID2, &varID, &levelID);
         const size_t offset = gridsize * levelID;
         double *single1 = zlevels_out + offset;
-        pstreamReadRecord(streamID2, single1, &zlevels_out_miss);
+        cdoReadRecord(streamID2, single1, &zlevels_out_miss);
       }
   }
 
@@ -277,7 +278,7 @@ Intlevel3d(void *process)
     if (str[0]) vlistDefVarUnits(vlistID3, oz3dvarID, str);
   }
 
-  pstreamDefVlist(streamID3, vlistID3);
+  cdoDefVlist(streamID3, vlistID3);
 
   maxlev = nlevi > nlevo ? nlevi : nlevo;
   nvars = vlistNvars(vlistID1);
@@ -346,18 +347,18 @@ Intlevel3d(void *process)
       for (varID = 0; varID < nvars; ++varID) vars[varID] = false;
 
       taxisCopyTimestep(taxisID3, taxisID1);
-      pstreamDefTimestep(streamID3, tsID);
+      cdoDefTimestep(streamID3, tsID);
 
       /*
        * Read the whole 3d data field
        */
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
           const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
           const size_t offset = gridsize * levelID;
           double *single1 = vardata1[varID] + offset;
-          pstreamReadRecord(streamID1, single1, &varnmiss[varID][levelID]);
+          cdoReadRecord(streamID1, single1, &varnmiss[varID][levelID]);
           vars[varID] = true;
         }
 
@@ -397,8 +398,8 @@ Intlevel3d(void *process)
                 {
                   size_t offset = gridsize * levelID;
                   double *single2 = vardata2[varID] + offset;
-                  pstreamDefRecord(streamID3, varID, levelID);
-                  pstreamWriteRecord(streamID3, single2, varnmiss[varID][levelID]);
+                  cdoDefRecord(streamID3, varID, levelID);
+                  cdoWriteRecord(streamID3, single2, varnmiss[varID][levelID]);
                 }
             }
         }
@@ -410,8 +411,8 @@ Intlevel3d(void *process)
         {
           const size_t offset = gridsize * levelID;
           double *single2 = zlevels_out + offset;
-          pstreamDefRecord(streamID3, oz3dvarID, levelID);
-          pstreamWriteRecord(streamID3, single2, 0);
+          cdoDefRecord(streamID3, oz3dvarID, levelID);
+          cdoWriteRecord(streamID3, single2, 0);
         }
       tsID++;
     }
