@@ -25,26 +25,87 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
+
 #include "datetime.h"
 #include "printinfo.h"
 
 int get_tunits(const char *unit, int *incperiod, int *incunit, int *tunit);
 
+size_t
+inttime(double fac1, double fac2, size_t gridsize, const double *single1, const double *single2, std::vector<double> &array,
+        bool withMissval, double missval1, double missval2)
+{
+  size_t nmiss = 0;
+
+  if (withMissval)
+    {
+      for (size_t i = 0; i < gridsize; i++)
+        {
+          if (!DBL_IS_EQUAL(single1[i], missval1) && !DBL_IS_EQUAL(single2[i], missval2))
+            array[i] = single1[i] * fac1 + single2[i] * fac2;
+          else if (DBL_IS_EQUAL(single1[i], missval1) && !DBL_IS_EQUAL(single2[i], missval2) && fac2 >= 0.5)
+            array[i] = single2[i];
+          else if (DBL_IS_EQUAL(single2[i], missval2) && !DBL_IS_EQUAL(single1[i], missval1) && fac1 >= 0.5)
+            array[i] = single1[i];
+          else
+            {
+              array[i] = missval1;
+              nmiss++;
+            }
+        }
+    }
+  else
+    {
+      for (size_t i = 0; i < gridsize; i++) array[i] = single1[i] * fac1 + single2[i] * fac2;
+    }
+
+  return nmiss;
+}
+
+static void
+juldateAddIncrement(JulianDate &juldate, int64_t ijulinc, int calendar, int tunit)
+{
+  if (tunit == TUNIT_MONTH || tunit == TUNIT_YEAR)
+    {
+      int64_t vdate;
+      int vtime;
+      julianDateDecode(calendar, juldate, &vdate, &vtime);
+
+      int year, month, day;
+      cdiDecodeDate(vdate, &year, &month, &day);
+
+      month += (int) ijulinc;
+
+      while (month > 12)
+        {
+          month -= 12;
+          year++;
+        }
+      while (month < 1)
+        {
+          month += 12;
+          year--;
+        }
+
+      vdate = cdiEncodeDate(year, month, day);
+
+      juldate = julianDateEncode(calendar, vdate, vtime);
+    }
+  else
+    {
+      juldate = julianDateAddSeconds(ijulinc, juldate);
+    }
+}
+
 void *
 Inttime(void *process)
 {
-  int streamID2 = -1;
-  int nlevel;
+  CdoStreamID streamID2 = CDO_STREAM_UNDEF;
   int varID, levelID;
   int64_t vdate;
   int vtime;
-  size_t gridsize;
-  size_t offset;
   int incperiod = 0, incunit = 3600, tunit = TUNIT_HOUR;
   int year, month, day, hour, minute, second;
-  double *single1, *single2;
-  double *vardatap;
 
   cdoInitialize(process);
 
@@ -90,36 +151,42 @@ Inttime(void *process)
       get_tunits(timeunits, &incperiod, &incunit, &tunit);
     }
   /* increment in seconds */
-  int64_t ijulinc = (int64_t) incperiod * incunit;
+  const int64_t ijulinc = (int64_t) incperiod * incunit;
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
 
+  VarList varList1, varList2;
+  varListInit(varList1, vlistID1);
+  varListInit(varList2, vlistID2);
+  
   if (ijulinc == 0) vlistDefNtsteps(vlistID2, 1);
 
   const int nvars = vlistNvars(vlistID1);
 
   const int maxrecs = vlistNrecs(vlistID1);
-  std::vector<RecordInfo> recinfo(maxrecs);
+  std::vector<RecordInfo> recList(maxrecs);
 
   const size_t gridsizemax = vlistGridsizeMax(vlistID1);
   std::vector<double> array(gridsizemax);
 
-  std::vector<std::vector<size_t>> nmiss1(nvars);
-  std::vector<std::vector<size_t>> nmiss2(nvars);
-  double **vardata1 = (double **) Malloc(nvars * sizeof(double *));
-  double **vardata2 = (double **) Malloc(nvars * sizeof(double *));
+  std::vector<std::vector<std::vector<size_t>>> nmiss(2);
+  nmiss[0].resize(nvars);
+  nmiss[1].resize(nvars);
+  std::vector<std::vector<std::vector<double>>> vardata(2);
+  vardata[0].resize(nvars);
+  vardata[1].resize(nvars);
 
   for (varID = 0; varID < nvars; varID++)
     {
-      gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-      nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-      nmiss1[varID].resize(nlevel);
-      nmiss2[varID].resize(nlevel);
-      vardata1[varID] = (double *) Malloc(gridsize * nlevel * sizeof(double));
-      vardata2[varID] = (double *) Malloc(gridsize * nlevel * sizeof(double));
+      const size_t gridsize = varList1[varID].gridsize;
+      const int nlevel = varList1[varID].nlevels;
+      nmiss[0][varID].resize(nlevel);
+      nmiss[1][varID].resize(nlevel);
+      vardata[0][varID].resize(gridsize * nlevel);
+      vardata[1][varID].resize(gridsize * nlevel);
     }
 
   const int taxisID1 = vlistInqTaxis(vlistID1);
@@ -138,16 +205,17 @@ Inttime(void *process)
       cdoPrint("ijulinc = %lld", ijulinc);
     }
 
+  int curFirst = 0, curSecond = 1;
+
   int tsID = 0;
   int nrecs = cdoStreamInqTimestep(streamID1, tsID++);
   JulianDate juldate1 = julianDateEncode(calendar, taxisInqVdate(taxisID1), taxisInqVtime(taxisID1));
   for (int recID = 0; recID < nrecs; recID++)
     {
-      pstreamInqRecord(streamID1, &varID, &levelID);
-      gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-      offset = gridsize * levelID;
-      single1 = vardata1[varID] + offset;
-      pstreamReadRecord(streamID1, single1, &nmiss1[varID][levelID]);
+      cdoInqRecord(streamID1, &varID, &levelID);
+      const size_t offset = varList1[varID].gridsize * levelID;
+      double *single1 = &vardata[curFirst][varID][offset];
+      cdoReadRecord(streamID1, single1, &nmiss[curFirst][varID][levelID]);
     }
 
   if (Options::cdoVerbose)
@@ -173,16 +241,15 @@ Inttime(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
-          recinfo[recID].varID = varID;
-          recinfo[recID].levelID = levelID;
-          recinfo[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+          recList[recID].varID = varID;
+          recList[recID].levelID = levelID;
+          recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
 
-          gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-          offset = gridsize * levelID;
-          single2 = vardata2[varID] + offset;
-          pstreamReadRecord(streamID1, single2, &nmiss2[varID][levelID]);
+          const size_t offset = varList1[varID].gridsize * levelID;
+          double *single2 = &vardata[curSecond][varID][offset];
+          cdoReadRecord(streamID1, single2, &nmiss[curSecond][varID][levelID]);
         }
 
       while (julianDateToSeconds(juldate) <= julianDateToSeconds(juldate2))
@@ -195,122 +262,53 @@ Inttime(void *process)
               if (Options::cdoVerbose)
                 {
                   char vdatestr[32], vtimestr[32];
-                  /*
-                  cdoPrint("juldate1 %f", julianDateToSeconds(juldate1));
-                  cdoPrint("juldate  %f", julianDateToSeconds(juldate));
-                  cdoPrint("juldate2 %f", julianDateToSeconds(juldate2));
-                  */
                   date2str(vdate, vdatestr, sizeof(vdatestr));
                   time2str(vtime, vtimestr, sizeof(vtimestr));
                   cdoPrint("%s %s  %f  %d", vdatestr, vtimestr, julianDateToSeconds(juldate), calendar);
                 }
 
-              if (streamID2 == -1)
+              if (streamID2 == CDO_STREAM_UNDEF)
                 {
-                  streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-                  pstreamDefVlist(streamID2, vlistID2);
+                  streamID2 = cdoOpenWrite(1);
+                  cdoDefVlist(streamID2, vlistID2);
                 }
 
               taxisDefVdate(taxisID2, vdate);
               taxisDefVtime(taxisID2, vtime);
-              pstreamDefTimestep(streamID2, tsIDo++);
+              cdoDefTimestep(streamID2, tsIDo++);
 
-              const double fac1
-                  = julianDateToSeconds(julianDateSub(juldate2, juldate)) / julianDateToSeconds(julianDateSub(juldate2, juldate1));
-              const double fac2
-                  = julianDateToSeconds(julianDateSub(juldate, juldate1)) / julianDateToSeconds(julianDateSub(juldate2, juldate1));
+              const double diff = julianDateToSeconds(julianDateSub(juldate2, juldate1));
+              const double fac1 = julianDateToSeconds(julianDateSub(juldate2, juldate)) / diff;
+              const double fac2 = julianDateToSeconds(julianDateSub(juldate, juldate1)) / diff;
 
               for (int recID = 0; recID < nrecs; recID++)
                 {
-                  varID = recinfo[recID].varID;
-                  levelID = recinfo[recID].levelID;
-                  gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-                  offset = gridsize * levelID;
-                  single1 = vardata1[varID] + offset;
-                  single2 = vardata2[varID] + offset;
+                  varID = recList[recID].varID;
+                  levelID = recList[recID].levelID;
+                  const size_t gridsize = varList1[varID].gridsize;
+                  const size_t offset = gridsize * levelID;
+                  const double *single1 = &vardata[curFirst][varID][offset];
+                  const double *single2 = &vardata[curSecond][varID][offset];
 
-                  size_t nmiss3 = 0;
+                  const bool withMissval = nmiss[curFirst][varID][levelID] || nmiss[curSecond][varID][levelID];
+                  const size_t nmiss3 = inttime(fac1, fac2, gridsize, single1, single2, array,
+                                                withMissval, varList1[varID].missval, varList2[varID].missval);
 
-                  if (nmiss1[varID][levelID] > 0 || nmiss2[varID][levelID] > 0)
-                    {
-                      const double missval1 = vlistInqVarMissval(vlistID1, varID);
-                      const double missval2 = vlistInqVarMissval(vlistID2, varID);
-
-                      for (size_t i = 0; i < gridsize; i++)
-                        {
-                          if (!DBL_IS_EQUAL(single1[i], missval1) && !DBL_IS_EQUAL(single2[i], missval2))
-                            array[i] = single1[i] * fac1 + single2[i] * fac2;
-                          else if (DBL_IS_EQUAL(single1[i], missval1) && !DBL_IS_EQUAL(single2[i], missval2) && fac2 >= 0.5)
-                            array[i] = single2[i];
-                          else if (DBL_IS_EQUAL(single2[i], missval2) && !DBL_IS_EQUAL(single1[i], missval1) && fac1 >= 0.5)
-                            array[i] = single1[i];
-                          else
-                            {
-                              array[i] = missval1;
-                              nmiss3++;
-                            }
-                        }
-                    }
-                  else
-                    {
-                      for (size_t i = 0; i < gridsize; i++) array[i] = single1[i] * fac1 + single2[i] * fac2;
-                    }
-
-                  pstreamDefRecord(streamID2, varID, levelID);
-                  pstreamWriteRecord(streamID2, array.data(), nmiss3);
+                  cdoDefRecord(streamID2, varID, levelID);
+                  cdoWriteRecord(streamID2, array.data(), nmiss3);
                 }
             }
 
           if (ijulinc == 0) break;
 
-          if (tunit == TUNIT_MONTH || tunit == TUNIT_YEAR)
-            {
-              julianDateDecode(calendar, juldate, &vdate, &vtime);
-
-              cdiDecodeDate(vdate, &year, &month, &day);
-
-              month += (int) ijulinc;
-
-              while (month > 12)
-                {
-                  month -= 12;
-                  year++;
-                }
-              while (month < 1)
-                {
-                  month += 12;
-                  year--;
-                }
-
-              vdate = cdiEncodeDate(year, month, day);
-
-              juldate = julianDateEncode(calendar, vdate, vtime);
-            }
-          else
-            {
-              juldate = julianDateAddSeconds(ijulinc, juldate);
-            }
+          juldateAddIncrement(juldate, ijulinc, calendar, tunit);
         }
 
       juldate1 = juldate2;
-      for (varID = 0; varID < nvars; varID++)
-        {
-          vardatap = vardata1[varID];
-          vardata1[varID] = vardata2[varID];
-          vardata2[varID] = vardatap;
-        }
+      std::swap(curFirst, curSecond);
     }
 
-  for (varID = 0; varID < nvars; varID++)
-    {
-      Free(vardata1[varID]);
-      Free(vardata2[varID]);
-    }
-
-  Free(vardata1);
-  Free(vardata2);
-
-  if (streamID2 != -1) cdoStreamClose(streamID2);
+  if (streamID2 != CDO_STREAM_UNDEF) cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
   if (tsIDo == 0) cdoWarning("date/time out of time axis, no time step interpolated!");
