@@ -24,7 +24,7 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
+
 #include "listarray.h"
 #include "util_files.h"
 
@@ -45,10 +45,10 @@ Intyear(void *process)
   const int nyears = listArrayInt.argvToInt(operatorArgc(), operatorArgv());
   int *iyears = listArrayInt.data();
 
-  std::vector<int> streamIDs(nyears);
+  std::vector<CdoStreamID> streamIDs(nyears);
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-  const int streamID2 = cdoStreamOpenRead(cdoStreamName(1));
+  CdoStreamID streamID1 = cdoOpenRead(0);
+  CdoStreamID streamID2 = cdoOpenRead(1);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = cdoStreamInqVlist(streamID2);
@@ -72,15 +72,15 @@ Intyear(void *process)
 
   const char *refname = cdoGetObase();
   filesuffix[0] = 0;
-  cdoGenFileSuffix(filesuffix, sizeof(filesuffix), pstreamInqFiletype(streamID1), vlistID1, refname);
+  cdoGenFileSuffix(filesuffix, sizeof(filesuffix), cdoInqFiletype(streamID1), vlistID1, refname);
 
   for (int iy = 0; iy < nyears; iy++)
     {
       sprintf(filename + nchars, "%04d", iyears[iy]);
       if (filesuffix[0]) sprintf(filename + nchars + 4, "%s", filesuffix);
 
-      streamIDs[iy] = cdoStreamOpenWrite(filename);
-      pstreamDefVlist(streamIDs[iy], vlistID3);
+      streamIDs[iy] = cdoOpenWrite(filename);
+      cdoDefVlist(streamIDs[iy], vlistID3);
     }
 
   int tsID = 0;
@@ -104,16 +104,16 @@ Intyear(void *process)
           const int64_t vdate3 = vdate1 - year1 * 10000 + iyears[iy] * 10000;
           taxisDefVdate(taxisID3, vdate3);
           taxisDefVtime(taxisID3, vtime);
-          pstreamDefTimestep(streamIDs[iy], tsID);
+          cdoDefTimestep(streamIDs[iy], tsID);
         }
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamInqRecord(streamID2, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID2, &varID, &levelID);
 
-          pstreamReadRecord(streamID1, array1.data(), &nmiss1);
-          pstreamReadRecord(streamID2, array2.data(), &nmiss2);
+          cdoReadRecord(streamID1, array1.data(), &nmiss1);
+          cdoReadRecord(streamID2, array2.data(), &nmiss2);
 
           gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
 
@@ -124,7 +124,7 @@ Intyear(void *process)
 
               nmiss3 = 0;
 
-              if (nmiss1 > 0 || nmiss2 > 0)
+              if (nmiss1 || nmiss2)
                 {
                   const double missval1 = vlistInqVarMissval(vlistID1, varID);
                   const double missval2 = vlistInqVarMissval(vlistID2, varID);
@@ -153,8 +153,8 @@ Intyear(void *process)
                   for (size_t i = 0; i < gridsize; i++) array3[i] = array1[i] * fac1 + array2[i] * fac2;
                 }
 
-              pstreamDefRecord(streamIDs[iy], varID, levelID);
-              pstreamWriteRecord(streamIDs[iy], array3.data(), nmiss3);
+              cdoDefRecord(streamIDs[iy], varID, levelID);
+              cdoWriteRecord(streamIDs[iy], array3.data(), nmiss3);
             }
         }
 
