@@ -29,7 +29,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 
 static void
 invertLonDes(int vlistID)
@@ -281,7 +280,7 @@ Invert(void *process)
   const int operfunc1 = cdoOperatorF1(operatorID);
   const int operfunc2 = cdoOperatorF2(operatorID);
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -298,9 +297,9 @@ Invert(void *process)
         invertLonDes(vlistID2);
     }
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
+  CdoStreamID streamID2 = cdoOpenWrite(1);
 
-  pstreamDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID2, vlistID2);
 
   const size_t gridsize = vlistGridsizeMax(vlistID1);
 
@@ -312,14 +311,14 @@ Invert(void *process)
     {
       taxisCopyTimestep(taxisID2, taxisID1);
 
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, array1.data(), &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, array1.data(), &nmiss);
 
-          pstreamDefRecord(streamID2, varID, levelID);
+          cdoDefRecord(streamID2, varID, levelID);
 
           if (operfunc1 == func_all || operfunc1 == func_fld)
             {
@@ -330,11 +329,11 @@ Invert(void *process)
               else
                 invertLonData(array1.data(), array2.data(), gridID1);
 
-              pstreamWriteRecord(streamID2, array2.data(), nmiss);
+              cdoWriteRecord(streamID2, array2.data(), nmiss);
             }
           else
             {
-              pstreamWriteRecord(streamID2, array1.data(), nmiss);
+              cdoWriteRecord(streamID2, array1.data(), nmiss);
             }
         }
       tsID++;
