@@ -24,7 +24,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 
 static void
 invertLevDes(int vlistID)
@@ -94,14 +93,14 @@ Invertlev(void *process)
 
   cdoInitialize(process);
 
-  const bool lcopy = UNCHANGED_RECORD;
+  const bool lcopy = unchangedRecord();
 
   cdoOperatorAdd("invertlev", func_all, 0, nullptr);
 
   const int operatorID = cdoOperatorID();
   const int operfunc = cdoOperatorF1(operatorID);
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -112,9 +111,9 @@ Invertlev(void *process)
 
   if (operfunc == func_all || operfunc == func_hrd) invertLevDes(vlistID2);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
+  CdoStreamID streamID2 = cdoOpenWrite(1);
 
-  pstreamDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID2, vlistID2);
 
   const size_t gridsizemax = vlistGridsizeMax(vlistID1);
   std::vector<double> array(gridsizemax);
@@ -146,11 +145,11 @@ Invertlev(void *process)
     {
       taxisCopyTimestep(taxisID2, taxisID1);
 
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           if (vardata[varID].size())
             {
@@ -158,20 +157,20 @@ Invertlev(void *process)
               const size_t gridsize = gridInqSize(gridID);
               const size_t offset = gridsize * levelID;
 
-              pstreamReadRecord(streamID1, &vardata[varID][offset], &nmiss);
+              cdoReadRecord(streamID1, &vardata[varID][offset], &nmiss);
               varnmiss[varID][levelID] = nmiss;
             }
           else
             {
-              pstreamDefRecord(streamID2, varID, levelID);
+              cdoDefRecord(streamID2, varID, levelID);
               if (lcopy)
                 {
-                  pstreamCopyRecord(streamID2, streamID1);
+                  cdoCopyRecord(streamID2, streamID1);
                 }
               else
                 {
-                  pstreamReadRecord(streamID1, array.data(), &nmiss);
-                  pstreamWriteRecord(streamID2, array.data(), nmiss);
+                  cdoReadRecord(streamID1, array.data(), &nmiss);
+                  cdoWriteRecord(streamID2, array.data(), nmiss);
                 }
             }
         }
@@ -186,12 +185,12 @@ Invertlev(void *process)
               nlevel = zaxisInqSize(zaxisID);
               for (levelID = 0; levelID < nlevel; levelID++)
                 {
-                  pstreamDefRecord(streamID2, varID, levelID);
+                  cdoDefRecord(streamID2, varID, levelID);
 
                   size_t offset = gridsize * (nlevel - levelID - 1);
                   nmiss = varnmiss[varID][nlevel - levelID - 1];
 
-                  pstreamWriteRecord(streamID2, &vardata[varID][offset], nmiss);
+                  cdoWriteRecord(streamID2, &vardata[varID][offset], nmiss);
                 }
             }
         }
