@@ -18,7 +18,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 
 #include "color.h"
 
@@ -249,7 +248,7 @@ FlowImagingLIC(int n_xres, int n_yres, float *pVectr, unsigned char *pNoise, uns
                float krnlen)
 {
 
-/// for each pixel in the 2D output LIC image///
+  /// for each pixel in the 2D output LIC image///
 
 #ifdef _OPENMP
 #pragma omp parallel for default(none) shared(pNoise, pVectr, n_xres, n_yres, pImage, p_LUT0, p_LUT1, krnlen)
@@ -261,23 +260,23 @@ FlowImagingLIC(int n_xres, int n_yres, float *pVectr, unsigned char *pNoise, uns
       int advcts;                       /// number of ADVeCTion stepS per direction (a step counter)
       int ADVCTS = (int) (krnlen * 3);  /// MAXIMUM number of advection steps per direction to break dead loops
 
-      float vctr_x;          /// x-component  of the VeCToR at the forefront point
-      float vctr_y;          /// y-component  of the VeCToR at the forefront point
-      float clp0_x;          /// x-coordinate of CLiP point 0 (current)
-      float clp0_y;          /// y-coordinate of CLiP point 0	(current)
-      float clp1_x;          /// x-coordinate of CLiP point 1 (next   )
-      float clp1_y;          /// y-coordinate of CLiP point 1 (next   )
-      float samp_x;          /// x-coordinate of the SAMPle in the current pixel
-      float samp_y;          /// y-coordinate of the SAMPle in the current pixel
-      float tmpLen;          /// TeMPorary LENgth of a trial clipped-segment
-      float segLen;          /// SEGment   LENgth
-      float curLen;          /// CURrent   LENgth of the streamline
-      float prvLen;          /// PReVious  LENgth of the streamline
-      float W_ACUM;          /// ACcuMulated Weight from the seed to the current streamline forefront
-      float texVal;          /// TEXture VALue
-      float smpWgt;          /// WeiGhT of the current SaMPle
-      float t_acum[2];       /// two ACcUMulated composite Textures for the two directions, perspectively
-      float w_acum[2];       /// two ACcUMulated Weighting values   for the two directions, perspectively
+      float vctr_x;             /// x-component  of the VeCToR at the forefront point
+      float vctr_y;             /// y-component  of the VeCToR at the forefront point
+      float clp0_x;             /// x-coordinate of CLiP point 0 (current)
+      float clp0_y;             /// y-coordinate of CLiP point 0	(current)
+      float clp1_x;             /// x-coordinate of CLiP point 1 (next   )
+      float clp1_y;             /// y-coordinate of CLiP point 1 (next   )
+      float samp_x;             /// x-coordinate of the SAMPle in the current pixel
+      float samp_y;             /// y-coordinate of the SAMPle in the current pixel
+      float tmpLen;             /// TeMPorary LENgth of a trial clipped-segment
+      float segLen;             /// SEGment   LENgth
+      float curLen;             /// CURrent   LENgth of the streamline
+      float prvLen;             /// PReVious  LENgth of the streamline
+      float W_ACUM;             /// ACcuMulated Weight from the seed to the current streamline forefront
+      float texVal;             /// TEXture VALue
+      float smpWgt;             /// WeiGhT of the current SaMPle
+      float t_acum[2];          /// two ACcUMulated composite Textures for the two directions, perspectively
+      float w_acum[2];          /// two ACcUMulated Weighting values   for the two directions, perspectively
       float *wgtLUT = nullptr;  /// WeiGhT Look Up Table pointing to the target filter LUT
       float len2ID = (DISCRETE_FILTER_SIZE - 1) / krnlen;  /// map a curve LENgth TO an ID in the LUT
       for (int i = 0; i < n_xres; i++)
@@ -454,7 +453,7 @@ Lic(void *process)
   if (status != 0) cdoAbort("Error during read of color palette table %s", cpt_file);
 
   const char *obasename = cdoGetStreamName(1);
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   int vlistID1 = cdoStreamInqVlist(streamID1);
 
@@ -505,15 +504,15 @@ Lic(void *process)
     {
       /*
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
       */
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           if (varID == varID1 || varID == varID2)
             {
-              pstreamReadRecord(streamID1, array1.data(), &nmiss);
+              cdoReadRecord(streamID1, array1.data(), &nmiss);
               if (nmiss) cdoAbort("Missing values unsupported!");
 
               gridsize = gridInqSize(gridID);
