@@ -22,8 +22,7 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "grid.h"
-#include "pstream_int.h"
+#include <mpim_grid.h>
 
 #ifdef HAVE_LIBMAGICS
 
@@ -212,7 +211,7 @@ maggraph(const char *plotfile, const char *varname, const char *varunits, long n
     {
       ntime_steps = nts[0];
       fprintf(stderr, " %ld %ld\n", nfiles, ntime_steps);
-      fprintf(stderr, "STAT  %d\n", (int)stat);
+      fprintf(stderr, "STAT  %d\n", (int) stat);
     }
 
   if (stat)
@@ -283,7 +282,7 @@ maggraph(const char *plotfile, const char *varname, const char *varunits, long n
 
   if (DBG)
     {
-      fprintf(stderr, "STAT  %d\n", (int)stat);
+      fprintf(stderr, "STAT  %d\n", (int) stat);
     }
 
   if (stat)
@@ -378,7 +377,7 @@ maggraph(const char *plotfile, const char *varname, const char *varunits, long n
          Construct the date_time_str array
       */
 
-      if (DBG) fprintf(stderr, "STAT  %d\n", (int)stat);
+      if (DBG) fprintf(stderr, "STAT  %d\n", (int) stat);
 
       for (fileID = 0; fileID < nfiles; fileID++)
         {
@@ -930,7 +929,7 @@ Maggraph(void *process)
     {
       if (DBG) fprintf(stderr, " file %d is %s\n", fileID, cdoGetStreamName(fileID));
 
-      int streamID = cdoStreamOpenRead(cdoStreamName(fileID));
+      CdoStreamID streamID = cdoOpenRead((fileID));
 
       int vlistID = cdoStreamInqVlist(streamID);
       int taxisID = vlistInqTaxis(vlistID);
@@ -981,8 +980,8 @@ Maggraph(void *process)
             }
 
           double val;
-          pstreamInqRecord(streamID, &varID, &levelID);
-          pstreamReadRecord(streamID, &val, &nmiss);
+          cdoInqRecord(streamID, &varID, &levelID);
+          cdoReadRecord(streamID, &val, &nmiss);
           datatab[fileID][tsID] = val;
           vdate[fileID][tsID] = taxisInqVdate(taxisID);
           vtime[fileID][tsID] = taxisInqVtime(taxisID);
