@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -21,8 +21,7 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "grid.h"
-#include "pstream_int.h"
+#include <mpim_grid.h>
 
 #ifdef HAVE_LIBMAGICS
 
@@ -32,6 +31,7 @@
 #include "results_template_parser.h"
 #include "StringUtilities.h"
 #include "util_string.h"
+#include "printinfo.h"
 
 #define DBG 0
 
@@ -120,7 +120,7 @@ const char *STD_COLOUR_TABLE[] = { "red",
                                    "purple",
                                    "white" };
 
-char **USR_COLOUR_TABLE = NULL;
+char **USR_COLOUR_TABLE = nullptr;
 
 int STD_COLOUR_COUNT = sizeof(STD_COLOUR_TABLE) / sizeof(char *);
 int USR_COLOUR_COUNT = 0;
@@ -170,12 +170,12 @@ int checkdevice(char *device_in);
 int checkprojection(char *projection_in);
 
 /* Magics default values */
-int COUNT = 10, isRGB = FALSE, THICKNESS = 1, NUM_LEVELS = 0, FILE_SPLIT = FALSE;
-double YMIN = 1.0e+200, YMAX = -1.0e+200, INTERVAL = 8.0, RESOLUTION = 10.0f, *LEV_LIST = NULL;
+int COUNT = 10, isRGB = false, THICKNESS = 1, NUM_LEVELS = 0, FILE_SPLIT = false;
+double YMIN = 1.0e+200, YMAX = -1.0e+200, INTERVAL = 8.0, RESOLUTION = 10.0f, *LEV_LIST = nullptr;
 double LAT_MIN = 1.0e+200, LAT_MAX = -1.e+200;
 double LON_MIN = 1.0e+200, LON_MAX = -1.e+200;
-const char *COLOUR = NULL, *COLOUR_MIN = NULL, *COLOUR_MAX = NULL, *STYLE = NULL, *DEVICE = NULL, *COLOUR_TRIAD = NULL,
-           *PROJECTION = NULL;
+const char *COLOUR = nullptr, *COLOUR_MIN = nullptr, *COLOUR_MAX = nullptr, *STYLE = nullptr, *DEVICE = nullptr,
+           *COLOUR_TRIAD = nullptr, *PROJECTION = nullptr;
 
 static void
 magplot(const char *plotfile, int operatorID, const char *varname, const char *units, long nlon, long nlat, double *grid_center_lon,
@@ -188,7 +188,7 @@ magplot(const char *plotfile, int operatorID, const char *varname, const char *u
   char *titlename;
   int j, split_str_count;
   const char *sep_char = "=";
-  char **split_str = NULL;
+  char **split_str = nullptr;
   char tempname[256];
 
   if (DBG)
@@ -202,32 +202,32 @@ magplot(const char *plotfile, int operatorID, const char *varname, const char *u
         {
           split_str_count = 0;
           sep_char = "=";
-          split_str_count = StringSplitWithSeperator(params[i], sep_char, &split_str);
+          split_str_count = cstrSplitWithSeperator(params[i], sep_char, &split_str);
 
-          if (!strcmp(split_str[0], "min")) fprintf(stderr, "Min Val %g\n", YMIN);
-          if (!strcmp(split_str[0], "max")) fprintf(stderr, "Max Val %g\n", YMAX);
-          // if ( !strcmp( split_str[0],"resolution" ) ) fprintf(
+          if (cstrIsEqual(split_str[0], "min")) fprintf(stderr, "Min Val %g\n", YMIN);
+          if (cstrIsEqual(split_str[0], "max")) fprintf(stderr, "Max Val %g\n", YMAX);
+          // if ( cstrIsEqual( split_str[0],"resolution" ) ) fprintf(
           // stderr,"RESOLUTION %g\n",RESOLUTION );
-          if (!strcmp(split_str[0], "colour")) fprintf(stderr, "COLOUR %s\n", COLOUR);
-          if (!strcmp(split_str[0], "colour_min")) fprintf(stderr, "COLOUR %s\n", COLOUR_MIN);
-          if (!strcmp(split_str[0], "colour_max")) fprintf(stderr, "COLOUR %s\n", COLOUR_MAX);
-          if (!strcmp(split_str[0], "interval")) fprintf(stderr, "INTERVAL %f\n", INTERVAL);
-          if (!strcmp(split_str[0], "count")) fprintf(stderr, "COUNT %d\n", COUNT);
+          if (cstrIsEqual(split_str[0], "colour")) fprintf(stderr, "COLOUR %s\n", COLOUR);
+          if (cstrIsEqual(split_str[0], "colour_min")) fprintf(stderr, "COLOUR %s\n", COLOUR_MIN);
+          if (cstrIsEqual(split_str[0], "colour_max")) fprintf(stderr, "COLOUR %s\n", COLOUR_MAX);
+          if (cstrIsEqual(split_str[0], "interval")) fprintf(stderr, "INTERVAL %f\n", INTERVAL);
+          if (cstrIsEqual(split_str[0], "count")) fprintf(stderr, "COUNT %d\n", COUNT);
 
-          if (!strcmp(split_str[0], "list"))
+          if (cstrIsEqual(split_str[0], "list"))
             {
               for (j = 0; j < split_str_count; j++) fprintf(stderr, "LIST %f\n", LEV_LIST[j]);
             }
 
-          if (!strcmp(split_str[0], "thickness")) fprintf(stderr, "THICKNESS %d\n", THICKNESS);
-          if (!strcmp(split_str[0], "style")) fprintf(stderr, "STYLE %s\n", STYLE);
-          if (!strcmp(split_str[0], "device")) fprintf(stderr, "DEVICE %s\n", DEVICE);
-          if (!strcmp(split_str[0], "step_freq")) fprintf(stderr, "STEP_FREQ %d\n", STEP_FREQ);
-          if (!strcmp(split_str[0], "lat_min")) fprintf(stderr, "Lat Min Val %g\n", LAT_MIN);
-          if (!strcmp(split_str[0], "lat_max")) fprintf(stderr, "Lat Max Val %g\n", LAT_MAX);
-          if (!strcmp(split_str[0], "lon_min")) fprintf(stderr, "Lon Min Val %g\n", LON_MIN);
-          if (!strcmp(split_str[0], "lon_max")) fprintf(stderr, "Lon Max Val %g\n", LON_MAX);
-          if (!strcmp(split_str[0], "projection")) fprintf(stderr, "PROJECTION %s\n", PROJECTION);
+          if (cstrIsEqual(split_str[0], "thickness")) fprintf(stderr, "THICKNESS %d\n", THICKNESS);
+          if (cstrIsEqual(split_str[0], "style")) fprintf(stderr, "STYLE %s\n", STYLE);
+          if (cstrIsEqual(split_str[0], "device")) fprintf(stderr, "DEVICE %s\n", DEVICE);
+          if (cstrIsEqual(split_str[0], "step_freq")) fprintf(stderr, "STEP_FREQ %d\n", STEP_FREQ);
+          if (cstrIsEqual(split_str[0], "lat_min")) fprintf(stderr, "Lat Min Val %g\n", LAT_MIN);
+          if (cstrIsEqual(split_str[0], "lat_max")) fprintf(stderr, "Lat Max Val %g\n", LAT_MAX);
+          if (cstrIsEqual(split_str[0], "lon_min")) fprintf(stderr, "Lon Min Val %g\n", LON_MIN);
+          if (cstrIsEqual(split_str[0], "lon_max")) fprintf(stderr, "Lon Max Val %g\n", LON_MAX);
+          if (cstrIsEqual(split_str[0], "projection")) fprintf(stderr, "PROJECTION %s\n", PROJECTION);
 
           Free(split_str);
         }
@@ -586,7 +586,7 @@ init_MAGICS()
    * xml file  ** begin ** */
   mag_setc("page_id_line", "off");
   mag_setc("output_name_first_page_number", "off");
-  if (FILE_SPLIT == TRUE) mag_setc("output_ps_split", "on");
+  if (FILE_SPLIT == true) mag_setc("output_ps_split", "on");
 }
 
 static void
@@ -600,11 +600,11 @@ static void
 VerifyPlotParameters(int num_param, char **param_names, int opID)
 {
   int i, j, k;
-  int found = FALSE, syntax = TRUE, halt_flag = FALSE,
-      /* file_found = TRUE, */ split_str_count;
+  bool found = false, syntax = true, halt_flag = false;
+  int split_str_count;
   int param_count = 0;
-  const char **params = NULL;
-  char **split_str = NULL, **split_str1 = NULL;
+  const char **params = nullptr;
+  char **split_str = nullptr, **split_str1 = nullptr;
   const char *sep_char = "=";
   char *temp_str;
   const char orig_char = ';', rep_char = ',';
@@ -622,9 +622,9 @@ VerifyPlotParameters(int num_param, char **param_names, int opID)
   for (i = 0; i < num_param; ++i)
     {
       split_str_count = 0;
-      found = FALSE;
-      syntax = TRUE;
-      split_str_count = StringSplitWithSeperator(param_names[i], sep_char, &split_str);
+      found = false;
+      syntax = true;
+      split_str_count = cstrSplitWithSeperator(param_names[i], sep_char, &split_str);
 
       if (DBG) fprintf(stderr, "Verifying params!\n");
 
@@ -649,91 +649,92 @@ VerifyPlotParameters(int num_param, char **param_names, int opID)
 
           for (j = 0; j < param_count; ++j)
             {
-              if (!strcmp(split_str[0], params[j]))
+              if (cstrIsEqual(split_str[0], params[j]))
                 {
-                  found = TRUE;
-                  if (!strcmp(split_str[0], "colour") || !strcmp(split_str[0], "style") || !strcmp(split_str[0], "colour_min")
-                      || !strcmp(split_str[0], "colour_max") || !strcmp(split_str[0], "RGB")
-                      || !strcmp(split_str[0], "colour_triad") || !strcmp(split_str[0], "device")
-                      || !strcmp(split_str[0], "file_split") || !strcmp(split_str[0], "projection"))
+                  found = true;
+                  if (cstrIsEqual(split_str[0], "colour") || cstrIsEqual(split_str[0], "style")
+                      || cstrIsEqual(split_str[0], "colour_min") || cstrIsEqual(split_str[0], "colour_max")
+                      || cstrIsEqual(split_str[0], "RGB") || cstrIsEqual(split_str[0], "colour_triad")
+                      || cstrIsEqual(split_str[0], "device") || cstrIsEqual(split_str[0], "file_split")
+                      || cstrIsEqual(split_str[0], "projection"))
                     {
-                      if (IsNumeric(split_str[1]))
-                        syntax = FALSE;
+                      if (cstrIsNumeric(split_str[1]))
+                        syntax = false;
                       else
                         {
-                          if (!strcmp(split_str[0], "RGB") || !strcmp(split_str[0], "file_split"))
+                          if (cstrIsEqual(split_str[0], "RGB") || cstrIsEqual(split_str[0], "file_split"))
                             {
                               temp_str = strdup(split_str[1]);
-                              StrToUpperCase(temp_str);
-                              if (strcmp(temp_str, "TRUE") && strcmp(temp_str, "FALSE"))
-                                syntax = FALSE;
+                              cstrToUpperCase(temp_str);
+                              if (strcmp(temp_str, "true") && strcmp(temp_str, "false"))
+                                syntax = false;
                               else
                                 {
-                                  if (!strcmp(split_str[0], "RGB"))
+                                  if (cstrIsEqual(split_str[0], "RGB"))
                                     {
-                                      if (!strcmp(temp_str, "TRUE"))
-                                        isRGB = TRUE;
+                                      if (cstrIsEqual(temp_str, "true"))
+                                        isRGB = true;
                                       else
-                                        isRGB = FALSE;
+                                        isRGB = false;
                                     }
-                                  else if (!strcmp(split_str[0], "file_split"))
+                                  else if (cstrIsEqual(split_str[0], "file_split"))
                                     {
-                                      if (!strcmp(temp_str, "TRUE"))
-                                        FILE_SPLIT = TRUE;
+                                      if (cstrIsEqual(temp_str, "true"))
+                                        FILE_SPLIT = true;
                                       else
-                                        FILE_SPLIT = FALSE;
+                                        FILE_SPLIT = false;
                                     }
                                 }
                             }
-                          else if (!strcmp(split_str[0], "style"))
+                          else if (cstrIsEqual(split_str[0], "style"))
                             {
-                              if (checkstyle(split_str[1])) syntax = FALSE;
+                              if (checkstyle(split_str[1])) syntax = false;
                             }
-                          else if (!strcmp(split_str[0], "colour") || !strcmp(split_str[0], "colour_min")
-                                   || !strcmp(split_str[0], "colour_max"))
+                          else if (cstrIsEqual(split_str[0], "colour") || cstrIsEqual(split_str[0], "colour_min")
+                                   || cstrIsEqual(split_str[0], "colour_max"))
                             {
 
                               if (checkcolour(split_str[1]))
-                                syntax = FALSE;
+                                syntax = false;
                               else
                                 {
-                                  if (!strcmp(split_str[0], "colour"))
+                                  if (cstrIsEqual(split_str[0], "colour"))
                                     {
                                       temp_str = strdup(split_str[1]);
                                       if (!isRGB)
-                                        StrToLowerCase(temp_str);
+                                        cstrToLowerCase(temp_str);
                                       else
                                         {
-                                          StrToUpperCase(temp_str);
-                                          StrReplaceChar(temp_str, orig_char, rep_char); /* replace ';' in RGB
+                                          cstrToUpperCase(temp_str);
+                                          cstrReplaceChar(temp_str, orig_char, rep_char); /* replace ';' in RGB
                                                                                             format to ',' */
                                         }
                                       COLOUR = temp_str;
                                       if (DBG) fprintf(stderr, "COLOUR %s\n", COLOUR);
                                     }
-                                  if (!strcmp(split_str[0], "colour_min"))
+                                  if (cstrIsEqual(split_str[0], "colour_min"))
                                     {
                                       temp_str = strdup(split_str[1]);
                                       if (!isRGB)
-                                        StrToLowerCase(temp_str);
+                                        cstrToLowerCase(temp_str);
                                       else
                                         {
-                                          StrToUpperCase(temp_str);
-                                          StrReplaceChar(temp_str, orig_char, rep_char); /* replace ';' in RGB
+                                          cstrToUpperCase(temp_str);
+                                          cstrReplaceChar(temp_str, orig_char, rep_char); /* replace ';' in RGB
                                                                                             format to ',' */
                                         }
                                       COLOUR_MIN = temp_str;
                                       if (DBG) fprintf(stderr, "COLOUR %s\n", COLOUR_MIN);
                                     }
-                                  if (!strcmp(split_str[0], "colour_max"))
+                                  if (cstrIsEqual(split_str[0], "colour_max"))
                                     {
                                       temp_str = strdup(split_str[1]);
                                       if (!isRGB)
-                                        StrToLowerCase(temp_str);
+                                        cstrToLowerCase(temp_str);
                                       else
                                         {
-                                          StrToUpperCase(temp_str);
-                                          StrReplaceChar(temp_str, orig_char, rep_char); /* replace ';' in RGB
+                                          cstrToUpperCase(temp_str);
+                                          cstrReplaceChar(temp_str, orig_char, rep_char); /* replace ';' in RGB
                                                                                             format to ',' */
                                         }
                                       COLOUR_MAX = temp_str;
@@ -741,96 +742,95 @@ VerifyPlotParameters(int num_param, char **param_names, int opID)
                                     }
                                 }
                             }
-                          else if (!strcmp(split_str[0], "device"))
+                          else if (cstrIsEqual(split_str[0], "device"))
                             {
-                              if (checkdevice(split_str[1])) syntax = FALSE;
+                              if (checkdevice(split_str[1])) syntax = false;
                             }
-                          else if (!strcmp(split_str[0], "colour_triad"))
+                          else if (cstrIsEqual(split_str[0], "colour_triad"))
                             {
                               temp_str = strdup(split_str[1]);
-                              StrToUpperCase(temp_str);
+                              cstrToUpperCase(temp_str);
                               if (strcmp(temp_str, "CW") && strcmp(temp_str, "ACW"))
-                                syntax = FALSE;
+                                syntax = false;
                               else
                                 {
                                   if (DBG) fprintf(stderr, "TRIAD check  %s!\n", temp_str);
-                                  if (!strcmp(temp_str, "CW"))
+                                  if (cstrIsEqual(temp_str, "CW"))
                                     COLOUR_TRIAD = "clockwise";
                                   else
                                     COLOUR_TRIAD = "anti_clockwise";
                                 }
                             }
-                          else if (!strcmp(split_str[0], "projection"))
+                          else if (cstrIsEqual(split_str[0], "projection"))
                             {
-                              if (checkprojection(split_str[1])) syntax = FALSE;
+                              if (checkprojection(split_str[1])) syntax = false;
                             }
                         }
                     }
 
-                  if (!strcmp(split_str[0], "min") || !strcmp(split_str[0], "max") || !strcmp(split_str[0], "lat_min")
-                      || !strcmp(split_str[0], "lat_max") || !strcmp(split_str[0], "lon_min") || !strcmp(split_str[0], "lon_max")
-                      || !strcmp(split_str[0], "count") || !strcmp(split_str[0], "interval") || !strcmp(split_str[0], "thickness")
-                      || !strcmp(split_str[0], "resolution") || !strcmp(split_str[0], "step_freq"))
+                  if (cstrIsEqual(split_str[0], "min") || cstrIsEqual(split_str[0], "max") || cstrIsEqual(split_str[0], "lat_min")
+                      || cstrIsEqual(split_str[0], "lat_max") || cstrIsEqual(split_str[0], "lon_min")
+                      || cstrIsEqual(split_str[0], "lon_max") || cstrIsEqual(split_str[0], "count")
+                      || cstrIsEqual(split_str[0], "interval") || cstrIsEqual(split_str[0], "thickness")
+                      || cstrIsEqual(split_str[0], "resolution") || cstrIsEqual(split_str[0], "step_freq"))
                     {
-                      if (!IsNumeric(split_str[1]))
-                        syntax = FALSE;
+                      if (!cstrIsNumeric(split_str[1]))
+                        syntax = false;
                       else
                         {
-                          if (!strcmp(split_str[0], "min"))
+                          if (cstrIsEqual(split_str[0], "min"))
                             {
                               YMIN = atof(split_str[1]);
                             }
-                          if (!strcmp(split_str[0], "max"))
+                          if (cstrIsEqual(split_str[0], "max"))
                             {
                               YMAX = atof(split_str[1]);
                             }
-                          if (!strcmp(split_str[0], "count"))
+                          if (cstrIsEqual(split_str[0], "count"))
                             {
                               COUNT = atoi(split_str[1]);
                             }
-                          if (!strcmp(split_str[0], "interval"))
+                          if (cstrIsEqual(split_str[0], "interval"))
                             {
                               INTERVAL = atof(split_str[1]);
                             }
-                          if (!strcmp(split_str[0], "thickness"))
+                          if (cstrIsEqual(split_str[0], "thickness"))
                             {
                               THICKNESS = atoi(split_str[1]);
                             }
-                          if (!strcmp(split_str[0], "resolution"))
+                          if (cstrIsEqual(split_str[0], "resolution"))
                             {
                               RESOLUTION = atoi(split_str[1]);
                             }
-                          if (!strcmp(split_str[0], "step_freq"))
+                          if (cstrIsEqual(split_str[0], "step_freq"))
                             {
                               STEP_FREQ = atoi(split_str[1]);
                             }
-                          if (!strcmp(split_str[0], "lat_min"))
+                          if (cstrIsEqual(split_str[0], "lat_min"))
                             {
                               LAT_MIN = atof(split_str[1]);
                             }
-                          if (!strcmp(split_str[0], "lat_max"))
+                          if (cstrIsEqual(split_str[0], "lat_max"))
                             {
                               LAT_MAX = atof(split_str[1]);
                             }
-                          if (!strcmp(split_str[0], "lon_min"))
+                          if (cstrIsEqual(split_str[0], "lon_min"))
                             {
                               LON_MIN = atof(split_str[1]);
                             }
-                          if (!strcmp(split_str[0], "lon_max"))
+                          if (cstrIsEqual(split_str[0], "lon_max"))
                             {
                               LON_MAX = atof(split_str[1]);
                             }
                         }
                     }
 
-                  if (!strcmp(split_str[0], "colour_table"))
+                  if (cstrIsEqual(split_str[0], "colour_table"))
                     {
-                      if ((fp = fopen(split_str[1], "r")) == NULL)
+                      if ((fp = fopen(split_str[1], "r")) == nullptr)
                         {
-                          fprintf(stderr, "Input Color Table File not found in "
-                                          "specified path '%s'\n",
-                                  split_str[1]);
-                          halt_flag = TRUE;
+                          fprintf(stderr, "Input Color Table File not found in specified path '%s'\n", split_str[1]);
+                          halt_flag = true;
                         }
                       else
                         {
@@ -838,21 +838,21 @@ VerifyPlotParameters(int num_param, char **param_names, int opID)
                         }
                     }
 
-                  if (!strcmp(split_str[0], "list"))
+                  if (cstrIsEqual(split_str[0], "list"))
                     {
                       sep_char = ";";
-                      split_str_count = StringSplitWithSeperator(split_str[1], sep_char, &split_str1);
+                      split_str_count = cstrSplitWithSeperator(split_str[1], sep_char, &split_str1);
                       if (!split_str_count)
                         {
-                          syntax = FALSE;
+                          syntax = false;
                         }
                       else
                         {
                           for (k = 0; k < split_str_count; k++)
                             {
-                              if (!IsNumeric(split_str1[k])) syntax = FALSE;
+                              if (!cstrIsNumeric(split_str1[k])) syntax = false;
                             }
-                          if (syntax == TRUE)
+                          if (syntax == true)
                             {
                               NUM_LEVELS = split_str_count;
                               LEV_LIST = (double *) Malloc(sizeof(double) * split_str_count);
@@ -865,32 +865,29 @@ VerifyPlotParameters(int num_param, char **param_names, int opID)
                         }
                     }
                   sep_char = "=";
-                } /*** if( !strcmp( split_str[0], params[j] ) )  ***/
+                } /*** if( cstrIsEqual( split_str[0], params[j] ) )  ***/
             }     /*** Loop over param count ***/
         }         /*** ( split_str_count > 1 ) ***/
       else
         {
-          syntax = FALSE;
+          syntax = false;
         }
 
-      if (found == FALSE)
+      if (!found)
         {
-          halt_flag = TRUE;
+          halt_flag = true;
           fprintf(stderr, "Invalid parameter  '%s'\n", param_names[i]);
         }
-      if (found == TRUE && syntax == FALSE)
+      if (found && syntax == false)
         {
-          halt_flag = TRUE;
+          halt_flag = true;
           fprintf(stderr, "Invalid parameter specification  '%s'\n", param_names[i]);
         }
 
       if (split_str) Free(split_str);
     } /*** Loop over params ****/
 
-  if (halt_flag == TRUE)
-    {
-      exit(0);
-    }
+  if (halt_flag) exit(0);
 }
 
 int
@@ -899,7 +896,7 @@ checkcolour(char *colour_in)
   int i, n;
   int split_str_count;
   const char *sep_char = ",";
-  char **split_str = NULL;
+  char **split_str = nullptr;
   float rgb_values[3];
   char temp[256];
   char *ref;
@@ -908,10 +905,9 @@ checkcolour(char *colour_in)
 
   if (isRGB)
     {
-      if (strchr(colour_in, ';') == NULL || strstr(colour_in, "RGB(") == NULL)
+      if (strchr(colour_in, ';') == nullptr || strstr(colour_in, "RGB(") == nullptr)
         {
-          cdoWarning("Found 'RGB=true',Specify Colour in 'RGB(r;g;b)' ( where "
-                     "r,g,b in [0.0,1.0] ) format!");
+          cdoWarning("Found 'RGB=true',Specify Colour in 'RGB(r;g;b)' ( where r,g,b in [0.0,1.0] ) format!");
           Free(split_str);
           return 1;
         }
@@ -934,7 +930,7 @@ checkcolour(char *colour_in)
       if (DBG) fprintf(stdout, "  count %d  modified color %s \n", (int) strlen(temp), temp);
 
       sep_char = ";";
-      split_str_count = StringSplitWithSeperator(temp, sep_char, &split_str);
+      split_str_count = cstrSplitWithSeperator(temp, sep_char, &split_str);
 
       if (split_str_count != 3)
         {
@@ -958,18 +954,17 @@ checkcolour(char *colour_in)
     }
   else
     {
-      if (strchr(colour_in, ';') != NULL || strstr(colour_in, "RGB(") != NULL)
+      if (strchr(colour_in, ';') != nullptr || strstr(colour_in, "RGB(") != nullptr)
         {
-          cdoWarning("Found Colour with 'RGB(r;g;b)' format, set parameter "
-                     "RGB='true' !");
+          cdoWarning("Found Colour with 'RGB(r;g;b)' format, set parameter RGB='true' !");
           Free(split_str);
           return 1;
         }
 
-      StrToLowerCase(colour_in);
+      cstrToLowerCase(colour_in);
       for (i = 0; i < STD_COLOUR_COUNT; i++)
         {
-          if (!strcmp(STD_COLOUR_TABLE[i], colour_in)) return 0;
+          if (cstrIsEqual(STD_COLOUR_TABLE[i], colour_in)) return 0;
         }
       cdoWarning("Specified Colour not in Standard colour list, resetting to "
                  "blue(default colour)!");
@@ -1025,7 +1020,7 @@ ReadColourTable(char *filepath)
 
       if (!checkcolour(temp_table[i]))
         {
-          if (isRGB) StrReplaceChar(temp_table[i], orig_char, rep_char); /* replace ';' in RGB format to ',' */
+          if (isRGB) cstrReplaceChar(temp_table[i], orig_char, rep_char); /* replace ';' in RGB format to ',' */
 
           if (DBG) fprintf(stdout, "Before appending %s\n", temp_table[i]);
 
@@ -1052,15 +1047,15 @@ ReadColourTable(char *filepath)
 int
 checkstyle(char *style_in)
 {
-  int i, found = FALSE;
-  StrToUpperCase(style_in);
-  for (i = 0; i < STYLE_COUNT; i++)
+  bool found = false;
+  cstrToUpperCase(style_in);
+  for (int i = 0; i < STYLE_COUNT; i++)
     {
       if (DBG) fprintf(stderr, "Input %s ref %s\n", style_in, STYLE_TABLE[i]);
 
-      if (!strcmp(STYLE_TABLE[i], style_in))
+      if (cstrIsEqual(STYLE_TABLE[i], style_in))
         {
-          found = TRUE;
+          found = true;
           STYLE = style_in;
           return 0;
         }
@@ -1074,20 +1069,20 @@ checkstyle(char *style_in)
 int
 checkdevice(char *device_in)
 {
-  int i, found = FALSE;
-  StrToUpperCase(device_in);
-  for (i = 0; i < DEVICE_COUNT; i++)
+  bool found = false;
+  cstrToUpperCase(device_in);
+  for (int i = 0; i < DEVICE_COUNT; i++)
     {
       if (DBG) fprintf(stderr, "Input %s ref %s\n", device_in, DEVICE_TABLE[i]);
 
-      if (!strcmp(DEVICE_TABLE[i], device_in))
+      if (cstrIsEqual(DEVICE_TABLE[i], device_in))
         {
-          found = TRUE;
+          found = true;
 
           DEVICE = device_in;
           if (DBG) fprintf(stderr, "DEVICE %s\n", DEVICE);
 
-          if (!strcmp("GIF_ANIMATION", device_in) || !strcmp("KML", device_in))
+          if (cstrIsEqual("GIF_ANIMATION", device_in) || cstrIsEqual("KML", device_in))
             {
               ANIM_FLAG = 1;
               STEP_FREQ = 1;
@@ -1104,17 +1099,17 @@ checkdevice(char *device_in)
 int
 checkprojection(char *projection_in)
 {
-  int i, found = FALSE;
+  bool found = false;
 
-  /*StrToUpperCase( projection_in );*/
+  /*cstrToUpperCase( projection_in );*/
 
-  for (i = 0; i < PROJECTION_COUNT; i++)
+  for (int i = 0; i < PROJECTION_COUNT; i++)
     {
       if (DBG) fprintf(stderr, "Input %s ref %s\n", projection_in, PROJECTION_TABLE[i]);
 
-      if (!strcmp(PROJECTION_TABLE[i], projection_in))
+      if (cstrIsEqual(PROJECTION_TABLE[i], projection_in))
         {
-          found = TRUE;
+          found = true;
           PROJECTION = projection_in;
           return 0;
         }
@@ -1150,9 +1145,9 @@ Magplot(void *process)
   int nparam = operatorArgc();
   char **pnames = operatorArgv();
 
-  CONTOUR = cdoOperatorAdd("contour", 0, 0, NULL);
-  SHADED = cdoOperatorAdd("shaded", 0, 0, NULL);
-  GRFILL = cdoOperatorAdd("grfill", 0, 0, NULL);
+  CONTOUR = cdoOperatorAdd("contour", 0, 0, nullptr);
+  SHADED = cdoOperatorAdd("shaded", 0, 0, nullptr);
+  GRFILL = cdoOperatorAdd("grfill", 0, 0, nullptr);
 
   int operatorID = cdoOperatorID();
 
@@ -1166,7 +1161,7 @@ Magplot(void *process)
       VerifyPlotParameters(nparam, pnames, operatorID);
     }
 
-  int streamID = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID = cdoOpenRead(0);
 
   int vlistID = cdoStreamInqVlist(streamID);
   int taxisID = vlistInqTaxis(vlistID);
@@ -1267,8 +1262,8 @@ Magplot(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID, &varID, &levelID);
-          pstreamReadRecord(streamID, array, &nmiss);
+          cdoInqRecord(streamID, &varID, &levelID);
+          cdoReadRecord(streamID, array, &nmiss);
 
           if (nmiss) cdoSetNAN(vlistInqVarMissval(vlistID, varID), gridsize, array);
 
@@ -1288,8 +1283,8 @@ Magplot(void *process)
                 }
 
               if (DBG) fprintf(stderr, "Plot %d\n", varID);
-              magplot(cdoGetStreamName(1).c_str(), operatorID, varname, units, nlon, nlat, grid_center_lon, grid_center_lat, array,
-                      nparam, pnames, datetimestr, lregular);
+              magplot(cdoGetStreamName(1), operatorID, varname, units, nlon, nlat, grid_center_lon, grid_center_lat, array, nparam,
+                      pnames, datetimestr, lregular);
             }
           else
             fprintf(stderr, "operator not implemented\n");
@@ -1319,11 +1314,11 @@ Magplot(void *process)
 
   if (ANIM_FLAG)
     {
-      if (FILE_SPLIT == TRUE) cdoWarning("File split parameter ignored!!!");
+      if (FILE_SPLIT == true) cdoWarning("File split parameter ignored!!!");
     }
   quit_MAGICS();
 
-  pstreamClose(streamID);
+  cdoStreamClose(streamID);
 
   if (array) Free(array);
   if (grid_center_lon) Free(grid_center_lon);
@@ -1338,5 +1333,5 @@ Magplot(void *process)
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
