@@ -21,8 +21,8 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "grid.h"
-#include "pstream_int.h"
+#include <mpim_grid.h>
+
 #include "printinfo.h"
 
 #ifdef HAVE_LIBMAGICS
@@ -253,7 +253,8 @@ VerifyVectorParameters(int num_param, char **param_names, int opID)
                 {
                   found = true;
 
-                  if (cstrIsEqual(split_str[0], "thin_fac") || cstrIsEqual(split_str[0], "unit_vec") || cstrIsEqual(split_str[0], "step_freq"))
+                  if (cstrIsEqual(split_str[0], "thin_fac") || cstrIsEqual(split_str[0], "unit_vec")
+                      || cstrIsEqual(split_str[0], "step_freq"))
                     {
                       if (!cstrIsNumeric(split_str[1])) syntax = false;
                     }
@@ -335,7 +336,7 @@ Magvector(void *process)
       VerifyVectorParameters(nparam, pnames, operatorID);
     }
 
-  int streamID = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID = cdoOpenRead(0);
 
   int vlistID = cdoStreamInqVlist(streamID);
   int taxisID = vlistInqTaxis(vlistID);
@@ -409,7 +410,7 @@ Magvector(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID, &varID, &levelID);
+          cdoInqRecord(streamID, &varID, &levelID);
 
           vlistInqVarName(vlistID, varID, varname);
 
@@ -419,7 +420,7 @@ Magvector(void *process)
                                                                           'var131, as per NC 'u' */
                 {
                   if (DBG) fprintf(stderr, "Found U VEL in Varname %s\n", varname);
-                  pstreamReadRecord(streamID, uarray, &nmiss);
+                  cdoReadRecord(streamID, uarray, &nmiss);
                   if (nmiss) cdoSetNAN(vlistInqVarMissval(vlistID, varID), gridsize, uarray);
                   found++;
                 }
@@ -427,7 +428,7 @@ Magvector(void *process)
                                                                           'var132, as per NC 'v'*/
                 {
                   if (DBG) fprintf(stderr, "Found V VEL in Varname %s\n", varname);
-                  pstreamReadRecord(streamID, varray, &nmiss);
+                  cdoReadRecord(streamID, varray, &nmiss);
                   if (nmiss) cdoSetNAN(vlistInqVarMissval(vlistID, varID), gridsize, varray);
                   found++;
                 }
@@ -444,8 +445,8 @@ Magvector(void *process)
           if (found == 2)
             {
               if (DBG) fprintf(stderr, "Found Both U & V VEL, Creating vector fields! \n");
-              magvector(cdoGetStreamName(1), operatorID, varname, nlon, nlat, grid_center_lon, grid_center_lat, uarray,
-                        varray, nparam, pnames, datetimestr);
+              magvector(cdoGetStreamName(1), operatorID, varname, nlon, nlat, grid_center_lon, grid_center_lat, uarray, varray,
+                        nparam, pnames, datetimestr);
             }
           else if (found == 1)
             {
