@@ -26,8 +26,7 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
-#include "grid.h"
+#include <mpim_grid.h>
 
 #define MAX_LINE 256
 
@@ -89,12 +88,13 @@ ReadCoords(double *xvals, double *yvals, const char *polyfile, FILE *fp)
   return number;
 }
 
-void genlonlatbox(int argc_offset, int gridID1, long *lat1, long *lat2, long *lon11, long *lon12, long *lon21, long *lon22);
+void genlonlatbox(int argc_offset, int gridID1, long &lat1, long &lat2, long &lon11, long &lon12, long &lon21, long &lon22);
 
-void genindexbox(int argc_offset, int gridID1, long *lat1, long *lat2, long *lon11, long *lon12, long *lon21, long *lon22);
+void genindexbox(int argc_offset, int gridID1, long &lat1, long &lat2, long &lon11, long &lon12, long &lon21, long &lon22);
 
 static void
-maskbox(std::vector<bool> &mask, const int gridID, const long lat1, const long lat2, const long lon11, const long lon12, const long lon21, const long lon22)
+maskbox(std::vector<bool> &mask, const int gridID, const long lat1, const long lat2, const long lon11, const long lon12,
+        const long lon21, const long lon22)
 {
   const long nlon = (long) gridInqXsize(gridID);
   const long nlat = (long) gridInqYsize(gridID);
@@ -105,13 +105,13 @@ maskbox(std::vector<bool> &mask, const int gridID, const long lat1, const long l
         mask[nlon * ilat + ilon] = false;
 }
 
-void getlonlatparams(int argc_offset, double *xlon1, double *xlon2, double *xlat1, double *xlat2);
+void getlonlatparams(int argc_offset, double &xlon1, double &xlon2, double &xlat1, double &xlat2);
 
 static void
 maskbox_cell(std::vector<bool> &mask, const int gridID)
 {
   double xlon1 = 0, xlon2 = 0, xlat1 = 0, xlat2 = 0;
-  getlonlatparams(0, &xlon1, &xlon2, &xlat1, &xlat2);
+  getlonlatparams(0, xlon1, xlon2, xlat1, xlat2);
 
   const size_t gridsize = gridInqSize(gridID);
 
@@ -131,12 +131,7 @@ maskbox_cell(std::vector<bool> &mask, const int gridID)
 
   if (xlon1 > xlon2) cdoAbort("The second longitude have to be greater than the first one!");
 
-  if (xlat1 > xlat2)
-    {
-      double xtemp = xlat1;
-      xlat1 = xlat2;
-      xlat2 = xtemp;
-    }
+  if (xlat1 > xlat2) std::swap(xlat1, xlat2);
 
   for (size_t i = 0; i < gridsize; ++i)
     {
@@ -156,7 +151,8 @@ maskbox_cell(std::vector<bool> &mask, const int gridID)
 }
 
 static void
-maskregion(std::vector<bool> &mask, const int gridID, const std::vector<double> &xcoords, const std::vector<double> &ycoords, const size_t nofcoords)
+maskregion(std::vector<bool> &mask, const int gridID, const std::vector<double> &xcoords, const std::vector<double> &ycoords,
+           const size_t nofcoords)
 {
   size_t nlon = gridInqXsize(gridID);
   size_t nlat = gridInqYsize(gridID);
@@ -177,10 +173,10 @@ maskregion(std::vector<bool> &mask, const int gridID, const std::vector<double>
   }
 
   double xmin, xmax;
-  arrayMinMax(nlon, &xvals[0], &xmin, &xmax);
+  arrayMinMax(nlon, &xvals[0], xmin, xmax);
 
   double ymin, ymax;
-  arrayMinMax(nlat, &yvals[0], &ymin, &ymax);
+  arrayMinMax(nlat, &yvals[0], ymin, ymax);
 
   for (size_t ilat = 0; ilat < nlat; ilat++)
     {
@@ -254,7 +250,7 @@ Maskbox(void *process)
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -295,8 +291,8 @@ Maskbox(void *process)
       vars[varID] = (gridID == vlistInqVarGrid(vlistID1, varID));
     }
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   size_t gridsize = gridInqSize(gridID);
   std::vector<double> array(gridsize);
@@ -311,13 +307,13 @@ Maskbox(void *process)
         }
       else
         {
-          genlonlatbox(0, gridID, &lat1, &lat2, &lon11, &lon12, &lon21, &lon22);
+          genlonlatbox(0, gridID, lat1, lat2, lon11, lon12, lon21, lon22);
           maskbox(mask, gridID, lat1, lat2, lon11, lon12, lon21, lon22);
         }
     }
   else if (operatorID == MASKINDEXBOX)
     {
-      genindexbox(0, gridID, &lat1, &lat2, &lon11, &lon12, &lon21, &lon22);
+      genindexbox(0, gridID, lat1, lat2, lon11, lon12, lon21, lon22);
       maskbox(mask, gridID, lat1, lat2, lon11, lon12, lon21, lon22);
     }
   else if (operatorID == MASKREGION)
@@ -348,16 +344,16 @@ Maskbox(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           if (vars[varID])
             {
               size_t nmiss;
-              pstreamReadRecord(streamID1, &array[0], &nmiss);
+              cdoReadRecord(streamID1, &array[0], &nmiss);
 
               const double missval = vlistInqVarMissval(vlistID1, varID);
               for (size_t i = 0; i < gridsize; i++)
@@ -365,8 +361,8 @@ Maskbox(void *process)
 
               nmiss = arrayNumMV(gridsize, &array[0], missval);
 
-              pstreamDefRecord(streamID2, varID, levelID);
-              pstreamWriteRecord(streamID2, &array[0], nmiss);
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, &array[0], nmiss);
             }
         }
 
