@@ -24,8 +24,7 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "grid.h"
-#include "pstream_int.h"
+#include <mpim_grid.h>
 
 static void
 mastrfu(int gridID, int zaxisID, std::vector<std::vector<double>> &field1, std::vector<std::vector<double>> &field2, size_t nmiss,
@@ -92,7 +91,7 @@ Mastrfu(void *process)
 
   cdoInitialize(process);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   int vlistID1 = cdoStreamInqVlist(streamID1);
 
@@ -130,8 +129,8 @@ Mastrfu(void *process)
   vlistDefVarUnits(vlistID2, 0, "kg/s");
   vlistDefVarDatatype(vlistID2, 0, CDI_DATATYPE_FLT32);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   VECTOR_2D(double, array1, nlev, gridsize);
   VECTOR_2D(double, array2, nlev, gridsize);
@@ -140,13 +139,13 @@ Mastrfu(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       nmiss = 0;
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, array1[levelID].data(), &nmiss1);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, array1[levelID].data(), &nmiss1);
           nmiss += nmiss1;
         }
 
@@ -156,8 +155,8 @@ Mastrfu(void *process)
         {
           varID = 0;
           levelID = recID;
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, array2[levelID].data(), nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, array2[levelID].data(), nmiss);
         }
 
       tsID++;
