@@ -24,77 +24,65 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
+
 #include "util_files.h"
 
 static void
 checkDupEntry(int vlistID1, int vlistID2, const char *filename)
 {
   char vname1[CDI_MAX_NAME], vname2[CDI_MAX_NAME];
-  int k;
-  int mlev1 = 0, mlev2 = 0;
   std::vector<double> lev1, lev2;
 
-  int nvars1 = vlistNvars(vlistID1);
-  int nvars2 = vlistNvars(vlistID2);
+  const int nvars1 = vlistNvars(vlistID1);
+  const int nvars2 = vlistNvars(vlistID2);
 
   for (int varID1 = 0; varID1 < nvars1; ++varID1)
     {
       vlistInqVarName(vlistID1, varID1, vname1);
-      int param1 = vlistInqVarParam(vlistID1, varID1);
-      int gridID1 = vlistInqVarGrid(vlistID1, varID1);
-      int zaxisID1 = vlistInqVarZaxis(vlistID1, varID1);
-      int gtype1 = gridInqType(gridID1);
-      size_t gsize1 = gridInqSize(gridID1);
-      int ztype1 = zaxisInqType(zaxisID1);
-      int nlev1 = zaxisInqSize(zaxisID1);
-      if (nlev1 > mlev1)
-        {
-          mlev1 = nlev1;
-          lev1.resize(mlev1);
-        }
+      const int param1 = vlistInqVarParam(vlistID1, varID1);
+      const int gridID1 = vlistInqVarGrid(vlistID1, varID1);
+      const int zaxisID1 = vlistInqVarZaxis(vlistID1, varID1);
+      const int gtype1 = gridInqType(gridID1);
+      const size_t gsize1 = gridInqSize(gridID1);
+      const int ztype1 = zaxisInqType(zaxisID1);
+      const size_t nlev1 = zaxisInqSize(zaxisID1);
+      if (nlev1 > lev1.size()) lev1.resize(nlev1);
       cdoZaxisInqLevels(zaxisID1, lev1.data());
 
       for (int varID2 = 0; varID2 < nvars2; ++varID2)
         {
           vlistInqVarName(vlistID2, varID2, vname2);
-          int param2 = vlistInqVarParam(vlistID2, varID2);
-          int gridID2 = vlistInqVarGrid(vlistID2, varID2);
-          int zaxisID2 = vlistInqVarZaxis(vlistID2, varID2);
-          int gtype2 = gridInqType(gridID2);
-          size_t gsize2 = gridInqSize(gridID2);
-          int ztype2 = zaxisInqType(zaxisID2);
-          int nlev2 = zaxisInqSize(zaxisID2);
+          const int param2 = vlistInqVarParam(vlistID2, varID2);
+          const int gridID2 = vlistInqVarGrid(vlistID2, varID2);
+          const int zaxisID2 = vlistInqVarZaxis(vlistID2, varID2);
+          const int gtype2 = gridInqType(gridID2);
+          const size_t gsize2 = gridInqSize(gridID2);
+          const int ztype2 = zaxisInqType(zaxisID2);
+          const size_t nlev2 = zaxisInqSize(zaxisID2);
           if (gtype1 == gtype2 && gsize1 == gsize2 && ztype1 == ztype2 && nlev1 == nlev2)
             {
-              if (nlev2 > mlev2)
-                {
-                  mlev2 = nlev2;
-                  lev2.resize(mlev2);
-                }
+              if (nlev2 > lev2.size()) lev2.resize(nlev2);
               cdoZaxisInqLevels(zaxisID2, lev2.data());
 
               if (zaxisInqLevels(zaxisID1, nullptr) && zaxisInqLevels(zaxisID2, nullptr))
                 {
-                  for (k = 0; k < nlev2; ++k)
-                    if (!IS_EQUAL(lev1[k], lev2[k])) break;
+                  for (size_t k = 0; k < nlev2; ++k)
+                    if (!IS_EQUAL(lev1[k], lev2[k])) return;
+                }
 
-                  if (k == nlev2)
-                    {
-                      if (param1 < 0 || param2 < 0)
-                        {
-                          if (strcmp(vname1, vname2) == 0) cdoWarning("Duplicate entry of parameter %s in %s!", vname2, filename);
-                        }
-                      else
-                        {
-                          if (param1 == param2)
-                            {
-                              char paramstr[32];
-                              cdiParamToString(param2, paramstr, sizeof(paramstr));
-                              cdoWarning("Duplicate entry of parameter %s in %s!", paramstr, filename);
-                            }
-                        }
-                    }
+              if ((param1 < 0 || param2 < 0) && strcmp(vname1, vname2) == 0)
+                {
+                  cdoWarning("Duplicate entry of parameter name %s in %s!", vname2, filename);
+                }
+              else if (param1 == param2)
+                {
+                  char paramstr[32];
+                  cdiParamToString(param2, paramstr, sizeof(paramstr));
+                  cdoWarning("Duplicate entry of parameter %s in %s!", paramstr, filename);
+                }
+              else if (param1 != param2 && strcmp(vname1, vname2) == 0)
+                {
+                  cdoWarning("Duplicate entry of parameter name %s with different IDs in %s!", vname2, filename);
                 }
             }
         }
@@ -113,10 +101,24 @@ int vlistConstVars(int vlistID)
 }
 */
 
+static int
+getTaxisindex(const std::vector<int> &vlistIDs)
+{
+  if (vlistNtsteps(vlistIDs[0]) == 0)
+    {
+      const int nmerge = vlistIDs.size();
+      for (int index = 1; index < nmerge; index++)
+        {
+          if (vlistNtsteps(vlistIDs[index]) != 0) return index;
+        }
+    }
+
+  return 0;
+}
+
 void *
 Merge(void *process)
 {
-  int streamID1 = -1;
   int varID, varID2;
   int nrecs = 0;
   int levelID, levelID2;
@@ -125,60 +127,44 @@ Merge(void *process)
 
   cdoInitialize(process);
 
-  bool lcopy = UNCHANGED_RECORD;
+  const bool lcopy = unchangedRecord();
 
-  int streamCnt = cdoStreamCnt();
-  int nmerge = streamCnt - 1;
+  const int streamCnt = cdoStreamCnt();
+  const int nmerge = streamCnt - 1;
 
   const char *ofilename = cdoGetStreamName(streamCnt - 1);
 
   if (!Options::cdoOverwriteMode && fileExists(ofilename) && !userFileOverwrite(ofilename))
     cdoAbort("Outputfile %s already exists!", ofilename);
 
-  std::vector<int> streamIDs(nmerge);
+  std::vector<CdoStreamID> streamIDs(nmerge);
   std::vector<int> vlistIDs(nmerge);
   std::vector<int> numrecs(nmerge);
   std::vector<int> numsteps(nmerge);
 
   for (index = 0; index < nmerge; index++)
     {
-      streamIDs[index] = cdoStreamOpenRead(cdoStreamName(index));
+      streamIDs[index] = cdoOpenRead((index));
       vlistIDs[index] = cdoStreamInqVlist(streamIDs[index]);
     }
 
-  int taxisindex = 0;
-  int vlistID1 = vlistIDs[0];
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  if (vlistNtsteps(vlistID1) == 0)
-    for (index = 1; index < nmerge; index++)
-      {
-        vlistID1 = vlistIDs[index];
-        if (vlistNtsteps(vlistID1) != 0)
-          {
-            taxisindex = index;
-            taxisID1 = vlistInqTaxis(vlistID1);
-            break;
-          }
-      }
+  const int taxisindex = getTaxisindex(vlistIDs);
 
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistIDs[taxisindex]);
+  const int taxisID2 = taxisDuplicate(taxisID1);
 
-  int vlistID2 = vlistCreate();
+  const int vlistID2 = vlistCreate();
   vlistCopy(vlistID2, vlistIDs[0]);
   for (index = 1; index < nmerge; index++)
     {
-      checkDupEntry(vlistID2, vlistIDs[index], cdoGetStreamName(index));
-      /* vlistCat(vlistID2, vlistIDs[index]); */
+      checkDupEntry(vlistID2, vlistIDs[index], cdoGetCommandFromInStream(index));
       vlistMerge(vlistID2, vlistIDs[index]);
     }
 
-  for (index = 0; index < nmerge; index++) numsteps[index] = -1;
-
   int numconst = 0;
   for (index = 0; index < nmerge; index++)
     {
-      vlistID1 = vlistIDs[index];
-      numsteps[index] = vlistNtsteps(vlistID1);
+      numsteps[index] = vlistNtsteps(vlistIDs[index]);
       if (numsteps[index] == 0) numsteps[index] = 1;
       if (numsteps[index] == 1) numconst++;
     }
@@ -188,8 +174,8 @@ Merge(void *process)
       {
         if (numsteps[index] == 1)
           {
-            vlistID1 = vlistIDs[index];
-            int nvars = vlistNvars(vlistID1);
+            const int vlistID1 = vlistIDs[index];
+            const int nvars = vlistNvars(vlistID1);
             for (int varID = 0; varID < nvars; ++varID)
               {
                 varID2 = vlistMergedVar(vlistID1, varID);
@@ -204,15 +190,15 @@ Merge(void *process)
       vlistPrint(vlistID2);
     }
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(streamCnt - 1));
+  CdoStreamID streamID2 = cdoOpenWrite((streamCnt - 1));
 
   vlistDefTaxis(vlistID2, taxisID2);
-  pstreamDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID2, vlistID2);
 
   std::vector<double> array;
   if (!lcopy)
     {
-      size_t gridsizemax = vlistGridsizeMax(vlistID2);
+      const size_t gridsizemax = vlistGridsizeMax(vlistID2);
       array.resize(gridsizemax);
     }
 
@@ -221,11 +207,8 @@ Merge(void *process)
     {
       for (index = 0; index < nmerge; index++)
         {
-          streamID1 = streamIDs[index];
-          vlistID1 = vlistIDs[index];
-          if (vlistID1 == -1) continue;
-
-          numrecs[index] = cdoStreamInqTimestep(streamID1, tsID);
+          if (vlistIDs[index] == -1) continue;
+          numrecs[index] = cdoStreamInqTimestep(streamIDs[index], tsID);
         }
 
       for (index = 0; index < nmerge; index++)
@@ -271,34 +254,34 @@ Merge(void *process)
         }
 
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (index = 0; index < nmerge; index++)
         {
-          streamID1 = streamIDs[index];
-          vlistID1 = vlistIDs[index];
+          CdoStreamID streamID1 = streamIDs[index];
+          const int vlistID1 = vlistIDs[index];
           nrecs = numrecs[index];
 
           if (vlistID1 == -1) continue;
 
           for (int recID = 0; recID < nrecs; recID++)
             {
-              pstreamInqRecord(streamID1, &varID, &levelID);
+              cdoInqRecord(streamID1, &varID, &levelID);
 
               varID2 = vlistMergedVar(vlistID1, varID);
               levelID2 = vlistMergedLevel(vlistID1, varID, levelID);
 
               if (Options::cdoVerbose) cdoPrint("var %d %d %d %d", varID, levelID, varID2, levelID2);
 
-              pstreamDefRecord(streamID2, varID2, levelID2);
+              cdoDefRecord(streamID2, varID2, levelID2);
               if (lcopy)
                 {
-                  pstreamCopyRecord(streamID2, streamID1);
+                  cdoCopyRecord(streamID2, streamID1);
                 }
               else
                 {
-                  pstreamReadRecord(streamID1, array.data(), &nmiss);
-                  pstreamWriteRecord(streamID2, array.data(), nmiss);
+                  cdoReadRecord(streamID1, array.data(), &nmiss);
+                  cdoWriteRecord(streamID2, array.data(), nmiss);
                 }
             }
         }
