@@ -24,13 +24,12 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 #include "util_files.h"
 #include "printinfo.h"
 
 struct StreamFile
 {
-  int streamID;
+  CdoStreamID streamID;
   int vlistID;
   int taxisID;
   int tsID;
@@ -39,20 +38,19 @@ struct StreamFile
   int nrecs;
 };
 
-
 static bool
 getenvSkipSameTime()
 {
   bool skipSameTime = false;
 
-  const char *envstr = getenv("SKIPSAMETIME");
+  const char *envstr = getenv("SKIP_SAME_TIME");
   if (envstr)
     {
-      int ival = atoi(envstr);
+      const int ival = atoi(envstr);
       if (ival == 1)
         {
           skipSameTime = true;
-          if (Options::cdoVerbose) cdoPrint("Set SKIPSAMETIME to %d", ival);
+          if (Options::cdoVerbose) cdoPrint("Set SKIP_SAME_TIME to %d", ival);
         }
     }
 
@@ -66,7 +64,7 @@ openAllFiles(int nfiles, std::vector<StreamFile> &sf)
     {
       if (Options::cdoVerbose) cdoPrint("process: %s", cdoGetStreamName(fileID));
 
-      sf[fileID].streamID = cdoStreamOpenRead(cdoStreamName(fileID));
+      sf[fileID].streamID = cdoOpenRead((fileID));
       sf[fileID].vlistID = cdoStreamInqVlist(sf[fileID].streamID);
       sf[fileID].taxisID = vlistInqTaxis(sf[fileID].vlistID);
     }
@@ -82,7 +80,7 @@ readFirstTimestep(int nfiles, std::vector<StreamFile> &sf)
       if (sf[fileID].nrecs == 0)
         {
           cdoStreamClose(sf[fileID].streamID);
-          sf[fileID].streamID = -1;
+          sf[fileID].streamID = CDO_STREAM_UNDEF;
         }
       else
         {
@@ -102,9 +100,9 @@ Mergetime(void *process)
 
   cdoInitialize(process);
 
-  bool skipSameTime = getenvSkipSameTime();
+  const bool skipSameTime = getenvSkipSameTime();
 
-  const bool lcopy = UNCHANGED_RECORD;
+  const bool lcopy = unchangedRecord();
 
   const int nfiles = cdoStreamCnt() - 1;
   std::vector<StreamFile> sf(nfiles);
@@ -121,7 +119,7 @@ Mergetime(void *process)
   if (!Options::cdoOverwriteMode && fileExists(ofilename) && !userFileOverwrite(ofilename))
     cdoAbort("Outputfile %s already exists!", ofilename);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(nfiles));
+  CdoStreamID streamID2 = cdoOpenWrite((nfiles));
 
   std::vector<double> array;
   if (!lcopy)
@@ -139,7 +137,7 @@ Mergetime(void *process)
       int vtime = 0;
       for (int fileID = 0; fileID < nfiles; fileID++)
         {
-          if (sf[fileID].streamID != -1)
+          if (sf[fileID].streamID != CDO_STREAM_UNDEF)
             if (next_fileID == -1 || sf[fileID].vdate < vdate || (sf[fileID].vdate == vdate && sf[fileID].vtime < vtime))
               {
                 next_fileID = fileID;
@@ -158,7 +156,7 @@ Mergetime(void *process)
             char vdatestr[32], vtimestr[32];
             date2str(vdate, vdatestr, sizeof(vdatestr));
             time2str(vtime, vtimestr, sizeof(vtimestr));
-            cdoPrint("Timestep %4d in stream %d (%s %s) already exists, skipped!", sf[fileID].tsID + 1, sf[fileID].streamID,
+            cdoPrint("Timestep %4d in stream %d (%s %s) already exists, skipped!", sf[fileID].tsID + 1, sf[fileID].streamID->getID(),
                      vdatestr, vtimestr);
             process_timestep = false;
           }
@@ -173,34 +171,34 @@ Mergetime(void *process)
               taxisID2 = taxisDuplicate(taxisID1);
               vlistDefTaxis(vlistID2, taxisID2);
 
-              pstreamDefVlist(streamID2, vlistID2);
+              cdoDefVlist(streamID2, vlistID2);
             }
 
           last_vdate = vdate;
           last_vtime = vtime;
 
           taxisCopyTimestep(taxisID2, sf[fileID].taxisID);
-          pstreamDefTimestep(streamID2, tsID2);
+          cdoDefTimestep(streamID2, tsID2);
 
           for (int recID = 0; recID < sf[fileID].nrecs; recID++)
             {
               int varID, levelID;
-              pstreamInqRecord(sf[fileID].streamID, &varID, &levelID);
+              cdoInqRecord(sf[fileID].streamID, &varID, &levelID);
 
               if (tsID2 > 0 && sf[fileID].tsID == 0)
                 if (vlistInqVarTimetype(sf[fileID].vlistID, varID) == TIME_CONSTANT) continue;
 
-              pstreamDefRecord(streamID2, varID, levelID);
+              cdoDefRecord(streamID2, varID, levelID);
 
               if (lcopy)
                 {
-                  pstreamCopyRecord(streamID2, sf[fileID].streamID);
+                  cdoCopyRecord(streamID2, sf[fileID].streamID);
                 }
               else
                 {
                   size_t nmiss;
-                  pstreamReadRecord(sf[fileID].streamID, array.data(), &nmiss);
-                  pstreamWriteRecord(streamID2, array.data(), nmiss);
+                  cdoReadRecord(sf[fileID].streamID, array.data(), &nmiss);
+                  cdoWriteRecord(streamID2, array.data(), nmiss);
                 }
             }
 
@@ -211,7 +209,7 @@ Mergetime(void *process)
       if (sf[fileID].nrecs == 0)
         {
           cdoStreamClose(sf[fileID].streamID);
-          sf[fileID].streamID = -1;
+          sf[fileID].streamID = CDO_STREAM_UNDEF;
         }
       else
         {
