@@ -24,8 +24,8 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
-#include "grid.h"
+
+#include <mpim_grid.h>
 
 void
 rotate_uv(double *u_i, double *v_j, int ix, int iy, double *lon, double *lat, double *u_lon, double *v_lat)
@@ -195,7 +195,7 @@ Mrotuv(void *process)
 
   cdoInitialize(process);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   int vlistID1 = cdoStreamInqVlist(streamID1);
 
@@ -297,11 +297,11 @@ Mrotuv(void *process)
   vlistDefTaxis(vlistID2, taxisID2);
   vlistDefTaxis(vlistID3, taxisID3);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  int streamID3 = cdoStreamOpenWrite(cdoStreamName(2));
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  CdoStreamID streamID3 = cdoOpenWrite(2);
 
-  pstreamDefVlist(streamID2, vlistID2);
-  pstreamDefVlist(streamID3, vlistID3);
+  cdoDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID3, vlistID3);
 
   double missval1 = vlistInqVarMissval(vlistID1, uid);
   double missval2 = vlistInqVarMissval(vlistID1, vid);
@@ -324,16 +324,16 @@ Mrotuv(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
       taxisCopyTimestep(taxisID3, taxisID1);
-      pstreamDefTimestep(streamID3, tsID);
+      cdoDefTimestep(streamID3, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
-          if (varID == uid) pstreamReadRecord(streamID1, urfield[levelID].data(), &nmiss1);
-          if (varID == vid) pstreamReadRecord(streamID1, vrfield[levelID].data(), &nmiss2);
+          if (varID == uid) cdoReadRecord(streamID1, urfield[levelID].data(), &nmiss1);
+          if (varID == vid) cdoReadRecord(streamID1, vrfield[levelID].data(), &nmiss2);
         }
 
       for (levelID = 0; levelID < nlevs; levelID++)
@@ -388,10 +388,10 @@ Mrotuv(void *process)
               vfield[IX2D(nlat - 1, i, nlon)] = vhelp[IX2D(nlat - 1, i + 1, nlon + 2)];
             }
 
-          pstreamDefRecord(streamID2, 0, levelID);
-          pstreamWriteRecord(streamID2, ufield.data(), nmiss1);
-          pstreamDefRecord(streamID3, 0, levelID);
-          pstreamWriteRecord(streamID3, vfield.data(), nmiss2);
+          cdoDefRecord(streamID2, 0, levelID);
+          cdoWriteRecord(streamID2, ufield.data(), nmiss1);
+          cdoDefRecord(streamID3, 0, levelID);
+          cdoWriteRecord(streamID3, vfield.data(), nmiss2);
         }
 
       tsID++;
