@@ -24,8 +24,8 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
-#include "grid.h"
+
+#include <mpim_grid.h>
 
 /*
 !----------------------------------------------------------------------
@@ -156,8 +156,7 @@ uv_to_p_grid(size_t nlon, size_t nlat, double *grid1x, double *grid1y, double *g
   double gx2, gy2;
 
   size_t gridsizex = (nlon + 2) * nlat;
-  std::vector<double> gxhelp(gridsizex);
-  std::vector<double> gyhelp(gridsizex);
+  std::vector<double> gxhelp(gridsizex), gyhelp(gridsizex);
 
   /* load to a help field */
   for (size_t j = 0; j < nlat; j++)
@@ -262,11 +261,11 @@ Mrotuvb(void *process)
   if (operatorArgc() == 1)
     if (strcmp(operatorArgv()[0], "noint") == 0) gpint = false;
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-  int streamID2 = cdoStreamOpenRead(cdoStreamName(1));
+  CdoStreamID streamID1 = cdoOpenRead(0);
+  CdoStreamID streamID2 = cdoOpenRead(1);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = cdoStreamInqVlist(streamID2);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = cdoStreamInqVlist(streamID2);
 
   int nvars = vlistNvars(vlistID1);
   if (nvars > 1) cdoAbort("More than one variable found in %s", cdoGetStreamName(0));
@@ -275,7 +274,7 @@ Mrotuvb(void *process)
 
   int gridID1 = vlistGrid(vlistID1, 0);
   int gridID2 = vlistGrid(vlistID2, 0);
-  size_t gridsize = gridInqSize(gridID1);
+  const size_t gridsize = gridInqSize(gridID1);
   if (gpint && gridID1 == gridID2) cdoAbort("Input grids are the same, use parameter >noint< to disable interpolation!");
   if (!gpint && gridID1 != gridID2) cdoAbort("Input grids are not the same!");
   if (gridsize != gridInqSize(gridID2)) cdoAbort("Grids have different size!");
@@ -291,15 +290,12 @@ Mrotuvb(void *process)
 
   if (gridsize != gridInqSize(gridID2)) cdoAbort("Internal problem: gridsize changed!");
 
-  size_t nlon = gridInqXsize(gridID1);
-  size_t nlat = gridInqYsize(gridID1);
+  const size_t nlon = gridInqXsize(gridID1);
+  const size_t nlat = gridInqYsize(gridID1);
 
-  std::vector<double> grid1x(gridsize);
-  std::vector<double> grid1y(gridsize);
-  std::vector<double> grid2x(gridsize);
-  std::vector<double> grid2y(gridsize);
-  std::vector<double> grid3x(gridsize);
-  std::vector<double> grid3y(gridsize);
+  std::vector<double> grid1x(gridsize), grid1y(gridsize);
+  std::vector<double> grid2x(gridsize), grid2y(gridsize);
+  std::vector<double> grid3x(gridsize), grid3y(gridsize);
 
   gridInqXvals(gridID1, grid1x.data());
   gridInqYvals(gridID1, grid1y.data());
@@ -331,11 +327,11 @@ Mrotuvb(void *process)
     }
   else
     {
-      arrayCopy(gridsize, grid1x.data(), grid3x.data());
-      arrayCopy(gridsize, grid1y.data(), grid3y.data());
+      grid3x = grid1x;
+      grid3y = grid1y;
     }
 
-  int gridID3 = gridCreate(GRID_CURVILINEAR, gridsize);
+  const int gridID3 = gridCreate(GRID_CURVILINEAR, gridsize);
   gridDefDatatype(gridID3, gridInqDatatype(gridID1));
   gridDefXsize(gridID3, nlon);
   gridDefYsize(gridID3, nlat);
@@ -366,16 +362,14 @@ Mrotuvb(void *process)
 
   if (Options::cdoVerbose) vlistPrint(vlistID3);
 
-  int streamID3 = cdoStreamOpenWrite(cdoStreamName(2));
-  pstreamDefVlist(streamID3, vlistID3);
+  CdoStreamID streamID3 = cdoOpenWrite(2);
+  cdoDefVlist(streamID3, vlistID3);
 
   double missval1 = vlistInqVarMissval(vlistID1, 0);
   double missval2 = vlistInqVarMissval(vlistID2, 0);
 
-  std::vector<double> ufield(gridsize);
-  std::vector<double> vfield(gridsize);
-  std::vector<double> urfield(gridsize);
-  std::vector<double> vrfield(gridsize);
+  std::vector<double> ufield(gridsize), vfield(gridsize);
+  std::vector<double> urfield(gridsize), vrfield(gridsize);
 
   std::vector<double> uhelp, vhelp;
   if (gpint)
@@ -390,7 +384,7 @@ Mrotuvb(void *process)
     {
       taxisCopyTimestep(taxisID3, taxisID1);
 
-      pstreamDefTimestep(streamID3, tsID);
+      cdoDefTimestep(streamID3, tsID);
 
       nrecs2 = cdoStreamInqTimestep(streamID2, tsID);
 
@@ -398,11 +392,11 @@ Mrotuvb(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID1, &levelID);
-          pstreamInqRecord(streamID2, &varID2, &levelID);
+          cdoInqRecord(streamID1, &varID1, &levelID);
+          cdoInqRecord(streamID2, &varID2, &levelID);
 
-          pstreamReadRecord(streamID1, ufield.data(), &nmiss1);
-          pstreamReadRecord(streamID2, vfield.data(), &nmiss2);
+          cdoReadRecord(streamID1, ufield.data(), &nmiss1);
+          cdoReadRecord(streamID2, vfield.data(), &nmiss2);
 
           /* remove missing values */
           if (nmiss1 || nmiss2)
@@ -475,10 +469,10 @@ Mrotuvb(void *process)
           */
           nmiss1 = 0;
           nmiss2 = 0;
-          pstreamDefRecord(streamID3, 0, levelID);
-          pstreamWriteRecord(streamID3, urfield.data(), nmiss1);
-          pstreamDefRecord(streamID3, 1, levelID);
-          pstreamWriteRecord(streamID3, vrfield.data(), nmiss2);
+          cdoDefRecord(streamID3, 0, levelID);
+          cdoWriteRecord(streamID3, urfield.data(), nmiss1);
+          cdoDefRecord(streamID3, 1, levelID);
+          cdoWriteRecord(streamID3, vrfield.data(), nmiss2);
         }
 
       tsID++;
