@@ -31,7 +31,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 
 void *
 Ninfo(void *process)
@@ -67,7 +66,7 @@ Ninfo(void *process)
   const int operatorID = cdoOperatorID();
   const int operfunc = cdoOperatorF1(operatorID);
 
-  const int streamID = cdoStreamOpenRead(0);
+  CdoStreamID streamID = cdoOpenRead(0);
   const int vlistID = cdoStreamInqVlist(streamID);
 
   const int nvars = vlistNvars(vlistID);
