@@ -44,10 +44,10 @@ printValues(const int nvalues, const std::vector<std::string> &values)
             case CDI_DATATYPE_INT16: printf("%ds", literal_to_int(value)); break;
             case CDI_DATATYPE_INT32: printf("%d", literal_to_int(value)); break;
             case CDI_DATATYPE_FLT32:
-              printf("%sf", double_to_attstr(Options::CDO_flt_digits, fltstr, sizeof(fltstr), literal_to_double(value)));
+              printf("%sf", doubleToAttstr(Options::CDO_flt_digits, fltstr, sizeof(fltstr), literal_to_double(value)));
               break;
             case CDI_DATATYPE_FLT64:
-              printf("%s", double_to_attstr(Options::CDO_dbl_digits, fltstr, sizeof(fltstr), literal_to_double(value)));
+              printf("%s", doubleToAttstr(Options::CDO_dbl_digits, fltstr, sizeof(fltstr), literal_to_double(value)));
               break;
             default: printf("\"%s\"", value);
             }
