@@ -30,8 +30,8 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "grid.h"
-#include "pstream_int.h"
+#include <mpim_grid.h>
+#include "gridreference.h"
 #include "printinfo.h"
 
 static void
@@ -46,9 +46,8 @@ outputarr(int dig, size_t gridsize, std::vector<double> &array)
 static void
 outputsp(size_t gridsize, std::vector<double> &array, long ntr)
 {
-  double minval = array[0];
-  double maxval = array[0];
-  arrayMinMax(gridsize, array.data(), &minval, &maxval);
+  double minval, maxval;
+  arrayMinMax(gridsize, array.data(), minval, maxval);
   if (/* T11 */ minval >= -1 && maxval <= 12)
     {
       double *spc = array.data();
@@ -273,7 +272,7 @@ Output(void *process)
 
   for (int indf = 0; indf < cdoStreamCnt(); indf++)
     {
-      int streamID = cdoStreamOpenRead(cdoStreamName(indf));
+      CdoStreamID streamID = cdoOpenRead((indf));
 
       int vlistID = cdoStreamInqVlist(streamID);
 
@@ -297,7 +296,7 @@ Output(void *process)
           if (gridtype == GRID_GME) gridID = gridToUnstructured(gridID, 0);
 
           if (gridtype != GRID_UNSTRUCTURED && gridtype != GRID_CURVILINEAR) gridID = gridToCurvilinear(gridID, 0);
-          
+
           if (gridtype == GRID_UNSTRUCTURED)
             {
               if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
@@ -339,21 +338,21 @@ Output(void *process)
 
           for (int recID = 0; recID < nrecs; recID++)
             {
-              pstreamInqRecord(streamID, &varID, &levelID);
+              cdoInqRecord(streamID, &varID, &levelID);
 
               vlistInqVarName(vlistID, varID, name);
-              int param = vlistInqVarParam(vlistID, varID);
-              int code = vlistInqVarCode(vlistID, varID);
-              int gridID = vlistInqVarGrid(vlistID, varID);
-              int zaxisID = vlistInqVarZaxis(vlistID, varID);
-              int datatype = vlistInqVarDatatype(vlistID, varID);
-              int dig = (datatype == CDI_DATATYPE_FLT64) ? Options::CDO_dbl_digits : Options::CDO_flt_digits;
-              size_t nwpv = (vlistInqVarNumber(vlistID, varID) == CDI_COMP) ? 2 : 1;
-              size_t gridsize = nwpv * gridInqSize(gridID);
+              const int param = vlistInqVarParam(vlistID, varID);
+              const int code = vlistInqVarCode(vlistID, varID);
+              const int gridID = vlistInqVarGrid(vlistID, varID);
+              const int zaxisID = vlistInqVarZaxis(vlistID, varID);
+              const int datatype = vlistInqVarDatatype(vlistID, varID);
+              const int dig = (datatype == CDI_DATATYPE_FLT64) ? Options::CDO_dbl_digits : Options::CDO_flt_digits;
+              const size_t nwpv = (vlistInqVarNumber(vlistID, varID) == CDI_COMP) ? 2 : 1;
+              const size_t gridsize = nwpv * gridInqSize(gridID);
               size_t nlon = gridInqXsize(gridID);
               size_t nlat = gridInqYsize(gridID);
-              double level = cdoZaxisInqLevel(zaxisID, levelID);
-              double missval = vlistInqVarMissval(vlistID, varID);
+              const double level = cdoZaxisInqLevel(zaxisID, levelID);
+              const double missval = vlistInqVarMissval(vlistID, varID);
 
               cdiParamToString(param, paramstr, sizeof(paramstr));
 
@@ -363,10 +362,10 @@ Output(void *process)
                   nlat = 1;
                 }
 
-              pstreamReadRecord(streamID, array.data(), &nmiss);
+              cdoReadRecord(streamID, array.data(), &nmiss);
 
               if (operatorID == OUTPUTSRV)
-                fprintf(stdout, "%4d %8g %8lld %4d %8zu %8zu %d %d\n", code, level, vdate, vtime, nlon, nlat, 0, 0);
+                fprintf(stdout, "%4d %8g %8ld %4d %8zu %8zu %d %d\n", code, level, (long)vdate, vtime, nlon, nlat, 0, 0);
 
               if (operatorID == OUTPUTEXT) fprintf(stdout, "%8lld %4d %8g %8zu\n", vdate, code, level, gridsize);
 
@@ -422,10 +421,9 @@ Output(void *process)
                 }
               else if (operatorID == OUTPUTTAB)
                 {
-                  bool l2d = false;
-                  int xsize = gridInqXsize(gridID);
+                  const bool l2d = (gridtype == GRID_CURVILINEAR);
+                  const int xsize = gridInqXsize(gridID);
                   // int ysize = gridInqYsize(gridID);
-                  if (gridtype == GRID_CURVILINEAR) l2d = true;
 
                   for (size_t i = 0; i < gridsize; i++)
                     {
@@ -436,8 +434,8 @@ Output(void *process)
                           yind /= xsize;
                           xind -= yind * xsize;
                         }
-                      double lon = grid_center_lon[i];
-                      double lat = grid_center_lat[i];
+                      const double lon = grid_center_lon[i];
+                      const double lat = grid_center_lat[i];
 
                       for (k = 0; k < nkeys; ++k)
                         {
