@@ -34,8 +34,8 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "grid.h"
-#include "pstream_int.h"
+#include <mpim_grid.h>
+#include "gridreference.h"
 #include "color.h"
 #include "printinfo.h"
 #include "interpol.h"
@@ -78,7 +78,7 @@ static void
 output_vrml(int nlon, int nlat, int ngp, double *restrict array, double missval, CPT *cpt)
 {
   double minval, maxval, meanval;
-  arrayMinMaxMeanMV(ngp, array, missval, &minval, &maxval, &meanval);
+  arrayMinMaxMeanMV(ngp, array, missval, minval, maxval, meanval);
 
   double dx = 10. / nlon;
 
@@ -193,17 +193,17 @@ Outputgmt(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  int OUTPUTCENTER    = cdoOperatorAdd("gmtxyz",          0, 0, nullptr);
-  int OUTPUTCENTER2   = cdoOperatorAdd("outputcenter2",   0, 0, nullptr);
-  int OUTPUTCENTERCPT = cdoOperatorAdd("outputcentercpt", 0, 0, nullptr);
-  int OUTPUTBOUNDS    = cdoOperatorAdd("gmtcells",        0, 0, nullptr);
-  int OUTPUTBOUNDSCPT = cdoOperatorAdd("outputboundscpt", 0, 0, nullptr);
-  int OUTPUTVECTOR    = cdoOperatorAdd("outputvector",    0, 0, nullptr);
-  int OUTPUTTRI       = cdoOperatorAdd("outputtri",       0, 0, nullptr);
-  int OUTPUTVRML      = cdoOperatorAdd("outputvrml",      0, 0, nullptr);
+  const int OUTPUTCENTER    = cdoOperatorAdd("gmtxyz",          0, 0, nullptr);
+  const int OUTPUTCENTER2   = cdoOperatorAdd("outputcenter2",   0, 0, nullptr);
+  const int OUTPUTCENTERCPT = cdoOperatorAdd("outputcentercpt", 0, 0, nullptr);
+  const int OUTPUTBOUNDS    = cdoOperatorAdd("gmtcells",        0, 0, nullptr);
+  const int OUTPUTBOUNDSCPT = cdoOperatorAdd("outputboundscpt", 0, 0, nullptr);
+  const int OUTPUTVECTOR    = cdoOperatorAdd("outputvector",    0, 0, nullptr);
+  const int OUTPUTTRI       = cdoOperatorAdd("outputtri",       0, 0, nullptr);
+  const int OUTPUTVRML      = cdoOperatorAdd("outputvrml",      0, 0, nullptr);
   // clang-format on
 
-  int operatorID = cdoOperatorID();
+  const int operatorID = cdoOperatorID();
 
   if (operatorID == OUTPUTVECTOR)
     {
@@ -223,35 +223,40 @@ Outputgmt(void *process)
       FILE *cpt_fp = fopen(cpt_file, "r");
       if (cpt_fp == nullptr) cdoAbort("Open failed on color palette table %s", cpt_file);
 
-      int status = cptRead(cpt_fp, &cpt);
+      const int status = cptRead(cpt_fp, &cpt);
       if (status != 0) cdoAbort("Error during read of color palette table %s", cpt_file);
 
       if (Options::cdoVerbose) cptWrite(stderr, cpt);
     }
 
-  int streamID = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID = cdoOpenRead(0);
 
-  int vlistID = cdoStreamInqVlist(streamID);
-  int taxisID = vlistInqTaxis(vlistID);
+  const int vlistID = cdoStreamInqVlist(streamID);
+  const int taxisID = vlistInqTaxis(vlistID);
 
   int varID = 0;
   vlistInqVarName(vlistID, varID, varname);
-  int code = vlistInqVarCode(vlistID, varID);
+  const int code = vlistInqVarCode(vlistID, varID);
   int gridID = vlistInqVarGrid(vlistID, varID);
-  int zaxisID = vlistInqVarZaxis(vlistID, varID);
-  double missval = vlistInqVarMissval(vlistID, varID);
+  const int zaxisID = vlistInqVarZaxis(vlistID, varID);
+  const double missval = vlistInqVarMissval(vlistID, varID);
+
+  const size_t gridsize = gridInqSize(gridID);
 
-  size_t gridsize = gridInqSize(gridID);
   int gridtype = gridInqType(gridID);
-
-  if (gridtype == GRID_GME) gridID = gridToUnstructured(gridID, 1);
+  if (gridtype == GRID_GME || gridtype == GRID_GAUSSIAN_REDUCED)
+    {
+      gridID = gridToUnstructured(gridID, 1);
+      gridtype = gridInqType(gridID);
+    }
 
   if (gridtype != GRID_UNSTRUCTURED && gridtype != GRID_CURVILINEAR)
     {
       gridID = gridToCurvilinear(gridID, 1);
+      gridtype = gridInqType(gridID);
       lgrid_gen_bounds = true;
     }
-  
+
   if (gridtype == GRID_UNSTRUCTURED)
     {
       if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
@@ -264,8 +269,7 @@ Outputgmt(void *process)
         }
     }
 
-  if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
-    cdoAbort("Cell center coordinates missing!");
+  if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0) cdoAbort("Cell center coordinates missing!");
 
   int nlon = gridInqXsize(gridID);
   int nlat = gridInqYsize(gridID);
@@ -303,9 +307,9 @@ Outputgmt(void *process)
       if (operatorID == OUTPUTBOUNDS || operatorID == OUTPUTBOUNDSCPT) cdoAbort("Bounds not available hovmoeller data!");
     }
 
-  int ncorner = (gridInqType(gridID) == GRID_UNSTRUCTURED) ? gridInqNvertex(gridID) : 4;
+  const int ncorner = (gridInqType(gridID) == GRID_UNSTRUCTURED) ? gridInqNvertex(gridID) : 4;
 
-  bool grid_is_circular = gridIsCircular(gridID);
+  const bool grid_is_circular = gridIsCircular(gridID);
 
   std::vector<double> grid_center_lat(gridsize), grid_center_lon(gridsize);
   gridInqYvals(gridID, grid_center_lat.data());
@@ -333,7 +337,7 @@ Outputgmt(void *process)
 
       for (int j = 0; j < nlat; ++j)
         {
-          int ij2 = j * (nlon + 1);
+          const int ij2 = j * (nlon + 1);
           grid_center_lon2[ij2 + nlon] += 360;
         }
 
@@ -351,7 +355,7 @@ Outputgmt(void *process)
   if (luse_grid_corner)
     {
       if (ncorner == 0) cdoAbort("Number of cell corners undefined!");
-      size_t nalloc = ncorner * gridsize;
+      const size_t nalloc = ncorner * gridsize;
       grid_corner_lat.resize(nalloc);
       grid_corner_lon.resize(nalloc);
 
@@ -420,8 +424,8 @@ Outputgmt(void *process)
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID, tsID)))
     {
-      int64_t vdate = taxisInqVdate(taxisID);
-      int vtime = taxisInqVtime(taxisID);
+      const int64_t vdate = taxisInqVdate(taxisID);
+      const int vtime = taxisInqVtime(taxisID);
 
       date2str(vdate, vdatestr, sizeof(vdatestr));
       time2str(vtime, vtimestr, sizeof(vtimestr));
@@ -445,29 +449,29 @@ Outputgmt(void *process)
 
           if (operatorID == OUTPUTVECTOR) fprintf(stdout, "# Increment = %d\n", ninc);
           fprintf(stdout, "#\n");
-          fprintf(stdout, "# File  = %s\n", cdoGetStreamName(0));
-          fprintf(stdout, "# Date  = %s\n", vdatestr);
-          fprintf(stdout, "# Time  = %s\n", vtimestr);
-          fprintf(stdout, "# Name  = %s\n", varname);
-          fprintf(stdout, "# Code  = %d\n", code);
+          fprintf(stdout, "# Stream = %s\n", cdoGetStreamName(0));
+          fprintf(stdout, "# Date   = %s\n", vdatestr);
+          fprintf(stdout, "# Time   = %s\n", vtimestr);
+          fprintf(stdout, "# Name   = %s\n", varname);
+          fprintf(stdout, "# Code   = %d\n", code);
         }
 
       varID0 = varID;
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID, &varID, &levelID);
+          cdoInqRecord(streamID, &varID, &levelID);
 
           if (varID != varID0) continue;
           if (recID > 0 && !lzon && !lmer) continue;
 
-          pstreamReadRecord(streamID, array.data(), &nmiss);
+          cdoReadRecord(streamID, array.data(), &nmiss);
 
           if (operatorID == OUTPUTCENTER2 && grid_is_circular) make_cyclic(array.data(), array2.data(), nlon, nlat);
 
-          double level = zaxis_center_lev[levelID];
+          const double level = zaxis_center_lev[levelID];
 
-          if ((tsID == 0 || lzon || lmer) && operatorID != OUTPUTTRI) fprintf(stdout, "# Level = %g\n", level);
+          if ((tsID == 0 || lzon || lmer) && operatorID != OUTPUTTRI) fprintf(stdout, "# Level  = %g\n", level);
           if (lhov) fprintf(stdout, "# Timestep = %d\n", tsID + 1);
 
           if (operatorID != OUTPUTTRI) fprintf(stdout, "#\n");
@@ -477,8 +481,8 @@ Outputgmt(void *process)
               if (Options::cdoVerbose)
                 {
                   double minval, maxval, meanval;
-                  arrayMinMaxMeanMV(gridsize, array.data(), missval, &minval, &maxval, &meanval);
-                  double range = maxval - minval;
+                  arrayMinMaxMeanMV(gridsize, array.data(), missval, minval, maxval, meanval);
+                  const double range = maxval - minval;
                   fprintf(stderr, "makecpt -T%g/%g/%g -Crainbow > gmt.cpt\n", minval, maxval, range / 20);
                   fprintf(stderr, "pscontour -K -JQ0/10i -Rd -I -Cgmt.cpt data.gmt > gmtplot.ps\n");
                   fprintf(stderr, "pscoast -O -J -R -Dc -W -B40g20 >> gmtplot.ps\n");
@@ -517,20 +521,18 @@ Outputgmt(void *process)
             }
           else if (operatorID == OUTPUTTRI)
             {
-              int c1, c2, c3;
-              int ip1;
               if (gridInqType(gridID) != GRID_CURVILINEAR) cdoAbort("Unsupported grid!");
 
-              int mlon = nlon - 1;
-              /* if ( gridIsCircular(gridID) ) mlon = nlon; */
+              const int mlon = nlon - 1;
+              // if ( gridIsCircular(gridID) ) mlon = nlon;
               for (int j = 0; j < nlat - 1; ++j)
                 for (int i = 0; i < mlon; ++i)
                   {
-                    ip1 = i + 1;
+                    int ip1 = i + 1;
                     if (i == nlon - 1) ip1 = 0;
-                    c1 = (j) *nlon + ip1;
-                    c2 = (j) *nlon + i;
-                    c3 = (j + 1) * nlon + i;
+                    int c1 = (j) *nlon + ip1;
+                    int c2 = (j) *nlon + i;
+                    int c3 = (j + 1) * nlon + i;
                     fprintf(stdout, "%d   %d   %d\n", c1, c2, c3);
                     c1 = (j) *nlon + i + 1;
                     c2 = (j + 1) * nlon + i;
@@ -543,13 +545,13 @@ Outputgmt(void *process)
               if (nrecs < 2) cdoAbort("Too few fields!");
 
               arrayCopy(gridsize, array.data(), uf.data());
-              pstreamInqRecord(streamID, &varID, &levelID);
-              pstreamReadRecord(streamID, vf.data(), &nmiss);
+              cdoInqRecord(streamID, &varID, &levelID);
+              cdoReadRecord(streamID, vf.data(), &nmiss);
 
               for (int j = 0; j < nlat; j += ninc)
                 for (int i = 0; i < nlon; i += ninc)
                   {
-                    /* compute length of velocity vector */
+                    // compute length of velocity vector
                     auv[IX2D(j, i, nlon)]
                         = sqrt(uf[IX2D(j, i, nlon)] * uf[IX2D(j, i, nlon)] + vf[IX2D(j, i, nlon)] * vf[IX2D(j, i, nlon)]);
 
@@ -576,8 +578,8 @@ Outputgmt(void *process)
               if (Options::cdoVerbose)
                 {
                   double minval, maxval, meanval;
-                  arrayMinMaxMeanMV(gridsize, array.data(), missval, &minval, &maxval, &meanval);
-                  double range = maxval - minval;
+                  arrayMinMaxMeanMV(gridsize, array.data(), missval, minval, maxval, meanval);
+                  const double range = maxval - minval;
                   fprintf(stderr, "makecpt -T%g/%g/%g -Crainbow > gmt.cpt\n", minval, maxval, range / 20);
                   fprintf(stderr, "psxy -K -JQ0/10i -Rd -L -Cgmt.cpt -m data.gmt > gmtplot.ps\n");
                   // fprintf(stderr, "psxy -K -Jx0.028id -Rd -L -Cgmt.cpt -m
@@ -684,7 +686,7 @@ Outputgmt(void *process)
                     {
                       const double *lon_bounds = grid_corner_lon.data() + i * ncorner;
                       const double *lat_bounds = grid_corner_lat.data() + i * ncorner;
-                      int ncorner_new = check_ncorner(ncorner, lon_bounds, lat_bounds);
+                      const int ncorner_new = check_ncorner(ncorner, lon_bounds, lat_bounds);
 
                       for (int ic = 0; ic < ncorner_new; ic++) fprintf(stdout, "   %g  %g\n", lon_bounds[ic], lat_bounds[ic]);
                       fprintf(stdout, "   %g  %g\n", lon_bounds[0], lat_bounds[0]);
