@@ -28,70 +28,47 @@
 #include <limits.h>
 
 #include <cdi.h>
-
 #include "cdo_int.h"
-#include "pstream_int.h"
 #include "datetime.h"
 
 #define NALLOC_INC 1024
 
 static int
-get_type_values(int datatype, double *tmin, double *tmax)
+get_type_values(int datatype, double &tmin, double &tmax)
 {
   int status = 0;
 
+  // clang-format off
   switch (datatype)
     {
-    case CDI_DATATYPE_INT8:
-      *tmin = -SCHAR_MAX + 1;
-      *tmax = SCHAR_MAX;
-      break;
-    case CDI_DATATYPE_UINT8:
-      *tmin = 0;
-      *tmax = UCHAR_MAX - 1;
-      break;
-    case CDI_DATATYPE_INT16:
-      *tmin = -SHRT_MAX + 1;
-      *tmax = SHRT_MAX;
-      break;
-    case CDI_DATATYPE_UINT16:
-      *tmin = 0;
-      *tmax = USHRT_MAX - 1;
-      break;
-    case CDI_DATATYPE_INT32:
-      *tmin = -INT_MAX + 1;
-      *tmax = INT_MAX;
-      break;
-    case CDI_DATATYPE_UINT32:
-      *tmin = 0;
-      *tmax = UINT_MAX - 1;
-      break;
+    case CDI_DATATYPE_INT8:   tmin = -SCHAR_MAX + 1;  tmax = SCHAR_MAX;     break;
+    case CDI_DATATYPE_UINT8:  tmin = 0;               tmax = UCHAR_MAX - 1; break;
+    case CDI_DATATYPE_INT16:  tmin = -SHRT_MAX + 1;   tmax = SHRT_MAX;      break;
+    case CDI_DATATYPE_UINT16: tmin = 0;               tmax = USHRT_MAX - 1; break;
+    case CDI_DATATYPE_INT32:  tmin = -INT_MAX + 1;    tmax = INT_MAX;       break;
+    case CDI_DATATYPE_UINT32: tmin = 0;               tmax = UINT_MAX - 1;  break;
     default: status = 1; break;
     }
+  // clang-format on
 
   return status;
 }
 
 static int
-compute_scale(int datatype, double fmin, double fmax, double *scale_factor, double *add_offset)
+computeScaleAndOffset(int datatype, double fmin, double fmax, double &scale_factor, double &add_offset)
 {
+  scale_factor = 1.0;
+  add_offset = 0.0;
+
   double tmin, tmax;
-  double ao = 0.0, sf = 1.0;
-
-  *scale_factor = sf;
-  *add_offset = ao;
-
-  if (get_type_values(datatype, &tmin, &tmax)) return 1;
+  if (get_type_values(datatype, tmin, tmax)) return 1;
 
   if (IS_NOT_EQUAL(fmin, fmax))
     {
-      sf = (fmax - fmin) / (tmax - tmin);
-      ao = ((fmax + fmin) - sf * (tmin + tmax)) / 2;
+      scale_factor = (fmax - fmin) / (tmax - tmin);
+      add_offset = ((fmax + fmin) - scale_factor * (tmin + tmax)) / 2;
     }
 
-  *scale_factor = sf;
-  *add_offset = ao;
-
   return 0;
 }
 
@@ -100,13 +77,12 @@ Pack(void *process)
 {
   int nrecs;
   int varID, levelID;
-  int nalloc = 0;
   int datatype = CDI_DATATYPE_INT16;
   DateTimeList dtlist;
 
   cdoInitialize(process);
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -115,20 +91,17 @@ Pack(void *process)
   const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
+  VarList varList;
+  varListInit(varList, vlistID1);
+
   const int nvars = vlistNvars(vlistID1);
-  std::vector<bool> varIsConst(nvars);
-  for (int varID = 0; varID < nvars; ++varID) varIsConst[varID] = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
 
   FieldVector3D vars;
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
-      if (tsID >= nalloc)
-        {
-          nalloc += NALLOC_INC;
-          vars.resize(nalloc);
-        }
+      if ((size_t)tsID >= vars.size()) vars.resize(vars.size() + NALLOC_INC);
 
       dtlist.taxisInqTimestep(taxisID1, tsID);
 
@@ -136,12 +109,11 @@ Pack(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          const int gridID = vlistInqVarGrid(vlistID1, varID);
-          const size_t gridsize = gridInqSize(gridID);
-          vars[tsID][varID][levelID].ptr = (double *) Malloc(gridsize * sizeof(double));
+          cdoInqRecord(streamID1, &varID, &levelID);
+          const size_t gridsize = varList[varID].gridsize;
+          vars[tsID][varID][levelID].resize(gridsize);
           size_t nmiss;
-          pstreamReadRecord(streamID1, vars[tsID][varID][levelID].ptr, &nmiss);
+          cdoReadRecord(streamID1, vars[tsID][varID][levelID].vec.data(), &nmiss);
           vars[tsID][varID][levelID].nmiss = nmiss;
         }
 
@@ -169,31 +141,28 @@ Pack(void *process)
     {
       double fmin = 1.e300;
       double fmax = -1.e300;
-      double sf, ao;
       size_t nmisspv = 0;
 
-      const int gridID = vlistInqVarGrid(vlistID1, varID);
-      const double missval1 = vlistInqVarMissval(vlistID1, varID);
-      const size_t gridsize = gridInqSize(gridID);
-      const int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-
-      for (levelID = 0; levelID < nlevel; levelID++)
+      const int timetype = varList[varID].timetype;
+      const double missval1 = varList[varID].missval;
+      const size_t gridsize = varList[varID].gridsize;
+      const int nlevels = varList[varID].nlevels;
+      for (levelID = 0; levelID < nlevels; levelID++)
         {
           for (int tsID = 0; tsID < nts; tsID++)
             {
-              double zmin = 1.e300;
-              double zmax = -1.e300;
-              if (tsID > 0 && varIsConst[varID]) continue;
-              double *array = vars[tsID][varID][levelID].ptr;
-              size_t nmiss = vars[tsID][varID][levelID].nmiss;
-              if (nmiss > 0)
+              double zmin, zmax;
+              if (tsID > 0 && timetype == TIME_CONSTANT) continue;
+              const double *array = vars[tsID][varID][levelID].vec.data();
+              const size_t nmiss = vars[tsID][varID][levelID].nmiss;
+              if (nmiss)
                 {
                   nmisspv += nmiss;
-                  arrayMinMaxMV(gridsize, array, missval1, &zmin, &zmax);
+                  arrayMinMaxMV(gridsize, array, missval1, zmin, zmax);
                 }
               else
                 {
-                  arrayMinMax(gridsize, array, &zmin, &zmax);
+                  arrayMinMax(gridsize, array, zmin, zmax);
                 }
 
               if (zmin < fmin) fmin = zmin;
@@ -202,24 +171,24 @@ Pack(void *process)
         }
 
       vlistDefVarDatatype(vlistID2, varID, datatype);
-      double missval2 = vlistInqVarMissval(vlistID2, varID);
+      const double missval2 = vlistInqVarMissval(vlistID2, varID);
 
-      if (nmisspv > 0)
+      if (nmisspv)
         {
           double tmin, tmax;
-          if (!get_type_values(datatype, &tmin, &tmax))
+          if (!get_type_values(datatype, tmin, tmax))
             {
               if (!(missval2 < tmin || missval2 > tmax))
                 cdoWarning("new missing value %g is inside data range (%g - %g)!", missval2, tmin, tmax);
 
-              for (levelID = 0; levelID < nlevel; levelID++)
+              for (levelID = 0; levelID < nlevels; levelID++)
                 {
                   for (int tsID = 0; tsID < nts; tsID++)
                     {
-                      if (tsID > 0 && varIsConst[varID]) continue;
-                      double *array = vars[tsID][varID][levelID].ptr;
+                      if (tsID > 0 && timetype == TIME_CONSTANT) continue;
+                      auto &array = vars[tsID][varID][levelID].vec;
                       const size_t nmiss = vars[tsID][varID][levelID].nmiss;
-                      if (nmiss > 0)
+                      if (nmiss)
                         for (size_t i = 0; i < gridsize; ++i)
                           if (DBL_IS_EQUAL(array[i], missval1)) array[i] = missval2;
                     }
@@ -229,7 +198,8 @@ Pack(void *process)
 
       // printf("fmin %g fmax %g missval %g\n", fmin, fmax,
       // vlistInqVarMissval(vlistID2, varID));
-      if (!compute_scale(datatype, fmin, fmax, &sf, &ao))
+      double sf, ao;
+      if (!computeScaleAndOffset(datatype, fmin, fmax, sf, ao))
         {
           // printf("sf = %g ao = %g \n", sf, ao);
           // printf("smin %g smax %g\n", (fmin - ao)/sf, (fmax -ao)/sf);
@@ -239,32 +209,29 @@ Pack(void *process)
         }
     }
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   for (int tsID = 0; tsID < nts; tsID++)
     {
       dtlist.taxisDefTimestep(taxisID2, tsID);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (varID = 0; varID < nvars; varID++)
         {
-          if (tsID > 0 && varIsConst[varID]) continue;
-          const int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-          for (levelID = 0; levelID < nlevel; levelID++)
+          const int timetype = varList[varID].timetype;
+          if (tsID > 0 && timetype == TIME_CONSTANT) continue;
+          const int nlevels = varList[varID].nlevels;
+          for (levelID = 0; levelID < nlevels; levelID++)
             {
-              if (vars[tsID][varID][levelID].ptr)
+              if (!vars[tsID][varID][levelID].empty())
                 {
                   const size_t nmiss = vars[tsID][varID][levelID].nmiss;
-                  pstreamDefRecord(streamID2, varID, levelID);
-                  pstreamWriteRecord(streamID2, vars[tsID][varID][levelID].ptr, nmiss);
-                  Free(vars[tsID][varID][levelID].ptr);
-                  vars[tsID][varID][levelID].ptr = nullptr;
+                  cdoDefRecord(streamID2, varID, levelID);
+                  cdoWriteRecord(streamID2, vars[tsID][varID][levelID].vec.data(), nmiss);
                 }
             }
         }
-
-      fieldsFree(vlistID1, vars[tsID]);
     }
 
   cdoStreamClose(streamID2);
