@@ -26,7 +26,6 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
 
 void *
 Pardup(void *process)
@@ -58,7 +57,7 @@ Pardup(void *process)
   else
     cdoAbort("operator not implemented!");
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   int vlistID1 = cdoStreamInqVlist(streamID1);
   int vlistID2 = vlistDuplicate(vlistID1);
@@ -70,7 +69,7 @@ Pardup(void *process)
   int nvars = vlistNvars(vlistID1);
 
   int maxrecs = vlistNrecs(vlistID1);
-  std::vector<RecordInfo> recinfo(maxrecs);
+  std::vector<RecordInfo> recList(maxrecs);
 
   size_t gridsize = vlistGridsizeMax(vlistID1);
   std::vector<double> array(gridsize);
@@ -92,36 +91,36 @@ Pardup(void *process)
         vlistDefVarParam(vlistID2, varID + nvars * i, cdiEncodeParam(-(varID + nvars * i + 1), 255, 255));
     }
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
-          recinfo[recID].varID = varID;
-          recinfo[recID].levelID = levelID;
-          recinfo[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+          recList[recID].varID = varID;
+          recList[recID].levelID = levelID;
+          recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
 
           gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
           offset = gridsize * levelID;
           single = &vardata[varID][offset];
 
-          pstreamReadRecord(streamID1, single, &nmiss);
+          cdoReadRecord(streamID1, single, &nmiss);
           varnmiss[varID][levelID] = nmiss;
         }
 
       for (int i = 0; i < nmul; i++)
         for (int recID = 0; recID < nrecs; recID++)
           {
-            varID = recinfo[recID].varID;
-            levelID = recinfo[recID].levelID;
+            varID = recList[recID].varID;
+            levelID = recList[recID].levelID;
             varID2 = varID + i * nvars;
 
             gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
@@ -130,8 +129,8 @@ Pardup(void *process)
             nmiss = varnmiss[varID][levelID];
 
             arrayCopy(gridsize, single, array.data());
-            pstreamDefRecord(streamID2, varID2, levelID);
-            pstreamWriteRecord(streamID2, array.data(), nmiss);
+            cdoDefRecord(streamID2, varID2, levelID);
+            cdoWriteRecord(streamID2, array.data(), nmiss);
           }
 
       tsID++;
