@@ -23,7 +23,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 #include "printinfo.h"
 
 void *
@@ -50,7 +49,7 @@ Pinfo(void *process)
 
   int operatorID = cdoOperatorID();
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   int vlistID1 = cdoStreamInqVlist(streamID1);
   int vlistID2 = vlistDuplicate(vlistID1);
@@ -59,8 +58,8 @@ Pinfo(void *process)
   int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   size_t gridsize = vlistGridsizeMax(vlistID1);
 
@@ -75,7 +74,7 @@ Pinfo(void *process)
       int vtime = taxisInqVtime(taxisID1);
 
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       date2str(vdate, vdatestr, sizeof(vdatestr));
       time2str(vtime, vtimestr, sizeof(vtimestr));
@@ -91,8 +90,8 @@ Pinfo(void *process)
                 fprintf(stdout, "   Rec :       Date  Time    Code  Level    Size    Miss :     Minimum        Mean     Maximum\n");
             }
 
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, array1.data(), &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, array1.data(), &nmiss);
 
           indg += 1;
           int code = vlistInqVarCode(vlistID1, varID);
@@ -119,15 +118,15 @@ Pinfo(void *process)
             }
           else
             {
-              if (nmiss > 0)
+              if (nmiss)
                 {
-                  ivals = arrayMinMaxMeanMV(gridsize, array1.data(), missval, &arrmin, &arrmax, &arrmean);
+                  ivals = arrayMinMaxMeanMV(gridsize, array1.data(), missval, arrmin, arrmax, arrmean);
                   imiss = gridsize - ivals;
                   gridsize = ivals;
                 }
               else
                 {
-                  arrayMinMaxMean(gridsize, array1.data(), &arrmin, &arrmax, &arrmean);
+                  arrayMinMaxMean(gridsize, array1.data(), arrmin, arrmax, arrmean);
                 }
 
               if (gridsize)
@@ -139,13 +138,13 @@ Pinfo(void *process)
                   fprintf(stdout, "                     nan\n");
                 }
 
-              if (imiss != nmiss && nmiss > 0) fprintf(stdout, "Found %zu of %zu missing values!\n", imiss, nmiss);
+              if (imiss != nmiss && nmiss) fprintf(stdout, "Found %zu of %zu missing values!\n", imiss, nmiss);
             }
 
           arrayCopy(gridsize, array1.data(), array2.data());
 
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, array2.data(), nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, array2.data(), nmiss);
         }
 
       tsID++;
