@@ -26,7 +26,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 #include "after_vertint.h"
 #include "stdnametable.h"
 #include "util_string.h"
@@ -61,7 +60,7 @@ Pressure(void *process)
 
   int operatorID = cdoOperatorID();
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   int vlistID1 = cdoStreamInqVlist(streamID1);
 
@@ -162,10 +161,10 @@ Pressure(void *process)
         {
           vlistInqVarName(vlistID1, varID, varname);
 
-          strtolower(varname);
+          cstrToLowerCase(varname);
 
           //                       ECHAM                         ECMWF
-          if (cstrIsEqual(varname, "geosp") || cstrIsEqual(varname, "z") )
+          if (cstrIsEqual(varname, "geosp") || cstrIsEqual(varname, "z"))
             code = 129;
           else if (cstrIsEqual(varname, "st") || cstrIsEqual(varname, "t"))
             code = 130;
@@ -226,23 +225,23 @@ Pressure(void *process)
   int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           if (varID == pvarID)
             {
-              pstreamReadRecord(streamID1, array.data(), &nmiss);
-              if (nmiss > 0) cdoAbort("Missing valus unsupported!");
+              cdoReadRecord(streamID1, array.data(), &nmiss);
+              if (nmiss) cdoAbort("Missing valus unsupported!");
             }
         }
 
@@ -254,7 +253,7 @@ Pressure(void *process)
             arrayCopy(gridsize, array.data(), ps_prog.data());
 
           // check range of ps_prog
-          arrayMinMaxMask(gridsize, ps_prog.data(), nullptr, &minval, &maxval);
+          arrayMinMax(gridsize, ps_prog.data(), minval, maxval);
           if (minval < MIN_PS || maxval > MAX_PS) cdoWarning("Surface pressure out of range (min=%g max=%g)!", minval, maxval);
 
           presh(full_press.data(), half_press.data(), vct, ps_prog.data(), nhlevf, gridsize);
@@ -285,9 +284,9 @@ Pressure(void *process)
       varID = 0;
       for (levelID = 0; levelID < nlevel; levelID++)
         {
-          pstreamDefRecord(streamID2, varID, levelID);
+          cdoDefRecord(streamID2, varID, levelID);
           size_t offset = levelID * gridsize;
-          pstreamWriteRecord(streamID2, pout + offset, 0);
+          cdoWriteRecord(streamID2, pout + offset, 0);
         }
 
       tsID++;
