@@ -41,9 +41,10 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
+
 #include "remap.h"
-#include "grid.h"
+#include <mpim_grid.h>
+#include "griddes.h"
 #include "cdo_options.h"
 #include "timer.h"
 
@@ -181,7 +182,7 @@ remapPrintInfo(int operfunc, bool remap_genweights, RemapGrid &src_grid, RemapGr
   strcat(line, tmpstr);
   strcat(line, " grid");
 
-  if (nmiss > 0)
+  if (nmiss)
     {
       snprintf(tmpstr, sizeof(tmpstr), ", with source mask (%zu)", gridInqSize(src_grid.gridID) - nmiss);
       strcat(line, tmpstr);
@@ -210,7 +211,7 @@ remapPrintWarning(const char *remap_file, int operfunc, RemapGrid &src_grid, siz
   strcat(line, tmpstr);
   strcat(line, " grid");
 
-  if (nmiss > 0)
+  if (nmiss)
     {
       snprintf(tmpstr, sizeof(tmpstr), " with mask (%zu)", gridInqSize(src_grid.gridID) - nmiss);
       strcat(line, tmpstr);
@@ -283,8 +284,7 @@ remapGetenv(void)
       else if (strcmp(envstr, "merge") == 0)
         sort_mode = MERGE_SORT;
 
-      if (Options::cdoVerbose)
-        cdoPrint("Set sort_mode to %s", sort_mode == HEAP_SORT ? "HEAP_SORT" : "MERGE_SORT");
+      if (Options::cdoVerbose) cdoPrint("Set sort_mode to %s", sort_mode == HEAP_SORT ? "HEAP_SORT" : "MERGE_SORT");
     }
 
   envstr = getenv("REMAP_THRESHHOLD");
@@ -763,7 +763,7 @@ remapField(RemapMethod mapType, remapType *remap, int numNeighbors, double *arra
 }
 
 static void
-remapAddOperators(void)
+addOperators(void)
 {
   // clang-format off
   cdoOperatorAdd("remap",        REMAP,        0, nullptr);
@@ -795,7 +795,7 @@ Remap(void *argument)
   RemapMethod mapType(RemapMethod::UNDEF);
   SubmapType submapType(SubmapType::NONE);
   bool remap_genweights = Options::REMAP_genweights;
-  int streamID2 = -1;
+  CdoStreamID streamID2 = CDO_STREAM_UNDEF;
   int nrecs;
   int varID, levelID;
   size_t nmiss1, nmiss2 = 0;
@@ -808,7 +808,7 @@ Remap(void *argument)
 
   cdoInitialize(argument);
 
-  remapAddOperators();
+  addOperators();
 
   const int operatorID = cdoOperatorID();
   int operfunc = cdoOperatorF1(operatorID);
@@ -848,9 +848,9 @@ Remap(void *argument)
 
   if (gridInqType(gridID2) == GRID_GENERIC) cdoAbort("Unsupported target grid type (generic)!");
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  const int filetype = pstreamInqFiletype(streamID1);
+  const int filetype = cdoInqFiletype(streamID1);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -954,8 +954,8 @@ Remap(void *argument)
 
   if (!writeRemapWeightsOnly)
     {
-      streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-      pstreamDefVlist(streamID2, vlistID2);
+      streamID2 = cdoOpenWrite(1);
+      cdoDefVlist(streamID2, vlistID2);
     }
 
   int tsID = 0;
@@ -963,12 +963,12 @@ Remap(void *argument)
     {
       taxisCopyTimestep(taxisID2, taxisID1);
 
-      if (!writeRemapWeightsOnly) pstreamDefTimestep(streamID2, tsID);
+      if (!writeRemapWeightsOnly) cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, &array1[0], &nmiss1);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, &array1[0], &nmiss1);
 
           const int gridID1 = vlistInqVarGrid(vlistID1, varID);
           const double missval = vlistInqVarMissval(vlistID1, varID);
@@ -1155,7 +1155,8 @@ Remap(void *argument)
 
               // calculate some statistics
               if (Options::cdoVerbose)
-                remapStat(remapOrder, remaps[r].src_grid, remaps[r].tgt_grid, remaps[r].vars, array1.data(), array2.data(), missval);
+                remapStat(remapOrder, remaps[r].src_grid, remaps[r].tgt_grid, remaps[r].vars, array1.data(), array2.data(),
+                          missval);
 
               if (gridInqType(gridID2) == GRID_GME)
                 {
@@ -1172,8 +1173,8 @@ Remap(void *argument)
               nmiss2 = arrayNumMV(gridsize2, array2.data(), missval);
             }
 
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, array2.data(), nmiss2);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, array2.data(), nmiss2);
         }
 
       tsID++;
@@ -1184,8 +1185,8 @@ Remap(void *argument)
 WRITE_REMAP:
 
   if (writeRemapWeightsOnly)
-    remapWriteDataScrip(cdoGetStreamName(1), mapType, submapType, numNeighbors, remapOrder, remaps[r].src_grid,
-                        remaps[r].tgt_grid, remaps[r].vars);
+    remapWriteDataScrip(cdoGetStreamName(1), mapType, submapType, numNeighbors, remapOrder, remaps[r].src_grid, remaps[r].tgt_grid,
+                        remaps[r].vars);
 
   cdoStreamClose(streamID1);
 
