@@ -20,11 +20,10 @@
       Remapeta     remapeta          Model to model level interpolation
 */
 
-#include "hetaeta.h"
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
+#include "hetaeta.h"
 #include "after_vertint.h"
 #include "stdnametable.h"
 #include "util_string.h"
@@ -32,6 +31,7 @@
 #include "const.h"
 #include "cdo_options.h"
 #include "cdo_zaxis.h"
+#include "cdi_lockedIO.h"
 
 static void
 setmissval(long nvals, const int *imiss, double missval, double *array)
@@ -140,7 +140,7 @@ vctFromFile(const char *filename, int *nvct, std::vector<double> &vct2)
 }
 
 static void
-vertSum(std::vector<double> &sum, const double *var3d, size_t gridsize, size_t  nlevel)
+vertSum(std::vector<double> &sum, const double *var3d, size_t gridsize, size_t nlevel)
 {
   for (size_t i = 0; i < gridsize; ++i) sum[i] = 0;
 
@@ -152,7 +152,7 @@ vertSum(std::vector<double> &sum, const double *var3d, size_t gridsize, size_t
 }
 
 static void
-vertSumw(std::vector<double> &sum, const double *var3d, size_t gridsize, size_t  nlevel, const std::vector<double> &deltap)
+vertSumw(std::vector<double> &sum, const double *var3d, size_t gridsize, size_t nlevel, const std::vector<double> &deltap)
 {
   for (size_t i = 0; i < gridsize; ++i) sum[i] = 0;
 
@@ -273,14 +273,14 @@ Remapeta(void *process)
   if (Options::cdoVerbose)
     for (i = 0; i < nhlevf2 + 1; ++i) cdoPrint("vct2: %5d %25.17f %25.17f", i, vct2[i], vct2[nvct2 / 2 + i]);
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   if (operatorArgc() == 2)
     {
       lfis2 = true;
 
       const char *fname = operatorArgv()[1];
-      const int streamID = streamOpenReadLocked(fname);
+      int streamID = streamOpenReadLocked(fname);
       const int vlistID1 = streamInqVlist(streamID);
 
       streamInqRecord(streamID, &varID, &levelID);
@@ -304,7 +304,7 @@ Remapeta(void *process)
         }
 
       // check range of surface_geopotential
-      arrayMinMaxMask(nfis2gp, fis2.data(), imiss, &minval, &maxval);
+      arrayMinMaxMask(nfis2gp, fis2.data(), imiss, minval, maxval);
       if (minval < MIN_FIS || maxval > MAX_FIS)
         cdoWarning("%s out of range (min=%g max=%g)!", var_stdname(surface_geopotential), minval, maxval);
 
@@ -359,8 +359,8 @@ Remapeta(void *process)
   if (Options::cdoVerbose)
     for (i = 0; i < nvct1 / 2; ++i) cdoPrint("vct1: %5d %25.17f %25.17f", i, vct1[i], vct1[nvct1 / 2 + i]);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   if (zaxisIDh == -1) cdoWarning("No 3D variable with hybrid sigma pressure coordinate found!");
 
@@ -377,10 +377,10 @@ Remapeta(void *process)
       if (code <= 0 || code == 255)
         {
           vlistInqVarName(vlistID1, varID, varname);
-          strtolower(varname);
+          cstrToLowerCase(varname);
 
           vlistInqVarStdname(vlistID1, varID, stdname);
-          strtolower(stdname);
+          cstrToLowerCase(stdname);
 
           code = echamcode_from_stdname(stdname);
 
@@ -532,15 +532,15 @@ Remapeta(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
           const int zaxisID = vlistInqVarZaxis(vlistID1, varID);
           const int nlevel = zaxisInqSize(zaxisID);
           const size_t offset = gridsize * levelID;
-          pstreamReadRecord(streamID1, array.data(), &nmiss);
+          cdoReadRecord(streamID1, array.data(), &nmiss);
 
           if (zaxisIDh != -1)
             {
@@ -569,25 +569,25 @@ Remapeta(void *process)
                 }
               else
                 {
-                  pstreamDefRecord(streamID2, varID, levelID);
-                  pstreamWriteRecord(streamID2, array.data(), nmiss);
+                  cdoDefRecord(streamID2, varID, levelID);
+                  cdoWriteRecord(streamID2, array.data(), nmiss);
                 }
             }
           else
             {
-              pstreamDefRecord(streamID2, varID, levelID);
-              pstreamWriteRecord(streamID2, array.data(), nmiss);
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, array.data(), nmiss);
             }
         }
 
       if (zaxisIDh != -1)
         {
           // check range of ps_prog
-          arrayMinMaxMask(gridsize, ps1.data(), imiss, &minval, &maxval);
+          arrayMinMaxMask(gridsize, ps1.data(), imiss, minval, maxval);
           if (minval < MIN_PS || maxval > MAX_PS) cdoWarning("Surface pressure out of range (min=%g max=%g)!", minval, maxval);
 
           // check range of geop
-          arrayMinMaxMask(gridsize, fis1.data(), imiss, &minval, &maxval);
+          arrayMinMaxMask(gridsize, fis1.data(), imiss, minval, maxval);
           if (minval < MIN_FIS || maxval > MAX_FIS) cdoWarning("Orography out of range (min=%g max=%g)!", minval, maxval);
         }
 
@@ -603,7 +603,7 @@ Remapeta(void *process)
               const size_t offset = gridsize * levelID;
               single2 = t1 + offset;
 
-              arrayMinMaxMask(gridsize, single2, imiss, &minval, &maxval);
+              arrayMinMaxMask(gridsize, single2, imiss, minval, maxval);
               if (minval < MIN_T || maxval > MAX_T)
                 cdoWarning("Input temperature at level %d out of range (min=%g max=%g)!", levelID + 1, minval, maxval);
             }
@@ -617,7 +617,7 @@ Remapeta(void *process)
 
               corr_hum(gridsize, single2, MIN_Q);
 
-              arrayMinMaxMask(gridsize, single2, imiss, &minval, &maxval);
+              arrayMinMaxMask(gridsize, single2, imiss, minval, maxval);
               if (minval < MIN_Q || maxval > MAX_Q)
                 cdoWarning("Input humidity at level %d out of range (min=%g max=%g)!", levelID + 1, minval, maxval);
             }
@@ -626,8 +626,8 @@ Remapeta(void *process)
       if (nvars3D || ltq)
         {
           if (Options::Timer) timer_start(timer_hetaeta);
-          hetaeta(ltq, gridsize, imiss, nhlevf1, a1, b1, fis1.data(), ps1.data(), t1, q1, nhlevf2, a2, b2, fis2.data(), ps2.data(), t2, q2, nvars3D, vars1,
-                  vars2, tscor, pscor, secor);
+          hetaeta(ltq, gridsize, imiss, nhlevf1, a1, b1, fis1.data(), ps1.data(), t1, q1, nhlevf2, a2, b2, fis2.data(), ps2.data(),
+                  t2, q2, nvars3D, vars1, vars2, tscor, pscor, secor);
           if (Options::Timer) timer_stop(timer_hetaeta);
         }
 
@@ -638,8 +638,8 @@ Remapeta(void *process)
           varID = sgeopotID;
           levelID = 0;
           setmissval(gridsize, imiss, missval, fis2.data());
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, fis2.data(), nmissout);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, fis2.data(), nmissout);
         }
 
       if (zaxisIDh != -1 && lnpsID != -1)
@@ -650,8 +650,8 @@ Remapeta(void *process)
           varID = presID;
           levelID = 0;
           setmissval(gridsize, imiss, missval, ps2.data());
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, ps2.data(), nmissout);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, ps2.data(), nmissout);
         }
 
       if (ltq)
@@ -663,13 +663,13 @@ Remapeta(void *process)
               const size_t offset = gridsize * levelID;
               single2 = t2 + offset;
 
-              arrayMinMaxMask(gridsize, single2, imiss, &minval, &maxval);
+              arrayMinMaxMask(gridsize, single2, imiss, minval, maxval);
               if (minval < MIN_T || maxval > MAX_T)
                 cdoWarning("Output temperature at level %d out of range (min=%g max=%g)!", levelID + 1, minval, maxval);
 
               setmissval(gridsize, imiss, missval, single2);
-              pstreamDefRecord(streamID2, varID, levelID);
-              pstreamWriteRecord(streamID2, single2, nmissout);
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, single2, nmissout);
             }
 
           varID = sqID;
@@ -684,13 +684,13 @@ Remapeta(void *process)
               if (levelID < nctop)
                 for (size_t i = 0; i < gridsize; ++i) single2[i] = cconst;
 
-              arrayMinMaxMask(gridsize, single2, imiss, &minval, &maxval);
+              arrayMinMaxMask(gridsize, single2, imiss, minval, maxval);
               if (minval < MIN_Q || maxval > MAX_Q)
                 cdoWarning("Output humidity at level %d out of range (min=%g max=%g)!", levelID + 1, minval, maxval);
 
               setmissval(gridsize, imiss, missval, single2);
-              pstreamDefRecord(streamID2, varID, levelID);
-              pstreamWriteRecord(streamID2, single2, nmissout);
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, single2, nmissout);
             }
         }
 
@@ -743,8 +743,8 @@ Remapeta(void *process)
                 }
 
               setmissval(gridsize, imiss, missval, single2);
-              pstreamDefRecord(streamID2, varID, levelID);
-              pstreamWriteRecord(streamID2, single2, nmissout);
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, single2, nmissout);
             }
         }
 
