@@ -24,7 +24,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 
 #define MAX_VARS 1024
 
@@ -46,20 +45,24 @@ Replace(void *process)
 
   cdoInitialize(process);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID3 = taxisDuplicate(taxisID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID3 = taxisDuplicate(taxisID1);
 
-  int streamID2 = cdoStreamOpenRead(cdoStreamName(1));
+  CdoStreamID streamID2 = cdoOpenRead(1);
 
-  int vlistID2 = cdoStreamInqVlist(streamID2);
+  const int vlistID2 = cdoStreamInqVlist(streamID2);
 
-  /* compare all variables in vlistID2 */
+  VarList varList1, varList2;
+  varListInit(varList1, vlistID1);
+  varListInit(varList2, vlistID2);
 
-  int nvars1 = vlistNvars(vlistID1);
-  int nvars2 = vlistNvars(vlistID2);
+  // compare all variables in vlistID2
+
+  const int nvars1 = vlistNvars(vlistID1);
+  const int nvars2 = vlistNvars(vlistID2);
 
   for (varID2 = 0; varID2 < nvars2; varID2++)
     {
@@ -73,11 +76,11 @@ Replace(void *process)
 
       if (varID1 < nvars1)
         {
-          size_t gridsize1 = gridInqSize(vlistInqVarGrid(vlistID1, varID1));
-          int nlevel1 = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID1));
+          const size_t gridsize1 = varList1[varID1].gridsize;
+          const int nlevel1 = varList1[varID1].nlevels;
 
-          size_t gridsize2 = gridInqSize(vlistInqVarGrid(vlistID2, varID2));
-          int nlevel2 = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID2));
+          const size_t gridsize2 = varList2[varID2].gridsize;
+          const int nlevel2 = varList2[varID2].nlevels;
 
           if (gridsize1 != gridsize2) cdoAbort("Variables have different gridsize!");
 
@@ -105,9 +108,9 @@ Replace(void *process)
         {
           varID1 = varlist1[idx];
           varID2 = varlist2[idx];
-          int nlevel1 = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID1));
-          int nlevel2 = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID2));
-          size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID2));
+          const int nlevel1 = varList1[varID1].nlevels;
+          const int nlevel2 = varList2[varID2].nlevels;
+          const size_t gridsize = varList2[varID2].gridsize;
           vardata2[idx].resize(nlevel2 * gridsize);
           varnmiss2[idx].resize(nlevel2);
           varlevel[idx].resize(nlevel1);
@@ -140,17 +143,17 @@ Replace(void *process)
         }
     }
 
-  int vlistID3 = vlistDuplicate(vlistID1);
+  const int vlistID3 = vlistDuplicate(vlistID1);
 
-  int streamID3 = cdoStreamOpenWrite(cdoStreamName(2));
+  CdoStreamID streamID3 = cdoOpenWrite(2);
 
   vlistDefTaxis(vlistID3, taxisID3);
-  pstreamDefVlist(streamID3, vlistID3);
+  cdoDefVlist(streamID3, vlistID3);
 
   size_t gridsize = vlistGridsizeMax(vlistID1);
   std::vector<double> array(gridsize);
 
-  int nts2 = vlistNtsteps(vlistID2);
+  const int nts2 = vlistNtsteps(vlistID2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
@@ -164,25 +167,24 @@ Replace(void *process)
 
           for (int recID = 0; recID < nrecs2; recID++)
             {
-              pstreamInqRecord(streamID2, &varID, &levelID);
+              cdoInqRecord(streamID2, &varID, &levelID);
 
               for (idx = 0; idx < nchvars; idx++)
                 if (varlist2[idx] == varID)
                   {
-                    size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
-                    size_t offset = gridsize * levelID;
-                    pstreamReadRecord(streamID2, &vardata2[idx][offset], &nmiss);
+                    const size_t offset = varList2[varID].gridsize * levelID;
+                    cdoReadRecord(streamID2, &vardata2[idx][offset], &nmiss);
                     varnmiss2[idx][levelID] = nmiss;
                     break;
                   }
             }
         }
 
-      pstreamDefTimestep(streamID3, tsID);
+      cdoDefTimestep(streamID3, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           double *parray = array.data();
 
@@ -192,18 +194,17 @@ Replace(void *process)
                 levelID2 = varlevel[idx][levelID];
                 if (levelID2 != -1)
                   {
-                    size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-                    size_t offset = gridsize * levelID2;
+                    const size_t offset = varList1[varID].gridsize * levelID2;
                     parray = &vardata2[idx][offset];
                     nmiss = varnmiss2[idx][levelID2];
                     break;
                   }
               }
 
-          if (idx == nchvars) pstreamReadRecord(streamID1, parray, &nmiss);
+          if (idx == nchvars) cdoReadRecord(streamID1, parray, &nmiss);
 
-          pstreamDefRecord(streamID3, varID, levelID);
-          pstreamWriteRecord(streamID3, parray, nmiss);
+          cdoDefRecord(streamID3, varID, levelID);
+          cdoWriteRecord(streamID3, parray, nmiss);
         }
 
       tsID++;
