@@ -27,7 +27,7 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
+
 #include "listarray.h"
 
 void *
@@ -45,12 +45,12 @@ Replacevalues(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  int SETVALS  = cdoOperatorAdd("setvals",  0, 0, "I1,O1,...,In,On");
-  int SETRTOC  = cdoOperatorAdd("setrtoc",  0, 0, "range (min, max), value");
-  int SETRTOC2 = cdoOperatorAdd("setrtoc2", 0, 0, "range (min, max), value1, value2");
+  const int SETVALS  = cdoOperatorAdd("setvals",  0, 0, "I1,O1,...,In,On");
+  const int SETRTOC  = cdoOperatorAdd("setrtoc",  0, 0, "range (min, max), value");
+  const int SETRTOC2 = cdoOperatorAdd("setrtoc2", 0, 0, "range (min, max), value1, value2");
   // clang-format on
 
-  int operatorID = cdoOperatorID();
+  const int operatorID = cdoOperatorID();
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
@@ -78,50 +78,48 @@ Replacevalues(void *process)
       newval2 = parameter2double(operatorArgv()[3]);
     }
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
+  CdoStreamID streamID2 = cdoOpenWrite(1);
 
-  pstreamDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID2, vlistID2);
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
-
-  double *array = (double *) Malloc(gridsize * sizeof(double));
+  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  std::vector<double> array(gridsizemax);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, array, &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, array.data(), &nmiss);
 
-          gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-          double missval = vlistInqVarMissval(vlistID1, varID);
+          const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
+          const double missval = vlistInqVarMissval(vlistID1, varID);
 
           if (operatorID == SETVALS)
             {
               for (size_t i = 0; i < gridsize; i++)
                 if (!DBL_IS_EQUAL(array[i], missval))
                   {
-                    /* printf("\nelem %d val %f ",i,array[i]); */
+                    // printf("\nelem %d val %f ",i,array[i]);
                     for (int j = 0; j < nvals; j++)
                       {
                         if (DBL_IS_EQUAL(array[i], fltarr[j * 2]))
                           {
                             array[i] = fltarr[j * 2 + 1];
-                            /* printf("j=%d %f %f
-                             * ",j,fltarr[j*2],fltarr[j*2+1]); */
+                            // printf("j=%d %f %f",j,fltarr[j*2],fltarr[j*2+1]);
                             break;
                           }
                       }
@@ -140,15 +138,12 @@ Replacevalues(void *process)
               for (size_t i = 0; i < gridsize; i++)
                 if (!DBL_IS_EQUAL(array[i], missval))
                   {
-                    if (array[i] >= rmin && array[i] <= rmax)
-                      array[i] = newval;
-                    else
-                      array[i] = newval2;
+                    array[i] = (array[i] >= rmin && array[i] <= rmax) ? newval : newval2;
                   }
             }
 
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, array, nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, array.data(), nmiss);
         }
 
       tsID++;
@@ -157,8 +152,6 @@ Replacevalues(void *process)
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
-  if (array) Free(array);
-
   cdoFinish();
 
   return nullptr;
