@@ -25,8 +25,8 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
-#include "grid.h"
+
+#include <mpim_grid.h>
 
 static void
 rot_uv_back(int gridID, double *us, double *vs)
@@ -35,44 +35,35 @@ rot_uv_back(int gridID, double *us, double *vs)
   if (gridInqType(gridID) == GRID_PROJECTION && gridInqProjType(gridID) == CDI_PROJ_RLL)
     gridInqParamRLL(gridID, &xpole, &ypole, &angle);
 
-  size_t nlon = gridInqXsize(gridID);
-  size_t nlat = gridInqYsize(gridID);
+  const size_t nlon = gridInqXsize(gridID);
+  const size_t nlat = gridInqYsize(gridID);
 
-  double *xvals = (double *) Malloc(nlon * sizeof(double));
-  double *yvals = (double *) Malloc(nlat * sizeof(double));
-  gridInqXvals(gridID, xvals);
-  gridInqYvals(gridID, yvals);
+  std::vector<double> xvals(nlon);
+  std::vector<double> yvals(nlat);
+  gridInqXvals(gridID, xvals.data());
+  gridInqYvals(gridID, yvals.data());
 
   /* Convert lat/lon units if required */
   char units[CDI_MAX_NAME];
   gridInqXunits(gridID, units);
   grid_to_degree(units, 1, &angle, "angle");
   grid_to_degree(units, 1, &xpole, "xpole");
-  grid_to_degree(units, nlon, xvals, "grid center lon");
+  grid_to_degree(units, nlon, xvals.data(), "grid center lon");
   gridInqYunits(gridID, units);
   grid_to_degree(units, 1, &ypole, "ypole");
-  grid_to_degree(units, nlat, yvals, "grid center lat");
+  grid_to_degree(units, nlat, yvals.data(), "grid center lat");
 
   if (xpole > 180) xpole -= 360;
   if (angle > 180) angle -= 360;
 
-  double u, v;
   for (size_t ilat = 0; ilat < nlat; ilat++)
     for (size_t ilon = 0; ilon < nlon; ilon++)
       {
-        size_t i = ilat * nlon + ilon;
-
-        double xval = lamrot_to_lam(yvals[ilat], xvals[ilon], ypole, xpole, angle);
-        double yval = phirot_to_phi(yvals[ilat], xvals[ilon], ypole, angle);
-
-        usvs_to_uv(us[i], vs[i], yval, xval, ypole, xpole, &u, &v);
-
-        us[i] = u;
-        vs[i] = v;
+        const size_t i = ilat * nlon + ilon;
+        const double xval = lamrot_to_lam(yvals[ilat], xvals[ilon], ypole, xpole, angle);
+        const double yval = phirot_to_phi(yvals[ilat], xvals[ilon], ypole, angle);
+        usvs_to_uv(us[i], vs[i], yval, xval, ypole, xpole, &us[i], &vs[i]);
       }
-
-  Free(xvals);
-  Free(yvals);
 }
 
 #define MAXARG 16384
@@ -81,10 +72,7 @@ void *
 Rotuv(void *process)
 {
   int varID, levelID;
-  int varID1, varID2, nlevel1, nlevel2;
-  size_t gridsize;
-  int code, gridID;
-  int nlevel;
+  int varID1, varID2;
   int nrecs;
   int chcodes[MAXARG];
   char *chvars[MAXARG];
@@ -118,15 +106,15 @@ Rotuv(void *process)
       for (int i = 0; i < nch; i++) chcodes[i] = parameter2int(operatorArgv()[i]);
     }
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
-  int nvars = vlistNvars(vlistID1);
+  const int nvars = vlistNvars(vlistID1);
 
-  int maxrecs = vlistNrecs(vlistID1);
-  std::vector<RecordInfo> recinfo(maxrecs);
+  const int maxrecs = vlistNrecs(vlistID1);
+  std::vector<RecordInfo> recList(maxrecs);
 
   size_t **varnmiss = (size_t **) Malloc(nvars * sizeof(size_t *));
   double **vardata = (double **) Malloc(nvars * sizeof(double *));
@@ -149,7 +137,7 @@ Rotuv(void *process)
     {
       for (varID = 0; varID < nvars; varID++)
         {
-          code = vlistInqVarCode(vlistID2, varID);
+          const int code = vlistInqVarCode(vlistID2, varID);
           for (int i = 0; i < nch; i++)
             if (code == chcodes[i]) lfound[i] = true;
         }
@@ -159,41 +147,41 @@ Rotuv(void *process)
 
   for (varID = 0; varID < nvars; varID++)
     {
-      gridID = vlistInqVarGrid(vlistID1, varID);
+      const int gridID = vlistInqVarGrid(vlistID1, varID);
       if (!(gridInqType(gridID) == GRID_PROJECTION && gridInqProjType(gridID) == CDI_PROJ_RLL))
         cdoAbort("Only rotated lon/lat grids supported!");
 
-      gridsize = gridInqSize(gridID);
-      nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+      const size_t gridsize = gridInqSize(gridID);
+      const int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
       varnmiss[varID] = (size_t *) Malloc(nlevel * sizeof(size_t));
       vardata[varID] = (double *) Malloc(gridsize * nlevel * sizeof(double));
     }
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
-          recinfo[recID].varID = varID;
-          recinfo[recID].levelID = levelID;
-          recinfo[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+          recList[recID].varID = varID;
+          recList[recID].levelID = levelID;
+          recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
 
-          gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-          size_t offset = gridsize * levelID;
+          const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
+          const size_t offset = gridsize * levelID;
           single = vardata[varID] + offset;
-          pstreamReadRecord(streamID1, single, &varnmiss[varID][levelID]);
+          cdoReadRecord(streamID1, single, &varnmiss[varID][levelID]);
           if (varnmiss[varID][levelID]) cdoAbort("Missing values unsupported for this operator!");
         }
 
@@ -208,16 +196,14 @@ Rotuv(void *process)
                 }
               else
                 {
-                  code = vlistInqVarCode(vlistID2, varID);
+                  const int code = vlistInqVarCode(vlistID2, varID);
                   if (code == chcodes[i]) break;
                 }
             }
 
-          if (varID == nvars)
-            cdoAbort("u-wind not found!");
-          else
-            usvar = vardata[varID];
+          if (varID == nvars) cdoAbort("u-wind not found!");
 
+          usvar = vardata[varID];
           varID1 = varID;
 
           for (varID = 0; varID < nvars; varID++)
@@ -229,16 +215,14 @@ Rotuv(void *process)
                 }
               else
                 {
-                  code = vlistInqVarCode(vlistID2, varID);
+                  const int code = vlistInqVarCode(vlistID2, varID);
                   if (code == chcodes[i + 1]) break;
                 }
             }
 
-          if (varID == nvars)
-            cdoAbort("v-wind not found!");
-          else
-            vsvar = vardata[varID];
+          if (varID == nvars) cdoAbort("v-wind not found!");
 
+          vsvar = vardata[varID];
           varID2 = varID;
 
           if (Options::cdoVerbose)
@@ -254,10 +238,10 @@ Rotuv(void *process)
                          vlistInqVarCode(vlistID1, varID2), chcodes[i + 1]);
             }
 
-          gridID = vlistInqVarGrid(vlistID1, varID);
-          gridsize = gridInqSize(gridID);
-          nlevel1 = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID1));
-          nlevel2 = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID2));
+          const int gridID = vlistInqVarGrid(vlistID1, varID);
+          const size_t gridsize = gridInqSize(gridID);
+          const int nlevel1 = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID1));
+          const int nlevel2 = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID2));
 
           if (nlevel1 != nlevel2) cdoAbort("u-wind and v-wind have different number of levels!");
 
@@ -270,14 +254,14 @@ Rotuv(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          varID = recinfo[recID].varID;
-          levelID = recinfo[recID].levelID;
-          gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-          size_t offset = gridsize * levelID;
+          varID = recList[recID].varID;
+          levelID = recList[recID].levelID;
+          const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
+          const size_t offset = gridsize * levelID;
           single = vardata[varID] + offset;
 
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, single, varnmiss[varID][levelID]);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, single, varnmiss[varID][levelID]);
         }
 
       tsID++;
