@@ -25,7 +25,7 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
+
 #include "percentiles.h"
 #include "datetime.h"
 
@@ -43,27 +43,27 @@ Runpctl(void *process)
 
   operatorInputArg("percentile number, number of timesteps");
   operatorCheckArgc(2);
-  double pn = parameter2double(operatorArgv()[0]);
+  const double pn = parameter2double(operatorArgv()[0]);
   percentile_check_number(pn);
-  int ndates = parameter2int(operatorArgv()[1]);
+  const int ndates = parameter2int(operatorArgv()[1]);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   taxisWithBounds(taxisID2);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
-  int nvars = vlistNvars(vlistID1);
+  const int nvars = vlistNvars(vlistID1);
 
-  int maxrecs = vlistNrecs(vlistID1);
-  std::vector<RecordInfo> recinfo(maxrecs);
+  const int maxrecs = vlistNrecs(vlistID1);
+  std::vector<RecordInfo> recList(maxrecs);
 
   DateTimeList dtlist;
   dtlist.setStat(timestat_date);
@@ -72,7 +72,7 @@ Runpctl(void *process)
   FieldVector3D vars1(ndates + 1);
   std::vector<double> array(ndates);
 
-  for (int its = 0; its < ndates; its++) fieldsFromVlist(vlistID1, vars1[its], FIELD_PTR);
+  for (int its = 0; its < ndates; its++) fieldsFromVlist(vlistID1, vars1[its], FIELD_VEC);
 
   int tsID;
   for (tsID = 0; tsID < ndates; tsID++)
@@ -84,16 +84,16 @@ Runpctl(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           if (tsID == 0)
             {
-              recinfo[recID].varID = varID;
-              recinfo[recID].levelID = levelID;
-              recinfo[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+              recList[recID].varID = varID;
+              recList[recID].levelID = levelID;
+              recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
             }
 
-          pstreamReadRecord(streamID1, vars1[tsID][varID][levelID].ptr, &nmiss);
+          cdoReadRecord(streamID1, vars1[tsID][varID][levelID].vec.data(), &nmiss);
           vars1[tsID][varID][levelID].nmiss = nmiss;
         }
     }
@@ -105,48 +105,49 @@ Runpctl(void *process)
         {
           if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
 
-          size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-          int nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-          double missval = vlistInqVarMissval(vlistID1, varID);
+          const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
+          const int nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+          const double missval = vlistInqVarMissval(vlistID1, varID);
 
           for (levelID = 0; levelID < nlevels; levelID++)
             {
+              Field &rvars1_0 = vars1[0][varID][levelID];
               nmiss = 0;
               for (size_t i = 0; i < gridsize; i++)
                 {
                   int j = 0;
                   for (int inp = 0; inp < ndates; inp++)
                     {
-                      double val = vars1[inp][varID][levelID].ptr[i];
+                      const double val = vars1[inp][varID][levelID].vec[i];
                       if (!DBL_IS_EQUAL(val, missval)) array[j++] = val;
                     }
 
                   if (j > 0)
                     {
-                      vars1[0][varID][levelID].ptr[i] = percentile(array.data(), j, pn);
+                      rvars1_0.vec[i] = percentile(array.data(), j, pn);
                     }
                   else
                     {
-                      vars1[0][varID][levelID].ptr[i] = missval;
+                      rvars1_0.vec[i] = missval;
                       nmiss++;
                     }
                 }
-              vars1[0][varID][levelID].nmiss = nmiss;
+              rvars1_0.nmiss = nmiss;
             }
         }
 
       dtlist.statTaxisDefTimestep(taxisID2, ndates);
-      pstreamDefTimestep(streamID2, otsID);
+      cdoDefTimestep(streamID2, otsID);
 
       for (int recID = 0; recID < maxrecs; recID++)
         {
-          if (otsID && recinfo[recID].lconst) continue;
+          if (otsID && recList[recID].lconst) continue;
 
-          int varID = recinfo[recID].varID;
-          int levelID = recinfo[recID].levelID;
+          const int varID = recList[recID].varID;
+          const int levelID = recList[recID].levelID;
 
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, vars1[0][varID][levelID].ptr, vars1[0][varID][levelID].nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, vars1[0][varID][levelID].vec.data(), vars1[0][varID][levelID].nmiss);
         }
 
       otsID++;
@@ -156,23 +157,21 @@ Runpctl(void *process)
       vars1[ndates] = vars1[0];
       for (int inp = 0; inp < ndates; inp++) vars1[inp] = vars1[inp + 1];
 
-      int nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      const int nrecs = cdoStreamInqTimestep(streamID1, tsID);
       if (nrecs == 0) break;
 
       dtlist.taxisInqTimestep(taxisID1, ndates - 1);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, vars1[ndates - 1][varID][levelID].ptr, &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, vars1[ndates - 1][varID][levelID].vec.data(), &nmiss);
           vars1[ndates - 1][varID][levelID].nmiss = nmiss;
         }
 
       tsID++;
     }
 
-  for (int its = 0; its < ndates; its++) fieldsFree(vlistID1, vars1[its]);
-
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
