@@ -34,28 +34,12 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
+
 #include "datetime.h"
 
-void *
-Runstat(void *process)
+static void
+addOperators(void)
 {
-  TimeStat timestat_date = TimeStat::MEAN;
-  int varID;
-  int levelID;
-  size_t nmiss;
-  bool runstat_nomiss = false;
-
-  cdoInitialize(process);
-
-  char *envstr = getenv("RUNSTAT_NOMISS");
-  if (envstr)
-    {
-      char *endptr;
-      int envval = (int) strtol(envstr, &endptr, 10);
-      if (envval == 1) runstat_nomiss = true;
-    }
-
   // clang-format off
   cdoOperatorAdd("runrange", func_range, 0, nullptr);
   cdoOperatorAdd("runmin",   func_min,   0, nullptr);
@@ -67,29 +51,49 @@ Runstat(void *process)
   cdoOperatorAdd("runvar1",  func_var1,  0, nullptr);
   cdoOperatorAdd("runstd",   func_std,   0, nullptr);
   cdoOperatorAdd("runstd1",  func_std1,  0, nullptr);
+  // clang-format on
+}
 
-  int operatorID = cdoOperatorID();
+void *
+Runstat(void *process)
+{
+  TimeStat timestat_date = TimeStat::MEAN;
+  int varID, levelID;
+  bool runstat_nomiss = false;
+
+  cdoInitialize(process);
+
+  char *envstr = getenv("RUNSTAT_NOMISS");
+  if (envstr)
+    {
+      char *endptr;
+      const int envval = (int) strtol(envstr, &endptr, 10);
+      if (envval == 1) runstat_nomiss = true;
+    }
+
+  addOperators();
+
+  const int operatorID = cdoOperatorID();
   int operfunc = cdoOperatorF1(operatorID);
 
-  bool lrange  = operfunc == func_range;
-  bool lmean   = operfunc == func_mean || operfunc == func_avg;
-  bool lstd    = operfunc == func_std || operfunc == func_std1;
-  bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
-  int  divisor = operfunc == func_std1 || operfunc == func_var1;
-  // clang-format on
-  bool lvars2 = lvarstd || lrange;
+  const bool lrange = operfunc == func_range;
+  const bool lmean = operfunc == func_mean || operfunc == func_avg;
+  const bool lstd = operfunc == func_std || operfunc == func_std1;
+  const bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
+  const int  divisor = operfunc == func_std1 || operfunc == func_var1;
+  const bool lvars2 = lvarstd || lrange;
 
   operatorInputArg("number of timesteps");
   operatorCheckArgc(1);
   int ndates = parameter2int(operatorArgv()[0]);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   taxisWithBounds(taxisID2);
   vlistDefTaxis(vlistID2, taxisID2);
   /*  Number of timestep will be reduced compared to the input
@@ -101,11 +105,11 @@ Runstat(void *process)
       if (nsteps > 0) vlistDefNtsteps(vlistID2, nsteps);
     }
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
-  int maxrecs = vlistNrecs(vlistID1);
-  std::vector<RecordInfo> recinfo(maxrecs);
+  const int maxrecs = vlistNrecs(vlistID1);
+  std::vector<RecordInfo> recList(maxrecs);
 
   DateTimeList dtlist;
   dtlist.setStat(timestat_date);
@@ -117,12 +121,12 @@ Runstat(void *process)
 
   for (int its = 0; its < ndates; its++)
     {
-      fieldsFromVlist(vlistID1, vars1[its], FIELD_PTR);
-      if (!runstat_nomiss) fieldsFromVlist(vlistID1, samp1[its], FIELD_PTR);
-      if (lvars2) fieldsFromVlist(vlistID1, vars2[its], FIELD_PTR);
+      fieldsFromVlist(vlistID1, vars1[its], FIELD_VEC);
+      if (!runstat_nomiss) fieldsFromVlist(vlistID1, samp1[its], FIELD_VEC);
+      if (lvars2) fieldsFromVlist(vlistID1, vars2[its], FIELD_VEC);
     }
 
-  size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
   std::vector<bool> imask(gridsizemax);
 
   int tsID = 0;
@@ -135,58 +139,56 @@ Runstat(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           if (tsID == 0)
             {
-              recinfo[recID].varID = varID;
-              recinfo[recID].levelID = levelID;
-              recinfo[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+              recList[recID].varID = varID;
+              recList[recID].levelID = levelID;
+              recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
             }
 
           Field &rvars1 = vars1[tsID][varID][levelID];
 
-          size_t gridsize = rvars1.size;
+          size_t fieldsize = rvars1.size;
 
-          pstreamReadRecord(streamID1, rvars1.ptr, &nmiss);
-          rvars1.nmiss = nmiss;
+          cdoReadRecord(streamID1, rvars1.vec.data(), &rvars1.nmiss);
           if (lrange)
             {
               vars2[tsID][varID][levelID].nmiss = rvars1.nmiss;
-              for (size_t i = 0; i < gridsize; i++) vars2[tsID][varID][levelID].ptr[i] = rvars1.ptr[i];
+              vars2[tsID][varID][levelID].vec = rvars1.vec;
             }
 
-          if (runstat_nomiss && nmiss > 0) cdoAbort("Missing values supported was swichted off by env. RUNSTAT_NOMISS!");
+          if (runstat_nomiss && rvars1.nmiss) cdoAbort("Missing values supported was swichted off by env. RUNSTAT_NOMISS!");
 
           if (!runstat_nomiss)
             {
-              double missval = rvars1.missval;
+              const double missval = rvars1.missval;
 
-              for (size_t i = 0; i < gridsize; i++) imask[i] = !DBL_IS_EQUAL(rvars1.ptr[i], missval);
-
-              for (size_t i = 0; i < gridsize; i++) samp1[tsID][varID][levelID].ptr[i] = (double) imask[i];
+              for (size_t i = 0; i < fieldsize; i++) imask[i] = !DBL_IS_EQUAL(rvars1.vec[i], missval);
+              for (size_t i = 0; i < fieldsize; i++) samp1[tsID][varID][levelID].vec[i] = (double) imask[i];
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(tsID, gridsize, imask, samp1, varID, levelID)
+#pragma omp parallel for default(none) shared(tsID, imask, samp1, varID, levelID, fieldsize)
 #endif
               for (int inp = 0; inp < tsID; inp++)
                 {
-                  double *ptr = samp1[inp][varID][levelID].ptr;
-                  for (size_t i = 0; i < gridsize; i++)
-                    if (imask[i]) ptr[i]++;
+                  auto &samp = samp1[inp][varID][levelID].vec;
+                  for (size_t i = 0; i < fieldsize; i++)
+                    if (imask[i]) samp[i]++;
                 }
             }
 
           if (lvarstd)
             {
-              farmoq(vars2[tsID][varID][levelID], vars1[tsID][varID][levelID]);
+              vfarmoq(vars2[tsID][varID][levelID], vars1[tsID][varID][levelID]);
 #ifdef _OPENMP
 #pragma omp parallel for default(none) shared(tsID, vars1, vars2, varID, levelID, rvars1)
 #endif
               for (int inp = 0; inp < tsID; inp++)
                 {
-                  farsumq(vars2[inp][varID][levelID], rvars1);
-                  farsum(vars1[inp][varID][levelID], rvars1);
+                  vfarsumq(vars2[inp][varID][levelID], rvars1);
+                  vfarsum(vars1[inp][varID][levelID], rvars1);
                 }
             }
           else if (lrange)
@@ -196,18 +198,18 @@ Runstat(void *process)
 #endif
               for (int inp = 0; inp < tsID; inp++)
                 {
-                  farmin(vars2[inp][varID][levelID], rvars1);
-                  farmax(vars1[inp][varID][levelID], rvars1);
+                  vfarmin(vars2[inp][varID][levelID], rvars1);
+                  vfarmax(vars1[inp][varID][levelID], rvars1);
                 }
             }
           else
             {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(tsID, vars1, operfunc, varID, levelID, rvars1)
+#pragma omp parallel for default(none) shared(tsID, vars1, varID, levelID, rvars1, operfunc)
 #endif
               for (int inp = 0; inp < tsID; inp++)
                 {
-                  farfun(vars1[inp][varID][levelID], rvars1, operfunc);
+                  vfarfun(vars1[inp][varID][levelID], rvars1, operfunc);
                 }
             }
         }
@@ -218,19 +220,19 @@ Runstat(void *process)
     {
       for (int recID = 0; recID < maxrecs; recID++)
         {
-          if (recinfo[recID].lconst) continue;
+          if (recList[recID].lconst) continue;
 
-          int varID = recinfo[recID].varID;
-          int levelID = recinfo[recID].levelID;
+          const int varID = recList[recID].varID;
+          const int levelID = recList[recID].levelID;
           Field &rvars1 = vars1[0][varID][levelID];
-          int nsets = ndates;
+          const int nsets = ndates;
 
           if (lmean)
             {
               if (!runstat_nomiss)
-                fardiv(rvars1, samp1[0][varID][levelID]);
+                vfardiv(rvars1, samp1[0][varID][levelID]);
               else
-                farcdiv(rvars1, (double) nsets);
+                vfarcdiv(rvars1, (double) nsets);
             }
           else if (lvarstd)
             {
@@ -238,38 +240,38 @@ Runstat(void *process)
               if (!runstat_nomiss)
                 {
                   if (lstd)
-                    farstd(rvars1, rvars2, samp1[0][varID][levelID], divisor);
+                    vfarstd(rvars1, rvars2, samp1[0][varID][levelID], divisor);
                   else
-                    farvar(rvars1, rvars2, samp1[0][varID][levelID], divisor);
+                    vfarvar(rvars1, rvars2, samp1[0][varID][levelID], divisor);
                 }
               else
                 {
                   if (lstd)
-                    farcstd(rvars1, rvars2, nsets, divisor);
+                    vfarcstd(rvars1, rvars2, nsets, divisor);
                   else
-                    farcvar(rvars1, rvars2, nsets, divisor);
+                    vfarcvar(rvars1, rvars2, nsets, divisor);
                 }
             }
           else if (lrange)
             {
               Field &rvars2 = vars2[0][varID][levelID];
-              farsub(rvars1, rvars2);
+              vfarsub(rvars1, rvars2);
             }
         }
 
       dtlist.statTaxisDefTimestep(taxisID2, ndates);
-      pstreamDefTimestep(streamID2, otsID);
+      cdoDefTimestep(streamID2, otsID);
 
       for (int recID = 0; recID < maxrecs; recID++)
         {
-          if (otsID && recinfo[recID].lconst) continue;
+          if (otsID && recList[recID].lconst) continue;
 
-          int varID = recinfo[recID].varID;
-          int levelID = recinfo[recID].levelID;
+          const int varID = recList[recID].varID;
+          const int levelID = recList[recID].levelID;
           Field &rvars1 = vars1[0][varID][levelID];
 
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, rvars1.ptr, rvars1.nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, rvars1.vec.data(), rvars1.nmiss);
         }
 
       otsID++;
@@ -287,58 +289,56 @@ Runstat(void *process)
           if (lvars2) vars2[inp] = vars2[inp + 1];
         }
 
-      int nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      const int nrecs = cdoStreamInqTimestep(streamID1, tsID);
       if (nrecs == 0) break;
 
       dtlist.taxisInqTimestep(taxisID1, ndates - 1);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           Field &rvars1 = vars1[ndates - 1][varID][levelID];
 
-          size_t gridsize = rvars1.size;
+          size_t fieldsize = rvars1.size;
 
-          pstreamReadRecord(streamID1, rvars1.ptr, &nmiss);
-          rvars1.nmiss = nmiss;
+          cdoReadRecord(streamID1, rvars1.vec.data(), &rvars1.nmiss);
           if (lrange)
             {
               vars2[ndates - 1][varID][levelID].nmiss = rvars1.nmiss;
-              for (size_t i = 0; i < gridsize; i++) vars2[ndates - 1][varID][levelID].ptr[i] = rvars1.ptr[i];
+              for (size_t i = 0; i < fieldsize; i++) vars2[ndates - 1][varID][levelID].vec[i] = rvars1.vec[i];
             }
 
-          if (runstat_nomiss && nmiss > 0) cdoAbort("Missing values supported swichted off!");
+          if (runstat_nomiss && rvars1.nmiss) cdoAbort("Missing values supported swichted off!");
 
           if (!runstat_nomiss)
             {
-              double missval = rvars1.missval;
+              const double missval = rvars1.missval;
 
-              for (size_t i = 0; i < gridsize; i++) imask[i] = !DBL_IS_EQUAL(rvars1.ptr[i], missval);
-
-              for (size_t i = 0; i < gridsize; i++) samp1[ndates - 1][varID][levelID].ptr[i] = (double) imask[i];
+              for (size_t i = 0; i < fieldsize; i++) imask[i] = !DBL_IS_EQUAL(rvars1.vec[i], missval);
+              for (size_t i = 0; i < fieldsize; i++) samp1[ndates - 1][varID][levelID].vec[i] = (double) imask[i];
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(ndates, imask, gridsize, samp1, varID, levelID)
+#pragma omp parallel for default(none) shared(ndates, fieldsize, imask, samp1, varID, levelID)
 #endif
               for (int inp = 0; inp < ndates - 1; inp++)
                 {
-                  double *ptr = samp1[inp][varID][levelID].ptr;
-                  for (size_t i = 0; i < gridsize; i++)
-                    if (imask[i]) ptr[i]++;
+                  auto &samp = samp1[inp][varID][levelID].vec;
+                  for (size_t i = 0; i < fieldsize; i++)
+                    if (imask[i]) samp[i]++;
                 }
             }
 
           if (lvarstd)
             {
-              farmoq(vars2[ndates - 1][varID][levelID], vars1[ndates - 1][varID][levelID]);
+              vfarmoq(vars2[ndates - 1][varID][levelID], vars1[ndates - 1][varID][levelID]);
 #ifdef _OPENMP
 #pragma omp parallel for default(none) shared(ndates, vars1, vars2, varID, levelID, rvars1)
 #endif
               for (int inp = 0; inp < ndates - 1; inp++)
                 {
-                  farsumq(vars2[inp][varID][levelID], rvars1);
-                  farsum(vars1[inp][varID][levelID], rvars1);
+                  vfarsumq(vars2[inp][varID][levelID], rvars1);
+                  vfarsum(vars1[inp][varID][levelID], rvars1);
                 }
             }
           else if (lrange)
@@ -348,18 +348,18 @@ Runstat(void *process)
 #endif
               for (int inp = 0; inp < ndates - 1; inp++)
                 {
-                  farmin(vars2[inp][varID][levelID], rvars1);
-                  farmax(vars1[inp][varID][levelID], rvars1);
+                  vfarmin(vars2[inp][varID][levelID], rvars1);
+                  vfarmax(vars1[inp][varID][levelID], rvars1);
                 }
             }
           else
             {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(ndates, vars1, varID, levelID, operfunc, rvars1)
+#pragma omp parallel for default(none) shared(ndates, vars1, varID, levelID, rvars1, operfunc)
 #endif
               for (int inp = 0; inp < ndates - 1; inp++)
                 {
-                  farfun(vars1[inp][varID][levelID], rvars1, operfunc);
+                  vfarfun(vars1[inp][varID][levelID], rvars1, operfunc);
                 }
             }
         }
@@ -367,13 +367,6 @@ Runstat(void *process)
       tsID++;
     }
 
-  for (int its = 0; its < ndates; its++)
-    {
-      fieldsFree(vlistID1, vars1[its]);
-      if (!runstat_nomiss) fieldsFree(vlistID1, samp1[its]);
-      if (lvars2) fieldsFree(vlistID1, vars2[its]);
-    }
-
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
