@@ -17,8 +17,9 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
-#include "grid.h"
+#include "cdi_lockedIO.h"
+
+#include <mpim_grid.h>
 #include "grid_point_search.h"
 #include "verifygrid.h"
 
@@ -246,7 +247,7 @@ compute_child_from_parent(CellIndex *cellindex1, CellIndex *cellindex2)
 static void
 read_coordinates(const char *filename, long n, double *lon, double *lat, int nv, double *lon_bnds, double *lat_bnds)
 {
-  int streamID = streamOpenRead(filename);
+  CdiStreamID streamID = streamOpenRead(filename);
   int vlistID = streamInqVlist(streamID);
   int ngrids = vlistNgrids(vlistID);
   int gridID = -1;
@@ -313,8 +314,8 @@ compute_child_from_bounds(CellIndex *cellindex2, long ncells2, double *grid_cent
       for (int corner_no = 0; corner_no < ncorner; corner_no++)
         {
           // Conversion of corner spherical coordinates to Cartesian coordinates.
-          cdoLLtoXYZ(grid_corner_lon2[cell_no2 * ncorner + corner_no], grid_corner_lat2[cell_no2 * ncorner + corner_no],
-                     corner_coordinates);
+          gcLLtoXYZ(grid_corner_lon2[cell_no2 * ncorner + corner_no], grid_corner_lat2[cell_no2 * ncorner + corner_no],
+                    corner_coordinates);
 
           // The components of the result vector are appended to the list of cell corner coordinates.
           int off = corner_no * 3;
@@ -370,7 +371,7 @@ compute_child_from_bounds(CellIndex *cellindex2, long ncells2, double *grid_cent
           size_t cell_no1 = nbr_addr[i];
           if (cell_no1 < SIZE_MAX)
             {
-              cdoLLtoXYZ(grid_center_lon1[cell_no1], grid_center_lat1[cell_no1], center_point_xyz);
+              gcLLtoXYZ(grid_center_lon1[cell_no1], grid_center_lat1[cell_no1], center_point_xyz);
 
               switch (coordinate_to_ignore)
                 {
@@ -412,24 +413,14 @@ compute_child_from_coordinates(CellIndex *cellindex1, CellIndex *cellindex2)
   const long ncells1 = cellindex1->ncells;
   const long ncells2 = cellindex2->ncells;
 
-  double *lon1 = (double *) Malloc(ncells1 * sizeof(double));
-  double *lat1 = (double *) Malloc(ncells1 * sizeof(double));
-  double *lon2 = (double *) Malloc(ncells2 * sizeof(double));
-  double *lat2 = (double *) Malloc(ncells2 * sizeof(double));
-  double *lon2_bnds = (double *) Malloc(3 * ncells2 * sizeof(double));
-  double *lat2_bnds = (double *) Malloc(3 * ncells2 * sizeof(double));
+  std::vector<double> lon1(ncells1), lat1(ncells1), lon2(ncells2), lat2(ncells2);
+  std::vector<double> lon2_bnds(3 * ncells2), lat2_bnds(3 * ncells2);
 
-  read_coordinates(cellindex1->filename, ncells1, lon1, lat1, 0, nullptr, nullptr);
-  read_coordinates(cellindex2->filename, ncells2, lon2, lat2, 3, lon2_bnds, lat2_bnds);
+  read_coordinates(cellindex1->filename, ncells1, lon1.data(), lat1.data(), 0, nullptr, nullptr);
+  read_coordinates(cellindex2->filename, ncells2, lon2.data(), lat2.data(), 3, lon2_bnds.data(), lat2_bnds.data());
 
-  compute_child_from_bounds(cellindex2, ncells2, lon2, lat2, lon2_bnds, lat2_bnds, ncells1, lon1, lat1);
-
-  Free(lon1);
-  Free(lat1);
-  Free(lon2);
-  Free(lat2);
-  Free(lon2_bnds);
-  Free(lat2_bnds);
+  compute_child_from_bounds(cellindex2, ncells2, lon2.data(), lat2.data(), lon2_bnds.data(), lat2_bnds.data(), ncells1, lon1.data(),
+                            lat1.data());
 }
 
 static void
@@ -510,8 +501,6 @@ samplegrid(double missval, long nci, CellIndex **cellindex, double *array1, doub
     }
 }
 
-#include "pstream_int.h"
-
 void *
 Samplegridicon(void *process)
 {
@@ -538,7 +527,7 @@ Samplegridicon(void *process)
 
   const int gridID2 = read_grid(operatorArgv()[nsamplegrids - 1]);
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
   const int vlistID1 = cdoStreamInqVlist(streamID1);
 
   long gridsize = vlistGridsizeMax(vlistID1);
@@ -570,11 +559,11 @@ Samplegridicon(void *process)
       vlistChangeGridIndex(vlistID3, index, gridID2);
     }
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
-  const int streamID3 = cdoStreamOpenWrite(cdoStreamName(2));
-  pstreamDefVlist(streamID3, vlistID3);
+  CdoStreamID streamID3 = cdoOpenWrite(2);
+  cdoDefVlist(streamID3, vlistID3);
 
   long gridsize2 = gridInqSize(gridID2);
   if (Options::cdoVerbose) cdoPrint("Target gridsize = %ld", gridsize2);
@@ -587,26 +576,26 @@ Samplegridicon(void *process)
     {
       taxisCopyTimestep(taxisID2, taxisID1);
       taxisCopyTimestep(taxisID3, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
-      pstreamDefTimestep(streamID3, tsID);
+      cdoDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID3, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
           size_t nmiss;
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, array1, &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, array1, &nmiss);
 
           const double missval = vlistInqVarMissval(vlistID1, varID);
 
           samplegrid(missval, nsamplegrids, &cellindex[0], array1, array2, array3);
 
           nmiss = arrayNumMV(gridsize2, array2, missval);
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, array2, nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, array2, nmiss);
 
           nmiss = arrayNumMV(gridsize2, array3, missval);
-          pstreamDefRecord(streamID3, varID, levelID);
-          pstreamWriteRecord(streamID3, array3, nmiss);
+          cdoDefRecord(streamID3, varID, levelID);
+          cdoWriteRecord(streamID3, array3, nmiss);
         }
 
       tsID++;
