@@ -25,7 +25,7 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
+
 #include "percentiles_hist.h"
 #include "percentiles.h"
 #include "datetime.h"
@@ -42,7 +42,6 @@ Seaspctl(void *process)
   int nlevels;
   int oldmon = 0;
   int season_start;
-  double missval;
 
   cdoInitialize(process);
 
@@ -54,9 +53,9 @@ Seaspctl(void *process)
 
   season_start = get_season_start();
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-  const int streamID2 = cdoStreamOpenRead(cdoStreamName(1));
-  const int streamID3 = cdoStreamOpenRead(cdoStreamName(2));
+  CdoStreamID streamID1 = cdoOpenRead(0);
+  CdoStreamID streamID2 = cdoOpenRead(1);
+  CdoStreamID streamID3 = cdoOpenRead(2);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = cdoStreamInqVlist(streamID2);
@@ -75,44 +74,33 @@ Seaspctl(void *process)
   taxisWithBounds(taxisID4);
   vlistDefTaxis(vlistID4, taxisID4);
 
-  const int streamID4 = cdoStreamOpenWrite(cdoStreamName(3));
-  pstreamDefVlist(streamID4, vlistID4);
+  CdoStreamID streamID4 = cdoOpenWrite(3);
+  cdoDefVlist(streamID4, vlistID4);
 
   const int nvars = vlistNvars(vlistID1);
 
   const int maxrecs = vlistNrecs(vlistID1);
-  std::vector<RecordInfo> recinfo(maxrecs);
+  std::vector<RecordInfo> recList(maxrecs);
 
   DateTimeList dtlist;
   dtlist.setStat(timestat_date);
   dtlist.setCalendar(taxisInqCalendar(taxisID1));
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
+  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field;
-  fieldInit(field);
-  field.ptr = (double *) Malloc(gridsize * sizeof(double));
+  field.resize(gridsizemax);
 
-  Field **vars1 = (Field **) Malloc(nvars * sizeof(Field *));
+  FieldVector2D vars1;
+  fieldsFromVlist(vlistID1, vars1, FIELD_VEC);
   HISTOGRAM_SET *hset = hsetCreate(nvars);
 
   for (varID = 0; varID < nvars; varID++)
     {
       gridID = vlistInqVarGrid(vlistID1, varID);
-      gridsize = gridInqSize(gridID);
       nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-      missval = vlistInqVarMissval(vlistID1, varID);
 
-      vars1[varID] = (Field *) Malloc(nlevels * sizeof(Field));
       hsetCreateVarLevels(hset, varID, nlevels, gridID);
-
-      for (levelID = 0; levelID < nlevels; levelID++)
-        {
-          vars1[varID][levelID].grid = gridID;
-          vars1[varID][levelID].nmiss = 0;
-          vars1[varID][levelID].missval = missval;
-          vars1[varID][levelID].ptr = (double *) Malloc(gridsize * sizeof(double));
-        }
     }
 
   int tsID = 0;
@@ -121,29 +109,26 @@ Seaspctl(void *process)
     {
       nrecs = cdoStreamInqTimestep(streamID2, otsID);
       if (nrecs != cdoStreamInqTimestep(streamID3, otsID))
-        cdoAbort("Number of records at time step %d of %s and %s differ!", otsID + 1, cdoGetStreamName(1),
-                 cdoGetStreamName(2));
+        cdoAbort("Number of records at time step %d of %s and %s differ!", otsID + 1, cdoGetStreamName(1), cdoGetStreamName(2));
 
       const int64_t vdate2 = taxisInqVdate(taxisID2);
       const int vtime2 = taxisInqVtime(taxisID2);
       const int64_t vdate3 = taxisInqVdate(taxisID3);
       const int vtime3 = taxisInqVtime(taxisID3);
       if (vdate2 != vdate3 || vtime2 != vtime3)
-        cdoAbort("Verification dates at time step %d of %s and %s differ!", otsID + 1, cdoGetStreamName(1),
-                 cdoGetStreamName(2));
+        cdoAbort("Verification dates at time step %d of %s and %s differ!", otsID + 1, cdoGetStreamName(1), cdoGetStreamName(2));
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID2, &varID, &levelID);
-          pstreamReadRecord(streamID2, vars1[varID][levelID].ptr, &nmiss);
+          cdoInqRecord(streamID2, &varID, &levelID);
+          cdoReadRecord(streamID2, vars1[varID][levelID].vec.data(), &nmiss);
           vars1[varID][levelID].nmiss = nmiss;
         }
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID3, &varID, &levelID);
-          pstreamReadRecord(streamID3, field.ptr, &nmiss);
-          field.nmiss = nmiss;
+          cdoInqRecord(streamID3, &varID, &levelID);
+          cdoReadRecord(streamID3, field.vec.data(), &field.nmiss);
           field.grid = vars1[varID][levelID].grid;
           field.missval = vars1[varID][levelID].missval;
 
@@ -178,14 +163,14 @@ Seaspctl(void *process)
 
           for (int recID = 0; recID < nrecs; recID++)
             {
-              pstreamInqRecord(streamID1, &varID, &levelID);
+              cdoInqRecord(streamID1, &varID, &levelID);
               if (tsID == 0)
                 {
-                  recinfo[recID].varID = varID;
-                  recinfo[recID].levelID = levelID;
-                  recinfo[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+                  recList[recID].varID = varID;
+                  recList[recID].levelID = levelID;
+                  recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
                 }
-              pstreamReadRecord(streamID1, vars1[varID][levelID].ptr, &nmiss);
+              cdoReadRecord(streamID1, vars1[varID][levelID].vec.data(), &nmiss);
               vars1[varID][levelID].nmiss = nmiss;
 
               hsetAddVarLevelValues(hset, varID, levelID, &vars1[varID][levelID]);
@@ -207,34 +192,24 @@ Seaspctl(void *process)
         }
 
       dtlist.statTaxisDefTimestep(taxisID4, nsets);
-      pstreamDefTimestep(streamID4, otsID);
+      cdoDefTimestep(streamID4, otsID);
 
       for (int recID = 0; recID < maxrecs; recID++)
         {
-          if (otsID && recinfo[recID].lconst) continue;
+          if (otsID && recList[recID].lconst) continue;
 
-          const int varID = recinfo[recID].varID;
-          const int levelID = recinfo[recID].levelID;
-          pstreamDefRecord(streamID4, varID, levelID);
-          pstreamWriteRecord(streamID4, vars1[varID][levelID].ptr, vars1[varID][levelID].nmiss);
+          const int varID = recList[recID].varID;
+          const int levelID = recList[recID].levelID;
+          cdoDefRecord(streamID4, varID, levelID);
+          cdoWriteRecord(streamID4, vars1[varID][levelID].vec.data(), vars1[varID][levelID].nmiss);
         }
 
       if (nrecs == 0) break;
       otsID++;
     }
 
-  for (varID = 0; varID < nvars; varID++)
-    {
-      nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-      for (levelID = 0; levelID < nlevels; levelID++) Free(vars1[varID][levelID].ptr);
-      Free(vars1[varID]);
-    }
-
-  Free(vars1);
   hsetDestroy(hset);
 
-  if (field.ptr) Free(field.ptr);
-
   cdoStreamClose(streamID4);
   cdoStreamClose(streamID3);
   cdoStreamClose(streamID2);
