@@ -33,7 +33,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 #include "datetime.h"
 #include "printinfo.h"
 #include "cdo_season.h"
@@ -47,7 +46,6 @@ Seasstat(void *process)
   int nrecs;
   int varID, levelID;
   int year, month, day, seas0 = 0;
-  size_t nmiss;
   int oldmon = 0;
   int nseason = 0;
   const char *seas_name[4];
@@ -80,7 +78,7 @@ Seasstat(void *process)
   const int season_start = get_season_start();
   get_season_name(seas_name);
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -91,11 +89,11 @@ Seasstat(void *process)
   if (taxisInqType(taxisID2) == TAXIS_FORECAST) taxisDefType(taxisID2, TAXIS_RELATIVE);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   const int maxrecs = vlistNrecs(vlistID1);
-  std::vector<RecordInfo> recinfo(maxrecs);
+  std::vector<RecordInfo> recList(maxrecs);
 
   DateTimeList dtlist;
   dtlist.setStat(timestat_date);
@@ -104,13 +102,12 @@ Seasstat(void *process)
   const size_t gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field;
-  fieldInit(field);
-  field.ptr = (double *) Malloc(gridsizemax * sizeof(double));
+  field.resize(gridsizemax);
 
   FieldVector2D samp1, vars1, vars2;
   fieldsFromVlist(vlistID1, samp1, FIELD_NONE);
-  fieldsFromVlist(vlistID1, vars1, FIELD_PTR);
-  if (lvars2) fieldsFromVlist(vlistID1, vars2, FIELD_PTR);
+  fieldsFromVlist(vlistID1, vars1, FIELD_VEC);
+  if (lvars2) fieldsFromVlist(vlistID1, vars2, FIELD_VEC);
 
   int tsID = 0;
   int otsID = 0;
@@ -149,69 +146,65 @@ Seasstat(void *process)
 
           for (int recID = 0; recID < nrecs; recID++)
             {
-              pstreamInqRecord(streamID1, &varID, &levelID);
+              cdoInqRecord(streamID1, &varID, &levelID);
 
               if (tsID == 0)
                 {
-                  recinfo[recID].varID = varID;
-                  recinfo[recID].levelID = levelID;
-                  recinfo[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+                  recList[recID].varID = varID;
+                  recList[recID].levelID = levelID;
+                  recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
                 }
 
               Field &rsamp1 = samp1[varID][levelID];
               Field &rvars1 = vars1[varID][levelID];
 
-              const size_t gridsize = rvars1.size;
+              const size_t fieldsize = rvars1.size;
 
               if (nsets == 0)
                 {
-                  pstreamReadRecord(streamID1, rvars1.ptr, &nmiss);
-                  rvars1.nmiss = nmiss;
+                  cdoReadRecord(streamID1, rvars1.vec.data(), &rvars1.nmiss);
                   if (lrange)
                     {
                       vars2[varID][levelID].nmiss = rvars1.nmiss;
-                      for (size_t i = 0; i < gridsize; i++) vars2[varID][levelID].ptr[i] = rvars1.ptr[i];
+                      vars2[varID][levelID].vec = rvars1.vec;
                     }
 
-                  if (nmiss > 0 || rsamp1.ptr)
+                  if (rvars1.nmiss || !rsamp1.empty())
                     {
-                      if (rsamp1.ptr == nullptr) rsamp1.ptr = (double *) Malloc(gridsize * sizeof(double));
+                      if (rsamp1.empty()) rsamp1.resize(fieldsize);
 
-                      for (size_t i = 0; i < gridsize; i++) rsamp1.ptr[i] = !DBL_IS_EQUAL(rvars1.ptr[i], rvars1.missval);
+                      for (size_t i = 0; i < fieldsize; i++)
+                        rsamp1.vec[i] = !DBL_IS_EQUAL(rvars1.vec[i], rvars1.missval);
                     }
                 }
               else
                 {
-                  pstreamReadRecord(streamID1, field.ptr, &nmiss);
-                  field.nmiss = nmiss;
+                  cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
+                  field.size = fieldsize;
                   field.grid = rvars1.grid;
                   field.missval = rvars1.missval;
 
-                  if (field.nmiss > 0 || rsamp1.ptr)
+                  if (field.nmiss || !rsamp1.empty())
                     {
-                      if (rsamp1.ptr == nullptr)
-                        {
-                          rsamp1.ptr = (double *) Malloc(gridsize * sizeof(double));
-                          for (size_t i = 0; i < gridsize; i++) rsamp1.ptr[i] = nsets;
-                        }
+                      if (rsamp1.empty()) rsamp1.resize(fieldsize, nsets);
 
-                      for (size_t i = 0; i < gridsize; i++)
-                        if (!DBL_IS_EQUAL(field.ptr[i], rvars1.missval)) rsamp1.ptr[i]++;
+                      for (size_t i = 0; i < fieldsize; i++)
+                        if (!DBL_IS_EQUAL(field.vec[i], rvars1.missval)) rsamp1.vec[i]++;
                     }
 
                   if (lvarstd)
                     {
-                      farsumq(vars2[varID][levelID], field);
-                      farsum(rvars1, field);
+                      vfarsumq(vars2[varID][levelID], field);
+                      vfarsum(rvars1, field);
                     }
                   else if (lrange)
                     {
-                      farmin(vars2[varID][levelID], field);
-                      farmax(rvars1, field);
+                      vfarmin(vars2[varID][levelID], field);
+                      vfarmax(rvars1, field);
                     }
                   else
                     {
-                      farfun(rvars1, field, operfunc);
+                      vfarfun(rvars1, field, operfunc);
                     }
                 }
             }
@@ -219,12 +212,11 @@ Seasstat(void *process)
           if (nsets == 0 && lvarstd)
             for (int recID = 0; recID < maxrecs; recID++)
               {
-                if (recinfo[recID].lconst) continue;
+                if (recList[recID].lconst) continue;
 
-                const int varID = recinfo[recID].varID;
-                const int levelID = recinfo[recID].levelID;
-
-                farmoq(vars2[varID][levelID], vars1[varID][levelID]);
+                const int varID = recList[recID].varID;
+                const int levelID = recList[recID].levelID;
+                vfarmoq(vars2[varID][levelID], vars1[varID][levelID]);
               }
 
           vdate1 = vdate;
@@ -237,42 +229,42 @@ Seasstat(void *process)
 
       for (int recID = 0; recID < maxrecs; recID++)
         {
-          if (recinfo[recID].lconst) continue;
+          if (recList[recID].lconst) continue;
 
-          const int varID = recinfo[recID].varID;
-          const int levelID = recinfo[recID].levelID;
+          const int varID = recList[recID].varID;
+          const int levelID = recList[recID].levelID;
           Field &rsamp1 = samp1[varID][levelID];
           Field &rvars1 = vars1[varID][levelID];
 
           if (lmean)
             {
-              if (rsamp1.ptr)
-                fardiv(rvars1, rsamp1);
+              if (!rsamp1.empty())
+                vfardiv(rvars1, rsamp1);
               else
-                farcdiv(rvars1, (double) nsets);
+                vfarcdiv(rvars1, (double) nsets);
             }
           else if (lvarstd)
             {
               Field &rvars2 = vars2[varID][levelID];
-              if (rsamp1.ptr)
+              if (!rsamp1.empty())
                 {
                   if (lstd)
-                    farstd(rvars1, rvars2, rsamp1, divisor);
+                    vfarstd(rvars1, rvars2, rsamp1, divisor);
                   else
-                    farvar(rvars1, rvars2, rsamp1, divisor);
+                    vfarvar(rvars1, rvars2, rsamp1, divisor);
                 }
               else
                 {
                   if (lstd)
-                    farcstd(rvars1, rvars2, nsets, divisor);
+                    vfarcstd(rvars1, rvars2, nsets, divisor);
                   else
-                    farcvar(rvars1, rvars2, nsets, divisor);
+                    vfarcvar(rvars1, rvars2, nsets, divisor);
                 }
             }
           else if (lrange)
             {
               Field &rvars2 = vars2[varID][levelID];
-              farsub(rvars1, rvars2);
+              vfarsub(rvars1, rvars2);
             }
         }
 
@@ -289,7 +281,7 @@ Seasstat(void *process)
         }
 
       dtlist.statTaxisDefTimestep(taxisID2, nsets);
-      pstreamDefTimestep(streamID2, otsID);
+      cdoDefTimestep(streamID2, otsID);
 
       if (nsets < 3)
         {
@@ -300,26 +292,20 @@ Seasstat(void *process)
 
       for (int recID = 0; recID < maxrecs; recID++)
         {
-          if (otsID && recinfo[recID].lconst) continue;
+          if (otsID && recList[recID].lconst) continue;
 
-          const int varID = recinfo[recID].varID;
-          const int levelID = recinfo[recID].levelID;
+          const int varID = recList[recID].varID;
+          const int levelID = recList[recID].levelID;
           Field &rvars1 = vars1[varID][levelID];
 
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, rvars1.ptr, rvars1.nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, rvars1.vec.data(), rvars1.nmiss);
         }
 
       if (nrecs == 0) break;
       otsID++;
     }
 
-  fieldsFree(vlistID1, samp1);
-  fieldsFree(vlistID1, vars1);
-  if (lvars2) fieldsFree(vlistID1, vars2);
-
-  if (field.ptr) Free(field.ptr);
-
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
