@@ -23,12 +23,13 @@
 */
 
 #include <cdi.h>
+#include <mpim_grid.h>
+
+#include "grid_define.h"
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "grid.h"
-#include "pstream_int.h"
-
+#include "gridreference.h"
 
 static void
 correct_xvals(const long nlon, const long inc, double *xvals)
@@ -51,7 +52,9 @@ correct_xvals(const long nlon, const long inc, double *xvals)
 }
 
 static void
-gengridxyvals(const int gridtype, const int gridID1, const int gridID2, const long nlon, const long nlat, const long nlon2, const long nlat2, const long lat1, const long lat2, const long lon11, const long lon12, const long lon21, const long lon22, const char *xunits)
+gengridxyvals(const int gridtype, const int gridID1, const int gridID2, const long nlon, const long nlat, const long nlon2,
+              const long nlat2, const long lat1, const long lat2, const long lon11, const long lon12, const long lon21,
+              const long lon22, const char *xunits)
 {
   std::vector<double> xvals1, yvals1, xvals2, yvals2;
 
@@ -323,15 +326,15 @@ gengridcell(int gridID1, size_t gridsize2, long *cellidx)
 }
 
 void
-genlonlatboxReg2d(int gridID, double xlon1, double xlon2, double xlat1, double xlat2, long *lat1, long *lat2, long *lon11,
-                  long *lon12, long *lon21, long *lon22)
+genlonlatboxReg2d(int gridID, double xlon1, double xlon2, double xlat1, double xlat2, long &lat1, long &lat2, long &lon11,
+                  long &lon12, long &lon21, long &lon22)
 {
-  *lat1 = 0;
-  *lat2 = 0;
-  *lon11 = 1;
-  *lon12 = 0;
-  *lon21 = 0;
-  *lon22 = 0;
+  lat1 = 0;
+  lat2 = 0;
+  lon11 = 1;
+  lon12 = 0;
+  lon21 = 0;
+  lon22 = 0;
 
   if (IS_NOT_EQUAL(xlon1, xlon2))
     {
@@ -356,7 +359,7 @@ genlonlatboxReg2d(int gridID, double xlon1, double xlon2, double xlat1, double x
       gridInqXvals(gridID, xvals.data());
       gridInqXunits(gridID, xunits);
       if (strncmp(xunits, "radian", 6) == 0) xfact = RAD2DEG;
-      for (long ilat = 0; ilat < nlat; ilat++) yvals[ilat] *= yfact;
+      for (long ilon = 0; ilon < nlon; ilon++) xvals[ilon] *= xfact;
 
       xlon2 -= 360 * floor((xlon1 - xvals[0]) / 360);
       xlon1 -= 360 * floor((xlon1 - xvals[0]) / 360);
@@ -368,33 +371,33 @@ genlonlatboxReg2d(int gridID, double xlon1, double xlon2, double xlat1, double x
       gridInqYvals(gridID, yvals.data());
       gridInqYunits(gridID, yunits);
       if (strncmp(yunits, "radian", 6) == 0) yfact = RAD2DEG;
-      for (long ilon = 0; ilon < nlon; ilon++) xvals[ilon] *= xfact;
+      for (long ilat = 0; ilat < nlat; ilat++) yvals[ilat] *= yfact;
     }
 
   if (nlon > 0)
     {
       // while ( nlon == 1 || (xvals[nlon-1] - xvals[0]) >= 360 ) nlon--;
 
-      for (*lon21 = 0; *lon21 < nlon && xvals[*lon21] < xlon1; (*lon21)++)
+      for (lon21 = 0; lon21 < nlon && xvals[lon21] < xlon1; lon21++)
         ;
-      for (*lon22 = *lon21; *lon22 < nlon && xvals[*lon22] < xlon2; (*lon22)++)
+      for (lon22 = lon21; lon22 < nlon && xvals[lon22] < xlon2; lon22++)
         ;
 
-      if (*lon22 >= nlon || xvals[*lon22] > xlon2) (*lon22)--;
+      if (lon22 >= nlon || xvals[lon22] > xlon2) lon22--;
 
       xlon1 -= 360;
       xlon2 -= 360;
 
-      for (*lon11 = 0; xvals[*lon11] < xlon1; (*lon11)++)
+      for (lon11 = 0; xvals[lon11] < xlon1; lon11++)
         ;
-      for (*lon12 = *lon11; *lon12 < nlon && xvals[*lon12] < xlon2; (*lon12)++)
+      for (lon12 = lon11; lon12 < nlon && xvals[lon12] < xlon2; lon12++)
         ;
 
-      // (*lon12)--;
-      if (*lon12 >= nlon || xvals[*lon12] > xlon2) (*lon12)--;
-      if (*lon12 >= 0 && IS_EQUAL(xvals[*lon12], xvals[*lon21])) (*lon12)--;
+      // lon12--;
+      if (lon12 >= nlon || xvals[lon12] > xlon2) lon12--;
+      if (lon12 >= 0 && IS_EQUAL(xvals[lon12], xvals[lon21])) lon12--;
 
-      if (*lon12 - *lon11 + 1 + *lon22 - *lon21 + 1 <= 0) cdoAbort("Longitudinal dimension is too small!");
+      if (lon12 - lon11 + 1 + lon22 - lon21 + 1 <= 0) cdoAbort("Longitudinal dimension is too small!");
     }
 
   if (nlat > 0)
@@ -403,16 +406,16 @@ genlonlatboxReg2d(int gridID, double xlon1, double xlon2, double xlat1, double x
         {
           if (xlat1 > xlat2)
             {
-              for (*lat1 = 0; *lat1 < nlat && yvals[*lat1] > xlat1; (*lat1)++)
+              for (lat1 = 0; lat1 < nlat && yvals[lat1] > xlat1; lat1++)
                 ;
-              for (*lat2 = nlat - 1; *lat2 && yvals[*lat2] < xlat2; (*lat2)--)
+              for (lat2 = nlat - 1; lat2 && yvals[lat2] < xlat2; lat2--)
                 ;
             }
           else
             {
-              for (*lat1 = 0; *lat1 < nlat && yvals[*lat1] > xlat2; (*lat1)++)
+              for (lat1 = 0; lat1 < nlat && yvals[lat1] > xlat2; lat1++)
                 ;
-              for (*lat2 = nlat - 1; *lat2 && yvals[*lat2] < xlat1; (*lat2)--)
+              for (lat2 = nlat - 1; lat2 && yvals[lat2] < xlat1; lat2--)
                 ;
             }
         }
@@ -420,27 +423,27 @@ genlonlatboxReg2d(int gridID, double xlon1, double xlon2, double xlat1, double x
         {
           if (xlat1 < xlat2)
             {
-              for (*lat1 = 0; *lat1 < nlat && yvals[*lat1] < xlat1; (*lat1)++)
+              for (lat1 = 0; lat1 < nlat && yvals[lat1] < xlat1; lat1++)
                 ;
-              for (*lat2 = nlat - 1; *lat2 && yvals[*lat2] > xlat2; (*lat2)--)
+              for (lat2 = nlat - 1; lat2 && yvals[lat2] > xlat2; lat2--)
                 ;
             }
           else
             {
-              for (*lat1 = 0; *lat1 < nlat && yvals[*lat1] < xlat2; (*lat1)++)
+              for (lat1 = 0; lat1 < nlat && yvals[lat1] < xlat2; lat1++)
                 ;
-              for (*lat2 = nlat - 1; *lat2 && yvals[*lat2] > xlat1; (*lat2)--)
+              for (lat2 = nlat - 1; lat2 && yvals[lat2] > xlat1; lat2--)
                 ;
             }
         }
 
-      if (*lat2 - *lat1 + 1 <= 0) cdoAbort("Latitudinal dimension is too small!");
+      if (lat2 - lat1 + 1 <= 0) cdoAbort("Latitudinal dimension is too small!");
     }
 }
 
 static void
-genlonlatboxCurv(const int gridID, double xlon1, double xlon2, double xlat1, double xlat2, long *lat1, long *lat2, long *lon11,
-                 long *lon12, long *lon21, long *lon22)
+genlonlatboxCurv(const int gridID, double xlon1, double xlon2, double xlat1, double xlat2, long &lat1, long &lat2, long &lon11,
+                 long &lon12, long &lon21, long &lon22)
 {
   const long nlon = gridInqXsize(gridID);
   const long nlat = gridInqYsize(gridID);
@@ -477,16 +480,14 @@ genlonlatboxCurv(const int gridID, double xlon1, double xlon2, double xlat1, dou
       xlat2 = xtemp;
     }
 
-  *lat1 = nlat - 1;
-  *lat2 = 0;
-  *lon11 = 0;
-  *lon12 = -1;
-  /*
-   *lon11 = nlon-1;
-   *lon12 = 0;
-   */
-  *lon21 = nlon - 1;
-  *lon22 = 0;
+  lat1 = nlat - 1;
+  lat2 = 0;
+  lon11 = 0;
+  lon12 = -1;
+  // lon11 = nlon-1;
+  // lon12 = 0;
+  lon21 = nlon - 1;
+  lon22 = 0;
 
   bool lp2 = false;
   double xfirst, xlast, ylast;
@@ -500,8 +501,8 @@ genlonlatboxCurv(const int gridID, double xlon1, double xlon2, double xlat1, dou
           if (ylast >= xlat1 && ylast <= xlat2)
             if (xlon1 <= xlast && xlon2 > xlast && (xlon2 - xlon1) < 360)
               {
-                *lon11 = nlon - 1;
-                *lon12 = 0;
+                lon11 = nlon - 1;
+                lon12 = 0;
                 lp2 = true;
                 break;
               }
@@ -525,17 +526,17 @@ genlonlatboxCurv(const int gridID, double xlon1, double xlon2, double xlat1, dou
 
                 if (xval >= xlon1 && xval <= xlast)
                   {
-                    if (ilon < *lon21) *lon21 = ilon;
-                    if (ilon > *lon22) *lon22 = ilon;
-                    if (ilat < *lat1) *lat1 = ilat;
-                    if (ilat > *lat2) *lat2 = ilat;
+                    if (ilon < lon21) lon21 = ilon;
+                    if (ilon > lon22) lon22 = ilon;
+                    if (ilat < lat1) lat1 = ilat;
+                    if (ilat > lat2) lat2 = ilat;
                   }
                 else if (xval >= xfirst && xval <= xlon2)
                   {
-                    if (ilon < *lon11) *lon11 = ilon;
-                    if (ilon > *lon12) *lon12 = ilon;
-                    if (ilat < *lat1) *lat1 = ilat;
-                    if (ilat > *lat2) *lat2 = ilat;
+                    if (ilon < lon11) lon11 = ilon;
+                    if (ilon > lon12) lon12 = ilon;
+                    if (ilat < lat1) lat1 = ilat;
+                    if (ilat > lat2) lat2 = ilat;
                   }
               }
             else
@@ -543,41 +544,40 @@ genlonlatboxCurv(const int gridID, double xlon1, double xlon2, double xlat1, dou
                 if (((xval >= xlon1 && xval <= xlon2) || (xval - 360 >= xlon1 && xval - 360 <= xlon2)
                      || (xval + 360 >= xlon1 && xval + 360 <= xlon2)))
                   {
-                    if (ilon < *lon21) *lon21 = ilon;
-                    if (ilon > *lon22) *lon22 = ilon;
-                    if (ilat < *lat1) *lat1 = ilat;
-                    if (ilat > *lat2) *lat2 = ilat;
+                    if (ilon < lon21) lon21 = ilon;
+                    if (ilon > lon22) lon22 = ilon;
+                    if (ilat < lat1) lat1 = ilat;
+                    if (ilat > lat2) lat2 = ilat;
                   }
               }
             /*
             if ( xval >= xlon1 && xval <= xlon2 )
               {
-                if ( ilon < *lon21 ) *lon21 = ilon;
-                if ( ilon > *lon22 ) *lon22 = ilon;
-                if ( ilat < *lat1 ) *lat1 = ilat;
-                if ( ilat > *lat2 ) *lat2 = ilat;
+                if ( ilon < lon21 ) lon21 = ilon;
+                if ( ilon > lon22 ) lon22 = ilon;
+                if ( ilat < lat1 ) lat1 = ilat;
+                if ( ilat > lat2 ) lat2 = ilat;
               }
             else if ( xval >= xlon1-360 && xval <= xlon2-360 )
               {
-                if ( ilon < *lon11 ) *lon11 = ilon;
-                if ( ilon > *lon12 ) *lon12 = ilon;
-                if ( ilat < *lat1 ) *lat1 = ilat;
-                if ( ilat > *lat2 ) *lat2 = ilat;
+                if ( ilon < lon11 ) lon11 = ilon;
+                if ( ilon > lon12 ) lon12 = ilon;
+                if ( ilat < lat1 ) lat1 = ilat;
+                if ( ilat > lat2 ) lat2 = ilat;
               }
             */
           }
       }
-  /*
-  while ( *lon12 >= *lon21 ) (*lon12)--;
-  if ( *lon12 <= *lon11 ) { *lon11 = nlon-1; *lon12 = 0; }
-  */
-  if (*lon12 == 0 && *lon11 > 0) *lon11 = -1;
 
-  if (*lat2 - *lat1 + 1 <= 0) cdoAbort("Latitudinal dimension is too small!");
+  // while ( lon12 >= lon21 ) lon12--;
+  // if ( lon12 <= lon11 ) { lon11 = nlon-1; lon12 = 0; }
+  if (lon12 == 0 && lon11 > 0) lon11 = -1;
+
+  if (lat2 - lat1 + 1 <= 0) cdoAbort("Latitudinal dimension is too small!");
 }
 
 void
-getlonlatparams(int argc_offset, double *xlon1, double *xlon2, double *xlat1, double *xlat2)
+getlonlatparams(int argc_offset, double &xlon1, double &xlon2, double &xlat1, double &xlat2)
 {
   bool lset = false;
 
@@ -587,10 +587,10 @@ getlonlatparams(int argc_offset, double *xlon1, double *xlon2, double *xlat1, do
       const char *gridname = operatorArgv()[argc_offset + 0];
       if (strcmp(gridname, "europe") == 0)
         {
-          *xlon1 = -30;
-          *xlon2 = 60;
-          *xlat1 = 30;
-          *xlat2 = 80;
+          xlon1 = -30;
+          xlon2 = 60;
+          xlat1 = 30;
+          xlat2 = 80;
           lset = true;
         }
     }
@@ -599,18 +599,18 @@ getlonlatparams(int argc_offset, double *xlon1, double *xlon2, double *xlat1, do
     {
       operatorCheckArgc(argc_offset + 4);
 
-      *xlon1 = parameter2double(operatorArgv()[argc_offset + 0]);
-      *xlon2 = parameter2double(operatorArgv()[argc_offset + 1]);
-      *xlat1 = parameter2double(operatorArgv()[argc_offset + 2]);
-      *xlat2 = parameter2double(operatorArgv()[argc_offset + 3]);
+      xlon1 = parameter2double(operatorArgv()[argc_offset + 0]);
+      xlon2 = parameter2double(operatorArgv()[argc_offset + 1]);
+      xlat1 = parameter2double(operatorArgv()[argc_offset + 2]);
+      xlat2 = parameter2double(operatorArgv()[argc_offset + 3]);
     }
 }
 
 void
-genlonlatbox(int argc_offset, int gridID, long *lat1, long *lat2, long *lon11, long *lon12, long *lon21, long *lon22)
+genlonlatbox(int argc_offset, int gridID, long &lat1, long &lat2, long &lon11, long &lon12, long &lon21, long &lon22)
 {
   double xlon1 = 0, xlon2 = 0, xlat1 = 0, xlat2 = 0;
-  getlonlatparams(argc_offset, &xlon1, &xlon2, &xlat1, &xlat2);
+  getlonlatparams(argc_offset, xlon1, xlon2, xlat1, xlat2);
 
   const int gridtype = gridInqType(gridID);
 
@@ -621,11 +621,11 @@ genlonlatbox(int argc_offset, int gridID, long *lat1, long *lat2, long *lon11, l
 }
 
 static int
-genlonlatgrid(int gridID1, long *lat1, long *lat2, long *lon11, long *lon12, long *lon21, long *lon22)
+genlonlatgrid(int gridID1, long &lat1, long &lat2, long &lon11, long &lon12, long &lon21, long &lon22)
 {
   genlonlatbox(0, gridID1, lat1, lat2, lon11, lon12, lon21, lon22);
 
-  return gengrid(gridID1, *lat1, *lat2, *lon11, *lon12, *lon21, *lon22);
+  return gengrid(gridID1, lat1, lat2, lon11, lon12, lon21, lon22);
 }
 
 static int
@@ -633,7 +633,7 @@ gencellgrid(int gridID1, long *gridsize2, std::vector<long> &cellidx)
 {
   const long cellinc = 4096;
 
-  int argc_offset = 0;
+  const int argc_offset = 0;
   operatorCheckArgc(argc_offset + 4);
 
   double xlon1 = parameter2double(operatorArgv()[argc_offset + 0]);
@@ -641,19 +641,8 @@ gencellgrid(int gridID1, long *gridsize2, std::vector<long> &cellidx)
   double xlat1 = parameter2double(operatorArgv()[argc_offset + 2]);
   double xlat2 = parameter2double(operatorArgv()[argc_offset + 3]);
 
-  double x;
-  if (xlon1 >= xlon2)
-    {
-      x = xlon1;
-      xlon1 = xlon2;
-      xlon2 = x;
-    }
-  if (xlat1 >= xlat2)
-    {
-      x = xlat1;
-      xlat1 = xlat2;
-      xlat2 = x;
-    }
+  if (xlon1 >= xlon2) std::swap(xlon1, xlon2);
+  if (xlat1 >= xlat2) std::swap(xlat1, xlat2);
 
   const int gridtype = gridInqType(gridID1);
   const size_t gridsize1 = gridInqSize(gridID1);
@@ -669,12 +658,11 @@ gencellgrid(int gridID1, long *gridsize2, std::vector<long> &cellidx)
         }
     }
 
-  if (gridInqXvals(gridID1, nullptr) == 0 || gridInqXvals(gridID1, nullptr) == 0)
-    cdoAbort("Grid cell coordinates missing!");
+  if (gridInqXvals(gridID1, nullptr) == 0 || gridInqXvals(gridID1, nullptr) == 0) cdoAbort("Grid cell coordinates missing!");
 
   std::vector<double> xvals(gridsize1);
   std::vector<double> yvals(gridsize1);
-  
+
   gridInqXvals(gridID1, xvals.data());
   gridInqYvals(gridID1, yvals.data());
 
@@ -717,37 +705,32 @@ gencellgrid(int gridID1, long *gridsize2, std::vector<long> &cellidx)
 }
 
 static void
-checkBounds(const char *txt, const long minVal, const long maxVal, long *lval)
+checkBounds(const char *txt, const long minVal, const long maxVal, long &lval)
 {
-  if (*lval < minVal)
+  if (lval < minVal)
     {
       cdoWarning("%s index out of range, set to %ld!", txt, minVal);
-      *lval = minVal;
+      lval = minVal;
     }
 
-  if (*lval > maxVal)
+  if (lval > maxVal)
     {
       cdoWarning("%s index out of range, set to %ld!", txt, maxVal);
-      *lval = maxVal;
+      lval = maxVal;
     }
 }
 
 void
-genindexbox(int argc_offset, int gridID1, long *lat1, long *lat2, long *lon11, long *lon12, long *lon21, long *lon22)
+genindexbox(int argc_offset, int gridID1, long &lat1, long &lat2, long &lon11, long &lon12, long &lon21, long &lon22)
 {
   operatorCheckArgc(argc_offset + 4);
 
-  *lon11 = parameter2int(operatorArgv()[argc_offset + 0]);
-  *lon12 = parameter2int(operatorArgv()[argc_offset + 1]);
-  *lat1 = parameter2int(operatorArgv()[argc_offset + 2]);
-  *lat2 = parameter2int(operatorArgv()[argc_offset + 3]);
+  lon11 = parameter2int(operatorArgv()[argc_offset + 0]);
+  lon12 = parameter2int(operatorArgv()[argc_offset + 1]);
+  lat1 = parameter2int(operatorArgv()[argc_offset + 2]);
+  lat2 = parameter2int(operatorArgv()[argc_offset + 3]);
 
-  if (*lat1 > *lat2)
-    {
-      const long temp = *lat1;
-      *lat1 = *lat2;
-      *lat2 = temp;
-    }
+  if (lat1 > lat2) std::swap(lat1, lat2);
 
   const long nlon = gridInqXsize(gridID1);
   const long nlat = gridInqYsize(gridID1);
@@ -757,51 +740,52 @@ genindexbox(int argc_offset, int gridID1, long *lat1, long *lat2, long *lon11, l
   checkBounds("First longitude", 1, nlon, lon11);
   checkBounds("Last longitude", 1, nlon, lon12);
 
-  (*lon11)--;
-  (*lon12)--;
-  (*lat1)--;
-  (*lat2)--;
+  lon11--;
+  lon12--;
+  lat1--;
+  lat2--;
 
-  if (*lon11 > *lon12)
+  if (lon11 > lon12)
     {
-      *lon21 = *lon11;
-      *lon22 = nlon - 1;
-      *lon11 = 0;
+      lon21 = lon11;
+      lon22 = nlon - 1;
+      lon11 = 0;
     }
   else
     {
-      if (*lon12 > nlon - 1)
+      if (lon12 > nlon - 1)
         {
-          *lon21 = *lon11;
-          *lon22 = nlon - 1;
-          *lon11 = 0;
-          *lon12 = 0;
+          lon21 = lon11;
+          lon22 = nlon - 1;
+          lon11 = 0;
+          lon12 = 0;
         }
       else
         {
-          *lon21 = 0;
-          *lon22 = -1;
+          lon21 = 0;
+          lon22 = -1;
         }
     }
 }
 
 static int
-genindexgrid(int gridID1, long *lat1, long *lat2, long *lon11, long *lon12, long *lon21, long *lon22)
+genindexgrid(int gridID1, long &lat1, long &lat2, long &lon11, long &lon12, long &lon21, long &lon22)
 {
   genindexbox(0, gridID1, lat1, lat2, lon11, lon12, lon21, lon22);
 
   int gridID2 = -1;
   if (gridInqType(gridID1) == GRID_PROJECTION && gridInqProjType(gridID1) == CDI_PROJ_LCC)
-    gridID2 = cdo_define_subgrid_grid(gridID1, *lon11, *lon12, *lat1, *lat2);
+    gridID2 = cdo_define_subgrid_grid(gridID1, lon11, lon12, lat1, lat2);
   else
-    gridID2 = gengrid(gridID1, *lat1, *lat2, *lon11, *lon12, *lon21, *lon22);
+    gridID2 = gengrid(gridID1, lat1, lat2, lon11, lon12, lon21, lon22);
 
   return gridID2;
 }
 
 template <typename T>
 static void
-window(const int nwpv, const T *array1, const int gridID1, T *array2, const long lat1, const long lat2, const long lon11, const long lon12, const long lon21, const long lon22)
+window(const int nwpv, const T *array1, const int gridID1, T *array2, const long lat1, const long lat2, const long lon11,
+       const long lon12, const long lon21, const long lon22)
 {
   const long nlon = gridInqXsize(gridID1);
 
@@ -833,7 +817,7 @@ window(const int nwpv, const T *array1, const int gridID1, T *array2, const long
 
 template <typename T>
 static void
-window_cell(const int nwpv, const T *array1, T *array2, const long gridsize2, const long *restrict cellidx)
+window_cell(const int nwpv, const std::vector<T> &array1, std::vector<T> &array2, const long gridsize2, const std::vector<long> &restrict cellidx)
 {
   if (nwpv == 2)
     {
@@ -876,7 +860,7 @@ Selbox(void *process)
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -900,7 +884,8 @@ Selbox(void *process)
           || (gridtype == GRID_PROJECTION && gridInqProjType(gridID1) == CDI_PROJ_RLL)
           || (gridtype == GRID_PROJECTION && gridInqProjType(gridID1) == CDI_PROJ_LCC)
           || (gridtype == GRID_PROJECTION && gridInqProjType(gridID1) == CDI_PROJ_STERE)
-          || (operatorID == SELINDEXBOX && (gridtype == GRID_GENERIC || gridtype == GRID_PROJECTION) && gridInqXsize(gridID1) > 0 && gridInqYsize(gridID1) > 0)
+          || (operatorID == SELINDEXBOX && (gridtype == GRID_GENERIC || gridtype == GRID_PROJECTION) && gridInqXsize(gridID1) > 0
+              && gridInqYsize(gridID1) > 0)
           || (operatorID == SELLONLATBOX && gridtype == GRID_UNSTRUCTURED))
         {
           if (operatorID == SELLONLATBOX)
@@ -911,12 +896,12 @@ Selbox(void *process)
               if (gridtype == GRID_UNSTRUCTURED)
                 gridID2 = gencellgrid(gridID1, &sbox[index].nvals, sbox[index].cellidx);
               else
-                gridID2 = genlonlatgrid(gridID1, &sbox[index].lat1, &sbox[index].lat2, &sbox[index].lon11, &sbox[index].lon12,
-                                        &sbox[index].lon21, &sbox[index].lon22);
+                gridID2 = genlonlatgrid(gridID1, sbox[index].lat1, sbox[index].lat2, sbox[index].lon11, sbox[index].lon12,
+                                        sbox[index].lon21, sbox[index].lon22);
             }
           else
-            gridID2 = genindexgrid(gridID1, &sbox[index].lat1, &sbox[index].lat2, &sbox[index].lon11, &sbox[index].lon12,
-                                   &sbox[index].lon21, &sbox[index].lon22);
+            gridID2 = genindexgrid(gridID1, sbox[index].lat1, sbox[index].lat2, sbox[index].lon11, sbox[index].lon12,
+                                   sbox[index].lon21, sbox[index].lon22);
 
           sbox[index].gridID1 = gridID1;
           sbox[index].gridID2 = gridID2;
@@ -953,8 +938,8 @@ Selbox(void *process)
 
   if (varID >= nvars) cdoWarning("No variables selected!");
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   size_t gridsizemax = vlistGridsizeMax(vlistID1);
   if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
@@ -978,17 +963,17 @@ Selbox(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
           if (Options::CDO_Memtype == MEMTYPE_FLOAT)
-            pstreamReadRecordF(streamID1, array1f.data(), &nmiss);
+            cdoReadRecordF(streamID1, array1f.data(), &nmiss);
           else
-            pstreamReadRecord(streamID1, array1.data(), &nmiss);
+            cdoReadRecord(streamID1, array1.data(), &nmiss);
 
-          pstreamDefRecord(streamID2, varID, levelID);
+          cdoDefRecord(streamID2, varID, levelID);
 
           if (vars[varID])
             {
@@ -1007,7 +992,7 @@ Selbox(void *process)
               if (Options::CDO_Memtype == MEMTYPE_FLOAT)
                 {
                   if (operatorID == SELLONLATBOX && gridtype == GRID_UNSTRUCTURED)
-                    window_cell(nwpv, array1f.data(), array2f.data(), gridsize2, sbox[index].cellidx.data());
+                    window_cell(nwpv, array1f, array2f, gridsize2, sbox[index].cellidx);
                   else
                     window(nwpv, array1f.data(), gridID1, array2f.data(), sbox[index].lat1, sbox[index].lat2, sbox[index].lon11,
                            sbox[index].lon12, sbox[index].lon21, sbox[index].lon22);
@@ -1015,7 +1000,7 @@ Selbox(void *process)
               else
                 {
                   if (operatorID == SELLONLATBOX && gridtype == GRID_UNSTRUCTURED)
-                    window_cell(nwpv, array1.data(), array2.data(), gridsize2, sbox[index].cellidx.data());
+                    window_cell(nwpv, array1, array2, gridsize2, sbox[index].cellidx);
                   else
                     window(nwpv, array1.data(), gridID1, array2.data(), sbox[index].lat1, sbox[index].lat2, sbox[index].lon11,
                            sbox[index].lon12, sbox[index].lon21, sbox[index].lon22);
@@ -1031,16 +1016,16 @@ Selbox(void *process)
                 }
 
               if (Options::CDO_Memtype == MEMTYPE_FLOAT)
-                pstreamWriteRecordF(streamID2, array2f.data(), nmiss);
+                cdoWriteRecordF(streamID2, array2f.data(), nmiss);
               else
-                pstreamWriteRecord(streamID2, array2.data(), nmiss);
+                cdoWriteRecord(streamID2, array2.data(), nmiss);
             }
           else
             {
               if (Options::CDO_Memtype == MEMTYPE_FLOAT)
-                pstreamWriteRecordF(streamID2, array1f.data(), nmiss);
+                cdoWriteRecordF(streamID2, array1f.data(), nmiss);
               else
-                pstreamWriteRecord(streamID2, array1.data(), nmiss);
+                cdoWriteRecord(streamID2, array1.data(), nmiss);
             }
         }
 
