@@ -21,22 +21,21 @@
       Select      select         Select fields
 */
 
-#include "cdo_debugOutput.h"
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 #include "datetime.h"
 #include "sellist.h"
 #include "printinfo.h"
 #include "progress.h"
 
+
 double datestrToDouble(const char *datestr, int opt);
 
 bool *cdo_read_timestepmask(const char *maskfile, int *n);
 
 static void
-write_const_vars(const int streamID2, const int vlistID2, const int nvars, std::vector<std::vector<double>> &vardata2)
+write_const_vars(CdoStreamID streamID2, const int vlistID2, const int nvars, std::vector<std::vector<double>> &vardata2)
 {
   for (int varID2c = 0; varID2c < nvars; ++varID2c)
     {
@@ -49,8 +48,8 @@ write_const_vars(const int streamID2, const int vlistID2, const int nvars, std::
             {
               double *pdata = &vardata2[varID2c][gridsize * levelID2c];
               const size_t nmiss = arrayNumMV(gridsize, pdata, missval);
-              pstreamDefRecord(streamID2, varID2c, levelID2c);
-              pstreamWriteRecord(streamID2, pdata, nmiss);
+              cdoDefRecord(streamID2, varID2c, levelID2c);
+              cdoWriteRecord(streamID2, pdata, nmiss);
             }
           vardata2[varID2c].clear();
           vardata2[varID2c].shrink_to_fit();
@@ -96,11 +95,31 @@ eval_timestepmask(const char *maskfile, KVList &kvlist)
   Free(imask);
 }
 
+const char *
+steptypeName(const int tsteptype)
+{
+  const char *steptype;
+  // clang-format off
+  if      (tsteptype == TSTEP_INSTANT)  steptype = "instant";
+  else if (tsteptype == TSTEP_INSTANT2) steptype = "instant";
+  else if (tsteptype == TSTEP_INSTANT3) steptype = "instant";
+  else if (tsteptype == TSTEP_MIN)      steptype = "min";
+  else if (tsteptype == TSTEP_MAX)      steptype = "max";
+  else if (tsteptype == TSTEP_AVG)      steptype = "avg";
+  else if (tsteptype == TSTEP_ACCUM)    steptype = "accum";
+  else if (tsteptype == TSTEP_RANGE)    steptype = "range";
+  else if (tsteptype == TSTEP_DIFF)     steptype = "diff";
+  else if (tsteptype == TSTEP_SUM)      steptype = "sum";
+  else steptype = "unknown";
+  // clang-format on
+  return steptype;
+}
+
 void *
 Select(void *process)
 {
   bool lconstvars = true;
-  int streamID2 = CDI_UNDEFID;
+  CdoStreamID streamID2 = CDO_STREAM_UNDEF;
   int nrecs;
   int nvars, nvars2 = 0;
   int varID, levelID;
@@ -125,7 +144,7 @@ Select(void *process)
   const int SELECT = cdoOperatorAdd("select", 0, 0, "parameter list");
   const int DELETE = cdoOperatorAdd("delete", 0, 0, "parameter list");
 
-  const bool lcopy = UNCHANGED_RECORD;
+  const bool lcopy = unchangedRecord();
 
   const int operatorID = cdoOperatorID();
 
@@ -197,7 +216,7 @@ Select(void *process)
       if (!Options::cdoVerbose && nfiles > 1) Progress::update(0, 1, (indf + 1.) / nfiles);
       if (Options::cdoVerbose) cdoPrint("Process file: %s", cdoGetStreamName(indf));
 
-      const int streamID1 = cdoStreamOpenRead(cdoStreamName(indf));
+      CdoStreamID streamID1 = cdoOpenRead((indf));
 
       const int vlistID1 = cdoStreamInqVlist(streamID1);
       const int taxisID1 = vlistInqTaxis(vlistID1);
@@ -225,9 +244,10 @@ Select(void *process)
                 }
             }
 
-          bool lvarsel = SELLIST_NVAL(code) || SELLIST_NVAL(ltype) || SELLIST_NVAL(zaxisnum) || SELLIST_NVAL(gridnum)
-                         || SELLIST_NVAL(name) || SELLIST_NVAL(param) || SELLIST_NVAL(zaxisname) || SELLIST_NVAL(gridname)
-                         || SELLIST_NVAL(steptype);
+          const bool findVariable = SELLIST_NVAL(code) || SELLIST_NVAL(name) || SELLIST_NVAL(param);
+
+          bool lvarsel = findVariable || SELLIST_NVAL(ltype) || SELLIST_NVAL(zaxisnum) || SELLIST_NVAL(gridnum)
+                         || SELLIST_NVAL(zaxisname) || SELLIST_NVAL(gridname) || SELLIST_NVAL(steptype);
 
           bool llevsel = SELLIST_NVAL(level) || SELLIST_NVAL(levidx);
 
@@ -263,18 +283,7 @@ Select(void *process)
               gridname = gname;
 
               const int tsteptype = vlistInqVarTsteptype(vlistID1, varID);
-              // clang-format off
-              if      (tsteptype == TSTEP_INSTANT)  steptype = "instant";
-              else if (tsteptype == TSTEP_INSTANT2) steptype = "instant";
-              else if (tsteptype == TSTEP_INSTANT3) steptype = "instant";
-              else if (tsteptype == TSTEP_MIN)      steptype = "min";
-              else if (tsteptype == TSTEP_MAX)      steptype = "max";
-              else if (tsteptype == TSTEP_AVG)      steptype = "avg";
-              else if (tsteptype == TSTEP_ACCUM)    steptype = "accum";
-              else if (tsteptype == TSTEP_RANGE)    steptype = "range";
-              else if (tsteptype == TSTEP_DIFF)     steptype = "diff";
-              else steptype = "unknown";
-              // clang-format on
+              steptype = steptypeName(tsteptype);
 
               vars[varID] = false;
 
@@ -295,11 +304,9 @@ Select(void *process)
                                ? (found_ltype || found_zaxis || found_zname)
                                : true;
 
-              if (!vars[varID] && lgrid && lvar) vars[varID] = true;
-              if (!vars[varID] && lvert && lvar) vars[varID] = true;
-              if (!vars[varID] && lstep && lvar) vars[varID] = true;
+              if (lvar && lgrid && lvert && lstep) vars[varID] = true;
 
-              if (!vars[varID] && !lvar)
+              if (!vars[varID] && !lvar && !findVariable)
                 {
                   if (found_grid || found_gname)
                     vars[varID] = true;
@@ -477,7 +484,7 @@ Select(void *process)
                   if (ptimestep < 0)
                     {
                       if (nfiles != 1) cdoAbort("Negative timesteps only supported with one input stream!");
-                      if (ntsteps < 0 && !processSelf().inputStreams[0]->ispipe)
+                      if (ntsteps < 0 && !cdoAssertFilesOnly() == false)
                         {
                           int tsID = 0;
                           while (cdoStreamInqTimestep(streamID1, tsID)) tsID++;
@@ -612,20 +619,20 @@ Select(void *process)
           if (copytimestep)
             {
               taxisCopyTimestep(taxisID2, taxisID1);
-              if (streamID2 == CDI_UNDEFID)
+              if (streamID2 == CDO_STREAM_UNDEF)
                 {
                   const bool lasttimestep = (nfiles == 1) && (ntsteps2 > 1) && (ntsteps2 == (tsID1 + 1));
                   if (lasttimestep && tsID2 == 0) ntsteps2 = 1;
                   if (ntsteps2 == 0 || ntsteps2 == 1) vlistDefNtsteps(vlistID2, ntsteps2);
-                  streamID2 = cdoStreamOpenWrite(cdoStreamName(nfiles));
+                  streamID2 = cdoOpenWrite((nfiles));
                   vlistDefTaxis(vlistID2, taxisID2);
-                  pstreamDefVlist(streamID2, vlistID2);
+                  cdoDefVlist(streamID2, vlistID2);
                 }
-              pstreamDefTimestep(streamID2, tsID2);
+              cdoDefTimestep(streamID2, tsID2);
 
               for (int recID = 0; recID < nrecs; ++recID)
                 {
-                  pstreamInqRecord(streamID1, &varID, &levelID);
+                  cdoInqRecord(streamID1, &varID, &levelID);
                   if (vlistInqFlag(vlistID0, varID, levelID) == true)
                     {
                       if (lconstvars && tsID2 > 0 && tsID1 == 0)
@@ -635,16 +642,16 @@ Select(void *process)
                       const int levelID2 = vlistFindLevel(vlistID2, varID, levelID);
                       if (lcopy_const && tsID2 == 0) write_const_vars(streamID2, vlistID2, varID2, vardata2);
 
-                      pstreamDefRecord(streamID2, varID2, levelID2);
+                      cdoDefRecord(streamID2, varID2, levelID2);
                       if (lcopy)
                         {
-                          pstreamCopyRecord(streamID2, streamID1);
+                          cdoCopyRecord(streamID2, streamID1);
                         }
                       else
                         {
                           size_t nmiss;
-                          pstreamReadRecord(streamID1, array.data(), &nmiss);
-                          pstreamWriteRecord(streamID2, array.data(), nmiss);
+                          cdoReadRecord(streamID1, array.data(), &nmiss);
+                          cdoWriteRecord(streamID2, array.data(), nmiss);
                         }
                     }
                 }
@@ -657,7 +664,7 @@ Select(void *process)
             {
               for (int recID = 0; recID < nrecs; ++recID)
                 {
-                  pstreamInqRecord(streamID1, &varID, &levelID);
+                  cdoInqRecord(streamID1, &varID, &levelID);
                   if (vlistInqFlag(vlistID0, varID, levelID) == true)
                     {
                       const int varID2 = vlistFindVar(vlistID2, varID);
@@ -671,7 +678,7 @@ Select(void *process)
                               vardata2[varID2].resize(gridsize * nlevel);
                             }
                           size_t nmiss;
-                          pstreamReadRecord(streamID1, &vardata2[varID2][gridsize * levelID2], &nmiss);
+                          cdoReadRecord(streamID1, &vardata2[varID2][gridsize * levelID2], &nmiss);
                         }
                     }
                 }
@@ -701,7 +708,7 @@ END_LABEL:
   SELLIST_CHECK_FLAG(season);
   SELLIST_CHECK_FLAG(date);
 
-  if (streamID2 != CDI_UNDEFID) cdoStreamClose(streamID2);
+  if (streamID2 != CDO_STREAM_UNDEF) cdoStreamClose(streamID2);
 
   vlistDestroy(vlistID0);
   vlistDestroy(vlistID2);
