@@ -17,16 +17,13 @@
 
 #include <cdi.h>
 
-#include "cdo_debugOutput.h"
 #include "cdo_int.h"
-#include "pstream_int.h"
+
 #include "listarray.h"
 
 // NOTE: All operators in this module works only on GRIB edition 1 files!
 
-extern "C" {
-void streamGrbChangeParameterIdentification(int code, int ltype, int lev);
-}
+extern "C" void streamGrbChangeParameterIdentification(int code, int ltype, int lev);
 
 /*
 Supported notations:
@@ -196,7 +193,7 @@ Selmulti(void *process)
 
   // operatorCheckArgc(1);
   filenameOrString = operatorArgv()[0];
-  if (CdoDebug::cdoDebugExt)
+  if (cdoDebugExt)
     {
       printf("Given operator arguments (nr=%d): \n", operatorArgc());
       for (int i = 0; i < operatorArgc(); i++) printf("%s", operatorArgv()[i]);
@@ -226,14 +223,14 @@ Selmulti(void *process)
                "%s",
                filenameOrString);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   int vlistID1 = cdoStreamInqVlist(streamID1);
 
   vlistClearFlag(vlistID1);
   int nvars = vlistNvars(vlistID1);
 
-  if (CdoDebug::cdoDebugExt) cdoPrint(" Total number of variables: %d", nvars);
+  Debug(cdoDebugExt," Total number of variables: %d", nvars);
 
   for (varID = 0; varID < nvars; varID++)
     {
@@ -257,7 +254,7 @@ Selmulti(void *process)
           for (int ii = 0; ii < NUMTUPLES; ii++)
             {
               TUPLEREC *tuplerec = getSelTuple(ii);
-              // if ( CdoDebug::cdoDebugExt ) cdoPrint(" Processing: (code %d,
+              // if ( cdoDebugExt ) cdoPrint(" Processing: (code %d,
               // ltype %d, level %d);  nvars=%d, varID=%d", code, ltype,
               // (int)level, nvars, varID);
               // Note: When the list is Empty then function
@@ -273,7 +270,7 @@ Selmulti(void *process)
                         {
                         case 0:  // operator decides ...
                           vlistDefFlag(vlistID1, varID, levelID, true);
-                          if (CdoDebug::cdoDebugExt)
+                          if (cdoDebugExt)
                             {
                               if (!tuplerec->simpleMath)
                                 cdoPrint(" Selecting : (code %3i, ltype %3i, level %3i)   [varID(%d),levelID(%d)]", code, ltype,
@@ -286,7 +283,7 @@ Selmulti(void *process)
                           break;
                         case 1:
                           vlistDefFlag(vlistID1, varID, levelID, true);
-                          if (CdoDebug::cdoDebugExt)
+                          if (cdoDebugExt)
                             {
                               if (!tuplerec->simpleMath)
                                 cdoPrint(" Selecting : (code %3i, ltype %3i, level %3i)   [varID(%d),levelID(%d)]", code, ltype,
@@ -299,7 +296,7 @@ Selmulti(void *process)
                           break;
                         case 2:
                           vlistDefFlag(vlistID1, varID, levelID, false);
-                          if (CdoDebug::cdoDebugExt)
+                          if (cdoDebugExt)
                             {
                               if (!tuplerec->simpleMath)
                                 cdoPrint(" Selecting for removal: (code %3i, ltype %3i, level %3i)   [varID(%d),levelID(%d)]", code,
@@ -318,7 +315,7 @@ Selmulti(void *process)
                         {
                         case 0:  // operator decides ...
                           vlistDefFlag(vlistID1, varID, levelID, false);
-                          if (CdoDebug::cdoDebugExt)
+                          if (cdoDebugExt)
                             {
                               if (!tuplerec->simpleMath)
                                 cdoPrint(" Selecting for removal: (code %3i, ltype %3i, level %3i)   [varID(%d),levelID(%d)]", code,
@@ -331,7 +328,7 @@ Selmulti(void *process)
                           break;
                         case 1:
                           vlistDefFlag(vlistID1, varID, levelID, true);
-                          if (CdoDebug::cdoDebugExt)
+                          if (cdoDebugExt)
                             {
                               if (!tuplerec->simpleMath)
                                 cdoPrint(" Selecting : (code %3i, ltype %3i, level %3i)   [varID(%d),levelID(%d)]", code, ltype,
@@ -344,7 +341,7 @@ Selmulti(void *process)
                           break;
                         case 2:
                           vlistDefFlag(vlistID1, varID, levelID, false);
-                          if (CdoDebug::cdoDebugExt)
+                          if (cdoDebugExt)
                             {
                               if (!tuplerec->simpleMath)
                                 cdoPrint(" Selecting for removal: (code %3i, ltype %3i, level %3i)   [varID(%d),levelID(%d)]", code,
@@ -363,7 +360,7 @@ Selmulti(void *process)
         }      // end for ( levelID
     }          // end for ( varID
 
-  if (CdoDebug::cdoDebugExt) cdoPrint(" Writing the selected fields ...");
+  Debug(cdoDebugExt," Writing the selected fields ...");
 
   int vlistID2 = vlistCreate();
   cdoVlistCopyFlag(vlistID2, vlistID1);
@@ -379,8 +376,8 @@ Selmulti(void *process)
 
   int nrecs = vlistNrecs(vlistID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   size_t gridsize = vlistGridsizeMax(vlistID1);
   if (vlistNumber(vlistID1) != CDI_REAL) gridsize *= 2;
@@ -390,11 +387,11 @@ Selmulti(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
           missval = vlistInqVarMissval(vlistID1, varID);
 
           if (vlistInqFlag(vlistID1, varID, levelID) == true)
@@ -419,7 +416,7 @@ Selmulti(void *process)
                     {
                       if (operatorID == CHANGEMULTI)
                         {
-                          if (CdoDebug::cdoDebugExt)
+                          if (cdoDebugExt)
                             cdoPrint(" Processing: (code %d, ltype %d, level %d);  nvars=%d, varID=%d => (selcode %d, selltype "
                                      "%d, sellevel %d) => change (%d,%d,%d)",
                                      code, ltype, (int) level, nvars, varID, selcode, selltype, sellevel, tuplerec->changedCode,
@@ -438,7 +435,7 @@ Selmulti(void *process)
                         }
                       else
                         {
-                          if (CdoDebug::cdoDebugExt)
+                          if (cdoDebugExt)
                             cdoPrint(" Processing: (code %d, ltype %d, level %d);  nvars=%d, varID=%d => (selcode %d, "
                                      "selltype %d, sellevel %d)",
                                      code, ltype, (int) level, nvars, varID, selcode, selltype, sellevel);
@@ -467,29 +464,29 @@ Selmulti(void *process)
                            code, ltype, (int) (level), varID, levelID, varID2, levelID2);
                   continue;
                 }
-              pstreamDefRecord(streamID2, varID2, levelID2);
+              cdoDefRecord(streamID2, varID2, levelID2);
 
               if (lcopy)
                 {
-                  if (CdoDebug::cdoDebugExt)
+                  if (cdoDebugExt)
                     cdoPrint(" Copying record [%4d] with (code %3i, ltype %3i, level %3i)   [varID(%d),levelID(%d)]", recID, code,
                              ltype, (int) (level), varID, levelID);
-                  pstreamCopyRecord(streamID2, streamID1);
+                  cdoCopyRecord(streamID2, streamID1);
                 }
               else
                 {
-                  pstreamReadRecord(streamID1, array, &nmiss);
+                  cdoReadRecord(streamID1, array, &nmiss);
 
                   if (!simpleMath)
                     {
-                      if (CdoDebug::cdoDebugExt)
+                      if (cdoDebugExt)
                         cdoPrint(" Writing record [%4d] with (code %3i, ltype %3i, level %3i)   [varID(%d),levelID(%d)]", recID,
                                  code, ltype, (int) (level), varID, levelID);
-                      pstreamWriteRecord(streamID2, array, nmiss);
+                      cdoWriteRecord(streamID2, array, nmiss);
                     }
                   else  // 1:  simple array arithmetics ( *,+)
                     {
-                      if (CdoDebug::cdoDebugExt)
+                      if (cdoDebugExt)
                         cdoPrint(" Writing record [%4d] with (code %3i, ltype %3i, level %3i)   [varID(%d),levelID(%d)]; "
                                  "SCALE=%f; OFFSET=%f",
                                  recID, code, ltype, (int) (level), varID, levelID, scale, offset);
@@ -500,7 +497,7 @@ Selmulti(void *process)
                             // If SCALE and/or OFFSET are defined, then the data
                             // values are scaled as SCALE*(VALUE-OFFSET).
                           }
-                      pstreamWriteRecord(streamID2, array, nmiss);
+                      cdoWriteRecord(streamID2, array, nmiss);
                     }
                 }  // end of else ( lcopy )
             }      // end if ( vlistInqFlag(vlistID1, varID, levelID) == true )
@@ -513,7 +510,7 @@ Selmulti(void *process)
         zaxisID = vlistInqVarZaxis(vlistID1, varID);
         level = zaxisInqLevel(zaxisID, levelID);
         ltype = zaxis2ltype(zaxisID);
-        if ( CdoDebug::cdoDebugExt ) cdoPrint(" Removing record [%4d] with (code
+        if ( cdoDebugExt ) cdoPrint(" Removing record [%4d] with (code
         %3i, ltype %3i, level %3i)   [varID(%d),levelID(%d)]",recID, code,
         ltype, (int)(level), varID, levelID);
         }
@@ -532,7 +529,7 @@ Selmulti(void *process)
 
   cdoFinish();
 
-  CdoDebug::cdoDebugExt = 0;
+  cdoDebugExt = 0;
 
   return 0;
 }
@@ -601,7 +598,7 @@ checkListContainsInt(int value, ListArray<int> *list, int arraylen)
 
   if (arraylen == 0)
     {
-      // if ( CdoDebug::cdoDebugExt ) cdoPrint(" value=%d: found. (list empty)");
+      // if ( cdoDebugExt ) cdoPrint(" value=%d: found. (list empty)");
       return 1;
     }
   for (i = 0; i < arraylen; i++)
@@ -609,17 +606,17 @@ checkListContainsInt(int value, ListArray<int> *list, int arraylen)
       if (array[i] == -1)
         {  // this is for '*' selection; can be any code, any level or any
            // level-type
-          // if ( CdoDebug::cdoDebugExt ) cdoPrint(" value=%d: found.");
+          // if ( cdoDebugExt ) cdoPrint(" value=%d: found.");
           return 1;
         }
 
       if (array[i] == value)
         {
-          // if ( CdoDebug::cdoDebugExt ) cdoPrint(" value=%d: found.");
+          // if ( cdoDebugExt ) cdoPrint(" value=%d: found.");
           return 1;
         }
     }
-  // if ( CdoDebug::cdoDebugExt ) cdoPrint(" value=%d: NOT found.");
+  // if ( cdoDebugExt ) cdoPrint(" value=%d: NOT found.");
   return 0;
 }
 
@@ -673,7 +670,7 @@ goToNextSeparator(char *pline)
         }
     }
   if (separatorFound) pline++;
-  if (CdoDebug::cdoDebugExt >= 100) cdoPrint("goToNextSeparator():  pline= ('%s') ", pline);
+  Debug(cdoDebugExt >= 100,"goToNextSeparator():  pline= ('%s') ", pline);
   // while ( isspace((int) *pline) ) pline++;
   pline = removeSpaces(pline);
   return pline;
@@ -692,19 +689,19 @@ strContains(char *str, const char *substr)
 
   if (lensub > lenstr)
     {
-      if (CdoDebug::cdoDebugExt >= 100)
+      if (cdoDebugExt >= 100)
         cdoPrint("strContains():  substr('%s') NOT found in str('%s');  lensub(%zu)>lenstr(%zu) ", substr, str, lensub, lenstr);
       return nullptr;
     }
   char *rv = strstr(str, substr);
   if (rv)
     {
-      if (CdoDebug::cdoDebugExt >= 100) cdoPrint("strContains():  substr('%s') FOUND in str('%s')", substr, str);
+      Debug(cdoDebugExt >= 100,"strContains():  substr('%s') FOUND in str('%s')", substr, str);
       return (rv + lensub);  // points after subStr ..
     }
   else
     {
-      if (CdoDebug::cdoDebugExt >= 100) cdoPrint("strContains():  substr('%s') NOT found in str('%s')", substr, str);
+      Debug(cdoDebugExt >= 100,"strContains():  substr('%s') NOT found in str('%s')", substr, str);
       return rv;
     }
 }
@@ -722,10 +719,10 @@ findParamEnd(char *str)
   if (ptrEnding != nullptr)
     {
       ptrEnding = removeSpaces(ptrEnding);
-      if (CdoDebug::cdoDebugExt >= 100) cdoPrint(" ptrEnding='%s'", ptrEnding);
+      Debug(cdoDebugExt >= 100," ptrEnding='%s'", ptrEnding);
       return ptrEnding;
     }
-  if (CdoDebug::cdoDebugExt >= 100) cdoPrint(" ptrEnding=end-of-string");
+  Debug(cdoDebugExt >= 100," ptrEnding=end-of-string");
   size_t lenstr = strlen(str);
   ptrEnding = str + lenstr;
   ptrEnding = removeSpaces(ptrEnding);
@@ -745,10 +742,10 @@ findTupleEnd(char *str)
   if (ptrEnding != nullptr)
     {
       ptrEnding = removeSpaces(ptrEnding);
-      if (CdoDebug::cdoDebugExt >= 100) cdoPrint(" findTupleEnd='%s'", ptrEnding);
+      Debug(cdoDebugExt >= 100," findTupleEnd='%s'", ptrEnding);
       return ptrEnding;
     }
-  if (CdoDebug::cdoDebugExt >= 100) cdoPrint(" findTupleEnd=end-of-string");
+  Debug(cdoDebugExt >= 100," findTupleEnd=end-of-string");
   size_t lenstr = strlen(str);
   ptrEnding = str + lenstr;
   ptrEnding = removeSpaces(ptrEnding);
@@ -765,19 +762,19 @@ readlineForParsing(FILE *gfp, char *strToParsePtr, char *line)
     }
   else if (strToParsePtr != nullptr)  // we parse a given string
     {
-      if (CdoDebug::cdoDebugExt >= 30) cdoPrint("%s(): Parsing selection string:  %s", __func__, strToParsePtr);
+      if (cdoDebugExt >= 30) cdoPrint("%s(): Parsing selection string:  %s", __func__, strToParsePtr);
       char *tpEnd = nullptr;
       if (strlen(strToParsePtr) > 0) tpEnd = findTupleEnd(strToParsePtr);
       if (tpEnd == nullptr)
         {
-          if (CdoDebug::cdoDebugExt >= 100) cdoPrint("%s(): End of selection string reached.", __func__);
+          Debug(cdoDebugExt >= 100,"%s(): End of selection string reached.", __func__);
           return nullptr;
         }
       else
         {
           tpEnd[0] = 0;
           if (strlen(strToParsePtr) <= MAX_LINE_LEN) strcpy(line, strToParsePtr);
-          if (CdoDebug::cdoDebugExt >= 100) cdoPrint("%s(): Current selection line=%s", __func__, line);
+          Debug(cdoDebugExt >= 100,"%s(): Current selection line=%s", __func__, line);
           strToParsePtr = tpEnd + 1;
           return strToParsePtr;
         }
@@ -815,17 +812,17 @@ multiSelectionParser(const char *filenameOrString)
       // cdo selmulti,'{(33/34;105;10);(11/32,8;109;51/52/53/54/55)}'
       // - or -
       // cdo changemulti,'{(134;1;0|1;105;0);(6;1;0|6;105;0)}'
-      strncpy(strToParse, filenameOrString, MAX_LINE_LEN-1);
+      strncpy(strToParse, filenameOrString, MAX_LINE_LEN - 1);
       strToParsePtr = &strToParse[0];
       if (strToParsePtr[0] == '{') strToParsePtr++;
       int strLn = strlen(strToParsePtr);
       if (strToParsePtr[strLn - 1] == '}') strToParsePtr[strLn - 1] = 0;
-      if (CdoDebug::cdoDebugExt) cdoPrint(" Parsing selection string:  %s", strToParsePtr);
+      Debug(cdoDebugExt," Parsing selection string:  %s", strToParsePtr);
     }
   else
     {
       gfp = fopen(filenameOrString, "r");
-      if (CdoDebug::cdoDebugExt) cdoPrint(" Parsing file:  %s", filenameOrString);
+      Debug(cdoDebugExt," Parsing file:  %s", filenameOrString);
       if (gfp == nullptr)
         {
           cdoAbort(" Missing file:  %s", filenameOrString);
@@ -838,7 +835,7 @@ multiSelectionParser(const char *filenameOrString)
       if (line[0] == '#') continue;
       if (line[0] == '\0') continue;
       pline = line;
-      if (CdoDebug::cdoDebugExt >= 30) cdoPrint(": Line: %s", pline);
+      if (cdoDebugExt >= 30) cdoPrint(": Line: %s", pline);
       while (isspace((int) *pline)) pline++;
       if (pline[0] == '\0') continue;
       strpos = strContains(pline, "SELECT, ");
@@ -859,7 +856,7 @@ multiSelectionParser(const char *filenameOrString)
         }
       if (strpos != nullptr)  // we have SELECT ..
         {
-          if (CdoDebug::cdoDebugExt) cdoPrint(" Parsing notation SELECT: %s", strpos);
+          Debug(cdoDebugExt," Parsing notation SELECT: %s", strpos);
           pline = strpos;
           tuplerec = TUPLERECNew();
           tuplerec->sel_or_del_or_change = selectionRec;
@@ -867,16 +864,16 @@ multiSelectionParser(const char *filenameOrString)
           // SELECT, PARAMETER=11/17, LEVTYPE=105, LEVEL=2
           while ((pline != nullptr) && (strlen(pline) != 0))
             {
-              if (CdoDebug::cdoDebugExt >= 100) cdoPrint("pline='%s'", pline);
+              Debug(cdoDebugExt >= 100,"pline='%s'", pline);
               strpos = strContains(pline, "PARAMETER=");
               if ((strpos) != nullptr)
                 {
-                  if (CdoDebug::cdoDebugExt >= 100) cdoPrint(": PARAMETER=%s", strpos);
+                  Debug(cdoDebugExt >= 100,": PARAMETER=%s", strpos);
                   pline = strpos;
                   pline = removeSpaces(pline);
                   parEnd = findParamEnd(pline);
                   if ((!parEnd) && (!pline))
-                    if (CdoDebug::cdoDebugExt >= 100) cdoPrint("strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                    Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                   while ((pline != parEnd) && (strlen(pline) > 0))
                     {
                       pline = removeSpaces(pline);
@@ -884,7 +881,7 @@ multiSelectionParser(const char *filenameOrString)
                         val = -1;
                       else
                         val = atoi(pline);
-                      if (CdoDebug::cdoDebugExt >= 100) cdoPrint("code=%d", val);
+                      Debug(cdoDebugExt >= 100,"code=%d", val);
                       tuplerec->ncodes = push_backIntList(val, tuplerec->codeLST, tuplerec->ncodes);
                       strpos = goToNextSeparator(pline);
                       pline = strpos;
@@ -892,16 +889,16 @@ multiSelectionParser(const char *filenameOrString)
                       pline = skipSeparator(strpos);
                     }
                 }
-              if (CdoDebug::cdoDebugExt >= 100) cdoPrint("pline='%s'", pline);
+              Debug(cdoDebugExt >= 100,"pline='%s'", pline);
               strpos = strContains(pline, "LEVTYPE=");
               if ((strpos) != nullptr)
                 {
-                  if (CdoDebug::cdoDebugExt >= 100) cdoPrint(": LEVTYPE=%s", strpos);
+                  Debug(cdoDebugExt >= 100,": LEVTYPE=%s", strpos);
                   pline = strpos;
                   pline = removeSpaces(pline);
                   parEnd = findParamEnd(pline);
                   if ((!parEnd) && (!pline))
-                    if (CdoDebug::cdoDebugExt >= 100) cdoPrint("strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                    Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                   while ((pline != parEnd) && (strlen(pline) > 0))
                     {
                       pline = removeSpaces(pline);
@@ -912,13 +909,13 @@ multiSelectionParser(const char *filenameOrString)
                       if (val == 100)
                         {
                           convertLevelsHPa2Pa = 1;
-                          if (CdoDebug::cdoDebugExt >= 1)
+                          if (cdoDebugExt >= 1)
                             cdoPrint("Detected levelType=100 ! HIP selection specifies HPa but CDO has levels in "
                                      "Pascals.\nSelection HPa's will be converted into Pa's.");
                         }
                       else
                         convertLevelsHPa2Pa = 0;
-                      if (CdoDebug::cdoDebugExt >= 100) cdoPrint("levelType=%d", val);
+                      Debug(cdoDebugExt >= 100,"levelType=%d", val);
                       tuplerec->nlevelTypes = push_backIntList(val, tuplerec->levelTypeLST, tuplerec->nlevelTypes);
                       strpos = goToNextSeparator(pline);
                       pline = strpos;
@@ -926,16 +923,16 @@ multiSelectionParser(const char *filenameOrString)
                       pline = skipSeparator(strpos);
                     }
                 }
-              if (CdoDebug::cdoDebugExt >= 100) cdoPrint("pline='%s'", pline);
+              Debug(cdoDebugExt >= 100,"pline='%s'", pline);
               strpos = strContains(pline, "LEVEL=");
               if ((strpos) != nullptr)
                 {
-                  if (CdoDebug::cdoDebugExt >= 100) cdoPrint(": LEVEL=%s", strpos);
+                  Debug(cdoDebugExt >= 100,": LEVEL=%s", strpos);
                   pline = strpos;
                   pline = removeSpaces(pline);
                   parEnd = findParamEnd(pline);
                   if ((!parEnd) && (!pline))
-                    if (CdoDebug::cdoDebugExt >= 100) cdoPrint("strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                    Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                   while ((pline != parEnd) && (strlen(pline) > 0))
                     {
                       pline = removeSpaces(pline);
@@ -947,7 +944,7 @@ multiSelectionParser(const char *filenameOrString)
                         {
                           val *= 100;
                         }
-                      if (CdoDebug::cdoDebugExt >= 100) cdoPrint("level=%d", val);
+                      Debug(cdoDebugExt >= 100,"level=%d", val);
                       tuplerec->nlevels = push_backIntList(val, tuplerec->levelLST, tuplerec->nlevels);
                       strpos = goToNextSeparator(pline);
                       pline = strpos;
@@ -955,21 +952,21 @@ multiSelectionParser(const char *filenameOrString)
                       pline = skipSeparator(strpos);
                     }
                 }
-              if (CdoDebug::cdoDebugExt >= 100) cdoPrint("pline='%s' (check SCALE=...)", pline);
+              Debug(cdoDebugExt >= 100,"pline='%s' (check SCALE=...)", pline);
               strpos = strContains(pline, "SCALE=");
               if ((strpos) != nullptr)
                 {
-                  if (CdoDebug::cdoDebugExt >= 100) cdoPrint(": SCALE= %s", strpos);
+                  Debug(cdoDebugExt >= 100,": SCALE= %s", strpos);
                   pline = strpos;
                   pline = removeSpaces(pline);
                   parEnd = findParamEnd(pline);
                   if ((!parEnd) && (!pline))
-                    if (CdoDebug::cdoDebugExt >= 100) cdoPrint("strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                    Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                   while ((pline != parEnd) && (strlen(pline) > 0))
                     {
                       pline = removeSpaces(pline);
                       floatval = atof(pline);
-                      if (CdoDebug::cdoDebugExt >= 100) cdoPrint("scale=%f", floatval);
+                      Debug(cdoDebugExt >= 100,"scale=%f", floatval);
                       tuplerec->simpleMath = 1;    // 1:  simple array arithmetics
                                                    // ( *,+), 0: do nothing
                       tuplerec->scale = floatval;  // tuplerec->offset = 0.0;
@@ -979,21 +976,21 @@ multiSelectionParser(const char *filenameOrString)
                       pline = skipSeparator(strpos);
                     }
                 }
-              if (CdoDebug::cdoDebugExt >= 100) cdoPrint("pline='%s' (check OFFSET=...)", pline);
+              Debug(cdoDebugExt >= 100,"pline='%s' (check OFFSET=...)", pline);
               strpos = strContains(pline, "OFFSET=");
               if ((strpos) != nullptr)
                 {
-                  if (CdoDebug::cdoDebugExt >= 100) cdoPrint(": OFFSET= %s", strpos);
+                  Debug(cdoDebugExt >= 100,": OFFSET= %s", strpos);
                   pline = strpos;
                   pline = removeSpaces(pline);
                   parEnd = findParamEnd(pline);
                   if ((!parEnd) && (!pline))
-                    if (CdoDebug::cdoDebugExt >= 100) cdoPrint("strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                    Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                   while ((pline != parEnd) && (strlen(pline) > 0))
                     {
                       pline = removeSpaces(pline);
                       floatval = atof(pline);
-                      if (CdoDebug::cdoDebugExt >= 100) cdoPrint("offset=%f", floatval);
+                      Debug(cdoDebugExt >= 100,"offset=%f", floatval);
                       tuplerec->simpleMath = 1;     // 1:  simple array arithmetics
                                                     // ( *,+), 0: do nothing
                       tuplerec->offset = floatval;  // tuplerec->scale = 1.0;
@@ -1032,7 +1029,7 @@ multiSelectionParser(const char *filenameOrString)
         }
       if (strpos != nullptr)
         {
-          if (CdoDebug::cdoDebugExt)
+          if (cdoDebugExt)
             cdoPrint(" Parsing notation (code(s),..; levelType(s),..; level(s),..) : %s; [selectionRec =%d]", strpos, selectionRec);
           if (selectionRec == 3) strpos = strContains(pline, "(");
           pline = strpos;
@@ -1042,13 +1039,13 @@ multiSelectionParser(const char *filenameOrString)
           // (33/34; 105; 10)
           while ((pline != nullptr) && (strlen(pline) != 0) && (pline[0] != ')'))
             {
-              if (CdoDebug::cdoDebugExt >= 100) cdoPrint("[1]: pline='%s'", pline);
+              Debug(cdoDebugExt >= 100,"[1]: pline='%s'", pline);
               // 1st is code
               {
                 pline = removeSpaces(pline);
                 parEnd = findParamEnd(pline);
                 if ((!parEnd) && (!pline))
-                  if (CdoDebug::cdoDebugExt >= 100) cdoPrint("strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                  Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                 while ((pline != parEnd) && (strlen(pline) > 0))
                   {
                     pline = removeSpaces(pline);
@@ -1056,7 +1053,7 @@ multiSelectionParser(const char *filenameOrString)
                       val = -1;
                     else
                       val = atoi(pline);
-                    if (CdoDebug::cdoDebugExt >= 100) cdoPrint("code=%d", val);
+                    Debug(cdoDebugExt >= 100,"code=%d", val);
                     tuplerec->ncodes = push_backIntList(val, tuplerec->codeLST, tuplerec->ncodes);
                     strpos = goToNextSeparator(pline);
                     if (!strpos)
@@ -1070,12 +1067,12 @@ multiSelectionParser(const char *filenameOrString)
                   }
               }
               // 2nd is level type
-              if (CdoDebug::cdoDebugExt >= 100) cdoPrint("[2]: pline='%s'", pline);
+              Debug(cdoDebugExt >= 100,"[2]: pline='%s'", pline);
               {
                 pline = removeSpaces(pline);
                 parEnd = findParamEnd(pline);
                 if ((!parEnd) && (!pline))
-                  if (CdoDebug::cdoDebugExt >= 100) cdoPrint("strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                  Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                 while ((pline != parEnd) && (strlen(pline) > 0))
                   {
                     pline = removeSpaces(pline);
@@ -1086,14 +1083,14 @@ multiSelectionParser(const char *filenameOrString)
                     if (val == 100)
                       {
                         convertLevelsHPa2Pa = 1;
-                        if (CdoDebug::cdoDebugExt >= 1)
+                        if (cdoDebugExt >= 1)
                           cdoPrint("Detected levelType=100 ! Selection specifies HPa but CDO has levels in "
                                    "Pascals.\nSelection HPa's will be converted into Pa's.");
                       }
                     else
                       convertLevelsHPa2Pa = 0;
 
-                    if (CdoDebug::cdoDebugExt >= 100) cdoPrint("levelType=%d", val);
+                    Debug(cdoDebugExt >= 100,"levelType=%d", val);
                     tuplerec->nlevelTypes = push_backIntList(val, tuplerec->levelTypeLST, tuplerec->nlevelTypes);
                     strpos = goToNextSeparator(pline);
                     if (!strpos)
@@ -1107,12 +1104,12 @@ multiSelectionParser(const char *filenameOrString)
                   }
               }
               // 3rd is level
-              if (CdoDebug::cdoDebugExt >= 100) cdoPrint("[3]: pline='%s'", pline);
+              Debug(cdoDebugExt >= 100,"[3]: pline='%s'", pline);
               {
                 pline = removeSpaces(pline);
                 parEnd = findParamEnd(pline);
                 if ((!parEnd) && (!pline))
-                  if (CdoDebug::cdoDebugExt >= 100) cdoPrint("strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                  Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                 while ((pline != parEnd) && (strlen(pline) > 0))
                   {
                     pline = removeSpaces(pline);
@@ -1124,11 +1121,11 @@ multiSelectionParser(const char *filenameOrString)
                       {
                         val *= 100;
                       }
-                    if (CdoDebug::cdoDebugExt >= 100) cdoPrint("level=%d", val);
+                    Debug(cdoDebugExt >= 100,"level=%d", val);
                     tuplerec->nlevels = push_backIntList(val, tuplerec->levelLST, tuplerec->nlevels);
-                    if (CdoDebug::cdoDebugExt >= 100) cdoPrint("strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                    Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                     strpos = goToNextSeparator(pline);
-                    if (CdoDebug::cdoDebugExt >= 100) cdoPrint("strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                    Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                     if (!strpos)
                       {
                         strpos = strContains(pline, "|");  // compact notation for  changemulti
@@ -1136,7 +1133,7 @@ multiSelectionParser(const char *filenameOrString)
                           pline = strpos - 1;  // strContains returns character after...
                         else
                           pline = parEnd;
-                        if (CdoDebug::cdoDebugExt >= 100) cdoPrint("strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                        Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                         break;
                       }
                     else
@@ -1149,7 +1146,7 @@ multiSelectionParser(const char *filenameOrString)
               // cdo
               // changemulti,'{(134;1;*|1;105;*);{(6;1;*|6;105;*)};{(246;*;*|76;*;*)};{(247;*;*|58;*;*)};{(248;*;*|71;*;*)}'
               // fileIN fileOUT
-              if (CdoDebug::cdoDebugExt >= 100) cdoPrint("[OPT]: pline='%s'", pline);
+              Debug(cdoDebugExt >= 100,"[OPT]: pline='%s'", pline);
               {
                 pline = removeSpaces(pline);
                 // pline points to: "=1;105;*);....."
@@ -1161,13 +1158,13 @@ multiSelectionParser(const char *filenameOrString)
                     parEnd = findParamEnd(pline);
                     if ((!parEnd) || (pline[0] == 0))
                       cdoAbort("Channot parse: strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
-                    if (CdoDebug::cdoDebugExt >= 100) cdoPrint("strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                    Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                     if (pline[0] == '*')
                       val = -1;
                     else
                       val = atoi(pline);
                     tuplerec->changedCode = val;
-                    if (CdoDebug::cdoDebugExt >= 100) cdoPrint("changedCode=%d", val);
+                    Debug(cdoDebugExt >= 100,"changedCode=%d", val);
                     strpos = goToNextSeparator(pline);
                     if (!strpos) cdoAbort("Channot parse: strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                     pline = skipSeparator(strpos);
@@ -1175,13 +1172,13 @@ multiSelectionParser(const char *filenameOrString)
                     parEnd = findParamEnd(pline);
                     if ((!parEnd) || (pline[0] == 0))
                       cdoAbort("Channot parse: strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
-                    if (CdoDebug::cdoDebugExt >= 100) cdoPrint("strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                    Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                     if (pline[0] == '*')
                       val = -1;
                     else
                       val = atoi(pline);
                     tuplerec->changedLevelType = val;
-                    if (CdoDebug::cdoDebugExt >= 100) cdoPrint("changedLevelType=%d", val);
+                    Debug(cdoDebugExt >= 100,"changedLevelType=%d", val);
                     strpos = goToNextSeparator(pline);
                     if (!strpos) cdoAbort("Channot parse: strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                     pline = skipSeparator(strpos);
@@ -1189,18 +1186,18 @@ multiSelectionParser(const char *filenameOrString)
                     parEnd = findParamEnd(pline);
                     if ((!parEnd) || (pline[0] == 0))
                       cdoAbort("Channot parse: strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
-                    if (CdoDebug::cdoDebugExt >= 100) cdoPrint("strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                    Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                     if (pline[0] == '*')
                       val = -1;
                     else
                       val = atoi(pline);
                     tuplerec->changedLevel = val;
-                    if (CdoDebug::cdoDebugExt >= 100) cdoPrint("changedLevel=%d", val);
+                    Debug(cdoDebugExt >= 100,"changedLevel=%d", val);
                     pline = parEnd;
                   }  // changemulti specification
                 parEnd = findParamEnd(pline);
                 if ((!parEnd) && (!pline))
-                  if (CdoDebug::cdoDebugExt >= 100) cdoPrint("strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
+                  Debug(cdoDebugExt >= 100,"strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
                 while ((pline != parEnd) && (strlen(pline) > 0))
                   {
                     pline = removeSpaces(pline);
@@ -1212,7 +1209,7 @@ multiSelectionParser(const char *filenameOrString)
                       {
                         val *= 100;
                       }
-                    if (CdoDebug::cdoDebugExt >= 100) cdoPrint("level=%d", val);
+                    Debug(cdoDebugExt >= 100,"level=%d", val);
                     tuplerec->nlevels = push_backIntList(val, tuplerec->levelLST, tuplerec->nlevels);
                     strpos = goToNextSeparator(pline);
                     if (!strpos)
@@ -1238,7 +1235,7 @@ multiSelectionParser(const char *filenameOrString)
 void
 printSelectionTuples()
 {
-  if (CdoDebug::cdoDebugExt) cdoPrint(" Printing selection tuples:");
+  Debug(cdoDebugExt," Printing selection tuples:");
 
   int ii, ri;
   for (ii = 0; ii < NUMTUPLES; ii++)
@@ -1248,7 +1245,7 @@ printSelectionTuples()
       strcpy(strval, "(");
       char bff[200];
       bff[0] = '\0';
-      if (CdoDebug::cdoDebugExt)
+      if (cdoDebugExt)
         cdoPrint(" Selection tuple [%d]: ncodes=%d, nlevelTypes=%d, nlevels=%d", ii, tuplerec->ncodes, tuplerec->nlevelTypes,
                  tuplerec->nlevels);
       for (ri = 0; ri < tuplerec->ncodes; ri++)
@@ -1289,15 +1286,15 @@ printSelectionTuples()
       // 3:change
       if (tuplerec->sel_or_del_or_change == 1)
         {
-          if (CdoDebug::cdoDebugExt) cdoPrint(" Selection tuple [%d] = %s (select)", ii, strval);
+          Debug(cdoDebugExt," Selection tuple [%d] = %s (select)", ii, strval);
         }
       else if (tuplerec->sel_or_del_or_change == 2)
         {
-          if (CdoDebug::cdoDebugExt) cdoPrint(" Selection tuple [%d] = %s (delete)", ii, strval);
+          Debug(cdoDebugExt," Selection tuple [%d] = %s (delete)", ii, strval);
         }
       if (tuplerec->sel_or_del_or_change == 3)
         {
-          if (CdoDebug::cdoDebugExt)
+          if (cdoDebugExt)
             cdoPrint(" Selection tuple [%d] = %s (change) => (%d; %d;%d;)", ii, strval, tuplerec->changedCode,
                      tuplerec->changedLevelType, tuplerec->changedLevel);
         }
