@@ -19,7 +19,6 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
 
 void *
 Seloperator(void *process)
@@ -36,7 +35,7 @@ Seloperator(void *process)
 
   cdoInitialize(process);
 
-  const bool lcopy = UNCHANGED_RECORD;
+  const bool lcopy = unchangedRecord();
 
   operatorInputArg("code, ltype, level");
 
@@ -45,7 +44,7 @@ Seloperator(void *process)
 
   if (operatorArgc() == 3) slevel = parameter2double(operatorArgv()[2]);
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
   const int vlistID1 = cdoStreamInqVlist(streamID1);
 
   const int nvars = vlistNvars(vlistID1);
@@ -87,8 +86,8 @@ Seloperator(void *process)
   const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   std::vector<double> array;
   if (!lcopy)
@@ -101,26 +100,26 @@ Seloperator(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
           if (vlistInqFlag(vlistID1, varID, levelID) == true)
             {
               varID2 = vlistFindVar(vlistID2, varID);
               levelID2 = vlistFindLevel(vlistID2, varID, levelID);
 
-              pstreamDefRecord(streamID2, varID2, levelID2);
+              cdoDefRecord(streamID2, varID2, levelID2);
 
               if (lcopy)
                 {
-                  pstreamCopyRecord(streamID2, streamID1);
+                  cdoCopyRecord(streamID2, streamID1);
                 }
               else
                 {
-                  pstreamReadRecord(streamID1, array.data(), &nmiss);
-                  pstreamWriteRecord(streamID2, array.data(), nmiss);
+                  cdoReadRecord(streamID1, array.data(), &nmiss);
+                  cdoWriteRecord(streamID2, array.data(), nmiss);
                 }
             }
         }
