@@ -24,7 +24,7 @@
 #include <cdi.h>
 
 #include "cdo_int.h" /* processSelf */
-#include "pstream_int.h"
+
 #include "listarray.h"
 
 void *
@@ -49,9 +49,9 @@ Selrec(void *process)
       for (int i = 0; i < nsel; i++) cdoPrint("intarr entry: %d %d", i, intarr[i]);
     }
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  const int filetype = pstreamInqFiletype(streamID1);
+  const int filetype = cdoInqFiletype(streamID1);
 
   if (filetype == CDI_FILETYPE_NC || filetype == CDI_FILETYPE_NC2 || filetype == CDI_FILETYPE_NC4 || filetype == CDI_FILETYPE_NC4C
       || filetype == CDI_FILETYPE_NC5)
@@ -64,27 +64,27 @@ Selrec(void *process)
   const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   int recordID = 0;
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
           recordID++;
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           for (int i = 0; i < nsel; i++)
             {
               if (recordID == intarr[i])
                 {
-                  pstreamDefRecord(streamID2, varID, levelID);
-                  pstreamCopyRecord(streamID2, streamID1);
+                  cdoDefRecord(streamID2, varID, levelID);
+                  cdoCopyRecord(streamID2, streamID1);
 
                   break;
                 }
