@@ -33,7 +33,7 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
+
 #include "listarray.h"
 #include "util_string.h"
 
@@ -53,7 +53,7 @@ season_to_months(const char *season, int *imonths)
     {
       if (len > 12) cdoAbort("Too many months %d (limit=12)!", (int) len);
       char *season_u = strdup(season);
-      strtoupper(season_u);
+      cstrToUpperCase(season_u);
       const char *sstr = strstr(smons, season_u);
       free(season_u);
       if (sstr == nullptr) cdoAbort("Season %s not available!", season);
@@ -184,7 +184,7 @@ datelist(ListArray<double> &listArrayFlt)
 void *
 Seltime(void *process)
 {
-  int streamID2 = -1;
+  CdoStreamID streamID2 = CDO_STREAM_UNDEF;
   int nrecs;
   int varID, levelID;
   int nsel = 0;
@@ -204,7 +204,7 @@ Seltime(void *process)
 
   cdoInitialize(process);
 
-  const bool lcopy = UNCHANGED_RECORD;
+  const bool lcopy = unchangedRecord();
 
   // clang-format off
   const int SELTIMESTEP = cdoOperatorAdd("seltimestep", func_step,     0, "timesteps");
@@ -277,7 +277,7 @@ Seltime(void *process)
       for (i = 0; i < nsel; i++) selfound[i] = false;
     }
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -305,7 +305,7 @@ Seltime(void *process)
         {
           if (intarr[i] < 0)
             {
-              if (ntsteps < 0 && !processSelf().inputStreams[0]->ispipe)
+              if (ntsteps < 0 && !cdoAssertFilesOnly() == false)
                 {
                   int tsID = 0;
                   while (cdoStreamInqTimestep(streamID1, tsID)) tsID++;
@@ -367,7 +367,7 @@ Seltime(void *process)
 
                   for (levelID = 0; levelID < nlevel; levelID++)
                     {
-                      vars[tsID][varID][levelID].ptr = (double *) Malloc(gridsize * sizeof(double));
+                      vars[tsID][varID][levelID].resize(gridsize);
                     }
                 }
             }
@@ -464,8 +464,8 @@ Seltime(void *process)
           taxisCopyTimestep(taxisID2, taxisID1);
           if (tsID2 == 0)
             {
-              streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-              pstreamDefVlist(streamID2, vlistID2);
+              streamID2 = cdoOpenWrite(1);
+              cdoDefVlist(streamID2, vlistID2);
             }
 
           if (lnts1 && ncts == 0)
@@ -481,18 +481,18 @@ Seltime(void *process)
                 {
                   taxisDefVdate(taxisID2, vdate_list[it]);
                   taxisDefVtime(taxisID2, vtime_list[it]);
-                  pstreamDefTimestep(streamID2, tsID2++);
+                  cdoDefTimestep(streamID2, tsID2++);
 
                   for (varID = 0; varID < nvars; varID++)
                     {
                       if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT && tsID2 > 1) continue;
-                      const int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-                      for (levelID = 0; levelID < nlevel; levelID++)
+                      const int nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+                      for (levelID = 0; levelID < nlevels; levelID++)
                         {
-                          pstreamDefRecord(streamID2, varID, levelID);
-                          double *single = vars[it][varID][levelID].ptr;
+                          cdoDefRecord(streamID2, varID, levelID);
+                          double *single = vars[it][varID][levelID].vec.data();
                           const size_t nmiss = vars[it][varID][levelID].nmiss;
-                          pstreamWriteRecord(streamID2, single, nmiss);
+                          cdoWriteRecord(streamID2, single, nmiss);
                         }
                     }
                 }
@@ -507,7 +507,7 @@ Seltime(void *process)
               process_nts1 = true;
             }
 
-          pstreamDefTimestep(streamID2, tsID2++);
+          cdoDefTimestep(streamID2, tsID2++);
 
           if (tsID > 0 && lconstout)
             {
@@ -520,10 +520,10 @@ Seltime(void *process)
                       const int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
                       for (levelID = 0; levelID < nlevel; levelID++)
                         {
-                          pstreamDefRecord(streamID2, varID, levelID);
-                          double *single = vars[nts][varID][levelID].ptr;
+                          cdoDefRecord(streamID2, varID, levelID);
+                          double *single = vars[nts][varID][levelID].vec.data();
                           const size_t nmiss = vars[nts][varID][levelID].nmiss;
-                          pstreamWriteRecord(streamID2, single, nmiss);
+                          cdoWriteRecord(streamID2, single, nmiss);
                         }
                     }
                 }
@@ -531,16 +531,16 @@ Seltime(void *process)
 
           for (int recID = 0; recID < nrecs; recID++)
             {
-              pstreamInqRecord(streamID1, &varID, &levelID);
-              pstreamDefRecord(streamID2, varID, levelID);
+              cdoInqRecord(streamID1, &varID, &levelID);
+              cdoDefRecord(streamID2, varID, levelID);
               if (lcopy)
                 {
-                  pstreamCopyRecord(streamID2, streamID1);
+                  cdoCopyRecord(streamID2, streamID1);
                 }
               else
                 {
-                  pstreamReadRecord(streamID1, array.data(), &nmiss);
-                  pstreamWriteRecord(streamID2, array.data(), nmiss);
+                  cdoReadRecord(streamID1, array.data(), &nmiss);
+                  cdoWriteRecord(streamID2, array.data(), nmiss);
                 }
             }
         }
@@ -565,12 +565,10 @@ Seltime(void *process)
                         for (varID = 0; varID < nvars; varID++)
                           {
                             if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
-                            const int gridID = vlistInqVarGrid(vlistID1, varID);
-                            const size_t gridsize = gridInqSize(gridID);
-                            const int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-                            for (levelID = 0; levelID < nlevel; levelID++)
+                            const int nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+                            for (levelID = 0; levelID < nlevels; levelID++)
                               {
-                                memcpy(vars[it][varID][levelID].ptr, vars[it + 1][varID][levelID].ptr, gridsize * sizeof(double));
+                                vars[it][varID][levelID].vec = vars[it + 1][varID][levelID].vec;
                                 vars[it][varID][levelID].nmiss = vars[it + 1][varID][levelID].nmiss;
                               }
                           }
@@ -584,11 +582,11 @@ Seltime(void *process)
 
               for (int recID = 0; recID < nrecs; recID++)
                 {
-                  pstreamInqRecord(streamID1, &varID, &levelID);
+                  cdoInqRecord(streamID1, &varID, &levelID);
                   if (lnts1 || (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT))
                     {
-                      double *single = vars[nts][varID][levelID].ptr;
-                      pstreamReadRecord(streamID1, single, &nmiss);
+                      double *single = vars[nts][varID][levelID].vec.data();
+                      cdoReadRecord(streamID1, single, &nmiss);
                       vars[nts][varID][levelID].nmiss = nmiss;
                     }
                 }
@@ -598,7 +596,7 @@ Seltime(void *process)
       tsID++;
     }
 
-  if (streamID2 != -1) cdoStreamClose(streamID2);
+  if (streamID2 != CDO_STREAM_UNDEF) cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
   if (operatorID == SELSMON)
@@ -664,11 +662,6 @@ Seltime(void *process)
         }
     }
 
-  if (lnts1 || nconst)
-    {
-      for (tsID = 0; tsID < nts1; tsID++) fieldsFree(vlistID2, vars[tsID]);
-    }
-
   vlistDestroy(vlistID2);
 
   if (tsID2 == 0) cdoAbort("No timesteps selected!");
