@@ -36,7 +36,7 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
+
 #include "listarray.h"
 #include "util_wildcards.h"
 
@@ -61,7 +61,7 @@ Selvar(void *process)
 
   cdoInitialize(process);
 
-  const bool lcopy = UNCHANGED_RECORD;
+  const bool lcopy = unchangedRecord();
 
 #define INVERTS_SELECTION(id) (cdoOperatorF2(id) & 1)
 #define TAKES_STRINGS(id) (cdoOperatorF2(id) & 2)
@@ -129,7 +129,7 @@ Selvar(void *process)
   if ( nsel == 0 )
     cdoAbort("missing code argument!");
   */
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   int nvars = vlistNvars(vlistID1);
@@ -297,8 +297,8 @@ Selvar(void *process)
   const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   std::vector<double> array;
   if (!lcopy)
@@ -313,25 +313,25 @@ Selvar(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
           if (vlistInqFlag(vlistID1, varID, levelID) == true)
             {
               varID2 = vlistFindVar(vlistID2, varID);
               levelID2 = vlistFindLevel(vlistID2, varID, levelID);
 
-              pstreamDefRecord(streamID2, varID2, levelID2);
+              cdoDefRecord(streamID2, varID2, levelID2);
               if (lcopy)
                 {
-                  pstreamCopyRecord(streamID2, streamID1);
+                  cdoCopyRecord(streamID2, streamID1);
                 }
               else
                 {
-                  pstreamReadRecord(streamID1, array.data(), &nmiss);
-                  pstreamWriteRecord(streamID2, array.data(), nmiss);
+                  cdoReadRecord(streamID1, array.data(), &nmiss);
+                  cdoWriteRecord(streamID2, array.data(), nmiss);
                 }
             }
         }
