@@ -24,7 +24,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 #include "timebase.h"
 
 void *
@@ -43,12 +42,12 @@ Selyearidx(void *process)
 
   const bool ltime = cdoOperatorF1(cdoOperatorID());
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
   const int vlistID1 = cdoStreamInqVlist(streamID1);
 
   const int taxisID1 = vlistInqTaxis(vlistID1);
 
-  const int streamID2 = cdoStreamOpenRead(cdoStreamName(1));
+  CdoStreamID streamID2 = cdoOpenRead(1);
   const int vlistID2 = cdoStreamInqVlist(streamID2);
   const int taxisID2 = vlistInqTaxis(vlistID2);
 
@@ -58,34 +57,36 @@ Selyearidx(void *process)
   const int taxisID3 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID3, taxisID3);
 
-  const int streamID3 = cdoStreamOpenWrite(cdoStreamName(2));
-  pstreamDefVlist(streamID3, vlistID3);
+  CdoStreamID streamID3 = cdoOpenWrite(2);
+  cdoDefVlist(streamID3, vlistID3);
+
+  VarList varList1, varList2;
+  varListInit(varList1, vlistID1);
+  varListInit(varList2, vlistID2);
 
   const int maxrecs = vlistNrecs(vlistID1);
-  std::vector<RecordInfo> recinfo(maxrecs);
+  std::vector<RecordInfo> recList(maxrecs);
 
   const size_t gridsizemax = vlistGridsizeMax(vlistID1);
 
   std::vector<double> array(gridsizemax);
 
   FieldVector2D vars1, vars2;
-  fieldsFromVlist(vlistID1, vars1, FIELD_PTR);
-  fieldsFromVlist(vlistID1, vars2, FIELD_PTR);
+  fieldsFromVlist(vlistID1, vars1, FIELD_VEC);
+  fieldsFromVlist(vlistID1, vars2, FIELD_VEC);
 
   const int nvars = vlistNvars(vlistID1);
   for (varID = 0; varID < nvars; ++varID)
     {
-      const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-      const int nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-      const double missval = vlistInqVarMissval(vlistID2, varID);
-      for (levelID = 0; levelID < nlevs; ++levelID)
+      const size_t gridsize = varList1[varID].gridsize;
+      const int nlevels = varList1[varID].nlevels;
+      const double missval = varList2[varID].missval;
+      for (levelID = 0; levelID < nlevels; ++levelID)
         {
-          for (size_t i = 0; i < gridsize; ++i) vars2[varID][levelID].ptr[i] = missval;
+          for (size_t i = 0; i < gridsize; ++i) vars2[varID][levelID].vec[i] = missval;
         }
     }
 
-  const int calendar = taxisInqCalendar(taxisID2);
-
   int tsID = 0;
   int tsID2 = 0;
   int tsID3 = 0;
@@ -103,15 +104,15 @@ Selyearidx(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, vars1[varID][levelID].ptr, &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, vars1[varID][levelID].vec.data(), &nmiss);
           vars1[varID][levelID].nmiss = nmiss;
 
           if (tsID == 0)
             {
-              recinfo[recID].varID = varID;
-              recinfo[recID].levelID = levelID;
-              recinfo[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+              recList[recID].varID = varID;
+              recList[recID].levelID = levelID;
+              recList[recID].lconst = varList1[varID].timetype == TIME_CONSTANT;
             }
         }
 
@@ -123,23 +124,19 @@ Selyearidx(void *process)
           cdiDecodeDate(vdate, &year, &month, &day);
           cdiDecodeTime(vtime, &hour, &minute, &second);
 
-          if (year1 != year) break;
+          if (ltime == false && year1 != year) break;
 
           for (int recID = 0; recID < nrecs; recID++)
             {
-              pstreamInqRecord(streamID2, &varID, &levelID);
-              pstreamReadRecord(streamID2, array.data(), &nmiss);
+              cdoInqRecord(streamID2, &varID, &levelID);
+              cdoReadRecord(streamID2, array.data(), &nmiss);
 
-              const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
+              const size_t gridsize = varList2[varID].gridsize;
               for (size_t i = 0; i < gridsize; ++i)
-                if (nsets == (int) lround(vars1[varID][levelID].ptr[i]))
+                if (nsets == (int) lround(vars1[varID][levelID].vec[i]))
                   {
                     if (lexactdate) taxisCopyTimestep(taxisID3, taxisID2);
-                    if (ltime)
-                      vars2[varID][levelID].ptr[i]
-                          = date_to_julday(calendar, vdate) + (hour * 3600 + minute * 60 + second) / 86400.;
-                    else
-                      vars2[varID][levelID].ptr[i] = array[i];
+                    vars2[varID][levelID].vec[i] = array[i];
                   }
             }
 
@@ -150,19 +147,16 @@ Selyearidx(void *process)
       if (nsets)
         {
           if (!lexactdate) taxisCopyTimestep(taxisID3, taxisID1);
-          pstreamDefTimestep(streamID3, tsID3);
+          cdoDefTimestep(streamID3, tsID3);
 
           for (int recID = 0; recID < maxrecs; recID++)
             {
-              if (tsID && recinfo[recID].lconst) continue;
+              if (tsID && recList[recID].lconst) continue;
 
-              varID = recinfo[recID].varID;
-              levelID = recinfo[recID].levelID;
-              pstreamDefRecord(streamID3, varID, levelID);
-              const size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
-              const double missval = vlistInqVarMissval(vlistID2, varID);
-              nmiss = arrayNumMV(gridsize, vars2[varID][levelID].ptr, missval);
-              pstreamWriteRecord(streamID3, vars2[varID][levelID].ptr, nmiss);
+              varID = recList[recID].varID;
+              levelID = recList[recID].levelID;
+              cdoDefRecord(streamID3, varID, levelID);
+              cdoWriteRecord(streamID3, vars2[varID][levelID].vec.data(), fieldNumMiss(vars2[varID][levelID]));
             }
 
           tsID3++;
@@ -172,9 +166,6 @@ Selyearidx(void *process)
       tsID++;
     }
 
-  fieldsFree(vlistID1, vars2);
-  fieldsFree(vlistID1, vars1);
-
   cdoStreamClose(streamID3);
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
