@@ -29,7 +29,6 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
 
 int
 stringToParam(const char *paramstr)
@@ -94,7 +93,7 @@ Set(void *process)
       newlevel = parameter2double(operatorArgv()[0]);
     }
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -158,8 +157,8 @@ Set(void *process)
     }
 
   // vlistPrint(vlistID2);
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   size_t gridsize = vlistGridsizeMax(vlistID1);
   if (vlistNumber(vlistID1) != CDI_REAL) gridsize *= 2;
@@ -169,15 +168,15 @@ Set(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID1)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID1);
+      cdoDefTimestep(streamID2, tsID1);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamDefRecord(streamID2, varID, levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoDefRecord(streamID2, varID, levelID);
 
-          pstreamReadRecord(streamID1, array.data(), &nmiss);
-          pstreamWriteRecord(streamID2, array.data(), nmiss);
+          cdoReadRecord(streamID1, array.data(), &nmiss);
+          cdoWriteRecord(streamID2, array.data(), nmiss);
         }
 
       tsID1++;
