@@ -19,7 +19,7 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
+
 #include "parse_literals.h"
 #include "pmlist.h"
 #include "util_wildcards.h"
@@ -143,7 +143,8 @@ setAttributes(KVList &kvlist, int vlistID)
           for (int idx = 0; idx < nv; ++idx)
             {
               const int varID = varIDs[idx];
-              // if ( Options::cdoVerbose ) printf("varID, cdiID, attname %d %d %s %d\n", varID, cdiID, attname, (int)strlen(attname));
+              // if ( Options::cdoVerbose ) printf("varID, cdiID, attname %d %d %s %d\n", varID, cdiID, attname,
+              // (int)strlen(attname));
               if (dtype == CDI_DATATYPE_INT8 || dtype == CDI_DATATYPE_INT16 || dtype == CDI_DATATYPE_INT32)
                 {
                   std::vector<int> ivals(nvalues);
@@ -188,7 +189,7 @@ Setattribute(void *process)
 
   cdoOperatorAdd("setattribute", 0, 0, "attributes");
 
-  const bool lcopy = UNCHANGED_RECORD;
+  const bool lcopy = unchangedRecord();
 
   const int operatorID = cdoOperatorID();
 
@@ -200,7 +201,7 @@ Setattribute(void *process)
   PMList pmlist;
   KVList kvlist;
   kvlist.name = "SETATTRIBUTES";
-  if (kvlist.parseArguments(natts, (const char **)operatorArgv()) != 0) cdoAbort("Parse error!");
+  if (kvlist.parseArguments(natts, (const char **) operatorArgv()) != 0) cdoAbort("Parse error!");
   if (Options::cdoVerbose) kvlist.print();
 
   KVList *pkvlist = &kvlist;
@@ -221,7 +222,7 @@ Setattribute(void *process)
         }
     }
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -232,8 +233,8 @@ Setattribute(void *process)
   const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   std::vector<double> array;
   if (!lcopy)
@@ -247,22 +248,22 @@ Setattribute(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamDefRecord(streamID2, varID, levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoDefRecord(streamID2, varID, levelID);
 
           if (lcopy)
             {
-              pstreamCopyRecord(streamID2, streamID1);
+              cdoCopyRecord(streamID2, streamID1);
             }
           else
             {
               size_t nmiss;
-              pstreamReadRecord(streamID1, array.data(), &nmiss);
-              pstreamWriteRecord(streamID2, array.data(), nmiss);
+              cdoReadRecord(streamID1, array.data(), &nmiss);
+              cdoWriteRecord(streamID2, array.data(), nmiss);
             }
         }
 
