@@ -26,12 +26,12 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
-#include "grid.h"
 
-void genlonlatbox(int argc_offset, int gridID1, long *lat1, long *lat2, long *lon11, long *lon12, long *lon21, long *lon22);
+#include <mpim_grid.h>
 
-void genindexbox(int argc_offset, int gridID1, long *lat1, long *lat2, long *lon11, long *lon12, long *lon21, long *lon22);
+void genlonlatbox(int argc_offset, int gridID1, long &lat1, long &lat2, long &lon11, long &lon12, long &lon21, long &lon22);
+
+void genindexbox(int argc_offset, int gridID1, long &lat1, long &lat2, long &lon11, long &lon12, long &lon21, long &lon22);
 
 static void
 setcbox(double constant, double *array, int gridID, long lat1, long lat2, long lon11, long lon12, long lon21, long lon22)
@@ -70,7 +70,7 @@ Setbox(void *process)
 
   const double constant = parameter2double(operatorArgv()[0]);
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
 
@@ -97,9 +97,9 @@ Setbox(void *process)
   operatorInputArg(cdoOperatorEnter(operatorID));
 
   if (operatorID == SETCLONLATBOX)
-    genlonlatbox(1, gridID, &lat1, &lat2, &lon11, &lon12, &lon21, &lon22);
+    genlonlatbox(1, gridID, lat1, lat2, lon11, lon12, lon21, lon22);
   else
-    genindexbox(1, gridID, &lat1, &lat2, &lon11, &lon12, &lon21, &lon22);
+    genindexbox(1, gridID, lat1, lat2, lon11, lon12, lon21, lon22);
 
   const int vlistID2 = vlistDuplicate(vlistID1);
 
@@ -111,9 +111,9 @@ Setbox(void *process)
   std::vector<bool> vars(nvars);
   for (varID = 0; varID < nvars; varID++) vars[varID] = (gridID == vlistInqVarGrid(vlistID1, varID));
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
+  CdoStreamID streamID2 = cdoOpenWrite(1);
 
-  pstreamDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID2, vlistID2);
 
   const size_t gridsize = gridInqSize(gridID);
   std::vector<double> array(gridsize);
@@ -122,22 +122,22 @@ Setbox(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           if (vars[varID])
             {
-              pstreamReadRecord(streamID1, array.data(), &nmiss);
+              cdoReadRecord(streamID1, array.data(), &nmiss);
 
               setcbox(constant, array.data(), gridID, lat1, lat2, lon11, lon12, lon21, lon22);
 
               const double missval = vlistInqVarMissval(vlistID1, varID);
               nmiss = arrayNumMV(gridsize, array.data(), missval);
-              pstreamDefRecord(streamID2, varID, levelID);
-              pstreamWriteRecord(streamID2, array.data(), nmiss);
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, array.data(), nmiss);
             }
         }
 
