@@ -27,12 +27,13 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
+#include "cdi_lockedIO.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
-#include "grid.h"
+#include <mpim_grid.h>
+#include "griddes.h"
+#include "gridreference.h"
 #include "util_files.h"
 
-
 static void
 changeGrid(const int vlistID1, const int vlistID2, const int gridID2)
 {
@@ -51,8 +52,8 @@ changeGrid(const int vlistID1, const int vlistID2, const int gridID2)
 }
 
 static void
-setGridtype(const int vlistID1, const int vlistID2, const int gridtype, const char *gridname, const bool lregular, const bool lregularnn,
-            const bool ldereference, const bool lbounds, std::vector<int> &grid2_vgpm)
+setGridtype(const int vlistID1, const int vlistID2, const int gridtype, const char *gridname, const bool lregular,
+            const bool lregularnn, const bool ldereference, const bool lbounds, std::vector<int> &grid2_vgpm)
 {
   int gridID2;
   bool lrgrid = false;
@@ -197,15 +198,15 @@ readAreaFromFile(const char *areafile, std::vector<double> &gridcellArea)
   if (!fileExists(areafile))
     {
       auto const pos = filename.find_last_of(':');
-      if (pos > 1 && pos < (filename.size()-1))
+      if (pos > 1 && pos < (filename.size() - 1))
         {
-          varname = filename.substr(pos+1);
+          varname = filename.substr(pos + 1);
           filename = filename.substr(0, pos);
           searchName = true;
         }
     }
 
-  const int streamID = streamOpenReadLocked(filename.c_str());
+  int streamID = streamOpenReadLocked(filename.c_str());
   const int vlistID = streamInqVlist(streamID);
 
   int svarID = 0;
@@ -314,9 +315,10 @@ Setgrid(void *process)
 
       if (Options::cdoVerbose)
         {
-          const size_t areasize = gridcellArea.size();;
+          const size_t areasize = gridcellArea.size();
+
           double arrmean, arrmin, arrmax;
-          arrayMinMaxMean(areasize, gridcellArea.data(), &arrmin, &arrmax, &arrmean);
+          arrayMinMaxMean(areasize, gridcellArea.data(), arrmin, arrmax, arrmean);
           cdoPrint("gridcellAreas: %zu %#12.5g%#12.5g%#12.5g", areasize, arrmin, arrmean, arrmax);
         }
     }
@@ -324,7 +326,7 @@ Setgrid(void *process)
     {
       operatorCheckArgc(1);
       const char *maskfile = operatorArgv()[0];
-      const int streamID = streamOpenReadLocked(maskfile);
+      int streamID = streamOpenReadLocked(maskfile);
       const int vlistID = streamInqVlist(streamID);
 
       nrecs = streamInqTimestep(streamID, 0);
@@ -364,7 +366,7 @@ Setgrid(void *process)
       griduri = operatorArgv()[0];
     }
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -420,9 +422,9 @@ Setgrid(void *process)
       unsetGridMask(vlistID1, vlistID2);
     }
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
+  CdoStreamID streamID2 = cdoOpenWrite(1);
 
-  pstreamDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID2, vlistID2);
   // vlistPrint(vlistID2);
 
   size_t gridsize = (lregular || lregularnn) ? vlistGridsizeMax(vlistID2) : vlistGridsizeMax(vlistID1);
@@ -434,14 +436,14 @@ Setgrid(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamDefRecord(streamID2, varID, levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoDefRecord(streamID2, varID, levelID);
 
-          pstreamReadRecord(streamID1, array.data(), &nmiss);
+          cdoReadRecord(streamID1, array.data(), &nmiss);
 
           const int gridID1 = vlistInqVarGrid(vlistID1, varID);
           if (lregular || lregularnn)
@@ -462,7 +464,7 @@ Setgrid(void *process)
                 if (grid2_vgpm[i]) array[j++] = array[i];
             }
 
-          pstreamWriteRecord(streamID2, array.data(), nmiss);
+          cdoWriteRecord(streamID2, array.data(), nmiss);
         }
 
       tsID++;
