@@ -24,8 +24,8 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
-#include "grid.h"
+
+#include <mpim_grid.h>
 
 static int
 gentpngrid(int gridID1)
@@ -412,7 +412,7 @@ Sethalo(void *process)
 
   const int operatorID = cdoOperatorID();
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
   const int vlistID1 = cdoStreamInqVlist(streamID1);
 
   int ngrids = vlistNgrids(vlistID1);
@@ -465,8 +465,8 @@ Sethalo(void *process)
   std::vector<bool> vars(nvars);
   for (varID = 0; varID < nvars; varID++) vars[varID] = (gridID1 == vlistInqVarGrid(vlistID1, varID));
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   const size_t gridsize = gridInqSize(gridID1);
   std::vector<double> array1(gridsize);
@@ -478,15 +478,15 @@ Sethalo(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           if (vars[varID])
             {
-              pstreamReadRecord(streamID1, array1.data(), &nmiss);
+              cdoReadRecord(streamID1, array1.data(), &nmiss);
 
               if (operatorID == SETHALO)
                 halo(array1.data(), gridID1, array2.data(), lhalo, rhalo);
@@ -499,8 +499,8 @@ Sethalo(void *process)
                   nmiss = arrayNumMV(gridsize2, array2.data(), missval);
                 }
 
-              pstreamDefRecord(streamID2, varID, levelID);
-              pstreamWriteRecord(streamID2, array2.data(), nmiss);
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, array2.data(), nmiss);
             }
         }
 
