@@ -26,7 +26,7 @@
 #include "cdo_int.h"
 #include "param_conversion.h"
 #include "cdo_cmor.h"
-#include "pstream_int.h"
+
 #include "pmlist.h"
 #include "convert_units.h"
 #include "util_files.h"
@@ -53,7 +53,13 @@ paramToString(int param, char *paramstr, int maxlen)
   if (len >= maxlen || len < 0) fprintf(stderr, "Internal problem (%s): size of input string is too small!\n", __func__);
 }
 
-typedef enum { CODE_NUMBER, PARAMETER_ID, VARIABLE_NAME, STANDARD_NAME } pt_mode_t;
+typedef enum
+{
+  CODE_NUMBER,
+  PARAMETER_ID,
+  VARIABLE_NAME,
+  STANDARD_NAME
+} pt_mode_t;
 
 static void
 applyParameterList(pt_mode_t ptmode, PMList &pmlist, int nvars, int vlistID2, std::vector<CmorVar> &vars)
@@ -183,6 +189,8 @@ applyParameterList(pt_mode_t ptmode, PMList &pmlist, int nvars, int vlistID2, st
                 vlistDefVarParam(vlistID2, varID, cdiEncodeParam(parameter2int(value), ptab, 255));
               else if (lv1 && cstrIsEqual(key, "out_code"))
                 vlistDefVarParam(vlistID2, varID, cdiEncodeParam(parameter2int(value), ptab, 255));
+              else if (lv1 && cstrIsEqual(key, "uvRelativeToGrid"))
+                cdiDefKeyInt(vlistID2, varID, CDI_KEY_UVRELATIVETOGRID, parameter2bool(value));
               else if (lv1 && cstrIsEqual(key, "comment"))
                 cdiDefAttTxt(vlistID2, varID, key, (int) strlen(value), value);
               else if (lv1 && cstrIsEqual(key, "chunktype"))
@@ -356,7 +364,7 @@ Setpartab(void *process)
 
   if (Options::cdoVerbose) cdoPrint("Table format version %d", tableformat);
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   int vlistID2 = vlistDuplicate(vlistID1);
@@ -371,10 +379,7 @@ Setpartab(void *process)
 
   if (tableformat == 0)
     {
-      /*
-        for ( int varID = 0; varID < nvars; varID++ )
-          vlistDefVarTable(vlistID2, varID, tableID);
-      */
+      // for ( int varID = 0; varID < nvars; varID++ ) vlistDefVarTable(vlistID2, varID, tableID);
       char name[CDI_MAX_NAME], longname[CDI_MAX_NAME], units[CDI_MAX_NAME];
       for (int varID = 0; varID < nvars; varID++)
         {
@@ -400,7 +405,7 @@ Setpartab(void *process)
         }
     }
   else
-    {      
+    {
       {
         const char *filename = operatorArgv()[0];
         FILE *fp = fopen(filename, "r");
@@ -464,8 +469,8 @@ Setpartab(void *process)
   vlistDefTaxis(vlistID2, taxisID2);
 
   // vlistPrint(vlistID2);
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   size_t gridsizemax = vlistGridsizeMax(vlistID1);
   if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
@@ -475,13 +480,13 @@ Setpartab(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID1)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID1);
+      cdoDefTimestep(streamID2, tsID1);
 
       cmor_check_init(nvars, vars);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           CmorVar &var = vars[varID];
           int varID2 = varID;
@@ -498,15 +503,15 @@ Setpartab(void *process)
                 }
             }
 
-          pstreamDefRecord(streamID2, varID2, levelID2);
+          cdoDefRecord(streamID2, varID2, levelID2);
 
-          pstreamReadRecord(streamID1, array.data(), &nmiss);
+          cdoReadRecord(streamID1, array.data(), &nmiss);
 
           const double missval = vlistInqVarMissval(vlistID2, varID2);
           size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID2));
           if (vlistInqVarNumber(vlistID2, varID2) != CDI_REAL) gridsize *= 2;
 
-          if (nmiss > 0 && var.changemissval)
+          if (nmiss && var.changemissval)
             {
               for (size_t i = 0; i < gridsize; ++i)
                 {
@@ -543,7 +548,7 @@ Setpartab(void *process)
             }
 #endif
 
-          pstreamWriteRecord(streamID2, array.data(), nmiss);
+          cdoWriteRecord(streamID2, array.data(), nmiss);
 
           cmor_check_prep(var, gridsize, missval, array.data());
         }
