@@ -35,7 +35,7 @@
 #include "cdo_int.h"
 #include "param_conversion.h"
 #include "calendar.h"
-#include "pstream_int.h"
+
 #include "util_string.h"
 #include "datetime.h"
 
@@ -378,7 +378,7 @@ Settime(void *process)
     {
       operatorCheckArgc(1);
       char *cname = operatorArgv()[0];
-      strtolower(cname);
+      cstrToLowerCase(cname);
       // clang-format off
       if      (cstrIsEqual(cname, "standard")) newcalendar = CALENDAR_STANDARD;
       else if (cstrIsEqual(cname, "gregorian")) newcalendar = CALENDAR_GREGORIAN;
@@ -399,7 +399,7 @@ Settime(void *process)
       newval = parameter2int(operatorArgv()[0]);
     }
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   int vlistID1 = cdoStreamInqVlist(streamID1);
   int vlistID2 = vlistDuplicate(vlistID1);
@@ -464,10 +464,10 @@ Settime(void *process)
         cdoAbort("This operator does not work with pipes!");
       */
       if (taxisInqType(taxisID1) == TAXIS_ABSOLUTE)
-        {/* 
-           if ( CdoDefault::FileType != CDI_FILETYPE_NC )
-             cdoAbort("This operator does not work on an absolute time axis!");
-          */
+        { /*
+            if ( CdoDefault::FileType != CDI_FILETYPE_NC )
+              cdoAbort("This operator does not work on an absolute time axis!");
+           */
           cdoPrint("Changing absolute to relative time axis!");
           taxisID2 = cdoTaxisCreate(TAXIS_RELATIVE);
         }
@@ -507,7 +507,7 @@ Settime(void *process)
 
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = -1;
+  CdoStreamID streamID2 = CDO_STREAM_UNDEF;
 
   size_t gridsizemax = vlistGridsizeMax(vlistID1);
   if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
@@ -623,21 +623,21 @@ Settime(void *process)
             }
         }
 
-      if (streamID2 == -1)
+      if (streamID2 == CDO_STREAM_UNDEF)
         {
-          streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-          pstreamDefVlist(streamID2, vlistID2);
+          streamID2 = cdoOpenWrite(1);
+          cdoDefVlist(streamID2, vlistID2);
         }
 
-      pstreamDefTimestep(streamID2, tsID1);
+      cdoDefTimestep(streamID2, tsID1);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamDefRecord(streamID2, varID, levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoDefRecord(streamID2, varID, levelID);
 
-          pstreamReadRecord(streamID1, &array[0], &nmiss);
-          pstreamWriteRecord(streamID2, &array[0], nmiss);
+          cdoReadRecord(streamID1, &array[0], &nmiss);
+          cdoWriteRecord(streamID2, &array[0], nmiss);
         }
 
       tsID1++;
