@@ -25,7 +25,7 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
+
 #include "cdo_zaxis.h"
 
 void genLayerBounds(int nlev, double *levels, double *lbounds, double *ubounds);
@@ -97,7 +97,7 @@ Setzaxis(void *process)
         }
     }
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -106,7 +106,7 @@ Setzaxis(void *process)
   const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
+  CdoStreamID streamID2 = cdoOpenWrite(1);
 
   if (operatorID == SETZAXIS)
     {
@@ -152,7 +152,7 @@ Setzaxis(void *process)
         }
     }
 
-  pstreamDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID2, vlistID2);
 
   size_t gridsize = vlistGridsizeMax(vlistID1);
   if (vlistNumber(vlistID1) != CDI_REAL) gridsize *= 2;
@@ -162,15 +162,15 @@ Setzaxis(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamDefRecord(streamID2, varID, levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoDefRecord(streamID2, varID, levelID);
 
-          pstreamReadRecord(streamID1, array.data(), &nmiss);
-          pstreamWriteRecord(streamID2, array.data(), nmiss);
+          cdoReadRecord(streamID1, array.data(), &nmiss);
+          cdoWriteRecord(streamID2, array.data(), nmiss);
         }
 
       tsID++;
