@@ -18,7 +18,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 
 void
 printAtts(int vlistID, int varOrGlobal, int natts, char *argument)
@@ -30,7 +29,7 @@ printAtts(int vlistID, int varOrGlobal, int natts, char *argument)
       vlistInqVarStdname(vlistID, varOrGlobal, stdname);
       vlistInqVarLongname(vlistID, varOrGlobal, longname);
       vlistInqVarUnits(vlistID, varOrGlobal, units);
-      double misval = vlistInqVarMissval(vlistID, varOrGlobal);
+      const double missval = vlistInqVarMissval(vlistID, varOrGlobal);
       if (argument)
         {
           if (cstrIsEqual("standard_name", argument) && stdname[0])
@@ -50,7 +49,7 @@ printAtts(int vlistID, int varOrGlobal, int natts, char *argument)
             }
           else if (cstrIsEqual("missing_value", argument))
             {
-              fprintf(stdout, "  missing_value = \"%e\"\n", misval);
+              fprintf(stdout, "  missing_value = \"%e\"\n", missval);
               found++;
             }
         }
@@ -59,9 +58,10 @@ printAtts(int vlistID, int varOrGlobal, int natts, char *argument)
           if (stdname[0]) fprintf(stdout, "  standard_name = \"%s\"\n", stdname);
           if (longname[0]) fprintf(stdout, "  long_name = \"%s\"\n", longname);
           if (units[0]) fprintf(stdout, "  units = \"%s\"\n", units);
-          fprintf(stdout, "  missing_value = \"%e\"\n", misval);
+          fprintf(stdout, "  missing_value = \"%e\"\n", missval);
         }
     }
+
   if (found == 0)
     {
       for (int i = 0; i < natts; i++)
@@ -70,6 +70,7 @@ printAtts(int vlistID, int varOrGlobal, int natts, char *argument)
           char *value = (char *) Malloc(4 * CDI_MAX_NAME * sizeof(char));
           int type, len;
           cdiInqAtt(vlistID, varOrGlobal, i, name, &type, &len);
+
           if (argument)
             {
               if (!cstrIsEqual(argument, name))
@@ -77,6 +78,7 @@ printAtts(int vlistID, int varOrGlobal, int natts, char *argument)
               else
                 found++;
             }
+
           switch (type)
             {
             case CDI_DATATYPE_TXT:
@@ -88,15 +90,18 @@ printAtts(int vlistID, int varOrGlobal, int natts, char *argument)
               cdiInqAttInt(vlistID, varOrGlobal, name, len, (int *) value);
               fprintf(stdout, "  %s = %i\n", name, *(int *) value);
               break;
+            case CDI_DATATYPE_FLT32:
             case CDI_DATATYPE_FLT64:
               cdiInqAttFlt(vlistID, varOrGlobal, name, len, (double *) value);
-              fprintf(stdout, "  %s = %e\n", name, *(double *) value);
+              fprintf(stdout, "  %s = %g\n", name, *(double *) value);
               break;
-            default: cdoWarning("Unsupported type %i name %s\n", type, name);
+            default: cdoWarning("Unsupported type %i name %s", type, name);
             }
+
           Free(value);
         }
     }
+
   if (argument && found == 0)
     {
       if (varOrGlobal != CDI_GLOBAL)
@@ -137,7 +142,7 @@ Showattribute(void *process)
 
   const int operatorID = cdoOperatorID();
 
-  const int streamID = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID = cdoOpenRead(0);
   const int vlistID = cdoStreamInqVlist(streamID);
   const int nvars = vlistNvars(vlistID);
 
