@@ -35,7 +35,7 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
+
 #include "Showattribute.h"
 #include "printinfo.h"
 
@@ -69,7 +69,7 @@ Showinfo(void *process)
 
   const int operatorID = cdoOperatorID();
 
-  const int streamID = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID = cdoOpenRead(0);
 
   const int vlistID = cdoStreamInqVlist(streamID);
 
