@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -20,33 +20,36 @@
 
       Smoothstat       smooth9             running 9-point-average
 */
-#include <time.h>  // clock()
 
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
-#include "grid.h"
+#include "param_conversion.h"
+#include "cdo_wtime.h"
+#include <mpim_grid.h>
+#include "gridreference.h"
 #include "constants.h"  // planet radius
 #include "pmlist.h"
-#include "cdoOptions.h"
-
-#include "grid_search.h"
+#include "cdo_options.h"
+#include "progress.h"
+#include "cimdOmp.h"
+#include "grid_point_search.h"
 
 enum
 {
   FORM_LINEAR
 };
+
 static const char *Form[] = { "linear" };
 
-typedef struct
+struct smoothpoint_t
 {
   size_t maxpoints;
   int form;
   double radius;
   double weight0;
   double weightR;
-} smoothpoint_t;
+};
 
 static void
 smooth(int gridID, double missval, const double *restrict array1, double *restrict array2, size_t *nmiss, smoothpoint_t spoint)
@@ -60,18 +63,31 @@ smooth(int gridID, double missval, const double *restrict array1, double *restri
   uint8_t *mask = vmask.data();
   for (size_t i = 0; i < gridsize; ++i) mask[i] = !DBL_IS_EQUAL(array1[i], missval);
 
-  std::vector<double> xvals(gridsize);
-  std::vector<double> yvals(gridsize);
+  int gridtype = gridInqType(gridID);
 
-  if (gridInqType(gridID) == GRID_GME) gridID = gridToUnstructured(gridID, 0);
+  if (gridtype == GRID_GME) gridID = gridToUnstructured(gridID, 0);
 
-  if (gridInqType(gridID) != GRID_UNSTRUCTURED && gridInqType(gridID) != GRID_CURVILINEAR)
-    gridID = gridToCurvilinear(gridID, 0);
+  if (gridtype != GRID_UNSTRUCTURED && gridtype != GRID_CURVILINEAR) gridID = gridToCurvilinear(gridID, 0);
 
+  if (gridtype == GRID_UNSTRUCTURED)
+    {
+      if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
+        {
+          if (gridInqNumber(gridID) > 0)
+            {
+              gridID = referenceToGrid(gridID);
+              if (gridID == -1) cdoAbort("Reference to source grid not found!");
+            }
+        }
+    }
+
+  if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0) cdoAbort("Cell center coordinates missing!");
+
+  std::vector<double> xvals(gridsize), yvals(gridsize);
   gridInqXvals(gridID, &xvals[0]);
   gridInqYvals(gridID, &yvals[0]);
 
-  /* Convert lat/lon units if required */
+  // Convert lat/lon units if required
   char units[CDI_MAX_NAME];
   gridInqXunits(gridID, units);
   grid_to_radian(units, gridsize, &xvals[0], "grid center lon");
@@ -81,48 +97,46 @@ smooth(int gridID, double missval, const double *restrict array1, double *restri
   std::vector<knnWeightsType> knnWeights;
   for (int i = 0; i < Threading::ompNumThreads; ++i) knnWeights.push_back(knnWeightsType(numNeighbors));
 
-  clock_t start, finish;
-
-  start = clock();
+  double start = Options::cdoVerbose ? cdo_get_wtime() : 0;
 
   bool xIsCyclic = false;
   size_t dims[2] = { gridsize, 0 };
-  GridSearch *gs = gridsearch_create(xIsCyclic, dims, gridsize, &xvals[0], &yvals[0]);
+  GridPointSearch *gps = gridPointSearchCreate(xIsCyclic, dims, gridsize, &xvals[0], &yvals[0]);
 
-  gs->searchRadius = spoint.radius;
+  gps->searchRadius = spoint.radius;
 
-  finish = clock();
+  if (Options::cdoVerbose) cdoPrint("Point search created: %.2f seconds", cdo_get_wtime() - start);
 
-  if (cdoVerbose) cdoPrint("Point search created: %.2f seconds", ((double) (finish - start)) / CLOCKS_PER_SEC);
+  if (Options::cdoVerbose) Progress::init();
 
-  if (cdoVerbose) progressInit();
-
-  start = clock();
+  start = Options::cdoVerbose ? cdo_get_wtime() : 0;
 
+  size_t naddsMax = 0;
   size_t nmissx = 0;
   double findex = 0;
 
 #ifdef HAVE_OPENMP4
-#pragma omp parallel for schedule(dynamic) default(none) reduction(+ : nmissx) shared(findex, \
-    cdoVerbose, knnWeights, spoint, mask, array1, array2, xvals, yvals, gs, gridsize, missval)
+#pragma omp parallel for schedule(dynamic) default(none) reduction(+ : nmissx) reduction(max : naddsMax) shared( \
+    findex, Options::cdoVerbose, knnWeights, spoint, mask, array1, array2, xvals, yvals, gps, gridsize, missval)
 #endif
   for (size_t i = 0; i < gridsize; ++i)
     {
-      int ompthID = cdo_omp_get_thread_num();
+      const int ompthID = cdo_omp_get_thread_num();
 
 #ifdef _OPENMP
 #pragma omp atomic
 #endif
       findex++;
-      if (cdoVerbose && cdo_omp_get_thread_num() == 0) progressStatus(0, 1, findex / gridsize);
+      if (Options::cdoVerbose && cdo_omp_get_thread_num() == 0) Progress::update(0, 1, findex / gridsize);
 
-      gridSearchPoint(gs, xvals[i], yvals[i], knnWeights[ompthID]);
+      gridSearchPointSmooth(gps, xvals[i], yvals[i], knnWeights[ompthID]);
 
       // Compute weights based on inverse distance if mask is false, eliminate those points
-      size_t nadds = knnWeights[ompthID].compute_weights(mask, spoint.radius, spoint.weight0, spoint.weightR);
+      const size_t nadds = knnWeights[ompthID].computeWeights(mask, spoint.radius, spoint.weight0, spoint.weightR);
+      if (nadds > naddsMax) naddsMax = nadds;
       if (nadds)
         {
-          array2[i] = knnWeights[ompthID].array_weights_sum(array1);
+          array2[i] = knnWeights[ompthID].arrayWeightsSum(array1);
         }
       else
         {
@@ -133,13 +147,12 @@ smooth(int gridID, double missval, const double *restrict array1, double *restri
 
   *nmiss = nmissx;
 
-  progressStatus(0, 1, 1);
+  Progress::update(0, 1, 1);
 
-  finish = clock();
+  if (Options::cdoVerbose) cdoPrint("Point search nearest: %.2f seconds", cdo_get_wtime() - start);
+  if (Options::cdoVerbose) cdoPrint("Max points found: %zu", naddsMax);
 
-  if (cdoVerbose) cdoPrint("Point search nearest: %.2f seconds", ((double) (finish - start)) / CLOCKS_PER_SEC);
-
-  if (gs) gridsearch_delete(gs);
+  if (gps) gridPointSearchDelete(gps);
 
   if (gridID0 != gridID) gridDestroy(gridID);
 }
@@ -240,7 +253,7 @@ smooth9(int gridID, double missval, const double *restrict array1, double *restr
               smooth9_sum(((i + 1) * nlon) + j + 1, mask, 0.3, array1, &avg, &divavg);
             }
 
-          if (fabs(divavg) > 0)
+          if (std::fabs(divavg) > 0)
             {
               array2[i * nlon + j] = avg / divavg;
             }
@@ -256,7 +269,7 @@ smooth9(int gridID, double missval, const double *restrict array1, double *restr
 double
 radius_str_to_deg(const char *string)
 {
-  char *endptr = NULL;
+  char *endptr = nullptr;
   double radius = strtod(string, &endptr);
 
   if (*endptr != 0)
@@ -294,35 +307,33 @@ convert_form(const char *formstr)
 static void
 smoothGetParameter(int *xnsmooth, smoothpoint_t *spoint)
 {
-  int pargc = operatorArgc();
+  const int pargc = operatorArgc();
   if (pargc)
     {
-      char **pargv = operatorArgv();
+      const char **pargv = (const char **) operatorArgv();
 
-      list_t *kvlist = list_new(sizeof(keyValues_t *), free_keyval, "SMOOTH");
-      if (kvlist_parse_cmdline(kvlist, pargc, pargv) != 0) cdoAbort("Parse error!");
-      if (cdoVerbose) kvlist_print(kvlist);
+      KVList kvlist;
+      kvlist.name = "SMOOTH";
+      if (kvlist.parseArguments(pargc, pargv) != 0) cdoAbort("Parse error!");
+      if (Options::cdoVerbose) kvlist.print();
 
-      for (listNode_t *kvnode = kvlist->head; kvnode; kvnode = kvnode->next)
+      for (const auto &kv : kvlist)
         {
-          keyValues_t *kv = *(keyValues_t **) kvnode->data;
-          const char *key = kv->key;
-          if (kv->nvalues > 1) cdoAbort("Too many values for parameter key >%s<!", key);
-          if (kv->nvalues < 1) cdoAbort("Missing value for parameter key >%s<!", key);
-          const char *value = kv->values[0];
+          const char *key = kv.key.c_str();
+          if (kv.nvalues > 1) cdoAbort("Too many values for parameter key >%s<!", key);
+          if (kv.nvalues < 1) cdoAbort("Missing value for parameter key >%s<!", key);
+          const char *value = kv.values[0].c_str();
 
           // clang-format off
-          if      (STR_IS_EQ(key, "nsmooth")) *xnsmooth = parameter2int(value);
-          else if (STR_IS_EQ(key, "maxpoints")) spoint->maxpoints = parameter2sizet(value);
-          else if (STR_IS_EQ(key, "weight0")) spoint->weight0 = parameter2double(value);
-          else if (STR_IS_EQ(key, "weightR")) spoint->weightR = parameter2double(value);
-          else if (STR_IS_EQ(key, "radius")) spoint->radius = radius_str_to_deg(value);
-          else if (STR_IS_EQ(key, "form")) spoint->form = convert_form(value);
+          if      (cstrIsEqual(key, "nsmooth")) *xnsmooth = parameter2int(value);
+          else if (cstrIsEqual(key, "maxpoints")) spoint->maxpoints = parameter2sizet(value);
+          else if (cstrIsEqual(key, "weight0")) spoint->weight0 = parameter2double(value);
+          else if (cstrIsEqual(key, "weightR")) spoint->weightR = parameter2double(value);
+          else if (cstrIsEqual(key, "radius")) spoint->radius = radius_str_to_deg(value);
+          else if (cstrIsEqual(key, "form")) spoint->form = convert_form(value);
           else cdoAbort("Invalid parameter key >%s<!", key);
           // clang-format on
         }
-
-      list_destroy(kvlist);
     }
 }
 
@@ -341,37 +352,38 @@ Smooth(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  int SMOOTH  = cdoOperatorAdd("smooth",   0,   0, NULL);
-  int SMOOTH9 = cdoOperatorAdd("smooth9",  0,   0, NULL);
+  const int SMOOTH  = cdoOperatorAdd("smooth",   0,   0, nullptr);
+  const int SMOOTH9 = cdoOperatorAdd("smooth9",  0,   0, nullptr);
   // clang-format on
 
-  int operatorID = cdoOperatorID();
+  const int operatorID = cdoOperatorID();
 
   if (operatorID == SMOOTH) smoothGetParameter(&xnsmooth, &spoint);
 
   if (spoint.radius < 0 || spoint.radius > 180) cdoAbort("%s=%g out of bounds (0-180 deg)!", "radius", spoint.radius);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int nvars = vlistNvars(vlistID1);
+  const int nvars = vlistNvars(vlistID1);
   std::vector<bool> varIDs(nvars);
 
   for (int varID = 0; varID < nvars; ++varID)
     {
-      int gridID = vlistInqVarGrid(vlistID1, varID);
-      int gridtype = gridInqType(gridID);
-      if (gridtype == GRID_GAUSSIAN || gridtype == GRID_LONLAT || gridtype == GRID_CURVILINEAR)
+      const int gridID = vlistInqVarGrid(vlistID1, varID);
+      const int gridtype = gridInqType(gridID);
+      if (gridtype == GRID_GAUSSIAN || gridtype == GRID_LONLAT || gridtype == GRID_CURVILINEAR
+          || (operatorID == SMOOTH9 && gridtype == GRID_GENERIC && gridInqXsize(gridID) > 0 && gridInqYsize(gridID) > 0))
         {
           varIDs[varID] = true;
         }
-      else if (gridtype == GRID_UNSTRUCTURED && operatorID == SMOOTH)
+      else if (operatorID == SMOOTH && gridtype == GRID_UNSTRUCTURED)
         {
           varIDs[varID] = true;
         }
@@ -386,7 +398,7 @@ Smooth(void *process)
 
   size_t gridsizemax = vlistGridsizeMax(vlistID1);
   if (gridsizemax < spoint.maxpoints) spoint.maxpoints = gridsizemax;
-  if (cdoVerbose)
+  if (Options::cdoVerbose && operatorID == SMOOTH)
     cdoPrint("nsmooth = %d, maxpoints = %zu, radius = %gdeg, form = %s, weight0 = %g, weightR = %g", xnsmooth, spoint.maxpoints,
              spoint.radius, Form[spoint.form], spoint.weight0, spoint.weightR);
 
@@ -395,22 +407,22 @@ Smooth(void *process)
   std::vector<double> array1(gridsizemax);
   std::vector<double> array2(gridsizemax);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1), cdoFiletype());
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
           int varID, levelID;
           size_t nmiss;
 
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, array1.data(), &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, array1.data(), &nmiss);
 
           if (varIDs[varID])
             {
@@ -427,23 +439,23 @@ Smooth(void *process)
                   arrayCopy(gridsizemax, array2.data(), array1.data());
                 }
 
-              pstreamDefRecord(streamID2, varID, levelID);
-              pstreamWriteRecord(streamID2, array2.data(), nmiss);
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, array2.data(), nmiss);
             }
           else
             {
-              pstreamDefRecord(streamID2, varID, levelID);
-              pstreamWriteRecord(streamID2, array1.data(), nmiss);
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, array1.data(), nmiss);
             }
         }
 
       tsID++;
     }
 
-  pstreamClose(streamID2);
-  pstreamClose(streamID1);
+  cdoStreamClose(streamID2);
+  cdoStreamClose(streamID1);
 
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
