@@ -25,7 +25,6 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
 
 struct levinfo_t
 {
@@ -152,7 +151,7 @@ Sort(void *process)
       if (iarg < 0) cmpvarlev = cmpvarlevelrev;
     }
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -169,8 +168,8 @@ Sort(void *process)
       ;
   */
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   const int nvars = vlistNvars(vlistID1);
 
@@ -195,11 +194,11 @@ Sort(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           if (tsID == 0)
             {
@@ -217,7 +216,7 @@ Sort(void *process)
           offset = gridsize * levelID;
           single = vardata[varID] + offset;
 
-          pstreamReadRecord(streamID1, single, &nmiss);
+          cdoReadRecord(streamID1, single, &nmiss);
 
           setNmiss(varID, levelID, nvars, varInfo, nmiss);
           // varInfo[varID].levInfo[levelID].nmiss = nmiss;
@@ -274,8 +273,8 @@ Sort(void *process)
                   offset = gridsize * levelID;
                   single = vardata[varID] + offset;
 
-                  pstreamDefRecord(streamID2, varID, levelID);
-                  pstreamWriteRecord(streamID2, single, nmiss);
+                  cdoDefRecord(streamID2, varID, levelID);
+                  cdoWriteRecord(streamID2, single, nmiss);
                 }
             }
         }
