@@ -24,7 +24,7 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
+
 #include "printinfo.h"
 #include "timebase.h"
 
@@ -66,7 +66,7 @@ Sorttimestamp(void *process)
   int xtsID = 0;
   for (int fileID = 0; fileID < nfiles; fileID++)
     {
-      const int streamID1 = cdoStreamOpenRead(cdoStreamName(fileID));
+      CdoStreamID streamID1 = cdoOpenRead((fileID));
 
       const int vlistID1 = cdoStreamInqVlist(streamID1);
       const int taxisID1 = vlistInqTaxis(vlistID1);
@@ -106,11 +106,11 @@ Sorttimestamp(void *process)
 
           for (int recID = 0; recID < nrecs; recID++)
             {
-              pstreamInqRecord(streamID1, &varID, &levelID);
+              cdoInqRecord(streamID1, &varID, &levelID);
               gridID = vlistInqVarGrid(vlistID1, varID);
               const size_t gridsize = gridInqSize(gridID);
-              vars[xtsID][varID][levelID].ptr = (double *) Malloc(gridsize * sizeof(double));
-              pstreamReadRecord(streamID1, vars[xtsID][varID][levelID].ptr, &nmiss);
+              vars[xtsID][varID][levelID].resize(gridsize);
+              cdoReadRecord(streamID1, vars[xtsID][varID][levelID].vec.data(), &nmiss);
               vars[xtsID][varID][levelID].nmiss = nmiss;
             }
 
@@ -139,8 +139,8 @@ Sorttimestamp(void *process)
 
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(nfiles));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite((nfiles));
+  cdoDefVlist(streamID2, vlistID2);
 
   int tsID2 = 0;
   for (tsID = 0; tsID < nts; tsID++)
@@ -166,23 +166,21 @@ Sorttimestamp(void *process)
 
       taxisDefVdate(taxisID2, vdate[xtsID]);
       taxisDefVtime(taxisID2, vtime[xtsID]);
-      pstreamDefTimestep(streamID2, tsID2++);
+      cdoDefTimestep(streamID2, tsID2++);
 
       for (varID = 0; varID < nvars; varID++)
         {
           const int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
           for (levelID = 0; levelID < nlevel; levelID++)
             {
-              if (vars[xtsID][varID][levelID].ptr)
+              if (!vars[xtsID][varID][levelID].empty())
                 {
                   nmiss = vars[xtsID][varID][levelID].nmiss;
-                  pstreamDefRecord(streamID2, varID, levelID);
-                  pstreamWriteRecord(streamID2, vars[xtsID][varID][levelID].ptr, nmiss);
+                  cdoDefRecord(streamID2, varID, levelID);
+                  cdoWriteRecord(streamID2, vars[xtsID][varID][levelID].vec.data(), nmiss);
                 }
             }
         }
-
-      fieldsFree(vlistID2, vars[xtsID]);
     }
 
   cdoStreamClose(streamID2);
