@@ -22,7 +22,7 @@
 */
 
 #include "cdo_int.h"
-#include "grid.h"
+#include <mpim_grid.h>
 
 #define NTR2NSP(ntr) ((ntr + 1) * (ntr + 2))
 #define NSP2NTR(nsp) ((int) ((((sqrt((double) (4 * nsp + 1))) - 3) / 2)))
