@@ -30,8 +30,8 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "grid.h"
-#include "pstream_int.h"
+#include <mpim_grid.h>
+#include "griddes.h"
 #include "specspace.h"
 #include "listarray.h"
 
@@ -44,12 +44,12 @@ Spectral(void *process)
   int gridID1 = -1, gridID2 = -1;
   size_t nmiss;
   std::vector<int> waves;
-  SPTRANS *sptrans = nullptr;
+  SPTRANS sptrans;
   ListArray<int> listArrayInt;
 
   cdoInitialize(process);
 
-  const bool lcopy = UNCHANGED_RECORD;
+  const bool lcopy = unchangedRecord();
 
   // clang-format off
   const int GP2SP  = cdoOperatorAdd("gp2sp",  0, 0, nullptr);
@@ -78,7 +78,7 @@ Spectral(void *process)
     }
   // clang-format on
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -141,7 +141,7 @@ Spectral(void *process)
           gridID2 = gridIDsp;
 
           ntr = gridInqTrunc(gridID2);
-          sptrans = sptrans_new(nlon, nlat, ntr, 0);
+          sptrans.init(nlon, nlat, ntr, 0);
         }
     }
   else if (lsp2gp)
@@ -171,7 +171,7 @@ Spectral(void *process)
           const long ntr = gridInqTrunc(gridID1);
           const long nlon = gridInqXsize(gridID2);
           const long nlat = gridInqYsize(gridID2);
-          sptrans = sptrans_new(nlon, nlat, ntr, 0);
+          sptrans.init(nlon, nlat, ntr, 0);
         }
     }
   else if (operatorID == SP2SP)
@@ -224,8 +224,8 @@ Spectral(void *process)
 
   if (gridID1 != -1) vlistChangeGrid(vlistID2, gridID1, gridID2);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   const size_t gridsizemax = vlistGridsizeMax(vlistID1);
   std::vector<double> array1(gridsizemax);
@@ -236,15 +236,15 @@ Spectral(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           if (vars[varID])
             {
-              pstreamReadRecord(streamID1, array1.data(), &nmiss);
+              cdoReadRecord(streamID1, array1.data(), &nmiss);
               if (nmiss) cdoAbort("Missing values unsupported for spectral data!");
 
               gridID1 = vlistInqVarGrid(vlistID1, varID);
@@ -257,20 +257,20 @@ Spectral(void *process)
               else if (operatorID == SPCUT)
                 speccut(gridID1, array1.data(), array2.data(), waves.data());
 
-              pstreamDefRecord(streamID2, varID, levelID);
-              pstreamWriteRecord(streamID2, array2.data(), nmiss);
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, array2.data(), nmiss);
             }
           else
             {
-              pstreamDefRecord(streamID2, varID, levelID);
+              cdoDefRecord(streamID2, varID, levelID);
               if (lcopy)
                 {
-                  pstreamCopyRecord(streamID2, streamID1);
+                  cdoCopyRecord(streamID2, streamID1);
                 }
               else
                 {
-                  pstreamReadRecord(streamID1, array1.data(), &nmiss);
-                  pstreamWriteRecord(streamID2, array1.data(), nmiss);
+                  cdoReadRecord(streamID1, array1.data(), &nmiss);
+                  cdoWriteRecord(streamID2, array1.data(), nmiss);
                 }
             }
         }
@@ -281,8 +281,6 @@ Spectral(void *process)
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
-  sptrans_delete(sptrans);
-
   cdoFinish();
 
   return nullptr;
