@@ -30,7 +30,7 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
+
 #include "cdo_history.h"
 #include "util_files.h"
 
@@ -57,13 +57,14 @@ Split(void *process)
   size_t nmiss;
   bool swap_obase = false;
   const char *uuid_attribute = nullptr;
-  std::vector<int> vlistIDs, streamIDs;
+  std::vector<int> vlistIDs;
+  std::vector<CdoStreamID> streamIDs;
 
   cdoInitialize(process);
 
   if (processSelf().m_ID != 0) cdoAbort("This operator can't be combined with other operators!");
 
-  const bool lcopy = UNCHANGED_RECORD;
+  const bool lcopy = unchangedRecord();
 
   // clang-format off
   const int SPLITCODE   = cdoOperatorAdd("splitcode",   0, 0, nullptr);
@@ -87,7 +88,7 @@ Split(void *process)
         cdoAbort("Unknown parameter: >%s<", operatorArgv()[0]);
     }
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
   const int vlistID1 = cdoStreamInqVlist(streamID1);
 
   const int nvars = vlistNvars(vlistID1);
@@ -100,7 +101,7 @@ Split(void *process)
 
   const char *refname = cdoGetObase();
   filesuffix[0] = 0;
-  cdoGenFileSuffix(filesuffix, sizeof(filesuffix), pstreamInqFiletype(streamID1), vlistID1, refname);
+  cdoGenFileSuffix(filesuffix, sizeof(filesuffix), cdoInqFiletype(streamID1), vlistID1, refname);
 
   if (operatorID == SPLITCODE)
     {
@@ -161,7 +162,7 @@ Split(void *process)
               gen_filename(filename, swap_obase, cdoGetObase(), filesuffix);
             }
 
-          streamIDs[index] = cdoStreamOpenWrite(filename);
+          streamIDs[index] = cdoOpenWrite(filename);
         }
     }
   else if (operatorID == SPLITPARAM)
@@ -214,7 +215,7 @@ Split(void *process)
           strcat(filename, paramstr);
           gen_filename(filename, swap_obase, cdoGetObase(), filesuffix);
 
-          streamIDs[index] = cdoStreamOpenWrite(filename);
+          streamIDs[index] = cdoOpenWrite(filename);
         }
     }
   else if (operatorID == SPLITTABNUM)
@@ -262,7 +263,7 @@ Split(void *process)
           sprintf(filename + nchars, "%03d", tabnums[index]);
           gen_filename(filename, swap_obase, cdoGetObase(), filesuffix);
 
-          streamIDs[index] = cdoStreamOpenWrite(filename);
+          streamIDs[index] = cdoOpenWrite(filename);
         }
     }
   else if (operatorID == SPLITNAME)
@@ -293,7 +294,7 @@ Split(void *process)
           strcat(filename, varname);
           gen_filename(filename, swap_obase, cdoGetObase(), filesuffix);
 
-          streamIDs[index] = cdoStreamOpenWrite(filename);
+          streamIDs[index] = cdoOpenWrite(filename);
         }
     }
   else if (operatorID == SPLITLEVEL)
@@ -344,7 +345,7 @@ Split(void *process)
           sprintf(filename + nchars, "%06g", levels[index]);
           gen_filename(filename, swap_obase, cdoGetObase(), filesuffix);
 
-          streamIDs[index] = cdoStreamOpenWrite(filename);
+          streamIDs[index] = cdoOpenWrite(filename);
         }
     }
   else if (operatorID == SPLITGRID)
@@ -381,7 +382,7 @@ Split(void *process)
           sprintf(filename + nchars, "%02d", vlistGridIndex(vlistID1, gridIDs[index]) + 1);
           gen_filename(filename, swap_obase, cdoGetObase(), filesuffix);
 
-          streamIDs[index] = cdoStreamOpenWrite(filename);
+          streamIDs[index] = cdoOpenWrite(filename);
         }
     }
   else if (operatorID == SPLITZAXIS)
@@ -416,7 +417,7 @@ Split(void *process)
           sprintf(filename + nchars, "%02d", vlistZaxisIndex(vlistID1, zaxisIDs[index]) + 1);
           gen_filename(filename, swap_obase, cdoGetObase(), filesuffix);
 
-          streamIDs[index] = cdoStreamOpenWrite(filename);
+          streamIDs[index] = cdoOpenWrite(filename);
         }
     }
   else
@@ -428,7 +429,7 @@ Split(void *process)
     {
       if (uuid_attribute) cdo_def_tracking_id(vlistIDs[index], uuid_attribute);
 
-      pstreamDefVlist(streamIDs[index], vlistIDs[index]);
+      cdoDefVlist(streamIDs[index], vlistIDs[index]);
     }
 
   std::vector<double> array;
@@ -443,11 +444,11 @@ Split(void *process)
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
-      for (int index = 0; index < nsplit; index++) pstreamDefTimestep(streamIDs[index], tsID);
+      for (int index = 0; index < nsplit; index++) cdoDefTimestep(streamIDs[index], tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           const int index = vlistInqIndex(vlistID1, varID, levelID);
           vlistID2 = vlistIDs[index];
@@ -457,15 +458,15 @@ Split(void *process)
             printf("%d %d %d %d %d %d\n", index, vlistID2, varID, levelID,
             varID2, levelID2);
           */
-          pstreamDefRecord(streamIDs[index], varID2, levelID2);
+          cdoDefRecord(streamIDs[index], varID2, levelID2);
           if (lcopy)
             {
-              pstreamCopyRecord(streamIDs[index], streamID1);
+              cdoCopyRecord(streamIDs[index], streamID1);
             }
           else
             {
-              pstreamReadRecord(streamID1, array.data(), &nmiss);
-              pstreamWriteRecord(streamIDs[index], array.data(), nmiss);
+              cdoReadRecord(streamID1, array.data(), &nmiss);
+              cdoWriteRecord(streamIDs[index], array.data(), nmiss);
             }
         }
 
