@@ -24,7 +24,7 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
+
 #include "util_files.h"
 
 void *
@@ -40,9 +40,9 @@ Splitrec(void *process)
 
   if (processSelf().m_ID != 0) cdoAbort("This operator can't be combined with other operators!");
 
-  const bool lcopy = UNCHANGED_RECORD;
+  const bool lcopy = unchangedRecord();
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
   const int vlistID1 = cdoStreamInqVlist(streamID1);
 
   int nrecs = vlistNrecs(vlistID1);
@@ -52,7 +52,7 @@ Splitrec(void *process)
 
   const char *refname = cdoGetObase();
   filesuffix[0] = 0;
-  cdoGenFileSuffix(filesuffix, sizeof(filesuffix), pstreamInqFiletype(streamID1), vlistID1, refname);
+  cdoGenFileSuffix(filesuffix, sizeof(filesuffix), cdoInqFiletype(streamID1), vlistID1, refname);
 
   std::vector<double> array;
   if (!lcopy)
@@ -68,7 +68,7 @@ Splitrec(void *process)
     {
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           vlistClearFlag(vlistID1);
           vlistDefFlag(vlistID1, varID, levelID, true);
@@ -82,23 +82,23 @@ Splitrec(void *process)
 
           if (Options::cdoVerbose) cdoPrint("create file %s", filename);
 
-          const int streamID2 = cdoStreamOpenWrite(filename);
+          CdoStreamID streamID2 = cdoOpenWrite(filename);
 
-          pstreamDefVlist(streamID2, vlistID2);
+          cdoDefVlist(streamID2, vlistID2);
 
           const int varID2 = vlistFindVar(vlistID2, varID);
           const int levelID2 = vlistFindLevel(vlistID2, varID, levelID);
 
-          pstreamDefTimestep(streamID2, 0);
-          pstreamDefRecord(streamID2, varID2, levelID2);
+          cdoDefTimestep(streamID2, 0);
+          cdoDefRecord(streamID2, varID2, levelID2);
           if (lcopy)
             {
-              pstreamCopyRecord(streamID2, streamID1);
+              cdoCopyRecord(streamID2, streamID1);
             }
           else
             {
-              pstreamReadRecord(streamID1, array.data(), &nmiss);
-              pstreamWriteRecord(streamID2, array.data(), nmiss);
+              cdoReadRecord(streamID1, array.data(), &nmiss);
+              cdoWriteRecord(streamID2, array.data(), nmiss);
             }
 
           cdoStreamClose(streamID2);
