@@ -25,7 +25,6 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
 #include "util_files.h"
 
 void *
@@ -36,15 +35,12 @@ Splitsel(void *process)
   int tsID;
   size_t nmiss;
   int i2 = 0;
-  int nchars;
   char filesuffix[32];
   char filename[8192];
 
   cdoInitialize(process);
 
-  if (processSelf().m_ID != 0) cdoAbort("This operator can't be combined with other operators!");
-
-  const bool lcopy = UNCHANGED_RECORD;
+  const bool lcopy = unchangedRecord();
 
   cdoOperatorAdd("splitsel", 0, 0, nullptr);
 
@@ -57,14 +53,13 @@ Splitsel(void *process)
   /*   if ( nargc > 1 ) noffset = parameter2int(operatorArgv()[1]); */
   /*   if ( nargc > 2 ) nskip   = parameter2int(operatorArgv()[2]); */
   /*   printf("%s %s %s\n", operatorArgv()[0],operatorArgv()[1],operatorArgv()[2]); */
-  double noffset = 0.0, nskip = 0.0;
   const double ndates = parameter2double(operatorArgv()[0]);
-  if (nargc > 1) noffset = parameter2double(operatorArgv()[1]);
-  if (nargc > 2) nskip = parameter2double(operatorArgv()[2]);
+  const double noffset = (nargc > 1) ? parameter2double(operatorArgv()[1]) : 0.0;
+  const double nskip = (nargc > 2) ? parameter2double(operatorArgv()[2]) : 0.0;
 
   if (Options::cdoVerbose) cdoPrint("nsets = %f, noffset = %f, nskip = %f", ndates, noffset, nskip);
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -75,11 +70,11 @@ Splitsel(void *process)
   vlistDefTaxis(vlistID2, taxisID2);
 
   strcpy(filename, cdoGetObase());
-  nchars = strlen(filename);
+  const int nchars = strlen(filename);
 
   const char *refname = cdoGetObase();
   filesuffix[0] = 0;
-  cdoGenFileSuffix(filesuffix, sizeof(filesuffix), pstreamInqFiletype(streamID1), vlistID1, refname);
+  cdoGenFileSuffix(filesuffix, sizeof(filesuffix), cdoInqFiletype(streamID1), vlistID1, refname);
 
   std::vector<double> array;
   //  if ( ! lcopy )
@@ -94,7 +89,7 @@ Splitsel(void *process)
   for (varID = 0; varID < nvars; varID++)
     if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) nconst++;
 
-  std::vector<std::vector<Field>> vars;
+  FieldVector2D vars;
   if (nconst)
     {
       vars.resize(nvars);
@@ -111,9 +106,8 @@ Splitsel(void *process)
 
               for (levelID = 0; levelID < nlevel; levelID++)
                 {
-                  fieldInit(vars[varID][levelID]);
                   vars[varID][levelID].grid = gridID;
-                  vars[varID][levelID].ptr = (double *) Malloc(gridsize * sizeof(double));
+                  vars[varID][levelID].resize(gridsize);
                 }
             }
         }
@@ -135,10 +129,10 @@ Splitsel(void *process)
       if (tsID == 0 && nconst)
         for (int recID = 0; recID < nrecs; recID++)
           {
-            pstreamInqRecord(streamID1, &varID, &levelID);
+            cdoInqRecord(streamID1, &varID, &levelID);
             if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT)
               {
-                pstreamReadRecord(streamID1, vars[varID][levelID].ptr, &nmiss);
+                cdoReadRecord(streamID1, vars[varID][levelID].vec.data(), &nmiss);
                 vars[varID][levelID].nmiss = nmiss;
               }
           }
@@ -151,7 +145,7 @@ Splitsel(void *process)
 
       if (Options::cdoVerbose) cdoPrint("create file %s", filename);
 
-      int streamID2 = -1;
+      CdoStreamID streamID2 = CDO_STREAM_UNDEF;
       int tsID2 = 0;
 
       for (; nsets < (int) (ndates * (index + 1)); nsets++)
@@ -161,13 +155,13 @@ Splitsel(void *process)
 
           taxisCopyTimestep(taxisID2, taxisID1);
 
-          if (streamID2 < 0)
+          if (streamID2 == CDO_STREAM_UNDEF)
             {
-              streamID2 = cdoStreamOpenWrite(filename);
-              pstreamDefVlist(streamID2, vlistID2);
+              streamID2 = cdoOpenWrite(filename);
+              cdoDefVlist(streamID2, vlistID2);
             }
 
-          pstreamDefTimestep(streamID2, tsID2);
+          cdoDefTimestep(streamID2, tsID2);
 
           if (tsID > 0 && tsID2 == 0 && nconst)
             {
@@ -175,12 +169,12 @@ Splitsel(void *process)
                 {
                   if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT)
                     {
-                      int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-                      for (levelID = 0; levelID < nlevel; levelID++)
+                      const int nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+                      for (levelID = 0; levelID < nlevels; levelID++)
                         {
-                          pstreamDefRecord(streamID2, varID, levelID);
+                          cdoDefRecord(streamID2, varID, levelID);
                           nmiss = vars[varID][levelID].nmiss;
-                          pstreamWriteRecord(streamID2, vars[varID][levelID].ptr, nmiss);
+                          cdoWriteRecord(streamID2, vars[varID][levelID].vec.data(), nmiss);
                         }
                     }
                 }
@@ -189,16 +183,16 @@ Splitsel(void *process)
           for (int recID = 0; recID < nrecs; recID++)
             {
 
-              pstreamInqRecord(streamID1, &varID, &levelID);
-              pstreamDefRecord(streamID2, varID, levelID);
+              cdoInqRecord(streamID1, &varID, &levelID);
+              cdoDefRecord(streamID2, varID, levelID);
               if (lcopy && !(tsID == 0 && nconst))
                 {
-                  pstreamCopyRecord(streamID2, streamID1);
+                  cdoCopyRecord(streamID2, streamID1);
                 }
               else
                 {
-                  pstreamReadRecord(streamID1, array.data(), &nmiss);
-                  pstreamWriteRecord(streamID2, array.data(), nmiss);
+                  cdoReadRecord(streamID1, array.data(), &nmiss);
+                  cdoWriteRecord(streamID2, array.data(), nmiss);
 
                   if (tsID == 0 && nconst)
                     {
@@ -206,7 +200,7 @@ Splitsel(void *process)
                         {
                           int gridID = vlistInqVarGrid(vlistID1, varID);
                           size_t gridsize = gridInqSize(gridID);
-                          arrayCopy(gridsize, array.data(), vars[varID][levelID].ptr);
+                          arrayCopy(gridsize, array.data(), vars[varID][levelID].vec.data());
                           vars[varID][levelID].nmiss = nmiss;
                         }
                     }
@@ -240,19 +234,6 @@ LABEL_END:
 
   cdoStreamClose(streamID1);
 
-  if (nconst)
-    {
-      for (varID = 0; varID < nvars; varID++)
-        {
-          if (vlistInqVarTimetype(vlistID2, varID) == TIME_CONSTANT)
-            {
-              int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
-              for (levelID = 0; levelID < nlevel; levelID++)
-                if (vars[varID][levelID].ptr) Free(vars[varID][levelID].ptr);
-            }
-        }
-    }
-
   vlistDestroy(vlistID2);
 
   cdoFinish();
