@@ -24,12 +24,10 @@
       Splittime  splitseas       Split seasons
 */
 
+#include <time.h>
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
-
-#include <time.h>
 #include "cdo_season.h"
 #include "util_files.h"
 
@@ -58,11 +56,12 @@ datetime_to_tm(int64_t date, int time)
 void *
 Splittime(void *process)
 {
-  int streamID2;
+  CdoStreamID streamID2;
   int varID;
   int nrecs;
   int levelID;
-  int streamIDs[MAX_STREAMS], tsIDs[MAX_STREAMS];
+  CdoStreamID streamIDs[MAX_STREAMS];
+  int tsIDs[MAX_STREAMS];
   int index = 0;
   size_t nmiss;
   char filesuffix[32];
@@ -70,9 +69,7 @@ Splittime(void *process)
 
   cdoInitialize(process);
 
-  if (processSelf().m_ID != 0) cdoAbort("This operator can't be combined with other operators!");
-
-  const bool lcopy = UNCHANGED_RECORD;
+  const bool lcopy = unchangedRecord();
 
   // clang-format off
   const int SPLITHOUR = cdoOperatorAdd("splithour", func_time, 10000, nullptr);
@@ -94,10 +91,10 @@ Splittime(void *process)
   const char *seas_name[4];
   get_season_name(seas_name);
 
-  for (int i = 0; i < MAX_STREAMS; i++) streamIDs[i] = -1;
+  for (int i = 0; i < MAX_STREAMS; i++) streamIDs[i] = CDO_STREAM_UNDEF;
   for (int i = 0; i < MAX_STREAMS; i++) tsIDs[i] = 0;
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -111,7 +108,7 @@ Splittime(void *process)
 
   const char *refname = cdoGetObase();
   filesuffix[0] = 0;
-  cdoGenFileSuffix(filesuffix, sizeof(filesuffix), pstreamInqFiletype(streamID1), vlistID1, refname);
+  cdoGenFileSuffix(filesuffix, sizeof(filesuffix), cdoInqFiletype(streamID1), vlistID1, refname);
 
   std::vector<double> array;
   //  if ( ! lcopy )
@@ -126,7 +123,7 @@ Splittime(void *process)
   for (varID = 0; varID < nvars; varID++)
     if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) nconst++;
 
-  std::vector<std::vector<Field>> vars;
+  FieldVector2D vars;
   if (nconst)
     {
       vars.resize(nvars);
@@ -143,9 +140,8 @@ Splittime(void *process)
 
               for (levelID = 0; levelID < nlevel; levelID++)
                 {
-                  fieldInit(vars[varID][levelID]);
                   vars[varID][levelID].grid = gridID;
-                  vars[varID][levelID].ptr = (double *) Malloc(gridsize * sizeof(double));
+                  vars[varID][levelID].resize(gridsize);
                 }
             }
         }
@@ -173,7 +169,7 @@ Splittime(void *process)
       if (index < 0 || index >= MAX_STREAMS) cdoAbort("Index out of range!");
 
       streamID2 = streamIDs[index];
-      if (streamID2 < 0)
+      if (streamID2 == CDO_STREAM_UNDEF)
         {
           if (operatorID == SPLITSEAS)
             {
@@ -201,12 +197,12 @@ Splittime(void *process)
 
           if (Options::cdoVerbose) cdoPrint("create file %s", filename);
 
-          streamID2 = cdoStreamOpenWrite(filename);
-          pstreamDefVlist(streamID2, vlistID2);
+          streamID2 = cdoOpenWrite(filename);
+          cdoDefVlist(streamID2, vlistID2);
           streamIDs[index] = streamID2;
         }
 
-      pstreamDefTimestep(streamID2, tsIDs[index]);
+      cdoDefTimestep(streamID2, tsIDs[index]);
 
       if (tsID > 0 && tsIDs[index] == 0 && nconst)
         {
@@ -214,12 +210,12 @@ Splittime(void *process)
             {
               if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT)
                 {
-                  const int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-                  for (levelID = 0; levelID < nlevel; levelID++)
+                  const int nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+                  for (levelID = 0; levelID < nlevels; levelID++)
                     {
-                      pstreamDefRecord(streamID2, varID, levelID);
+                      cdoDefRecord(streamID2, varID, levelID);
                       nmiss = vars[varID][levelID].nmiss;
-                      pstreamWriteRecord(streamID2, vars[varID][levelID].ptr, nmiss);
+                      cdoWriteRecord(streamID2, vars[varID][levelID].vec.data(), nmiss);
                     }
                 }
             }
@@ -227,17 +223,17 @@ Splittime(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamDefRecord(streamID2, varID, levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoDefRecord(streamID2, varID, levelID);
 
           if (lcopy && !(tsID == 0 && nconst))
             {
-              pstreamCopyRecord(streamID2, streamID1);
+              cdoCopyRecord(streamID2, streamID1);
             }
           else
             {
-              pstreamReadRecord(streamID1, array.data(), &nmiss);
-              pstreamWriteRecord(streamID2, array.data(), nmiss);
+              cdoReadRecord(streamID1, array.data(), &nmiss);
+              cdoWriteRecord(streamID2, array.data(), nmiss);
 
               if (tsID == 0 && nconst)
                 {
@@ -245,7 +241,7 @@ Splittime(void *process)
                     {
                       const int gridID = vlistInqVarGrid(vlistID1, varID);
                       const size_t gridsize = gridInqSize(gridID);
-                      arrayCopy(gridsize, array.data(), vars[varID][levelID].ptr);
+                      arrayCopy(gridsize, array.data(), vars[varID][levelID].vec.data());
                       vars[varID][levelID].nmiss = nmiss;
                     }
                 }
@@ -258,22 +254,9 @@ Splittime(void *process)
 
   cdoStreamClose(streamID1);
 
-  if (nconst)
-    {
-      for (varID = 0; varID < nvars; varID++)
-        {
-          if (vlistInqVarTimetype(vlistID2, varID) == TIME_CONSTANT)
-            {
-              const int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
-              for (levelID = 0; levelID < nlevel; levelID++)
-                if (vars[varID][levelID].ptr) Free(vars[varID][levelID].ptr);
-            }
-        }
-    }
-
   for (index = 0; index < MAX_STREAMS; index++)
     {
-      if (streamIDs[index] >= 0) cdoStreamClose(streamIDs[index]);
+      if (streamIDs[index] != CDO_STREAM_UNDEF) cdoStreamClose(streamIDs[index]);
     }
 
   vlistDestroy(vlistID2);
