@@ -27,7 +27,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 #include "util_files.h"
 
 #define MAX_YEARS 99999
@@ -35,7 +34,7 @@
 void *
 Splityear(void *process)
 {
-  int streamID2 = -1;
+  CdoStreamID streamID2 = CDO_STREAM_UNDEF;
   int varID;
   int nrecs;
   int levelID;
@@ -50,7 +49,7 @@ Splityear(void *process)
 
   if (processSelf().m_ID != 0) cdoAbort("This operator can't be combined with other operators!");
 
-  const bool lcopy = UNCHANGED_RECORD;
+  const bool lcopy = unchangedRecord();
 
   // clang-format off
   const int SPLITYEAR    = cdoOperatorAdd("splityear",     0, 10000, nullptr);
@@ -62,7 +61,7 @@ Splityear(void *process)
 
   arrayFill(MAX_YEARS, cyear, 0);
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -76,7 +75,7 @@ Splityear(void *process)
 
   const char *refname = cdoGetObase();
   filesuffix[0] = 0;
-  cdoGenFileSuffix(filesuffix, sizeof(filesuffix), pstreamInqFiletype(streamID1), vlistID1, refname);
+  cdoGenFileSuffix(filesuffix, sizeof(filesuffix), cdoInqFiletype(streamID1), vlistID1, refname);
 
   std::vector<double> array;
   // if ( ! lcopy )
@@ -91,7 +90,7 @@ Splityear(void *process)
   for (varID = 0; varID < nvars; varID++)
     if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) nconst++;
 
-  std::vector<std::vector<Field>> vars;
+  FieldVector2D vars;
   if (nconst)
     {
       vars.resize(nvars);
@@ -108,9 +107,8 @@ Splityear(void *process)
 
               for (levelID = 0; levelID < nlevel; levelID++)
                 {
-                  fieldInit(vars[varID][levelID]);
                   vars[varID][levelID].grid = gridID;
-                  vars[varID][levelID].ptr = (double *) Malloc(gridsize * sizeof(double));
+                  vars[varID][levelID].resize(gridsize);
                 }
             }
         }
@@ -144,7 +142,7 @@ Splityear(void *process)
 
               year1 = year2;
 
-              if (streamID2 >= 0) cdoStreamClose(streamID2);
+              if (streamID2 != CDO_STREAM_UNDEF) cdoStreamClose(streamID2);
 
               sprintf(filename + nchars, "%04d", year1);
               if (ic > 0) sprintf(filename + strlen(filename), "_%d", ic + 1);
@@ -152,8 +150,8 @@ Splityear(void *process)
 
               if (Options::cdoVerbose) cdoPrint("create file %s", filename);
 
-              streamID2 = cdoStreamOpenWrite(filename);
-              pstreamDefVlist(streamID2, vlistID2);
+              streamID2 = cdoOpenWrite(filename);
+              cdoDefVlist(streamID2, vlistID2);
             }
           mon1 = mon2;
         }
@@ -167,7 +165,7 @@ Splityear(void *process)
 
               index1 = index2;
 
-              if (streamID2 >= 0) cdoStreamClose(streamID2);
+              if (streamID2 != CDO_STREAM_UNDEF) cdoStreamClose(streamID2);
 
               sprintf(filename + nchars, "%04d", index1);
               // if ( ic > 0 ) sprintf(filename+strlen(filename), "_%d", ic+1);
@@ -175,12 +173,12 @@ Splityear(void *process)
 
               if (Options::cdoVerbose) cdoPrint("create file %s", filename);
 
-              streamID2 = cdoStreamOpenWrite(filename);
-              pstreamDefVlist(streamID2, vlistID2);
+              streamID2 = cdoOpenWrite(filename);
+              cdoDefVlist(streamID2, vlistID2);
             }
         }
 
-      pstreamDefTimestep(streamID2, tsID2);
+      cdoDefTimestep(streamID2, tsID2);
 
       if (tsID > 0 && tsID2 == 0 && nconst)
         {
@@ -188,12 +186,12 @@ Splityear(void *process)
             {
               if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT)
                 {
-                  const int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-                  for (levelID = 0; levelID < nlevel; levelID++)
+                  const int nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+                  for (levelID = 0; levelID < nlevels; levelID++)
                     {
                       nmiss = vars[varID][levelID].nmiss;
-                      pstreamDefRecord(streamID2, varID, levelID);
-                      pstreamWriteRecord(streamID2, vars[varID][levelID].ptr, nmiss);
+                      cdoDefRecord(streamID2, varID, levelID);
+                      cdoWriteRecord(streamID2, vars[varID][levelID].vec.data(), nmiss);
                     }
                 }
             }
@@ -201,17 +199,17 @@ Splityear(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamDefRecord(streamID2, varID, levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoDefRecord(streamID2, varID, levelID);
 
           if (lcopy && !(tsID == 0 && nconst))
             {
-              pstreamCopyRecord(streamID2, streamID1);
+              cdoCopyRecord(streamID2, streamID1);
             }
           else
             {
-              pstreamReadRecord(streamID1, array.data(), &nmiss);
-              pstreamWriteRecord(streamID2, array.data(), nmiss);
+              cdoReadRecord(streamID1, array.data(), &nmiss);
+              cdoWriteRecord(streamID2, array.data(), nmiss);
 
               if (tsID == 0 && nconst)
                 {
@@ -219,7 +217,7 @@ Splityear(void *process)
                     {
                       const int gridID = vlistInqVarGrid(vlistID1, varID);
                       const size_t gridsize = gridInqSize(gridID);
-                      arrayCopy(gridsize, array.data(), vars[varID][levelID].ptr);
+                      arrayCopy(gridsize, array.data(), vars[varID][levelID].vec.data());
                       vars[varID][levelID].nmiss = nmiss;
                     }
                 }
@@ -233,19 +231,6 @@ Splityear(void *process)
   cdoStreamClose(streamID1);
   cdoStreamClose(streamID2);
 
-  if (nconst)
-    {
-      for (varID = 0; varID < nvars; varID++)
-        {
-          if (vlistInqVarTimetype(vlistID2, varID) == TIME_CONSTANT)
-            {
-              const int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
-              for (levelID = 0; levelID < nlevel; levelID++)
-                if (vars[varID][levelID].ptr) Free(vars[varID][levelID].ptr);
-            }
-        }
-    }
-
   cdoFinish();
 
   return nullptr;
