@@ -18,7 +18,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 
 void *
 Template1(void *process)
@@ -29,9 +28,9 @@ Template1(void *process)
 
   cdoInitialize(process);
 
-  const bool lcopy = UNCHANGED_RECORD;
+  const bool lcopy = unchangedRecord();
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -40,9 +39,9 @@ Template1(void *process)
   const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
+  CdoStreamID streamID2 = cdoOpenWrite(1);
 
-  pstreamDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID2, vlistID2);
 
   std::vector<double> array;
   if (!lcopy)
@@ -55,21 +54,21 @@ Template1(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamDefRecord(streamID2, varID, levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoDefRecord(streamID2, varID, levelID);
 
           if (lcopy)
             {
-              pstreamCopyRecord(streamID2, streamID1);
+              cdoCopyRecord(streamID2, streamID1);
             }
           else
             {
-              pstreamReadRecord(streamID1, array.data(), &nmiss);
-              pstreamWriteRecord(streamID2, array.data(), nmiss);
+              cdoReadRecord(streamID1, array.data(), &nmiss);
+              cdoWriteRecord(streamID2, array.data(), nmiss);
             }
         }
 
@@ -95,7 +94,7 @@ Template2(void *process)
 
   cdoInitialize(process);
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -104,9 +103,9 @@ Template2(void *process)
   const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
+  CdoStreamID streamID2 = cdoOpenWrite(1);
 
-  pstreamDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID2, vlistID2);
 
   const size_t gridsizemax = vlistGridsizeMax(vlistID1);
   std::vector<double> array(gridsizemax);
@@ -115,15 +114,15 @@ Template2(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamDefRecord(streamID2, varID, levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoDefRecord(streamID2, varID, levelID);
 
-          pstreamReadRecord(streamID1, array.data(), &nmiss);
-          pstreamWriteRecord(streamID2, array.data(), nmiss);
+          cdoReadRecord(streamID1, array.data(), &nmiss);
+          cdoWriteRecord(streamID2, array.data(), nmiss);
         }
 
       tsID++;
