@@ -18,20 +18,19 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 
 void *
 Test(void *process)
 {
   /*
-  int streamID1, streamID2;
+  CdoStreamID streamID1, streamID2;
   */
 
   cdoInitialize(process);
 
   /*
-  streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-  streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
+  streamID1 = cdoOpenRead(0);
+  streamID2 = cdoOpenWrite(1);
 
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
@@ -45,15 +44,15 @@ void *
 Test2(void *process)
 {
   /*
-  int streamID1, streamID2, streamID3;
+  CdoStreamID streamID1, streamID2, streamID3;
   */
 
   cdoInitialize(process);
 
   /*
-  streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-  streamID2 = cdoStreamOpenRead(cdoStreamName(1));
-  streamID3 = cdoStreamOpenWrite(cdoStreamName(2));
+  streamID1 = cdoOpenRead(0);
+  streamID2 = cdoOpenRead(1);
+  streamID3 = cdoOpenWrite(2);
 
   cdoStreamClose(streamID3);
   cdoStreamClose(streamID2);
@@ -75,18 +74,18 @@ Testdata(void *process)
 
   int tsID2 = 0;
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   int vlistID1 = cdoStreamInqVlist(streamID1);
   int taxisID1 = vlistInqTaxis(vlistID1);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
+  CdoStreamID streamID2 = cdoOpenWrite(1);
 
   int vlistID2 = vlistDuplicate(vlistID1);
   int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  pstreamDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID2, vlistID2);
 
   size_t gridsize = vlistGridsizeMax(vlistID1);
   double *array = (double *) Malloc(gridsize * sizeof(double));
@@ -101,14 +100,14 @@ Testdata(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID1)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID2);
+      cdoDefTimestep(streamID2, tsID2);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamDefRecord(streamID2, varID, levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoDefRecord(streamID2, varID, levelID);
 
-          pstreamReadRecord(streamID1, array, &nmiss);
+          cdoReadRecord(streamID1, array, &nmiss);
 
           gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
           for (size_t i = 0; i < gridsize; ++i)
@@ -128,7 +127,7 @@ Testdata(void *process)
                        (unsigned int) cval[4 * i + 2], (unsigned int) cval[4 * i + 3], ival[i], fval[i]);
             }
 
-          pstreamWriteRecord(streamID2, array, nmiss);
+          cdoWriteRecord(streamID2, array, nmiss);
 
           fwrite(cval, 4, gridsize, fp);
         }
