@@ -28,7 +28,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 
 void *
 Timcount(void *process)
@@ -38,8 +37,6 @@ Timcount(void *process)
   int vtime0 = 0;
   int nrecs;
   int varID, levelID;
-  size_t nmiss;
-  int nwpv;  // number of words per value; real:1  complex:2
 
   cdoInitialize(process);
 
@@ -51,39 +48,38 @@ Timcount(void *process)
   cdoOperatorAdd("hourcount", 0,  4, nullptr);
   // clang-format on
 
-  int operatorID = cdoOperatorID();
+  const int operatorID = cdoOperatorID();
 
-  int cmplen = DATE_LEN - cdoOperatorF2(operatorID);
+  const int cmplen = DATE_LEN - cdoOperatorF2(operatorID);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
-  int nvars = vlistNvars(vlistID1);
+  const int nvars = vlistNvars(vlistID1);
   for (varID = 0; varID < nvars; varID++) vlistDefVarUnits(vlistID2, varID, "No.");
 
   if (cdoOperatorF2(operatorID) == 16) vlistDefNtsteps(vlistID2, 1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
-  int maxrecs = vlistNrecs(vlistID1);
-  std::vector<RecordInfo> recinfo(maxrecs);
+  const int maxrecs = vlistNrecs(vlistID1);
+  std::vector<RecordInfo> recList(maxrecs);
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
-  if (vlistNumber(vlistID1) != CDI_REAL) gridsize *= 2;
+  size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
 
   Field field;
-  fieldInit(field);
-  field.ptr = (double *) Malloc(gridsize * sizeof(double));
+  field.resize(gridsizemax);
 
   FieldVector2D vars1;
-  fieldsFromVlist(vlistID1, vars1, FIELD_PTR);
+  fieldsFromVlist(vlistID1, vars1, FIELD_VEC);
 
   int tsID = 0;
   int otsID = 0;
@@ -92,8 +88,8 @@ Timcount(void *process)
       int nsets = 0;
       while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
         {
-          int64_t vdate = taxisInqVdate(taxisID1);
-          int vtime = taxisInqVtime(taxisID1);
+          const int64_t vdate = taxisInqVdate(taxisID1);
+          const int vtime = taxisInqVtime(taxisID1);
 
           if (nsets == 0) SET_DATE(indate2, vdate, vtime);
           SET_DATE(indate1, vdate, vtime);
@@ -102,30 +98,28 @@ Timcount(void *process)
 
           for (int recID = 0; recID < nrecs; recID++)
             {
-              pstreamInqRecord(streamID1, &varID, &levelID);
+              cdoInqRecord(streamID1, &varID, &levelID);
 
               if (tsID == 0)
                 {
-                  recinfo[recID].varID = varID;
-                  recinfo[recID].levelID = levelID;
-                  recinfo[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+                  recList[recID].varID = varID;
+                  recList[recID].levelID = levelID;
+                  recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
                 }
 
-              nwpv = vars1[varID][levelID].nwpv;
-              gridsize = gridInqSize(vars1[varID][levelID].grid);
+              const size_t fieldsize = vars1[varID][levelID].size;
 
               if (nsets == 0)
                 {
-                  for (size_t i = 0; i < nwpv * gridsize; i++) vars1[varID][levelID].ptr[i] = vars1[varID][levelID].missval;
-                  vars1[varID][levelID].nmiss = gridsize;
+                  for (size_t i = 0; i < fieldsize; i++) vars1[varID][levelID].vec[i] = vars1[varID][levelID].missval;
+                  vars1[varID][levelID].nmiss = fieldsize;
                 }
 
-              pstreamReadRecord(streamID1, field.ptr, &nmiss);
-              field.nmiss = nmiss;
+              cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
               field.grid = vars1[varID][levelID].grid;
               field.missval = vars1[varID][levelID].missval;
 
-              farcount(vars1[varID][levelID], field);
+              vfarcount(vars1[varID][levelID], field);
             }
 
           vdate0 = vdate;
@@ -138,26 +132,22 @@ Timcount(void *process)
 
       taxisDefVdate(taxisID2, vdate0);
       taxisDefVtime(taxisID2, vtime0);
-      pstreamDefTimestep(streamID2, otsID);
+      cdoDefTimestep(streamID2, otsID);
 
       for (int recID = 0; recID < maxrecs; recID++)
         {
-          if (otsID && recinfo[recID].lconst) continue;
+          if (otsID && recList[recID].lconst) continue;
 
-          int varID = recinfo[recID].varID;
-          int levelID = recinfo[recID].levelID;
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, vars1[varID][levelID].ptr, vars1[varID][levelID].nmiss);
+          const int varID = recList[recID].varID;
+          const int levelID = recList[recID].levelID;
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, vars1[varID][levelID].vec.data(), vars1[varID][levelID].nmiss);
         }
 
       if (nrecs == 0) break;
       otsID++;
     }
 
-  fieldsFree(vlistID1, vars1);
-
-  if (field.ptr) Free(field.ptr);
-
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
