@@ -24,91 +24,82 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 
 void *
 Timcumsum(void *process)
 {
   int nrecs;
   int varID, levelID;
-  size_t nmiss;
 
   cdoInitialize(process);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
-  if (vlistNumber(vlistID1) != CDI_REAL) gridsize *= 2;
+  size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
 
   Field field;
-  fieldInit(field);
-  field.ptr = (double *) Malloc(gridsize * sizeof(double));
+  field.resize(gridsizemax);
 
   FieldVector2D vars1;
-  fieldsFromVlist(vlistID1, vars1, FIELD_PTR);
+  fieldsFromVlist(vlistID1, vars1, FIELD_VEC);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           Field &rvars1 = vars1[varID][levelID];
 
-          gridsize = gridInqSize(rvars1.grid);
+          const size_t fieldsize = rvars1.size;
 
           if (tsID == 0)
             {
-              pstreamReadRecord(streamID1, rvars1.ptr, &nmiss);
-              // rvars1.nmiss = nmiss;
-              if (nmiss)
-                for (size_t i = 0; i < gridsize; ++i)
-                  if (DBL_IS_EQUAL(rvars1.ptr[i], rvars1.missval)) rvars1.ptr[i] = 0;
+              cdoReadRecord(streamID1, rvars1.vec.data(), &rvars1.nmiss);
+              if (rvars1.nmiss)
+                for (size_t i = 0; i < fieldsize; ++i)
+                  if (DBL_IS_EQUAL(rvars1.vec[i], rvars1.missval)) rvars1.vec[i] = 0;
             }
           else
             {
-              pstreamReadRecord(streamID1, field.ptr, &nmiss);
-              // field.nmiss   = nmiss;
-              field.size = gridsize;
+              cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
+              field.size = fieldsize;
               field.grid = rvars1.grid;
               field.missval = rvars1.missval;
 
-              if (nmiss)
-                for (size_t i = 0; i < gridsize; ++i)
-                  if (DBL_IS_EQUAL(field.ptr[i], rvars1.missval)) field.ptr[i] = 0;
+              if (field.nmiss)
+                for (size_t i = 0; i < fieldsize; ++i)
+                  if (DBL_IS_EQUAL(field.vec[i], rvars1.missval)) field.vec[i] = 0;
 
-              farfun(rvars1, field, func_sum);
+              vfarsum(rvars1, field);
             }
 
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, rvars1.ptr, rvars1.nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, rvars1.vec.data(), rvars1.nmiss);
         }
 
       tsID++;
     }
 
-  fieldsFree(vlistID1, vars1);
-
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
-  if (field.ptr) Free(field.ptr);
-
   cdoFinish();
 
-  return 0;
+  return nullptr;
 }
