@@ -25,7 +25,6 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
 #include "percentiles_hist.h"
 #include "percentiles.h"
 #include "datetime.h"
@@ -46,59 +45,58 @@ Timselpctl(void *process)
 
   operatorInputArg("percentile number, nsets <,noffset <,nskip>>");
 
-  int nargc = operatorArgc();
+  const int nargc = operatorArgc();
   if (nargc < 2) cdoAbort("Too few arguments! Need %d found %d.", 2, nargc);
 
-  double pn = parameter2double(operatorArgv()[0]);
+  const double pn = parameter2double(operatorArgv()[0]);
   percentile_check_number(pn);
-  int ndates = parameter2int(operatorArgv()[1]);
+  const int ndates = parameter2int(operatorArgv()[1]);
   int noffset = 0, nskip = 0;
   if (nargc > 2) noffset = parameter2int(operatorArgv()[2]);
   if (nargc > 3) nskip = parameter2int(operatorArgv()[3]);
 
   if (Options::cdoVerbose) cdoPrint("nsets = %d, noffset = %d, nskip = %d", ndates, noffset, nskip);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-  int streamID2 = cdoStreamOpenRead(cdoStreamName(1));
-  int streamID3 = cdoStreamOpenRead(cdoStreamName(2));
+  CdoStreamID streamID1 = cdoOpenRead(0);
+  CdoStreamID streamID2 = cdoOpenRead(1);
+  CdoStreamID streamID3 = cdoOpenRead(2);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = cdoStreamInqVlist(streamID2);
-  int vlistID3 = cdoStreamInqVlist(streamID3);
-  int vlistID4 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = cdoStreamInqVlist(streamID2);
+  const int vlistID3 = cdoStreamInqVlist(streamID3);
+  const int vlistID4 = vlistDuplicate(vlistID1);
 
   vlistCompare(vlistID1, vlistID2, CMP_ALL);
   vlistCompare(vlistID1, vlistID3, CMP_ALL);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = vlistInqTaxis(vlistID2);
-  int taxisID3 = vlistInqTaxis(vlistID3);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = vlistInqTaxis(vlistID2);
+  const int taxisID3 = vlistInqTaxis(vlistID3);
   /* TODO - check that time axes 2 and 3 are equal */
 
-  int taxisID4 = taxisDuplicate(taxisID1);
+  const int taxisID4 = taxisDuplicate(taxisID1);
   taxisWithBounds(taxisID4);
   vlistDefTaxis(vlistID4, taxisID4);
 
-  int streamID4 = cdoStreamOpenWrite(cdoStreamName(3));
-  pstreamDefVlist(streamID4, vlistID4);
+  CdoStreamID streamID4 = cdoOpenWrite(3);
+  cdoDefVlist(streamID4, vlistID4);
 
-  int nvars = vlistNvars(vlistID1);
+  const int nvars = vlistNvars(vlistID1);
 
-  int maxrecs = vlistNrecs(vlistID1);
-  std::vector<RecordInfo> recinfo(maxrecs);
+  const int maxrecs = vlistNrecs(vlistID1);
+  std::vector<RecordInfo> recList(maxrecs);
 
   DateTimeList dtlist;
   dtlist.setStat(timestat_date);
   dtlist.setCalendar(taxisInqCalendar(taxisID1));
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
+  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field;
-  fieldInit(field);
-  field.ptr = (double *) Malloc(gridsize * sizeof(double));
+  field.resize(gridsizemax);
 
   FieldVector2D vars1;
-  fieldsFromVlist(vlistID1, vars1, FIELD_PTR);
+  fieldsFromVlist(vlistID1, vars1, FIELD_VEC);
   HISTOGRAM_SET *hset = hsetCreate(nvars);
 
   for (varID = 0; varID < nvars; varID++)
@@ -116,13 +114,13 @@ Timselpctl(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           if (tsID == 0)
             {
-              recinfo[recID].varID = varID;
-              recinfo[recID].levelID = levelID;
-              recinfo[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+              recList[recID].varID = varID;
+              recList[recID].levelID = levelID;
+              recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
             }
         }
     }
@@ -138,29 +136,26 @@ Timselpctl(void *process)
     {
       nrecs = cdoStreamInqTimestep(streamID2, otsID);
       if (nrecs != cdoStreamInqTimestep(streamID3, otsID))
-        cdoAbort("Number of records at time step %d of %s and %s differ!", otsID + 1, cdoGetStreamName(1),
-                 cdoGetStreamName(2));
+        cdoAbort("Number of records at time step %d of %s and %s differ!", otsID + 1, cdoGetStreamName(1), cdoGetStreamName(2));
 
-      int64_t vdate2 = taxisInqVdate(taxisID2);
-      int vtime2 = taxisInqVtime(taxisID2);
-      int64_t vdate3 = taxisInqVdate(taxisID3);
-      int vtime3 = taxisInqVtime(taxisID3);
+      const int64_t vdate2 = taxisInqVdate(taxisID2);
+      const int vtime2 = taxisInqVtime(taxisID2);
+      const int64_t vdate3 = taxisInqVdate(taxisID3);
+      const int vtime3 = taxisInqVtime(taxisID3);
       if (vdate2 != vdate3 || vtime2 != vtime3)
-        cdoAbort("Verification dates at time step %d of %s and %s differ!", otsID + 1, cdoGetStreamName(1),
-                 cdoGetStreamName(2));
+        cdoAbort("Verification dates at time step %d of %s and %s differ!", otsID + 1, cdoGetStreamName(1), cdoGetStreamName(2));
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID2, &varID, &levelID);
-          pstreamReadRecord(streamID2, vars1[varID][levelID].ptr, &nmiss);
+          cdoInqRecord(streamID2, &varID, &levelID);
+          cdoReadRecord(streamID2, vars1[varID][levelID].vec.data(), &nmiss);
           vars1[varID][levelID].nmiss = nmiss;
         }
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID3, &varID, &levelID);
-          pstreamReadRecord(streamID3, field.ptr, &nmiss);
-          field.nmiss = nmiss;
+          cdoInqRecord(streamID3, &varID, &levelID);
+          cdoReadRecord(streamID3, field.vec.data(), &field.nmiss);
           field.grid = vars1[varID][levelID].grid;
           field.missval = vars1[varID][levelID].missval;
 
@@ -178,16 +173,16 @@ Timselpctl(void *process)
 
             for (int recID = 0; recID < nrecs; recID++)
               {
-                pstreamInqRecord(streamID1, &varID, &levelID);
+                cdoInqRecord(streamID1, &varID, &levelID);
 
                 if (tsID == 0)
                   {
-                    recinfo[recID].varID = varID;
-                    recinfo[recID].levelID = levelID;
-                    recinfo[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+                    recList[recID].varID = varID;
+                    recList[recID].levelID = levelID;
+                    recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
                   }
 
-                pstreamReadRecord(streamID1, vars1[varID][levelID].ptr, &nmiss);
+                cdoReadRecord(streamID1, vars1[varID][levelID].vec.data(), &nmiss);
                 vars1[varID][levelID].nmiss = nmiss;
 
                 hsetAddVarLevelValues(hset, varID, levelID, &vars1[varID][levelID]);
@@ -208,16 +203,16 @@ Timselpctl(void *process)
         }
 
       dtlist.statTaxisDefTimestep(taxisID4, nsets);
-      pstreamDefTimestep(streamID4, otsID);
+      cdoDefTimestep(streamID4, otsID);
 
       for (int recID = 0; recID < maxrecs; recID++)
         {
-          if (otsID && recinfo[recID].lconst) continue;
+          if (otsID && recList[recID].lconst) continue;
 
-          int varID = recinfo[recID].varID;
-          int levelID = recinfo[recID].levelID;
-          pstreamDefRecord(streamID4, varID, levelID);
-          pstreamWriteRecord(streamID4, vars1[varID][levelID].ptr, vars1[varID][levelID].nmiss);
+          const int varID = recList[recID].varID;
+          const int levelID = recList[recID].levelID;
+          cdoDefRecord(streamID4, varID, levelID);
+          cdoWriteRecord(streamID4, vars1[varID][levelID].vec.data(), vars1[varID][levelID].nmiss);
         }
 
       if (nrecs == 0) break;
@@ -235,11 +230,8 @@ Timselpctl(void *process)
 
 LABEL_END:
 
-  fieldsFree(vlistID1, vars1);
   hsetDestroy(hset);
 
-  if (field.ptr) Free(field.ptr);
-
   cdoStreamClose(streamID4);
   cdoStreamClose(streamID3);
   cdoStreamClose(streamID2);
