@@ -73,7 +73,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 #include "datetime.h"
 #include "printinfo.h"
 
@@ -98,12 +97,14 @@ vlistSetFrequency(int vlistID, int comparelen)
 }
 
 static void
-timstatAddOperators(void)
+addOperators(void)
 {
   // clang-format off
   cdoOperatorAdd("timrange",   func_range,  DATE_LEN, nullptr);
   cdoOperatorAdd("timmin",     func_min,    DATE_LEN, nullptr);
   cdoOperatorAdd("timmax",     func_max,    DATE_LEN, nullptr);
+  cdoOperatorAdd("timminidx",  func_minidx, DATE_LEN, nullptr);
+  cdoOperatorAdd("timmaxidx",  func_maxidx, DATE_LEN, nullptr);
   cdoOperatorAdd("timsum",     func_sum,    DATE_LEN, nullptr);
   cdoOperatorAdd("timmean",    func_mean,   DATE_LEN, nullptr);
   cdoOperatorAdd("timavg",     func_avg,    DATE_LEN, nullptr);
@@ -164,33 +165,31 @@ Timstat(void *argument)
   int vtime0 = 0;
   int nrecs;
   int varID, levelID;
-  int streamID3 = -1;
+  CdoStreamID streamID3;
   int vlistID3, taxisID3 = -1;
-  size_t nmiss;
   bool lvfrac = false;
-  int nwpv;  // number of words per value; real:1  complex:2
   char indate1[DATE_LEN + 1], indate2[DATE_LEN + 1];
   double vfrac = 1;
 
   cdoInitialize(argument);
 
-  timstatAddOperators();
+  addOperators();
 
-  int operatorID = cdoOperatorID();
-  int operfunc = cdoOperatorF1(operatorID);
-  int comparelen = cdoOperatorF2(operatorID);
+  const int operatorID = cdoOperatorID();
+  const int operfunc = cdoOperatorF1(operatorID);
+  const int comparelen = cdoOperatorF2(operatorID);
 
-  bool lrange = operfunc == func_range;
-  bool lminidx = operfunc == func_minidx;
-  bool lmaxidx = operfunc == func_maxidx;
-  bool lmean = operfunc == func_mean || operfunc == func_avg;
-  bool lstd = operfunc == func_std || operfunc == func_std1;
-  bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
-  int divisor = operfunc == func_std1 || operfunc == func_var1;
+  const bool lrange = operfunc == func_range;
+  const bool lminidx = operfunc == func_minidx;
+  const bool lmaxidx = operfunc == func_maxidx;
+  const bool lmean = operfunc == func_mean || operfunc == func_avg;
+  const bool lstd = operfunc == func_std || operfunc == func_std1;
+  const bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
+  const int divisor = operfunc == func_std1 || operfunc == func_var1;
 
   if (operfunc == func_mean)
     {
-      int oargc = operatorArgc();
+      const int oargc = operatorArgc();
       char **oargv = operatorArgv();
 
       if (oargc == 1)
@@ -204,36 +203,41 @@ Timstat(void *argument)
         cdoAbort("Too many arguments!");
     }
 
-  int cmplen = DATE_LEN - comparelen;
+  const int cmplen = DATE_LEN - comparelen;
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
   if (cmplen == 0) vlistDefNtsteps(vlistID2, 1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   taxisWithBounds(taxisID2);
   if (taxisInqType(taxisID2) == TAXIS_FORECAST) taxisDefType(taxisID2, TAXIS_RELATIVE);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int nvars = vlistNvars(vlistID1);
+  const int nvars = vlistNvars(vlistID1);
+
+  if (lminidx || lmaxidx)
+    {
+      for (varID = 0; varID < nvars; ++varID)
+        vlistDefVarDatatype(vlistID2, varID, CDI_DATATYPE_INT32);
+    }
 
   vlistSetFrequency(vlistID2, comparelen);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   if (Options::cdoDiag)
     {
       char filename[8192];
-
       strcpy(filename, cdoOperatorName(operatorID));
       strcat(filename, "_");
       strcat(filename, cdoGetStreamName(1));
-      streamID3 = cdoStreamOpenWrite(filename);
+      streamID3 = cdoOpenWrite(filename);
 
       vlistID3 = vlistDuplicate(vlistID1);
 
@@ -250,11 +254,11 @@ Timstat(void *argument)
       taxisWithBounds(taxisID3);
       vlistDefTaxis(vlistID3, taxisID3);
 
-      pstreamDefVlist(streamID3, vlistID3);
+      cdoDefVlist(streamID3, vlistID3);
     }
 
-  int maxrecs = vlistNrecs(vlistID1);
-  std::vector<RecordInfo> recinfo(maxrecs);
+  const int maxrecs = vlistNrecs(vlistID1);
+  std::vector<RecordInfo> recList(maxrecs);
 
   DateTimeList dtlist;
   dtlist.setStat(timestat_date);
@@ -267,18 +271,18 @@ Timstat(void *argument)
   if (operfunc == func_mean && Options::CDO_Memtype == MEMTYPE_FLOAT) FIELD_MEMTYPE = MEMTYPE_FLOAT;
 
   Field field;
-  fieldInit(field);
   field.memtype = FIELD_MEMTYPE;
   if (FIELD_MEMTYPE == MEMTYPE_FLOAT)
-    field.ptrf = (float *) Malloc(gridsizemax * sizeof(float));
+    field.resizef(gridsizemax);
   else
-    field.ptr = (double *) Malloc(gridsizemax * sizeof(double));
+    field.resize(gridsizemax);
+
+  const bool lvars2 = lvarstd || lrange || lminidx || lmaxidx;
 
-  bool lvars2 = lvarstd || lrange || lminidx || lmaxidx;
   FieldVector2D samp1, vars1, vars2;
   fieldsFromVlist(vlistID1, samp1, FIELD_NONE);
-  fieldsFromVlist(vlistID1, vars1, FIELD_PTR);
-  if (lvars2) fieldsFromVlist(vlistID1, vars2, FIELD_PTR);
+  fieldsFromVlist(vlistID1, vars1, FIELD_VEC);
+  if (lvars2) fieldsFromVlist(vlistID1, vars2, FIELD_VEC);
 
   int tsID = 0;
   int otsID = 0;
@@ -288,8 +292,8 @@ Timstat(void *argument)
       while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
         {
           dtlist.taxisInqTimestep(taxisID1, nsets);
-          int64_t vdate = dtlist.getVdate(nsets);
-          int vtime = dtlist.getVtime(nsets);
+          const int64_t vdate = dtlist.getVdate(nsets);
+          const int vtime = dtlist.getVtime(nsets);
 
           if (nsets == 0) SET_DATE(indate2, vdate, vtime);
           SET_DATE(indate1, vdate, vtime);
@@ -298,94 +302,89 @@ Timstat(void *argument)
 
           for (int recID = 0; recID < nrecs; recID++)
             {
-              pstreamInqRecord(streamID1, &varID, &levelID);
+              cdoInqRecord(streamID1, &varID, &levelID);
 
               if (tsID == 0)
                 {
-                  recinfo[recID].varID = varID;
-                  recinfo[recID].levelID = levelID;
-                  recinfo[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+                  recList[recID].varID = varID;
+                  recList[recID].levelID = levelID;
+                  recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
                 }
 
               Field &rsamp1 = samp1[varID][levelID];
               Field &rvars1 = vars1[varID][levelID];
 
-              nwpv = rvars1.nwpv;
-              size_t gridsize = rvars1.size;
+              const size_t fieldsize = rvars1.size;
 
               if (nsets == 0)
                 {
-                  pstreamReadRecord(streamID1, rvars1.ptr, &nmiss);
-                  rvars1.nmiss = nmiss;
+                  cdoReadRecord(streamID1, rvars1.vec.data(), &rvars1.nmiss);
                   if (lrange)
                     {
-                      vars2[varID][levelID].nmiss = nmiss;
-                      for (size_t i = 0; i < nwpv * gridsize; i++) vars2[varID][levelID].ptr[i] = rvars1.ptr[i];
+                      vars2[varID][levelID].nmiss = rvars1.nmiss;
+                      vars2[varID][levelID].vec = rvars1.vec;
                     }
                   else if (lminidx || lmaxidx)
                     {
-                      vars2[varID][levelID].nmiss = nmiss;
-                      for (size_t i = 0; i < nwpv * gridsize; i++) vars2[varID][levelID].ptr[i] = rvars1.ptr[i];
-                      for (size_t i = 0; i < nwpv * gridsize; i++) rvars1.ptr[i] = 0;
+                      vars2[varID][levelID].nmiss = rvars1.nmiss;
+                      vars2[varID][levelID].vec = rvars1.vec;
+                      fieldFill(rvars1, 0);
                     }
 
-                  if (nmiss > 0 || rsamp1.ptr)
+                  if (rvars1.nmiss || !rsamp1.empty())
                     {
-                      if (rsamp1.ptr == nullptr) rsamp1.ptr = (double *) Malloc(nwpv * gridsize * sizeof(double));
-                      for (size_t i = 0; i < nwpv * gridsize; i++) rsamp1.ptr[i] = !DBL_IS_EQUAL(rvars1.ptr[i], rvars1.missval);
+                      if (rsamp1.empty()) rsamp1.resize(fieldsize);
+                      for (size_t i = 0; i < fieldsize; i++)
+                        rsamp1.vec[i] = !DBL_IS_EQUAL(rvars1.vec[i], rvars1.missval);
                     }
                 }
               else
                 {
                   if (Options::CDO_Memtype == MEMTYPE_FLOAT)
-                    pstreamReadRecordF(streamID1, field.ptrf, &nmiss);
+                    cdoReadRecordF(streamID1, field.vecf.data(), &field.nmiss);
                   else
-                    pstreamReadRecord(streamID1, field.ptr, &nmiss);
-                  field.nmiss = nmiss;
-                  field.size = gridsize;
+                    cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
+                  field.size = fieldsize;
                   field.grid = rvars1.grid;
                   field.missval = rvars1.missval;
-                  if (field.nmiss > 0 || rsamp1.ptr)
+
+                  if (field.nmiss || !rsamp1.empty())
                     {
-                      if (rsamp1.ptr == nullptr)
-                        {
-                          rsamp1.ptr = (double *) Malloc(nwpv * gridsize * sizeof(double));
-                          for (size_t i = 0; i < nwpv * gridsize; i++) rsamp1.ptr[i] = nsets;
-                        }
+                      if (rsamp1.empty()) rsamp1.resize(fieldsize, nsets);
 
                       if (Options::CDO_Memtype == MEMTYPE_FLOAT)
                         {
-                          for (size_t i = 0; i < nwpv * gridsize; i++)
-                            if (!DBL_IS_EQUAL(field.ptrf[i], rvars1.missval)) rsamp1.ptr[i]++;
+                          for (size_t i = 0; i < fieldsize; i++)
+                            if (!DBL_IS_EQUAL(field.vecf[i], rvars1.missval)) rsamp1.vec[i]++;
                         }
                       else
                         {
-                          for (size_t i = 0; i < nwpv * gridsize; i++)
-                            if (!DBL_IS_EQUAL(field.ptr[i], rvars1.missval)) rsamp1.ptr[i]++;
+                          for (size_t i = 0; i < fieldsize; i++)
+                            if (!DBL_IS_EQUAL(field.vec[i], rvars1.missval)) rsamp1.vec[i]++;
                         }
                     }
 
                   if (lvarstd)
                     {
-                      farsumq(vars2[varID][levelID], field);
-                      farsum(rvars1, field);
+                      vfarsumq(vars2[varID][levelID], field);
+                      vfarsum(rvars1, field);
                     }
                   else if (lrange)
                     {
-                      farmin(vars2[varID][levelID], field);
-                      farmax(rvars1, field);
+                      vfarmin(vars2[varID][levelID], field);
+                      vfarmax(rvars1, field);
                     }
                   else if (lminidx)
                     {
-                      farminidx(rvars1, vars2[varID][levelID], field, nsets);
+                      vfarminidx(rvars1, vars2[varID][levelID], field, nsets);
                     }
                   else if (lmaxidx)
                     {
-                      farmaxidx(rvars1, vars2[varID][levelID], field, nsets);
+                      vfarmaxidx(rvars1, vars2[varID][levelID], field, nsets);
                     }
                   else
                     {
-                      farfun(rvars1, field, operfunc);
+                      vfarfun(rvars1, field, operfunc);
                     }
                 }
             }
@@ -393,11 +392,11 @@ Timstat(void *argument)
           if (nsets == 0 && lvarstd)
             for (int recID = 0; recID < maxrecs; recID++)
               {
-                if (recinfo[recID].lconst) continue;
+                if (recList[recID].lconst) continue;
 
-                int varID = recinfo[recID].varID;
-                int levelID = recinfo[recID].levelID;
-                farmoq(vars2[varID][levelID], vars1[varID][levelID]);
+                const int varID = recList[recID].varID;
+                const int levelID = recList[recID].levelID;
+                vfarmoq(vars2[varID][levelID], vars1[varID][levelID]);
               }
 
           vdate0 = vdate;
@@ -410,42 +409,42 @@ Timstat(void *argument)
 
       for (int recID = 0; recID < maxrecs; recID++)
         {
-          if (recinfo[recID].lconst) continue;
+          if (recList[recID].lconst) continue;
 
-          int varID = recinfo[recID].varID;
-          int levelID = recinfo[recID].levelID;
+          const int varID = recList[recID].varID;
+          const int levelID = recList[recID].levelID;
           Field &rsamp1 = samp1[varID][levelID];
           Field &rvars1 = vars1[varID][levelID];
 
           if (lmean)
             {
-              if (rsamp1.ptr)
-                fardiv(rvars1, rsamp1);
+              if (!rsamp1.empty())
+                vfardiv(rvars1, rsamp1);
               else
-                farcdiv(rvars1, (double) nsets);
+                vfarcdiv(rvars1, (double) nsets);
             }
           else if (lvarstd)
             {
               Field &rvars2 = vars2[varID][levelID];
-              if (rsamp1.ptr)
+              if (!rsamp1.empty())
                 {
                   if (lstd)
-                    farstd(rvars1, rvars2, rsamp1, divisor);
+                    vfarstd(rvars1, rvars2, rsamp1, divisor);
                   else
-                    farvar(rvars1, rvars2, rsamp1, divisor);
+                    vfarvar(rvars1, rvars2, rsamp1, divisor);
                 }
               else
                 {
                   if (lstd)
-                    farcstd(rvars1, rvars2, nsets, divisor);
+                    vfarcstd(rvars1, rvars2, nsets, divisor);
                   else
-                    farcvar(rvars1, rvars2, nsets, divisor);
+                    vfarcvar(rvars1, rvars2, nsets, divisor);
                 }
             }
           else if (lrange)
             {
               Field &rvars2 = vars2[varID][levelID];
-              farsub(rvars1, rvars2);
+              vfarsub(rvars1, rvars2);
             }
         }
 
@@ -460,66 +459,65 @@ Timstat(void *argument)
       if (lvfrac && operfunc == func_mean)
         for (int recID = 0; recID < maxrecs; recID++)
           {
-            if (recinfo[recID].lconst) continue;
+            if (recList[recID].lconst) continue;
 
-            int varID = recinfo[recID].varID;
-            int levelID = recinfo[recID].levelID;
+            const int varID = recList[recID].varID;
+            const int levelID = recList[recID].levelID;
             Field &rsamp1 = samp1[varID][levelID];
             Field &rvars1 = vars1[varID][levelID];
 
-            int nwpv = rvars1.nwpv;
-            size_t gridsize = gridInqSize(rvars1.grid);
-            double missval = rvars1.missval;
-            if (rsamp1.ptr)
+            const size_t fieldsize = rvars1.size;
+            const double missval = rvars1.missval;
+            if (!rsamp1.empty())
               {
                 int irun = 0;
-                for (size_t i = 0; i < nwpv * gridsize; ++i)
+                for (size_t i = 0; i < fieldsize; ++i)
                   {
-                    if ((rsamp1.ptr[i] / nsets) < vfrac)
+                    if ((rsamp1.vec[i] / nsets) < vfrac)
                       {
-                        rvars1.ptr[i] = missval;
+                        rvars1.vec[i] = missval;
                         irun++;
                       }
                   }
 
-                if (irun) rvars1.nmiss = arrayNumMV(nwpv * gridsize, rvars1.ptr, missval);
+                if (irun) rvars1.nmiss = fieldNumMiss(rvars1);
               }
           }
 
       dtlist.statTaxisDefTimestep(taxisID2, nsets);
-      pstreamDefTimestep(streamID2, otsID);
+      cdoDefTimestep(streamID2, otsID);
 
       if (Options::cdoDiag)
         {
           dtlist.statTaxisDefTimestep(taxisID3, nsets);
-          pstreamDefTimestep(streamID3, otsID);
+          cdoDefTimestep(streamID3, otsID);
         }
 
       for (int recID = 0; recID < maxrecs; recID++)
         {
-          if (otsID && recinfo[recID].lconst) continue;
+          if (otsID && recList[recID].lconst) continue;
 
-          int varID = recinfo[recID].varID;
-          int levelID = recinfo[recID].levelID;
+          const int varID = recList[recID].varID;
+          const int levelID = recList[recID].levelID;
           Field &rsamp1 = samp1[varID][levelID];
           Field &rvars1 = vars1[varID][levelID];
 
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, rvars1.ptr, rvars1.nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, rvars1.vec.data(), rvars1.nmiss);
 
           if (Options::cdoDiag)
             {
-              double *sampptr = field.ptr;
-              if (rsamp1.ptr)
-                sampptr = rsamp1.ptr;
+              double *sampptr = field.vec.data();
+              if (!rsamp1.empty())
+                sampptr = rsamp1.vec.data();
               else
                 {
-                  size_t gridsize = rvars1.size;
-                  for (size_t i = 0; i < gridsize; ++i) sampptr[i] = nsets;
+                  const size_t fieldsize = rvars1.size;
+                  for (size_t i = 0; i < fieldsize; ++i) sampptr[i] = nsets;
                 }
 
-              pstreamDefRecord(streamID3, varID, levelID);
-              pstreamWriteRecord(streamID3, sampptr, 0);
+              cdoDefRecord(streamID3, varID, levelID);
+              cdoWriteRecord(streamID3, sampptr, 0);
             }
         }
 
@@ -527,16 +525,10 @@ Timstat(void *argument)
       otsID++;
     }
 
-  fieldsFree(vlistID1, samp1);
-  fieldsFree(vlistID1, vars1);
-  if (lvars2) fieldsFree(vlistID1, vars2);
-
   if (Options::cdoDiag) cdoStreamClose(streamID3);
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
-  if (field.ptr) Free(field.ptr);
-
   cdoFinish();
 
   return nullptr;
