@@ -24,7 +24,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 
 // correlation in time
 static void
@@ -141,43 +140,42 @@ Timstat2(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  cdoOperatorAdd("timcor",   func_cor,   0, nullptr);
-  cdoOperatorAdd("timcovar", func_covar, 0, nullptr);
+  cdoOperatorAdd("timcor",   func_cor,   5, nullptr);
+  cdoOperatorAdd("timcovar", func_covar, 3, nullptr);
   // clang-format on
 
-  int operatorID = cdoOperatorID();
-  int operfunc = cdoOperatorF1(operatorID);
+  const int operatorID = cdoOperatorID();
+  const int operfunc = cdoOperatorF1(operatorID);
+  const int nwork = cdoOperatorF2(operatorID);
 
-  int nwork = 0;
-  if (operfunc == func_cor)
-    nwork = 5;
-  else if (operfunc == func_covar)
-    nwork = 3;
+  CdoStreamID streamID1 = cdoOpenRead(0);
+  CdoStreamID streamID2 = cdoOpenRead(1);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-  int streamID2 = cdoStreamOpenRead(cdoStreamName(1));
-
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = cdoStreamInqVlist(streamID2);
-  int vlistID3 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = cdoStreamInqVlist(streamID2);
+  const int vlistID3 = vlistDuplicate(vlistID1);
 
   vlistCompare(vlistID1, vlistID2, CMP_ALL);
 
-  int nvars = vlistNvars(vlistID1);
+  VarList varList1, varList2;
+  varListInit(varList1, vlistID1);
+  varListInit(varList2, vlistID2);
+
+  const int nvars = vlistNvars(vlistID1);
   int nrecs = vlistNrecs(vlistID1);
-  int nrecs3 = nrecs;
+  const int nrecs3 = nrecs;
   std::vector<int> recVarID(nrecs);
   std::vector<int> recLevelID(nrecs);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
   // int taxisID2 = vlistInqTaxis(vlistID2);
-  int taxisID3 = taxisDuplicate(taxisID1);
+  const int taxisID3 = taxisDuplicate(taxisID1);
 
   vlistDefTaxis(vlistID3, taxisID3);
-  int streamID3 = cdoStreamOpenWrite(cdoStreamName(2));
-  pstreamDefVlist(streamID3, vlistID3);
+  CdoStreamID streamID3 = cdoOpenWrite(2);
+  cdoDefVlist(streamID3, vlistID3);
 
-  size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
   std::vector<double> array1(gridsizemax);
   std::vector<double> array2(gridsizemax);
 
@@ -186,9 +184,8 @@ Timstat2(void *process)
 
   for (varID = 0; varID < nvars; varID++)
     {
-      int gridID = vlistInqVarGrid(vlistID1, 0);
-      size_t gridsize = gridInqSize(gridID);
-      int nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+      const size_t gridsize = varList1[varID].gridsize;
+      const int nlevs = varList1[varID].nlevels;
 
       work[varID].resize(nlevs);
       nofvals[varID].resize(nlevs);
@@ -212,8 +209,8 @@ Timstat2(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamInqRecord(streamID2, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID2, &varID, &levelID);
 
           if (tsID == 0)
             {
@@ -221,13 +218,12 @@ Timstat2(void *process)
               recLevelID[recID] = levelID;
             }
 
-          size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
+          const size_t gridsize = varList1[varID].gridsize;
+          const double missval1 = varList1[varID].missval;
+          const double missval2 = varList2[varID].missval;
 
-          double missval1 = vlistInqVarMissval(vlistID1, varID);
-          double missval2 = vlistInqVarMissval(vlistID2, varID);
-
-          pstreamReadRecord(streamID1, &array1[0], &nmiss);
-          pstreamReadRecord(streamID2, &array2[0], &nmiss);
+          cdoReadRecord(streamID1, &array1[0], &nmiss);
+          cdoReadRecord(streamID2, &array2[0], &nmiss);
 
           if (operfunc == func_cor)
             {
@@ -248,17 +244,16 @@ Timstat2(void *process)
   tsID = 0;
   taxisDefVdate(taxisID3, vdate);
   taxisDefVtime(taxisID3, vtime);
-  pstreamDefTimestep(streamID3, tsID);
+  cdoDefTimestep(streamID3, tsID);
 
   for (int recID = 0; recID < nrecs3; recID++)
     {
       varID = recVarID[recID];
       levelID = recLevelID[recID];
 
-      size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-
-      double missval1 = vlistInqVarMissval(vlistID1, varID);
-      double missval2 = vlistInqVarMissval(vlistID2, varID);
+      const size_t gridsize = varList1[varID].gridsize;
+      const double missval1 = varList1[varID].missval;
+      const double missval2 = varList2[varID].missval;
 
       if (operfunc == func_cor)
         {
@@ -272,8 +267,8 @@ Timstat2(void *process)
                              &work[varID][levelID][1][0], &work[varID][levelID][2][0]);
         }
 
-      pstreamDefRecord(streamID3, varID, levelID);
-      pstreamWriteRecord(streamID3, &work[varID][levelID][0][0], nmiss);
+      cdoDefRecord(streamID3, varID, levelID);
+      cdoWriteRecord(streamID3, &work[varID][levelID][0][0], nmiss);
     }
 
   cdoStreamClose(streamID3);
