@@ -25,7 +25,7 @@
 
 #include "cdo_int.h"
 #include "calendar.h"
-#include "pstream_int.h"
+
 #include "datetime.h"
 #include "printinfo.h"
 #include "util_string.h"
@@ -141,7 +141,7 @@ printBounds(int taxisID, int calendar)
   getTimeInc(jdelta, vdate0, vdate1, &incperiod, &incunit);
 
   /* fprintf(stdout, "  %g  %g  %g  %d", jdelta, jdelta/3600, fmod(jdelta,3600), incperiod%3600);*/
-  int len = fprintf(stdout, " %3lld %s%s", incperiod, tunits[incunit], labs(incperiod) != 1 ? "s" : "");
+  int len = fprintf(stdout, " %3ld %s%s", (long)incperiod, tunits[incunit], labs(incperiod) != 1 ? "s" : "");
   for (int i = 0; i < 11 - len; ++i) fprintf(stdout, " ");
 }
 
@@ -250,7 +250,7 @@ Tinfo(void *process)
 
   cdoInitialize(process);
 
-  int streamID = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID = cdoOpenRead(0);
 
   int vlistID = cdoStreamInqVlist(streamID);
 
@@ -279,10 +279,10 @@ Tinfo(void *process)
               fprintf(stdout, "     RefTime = %s %s", vdatestr, vtimestr);
 
               int unit = taxisInqTunit(taxisID);
-              if (unit != CDI_UNDEFID) fprintf(stdout, "  Units = %s", tunit2str(unit));
+              if (unit != CDI_UNDEFID) fprintf(stdout, "  Units = %s", tunitToCstr(unit));
 
               int calendar = taxisInqCalendar(taxisID);
-              if (calendar != CDI_UNDEFID) fprintf(stdout, "  Calendar = %s", calendar2str(calendar));
+              if (calendar != CDI_UNDEFID) fprintf(stdout, "  Calendar = %s", calendarToCstr(calendar));
 
               if (taxisHasBounds(taxisID)) fprintf(stdout, "  Bounds = true");
 
@@ -299,7 +299,7 @@ Tinfo(void *process)
                   fprintf(stdout, "     Forecast RefTime = %s %s", vdatestr, vtimestr);
 
                   unit = taxisInqForecastTunit(taxisID);
-                  if (unit != CDI_UNDEFID) fprintf(stdout, "  Units = %s", tunit2str(unit));
+                  if (unit != CDI_UNDEFID) fprintf(stdout, "  Units = %s", tunitToCstr(unit));
 
                   fprintf(stdout, "\n");
 
@@ -345,7 +345,7 @@ Tinfo(void *process)
               getTimeInc(jdelta, vdate0, vdate, &incperiod, &incunit);
 
               /* fprintf(stdout, "  %g  %g  %g  %d", jdelta, jdelta/3600, fmod(jdelta,3600), incperiod%3600);*/
-              len = fprintf(stdout, " %3lld %s%s", incperiod, tunits[incunit], labs(incperiod) != 1 ? "s" : "");
+              len = fprintf(stdout, " %3ld %s%s", (long)incperiod, tunits[incunit], labs(incperiod) != 1 ? "s" : "");
               for (i = 0; i < 11 - len; ++i) fprintf(stdout, " ");
             }
           else
@@ -440,7 +440,7 @@ Tinfo(void *process)
   time2str(vtime, vtimestr, sizeof(vtimestr));
   fprintf(stdout, " End date            : %s %s\n", vdatestr, vtimestr);
 
-  fprintf(stdout, " Increment           : %3lld %s%s\n", incperiod0, tunits[incunit0], incperiod0 != 1 ? "s" : "");
+  fprintf(stdout, " Increment           : %3ld %s%s\n", (long)incperiod0, tunits[incunit0], incperiod0 != 1 ? "s" : "");
   fprintf(stdout, " Number of timesteps : %d\n", tsID);
   fprintf(stdout, " Gaps identified     : %d\n", ngaps);
 
