@@ -18,8 +18,8 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
-#include "cdo_defaultValues.h" // Namespace CdoDefault
+
+#include "cdo_defaultValues.h"  // Namespace CdoDefault
 
 void *
 Tocomplex(void *process)
@@ -35,7 +35,7 @@ Tocomplex(void *process)
 
   int operatorID = cdoOperatorID();
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   int vlistID1 = cdoStreamInqVlist(streamID1);
   int vlistID2 = vlistDuplicate(vlistID1);
@@ -53,8 +53,8 @@ Tocomplex(void *process)
   vlistDefTaxis(vlistID2, taxisID2);
 
   // if (CdoDefault::FileType != CDI_FILETYPE_EXT) cdoAbort("Complex numbers need EXTRA format; used CDO option -f ext!");
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   size_t gridsize = vlistGridsizeMax(vlistID1);
   std::vector<double> array1(gridsize);
@@ -65,16 +65,16 @@ Tocomplex(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID2++);
+      cdoDefTimestep(streamID2, tsID2++);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamDefRecord(streamID2, varID, levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoDefRecord(streamID2, varID, levelID);
 
           gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
 
-          pstreamReadRecord(streamID1, array1.data(), &nmiss);
+          cdoReadRecord(streamID1, array1.data(), &nmiss);
 
           if (operatorID == RETOCOMPLEX)
             {
@@ -93,7 +93,7 @@ Tocomplex(void *process)
                 }
             }
 
-          pstreamWriteRecord(streamID2, array2.data(), nmiss);
+          cdoWriteRecord(streamID2, array2.data(), nmiss);
         }
 
       tsID++;
