@@ -24,7 +24,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 
 void
 transxy(int gridID, double *array1, double *array2)
@@ -63,7 +62,7 @@ Transpose(void *process)
 
   cdoInitialize(process);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   int vlistID1 = cdoStreamInqVlist(streamID1);
   int vlistID2 = vlistDuplicate(vlistID1);
@@ -88,8 +87,8 @@ Transpose(void *process)
   int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   size_t gridsize = vlistGridsizeMax(vlistID1);
 
@@ -100,18 +99,18 @@ Transpose(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, array1.data(), &nmiss);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, array1.data(), &nmiss);
 
           int gridID = vlistInqVarGrid(vlistID1, varID);
           transxy(gridID, array1.data(), array2.data());
 
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, array2.data(), nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, array2.data(), nmiss);
         }
 
       tsID++;
