@@ -25,14 +25,13 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
 #include "cdo_options.h"
 #include "cimdOmp.h"
 
 #define NALLOC_INC 1024
 
 static double
-tstepcount(long nts, double missval1, double *array1, double refval)
+tstepcount(long nts, double missval1, const std::vector<double> &array1, double refval)
 {
   long j;
   long n = 0;
@@ -53,47 +52,41 @@ Tstepcount(void *process)
 {
   int nrecs;
   int gridID, varID, levelID;
-  int nalloc = 0;
   size_t nmiss;
   int64_t vdate = 0;
   int vtime = 0;
-  double refval = 0;
   struct memory_t
   {
-    double *array1;
+    std::vector<double> array1;
   };
 
   cdoInitialize(process);
 
-  if (operatorArgc() == 1) refval = parameter2double(operatorArgv()[0]);
+  double refval = (operatorArgc() == 1) ? parameter2double(operatorArgv()[0]) : 0;
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
   vlistDefNtsteps(vlistID2, 1);
 
-  int nvars = vlistNvars(vlistID1);
+  const int nvars = vlistNvars(vlistID1);
   for (varID = 0; varID < nvars; varID++) vlistDefVarUnits(vlistID2, varID, "steps");
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   FieldVector3D vars;
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
-      if (tsID >= nalloc)
-        {
-          nalloc += NALLOC_INC;
-          vars.resize(nalloc);
-        }
+      if ((size_t)tsID >= vars.size()) vars.resize(vars.size() + NALLOC_INC);
 
       vdate = taxisInqVdate(taxisID1);
       vtime = taxisInqVtime(taxisID1);
@@ -102,11 +95,11 @@ Tstepcount(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
           gridID = vlistInqVarGrid(vlistID1, varID);
-          size_t gridsize = gridInqSize(gridID);
-          vars[tsID][varID][levelID].ptr = (double *) Malloc(gridsize * sizeof(double));
-          pstreamReadRecord(streamID1, vars[tsID][varID][levelID].ptr, &nmiss);
+          const size_t gridsize = gridInqSize(gridID);
+          vars[tsID][varID][levelID].resize(gridsize);
+          cdoReadRecord(streamID1, vars[tsID][varID][levelID].vec.data(), &nmiss);
           vars[tsID][varID][levelID].nmiss = nmiss;
         }
 
@@ -115,55 +108,46 @@ Tstepcount(void *process)
 
   int nts = tsID;
 
-  memory_t *mem = (memory_t *) Malloc(Threading::ompNumThreads * sizeof(memory_t));
-  for (int i = 0; i < Threading::ompNumThreads; i++) mem[i].array1 = (double *) Malloc(nts * sizeof(double));
+  std::vector<memory_t> mem(Threading::ompNumThreads);
+  for (int i = 0; i < Threading::ompNumThreads; i++) mem[i].array1.resize(nts);
 
   for (varID = 0; varID < nvars; varID++)
     {
       gridID = vlistInqVarGrid(vlistID1, varID);
       double missval = vlistInqVarMissval(vlistID1, varID);
       size_t gridsize = gridInqSize(gridID);
-      int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-      for (levelID = 0; levelID < nlevel; levelID++)
+      const int nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+      for (levelID = 0; levelID < nlevels; levelID++)
         {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(gridsize, mem, vars, varID, levelID, nts, missval, refval) schedule(dynamic, 1)
+#pragma omp parallel for default(none) shared(gridsize, nts, refval, mem, vars, varID, levelID, missval) schedule(dynamic, 1)
 #endif
           for (size_t i = 0; i < gridsize; i++)
             {
-              int ompthID = cdo_omp_get_thread_num();
+              const int ompthID = cdo_omp_get_thread_num();
 
-              for (int tsID = 0; tsID < nts; tsID++) mem[ompthID].array1[tsID] = vars[tsID][varID][levelID].ptr[i];
+              for (int tsID = 0; tsID < nts; tsID++) mem[ompthID].array1[tsID] = vars[tsID][varID][levelID].vec[i];
 
-              double count = tstepcount(nts, missval, mem[ompthID].array1, refval);
-              vars[0][varID][levelID].ptr[i] = count;
+              const double count = tstepcount(nts, missval, mem[ompthID].array1, refval);
+              vars[0][varID][levelID].vec[i] = count;
             }
         }
     }
 
-  for (int i = 0; i < Threading::ompNumThreads; i++) Free(mem[i].array1);
-  Free(mem);
-
   taxisDefVdate(taxisID2, vdate);
   taxisDefVtime(taxisID2, vtime);
-  pstreamDefTimestep(streamID2, 0);
+  cdoDefTimestep(streamID2, 0);
 
   for (varID = 0; varID < nvars; varID++)
     {
-      gridID = vlistInqVarGrid(vlistID2, varID);
-      double missval = vlistInqVarMissval(vlistID2, varID);
-      size_t gridsize = gridInqSize(gridID);
-      int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
-      for (levelID = 0; levelID < nlevel; levelID++)
+      const int nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
+      for (levelID = 0; levelID < nlevels; levelID++)
         {
-          nmiss = arrayNumMV(gridsize, vars[0][varID][levelID].ptr, missval);
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, vars[0][varID][levelID].ptr, nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, vars[0][varID][levelID].vec.data(), fieldNumMiss(vars[0][varID][levelID]));
         }
     }
 
-  for (tsID = 0; tsID < nts; tsID++) fieldsFree(vlistID1, vars[tsID]);
-
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
