@@ -26,19 +26,21 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"  // ENABLE_DATA
 #endif
+
 #include <cassert>
 #include <cdi.h>
 
 #include "cdo_int.h"
+#include "cdo_zaxis.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
 #include "listarray.h"
-#include "grid.h"
+#include <mpim_grid.h>
+#include "griddes.h"
 #include "constants.h"
 #include "stdnametable.h"
 #include "timebase.h"
 
-#if defined(ENABLE_DATA)
+#ifdef ENABLE_DATA
 static const double etopo_scale = 3;
 static const double etopo_offset = 11000;
 static const unsigned short etopo[] = {
@@ -59,30 +61,30 @@ static const unsigned short mask[] = {
 #endif
 
 // Some Constants for creating temperatur and pressure for the standard atmosphere
-#define T_ZERO      (213.0)
-#define T_DELTA     (75.0)
-#define SCALEHEIGHT (10000.0) // [m]
-#define P_ZERO      (1013.25) // surface pressure [hPa]
-#define CC_R        (287.05)  // specific gas constant for air
+#define T_ZERO (213.0)
+#define T_DELTA (75.0)
+#define SCALEHEIGHT (10000.0)  // [m]
+#define P_ZERO (1013.25)       // surface pressure [hPa]
+#define CC_R (287.05)          // specific gas constant for air
 
 constexpr double TMP4PRESSURE = (C_EARTH_GRAV * SCALEHEIGHT) / (CC_R * T_ZERO);
 
-static
-double std_atm_temperatur(double height)
+static double
+std_atm_temperatur(double height)
 {
   // Compute the temperatur for the given height (in meters) according to the solution of the hydrostatic atmosphere
   return (T_ZERO + T_DELTA * exp((-1) * (height / SCALEHEIGHT)));
 }
 
-static
-double std_atm_pressure(double height)
+static double
+std_atm_pressure(double height)
 {
   // Compute the pressure for the given height (in meters) according to the solution of the hydrostatic atmosphere
   return (P_ZERO * exp((-1) * TMP4PRESSURE * log((exp(height / SCALEHEIGHT) * T_ZERO + T_DELTA) / (T_ZERO + T_DELTA))));
 }
 
-static
-void conv_generic_grid(const int gridID, const size_t gridsize, std::vector<double> &xvals2D, std::vector<double> &yvals2D)
+static void
+conv_generic_grid(const int gridID, const size_t gridsize, std::vector<double> &xvals2D, std::vector<double> &yvals2D)
 {
   const size_t xsize = gridInqXsize(gridID);
   const size_t ysize = gridInqYsize(gridID);
@@ -96,10 +98,10 @@ void conv_generic_grid(const int gridID, const size_t gridsize, std::vector<doub
   gridInqYvals(gridID, &ycoord[0]);
 
   double xmin, xmax;
-  arrayMinMax(xsize, &xcoord[0], &xmin, &xmax);
+  arrayMinMax(xsize, &xcoord[0], xmin, xmax);
 
   double ymin, ymax;
-  arrayMinMax(ysize, &ycoord[0], &ymin, &ymax);
+  arrayMinMax(ysize, &ycoord[0], ymin, ymax);
 
   const double xrange = xmax - xmin;
   const double yrange = ymax - ymin;
@@ -119,8 +121,7 @@ remap_nn_reg2d_reg2d(const size_t nx, const size_t ny, const double *restrict da
 
   const size_t nxvals = gridInqXsize(gridID);
   const size_t nyvals = gridInqYsize(gridID);
-  std::vector<double> xvals(nxvals);
-  std::vector<double> yvals(nyvals);
+  std::vector<double> xvals(nxvals), yvals(nyvals);
 
   gridInqXvals(gridID, &xvals[0]);
   gridInqYvals(gridID, &yvals[0]);
@@ -134,7 +135,7 @@ remap_nn_reg2d_reg2d(const size_t nx, const size_t ny, const double *restrict da
 
   for (size_t j = 0; j < nyvals; j++)
     {
-      double yval = yvals[j];
+      const double yval = yvals[j];
       for (size_t i = 0; i < nxvals; i++)
         {
           double xval = xvals[i];
@@ -154,10 +155,10 @@ remap_nn_reg2d_nonreg2d(const size_t nx, const size_t ny, const double *restrict
 {
   int gridID2 = gridID;
   const size_t gridsize = gridInqSize(gridID2);
-  std::vector<double> xvals(gridsize);
-  std::vector<double> yvals(gridsize);
+  std::vector<double> xvals(gridsize), yvals(gridsize);
 
-  if (gridInqType(gridID2) == GRID_GME) gridID2 = gridToUnstructured(gridID2, 0);
+  if (gridInqType(gridID2) == GRID_GME || gridInqType(gridID2) == GRID_GAUSSIAN_REDUCED)
+    gridID2 = gridToUnstructured(gridID2, 0);
 
   if (gridInqType(gridID2) != GRID_UNSTRUCTURED && gridInqType(gridID2) != GRID_CURVILINEAR)
     gridID2 = gridToCurvilinear(gridID2, 0);
@@ -197,8 +198,8 @@ remap_nn_reg2d(const size_t nx, const size_t ny, const double *restrict data, co
     remap_nn_reg2d_nonreg2d(nx, ny, data, gridID, array);
 }
 
-static
-int randomInit(const int operatorID)
+static int
+randomInit(const int operatorID)
 {
   unsigned int seed = 1;
   operatorInputArg(cdoOperatorEnter(operatorID));
@@ -214,34 +215,33 @@ int randomInit(const int operatorID)
   return gridID;
 }
 
-static
-void randomCompute(const size_t gridsize, std::vector<double> &array)
+static void
+randomCompute(const size_t gridsize, std::vector<double> &array)
 {
   for (size_t i = 0; i < gridsize; ++i) array[i] = ((double) rand()) / ((double) RAND_MAX);
 }
 
-static
-void sincosCompute(const size_t gridsize, std::vector<double> &array,
-                   const std::vector<double> &xvals, const std::vector<double> &yvals)
+static void
+sincosCompute(const size_t gridsize, std::vector<double> &array, const std::vector<double> &xvals, const std::vector<double> &yvals)
 {
   for (size_t i = 0; i < gridsize; ++i) array[i] = cos(1.0 * xvals[i]) * sin(2.0 * yvals[i]);
 }
 
-static
-void coshillCompute(const size_t gridsize, std::vector<double> &array,
-                    const std::vector<double> &xvals, const std::vector<double> &yvals)
+static void
+coshillCompute(const size_t gridsize, std::vector<double> &array, const std::vector<double> &xvals,
+               const std::vector<double> &yvals)
 {
   for (size_t i = 0; i < gridsize; ++i) array[i] = 2 - cos(acos(cos(xvals[i]) * cos(yvals[i])) / 1.2);
 }
 
-static
-void unpackData(const size_t datasize, std::vector<double> &data, const double scale, const double offset, const unsigned short *zdata)
+static void
+unpackData(const size_t datasize, std::vector<double> &data, const double scale, const double offset, const unsigned short *zdata)
 {
   for (size_t i = 0; i < datasize; ++i) data[i] = zdata[i] / scale - offset;
 }
 
-static
-int definePointGrid()
+static int
+definePointGrid()
 {
   int gridID = gridCreate(GRID_LONLAT, 1);
   gridDefXsize(gridID, 1);
@@ -253,8 +253,8 @@ int definePointGrid()
   return gridID;
 }
 
-static
-int defineZaxis(const bool lstdatm, const int nlevels, double *levels)
+static int
+defineZaxis(const bool lstdatm, const int nlevels, double *levels)
 {
   int zaxisID = -1;
 
@@ -268,14 +268,14 @@ int defineZaxis(const bool lstdatm, const int nlevels, double *levels)
     }
   else
     {
-      zaxisID = zaxisCreate(ZAXIS_SURFACE, 1);
+      zaxisID = zaxisFromName("surface");
     }
 
   return zaxisID;
 }
 
-static
-void definePressureAttributes(const int vlistID, const int varID)
+static void
+definePressureAttributes(const int vlistID, const int varID)
 {
   vlistDefVarName(vlistID, varID, "P");
   vlistDefVarParam(vlistID, varID, cdiEncodeParam(1, 255, 255));
@@ -284,8 +284,8 @@ void definePressureAttributes(const int vlistID, const int varID)
   vlistDefVarUnits(vlistID, varID, "hPa");
 }
 
-static
-void defineTemperatureAttributes(const int vlistID, const int varID)
+static void
+defineTemperatureAttributes(const int vlistID, const int varID)
 {
   vlistDefVarName(vlistID, varID, "T");
   vlistDefVarParam(vlistID, varID, cdiEncodeParam(130, 128, 255));
@@ -314,7 +314,7 @@ Vargen(void *process)
   const int SINCOS  = cdoOperatorAdd("sincos",  0, 0, "grid description file or name");
   const int COSHILL = cdoOperatorAdd("coshill", 0, 0, "grid description file or name");
   const int CONST   = cdoOperatorAdd("const",   0, 0, "constant value, grid description file or name");
-  const int FOR     = cdoOperatorAdd("for",     0, 0, "start, end, <increment>");
+  const int SEQ     = cdoOperatorAdd("seq",     0, 0, "start, end, <increment>");
   const int TOPO    = cdoOperatorAdd("topo",    0, 0, nullptr);
   const int TEMP    = cdoOperatorAdd("temp",    0, 0, nullptr);
   const int MASK    = cdoOperatorAdd("mask",    0, 0, nullptr);
@@ -357,7 +357,7 @@ Vargen(void *process)
       if (operatorArgc() == 1) gridID = cdoDefineGrid(operatorArgv()[0]);
       if (operatorArgc() > 1) cdoAbort("Too many arguments!");
     }
-  else if (operatorID == FOR)
+  else if (operatorID == SEQ)
     {
       operatorInputArg(cdoOperatorEnter(operatorID));
       if (operatorArgc() < 2) cdoAbort("Too few arguments!");
@@ -386,7 +386,7 @@ Vargen(void *process)
 
   const int vlistID = vlistCreate();
 
-  const int timetype = (operatorID == FOR) ? TIME_VARYING : TIME_CONSTANT;
+  const int timetype = (operatorID == SEQ) ? TIME_VARYING : TIME_CONSTANT;
 
   int varID = vlistDefVar(vlistID, gridID, zaxisID, timetype);
   /*
@@ -417,14 +417,14 @@ Vargen(void *process)
       || operatorID == TEMP || operatorID == MASK || operatorID == STDATM)
     vlistDefNtsteps(vlistID, 1);
 
-  const int streamID = cdoStreamOpenWrite(0);
+  CdoStreamID streamID = cdoOpenWrite(0);
 
-  pstreamDefVlist(streamID, vlistID);
+  cdoDefVlist(streamID, vlistID);
 
   const size_t gridsize = gridInqSize(gridID);
   std::vector<double> array(gridsize);
 
-  if (operatorID == FOR)
+  if (operatorID == SEQ)
     ntimesteps = 1.001 + ((rstop - rstart) / rinc);
   else
     {
@@ -443,14 +443,14 @@ Vargen(void *process)
       const int vtime = 0;
       taxisDefVdate(taxisID, vdate);
       taxisDefVtime(taxisID, vtime);
-      pstreamDefTimestep(streamID, tsID);
+      cdoDefTimestep(streamID, tsID);
 
       for (varID = 0; varID < nvars; varID++)
         {
           nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID, varID));
           for (int levelID = 0; levelID < nlevels; levelID++)
             {
-              pstreamDefRecord(streamID, varID, levelID);
+              cdoDefRecord(streamID, varID, levelID);
 
               if (operatorID == RANDOM)
                 {
@@ -497,7 +497,7 @@ Vargen(void *process)
                 }
               else if (operatorID == TOPO || operatorID == TEMP || operatorID == MASK)
                 {
-#if defined(ENABLE_DATA)
+#ifdef ENABLE_DATA
                   const size_t datasize = gridInqSize(gridIDdata);
                   std::vector<double> data(datasize);
 
@@ -519,7 +519,7 @@ Vargen(void *process)
                   cdoAbort("Operator support disabled!");
 #endif
                 }
-              else if (operatorID == FOR)
+              else if (operatorID == SEQ)
                 {
                   array[0] = rval;
                 }
@@ -528,7 +528,7 @@ Vargen(void *process)
                   array[0] = (varID == varID2) ? std_atm_temperatur(levels[levelID]) : std_atm_pressure(levels[levelID]);
                 }
 
-              pstreamWriteRecord(streamID, array.data(), 0);
+              cdoWriteRecord(streamID, array.data(), 0);
             }
         }
     }
