@@ -18,78 +18,42 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "param_conversion.h"
-#include "pstream_int.h"
-#include "pmlist.h"
 
-
-void *
-Varsstat(void *process)
+static void
+checkUniqueZaxis(int vlistID)
 {
-  int nrecs;
-  int varID, levelID;
-  size_t nmiss;
-
-  cdoInitialize(process);
-
-  // clang-format off
-  cdoOperatorAdd("varsrange", func_range, 0, nullptr);
-  cdoOperatorAdd("varsmin",   func_min,   0, nullptr);
-  cdoOperatorAdd("varsmax",   func_max,   0, nullptr);
-  cdoOperatorAdd("varssum",   func_sum,   0, nullptr);
-  cdoOperatorAdd("varsmean",  func_mean,  0, nullptr);
-  cdoOperatorAdd("varsavg",   func_avg,   0, nullptr);
-  cdoOperatorAdd("varsvar",   func_var,   0, nullptr);
-  cdoOperatorAdd("varsvar1",  func_var1,  0, nullptr);
-  cdoOperatorAdd("varsstd",   func_std,   0, nullptr);
-  cdoOperatorAdd("varsstd1",  func_std1,  0, nullptr);
-
-  const int operatorID = cdoOperatorID();
-  const int operfunc   = cdoOperatorF1(operatorID);
-
-  const bool lrange  = operfunc == func_range;
-  const bool lmean   = operfunc == func_mean || operfunc == func_avg;
-  const bool lstd    = operfunc == func_std || operfunc == func_std1;
-  const bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
-  const int  divisor = operfunc == func_std1 || operfunc == func_var1;
-  // clang-format on
-
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-  const int vlistID1 = cdoStreamInqVlist(streamID1);
-
-  const int nzaxis = vlistNzaxis(vlistID1);
-  const int zaxisID = vlistZaxis(vlistID1, 0);
-  const int nlevs = zaxisInqSize(zaxisID);
+  const int nzaxis = vlistNzaxis(vlistID);
+  const int zaxisID = vlistZaxis(vlistID, 0);
+  const int nlevels = zaxisInqSize(zaxisID);
   for (int index = 1; index < nzaxis; ++index)
     {
-      if (nlevs != zaxisInqSize(vlistZaxis(vlistID1, index)))
-        cdoAbort("Number of level differ!");
+      if (nlevels != zaxisInqSize(vlistZaxis(vlistID, index))) cdoAbort("Number of level differ!");
     }
+}
 
-  const int ngrids = vlistNgrids(vlistID1);
-  const int gridID = vlistGrid(vlistID1, 0);
+static void
+checkUniqueGridsize(int vlistID)
+{
+  const int ngrids = vlistNgrids(vlistID);
+  const int gridID = vlistGrid(vlistID, 0);
   const size_t gridsize = gridInqSize(gridID);
   for (int index = 0; index < ngrids; index++)
     {
-      if (gridsize != gridInqSize(vlistGrid(vlistID1, index)))
-        cdoAbort("Horizontal gridsize differ!");
-    }
-
-  const int timetype = vlistInqVarTimetype(vlistID1, 0);
-  const int nvars = vlistNvars(vlistID1);
-  for (varID = 1; varID < nvars; ++varID)
-    {
-      if (timetype != vlistInqVarTimetype(vlistID1, varID))
-        cdoAbort("Number of timesteps differ!");
+      if (gridsize != gridInqSize(vlistGrid(vlistID, index))) cdoAbort("Horizontal gridsize differ!");
     }
+}
 
+static void
+setAttributes(int vlistID1, int vlistID2, int varID2, int operatorID)
+{
   bool paramIsEqual = true;
   char name[CDI_MAX_NAME], zname[CDI_MAX_NAME];
-  int param = vlistInqVarParam(vlistID1, 0);
+  const int param = vlistInqVarParam(vlistID1, 0);
   vlistInqVarName(vlistID1, 0, name);
-  for (varID = 1; varID < nvars; ++varID)
+  int nvars = vlistNvars(vlistID1);
+  for (int varID = 1; varID < nvars; ++varID)
     {
-      int zparam = vlistInqVarParam(vlistID1, varID);
+      const int zparam = vlistInqVarParam(vlistID1, varID);
       vlistInqVarName(vlistID1, varID, zname);
       if (param != zparam || !cstrIsEqual(name, zname))
         {
@@ -98,8 +62,6 @@ Varsstat(void *process)
         }
     }
 
-  const int vlistID2 = vlistCreate();
-  const int varID2 = vlistDefVar(vlistID2, gridID, zaxisID, timetype);
   if (!paramIsEqual) strcpy(name, cdoOperatorName(operatorID));
   vlistDefVarName(vlistID2, varID2, name);
   if (paramIsEqual)
@@ -112,48 +74,97 @@ Varsstat(void *process)
       vlistInqVarUnits(vlistID1, 0, zname);
       if (*zname) vlistDefVarUnits(vlistID2, varID2, zname);
     }
+}
+
+static void
+addOperators(void)
+{
+  // clang-format off
+  cdoOperatorAdd("varsrange", func_range, 0, nullptr);
+  cdoOperatorAdd("varsmin",   func_min,   0, nullptr);
+  cdoOperatorAdd("varsmax",   func_max,   0, nullptr);
+  cdoOperatorAdd("varssum",   func_sum,   0, nullptr);
+  cdoOperatorAdd("varsmean",  func_mean,  0, nullptr);
+  cdoOperatorAdd("varsavg",   func_avg,   0, nullptr);
+  cdoOperatorAdd("varsvar",   func_var,   0, nullptr);
+  cdoOperatorAdd("varsvar1",  func_var1,  0, nullptr);
+  cdoOperatorAdd("varsstd",   func_std,   0, nullptr);
+  cdoOperatorAdd("varsstd1",  func_std1,  0, nullptr);
+  // clang-format on
+}
+
+void *
+Varsstat(void *process)
+{
+  int nrecs;
+  int varID, levelID;
+
+  cdoInitialize(process);
+
+  addOperators();
+
+  // clang-format off
+  const int operatorID = cdoOperatorID();
+  const int operfunc   = cdoOperatorF1(operatorID);
+
+  const bool lrange  = operfunc == func_range;
+  const bool lmean   = operfunc == func_mean || operfunc == func_avg;
+  const bool lstd    = operfunc == func_std || operfunc == func_std1;
+  const bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
+  const int  divisor = operfunc == func_std1 || operfunc == func_var1;
+  // clang-format on
+
+  CdoStreamID streamID1 = cdoOpenRead(0);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+
+  checkUniqueZaxis(vlistID1);
+  const int zaxisID = vlistZaxis(vlistID1, 0);
+  const int nlevels = zaxisInqSize(zaxisID);
+
+  checkUniqueGridsize(vlistID1);
+  const int gridID = vlistGrid(vlistID1, 0);
+  const size_t gridsize = gridInqSize(gridID);
+
+  const int timetype = vlistInqVarTimetype(vlistID1, 0);
+  const int nvars = vlistNvars(vlistID1);
+  for (varID = 1; varID < nvars; ++varID)
+    {
+      if (timetype != vlistInqVarTimetype(vlistID1, varID)) cdoAbort("Number of timesteps differ!");
+    }
+
+  const int vlistID2 = vlistCreate();
+  const int varID2 = vlistDefVar(vlistID2, gridID, zaxisID, timetype);
+  setAttributes(vlistID1, vlistID2, varID2, operatorID);
 
   const int taxisID1 = vlistInqTaxis(vlistID1);
   const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
-
-  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   Field field;
-  fieldInit(field);
-  field.ptr = (double *) Malloc(gridsizemax * sizeof(double));
+  field.resize(gridsize);
 
-  std::vector<Field> vars1(nlevs);
-  std::vector<Field> samp1(nlevs);
-  std::vector<Field> vars2;
-  if (lvarstd || lrange) vars2.resize(nlevs);
+  FieldVector vars1(nlevels);
+  FieldVector samp1(nlevels);
+  FieldVector vars2;
+  if (lvarstd || lrange) vars2.resize(nlevels);
 
-  for (levelID = 0; levelID < nlevs; levelID++)
+  for (levelID = 0; levelID < nlevels; levelID++)
     {
-      double missval = vlistInqVarMissval(vlistID1, 0);
+      const double missval = vlistInqVarMissval(vlistID1, 0);
 
-      fieldInit(vars1[levelID]);
-      fieldInit(samp1[levelID]);
       vars1[levelID].grid = gridID;
-      vars1[levelID].zaxis = zaxisID;
-      vars1[levelID].nsamp = 0;
-      vars1[levelID].nmiss = 0;
       vars1[levelID].missval = missval;
-      vars1[levelID].ptr = (double *) Malloc(gridsize * sizeof(double));
+      vars1[levelID].resize(gridsize);
       samp1[levelID].grid = gridID;
-      samp1[levelID].nmiss = 0;
       samp1[levelID].missval = missval;
-      samp1[levelID].ptr = nullptr;
       if (lvarstd || lrange)
         {
-          fieldInit(vars2[levelID]);
           vars2[levelID].grid = gridID;
-          vars2[levelID].nmiss = 0;
           vars2[levelID].missval = missval;
-          vars2[levelID].ptr = (double *) Malloc(gridsize * sizeof(double));
+          vars2[levelID].resize(gridsize);
         }
     }
 
@@ -161,110 +172,100 @@ Varsstat(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           vars1[levelID].nsamp++;
           if (lrange) vars2[levelID].nsamp++;
 
           if (varID == 0)
             {
-              pstreamReadRecord(streamID1, vars1[levelID].ptr, &nmiss);
-              vars1[levelID].nmiss = nmiss;
+              cdoReadRecord(streamID1, vars1[levelID].vec.data(), &vars1[levelID].nmiss);
               if (lrange)
                 {
-                  vars2[levelID].nmiss = nmiss;
-                  for (size_t i = 0; i < gridsize; i++) vars2[levelID].ptr[i] = vars1[levelID].ptr[i];
+                  vars2[levelID].nmiss = vars1[levelID].nmiss;
+                  vars2[levelID].vec = vars1[levelID].vec;
                 }
 
-              if (lvarstd)
-                {
-                  farmoq(vars2[levelID], vars1[levelID]);
-                }
+              if (lvarstd) vfarmoq(vars2[levelID], vars1[levelID]);
 
-              if (nmiss > 0 || samp1[levelID].ptr)
+              if (vars1[levelID].nmiss || !samp1[levelID].empty())
                 {
-                  if (samp1[levelID].ptr == nullptr) samp1[levelID].ptr = (double *) Malloc(gridsize * sizeof(double));
+                  if (samp1[levelID].empty()) samp1[levelID].resize(gridsize);
 
                   for (size_t i = 0; i < gridsize; i++)
-                    if (DBL_IS_EQUAL(vars1[levelID].ptr[i], vars1[levelID].missval))
-                      samp1[levelID].ptr[i] = 0.;
+                    samp1[levelID].vec[i] = !DBL_IS_EQUAL(vars1[levelID].vec[i], vars1[levelID].missval);
                 }
             }
           else
             {
-              pstreamReadRecord(streamID1, field.ptr, &nmiss);
-              field.nmiss = nmiss;
+              cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
               field.grid = vars1[levelID].grid;
               field.missval = vars1[levelID].missval;
 
-              if (field.nmiss > 0 || samp1[levelID].ptr)
+              if (field.nmiss || !samp1[levelID].empty())
                 {
-                  if (samp1[levelID].ptr == nullptr)
-                    {
-                      samp1[levelID].ptr = (double *) Malloc(gridsize * sizeof(double));
-                      for (size_t i = 0; i < gridsize; i++) samp1[levelID].ptr[i] = vars1[levelID].nsamp;
-                    }
+                  if (samp1[levelID].empty()) samp1[levelID].resize(gridsize, vars1[levelID].nsamp);
 
                   for (size_t i = 0; i < gridsize; i++)
-                    if (!DBL_IS_EQUAL(field.ptr[i], vars1[levelID].missval)) samp1[levelID].ptr[i]++;
+                    if (!DBL_IS_EQUAL(field.vec[i], field.missval)) samp1[levelID].vec[i]++;
                 }
 
               if (lvarstd)
                 {
-                  farsumq(vars2[levelID], field);
-                  farsum(vars1[levelID], field);
+                  vfarsumq(vars2[levelID], field);
+                  vfarsum(vars1[levelID], field);
                 }
               else if (lrange)
                 {
-                  farmin(vars2[levelID], field);
-                  farmax(vars1[levelID], field);
+                  vfarmin(vars2[levelID], field);
+                  vfarmax(vars1[levelID], field);
                 }
               else
                 {
-                  farfun(vars1[levelID], field, operfunc);
+                  vfarfun(vars1[levelID], field, operfunc);
                 }
             }
         }
 
-      for (levelID = 0; levelID < nlevs; levelID++)
+      for (levelID = 0; levelID < nlevels; levelID++)
         {
           if (vars1[levelID].nsamp)
             {
               if (lmean)
                 {
-                  if (samp1[levelID].ptr == nullptr)
-                    farcdiv(vars1[levelID], (double) vars1[levelID].nsamp);
+                  if (!samp1[levelID].empty())
+                    vfardiv(vars1[levelID], samp1[levelID]);
                   else
-                    fardiv(vars1[levelID], samp1[levelID]);
+                    vfarcdiv(vars1[levelID], (double) vars1[levelID].nsamp);
                 }
               else if (lvarstd)
                 {
-                  if (samp1[levelID].ptr == nullptr)
+                  if (!samp1[levelID].empty())
                     {
                       if (lstd)
-                        farcstd(vars1[levelID], vars2[levelID], vars1[levelID].nsamp, divisor);
+                        vfarstd(vars1[levelID], vars2[levelID], samp1[levelID], divisor);
                       else
-                        farcvar(vars1[levelID], vars2[levelID], vars1[levelID].nsamp, divisor);
+                        vfarvar(vars1[levelID], vars2[levelID], samp1[levelID], divisor);
                     }
                   else
                     {
                       if (lstd)
-                        farstd(vars1[levelID], vars2[levelID], samp1[levelID], divisor);
+                        vfarcstd(vars1[levelID], vars2[levelID], vars1[levelID].nsamp, divisor);
                       else
-                        farvar(vars1[levelID], vars2[levelID], samp1[levelID], divisor);
+                        vfarcvar(vars1[levelID], vars2[levelID], vars1[levelID].nsamp, divisor);
                     }
                 }
               else if (lrange)
                 {
-                  farsub(vars1[levelID], vars2[levelID]);
+                  vfarsub(vars1[levelID], vars2[levelID]);
                 }
 
-              pstreamDefRecord(streamID2, 0, levelID);
-              pstreamWriteRecord(streamID2, vars1[levelID].ptr, vars1[levelID].nmiss);
+              cdoDefRecord(streamID2, 0, levelID);
+              cdoWriteRecord(streamID2, vars1[levelID].vec.data(), vars1[levelID].nmiss);
               vars1[levelID].nsamp = 0;
             }
         }
@@ -272,15 +273,6 @@ Varsstat(void *process)
       tsID++;
     }
 
-  for (levelID = 0; levelID < nlevs; levelID++)
-    {
-      Free(vars1[levelID].ptr);
-      if (samp1[levelID].ptr) Free(samp1[levelID].ptr);
-      if (lvarstd) Free(vars2[levelID].ptr);
-    }
-
-  if (field.ptr) Free(field.ptr);
-
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
