@@ -21,12 +21,9 @@
 
 #include <cdi.h>
 #include "cdo_int.h"
-#include "grid.h"
+#include <mpim_grid.h>
+#include "gridreference.h"
 #include "verifygrid.h"
-#include "pstream_int.h"
-extern "C" {
-#include "lib/yac/geometry.h"
-}
 
 static void
 quick_sort(double *array, size_t array_length)
@@ -329,9 +326,9 @@ verify_grid(int gridtype, size_t gridsize, int gridno, int ngrids, int ncorner,
   for (int i = 0; i < ncorner; i++) no_cells_with_a_specific_no_of_corners[i] = 0;
 
   if (ngrids == 1)
-    cdoPrintBlue("Grid consists of %zu cells (type: %s), of which", gridsize, gridNamePtr(gridtype));
+    cdoPrint(Blue("Grid consists of %zu cells (type: %s), of which"), gridsize, gridNamePtr(gridtype));
   else
-    cdoPrintBlue("Grid no %u (of %u) consists of %zu cells (type: %s), of which", gridno + 1, ngrids, gridsize,
+    cdoPrint(Blue("Grid no %u (of %u) consists of %zu cells (type: %s), of which"), gridno + 1, ngrids, gridsize,
                  gridNamePtr(gridtype));
   // cdoPrint("");
 
@@ -365,7 +362,8 @@ verify_grid(int gridtype, size_t gridsize, int gridno, int ngrids, int ncorner,
       if (std::fabs(center_point_array[cell_no * 2 + 0] - center_point_array[(cell_no + 1) * 2 + 0]) > eps)
         {
           subarray_end = cell_no * 2;
-          if ((subarray_end - subarray_start) > 1) quick_sort_of_subarray_by_lat(center_point_array.data(), subarray_start, subarray_end);
+          if ((subarray_end - subarray_start) > 1)
+            quick_sort_of_subarray_by_lat(center_point_array.data(), subarray_start, subarray_end);
 
           subarray_start = subarray_end + 2;
         }
@@ -397,25 +395,22 @@ verify_grid(int gridtype, size_t gridsize, int gridno, int ngrids, int ncorner,
   /*
      Latitude and longitude are spherical coordinates on a unit circle. Each such coordinate tuple is transformed into a
      triple of Cartesian coordinates in Euclidean space. This is first done for the presumed center point of the cell
-     and then for all the corners of the cell. LLtoXYZ_deg is defined in clipping/geometry.h
+     and then for all the corners of the cell.
   */
 
   for (size_t cell_no = 0; cell_no < gridsize; cell_no++)
     {
-      /* Conversion of center point spherical coordinates to Cartesian coordinates. */
-
-      LLtoXYZ_deg(grid_center_lon[cell_no], grid_center_lat[cell_no], center_point_xyz);
+      // Conversion of center point spherical coordinates to Cartesian coordinates.
+      gcLLtoXYZdeg(grid_center_lon[cell_no], grid_center_lat[cell_no], center_point_xyz);
 
       for (int corner_no = 0; corner_no < ncorner; corner_no++)
         {
-          /* Conversion of corner spherical coordinates to Cartesian coordinates. */
+          // Conversion of corner spherical coordinates to Cartesian coordinates.
+          gcLLtoXYZdeg(grid_corner_lon[cell_no * ncorner + corner_no], grid_corner_lat[cell_no * ncorner + corner_no],
+                       corner_coordinates);
 
-          LLtoXYZ_deg(grid_corner_lon[cell_no * ncorner + corner_no], grid_corner_lat[cell_no * ncorner + corner_no],
-                      corner_coordinates);
-
-          /* The components of the result vector are appended to the list of cell corner coordinates. */
-
-          int off = corner_no * 3;
+          // The components of the result vector are appended to the list of cell corner coordinates.
+          const int off = corner_no * 3;
           cell_corners_xyz_open_cell[off + 0] = corner_coordinates[0];
           cell_corners_xyz_open_cell[off + 1] = corner_coordinates[1];
           cell_corners_xyz_open_cell[off + 2] = corner_coordinates[2];
@@ -449,8 +444,9 @@ verify_grid(int gridtype, size_t gridsize, int gridno, int ngrids, int ncorner,
       if (actual_number_of_corners < 3)
         {
           if (Options::cdoVerbose)
-            fprintf(stdout, "Less than three vertices found in cell no %zu. This cell is considered degenerate and "
-                            "will be omitted from further computation!\n",
+            fprintf(stdout,
+                    "Less than three vertices found in cell no %zu. This cell is considered degenerate and "
+                    "will be omitted from further computation!\n",
                     cell_no + 1);
 
           continue;
@@ -506,8 +502,9 @@ verify_grid(int gridtype, size_t gridsize, int gridno, int ngrids, int ncorner,
       if (actual_number_of_corners < 3)
         {
           if (Options::cdoVerbose)
-            fprintf(stdout, "Less than three vertices found in cell no %zu. This cell is considered degenerate and "
-                            "will be omitted from further computation!\n",
+            fprintf(stdout,
+                    "Less than three vertices found in cell no %zu. This cell is considered degenerate and "
+                    "will be omitted from further computation!\n",
                     cell_no + 1);
 
           continue;
@@ -592,8 +589,8 @@ verify_grid(int gridtype, size_t gridsize, int gridno, int ngrids, int ncorner,
         no_counterclockwise_cells += 1;
 
       // The winding numbers algorithm is used to test whether the presumed center point is within the bounds of the cell.
-      int winding_number
-        = winding_numbers_algorithm(cell_corners_plane_projection.data(), actual_number_of_corners + 1, center_point_plane_projection);
+      int winding_number = winding_numbers_algorithm(cell_corners_plane_projection.data(), actual_number_of_corners + 1,
+                                                     center_point_plane_projection);
 
       // if ( winding_number == 0 ) printf("%d,", cell_no+1);
       if (winding_number == 0) no_of_cells_with_center_points_out_of_bounds += 1;
@@ -614,20 +611,20 @@ verify_grid(int gridtype, size_t gridsize, int gridno, int ngrids, int ncorner,
 
   for (int i = 2; i < ncorner; i++)
     if (no_cells_with_a_specific_no_of_corners[i])
-      cdoPrintBlue("%9d cells have %d vertices", no_cells_with_a_specific_no_of_corners[i], i + 1);
+      cdoPrint(Blue("%9d cells have %d vertices"), no_cells_with_a_specific_no_of_corners[i], i + 1);
 
-  if (no_of_cells_with_duplicates) cdoPrintBlue("%9zu cells have duplicate vertices", no_of_cells_with_duplicates);
+  if (no_of_cells_with_duplicates) cdoPrint(Blue("%9zu cells have duplicate vertices"), no_of_cells_with_duplicates);
 
-  if (no_nonusable_cells) cdoPrintRed("%9zu cells have unusable vertices", no_nonusable_cells);
+  if (no_nonusable_cells) cdoPrint(Red("%9zu cells have unusable vertices"), no_nonusable_cells);
 
-  if (no_nonunique_cells) cdoPrintRed("%9zu cells are not unique", no_nonunique_cells);
+  if (no_nonunique_cells) cdoPrint(Red("%9zu cells are not unique"), no_nonunique_cells);
 
-  if (no_nonconvex_cells) cdoPrintRed("%9zu cells are non-convex", no_nonconvex_cells);
+  if (no_nonconvex_cells) cdoPrint(Red("%9zu cells are non-convex"), no_nonconvex_cells);
 
-  if (no_clockwise_cells) cdoPrintRed("%9zu cells have their vertices arranged in a clockwise order", no_clockwise_cells);
+  if (no_clockwise_cells) cdoPrint(Red("%9zu cells have their vertices arranged in a clockwise order"), no_clockwise_cells);
 
   if (no_of_cells_with_center_points_out_of_bounds)
-    cdoPrintRed("%9zu cells have their center points located outside their boundaries",
+    cdoPrint(Red("%9zu cells have their center points located outside their boundaries"),
                 no_of_cells_with_center_points_out_of_bounds);
 
   // cdoPrint("");
@@ -644,9 +641,9 @@ Verifygrid(void *argument)
 
   int operatorID = cdoOperatorID();
 
-  int streamID = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID = cdoOpenRead(0);
 
-  int vlistID = pstreamInqVlist(streamID);
+  int vlistID = cdoStreamInqVlist(streamID);
 
   int ngrids = vlistNgrids(vlistID);
   for (int gridno = 0; gridno < ngrids; ++gridno)
@@ -683,7 +680,6 @@ Verifygrid(void *argument)
             }
         }
 
-
       size_t gridsize = gridInqSize(gridID);
 
       if (lgeo)
@@ -734,9 +730,9 @@ Verifygrid(void *argument)
       else
         {
           if (ngrids == 1)
-            cdoPrintBlue("Grid consists of %zu points (type: %s)", gridsize, gridNamePtr(gridtype));
+            cdoPrint(Blue("Grid consists of %zu points (type: %s)"), gridsize, gridNamePtr(gridtype));
           else
-            cdoPrintBlue("Grid no %u (of %u) consists of %zu points (type: %s)", gridno + 1, ngrids, gridsize,
+            cdoPrint(Blue("Grid no %u (of %u) consists of %zu points (type: %s)"), gridno + 1, ngrids, gridsize,
                          gridNamePtr(gridtype));
           // cdoPrint("");
         }
