@@ -25,15 +25,14 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 
 #define IS_SURFACE_LEVEL(zaxisID) (zaxisInqType(zaxisID) == ZAXIS_SURFACE && zaxisInqSize(zaxisID) == 1)
 
 static void
-add_vars_mv(size_t gridsize, double missval, const double *restrict var1, const double *restrict var2, double *restrict var3)
+add_vars_mv(size_t gridsize, double missval, const std::vector<double> &var1, const std::vector<double> &var2, std::vector<double> &var3)
 {
-  double missval1 = missval;
-  double missval2 = missval;
+  const double missval1 = missval;
+  const double missval2 = missval;
   /*
   for ( size_t i = 0; i < gridsize; ++i )
     var3[i] = ADDMN(var2[i], var1[i]);
@@ -66,17 +65,12 @@ Vertcum(void *process)
   int VERTCUMHL = cdoOperatorAdd("vertcumhl",  0,  0, nullptr);
   // clang-format on
 
-  int operatorID = cdoOperatorID();
+  const int operatorID = cdoOperatorID();
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
-
-  int nvars = vlistNvars(vlistID1);
-  size_t **varnmiss = (size_t **) Malloc(nvars * sizeof(size_t *));
-  double ***vardata1 = (double ***) Malloc(nvars * sizeof(double **));
-  double ***vardata2 = (double ***) Malloc(nvars * sizeof(double **));
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
   int zaxisIDhl = -1;
 
@@ -84,11 +78,11 @@ Vertcum(void *process)
     {
       double *vct = nullptr;
       bool lhybrid = false;
-      int nzaxis = vlistNzaxis(vlistID1);
+      const int nzaxis = vlistNzaxis(vlistID1);
       for (int i = 0; i < nzaxis; ++i)
         {
-          int zaxisID = vlistZaxis(vlistID1, i);
-          int nlevs = zaxisInqSize(zaxisID);
+          const int zaxisID = vlistZaxis(vlistID1, i);
+          const int nlevs = zaxisInqSize(zaxisID);
 
           if (zaxisInqType(zaxisID) == ZAXIS_HYBRID && nlevs > 1)
             {
@@ -122,43 +116,52 @@ Vertcum(void *process)
       if (vct) Free(vct);
     }
 
+  VarList varList1, varList2;
+  varListInit(varList1, vlistID1);
+  varListInit(varList2, vlistID2);
+
+  const int nvars = vlistNvars(vlistID1);
+  std::vector<std::vector<size_t>> varnmiss(nvars);
+  std::vector<std::vector<std::vector<double>>> vardata1(nvars);
+  std::vector<std::vector<std::vector<double>>> vardata2(nvars);
+
   for (varID = 0; varID < nvars; varID++)
     {
-      size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-      int nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-      int nlevs2 = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
+      const size_t gridsize = varList1[varID].gridsize;
+      const int nlevs = varList1[varID].nlevels;
+      const int nlevs2 = varList2[varID].nlevels;
 
-      varnmiss[varID] = (size_t *) Malloc(nlevs * sizeof(size_t));
-      vardata1[varID] = (double **) Malloc(nlevs * sizeof(double *));
-      vardata2[varID] = (double **) Malloc(nlevs2 * sizeof(double *));
-      for (levelID = 0; levelID < nlevs; ++levelID) vardata1[varID][levelID] = (double *) Malloc(gridsize * sizeof(double));
-      for (levelID = 0; levelID < nlevs2; ++levelID) vardata2[varID][levelID] = (double *) Malloc(gridsize * sizeof(double));
+      varnmiss[varID].resize(nlevs);
+      vardata1[varID].resize(nlevs);
+      vardata2[varID].resize(nlevs2);
+      for (levelID = 0; levelID < nlevs; ++levelID) vardata1[varID][levelID].resize(gridsize);
+      for (levelID = 0; levelID < nlevs2; ++levelID) vardata2[varID][levelID].resize(gridsize);
     }
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          pstreamReadRecord(streamID1, vardata1[varID][levelID], &varnmiss[varID][levelID]);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          cdoReadRecord(streamID1, &vardata1[varID][levelID][0], &varnmiss[varID][levelID]);
         }
 
       for (varID = 0; varID < nvars; ++varID)
         {
-          double missval = vlistInqVarMissval(vlistID2, varID);
-          size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
-          int nlevs2 = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
+          const double missval = varList2[varID].missval;
+          const size_t gridsize = varList2[varID].gridsize;
+          const int nlevs2 = varList2[varID].nlevels;
 
           if (operatorID == VERTCUMHL && nlevs2 == nlevshl)
             {
@@ -180,10 +183,10 @@ Vertcum(void *process)
 
           if (operatorID == VERTCUMHL && nlevs2 == nlevshl)
             {
-              double *var1 = vardata2[varID][nlevs2 - 1];
+              std::vector<double> &var1 = vardata2[varID][nlevs2 - 1];
               for (levelID = 0; levelID < nlevs2; ++levelID)
                 {
-                  double *var2 = vardata2[varID][levelID];
+                  std::vector<double> &var2 = vardata2[varID][levelID];
                   for (size_t i = 0; i < gridsize; ++i)
                     {
                       if (IS_NOT_EQUAL(var1[i], 0))
@@ -197,36 +200,21 @@ Vertcum(void *process)
 
       for (varID = 0; varID < nvars; ++varID)
         {
-          double missval = vlistInqVarMissval(vlistID2, varID);
-          size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
-          int nlevs2 = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
+          const double missval = varList2[varID].missval;
+          const size_t gridsize = varList2[varID].gridsize;
+          const int nlevs2 = varList2[varID].nlevels;
           for (levelID = 0; levelID < nlevs2; ++levelID)
             {
-              double *single = vardata2[varID][levelID];
+              double *single = vardata2[varID][levelID].data();
               nmiss = arrayNumMV(gridsize, single, missval);
-              pstreamDefRecord(streamID2, varID, levelID);
-              pstreamWriteRecord(streamID2, single, nmiss);
+              cdoDefRecord(streamID2, varID, levelID);
+              cdoWriteRecord(streamID2, single, nmiss);
             }
         }
 
       tsID++;
     }
 
-  for (varID = 0; varID < nvars; ++varID)
-    {
-      int nlevs = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-      int nlevs2 = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
-      for (levelID = 0; levelID < nlevs; ++levelID) Free(vardata1[varID][levelID]);
-      for (levelID = 0; levelID < nlevs2; ++levelID) Free(vardata2[varID][levelID]);
-      Free(vardata1[varID]);
-      Free(vardata2[varID]);
-      Free(varnmiss[varID]);
-    }
-
-  Free(vardata1);
-  Free(vardata2);
-  Free(varnmiss);
-
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
