@@ -24,7 +24,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 #include "after_vertint.h"
 #include "listarray.h"
 #include "stdnametable.h"
@@ -52,11 +51,11 @@ is_height_axis(int zaxisID, int nlevel)
 static void
 change_height_zaxis(int vlistID1, int vlistID2, int zaxisID2)
 {
-  int nzaxis = vlistNzaxis(vlistID1);
+  const int nzaxis = vlistNzaxis(vlistID1);
   for (int iz = 0; iz < nzaxis; ++iz)
     {
-      int zaxisID = vlistZaxis(vlistID1, iz);
-      int nlevel = zaxisInqSize(zaxisID);
+      const int zaxisID = vlistZaxis(vlistID1, iz);
+      const int nlevel = zaxisInqSize(zaxisID);
 
       if (is_height_axis(zaxisID, nlevel) && nlevel > 1)
         {
@@ -91,22 +90,21 @@ Vertintap(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  int AP2PL     = cdoOperatorAdd("ap2pl",     func_pl, type_lin, "pressure levels in pascal");
-  int AP2PLX    = cdoOperatorAdd("ap2plx",    func_pl, type_lin, "pressure levels in pascal");
-  int AP2HL     = cdoOperatorAdd("ap2hl",     func_hl, type_lin, "height levels in meter");
-  int AP2HLX    = cdoOperatorAdd("ap2hlx",    func_hl, type_lin, "height levels in meter");
-  int AP2PL_LP  = cdoOperatorAdd("ap2pl_lp",  func_pl, type_log, "pressure levels in pascal");
-  int AP2PLX_LP = cdoOperatorAdd("ap2plx_lp", func_pl, type_log, "pressure levels in pascal");
+  const int AP2PL     = cdoOperatorAdd("ap2pl",     func_pl, type_lin, "pressure levels in pascal");
+  const int AP2PLX    = cdoOperatorAdd("ap2plx",    func_pl, type_lin, "pressure levels in pascal");
+  const int AP2HL     = cdoOperatorAdd("ap2hl",     func_hl, type_lin, "height levels in meter");
+  const int AP2HLX    = cdoOperatorAdd("ap2hlx",    func_hl, type_lin, "height levels in meter");
+  const int AP2PL_LP  = cdoOperatorAdd("ap2pl_lp",  func_pl, type_log, "pressure levels in pascal");
+  const int AP2PLX_LP = cdoOperatorAdd("ap2plx_lp", func_pl, type_log, "pressure levels in pascal");
   // clang-format on
 
-  int operatorID = cdoOperatorID();
-  bool useHightLevel = cdoOperatorF1(operatorID) == func_hl;
-  bool useLogType = cdoOperatorF2(operatorID) == type_log;
+  const int operatorID = cdoOperatorID();
+  const bool useHightLevel = cdoOperatorF1(operatorID) == func_hl;
+  const bool useLogType = cdoOperatorF2(operatorID) == type_log;
 
   if (operatorID == AP2PL || operatorID == AP2HL || operatorID == AP2PL_LP)
     {
-      char *envstr = getenv("EXTRAPOLATE");
-
+      const char *envstr = getenv("EXTRAPOLATE");
       if (envstr && isdigit((int) envstr[0]))
         {
           if (atoi(envstr) == 1) extrapolate = true;
@@ -126,15 +124,15 @@ Vertintap(void *process)
     {
       if (useHightLevel)
         {
-          double stdlev[] = { 10, 50, 100, 500, 1000, 5000, 10000, 15000, 20000, 25000, 30000 };
+          const double stdlev[] = { 10, 50, 100, 500, 1000, 5000, 10000, 15000, 20000, 25000, 30000 };
           nplev = sizeof(stdlev) / sizeof(*stdlev);
           plev.resize(nplev);
           for (int i = 0; i < nplev; ++i) plev[i] = stdlev[i];
         }
       else
         {
-          double stdlev[] = { 100000, 92500, 85000, 70000, 60000, 50000, 40000, 30000, 25000,
-                              20000,  15000, 10000, 7000,  5000,  3000,  2000,  1000 };
+          const double stdlev[] = { 100000, 92500, 85000, 70000, 60000, 50000, 40000, 30000, 25000,
+                                    20000,  15000, 10000, 7000,  5000,  3000,  2000,  1000 };
           nplev = sizeof(stdlev) / sizeof(*stdlev);
           plev.resize(nplev);
           for (int i = 0; i < nplev; ++i) plev[i] = stdlev[i];
@@ -148,27 +146,30 @@ Vertintap(void *process)
       for (int i = 0; i < nplev; ++i) plev[i] = larr[i];
     }
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  size_t gridsize = vlist_check_gridsize(vlistID1);
+  const size_t gridsize = vlist_check_gridsize(vlistID1);
 
-  int zaxistype = useHightLevel ? ZAXIS_HEIGHT : ZAXIS_PRESSURE;
-  int zaxisIDp = zaxisCreate(zaxistype, nplev);
+  const int zaxistype = useHightLevel ? ZAXIS_HEIGHT : ZAXIS_PRESSURE;
+  const int zaxisIDp = zaxisCreate(zaxistype, nplev);
   zaxisDefLevels(zaxisIDp, plev.data());
 
-  int nvars = vlistNvars(vlistID1);
+  VarList varList1, varList2;
+  varListInit(varList1, vlistID1);
+
+  const int nvars = vlistNvars(vlistID1);
 
   for (varID = 0; varID < nvars; varID++)
     {
       vlistInqVarStdname(vlistID1, varID, stdname);
-      strtolower(stdname);
+      cstrToLowerCase(stdname);
 
       // clang-format off
       if      (cstrIsEqual(stdname, var_stdname(surface_air_pressure))) psID = varID;
@@ -190,11 +191,11 @@ Vertintap(void *process)
   if (apressID == -1) cdoAbort("%s not found!", var_stdname(air_pressure));
 
   int zaxisIDh = -1;
-  int nzaxis = vlistNzaxis(vlistID1);
+  const int nzaxis = vlistNzaxis(vlistID1);
   for (int i = 0; i < nzaxis; i++)
     {
-      int zaxisID = vlistZaxis(vlistID1, i);
-      if (zaxisID == vlistInqVarZaxis(vlistID1, apressID))
+      const int zaxisID = vlistZaxis(vlistID1, i);
+      if (zaxisID == varList1[apressID].zaxisID)
         {
           bool mono_level = true;
           nlevel = zaxisInqSize(zaxisID);
@@ -224,14 +225,15 @@ Vertintap(void *process)
     }
 
   change_height_zaxis(vlistID1, vlistID2, zaxisIDp);
+  varListInit(varList2, vlistID2);
 
   std::vector<bool> vars(nvars);
   std::vector<bool> varinterp(nvars);
-  std::vector<size_t *> varnmiss(nvars);
-  std::vector<double *> vardata1(nvars);
-  std::vector<double *> vardata2(nvars);
+  std::vector<std::vector<size_t>> varnmiss(nvars);
+  std::vector<std::vector<double>> vardata1(nvars);
+  std::vector<std::vector<double>> vardata2(nvars);
 
-  int maxlev = nhlevh > nplev ? nhlevh : nplev;
+  const int maxlev = nhlevh > nplev ? nhlevh : nplev;
 
   std::vector<size_t> pnmiss;
   if (!extrapolate) pnmiss.resize(nplev);
@@ -239,7 +241,7 @@ Vertintap(void *process)
   // check levels
   if (zaxisIDh != -1)
     {
-      int nlev = zaxisInqSize(zaxisIDh);
+      const int nlev = zaxisInqSize(zaxisIDh);
       if (nlev != nhlev) cdoAbort("Internal error, wrong number of height level!");
       std::vector<double> levels(nlev);
       cdoZaxisInqLevels(zaxisIDh, &levels[0]);
@@ -282,19 +284,20 @@ Vertintap(void *process)
 
   for (varID = 0; varID < nvars; varID++)
     {
-      int gridID = vlistInqVarGrid(vlistID1, varID);
-      int zaxisID = vlistInqVarZaxis(vlistID1, varID);
-      int nlevel = zaxisInqSize(zaxisID);
+      const int gridID = varList1[varID].gridID;
+      const int zaxisID = varList1[varID].zaxisID;
+      const int nlevel = varList1[varID].nlevels;
 
       if (gridInqType(gridID) == GRID_SPECTRAL) cdoAbort("Spectral data unsupported!");
 
-      vardata1[varID] = (double *) Malloc(gridsize * nlevel * sizeof(double));
+      vardata1[varID].resize(gridsize * nlevel);
 
-      if (zaxisID == zaxisIDh || (is_height_axis(zaxisID, nlevel) && zaxisIDh != -1 && (nlevel == nhlevh || nlevel == nhlevf)))
+      varinterp[varID] = (zaxisID == zaxisIDh || (is_height_axis(zaxisID, nlevel) && zaxisIDh != -1 && (nlevel == nhlevh || nlevel == nhlevf)));
+
+      if (varinterp[varID])
         {
-          varinterp[varID] = true;
-          vardata2[varID] = (double *) Malloc(gridsize * nplev * sizeof(double));
-          varnmiss[varID] = (size_t *) Calloc(maxlev, sizeof(size_t));
+          vardata2[varID].resize(gridsize * nplev);
+          varnmiss[varID].resize(maxlev, 0);
         }
       else
         {
@@ -304,9 +307,7 @@ Vertintap(void *process)
               cdoWarning("Parameter %d has wrong number of levels, skipped! (param=%s nlevel=%d)", varID + 1, varname, nlevel);
             }
 
-          varinterp[varID] = false;
-          vardata2[varID] = vardata1[varID];
-          varnmiss[varID] = (size_t *) Malloc(nlevel * sizeof(size_t));
+          varnmiss[varID].resize(nlevel);
         }
     }
 
@@ -320,13 +321,13 @@ Vertintap(void *process)
 
   for (varID = 0; varID < nvars; ++varID)
     {
-      if (varinterp[varID] && vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT)
+      if (varinterp[varID] && varList1[varID].timetype == TIME_CONSTANT)
         vlistDefVarTimetype(vlistID2, varID, TIME_VARYING);
     }
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
+  CdoStreamID streamID2 = cdoOpenWrite(1);
 
-  pstreamDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
@@ -334,30 +335,28 @@ Vertintap(void *process)
       for (varID = 0; varID < nvars; ++varID)
         {
           vars[varID] = false;
-          nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+          nlevel = varList1[varID].nlevels;
           for (levelID = 0; levelID < nlevel; levelID++) varnmiss[varID][levelID] = 0;
         }
 
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
           /*
-          zaxisID  = vlistInqVarZaxis(vlistID1, varID);
-          nlevel   = zaxisInqSize(zaxisID);
-          if ( sortlevels && zaxisIDh != -1 && zaxisID == zaxisIDh && nlevel ==
-          nhlev )
+          zaxisID  = varList1[varID].zaxisID;
+          nlevel   = varList1[varID].nlevels;
+          if ( sortlevels && zaxisIDh != -1 && zaxisID == zaxisIDh && nlevel == nhlev )
             {
               levelID = (int) (zaxisInqLevel(zaxisIDh, levelID)-1);
               printf("levelID %d\n", levelID);
             }
           */
-          size_t offset = gridsize * levelID;
-          double *single1 = vardata1[varID] + offset;
-          pstreamReadRecord(streamID1, single1, &varnmiss[varID][levelID]);
-
+          const size_t offset = gridsize * levelID;
+          double *single1 = &vardata1[varID][offset];
+          cdoReadRecord(streamID1, single1, &varnmiss[varID][levelID]);
           vars[varID] = true;
         }
 
@@ -368,7 +367,7 @@ Vertintap(void *process)
         {
           if (psID != -1)
             {
-              arrayCopy(gridsize, vardata1[psID], &ps_prog[0]);
+              arrayCopy(gridsize, &vardata1[psID][0], &ps_prog[0]);
             }
           else if (dpressID != -1)
             {
@@ -378,16 +377,16 @@ Vertintap(void *process)
             }
           else
             {
-              arrayCopy(gridsize, vardata1[apressID] + gridsize * (nhlevf - 1), &ps_prog[0]);
+              arrayCopy(gridsize, &vardata1[apressID][gridsize * (nhlevf - 1)], &ps_prog[0]);
               // for ( size_t i = 0; i < gridsize; i++ )  ps_prog[i] = 110000;
             }
 
           /* check range of ps_prog */
           double minval, maxval;
-          arrayMinMaxMask(gridsize, &ps_prog[0], nullptr, &minval, &maxval);
+          arrayMinMax(gridsize, &ps_prog[0], minval, maxval);
           if (minval < MIN_PS || maxval > MAX_PS) cdoWarning("Surface pressure out of range (min=%g max=%g)!", minval, maxval);
 
-          arrayCopy(gridsize * nhlevf, vardata1[apressID], &full_press[0]);
+          arrayCopy(gridsize * nhlevf, &vardata1[apressID][0], &full_press[0]);
 
           for (size_t i = 0; i < gridsize; i++) half_press[i] = 0;
           for (int k = 1; k < nhlevf; k++)
@@ -411,17 +410,12 @@ Vertintap(void *process)
         {
           if (vars[varID])
             {
-              int zaxisID = vlistInqVarZaxis(vlistID1, varID);
-              int nlevel = zaxisInqSize(zaxisID);
-              double missval = vlistInqVarMissval(vlistID1, varID);
+              const int nlevel = varList1[varID].nlevels;
+              const double missval = varList1[varID].missval;
               if (varinterp[varID])
                 {
-                  double *hyb_press = nullptr;
-                  if (nlevel == nhlevf)
-                    hyb_press = &full_press[0];
-                  else if (nlevel == nhlevh)
-                    hyb_press = &half_press[0];
-                  else
+                  double *hyb_press = (nlevel == nhlevf) ? &full_press[0] : (nlevel == nhlevh) ? &half_press[0] : nullptr;
+                  if (hyb_press == nullptr)
                     {
                       vlistInqVarName(vlistID1, varID, varname);
                       cdoAbort("Number of generalized height level differ from full/half level (param=%s)!", varname);
@@ -432,9 +426,10 @@ Vertintap(void *process)
                       if (varnmiss[varID][levelID]) cdoAbort("Missing values unsupported for this operator!");
                     }
 
-                  interp_X(vardata1[varID], vardata2[varID], hyb_press, &vert_index[0], plev.data(), nplev, gridsize, nlevel, missval);
+                  interp_X(&vardata1[varID][0], &vardata2[varID][0], hyb_press, &vert_index[0], plev.data(), nplev, gridsize, nlevel,
+                           missval);
 
-                  if (!extrapolate) arrayCopy(nplev, &pnmiss[0], varnmiss[varID]);
+                  if (!extrapolate) arrayCopy(nplev, &pnmiss[0], varnmiss[varID].data());
                 }
             }
         }
@@ -443,13 +438,13 @@ Vertintap(void *process)
         {
           if (vars[varID])
             {
-              int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
+              const int nlevel = varList2[varID].nlevels;
               for (levelID = 0; levelID < nlevel; levelID++)
                 {
-                  size_t offset = gridsize * levelID;
-                  double *single2 = vardata2[varID] + offset;
-                  pstreamDefRecord(streamID2, varID, levelID);
-                  pstreamWriteRecord(streamID2, single2, varnmiss[varID][levelID]);
+                  const size_t offset = gridsize * levelID;
+                  double *single = varinterp[varID] ? &vardata2[varID][offset] : &vardata1[varID][offset];
+                  cdoDefRecord(streamID2, varID, levelID);
+                  cdoWriteRecord(streamID2, single, varnmiss[varID][levelID]);
                 }
             }
         }
@@ -460,13 +455,6 @@ Vertintap(void *process)
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
-  for (varID = 0; varID < nvars; varID++)
-    {
-      Free(varnmiss[varID]);
-      Free(vardata1[varID]);
-      if (varinterp[varID]) Free(vardata2[varID]);
-    }
-
   cdoFinish();
 
   return nullptr;
