@@ -25,7 +25,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 #include "after_vertint.h"
 #include "listarray.h"
 #include "stdnametable.h"
@@ -42,16 +41,16 @@ zaxis_is_hybrid(int zaxistype)
 static void
 change_hybrid_zaxis(int vlistID1, int vlistID2, int nvct, double *vct, int zaxisID2, int nhlevf, int nhlevh)
 {
-  int nzaxis = vlistNzaxis(vlistID1);
+  const int nzaxis = vlistNzaxis(vlistID1);
   for (int iz = 0; iz < nzaxis; ++iz)
     {
-      int zaxisID = vlistZaxis(vlistID1, iz);
-      int nlevel = zaxisInqSize(zaxisID);
-      int zaxistype = zaxisInqType(zaxisID);
+      const int zaxisID = vlistZaxis(vlistID1, iz);
+      const int nlevel = zaxisInqSize(zaxisID);
+      const int zaxistype = zaxisInqType(zaxisID);
 
       if (zaxis_is_hybrid(zaxistype) && (nlevel == nhlevh || nlevel == nhlevf))
         {
-          int nvct2 = zaxisInqVctSize(zaxisID);
+          const int nvct2 = zaxisInqVctSize(zaxisID);
           if (nvct2 == nvct && memcmp(vct, zaxisInqVctPtr(zaxisID), nvct * sizeof(double)) == 0)
             vlistChangeZaxisIndex(vlistID2, iz, zaxisID2);
         }
@@ -88,23 +87,23 @@ Vertintml(void *process)
   cdoInitialize(process);
 
   // clang-format off
-  int ML2PL     = cdoOperatorAdd("ml2pl",     func_pl, type_lin, "pressure levels in pascal");
-  int ML2PLX    = cdoOperatorAdd("ml2plx",    func_pl, type_lin, "pressure levels in pascal");
-  int ML2HL     = cdoOperatorAdd("ml2hl",     func_hl, type_lin, "height levels in meter");
-  int ML2HLX    = cdoOperatorAdd("ml2hlx",    func_hl, type_lin, "height levels in meter");
-  int ML2PL_LP  = cdoOperatorAdd("ml2pl_lp",  func_pl, type_log, "pressure levels in pascal");
-  int ML2PLX_LP = cdoOperatorAdd("ml2plx_lp", func_pl, type_log, "pressure levels in pascal");
-  int ML2HL_LP  = cdoOperatorAdd("ml2hl_lp",  func_hl, type_log, "height levels in meter");
-  int ML2HLX_LP = cdoOperatorAdd("ml2hlx_lp", func_hl, type_log, "height levels in meter");
+  const int ML2PL     = cdoOperatorAdd("ml2pl",     func_pl, type_lin, "pressure levels in pascal");
+  const int ML2PLX    = cdoOperatorAdd("ml2plx",    func_pl, type_lin, "pressure levels in pascal");
+  const int ML2HL     = cdoOperatorAdd("ml2hl",     func_hl, type_lin, "height levels in meter");
+  const int ML2HLX    = cdoOperatorAdd("ml2hlx",    func_hl, type_lin, "height levels in meter");
+  const int ML2PL_LP  = cdoOperatorAdd("ml2pl_lp",  func_pl, type_log, "pressure levels in pascal");
+  const int ML2PLX_LP = cdoOperatorAdd("ml2plx_lp", func_pl, type_log, "pressure levels in pascal");
+  const int ML2HL_LP  = cdoOperatorAdd("ml2hl_lp",  func_hl, type_log, "height levels in meter");
+  const int ML2HLX_LP = cdoOperatorAdd("ml2hlx_lp", func_hl, type_log, "height levels in meter");
   // clang-format on
 
-  int operatorID = cdoOperatorID();
-  bool useHightLevel = cdoOperatorF1(operatorID) == func_hl;
-  bool useLogType = cdoOperatorF2(operatorID) == type_log;
+  const int operatorID = cdoOperatorID();
+  const bool useHightLevel = cdoOperatorF1(operatorID) == func_hl;
+  const bool useLogType = cdoOperatorF2(operatorID) == type_log;
 
   if (operatorID == ML2PL || operatorID == ML2HL || operatorID == ML2PL_LP || operatorID == ML2HL_LP)
     {
-      char *envstr = getenv("EXTRAPOLATE");
+      const char *envstr = getenv("EXTRAPOLATE");
       if (envstr && isdigit((int) envstr[0]))
         {
           if (atoi(envstr) == 1) extrapolate = true;
@@ -122,21 +121,14 @@ Vertintml(void *process)
   std::vector<double> plev;
   if (operatorArgc() == 1 && cstrIsEqual(operatorArgv()[0], "default"))
     {
+      std::vector<double> stdlev;
       if (useHightLevel)
-        {
-          double stdlev[] = { 10, 50, 100, 500, 1000, 5000, 10000, 15000, 20000, 25000, 30000 };
-          nplev = sizeof(stdlev) / sizeof(*stdlev);
-          plev.resize(nplev);
-          for (int i = 0; i < nplev; ++i) plev[i] = stdlev[i];
-        }
+        stdlev = { 10, 50, 100, 500, 1000, 5000, 10000, 15000, 20000, 25000, 30000 };
       else
-        {
-          double stdlev[] = { 100000, 92500, 85000, 70000, 60000, 50000, 40000, 30000, 25000,
-                              20000,  15000, 10000, 7000,  5000,  3000,  2000,  1000 };
-          nplev = sizeof(stdlev) / sizeof(*stdlev);
-          plev.resize(nplev);
-          for (int i = 0; i < nplev; ++i) plev[i] = stdlev[i];
-        }
+        stdlev = { 100000, 92500, 85000, 70000, 60000, 50000, 40000, 30000, 25000,
+                   20000,  15000, 10000, 7000,  5000,  3000,  2000,  1000 };
+      plev = stdlev;
+      nplev = plev.size();
     }
   else
     {
@@ -146,19 +138,19 @@ Vertintml(void *process)
       for (int i = 0; i < nplev; ++i) plev[i] = larr[i];
     }
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  size_t gridsize = vlist_check_gridsize(vlistID1);
+  const size_t gridsize = vlist_check_gridsize(vlistID1);
 
-  int zaxistype = useHightLevel ? ZAXIS_HEIGHT : ZAXIS_PRESSURE;
-  int zaxisIDp = zaxisCreate(zaxistype, nplev);
+  const int zaxistype = useHightLevel ? ZAXIS_HEIGHT : ZAXIS_PRESSURE;
+  const int zaxisIDp = zaxisCreate(zaxistype, nplev);
   zaxisDefLevels(zaxisIDp, plev.data());
 
   int nvct = 0;
@@ -193,15 +185,19 @@ Vertintml(void *process)
         }
     }
 
-  int nvars = vlistNvars(vlistID1);
+  VarList varList1, varList2;
+  varListInit(varList1, vlistID1);
+  varListInit(varList2, vlistID2);
+
+  const int nvars = vlistNvars(vlistID1);
 
   std::vector<bool> vars(nvars);
   std::vector<bool> varinterp(nvars);
-  std::vector<size_t *> varnmiss(nvars);
-  std::vector<double *> vardata1(nvars);
-  std::vector<double *> vardata2(nvars);
+  std::vector<std::vector<size_t>> varnmiss(nvars);
+  std::vector<std::vector<double>> vardata1(nvars);
+  std::vector<std::vector<double>> vardata2(nvars);
 
-  int maxlev = nhlevh > nplev ? nhlevh : nplev;
+  const int maxlev = nhlevh > nplev ? nhlevh : nplev;
 
   std::vector<size_t> pnmiss;
   if (!extrapolate) pnmiss.resize(nplev);
@@ -209,7 +205,7 @@ Vertintml(void *process)
   // check levels
   if (zaxisIDh != -1)
     {
-      int nlev = zaxisInqSize(zaxisIDh);
+      const int nlev = zaxisInqSize(zaxisIDh);
       if (nlev != nhlev) cdoAbort("Internal error, wrong number of hybrid level!");
       std::vector<double> levels(nlev);
       cdoZaxisInqLevels(zaxisIDh, &levels[0]);
@@ -253,7 +249,7 @@ Vertintml(void *process)
   bool useTable = false;
   for (varID = 0; varID < nvars; varID++)
     {
-      int tableNum = tableInqNum(vlistInqVarTable(vlistID1, varID));
+      const int tableNum = tableInqNum(vlistInqVarTable(vlistID1, varID));
       if (tableNum > 0 && tableNum != 255)
         {
           useTable = true;
@@ -265,17 +261,17 @@ Vertintml(void *process)
 
   for (varID = 0; varID < nvars; varID++)
     {
-      int gridID = vlistInqVarGrid(vlistID1, varID);
-      int zaxisID = vlistInqVarZaxis(vlistID1, varID);
-      int zaxistype = zaxisInqType(zaxisID);
+      const int gridID = varList1[varID].gridID;
+      const int zaxisID = varList1[varID].zaxisID;
+      const int zaxistype = zaxisInqType(zaxisID);
       // gridsize = gridInqSize(gridID);
-      int nlevel = zaxisInqSize(zaxisID);
-      int instNum = institutInqCenter(vlistInqVarInstitut(vlistID1, varID));
-      int tableNum = tableInqNum(vlistInqVarTable(vlistID1, varID));
+      const int nlevel = zaxisInqSize(zaxisID);
+      const int instNum = institutInqCenter(vlistInqVarInstitut(vlistID1, varID));
+      const int tableNum = tableInqNum(vlistInqVarTable(vlistID1, varID));
 
       int code = vlistInqVarCode(vlistID1, varID);
 
-      int param = vlistInqVarParam(vlistID1, varID);
+      const int param = vlistInqVarParam(vlistID1, varID);
       cdiParamToString(param, paramstr, sizeof(paramstr));
       int pnum, pcat, pdis;
       cdiDecodeParam(param, &pnum, &pcat, &pdis);
@@ -312,17 +308,17 @@ Vertintml(void *process)
       if (Options::cdoVerbose)
         {
           vlistInqVarName(vlistID1, varID, varname);
-          cdoPrint("Mode = %d  Center = %d TableNum =%d Code = %d Param = %s Varname = %s varID = %d", mode, instNum, tableNum,
+          cdoPrint("Mode=%d  Center=%d  TableNum=%d  Code=%d  Param=%s  Varname=%s  varID=%d", mode, instNum, tableNum,
                    code, paramstr, varname, varID);
         }
 
       if (code <= 0 || code == 255)
         {
           vlistInqVarName(vlistID1, varID, varname);
-          strtolower(varname);
+          cstrToLowerCase(varname);
 
           vlistInqVarStdname(vlistID1, varID, stdname);
-          strtolower(stdname);
+          cstrToLowerCase(stdname);
 
           code = echamcode_from_stdname(stdname);
           if (code == -1)
@@ -364,17 +360,16 @@ Vertintml(void *process)
 
       if (gridInqType(gridID) == GRID_SPECTRAL) cdoAbort("Spectral data unsupported!");
 
-      if (varID == gheightID)
-        vardata1[varID] = (double *) Malloc(gridsize * (nlevel + 1) * sizeof(double));
-      else
-        vardata1[varID] = (double *) Malloc(gridsize * nlevel * sizeof(double));
+      const size_t maxlevel = (varID == gheightID) ? (nlevel + 1) : nlevel;
+      vardata1[varID].resize(gridsize * maxlevel);
 
-      // if ( zaxis_is_hybrid(zaxistype) && zaxisIDh != -1 && nlevel == nhlev )
-      if (zaxisID == zaxisIDh || (zaxis_is_hybrid(zaxistype) && zaxisIDh != -1 && (nlevel == nhlevh || nlevel == nhlevf)))
+      // varinterp[varID] = ( zaxis_is_hybrid(zaxistype) && zaxisIDh != -1 && nlevel == nhlev );
+      varinterp[varID] = (zaxisID == zaxisIDh || (zaxis_is_hybrid(zaxistype) && zaxisIDh != -1 && (nlevel == nhlevh || nlevel == nhlevf)));
+
+      if (varinterp[varID])
         {
-          varinterp[varID] = true;
-          vardata2[varID] = (double *) Malloc(gridsize * nplev * sizeof(double));
-          varnmiss[varID] = (size_t *) Calloc(maxlev, sizeof(size_t));
+          vardata2[varID].resize(gridsize * nplev);
+          varnmiss[varID].resize(maxlev, 0);
         }
       else
         {
@@ -384,9 +379,7 @@ Vertintml(void *process)
               cdoWarning("Parameter %d has wrong number of levels, skipped! (param=%s nlevel=%d)", varID + 1, varname, nlevel);
             }
 
-          varinterp[varID] = false;
-          vardata2[varID] = vardata1[varID];
-          varnmiss[varID] = (size_t *) Malloc(nlevel * sizeof(size_t));
+          varnmiss[varID].resize(nlevel);
         }
     }
 
@@ -450,16 +443,13 @@ Vertintml(void *process)
   if (zaxisIDh != -1)
     {
       double suma = 0, sumb = 0;
-      for (int i = 0; i < nhlevh; i++)
-        {
-          suma += vct[i];
-          sumb += vct[i + nhlevh];
-        }
+      for (int i = 0; i < nhlevh; ++i) suma += vct[i];
+      for (int i = 0; i < nhlevh; ++i) sumb += vct[i + nhlevh];
       if (!(suma > 0 || sumb > 0)) cdoWarning("VCT is empty!");
     }
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
@@ -467,14 +457,14 @@ Vertintml(void *process)
       for (varID = 0; varID < nvars; ++varID) vars[varID] = false;
 
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
-          // gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-          int zaxisID = vlistInqVarZaxis(vlistID1, varID);
-          int nlevel = zaxisInqSize(zaxisID);
+          cdoInqRecord(streamID1, &varID, &levelID);
+          // gridsize = varList1[varID].gridsize;
+          const int zaxisID = varList1[varID].zaxisID;
+          const int nlevel = varList1[varID].nlevels;
           /*
           if ( sortlevels && zaxisIDh != -1 && zaxisID == zaxisIDh && nlevel == nhlev )
             {
@@ -484,9 +474,9 @@ Vertintml(void *process)
           */
           if (linvertvct && zaxisIDh != -1 && zaxisID == zaxisIDh) levelID = nlevel - 1 - levelID;
 
-          size_t offset = gridsize * levelID;
-          double *single = vardata1[varID] + offset;
-          pstreamReadRecord(streamID1, single, &varnmiss[varID][levelID]);
+          const size_t offset = gridsize * levelID;
+          double *single = &vardata1[varID][offset];
+          cdoReadRecord(streamID1, single, &varnmiss[varID][levelID]);
           vars[varID] = true;
         }
 
@@ -495,14 +485,14 @@ Vertintml(void *process)
           if (sgeopot_needed)
             {
               if (sgeopotID != -1)
-                arrayCopy(gridsize, vardata1[sgeopotID], &sgeopot[0]);
+                arrayCopy(gridsize, &vardata1[sgeopotID][0], &sgeopot[0]);
               else if (geopotID != -1)
-                arrayCopy(gridsize, vardata1[geopotID] + gridsize * (nhlevf - 1), &sgeopot[0]);
+                arrayCopy(gridsize, &vardata1[geopotID][gridsize * (nhlevf - 1)], &sgeopot[0]);
 
               // check range of surface geopot
               if (extrapolate && (sgeopotID != -1 || geopotID != -1))
                 {
-                  arrayMinMaxMask(gridsize, &sgeopot[0], nullptr, &minval, &maxval);
+                  arrayMinMax(gridsize, &sgeopot[0], minval, maxval);
                   if (minval < MIN_FIS || maxval > MAX_FIS)
                     cdoWarning("Surface geopotential out of range (min=%g max=%g) [timestep:%d]!", minval, maxval, tsID + 1);
                   if (gridsize > 1 && minval >= 0 && maxval <= 9000 && IS_NOT_EQUAL(minval, maxval))
@@ -514,10 +504,10 @@ Vertintml(void *process)
           if (presID == lnpsID)
             for (size_t i = 0; i < gridsize; i++) ps_prog[i] = exp(vardata1[lnpsID][i]);
           else if (presID != -1)
-            arrayCopy(gridsize, vardata1[presID], &ps_prog[0]);
+            arrayCopy(gridsize, &vardata1[presID][0], &ps_prog[0]);
 
           // check range of ps_prog
-          arrayMinMaxMask(gridsize, &ps_prog[0], nullptr, &minval, &maxval);
+          arrayMinMax(gridsize, &ps_prog[0], minval, maxval);
           if (minval < MIN_PS || maxval > MAX_PS)
             cdoWarning("Surface pressure out of range (min=%g max=%g) [timestep:%d]!", minval, maxval, tsID + 1);
 
@@ -539,21 +529,17 @@ Vertintml(void *process)
         {
           if (vars[varID])
             {
-              int zaxisID = vlistInqVarZaxis(vlistID1, varID);
-              int nlevel = zaxisInqSize(zaxisID);
-              double missval = vlistInqVarMissval(vlistID1, varID);
+              const int nlevel = varList1[varID].nlevels;
+              const double missval = varList1[varID].missval;
               if (varinterp[varID])
                 {
                   /*
                   if ( nlevel == nhlevh )
                     {
-                      int i, k;
-                      double *vl1, *vl2;
-
                       for ( int k = 1; k < nlevel; k++ )
                         {
-                          vl1  = vardata1[varID] + gridsize*(k-1);
-                          vl2  = vardata1[varID] + gridsize*(k);
+                          double *vl1  = &vardata1[varID][gridsize*(k-1)];
+                          double *vl2  = &vardata1[varID][gridsize*(k)];
                           for ( size_t i = 0; i < gridsize; i++ )
                             vl1[i] = 0.5*(vl1[i] + vl2[i]);
                         }
@@ -561,12 +547,8 @@ Vertintml(void *process)
                       nlevel = nhlevf;
                     }
                   */
-                  double *hyb_press = nullptr;
-                  if (nlevel == nhlevf)
-                    hyb_press = &full_press[0];
-                  else if (nlevel == nhlevh)
-                    hyb_press = &half_press[0];
-                  else
+                  double *hyb_press = (nlevel == nhlevf) ? &full_press[0] : (nlevel == nhlevh) ? &half_press[0] : nullptr;
+                  if (hyb_press == nullptr)
                     {
                       vlistInqVarName(vlistID1, varID, varname);
                       cdoAbort("Number of hybrid level differ from full/half level (param=%s)!", varname);
@@ -583,22 +565,23 @@ Vertintml(void *process)
 
                       if (useLogType && extrapolate) cdoAbort("Log. extrapolation of temperature unsupported!");
 
-                      interp_T(&sgeopot[0], vardata1[varID], vardata2[varID], &full_press[0], &half_press[0], &vert_index[0], plev.data(),
-                               nplev, gridsize, nlevel, missval);
+                      interp_T(&sgeopot[0], &vardata1[varID][0], &vardata2[varID][0], &full_press[0], &half_press[0], &vert_index[0],
+                               plev.data(), nplev, gridsize, nlevel, missval);
                     }
                   else if (varID == gheightID)
                     {
                       for (size_t i = 0; i < gridsize; ++i) vardata1[varID][gridsize * nlevel + i] = sgeopot[i] / PlanetGrav;
 
-                      interp_Z(&sgeopot[0], vardata1[varID], vardata2[varID], &full_press[0], &half_press[0], &vert_index[0],
-                               vardata1[tempID], plev.data(), nplev, gridsize, nlevel, missval);
+                      interp_Z(&sgeopot[0], &vardata1[varID][0], &vardata2[varID][0], &full_press[0], &half_press[0], &vert_index[0],
+                               &vardata1[tempID][0], plev.data(), nplev, gridsize, nlevel, missval);
                     }
                   else
                     {
-                      interp_X(vardata1[varID], vardata2[varID], hyb_press, &vert_index[0], plev.data(), nplev, gridsize, nlevel, missval);
+                      interp_X(&vardata1[varID][0], &vardata2[varID][0], hyb_press, &vert_index[0], plev.data(), nplev, gridsize, nlevel,
+                               missval);
                     }
 
-                  if (!extrapolate) arrayCopy(nplev, &pnmiss[0], varnmiss[varID]);
+                  if (!extrapolate) arrayCopy(nplev, &pnmiss[0], varnmiss[varID].data());
                 }
             }
         }
@@ -607,14 +590,14 @@ Vertintml(void *process)
         {
           if (vars[varID])
             {
-              int nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
+              const size_t gridsize = varList2[varID].gridsize;
+              const int nlevel = varList2[varID].nlevels;
               for (levelID = 0; levelID < nlevel; levelID++)
                 {
-                  // gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
-                  size_t offset = gridsize * levelID;
-                  double *single = vardata2[varID] + offset;
-                  pstreamDefRecord(streamID2, varID, levelID);
-                  pstreamWriteRecord(streamID2, single, varnmiss[varID][levelID]);
+                  const size_t offset = gridsize * levelID;
+                  double *single = varinterp[varID] ? &vardata2[varID][offset] : &vardata1[varID][offset];
+                  cdoDefRecord(streamID2, varID, levelID);
+                  cdoWriteRecord(streamID2, single, varnmiss[varID][levelID]);
                 }
             }
         }
@@ -625,13 +608,6 @@ Vertintml(void *process)
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
-  for (varID = 0; varID < nvars; varID++)
-    {
-      Free(varnmiss[varID]);
-      Free(vardata1[varID]);
-      if (varinterp[varID]) Free(vardata2[varID]);
-    }
-
   if (vct) Free(vct);
 
   cdoFinish();
