@@ -34,8 +34,8 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
+#include "cdo_zaxis.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
 #include "pmlist.h"
 
 #define IS_SURFACE_LEVEL(zaxisID) (zaxisInqType(zaxisID) == ZAXIS_SURFACE && zaxisInqSize(zaxisID) == 1)
@@ -56,7 +56,7 @@ getSurfaceID(int vlistID)
         }
     }
 
-  if (surfID == -1) surfID = zaxisCreate(ZAXIS_SURFACE, 1);
+  if (surfID == -1) surfID = zaxisFromName("surface");
 
   return surfID;
 }
@@ -176,10 +176,8 @@ void *
 Vertstat(void *process)
 {
   int nrecs;
-  int gridID;
   int varID, levelID;
-  size_t nmiss;
-  struct vert_t
+  struct VertInfo
   {
     int zaxisID;
     int status;
@@ -216,7 +214,7 @@ Vertstat(void *process)
 
   // int applyWeights = lmean;
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
   const int vlistID1 = cdoStreamInqVlist(streamID1);
 
   vlistClearFlag(vlistID1);
@@ -234,8 +232,7 @@ Vertstat(void *process)
   setSurfaceID(vlistID2, surfID);
 
   const int nzaxis = vlistNzaxis(vlistID1);
-  int nlev, zaxisID;
-  std::vector<vert_t> vert(nzaxis);
+  std::vector<VertInfo> vert(nzaxis);
   if (needWeights)
     {
       bool useweights = true;
@@ -250,12 +247,12 @@ Vertstat(void *process)
 
       for (int index = 0; index < nzaxis; ++index)
         {
-          zaxisID = vlistZaxis(vlistID1, index);
-          nlev = zaxisInqSize(zaxisID);
+          const int zaxisID = vlistZaxis(vlistID1, index);
+          const int nlev = zaxisInqSize(zaxisID);
           vert[index].numlevel = 0;
           vert[index].status = 0;
           vert[index].zaxisID = zaxisID;
-          // if ( nlev > 1 )
+          // if (nlev > 1)
           {
             vert[index].numlevel = nlev;
             vert[index].thickness.resize(nlev);
@@ -267,46 +264,34 @@ Vertstat(void *process)
         }
     }
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
+  VarList varList;
+  varListInit(varList, vlistID1);
+
+  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field;
-  fieldInit(field);
-  field.ptr = (double *) Malloc(gridsize * sizeof(double));
+  field.resize(gridsizemax);
 
-  std::vector<Field> vars1(nvars);
-  std::vector<Field> samp1(nvars);
-  std::vector<Field> vars2;
+  FieldVector vars1(nvars);
+  FieldVector samp1(nvars);
+  FieldVector vars2;
   if (lvarstd || lrange) vars2.resize(nvars);
 
   for (varID = 0; varID < nvars; varID++)
     {
-      gridID = vlistInqVarGrid(vlistID1, varID);
-      gridsize = gridInqSize(gridID);
-      zaxisID = vlistInqVarZaxis(vlistID1, varID);
-      const double missval = vlistInqVarMissval(vlistID1, varID);
-
-      fieldInit(vars1[varID]);
-      fieldInit(samp1[varID]);
-      vars1[varID].grid = gridID;
-      vars1[varID].zaxis = zaxisID;
-      vars1[varID].nsamp = 0;
-      vars1[varID].nmiss = 0;
-      vars1[varID].missval = missval;
-      vars1[varID].ptr = (double *) Malloc(gridsize * sizeof(double));
-      samp1[varID].grid = gridID;
-      samp1[varID].nmiss = 0;
-      samp1[varID].missval = missval;
-      samp1[varID].ptr = nullptr;
+      vars1[varID].grid = varList[varID].gridID;
+      vars1[varID].missval = varList[varID].missval;
+      vars1[varID].resize(varList[varID].gridsize);
+      samp1[varID].grid = varList[varID].gridID;
+      samp1[varID].missval = varList[varID].missval;
       if (lvarstd || lrange)
         {
-          fieldInit(vars2[varID]);
-          vars2[varID].grid = gridID;
-          vars2[varID].nmiss = 0;
-          vars2[varID].missval = missval;
-          vars2[varID].ptr = (double *) Malloc(gridsize * sizeof(double));
+          vars2[varID].grid = varList[varID].gridID;
+          vars2[varID].missval = varList[varID].missval;
+          vars2[varID].resize(varList[varID].gridsize);
         }
     }
 
@@ -314,17 +299,18 @@ Vertstat(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           vars1[varID].nsamp++;
           if (lrange) vars2[varID].nsamp++;
-          gridsize = gridInqSize(vars1[varID].grid);
-          zaxisID = vars1[varID].zaxis;
-          nlev = zaxisInqSize(zaxisID);
+
+          const size_t gridsize = varList[varID].gridsize;
+          const int zaxisID = varList[varID].zaxisID;
+          const int nlev = varList[varID].nlevels;
 
           double layer_weight = 1.0;
           double layer_thickness = 1.0;
@@ -351,84 +337,79 @@ Vertstat(void *process)
 
           if (levelID == 0)
             {
-              pstreamReadRecord(streamID1, vars1[varID].ptr, &nmiss);
-              vars1[varID].nmiss = nmiss;
+              cdoReadRecord(streamID1, vars1[varID].vec.data(), &vars1[varID].nmiss);
               if (lrange)
                 {
-                  vars2[varID].nmiss = nmiss;
-                  for (size_t i = 0; i < gridsize; i++) vars2[varID].ptr[i] = vars1[varID].ptr[i];
+                  vars2[varID].nmiss = vars1[varID].nmiss;
+                  vars2[varID].vec = vars1[varID].vec;
                 }
 
-              if (operatorID == VERTINT && IS_NOT_EQUAL(layer_thickness, 1.0)) farcmul(vars1[varID], layer_thickness);
-              if (lmean && IS_NOT_EQUAL(layer_weight, 1.0)) farcmul(vars1[varID], layer_weight);
+              if (operatorID == VERTINT && IS_NOT_EQUAL(layer_thickness, 1.0)) vfarcmul(vars1[varID], layer_thickness);
+              if (lmean && IS_NOT_EQUAL(layer_weight, 1.0)) vfarcmul(vars1[varID], layer_weight);
 
               if (lvarstd)
                 {
                   if (IS_NOT_EQUAL(layer_weight, 1.0))
                     {
-                      farmoqw(vars2[varID], vars1[varID], layer_weight);
-                      farcmul(vars1[varID], layer_weight);
+                      vfarmoqw(vars2[varID], vars1[varID], layer_weight);
+                      vfarcmul(vars1[varID], layer_weight);
                     }
                   else
                     {
-                      farmoq(vars2[varID], vars1[varID]);
+                      vfarmoq(vars2[varID], vars1[varID]);
                     }
                 }
 
-              if (nmiss > 0 || samp1[varID].ptr || needWeights)
+              if (vars1[varID].nmiss || !samp1[varID].empty() || needWeights)
                 {
-                  if (samp1[varID].ptr == nullptr) samp1[varID].ptr = (double *) Malloc(gridsize * sizeof(double));
+                  if (samp1[varID].empty()) samp1[varID].resize(gridsize);
 
                   for (size_t i = 0; i < gridsize; i++)
-                    if (DBL_IS_EQUAL(vars1[varID].ptr[i], vars1[varID].missval))
-                      samp1[varID].ptr[i] = 0.;
+                    if (DBL_IS_EQUAL(vars1[varID].vec[i], vars1[varID].missval))
+                      samp1[varID].vec[i] = 0.;
                     else
-                      samp1[varID].ptr[i] = layer_weight;
+                      samp1[varID].vec[i] = layer_weight;
                 }
             }
           else
             {
-              pstreamReadRecord(streamID1, field.ptr, &nmiss);
-              field.nmiss = nmiss;
+              cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
               field.grid = vars1[varID].grid;
+              field.size = vars1[varID].size;
               field.missval = vars1[varID].missval;
 
-              if (operatorID == VERTINT && IS_NOT_EQUAL(layer_thickness, 1.0)) farcmul(field, layer_thickness);
-              if (lmean && IS_NOT_EQUAL(layer_weight, 1.0)) farcmul(field, layer_weight);
+              if (operatorID == VERTINT && IS_NOT_EQUAL(layer_thickness, 1.0)) vfarcmul(field, layer_thickness);
+              if (lmean && IS_NOT_EQUAL(layer_weight, 1.0)) vfarcmul(field, layer_weight);
 
-              if (field.nmiss > 0 || samp1[varID].ptr)
+              if (field.nmiss || !samp1[varID].empty())
                 {
-                  if (samp1[varID].ptr == nullptr)
-                    {
-                      samp1[varID].ptr = (double *) Malloc(gridsize * sizeof(double));
-                      for (size_t i = 0; i < gridsize; i++) samp1[varID].ptr[i] = vars1[varID].nsamp;
-                    }
+                  if (samp1[varID].empty()) samp1[varID].resize(gridsize, vars1[varID].nsamp);
 
                   for (size_t i = 0; i < gridsize; i++)
-                    if (!DBL_IS_EQUAL(field.ptr[i], vars1[varID].missval)) samp1[varID].ptr[i] += layer_weight;
+                    if (!DBL_IS_EQUAL(field.vec[i], vars1[varID].missval)) samp1[varID].vec[i] += layer_weight;
                 }
 
               if (lvarstd)
                 {
                   if (IS_NOT_EQUAL(layer_weight, 1.0))
                     {
-                      farsumqw(vars2[varID], field, layer_weight);
-                      farsumw(vars1[varID], field, layer_weight);
+                      vfarsumqw(vars2[varID], field, layer_weight);
+                      vfarsumw(vars1[varID], field, layer_weight);
                     }
                   else
                     {
-                      farsumq(vars2[varID], field);
-                      farsum(vars1[varID], field);
+                      vfarsumq(vars2[varID], field);
+                      vfarsum(vars1[varID], field);
                     }
                 }
               else if (lrange)
                 {
-                  farmin(vars2[varID], field);
-                  farmax(vars1[varID], field);
+                  vfarmin(vars2[varID], field);
+                  vfarmax(vars1[varID], field);
                 }
               else
                 {
-                  farfun(vars1[varID], field, operfunc);
+                  vfarfun(vars1[varID], field, operfunc);
                 }
             }
         }
@@ -439,35 +420,35 @@ Vertstat(void *process)
             {
               if (lmean)
                 {
-                  if (samp1[varID].ptr == nullptr)
-                    farcdiv(vars1[varID], (double) vars1[varID].nsamp);
+                  if (!samp1[varID].empty())
+                    vfardiv(vars1[varID], samp1[varID]);
                   else
-                    fardiv(vars1[varID], samp1[varID]);
+                    vfarcdiv(vars1[varID], (double) vars1[varID].nsamp);
                 }
               else if (lvarstd)
                 {
-                  if (samp1[varID].ptr == nullptr)
+                  if (!samp1[varID].empty())
                     {
                       if (lstd)
-                        farcstd(vars1[varID], vars2[varID], vars1[varID].nsamp, divisor);
+                        vfarstd(vars1[varID], vars2[varID], samp1[varID], divisor);
                       else
-                        farcvar(vars1[varID], vars2[varID], vars1[varID].nsamp, divisor);
+                        vfarvar(vars1[varID], vars2[varID], samp1[varID], divisor);
                     }
                   else
                     {
                       if (lstd)
-                        farstd(vars1[varID], vars2[varID], samp1[varID], divisor);
+                        vfarcstd(vars1[varID], vars2[varID], vars1[varID].nsamp, divisor);
                       else
-                        farvar(vars1[varID], vars2[varID], samp1[varID], divisor);
+                        vfarcvar(vars1[varID], vars2[varID], vars1[varID].nsamp, divisor);
                     }
                 }
               else if (lrange)
                 {
-                  farsub(vars1[varID], vars2[varID]);
+                  vfarsub(vars1[varID], vars2[varID]);
                 }
 
-              pstreamDefRecord(streamID2, varID, 0);
-              pstreamWriteRecord(streamID2, vars1[varID].ptr, vars1[varID].nmiss);
+              cdoDefRecord(streamID2, varID, 0);
+              cdoWriteRecord(streamID2, vars1[varID].vec.data(), vars1[varID].nmiss);
               vars1[varID].nsamp = 0;
             }
         }
@@ -475,15 +456,6 @@ Vertstat(void *process)
       tsID++;
     }
 
-  for (varID = 0; varID < nvars; varID++)
-    {
-      Free(vars1[varID].ptr);
-      if (samp1[varID].ptr) Free(samp1[varID].ptr);
-      if (lvarstd) Free(vars2[varID].ptr);
-    }
-
-  if (field.ptr) Free(field.ptr);
-
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
