@@ -24,7 +24,7 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
+
 #include "after_vertint.h"
 #include "util_string.h"
 
@@ -45,7 +45,7 @@ Vertwind(void *process)
 
   cdoInitialize(process);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   int vlistID1 = cdoStreamInqVlist(streamID1);
 
@@ -64,7 +64,7 @@ Vertwind(void *process)
       if (code <= 0)
         {
           vlistInqVarName(vlistID1, varID, varname);
-          strtolower(varname);
+          cstrToLowerCase(varname);
 
           // clang-format off
           if      (strcmp(varname, "st") == 0) code = temp_code;
@@ -152,30 +152,30 @@ Vertwind(void *process)
   int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
+  CdoStreamID streamID2 = cdoOpenWrite(1);
 
-  pstreamDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           size_t offset = (size_t) levelID * gridsize;
 
           if (varID == tempID)
-            pstreamReadRecord(streamID1, &temp[offset], &nmiss);
+            cdoReadRecord(streamID1, &temp[offset], &nmiss);
           else if (varID == sqID)
-            pstreamReadRecord(streamID1, &sq[offset], &nmiss);
+            cdoReadRecord(streamID1, &sq[offset], &nmiss);
           else if (varID == omegaID)
-            pstreamReadRecord(streamID1, &omega[offset], &nmiss);
+            cdoReadRecord(streamID1, &omega[offset], &nmiss);
           else if (varID == psID && zaxisInqType(zaxisID) == ZAXIS_HYBRID)
-            pstreamReadRecord(streamID1, ps_prog.data(), &nmiss);
+            cdoReadRecord(streamID1, ps_prog.data(), &nmiss);
         }
 
       if (zaxisInqType(zaxisID) == ZAXIS_HYBRID) presh(fpress.data(), hpress.data(), vct.data(), ps_prog.data(), nlevel, gridsize);
@@ -215,8 +215,8 @@ Vertwind(void *process)
           for (size_t i = 0; i < gridsize; i++)
             if (DBL_IS_EQUAL(wms[offset + i], missval_out)) nmiss_out++;
 
-          pstreamDefRecord(streamID2, 0, levelID);
-          pstreamWriteRecord(streamID2, &wms[offset], nmiss_out);
+          cdoDefRecord(streamID2, 0, levelID);
+          cdoWriteRecord(streamID2, &wms[offset], nmiss_out);
         }
 
       tsID++;
