@@ -24,7 +24,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 
 static const char WCT_NAME[] = "wind_chill_temperature";
 static const char WCT_LONGNAME[] = "Windchill temperature describes the fact that low temperatures are felt "
@@ -49,74 +48,64 @@ windchillTemperature(double t, double ff, double missval)
 }
 
 static void
-farexpr(Field *field1, Field field2, double (*expression)(double, double, double))
+farexpr(Field &field1, Field &field2, double (*expression)(double, double, double))
 {
-  size_t i, len;
-  const int grid1 = field1->grid;
-  const size_t nmiss1 = field1->nmiss;
-  const double missval1 = field1->missval;
-  double *array1 = field1->ptr;
-  const int grid2 = field2.grid;
-  const size_t nmiss2 = field2.nmiss;
+  const double missval1 = field1.missval;
   const double missval2 = field2.missval;
-  const double *array2 = field2.ptr;
 
-  len = gridInqSize(grid1);
-  if (len != gridInqSize(grid2)) cdoAbort("Fields have different gridsize (%s)", __func__);
+  const size_t len = field1.size;
+  if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
 
-  if (nmiss1 > 0 || nmiss2 > 0)
+  if (field1.nmiss || field2.nmiss)
     {
-      for (i = 0; i < len; i++)
-        if (DBL_IS_EQUAL(array1[i], missval1) || DBL_IS_EQUAL(array2[i], missval2))
-          array1[i] = missval1;
+      for (size_t i = 0; i < len; i++)
+        if (DBL_IS_EQUAL(field1.vec[i], missval1) || DBL_IS_EQUAL(field2.vec[i], missval2))
+          field1.vec[i] = missval1;
         else
-          array1[i] = expression(array1[i], array2[i], missval1);
+          field1.vec[i] = expression(field1.vec[i], field2.vec[i], missval1);
     }
   else
     {
-      for (i = 0; i < len; i++) array1[i] = expression(array1[i], array2[i], missval1);
+      for (size_t i = 0; i < len; i++) field1.vec[i] = expression(field1.vec[i], field2.vec[i], missval1);
     }
 
-  field1->nmiss = arrayNumMV(len, array1, missval1);
+  field1.nmiss = fieldNumMiss(field1);
 }
 
 void *
 Wct(void *process)
 {
   int nrecs, nrecs2;
-  size_t nmiss;
   int varID1, varID2;
   int levelID1, levelID2;
 
   cdoInitialize(process);
   cdoOperatorAdd("wct", 0, 0, nullptr);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-  int streamID2 = cdoStreamOpenRead(cdoStreamName(1));
+  CdoStreamID streamID1 = cdoOpenRead(0);
+  CdoStreamID streamID2 = cdoOpenRead(1);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = cdoStreamInqVlist(streamID2);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = cdoStreamInqVlist(streamID2);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
 
   vlistCompare(vlistID1, vlistID2, CMP_DIM);
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
+  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field1, field2;
-  fieldInit(field1);
-  fieldInit(field2);
-  field1.ptr = (double *) Malloc(gridsize * sizeof(double));
-  field2.ptr = (double *) Malloc(gridsize * sizeof(double));
+  field1.resize(gridsizemax);
+  field2.resize(gridsizemax);
 
   if (Options::cdoVerbose) cdoPrint("Number of timesteps: file1 %d, file2 %d", vlistNtsteps(vlistID1), vlistNtsteps(vlistID2));
 
-  int vlistID3 = vlistCreate();
-  int gridID = vlistInqVarGrid(vlistID1, FIRST_VAR);
-  int zaxisID = vlistInqVarZaxis(vlistID1, FIRST_VAR);
-  int varID3 = vlistDefVar(vlistID3, gridID, zaxisID, TIME_VARYING);
+  const int vlistID3 = vlistCreate();
+  const int gridID = vlistInqVarGrid(vlistID1, FIRST_VAR);
+  const int zaxisID = vlistInqVarZaxis(vlistID1, FIRST_VAR);
+  const int varID3 = vlistDefVar(vlistID3, gridID, zaxisID, TIME_VARYING);
 
-  int taxisID3 = cdoTaxisCreate(TAXIS_RELATIVE);
+  const int taxisID3 = cdoTaxisCreate(TAXIS_RELATIVE);
   taxisDefTunit(taxisID3, TUNIT_MINUTE);
   taxisDefCalendar(taxisID3, CALENDAR_STANDARD);
   taxisDefRdate(taxisID3, 19550101);
@@ -127,9 +116,8 @@ Wct(void *process)
   vlistDefVarLongname(vlistID3, varID3, WCT_LONGNAME);
   vlistDefVarUnits(vlistID3, varID3, WCT_UNITS);
 
-  int streamID3 = cdoStreamOpenWrite(cdoStreamName(2));
-
-  pstreamDefVlist(streamID3, vlistID3);
+  CdoStreamID streamID3 = cdoOpenWrite(2);
+  cdoDefVlist(streamID3, vlistID3);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
@@ -138,32 +126,27 @@ Wct(void *process)
       if (nrecs2 == 0) cdoAbort("Input streams have different number of timesteps!");
 
       taxisCopyTimestep(taxisID3, taxisID1);
-      pstreamDefTimestep(streamID3, tsID);
+      cdoDefTimestep(streamID3, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID1, &levelID1);
-          pstreamReadRecord(streamID1, field1.ptr, &nmiss);
-          field1.nmiss = nmiss;
+          cdoInqRecord(streamID1, &varID1, &levelID1);
+          cdoReadRecord(streamID1, field1.vec.data(), &field1.nmiss);
 
-          pstreamInqRecord(streamID2, &varID2, &levelID2);
-          pstreamReadRecord(streamID2, field2.ptr, &nmiss);
-          field2.nmiss = nmiss;
+          cdoInqRecord(streamID2, &varID2, &levelID2);
+          cdoReadRecord(streamID2, field2.vec.data(), &field2.nmiss);
 
           if (varID1 != varID2 || levelID1 != levelID2) cdoAbort("Input streams have different structure!");
 
           if (varID1 != FIRST_VAR) continue;
 
-          field1.grid = vlistInqVarGrid(vlistID1, varID1);
           field1.missval = vlistInqVarMissval(vlistID1, varID1);
-
-          field2.grid = vlistInqVarGrid(vlistID2, varID2);
           field2.missval = vlistInqVarMissval(vlistID2, varID2);
 
-          farexpr(&field1, field2, windchillTemperature);
+          farexpr(field1, field2, windchillTemperature);
 
-          pstreamDefRecord(streamID3, varID3, levelID1);
-          pstreamWriteRecord(streamID3, field1.ptr, field1.nmiss);
+          cdoDefRecord(streamID3, varID3, levelID1);
+          cdoWriteRecord(streamID3, field1.vec.data(), field1.nmiss);
         }
 
       tsID++;
@@ -173,9 +156,6 @@ Wct(void *process)
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
-  if (field1.ptr) Free(field1.ptr);
-  if (field2.ptr) Free(field2.ptr);
-
   cdoFinish();
 
   return nullptr;
