@@ -27,8 +27,8 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "grid.h"
-#include "pstream_int.h"
+#include <mpim_grid.h>
+#include "griddes.h"
 #include "specspace.h"
 #include "listarray.h"
 #include "util_string.h"
@@ -44,13 +44,13 @@ Wind(void *process)
   size_t nmiss;
   long ntr = -1;
   int varID1 = -1, varID2 = -1;
-  SPTRANS *sptrans = nullptr;
-  DVTRANS *dvtrans = nullptr;
+  SPTRANS sptrans;
+  DVTRANS dvtrans;
   char varname[CDI_MAX_NAME];
 
   cdoInitialize(process);
 
-  const bool lcopy = UNCHANGED_RECORD;
+  const bool lcopy = unchangedRecord();
 
   // clang-format off
   const int UV2DV  = cdoOperatorAdd("uv2dv",  0, 0, nullptr);
@@ -78,7 +78,7 @@ Wind(void *process)
     }
   // clang-format on
 
-  const int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   const int vlistID1 = cdoStreamInqVlist(streamID1);
   const int vlistID2 = vlistDuplicate(vlistID1);
@@ -101,7 +101,7 @@ Wind(void *process)
           if (pdis != 255 || code <= 0)
             {
               vlistInqVarName(vlistID1, varID, varname);
-              strtolower(varname);
+              cstrToLowerCase(varname);
               if (cstrIsEqual(varname, "u")) code = 131;
               if (cstrIsEqual(varname, "v")) code = 132;
             }
@@ -115,14 +115,14 @@ Wind(void *process)
           if (pdis != 255)  // GRIB2
             {
               vlistInqVarName(vlistID1, varID, varname);
-              strtolower(varname);
+              cstrToLowerCase(varname);
               if (cstrIsEqual(varname, "d")) code = 155;
               if (cstrIsEqual(varname, "vo")) code = 138;
             }
           else if (code <= 0)
             {
               vlistInqVarName(vlistID1, varID, varname);
-              strtolower(varname);
+              cstrToLowerCase(varname);
               if (cstrIsEqual(varname, "sd")) code = 155;
               if (cstrIsEqual(varname, "svo")) code = 138;
             }
@@ -207,7 +207,7 @@ Wind(void *process)
           const long nlat = gridInqYsize(gridID1);
           ntr = gridInqTrunc(gridID2);
 
-          sptrans = sptrans_new(nlon, nlat, ntr, 1);
+          sptrans.init(nlon, nlat, ntr, 1);
         }
     }
   else if (ldv2uv)
@@ -255,8 +255,8 @@ Wind(void *process)
           const long nlat = gridInqYsize(gridID2);
           ntr = gridInqTrunc(gridID1);
 
-          sptrans = sptrans_new(nlon, nlat, ntr, 0);
-          dvtrans = dvtrans_new(ntr);
+          sptrans.init(nlon, nlat, ntr, 0);
+          dvtrans.init(ntr);
         }
     }
   else if (operatorID == DV2PS)
@@ -287,8 +287,8 @@ Wind(void *process)
         }
     }
 
-  const int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   const size_t gridsizemax = vlistGridsizeMax(vlistID1);
   std::vector<double> array1(gridsizemax);
@@ -311,15 +311,15 @@ Wind(void *process)
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           if ((varID1 != -1 && varID2 != -1) && (varID == varID1 || varID == varID2))
             {
-              pstreamReadRecord(streamID1, array1.data(), &nmiss);
+              cdoReadRecord(streamID1, array1.data(), &nmiss);
               if (nmiss) cdoAbort("Missing values unsupported for spectral data!");
 
               const size_t gridsize = gridInqSize(gridID1);
@@ -332,15 +332,15 @@ Wind(void *process)
             }
           else
             {
-              pstreamDefRecord(streamID2, varID, levelID);
+              cdoDefRecord(streamID2, varID, levelID);
               if (lcopy)
                 {
-                  pstreamCopyRecord(streamID2, streamID1);
+                  cdoCopyRecord(streamID2, streamID1);
                 }
               else
                 {
-                  pstreamReadRecord(streamID1, array1.data(), &nmiss);
-                  pstreamWriteRecord(streamID2, array1.data(), nmiss);
+                  cdoReadRecord(streamID1, array1.data(), &nmiss);
+                  cdoWriteRecord(streamID2, array1.data(), nmiss);
                 }
             }
         }
@@ -363,14 +363,14 @@ Wind(void *process)
               for (size_t levelID = 0; levelID < nlev; levelID++)
                 {
                   const size_t offset = gridsize * levelID;
-                  pstreamDefRecord(streamID2, varID2, levelID);
-                  pstreamWriteRecord(streamID2, &ovar2[offset], 0);
+                  cdoDefRecord(streamID2, varID2, levelID);
+                  cdoWriteRecord(streamID2, &ovar2[offset], 0);
                 }
               for (size_t levelID = 0; levelID < nlev; levelID++)
                 {
                   const size_t offset = gridsize * levelID;
-                  pstreamDefRecord(streamID2, varID1, levelID);
-                  pstreamWriteRecord(streamID2, &ovar1[offset], 0);
+                  cdoDefRecord(streamID2, varID1, levelID);
+                  cdoWriteRecord(streamID2, &ovar1[offset], 0);
                 }
             }
           else if (ldv2uv)
@@ -378,14 +378,14 @@ Wind(void *process)
               for (size_t levelID = 0; levelID < nlev; levelID++)
                 {
                   const size_t offset = gridsize * levelID;
-                  pstreamDefRecord(streamID2, varID1, levelID);
-                  pstreamWriteRecord(streamID2, &ovar1[offset], 0);
+                  cdoDefRecord(streamID2, varID1, levelID);
+                  cdoWriteRecord(streamID2, &ovar1[offset], 0);
                 }
               for (size_t levelID = 0; levelID < nlev; levelID++)
                 {
                   const size_t offset = gridsize * levelID;
-                  pstreamDefRecord(streamID2, varID2, levelID);
-                  pstreamWriteRecord(streamID2, &ovar2[offset], 0);
+                  cdoDefRecord(streamID2, varID2, levelID);
+                  cdoWriteRecord(streamID2, &ovar2[offset], 0);
                 }
             }
         }
@@ -396,9 +396,6 @@ Wind(void *process)
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
-  sptrans_delete(sptrans);
-  if (dvtrans) dvtrans_delete(dvtrans);
-
   cdoFinish();
 
   return nullptr;
