@@ -24,15 +24,17 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
-#include "grid.h"
+
+#include <mpim_grid.h>
+#include "griddes.h"
+#include "gridreference.h"
 
 void *
 Writegrid(void *process)
 {
   cdoInitialize(process);
 
-  int streamID = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID = cdoOpenRead(0);
 
   int vlistID = cdoStreamInqVlist(streamID);
   int gridID = vlistGrid(vlistID, 0);
