@@ -24,7 +24,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 
 void *
 Writerandom(void *process)
@@ -36,7 +35,7 @@ Writerandom(void *process)
 
   cdoInitialize(process);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
   int vlistID1 = cdoStreamInqVlist(streamID1);
   int vlistID2 = vlistDuplicate(vlistID1);
@@ -45,15 +44,15 @@ Writerandom(void *process)
   int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
+  CdoStreamID streamID2 = cdoOpenWrite(1);
 
-  pstreamDefVlist(streamID2, vlistID2);
+  cdoDefVlist(streamID2, vlistID2);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
       taxisCopyTimestep(taxisID2, taxisID1);
-      pstreamDefTimestep(streamID2, tsID);
+      cdoDefTimestep(streamID2, tsID);
 
       std::vector<std::vector<double>> recdata(nrecs);
       std::vector<int> recvarID(nrecs);
@@ -63,12 +62,12 @@ Writerandom(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
           gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
           recvarID[recID] = varID;
           reclevelID[recID] = levelID;
           recdata[recID].resize(gridsize);
-          pstreamReadRecord(streamID1, recdata[recID].data(), &recnmiss[recID]);
+          cdoReadRecord(streamID1, recdata[recID].data(), &recnmiss[recID]);
         }
 
       for (int recID = 0; recID < nrecs; recID++) recindex[recID] = -1;
@@ -102,8 +101,8 @@ Writerandom(void *process)
           varID = recvarID[rindex];
           levelID = reclevelID[rindex];
           gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, recdata[rindex].data(), recnmiss[rindex]);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, recdata[rindex].data(), recnmiss[rindex]);
         }
 
       tsID++;
