@@ -69,7 +69,7 @@
 
 #include "cdo_int.h"
 #include "cdo_task.h"
-#include "pstream_int.h"
+
 #include "timer.h"
 #include "datetime.h"
 #include "printinfo.h"
@@ -86,11 +86,12 @@ enum
 class ReadArguments
 {
 public:
-  ReadArguments(FieldVector2D &input_vars) : vars(input_vars) { };
-  ~ReadArguments() { };
+  ReadArguments(FieldVector2D &input_vars) : vars(input_vars){};
+  ~ReadArguments(){};
   int tsIDnext;
-  int streamID, nrecs;
-  RecordInfo *recinfo;
+  CdoStreamID streamID;
+    int nrecs;
+  RecordInfo *recList;
   FieldVector2D &vars;
 };
 
@@ -103,8 +104,8 @@ cdoReadTimestep(void *rarg)
   size_t nmiss;
   ReadArguments *readarg = (ReadArguments *) rarg;
   FieldVector2D &input_vars = readarg->vars;
-  RecordInfo *recinfo = readarg->recinfo;
-  int streamID = readarg->streamID;
+  RecordInfo *recList = readarg->recList;
+  CdoStreamID streamID = readarg->streamID;
   int tsIDnext = readarg->tsIDnext;
   int nrecs = readarg->nrecs;
 
@@ -112,54 +113,30 @@ cdoReadTimestep(void *rarg)
 
   for (int recID = 0; recID < nrecs; ++recID)
     {
-      pstreamInqRecord(streamID, &varID, &levelID);
+      cdoInqRecord(streamID, &varID, &levelID);
 
-      if (tsIDnext == 1 && recinfo)
+      if (tsIDnext == 1 && recList)
         {
-          recinfo[recID].varID = varID;
-          recinfo[recID].levelID = levelID;
+          recList[recID].varID = varID;
+          recList[recID].levelID = levelID;
         }
 
       if (Options::CDO_Memtype == MEMTYPE_FLOAT)
-        pstreamReadRecordF(streamID, input_vars[varID][levelID].ptr2f, &nmiss);
+        cdoReadRecordF(streamID, input_vars[varID][levelID].vecf.data(), &nmiss);
       else
-        pstreamReadRecord(streamID, input_vars[varID][levelID].ptr2, &nmiss);
+        cdoReadRecord(streamID, input_vars[varID][levelID].vec.data(), &nmiss);
 
-      input_vars[varID][levelID].nmiss2 = nmiss;
+      input_vars[varID][levelID].nmiss = nmiss;
     }
 
   // timer_stop(timer_read);
 
   num_recs = cdoStreamInqTimestep(streamID, tsIDnext);
+  //num_recs = cdoStreamInqTimestep(streamID, tsIDnext);
 
   return ((void *) &num_recs);
 }
 
-static void
-cdoUpdateVars(int nvars, int vlistID, FieldVector2D &vars)
-{
-  for (int varID = 0; varID < nvars; varID++)
-    {
-      int nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID, varID));
-      for (int levelID = 0; levelID < nlevels; levelID++)
-        {
-          if (Options::CDO_Memtype == MEMTYPE_FLOAT)
-            {
-              float *tmp = vars[varID][levelID].ptrf;
-              vars[varID][levelID].ptrf = vars[varID][levelID].ptr2f;
-              vars[varID][levelID].ptr2f = tmp;
-            }
-          else
-            {
-              double *tmp = vars[varID][levelID].ptr;
-              vars[varID][levelID].ptr = vars[varID][levelID].ptr2;
-              vars[varID][levelID].ptr2 = tmp;
-            }
-          vars[varID][levelID].nmiss = vars[varID][levelID].nmiss2;
-        }
-    }
-}
-
 static void
 vlistSetFrequency(int vlistID, int comparelen)
 {
@@ -173,7 +150,7 @@ vlistSetFrequency(int vlistID, int comparelen)
 }
 
 static void
-timstatAddOperators(void)
+addOperators(void)
 {
   // clang-format off
   cdoOperatorAdd("xtimmin",    func_min,   DATE_LEN, nullptr);
@@ -213,16 +190,15 @@ XTimstat(void *process)
   int64_t vdate = 0, vdate0 = 0;
   int vtime = 0, vtime0 = 0;
   int varID;
-  int streamID3 = -1;
+  CdoStreamID streamID3 = CDO_STREAM_UNDEF;
   int vlistID3, taxisID3 = -1;
   bool lvfrac = false;
-  int nwpv;  // number of words per value; real:1  complex:2
   char indate1[DATE_LEN + 1], indate2[DATE_LEN + 1];
   double vfrac = 1;
 
   cdoInitialize(process);
 
-  timstatAddOperators();
+  addOperators();
 
   // clang-format off
   int operatorID = cdoOperatorID();
@@ -254,8 +230,8 @@ XTimstat(void *process)
 
   int cmplen = DATE_LEN - comparelen;
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-  // int streamID1 = cdoStreamOpenRead(cdoGetStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
+  // CdoStreamID streamID1 = cdoOpenRead(cdoGetStreamName(0));
 
   int vlistID1 = cdoStreamInqVlist(streamID1);
   int vlistID2 = vlistDuplicate(vlistID1);
@@ -272,8 +248,8 @@ XTimstat(void *process)
 
   vlistSetFrequency(vlistID2, comparelen);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
   if (Options::cdoDiag)
     {
@@ -281,7 +257,7 @@ XTimstat(void *process)
       strcpy(filename, cdoOperatorName(operatorID));
       strcat(filename, "_");
       strcat(filename, cdoGetStreamName(1));
-      streamID3 = cdoStreamOpenWrite(filename);
+      streamID3 = cdoOpenWrite(filename);
 
       vlistID3 = vlistDuplicate(vlistID1);
 
@@ -298,26 +274,26 @@ XTimstat(void *process)
       taxisWithBounds(taxisID3);
       vlistDefTaxis(vlistID3, taxisID3);
 
-      pstreamDefVlist(streamID3, vlistID3);
+      cdoDefVlist(streamID3, vlistID3);
     }
 
   DateTimeList dtlist;
   dtlist.setStat(timestat_date);
   dtlist.setCalendar(taxisInqCalendar(taxisID1));
 
-  size_t gridsizemax = vlistGridsizeMax(vlistID1);
-  if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
-
   int FIELD_MEMTYPE = 0;
   if (operfunc == func_mean && Options::CDO_Memtype == MEMTYPE_FLOAT) FIELD_MEMTYPE = FIELD_FLT;
-  FieldVector2D input_vars;
-  fieldsFromVlist(vlistID1, input_vars, FIELD_PTR | FIELD_PTR2 | FIELD_MEMTYPE);
+  FieldVector3D input_vars(2);
+  fieldsFromVlist(vlistID1, input_vars[0], FIELD_VEC | FIELD_MEMTYPE);
+  fieldsFromVlist(vlistID1, input_vars[1], FIELD_VEC | FIELD_MEMTYPE);
   FieldVector2D samp1, vars1, vars2;
   fieldsFromVlist(vlistID1, samp1, FIELD_NONE);
-  fieldsFromVlist(vlistID1, vars1, FIELD_PTR);
-  if (lvars2) fieldsFromVlist(vlistID1, vars2, FIELD_PTR);
+  fieldsFromVlist(vlistID1, vars1, FIELD_VEC);
+  if (lvars2) fieldsFromVlist(vlistID1, vars2, FIELD_VEC);
 
-  ReadArguments readarg(input_vars);
+  int curFirst = 0, curSecond = 1;
+
+  ReadArguments readarg(input_vars[curFirst]);
   readarg.streamID = streamID1;
 
   bool lparallelread = Options::CDO_Parallel_Read > 0;
@@ -339,7 +315,7 @@ XTimstat(void *process)
   int otsID = 0;
   int nrecs = cdoStreamInqTimestep(streamID1, tsID);
   int maxrecs = nrecs;
-  std::vector<RecordInfo> recinfo(maxrecs);
+  std::vector<RecordInfo> recList(maxrecs);
 
   tsID++;
   while (true)
@@ -358,7 +334,8 @@ XTimstat(void *process)
 
           readarg.tsIDnext = tsID;
           readarg.nrecs = nrecs;
-          readarg.recinfo = recinfo.data();
+          readarg.recList = recList.data();
+          readarg.vars = input_vars[curFirst];
 
           if (ltsfirst || !lparallelread)
             {
@@ -372,10 +349,11 @@ XTimstat(void *process)
 
           nrecs = *(int *) readresult;
 
-          cdoUpdateVars(nvars, vlistID1, input_vars);
+          // std::swap(curFirst, curSecond);
 
           if (nrecs && lparallelread)
             {
+              readarg.vars = input_vars[curFirst];
               readarg.tsIDnext = tsID + 1;
               cdo_task_start(read_task, cdoReadTimestep, &readarg);
             }
@@ -383,69 +361,64 @@ XTimstat(void *process)
           if (nsets == 0)
             {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(maxrecs, recinfo, input_vars, vars1, samp1) if (maxrecs > 1)
+#pragma omp parallel for default(none) shared(maxrecs, recList, curFirst, input_vars, vars1, samp1) if (maxrecs > 1)
 #endif
               for (int recID = 0; recID < maxrecs; recID++)
                 {
-                  int varID = recinfo[recID].varID;
-                  int levelID = recinfo[recID].levelID;
+                  int varID = recList[recID].varID;
+                  int levelID = recList[recID].levelID;
 
                   Field &rvars1 = vars1[varID][levelID];
-                  Field &rinput_var = input_vars[varID][levelID];
+                  Field &rinput_var = input_vars[curFirst][varID][levelID];
 
-                  int nwpv = rvars1.nwpv;
-                  size_t gridsize = rvars1.size;
+                  size_t fieldsize = rvars1.size;
                   size_t nmiss = rinput_var.nmiss;
 
-                  farcpy(rvars1, rinput_var);
+                  vfarcpy(rvars1, rinput_var);
                   rvars1.nmiss = nmiss;
-                  if (nmiss > 0 || samp1[varID][levelID].ptr)
+                  if (nmiss || !samp1[varID][levelID].empty())
                     {
-                      if (samp1[varID][levelID].ptr == nullptr)
-                        samp1[varID][levelID].ptr = (double *) malloc(nwpv * gridsize * sizeof(double));
+                      if (samp1[varID][levelID].empty())
+                        samp1[varID][levelID].resize(fieldsize);
 
-                      for (size_t i = 0; i < nwpv * gridsize; i++)
-                        samp1[varID][levelID].ptr[i] = !DBL_IS_EQUAL(rvars1.ptr[i], rvars1.missval);
+                      for (size_t i = 0; i < fieldsize; i++)
+                        samp1[varID][levelID].vec[i] = !DBL_IS_EQUAL(rvars1.vec[i], rvars1.missval);
                     }
                 }
             }
           else
             {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(lvarstd, nsets, maxrecs, recinfo, input_vars, vars1, samp1, vars2, operfunc) if (maxrecs > 1)
+#pragma omp parallel for default(none) \
+  shared(lvarstd, nsets, maxrecs, recList, curFirst, input_vars, vars1, samp1, vars2, operfunc) if (maxrecs > 1)
 #endif
               for (int recID = 0; recID < maxrecs; recID++)
                 {
-                  int varID = recinfo[recID].varID;
-                  int levelID = recinfo[recID].levelID;
+                  int varID = recList[recID].varID;
+                  int levelID = recList[recID].levelID;
 
                   Field &rvars1 = vars1[varID][levelID];
-                  Field &rinput_var = input_vars[varID][levelID];
+                  Field &rinput_var = input_vars[curFirst][varID][levelID];
 
-                  int nwpv = rvars1.nwpv;
-                  size_t gridsize = rvars1.size;
+                  size_t fieldsize = rvars1.size;
                   size_t nmiss = rinput_var.nmiss;
 
-                  if (nmiss > 0 || samp1[varID][levelID].ptr)
+                  if (nmiss || !samp1[varID][levelID].empty())
                     {
-                      if (samp1[varID][levelID].ptr == nullptr)
-                        {
-                          samp1[varID][levelID].ptr = (double *) malloc(nwpv * gridsize * sizeof(double));
-                          for (size_t i = 0; i < nwpv * gridsize; i++) samp1[varID][levelID].ptr[i] = nsets;
-                        }
+                      if (samp1[varID][levelID].empty()) samp1[varID][levelID].resize(fieldsize, nsets);
 
-                      for (size_t i = 0; i < nwpv * gridsize; i++)
-                        if (!DBL_IS_EQUAL(rinput_var.ptr[i], rvars1.missval)) samp1[varID][levelID].ptr[i]++;
+                      for (size_t i = 0; i < fieldsize; i++)
+                        if (!DBL_IS_EQUAL(rinput_var.vec[i], rvars1.missval)) samp1[varID][levelID].vec[i]++;
                     }
 
                   if (lvarstd)
                     {
-                      farsumq(vars2[varID][levelID], rinput_var);
-                      farsum(rvars1, rinput_var);
+                      vfarsumq(vars2[varID][levelID], rinput_var);
+                      vfarsum(rvars1, rinput_var);
                     }
                   else
                     {
-                      farfun(rvars1, rinput_var, operfunc);
+                      vfarfun(rvars1, rinput_var, operfunc);
                     }
                 }
             }
@@ -453,12 +426,12 @@ XTimstat(void *process)
           if (nsets == 0 && lvarstd)
             for (int recID = 0; recID < maxrecs; recID++)
               {
-                int varID = recinfo[recID].varID;
-                int levelID = recinfo[recID].levelID;
+                int varID = recList[recID].varID;
+                int levelID = recList[recID].levelID;
 
                 if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
 
-                farmoq(vars2[varID][levelID], vars1[varID][levelID]);
+                vfarmoq(vars2[varID][levelID], vars1[varID][levelID]);
               }
 
           vdate0 = vdate;
@@ -472,27 +445,27 @@ XTimstat(void *process)
       if (lmean)
         {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(vlistID1, nsets, maxrecs, recinfo, vars1, samp1) if (maxrecs > 1)
+#pragma omp parallel for default(none) shared(vlistID1, nsets, maxrecs, recList, vars1, samp1) if (maxrecs > 1)
 #endif
           for (int recID = 0; recID < maxrecs; recID++)
             {
-              int varID = recinfo[recID].varID;
-              int levelID = recinfo[recID].levelID;
+              int varID = recList[recID].varID;
+              int levelID = recList[recID].levelID;
 
               if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
 
-              if (samp1[varID][levelID].ptr == nullptr)
-                farcdiv(vars1[varID][levelID], (double) nsets);
+              if (!samp1[varID][levelID].empty())
+                vfardiv(vars1[varID][levelID], samp1[varID][levelID]);
               else
-                fardiv(vars1[varID][levelID], samp1[varID][levelID]);
+                vfarcdiv(vars1[varID][levelID], (double) nsets);
             }
         }
       else if (lvarstd)
         {
           for (int recID = 0; recID < maxrecs; recID++)
             {
-              int varID = recinfo[recID].varID;
-              int levelID = recinfo[recID].levelID;
+              int varID = recList[recID].varID;
+              int levelID = recList[recID].levelID;
 
               if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
 
@@ -500,19 +473,19 @@ XTimstat(void *process)
               Field &rvars1 = vars1[varID][levelID];
               Field &rvars2 = vars2[varID][levelID];
 
-              if (rsamp1.ptr == nullptr)
+              if (!rsamp1.empty())
                 {
                   if (lstd)
-                    farcstd(rvars1, rvars2, nsets, divisor);
+                    vfarstd(rvars1, rvars2, rsamp1, divisor);
                   else
-                    farcvar(rvars1, rvars2, nsets, divisor);
+                    vfarvar(rvars1, rvars2, rsamp1, divisor);
                 }
               else
                 {
                   if (lstd)
-                    farstd(rvars1, rvars2, rsamp1, divisor);
+                    vfarcstd(rvars1, rvars2, nsets, divisor);
                   else
-                    farvar(rvars1, rvars2, rsamp1, divisor);
+                    vfarcvar(rvars1, rvars2, nsets, divisor);
                 }
             }
         }
@@ -528,57 +501,56 @@ XTimstat(void *process)
       if (lvfrac && operfunc == func_mean)
         for (int recID = 0; recID < maxrecs; recID++)
           {
-            int varID = recinfo[recID].varID;
-            int levelID = recinfo[recID].levelID;
+            int varID = recList[recID].varID;
+            int levelID = recList[recID].levelID;
             Field &rvars1 = vars1[varID][levelID];
 
             if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
 
-            nwpv = rvars1.nwpv;
-            size_t gridsize = rvars1.size;
+            size_t fieldsize = rvars1.size;
             double missval = rvars1.missval;
-            if (samp1[varID][levelID].ptr)
+            if (!samp1[varID][levelID].empty())
               {
                 size_t irun = 0;
-                for (size_t i = 0; i < nwpv * gridsize; ++i)
+                for (size_t i = 0; i < fieldsize; ++i)
                   {
-                    if ((samp1[varID][levelID].ptr[i] / nsets) < vfrac)
+                    if ((samp1[varID][levelID].vec[i] / nsets) < vfrac)
                       {
-                        rvars1.ptr[i] = missval;
+                        rvars1.vec[i] = missval;
                         irun++;
                       }
                   }
 
-                if (irun) rvars1.nmiss = arrayNumMV(nwpv * gridsize, rvars1.ptr, missval);
+                if (irun) rvars1.nmiss = fieldNumMiss(rvars1);
               }
           }
 
       dtlist.statTaxisDefTimestep(taxisID2, nsets);
-      pstreamDefTimestep(streamID2, otsID);
+      cdoDefTimestep(streamID2, otsID);
 
       if (Options::cdoDiag)
         {
           dtlist.statTaxisDefTimestep(taxisID3, nsets);
-          pstreamDefTimestep(streamID3, otsID);
+          cdoDefTimestep(streamID3, otsID);
         }
 
       for (int recID = 0; recID < maxrecs; recID++)
         {
-          int varID = recinfo[recID].varID;
-          int levelID = recinfo[recID].levelID;
+          int varID = recList[recID].varID;
+          int levelID = recList[recID].levelID;
           Field &rvars1 = vars1[varID][levelID];
 
           if (otsID && vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
 
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, rvars1.ptr, rvars1.nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, rvars1.vec.data(), rvars1.nmiss);
 
           if (Options::cdoDiag)
             {
-              if (samp1[varID][levelID].ptr)
+              if (!samp1[varID][levelID].empty())
                 {
-                  pstreamDefRecord(streamID3, varID, levelID);
-                  pstreamWriteRecord(streamID3, samp1[varID][levelID].ptr, 0);
+                  cdoDefRecord(streamID3, varID, levelID);
+                  cdoWriteRecord(streamID3, samp1[varID][levelID].vec.data(), 0);
                 }
             }
         }
@@ -587,11 +559,6 @@ XTimstat(void *process)
       otsID++;
     }
 
-  fieldsFree(vlistID1, input_vars);
-  fieldsFree(vlistID1, vars1);
-  fieldsFree(vlistID1, samp1);
-  if (lvarstd) fieldsFree(vlistID1, vars2);
-
   if (Options::cdoDiag) cdoStreamClose(streamID3);
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
