@@ -34,7 +34,7 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
+
 #include "pmlist.h"
 
 #define MAX_DOY 373
@@ -76,7 +76,6 @@ Ydaystat(void *process)
   int year, month, day;
   int nrecs;
   int dayoy_nsets[MAX_DOY];
-  size_t nmiss;
   int64_t vdates[MAX_DOY];
   int vtimes[MAX_DOY];
   FieldVector2D vars1[MAX_DOY], vars2[MAX_DOY], samp1[MAX_DOY];
@@ -97,55 +96,52 @@ Ydaystat(void *process)
   cdoOperatorAdd("ydaystd",   func_std,   0, nullptr);
   cdoOperatorAdd("ydaystd1",  func_std1,  0, nullptr);
 
-  int operatorID = cdoOperatorID();
-  int operfunc = cdoOperatorF1(operatorID);
+  const int operatorID = cdoOperatorID();
+  const int operfunc = cdoOperatorF1(operatorID);
 
-  bool lrange  = operfunc == func_range;
-  bool lmean   = operfunc == func_mean || operfunc == func_avg;
-  bool lstd    = operfunc == func_std || operfunc == func_std1;
-  bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
-  int  divisor = operfunc == func_std1 || operfunc == func_var1;
+  const bool lrange  = operfunc == func_range;
+  const bool lmean   = operfunc == func_mean || operfunc == func_avg;
+  const bool lstd    = operfunc == func_std || operfunc == func_std1;
+  const bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
+  const int  divisor = operfunc == func_std1 || operfunc == func_var1;
   // clang-format on
-  bool lvars2 = lvarstd || lrange;
+  const bool lvars2 = lvarstd || lrange;
 
   for (int dayoy = 0; dayoy < MAX_DOY; ++dayoy) dayoy_nsets[dayoy] = 0;
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   if (taxisHasBounds(taxisID2)) taxisDeleteBounds(taxisID2);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
-  int maxrecs = vlistNrecs(vlistID1);
-  std::vector<RecordInfo> recinfo(maxrecs);
+  const int maxrecs = vlistNrecs(vlistID1);
+  std::vector<RecordInfo> recList(maxrecs);
 
-  size_t gridsizemax = vlistGridsizeMax(vlistID1);
+  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field;
-  fieldInit(field);
-  field.ptr = (double *) Malloc(gridsizemax * sizeof(double));
+  field.resize(gridsizemax);
 
   int tsID = 0;
   int otsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
-      int64_t vdate = taxisInqVdate(taxisID1);
-      int vtime = taxisInqVtime(taxisID1);
+      const int64_t vdate = taxisInqVdate(taxisID1);
+      const int vtime = taxisInqVtime(taxisID1);
 
       if (Options::cdoVerbose) cdoPrint("process timestep: %d %d %d", tsID + 1, vdate, vtime);
 
       cdiDecodeDate(vdate, &year, &month, &day);
 
-      int dayoy = 0;
-      if (month >= 1 && month <= 12) dayoy = (month - 1) * 31 + day;
-
+      const int dayoy = (month >= 1 && month <= 12) ? (month - 1) * 31 + day : 0;
       if (dayoy < 0 || dayoy >= MAX_DOY) cdoAbort("Day of year %d out of range (date=%d)!", dayoy, vdate);
 
       vdates[dayoy] = vdate;
@@ -154,76 +150,71 @@ Ydaystat(void *process)
       if (!vars1[dayoy].size())
         {
           fieldsFromVlist(vlistID1, samp1[dayoy], FIELD_NONE);
-          fieldsFromVlist(vlistID1, vars1[dayoy], FIELD_PTR);
-          if (lvars2) fieldsFromVlist(vlistID1, vars2[dayoy], FIELD_PTR);
+          fieldsFromVlist(vlistID1, vars1[dayoy], FIELD_VEC);
+          if (lvars2) fieldsFromVlist(vlistID1, vars2[dayoy], FIELD_VEC);
         }
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           if (tsID == 0)
             {
-              recinfo[recID].varID = varID;
-              recinfo[recID].levelID = levelID;
-              recinfo[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+              recList[recID].varID = varID;
+              recList[recID].levelID = levelID;
+              recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
             }
 
           Field &rsamp1 = samp1[dayoy][varID][levelID];
           Field &rvars1 = vars1[dayoy][varID][levelID];
-          int nsets = dayoy_nsets[dayoy];
 
-          size_t gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
+          const int nsets = dayoy_nsets[dayoy];
+          const size_t fieldsize = rvars1.size;
 
           if (nsets == 0)
             {
-              pstreamReadRecord(streamID1, rvars1.ptr, &nmiss);
-              rvars1.nmiss = nmiss;
+              cdoReadRecord(streamID1, rvars1.vec.data(), &rvars1.nmiss);
               if (lrange)
                 {
                   vars2[dayoy][varID][levelID].nmiss = rvars1.nmiss;
-                  for (size_t i = 0; i < gridsize; i++) vars2[dayoy][varID][levelID].ptr[i] = rvars1.ptr[i];
+                  vars2[dayoy][varID][levelID].vec = rvars1.vec;
                 }
 
-              if (nmiss > 0 || rsamp1.ptr)
+              if (rvars1.nmiss || !rsamp1.empty())
                 {
-                  if (rsamp1.ptr == nullptr) rsamp1.ptr = (double *) Malloc(gridsize * sizeof(double));
-
-                  for (size_t i = 0; i < gridsize; i++) rsamp1.ptr[i] = !DBL_IS_EQUAL(rvars1.ptr[i], rvars1.missval);
+                  if (rsamp1.empty()) rsamp1.resize(fieldsize);
+                  for (size_t i = 0; i < fieldsize; i++)
+                    rsamp1.vec[i] = !DBL_IS_EQUAL(rvars1.vec[i], rvars1.missval);
                 }
             }
           else
             {
-              pstreamReadRecord(streamID1, field.ptr, &nmiss);
-              field.nmiss = nmiss;
+              cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
+              field.size = fieldsize;
               field.grid = rvars1.grid;
               field.missval = rvars1.missval;
 
-              if (field.nmiss > 0 || rsamp1.ptr)
+              if (field.nmiss || !rsamp1.empty())
                 {
-                  if (rsamp1.ptr == nullptr)
-                    {
-                      rsamp1.ptr = (double *) Malloc(gridsize * sizeof(double));
-                      for (size_t i = 0; i < gridsize; i++) rsamp1.ptr[i] = nsets;
-                    }
+                  if (rsamp1.empty()) rsamp1.resize(fieldsize, nsets);
 
-                  for (size_t i = 0; i < gridsize; i++)
-                    if (!DBL_IS_EQUAL(field.ptr[i], rvars1.missval)) rsamp1.ptr[i]++;
+                  for (size_t i = 0; i < fieldsize; i++)
+                    if (!DBL_IS_EQUAL(field.vec[i], rvars1.missval)) rsamp1.vec[i]++;
                 }
 
               if (lvarstd)
                 {
-                  farsumq(vars2[dayoy][varID][levelID], field);
-                  farsum(rvars1, field);
+                  vfarsumq(vars2[dayoy][varID][levelID], field);
+                  vfarsum(rvars1, field);
                 }
               else if (lrange)
                 {
-                  farmin(vars2[dayoy][varID][levelID], field);
-                  farmax(rvars1, field);
+                  vfarmin(vars2[dayoy][varID][levelID], field);
+                  vfarmax(rvars1, field);
                 }
               else
                 {
-                  farfun(rvars1, field, operfunc);
+                  vfarfun(rvars1, field, operfunc);
                 }
             }
         }
@@ -231,12 +222,11 @@ Ydaystat(void *process)
       if (dayoy_nsets[dayoy] == 0 && lvarstd)
         for (int recID = 0; recID < maxrecs; recID++)
           {
-            if (recinfo[recID].lconst) continue;
+            if (recList[recID].lconst) continue;
 
-            int varID = recinfo[recID].varID;
-            int levelID = recinfo[recID].levelID;
-
-            farmoq(vars2[dayoy][varID][levelID], vars1[dayoy][varID][levelID]);
+            const int varID = recList[recID].varID;
+            const int levelID = recList[recID].levelID;
+            vfarmoq(vars2[dayoy][varID][levelID], vars1[dayoy][varID][levelID]);
           }
 
       dayoy_nsets[dayoy]++;
@@ -266,79 +256,67 @@ Ydaystat(void *process)
   for (int dayoy = 0; dayoy < MAX_DOY; dayoy++)
     if (dayoy_nsets[dayoy])
       {
-        int nsets = dayoy_nsets[dayoy];
+        const int nsets = dayoy_nsets[dayoy];
         for (int recID = 0; recID < maxrecs; recID++)
           {
-            if (recinfo[recID].lconst) continue;
+            if (recList[recID].lconst) continue;
 
-            int varID = recinfo[recID].varID;
-            int levelID = recinfo[recID].levelID;
+            const int varID = recList[recID].varID;
+            const int levelID = recList[recID].levelID;
             Field &rsamp1 = samp1[dayoy][varID][levelID];
             Field &rvars1 = vars1[dayoy][varID][levelID];
 
             if (lmean)
               {
-                if (rsamp1.ptr)
-                  fardiv(rvars1, rsamp1);
+                if (!rsamp1.empty())
+                  vfardiv(rvars1, rsamp1);
                 else
-                  farcdiv(rvars1, (double) nsets);
+                  vfarcdiv(rvars1, (double) nsets);
               }
             else if (lvarstd)
               {
                 Field &rvars2 = vars2[dayoy][varID][levelID];
-                if (rsamp1.ptr)
+                if (!rsamp1.empty())
                   {
                     if (lstd)
-                      farstd(rvars1, rvars2, rsamp1, divisor);
+                      vfarstd(rvars1, rvars2, rsamp1, divisor);
                     else
-                      farvar(rvars1, rvars2, rsamp1, divisor);
+                      vfarvar(rvars1, rvars2, rsamp1, divisor);
                   }
                 else
                   {
                     if (lstd)
-                      farcstd(rvars1, rvars2, nsets, divisor);
+                      vfarcstd(rvars1, rvars2, nsets, divisor);
                     else
-                      farcvar(rvars1, rvars2, nsets, divisor);
+                      vfarcvar(rvars1, rvars2, nsets, divisor);
                   }
               }
             else if (lrange)
               {
                 Field &rvars2 = vars2[dayoy][varID][levelID];
-                farsub(rvars1, rvars2);
+                vfarsub(rvars1, rvars2);
               }
           }
 
         taxisDefVdate(taxisID2, vdates[dayoy]);
         taxisDefVtime(taxisID2, vtimes[dayoy]);
-        pstreamDefTimestep(streamID2, otsID);
+        cdoDefTimestep(streamID2, otsID);
 
         for (int recID = 0; recID < maxrecs; recID++)
           {
-            if (otsID && recinfo[recID].lconst) continue;
+            if (otsID && recList[recID].lconst) continue;
 
-            int varID = recinfo[recID].varID;
-            int levelID = recinfo[recID].levelID;
+            const int varID = recList[recID].varID;
+            const int levelID = recList[recID].levelID;
             Field &rvars1 = vars1[dayoy][varID][levelID];
 
-            pstreamDefRecord(streamID2, varID, levelID);
-            pstreamWriteRecord(streamID2, rvars1.ptr, rvars1.nmiss);
+            cdoDefRecord(streamID2, varID, levelID);
+            cdoWriteRecord(streamID2, rvars1.vec.data(), rvars1.nmiss);
           }
 
         otsID++;
       }
 
-  for (int dayoy = 0; dayoy < MAX_DOY; dayoy++)
-    {
-      if (vars1[dayoy].size())
-        {
-          fieldsFree(vlistID1, samp1[dayoy]);
-          fieldsFree(vlistID1, vars1[dayoy]);
-          if (lvars2) fieldsFree(vlistID1, vars2[dayoy]);
-        }
-    }
-
-  if (field.ptr) Free(field.ptr);
-
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
