@@ -26,7 +26,7 @@
 #include "cdo_int.h"
 #include "param_conversion.h"
 #include "calendar.h"
-#include "pstream_int.h"
+
 #include "percentiles_hist.h"
 #include "percentiles.h"
 #include "datetime.h"
@@ -51,9 +51,7 @@ Ydrunpctl(void *process)
   int inp, its;
   size_t nmiss;
   int nlevels;
-  int year, month, day, dayoy;
-  int64_t vdate;
-  int vtime;
+  int year, month, day;
   int64_t vdates1[NDAY], vdates2[NDAY];
   int vtimes1[NDAY] /*, vtimes2[NDAY]*/;
   int nsets[NDAY];
@@ -65,81 +63,71 @@ Ydrunpctl(void *process)
 
   operatorInputArg("percentile number, number of timesteps");
   operatorCheckArgc(2);
-  double pn = parameter2double(operatorArgv()[0]);
+  const double pn = parameter2double(operatorArgv()[0]);
   percentile_check_number(pn);
-  int ndates = parameter2int(operatorArgv()[1]);
+  const int ndates = parameter2int(operatorArgv()[1]);
 
-  for (dayoy = 0; dayoy < NDAY; dayoy++)
-    {
-      nsets[dayoy] = 0;
-    }
+  for (int dayoy = 0; dayoy < NDAY; dayoy++) nsets[dayoy] = 0;
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-  int streamID2 = cdoStreamOpenRead(cdoStreamName(1));
-  int streamID3 = cdoStreamOpenRead(cdoStreamName(2));
+  CdoStreamID streamID1 = cdoOpenRead(0);
+  CdoStreamID streamID2 = cdoOpenRead(1);
+  CdoStreamID streamID3 = cdoOpenRead(2);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = cdoStreamInqVlist(streamID2);
-  int vlistID3 = cdoStreamInqVlist(streamID3);
-  int vlistID4 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = cdoStreamInqVlist(streamID2);
+  const int vlistID3 = cdoStreamInqVlist(streamID3);
+  const int vlistID4 = vlistDuplicate(vlistID1);
 
   vlistCompare(vlistID1, vlistID2, CMP_ALL);
   vlistCompare(vlistID1, vlistID3, CMP_ALL);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = vlistInqTaxis(vlistID2);
-  int taxisID3 = vlistInqTaxis(vlistID3);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = vlistInqTaxis(vlistID2);
+  const int taxisID3 = vlistInqTaxis(vlistID3);
   /* TODO - check that time axes 2 and 3 are equal */
 
-  int taxisID4 = taxisDuplicate(taxisID1);
+  const int taxisID4 = taxisDuplicate(taxisID1);
   if (taxisHasBounds(taxisID4)) taxisDeleteBounds(taxisID4);
   vlistDefTaxis(vlistID4, taxisID4);
 
-  int calendar = taxisInqCalendar(taxisID1);
-  int dpy = calendar_dpy(calendar);
+  const int calendar = taxisInqCalendar(taxisID1);
+  const int dpy = calendar_dpy(calendar);
 
-  int streamID4 = cdoStreamOpenWrite(cdoStreamName(3));
-  pstreamDefVlist(streamID4, vlistID4);
+  CdoStreamID streamID4 = cdoOpenWrite(3);
+  cdoDefVlist(streamID4, vlistID4);
 
-  int nvars = vlistNvars(vlistID1);
+  const int nvars = vlistNvars(vlistID1);
 
-  int maxrecs = vlistNrecs(vlistID1);
-  std::vector<RecordInfo> recinfo(maxrecs);
+  const int maxrecs = vlistNrecs(vlistID1);
+  std::vector<RecordInfo> recList(maxrecs);
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
+  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field;
-  fieldInit(field);
-  field.ptr = (double *) Malloc(gridsize * sizeof(double));
+  field.resize(gridsizemax);
 
   std::vector<CdoDateTime> datetime(ndates + 1);
 
   FieldVector3D vars1(ndates + 1);
-  for (its = 0; its < ndates; its++) fieldsFromVlist(vlistID1, vars1[its], FIELD_PTR);
+  for (its = 0; its < ndates; its++) fieldsFromVlist(vlistID1, vars1[its], FIELD_VEC);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID2, tsID)))
     {
       if (nrecs != cdoStreamInqTimestep(streamID3, tsID))
-        cdoAbort("Number of records at time step %d of %s and %s differ!", tsID + 1, cdoGetStreamName(1),
-                 cdoGetStreamName(2));
+        cdoAbort("Number of records at time step %d of %s and %s differ!", tsID + 1, cdoGetStreamName(1), cdoGetStreamName(2));
 
-      vdate = taxisInqVdate(taxisID2);
-      vtime = taxisInqVtime(taxisID2);
+      const int64_t vdate = taxisInqVdate(taxisID2);
+      const int vtime = taxisInqVtime(taxisID2);
 
       if (vdate != taxisInqVdate(taxisID3))
-        cdoAbort("Verification dates at time step %d of %s and %s differ!", tsID + 1, cdoGetStreamName(1),
-                 cdoGetStreamName(2));
+        cdoAbort("Verification dates at time step %d of %s and %s differ!", tsID + 1, cdoGetStreamName(1), cdoGetStreamName(2));
 
       if (Options::cdoVerbose) cdoPrint("process timestep: %d %d %d", tsID + 1, vdate, vtime);
 
       cdiDecodeDate(vdate, &year, &month, &day);
 
-      if (month >= 1 && month <= 12)
-        dayoy = (month - 1) * 31 + day;
-      else
-        dayoy = 0;
-
+      const int dayoy = (month >= 1 && month <= 12) ? (month - 1) * 31 + day : 0;
       if (dayoy < 0 || dayoy >= NDAY) cdoAbort("Day %d out of range!", dayoy);
 
       vdates2[dayoy] = vdate;
@@ -147,7 +135,7 @@ Ydrunpctl(void *process)
 
       if (!vars2[dayoy].size())
         {
-          fieldsFromVlist(vlistID2, vars2[dayoy], FIELD_PTR);
+          fieldsFromVlist(vlistID2, vars2[dayoy], FIELD_VEC);
           hsets[dayoy] = hsetCreate(nvars);
 
           for (varID = 0; varID < nvars; varID++)
@@ -161,15 +149,15 @@ Ydrunpctl(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID2, &varID, &levelID);
-          pstreamReadRecord(streamID2, vars2[dayoy][varID][levelID].ptr, &nmiss);
+          cdoInqRecord(streamID2, &varID, &levelID);
+          cdoReadRecord(streamID2, vars2[dayoy][varID][levelID].vec.data(), &nmiss);
           vars2[dayoy][varID][levelID].nmiss = nmiss;
         }
+
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID3, &varID, &levelID);
-          pstreamReadRecord(streamID3, field.ptr, &nmiss);
-          field.nmiss = nmiss;
+          cdoInqRecord(streamID3, &varID, &levelID);
+          cdoReadRecord(streamID3, field.vec.data(), &field.nmiss);
           field.grid = vars2[dayoy][varID][levelID].grid;
           field.missval = vars2[dayoy][varID][levelID].missval;
 
@@ -189,16 +177,16 @@ Ydrunpctl(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           if (tsID == 0)
             {
-              recinfo[recID].varID = varID;
-              recinfo[recID].levelID = levelID;
-              recinfo[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+              recList[recID].varID = varID;
+              recList[recID].levelID = levelID;
+              recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
             }
 
-          pstreamReadRecord(streamID1, vars1[tsID][varID][levelID].ptr, &nmiss);
+          cdoReadRecord(streamID1, vars1[tsID][varID][levelID].vec.data(), &nmiss);
           vars1[tsID][varID][levelID].nmiss = nmiss;
         }
     }
@@ -207,19 +195,18 @@ Ydrunpctl(void *process)
     {
       datetime_avg(dpy, ndates, datetime.data());
 
-      vdate = datetime[ndates].date;
-      vtime = datetime[ndates].time;
+      const int64_t vdate = datetime[ndates].date;
+      const int vtime = datetime[ndates].time;
 
       cdiDecodeDate(vdate, &year, &month, &day);
 
-      dayoy = (month >= 1 && month <= 12) ? (month - 1) * 31 + day : 0;
+      const int dayoy = (month >= 1 && month <= 12) ? (month - 1) * 31 + day : 0;
       if (dayoy < 0 || dayoy >= NDAY) cdoAbort("Day %d out of range!", dayoy);
 
       vdates1[dayoy] = vdate;
       vtimes1[dayoy] = vtime;
 
-      if (!vars2[dayoy].size())
-        cdoAbort("No data for day %d in %s and %s", dayoy, cdoGetStreamName(1), cdoGetStreamName(2));
+      if (!vars2[dayoy].size()) cdoAbort("No data for day %d in %s and %s", dayoy, cdoGetStreamName(1), cdoGetStreamName(2));
 
       for (varID = 0; varID < nvars; varID++)
         {
@@ -247,9 +234,9 @@ Ydrunpctl(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
-          pstreamReadRecord(streamID1, vars1[ndates - 1][varID][levelID].ptr, &nmiss);
+          cdoReadRecord(streamID1, vars1[ndates - 1][varID][levelID].vec.data(), &nmiss);
           vars1[ndates - 1][varID][levelID].nmiss = nmiss;
         }
 
@@ -275,7 +262,7 @@ Ydrunpctl(void *process)
       }
   */
   int otsID = 0;
-  for (dayoy = 0; dayoy < NDAY; dayoy++)
+  for (int dayoy = 0; dayoy < NDAY; dayoy++)
     if (nsets[dayoy])
       {
         if (getmonthday(vdates1[dayoy]) != getmonthday(vdates2[dayoy]))
@@ -293,40 +280,29 @@ Ydrunpctl(void *process)
 
         taxisDefVdate(taxisID4, vdates1[dayoy]);
         taxisDefVtime(taxisID4, vtimes1[dayoy]);
-        pstreamDefTimestep(streamID4, otsID);
+        cdoDefTimestep(streamID4, otsID);
 
         for (int recID = 0; recID < maxrecs; recID++)
           {
-            if (otsID && recinfo[recID].lconst) continue;
+            if (otsID && recList[recID].lconst) continue;
 
-            int varID = recinfo[recID].varID;
-            int levelID = recinfo[recID].levelID;
-            pstreamDefRecord(streamID4, varID, levelID);
-            pstreamWriteRecord(streamID4, vars2[dayoy][varID][levelID].ptr, vars2[dayoy][varID][levelID].nmiss);
+            const int varID = recList[recID].varID;
+            const int levelID = recList[recID].levelID;
+            cdoDefRecord(streamID4, varID, levelID);
+            cdoWriteRecord(streamID4, vars2[dayoy][varID][levelID].vec.data(), vars2[dayoy][varID][levelID].nmiss);
           }
 
         otsID++;
       }
 
-  for (its = 0; its < ndates; its++)
-    {
-      for (varID = 0; varID < nvars; varID++)
-        {
-          nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-          for (levelID = 0; levelID < nlevels; levelID++) Free(vars1[its][varID][levelID].ptr);
-        }
-    }
-
-  for (dayoy = 0; dayoy < NDAY; dayoy++)
+  for (int dayoy = 0; dayoy < NDAY; dayoy++)
     {
       if (vars2[dayoy].size())
         {
-          fieldsFree(vlistID2, vars2[dayoy]);
           hsetDestroy(hsets[dayoy]);
         }
     }
 
-  if (field.ptr) Free(field.ptr);
 
   cdoStreamClose(streamID4);
   cdoStreamClose(streamID3);
