@@ -34,7 +34,6 @@
 #include "cdo_int.h"
 #include "param_conversion.h"
 #include "calendar.h"
-#include "pstream_int.h"
 #include "datetime.h"
 
 #define NDAY 373
@@ -49,7 +48,6 @@ struct YDAY_STATS
   int vlist;
 };
 
-
 static YDAY_STATS *
 ydstatCreate(int vlistID)
 {
@@ -70,37 +68,12 @@ ydstatCreate(int vlistID)
 static void
 ydstatDestroy(YDAY_STATS *stats)
 {
-  int varID, levelID, nlevels;
-
-  if (stats != nullptr)
-    {
-      int nvars = vlistNvars(stats->vlist);
-
-      for (int dayoy = 0; dayoy < NDAY; dayoy++)
-        {
-          if (stats->vars1[dayoy].size())
-            {
-              for (varID = 0; varID < nvars; varID++)
-                {
-                  nlevels = zaxisInqSize(vlistInqVarZaxis(stats->vlist, varID));
-                  for (levelID = 0; levelID < nlevels; levelID++) Free(stats->vars1[dayoy][varID][levelID].ptr);
-                }
-            }
-          if (stats->vars2[dayoy].size())
-            {
-              for (varID = 0; varID < nvars; varID++)
-                {
-                  nlevels = zaxisInqSize(vlistInqVarZaxis(stats->vlist, varID));
-                  for (levelID = 0; levelID < nlevels; levelID++) Free(stats->vars2[dayoy][varID][levelID].ptr);
-                }
-            }
-        }
-      Free(stats);
-    }
+  if (stats != nullptr) Free(stats);
 }
 
 static void
-ydstatUpdate(YDAY_STATS *stats, int64_t vdate, int vtime, const FieldVector2D &vars1, const FieldVector2D &vars2, int nsets, int operfunc)
+ydstatUpdate(YDAY_STATS *stats, int64_t vdate, int vtime, const FieldVector2D &vars1, const FieldVector2D &vars2, int nsets,
+             int operfunc)
 {
   bool lvarstd = vars2.size();
 
@@ -109,7 +82,7 @@ ydstatUpdate(YDAY_STATS *stats, int64_t vdate, int vtime, const FieldVector2D &v
   int year, month, day;
   cdiDecodeDate(vdate, &year, &month, &day);
 
-  int dayoy = (month >= 1 && month <= 12) ? (month - 1) * 31 + day : 0;
+  const int dayoy = (month >= 1 && month <= 12) ? (month - 1) * 31 + day : 0;
   if (dayoy < 0 || dayoy >= NDAY) cdoAbort("day %d out of range!", dayoy);
 
   stats->vdate[dayoy] = vdate;
@@ -117,27 +90,25 @@ ydstatUpdate(YDAY_STATS *stats, int64_t vdate, int vtime, const FieldVector2D &v
 
   if (!stats->vars1[dayoy].size())
     {
-      fieldsFromVlist(stats->vlist, stats->vars1[dayoy], FIELD_PTR);
-      if (lvarstd) fieldsFromVlist(stats->vlist, stats->vars2[dayoy], FIELD_PTR);
+      fieldsFromVlist(stats->vlist, stats->vars1[dayoy], FIELD_VEC);
+      if (lvarstd) fieldsFromVlist(stats->vlist, stats->vars2[dayoy], FIELD_VEC);
     }
 
   for (int varID = 0; varID < nvars; varID++)
     {
       if (vlistInqVarTimetype(stats->vlist, varID) == TIME_CONSTANT) continue;
 
-      size_t gridsize = gridInqSize(vlistInqVarGrid(stats->vlist, varID));
-      int nlevels = zaxisInqSize(vlistInqVarZaxis(stats->vlist, varID));
-
+      const int nlevels = zaxisInqSize(vlistInqVarZaxis(stats->vlist, varID));
       for (int levelID = 0; levelID < nlevels; levelID++)
         {
           if (stats->nsets[dayoy] == 0)
             {
-              arrayCopy(gridsize, vars1[varID][levelID].ptr, stats->vars1[dayoy][varID][levelID].ptr);
+              stats->vars1[dayoy][varID][levelID].vec = vars1[varID][levelID].vec;
               stats->vars1[dayoy][varID][levelID].nmiss = vars1[varID][levelID].nmiss;
 
               if (lvarstd)
                 {
-                  arrayCopy(gridsize, vars2[varID][levelID].ptr, stats->vars2[dayoy][varID][levelID].ptr);
+                  stats->vars2[dayoy][varID][levelID].vec = vars2[varID][levelID].vec;
                   stats->vars2[dayoy][varID][levelID].nmiss = vars2[varID][levelID].nmiss;
                 }
             }
@@ -145,12 +116,12 @@ ydstatUpdate(YDAY_STATS *stats, int64_t vdate, int vtime, const FieldVector2D &v
             {
               if (lvarstd)
                 {
-                  farsum(stats->vars1[dayoy][varID][levelID], vars1[varID][levelID]);
-                  farsum(stats->vars2[dayoy][varID][levelID], vars2[varID][levelID]);
+                  vfarsum(stats->vars1[dayoy][varID][levelID], vars1[varID][levelID]);
+                  vfarsum(stats->vars2[dayoy][varID][levelID], vars2[varID][levelID]);
                 }
               else
                 {
-                  farfun(stats->vars1[dayoy][varID][levelID], vars1[varID][levelID], operfunc);
+                  vfarfun(stats->vars1[dayoy][varID][levelID], vars1[varID][levelID], operfunc);
                 }
             }
         }
@@ -163,9 +134,9 @@ static void
 ydstatFinalize(YDAY_STATS *stats, int operfunc)
 {
   int varID, levelID, nlevels;
-  int divisor = operfunc == func_std1 || operfunc == func_var1;
+  const int divisor = operfunc == func_std1 || operfunc == func_var1;
 
-  int nvars = vlistNvars(stats->vlist);
+  const int nvars = vlistNvars(stats->vlist);
 
   for (int dayoy = 0; dayoy < NDAY; dayoy++)
     if (stats->nsets[dayoy])
@@ -179,7 +150,7 @@ ydstatFinalize(YDAY_STATS *stats, int operfunc)
                 if (vlistInqVarTimetype(stats->vlist, varID) == TIME_CONSTANT) continue;
                 nlevels = zaxisInqSize(vlistInqVarZaxis(stats->vlist, varID));
                 for (levelID = 0; levelID < nlevels; levelID++)
-                  farcdiv(stats->vars1[dayoy][varID][levelID], (double) stats->nsets[dayoy]);
+                  vfarcdiv(stats->vars1[dayoy][varID][levelID], (double) stats->nsets[dayoy]);
               }
             break;
 
@@ -190,7 +161,7 @@ ydstatFinalize(YDAY_STATS *stats, int operfunc)
                 if (vlistInqVarTimetype(stats->vlist, varID) == TIME_CONSTANT) continue;
                 nlevels = zaxisInqSize(vlistInqVarZaxis(stats->vlist, varID));
                 for (levelID = 0; levelID < nlevels; levelID++)
-                  farcstd(stats->vars1[dayoy][varID][levelID], stats->vars2[dayoy][varID][levelID], stats->nsets[dayoy], divisor);
+                  vfarcstd(stats->vars1[dayoy][varID][levelID], stats->vars2[dayoy][varID][levelID], stats->nsets[dayoy], divisor);
               }
             break;
 
@@ -201,7 +172,7 @@ ydstatFinalize(YDAY_STATS *stats, int operfunc)
                 if (vlistInqVarTimetype(stats->vlist, varID) == TIME_CONSTANT) continue;
                 nlevels = zaxisInqSize(vlistInqVarZaxis(stats->vlist, varID));
                 for (levelID = 0; levelID < nlevels; levelID++)
-                  farcvar(stats->vars1[dayoy][varID][levelID], stats->vars2[dayoy][varID][levelID], stats->nsets[dayoy], divisor);
+                  vfarcvar(stats->vars1[dayoy][varID][levelID], stats->vars2[dayoy][varID][levelID], stats->nsets[dayoy], divisor);
               }
             break;
           }
@@ -216,7 +187,6 @@ Ydrunstat(void *process)
   int levelID;
   int tsID;
   int inp, its;
-  size_t nmiss;
 
   cdoInitialize(process);
 
@@ -232,32 +202,32 @@ Ydrunstat(void *process)
   cdoOperatorAdd("ydrunstd1",  func_std1,  0, nullptr);
   // clang-format on
 
-  int operatorID = cdoOperatorID();
-  int operfunc = cdoOperatorF1(operatorID);
+  const int operatorID = cdoOperatorID();
+  const int operfunc = cdoOperatorF1(operatorID);
 
   operatorInputArg("number of timesteps");
-  int ndates = parameter2int(operatorArgv()[0]);
+  const int ndates = parameter2int(operatorArgv()[0]);
 
-  bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
+  const bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   if (taxisHasBounds(taxisID2)) taxisDeleteBounds(taxisID2);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int calendar = taxisInqCalendar(taxisID1);
-  int dpy = calendar_dpy(calendar);
+  const int calendar = taxisInqCalendar(taxisID1);
+  const int dpy = calendar_dpy(calendar);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
-  int maxrecs = vlistNrecs(vlistID1);
-  std::vector<RecordInfo> recinfo(maxrecs);
+  const int maxrecs = vlistNrecs(vlistID1);
+  std::vector<RecordInfo> recList(maxrecs);
 
   std::vector<CdoDateTime> datetime(ndates + 1);
 
@@ -266,8 +236,8 @@ Ydrunstat(void *process)
 
   for (its = 0; its < ndates; its++)
     {
-      fieldsFromVlist(vlistID1, vars1[its], FIELD_PTR);
-      if (lvarstd) fieldsFromVlist(vlistID1, vars2[its], FIELD_PTR);
+      fieldsFromVlist(vlistID1, vars1[its], FIELD_VEC);
+      if (lvarstd) fieldsFromVlist(vlistID1, vars2[its], FIELD_VEC);
     }
 
   for (tsID = 0; tsID < ndates; tsID++)
@@ -280,34 +250,33 @@ Ydrunstat(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           if (tsID == 0)
             {
-              recinfo[recID].varID = varID;
-              recinfo[recID].levelID = levelID;
-              recinfo[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+              recList[recID].varID = varID;
+              recList[recID].levelID = levelID;
+              recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
             }
 
           Field &rvars1 = vars1[tsID][varID][levelID];
 
-          pstreamReadRecord(streamID1, rvars1.ptr, &nmiss);
-          rvars1.nmiss = nmiss;
+          cdoReadRecord(streamID1, rvars1.vec.data(), &rvars1.nmiss);
 
           if (lvarstd)
             {
-              farmoq(vars2[tsID][varID][levelID], vars1[tsID][varID][levelID]);
+              vfarmoq(vars2[tsID][varID][levelID], vars1[tsID][varID][levelID]);
               for (int inp = 0; inp < tsID; inp++)
                 {
-                  farsumq(vars2[inp][varID][levelID], rvars1);
-                  farsum(vars1[inp][varID][levelID], rvars1);
+                  vfarsumq(vars2[inp][varID][levelID], rvars1);
+                  vfarsum(vars1[inp][varID][levelID], rvars1);
                 }
             }
           else
             {
               for (int inp = 0; inp < tsID; inp++)
                 {
-                  farfun(vars1[inp][varID][levelID], rvars1, operfunc);
+                  vfarfun(vars1[inp][varID][levelID], rvars1, operfunc);
                 }
             }
         }
@@ -317,8 +286,8 @@ Ydrunstat(void *process)
     {
       datetime_avg(dpy, ndates, datetime.data());
 
-      int64_t vdate = datetime[ndates].date;
-      int vtime = datetime[ndates].time;
+      const int64_t vdate = datetime[ndates].date;
+      const int vtime = datetime[ndates].time;
 
       ydstatUpdate(stats, vdate, vtime, vars1[0], vars2[0], ndates, operfunc);
 
@@ -341,27 +310,26 @@ Ydrunstat(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           Field &rvars1 = vars1[ndates - 1][varID][levelID];
 
-          pstreamReadRecord(streamID1, rvars1.ptr, &nmiss);
-          rvars1.nmiss = nmiss;
+          cdoReadRecord(streamID1, rvars1.vec.data(), &rvars1.nmiss);
 
           if (lvarstd)
             {
               for (inp = 0; inp < ndates - 1; inp++)
                 {
-                  farsumq(vars2[inp][varID][levelID], rvars1);
-                  farsum(vars1[inp][varID][levelID], rvars1);
+                  vfarsumq(vars2[inp][varID][levelID], rvars1);
+                  vfarsum(vars1[inp][varID][levelID], rvars1);
                 }
-              farmoq(vars2[ndates - 1][varID][levelID], rvars1);
+              vfarmoq(vars2[ndates - 1][varID][levelID], rvars1);
             }
           else
             {
               for (inp = 0; inp < ndates - 1; inp++)
                 {
-                  farfun(vars1[inp][varID][levelID], rvars1, operfunc);
+                  vfarfun(vars1[inp][varID][levelID], rvars1, operfunc);
                 }
             }
         }
@@ -397,29 +365,23 @@ Ydrunstat(void *process)
       {
         taxisDefVdate(taxisID2, stats->vdate[dayoy]);
         taxisDefVtime(taxisID2, stats->vtime[dayoy]);
-        pstreamDefTimestep(streamID2, otsID);
+        cdoDefTimestep(streamID2, otsID);
 
         for (int recID = 0; recID < maxrecs; recID++)
           {
-            if (otsID && recinfo[recID].lconst) continue;
+            if (otsID && recList[recID].lconst) continue;
 
-            int varID = recinfo[recID].varID;
-            int levelID = recinfo[recID].levelID;
+            int varID = recList[recID].varID;
+            int levelID = recList[recID].levelID;
             Field &rvars1 = stats->vars1[dayoy][varID][levelID];
 
-            pstreamDefRecord(streamID2, varID, levelID);
-            pstreamWriteRecord(streamID2, rvars1.ptr, rvars1.nmiss);
+            cdoDefRecord(streamID2, varID, levelID);
+            cdoWriteRecord(streamID2, rvars1.vec.data(), rvars1.nmiss);
           }
 
         otsID++;
       }
 
-  for (its = 0; its < ndates; its++)
-    {
-      fieldsFree(vlistID1, vars1[its]);
-      if (lvarstd) fieldsFree(vlistID1, vars2[its]);
-    }
-
   ydstatDestroy(stats);
 
   cdoStreamClose(streamID2);
