@@ -26,7 +26,6 @@
 
 #include "cdo_int.h"
 #include "calendar.h"
-#include "pstream_int.h"
 #include "datetime.h"
 #include "printinfo.h"
 
@@ -34,17 +33,12 @@ void *
 Yearmonstat(void *process)
 {
   TimeStat timestat_date = TimeStat::MEAN;
-  int64_t vdate = 0, vdate0 = 0;
-  int vtime = 0, vtime0 = 0;
+  int64_t vdate0 = 0;
+  int vtime0 = 0;
   int nrecs;
   int varID, levelID;
-  int dpm;
   int year0 = 0, month0 = 0;
   int year, month, day;
-  size_t nmiss;
-  int nlevel;
-  long nsets;
-  double dsets;
   char vdatestr[32], vtimestr[32];
 
   cdoInitialize(process);
@@ -54,53 +48,52 @@ Yearmonstat(void *process)
   cdoOperatorAdd("yearmonavg",   func_avg,  0, nullptr);
   // clang-format on
 
-  int operatorID = cdoOperatorID();
-  int operfunc = cdoOperatorF1(operatorID);
+  const int operatorID = cdoOperatorID();
+  const int operfunc = cdoOperatorF1(operatorID);
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   taxisWithBounds(taxisID2);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
-  int nvars = vlistNvars(vlistID1);
+  const int nvars = vlistNvars(vlistID1);
 
-  int maxrecs = vlistNrecs(vlistID1);
-  std::vector<RecordInfo> recinfo(maxrecs);
+  const int maxrecs = vlistNrecs(vlistID1);
+  std::vector<RecordInfo> recList(maxrecs);
 
-  int calendar = taxisInqCalendar(taxisID1);
+  const int calendar = taxisInqCalendar(taxisID1);
   DateTimeList dtlist;
   dtlist.setStat(timestat_date);
   dtlist.setCalendar(calendar);
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
+  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field;
-  fieldInit(field);
-  field.ptr = (double *) Malloc(gridsize * sizeof(double));
+  field.resize(gridsizemax);
 
   FieldVector2D samp1, vars1;
   fieldsFromVlist(vlistID1, samp1, FIELD_NONE);
-  fieldsFromVlist(vlistID1, vars1, FIELD_PTR);
+  fieldsFromVlist(vlistID1, vars1, FIELD_VEC);
 
   int tsID = 0;
   int otsID = 0;
   while (true)
     {
-      nsets = 0;
-      dsets = 0;
+      long nsets = 0;
+      double dsets = 0;
       while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
         {
           dtlist.taxisInqTimestep(taxisID1, nsets);
-          vdate = dtlist.getVdate(nsets);
-          vtime = dtlist.getVtime(nsets);
+          const int64_t vdate = dtlist.getVdate(nsets);
+          const int vtime = dtlist.getVtime(nsets);
           cdiDecodeDate(vdate, &year, &month, &day);
 
           if (nsets == 0) year0 = year;
@@ -118,62 +111,59 @@ Yearmonstat(void *process)
               cdoAbort("Month does not change!");
             }
 
-          dpm = days_per_month(calendar, year, month);
+          const int dpm = days_per_month(calendar, year, month);
 
           for (int recID = 0; recID < nrecs; recID++)
             {
-              pstreamInqRecord(streamID1, &varID, &levelID);
+              cdoInqRecord(streamID1, &varID, &levelID);
 
               if (tsID == 0)
                 {
-                  recinfo[recID].varID = varID;
-                  recinfo[recID].levelID = levelID;
-                  recinfo[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+                  recList[recID].varID = varID;
+                  recList[recID].levelID = levelID;
+                  recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
                 }
 
-              gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
+              Field &rsamp1 = samp1[varID][levelID];
+              Field &rvars1 = vars1[varID][levelID];
+
+              const size_t fieldsize = rvars1.size;
 
               if (nsets == 0)
                 {
-                  pstreamReadRecord(streamID1, vars1[varID][levelID].ptr, &nmiss);
-                  vars1[varID][levelID].nmiss = nmiss;
+                  cdoReadRecord(streamID1, rvars1.vec.data(), &rvars1.nmiss);
 
-                  farcmul(vars1[varID][levelID], dpm);
+                  vfarcmul(rvars1, dpm);
 
-                  if (nmiss > 0 || samp1[varID][levelID].ptr)
+                  if (rvars1.nmiss || !rsamp1.empty())
                     {
-                      if (samp1[varID][levelID].ptr == nullptr)
-                        samp1[varID][levelID].ptr = (double *) Malloc(gridsize * sizeof(double));
+                      if (rsamp1.empty()) rsamp1.resize(fieldsize);
 
-                      for (size_t i = 0; i < gridsize; i++)
-                        if (DBL_IS_EQUAL(vars1[varID][levelID].ptr[i], vars1[varID][levelID].missval))
-                          samp1[varID][levelID].ptr[i] = 0;
+                      for (size_t i = 0; i < fieldsize; i++)
+                        if (DBL_IS_EQUAL(rvars1.vec[i], rvars1.missval))
+                          rsamp1.vec[i] = 0;
                         else
-                          samp1[varID][levelID].ptr[i] = dpm;
+                          rsamp1.vec[i] = dpm;
                     }
                 }
               else
                 {
-                  pstreamReadRecord(streamID1, field.ptr, &nmiss);
-                  field.nmiss = nmiss;
-                  field.grid = vars1[varID][levelID].grid;
-                  field.missval = vars1[varID][levelID].missval;
+                  cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
+                  field.size = fieldsize;
+                  field.grid = rvars1.grid;
+                  field.missval = rvars1.missval;
 
-                  farcmul(field, dpm);
+                  vfarcmul(field, dpm);
 
-                  if (field.nmiss > 0 || samp1[varID][levelID].ptr)
+                  if (field.nmiss || !rsamp1.empty())
                     {
-                      if (samp1[varID][levelID].ptr == nullptr)
-                        {
-                          samp1[varID][levelID].ptr = (double *) Malloc(gridsize * sizeof(double));
-                          for (size_t i = 0; i < gridsize; i++) samp1[varID][levelID].ptr[i] = dsets;
-                        }
+                      if (rsamp1.empty()) rsamp1.resize(fieldsize, dsets);
 
-                      for (size_t i = 0; i < gridsize; i++)
-                        if (!DBL_IS_EQUAL(field.ptr[i], vars1[varID][levelID].missval)) samp1[varID][levelID].ptr[i] += dpm;
+                      for (size_t i = 0; i < fieldsize; i++)
+                        if (!DBL_IS_EQUAL(field.vec[i], rvars1.missval)) rsamp1.vec[i] += dpm;
                     }
 
-                  farfun(vars1[varID][levelID], field, operfunc);
+                  vfarfun(rvars1, field, operfunc);
                 }
             }
 
@@ -190,13 +180,13 @@ Yearmonstat(void *process)
       for (varID = 0; varID < nvars; varID++)
         {
           if (vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT) continue;
-          nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-          for (levelID = 0; levelID < nlevel; levelID++)
+          int nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
+          for (levelID = 0; levelID < nlevels; levelID++)
             {
-              if (samp1[varID][levelID].ptr == nullptr)
-                farcdiv(vars1[varID][levelID], dsets);
+              if (samp1[varID][levelID].empty())
+                vfarcdiv(vars1[varID][levelID], dsets);
               else
-                fardiv(vars1[varID][levelID], samp1[varID][levelID]);
+                vfardiv(vars1[varID][levelID], samp1[varID][levelID]);
             }
         }
 
@@ -208,24 +198,23 @@ Yearmonstat(void *process)
         }
 
       dtlist.statTaxisDefTimestep(taxisID2, nsets);
-      pstreamDefTimestep(streamID2, otsID);
+      cdoDefTimestep(streamID2, otsID);
 
       for (int recID = 0; recID < maxrecs; recID++)
         {
-          if (otsID && recinfo[recID].lconst) continue;
+          if (otsID && recList[recID].lconst) continue;
 
-          int varID = recinfo[recID].varID;
-          int levelID = recinfo[recID].levelID;
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, vars1[varID][levelID].ptr, vars1[varID][levelID].nmiss);
+          const int varID = recList[recID].varID;
+          const int levelID = recList[recID].levelID;
+          Field &rvars1 = vars1[varID][levelID];
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, rvars1.vec.data(), rvars1.nmiss);
         }
 
       if (nrecs == 0) break;
       otsID++;
     }
 
-  if (field.ptr) Free(field.ptr);
-
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
