@@ -25,7 +25,6 @@
 
 #include "cdo_int.h"
 #include "param_conversion.h"
-#include "pstream_int.h"
 #include "percentiles_hist.h"
 #include "percentiles.h"
 
@@ -44,8 +43,6 @@ Ymonpctl(void *process)
 {
   int varID;
   int gridID;
-  int64_t vdate;
-  int vtime;
   int year, month, day;
   int levelID;
   size_t nmiss;
@@ -60,7 +57,7 @@ Ymonpctl(void *process)
   cdoOperatorAdd("ymonpctl", func_pctl, 0, nullptr);
 
   operatorInputArg("percentile number");
-  double pn = parameter2double(operatorArgv()[0]);
+  const double pn = parameter2double(operatorArgv()[0]);
   percentile_check_number(pn);
 
   for (month = 0; month < NMONTH; month++)
@@ -69,54 +66,51 @@ Ymonpctl(void *process)
       nsets[month] = 0;
     }
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
-  int streamID2 = cdoStreamOpenRead(cdoStreamName(1));
-  int streamID3 = cdoStreamOpenRead(cdoStreamName(2));
+  CdoStreamID streamID1 = cdoOpenRead(0);
+  CdoStreamID streamID2 = cdoOpenRead(1);
+  CdoStreamID streamID3 = cdoOpenRead(2);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = cdoStreamInqVlist(streamID2);
-  int vlistID3 = cdoStreamInqVlist(streamID3);
-  int vlistID4 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = cdoStreamInqVlist(streamID2);
+  const int vlistID3 = cdoStreamInqVlist(streamID3);
+  const int vlistID4 = vlistDuplicate(vlistID1);
 
   vlistCompare(vlistID1, vlistID2, CMP_ALL);
   vlistCompare(vlistID1, vlistID3, CMP_ALL);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = vlistInqTaxis(vlistID2);
-  int taxisID3 = vlistInqTaxis(vlistID3);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = vlistInqTaxis(vlistID2);
+  const int taxisID3 = vlistInqTaxis(vlistID3);
   /* TODO - check that time axes 2 and 3 are equal */
 
-  int taxisID4 = taxisDuplicate(taxisID1);
+  const int taxisID4 = taxisDuplicate(taxisID1);
   if (taxisHasBounds(taxisID4)) taxisDeleteBounds(taxisID4);
   vlistDefTaxis(vlistID4, taxisID4);
 
-  int streamID4 = cdoStreamOpenWrite(cdoStreamName(3));
-  pstreamDefVlist(streamID4, vlistID4);
+  CdoStreamID streamID4 = cdoOpenWrite(3);
+  cdoDefVlist(streamID4, vlistID4);
 
-  int nvars = vlistNvars(vlistID1);
+  const int nvars = vlistNvars(vlistID1);
 
-  int maxrecs = vlistNrecs(vlistID1);
-  std::vector<RecordInfo> recinfo(maxrecs);
+  const int maxrecs = vlistNrecs(vlistID1);
+  std::vector<RecordInfo> recList(maxrecs);
 
-  size_t gridsize = vlistGridsizeMax(vlistID1);
+  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
 
   Field field;
-  fieldInit(field);
-  field.ptr = (double *) Malloc(gridsize * sizeof(double));
+  field.resize(gridsizemax);
 
   int tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID2, tsID)))
     {
       if (nrecs != cdoStreamInqTimestep(streamID3, tsID))
-        cdoAbort("Number of records at time step %d of %s and %s differ!", tsID + 1, cdoGetStreamName(1),
-                 cdoGetStreamName(2));
+        cdoAbort("Number of records at time step %d of %s and %s differ!", tsID + 1, cdoGetStreamName(1), cdoGetStreamName(2));
 
-      vdate = taxisInqVdate(taxisID2);
-      vtime = taxisInqVtime(taxisID2);
+      const int64_t vdate = taxisInqVdate(taxisID2);
+      const int vtime = taxisInqVtime(taxisID2);
 
       if (vdate != taxisInqVdate(taxisID3))
-        cdoAbort("Verification dates at time step %d of %s and %s differ!", tsID + 1, cdoGetStreamName(1),
-                 cdoGetStreamName(2));
+        cdoAbort("Verification dates at time step %d of %s and %s differ!", tsID + 1, cdoGetStreamName(1), cdoGetStreamName(2));
 
       if (Options::cdoVerbose) cdoPrint("process timestep: %d %d %d", tsID + 1, vdate, vtime);
 
@@ -127,7 +121,7 @@ Ymonpctl(void *process)
 
       if (!vars1[month].size())
         {
-          fieldsFromVlist(vlistID1, vars1[month], FIELD_PTR);
+          fieldsFromVlist(vlistID1, vars1[month], FIELD_VEC);
           hsets[month] = hsetCreate(nvars);
 
           for (varID = 0; varID < nvars; varID++)
@@ -141,15 +135,15 @@ Ymonpctl(void *process)
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID2, &varID, &levelID);
-          pstreamReadRecord(streamID2, vars1[month][varID][levelID].ptr, &nmiss);
+          cdoInqRecord(streamID2, &varID, &levelID);
+          cdoReadRecord(streamID2, vars1[month][varID][levelID].vec.data(), &nmiss);
           vars1[month][varID][levelID].nmiss = nmiss;
         }
+
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID3, &varID, &levelID);
-          pstreamReadRecord(streamID3, field.ptr, &nmiss);
-          field.nmiss = nmiss;
+          cdoInqRecord(streamID3, &varID, &levelID);
+          cdoReadRecord(streamID3, field.vec.data(), &field.nmiss);
           field.grid = vars1[month][varID][levelID].grid;
           field.missval = vars1[month][varID][levelID].missval;
 
@@ -162,8 +156,8 @@ Ymonpctl(void *process)
   tsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
-      vdate = taxisInqVdate(taxisID1);
-      vtime = taxisInqVtime(taxisID1);
+      const int64_t vdate = taxisInqVdate(taxisID1);
+      const int vtime = taxisInqVtime(taxisID1);
 
       if (Options::cdoVerbose) cdoPrint("process timestep: %d %d %d", tsID + 1, vdate, vtime);
 
@@ -173,21 +167,20 @@ Ymonpctl(void *process)
       vdates1[month] = vdate;
       vtimes1[month] = vtime;
 
-      if (!vars1[month].size())
-        cdoAbort("No data for month %d in %s and %s", month, cdoGetStreamName(1), cdoGetStreamName(2));
+      if (!vars1[month].size()) cdoAbort("No data for month %d in %s and %s", month, cdoGetStreamName(1), cdoGetStreamName(2));
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           if (tsID == 0)
             {
-              recinfo[recID].varID = varID;
-              recinfo[recID].levelID = levelID;
-              recinfo[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+              recList[recID].varID = varID;
+              recList[recID].levelID = levelID;
+              recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
             }
 
-          pstreamReadRecord(streamID1, vars1[month][varID][levelID].ptr, &nmiss);
+          cdoReadRecord(streamID1, vars1[month][varID][levelID].vec.data(), &nmiss);
           vars1[month][varID][levelID].nmiss = nmiss;
 
           hsetAddVarLevelValues(hsets[month], varID, levelID, &vars1[month][varID][levelID]);
@@ -216,16 +209,16 @@ Ymonpctl(void *process)
 
         taxisDefVdate(taxisID4, vdates1[month]);
         taxisDefVtime(taxisID4, vtimes1[month]);
-        pstreamDefTimestep(streamID4, otsID);
+        cdoDefTimestep(streamID4, otsID);
 
         for (int recID = 0; recID < maxrecs; recID++)
           {
-            if (otsID && recinfo[recID].lconst) continue;
+            if (otsID && recList[recID].lconst) continue;
 
-            int varID = recinfo[recID].varID;
-            int levelID = recinfo[recID].levelID;
-            pstreamDefRecord(streamID4, varID, levelID);
-            pstreamWriteRecord(streamID4, vars1[month][varID][levelID].ptr, vars1[month][varID][levelID].nmiss);
+            const int varID = recList[recID].varID;
+            const int levelID = recList[recID].levelID;
+            cdoDefRecord(streamID4, varID, levelID);
+            cdoWriteRecord(streamID4, vars1[month][varID][levelID].vec.data(), vars1[month][varID][levelID].nmiss);
           }
 
         otsID++;
@@ -235,13 +228,10 @@ Ymonpctl(void *process)
     {
       if (vars1[month].size())
         {
-          fieldsFree(vlistID1, vars1[month]);
           hsetDestroy(hsets[month]);
         }
     }
 
-  if (field.ptr) Free(field.ptr);
-
   cdoStreamClose(streamID4);
   cdoStreamClose(streamID3);
   cdoStreamClose(streamID2);
