@@ -33,7 +33,6 @@
 #include <cdi.h>
 
 #include "cdo_int.h"
-#include "pstream_int.h"
 
 #define NMONTH 17
 
@@ -55,13 +54,11 @@ Ymonstat(void *process)
   int nrecs;
   int levelID;
   int month_nsets[NMONTH];
-  size_t nmiss;
   int64_t vdates[NMONTH];
   int vtimes[NMONTH];
   int mon[NMONTH];
   int nmon = 0;
   FieldVector2D vars1[NMONTH], vars2[NMONTH], samp1[NMONTH];
-  Field field;
 
   cdoInitialize(process);
 
@@ -77,45 +74,46 @@ Ymonstat(void *process)
   cdoOperatorAdd("ymonstd",   func_std,   0, nullptr);
   cdoOperatorAdd("ymonstd1",  func_std1,  0, nullptr);
 
-  int operatorID = cdoOperatorID();
-  int operfunc = cdoOperatorF1(operatorID);
+  const int operatorID = cdoOperatorID();
+  const int operfunc = cdoOperatorF1(operatorID);
 
-  bool lrange  = operfunc == func_range;
-  bool lmean   = operfunc == func_mean || operfunc == func_avg;
-  bool lstd    = operfunc == func_std || operfunc == func_std1;
-  bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
-  int  divisor = operfunc == func_std1 || operfunc == func_var1;
+  const bool lrange  = operfunc == func_range;
+  const bool lmean   = operfunc == func_mean || operfunc == func_avg;
+  const bool lstd    = operfunc == func_std || operfunc == func_std1;
+  const bool lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
+  const int  divisor = operfunc == func_std1 || operfunc == func_var1;
   // clang-format on
-  bool lvars2 = lvarstd || lrange;
+  const bool lvars2 = lvarstd || lrange;
 
   for (month = 0; month < NMONTH; month++) month_nsets[month] = 0;
 
-  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));
+  CdoStreamID streamID1 = cdoOpenRead(0);
 
-  int vlistID1 = cdoStreamInqVlist(streamID1);
-  int vlistID2 = vlistDuplicate(vlistID1);
+  const int vlistID1 = cdoStreamInqVlist(streamID1);
+  const int vlistID2 = vlistDuplicate(vlistID1);
 
-  int taxisID1 = vlistInqTaxis(vlistID1);
-  int taxisID2 = taxisDuplicate(taxisID1);
+  const int taxisID1 = vlistInqTaxis(vlistID1);
+  const int taxisID2 = taxisDuplicate(taxisID1);
   if (taxisHasBounds(taxisID2)) taxisDeleteBounds(taxisID2);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1));
-  pstreamDefVlist(streamID2, vlistID2);
+  CdoStreamID streamID2 = cdoOpenWrite(1);
+  cdoDefVlist(streamID2, vlistID2);
 
-  int maxrecs = vlistNrecs(vlistID1);
-  std::vector<RecordInfo> recinfo(maxrecs);
+  const int maxrecs = vlistNrecs(vlistID1);
+  std::vector<RecordInfo> recList(maxrecs);
 
-  size_t gridsizemax = vlistGridsizeMax(vlistID1);
-  fieldInit(field);
-  field.ptr = (double *) Malloc(gridsizemax * sizeof(double));
+  const size_t gridsizemax = vlistGridsizeMax(vlistID1);
+
+  Field field;
+  field.resize(gridsizemax);
 
   int tsID = 0;
   int otsID = 0;
   while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
     {
-      int64_t vdate = taxisInqVdate(taxisID1);
-      int vtime = taxisInqVtime(taxisID1);
+      const int64_t vdate = taxisInqVdate(taxisID1);
+      const int vtime = taxisInqVtime(taxisID1);
 
       if (Options::cdoVerbose) cdoPrint("process timestep: %d %d %d", tsID + 1, vdate, vtime);
 
@@ -130,76 +128,72 @@ Ymonstat(void *process)
         {
           mon[nmon++] = month;
           fieldsFromVlist(vlistID1, samp1[month], FIELD_NONE);
-          fieldsFromVlist(vlistID1, vars1[month], FIELD_PTR);
-          if (lvars2) fieldsFromVlist(vlistID1, vars2[month], FIELD_PTR);
+          fieldsFromVlist(vlistID1, vars1[month], FIELD_VEC);
+          if (lvars2) fieldsFromVlist(vlistID1, vars2[month], FIELD_VEC);
         }
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          pstreamInqRecord(streamID1, &varID, &levelID);
+          cdoInqRecord(streamID1, &varID, &levelID);
 
           if (tsID == 0)
             {
-              recinfo[recID].varID = varID;
-              recinfo[recID].levelID = levelID;
-              recinfo[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
+              recList[recID].varID = varID;
+              recList[recID].levelID = levelID;
+              recList[recID].lconst = vlistInqVarTimetype(vlistID1, varID) == TIME_CONSTANT;
             }
 
           Field &rsamp1 = samp1[month][varID][levelID];
           Field &rvars1 = vars1[month][varID][levelID];
-          int nsets = month_nsets[month];
 
-          size_t gridsize = rvars1.size;
+          const int nsets = month_nsets[month];
+          const size_t fieldsize = rvars1.size;
 
           if (nsets == 0)
             {
-              pstreamReadRecord(streamID1, rvars1.ptr, &nmiss);
-              rvars1.nmiss = nmiss;
+              cdoReadRecord(streamID1, rvars1.vec.data(), &rvars1.nmiss);
               if (lrange)
                 {
                   vars2[month][varID][levelID].nmiss = rvars1.nmiss;
-                  for (size_t i = 0; i < gridsize; i++) vars2[month][varID][levelID].ptr[i] = rvars1.ptr[i];
+                  vars2[month][varID][levelID].vec = rvars1.vec;
                 }
 
-              if (nmiss > 0 || rsamp1.ptr)
+              if (rvars1.nmiss || !rsamp1.empty())
                 {
-                  if (rsamp1.ptr == nullptr) rsamp1.ptr = (double *) Malloc(gridsize * sizeof(double));
+                  if (rsamp1.empty()) rsamp1.resize(fieldsize);
 
-                  for (size_t i = 0; i < gridsize; i++) rsamp1.ptr[i] = !DBL_IS_EQUAL(rvars1.ptr[i], rvars1.missval);
+                  for (size_t i = 0; i < fieldsize; i++)
+                    rsamp1.vec[i] = !DBL_IS_EQUAL(rvars1.vec[i], rvars1.missval);
                 }
             }
           else
             {
-              pstreamReadRecord(streamID1, field.ptr, &nmiss);
-              field.nmiss = nmiss;
+              cdoReadRecord(streamID1, field.vec.data(), &field.nmiss);
+              field.size = fieldsize;
               field.grid = rvars1.grid;
               field.missval = rvars1.missval;
 
-              if (field.nmiss > 0 || rsamp1.ptr)
+              if (field.nmiss || !rsamp1.empty())
                 {
-                  if (rsamp1.ptr == nullptr)
-                    {
-                      rsamp1.ptr = (double *) Malloc(gridsize * sizeof(double));
-                      for (size_t i = 0; i < gridsize; i++) rsamp1.ptr[i] = nsets;
-                    }
+                  if (rsamp1.empty()) rsamp1.resize(fieldsize, nsets);
 
-                  for (size_t i = 0; i < gridsize; i++)
-                    if (!DBL_IS_EQUAL(field.ptr[i], rvars1.missval)) rsamp1.ptr[i]++;
+                  for (size_t i = 0; i < fieldsize; i++)
+                    if (!DBL_IS_EQUAL(field.vec[i], rvars1.missval)) rsamp1.vec[i]++;
                 }
 
               if (lvarstd)
                 {
-                  farsumq(vars2[month][varID][levelID], field);
-                  farsum(rvars1, field);
+                  vfarsumq(vars2[month][varID][levelID], field);
+                  vfarsum(rvars1, field);
                 }
               else if (lrange)
                 {
-                  farmin(vars2[month][varID][levelID], field);
-                  farmax(rvars1, field);
+                  vfarmin(vars2[month][varID][levelID], field);
+                  vfarmax(rvars1, field);
                 }
               else
                 {
-                  farfun(rvars1, field, operfunc);
+                  vfarfun(rvars1, field, operfunc);
                 }
             }
         }
@@ -207,12 +201,11 @@ Ymonstat(void *process)
       if (month_nsets[month] == 0 && lvarstd)
         for (int recID = 0; recID < maxrecs; recID++)
           {
-            if (recinfo[recID].lconst) continue;
+            if (recList[recID].lconst) continue;
 
-            int varID = recinfo[recID].varID;
-            int levelID = recinfo[recID].levelID;
-
-            farmoq(vars2[month][varID][levelID], vars1[month][varID][levelID]);
+            const int varID = recList[recID].varID;
+            const int levelID = recList[recID].levelID;
+            vfarmoq(vars2[month][varID][levelID], vars1[month][varID][levelID]);
           }
 
       month_nsets[month]++;
@@ -250,81 +243,69 @@ Ymonstat(void *process)
   for (int i = 0; i < nmon; i++)
     {
       month = mon[i];
-      int nsets = month_nsets[month];
+      const int nsets = month_nsets[month];
       if (nsets == 0) cdoAbort("Internal problem, nsets[%d] not defined!", month);
 
       for (int recID = 0; recID < maxrecs; recID++)
         {
-          if (recinfo[recID].lconst) continue;
+          if (recList[recID].lconst) continue;
 
-          int varID = recinfo[recID].varID;
-          int levelID = recinfo[recID].levelID;
+          const int varID = recList[recID].varID;
+          const int levelID = recList[recID].levelID;
           Field &rsamp1 = samp1[month][varID][levelID];
           Field &rvars1 = vars1[month][varID][levelID];
 
           if (lmean)
             {
-              if (rsamp1.ptr)
-                fardiv(rvars1, rsamp1);
+              if (!rsamp1.empty())
+                vfardiv(rvars1, rsamp1);
               else
-                farcdiv(rvars1, (double) nsets);
+                vfarcdiv(rvars1, (double) nsets);
             }
           else if (lvarstd)
             {
               Field &rvars2 = vars2[month][varID][levelID];
-              if (rsamp1.ptr)
+              if (!rsamp1.empty())
                 {
                   if (lstd)
-                    farstd(rvars1, rvars2, rsamp1, divisor);
+                    vfarstd(rvars1, rvars2, rsamp1, divisor);
                   else
-                    farvar(rvars1, rvars2, rsamp1, divisor);
+                    vfarvar(rvars1, rvars2, rsamp1, divisor);
                 }
               else
                 {
                   if (lstd)
-                    farcstd(rvars1, rvars2, nsets, divisor);
+                    vfarcstd(rvars1, rvars2, nsets, divisor);
                   else
-                    farcvar(rvars1, rvars2, nsets, divisor);
+                    vfarcvar(rvars1, rvars2, nsets, divisor);
                 }
             }
           else if (lrange)
             {
               Field &rvars2 = vars2[month][varID][levelID];
-              farsub(rvars1, rvars2);
+              vfarsub(rvars1, rvars2);
             }
         }
 
       taxisDefVdate(taxisID2, vdates[month]);
       taxisDefVtime(taxisID2, vtimes[month]);
-      pstreamDefTimestep(streamID2, otsID);
+      cdoDefTimestep(streamID2, otsID);
 
       for (int recID = 0; recID < maxrecs; recID++)
         {
-          if (otsID && recinfo[recID].lconst) continue;
+          if (otsID && recList[recID].lconst) continue;
 
-          int varID = recinfo[recID].varID;
-          int levelID = recinfo[recID].levelID;
+          const int varID = recList[recID].varID;
+          const int levelID = recList[recID].levelID;
           Field &rvars1 = vars1[month][varID][levelID];
 
-          pstreamDefRecord(streamID2, varID, levelID);
-          pstreamWriteRecord(streamID2, rvars1.ptr, rvars1.nmiss);
+          cdoDefRecord(streamID2, varID, levelID);
+          cdoWriteRecord(streamID2, rvars1.vec.data(), rvars1.nmiss);
         }
 
       otsID++;
     }
 
-  for (month = 0; month < NMONTH; month++)
-    {
-      if (vars1[month].size())
-        {
-          fieldsFree(vlistID1, samp1[month]);
-          fieldsFree(vlistID1, vars1[month]);
-          if (lvars2) fieldsFree(vlistID1, vars2[month]);
-        }
-    }
-
-  if (field.ptr) Free(field.ptr);
-
   cdoStreamClose(streamID2);
   cdoStreamClose(streamID1);
 
