@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,8 @@
 #endif
 
 #include "cdo_int.h"
-#include "cdoOptions.h"
+#include "cdo_options.h"
+#include "cimdOmp.h"
 
 #ifdef SX
 #define NFFT 1024
@@ -46,12 +47,6 @@
 
 #define D60 (S60 + S60)
 
-long
-get_nfft(void)
-{
-  return (long) NFFT;
-}
-
 void
 fft_set(double *trigs, long *ifax, long n)
 {
@@ -106,12 +101,11 @@ fft_set(double *trigs, long *ifax, long n)
 }
 
 static int
-rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, long inc2, long inc3, long inc4, long lot, long n,
-       long ifac, long la)
+rpassc(const double *a, const double *b, double *c, double *d, const double *trigs, long inc1, long inc2, long inc3, long inc4,
+       long lot, long n, long ifac, long la)
 {
   /*
-     rpassc' - performs one pass through data as part;
-     of multiple real fft (fourier synthesis) routine;
+     rpassc - performs one pass through data as part of multiple real fft (fourier synthesis) routine;
 
      a is first real input vector
      b is equivalent to a + la * inc1
@@ -988,12 +982,11 @@ rpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
 }
 
 static int
-qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, long inc2, long inc3, long inc4, long lot, long n,
-       long ifac, long la)
+qpassc(const double *a, const double *b, double *c, double *d, const double *trigs, long inc1, long inc2, long inc3, long inc4,
+       long lot, long n, long ifac, long la)
 {
   /*
-     qpassc - performs one pass through data as part
-     of multiple real fft (fourier analysis) routine.
+     qpassc - performs one pass through data as part of multiple real fft (fourier analysis) routine.
 
      a      is first real input vector
      b      is equivalent to a + ifac * la * inc1
@@ -1912,7 +1905,7 @@ qpassc(double *a, double *b, double *c, double *d, double *trigs, long inc1, lon
 /* Fast Fourier Transform */
 /* ====================== */
 void
-fc2gp(double *restrict trig, long *restrict ifax, double *restrict fc, double *restrict gp, long nlat, long nlon, long nlev,
+fc2gp(double *restrict trig, const long *restrict ifax, double *restrict fc, double *restrict gp, long nlat, long nlon, long nlev,
       long nfc)
 {
   /* fc2gp performs fourier to gridpoint transforms using           */
@@ -1930,18 +1923,18 @@ fc2gp(double *restrict trig, long *restrict ifax, double *restrict fc, double *r
 
   if (ifax[9] != nlon) fprintf(stderr, "fc2gp: wrong initialization!\n");
 
-  long nfax = ifax[0];
+  const long nfax = ifax[0];
 
-  long jump = (nlon + 2);
-  long lot = nlev * nlat;
+  const long jump = (nlon + 2);
+  const long lot = nlev * nlat;
 
-  long nx = (nlon % 2 == 1) ? nlon : nlon + 1;
-  long nblox = 1 + (lot - 1) / NFFT;
+  const long nx = (nlon % 2 == 1) ? nlon : nlon + 1;
+  const long nblox = 1 + (lot - 1) / NFFT;
   long nvex = lot - (nblox - 1) * NFFT;
-  long nvex0 = nvex;
+  const long nvex0 = nvex;
 
-  long nthmax = (nblox < Threading::ompNumThreads) ? nblox : Threading::ompNumThreads;
-  long nvals = lot * jump;
+  const long nthmax = (nblox < Threading::ompNumThreads) ? nblox : Threading::ompNumThreads;
+  const long nvals = lot * jump;
 
   double *restrict wfc = (double *) Malloc(nvals * sizeof(double));
   VECTOR_2D(double, wgp2d, nthmax, nvals);
@@ -1976,11 +1969,11 @@ fc2gp(double *restrict trig, long *restrict ifax, double *restrict fc, double *r
 #endif
   for (long nb = 0; nb < nblox; nb++)
     {
-      int ompthID = cdo_omp_get_thread_num();
+      const int ompthID = cdo_omp_get_thread_num();
       double *restrict wgp = &wgp2d[ompthID][0];
 
-      long istart = istartv[nb];
-      long nvex = (nb == 0) ? nvex0 : NFFT;
+      const long istart = istartv[nb];
+      const long nvex = (nb == 0) ? nvex0 : NFFT;
 
       long i = istart;
 #if defined(SX)
@@ -2005,7 +1998,7 @@ fc2gp(double *restrict trig, long *restrict ifax, double *restrict fc, double *r
       long la = 1;
       for (long k = 0; k < nfax; ++k)
         {
-          long ifac = ifax[k + 1];
+          const long ifac = ifax[k + 1];
 
           if (k & 1)
             rpassc(wgp, wgp + la, wfc + ia, wfc + ia + ifac * la, trig, 1, 1, nx, jump, nvex, nlon, ifac, la);
@@ -2059,7 +2052,7 @@ fc2gp(double *restrict trig, long *restrict ifax, double *restrict fc, double *r
 }
 
 void
-gp2fc(double *trig, long *ifax, const double *restrict gp, double *restrict fc, long nlat, long nlon, long nlev, long nfc)
+gp2fc(double *trig, const long *ifax, const double *restrict gp, double *restrict fc, long nlat, long nlon, long nlev, long nfc)
 {
   long lot, fou, ia, ifac, jump, k, la;
   long lat, lev, lon, nfax, rix, wix;
