@@ -27,15 +27,14 @@
 #include "dmemory.h"
 #include "constants.h"
 #include "cimdOmp.h"
+#include "mpmo.h"
 
 #define OPENMP4 201307
 #if defined(_OPENMP) && defined(OPENMP4) && _OPENMP >= OPENMP4
 #define HAVE_OPENMP4 1
 #endif
 
-extern "C" {
-void gaussaw(double *pa, double *pw, size_t nlat);
-}
+extern "C" void gaussianLatitudes(double *latitudes, double *weights, size_t nlat);
 
 static void
 jspleg1(double *pleg, double plat, long ktrunc, double *work)
@@ -293,37 +292,29 @@ after_legini_full(long ntr, long nlat, double *restrict poli, double *restrict p
   long waves = ntr + 1;
   long dimsp = (ntr + 1) * (ntr + 2);
 
-  std::vector<double> gmu(nlat);
-  std::vector<double> gwt(nlat);
-
-  gaussaw(gmu.data(), gwt.data(), nlat);
+  std::vector<double> gmu(nlat),  gwt(nlat);
+  gaussianLatitudes(gmu.data(), gwt.data(), nlat);
 
 #ifndef _OPENMP
-  std::vector<double> pnm(dimsp);
-  std::vector<double> hnm(dimsp);
-  std::vector<double> ztemp1(waves << 1);
-  std::vector<double> ztemp2(waves << 1);
+  std::vector<double> pnm(dimsp), hnm(dimsp), ztemp1(waves << 1), ztemp2(waves << 1);
 #endif
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(nlat, dimsp, waves, poli, pold, pdev, pol2, pol3, coslat, gmu, gwt, PlanetRadius)
+#pragma omp parallel for default(none) shared(nlat, waves, dimsp, poli, pold, pdev, pol2, pol3, coslat, gmu, gwt, PlanetRadius)
 #endif
   for (long jgl = 0; jgl < nlat; ++jgl)
     {
 #ifdef _OPENMP
-      std::vector<double> pnm(dimsp);
-      std::vector<double> hnm(dimsp);
-      std::vector<double> ztemp1(waves << 1);
-      std::vector<double> ztemp2(waves << 1);
+      std::vector<double> pnm(dimsp), hnm(dimsp), ztemp1(waves << 1), ztemp2(waves << 1);
 #endif
-      double gmusq = 1.0 - gmu[jgl] * gmu[jgl];
+      const double gmusq = 1.0 - gmu[jgl] * gmu[jgl];
       coslat[jgl] = sqrt(gmusq);
 
       phcs(pnm.data(), hnm.data(), waves, gmu[jgl], ztemp1.data(), ztemp2.data());
 
-      double zgwt = gwt[jgl];
-      double zrafgmusqr = 1. / (PlanetRadius * gmusq);
-      double zradsqrtgmusqr = 1. / (-PlanetRadius * sqrt(gmusq));
+      const double zgwt = gwt[jgl];
+      const double zrafgmusqr = 1. / (PlanetRadius * gmusq);
+      const double zradsqrtgmusqr = 1. / (-PlanetRadius * sqrt(gmusq));
 
       const int lpold = pold != nullptr;
       const int lpdev = pdev != nullptr;
@@ -348,27 +339,25 @@ void
 after_legini(long ntr, long nlat, double *restrict poli, double *restrict pold, double *restrict coslat)
 {
   long waves = ntr + 1;
-  long dimpnm = (ntr + 1) * (ntr + 4) / 2;
+  const long dimpnm = (ntr + 1) * (ntr + 4) / 2;
 
-  std::vector<double> gmu(nlat);
-  std::vector<double> gwt(nlat);
 #ifdef _OPENMP
   std::vector<std::vector<double>> pnm2(Threading::ompNumThreads);
   std::vector<std::vector<double>> work2(Threading::ompNumThreads);
   for (long i = 0; i < Threading::ompNumThreads; ++i) pnm2[i].resize(dimpnm);
   for (long i = 0; i < Threading::ompNumThreads; ++i) work2[i].resize(3 * waves);
 #else
-  std::vector<double> pnm(dimpnm);
-  std::vector<double> work(3 * waves);
+  std::vector<double> pnm(dimpnm), work(3 * waves);
 #endif
 
-  gaussaw(gmu.data(), gwt.data(), nlat);
+  std::vector<double> gmu(nlat), gwt(nlat);
+  gaussianLatitudes(gmu.data(), gwt.data(), nlat);
   for (long jgl = 0; jgl < nlat; ++jgl) gwt[jgl] *= 0.5;
 
   for (long jgl = 0; jgl < nlat; ++jgl) coslat[jgl] = sqrt(1.0 - gmu[jgl] * gmu[jgl]);
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(nlat, ntr, waves, pnm2, work2, gwt, gmu, poli, pold)
+#pragma omp parallel for default(none) shared(nlat, waves, ntr, pnm2, work2, gwt, gmu, poli, pold)
 #endif
   for (long jgl = 0; jgl < nlat / 2; jgl++)
     {
@@ -377,7 +366,7 @@ after_legini(long ntr, long nlat, double *restrict poli, double *restrict pold,
       double *pnm = pnm2[ompthID].data();
       double *work = work2[ompthID].data();
 #endif
-      double zgwt = gwt[jgl];
+      const double zgwt = gwt[jgl];
 
       jspleg1(&pnm[0], gmu[jgl], ntr, &work[0]);
 
