@@ -20,10 +20,7 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-/* =============================================== */
-/* These include files should be standard on all   */
-/* UNIX systems.                                   */
-/* =============================================== */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -31,13 +28,9 @@
 
 #include "error.h"
 #include "dmemory.h"
-
-#ifndef true
-#define true 1
-#endif
-#ifndef false
-#define false 0
-#endif
+#include "process_int.h"
+#include "transform.h"
+#include "cdo_task.h"
 
 #define MaxLevel 1024
 
@@ -75,8 +68,8 @@ struct Control
   int Szip;
 
   int istreamID;
-  int ostreamID;
-  int ostreamID2;
+  CdoStreamID ostreamID;
+  CdoStreamID ostreamID2;
   int ivlistID;
   int ovlistID;
   int ovlistID2;
@@ -169,23 +162,6 @@ struct Variable
   int *samp;
 };
 
-/* FFT */
-void fft_set(double *trigs, long *ifax, long n);
-void fc2gp(double *trig, const long *ifax, double *fc, double *gp, long nlat, long nlon, long nlev, long nfc);
-void gp2fc(double *trig, const long *ifax, const double *gp, double *fc, long nlat, long nlon, long nlev, long nfc);
-
-/* Convert Spectral Array to new resolution */
-void sp2sp(double *arrayIn, long truncIn, double *arrayOut, long truncOut);
-void sp2fc(const double *sa, double *fa, const double *poli, long nlev, long nlat, long nfc, long nt);
-void fc2sp(double *fa, double *sa, const double *poli, long klev, long nlat, long nfc, long nt);
-
-/* Physc */
-void dv2ps(const double * div, double * pot, long nlev, long ntr);
-void dv2uv(const double *d, const double *o, double *u, double *v, double *f, double *g, long nt, long nsp, long nlev);
-void scaluv(double *fu, const double *rclat, long nlat, long lot);
-void uv2dv(double *fu, double *fv, double *sd, double *sv, double *pol2, double *pol3, long klev, long nlat, long nt);
-void geninx(long ntr, double *f, double *g);
-
 /* clang-format off */
 #define    LOW_CLOUD   34
 #define    MID_CLOUD   35
@@ -265,4 +241,15 @@ void after_EchamDependencies(struct Variable *vars, int ncodes, int type, int so
 
 void after_legini_setup(struct Control *globs, struct Variable *vars);
 
+
+template <typename... Args>
+void
+afterAbort(const std::string &format, Args const &... args)
+{
+  extern void *afterReadTask;
+  extern bool afterReadAsync;
+  if (afterReadAsync) cdo_task_wait(afterReadTask);
+  cdoAbort(format, args...);
+}
+
 #endif /* AFTERBURNER_H */
