@@ -20,29 +20,28 @@
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
+
 #ifdef CDO
 #include "cdo_int.h"
 #include "cdo_math.h"
 #include "pstream_write.h"
-#define streamDefRecord pstreamDefRecord
-#define streamWriteRecord pstreamWriteRecord
+#define streamDefRecord cdoDefRecord
+#define streamWriteRecord cdoWriteRecord
 #else
 #define OPENMP4 201307
 #ifdef _OPENMP &&defined(OPENMP4) && _OPENMP >= OPENMP4
 #define HAVE_OPENMP4 1
 #endif
 #endif
+
 #endif
 
-#include "error.h"
 #include "afterburner.h"
 #include "constants.h"
 #include "compare.h"
 #include "after_vertint.h"
-#include "pstream_int.h"
 
 int afterDebug = 0;
-int labort_after = true;
 
 static char *
 FieldName(int code, const char *text)
@@ -107,7 +106,7 @@ alloc_dp(int words, const char *array_name)
   if (words > 0)
     {
       result = (double *) Malloc(words * sizeof(double));
-      if (result == nullptr) SysError(array_name, "No Memory!");
+      if (result == nullptr) cdoSysError(array_name, "No Memory!");
     }
 
   return result;
@@ -142,7 +141,7 @@ VarQuaSum(double *Variance, const double *restrict Sum, int len, int n)
 static void
 AddVector(double *dest, const double *src, size_t len, size_t *nmiss, double missval)
 {
-  if (*nmiss > 0)
+  if (*nmiss)
     {
       arrayAddArrayMV(len, dest, src, missval);
       *nmiss = arrayNumMV(len, dest, missval);
@@ -169,7 +168,7 @@ Sub2Vectors(double *dest, const double *restrict srcA, const double *restrict sr
 static void
 MultVectorScalar(double *dest, const double *restrict src, double factor, size_t len, size_t nmiss, double missval)
 {
-  if (nmiss > 0)
+  if (nmiss)
     {
       for (size_t i = 0; i < len; i++)
         {
@@ -210,7 +209,7 @@ after_read_vct(const char *vctfile, double **vct, int *nvct)
   double va, vb;
 
   FILE *fp = fopen(vctfile, "r");
-  if (fp == nullptr) SysError("Open failed on %s", vctfile);
+  if (fp == nullptr) cdoSysError("Open failed on %s", vctfile);
 
   while (fgets(line, 1023, fp)) nvct++;
 
@@ -487,10 +486,7 @@ CheckAnalyses(struct Variable *vars)
         && code != VELOPOT && code != V_WIND && code != RHUMIDITY && code != GEOPOTHEIGHT && code != PS
         && vars[code].spectral == nullptr && vars[code].grid == nullptr)
       {
-        if (labort_after)
-          Error("Code  %3d not found", code);
-        else
-          Warning("Code  %3d not found", code);
+        afterAbort("Code  %3d not found", code);
       }
 }
 
@@ -606,7 +602,7 @@ after_processPL(struct Control *globs, struct Variable *vars)
       for (code = 0; code < MaxCodes; code++)
         if (vars[code].selected)
           {
-            if (!vars[code].spectral) Error("Code %d not available on spectral space!", code);
+            if (!vars[code].spectral) afterAbort("Code %d not available on spectral space!", code);
 
             nlevel = zaxisInqSize(vars[code].ozaxisID);
             for (lindex = 0; lindex < nlevel; lindex++)
@@ -682,7 +678,7 @@ after_processPL(struct Control *globs, struct Variable *vars)
       for (code = 0; code < MaxCodes; code++)
         if (vars[code].selected)
           {
-            if (!vars[code].fourier) Error("Code %d not available on fourier space!", code);
+            if (!vars[code].fourier) afterAbort("Code %d not available on fourier space!", code);
 
             nlevel = zaxisInqSize(vars[code].ozaxisID);
             for (lindex = 0; lindex < nlevel; lindex++)
@@ -706,7 +702,7 @@ after_processPL(struct Control *globs, struct Variable *vars)
       for (code = 0; code < MaxCodes; code++)
         if (vars[code].selected)
           {
-            if (!vars[code].fourier) Error("Code %d not available on zonal mean!", code);
+            if (!vars[code].fourier) afterAbort("Code %d not available on zonal mean!", code);
 
             nlevel = zaxisInqSize(vars[code].ozaxisID);
             for (lindex = 0; lindex < nlevel; lindex++)
@@ -966,7 +962,7 @@ Omega(double *omega_in, double *divergence, double *u_wind, double *v_wind, doub
       for (i = 0; i < dimgp; i++) omega[i] = 0.5 * (omega[i] + omega[i + dimgp]);
     }
 
-/* compute full level part of vertical velocity */
+    /* compute full level part of vertical velocity */
 
 #if defined(_OPENMP)
 #pragma omp parallel for default(shared) private(i, omega, halfp, fullp, uwind, vwind, DeltaHybrid, Cterm, Pterm)
@@ -1126,7 +1122,8 @@ LayerCloud(double *cc, double *ll, double pmax, double pmin, int DimGP, int Half
   for (k = MaxLev + 1; k <= MinLev; k++)
     {
       for (i = 0; i < DimGP; i++)
-        ll[i] *= (1. - cdo::max(cc[i + (k - 1) * DimGP], cc[i + k * DimGP])) / (1. - cdo::min(cc[i + (k - 1) * DimGP], 1. - ZEPSEC));
+        ll[i]
+            *= (1. - cdo::max(cc[i + (k - 1) * DimGP], cc[i + k * DimGP])) / (1. - cdo::min(cc[i + (k - 1) * DimGP], 1. - ZEPSEC));
     }
   for (i = 0; i < DimGP; i++) ll[i] = 1. - ll[i];
 }
@@ -1519,7 +1516,7 @@ after_processML(struct Control *globs, struct Variable *vars)
 
   if (globs->MeanCount == 1)
     {
-      if (globs->Debug) Message("TermCount = %d MeanCount = %d", globs->TermCount, globs->MeanCount);
+      if (globs->Debug) cdoPrint("TermCount = %d MeanCount = %d", globs->TermCount, globs->MeanCount);
       globs->StartDate = globs->OldDate;
     }
 
@@ -1581,7 +1578,7 @@ after_processML(struct Control *globs, struct Variable *vars)
       for (code = 0; code < MaxCodes; code++)
         if (vars[code].selected)
           {
-            if (!vars[code].spectral) Error("Code %d not available on spectral space!", code);
+            if (!vars[code].spectral) afterAbort("Code %d not available on spectral space!", code);
 
             nlevel = zaxisInqSize(vars[code].ozaxisID);
             for (lindex = 0; lindex < nlevel; lindex++)
@@ -1656,7 +1653,7 @@ after_processML(struct Control *globs, struct Variable *vars)
       for (code = 0; code < MaxCodes; code++)
         if (vars[code].selected)
           {
-            if (!vars[code].fourier) Error("Code %d not available on fourier space!", code);
+            if (!vars[code].fourier) afterAbort("Code %d not available on fourier space!", code);
 
             nlevel = zaxisInqSize(vars[code].ozaxisID);
             for (lindex = 0; lindex < nlevel; lindex++)
@@ -1680,7 +1677,7 @@ after_processML(struct Control *globs, struct Variable *vars)
       for (code = 0; code < MaxCodes; code++)
         if (vars[code].selected)
           {
-            if (!vars[code].fourier) Error("Code %d not available on zonal mean!", code);
+            if (!vars[code].fourier) afterAbort("Code %d not available on zonal mean!", code);
 
             nlevel = zaxisInqSize(vars[code].ozaxisID);
             for (lindex = 0; lindex < nlevel; lindex++)
@@ -1723,12 +1720,12 @@ after_processML(struct Control *globs, struct Variable *vars)
             }
           else if (vars[PS].hybrid)
             {
-              Warning("log surface pressure (code 152) not found - using surface pressure (code 134)!");
+              cdoWarning("log surface pressure (code 152) not found - using surface pressure (code 134)!");
               arrayCopy(globs->DimGP, vars[PS].hybrid, vars[PS_PROG].hybrid);
             }
           else
             {
-              Error("surface pressure not found!");
+              afterAbort("surface pressure not found!");
             }
         }
       vars[LNPS].needed = vars[LNPS].selected;
@@ -1742,7 +1739,7 @@ after_processML(struct Control *globs, struct Variable *vars)
             {
               if (vars[GEOPOTENTIAL].selected || globs->Type >= 30)
                 {
-                  Warning("Orography not found - using zero orography!");
+                  cdoWarning("Orography not found - using zero orography!");
                   arrayFill(globs->DimGP, globs->Orography, 0.0);
                 }
             }
@@ -1808,7 +1805,7 @@ after_processML(struct Control *globs, struct Variable *vars)
           for (code = 0; code < MaxCodes; code++)
             if (vars[code].selected)
               {
-                if (vars[code].hybrid == nullptr) Error("Internal problem. Code %d not allocated!", code);
+                if (vars[code].hybrid == nullptr) afterAbort("Internal problem. Code %d not allocated!", code);
 
                 nlevel = zaxisInqSize(vars[code].ozaxisID);
                 for (lindex = 0; lindex < nlevel; lindex++)
@@ -1889,7 +1886,7 @@ after_processML(struct Control *globs, struct Variable *vars)
                   }
                 else if (code == GEOPOTHEIGHT)
                   {
-                    if (vars[TEMPERATURE].hybrid == nullptr) Error("Code  130 not found!");
+                    if (vars[TEMPERATURE].hybrid == nullptr) afterAbort("Code  130 not found!");
                     interp_Z(globs->Orography, vars[GEOPOTHEIGHT].hybrid, vars[GEOPOTHEIGHT].grid, vars[FULL_PRESS].hybrid,
                              vars[HALF_PRESS].hybrid, globs->vert_index, vars[TEMPERATURE].hybrid, pressureLevel,
                              globs->NumLevelRequest, globs->DimGP, globs->NumLevel, vars[code].missval);
@@ -2049,7 +2046,7 @@ after_processML(struct Control *globs, struct Variable *vars)
       for (code = 0; code < MaxCodes; code++)
         if (vars[code].needed && vars[code].grid && (vars[code].sfit || globs->Type < 70))
           {
-            if (vars[code].nmiss > 0) Error("Missing values for code %d unsupported with TYPE > 30!", code);
+            if (vars[code].nmiss) afterAbort("Missing values for code %d unsupported with TYPE > 30!", code);
 
             if (vars[code].fourier == nullptr)
               {
@@ -2075,7 +2072,7 @@ after_processML(struct Control *globs, struct Variable *vars)
       for (code = 0; code < MaxCodes; code++)
         if (vars[code].selected)
           {
-            if (!vars[code].fourier) Error("Code %d not available on fourier space!", code);
+            if (!vars[code].fourier) afterAbort("Code %d not available on fourier space!", code);
 
             nlevel = zaxisInqSize(vars[code].ozaxisID);
             for (lindex = 0; lindex < nlevel; lindex++)
@@ -2099,7 +2096,7 @@ after_processML(struct Control *globs, struct Variable *vars)
       for (code = 0; code < MaxCodes; code++)
         if (vars[code].selected)
           {
-            if (!vars[code].fourier) Error("Code %d not available on zonal mean!", code);
+            if (!vars[code].fourier) afterAbort("Code %d not available on zonal mean!", code);
 
             nlevel = zaxisInqSize(vars[code].ozaxisID);
             for (lindex = 0; lindex < nlevel; lindex++)
@@ -2144,7 +2141,7 @@ after_processML(struct Control *globs, struct Variable *vars)
             vars[DIVERGENCE].spectral = alloc_dp(globs->DimSP * globs->NumLevelRequest, "vars[DIVERGENCE].spectral");
           if (vars[VORTICITY].spectral == nullptr)
             vars[VORTICITY].spectral = alloc_dp(globs->DimSP * globs->NumLevelRequest, "vars[VORTICITY].spectral");
-          if ((vars[U_WIND].fourier == 0 || vars[V_WIND].fourier == 0) && globs->NumLevelRequest) Error("uwind or vwind missing!");
+          if ((vars[U_WIND].fourier == 0 || vars[V_WIND].fourier == 0) && globs->NumLevelRequest) afterAbort("uwind or vwind missing!");
           uv2dv(vars[U_WIND].fourier, vars[V_WIND].fourier, vars[DIVERGENCE].spectral, vars[VORTICITY].spectral, globs->pol2,
                 globs->pol3, globs->NumLevelRequest, globs->Latitudes, globs->Truncation);
         }
@@ -2229,7 +2226,7 @@ after_processML(struct Control *globs, struct Variable *vars)
       for (code = 0; code < MaxCodes; code++)
         if (vars[code].selected)
           {
-            if (!vars[code].fourier) Error("Code %d not available on fourier space!", code);
+            if (!vars[code].fourier) afterAbort("Code %d not available on fourier space!", code);
 
             nlevel = zaxisInqSize(vars[code].ozaxisID);
             for (lindex = 0; lindex < nlevel; lindex++)
@@ -2253,7 +2250,7 @@ after_processML(struct Control *globs, struct Variable *vars)
       for (code = 0; code < MaxCodes; code++)
         if (vars[code].selected)
           {
-            if (!vars[code].fourier) Error("Code %d not available on zonal mean!", code);
+            if (!vars[code].fourier) afterAbort("Code %d not available on zonal mean!", code);
 
             nlevel = zaxisInqSize(vars[code].ozaxisID);
             for (lindex = 0; lindex < nlevel; lindex++)
@@ -2347,7 +2344,7 @@ after_AnalysisAddRecord(struct Control *globs, struct Variable *vars, int code,
             }
         }
       else
-        Error("Only pressure level data supported for spectral data!");
+        afterAbort("Only pressure level data supported for spectral data!");
     }
   else
     {
@@ -2370,7 +2367,7 @@ after_AnalysisAddRecord(struct Control *globs, struct Variable *vars, int code,
           arrayCopy(globs->DimGP, globs->Field, vars[code].grid0);
         }
 
-      if (globs->Mean > 0 && (nmiss > 0 || vars[code].samp))
+      if (globs->Mean > 0 && (nmiss || vars[code].samp))
         {
           if (vars[code].samp == nullptr)
             {
@@ -2431,7 +2428,7 @@ after_EchamAddRecord(struct Control *globs, struct Variable *vars, int code, int
       if (nlevel > 1 && leveltype == ZAXIS_HYBRID) vars[code].plev = globs->NumLevelRequest;
 
       if (gridInqTrunc(gridID) != globs->Truncation)
-        Error("Resolution error. Required %d - Found %d", globs->Truncation, gridInqTrunc(gridID));
+        afterAbort("Resolution error. Required %d - Found %d", globs->Truncation, gridInqTrunc(gridID));
     }
   else
     {
@@ -2444,7 +2441,7 @@ after_EchamAddRecord(struct Control *globs, struct Variable *vars, int code, int
           vars[code].sfit = true;
         }
 
-      if (globs->Mean > 0 && (nmiss > 0 || vars[code].samp))
+      if (globs->Mean > 0 && (nmiss || vars[code].samp))
         {
           if (vars[code].samp == nullptr)
             {
@@ -2487,7 +2484,7 @@ MakeDependencies(struct Variable *vars, int varcode, int depcode)
         {
           if (vars[depcode].ivarID == -1)
             {
-              Error("code %d undefined, needed to compute code %d", depcode, varcode);
+              afterAbort("code %d undefined, needed to compute code %d", depcode, varcode);
             }
           else
             {
