@@ -41,174 +41,155 @@ fpe_errstr(int fpeRaised)
 }
 
 void
-arrayMinMax(const size_t len, const double *array, double *rmin, double *rmax)
+arrayMinMax(const size_t len, const double *array, double &rmin, double &rmax)
 {
-  double min = DBL_MAX;
-  double max = -DBL_MAX;
+  double zmin = DBL_MAX;
+  double zmax = -DBL_MAX;
 
   // #pragma omp parallel for default(none) shared(min, max, array, gridsize)
   // reduction(+:mean) #pragma omp simd reduction(+:mean) reduction(min:min)
   // reduction(max:max) aligned(array:16)
   for (size_t i = 0; i < len; ++i)
     {
-      if (array[i] < min) min = array[i];
-      if (array[i] > max) max = array[i];
+      if (array[i] < zmin) zmin = array[i];
+      if (array[i] > zmax) zmax = array[i];
     }
 
-  if (rmin) *rmin = min;
-  if (rmax) *rmax = max;
+  rmin = zmin;
+  rmax = zmax;
 }
 
 size_t
-arrayMinMaxMV(const size_t len, const double *array, const double missval, double *rmin, double *rmax)
+arrayMinMaxMV(const size_t len, const double *array, const double missval, double &rmin, double &rmax)
 {
-  double min = DBL_MAX;
-  double max = -DBL_MAX;
+  double zmin = DBL_MAX;
+  double zmax = -DBL_MAX;
 
   size_t nvals = 0;
   for (size_t i = 0; i < len; ++i)
     {
       if (!DBL_IS_EQUAL(array[i], missval))
         {
-          if (array[i] < min) min = array[i];
-          if (array[i] > max) max = array[i];
+          if (array[i] < zmin) zmin = array[i];
+          if (array[i] > zmax) zmax = array[i];
           nvals++;
         }
     }
 
-  if (rmin) *rmin = min;
-  if (rmax) *rmax = max;
+  rmin = zmin;
+  rmax = zmax;
 
   return nvals;
 }
 
 void
-arrayMinMaxSum(const size_t len, const double *array, double *rmin, double *rmax, double *rsum)
+arrayMinMaxSum(const size_t len, const double *array, double &rmin, double &rmax, double &rsum)
 {
-  double min = DBL_MAX;
-  double max = -DBL_MAX;
-  double sum = 0;
+  // rmin, rmax and rsum will be initialized in Info
 
   // #pragma omp parallel for default(none) shared(min, max, array, gridsize)
   // reduction(+:mean) #pragma omp simd reduction(+:mean) reduction(min:min)
   // reduction(max:max) aligned(array:16)
   for (size_t i = 0; i < len; ++i)
     {
-      if (array[i] < min) min = array[i];
-      if (array[i] > max) max = array[i];
-      sum += array[i];
+      if (array[i] < rmin) rmin = array[i];
+      if (array[i] > rmax) rmax = array[i];
+      rsum += array[i];
     }
-
-  if (rmin) *rmin = min;
-  if (rmax) *rmax = max;
-  if (rsum) *rsum = sum;
 }
 
 void
-arrayMinMaxSum_f(const size_t len, const float *array, double *rmin, double *rmax, double *rsum)
+arrayMinMaxSum_f(const size_t len, const float *array, double &rmin, double &rmax, double &rsum)
 {
-  double min = DBL_MAX;
-  double max = -DBL_MAX;
-  double sum = 0;
+  // rmin, rmax and rsum will be initialized in Info
 
   // #pragma omp parallel for default(none) shared(min, max, array, gridsize)
   // reduction(+:mean) #pragma omp simd reduction(+:mean) reduction(min:min)
   // reduction(max:max) aligned(array:16)
   for (size_t i = 0; i < len; ++i)
     {
-      if (array[i] < min) min = array[i];
-      if (array[i] > max) max = array[i];
-      sum += array[i];
+      if (array[i] < rmin) rmin = array[i];
+      if (array[i] > rmax) rmax = array[i];
+      rsum += array[i];
     }
-
-  if (rmin) *rmin = min;
-  if (rmax) *rmax = max;
-  if (rsum) *rsum = sum;
 }
 
 size_t
-arrayMinMaxSumMV(const size_t len, const double *array, const double missval, double *rmin, double *rmax, double *rsum)
+arrayMinMaxSumMV(const size_t len, const double *array, const double missval, double &rmin, double &rmax, double &rsum)
 {
-  double min = DBL_MAX;
-  double max = -DBL_MAX;
-  double sum = 0;
+  // rmin, rmax and rsum will be initialized in Info
 
   size_t nvals = 0;
   for (size_t i = 0; i < len; ++i)
     {
       if (!DBL_IS_EQUAL(array[i], missval))
         {
-          if (array[i] < min) min = array[i];
-          if (array[i] > max) max = array[i];
-          sum += array[i];
+          if (array[i] < rmin) rmin = array[i];
+          if (array[i] > rmax) rmax = array[i];
+          rsum += array[i];
           nvals++;
         }
     }
 
-  if (nvals == 0) min = missval;
-  if (nvals == 0) max = missval;
-
-  if (rmin) *rmin = min;
-  if (rmax) *rmax = max;
-  if (rsum) *rsum = sum;
+  if (nvals == 0 && IS_EQUAL(rmin,  DBL_MAX)) rmin = missval;
+  if (nvals == 0 && IS_EQUAL(rmax, -DBL_MAX)) rmax = missval;
 
   return nvals;
 }
 
 size_t
-arrayMinMaxSumMV_f(const size_t len, const float *array, const double missval, double *rmin, double *rmax, double *rsum)
+arrayMinMaxSumMV_f(const size_t len, const float *array, const double missval, double &rmin, double &rmax, double &rsum)
 {
-  double min = DBL_MAX;
-  double max = -DBL_MAX;
-  double sum = 0;
+  // rmin, rmax and rsum will be initialized in Info
 
   size_t nvals = 0;
   for (size_t i = 0; i < len; ++i)
     {
       if (!DBL_IS_EQUAL(array[i], missval))
         {
-          if (array[i] < min) min = array[i];
-          if (array[i] > max) max = array[i];
-          sum += array[i];
+          if (array[i] < rmin) rmin = array[i];
+          if (array[i] > rmax) rmax = array[i];
+          rsum += array[i];
           nvals++;
         }
     }
 
-  if (nvals == 0) min = missval;
-  if (nvals == 0) max = missval;
 
-  if (rmin) *rmin = min;
-  if (rmax) *rmax = max;
-  if (rsum) *rsum = sum;
+  if (nvals == 0 && IS_EQUAL(rmin,  DBL_MAX)) rmin = missval;
+  if (nvals == 0 && IS_EQUAL(rmax, -DBL_MAX)) rmax = missval;
 
   return nvals;
 }
 
 void
-arrayMinMaxMean(const size_t len, const double *array, double *rmin, double *rmax, double *rmean)
+arrayMinMaxMean(const size_t len, const double *array, double &rmin, double &rmax, double &rmean)
 {
-  double sum;
-  arrayMinMaxSum(len, array, rmin, rmax, &sum);
+  rmin = DBL_MAX;
+  rmax = -DBL_MAX;
+  double sum = 0.0;
+  arrayMinMaxSum(len, array, rmin, rmax, sum);
 
-  if (rmean) *rmean = len ? sum / (double) len : 0;
+  rmean = len ? sum / (double) len : 0;
 }
 
 size_t
-arrayMinMaxMeanMV(const size_t len, const double *array, const double missval, double *rmin, double *rmax, double *rmean)
+arrayMinMaxMeanMV(const size_t len, const double *array, const double missval, double &rmin, double &rmax, double &rmean)
 {
-  double sum;
-  size_t nvals = arrayMinMaxSumMV(len, array, missval, rmin, rmax, &sum);
+  rmin = DBL_MAX;
+  rmax = -DBL_MAX;
+  double sum = 0.0;
+  size_t nvals = arrayMinMaxSumMV(len, array, missval, rmin, rmax, sum);
 
-  if (rmean) *rmean = nvals ? sum / (double) nvals : missval;
+  rmean = nvals ? sum / (double) nvals : missval;
 
   return nvals;
 }
 
 void
-arrayMinMaxMask(const size_t len, const double *array, int *mask, double *rmin, double *rmax)
+arrayMinMaxMask(const size_t len, const double *array, int *mask, double &rmin, double &rmax)
 {
-  double xmin = DBL_MAX;
-  double xmax = -DBL_MAX;
+  double zmin = DBL_MAX;
+  double zmax = -DBL_MAX;
 
   if (mask)
     {
@@ -216,10 +197,8 @@ arrayMinMaxMask(const size_t len, const double *array, int *mask, double *rmin,
         {
           if (!mask[i])
             {
-              if (array[i] > xmax)
-                xmax = array[i];
-              else if (array[i] < xmin)
-                xmin = array[i];
+              if (array[i] < zmin) zmin = array[i];
+              if (array[i] > zmax) zmax = array[i];
             }
         }
     }
@@ -227,15 +206,13 @@ arrayMinMaxMask(const size_t len, const double *array, int *mask, double *rmin,
     {
       for (size_t i = 0; i < len; ++i)
         {
-          if (array[i] > xmax)
-            xmax = array[i];
-          else if (array[i] < xmin)
-            xmin = array[i];
+          if (array[i] < zmin) zmin = array[i];
+          if (array[i] > zmax) zmax = array[i];
         }
     }
 
-  if (rmin) *rmin = xmin;
-  if (rmax) *rmax = xmax;
+  rmin = zmin;
+  rmax = zmax;
 }
 
 void
@@ -313,60 +290,58 @@ arrayNumMV_f(const size_t len, const float *restrict array, const double missval
 }
 
 double
-arrayMin(const size_t len, const double *restrict array)
+arrayMin(const size_t len, const std::vector<double> &v)
 {
-  assert(array != nullptr);
+  assert(v.size() >= len);
 
-  double min = array[0];
+  double min = v[0];
 
   for (size_t i = 0; i < len; ++i)
-    if (array[i] < min) min = array[i];
+    if (v[i] < min) min = v[i];
 
   return min;
 }
 
 double
-arrayMax(const size_t len, const double *restrict array)
+arrayMax(const size_t len, const std::vector<double> &v)
 {
-  assert(array != nullptr);
+  assert(v.size() >= len);
 
-  double max = array[0];
+  double max = v[0];
 
   for (size_t i = 0; i < len; ++i)
-    if (array[i] > max) max = array[i];
+    if (v[i] > max) max = v[i];
 
   return max;
 }
 
 double
-arrayRange(const size_t len, const double *restrict array)
+arrayRange(const size_t len, const std::vector<double> &v)
 {
-  assert(array != nullptr);
+  assert(v.size() >= len);
 
-  double min = array[0];
-  double max = array[0];
+  double min = v[0];
+  double max = v[0];
 
   for (size_t i = 0; i < len; ++i)
     {
-      if (array[i] < min) min = array[i];
-      if (array[i] > max) max = array[i];
+      if (v[i] < min) min = v[i];
+      if (v[i] > max) max = v[i];
     }
 
-  double range = max - min;
-
-  return range;
+  return (max - min);
 }
 
 double
-arrayMinMV(const size_t len, const double *restrict array, const double missval)
+arrayMinMV(const size_t len, const std::vector<double> &v, const double missval)
 {
-  assert(array != nullptr);
+  assert(v.size() >= len);
 
   double min = DBL_MAX;
 
   for (size_t i = 0; i < len; ++i)
-    if (!DBL_IS_EQUAL(array[i], missval))
-      if (array[i] < min) min = array[i];
+    if (!DBL_IS_EQUAL(v[i], missval))
+      if (v[i] < min) min = v[i];
 
   if (IS_EQUAL(min, DBL_MAX)) min = missval;
 
@@ -374,15 +349,15 @@ arrayMinMV(const size_t len, const double *restrict array, const double missval)
 }
 
 double
-arrayMaxMV(const size_t len, const double *restrict array, const double missval)
+arrayMaxMV(const size_t len, const std::vector<double> &v, const double missval)
 {
-  assert(array != nullptr);
+  assert(v.size() >= len);
 
   double max = -DBL_MAX;
 
   for (size_t i = 0; i < len; ++i)
-    if (!DBL_IS_EQUAL(array[i], missval))
-      if (array[i] > max) max = array[i];
+    if (!DBL_IS_EQUAL(v[i], missval))
+      if (v[i] > max) max = v[i];
 
   if (IS_EQUAL(max, -DBL_MAX)) max = missval;
 
@@ -390,19 +365,19 @@ arrayMaxMV(const size_t len, const double *restrict array, const double missval)
 }
 
 double
-arrayRangeMV(const size_t len, const double *restrict array, const double missval)
+arrayRangeMV(const size_t len, const std::vector<double> &v, const double missval)
 {
-  assert(array != nullptr);
+  assert(v.size() >= len);
 
   double min = DBL_MAX;
   double max = -DBL_MAX;
 
   for (size_t i = 0; i < len; ++i)
     {
-      if (!DBL_IS_EQUAL(array[i], missval))
+      if (!DBL_IS_EQUAL(v[i], missval))
         {
-          if (array[i] < min) min = array[i];
-          if (array[i] > max) max = array[i];
+          if (v[i] < min) min = v[i];
+          if (v[i] > max) max = v[i];
         }
     }
 
@@ -416,21 +391,21 @@ arrayRangeMV(const size_t len, const double *restrict array, const double missva
 }
 
 double
-arraySum(const size_t len, const double *restrict array)
+arraySum(const size_t len, const std::vector<double> &v)
 {
-  assert(array != nullptr);
+  assert(v.size() >= len);
 
   double sum = 0;
 
-  for (size_t i = 0; i < len; ++i) sum += array[i];
+  for (size_t i = 0; i < len; ++i) sum += v[i];
 
   return sum;
 }
 
 double
-arraySumMV(const size_t len, const double *restrict array, const double missval)
+arraySumMV(const size_t len, const std::vector<double> &v, const double missval)
 {
-  assert(array != nullptr);
+  assert(v.size() >= len);
 
   double sum = 0;
   size_t nvals = 0;
@@ -438,18 +413,18 @@ arraySumMV(const size_t len, const double *restrict array, const double missval)
   if (DBL_IS_NAN(missval))
     {
       for (size_t i = 0; i < len; ++i)
-        if (!DBL_IS_EQUAL(array[i], missval))
+        if (!DBL_IS_EQUAL(v[i], missval))
           {
-            sum += array[i];
+            sum += v[i];
             nvals++;
           }
     }
   else
     {
       for (size_t i = 0; i < len; ++i)
-        if (IS_NOT_EQUAL(array[i], missval))
+        if (IS_NOT_EQUAL(v[i], missval))
           {
-            sum += array[i];
+            sum += v[i];
             nvals++;
           }
     }
@@ -460,26 +435,26 @@ arraySumMV(const size_t len, const double *restrict array, const double missval)
 }
 
 double
-arrayMean(const size_t len, const double *restrict array)
+arrayMean(const size_t len, const std::vector<double> &v)
 {
-  assert(array != nullptr);
+  assert(v.size() >= len);
 
-  double sum = arraySum(len, array);
+  const double sum = arraySum(len, v);
 
   return sum / len;
 }
 
 double
-arrayMeanMV(const size_t len, const double *restrict array, const double missval)
+arrayMeanMV(const size_t len, const std::vector<double> &v, const double missval)
 {
-  assert(array != nullptr);
+  assert(v.size() >= len);
 
   double sum = 0, sumw = 0;
 
   for (size_t i = 0; i < len; ++i)
-    if (!DBL_IS_EQUAL(array[i], missval))
+    if (!DBL_IS_EQUAL(v[i], missval))
       {
-        sum += array[i];
+        sum += v[i];
         sumw += 1;
       }
 
@@ -488,16 +463,16 @@ arrayMeanMV(const size_t len, const double *restrict array, const double missval
 }
 
 double
-arrayWeightedMean(const size_t len, const double *restrict array, const double *restrict w, const double missval)
+arrayWeightedMean(const size_t len, const std::vector<double> &v, const std::vector<double> &w, const double missval)
 {
-  assert(array != nullptr);
-  assert(w != nullptr);
+  assert(v.size() >= len);
+  assert(w.size() >= len);
 
   double sum = 0, sumw = 0;
 
   for (size_t i = 0; i < len; ++i)
     {
-      sum += w[i] * array[i];
+      sum += w[i] * v[i];
       sumw += w[i];
     }
 
@@ -505,18 +480,18 @@ arrayWeightedMean(const size_t len, const double *restrict array, const double *
 }
 
 double
-arrayWeightedMeanMV(const size_t len, const double *restrict array, const double *restrict w, const double missval)
+arrayWeightedMeanMV(const size_t len, const std::vector<double> &v, const std::vector<double> &w, const double missval)
 {
-  assert(array != nullptr);
-  assert(w != nullptr);
+  assert(v.size() >= len);
+  assert(w.size() >= len);
 
-  double missval1 = missval, missval2 = missval;
+  const double missval1 = missval, missval2 = missval;
   double sum = 0, sumw = 0;
 
   for (size_t i = 0; i < len; ++i)
-    if (!DBL_IS_EQUAL(array[i], missval1) && !DBL_IS_EQUAL(w[i], missval1))
+    if (!DBL_IS_EQUAL(v[i], missval1) && !DBL_IS_EQUAL(w[i], missval1))
       {
-        sum += w[i] * array[i];
+        sum += w[i] * v[i];
         sumw += w[i];
       }
 
@@ -524,16 +499,16 @@ arrayWeightedMeanMV(const size_t len, const double *restrict array, const double
 }
 
 double
-arrayAvgMV(const size_t len, const double *restrict array, const double missval)
+arrayAvgMV(const size_t len, const std::vector<double> &v, const double missval)
 {
-  assert(array != nullptr);
+  assert(v.size() >= len);
 
-  double missval1 = missval, missval2 = missval;
+  const double missval1 = missval, missval2 = missval;
   double sum = 0, sumw = 0;
 
   for (size_t i = 0; i < len; ++i)
     {
-      sum = ADDMN(sum, array[i]);
+      sum = ADDMN(sum, v[i]);
       sumw += 1;
     }
 
@@ -541,18 +516,18 @@ arrayAvgMV(const size_t len, const double *restrict array, const double missval)
 }
 
 double
-arrayWeightedAvgMV(const size_t len, const double *restrict array, const double *restrict w, const double missval)
+arrayWeightedAvgMV(const size_t len, const std::vector<double> &v, const std::vector<double> &w, const double missval)
 {
-  assert(array != nullptr);
-  assert(w != nullptr);
+  assert(v.size() >= len);
+  assert(w.size() >= len);
 
-  double missval1 = missval, missval2 = missval;
+  const double missval1 = missval, missval2 = missval;
   double sum = 0, sumw = 0;
 
   for (size_t i = 0; i < len; ++i)
     if (!DBL_IS_EQUAL(w[i], missval))
       {
-        sum = ADDMN(sum, MULMN(w[i], array[i]));
+        sum = ADDMN(sum, MULMN(w[i], v[i]));
         sumw = ADDMN(sumw, w[i]);
       }
 
