@@ -1,12 +1,12 @@
 #include "cdo_int.h"
 
-int cdoTaxisCreate(int taxisType)
+int
+cdoTaxisCreate(int taxisType)
 {
   if (CdoDefault::TaxisType != CDI_UNDEFID) taxisType = CdoDefault::TaxisType;
   return taxisCreate(taxisType);
 }
 
-
 void
 cdoSetNAN(double missval, size_t gridsize, double *array)
 {
@@ -17,4 +17,3 @@ cdoSetNAN(double missval, size_t gridsize, double *array)
         if (DBL_IS_EQUAL(array[i], missval)) array[i] = newmissval;
     }
 }
-
