@@ -17,6 +17,7 @@
 
 #include <cdi.h>
 #include "cdo_options.h"
+#include "cdo_output.h"
 #include "config.h"
 
 #include <cstring>
@@ -27,10 +28,11 @@ const char *progname;
 char File_Suffix[32];
 const char *Version = "Climate Data Operators version " VERSION " (http://mpimet.mpg.de/cdo)";
 const char *Username;
-}
+}  // namespace Cdo
 
 namespace Options
 {
+int numStreamWorker;
 bool benchmark = false;
 bool silentMode = false;
 
@@ -44,8 +46,8 @@ bool Timer = false;
 
 bool CheckDatarange = false;
 
-int CDO_flt_digits = 7; //TODO:rename
-int CDO_dbl_digits = 15;// TODO:rename
+int CDO_flt_digits = 7;   // TODO:rename
+int CDO_dbl_digits = 15;  // TODO:rename
 
 bool Use_FFTW = true;
 bool VersionInfo = true;
@@ -69,7 +71,6 @@ char **cdoVarnames = nullptr;
 
 bool REMAP_genweights = true;
 
-
 const char *cdoExpName = nullptr;
 }  // namespace Options
 
@@ -97,4 +98,37 @@ cdoComment(void)
   return comment;
 }
 
+void
+set_comp(int fileID, int filetype)
+{
+  if (Options::cdoCompress)
+    {
+      if (filetype == CDI_FILETYPE_GRB)
+        {
+          Options::cdoCompType = CDI_COMPRESS_SZIP;
+          Options::cdoCompLevel = 0;
+        }
+      else if (filetype == CDI_FILETYPE_NC4 || filetype == CDI_FILETYPE_NC4C)
+        {
+          Options::cdoCompType = CDI_COMPRESS_ZIP;
+          Options::cdoCompLevel = 1;
+        }
+    }
 
+  if (Options::cdoCompType != CDI_COMPRESS_NONE)
+    {
+      streamDefCompType(fileID, Options::cdoCompType);
+      streamDefCompLevel(fileID, Options::cdoCompLevel);
+
+      if (Options::cdoCompType == CDI_COMPRESS_SZIP
+          && (filetype != CDI_FILETYPE_GRB && filetype != CDI_FILETYPE_GRB2 && filetype != CDI_FILETYPE_NC4
+              && filetype != CDI_FILETYPE_NC4C))
+        cdoWarning("SZIP compression not available for non GRIB/NetCDF4 data!");
+
+      if (Options::cdoCompType == CDI_COMPRESS_JPEG && filetype != CDI_FILETYPE_GRB2)
+        cdoWarning("JPEG compression not available for non GRIB2 data!");
+
+      if (Options::cdoCompType == CDI_COMPRESS_ZIP && (filetype != CDI_FILETYPE_NC4 && filetype != CDI_FILETYPE_NC4C))
+        cdoWarning("Deflate compression not available for non NetCDF4 data!");
+    }
+}
