@@ -23,10 +23,11 @@ extern const char *progname;
 extern char File_Suffix[32];  // refactor: added keyword extern
 extern const char *Version;
 extern const char *Username;
-}
+}  // namespace Cdo
 
 namespace Options
 {
+extern int numStreamWorker;
 extern bool benchmark;
 extern bool silentMode;
 
@@ -51,7 +52,7 @@ extern bool cdoDiag;
 
 extern int CDO_Memtype;
 #define MEMTYPE_DOUBLE 1
-#define MEMTYPE_FLOAT  2
+#define MEMTYPE_FLOAT 2
 extern bool CDO_Parallel_Read;
 
 extern int CDO_Reduce_Dim;
@@ -76,4 +77,8 @@ extern bool cdoLockIO;
 }  // namespace Threading
 
 const char *cdoComment(void);
+
+void
+set_comp(int fileID, int filetype);
+
 #endif
