@@ -16,10 +16,12 @@
 */
 #include <cdi.h>
 #include "cdo_options.h"
-#include "pstream_int.h"
-#include "exception.h"
+#include <vector>
+
+#include "cdo_output.h"
 #include "dmemory.h"
 #include "compare.h"
+#include "cdi_lockedIO.h"
 
 bool *
 cdo_read_timestepmask(const char *maskfile, int *n)
