@@ -24,8 +24,9 @@
 #include "cdo_vlist.h"
 #include "dmemory.h"
 #include "compare.h"
-#include "exception.h"
+#include "cdo_output.h"
 #include "functs.h"
+
 char *
 cdoVlistInqVarName(int vlistID, int varID, char *name)
 {
@@ -37,7 +38,8 @@ double
 cdoZaxisInqLevel(int zaxisID, int levelID)
 {
   int zaxistype = zaxisInqType(zaxisID);
-  double level = zaxisInqLevels(zaxisID, nullptr) ? zaxisInqLevel(zaxisID, levelID) : (zaxistype == ZAXIS_SURFACE) ? 0 : levelID + 1;
+  double level
+      = zaxisInqLevels(zaxisID, nullptr) ? zaxisInqLevel(zaxisID, levelID) : (zaxistype == ZAXIS_SURFACE) ? 0 : levelID + 1;
   return level;
 }
 
@@ -116,8 +118,8 @@ compare_lon_reg2d(size_t xsize, int gridID1, int gridID2)
 static void
 compare_grid_unstructured(int gridID1, int gridID2)
 {
-  if (gridInqXvals(gridID1, nullptr) && gridInqXvals(gridID1, nullptr) == gridInqXvals(gridID2, nullptr) && gridInqYvals(gridID1, nullptr)
-      && gridInqYvals(gridID1, nullptr) == gridInqYvals(gridID2, nullptr))
+  if (gridInqXvals(gridID1, nullptr) && gridInqXvals(gridID1, nullptr) == gridInqXvals(gridID2, nullptr)
+      && gridInqYvals(gridID1, nullptr) && gridInqYvals(gridID1, nullptr) == gridInqYvals(gridID2, nullptr))
     {
       size_t gridsize = gridInqSize(gridID1);
       std::vector<double> xvals1(gridsize);
@@ -183,8 +185,8 @@ vlistCompareNames(int vlistID1, int varID1, int vlistID2, int varID2)
   char name1[CDI_MAX_NAME], name2[CDI_MAX_NAME];
   vlistInqVarName(vlistID1, varID1, name1);
   vlistInqVarName(vlistID2, varID2, name2);
-  strtolower(name1);
-  strtolower(name2);
+  cstrToLowerCase(name1);
+  cstrToLowerCase(name2);
   return strcmp(name1, name2);
 }
 
@@ -237,8 +239,8 @@ vlistCheckNames(int vlistID1, int vlistID2)
   int varID;
   int nvars = vlistNvars(vlistID1);
 
-  //std::vector<std::array<char,CDI_MAX_NAME>> names1(nvars); C++14?
-  //std::vector<std::array<char,CDI_MAX_NAME>> names2(nvars);
+  // std::vector<std::array<char,CDI_MAX_NAME>> names1(nvars); C++14?
+  // std::vector<std::array<char,CDI_MAX_NAME>> names2(nvars);
   std::vector<std::vector<char>> names1(nvars);
   std::vector<std::vector<char>> names2(nvars);
   for (varID = 0; varID < nvars; varID++) names1[varID].resize(CDI_MAX_NAME);
@@ -362,7 +364,7 @@ vlistMap(int vlistID1, int vlistID2, int flag, int mapflag, std::map<int, int> &
         {
           for (varID1 = 0; varID1 < nvars1; varID1++)
             {
-              if (cstrIsEqual(names1[varID1].data(), names2[varID2].data())) break;          
+              if (cstrIsEqual(names1[varID1].data(), names2[varID2].data())) break;
             }
           if (varID1 == nvars1)
             {
@@ -380,7 +382,7 @@ vlistMap(int vlistID1, int vlistID2, int flag, int mapflag, std::map<int, int> &
         {
           for (varID2 = 0; varID2 < nvars2; varID2++)
             {
-              if (cstrIsEqual(names1[varID1].data(), names2[varID2].data())) break;          
+              if (cstrIsEqual(names1[varID1].data(), names2[varID2].data())) break;
             }
           if (varID2 == nvars2)
             {
@@ -465,12 +467,9 @@ vlistIsSzipped(int vlistID)
 int
 vlistInqNWPV(int vlistID, int varID)
 {
-  int nwpv;  // number of words per value; real:1  complex:2
-
-  if (vlistInqVarDatatype(vlistID, varID) == CDI_DATATYPE_CPX32 || vlistInqVarDatatype(vlistID, varID) == CDI_DATATYPE_CPX64)
-    nwpv = 2;
-  else
-    nwpv = 1;
+  int datatype = vlistInqVarDatatype(vlistID, varID);
+  // number of words per value; real:1  complex:2
+  int nwpv = (datatype == CDI_DATATYPE_CPX32 || datatype == CDI_DATATYPE_CPX64) ? 2 : 1;
 
   return nwpv;
 }
@@ -479,14 +478,13 @@ size_t
 vlist_check_gridsize(int vlistID)
 {
   bool lerror = false;
-  int ngrids = vlistNgrids(vlistID);
-  int gridID = vlistGrid(vlistID, 0);
-  size_t ngp = gridInqSize(gridID);
+  const size_t ngp = gridInqSize(vlistGrid(vlistID, 0));
 
-  /* check gridsize */
+  // check gridsize
+  const int ngrids = vlistNgrids(vlistID);
   for (int index = 0; index < ngrids; ++index)
     {
-      gridID = vlistGrid(vlistID, index);
+      int gridID = vlistGrid(vlistID, index);
       if (ngp != gridInqSize(gridID))
         {
           lerror = true;
@@ -500,7 +498,7 @@ vlist_check_gridsize(int vlistID)
       cdoPrint("Horizontal grids found:");
       for (int index = 0; index < ngrids; ++index)
         {
-          gridID = vlistGrid(vlistID, index);
+          int gridID = vlistGrid(vlistID, index);
           cdoPrint("  grid=%d  type=%s  points=%zu", index + 1, gridNamePtr(gridInqType(gridID)), gridInqSize(gridID));
         }
       cdoAbort("The input stream contains variables on different horizontal grids!");
@@ -697,3 +695,21 @@ vlist_get_psvarid(int vlistID, int zaxisID)
 
   return psvarid;
 }
+
+
+void varListInit(VarList &vl, int vlistID)
+{
+  const int nvars = vlistNvars(vlistID);
+  vl.resize(nvars);
+
+  for (int varID = 0; varID < nvars; ++varID)
+    {
+      vl[varID].gridID = vlistInqVarGrid(vlistID, varID);
+      vl[varID].zaxisID = vlistInqVarZaxis(vlistID, varID);
+      vl[varID].gridsize = gridInqSize(vl[varID].gridID);
+      vl[varID].nlevels = zaxisInqSize(vl[varID].zaxisID);
+      vl[varID].datatype = vlistInqVarDatatype(vlistID, varID);
+      vl[varID].missval = vlistInqVarMissval(vlistID, varID);
+      vl[varID].timetype = vlistInqVarTimetype(vlistID, varID);
+    }
+}
