@@ -17,6 +17,7 @@
 #ifndef CDO_VLIST_H
 #define CDO_VLIST_H
 
+#include <vector>
 #include <map>
 
 void vlistMap(int vlistID1, int vlistID2, int flag, int mapflag, std::map<int, int> &mapOfVarIDs);
@@ -30,5 +31,20 @@ int vlist_get_psvarid(int vlistID, int zaxisID);
 double *vlist_read_vct(int vlistID, int *rzaxisIDh, int *rnvct, int *rnhlev, int *rnhlevf, int *rnhlevh);
 void vlist_change_hybrid_zaxis(int vlistID1, int vlistID2, int zaxisID1, int zaxisID2);
 
-void cdoCompareGrids(int gridID1, int gridID2); //TODO: Check if this belongs here or if it should be in griddes.*
+void cdoCompareGrids(int gridID1, int gridID2);  // TODO: Check if this belongs here or if it should be in griddes.*
+
+struct ListVar
+{
+  int gridID;
+  int zaxisID;
+  size_t gridsize;
+  int nlevels;
+  int datatype;
+  double missval;
+  int timetype;
+};
+
+using VarList = std::vector<ListVar>;
+void varListInit(VarList &vl, int vlistID);
+
 #endif
