@@ -24,7 +24,7 @@
 #include "util_string.h"
 #include "parse_literals.h"
 #include "pmlist.h"
-#include "exception.h"
+#include "cdo_output.h"
 #include "compare.h"
 
 #define MAX_LINE_LEN 65536
@@ -356,13 +356,13 @@ zaxisFromName(const char *zaxisnameptr)
   size_t len;
 
   char *zaxisname = strdup(zaxisnameptr);
-  strtolower(zaxisname);
+  cstrToLowerCase(zaxisname);
 
   zaxis_t zaxis;
   zaxisInit(zaxis);
 
   const char *pline = zaxisname;
-  if (cmpstr(pline, "surface") == 0) /* surface */
+  if (cmpstr(pline, "surface") == 0) // surface
     {
       zaxis.type = ZAXIS_SURFACE;
       zaxis.size = 1;
