@@ -22,7 +22,6 @@
 #include <cdi.h>
 #include "cdo_int.h"
 
-
 #define TO_KELVIN(x) ((x) + 273.15)
 #define MISSVAL -9.0E33
 
@@ -52,7 +51,8 @@ static void
 readNcFile(const char path[], double **vars, int nvars, int nts)
 {
   int taxisID;
-  int vlistID, varID, streamID, tsID;
+  int vlistID, varID, tsID;
+  CdiStreamID streamID;
   size_t nmiss;
   int nrecs;
 
