@@ -25,20 +25,9 @@
 #include <cmath>
 #include <string.h>
 
-#ifdef __xlC__ /* performance problem on IBM */
 #ifndef DBL_IS_NAN
-#define DBL_IS_NAN(x) ((x) != (x))
-#endif
-#else
-#ifndef DBL_IS_NAN
-#if defined(HAVE_DECL_ISNAN) && !defined(DISABLE_ISNAN)
 #define DBL_IS_NAN(x) (std::isnan(x))
-#elif defined(FP_NAN)
-#define DBL_IS_NAN(x) (std::fpclassify(x) == FP_NAN)
-#else
-#define DBL_IS_NAN(x) ((x) != (x))
-#endif
-#endif
+//#define DBL_IS_NAN(x) ((x) != (x))
 #endif
 
 #ifndef DBL_IS_EQUAL
@@ -54,8 +43,8 @@
 #define cmpstr(s1, s2) (strncmp(s1, s2, strlen(s2)))
 #define cmpstrlen(s1, s2, len) (strncmp(s1, s2, len = strlen(s2)))
 
-inline bool
-cstrIsEqual(const char *x, const char *y) 
+static inline bool
+cstrIsEqual(const char *x, const char *y)
 {
   return (*x == *y) && strcmp(x, y) == 0;
 }
