@@ -53,7 +53,7 @@ getTimestatDate(TimeStat *tstat_date)
 
       memcpy(envstrl, envstr, 8);
       envstrl[7] = 0;
-      strtolower(envstrl);
+      cstrToLowerCase(envstrl);
 
       // clang-format off
       if      (memcmp(envstrl, "first", 5) == 0)   env_date = TimeStat::FIRST;
@@ -76,28 +76,33 @@ DateTimeList::init()
   if (!dateTimeInit) getTimestatDate(&CDO_Timestat_Date);
   dateTimeInit = true;
 }
-int64_t DateTimeList::getVdate(int tsID)
+
+int64_t
+DateTimeList::getVdate(int tsID)
 {
   if (tsID < 0 || (size_t) tsID >= this->size) cdoAbort("Internal error; tsID out of bounds!");
 
   return this->dtinfo[tsID].c.date;
-} 
+}
 
-int DateTimeList::getVtime(int tsID)
+int
+DateTimeList::getVtime(int tsID)
 {
   if (tsID < 0 || (size_t) tsID >= this->size) cdoAbort("Internal error; tsID out of bounds!");
-  
+
   return this->dtinfo[tsID].c.time;
 }
 
- void DateTimeList::shift()
+void
+DateTimeList::shift()
 {
   for (size_t inp = 0; inp < this->size - 1; inp++) this->dtinfo[inp] = this->dtinfo[inp + 1];
 }
+
 void
 DateTimeList::taxisInqTimestep(int taxisID, int tsID)
 {
-  size_t NALLOC = 128;
+  constexpr size_t NALLOC = 128;
 
   if ((size_t) tsID >= this->nalloc)
     {
@@ -135,7 +140,7 @@ DateTimeList::taxisInqTimestep(int taxisID, int tsID)
       if (CDO_Timestat_Bounds && this->dtinfo[tsID].v.date == this->dtinfo[tsID].b[1].date
           && this->dtinfo[tsID].v.time == this->dtinfo[tsID].b[1].time)
         {
-          int calendar = this->calendar;
+          const int calendar = this->calendar;
 
           int64_t vdate = this->dtinfo[tsID].b[0].date;
           int vtime = this->dtinfo[tsID].b[0].time;
