@@ -21,7 +21,6 @@
 
 #include "cdo_int.h"
 #include "cdo_math.h"
-#include "pstream_int.h"
 #include "timebase.h"
 #include "ecautil.h"
 
@@ -34,7 +33,7 @@ static unsigned long
 gregdate_to_julday(int year,  /* Gregorian year */
                    int month, /* Gregorian month (1-12) */
                    int day    /* Gregorian day (1-31) */
-                   )
+)
 {
 #if INT_MAX <= 0X7FFF
   long igreg = 15 + 31 * (10 + (12 * 1582));
@@ -107,7 +106,7 @@ gregdate_to_julday(int year,  /* Gregorian year */
  *
  * @return the day-of-year
  */
-unsigned long
+static unsigned long
 day_of_year(int64_t date)
 {
   const int year = date / 10000;
@@ -145,22 +144,17 @@ day_of_year(int64_t date)
 static void
 count(Field &field1, const Field &field2, double mode)
 {
-  size_t i;
-  const int grid1 = field1.grid;
-  const size_t nmiss1 = field1.nmiss;
   const double missval1 = field1.missval;
-  double *array1 = field1.ptr;
-  const int grid2 = field2.grid;
   const double missval2 = field2.missval;
-  const double *array2 = field2.ptr;
+  auto &array1 = field1.vec;
+  const auto &array2 = field2.vec;
 
-  size_t len = gridInqSize(grid1);
+  size_t len = field1.size;
+  if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
 
-  if (len != gridInqSize(grid2)) cdoAbort("Fields have different gridsize (%s)", __func__);
-
-  if (nmiss1 > 0)
+  if (field1.nmiss)
     {
-      for (i = 0; i < len; i++)
+      for (size_t i = 0; i < len; i++)
         {
           if (DBL_IS_EQUAL(array2[i], missval2))
             {
@@ -186,11 +180,11 @@ count(Field &field1, const Field &field2, double mode)
             }
         }
 
-      field1.nmiss = arrayNumMV(len, array1, missval1);
+      field1.nmiss = fieldNumMiss(field1);
     }
   else
     {
-      for (i = 0; i < len; i++)
+      for (size_t i = 0; i < len; i++)
         {
           if (DBL_IS_EQUAL(array2[i], missval2))
             {
@@ -224,33 +218,27 @@ count(Field &field1, const Field &field2, double mode)
 static void
 selcomp(Field &field1, const Field &field2, int (*compare)(double, double))
 {
-  size_t i;
-  const int grid1 = field1.grid;
-  const size_t nmiss1 = field1.nmiss;
   const double missval1 = field1.missval;
-  double *array1 = field1.ptr;
-  const int grid2 = field2.grid;
-  const size_t nmiss2 = field2.nmiss;
   const double missval2 = field2.missval;
-  const double *array2 = field2.ptr;
+  auto &array1 = field1.vec;
+  const auto &array2 = field2.vec;
 
-  size_t len = gridInqSize(grid1);
+  size_t len = field1.size;
+  if (len != field2.size) cdoAbort("Fields have different size (%s)", __func__);
 
-  if (len != gridInqSize(grid2)) cdoAbort("Fields have different gridsize (%s)", __func__);
-
-  if (nmiss1 > 0 || nmiss2 > 0)
+  if (field1.nmiss || field2.nmiss)
     {
-      for (i = 0; i < len; i++)
+      for (size_t i = 0; i < len; i++)
         if (DBL_IS_EQUAL(array1[i], missval1) || DBL_IS_EQUAL(array2[i], missval2) || !compare(array1[i], array2[i]))
           array1[i] = missval1;
     }
   else
     {
-      for (i = 0; i < len; i++)
+      for (size_t i = 0; i < len; i++)
         if (!compare(array1[i], array2[i])) array1[i] = missval1;
     }
 
-  field1.nmiss = arrayNumMV(len, array1, missval1);
+  field1.nmiss = fieldNumMiss(field1);
 }
 
 /**
@@ -271,30 +259,27 @@ selcomp(Field &field1, const Field &field2, int (*compare)(double, double))
 static void
 selcompc(Field &field, double c, int (*compare)(double, double))
 {
-  size_t i;
-  const int grid = field.grid;
-  const size_t nmiss = field.nmiss;
   const double missval = field.missval;
-  double *array = field.ptr;
+  auto &array = field.vec;
 
-  size_t len = gridInqSize(grid);
+  size_t len = field.size;
 
   if (DBL_IS_EQUAL(c, missval))
     {
-      for (i = 0; i < len; i++) array[i] = missval;
+      for (size_t i = 0; i < len; i++) array[i] = missval;
     }
-  else if (nmiss > 0)
+  else if (field.nmiss)
     {
-      for (i = 0; i < len; i++)
+      for (size_t i = 0; i < len; i++)
         if (DBL_IS_EQUAL(array[i], missval) || !compare(array[i], c)) array[i] = missval;
     }
   else
     {
-      for (i = 0; i < len; i++)
+      for (size_t i = 0; i < len; i++)
         if (!compare(array[i], c)) array[i] = missval;
     }
 
-  field.nmiss = arrayNumMV(len, array, missval);
+  field.nmiss = fieldNumMiss(field);
 }
 
 static int
@@ -334,145 +319,138 @@ ne(double a, double b)
 }
 
 void
-farnum(Field &field1, const Field &field2)
+vfarnum(Field &field1, const Field &field2)
 {
   count(field1, field2, 0.0);
 }
 
 void
-farnum2(Field &field1, const Field &field2)
+vfarnum2(Field &field1, const Field &field2)
 {
   count(field1, field2, 1.0);
 }
 
 void
-farnum3(Field &field1, const Field &field2, double n)
+vfarnum3(Field &field1, const Field &field2, double n)
 {
   count(field1, field2, n);
 }
 
 void
-farsel(Field &field1, const Field &field2)
+vfarsel(Field &field1, const Field &field2)
 {
-  size_t i;
-  const int grid1 = field1.grid;
   const double missval1 = field1.missval;
-  double *array1 = field1.ptr;
-  const int grid2 = field2.grid;
-  const size_t nmiss2 = field2.nmiss;
   const double missval2 = field2.missval;
-  const double *array2 = field2.ptr;
+  auto &array1 = field1.vec;
+  const auto &array2 = field2.vec;
 
-  size_t len = gridInqSize(grid1);
+  const size_t len = field1.size;
+  if (len != field2.size) cdoAbort("Fields have different gridsize (%s)", __func__);
 
-  if (len != gridInqSize(grid2)) cdoAbort("Fields have different gridsize (%s)", __func__);
-
-  if (nmiss2 > 0)
+  if (field2.nmiss)
     {
-      for (i = 0; i < len; i++)
+      for (size_t i = 0; i < len; i++)
         if (DBL_IS_EQUAL(array2[i], missval2) || DBL_IS_EQUAL(array2[i], 0.0)) array1[i] = missval1;
     }
   else
     {
-      for (i = 0; i < len; i++)
+      for (size_t i = 0; i < len; i++)
         if (IS_EQUAL(array2[i], 0.0)) array1[i] = missval1;
     }
 
-  field1.nmiss = arrayNumMV(len, array1, missval1);
+  field1.nmiss = fieldNumMiss(field1);
 }
 
 void
-farselle(Field &field1, const Field &field2)
+vfarselle(Field &field1, const Field &field2)
 {
   selcomp(field1, field2, le);
 }
 
 void
-farsellt(Field &field1, const Field &field2)
+vfarsellt(Field &field1, const Field &field2)
 {
   selcomp(field1, field2, lt);
 }
 
 void
-farselge(Field &field1, const Field &field2)
+vfarselge(Field &field1, const Field &field2)
 {
   selcomp(field1, field2, ge);
 }
 
 void
-farselgt(Field &field1, const Field &field2)
+vfarselgt(Field &field1, const Field &field2)
 {
   selcomp(field1, field2, gt);
 }
 
 void
-farseleq(Field &field1, const Field &field2)
+vfarseleq(Field &field1, const Field &field2)
 {
   selcomp(field1, field2, eq);
 }
 
 void
-farselne(Field &field1, const Field &field2)
+vfarselne(Field &field1, const Field &field2)
 {
   selcomp(field1, field2, ne);
 }
 
 void
-farsellec(Field &field, double c)
+vfarsellec(Field &field, double c)
 {
   selcompc(field, c, le);
 }
 
 void
-farselltc(Field &field, double c)
+vfarselltc(Field &field, double c)
 {
   selcompc(field, c, lt);
 }
 
 void
-farselgec(Field &field, double c)
+vfarselgec(Field &field, double c)
 {
   selcompc(field, c, ge);
 }
 
 void
-farseleqc(Field &field, double c)
+vfarseleqc(Field &field, double c)
 {
   selcompc(field, c, eq);
 }
 
 void
-farselnec(Field &field, double c)
+vfarselnec(Field &field, double c)
 {
   selcompc(field, c, ne);
 }
 
 void
-farselgtc(Field &field, double c)
+vfarselgtc(Field &field, double c)
 {
   selcompc(field, c, gt);
 }
 
 void
-updateHist(Field *field[2], int nlevels, size_t gridsize, double *yvals, bool onlyNorth)
+updateHist(FieldVector2D &field, int nlevels, size_t gridsize, std::vector<double> &yvals, bool onlyNorth)
 {
-  int levelID;
-
-  for (levelID = 0; levelID < nlevels; levelID++)
+  for (int levelID = 0; levelID < nlevels; levelID++)
     for (size_t i = 0; i < gridsize; i++)
       if (onlyNorth)
         {
-          if (yvals[i] >= 0.0) field[1][levelID].ptr[i] = field[0][levelID].ptr[i];
+          if (yvals[i] >= 0.0) field[1][levelID].vec[i] = field[0][levelID].vec[i];
         }
       else
-        field[1][levelID].ptr[i] = field[0][levelID].ptr[i];
+        field[1][levelID].vec[i] = field[0][levelID].vec[i];
 }
 
 void
-adjustEndDate(int nlevels, size_t gridsize, double *yvals, double missval, int64_t ovdate, Field *startDateWithHist[2],
-              Field *endDateWithHist[2])
+adjustEndDate(int nlevels, size_t gridsize, std::vector<double> &yvals, double missval, int64_t ovdate, FieldVector2D &startDateWithHist,
+              FieldVector2D &endDateWithHist)
 {
-  int64_t ovdateSouth = cdo::min(cdiEncodeDate(ovdate / 10000, 6, 30), ovdate);
+  const int64_t ovdateSouth = cdo::min(cdiEncodeDate(ovdate / 10000, 6, 30), ovdate);
 
   for (int levelID = 0; levelID < nlevels; levelID++)
     {
@@ -481,27 +459,27 @@ adjustEndDate(int nlevels, size_t gridsize, double *yvals, double missval, int64
           /* start with southern sphere */
           if (yvals[i] < 0)
             {
-              if (DBL_IS_EQUAL(startDateWithHist[1][levelID].ptr[i], missval))
+              if (DBL_IS_EQUAL(startDateWithHist[1][levelID].vec[i], missval))
                 {
-                  endDateWithHist[0][levelID].ptr[i] = missval;
+                  endDateWithHist[0][levelID].vec[i] = missval;
                   continue;
                 }
-              if (DBL_IS_EQUAL(endDateWithHist[0][levelID].ptr[i], missval))
+              if (DBL_IS_EQUAL(endDateWithHist[0][levelID].vec[i], missval))
                 {
-                  endDateWithHist[0][levelID].ptr[i] = ovdateSouth;
+                  endDateWithHist[0][levelID].vec[i] = ovdateSouth;
                 }
             }
           else
             {
-              if (DBL_IS_EQUAL(startDateWithHist[0][levelID].ptr[i], missval))
+              if (DBL_IS_EQUAL(startDateWithHist[0][levelID].vec[i], missval))
                 {
-                  endDateWithHist[0][levelID].ptr[i] = missval;
+                  endDateWithHist[0][levelID].vec[i] = missval;
                   continue;
                 }
 
-              if (DBL_IS_EQUAL(endDateWithHist[0][levelID].ptr[i], missval))
+              if (DBL_IS_EQUAL(endDateWithHist[0][levelID].vec[i], missval))
                 {
-                  endDateWithHist[0][levelID].ptr[i] = ovdate;
+                  endDateWithHist[0][levelID].vec[i] = ovdate;
                 }
             }
         }
@@ -509,8 +487,8 @@ adjustEndDate(int nlevels, size_t gridsize, double *yvals, double missval, int64
 }
 
 void
-computeGsl(int nlevels, size_t gridsize, double *yvals, double missval, Field *startDateWithHist[2], Field *endDateWithHist[2],
-           Field *gslDuration, Field *gslFirstDay, bool useCurrentYear)
+computeGsl(int nlevels, size_t gridsize, std::vector<double> &yvals, double missval, FieldVector2D &startDateWithHist, FieldVector2D &endDateWithHist,
+           FieldVector &gslDuration, FieldVector &gslFirstDay, bool useCurrentYear)
 {
   int levelID;
   double firstDay, duration;
@@ -524,22 +502,22 @@ computeGsl(int nlevels, size_t gridsize, double *yvals, double missval, Field *s
               /* start with southern sphere */
               if (yvals[i] < 0.0)
                 {
-                  duration = (double) (date_to_julday(CALENDAR_PROLEPTIC, (int64_t) endDateWithHist[0][levelID].ptr[i])
-                                       - date_to_julday(CALENDAR_PROLEPTIC, (int64_t) startDateWithHist[1][levelID].ptr[i]));
+                  duration = (double) (date_to_julday(CALENDAR_PROLEPTIC, (int64_t) endDateWithHist[0][levelID].vec[i])
+                                       - date_to_julday(CALENDAR_PROLEPTIC, (int64_t) startDateWithHist[1][levelID].vec[i]));
                 }
               else
                 {
-                  duration = (double) (date_to_julday(CALENDAR_PROLEPTIC, (int64_t) endDateWithHist[1][levelID].ptr[i])
-                                       - date_to_julday(CALENDAR_PROLEPTIC, (int64_t) startDateWithHist[1][levelID].ptr[i]));
+                  duration = (double) (date_to_julday(CALENDAR_PROLEPTIC, (int64_t) endDateWithHist[1][levelID].vec[i])
+                                       - date_to_julday(CALENDAR_PROLEPTIC, (int64_t) startDateWithHist[1][levelID].vec[i]));
                 }
 
-              if (DBL_IS_EQUAL(startDateWithHist[1][levelID].ptr[i], missval))
+              if (DBL_IS_EQUAL(startDateWithHist[1][levelID].vec[i], missval))
                 firstDay = missval;
               else
-                firstDay = (double) day_of_year((int64_t) startDateWithHist[1][levelID].ptr[i]);
+                firstDay = (double) day_of_year((int64_t) startDateWithHist[1][levelID].vec[i]);
 
-              gslDuration[levelID].ptr[i] = duration;
-              gslFirstDay[levelID].ptr[i] = firstDay;
+              gslDuration[levelID].vec[i] = duration;
+              gslFirstDay[levelID].vec[i] = firstDay;
             }
         }
     }
@@ -553,21 +531,21 @@ computeGsl(int nlevels, size_t gridsize, double *yvals, double missval, Field *s
               /* start with southern sphere */
               if (yvals[i] < 0.0)
                 {
-                  gslDuration[levelID].ptr[i] = missval;
-                  gslFirstDay[levelID].ptr[i] = missval;
+                  gslDuration[levelID].vec[i] = missval;
+                  gslFirstDay[levelID].vec[i] = missval;
                 }
               else
                 {
-                  duration = (double) (date_to_julday(CALENDAR_PROLEPTIC, (int64_t) endDateWithHist[0][levelID].ptr[i])
-                                       - date_to_julday(CALENDAR_PROLEPTIC, (int64_t) startDateWithHist[0][levelID].ptr[i]));
+                  duration = (double) (date_to_julday(CALENDAR_PROLEPTIC, (int64_t) endDateWithHist[0][levelID].vec[i])
+                                       - date_to_julday(CALENDAR_PROLEPTIC, (int64_t) startDateWithHist[0][levelID].vec[i]));
 
-                  if (DBL_IS_EQUAL(startDateWithHist[0][levelID].ptr[i], missval))
+                  if (DBL_IS_EQUAL(startDateWithHist[0][levelID].vec[i], missval))
                     firstDay = missval;
                   else
-                    firstDay = (double) day_of_year((int64_t) startDateWithHist[0][levelID].ptr[i]);
+                    firstDay = (double) day_of_year((int64_t) startDateWithHist[0][levelID].vec[i]);
 
-                  gslDuration[levelID].ptr[i] = duration;
-                  gslFirstDay[levelID].ptr[i] = firstDay;
+                  gslDuration[levelID].vec[i] = duration;
+                  gslFirstDay[levelID].vec[i] = firstDay;
                 }
             }
         }
@@ -575,30 +553,30 @@ computeGsl(int nlevels, size_t gridsize, double *yvals, double missval, Field *s
 
   for (levelID = 0; levelID < nlevels; levelID++)
     {
-      gslDuration[levelID].nmiss = arrayNumMV(gridsize, gslDuration[levelID].ptr, missval);
-      gslFirstDay[levelID].nmiss = arrayNumMV(gridsize, gslFirstDay[levelID].ptr, missval);
+      gslDuration[levelID].nmiss = fieldNumMiss(gslDuration[levelID]);
+      gslFirstDay[levelID].nmiss = fieldNumMiss(gslFirstDay[levelID]);
     }
 }
 
 void
-writeGslStream(int ostreamID, int otaxisID, int otsID, int ovarID1, int ovarID2, int ivlistID1, int first_var_id,
-               Field *gslDuration, Field *gslFirstDay, int64_t vdate, int vtime, int nlevels)
+writeGslStream(CdoStreamID ostreamID, int otaxisID, int otsID, int ovarID1, int ovarID2, int ivlistID1, int first_var_id,
+               FieldVector &gslDuration, FieldVector &gslFirstDay, int64_t vdate, int vtime, int nlevels)
 {
   (void) ivlistID1;
   (void) first_var_id;
 
   taxisDefVdate(otaxisID, vdate);
   taxisDefVtime(otaxisID, vtime);
-  pstreamDefTimestep(ostreamID, otsID);
+  cdoDefTimestep(ostreamID, otsID);
 
   for (int levelID = 0; levelID < nlevels; levelID++)
     {
-      pstreamDefRecord(ostreamID, ovarID1, levelID);
-      pstreamWriteRecord(ostreamID, gslDuration[levelID].ptr, gslDuration[levelID].nmiss);
+      cdoDefRecord(ostreamID, ovarID1, levelID);
+      cdoWriteRecord(ostreamID, gslDuration[levelID].vec.data(), gslDuration[levelID].nmiss);
     }
   for (int levelID = 0; levelID < nlevels; levelID++)
     {
-      pstreamDefRecord(ostreamID, ovarID2, levelID);
-      pstreamWriteRecord(ostreamID, gslFirstDay[levelID].ptr, gslFirstDay[levelID].nmiss);
+      cdoDefRecord(ostreamID, ovarID2, levelID);
+      cdoWriteRecord(ostreamID, gslFirstDay[levelID].vec.data(), gslFirstDay[levelID].nmiss);
     }
 }
