@@ -91,79 +91,79 @@ struct func_t
 {
   int type;
   int flag;
-  const char *name;  // function name
+  const char *name;    // function name
   void (*func)(void);  // pointer to function
 };
 
 static func_t fun_sym_tbl[] = {
   // scalar functions
-  { FT_STD, 0, "abs",   (void(*)(void)) ((double(*)(double)) fabs) }, // math functions could be inlined
-  { FT_STD, 0, "floor", (void(*)(void)) ((double(*)(double)) floor) },
-  { FT_STD, 0, "ceil", (void(*)(void)) ((double(*)(double)) ceil) },
-  { FT_STD, 0, "sqrt", (void(*)(void)) ((double(*)(double)) sqrt) },
-  { FT_STD, 0, "exp", (void(*)(void)) ((double(*)(double)) exp) },
-  { FT_STD, 0, "erf", (void(*)(void)) ((double(*)(double)) erf) },
-  { FT_STD, 0, "log", (void(*)(void)) ((double(*)(double)) log) },
-  { FT_STD, 0, "ln", (void(*)(void)) ((double(*)(double)) log) },
-  { FT_STD, 0, "log10", (void(*)(void)) ((double(*)(double)) log10) },
-  { FT_STD, 0, "sin", (void(*)(void)) ((double(*)(double)) sin) },
-  { FT_STD, 0, "cos", (void(*)(void)) ((double(*)(double)) cos) },
-  { FT_STD, 0, "tan", (void(*)(void)) ((double(*)(double)) tan) },
-  { FT_STD, 0, "sinh", (void(*)(void)) ((double(*)(double)) sinh) },
-  { FT_STD, 0, "cosh", (void(*)(void)) ((double(*)(double)) cosh) },
-  { FT_STD, 0, "tanh", (void(*)(void)) ((double(*)(double)) tanh) },
-  { FT_STD, 0, "asin", (void(*)(void)) ((double(*)(double)) asin) },
-  { FT_STD, 0, "acos", (void(*)(void)) ((double(*)(double)) acos) },
-  { FT_STD, 0, "atan", (void(*)(void)) ((double(*)(double)) atan) },
-  { FT_STD, 0, "asinh", (void(*)(void)) ((double(*)(double)) asinh) },
-  { FT_STD, 0, "acosh", (void(*)(void)) ((double(*)(double)) acosh) },
-  { FT_STD, 0, "atanh", (void(*)(void)) ((double(*)(double)) atanh) },
-  { FT_STD, 0, "gamma", (void(*)(void)) ((double(*)(double)) tgamma) },
-  { FT_STD, 0, "float", reinterpret_cast<void(*)(void)> (&f_float) },
-  { FT_STD, 0, "int", reinterpret_cast<void(*)(void)> (&f_int) },
-  { FT_STD, 0, "nint", reinterpret_cast<void(*)(void)> (&f_nint) },
-  { FT_STD, 0, "sqr", reinterpret_cast<void(*)(void)> (&f_sqr) },
-  { FT_STD, 0, "rad", reinterpret_cast<void(*)(void)> (&f_rad) },
-  { FT_STD, 0, "deg", reinterpret_cast<void(*)(void)> (&f_deg) },
+  { FT_STD, 0, "abs", (void (*)(void))((double (*)(double)) fabs) },  // math functions could be inlined
+  { FT_STD, 0, "floor", (void (*)(void))((double (*)(double)) floor) },
+  { FT_STD, 0, "ceil", (void (*)(void))((double (*)(double)) ceil) },
+  { FT_STD, 0, "sqrt", (void (*)(void))((double (*)(double)) sqrt) },
+  { FT_STD, 0, "exp", (void (*)(void))((double (*)(double)) exp) },
+  { FT_STD, 0, "erf", (void (*)(void))((double (*)(double)) erf) },
+  { FT_STD, 0, "log", (void (*)(void))((double (*)(double)) log) },
+  { FT_STD, 0, "ln", (void (*)(void))((double (*)(double)) log) },
+  { FT_STD, 0, "log10", (void (*)(void))((double (*)(double)) log10) },
+  { FT_STD, 0, "sin", (void (*)(void))((double (*)(double)) sin) },
+  { FT_STD, 0, "cos", (void (*)(void))((double (*)(double)) cos) },
+  { FT_STD, 0, "tan", (void (*)(void))((double (*)(double)) tan) },
+  { FT_STD, 0, "sinh", (void (*)(void))((double (*)(double)) sinh) },
+  { FT_STD, 0, "cosh", (void (*)(void))((double (*)(double)) cosh) },
+  { FT_STD, 0, "tanh", (void (*)(void))((double (*)(double)) tanh) },
+  { FT_STD, 0, "asin", (void (*)(void))((double (*)(double)) asin) },
+  { FT_STD, 0, "acos", (void (*)(void))((double (*)(double)) acos) },
+  { FT_STD, 0, "atan", (void (*)(void))((double (*)(double)) atan) },
+  { FT_STD, 0, "asinh", (void (*)(void))((double (*)(double)) asinh) },
+  { FT_STD, 0, "acosh", (void (*)(void))((double (*)(double)) acosh) },
+  { FT_STD, 0, "atanh", (void (*)(void))((double (*)(double)) atanh) },
+  { FT_STD, 0, "gamma", (void (*)(void))((double (*)(double)) tgamma) },
+  { FT_STD, 0, "float", reinterpret_cast<void (*)(void)>(&f_float) },
+  { FT_STD, 0, "int", reinterpret_cast<void (*)(void)>(&f_int) },
+  { FT_STD, 0, "nint", reinterpret_cast<void (*)(void)>(&f_nint) },
+  { FT_STD, 0, "sqr", reinterpret_cast<void (*)(void)>(&f_sqr) },
+  { FT_STD, 0, "rad", reinterpret_cast<void (*)(void)>(&f_rad) },
+  { FT_STD, 0, "deg", reinterpret_cast<void (*)(void)>(&f_deg) },
 
   // constant functions
-  { FT_CONST, 0, "ngp", reinterpret_cast<void(*)(void)> (&pt_ngp) },          // number of horizontal grid points
-  { FT_CONST, 0, "nlev", reinterpret_cast<void(*)(void)> (&pt_nlev) },        // number of vertical levels
-  { FT_CONST, 0, "size", reinterpret_cast<void(*)(void)> (&pt_size) },        // ngp*nlev
-  { FT_CONST, 0, "missval", reinterpret_cast<void(*)(void)> (&pt_missval) },  // Returns the missing value of a variable
+  { FT_CONST, 0, "ngp", reinterpret_cast<void (*)(void)>(&pt_ngp) },          // number of horizontal grid points
+  { FT_CONST, 0, "nlev", reinterpret_cast<void (*)(void)>(&pt_nlev) },        // number of vertical levels
+  { FT_CONST, 0, "size", reinterpret_cast<void (*)(void)>(&pt_size) },        // ngp*nlev
+  { FT_CONST, 0, "missval", reinterpret_cast<void (*)(void)>(&pt_missval) },  // Returns the missing value of a variable
 
   // CDO field functions (Reduce grid to point)
-  { FT_FLD, 0, "fldmin", reinterpret_cast<void(*)(void)> (&fldmin) },
-  { FT_FLD, 0, "fldmax", reinterpret_cast<void(*)(void)> (&fldmax) },
-  { FT_FLD, 0, "fldsum", reinterpret_cast<void(*)(void)> (&fldsum) },
-  { FT_FLD, 1, "fldmean", reinterpret_cast<void(*)(void)> (&fldmeanw) },
-  { FT_FLD, 1, "fldavg", reinterpret_cast<void(*)(void)> (&fldavgw) },
-  { FT_FLD, 1, "fldstd", reinterpret_cast<void(*)(void)> (&fldstdw) },
-  { FT_FLD, 1, "fldstd1", reinterpret_cast<void(*)(void)> (&fldstd1w) },
-  { FT_FLD, 1, "fldvar", reinterpret_cast<void(*)(void)> (&fldvarw) },
-  { FT_FLD, 1, "fldvar1", reinterpret_cast<void(*)(void)> (&fldvar1w) },
+  { FT_FLD, 0, "fldmin", reinterpret_cast<void (*)(void)>(&vfldmin) },
+  { FT_FLD, 0, "fldmax", reinterpret_cast<void (*)(void)>(&vfldmax) },
+  { FT_FLD, 0, "fldsum", reinterpret_cast<void (*)(void)>(&vfldsum) },
+  { FT_FLD, 1, "fldmean", reinterpret_cast<void (*)(void)>(&vfldmeanw) },
+  { FT_FLD, 1, "fldavg", reinterpret_cast<void (*)(void)>(&vfldavgw) },
+  { FT_FLD, 1, "fldstd", reinterpret_cast<void (*)(void)>(&vfldstdw) },
+  { FT_FLD, 1, "fldstd1", reinterpret_cast<void (*)(void)>(&vfldstd1w) },
+  { FT_FLD, 1, "fldvar", reinterpret_cast<void (*)(void)>(&vfldvarw) },
+  { FT_FLD, 1, "fldvar1", reinterpret_cast<void (*)(void)>(&vfldvar1w) },
 
   // CDO zonal functions (Reduce grid to point)
-  { FT_ZON, 0, "zonmin", reinterpret_cast<void(*)(void)> (&zonmin) },
-  { FT_ZON, 0, "zonmax", reinterpret_cast<void(*)(void)> (&zonmax) },
-  { FT_ZON, 0, "zonsum", reinterpret_cast<void(*)(void)> (&zonsum) },
-  { FT_ZON, 0, "zonmean", reinterpret_cast<void(*)(void)> (&zonmean) },
-  { FT_ZON, 0, "zonavg", reinterpret_cast<void(*)(void)> (&zonavg) },
-  { FT_ZON, 0, "zonstd", reinterpret_cast<void(*)(void)> (&zonstd) },
-  { FT_ZON, 0, "zonstd1", reinterpret_cast<void(*)(void)> (&zonstd1) },
-  { FT_ZON, 0, "zonvar", reinterpret_cast<void(*)(void)> (&zonvar) },
-  { FT_ZON, 0, "zonvar1", reinterpret_cast<void(*)(void)> (&zonvar1) },
+  { FT_ZON, 0, "zonmin", reinterpret_cast<void (*)(void)>(&zonmin) },
+  { FT_ZON, 0, "zonmax", reinterpret_cast<void (*)(void)>(&zonmax) },
+  { FT_ZON, 0, "zonsum", reinterpret_cast<void (*)(void)>(&zonsum) },
+  { FT_ZON, 0, "zonmean", reinterpret_cast<void (*)(void)>(&zonmean) },
+  { FT_ZON, 0, "zonavg", reinterpret_cast<void (*)(void)>(&zonavg) },
+  { FT_ZON, 0, "zonstd", reinterpret_cast<void (*)(void)>(&zonstd) },
+  { FT_ZON, 0, "zonstd1", reinterpret_cast<void (*)(void)>(&zonstd1) },
+  { FT_ZON, 0, "zonvar", reinterpret_cast<void (*)(void)>(&zonvar) },
+  { FT_ZON, 0, "zonvar1", reinterpret_cast<void (*)(void)>(&zonvar1) },
 
   // CDO field functions (Reduce level to point)
-  { FT_VERT, 0, "vertmin", reinterpret_cast<void(*)(void)> (&fldmin) },
-  { FT_VERT, 0, "vertmax", reinterpret_cast<void(*)(void)> (&fldmax) },
-  { FT_VERT, 0, "vertsum", reinterpret_cast<void(*)(void)> (&fldsum) },
-  { FT_VERT, 1, "vertmean", reinterpret_cast<void(*)(void)> (&fldmeanw) },
-  { FT_VERT, 1, "vertavg", reinterpret_cast<void(*)(void)> (&fldavgw) },
-  { FT_VERT, 1, "vertstd", reinterpret_cast<void(*)(void)> (&fldstdw) },
-  { FT_VERT, 1, "vertstd1", reinterpret_cast<void(*)(void)> (&fldstd1w) },
-  { FT_VERT, 1, "vertvar", reinterpret_cast<void(*)(void)> (&fldvarw) },
-  { FT_VERT, 1, "vertvar1", reinterpret_cast<void(*)(void)> (&fldvar1w) },
+  { FT_VERT, 0, "vertmin", reinterpret_cast<void (*)(void)>(&vfldmin) },
+  { FT_VERT, 0, "vertmax", reinterpret_cast<void (*)(void)>(&vfldmax) },
+  { FT_VERT, 0, "vertsum", reinterpret_cast<void (*)(void)>(&vfldsum) },
+  { FT_VERT, 1, "vertmean", reinterpret_cast<void (*)(void)>(&vfldmeanw) },
+  { FT_VERT, 1, "vertavg", reinterpret_cast<void (*)(void)>(&vfldavgw) },
+  { FT_VERT, 1, "vertstd", reinterpret_cast<void (*)(void)>(&vfldstdw) },
+  { FT_VERT, 1, "vertstd1", reinterpret_cast<void (*)(void)>(&vfldstd1w) },
+  { FT_VERT, 1, "vertvar", reinterpret_cast<void (*)(void)>(&vfldvarw) },
+  { FT_VERT, 1, "vertvar1", reinterpret_cast<void (*)(void)>(&vfldvar1w) },
 
   { FT_COORD, 0, "clon", nullptr },
   { FT_COORD, 0, "clat", nullptr },
@@ -171,16 +171,16 @@ static func_t fun_sym_tbl[] = {
   { FT_COORD, 0, "gridarea", nullptr },
   { FT_COORD, 0, "gridweight", nullptr },
 
-  { FT_0, 0, "ctimestep", reinterpret_cast<void(*)(void)> (&ts_ctimestep) },
-  { FT_0, 0, "cdate", reinterpret_cast<void(*)(void)> (&ts_cdate) },
-  { FT_0, 0, "ctime", reinterpret_cast<void(*)(void)> (&ts_ctime) },
-  { FT_0, 0, "cdeltat", reinterpret_cast<void(*)(void)> (&ts_cdeltat) },
-  { FT_0, 0, "cday", reinterpret_cast<void(*)(void)> (&ts_cday) },
-  { FT_0, 0, "cmonth", reinterpret_cast<void(*)(void)> (&ts_cmonth) },
-  { FT_0, 0, "cyear", reinterpret_cast<void(*)(void)> (&ts_cyear) },
-  { FT_0, 0, "csecond", reinterpret_cast<void(*)(void)> (&ts_csecond) },
-  { FT_0, 0, "cminute", reinterpret_cast<void(*)(void)> (&ts_cminute) },
-  { FT_0, 0, "chour", reinterpret_cast<void(*)(void)> (&ts_chour) },
+  { FT_0, 0, "ctimestep", reinterpret_cast<void (*)(void)>(&ts_ctimestep) },
+  { FT_0, 0, "cdate", reinterpret_cast<void (*)(void)>(&ts_cdate) },
+  { FT_0, 0, "ctime", reinterpret_cast<void (*)(void)>(&ts_ctime) },
+  { FT_0, 0, "cdeltat", reinterpret_cast<void (*)(void)>(&ts_cdeltat) },
+  { FT_0, 0, "cday", reinterpret_cast<void (*)(void)>(&ts_cday) },
+  { FT_0, 0, "cmonth", reinterpret_cast<void (*)(void)>(&ts_cmonth) },
+  { FT_0, 0, "cyear", reinterpret_cast<void (*)(void)>(&ts_cyear) },
+  { FT_0, 0, "csecond", reinterpret_cast<void (*)(void)>(&ts_csecond) },
+  { FT_0, 0, "cminute", reinterpret_cast<void (*)(void)>(&ts_cminute) },
+  { FT_0, 0, "chour", reinterpret_cast<void (*)(void)>(&ts_chour) },
 
   { FT_1C, 0, "sellevel", nullptr },
   { FT_1C, 0, "sellevidx", nullptr },
@@ -269,14 +269,23 @@ expr_con_con(const int oper, const nodeType *p1, const nodeType *p2)
   const double cval2 = p2->u.con.value;
 
   // clang-format off
-  switch ( oper )
+  switch (oper)
     {
+    case LT:   cval1 = COMPLT(cval1, cval2); break;
+    case GT:   cval1 = COMPGT(cval1, cval2); break;
+    case LE:   cval1 = COMPLE(cval1, cval2); break;
+    case GE:   cval1 = COMPGE(cval1, cval2); break;
+    case NE:   cval1 = COMPNE(cval1, cval2); break;
+    case EQ:   cval1 = COMPEQ(cval1, cval2); break;
+    case LEG:  cval1 = COMPLEG(cval1, cval2); break;
+    case AND:  cval1 = COMPAND(cval1, cval2); break;
+    case OR:   cval1 = COMPOR(cval1, cval2); break;
     case '+':  cval1 = cval1 + cval2; break;
     case '-':  cval1 = cval1 - cval2; break;
     case '*':  cval1 = cval1 * cval2; break;
     case '/':  cval1 = cval1 / cval2; break;
     case '^':  cval1 = pow(cval1, cval2); break;
-    default:   cdoAbort("%s: operator %c unsupported!", __func__, oper); break;
+    default:   cdoAbort("%s: operator %d unsupported!", __func__, oper); break;
     }
   // clang-format on
 
@@ -294,7 +303,7 @@ oper_expr_con_var(const int oper, const bool nmiss, const size_t n, const double
   // clang-format off
   if (nmiss)
     {
-      switch ( oper )
+      switch (oper)
         {
         case LT:  for (i=0; i<n; ++i) odat[i] = MVCOMPLT(cval, idat[i], missval1); break;
         case GT:  for (i=0; i<n; ++i) odat[i] = MVCOMPGT(cval, idat[i], missval1); break;
@@ -310,12 +319,12 @@ oper_expr_con_var(const int oper, const bool nmiss, const size_t n, const double
         case '-': for (i=0; i<n; ++i) odat[i] = SUBMN(cval, idat[i]); break;
         case '*': for (i=0; i<n; ++i) odat[i] = MULMN(cval, idat[i]); break;
         case '/': for (i=0; i<n; ++i) odat[i] = DIVMN(cval, idat[i]); break;
-        default: cdoAbort("%s: operator %c unsupported!", __func__, oper); break;
+        default: cdoAbort("%s: operator %d unsupported!", __func__, oper); break;
         }
     }
   else
     {
-      switch ( oper )
+      switch (oper)
         {
         case LT:  for (i=0; i<n; ++i) odat[i] = COMPLT(cval, idat[i]); break;
         case GT:  for (i=0; i<n; ++i) odat[i] = COMPGT(cval, idat[i]); break;
@@ -331,7 +340,7 @@ oper_expr_con_var(const int oper, const bool nmiss, const size_t n, const double
         case '-': for (i=0; i<n; ++i) odat[i] = cval - idat[i]; break;
         case '*': for (i=0; i<n; ++i) odat[i] = cval * idat[i]; break;
         case '/': for (i=0; i<n; ++i) odat[i] = DIVMN(cval, idat[i]); break;
-        default: cdoAbort("%s: operator %c unsupported!", __func__, oper); break;
+        default: cdoAbort("%s: operator %d unsupported!", __func__, oper); break;
         }
     }
   // clang-format on
@@ -362,7 +371,7 @@ oper_expr_var_con(const int oper, const bool nmiss, const size_t n, const double
         case '-': for (i=0; i<n; ++i) odat[i] = SUBMN(idat[i], cval); break;
         case '*': for (i=0; i<n; ++i) odat[i] = MULMN(idat[i], cval); break;
         case '/': for (i=0; i<n; ++i) odat[i] = DIVMN(idat[i], cval); break;
-        default: cdoAbort("%s: operator '%c' unsupported!", __func__, oper); break;
+        default: cdoAbort("%s: operator %d unsupported!", __func__, oper); break;
         }
     }
   else
@@ -386,7 +395,7 @@ oper_expr_var_con(const int oper, const bool nmiss, const size_t n, const double
           if (IS_EQUAL(cval, 0)) for (i=0; i<n; ++i) odat[i] = missval1;
           else                   for (i=0; i<n; ++i) odat[i] = idat[i] / cval;
           break;
-        default: cdoAbort("%s: operator '%c' unsupported!", __func__, oper); break;
+        default: cdoAbort("%s: operator %d unsupported!", __func__, oper); break;
         }
     }
   // clang-format on
@@ -417,7 +426,7 @@ oper_expr_var_var(const int oper, const bool nmiss, const size_t n, const double
         case '-': for (i=0; i<n; ++i) odat[i] = SUBMN(idat1[i], idat2[i]); break;
         case '*': for (i=0; i<n; ++i) odat[i] = MULMN(idat1[i], idat2[i]); break;
         case '/': for (i=0; i<n; ++i) odat[i] = DIVMN(idat1[i], idat2[i]); break;
-        default: cdoAbort("%s: operator %d (%c) unsupported!", __func__, oper, oper); break;
+        default: cdoAbort("%s: operator %d unsupported!", __func__, oper); break;
         }
     }
   else
@@ -438,7 +447,7 @@ oper_expr_var_var(const int oper, const bool nmiss, const size_t n, const double
         case '-': for (i=0; i<n; ++i) odat[i] = idat1[i] - idat2[i]; break;
         case '*': for (i=0; i<n; ++i) odat[i] = idat1[i] * idat2[i]; break;
         case '/': for (i = 0; i < n; ++i) odat[i] = IS_EQUAL(idat2[i], 0.) ? missval1 : idat1[i] / idat2[i]; break;
-        default: cdoAbort("%s: operator %d (%c) unsupported!", __func__, oper, oper); break;
+        default: cdoAbort("%s: operator %d unsupported!", __func__, oper); break;
         }
     }
   // clang-format on
@@ -472,7 +481,7 @@ expr_con_var(const int init, const int oper, const nodeType *p1, const nodeType
       double cval = p1->u.con.value;
       if (datatype == CDI_DATATYPE_FLT32 && isCompare(oper)) cval = (float) cval;
 
-      oper_expr_con_var(oper, nmiss > 0, n, missval1, missval2, odat, cval, idat);
+      oper_expr_con_var(oper, nmiss, n, missval1, missval2, odat, cval, idat);
 
       p->param.nmiss = arrayNumMV(n, odat, missval1);
     }
@@ -508,7 +517,7 @@ expr_var_con(const int init, const int oper, const nodeType *p1, const nodeType
       double cval = p2->u.con.value;
       if (datatype == CDI_DATATYPE_FLT32 && isCompare(oper)) cval = (float) cval;
 
-      oper_expr_var_con(oper, nmiss > 0, n, missval1, missval2, odat, idat, cval);
+      oper_expr_var_con(oper, nmiss, n, missval1, missval2, odat, idat, cval);
 
       p->param.nmiss = arrayNumMV(n, odat, missval1);
     }
@@ -607,7 +616,7 @@ expr_var_var(int init, int oper, nodeType *p1, nodeType *p2)
           const double *restrict idat1 = p1->param.data + loff1;
           const double *restrict idat2 = p2->param.data + loff2;
           double *restrict odat = p->param.data + loff;
-          const size_t nmiss = nmiss1 > 0 || nmiss2 > 0;
+          const size_t nmiss = nmiss1 || nmiss2;
 
           if (ngp1 != ngp2)
             {
@@ -669,7 +678,8 @@ ex_copy_con(int init, nodeType *p2, const nodeType *p1)
 {
   const double cval = p1->u.con.value;
 
-  if (Options::cdoVerbose) cdoPrint("\t%s\tcopy\t%s[L%zu][N%zu] = %g", ExIn[init], p2->param.name, p2->param.nlev, p2->param.ngp, cval);
+  if (Options::cdoVerbose)
+    cdoPrint("\t%s\tcopy\t%s[L%zu][N%zu] = %g", ExIn[init], p2->param.name, p2->param.nlev, p2->param.ngp, cval);
 
   const size_t ngp = p2->param.ngp;
   assert(ngp > 0);
@@ -718,7 +728,7 @@ expr(const int init, const int oper, nodeType *p1, nodeType *p2)
         case '-': coper = "-"; break;
         case '*': coper = "*"; break;
         case '/': coper = "/"; break;
-        default: cdoAbort("Internal error, expr operator >%c< not implemented!\n", oper);
+        default: cdoAbort("Internal error, expr operator %d not implemented!\n", oper);
         }
     }
 
@@ -748,7 +758,8 @@ expr(const int init, const int oper, nodeType *p1, nodeType *p2)
   else if (p1->type == typeCon && p2->type == typeCon)
     {
       p = expr_con_con(oper, p1, p2);
-      if (Options::cdoVerbose) cdoPrint("\t%s\tarith\t%g = %g %s %g", ExIn[init], p->u.con.value, p1->u.con.value, coper, p2->u.con.value);
+      if (Options::cdoVerbose)
+        cdoPrint("\t%s\tarith\t%g = %g %s %g", ExIn[init], p->u.con.value, p1->u.con.value, coper, p2->u.con.value);
     }
   else
     cdoAbort("Internal problem!");
@@ -857,7 +868,7 @@ ex_fun_var(const int init, const int funcID, nodeType *p1)
       if (functype == FT_STD)
         {
           double (*exprfunc)(double) = (double (*)(double)) fun_sym_tbl[funcID].func;
-          if (nmiss > 0)
+          if (nmiss)
             {
               for (size_t i = 0; i < ngp * nlev; i++)
                 {
@@ -885,43 +896,45 @@ ex_fun_var(const int init, const int funcID, nodeType *p1)
       else if (functype == FT_FLD)
         {
           Field field;
-          double *weights = nullptr;
+          field.resize(ngp);
           if (funcflag == 1)
             {
-              weights = p1->param.weight;
-              assert(weights != nullptr);
+              assert(p1->param.weight != nullptr);
+              field.weightv.resize(ngp);
             }
 
           double (*exprfunc)(const Field &) = (double (*)(const Field &)) fun_sym_tbl[funcID].func;
           for (size_t k = 0; k < nlev; k++)
             {
-              fld_field_init(field, nmiss, missval, ngp, p1data + k * ngp, weights);
+              fld_field_init(field, nmiss, missval, ngp, p1data + k * ngp, p1->param.weight);
               pdata[k] = exprfunc(field);
             }
         }
       else if (functype == FT_ZON)
         {
           Field field1, field2;
+          field1.resize(ngp);
+          field2.resize(nlat);
           void (*exprfunc)(const Field &, Field &) = (void (*)(const Field &, Field &)) fun_sym_tbl[funcID].func;
           for (size_t k = 0; k < nlev; k++)
             {
-              fld_field_init(field1, nmiss, missval, ngp, p1data + k * ngp, nullptr);
+              fld_field_init(field1, nmiss, missval, ngp, &p1data[k * ngp], nullptr);
               field1.grid = gridID;
-              fld_field_init(field2, nmiss, missval, nlat, &pdata[k], nullptr);
+              fld_field_init(field2, nmiss, missval, nlat, &pdata[k * nlat], nullptr);
               exprfunc(field1, field2);
+              arrayCopy(nlat, field2.vec.data(), &pdata[k * nlat]);
             }
         }
       else if (functype == FT_VERT)
         {
           Field field;
-          std::vector<double> weights;
-          if (funcflag == 1) vert_weights(p1->param.zaxisID, nlev, weights);
-          std::vector<double> array(nlev);
+          field.resize(nlev);
+          if (funcflag == 1) vert_weights(p1->param.zaxisID, nlev, field.weightv);
           double (*exprfunc)(const Field &) = (double (*)(const Field &)) fun_sym_tbl[funcID].func;
           for (size_t i = 0; i < ngp; i++)
             {
-              for (size_t k = 0; k < nlev; k++) array[k] = p1data[k * ngp + i];
-              fld_field_init(field, nmiss, missval, nlev, array.data(), weights.data());
+              for (size_t k = 0; k < nlev; k++) field.vec[k] = p1data[k * ngp + i];
+              fld_field_init(field, nmiss, missval, nlev, nullptr, nullptr);
               pdata[i] = exprfunc(field);
             }
         }
@@ -1028,6 +1041,12 @@ fun1c(int init, int funcID, nodeType *p1, double value, parseParamType *parse_ar
   param_meta_copy(p->param, p1->param);
   p->param.name = p->u.var.nm;
 
+  if (init)
+    {
+      if (p1->param.longname) p->param.longname = strdup(p1->param.longname);
+      if (p1->param.units) p->param.units = strdup(p1->param.units);
+    }
+
   p->param.nlev = 1;
 
   const int zaxisID = p1->param.zaxisID;
@@ -1052,7 +1071,7 @@ fun1c(int init, int funcID, nodeType *p1, double value, parseParamType *parse_ar
 
   if (cstrIsEqual(funcname, "sellevidx"))
     {
-      long ilevidx = lround(value);
+      const long ilevidx = lround(value);
       if (ilevidx < 1 || ilevidx > (long) nlev)
         cdoAbort("%s(): level index %ld out of range (range: 1-%zu)!", funcname, ilevidx, nlev);
       levidx = (size_t) ilevidx - 1;
@@ -1078,7 +1097,7 @@ fun1c(int init, int funcID, nodeType *p1, double value, parseParamType *parse_ar
       double *pdata = p->param.data;
       const double *p1data = p1->param.data + ngp * levidx;
       arrayCopy(ngp, p1data, pdata);
-      if (nmiss > 0) nmiss = arrayNumMV(ngp, pdata, missval);
+      if (nmiss) nmiss = arrayNumMV(ngp, pdata, missval);
       p->param.nmiss = nmiss;
     }
 
@@ -1107,6 +1126,12 @@ fun2c(int init, int funcID, nodeType *p1, double value1, double value2, parsePar
   param_meta_copy(p->param, p1->param);
   p->param.name = p->u.var.nm;
 
+  if (init)
+    {
+      if (p1->param.longname) p->param.longname = strdup(p1->param.longname);
+      if (p1->param.units) p->param.units = strdup(p1->param.units);
+    }
+
   const int zaxisID = p1->param.zaxisID;
   const int coordID = params_get_coordID(parse_arg, 'z', zaxisID);
   size_t levidx1 = 0;
@@ -1128,11 +1153,11 @@ fun2c(int init, int funcID, nodeType *p1, double value1, double value2, parsePar
   if (cstrIsEqual(funcname, "sellevidxrange"))
     {
       if (value2 < value1) cdoAbort("%s(): first level index is greater than last level index!", funcname);
-      long ilevidx1 = lround(value1);
+      const long ilevidx1 = lround(value1);
       if (ilevidx1 < 1 || ilevidx1 > (long) nlev)
         cdoAbort("%s(): level index1 %ld out of range (range: 1-%zu)!", funcname, ilevidx1, nlev);
       levidx1 = (size_t) ilevidx1 - 1;
-      long ilevidx2 = lround(value2);
+      const long ilevidx2 = lround(value2);
       if (ilevidx2 < 1 || ilevidx2 > (long) nlev)
         cdoAbort("%s(): level index2 %ld out of range (range: 1-%zu)!", funcname, ilevidx2, nlev);
       levidx2 = (size_t) ilevidx2 - 1;
@@ -1171,7 +1196,7 @@ fun2c(int init, int funcID, nodeType *p1, double value1, double value2, parsePar
       double *pdata = p->param.data;
       const double *p1data = p1->param.data + ngp * levidx1;
       arrayCopy(ngp * nlevout, p1data, pdata);
-      if (nmiss > 0) nmiss = arrayNumMV(ngp * nlevout, pdata, missval);
+      if (nmiss) nmiss = arrayNumMV(ngp * nlevout, pdata, missval);
       p->param.nmiss = nmiss;
     }
 
@@ -1247,7 +1272,7 @@ ex_uminus_var(const int init, nodeType *p1)
       double *restrict pdata = p->param.data;
       const double *restrict p1data = p1->param.data;
 
-      if (nmiss > 0)
+      if (nmiss)
         {
           for (size_t i = 0; i < ngp * nlev; ++i) pdata[i] = DBL_IS_EQUAL(p1data[i], missval) ? missval : -(p1data[i]);
         }
@@ -1320,7 +1345,7 @@ ex_not_var(const int init, nodeType *p1)
       double *restrict pdata = p->param.data;
       const double *restrict p1data = p1->param.data;
 
-      if (nmiss > 0)
+      if (nmiss)
         {
           for (size_t i = 0; i < ngp * nlev; ++i) pdata[i] = DBL_IS_EQUAL(p1data[i], missval) ? missval : COMPNOT(p1data[i]);
         }
@@ -1371,32 +1396,87 @@ ex_not(const int init, nodeType *p1)
   return p;
 }
 
+static void
+strAddNodeInfo(char *string, size_t stringlen, nodeType *p, const char *ext)
+{
+  const size_t len = strlen(string);
+  if (p->type == typeCon)
+    snprintf(string + len, stringlen - len, "%g%s", p->u.con.value, ext);
+  else
+    snprintf(string + len, stringlen - len, "%s[L%zu][N%zu]%s", p->u.var.nm, p->param.nlev, p->param.ngp, ext);
+}
+
 static nodeType *
 ex_ifelse(const int init, nodeType *p1, nodeType *p2, nodeType *p3)
 {
-  if (p1->type == typeCon) cdoAbort("expr?expr:expr: First expression is a constant but must be a variable!");
-
   if (Options::cdoVerbose)
     {
-      fprintf(stderr, "cdo expr:\t%s\tifelse\t%s[L%zu][N%zu] ? ", ExIn[init], p1->u.var.nm, p1->param.nlev, p1->param.ngp);
-      if (p2->type == typeCon)
-        fprintf(stderr, "%g : ", p2->u.con.value);
-      else
-        fprintf(stderr, "%s[L%zu][N%zu] : ", p2->u.var.nm, p2->param.nlev, p2->param.ngp);
-      if (p3->type == typeCon)
-        fprintf(stderr, "%g\n", p3->u.con.value);
-      else
-        fprintf(stderr, "%s[L%zu][N%zu]\n", p3->u.var.nm, p3->param.nlev, p3->param.ngp);
+      char strbuffer[1024];
+      snprintf(strbuffer, sizeof(strbuffer), "\t%s\tifelse\t", ExIn[init]);
+      strAddNodeInfo(strbuffer, sizeof(strbuffer), p1, " ? ");
+      strAddNodeInfo(strbuffer, sizeof(strbuffer), p2, " : ");
+      strAddNodeInfo(strbuffer, sizeof(strbuffer), p3, "");
+      cdoPrint(strbuffer);
     }
 
-  const size_t nmiss1 = p1->param.nmiss;
-  const size_t ngp1 = p1->param.ngp;
-  const size_t nlev1 = p1->param.nlev;
-  const double missval1 = p1->param.missval;
+  nodeType *p0 = nullptr;
+  nodeType *pnodes[3] = {p1, p2, p3};
+  for (unsigned i = 0; i < 3; ++i)
+    if (pnodes[i]->type != typeCon)
+      {
+        p0 = pnodes[i];
+        break;
+      }
+  
+  if (p0 == nullptr) cdoAbort("expr?expr:expr: no data variable found!");
 
-  size_t ngp = ngp1;
-  size_t nlev = nlev1;
-  nodeType *px = p1;
+  
+  double missval = p0->param.missval;
+  size_t ngp = p0->param.ngp;
+  size_t nlev = p0->param.nlev;
+  nodeType *px = p0;
+
+
+  size_t nmiss1 = 0;
+  double missval1 = missval;
+  double *pdata1 = nullptr;
+  size_t ngp1 = 1;
+  size_t nlev1 = 1;
+
+  if (p1->type == typeCon)
+    {
+      pdata1 = &p1->u.con.value;
+    }
+  else
+    {
+      nmiss1 = p1->param.nmiss;
+      ngp1 = p1->param.ngp;
+      nlev1 = p1->param.nlev;
+      missval1 = p1->param.missval;
+      pdata1 = p1->param.data;
+
+      if (ngp1 > 1 && ngp1 != ngp)
+        {
+          if (ngp == 1)
+            {
+              ngp = ngp1;
+              px = p1;
+            }
+          else
+            cdoAbort("expr?expr:expr: Number of grid points differ (ngp = %zu, ngp1 = %zu)", ngp, ngp1);
+        }
+
+      if (nlev1 > 1 && nlev1 != nlev)
+        {
+          if (nlev == 1)
+            {
+              nlev = nlev1;
+              px = p1;
+            }
+          else
+            cdoAbort("expr?expr:expr: Number of levels differ (nlev = %zu, nlev1 = %zu)", nlev, nlev1);
+        }
+    }
 
   double missval2 = missval1;
   double *pdata2 = nullptr;
@@ -1422,7 +1502,7 @@ ex_ifelse(const int init, nodeType *p1, nodeType *p2, nodeType *p3)
               px = p2;
             }
           else
-            cdoAbort("expr?expr:expr: Number of grid points differ (ngp1 = %zu, ngp2 = %zu)", ngp1, ngp2);
+            cdoAbort("expr?expr:expr: Number of grid points differ (ngp = %zu, ngp2 = %zu)", ngp, ngp2);
         }
 
       if (nlev2 > 1 && nlev2 != nlev)
@@ -1461,7 +1541,7 @@ ex_ifelse(const int init, nodeType *p1, nodeType *p2, nodeType *p3)
               px = p3;
             }
           else
-            cdoAbort("expr?expr:expr: Number of grid points differ (ngp1 = %zu, ngp3 = %zu)", ngp1, ngp3);
+            cdoAbort("expr?expr:expr: Number of grid points differ (ngp = %zu, ngp3 = %zu)", ngp, ngp3);
         }
 
       if (nlev3 > 1 && nlev3 != nlev)
@@ -1488,7 +1568,6 @@ ex_ifelse(const int init, nodeType *p1, nodeType *p2, nodeType *p3)
   if (!init)
     {
       size_t nmiss = 0;
-      double *pdata1 = p1->param.data;
 
       p->param.data = (double *) Malloc(ngp * nlev * sizeof(double));
 
@@ -1667,9 +1746,10 @@ expr_run(nodeType *p, parseParamType *parse_arg)
     case typeVar:
       {
         const char *vnm = p->u.var.nm;
+        // printf("%s: ngp %zu nlev %zu gID %d zID %d\n", vnm, rnode->param.ngp, rnode->param.nlev, rnode->param.gridID, rnode->param.zaxisID);
         varID = param_search_name_rev(parse_arg->nparams, params, vnm);
         // varID = param_search_name_rev2(parse_arg->nparams, params, vnm, rnode->param.gridID, rnode->param.zaxisID);
-        // printf("found %s varID %d\n", vnm, varID);
+        //XXX printf("found %s varID %d  cnparams %d  nparams %d\n", vnm, varID, parse_arg->cnparams, parse_arg->nparams);
         // if ( varID != -1 ) printf("   gridID %d zaxisID %d\n", params[varID].gridID, params[varID].zaxisID);
         if (varID == -1 && init)
           {
@@ -1682,17 +1762,16 @@ expr_run(nodeType *p, parseParamType *parse_arg)
                     char *varname = strdup(vnm);
                     varname[len - 2] = 0;
                     varID = param_search_name_rev(parse_arg->nparams, params, varname);
-                    free(varname);
                     if (varID == -1)
                       {
                         cdoAbort("Coordinate %c: variable >%s< not found!", coord, varname);
                       }
                     else
                       {
-                        int nvarID = parse_arg->nparams;
+                        const int nvarID = parse_arg->nparams;
                         if (nvarID >= parse_arg->maxparams) cdoAbort("Too many parameter (limit=%d)", parse_arg->maxparams);
 
-                        int coordID = params_get_coordID(parse_arg, coord, params[varID].gridID);
+                        const int coordID = params_get_coordID(parse_arg, coord, params[varID].gridID);
                         parse_arg->coords[coordID].needed = true;
                         const char *units = parse_arg->coords[coordID].units;
                         const char *longname = parse_arg->coords[coordID].longname;
@@ -1710,15 +1789,16 @@ expr_run(nodeType *p, parseParamType *parse_arg)
                         if (units) params[nvarID].units = strdup(units);
                         if (longname) params[nvarID].longname = strdup(longname);
                         parse_arg->nparams++;
+                        parse_arg->cnparams++;
                         varID = nvarID;
                       }
+                    free(varname);
                   }
                 else if (coord == 'z')
                   {
                     char *varname = strdup(vnm);
                     varname[len - 2] = 0;
                     varID = param_search_name_rev(parse_arg->nparams, params, varname);
-                    free(varname);
                     if (varID == -1)
                       {
                         cdoAbort("Coordinate %c: variable >%s< not found!", coord, varname);
@@ -1745,8 +1825,10 @@ expr_run(nodeType *p, parseParamType *parse_arg)
                         if (units) params[nvarID].units = strdup(units);
                         if (longname) params[nvarID].longname = strdup(longname);
                         parse_arg->nparams++;
+                        parse_arg->cnparams++;
                         varID = nvarID;
                       }
+                    free(varname);
                   }
               }
           }
@@ -1876,9 +1958,9 @@ expr_run(nodeType *p, parseParamType *parse_arg)
 
             if (init)
               {
-                //printf("%s: type %d %d %d\n", varname2, rnode->type, p->u.opr.op[0]->type, p->u.opr.op[1]->type);
-                //printf("%s: ngp %d nlev %d gID %d zID %d\n", varname2, rnode->param.ngp, rnode->param.nlev, rnode->param.gridID, rnode->param.zaxisID);
-                //varID = param_search_name_rev2(parse_arg->nparams, params, varname2, rnode->param.gridID, rnode->param.zaxisID);
+                // printf("%s: type %d %d %d\n", varname2, rnode->type, p->u.opr.op[0]->type, p->u.opr.op[1]->type);
+                //XXX printf("%s: ngp %zu nlev %zu gID %d zID %d\n", varname2, rnode->param.ngp, rnode->param.nlev, rnode->param.gridID, rnode->param.zaxisID);
+                // varID = param_search_name_rev2(parse_arg->nparams, params, varname2, rnode->param.gridID, rnode->param.zaxisID);
                 varID = param_search_name_rev(parse_arg->nparams, params, varname2);
                 if (varID >= 0)
                   {
@@ -1909,6 +1991,7 @@ expr_run(nodeType *p, parseParamType *parse_arg)
                     params[varID].nmiss = 0;
                     params[varID].name = strdup(varname2);
                     parse_arg->nparams++;
+                    parse_arg->cnparams++;
                   }
                 else if (p->u.opr.op[1]->type != typeCon)
                   {
@@ -1923,16 +2006,16 @@ expr_run(nodeType *p, parseParamType *parse_arg)
                     if (rnode->param.units) params[varID].units = strdup(rnode->param.units);
                     if (rnode->param.longname) params[varID].longname = strdup(rnode->param.longname);
                     parse_arg->nparams++;
+                    parse_arg->cnparams++;
                   }
               }
             else
               {
+                //XXX printf(">> %s: ngp %zu nlev %zu gID %d zID %d\n", varname2, rnode->param.ngp, rnode->param.nlev, rnode->param.gridID, rnode->param.zaxisID);
+                // varID = param_search_name_rev2(parse_arg->nparams, params, varname2, rnode->param.gridID, rnode->param.zaxisID);
                 varID = param_search_name_rev(parse_arg->nparams, params, varname2);
-                //varID = param_search_name_rev2(parse_arg->nparams, params, varname2, rnode->param.gridID, rnode->param.zaxisID);
-                if (varID < 0)
-                  cdoAbort("Variable >%s< not found!", varname2);
-                else if (params[varID].coord)
-                  cdoAbort("Coordinate variable %s is read only!", varname2);
+                if (varID < 0) cdoAbort("Variable >%s< not found!", varname2);
+                if (params[varID].coord) cdoAbort("Coordinate variable %s is read only!", varname2);
                 param_meta_copy(p->param, params[varID]);
                 p->param.name = params[varID].name;
                 p->param.data = params[varID].data;
