@@ -17,10 +17,6 @@
 
 #include <stdio.h>
 
-#ifndef fileno
-int fileno(FILE *stream);
-#endif
-
 extern int CDO_parser_errorno;
 
 enum
@@ -38,7 +34,16 @@ enum
   CLEN
 };
 
-typedef enum { typeCon, typeVar, typeFun, typeFun1c, typeFun2c, typeOpr, typeCom } nodeEnum;
+typedef enum
+{
+  typeCon,
+  typeVar,
+  typeFun,
+  typeFun1c,
+  typeFun2c,
+  typeOpr,
+  typeCom
+} nodeEnum;
 
 // commands
 struct comNodeType
@@ -91,7 +96,11 @@ struct oprNodeType
   struct nodeTypeTag *op[1];  // operands (expandable)
 };
 
-enum {PARAM_VAR, PARAM_CONST};
+enum
+{
+  PARAM_VAR,
+  PARAM_CONST
+};
 
 // parameter
 struct paramType
@@ -156,6 +165,7 @@ struct parseParamType
   bool *needed;
   int maxparams;
   int nparams;
+  int cnparams;    // current number of valid params
   int nvars1;
   int ncoords;
   int maxcoords;
