@@ -16,20 +16,18 @@
 */
 
 #include "cdo_int.h"
-#include "grid.h"
+#include <mpim_grid.h>
 
 void
 fld_field_init(Field &field, size_t nmiss, double missval, size_t ngp, double *array, double *w)
 {
-  fieldInit(field);
-
   field.size = ngp;
   field.nmiss = nmiss;
   field.missval = missval;
-  field.ptr = array;
-  field.weight = w;
+  if (array != nullptr) for (size_t i = 0; i < ngp; ++i) field.vec[i] = array[i];
+  if (w != nullptr) for (size_t i = 0; i < ngp; ++i) field.weightv[i] = w[i];
 }
-
+/*
 double *
 fld_weights(int gridID, size_t ngp)
 {
@@ -49,21 +47,21 @@ fld_weights(int gridID, size_t ngp)
 
   return weights;
 }
-
+*/
 int getLayerThickness(bool useweights, bool genbounds, int index, int zaxisID, int nlev, double *thickness, double *weights);
 
 void
 vert_weights(int zaxisID, size_t nlev, std::vector<double> &weights)
 {
-  static bool lwarn = true;
   weights.resize(nlev);
   std::vector<double> thickness(nlev);
   for (size_t i = 0; i < nlev; ++i) weights[i] = 1;
 
   if (nlev > 1)
     {
+      static bool lwarn = true;
       int wstatus = getLayerThickness(1, 0, 0, zaxisID, nlev, thickness.data(), weights.data());
-      if (wstatus != 0 && lwarn && nlev > 1)
+      if (wstatus == 0 && lwarn && nlev > 1)
         {
           lwarn = false;
           cdoWarning("Layer bounds not available, using constant vertical weights!");
