@@ -16,26 +16,22 @@
 */
 
 #include <cdi.h>
-
 #include "cdo_int.h"
 #include "array.h"
 
 static void
 faraddcplx(Field &field1, const Field &field2)
 {
-  int nwpv = field1.nwpv;
-  int grid1 = field1.grid;
-  int grid2 = field2.grid;
-  double missval1 = field1.missval;
-  double missval2 = field2.missval;
-  double *restrict array1 = field1.ptr;
-  const double *restrict array2 = field2.ptr;
+  const double missval1 = field1.missval;
+  const double missval2 = field2.missval;
+  auto &array1 = field1.vec;
+  const auto &array2 = field2.vec;
 
-  if (nwpv != 2) nwpv = 1;
+  if (field1.nwpv != 2) cdoAbort("Field1 is not complex!");
+  if (field2.nwpv != 2) cdoAbort("Field2 is not complex!");
 
-  size_t gridsize = gridInqSize(grid1);
-  size_t len = nwpv * gridsize;
-  if (len != (nwpv * gridInqSize(grid2))) cdoAbort("Fields have different gridsize (%s)", __func__);
+  const size_t gridsize = gridInqSize(field1.grid);
+  if (gridsize != gridInqSize(field2.grid)) cdoAbort("Fields have different size (%s)", __func__);
 
   for (size_t i = 0; i < gridsize; i++)
     {
@@ -47,19 +43,16 @@ faraddcplx(Field &field1, const Field &field2)
 static void
 farsubcplx(Field &field1, const Field &field2)
 {
-  int nwpv = field1.nwpv;
-  int grid1 = field1.grid;
-  int grid2 = field2.grid;
-  double missval1 = field1.missval;
-  double missval2 = field2.missval;
-  double *restrict array1 = field1.ptr;
-  const double *restrict array2 = field2.ptr;
+  const double missval1 = field1.missval;
+  const double missval2 = field2.missval;
+  auto &array1 = field1.vec;
+  const auto &array2 = field2.vec;
 
-  if (nwpv != 2) nwpv = 1;
+  if (field1.nwpv != 2) cdoAbort("Field1 is not complex!");
+  if (field2.nwpv != 2) cdoAbort("Field2 is not complex!");
 
-  size_t gridsize = gridInqSize(grid1);
-  size_t len = nwpv * gridsize;
-  if (len != (nwpv * gridInqSize(grid2))) cdoAbort("Fields have different gridsize (%s)", __func__);
+  const size_t gridsize = gridInqSize(field1.grid);
+  if (gridsize != gridInqSize(field2.grid)) cdoAbort("Fields have different size (%s)", __func__);
 
   for (size_t i = 0; i < gridsize; i++)
     {
@@ -71,19 +64,16 @@ farsubcplx(Field &field1, const Field &field2)
 static void
 farmulcplx(Field &field1, const Field &field2)
 {
-  int nwpv = field1.nwpv;
-  int grid1 = field1.grid;
-  int grid2 = field2.grid;
-  double missval1 = field1.missval;
-  double missval2 = field2.missval;
-  double *restrict array1 = field1.ptr;
-  const double *restrict array2 = field2.ptr;
+  const double missval1 = field1.missval;
+  const double missval2 = field2.missval;
+  auto &array1 = field1.vec;
+  const auto &array2 = field2.vec;
 
-  if (nwpv != 2) nwpv = 1;
+  if (field1.nwpv != 2) cdoAbort("Field1 is not complex!");
+  if (field2.nwpv != 2) cdoAbort("Field2 is not complex!");
 
-  size_t gridsize = gridInqSize(grid1);
-  size_t len = nwpv * gridsize;
-  if (len != (nwpv * gridInqSize(grid2))) cdoAbort("Fields have different gridsize (%s)", __func__);
+  const size_t gridsize = gridInqSize(field1.grid);
+  if (gridsize != gridInqSize(field2.grid)) cdoAbort("Fields have different size (%s)", __func__);
 
   // z1 x z2 = (x1x2 - y1y2) + i(x1y2 + x2y1)
   for (size_t i = 0; i < gridsize; i++)
@@ -98,33 +88,30 @@ farmulcplx(Field &field1, const Field &field2)
 static void
 fardivcplx(Field &field1, const Field &field2)
 {
-  int nwpv = field1.nwpv;
-  int grid1 = field1.grid;
-  int grid2 = field2.grid;
-  double missval1 = field1.missval;
-  double missval2 = field2.missval;
-  double *restrict array1 = field1.ptr;
-  const double *restrict array2 = field2.ptr;
+  const double missval1 = field1.missval;
+  const double missval2 = field2.missval;
+  auto &array1 = field1.vec;
+  const auto &array2 = field2.vec;
 
-  if (nwpv != 2) nwpv = 1;
+  if (field1.nwpv != 2) cdoAbort("Field1 is not complex!");
+  if (field2.nwpv != 2) cdoAbort("Field2 is not complex!");
 
-  size_t gridsize = gridInqSize(grid1);
-  size_t len = nwpv * gridsize;
-  if (len != (nwpv * gridInqSize(grid2))) cdoAbort("Fields have different gridsize (%s)", __func__);
+  const size_t gridsize = gridInqSize(field1.grid);
+  if (gridsize != gridInqSize(field2.grid)) cdoAbort("Fields have different size (%s)", __func__);
 
   // z1 / z2 = (x1x2 + y1y2) / (x2x2 + y2y2) + i (y1x2 - x1y2) / (x2x2 + y2y2)
   for (size_t i = 0; i < gridsize; i++)
     {
-      double a1r = array1[2 * i];
-      double a1i = array1[2 * i + 1];
-      double denominator = ADDMN(MULMN(array2[2 * i], array2[2 * i]), MULMN(array2[2 * i + 1], array2[2 * i + 1]));
+      const double a1r = array1[2 * i];
+      const double a1i = array1[2 * i + 1];
+      const double denominator = ADDMN(MULMN(array2[2 * i], array2[2 * i]), MULMN(array2[2 * i + 1], array2[2 * i + 1]));
       array1[2 * i] = DIVMN(ADDMN(MULMN(a1r, array2[2 * i]), MULMN(a1i, array2[2 * i + 1])), denominator);
       array1[2 * i + 1] = DIVMN(SUBMN(MULMN(a1i, array2[2 * i]), MULMN(a1r, array2[2 * i + 1])), denominator);
     }
 }
 
 void
-farfuncplx(Field &field1, const Field &field2, int function)
+vfarfuncplx(Field &field1, const Field &field2, int function)
 {
   // clang-format off
   switch (function)
