@@ -15,21 +15,18 @@
   GNU General Public License for more details.
 */
 
-#include "cdo_int.h"
 #include <cdi.h>
+#include "cdo_int.h"
 
 static void
-farcmulcplx(Field &field, const double rconst[2])
+vfarcmulcplx(Field &field, const double rconst[2])
 {
-  int nwpv = field.nwpv;
-  int grid = field.grid;
-  double missval1 = field.missval;
-  double missval2 = field.missval;
-  double *array = field.ptr;
+  const int grid = field.grid;
+  const double missval1 = field.missval;
+  const double missval2 = field.missval;
+  auto &array = field.vec;
 
-  if (nwpv != 2) nwpv = 1;
-
-  size_t gridsize = gridInqSize(grid);
+  const size_t gridsize = gridInqSize(grid);
 
   // z1 x z2 = (x1x2 - y1y2) + i(x1y2 + x2y1)
   for (size_t i = 0; i < gridsize; i++)
@@ -42,17 +39,14 @@ farcmulcplx(Field &field, const double rconst[2])
 }
 
 static void
-farcdivcplx(Field &field, const double rconst[2])
+vfarcdivcplx(Field &field, const double rconst[2])
 {
-  int nwpv = field.nwpv;
-  int grid = field.grid;
-  double missval1 = field.missval;
-  double missval2 = field.missval;
-  double *array = field.ptr;
+  const int grid = field.grid;
+  const double missval1 = field.missval;
+  const double missval2 = field.missval;
+  auto &array = field.vec;
 
-  if (nwpv != 2) nwpv = 1;
-
-  size_t gridsize = gridInqSize(grid);
+  const size_t gridsize = gridInqSize(grid);
 
   // z1 / z2 = (x1x2 + y1y2) / (x2x2 + y2y2) + i (y1x2 - x1y2) / (x2x2 + y2y2)
   for (size_t i = 0; i < gridsize; i++)
@@ -66,17 +60,14 @@ farcdivcplx(Field &field, const double rconst[2])
 }
 
 static void
-farcaddcplx(Field &field, const double rconst[2])
+vfarcaddcplx(Field &field, const double rconst[2])
 {
-  int nwpv = field.nwpv;
-  int grid = field.grid;
-  double missval1 = field.missval;
-  double missval2 = field.missval;
-  double *array = field.ptr;
+  const int grid = field.grid;
+  const double missval1 = field.missval;
+  const double missval2 = field.missval;
+  auto &array = field.vec;
 
-  if (nwpv != 2) nwpv = 1;
-
-  size_t gridsize = gridInqSize(grid);
+  const size_t gridsize = gridInqSize(grid);
 
   for (size_t i = 0; i < gridsize; i++)
     {
@@ -86,17 +77,14 @@ farcaddcplx(Field &field, const double rconst[2])
 }
 
 static void
-farcsubcplx(Field &field, const double rconst[2])
+vfarcsubcplx(Field &field, const double rconst[2])
 {
-  int nwpv = field.nwpv;
-  int grid = field.grid;
-  double missval1 = field.missval;
-  double missval2 = field.missval;
-  double *array = field.ptr;
+  const int grid = field.grid;
+  const double missval1 = field.missval;
+  const double missval2 = field.missval;
+  auto &array = field.vec;
 
-  if (nwpv != 2) nwpv = 1;
-
-  size_t gridsize = gridInqSize(grid);
+  const size_t gridsize = gridInqSize(grid);
 
   for (size_t i = 0; i < gridsize; i++)
     {
@@ -106,14 +94,14 @@ farcsubcplx(Field &field, const double rconst[2])
 }
 
 void
-farcfuncplx(Field &field, const double rconst[2], int function)
+vfarcfuncplx(Field &field, const double rconst[2], int function)
 {
   switch (function)
     {
-    case func_add: farcaddcplx(field, rconst); break;
-    case func_sub: farcsubcplx(field, rconst); break;
-    case func_mul: farcmulcplx(field, rconst); break;
-    case func_div: farcdivcplx(field, rconst); break;
+    case func_add: vfarcaddcplx(field, rconst); break;
+    case func_sub: vfarcsubcplx(field, rconst); break;
+    case func_mul: vfarcmulcplx(field, rconst); break;
+    case func_div: vfarcdivcplx(field, rconst); break;
     default: cdoAbort("%s: function %d not implemented!", __func__, function);
     }
 }
