@@ -16,8 +16,9 @@
 #include <vector>
 
 #include <cdi.h>
-#include "grid.h"
-#include "exception.h"
+#include <mpim_grid.h>
+#include "gridreference.h"
+#include "cdo_output.h"
 #include "compare.h"
 #include "cimdOmp.h"
 #include "progress.h"
@@ -292,57 +293,71 @@ gridGenAreaReg2D(const int gridID, double *area, bool lweights)
 
   int status = 0;
 
-  const size_t gridsize = gridInqSize(gridID);
-  const size_t nlon = gridInqXsize(gridID);
-  const size_t nlat = gridInqYsize(gridID);
-
-  if (gridInqYvals(gridID, nullptr) == 0 || gridInqXvals(gridID, nullptr) == 0)
+  if (gridInqXvals(gridID, nullptr) == 0 && gridInqYvals(gridID, nullptr) == 0 && gridInqXbounds(gridID, nullptr) == 0
+      && gridInqYbounds(gridID, nullptr) == 0)
     {
       cdoWarning("Computation of grid cell area weights failed, grid cell center coordinates missing!");
       return 1;
     }
 
-  std::vector<double> grid_center_lon(nlon);
-  std::vector<double> grid_center_lat(nlat);
+  char unitstr[CDI_MAX_NAME];
+  size_t gridsize = gridInqSize(gridID);
+  size_t nlon = gridInqXsize(gridID);
+  size_t nlat = gridInqYsize(gridID);
+  if (!nlon) nlon = 1;
+  if (!nlat) nlat = 1;
+  std::vector<double> grid_corner_lon(nlon * 2);
+  std::vector<double> grid_corner_lat(nlat * 2);
 
-  gridInqXvals(gridID, grid_center_lon.data());
-  gridInqYvals(gridID, grid_center_lat.data());
-
-  std::vector<double> grid_corner_lon(2 * nlon);
-  std::vector<double> grid_corner_lat(2 * nlat);
-
-  if (gridInqYbounds(gridID, nullptr) && gridInqXbounds(gridID, nullptr))
+  if (gridInqXbounds(gridID, nullptr))
     {
       gridInqXbounds(gridID, grid_corner_lon.data());
-      gridInqYbounds(gridID, grid_corner_lat.data());
+      gridInqXunits(gridID, unitstr);
     }
   else if (lweights && nlon == 1)
     {
       grid_corner_lon[0] = 0;
-      grid_corner_lon[1] = 1;
-      grid_gen_bounds(nlat, grid_center_lat.data(), grid_corner_lat.data());
-      grid_check_lat_borders(2 * nlat, grid_corner_lat.data());
+      grid_corner_lon[1] = .01;
+      strcpy(unitstr, "radian");
     }
-  else if (nlon == 1 || nlat == 1)
+  else if (nlon == 1)
     {
       return 1;
     }
   else
     {
-      grid_gen_bounds(nlon, grid_center_lon.data(), grid_corner_lon.data());
-      grid_gen_bounds(nlat, grid_center_lat.data(), grid_corner_lat.data());
-      grid_check_lat_borders(2 * nlat, grid_corner_lat.data());
+      std::vector<double> grid_center_lon(nlon);
+      gridInqXvals(gridID, grid_center_lon.data());
+      grid_gen_bounds(nlon, grid_center_lon, grid_corner_lon);
+      gridInqXunits(gridID, unitstr);
     }
 
-  char unitstr[CDI_MAX_NAME];
+  if (gridInqYbounds(gridID, nullptr))
+    {
+      gridInqYbounds(gridID, grid_corner_lat.data());
+      gridInqYunits(gridID, unitstr);
+    }
+  else if (lweights && nlat == 1)
+    {
+      grid_corner_lat[0] = 0;
+      grid_corner_lat[1] = .01;
+      strcpy(unitstr, "radian");
+    }
+  else if (nlat == 1)
+    {
+      return 1;
+    }
+  else
+    {
+      std::vector<double> grid_center_lat(nlat);
+      gridInqYvals(gridID, grid_center_lat.data());
+      grid_gen_bounds(nlat, grid_center_lat, grid_corner_lat);
+      grid_check_lat_borders(nlat * 2, grid_corner_lat.data());
+      gridInqYunits(gridID, unitstr);
+    }
 
-  gridInqXunits(gridID, unitstr);
-  grid_to_radian(unitstr, nlon, grid_center_lon.data(), "grid1 center longitudes");
-  grid_to_radian(unitstr, nlon * 2, grid_corner_lon.data(), "grid1 corner longitudes");
-
-  gridInqYunits(gridID, unitstr);
-  grid_to_radian(unitstr, nlat, grid_center_lat.data(), "grid1 center latitudes");
-  grid_to_radian(unitstr, nlat * 2, grid_corner_lat.data(), "grid1 corner latitudes");
+  grid_to_radian(unitstr, nlon * 2, grid_corner_lon.data(), "grid cell corner longitudes");
+  grid_to_radian(unitstr, nlat * 2, grid_corner_lat.data(), "grid cell corner latitudes");
 
   int nv = 4;
   double findex = 0;
@@ -350,7 +365,7 @@ gridGenAreaReg2D(const int gridID, double *area, bool lweights)
   Progress::init();
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(findex, area, nv, grid_corner_lon, grid_corner_lat)
+#pragma omp parallel for default(none) shared(gridsize, nlon, findex, area, nv, grid_corner_lon, grid_corner_lat)
 #endif
   for (size_t i = 0; i < gridsize; ++i)
     {
@@ -371,7 +386,8 @@ gridGenAreaReg2D(const int gridID, double *area, bool lweights)
   return status;
 }
 
-int gridGenAreaReg2Dweights(const int gridID, double *area)
+int
+gridGenAreaReg2Dweights(const int gridID, double *area)
 {
   bool lweights = true;
   return gridGenAreaReg2D(gridID, area, lweights);
@@ -389,7 +405,7 @@ gridGenAreaUnstruct(int gridID, double *area)
 
   if (gridtype != GRID_UNSTRUCTURED && gridtype != GRID_CURVILINEAR)
     {
-      if (gridtype == GRID_GME)
+      if (gridtype == GRID_GME || gridtype == GRID_GAUSSIAN_REDUCED)
         {
           lgriddestroy = true;
           gridID = gridToUnstructured(gridID, 1);
@@ -458,11 +474,9 @@ gridGenAreaUnstruct(int gridID, double *area)
     {
       if (lgrid_gen_bounds)
         {
-          size_t nlon = gridInqXsize(gridID);
-          size_t nlat = gridInqYsize(gridID);
-          double dlon = 0;
-          if (nlon == 1) dlon = 1;
-
+          const size_t nlon = gridInqXsize(gridID);
+          const size_t nlat = gridInqYsize(gridID);
+          const double dlon = (nlon == 1) ? 1 : 0;
           grid_cell_center_to_bounds_X2D(xunitstr, nlon, nlat, grid_center_lon.data(), grid_corner_lon.data(), dlon);
           grid_cell_center_to_bounds_Y2D(yunitstr, nlon, nlat, grid_center_lat.data(), grid_corner_lat.data());
         }
@@ -486,8 +500,8 @@ gridGenAreaUnstruct(int gridID, double *area)
   Progress::init();
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(findex, gridsize, area, nv, grid_corner_lon, grid_corner_lat, grid_center_lon, \
-                                              grid_center_lat)
+#pragma omp parallel for default(none) \
+  shared(gridsize, nv, findex, area, grid_corner_lon, grid_corner_lat, grid_center_lon, grid_center_lat)
 #endif
   for (size_t i = 0; i < gridsize; ++i)
     {
@@ -525,7 +539,8 @@ gridGenArea(const int gridID, double *area)
       status = gridGenAreaReg2D(gridID, area, lweights);
     }
   else if (projtype == CDI_PROJ_RLL || projtype == CDI_PROJ_LAEA || projtype == CDI_PROJ_SINU || projtype == CDI_PROJ_LCC
-           || projtype == CDI_PROJ_STERE || gridtype == GRID_GME || gridtype == GRID_CURVILINEAR || gridtype == GRID_UNSTRUCTURED)
+           || projtype == CDI_PROJ_STERE || gridtype == GRID_GME || gridtype == GRID_CURVILINEAR || gridtype == GRID_UNSTRUCTURED
+           || gridtype == GRID_GAUSSIAN_REDUCED)
     {
       status = gridGenAreaUnstruct(gridID, area);
     }
