@@ -15,9 +15,8 @@
   GNU General Public License for more details.
 */
 #include <cdi.h>
-#include "cdo_debugOutput.h"
 #include "cdo_int.h"
-#include "grid.h"
+#include <mpim_grid.h>
 
 #include <cstring>
 
@@ -55,11 +54,11 @@ cdo_define_destagered_grid(int gridID_u_stag, int gridID_v_stag, double *destagG
   degrees rlat      : first = -30.8  last = 24.1  inc = 0.1  degrees northpole :
   lon = -195  lat = 30
   */
-  if (CdoDebug::cdoDebugExt)
+  if (cdoDebugExt)
     cdoPrint("%s(gridID_u=%d,gridID_v=%d,destagGridOffsets(%02.1f,%02.1f)) ...\n", __func__, gridID_u_stag, gridID_v_stag,
              destagGridOffsets[0], destagGridOffsets[1]);
 
-  if (CdoDebug::cdoDebugExt > 1)
+  if (cdoDebugExt > 1)
     {
       cdoPrintGrid(gridID_u_stag, 1);
       cdoPrintGrid(gridID_v_stag, 1);
@@ -82,7 +81,7 @@ cdo_define_destagered_grid(int gridID_u_stag, int gridID_v_stag, double *destagG
 
   int gridID_uv_destag = gridDuplicate(gridID_u_stag);
 
-  if (CdoDebug::cdoDebugExt)
+  if (cdoDebugExt)
     {
       cdoPrintGrid(gridID_uv_destag, 1);
 
@@ -121,7 +120,7 @@ cdo_define_destagered_grid(int gridID_u_stag, int gridID_v_stag, double *destagG
   gridGenYvals(gridtype, ysize, yfirst, ylast, yinc, yvals.data());
   gridDefYvals(gridID_uv_destag, yvals.data());
 
-  if (CdoDebug::cdoDebugExt)
+  if (cdoDebugExt)
     {
       cdoPrint("%s():", __func__);
       cdoPrintGrid(gridID_uv_destag, 1);
@@ -191,7 +190,7 @@ cdo_define_sample_grid(int gridSrcID, int sampleFactor)
               longitudeOfFirstGridPointInDegrees = -7.89
               latitudeOfFirstGridPointInDegrees = 42.935
   */
-  if (CdoDebug::cdoDebugExt) cdoPrint("%s(gridSrcID=%d, sampleFactor=%d) ...", __func__, gridSrcID, sampleFactor);
+  Debug(cdoDebugExt,"%s(gridSrcID=%d, sampleFactor=%d) ...", __func__, gridSrcID, sampleFactor);
 
   int gridtype = gridInqType(gridSrcID);
   if (!(gridtype == GRID_GAUSSIAN || gridtype == GRID_LONLAT || gridtype == GRID_PROJECTION || gridtype == GRID_CURVILINEAR
@@ -206,7 +205,7 @@ cdo_define_sample_grid(int gridSrcID, int sampleFactor)
     cdoAbort("%s(): Unsupported sampleFactor (%d)! Note that: gridXsize = %d, gridYsize = %d", __func__, sampleFactor, gridXsize,
              gridYsize);
 
-  if (CdoDebug::cdoDebugExt > 20) cdoPrintGrid(gridSrcID, 1);
+  if (cdoDebugExt > 20) cdoPrintGrid(gridSrcID, 1);
 
   int xsize = (gridXsize + (sampleFactor - 1)) / sampleFactor;  // HARM36_L25: (789 + 2-1) / 2 = 395
   int ysize = (gridYsize + (sampleFactor - 1)) / sampleFactor;
@@ -218,7 +217,6 @@ cdo_define_sample_grid(int gridSrcID, int sampleFactor)
 
   gridDefNP(gridID_sampled, gridInqNP(gridSrcID));
   gridDefDatatype(gridID_sampled, gridInqDatatype(gridSrcID));
-  if (gridInqUvRelativeToGrid(gridSrcID)) gridDefUvRelativeToGrid(gridID_sampled, 1);
 
   grid_copy_attributes(gridSrcID, gridID_sampled);
 
@@ -256,7 +254,7 @@ cdo_define_sample_grid(int gridSrcID, int sampleFactor)
         }
     }
 
-  if (CdoDebug::cdoDebugExt > 20)
+  if (cdoDebugExt > 20)
     {
       cdoPrint("cdo SampleGrid: define_sample_grid(): ");
       cdoPrintGrid(gridID_sampled, 1);
@@ -327,7 +325,7 @@ cdo_define_subgrid_grid(int gridSrcID, int subI0, int subI1, int subJ0, int subJ
               longitudeOfFirstGridPointInDegrees = ...
               latitudeOfFirstGridPointInDegrees = ...
   */
-  if (CdoDebug::cdoDebugExt)
+  if (cdoDebugExt)
     cdoPrint("%s(gridSrcID=%d, (subI0,subI1,subJ0,subJ1) = (%d,%d,%d,%d) ...", __func__, gridSrcID, subI0, subI1, subJ0, subJ1);
 
   int gridXsize = gridInqXsize(gridSrcID);
@@ -351,9 +349,9 @@ cdo_define_subgrid_grid(int gridSrcID, int subI0, int subI1, int subJ0, int subJ
   x_0 = grid_missval;
   y_0 = grid_missval;
 
-  if (CdoDebug::cdoDebugExt > 20) cdoPrintGrid(gridSrcID, 1);
+  if (cdoDebugExt > 20) cdoPrintGrid(gridSrcID, 1);
 
-  if (CdoDebug::cdoDebugExt)
+  if (cdoDebugExt)
     {
       cdoPrint("%s() Original LCC grid:", __func__);
       cdoPrint("grid Xsize   %d, grid Ysize   %d", gridXsize, gridYsize);
@@ -365,7 +363,7 @@ cdo_define_subgrid_grid(int gridSrcID, int subI0, int subI1, int subJ0, int subJ
   xval_0 = gridInqXval(gridIDcurvl, 0);
   yval_0 = gridInqYval(gridIDcurvl, 0);
 
-  if (CdoDebug::cdoDebugExt)
+  if (cdoDebugExt)
     {
       cdoPrint("%s() Original LCC grid as curvilinear (with lats-lons computed):", __func__);
       cdoPrint("grid Xsize   %zu, grid Ysize   %zu", gridInqXsize(gridIDcurvl), gridInqYsize(gridIDcurvl));
@@ -395,14 +393,13 @@ cdo_define_subgrid_grid(int gridSrcID, int subI0, int subI1, int subJ0, int subJ
 
   gridDefNP(gridID_sampled, gridInqNP(gridSrcID));
   gridDefDatatype(gridID_sampled, gridInqDatatype(gridSrcID));
-  if (gridInqUvRelativeToGrid(gridSrcID)) gridDefUvRelativeToGrid(gridID_sampled, 1);
 
   grid_copy_attributes(gridSrcID, gridID_sampled);
 
   xval_0 = gridInqXval(gridIDcurvl, subJ0 * gridXsize + subI0);
   yval_0 = gridInqYval(gridIDcurvl, subJ0 * gridXsize + subI0);
 
-  if (CdoDebug::cdoDebugExt)
+  if (cdoDebugExt)
     {
       cdoPrint("%s()  Sub-grid:", __func__);
       cdoPrint("grid Xsize   %zu, grid Ysize   %zu", gridInqXsize(gridID_sampled), gridInqYsize(gridID_sampled));
@@ -413,7 +410,7 @@ cdo_define_subgrid_grid(int gridSrcID, int subI0, int subI1, int subJ0, int subJ
 
   gridDestroy(gridIDcurvl);
 
-  if (CdoDebug::cdoDebugExt > 20)
+  if (cdoDebugExt > 20)
     {
       cdoPrint("%s(): ", __func__);
       cdoPrintGrid(gridID_sampled, 1);
