@@ -15,9 +15,11 @@
   GNU General Public License for more details.
 */
 
+#include <valarray>
+
 #include <cdi.h>
 #include "cdo_int.h"
-#include "grid.h"
+#include <mpim_grid.h>
 #include "griddes.h"
 #include "util_string.h"
 
@@ -198,17 +200,16 @@ gen_grid_lonlat(GridDesciption &grid, const char *pline, double inc, double lon1
 
       if (lbounds && nlon > 1 && nlat > 1)
         {
-          std::vector<double> xbounds(2 * nlon);
-          std::vector<double> ybounds(2 * nlat);
+          std::vector<double> xbounds(2 * nlon), ybounds(2 * nlat);
 
-          grid_gen_bounds(nlon, grid.xvals.data(), xbounds.data());
-          grid_gen_bounds(nlat, yvals.data(), ybounds.data());
+          grid_gen_bounds(nlon, grid.xvals, xbounds);
+          grid_gen_bounds(nlat, yvals, ybounds);
           grid_check_lat_borders(2 * nlat, ybounds.data());
 
           grid.xbounds.resize(4 * gridsize);
           grid.ybounds.resize(4 * gridsize);
-          grid_gen_xbounds2D(nlon, nlat, xbounds.data(), grid.xbounds.data());
-          grid_gen_ybounds2D(nlon, nlat, ybounds.data(), grid.ybounds.data());
+          grid_gen_xbounds2D(nlon, nlat, xbounds, grid.xbounds);
+          grid_gen_ybounds2D(nlon, nlat, ybounds, grid.ybounds);
         }
     }
 }
@@ -231,6 +232,98 @@ gen_grid_gme(GridDesciption &grid, const char *pline)
     }
 }
 
+static void
+gaussianLatitudesInDegrees(std::vector<double> &lats, std::vector<double> &lat_bounds, size_t nlat)
+{
+  // lats(nlat)
+  // lat_bounds(nlat+1)
+  std::vector<double> latw(nlat), latw_cumsum(nlat);
+
+  gaussianLatitudes(&lats[0], &latw[0], nlat);
+
+  for (size_t j = 0; j < nlat; ++j) lats[j] = RAD2DEG*std::asin(lats[j]);
+
+  latw_cumsum[0] = latw[0];
+  for (size_t j = 1; j < nlat; ++j) latw_cumsum[j] = latw_cumsum[j-1] + latw[j];
+
+  lat_bounds[0] = 1.0;
+  for (size_t j = 1; j < nlat; ++j) lat_bounds[j] = 1.0 - latw_cumsum[j-1];
+  lat_bounds[nlat] = -1.0;
+  
+  for (size_t j = 0; j < nlat+1; ++j) lat_bounds[j] = RAD2DEG*std::asin(lat_bounds[j]);
+}
+
+static void
+gen_grid_gea(GridDesciption &grid, const char *pline)
+{
+  if (isdigit((int) *pline))
+    {
+      char *endptr = (char *) pline;
+      const double dx = strtod(pline, &endptr);
+      const double dy = dx;
+      if (*endptr == 0)
+        {
+          constexpr double re = 6378.137;
+          constexpr double f = 1.0 / 298.257223563;
+          constexpr double rp = re * (1.0 - f);
+          constexpr double polar_circumference = 2.0 * M_PI * rp;
+          constexpr double equator_circumference = 2.0 * M_PI * re;
+
+          size_t nlat = 0.5 * polar_circumference / dy;
+          if (nlat&2) nlat = nlat + 1;
+
+          std::vector<double> lats(nlat), lat_bounds(nlat+1);
+          gaussianLatitudesInDegrees(lats, lat_bounds, nlat);
+
+          std::vector<double> cell_height(nlat);
+          for (size_t j = 0; j < nlat; ++j) cell_height[j] = 0.25 * polar_circumference * (lat_bounds[j]-lat_bounds[j+1]) / 90.0;
+
+          size_t nlone = equator_circumference / dx;
+          if (nlone%2) nlone++;
+
+          std::vector<int> rowlon(nlat);
+          size_t ncells = 0;
+          for (size_t j = 0; j < nlat; ++j)
+            {
+              const double rlat = re*std::cos(DEG2RAD*lats[j]);
+              const double circumference = 2.0 * M_PI * rlat;
+              const double dx_to_use = dx * dy / cell_height[j];
+    
+              size_t nlon = std::max((int)std::lround(circumference / dx_to_use), 1);
+              if (nlon%2) nlon++;    
+
+              rowlon[j] = nlon;
+              ncells += nlon;
+            }
+
+          // printf("%zu %zu %zu %zu %g\n", ncells, nlone, nlat, nlone*nlat, 100.0*ncells/(nlone*nlat));
+
+          std::vector<double> lons(ncells);
+          size_t ij = 0;
+          for (size_t j = 0; j < nlat; ++j)
+            {
+              const size_t nlon = rowlon[j];
+              for (size_t i = 0; i < nlon; ++i) lons[ij++] = i * 360. / nlon;
+            }
+
+          grid.type = GRID_GAUSSIAN_REDUCED;
+          grid.size = ncells;
+          grid.xsize = ncells;
+          grid.ysize = nlat;
+          grid.np = nlat/2;
+          grid.xvals.resize(ncells);
+          grid.yvals.resize(nlat);
+          grid.ybounds.resize(nlat*2);
+          grid.rowlon.resize(nlat);
+          for (size_t i = 0; i < ncells; ++i) grid.xvals[i] = lons[i];
+          for (size_t j = 0; j < nlat; ++j) grid.yvals[j] = lats[j];
+          for (size_t j = 0; j < nlat; ++j) grid.ybounds[j*2+1] = lat_bounds[j];
+          for (size_t j = 0; j < nlat; ++j) grid.ybounds[j*2] = lat_bounds[j+1];
+          for (size_t j = 0; j < nlat; ++j) grid.rowlon[j] = rowlon[j];
+        }
+    }
+}
+
 static void
 gen_grid_reg2d(GridDesciption &grid, const char *pline)
 {
@@ -239,7 +332,8 @@ gen_grid_reg2d(GridDesciption &grid, const char *pline)
       grid.type = GRID_LONLAT;
       grid.xsize = atoi(pline);
       while (isdigit((int) *pline)) pline++;
-      if (*pline == 'x' || *pline == '/' || *pline == '_') pline++;
+      if (*pline == 'x' || *pline == '/' || *pline == '_')
+        pline++;
       else
         {
           grid.type = CDI_UNDEFID;
@@ -247,7 +341,7 @@ gen_grid_reg2d(GridDesciption &grid, const char *pline)
         }
       grid.ysize = atoi(pline);
       while (isdigit((int) *pline)) pline++;
-          
+
       grid.def_xfirst = true;
       grid.def_yfirst = true;
     }
@@ -289,9 +383,9 @@ gridFromName(const char *gridnameptr)
   char *endptr;
 
   char *gridname = strdup(gridnameptr);
-  strtolower(gridname);
+  cstrToLowerCase(gridname);
 
-  if (gridname[0] == 't') // t<RES>grid or t<RES>spec
+  if (gridname[0] == 't')  // t<RES>grid or t<RES>spec
     {
       int off = 0;
       int type = 'q';
@@ -306,7 +400,7 @@ gridFromName(const char *gridnameptr)
           off = 1;
         }
 
-      pline = &gridname[off+1];
+      pline = &gridname[off + 1];
       if (isdigit((int) *pline))
         {
           grid.ntr = atoi(pline);
@@ -330,35 +424,51 @@ gridFromName(const char *gridnameptr)
                 grid.ysize = ntr_to_nlat(grid.ntr);
 
               grid.np = grid.ysize / 2;
-              grid.xsize = (cmpstrlen(pline, "zon", len) == 0) ? 1 :
-                ((type == 'c') ? nlat_to_nlon_cubic(grid.ysize) : nlat_to_nlon(grid.ysize));
+              grid.xsize = (cmpstrlen(pline, "zon", len) == 0)
+                               ? 1
+                               : ((type == 'c') ? nlat_to_nlon_cubic(grid.ysize) : nlat_to_nlon(grid.ysize));
 
               grid.def_xfirst = true;
               grid.def_yfirst = true;
+              grid.yvals.resize(grid.ysize);
+              grid.ybounds.resize(grid.ysize*2);
+
+              size_t nlat = grid.ysize;
+              std::vector<double> lats(nlat), lat_bounds(nlat+1);
+              gaussianLatitudesInDegrees(lats, lat_bounds, nlat);
+
+              for (size_t j = 0; j < nlat; ++j) grid.yvals[j] = lats[j];
+              for (size_t j = 0; j < nlat; ++j) grid.ybounds[j*2+1] = lat_bounds[j];
+              for (size_t j = 0; j < nlat; ++j) grid.ybounds[j*2] = lat_bounds[j+1];
             }
         }
     }
-  else if (gridname[0] == 'r') // r<LON>x<LAT>; regular 2D grid
+  else if (gridname[0] == 'r')  // r<LON>x<LAT>; regular 2D grid
     {
       pline = &gridname[1];
       gen_grid_reg2d(grid, pline);
     }
-  else if (cmpstrlen(gridname, "lon=", len) == 0) // lon=<LON>_lat=<LAT>; one gridpoint
+  else if (cmpstrlen(gridname, "lon=", len) == 0)  // lon=<LON>_lat=<LAT>; one gridpoint
     {
       pline = &gridname[len];
       gen_grid_point(grid, pline);
     }
-  else if (gridname[0] == 'g' && gridname[1] == 'm' && gridname[2] == 'e') // gme<NI>
+  else if (gridname[0] == 'g' && gridname[1] == 'm' && gridname[2] == 'e')  // gme<NI>
     {
       pline = &gridname[3];
       gen_grid_gme(grid, pline);
     }
-  else if (gridname[0] == 'n' && gridname[1] == 'i') // ni<NI>
+  else if (gridname[0] == 'n' && gridname[1] == 'i')  // ni<NI>
     {
       pline = &gridname[2];
       gen_grid_gme(grid, pline);
     }
-  else if (gridname[0] == 'n') // N<xxx>
+  else if (gridname[0] == 'g' && gridname[1] == 'e' && gridname[2] == 'a')  // gea<DX>: gaussian reduced equal area
+    {
+      pline = &gridname[3];
+      gen_grid_gea(grid, pline);
+    }
+  else if (gridname[0] == 'n')  // N<xxx>
     {
       pline = &gridname[1];
       if (isdigit((int) *pline))
@@ -389,7 +499,7 @@ gridFromName(const char *gridnameptr)
             }
         }
     }
-  else if (gridname[0] == 'o') // O<xxx>
+  else if (gridname[0] == 'o')  // O<xxx>
     {
       pline = &gridname[1];
       if (isdigit((int) *pline))
@@ -420,7 +530,7 @@ gridFromName(const char *gridnameptr)
             }
         }
     }
-  else if (gridname[0] == 'g' && isdigit(gridname[1])) // g<LON>x<LAT> or g<SIZE>
+  else if (gridname[0] == 'g' && isdigit(gridname[1]))  // g<LON>x<LAT> or g<SIZE>
     {
       pline = &gridname[1];
       if (isdigit((int) *pline))
