@@ -56,12 +56,12 @@ init(void)
 
 // 20 triangles
 static const TriangleList triangles
-    = { {{ 0, 1, 2 }},  {{ 0, 2, 3 }},  {{ 0, 3, 4 }},  {{ 0, 4, 5 }},  {{ 0, 5, 1 }},   {{ 6, 2, 1 }},  {{ 7, 3, 2 }},
-        {{ 8, 4, 3 }},  {{ 9, 5, 4 }},  {{ 10, 1, 5 }}, {{ 2, 6, 7 }},  {{ 3, 7, 8 }},   {{ 4, 8, 9 }},  {{ 5, 9, 10 }},
-        {{ 1, 10, 6 }}, {{ 11, 7, 6 }}, {{ 11, 8, 7 }}, {{ 11, 9, 8 }}, {{ 11, 10, 9 }}, {{ 11, 6, 10 }} };
+    = { { { 0, 1, 2 } },  { { 0, 2, 3 } },  { { 0, 3, 4 } },  { { 0, 4, 5 } },   { { 0, 5, 1 } },
+        { { 6, 2, 1 } },  { { 7, 3, 2 } },  { { 8, 4, 3 } },  { { 9, 5, 4 } },   { { 10, 1, 5 } },
+        { { 2, 6, 7 } },  { { 3, 7, 8 } },  { { 4, 8, 9 } },  { { 5, 9, 10 } },  { { 1, 10, 6 } },
+        { { 11, 7, 6 } }, { { 11, 8, 7 } }, { { 11, 9, 8 } }, { { 11, 10, 9 } }, { { 11, 6, 10 } } };
 }  // namespace icosahedron
 
-
 using Lookup = std::map<std::pair<Index, Index>, Index>;
 
 static Index
@@ -71,8 +71,7 @@ vertexForEdge(Lookup &lookup, VertexList &vertices, Index first, Index second)
   if (key.first > key.second) std::swap(key.first, key.second);
 
   const auto inserted = lookup.insert({ key, vertices.size() });
-  if (inserted.second)
-    vertices.push_back((vertices[first] + vertices[second]).normalised());
+  if (inserted.second) vertices.push_back((vertices[first] + vertices[second]).normalised());
 
   return inserted.first->second;
 }
@@ -84,18 +83,17 @@ subdivide(VertexList &vertices, TriangleList triangles)
   Triangle mid;
 
   const size_t n = triangles.size();
-  TriangleList result(4*n);
+  TriangleList result(4 * n);
   for (size_t i = 0; i < n; ++i)
     {
       const auto &each = triangles[i];
 
-      for (int edge = 0; edge < 3; ++edge)
-        mid[edge] = vertexForEdge(lookup, vertices, each[edge], each[(edge + 1) % 3]);
+      for (int edge = 0; edge < 3; ++edge) mid[edge] = vertexForEdge(lookup, vertices, each[edge], each[(edge + 1) % 3]);
 
-      result[i*4+0] = {{ each[0], mid[0], mid[2] }};
-      result[i*4+1] = {{ each[1], mid[1], mid[0] }};
-      result[i*4+2] = {{ each[2], mid[2], mid[1] }};
-      result[i*4+3] = mid;
+      result[i * 4 + 0] = { { each[0], mid[0], mid[2] } };
+      result[i * 4 + 1] = { { each[1], mid[1], mid[0] } };
+      result[i * 4 + 2] = { { each[2], mid[2], mid[1] } };
+      result[i * 4 + 3] = mid;
     }
 
   return result;
