@@ -25,11 +25,12 @@
 #include "cdo_options.h"
 #include "dmemory.h"
 
-#include "grid.h"
 #include "grid_point_search.h"
+#include <mpim_grid.h>
 #include "kdtreelib/kdtree.h"
 #include "nanoflann.hpp"
-extern "C" {
+extern "C"
+{
 #include "lib/yac/sphere_part.h"
 }
 
@@ -197,12 +198,12 @@ gps_create_kdtree(size_t n, const double *restrict lons, const double *restrict
   kdata_t max[3] = { -1.e9, -1.e9, -1.e9 };
 
 #ifdef HAVE_OPENMP45
-#pragma omp parallel for reduction(min : min[ : 3]) reduction(max : max[ : 3])
+#pragma omp parallel for reduction(min : min[:3]) reduction(max : max[:3])
 #endif
   for (size_t i = 0; i < n; i++)
     {
       kdata_t *restrict point = pointlist[i].point;
-      cdoLLtoXYZ(lons[i], lats[i], point);
+      gcLLtoXYZ(lons[i], lats[i], point);
       for (unsigned j = 0; j < 3; ++j)
         {
           min[j] = point[j] < min[j] ? point[j] : min[j];
@@ -239,12 +240,12 @@ gps_create_nanoflann(size_t n, const double *restrict lons, const double *restri
   // Generating  Point Cloud
   pointcloud->pts.resize(n);
 #ifdef HAVE_OPENMP45
-#pragma omp parallel for reduction(min : min[ : 3]) reduction(max : max[ : 3])
+#pragma omp parallel for reduction(min : min[:3]) reduction(max : max[:3])
 #endif
   for (size_t i = 0; i < n; i++)
     {
       double point[3];
-      cdoLLtoXYZ(lons[i], lats[i], point);
+      gcLLtoXYZ(lons[i], lats[i], point);
       pointcloud->pts[i].x = point[0];
       pointcloud->pts[i].y = point[1];
       pointcloud->pts[i].z = point[2];
@@ -289,12 +290,12 @@ gps_create_spherepart(size_t n, const double *restrict lons, const double *restr
   double max[3] = { -1.e9, -1.e9, -1.e9 };
 
 #ifdef HAVE_OPENMP45
-#pragma omp parallel for reduction(min : min[ : 3]) reduction(max : max[ : 3])
+#pragma omp parallel for reduction(min : min[:3]) reduction(max : max[:3])
 #endif
   for (size_t i = 0; i < n; i++)
     {
       double *restrict point = coordinates_xyz[i];
-      cdoLLtoXYZ(lons[i], lats[i], point);
+      gcLLtoXYZ(lons[i], lats[i], point);
       for (unsigned j = 0; j < 3; ++j)
         {
           min[j] = point[j] < min[j] ? point[j] : min[j];
@@ -360,7 +361,7 @@ gps_create_full(size_t n, const double *restrict lons, const double *restrict la
 #endif
   for (size_t i = 0; i < n; i++)
     {
-      cdoLLtoXYZ(lons[i], lats[i], p[i]);
+      gcLLtoXYZ(lons[i], lats[i], p[i]);
     }
 
   full->n = n;
@@ -454,7 +455,7 @@ gps_nearest_kdtree(void *search_container, double lon, double lat, double search
   double sqrDist = sqrDistMax;
 
   double query_pt[3];
-  cdoLLtoXYZ(lon, lat, query_pt);
+  gcLLtoXYZ(lon, lat, query_pt);
 
   if (!gps->extrapolate)
     for (unsigned j = 0; j < 3; ++j)
@@ -482,7 +483,7 @@ gps_nearest_nanoflann(void *search_container, double lon, double lat, double sea
   double sqrDistMax = cdo::sqr(searchRadius);
 
   double query_pt[3];
-  cdoLLtoXYZ(lon, lat, query_pt);
+  gcLLtoXYZ(lon, lat, query_pt);
 
   if (!gps->extrapolate)
     for (unsigned j = 0; j < 3; ++j)
@@ -510,7 +511,7 @@ gps_nearest_spherepart(void *search_container, double lon, double lat, double se
                        GridPointSearch *gps)
 {
   double query_pt[1][3];
-  cdoLLtoXYZ(lon, lat, query_pt[0]);
+  gcLLtoXYZ(lon, lat, query_pt[0]);
 
   if (!gps->extrapolate)
     for (unsigned j = 0; j < 3; ++j)
@@ -551,7 +552,7 @@ gps_nearest_full(void *search_container, double lon, double lat, double searchRa
   double sqrDistMax = cdo::sqr(searchRadius);
 
   double query_pt[3];
-  cdoLLtoXYZ(lon, lat, query_pt);
+  gcLLtoXYZ(lon, lat, query_pt);
 
   size_t n = full->n;
   size_t closestpt = n;
@@ -646,7 +647,7 @@ gps_qnearest_kdtree(GridPointSearch *gps, double lon, double lat, double searchR
   struct pqueue *result = nullptr;
 
   kdata_t query_pt[3];
-  cdoLLtoXYZ(lon, lat, query_pt);
+  gcLLtoXYZ(lon, lat, query_pt);
 
   if (!gps->extrapolate)
     for (unsigned j = 0; j < 3; ++j)
@@ -691,7 +692,7 @@ gps_qnearest_nanoflann(GridPointSearch *gps, double lon, double lat, double sear
   double sqrDistMax = cdo::sqr(searchRadius);
 
   double query_pt[3];
-  cdoLLtoXYZ(lon, lat, query_pt);
+  gcLLtoXYZ(lon, lat, query_pt);
 
   if (!gps->extrapolate)
     for (unsigned j = 0; j < 3; ++j)
@@ -711,7 +712,7 @@ gps_qnearest_spherepart(GridPointSearch *gps, double lon, double lat, double sea
   if (gps)
     {
       double query_pt[1][3];
-      cdoLLtoXYZ(lon, lat, query_pt[0]);
+      gcLLtoXYZ(lon, lat, query_pt[0]);
 
       if (!gps->extrapolate)
         for (unsigned j = 0; j < 3; ++j)
