@@ -117,16 +117,19 @@ grid_print_attributes(FILE *fp, int gridID)
 
       if (atttype == CDI_DATATYPE_TXT)
         {
-          size_t attSize = (size_t)(attlen + 1) * sizeof(char);
+          const size_t attSize = (size_t)(attlen + 1) * sizeof(char);
           char *atttxt = (char *) resizeBuffer(&attBuf, &attBufSize, attSize);
           cdiInqAttTxt(cdiID, varID, attname, attlen, atttxt);
           atttxt[attlen] = 0;
-          fprintf(fp, "%s = \"%s\"\n", attname, atttxt);
+          if (strchr(atttxt, '"'))
+            fprintf(fp, "%s = '%s'\n", attname, atttxt);
+          else
+            fprintf(fp, "%s = \"%s\"\n", attname, atttxt);
         }
       else if (atttype == CDI_DATATYPE_INT8 || atttype == CDI_DATATYPE_UINT8 || atttype == CDI_DATATYPE_INT16
                || atttype == CDI_DATATYPE_UINT16 || atttype == CDI_DATATYPE_INT32 || atttype == CDI_DATATYPE_UINT32)
         {
-          size_t attSize = (size_t) attlen * sizeof(int);
+          const size_t attSize = (size_t) attlen * sizeof(int);
           int *attint = (int *) resizeBuffer(&attBuf, &attBufSize, attSize);
           cdiInqAttInt(cdiID, varID, attname, attlen, &attint[0]);
           fprintf(fp, "%s =", attname);
@@ -135,16 +138,16 @@ grid_print_attributes(FILE *fp, int gridID)
         }
       else if (atttype == CDI_DATATYPE_FLT32 || atttype == CDI_DATATYPE_FLT64)
         {
-          size_t attSize = (size_t) attlen * sizeof(double);
+          const size_t attSize = (size_t) attlen * sizeof(double);
           double *attflt = (double *) resizeBuffer(&attBuf, &attBufSize, attSize);
           cdiInqAttFlt(cdiID, varID, attname, attlen, attflt);
           fprintf(fp, "%s =", attname);
           if (atttype == CDI_DATATYPE_FLT32)
             for (int i = 0; i < attlen; ++i)
-              fprintf(fp, " %sf", double_to_attstr(Options::CDO_flt_digits, fltstr, sizeof(fltstr), attflt[i]));
+              fprintf(fp, " %sf", doubleToAttstr(Options::CDO_flt_digits, fltstr, sizeof(fltstr), attflt[i]));
           else
             for (int i = 0; i < attlen; ++i)
-              fprintf(fp, " %s", double_to_attstr(Options::CDO_dbl_digits, fltstr, sizeof(fltstr), attflt[i]));
+              fprintf(fp, " %s", doubleToAttstr(Options::CDO_dbl_digits, fltstr, sizeof(fltstr), attflt[i]));
           fprintf(fp, "\n");
         }
     }
@@ -158,19 +161,19 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
   size_t xdim, ydim;
   char attstr[CDI_MAX_NAME];
   char attstr2[CDI_MAX_NAME];
-  size_t nxvals = gridInqXvals(gridID, nullptr);
-  size_t nyvals = gridInqYvals(gridID, nullptr);
-  size_t nxbounds = gridInqXbounds(gridID, nullptr);
-  size_t nybounds = gridInqYbounds(gridID, nullptr);
+  const size_t nxvals = gridInqXvals(gridID, nullptr);
+  const size_t nyvals = gridInqYvals(gridID, nullptr);
+  const size_t nxbounds = gridInqXbounds(gridID, nullptr);
+  const size_t nybounds = gridInqYbounds(gridID, nullptr);
 
-  size_t gridsize = gridInqSize(gridID);
-  size_t xsize = gridInqXsize(gridID);
-  size_t ysize = gridInqYsize(gridID);
-  int type = gridInqType(gridID);
-  int nvertex = gridInqNvertex(gridID);
-  int datatype = gridInqDatatype(gridID);
-  int xstrlen = gridInqXIsc(gridID);
-  int ystrlen = gridInqYIsc(gridID);
+  const size_t gridsize = gridInqSize(gridID);
+  const size_t xsize = gridInqXsize(gridID);
+  const size_t ysize = gridInqYsize(gridID);
+  const int type = gridInqType(gridID);
+  const int nvertex = gridInqNvertex(gridID);
+  const int datatype = gridInqDatatype(gridID);
+  const int xstrlen = gridInqXIsc(gridID);
+  const int ystrlen = gridInqYIsc(gridID);
 
   char **xcvals = nullptr;
   if (xstrlen)
@@ -206,7 +209,7 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
           }
     }
 
-  int dig = (datatype == CDI_DATATYPE_FLT64) ? Options::CDO_dbl_digits : Options::CDO_flt_digits;
+  const int dig = (datatype == CDI_DATATYPE_FLT64) ? Options::CDO_dbl_digits : Options::CDO_flt_digits;
 
   fprintf(fp, "gridtype  = %s\n", gridNamePtr(type));
   fprintf(fp, "gridsize  = %zu\n", gridsize);
@@ -293,8 +296,8 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
 
         if (type == GRID_UNSTRUCTURED)
           {
-            int number = gridInqNumber(gridID);
-            int position = gridInqPosition(gridID);
+            const int number = gridInqNumber(gridID);
+            const int position = gridInqPosition(gridID);
             // const unsigned char *d;
             if (number > 0)
               {
@@ -310,7 +313,7 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
               }
           }
 
-        if (nxvals > 0 && type != GRID_GAUSSIAN_REDUCED)
+        if (nxvals > 0)
           {
             double xfirst = 0.0, xinc = 0.0;
 
@@ -420,9 +423,6 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
             printIntsPrefixAutoBrk(fp, prefix, (int) sizeof(prefix) - 1, (size_t)(ysize > 0 ? ysize : 0), rowlon.data());
           }
 
-        int uvRelativeToGrid = gridInqUvRelativeToGrid(gridID);
-        if (uvRelativeToGrid > 0) fprintf(fp, "uvRelativeToGrid = %d\n", uvRelativeToGrid);
-
 #ifdef HIRLAM_EXTENSIONS
         {
           int scanningMode = gridInqScanningMode(gridID);
@@ -485,7 +485,7 @@ grid_print_kernel(int gridID, int opt, FILE *fp)
       printMask(fp, prefix, sizeof(prefix) - 1, (size_t)(gridsize > 0 ? gridsize : 0), mask.data());
     }
 
-  int projID = gridInqProj(gridID);
+  const int projID = gridInqProj(gridID);
   if (projID != CDI_UNDEFID && gridInqType(projID) == GRID_PROJECTION) grid_print_kernel(projID, opt, fp);
 }
 
