@@ -206,8 +206,6 @@ grid_read_data(size_t ikv, size_t nkv, KVMap *kvmap, GridDesciption &grid, size_
         }
       else if (cstrIsEqual(key, "a"))
         grid.a = parameter2double(value);
-      else if (cstrIsEqual(key, "uvRelativeToGrid"))
-        grid.uvRelativeToGrid = parameter2bool(value);
       else if (cstrIsEqual(key, "xvals"))
         {
           size_t size = (grid.type == GRID_CURVILINEAR || grid.type == GRID_UNSTRUCTURED) ? grid.size : grid.xsize;
