@@ -15,10 +15,15 @@
   GNU General Public License for more details.
 */
 
+#include <ctype.h>
+
 #include <cdi.h>
-#include "cdo_int.h"
-#include "grid.h"
+
 #include "griddes.h"
+#include <mpim_grid.h>
+#include "cdo_output.h"
+#include "compare.h"
+
 
 static void
 skip_nondigit_lines(FILE *gfp)
@@ -76,9 +81,8 @@ input_darray(FILE *gfp, size_t n_values, double *array)
 }
 
 int
-grid_read_pingo(FILE *gfp, const char *dname)
+grid_read_pingo(FILE *gfp)
 {
-  UNUSED(dname);
   int gridID = -1;
   int i;
 
@@ -154,11 +158,11 @@ grid_read_pingo(FILE *gfp, const char *dname)
           }
 
       bool lgauss = false;
-      if (nlat > 2) /* check if gaussian */
+      if (nlat > 2) // check if gaussian
         {
           std::vector<double> yvals(grid.ysize);
           std::vector<double> yw(grid.ysize);
-          gaussaw(yvals.data(), yw.data(), grid.ysize);
+          gaussianLatitudes(yvals.data(), yw.data(), grid.ysize);
 
           for (i = 0; i < (int) grid.ysize; i++) yvals[i] = asin(yvals[i]) * RAD2DEG;
 
