@@ -24,12 +24,9 @@
 #include "cdo_options.h"
 #include "cdo_int.h"
 #include "griddes.h"
-#include "error.h"
-#include "cdo_debugOutput.h"
 #include "util_wildcards.h"
+#include "grid_read_pingo.h"
 
-int grid_read(FILE *gfp, const char *dname);
-int grid_read_pingo(FILE *gfp, const char *dname);
 
 void
 GridDesciption::init()
@@ -73,7 +70,6 @@ GridDesciption::init()
   this->xdimname[0] = 0;
   this->ydimname[0] = 0;
   this->vdimname[0] = 0;
-  this->uvRelativeToGrid = false;
   this->scanningMode = 64;
   /*
     scanningMode  = 128 * iScansNegatively + 64 * jScansPositively + 32 * jPointsAreConsecutive;
@@ -113,6 +109,43 @@ getoptname(char *optname, const char *optstring, int nopt)
   return nerr;
 }
 
+static void
+genXboundsRegular(GridDesciption &grid)
+{
+  grid.nvertex = 2;
+  grid.xbounds.resize(grid.xsize * grid.nvertex);
+  for (size_t i = 0; i < grid.xsize - 1; ++i)
+    {
+      grid.xbounds[2 * i + 1] = 0.5 * (grid.xvals[i] + grid.xvals[i + 1]);
+      grid.xbounds[2 * (i + 1)] = 0.5 * (grid.xvals[i] + grid.xvals[i + 1]);
+    }
+  grid.xbounds[0] = 2 * grid.xvals[0] - grid.xbounds[1];
+  grid.xbounds[2 * grid.xsize - 1] = 2 * grid.xvals[grid.xsize - 1] - grid.xbounds[2 * (grid.xsize - 1)];
+}
+
+static void
+genYboundsRegular(GridDesciption &grid)
+{
+  grid.nvertex = 2;
+  grid.ybounds.resize(grid.ysize * grid.nvertex);
+  for (size_t i = 0; i < grid.ysize - 1; ++i)
+    {
+      grid.ybounds[2 * i + 1] = 0.5 * (grid.yvals[i] + grid.yvals[i + 1]);
+      grid.ybounds[2 * (i + 1)] = 0.5 * (grid.yvals[i] + grid.yvals[i + 1]);
+    }
+
+  if (grid.yvals[0] > grid.yvals[grid.ysize - 1])
+    {
+      grid.ybounds[0] = 90;
+      grid.ybounds[grid.ysize * grid.nvertex - 1] = -90;
+    }
+  else
+    {
+      grid.ybounds[0] = -90;
+      grid.ybounds[grid.ysize * grid.nvertex - 1] = 90;
+    }
+}
+
 int
 gridDefine(GridDesciption &grid)
 {
@@ -128,17 +161,17 @@ gridDefine(GridDesciption &grid)
       {
         if (grid.size != 1)
           {
-            if (grid.xsize == 0 && grid.type != GRID_GAUSSIAN_REDUCED) Error("xsize undefined!");
-            if (grid.ysize == 0) Error("ysize undefined!");
+            if (grid.xsize == 0 && grid.type != GRID_GAUSSIAN_REDUCED) cdoAbort("xsize undefined!");
+            if (grid.ysize == 0) cdoAbort("ysize undefined!");
           }
 
         if (grid.size == 0) grid.size = grid.xsize * grid.ysize;
 
-        if (grid.xsize && grid.size != grid.xsize * grid.ysize)
-          Error("Inconsistent grid declaration: xsize*ysize!=gridsize (xsize=%zu ysize=%zu gridsize=%zu)", grid.xsize, grid.ysize,
+        if (grid.type != GRID_GAUSSIAN_REDUCED && grid.xsize && grid.size != grid.xsize * grid.ysize)
+          cdoAbort("Inconsistent grid declaration: xsize*ysize!=gridsize (xsize=%zu ysize=%zu gridsize=%zu)", grid.xsize, grid.ysize,
                 grid.size);
 
-        // if ( grid.size < 0 || grid.size > INT_MAX ) Error("grid size (%ld) out of bounds (0 - %d)!", grid.size, INT_MAX);
+        // if ( grid.size < 0 || grid.size > INT_MAX ) cdoAbort("grid size (%ld) out of bounds (0 - %d)!", grid.size, INT_MAX);
 
         gridID = gridCreate(grid.type, grid.size);
 
@@ -146,7 +179,6 @@ gridDefine(GridDesciption &grid)
         if (grid.ysize > 0) gridDefYsize(gridID, grid.ysize);
         if (grid.np > 0) gridDefNP(gridID, grid.np);
 
-        if (grid.uvRelativeToGrid) gridDefUvRelativeToGrid(gridID, 1);
         if (grid.nvertex) gridDefNvertex(gridID, grid.nvertex);
 
         if ((grid.def_xfirst || grid.def_xlast || grid.def_xinc) && grid.xvals.size() == 0)
@@ -154,18 +186,7 @@ gridDefine(GridDesciption &grid)
             grid.xvals.resize(grid.xsize);
             gridGenXvals(grid.xsize, grid.xfirst, grid.xlast, grid.xinc, grid.xvals.data());
 
-            if (grid.genBounds && grid.xbounds.size() == 0 && grid.xsize > 1)
-              {
-                grid.nvertex = 2;
-                grid.xbounds.resize(grid.xsize * grid.nvertex);
-                for (size_t i = 0; i < grid.xsize - 1; ++i)
-                  {
-                    grid.xbounds[2 * i + 1] = 0.5 * (grid.xvals[i] + grid.xvals[i + 1]);
-                    grid.xbounds[2 * (i + 1)] = 0.5 * (grid.xvals[i] + grid.xvals[i + 1]);
-                  }
-                grid.xbounds[0] = 2 * grid.xvals[0] - grid.xbounds[1];
-                grid.xbounds[2 * grid.xsize - 1] = 2 * grid.xvals[grid.xsize - 1] - grid.xbounds[2 * (grid.xsize - 1)];
-              }
+            if (grid.genBounds && grid.xbounds.size() == 0 && grid.xsize > 1) genXboundsRegular(grid);
           }
 
         if ((grid.def_yfirst || grid.def_ylast || grid.def_yinc) && grid.yvals.size() == 0)
@@ -174,27 +195,7 @@ gridDefine(GridDesciption &grid)
             grid.yvals.resize(grid.ysize);
             gridGenYvals(grid.type, grid.ysize, grid.yfirst, grid.ylast, grid.yinc, grid.yvals.data());
 
-            if (grid.genBounds && grid.ybounds.size() == 0 && grid.ysize > 1)
-              {
-                grid.nvertex = 2;
-                grid.ybounds.resize(grid.ysize * grid.nvertex);
-                for (size_t i = 0; i < grid.ysize - 1; ++i)
-                  {
-                    grid.ybounds[2 * i + 1] = 0.5 * (grid.yvals[i] + grid.yvals[i + 1]);
-                    grid.ybounds[2 * (i + 1)] = 0.5 * (grid.yvals[i] + grid.yvals[i + 1]);
-                  }
-
-                if (grid.yvals[0] > grid.yvals[grid.ysize - 1])
-                  {
-                    grid.ybounds[0] = 90;
-                    grid.ybounds[grid.ysize * grid.nvertex - 1] = -90;
-                  }
-                else
-                  {
-                    grid.ybounds[0] = -90;
-                    grid.ybounds[grid.ysize * grid.nvertex - 1] = 90;
-                  }
-              }
+            if (grid.genBounds && grid.ybounds.size() == 0 && grid.ysize > 1) genYboundsRegular(grid);
           }
 
         if (grid.xvals.size()) gridDefXvals(gridID, grid.xvals.data());
@@ -215,8 +216,8 @@ gridDefine(GridDesciption &grid)
 
         if (grid.type == GRID_CURVILINEAR)
           {
-            if (grid.xsize == 0) Error("xsize undefined!");
-            if (grid.ysize == 0) Error("ysize undefined!");
+            if (grid.xsize == 0) cdoAbort("xsize undefined!");
+            if (grid.ysize == 0) cdoAbort("ysize undefined!");
             gridDefXsize(gridID, grid.xsize);
             gridDefYsize(gridID, grid.ysize);
           }
@@ -242,7 +243,7 @@ gridDefine(GridDesciption &grid)
       }
     case GRID_SPECTRAL:
       {
-        if (grid.ntr == 0) Error("truncation undefined!");
+        if (grid.ntr == 0) cdoAbort("truncation undefined!");
         if (grid.size == 0) grid.size = (grid.ntr + 1) * (grid.ntr + 2);
 
         gridID = gridCreate(grid.type, grid.size);
@@ -254,9 +255,9 @@ gridDefine(GridDesciption &grid)
       }
     case GRID_GME:
       {
-        if (grid.nd == 0) Error("nd undefined!");
-        if (grid.ni == 0) Error("ni undefined!");
-        if (grid.size == 0) Error("size undefined!");
+        if (grid.nd == 0) cdoAbort("nd undefined!");
+        if (grid.ni == 0) cdoAbort("ni undefined!");
+        if (grid.size == 0) cdoAbort("size undefined!");
 
         gridID = gridCreate(grid.type, grid.size);
 
@@ -269,9 +270,9 @@ gridDefine(GridDesciption &grid)
     default:
       {
         if (grid.type == -1)
-          Error("Undefined grid type!");
+          cdoAbort("Undefined grid type!");
         else
-          Error("Unsupported grid type: %s", gridNamePtr(grid.type));
+          cdoAbort("Unsupported grid type: %s", gridNamePtr(grid.type));
 
         break;
       }
@@ -300,7 +301,7 @@ gridFromMPAS(const char *filename)
   int mpasGridID = -1;
 
   openLock();
-  int streamID = streamOpenRead(filename);
+  CdiStreamID streamID = streamOpenRead(filename);
   openUnlock();
   if (streamID >= 0)
     {
@@ -312,17 +313,23 @@ gridFromMPAS(const char *filename)
       for (int varID = 0; varID < nvars; ++varID)
         {
           vlistInqVarName(vlistID, varID, varname);
-          if      (cstrIsEqual(varname, "latCell")) latCellID = varID;
-          else if (cstrIsEqual(varname, "lonCell")) lonCellID = varID;
-          else if (cstrIsEqual(varname, "latVertex")) latVertexID = varID;
-          else if (cstrIsEqual(varname, "lonVertex")) lonVertexID = varID;
-          else if (cstrIsEqual(varname, "areaCell")) areaCellID = varID;
-          else if (cstrIsEqual(varname, "verticesOnCell")) verticesOnCellID = varID;
+          if (cstrIsEqual(varname, "latCell"))
+            latCellID = varID;
+          else if (cstrIsEqual(varname, "lonCell"))
+            lonCellID = varID;
+          else if (cstrIsEqual(varname, "latVertex"))
+            latVertexID = varID;
+          else if (cstrIsEqual(varname, "lonVertex"))
+            lonVertexID = varID;
+          else if (cstrIsEqual(varname, "areaCell"))
+            areaCellID = varID;
+          else if (cstrIsEqual(varname, "verticesOnCell"))
+            verticesOnCellID = varID;
         }
 
-      //printf("%d %d %d %d %d %d\n", latCellID, lonCellID, areaCellID, latVertexID, lonVertexID, verticesOnCellID);
-      if (latCellID == -1 || lonCellID == -1 || areaCellID == -1
-          || latVertexID == -1 || lonVertexID == -1 || verticesOnCellID == -1)
+      // printf("%d %d %d %d %d %d\n", latCellID, lonCellID, areaCellID, latVertexID, lonVertexID, verticesOnCellID);
+      if (latCellID == -1 || lonCellID == -1 || areaCellID == -1 || latVertexID == -1 || lonVertexID == -1
+          || verticesOnCellID == -1)
         {
           cdoWarning("MPAS grid variable missing!");
           return -1;
@@ -336,9 +343,9 @@ gridFromMPAS(const char *filename)
       const size_t nvertices = gridInqSize(vlistInqVarGrid(vlistID, latVertexID));
       std::vector<double> latVertex(nvertices);
       std::vector<double> lonVertex(nvertices);
-      std::vector<double> verticesOnCell(nv*ncells);
-      std::vector<double> xbounds(nv*ncells);
-      std::vector<double> ybounds(nv*ncells);
+      std::vector<double> verticesOnCell(nv * ncells);
+      std::vector<double> xbounds(nv * ncells);
+      std::vector<double> ybounds(nv * ncells);
 
       const int nrecs = streamInqTimestep(streamID, 0);
       for (int recID = 0; recID < nrecs; recID++)
@@ -346,28 +353,34 @@ gridFromMPAS(const char *filename)
           size_t nmiss;
           int varID, levelID;
           streamInqRecord(streamID, &varID, &levelID);
-          if      (varID == latCellID) streamReadRecord(streamID, latCell.data(), &nmiss);
-          else if (varID == lonCellID) streamReadRecord(streamID, lonCell.data(), &nmiss);
-          else if (varID == latVertexID) streamReadRecord(streamID, latVertex.data(), &nmiss);
-          else if (varID == lonVertexID) streamReadRecord(streamID, lonVertex.data(), &nmiss);
-          else if (varID == areaCellID) streamReadRecord(streamID, areaCell.data(), &nmiss);
-          else if (varID == verticesOnCellID) streamReadRecord(streamID, verticesOnCell.data(), &nmiss);
+          if (varID == latCellID)
+            streamReadRecord(streamID, latCell.data(), &nmiss);
+          else if (varID == lonCellID)
+            streamReadRecord(streamID, lonCell.data(), &nmiss);
+          else if (varID == latVertexID)
+            streamReadRecord(streamID, latVertex.data(), &nmiss);
+          else if (varID == lonVertexID)
+            streamReadRecord(streamID, lonVertex.data(), &nmiss);
+          else if (varID == areaCellID)
+            streamReadRecord(streamID, areaCell.data(), &nmiss);
+          else if (varID == verticesOnCellID)
+            streamReadRecord(streamID, verticesOnCell.data(), &nmiss);
         }
       streamClose(streamID);
 
       mpasGridID = gridCreate(GRID_UNSTRUCTURED, ncells);
-      gridDefXvals(mpasGridID, lonCell.data());
-      gridDefYvals(mpasGridID, latCell.data());
-      gridDefXunits(mpasGridID, "radians");
-      gridDefYunits(mpasGridID, "radians");
-      gridDefArea(mpasGridID, areaCell.data());
+      gridDefXvals(mpasGridID, lonCell.data());  // CDIcall
+      gridDefYvals(mpasGridID, latCell.data());  // CDIcall
+      gridDefXunits(mpasGridID, "radians");      // CDIcall
+      gridDefYunits(mpasGridID, "radians");      // CDIcall
+      gridDefArea(mpasGridID, areaCell.data());  // CDIcall
       bool lbounds = true;
       double xc[nv], yc[nv];
       for (size_t i = 0; i < ncells; ++i)
         for (size_t k = 0; k < nv; ++k)
           {
-            long cellIndex = lround(verticesOnCell[i*nv + k]);
-            if (cellIndex < 0 || cellIndex > (long)nvertices)
+            long cellIndex = lround(verticesOnCell[i * nv + k]);
+            if (cellIndex < 0 || cellIndex > (long) nvertices)
               {
                 lbounds = false;
               }
@@ -375,8 +388,8 @@ gridFromMPAS(const char *filename)
               {
                 if (k == 5)
                   {
-                    xc[k] = xc[k-1];
-                    yc[k] = yc[k-1];
+                    xc[k] = xc[k - 1];
+                    yc[k] = yc[k - 1];
                   }
                 else
                   {
@@ -385,11 +398,11 @@ gridFromMPAS(const char *filename)
               }
             else
               {
-                xc[k] = lonVertex[cellIndex-1];
-                yc[k] = latVertex[cellIndex-1];
+                xc[k] = lonVertex[cellIndex - 1];
+                yc[k] = latVertex[cellIndex - 1];
               }
-            xbounds[i*nv + k] = xc[k];
-            ybounds[i*nv + k] = yc[k];
+            xbounds[i * nv + k] = xc[k];
+            ybounds[i * nv + k] = yc[k];
           }
 
       if (lbounds)
@@ -452,20 +465,20 @@ cdoDefineGrid(const char *pgridfile)
     }
   else if (lmpas)
     {
-      if (CdoDebug::cdoDebug) cdoPrint("Grid from MPAS file");
+      Debug(cdoDebug,"Grid from MPAS file");
       gridID = gridFromMPAS(filename);
       if (gridID == -1) cdoAbort("Unsupported MPAS grid file %s!", filename);
     }
   else
     {
       char buffer[4];
-      if (read(fileno, buffer, 4) != 4) SysError("Read grid from %s failed!", filename);
+      if (read(fileno, buffer, 4) != 4) cdoSysError("Read grid from %s failed!", filename);
 
       close(fileno);
 
       if (cmpstrlen(buffer, "CDF", len) == 0)
         {
-          if (CdoDebug::cdoDebug) cdoPrint("Grid from NetCDF file");
+          Debug(cdoDebug,"Grid from NetCDF file");
           gridID = gridFromNCfile(filename);
         }
 
@@ -473,7 +486,7 @@ cdoDefineGrid(const char *pgridfile)
         {
           if (cmpstrlen(buffer + 1, "HDF", len) == 0)
             {
-              if (CdoDebug::cdoDebug) cdoPrint("Grid from HDF5 file");
+              Debug(cdoDebug,"Grid from HDF5 file");
               gridID = gridFromH5file(filename);
             }
         }
@@ -482,14 +495,14 @@ cdoDefineGrid(const char *pgridfile)
         {
           if (cmpstrlen(buffer + 1, "HDF", len) == 0)
             {
-              if (CdoDebug::cdoDebug) cdoPrint("Grid from NetCDF4 file");
+              Debug(cdoDebug,"Grid from NetCDF4 file");
               gridID = gridFromNCfile(filename);
             }
         }
 
       if (gridID == -1)
         {
-          if (CdoDebug::cdoDebug) cdoPrint("Grid from CDI file");
+          Debug(cdoDebug,"Grid from CDI file");
           openLock();
           int streamID = streamOpenRead(filename);
           openUnlock();
@@ -505,7 +518,7 @@ cdoDefineGrid(const char *pgridfile)
 
       if (gridID == -1)
         {
-          if (CdoDebug::cdoDebug) cdoPrint("grid from ASCII file");
+          Debug(cdoDebug,"grid from ASCII file");
           FILE *gfp = fopen(filename, "r");
           // size_t buffersize = 20*1024*1024;
           // char *buffer = (char*) Malloc(buffersize);
@@ -517,9 +530,9 @@ cdoDefineGrid(const char *pgridfile)
 
       if (gridID == -1)
         {
-          if (CdoDebug::cdoDebug) cdoPrint("grid from PINGO file");
+          Debug(cdoDebug,"grid from PINGO file");
           FILE *gfp = fopen(filename, "r");
-          gridID = grid_read_pingo(gfp, filename);
+          gridID = grid_read_pingo(gfp);
           fclose(gfp);
         }
 
