@@ -22,7 +22,7 @@
 
 class GridDesciption
 {
- public:
+public:
   std::vector<int> mask;
   std::vector<double> xvals;
   std::vector<double> yvals;
@@ -33,19 +33,18 @@ class GridDesciption
   double xfirst, yfirst;
   double xlast, ylast;
   double xinc, yinc;
-  double xpole, ypole, angle; // rotated north pole
+  double xpole, ypole, angle;  // rotated north pole
   int scanningMode;
   /*
     scanningMode  = 128 * iScansNegatively + 64 * jScansPositively + 32 * jPointsAreConsecutive;
-              64  = 128 * 0                + 64 *        1         + 32 * 0 
+              64  = 128 * 0                + 64 *        1         + 32 * 0
               00  = 128 * 0                + 64 *        0         + 32 * 0
               96  = 128 * 0                + 64 *        1         + 32 * 1
     Default  implicit scanning mode is 64: i and j scan positively, i points are consecutive (row-major)
   */
-  bool uvRelativeToGrid;
   double a;
   int datatype;
-  int isRotated; // true for rotated grids
+  int isRotated;  // true for rotated grids
   int type;
   int ntr;
   bool genBounds;
@@ -76,10 +75,7 @@ class GridDesciption
   char vdimname[CDI_MAX_NAME];
 
   void init();
-  GridDesciption()
-    {
-      init();
-    }
+  GridDesciption() { init(); }
 };
 
 int gridDefine(GridDesciption &grid);
@@ -88,9 +84,10 @@ int gridFromNCfile(const char *gridfile);
 int gridFromH5file(const char *gridfile);
 int gridFromName(const char *gridname);
 
-
 void writeNCgrid(const char *gridfile, int gridID, int *imask);
 
 int cdoDefineGrid(const char *gridfile);
 
+int grid_read(FILE *gfp, const char *dname);  // TODO: Find better place for this
+
 #endif /* GRIDDES_H */
