@@ -18,7 +18,7 @@
 
 #include "cdo_int.h"
 #include "cdo_math.h"
-#include "grid.h"
+#include <mpim_grid.h>
 #include "cdo_options.h"
 #include "interpol.h"
 #include "progress.h"
@@ -148,7 +148,7 @@ rect_grid_search2(long *imin, long *imax, double xmin, double xmax, long nxm, co
   *imin = nxm;
   *imax = -1;
 
-  bool lascend = (xm[0] < xm[nxm - 1]);
+  const bool lascend = (xm[0] < xm[nxm - 1]);
 
   long i1 = find_element(xmin, nxm, xm);
   long i2 = find_element(xmax, nxm, xm);
@@ -216,14 +216,14 @@ intlinarr2p(long nxm, long nym, double **fieldm, const double *xm, const double
 }
 
 static void
-intlinarr2(double missval, int lon_is_circular, size_t nxm, size_t nym, const double *restrict fieldm, const double *restrict xm,
-           const double *restrict ym, size_t gridsize2, double *field, const double *restrict x, const double *restrict y)
+intlinarr2(double missval, int lon_is_circular, size_t nxm, size_t nym, const double *restrict fieldm, const std::vector<double> &xm,
+           const std::vector<double> &ym, size_t gridsize2, double *field, const std::vector<double> &x, const std::vector<double> &y)
 {
   size_t nlon1 = nxm;
   double findex = 0;
 
   if (lon_is_circular) nlon1--;
-  size_t gridsize1 = nlon1 * nym;
+  const size_t gridsize1 = nlon1 * nym;
 
   std::vector<bool> grid1_mask(gridsize1);
   for (size_t jj = 0; jj < nym; ++jj)
@@ -232,8 +232,8 @@ intlinarr2(double missval, int lon_is_circular, size_t nxm, size_t nym, const do
   Progress::init();
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(findex, field, fieldm, x, y, xm, ym, nxm, nym, gridsize2, missval, nlon1, \
-                                              lon_is_circular, grid1_mask)
+#pragma omp parallel for default(none) \
+    shared(findex, field, fieldm, x, y, xm, ym, nxm, nym, gridsize2, missval, nlon1, lon_is_circular, grid1_mask)
 #endif
   for (size_t i = 0; i < gridsize2; ++i)
     {
@@ -248,7 +248,7 @@ intlinarr2(double missval, int lon_is_circular, size_t nxm, size_t nym, const do
       if (cdo_omp_get_thread_num() == 0) Progress::update(0, 1, findex / gridsize2);
 
       size_t ii, jj;
-      int lfound = rect_grid_search(&ii, &jj, x[i], y[i], nxm, nym, xm, ym);
+      int lfound = rect_grid_search(&ii, &jj, x[i], y[i], nxm, nym, xm.data(), ym.data());
 
       if (lfound)
         {
@@ -297,14 +297,14 @@ intlinarr(long nxm, double *ym, double *xm, int nx, double *y, double *x)
 }
 
 void
-intgridbil(Field *field1, Field *field2)
+intgridbil(Field &field1, Field &field2)
 {
   char xunits[CDI_MAX_NAME];
-  int gridID1 = field1->grid;
-  int gridID2 = field2->grid;
-  double *array1 = field1->ptr;
-  double *array2 = field2->ptr;
-  double missval = field1->missval;
+  int gridID1 = field1.grid;
+  int gridID2 = field2.grid;
+  double *array1 = field1.vec.data();
+  double *array2 = field2.vec.data();
+  const double missval = field1.missval;
 
   size_t nlon1 = gridInqXsize(gridID1);
   size_t nlat1 = gridInqYsize(gridID1);
@@ -343,8 +343,8 @@ intgridbil(Field *field1, Field *field2)
       if (lon_is_circular) lon1[nlon1 - 1] = lon1[0] + 2 * M_PI;
     }
 
-  size_t xsize2 = gridInqXsize(gridID2);
-  size_t ysize2 = gridInqYsize(gridID2);
+  const size_t xsize2 = gridInqXsize(gridID2);
+  const size_t ysize2 = gridInqYsize(gridID2);
 
   if (xsize2 == 1 && ysize2 == 1)
     {
@@ -398,10 +398,11 @@ intgridbil(Field *field1, Field *field2)
           if (gridInqType(gridID2) != GRID_UNSTRUCTURED && gridInqType(gridID2) != GRID_CURVILINEAR)
             gridID2 = gridToCurvilinear(gridID2, 0);
 
-          if (!(gridInqXvals(gridID2, nullptr) && gridInqYvals(gridID2, nullptr))) cdoAbort("Target grid has no coordinate values!");
+          if (!(gridInqXvals(gridID2, nullptr) && gridInqYvals(gridID2, nullptr)))
+            cdoAbort("Target grid has no coordinate values!");
         }
 
-      size_t gridsize2 = gridInqSize(gridID2);
+      const size_t gridsize2 = gridInqSize(gridID2);
 
       std::vector<double> xvals2(gridsize2);
       std::vector<double> yvals2(gridsize2);
@@ -438,10 +439,9 @@ intgridbil(Field *field1, Field *field2)
               }
         }
 
-      intlinarr2(missval, lon_is_circular, nlon1, nlat1, array1, lon1.data(), lat1.data(), gridsize2, array2, xvals2.data(),
-                 yvals2.data());
+      intlinarr2(missval, lon_is_circular, nlon1, nlat1, array1, lon1, lat1, gridsize2, array2, xvals2, yvals2);
 
-      field2->nmiss = arrayNumMV(gridsize2, array2, missval);
+      field2.nmiss = arrayNumMV(gridsize2, array2, missval);
     }
 
   Free(array1_2D);
@@ -449,7 +449,7 @@ intgridbil(Field *field1, Field *field2)
 
 /* source code from pingo */
 void
-interpolate(Field *field1, Field *field2)
+interpolate(Field &field1, Field &field2)
 {
   int i;
   double *lono_array, *lato_array, *lono, *lato;
@@ -476,11 +476,11 @@ interpolate(Field *field1, Field *field2)
   double faclon1, faclon2, faclat1, faclat2;
   int nlon, nlat, out_nlon, out_nlat;
 
-  gridIDi = field1->grid;
-  gridIDo = field2->grid;
-  arrayIn = field1->ptr;
-  arrayOut = field2->ptr;
-  missval = field1->missval;
+  gridIDi = field1.grid;
+  gridIDo = field2.grid;
+  arrayIn = field1.vec.data();
+  arrayOut = field2.vec.data();
+  missval = field1.missval;
 
   // gridsize_i = gridInqSize(gridIDi);
   gridsize_o = gridInqSize(gridIDo);
@@ -915,7 +915,7 @@ interpolate(Field *field1, Field *field2)
         }
     }
 
-  field2->nmiss = arrayNumMV(gridsize_o, arrayOut, missval);
+  field2.nmiss = arrayNumMV(gridsize_o, arrayOut, missval);
 
   Free(lon_array);
   Free(lat_array);
