@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -18,46 +18,45 @@
 #include "cdo_int.h"
 #include "datetime.h"
 #include "calendar.h"
+#include "timebase.h"
 
-juldate_t
-juldate_encode(int calendar, int64_t date, int time)
+JulianDate
+julianDateEncode(int calendar, int64_t date, int time)
 {
   int year, month, day, hour, minute, second;
-  juldate_t juldate;
-
   cdiDecodeDate(date, &year, &month, &day);
   cdiDecodeTime(time, &hour, &minute, &second);
 
+  JulianDate juldate;
   encode_caldaysec(calendar, year, month, day, hour, minute, second, &juldate.julday, &juldate.secofday);
 
   return juldate;
 }
 
 void
-juldate_decode(int calendar, juldate_t juldate, int64_t *date, int *time)
+julianDateDecode(int calendar, const JulianDate &juldate, int64_t *date, int *time)
 {
   int year, month, day, hour, minute, second;
-
   decode_caldaysec(calendar, juldate.julday, juldate.secofday, &year, &month, &day, &hour, &minute, &second);
 
   *date = cdiEncodeDate(year, month, day);
   *time = cdiEncodeTime(hour, minute, second);
 }
 
-juldate_t
-juldate_sub(juldate_t juldate2, juldate_t juldate1)
+JulianDate
+julianDateSub(const JulianDate &juldate2, const JulianDate &juldate1)
 {
-  juldate_t juldate;
+  JulianDate juldate;
 
   (void) julday_sub(juldate1.julday, juldate1.secofday, juldate2.julday, juldate2.secofday, &juldate.julday, &juldate.secofday);
 
   return juldate;
 }
 
-juldate_t
-juldate_add_seconds(int64_t seconds, juldate_t juldate)
+JulianDate
+julianDateAddSeconds(int64_t seconds, const JulianDate &juldate)
 {
-  juldate_t juldate_new = juldate;
+  JulianDate juldate_new = juldate;
 
   julday_add_seconds(seconds, &juldate_new.julday, &juldate_new.secofday);
 
@@ -65,8 +64,7 @@ juldate_add_seconds(int64_t seconds, juldate_t juldate)
 }
 
 double
-juldate_to_seconds(juldate_t juldate)
+julianDateToSeconds(const JulianDate &juldate)
 {
   return juldate.julday * 86400. + juldate.secofday;
-  ;
 }
