@@ -38,13 +38,13 @@ public:
   knnWeightsType(const size_t maxNeighbors) { init(maxNeighbors); }
 
   inline size_t
-  maxNeighbors()
+  maxNeighbors() const
   {
     return m_maxNeighbors;
   }
 
   inline size_t
-  numNeighbors()
+  numNeighbors() const
   {
     return m_numNeighbors;
   }
@@ -61,6 +61,18 @@ public:
     for (size_t i = 0; i < m_maxNeighbors; ++i) m_dist[i] = DBL_MAX;
   }
 
+  inline void
+  initAddr(const size_t numNeighbors)
+  {
+    for (size_t i = 0; i < numNeighbors; ++i) m_addr[i] = SIZE_MAX;
+  }
+
+  inline void
+  initDist(const size_t numNeighbors)
+  {
+    for (size_t i = 0; i < numNeighbors; ++i) m_dist[i] = DBL_MAX;
+  }
+
   inline void
   storeDistance(const size_t addr, const double distance, const size_t numNeighbors)
   {
@@ -129,12 +141,12 @@ public:
   }
 
   size_t
-  normalizeWeights(const double dist_tot)
+  normalizeWeights(const double dist_tot, size_t numNeighbors)
   {
     // Normalize weights and store the link
     size_t nadds = 0;
 
-    for (size_t n = 0; n < m_maxNeighbors; ++n)
+    for (size_t n = 0; n < numNeighbors; ++n)
       {
         if (m_mask[n])
           {
@@ -166,7 +178,7 @@ public:
           }
       }
 
-    return normalizeWeights(dist_tot);
+    return normalizeWeights(dist_tot, m_maxNeighbors);
   }
 
   size_t
@@ -179,16 +191,15 @@ public:
     for (size_t n = 0; n < m_maxNeighbors; ++n)
       {
         m_mask[n] = false;
-        if (m_addr[n] < SIZE_MAX)
-          if (src_grid_mask[m_addr[n]])
-            {
-              m_dist[n] = 1. / m_dist[n];
-              dist_tot += m_dist[n];
-              m_mask[n] = true;
-            }
+        if (m_addr[n] < SIZE_MAX && src_grid_mask[m_addr[n]])
+          {
+            m_dist[n] = 1. / m_dist[n];
+            dist_tot += m_dist[n];
+            m_mask[n] = true;
+          }
       }
 
-    return normalizeWeights(dist_tot);
+    return normalizeWeights(dist_tot, m_maxNeighbors);
   }
 
   size_t
@@ -198,23 +209,22 @@ public:
 
     double dist_tot = 0.;  // sum of neighbor distances (for normalizing)
 
-    for (size_t n = 0; n < m_maxNeighbors; ++n)
+    for (size_t n = 0; n < m_numNeighbors; ++n)
       {
         m_mask[n] = false;
-        if (m_addr[n] < SIZE_MAX)
-          if (src_grid_mask[m_addr[n]])
-            {
-              m_dist[n] = intlin(m_dist[n], weight0, 0, weightR, searchRadius);
-              dist_tot += m_dist[n];
-              m_mask[n] = true;
-            }
+        if (m_addr[n] < SIZE_MAX && src_grid_mask[m_addr[n]])
+          {
+            m_dist[n] = intlin(m_dist[n], weight0, 0, weightR, searchRadius);
+            dist_tot += m_dist[n];
+            m_mask[n] = true;
+          }
       }
 
-    return normalizeWeights(dist_tot);
+    return normalizeWeights(dist_tot, m_numNeighbors);
   }
 
   double
-  arrayWeightsSum(const double *array)
+  arrayWeightsSum(const double *array) const
   {
     double result = 0;
     for (size_t n = 0; n < m_numNeighbors; ++n) result += array[m_addr[n]] * m_dist[n];
