@@ -30,6 +30,11 @@
  * along with YAC.  If not, see <http://www.gnu.org/licenses/gpl.txt>.
  */
 
+#ifdef HAVE_CONFIG_H
+// Get the definition of the 'restrict' keyword.
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -159,9 +164,9 @@ static void compute_cell_barycenter(
   barycenter[2] = 0.0;
 
   for (size_t k = 0; k < cell.num_corners; ++k) {
-    barycenter[0] += cell.coordinates_xyz[0+k*3];
-    barycenter[1] += cell.coordinates_xyz[1+k*3];
-    barycenter[2] += cell.coordinates_xyz[2+k*3];
+    barycenter[0] += cell.coordinates_xyz[k][0];
+    barycenter[1] += cell.coordinates_xyz[k][1];
+    barycenter[2] += cell.coordinates_xyz[k][2];
   }
   normalise_vector(barycenter);
 }
@@ -196,7 +201,7 @@ void yac_compute_concave_overlap_info (size_t N,
   struct grid_cell target_partial_cell =
     {.coordinates_x   = (double[3]){-1},
      .coordinates_y   = (double[3]){-1},
-     .coordinates_xyz = (double[3*3]){-1},
+     .coordinates_xyz = (double[3][3]){{-1}},
      .edge_type       = (enum yac_edge_type[3]) {GREAT_CIRCLE},
      .num_corners     = 3};
 
@@ -210,17 +215,17 @@ void yac_compute_concave_overlap_info (size_t N,
   }
 
   // common node point to all partial target cells
-  target_partial_cell.coordinates_xyz[0] = target_node_xyz[0];
-  target_partial_cell.coordinates_xyz[1] = target_node_xyz[1];
-  target_partial_cell.coordinates_xyz[2] = target_node_xyz[2];
+  target_partial_cell.coordinates_xyz[0][0] = target_node_xyz[0];
+  target_partial_cell.coordinates_xyz[0][1] = target_node_xyz[1];
+  target_partial_cell.coordinates_xyz[0][2] = target_node_xyz[2];
   XYZtoLL(target_node_xyz, target_partial_cell.coordinates_x,
           target_partial_cell.coordinates_y);
 
   for ( size_t num_corners = 0;
-        num_corners < ((size_t)(target_cell.num_corners)); ++num_corners ) {
+        num_corners < target_cell.num_corners; ++num_corners ) {
 
     size_t corner_a = num_corners;
-    size_t corner_b = (num_corners+1)%((size_t)(target_cell.num_corners));
+    size_t corner_b = (num_corners+1)%target_cell.num_corners;
 
     // skip clipping and area calculation for degenerated triangles
     //
@@ -238,24 +243,24 @@ void yac_compute_concave_overlap_info (size_t N,
     // Compared to the if statement below the alternative seems
     // to be quite costly.
 
-    if ( ( ( fabs(target_cell.coordinates_xyz[0+3*corner_a] -
-                  target_cell.coordinates_xyz[0+3*corner_b]) < tol ) &&
-           ( fabs(target_cell.coordinates_xyz[1+3*corner_a] -
-                  target_cell.coordinates_xyz[1+3*corner_b]) < tol ) &&
-           ( fabs(target_cell.coordinates_xyz[2+3*corner_a] -
-                  target_cell.coordinates_xyz[2+3*corner_b]) < tol ) ) ||
-         ( ( fabs(target_cell.coordinates_xyz[0+3*corner_a] -
-                  target_partial_cell.coordinates_xyz[0]) < tol    ) &&
-           ( fabs(target_cell.coordinates_xyz[1+3*corner_a] -
-                  target_partial_cell.coordinates_xyz[1]) < tol    ) &&
-           ( fabs(target_cell.coordinates_xyz[2+3*corner_a] -
-                  target_partial_cell.coordinates_xyz[2]) < tol    ) ) ||
-         ( ( fabs(target_cell.coordinates_xyz[0+3*corner_b] -
-                  target_partial_cell.coordinates_xyz[0]) < tol    ) &&
-           ( fabs(target_cell.coordinates_xyz[1+3*corner_b] -
-                  target_partial_cell.coordinates_xyz[1]) < tol    ) &&
-           ( fabs(target_cell.coordinates_xyz[2+3*corner_b] -
-                  target_partial_cell.coordinates_xyz[2]) < tol    ) ) )
+    if ( ( ( fabs(target_cell.coordinates_xyz[corner_a][0] -
+                  target_cell.coordinates_xyz[corner_b][0]) < tol ) &&
+           ( fabs(target_cell.coordinates_xyz[corner_a][1] -
+                  target_cell.coordinates_xyz[corner_b][1]) < tol ) &&
+           ( fabs(target_cell.coordinates_xyz[corner_a][2] -
+                  target_cell.coordinates_xyz[corner_b][2]) < tol ) ) ||
+         ( ( fabs(target_cell.coordinates_xyz[corner_a][0] -
+                  target_partial_cell.coordinates_xyz[0][0]) < tol    ) &&
+           ( fabs(target_cell.coordinates_xyz[corner_a][1] -
+                  target_partial_cell.coordinates_xyz[0][1]) < tol    ) &&
+           ( fabs(target_cell.coordinates_xyz[corner_a][2] -
+                  target_partial_cell.coordinates_xyz[0][2]) < tol    ) ) ||
+         ( ( fabs(target_cell.coordinates_xyz[corner_b][0] -
+                  target_partial_cell.coordinates_xyz[0][0]) < tol    ) &&
+           ( fabs(target_cell.coordinates_xyz[corner_b][1] -
+                  target_partial_cell.coordinates_xyz[0][1]) < tol    ) &&
+           ( fabs(target_cell.coordinates_xyz[corner_b][2] -
+                  target_partial_cell.coordinates_xyz[0][2]) < tol    ) ) )
        continue;
 
     target_partial_cell.coordinates_x[1] = target_cell.coordinates_x[corner_a];
@@ -263,12 +268,12 @@ void yac_compute_concave_overlap_info (size_t N,
     target_partial_cell.coordinates_x[2] = target_cell.coordinates_x[corner_b];
     target_partial_cell.coordinates_y[2] = target_cell.coordinates_y[corner_b];
 
-    target_partial_cell.coordinates_xyz[0+3*1] = target_cell.coordinates_xyz[0+3*corner_a];
-    target_partial_cell.coordinates_xyz[1+3*1] = target_cell.coordinates_xyz[1+3*corner_a];
-    target_partial_cell.coordinates_xyz[2+3*1] = target_cell.coordinates_xyz[2+3*corner_a];
-    target_partial_cell.coordinates_xyz[0+3*2] = target_cell.coordinates_xyz[0+3*corner_b];
-    target_partial_cell.coordinates_xyz[1+3*2] = target_cell.coordinates_xyz[1+3*corner_b];
-    target_partial_cell.coordinates_xyz[2+3*2] = target_cell.coordinates_xyz[2+3*corner_b];
+    target_partial_cell.coordinates_xyz[1][0] = target_cell.coordinates_xyz[corner_a][0];
+    target_partial_cell.coordinates_xyz[1][1] = target_cell.coordinates_xyz[corner_a][1];
+    target_partial_cell.coordinates_xyz[1][2] = target_cell.coordinates_xyz[corner_a][2];
+    target_partial_cell.coordinates_xyz[2][0] = target_cell.coordinates_xyz[corner_b][0];
+    target_partial_cell.coordinates_xyz[2][1] = target_cell.coordinates_xyz[corner_b][1];
+    target_partial_cell.coordinates_xyz[2][2] = target_cell.coordinates_xyz[corner_b][2];
 
     yac_cell_clipping ( N, source_cell, target_partial_cell, overlap_buffer);
 
@@ -293,6 +298,53 @@ void yac_compute_concave_overlap_info (size_t N,
 #endif
 }
 
+static inline double dotproduct(double a[], double b[]) {
+
+  return a[0] * b[0] + a[1] * b[1] + a[2] * b[2];
+}
+
+static int gc_cell_is_convex(struct grid_cell cell) {
+
+  double const sq_angle_tol = yac_angle_tol * yac_angle_tol;
+  double (* restrict coordinates_xyz)[3] = cell.coordinates_xyz;
+
+  // for all edges (we skip the edge 0->num_corners-1; because we do not need it
+  // to determine whether a polygon is convex or not
+  for (size_t i = 0; i < cell.num_corners-1; ++i) {
+
+    // compute norm vector for current edge
+    double edge_norm_vec[3];
+    crossproduct_d(
+      coordinates_xyz[i], coordinates_xyz[i+1], edge_norm_vec);
+    // (sin(edge_length))^2
+    double sq_sin_edge_length = edge_norm_vec[0] * edge_norm_vec[0] +
+                                edge_norm_vec[1] * edge_norm_vec[1] +
+                                edge_norm_vec[2] * edge_norm_vec[2];
+    // if the edge is too short -> skip edge
+    if (sq_sin_edge_length < sq_angle_tol) continue;
+
+    normalise_vector(edge_norm_vec);
+
+    size_t same_side_count = 0;
+
+    // for the corners of the triangle
+    // (from corner index 0 to start of the edge)
+    for (size_t j = 0; j < i; ++j)
+      same_side_count += dotproduct(edge_norm_vec, coordinates_xyz[j]) > 0.0;
+
+    // for the remaining corners
+    for (size_t j = i + 2; j < cell.num_corners; ++j)
+      same_side_count += dotproduct(edge_norm_vec, coordinates_xyz[j]) > 0.0;
+
+    // if not all corners are on the same side of the current edge -> concave
+    if ((same_side_count != 0) &&
+        (same_side_count != cell.num_corners - 2)) return 0;
+  }
+
+  // the cell is convex
+  return 1;
+}
+
 /* ------------------------- */
 
 void yac_compute_concave_overlap_areas (size_t N,
@@ -305,7 +357,9 @@ void yac_compute_concave_overlap_areas (size_t N,
   if ( target_cell.num_corners > 3 )
     target_cell_type = get_cell_type (target_cell);
 
-  if ( target_cell.num_corners < 4 || target_cell_type == LON_LAT_CELL ) {
+  if (( target_cell.num_corners < 4 || target_cell_type == LON_LAT_CELL ) ||
+      ((target_cell_type == GREAT_CIRCLE_CELL) &&
+       gc_cell_is_convex(target_cell))) {
     yac_compute_overlap_areas (N, source_cell, target_cell, partial_areas);
     return;
   }
@@ -318,7 +372,7 @@ void yac_compute_concave_overlap_areas (size_t N,
   struct grid_cell target_partial_cell =
     {.coordinates_x   = (double[3]){-1},
      .coordinates_y   = (double[3]){-1},
-     .coordinates_xyz = (double[3*3]){-1},
+     .coordinates_xyz = (double[3][3]){{-1}},
      .edge_type       = (enum yac_edge_type[3]) {GREAT_CIRCLE},
      .num_corners     = 3};
 
@@ -329,17 +383,17 @@ void yac_compute_concave_overlap_areas (size_t N,
   for ( size_t n = 0; n < N; n++) partial_areas[n] = 0.0;
 
   // common node point to all partial target cells
-  target_partial_cell.coordinates_xyz[0] = target_node_xyz[0];
-  target_partial_cell.coordinates_xyz[1] = target_node_xyz[1];
-  target_partial_cell.coordinates_xyz[2] = target_node_xyz[2];
+  target_partial_cell.coordinates_xyz[0][0] = target_node_xyz[0];
+  target_partial_cell.coordinates_xyz[0][1] = target_node_xyz[1];
+  target_partial_cell.coordinates_xyz[0][2] = target_node_xyz[2];
   XYZtoLL(target_node_xyz, target_partial_cell.coordinates_x,
           target_partial_cell.coordinates_y);
 
   for ( size_t num_corners = 0;
-        num_corners < ((size_t)(target_cell.num_corners)); ++num_corners ) {
+        num_corners < target_cell.num_corners; ++num_corners ) {
 
     size_t corner_a = num_corners;
-    size_t corner_b = (num_corners+1)%((size_t)(target_cell.num_corners));
+    size_t corner_b = (num_corners+1)%target_cell.num_corners;
 
     // skip clipping and area calculation for degenerated triangles
     //
@@ -357,24 +411,24 @@ void yac_compute_concave_overlap_areas (size_t N,
     // Compared to the if statement below the alternative seems
     // to be quite costly.
 
-    if ( ( ( fabs(target_cell.coordinates_xyz[0+3*corner_a] -
-                  target_cell.coordinates_xyz[0+3*corner_b]) < tol ) &&
-           ( fabs(target_cell.coordinates_xyz[1+3*corner_a] -
-                  target_cell.coordinates_xyz[1+3*corner_b]) < tol ) &&
-           ( fabs(target_cell.coordinates_xyz[2+3*corner_a] -
-                  target_cell.coordinates_xyz[2+3*corner_b]) < tol ) ) ||
-         ( ( fabs(target_cell.coordinates_xyz[0+3*corner_a] -
-                  target_partial_cell.coordinates_xyz[0]) < tol    ) &&
-           ( fabs(target_cell.coordinates_xyz[1+3*corner_a] -
-                  target_partial_cell.coordinates_xyz[1]) < tol    ) &&
-           ( fabs(target_cell.coordinates_xyz[2+3*corner_a] -
-                  target_partial_cell.coordinates_xyz[2]) < tol    ) ) ||
-         ( ( fabs(target_cell.coordinates_xyz[0+3*corner_b] -
-                  target_partial_cell.coordinates_xyz[0]) < tol    ) &&
-           ( fabs(target_cell.coordinates_xyz[1+3*corner_b] -
-                  target_partial_cell.coordinates_xyz[1]) < tol    ) &&
-           ( fabs(target_cell.coordinates_xyz[2+3*corner_b] -
-                  target_partial_cell.coordinates_xyz[2]) < tol    ) ) )
+    if ( ( ( fabs(target_cell.coordinates_xyz[corner_a][0] -
+                  target_cell.coordinates_xyz[corner_b][0]) < tol ) &&
+           ( fabs(target_cell.coordinates_xyz[corner_a][1] -
+                  target_cell.coordinates_xyz[corner_b][1]) < tol ) &&
+           ( fabs(target_cell.coordinates_xyz[corner_a][2] -
+                  target_cell.coordinates_xyz[corner_b][2]) < tol ) ) ||
+         ( ( fabs(target_cell.coordinates_xyz[corner_a][0] -
+                  target_partial_cell.coordinates_xyz[0][0]) < tol    ) &&
+           ( fabs(target_cell.coordinates_xyz[corner_a][1] -
+                  target_partial_cell.coordinates_xyz[0][1]) < tol    ) &&
+           ( fabs(target_cell.coordinates_xyz[corner_a][2] -
+                  target_partial_cell.coordinates_xyz[0][2]) < tol    ) ) ||
+         ( ( fabs(target_cell.coordinates_xyz[corner_b][0] -
+                  target_partial_cell.coordinates_xyz[0][0]) < tol    ) &&
+           ( fabs(target_cell.coordinates_xyz[corner_b][1] -
+                  target_partial_cell.coordinates_xyz[0][1]) < tol    ) &&
+           ( fabs(target_cell.coordinates_xyz[corner_b][2] -
+                  target_partial_cell.coordinates_xyz[0][2]) < tol    ) ) )
        continue;
 
     target_partial_cell.coordinates_x[1] = target_cell.coordinates_x[corner_a];
@@ -382,12 +436,12 @@ void yac_compute_concave_overlap_areas (size_t N,
     target_partial_cell.coordinates_x[2] = target_cell.coordinates_x[corner_b];
     target_partial_cell.coordinates_y[2] = target_cell.coordinates_y[corner_b];
 
-    target_partial_cell.coordinates_xyz[0+3*1] = target_cell.coordinates_xyz[0+3*corner_a];
-    target_partial_cell.coordinates_xyz[1+3*1] = target_cell.coordinates_xyz[1+3*corner_a];
-    target_partial_cell.coordinates_xyz[2+3*1] = target_cell.coordinates_xyz[2+3*corner_a];
-    target_partial_cell.coordinates_xyz[0+3*2] = target_cell.coordinates_xyz[0+3*corner_b];
-    target_partial_cell.coordinates_xyz[1+3*2] = target_cell.coordinates_xyz[1+3*corner_b];
-    target_partial_cell.coordinates_xyz[2+3*2] = target_cell.coordinates_xyz[2+3*corner_b];
+    target_partial_cell.coordinates_xyz[1][0] = target_cell.coordinates_xyz[corner_a][0];
+    target_partial_cell.coordinates_xyz[1][1] = target_cell.coordinates_xyz[corner_a][1];
+    target_partial_cell.coordinates_xyz[1][2] = target_cell.coordinates_xyz[corner_a][2];
+    target_partial_cell.coordinates_xyz[2][0] = target_cell.coordinates_xyz[corner_b][0];
+    target_partial_cell.coordinates_xyz[2][1] = target_cell.coordinates_xyz[corner_b][1];
+    target_partial_cell.coordinates_xyz[2][2] = target_cell.coordinates_xyz[corner_b][2];
 
     yac_cell_clipping ( N, source_cell, target_partial_cell, overlap_buffer);
 
@@ -409,11 +463,6 @@ void yac_compute_concave_overlap_areas (size_t N,
 
 /* ------------------------- */
 
-static double dotproduct(double a[], double b[]) {
-
-  return a[0] * b[0] + a[1] * b[1] + a[2] * b[2];
-}
-
 static void compute_norm_vector(double a[], double b[], double norm[]) {
 
   crossproduct_ld(a, b, norm);
@@ -517,7 +566,7 @@ static enum cell_type get_cell_type(struct grid_cell target_cell) {
          target_cell.edge_type[3] == LAT_CIRCLE)))
       return LON_LAT_CELL;
    else
-      for (size_t i = 0; i < ((size_t)(target_cell.num_corners)); ++i)
+      for (size_t i = 0; i < target_cell.num_corners; ++i)
          if (target_cell.edge_type[i] == LON_CIRCLE ||
              target_cell.edge_type[i] == GREAT_CIRCLE)
             count_great_circle_edges++;
@@ -591,7 +640,7 @@ static void point_list_clipping (struct point_list * source_list, int source_ord
     double * tgt_edge_coords[2] = {tgt_points[i].point->vec_coords,
                                    tgt_points[i].point->next->vec_coords};
     enum yac_edge_type tgt_edge_type = tgt_points[i].point->edge_type;
-    double * tgt_edge_norm_vec = tgt_points[i].edge_norm_vec;
+    double * curr_tgt_edge_norm_vec = tgt_points[i].edge_norm_vec;
 
     int prev_is_inside, curr_is_inside, last_curr_is_inside;
 
@@ -605,7 +654,7 @@ static void point_list_clipping (struct point_list * source_list, int source_ord
 
       if (j != ncs - 1)
         curr_is_inside = is_inside(curr_src_point->vec_coords,
-                                   tgt_edge_norm_vec, tgt_edge_type,
+                                   curr_tgt_edge_norm_vec, tgt_edge_type,
                                    target_ordering);
       else
         curr_is_inside = last_curr_is_inside;
@@ -695,10 +744,11 @@ static void point_list_clipping (struct point_list * source_list, int source_ord
 
                 // compute the angle between the previous source edge vertex and
                 // the two intersection points
-                int compare_angles_ret_value =
-                  compare_angles(
-                    get_vector_angle_2(src_edge_coords[0], intersection[0]),
-                    get_vector_angle_2(src_edge_coords[0], intersection[1]));
+                struct sin_cos_angle angle_a =
+                  get_vector_angle_2(src_edge_coords[0], intersection[0]);
+                struct sin_cos_angle angle_b =
+                  get_vector_angle_2(src_edge_coords[0], intersection[1]);
+                int compare_angles_ret_value = compare_angles(angle_a, angle_b);
 
                 // if the two angles are identical, we assume that both points
                 // are more or less identical to each other
@@ -795,30 +845,33 @@ static void point_list_clipping (struct point_list * source_list, int source_ord
         // the respective vertex is directly on the cutting edge.
         if (one_intersect_expected && (num_intersections == 0)) {
 
+          double temp_a, temp_b;
+
           // which of the two source edge vertices closer to the cutting edge
 
           switch (tgt_points[i].point->edge_type) {
 
             case (LON_CIRCLE) :
             case (GREAT_CIRCLE) :
-              if (fabs(dotproduct(src_edge_coords[0], tgt_edge_norm_vec)) <
-                  fabs(dotproduct(src_edge_coords[1], tgt_edge_norm_vec)))
-                prev_is_inside = 2;
-              else
-                curr_is_inside = 2;
+              temp_a = fabs(dotproduct(src_edge_coords[0],
+                                       curr_tgt_edge_norm_vec));
+              temp_b = fabs(dotproduct(src_edge_coords[1],
+                                       curr_tgt_edge_norm_vec));
               break;
             case (LAT_CIRCLE) :
-              if (fabs(1.0 - fabs(src_edge_coords[0][2]+tgt_edge_norm_vec[2])) <
-                  fabs(1.0 - fabs(src_edge_coords[1][2]+tgt_edge_norm_vec[2])))
-                prev_is_inside = 2;
-              else
-                curr_is_inside = 2;
+              temp_a = fabs(1.0 - fabs(src_edge_coords[0][2] +
+                                       curr_tgt_edge_norm_vec[2]));
+              temp_b = fabs(1.0 - fabs(src_edge_coords[1][2] +
+                                       curr_tgt_edge_norm_vec[2]));
               break;
             default:
               yac_internal_abort_message("invalid edge type\n", __FILE__, __LINE__);
               return;
           };
 
+          if (temp_a < temp_b) prev_is_inside = 2;
+          else curr_is_inside = 2;
+
           one_intersect_expected = 0;
 
           // if this is the first iteration
@@ -858,14 +911,14 @@ static void point_list_clipping (struct point_list * source_list, int source_ord
         // not
         if ((src_edge_type == LAT_CIRCLE) ^ (tgt_edge_type == LAT_CIRCLE)) {
 
-          double cross_src_z = (long double)src_edge_coords[0][0] *
-                               (long double)src_edge_coords[1][1] -
-                               (long double)src_edge_coords[0][1] *
-                               (long double)src_edge_coords[1][0];
-          double cross_tgt_z = (long double)tgt_edge_coords[0][0] *
-                               (long double)tgt_edge_coords[1][1] -
-                               (long double)tgt_edge_coords[0][1] *
-                               (long double)tgt_edge_coords[1][0];
+          double cross_src_z = (double)((long double)src_edge_coords[0][0] *
+                                        (long double)src_edge_coords[1][1] -
+                                        (long double)src_edge_coords[0][1] *
+                                        (long double)src_edge_coords[1][0]);
+          double cross_tgt_z = (double)((long double)tgt_edge_coords[0][0] *
+                                        (long double)tgt_edge_coords[1][1] -
+                                        (long double)tgt_edge_coords[0][1] *
+                                        (long double)tgt_edge_coords[1][0]);
 
           int same_ordering = source_ordering == target_ordering;
           int same_direction = (cross_src_z > 0) == (cross_tgt_z > 0);
@@ -1135,6 +1188,8 @@ void yac_cell_clipping (size_t N,
     if (source_ordering == -1) continue;
 
     struct point_list * overlap;
+    double fabs_tgt_coordinate_y = fabs(target_cell.coordinates_y[0]);
+    double fabs_src_coordinate_y = fabs(source_cell[n].coordinates_y[0]);
 
     // in this case the source an target cell are both LAT_CELL's than the
     // bigger one has to be the target cell
@@ -1143,8 +1198,7 @@ void yac_cell_clipping (size_t N,
     // include in the target cell
     if (((tgt_cell_type == LAT_CELL) && (src_cell_type == GREAT_CIRCLE_CELL)) ||
         ((tgt_cell_type == LAT_CELL) && (src_cell_type == LAT_CELL) &&
-         (fabs(target_cell.coordinates_y[0]) >
-          fabs(source_cell[n].coordinates_y[0])))) {
+         (fabs_tgt_coordinate_y > fabs_src_coordinate_y))) {
 
       copy_point_list(target_list, &temp_list);
 
@@ -1200,7 +1254,6 @@ void yac_cell_clipping (size_t N,
 void yac_correct_weights ( size_t nSourceCells, double * weight ) {
 
   static const size_t maxIter = 10; // number of iterations to get better accuracy of the weights
-  static double const tol = 1.0e-15;
 
   double weight_diff;
 #ifdef YAC_VERBOSE_CLIPPING
@@ -1359,13 +1412,13 @@ static size_t remove_zero_length_edges(struct point_list * list) {
               curr->vec_coords, curr->next->vec_coords),
            SIN_COS_LOW_TOL) <= 0)) {
       curr->to_be_removed = 1;
-    } else if (curr->edge_type == LAT_CIRCLE &&
-               curr->next->edge_type == LAT_CIRCLE &&
-               (fabs(get_angle(atan2(curr->vec_coords[1] ,
-                                     curr->vec_coords[0]),
-                               atan2(curr->next->next->vec_coords[1] ,
-                                     curr->next->next->vec_coords[0]))) <
-                M_PI_2)) {
+    } else {
+      double temp_a = atan2(curr->vec_coords[1], curr->vec_coords[0]);
+      double temp_b = atan2(curr->next->next->vec_coords[1],
+                            curr->next->next->vec_coords[0]);
+      if (curr->edge_type == LAT_CIRCLE &&
+          curr->next->edge_type == LAT_CIRCLE &&
+          (fabs(get_angle(temp_a, temp_b)) < M_PI_2))
       curr->next->to_be_removed = 1;
     }
 
@@ -1385,19 +1438,19 @@ static size_t generate_point_list(struct point_list * list,
   struct point_list_element * curr = get_free_point_list_element(list);
 
   list->start = curr;
-  curr->vec_coords[0] = cell.coordinates_xyz[0+0*3];
-  curr->vec_coords[1] = cell.coordinates_xyz[1+0*3];
-  curr->vec_coords[2] = cell.coordinates_xyz[2+0*3];
+  curr->vec_coords[0] = cell.coordinates_xyz[0][0];
+  curr->vec_coords[1] = cell.coordinates_xyz[0][1];
+  curr->vec_coords[2] = cell.coordinates_xyz[0][2];
 
-  for (size_t i = 1; i < ((size_t)(cell.num_corners)); ++i) {
+  for (size_t i = 1; i < cell.num_corners; ++i) {
 
     curr->next = get_free_point_list_element(list);
     curr->edge_type = cell.edge_type[i - 1];
     curr = curr->next;
 
-    curr->vec_coords[0] = cell.coordinates_xyz[0+i*3];
-    curr->vec_coords[1] = cell.coordinates_xyz[1+i*3];
-    curr->vec_coords[2] = cell.coordinates_xyz[2+i*3];
+    curr->vec_coords[0] = cell.coordinates_xyz[i][0];
+    curr->vec_coords[1] = cell.coordinates_xyz[i][1];
+    curr->vec_coords[2] = cell.coordinates_xyz[i][2];
     curr->edge_type = cell.edge_type[i];
   }
 
@@ -1459,7 +1512,7 @@ static void free_point_list(struct point_list * list) {
 
 static int is_empty_gc_cell(struct point_list * list, size_t num_edges) {
 
-  double const tol = 1e-6;
+#define NORM_TOL (1e-6)
 
   if ((num_edges == 2) &&
       !((list->start->edge_type == LAT_CIRCLE) ^
@@ -1486,16 +1539,17 @@ static int is_empty_gc_cell(struct point_list * list, size_t num_edges) {
 
     compute_norm_vector(curr->vec_coords, curr->next->vec_coords, norm);
 
-    if (((fabs(ref_norm[0] - norm[0]) > tol) ||
-         (fabs(ref_norm[1] - norm[1]) > tol) ||
-         (fabs(ref_norm[2] - norm[2]) > tol)) &&
-        ((fabs(ref_norm[0] + norm[0]) > tol) ||
-         (fabs(ref_norm[1] + norm[1]) > tol) ||
-         (fabs(ref_norm[2] + norm[2]) > tol)))
+    if (((fabs(ref_norm[0] - norm[0]) > NORM_TOL) ||
+         (fabs(ref_norm[1] - norm[1]) > NORM_TOL) ||
+         (fabs(ref_norm[2] - norm[2]) > NORM_TOL)) &&
+        ((fabs(ref_norm[0] + norm[0]) > NORM_TOL) ||
+         (fabs(ref_norm[1] + norm[1]) > NORM_TOL) ||
+         (fabs(ref_norm[2] + norm[2]) > NORM_TOL)))
       return 0;
   }
 
   return 1;
+#undef NORM_TOL
 }
 
 static void generate_overlap_cell(struct point_list * list,
@@ -1521,7 +1575,7 @@ static void generate_overlap_cell(struct point_list * list,
     free(cell->edge_type);
     cell->coordinates_x = (double *)malloc(num_edges * sizeof(*cell->coordinates_x));
     cell->coordinates_y = (double *)malloc(num_edges * sizeof(*cell->coordinates_y));
-    cell->coordinates_xyz = (double *)malloc(3 * num_edges * sizeof(*cell->coordinates_xyz));
+    cell->coordinates_xyz = (double(*)[3])malloc(num_edges * sizeof(*cell->coordinates_xyz));
     cell->edge_type = (enum yac_edge_type *)malloc(num_edges * sizeof(*cell->edge_type));
     cell->array_size = num_edges;
   }
@@ -1533,11 +1587,12 @@ static void generate_overlap_cell(struct point_list * list,
 
 
     XYZtoLL(curr->vec_coords, cell->coordinates_x+i, cell->coordinates_y+i);
-    cell->coordinates_xyz[0+i*3] = curr->vec_coords[0];
-    cell->coordinates_xyz[1+i*3] = curr->vec_coords[1];
-    cell->coordinates_xyz[2+i*3] = curr->vec_coords[2];
+    cell->coordinates_xyz[i][0] = curr->vec_coords[0];
+    cell->coordinates_xyz[i][1] = curr->vec_coords[1];
+    cell->coordinates_xyz[i][2] = curr->vec_coords[2];
     cell->edge_type[i] = curr->edge_type;
 
     curr = curr->next;
   }
 }
+
