@@ -45,6 +45,9 @@
 #define GEOMETRY_H
 
 #ifdef HAVE_CONFIG_H
+// We include 'config.h' in this header file (which is a bad practice)
+// because we need the definition of the 'restrict' keyword for the
+// inlined functions.
 #include "config.h"
 #endif
 
@@ -59,12 +62,17 @@
 #define M_SQRT3_4 (0.866025403784438646764) /* sqrt(0.75) */
 #endif
 
+struct sin_cos_angle {
+  double sin, cos;
+};
+
 #define SIN_COS_M_PI_2 ((struct sin_cos_angle){.sin = 1.0, .cos = 0.0}) /* PI/2 */
 #define SIN_COS_M_PI ((struct sin_cos_angle){.sin = 0.0, .cos = -1.0}) /* PI */
 #define SIN_COS_7_M_PI_4 \
   ((struct sin_cos_angle){.sin = -0.707106781186547524401, \
                           .cos = +0.707106781186547524401}) /* (7 * PI)/4 */
-#define SIN_COS_ZERO ((struct sin_cos_angle){.sin = 0.0, .cos = 1.0}) /* 0.0 */
+//#define SIN_COS_ZERO ((struct sin_cos_angle){.sin = 0.0, .cos = 1.0}) /* 0.0 */
+static const struct sin_cos_angle SIN_COS_ZERO = {0.0, 1.0}; /* 0.0 */
 #define SIN_COS_TOL \
   ((struct sin_cos_angle){.sin = yac_angle_tol, \
                           .cos = yac_cos_angle_tol}) /* yac_angle_tol */
@@ -93,10 +101,6 @@ struct edge {
    enum yac_edge_type edge_type;
 };
 
-struct sin_cos_angle {
-  double sin, cos;
-};
-
 /**
  * defines a spherical cap, which is used as a convex hull to describe the extents of subdomain
  */
@@ -180,15 +184,18 @@ static inline double get_angle (double a_lon, double b_lon) {
 
 /**
  * searches for overlapping cells between grids \n
- * the user needs to provide a dependency list containing an initial search result that is used as a starting point, which is then refined
+ * the user needs to provide a dependency list containing an initial search
+ * result that is used as a starting point, which is then refined
  * @param[in] src_grid source %grid
  * @param[in] tgt_grid target %grid
- * @param[in] initial_src_to_tgt_dep dependency list containing for each source cell a list of target cell that might overlap
- * @param[out] src_to_tgt_dep dependency list containing the result of the search
+ * @param[in] initial_src_to_tgt_dep dependency list containing for each source
+ *                                   cell a list of target cell that might
+ *                                   overlap
+ * @return dependency list containing the result of the search
  */
-void yac_find_overlapping_cells (struct grid * src_grid, struct grid * tgt_grid,
-                                 struct dep_list initial_src_to_tgt_dep,
-                                 struct dep_list * src_to_tgt_dep);
+struct dep_list * yac_find_overlapping_cells(
+  struct grid * src_grid, struct grid * tgt_grid,
+  struct dep_list * initial_src_to_tgt_dep);
 
 /**
  * searches for all overlapping cells of a given grid that overlap with a
@@ -508,13 +515,12 @@ void yac_get_cell_bounding_circle_unstruct_triangle(
  * @param[in]  a          coordinates of first point (xyz)
  * @param[in]  b          coordinates of second point (xyz)
  * @param[in]  c          coordinates of thrid point (xyz)
- * @param[in]  d          coordinates of fourth point (xyz)
  * @param[out] bnd_circle circumscribe circle
  * @remark it is assumed that all edges of the quad are either circles of
  *         longitude or latitude
  */
 void yac_get_cell_circumscribe_circle_reg_quad(
-   double a[3], double b[3], double c[3], double d[3],
+   double a[3], double b[3], double c[3],
    struct bounding_circle * bnd_circle);
 
 /**
@@ -522,13 +528,12 @@ void yac_get_cell_circumscribe_circle_reg_quad(
  * @param[in]  a          coordinates of first point (xyz)
  * @param[in]  b          coordinates of second point (xyz)
  * @param[in]  c          coordinates of thrid point (xyz)
- * @param[in]  d          coordinates of fourth point (xyz)
  * @param[out] bnd_circle bounding circle
  * @remark it is assumed that all edges of the quad are either circles of
  *         longitude or latitude
  */
 void yac_get_cell_bounding_circle_reg_quad(
-   double a[3], double b[3], double c[3], double d[3],
+   double a[3], double b[3], double c[3],
    struct bounding_circle * bnd_circle);
 
 /** \example test_grid_bnd_circle.c
@@ -560,9 +565,9 @@ void yac_get_grid_bounding_circle(struct grid * grid,
  */
 void yac_get_matching_grid_cells(struct grid * grid, struct bounding_circle extent,
                                  struct grid_cell ** matching_cells,
-                                 unsigned * curr_matching_cells_array_size,
-                                 unsigned ** local_ids, unsigned * curr_local_ids_array_size,
-                                 unsigned * num_matching_cells, unsigned offset);
+                                 size_t * curr_matching_cells_array_size,
+                                 unsigned ** local_ids, size_t * curr_local_ids_array_size,
+                                 size_t * num_matching_cells, size_t offset);
 
 /**
  * checks whether two extents overlap
@@ -570,8 +575,8 @@ void yac_get_matching_grid_cells(struct grid * grid, struct bounding_circle exte
  * @param[in] extent_b bounding circle
  * @return 0 if the bounding circles do not overlap
  */
-unsigned yac_extents_overlap(struct bounding_circle * extent_a,
-                             struct bounding_circle * extent_b);
+int yac_extents_overlap(struct bounding_circle * extent_a,
+                        struct bounding_circle * extent_b);
 
 /**
  * checks whether a point is within a bounding circle
@@ -579,8 +584,8 @@ unsigned yac_extents_overlap(struct bounding_circle * extent_a,
  * @param[in] bnd_circle bounding circle
  * @return 0 if point is not within the bounding circle
  */
-unsigned yac_point_in_bounding_circle(struct point point,
-                                      struct bounding_circle * bnd_circle);
+int yac_point_in_bounding_circle(struct point point,
+                                 struct bounding_circle * bnd_circle);
 
 /**
  * checks whether a point is within a bounding circle
@@ -588,8 +593,8 @@ unsigned yac_point_in_bounding_circle(struct point point,
  * @param[in] bnd_circle bounding circle
  * @return 0 if point is not within the bounding circle
  */
-unsigned yac_point_in_bounding_circle_vec(double point_vector[3],
-                                          struct bounding_circle * bnd_circle);
+int yac_point_in_bounding_circle_vec(double point_vector[3],
+                                     struct bounding_circle * bnd_circle);
 
 /**
  * converts lon-lat coordinates into xyz ones
@@ -619,7 +624,7 @@ static inline void LLtoXYZ(double lon, double lat, double p_out[]) {
  * @param[out] p_out xyz coordinates
  */
 static inline void LLtoXYZ_deg(double lon, double lat, double p_out[]) {
-   LLtoXYZ(lon*rad, lat*rad, p_out);
+   LLtoXYZ(lon*YAC_RAD, lat*YAC_RAD, p_out);
 }
 
 /**
@@ -645,9 +650,9 @@ static inline void crossproduct_ld (double a[], double b[], double cross[]) {
    long double a_[3] = {a[0], a[1], a[2]};
    long double b_[3] = {b[0], b[1], b[2]};
 
-   cross[0] = a_[1] * b_[2] - a_[2] * b_[1];
-   cross[1] = a_[2] * b_[0] - a_[0] * b_[2];
-   cross[2] = a_[0] * b_[1] - a_[1] * b_[0];
+   cross[0] = (double)(a_[1] * b_[2] - a_[2] * b_[1]);
+   cross[1] = (double)(a_[2] * b_[0] - a_[0] * b_[2]);
+   cross[2] = (double)(a_[0] * b_[1] - a_[1] * b_[0]);
 }
 
 /**
@@ -848,7 +853,7 @@ static inline int sum_angles(
   return (compare_angles(sum_, a) < 0) || (compare_angles(sum_, b) < 0);
 }
 
-//! computes (a-b), if (a-b) < 0, the result is (a+b+2*PI)
+//! computes (a-b), if (a-b) < 0, the result is (a-b+2*PI)
 //! a and be have to be in the range [0;2*PI[
 static inline struct sin_cos_angle sub_angles_no_check(
   struct sin_cos_angle a, struct sin_cos_angle b) {
@@ -857,16 +862,20 @@ static inline struct sin_cos_angle sub_angles_no_check(
                            a.cos * b.cos + a.sin * b.sin);
 }
 
-//! computes (a-b), if (a-b) < 0, the result is (a+b+2*PI)
+//! computes (a-b), if (a-b) <= 0, the result is (a-b+2*PI)
 //! a and be have to be in the range [0;2*PI[
 //! \returns 1 if (a-b) < 0
 static inline int sub_angles(
   struct sin_cos_angle a, struct sin_cos_angle b,
   struct sin_cos_angle * restrict sub) {
 
-  *sub = sub_angles_no_check(a, b);
+  int compare_result = compare_angles(a, b);
 
-  return compare_angles(a, b) < 0;
+  // return sin=0.0 and cos=1.0 if the angles are equal to each other,
+  // i.e. compare_result == 0
+  *sub = compare_result ? sub_angles_no_check(a, b) : SIN_COS_ZERO;
+
+  return compare_result < 0;
 }
 
 //! return angles in the range of [0;2PI[
@@ -917,7 +926,7 @@ static inline struct sin_cos_angle half_angle(struct sin_cos_angle angle) {
 //! I derived it based on \ref half_angle
 static inline struct sin_cos_angle quarter_angle(struct sin_cos_angle angle) {
 
-  double tan = fabs(angle.sin) / (1.0 + fabs(angle.cos));
+  double tan = fabs(angle.sin / (1.0 + fabs(angle.cos)));
   double one_plus_sq_tan = 1.0 + tan * tan;
   double sqrt_one_plus_sq_tan = sqrt(one_plus_sq_tan);
 
@@ -1017,8 +1026,10 @@ static inline void rotate_vector2(
 static inline void rotate_vector(
   double axis[], double angle, double v_in[], double v_out[]) {
 
+  double sin_angle = sin(angle);
+  double cos_angle = cos(angle);
   rotate_vector2(
-    axis, sin_cos_angle_new(sin(angle), cos(angle)), v_in, v_out);
+    axis, sin_cos_angle_new(sin_angle, cos_angle), v_in, v_out);
 }
 
 /**
@@ -1057,7 +1068,7 @@ int yac_do_intersect (struct edge edge_a, double a[3], double b[3],
  *         3 corners and assumes them to be great circle edges
  */
 void yac_triangulate_cell(
-  struct grid_cell cell, unsigned start_corner, struct grid_cell * triangles);
+  struct grid_cell cell, size_t start_corner, struct grid_cell * triangles);
 
 /**
  * splits given indices of a cell into triangles
@@ -1072,8 +1083,8 @@ void yac_triangulate_cell(
  *         for the argument triangle_indices
  */
 void yac_triangulate_cell_indices(
-  unsigned const * cell_indices, unsigned num_corners, unsigned start_corner,
-  unsigned * triangle_indices);
+  unsigned const * cell_indices, size_t num_corners, size_t start_corner,
+  size_t (*triangle_indices)[3]);
 
 /**
  * computes a spherical triangle that contains all given vertices,
@@ -1089,3 +1100,4 @@ void yac_compute_bnd_triangle(
   double * vertices, size_t num_vertices, double * triangle, size_t num_tests);
 
 #endif // GEOMETRY_H
+
