@@ -70,7 +70,9 @@ static inline int vector_is_between (
   // ||ab||^2 >= ||ap||^2 + ||bp||^2 => (Pythagorean theorem)
 
   // the tol is required in case p is very close to a or b
-  return sq_len_diff_ab + tol >= sq_len_diff_vec(a,p) + sq_len_diff_vec(b,p);
+  double sq_len_diff_vec_ap = sq_len_diff_vec(a,p);
+  double sq_len_diff_vec_bp = sq_len_diff_vec(b,p);
+  return sq_len_diff_ab + tol >= sq_len_diff_vec_ap + sq_len_diff_vec_bp;
 }
 
 static int vector_is_between_lat (double a[], double b[], double p[]) {
@@ -85,7 +87,8 @@ static int vector_is_between_lat (double a[], double b[], double p[]) {
 
    if (fabs(fabs(a[2]) - 1.0) < tol) return 1;
 
-   int flag = fabs(a[0]) > fabs(a[1]);
+   double fabs_a[2] = {fabs(a[0]), fabs(a[1])};
+   int flag = fabs_a[0] > fabs_a[1];
    double a_0 = a[flag], a_1 = a[flag^1];
    double b_0 = b[flag], b_1 = b[flag^1];
    double p_0 = p[flag], p_1 = p[flag^1];
@@ -172,13 +175,13 @@ static int vector_is_between_lat (double a[], double b[], double p[]) {
                    cross_cd[2] * cross_cd[2];
       sq_sin_ab = (double)sq_sin_ab_;
       sq_sin_cd = (double)sq_sin_cd_;
-      if ((sq_sin_ab != 0.0) && (sq_sin_cd != 0.0)) {
+      if ((sq_sin_ab > 0.0) && (sq_sin_cd > 0.0)) {
          long double sq_length_abxcd = cross_abxcd[0] * cross_abxcd[0] +
                                        cross_abxcd[1] * cross_abxcd[1] +
                                        cross_abxcd[2] * cross_abxcd[2];
          sq_sin_abxcd = (double)(sq_length_abxcd / (sq_sin_ab_ * sq_sin_cd_));
-         if (sq_length_abxcd != 0.0l) {
-            long double scale = 1.0l / sqrtl(sq_length_abxcd);
+         if (sq_length_abxcd > 0.0) {
+            long double scale = 1.0 / sqrtl(sq_length_abxcd);
             p[0] = (double)(cross_abxcd[0] * scale);
             p[1] = (double)(cross_abxcd[1] * scale);
             p[2] = (double)(cross_abxcd[2] * scale);
@@ -197,10 +200,12 @@ static int vector_is_between_lat (double a[], double b[], double p[]) {
       double sq_len_diff_ab = sq_len_diff_vec(a, b);
       double sq_len_diff_cd = sq_len_diff_vec(c, d);
 
-      return (vector_is_between(a, b, p, sq_len_diff_ab) << 0) |
-             (vector_is_between(a, b, q, sq_len_diff_ab) << 1) |
-             (vector_is_between(c, d, p, sq_len_diff_cd) << 2) |
-             (vector_is_between(c, d, q, sq_len_diff_cd) << 3);
+      int ret = 0;
+      ret |= vector_is_between(a, b, p, sq_len_diff_ab) << 0;
+      ret |= vector_is_between(a, b, q, sq_len_diff_ab) << 1;
+      ret |= vector_is_between(c, d, p, sq_len_diff_cd) << 2;
+      ret |= vector_is_between(c, d, q, sq_len_diff_cd) << 3;
+      return ret;
 
     // both edges are on the same great circle
     } else {
@@ -283,9 +288,9 @@ static int vector_is_between_lat (double a[], double b[], double p[]) {
       // edge cd has at least a length of yac_angle_tol
     } else {
 
-      double sin_a_cd = fabs((double)(cross_cd[0]) * a[0] +
-                             (double)(cross_cd[1]) * a[1] +
-                             (double)(cross_cd[2]) * a[2]) / sqrt(sq_sin_cd) ;
+      double sin_a_cd = fabs(((double)(cross_cd[0]) * a[0] +
+                              (double)(cross_cd[1]) * a[1] +
+                              (double)(cross_cd[2]) * a[2]) / sqrt(sq_sin_cd));
 
       // if ab is on the plane of cd
       if (sin_a_cd <= yac_angle_tol) {
@@ -314,9 +319,9 @@ static int vector_is_between_lat (double a[], double b[], double p[]) {
 
     // dot product is most accurate for angles around half PI
     // (for angles very close to half PI: alpha = fabs(acos(alpha))
-    double sin_c_ad = fabs((double)(cross_ab[0]) * c[0] +
-                           (double)(cross_ab[1]) * c[1] +
-                           (double)(cross_ab[2]) * c[2]) / sqrt(sq_sin_ab);
+    double sin_c_ad = fabs(((double)(cross_ab[0]) * c[0] +
+                            (double)(cross_ab[1]) * c[1] +
+                            (double)(cross_ab[2]) * c[2]) / sqrt(sq_sin_ab));
 
     // if cd is on the plane of ab
     if (sin_c_ad <= yac_angle_tol) {
@@ -834,9 +839,9 @@ int yac_loncxlonc_vec (double a[3], double b[3], double c[3], double d[3],
 
    int edge_is_pole_point = 0;
 
-   double * ref_point;
-
-   ref_point = (fabs(a[2]) > fabs(b[2]))?b:a;
+   double fabs_a_2 = fabs(a[2]);
+   double fabs_b_2 = fabs(b[2]);
+   double * ref_point = (fabs_a_2 > fabs_b_2)?b:a;
    // if both points are at the pole
    if (fabs(ref_point[2]) > 1.0-tol) {
       abs_norm_cross_ab[0] = 1;
@@ -847,8 +852,10 @@ int yac_loncxlonc_vec (double a[3], double b[3], double c[3], double d[3],
                                 ref_point[1]*ref_point[1]);
       abs_norm_cross_ab[0] = ref_point[1] * scale;
       abs_norm_cross_ab[1] = ref_point[0] * scale;
-      double max_abs_val = (fabs(abs_norm_cross_ab[0]) >
-                            fabs(abs_norm_cross_ab[1]))?(abs_norm_cross_ab[0]):
+      double fabs_abs_norm_cross_ab[2] = {fabs(abs_norm_cross_ab[0]),
+                                          fabs(abs_norm_cross_ab[1])};
+      double max_abs_val = (fabs_abs_norm_cross_ab[0] >
+                            fabs_abs_norm_cross_ab[1])?(abs_norm_cross_ab[0]):
                                                         (abs_norm_cross_ab[1]);
       if (max_abs_val < 0) {
          abs_norm_cross_ab[0] *= -1.0;
@@ -856,7 +863,9 @@ int yac_loncxlonc_vec (double a[3], double b[3], double c[3], double d[3],
       }
    }
 
-   ref_point = (fabs(c[2]) > fabs(d[2]))?d:c;
+   double fabs_c_2 = fabs(c[2]);
+   double fabs_d_2 = fabs(d[2]);
+   ref_point = (fabs_c_2 > fabs_d_2)?d:c;
    // if both points are at the pole
    if (fabs(ref_point[2]) > 1.0-tol) {
       abs_norm_cross_cd[0] = 0;
@@ -867,9 +876,11 @@ int yac_loncxlonc_vec (double a[3], double b[3], double c[3], double d[3],
                                 ref_point[1]*ref_point[1]);
       abs_norm_cross_cd[0] = ref_point[1] * scale;
       abs_norm_cross_cd[1] = ref_point[0] * scale;
-      double max_abs_val = (fabs(abs_norm_cross_cd[0]) >
-                            fabs(abs_norm_cross_cd[1]))?(abs_norm_cross_cd[0]):
-                                                        (abs_norm_cross_cd[1]);
+      double fabs_abs_norm_cross_cd[2] = {fabs(abs_norm_cross_cd[0]),
+                                          fabs(abs_norm_cross_cd[1])};
+      double max_abs_val =
+        (fabs_abs_norm_cross_cd[0] > fabs_abs_norm_cross_cd[1])?
+        (abs_norm_cross_cd[0]):(abs_norm_cross_cd[1]);
       if (max_abs_val < 0) {
          abs_norm_cross_cd[0] *= -1.0;
          abs_norm_cross_cd[1] *= -1.0;
@@ -1403,8 +1414,11 @@ int yac_gcxlatc(struct edge edge_a, struct edge edge_b,
  *          p and q will be identically, but only the p bits will be set
  **/
 
+#if defined __INTEL_COMPILER
 #pragma intel optimization_level 0
-
+#elif defined _CRAYC
+#pragma _CRI noopt
+#endif
 int yac_gcxlatc_vec(double a[3], double b[3], double c[3], double d[3],
                     double p[3], double q[3]) {
 
@@ -1417,31 +1431,34 @@ int yac_gcxlatc_vec(double a[3], double b[3], double c[3], double d[3],
                 cross_ab[1]*cross_ab[1]+
                 cross_ab[2]*cross_ab[2]);
 
+   double fabs_a_2 = fabs(a[2]);
+   double fabs_b_2 = fabs(b[2]);
+
    // if the great circle is the equator
-   if (fabs(a[2]) < tol && fabs(b[2]) < tol) {
+   if (fabs_a_2 < tol && fabs_b_2 < tol) {
 
       return yac_latcxlatc_vec(a, b, c, d, p, q);
 
    // if the great circle is  a circle of longitude
    } else if (scale < tol || fabs(cross_ab[2]/scale) < tol ||
-              fabs(fabs(a[2])-1.0) < 1e-13 ||
-              fabs(fabs(b[2])-1.0) < 1e-13) {
+              fabs(fabs_a_2-1.0) < 1e-13 ||
+              fabs(fabs_b_2-1.0) < 1e-13) {
 
       return yac_loncxlatc_vec(a, b, c, d, p, q);
    }
 
    double t[3], s[3];
 
-   if (fabs(a[2]) > fabs(b[2])) {
+   if (fabs_a_2 > fabs_b_2) {
 
-      double scale = c[2] / a[2];
+      scale = c[2] / a[2];
 
       t[0] = scale * a[0];
       t[1] = scale * a[1];
 
    } else {
 
-      double scale = c[2] / b[2];
+      scale = c[2] / b[2];
 
       t[0] = scale * b[0];
       t[1] = scale * b[1];
@@ -1451,16 +1468,16 @@ int yac_gcxlatc_vec(double a[3], double b[3], double c[3], double d[3],
 
    s[2] = 0;
 
-   if (fabs(a[2]) < tol)
+   if (fabs_a_2 < tol)
       s[0] = a[0], s[1] = a[1];
-   else if (fabs(b[2]) < tol)
+   else if (fabs_b_2 < tol)
       s[0] = b[0], s[1] = b[1];
-   else if (fabs(a[2]) > fabs(b[2])) {
-      double scale = b[2] / a[2];
+   else if (fabs_a_2 > fabs_b_2) {
+      scale = b[2] / a[2];
       s[0] = b[0] - scale * a[0];
       s[1] = b[1] - scale * a[1];
    } else {
-      double scale = a[2] / b[2];
+      scale = a[2] / b[2];
       s[0] = a[0] - scale * b[0];
       s[1] = a[1] - scale * b[1];
    }
@@ -1476,7 +1493,7 @@ int yac_gcxlatc_vec(double a[3], double b[3], double c[3], double d[3],
       double b_ = 2.0 * (t[0] * s[0] + t[1] * s[1]);
       double c_ = t[0] * t[0] + t[1] * t[1] + c[2] * c[2] - 1.0;
 
-      if (a_ == 0.0)
+      if (!(fabs(a_) > 0.0))
          yac_internal_abort_message("internal error", __FILE__, __LINE__);
 
       double temp = b_ * b_ - 4.0 * a_ * c_;
@@ -1517,9 +1534,15 @@ int yac_gcxlatc_vec(double a[3], double b[3], double c[3], double d[3],
 
    return result;
 }
+#if defined _CRAYC
+#pragma _CRI opt
+#endif
 
 static int gcxlatc_vec_(double a[3], double b[3], double c[3], double d[3]) {
 
+   double fabs_a_2 = fabs(a[2]);
+   double fabs_b_2 = fabs(b[2]);
+
    // if ab is a point
    if (compare_angles(get_vector_angle_2(a, b), SIN_COS_TOL) <= 0) {
 
@@ -1546,7 +1569,7 @@ static int gcxlatc_vec_(double a[3], double b[3], double c[3], double d[3]) {
 
    double t[3], s[3];
 
-   if (fabs(a[2]) > fabs(b[2])) {
+   if (fabs_a_2 > fabs_b_2) {
 
       double scale = c[2] / a[2];
 
@@ -1565,11 +1588,11 @@ static int gcxlatc_vec_(double a[3], double b[3], double c[3], double d[3]) {
 
    s[2] = 0;
 
-   if (fabs(a[2]) < tol)
+   if (fabs_a_2 < tol)
       s[0] = a[0], s[1] = a[1];
-   else if (fabs(b[2]) < tol)
+   else if (fabs_b_2 < tol)
       s[0] = b[0], s[1] = b[1];
-   else if (fabs(a[2]) > fabs(b[2])) {
+   else if (fabs_a_2 > fabs_b_2) {
       double scale = b[2] / a[2];
       s[0] = b[0] - scale * a[0];
       s[1] = b[1] - scale * a[1];
@@ -1836,3 +1859,4 @@ int yac_do_intersect (struct edge edge_a, double a[3], double b[3],
          exit(EXIT_FAILURE);
    };
 }
+
