@@ -30,6 +30,11 @@
  * along with YAC.  If not, see <http://www.gnu.org/licenses/gpl.txt>.
  */
 
+#ifdef HAVE_CONFIG_H
+// Get the definition of the 'restrict' keyword.
+#include "config.h"
+#endif
+
 #include <assert.h>
 #include <stdlib.h>
 #include <math.h>
@@ -101,47 +106,39 @@ struct temp_partition_data {
   int node_type;
 };
 
-static void sphere_part_do_cell_search(struct grid_search * search,
-                                       struct grid * grid_data,
-                                       struct dep_list * tgt_to_src_cells);
+static struct dep_list * sphere_part_do_cell_search(struct grid_search * search,
+                                                   struct grid * grid_data);
 static void sphere_part_do_cell_search_single(struct grid_search * search,
                                               struct grid_cell cell,
                                               size_t * n_cells,
                                               size_t * cells_size,
                                               size_t ** cells);
-static void sphere_part_do_point_search_c(struct grid_search * search,
-                                          struct grid * grid_data,
-                                          struct dep_list * tgt_to_src_cells);
-static void sphere_part_do_point_search_c2(struct grid_search * search,
-                                           double (*coordinates_xyz)[3],
-                                           size_t num_points,
-                                           struct dep_list * tgt_to_src_cells);
-static void sphere_part_do_point_search_c3(struct grid_search * search,
-                                           double (*coordinates_xyz)[3],
-                                           size_t num_points,
-                                           struct dep_list * tgt_to_src_cells,
-                                           struct points * points);
-static void sphere_part_do_point_search_p(struct grid_search * search,
-                                          struct grid * grid_data,
-                                          struct dep_list * target_to_src_points);
-static void sphere_part_do_point_search_p2(struct grid_search * search,
-                                           double (*coordinates_xyz)[3],
-                                           size_t num_points,
-                                           struct dep_list * target_to_src_points);
-static void sphere_part_do_point_search_p3(struct grid_search * search,
-                                           double (*coordinates_xyz)[3],
-                                           size_t num_points,
-                                           struct dep_list * target_to_src_points,
-                                           struct points * points);
+static struct dep_list * sphere_part_do_point_search_c(struct grid_search * search,
+                                                       struct grid * grid_data);
+static struct dep_list * sphere_part_do_point_search_c2(struct grid_search * search,
+                                                        double (*coordinates_xyz)[3],
+                                                        size_t num_points);
+static struct dep_list * sphere_part_do_point_search_c3(struct grid_search * search,
+                                                        double (*coordinates_xyz)[3],
+                                                        size_t num_points,
+                                                        struct points * points);
+static struct dep_list * sphere_part_do_point_search_p(struct grid_search * search,
+                                                       struct grid * grid_data);
+static struct dep_list * sphere_part_do_point_search_p2(struct grid_search * search,
+                                                        double (*coordinates_xyz)[3],
+                                                        size_t num_points);
+static struct dep_list * sphere_part_do_point_search_p3(struct grid_search * search,
+                                                        double (*coordinates_xyz)[3],
+                                                        size_t num_points,
+                                                        struct points * points);
 static void sphere_part_do_point_search_p4 (struct grid_search * search,
                                             double coordinate_xyz[3],
                                             size_t * n_points,
                                             size_t * points_size,
                                             size_t ** points);
-static void sphere_part_do_bnd_circle_search (struct grid_search * search,
-                                              struct bounding_circle * bnd_circles,
-                                              size_t n,
-                                              struct dep_list * bnd_to_cells);
+static struct dep_list * sphere_part_do_bnd_circle_search (struct grid_search * search,
+                                                           struct bounding_circle * bnd_circles,
+                                                           size_t n);
 static void delete_sphere_part_search(struct grid_search * search);
 
 static struct grid_search_vtable sphere_part_search_vtable =
@@ -521,7 +518,7 @@ struct grid_search * yac_sphere_part_search_new (struct grid * grid) {
 
    init_sphere_part_node(&(search->base_node));
 
-   size_t num_grid_cells = yac_get_num_grid_cells(grid);
+   size_t num_grid_cells = (size_t)yac_get_num_grid_cells(grid);
    size_t * local_cell_ids = malloc(num_grid_cells * sizeof(*local_cell_ids));
    search->local_cell_ids = local_cell_ids;
 
@@ -530,7 +527,8 @@ struct grid_search * yac_sphere_part_search_new (struct grid * grid) {
       malloc(num_grid_cells * sizeof(*part_data));
    yac_init_grid_cell(&dummy_cell);
    for (size_t i = 0; i < num_grid_cells; ++i) {
-      yac_get_grid_cell2(grid, i, &dummy_cell, &(part_data[i].bnd_circle));
+      yac_get_grid_cell2(
+         grid, (unsigned)i, &dummy_cell, &(part_data[i].bnd_circle));
       part_data[i].local_id = i;
    }
    yac_free_grid_cell(&dummy_cell);
@@ -1097,7 +1095,7 @@ void yac_point_sphere_part_search_NN(struct point_sphere_part_search * search,
 
     size_t curr_tree_depth = 0;
     struct point_id_xyz * points = NULL;
-    size_t num_points = 0;
+    size_t curr_num_points = 0;
 
     // get the matching leaf for the current point
     do {
@@ -1118,13 +1116,13 @@ void yac_point_sphere_part_search_NN(struct point_sphere_part_search * search,
         if (curr_node->flags & U_IS_LEAF) {
           if (curr_node->U_size > 0) {
             points = (struct point_id_xyz*)(curr_node->U);
-            num_points = curr_node->U_size;
+            curr_num_points = curr_node->U_size;
             break;
           } else {
             flags[curr_tree_depth] = T_FLAG;
             if (curr_node->flags & T_IS_LEAF) {
               points = (struct point_id_xyz*)(curr_node->T);
-              num_points = curr_node->T_size;
+              curr_num_points = curr_node->T_size;
               break;
             } else {
               curr_node = curr_node->T;
@@ -1140,13 +1138,13 @@ void yac_point_sphere_part_search_NN(struct point_sphere_part_search * search,
         if (curr_node->flags & T_IS_LEAF) {
           if (curr_node->T_size > 0) {
             points = (struct point_id_xyz*)(curr_node->T);
-            num_points = curr_node->T_size;
+            curr_num_points = curr_node->T_size;
             break;
           } else {
             flags[curr_tree_depth] = U_FLAG;
             if (curr_node->flags & U_IS_LEAF) {
               points = (struct point_id_xyz*)(curr_node->U);
-              num_points = curr_node->U_size;
+              curr_num_points = curr_node->U_size;
               break;
             } else {
               curr_node = curr_node->U;
@@ -1160,7 +1158,7 @@ void yac_point_sphere_part_search_NN(struct point_sphere_part_search * search,
 
     // if we do not have to do a finer search
     if (leaf_contains_matching_points(
-          points, num_points, curr_coordinates_xyz, local_point_ids,
+          points, curr_num_points, curr_coordinates_xyz, local_point_ids,
           local_point_ids_array_size, result_coordinates_xyz,
           result_coordinates_xyz_array_size, total_num_local_point_ids,
           num_local_point_ids + i)) {
@@ -1176,7 +1174,7 @@ void yac_point_sphere_part_search_NN(struct point_sphere_part_search * search,
       bnd_circle.inc_angle = SIN_COS_M_PI;
 
       check_leaf_NN(
-        points, num_points, curr_coordinates_xyz, &(bnd_circle.inc_angle),
+        points, curr_num_points, curr_coordinates_xyz, &(bnd_circle.inc_angle),
         result_coordinates_xyz, result_coordinates_xyz_array_size,
         local_point_ids, local_point_ids_array_size, total_num_local_point_ids,
         num_local_point_ids + i);
@@ -1239,7 +1237,8 @@ static size_t initial_point_bnd_search_NNN(
 
   for (num_results = n;
        (num_results < num_points) &&
-       (min_cos_angle == results_[num_results].cos_angle); ++num_results);
+       !(fabs(min_cos_angle - results_[num_results].cos_angle) > 0.0);
+       ++num_results);
 
   return num_results;
 }
@@ -1295,11 +1294,11 @@ static inline struct sin_cos_angle check_leaf_NNN(
     if (num_results_ > n) {
 
       size_t new_num_results;
-      double min_cos_angle = results_[n - 1].cos_angle;
+      min_cos_angle = results_[n - 1].cos_angle;
 
       for (new_num_results = n;
            (new_num_results < num_results_) &&
-           (min_cos_angle == results_[new_num_results].cos_angle);
+           !(fabs(min_cos_angle - results_[new_num_results].cos_angle) > 0.0);
            ++new_num_results);
       num_results_ = new_num_results;
     }
@@ -1468,7 +1467,7 @@ void yac_point_sphere_part_search_NNN(struct point_sphere_part_search * search,
 
     size_t curr_tree_depth = 0;
     struct point_id_xyz * points = search->points;
-    size_t num_points = 0;
+    size_t curr_num_points = 0;
 
     // get the matching leaf for the current point
     do {
@@ -1487,12 +1486,12 @@ void yac_point_sphere_part_search_NNN(struct point_sphere_part_search * search,
         if (curr_node->U_size < n) {
 
           flags[curr_tree_depth] = U_FLAG + T_FLAG;
-          num_points = curr_node->U_size + curr_node->T_size;
+          curr_num_points = curr_node->U_size + curr_node->T_size;
           break;
         } else if (curr_node->flags & U_IS_LEAF) {
 
           flags[curr_tree_depth] = U_FLAG;
-          num_points = curr_node->U_size;
+          curr_num_points = curr_node->U_size;
           break;
         } else {
 
@@ -1505,13 +1504,13 @@ void yac_point_sphere_part_search_NNN(struct point_sphere_part_search * search,
         if (curr_node->T_size < n) {
 
           flags[curr_tree_depth] = U_FLAG + T_FLAG;
-          num_points = curr_node->U_size + curr_node->T_size;
+          curr_num_points = curr_node->U_size + curr_node->T_size;
           break;
         } else if (curr_node->flags & T_IS_LEAF) {
 
           points += curr_node->U_size;
           flags[curr_tree_depth] = T_FLAG;
-          num_points = curr_node->T_size;
+          curr_num_points = curr_node->T_size;
           break;
         } else {
 
@@ -1524,11 +1523,11 @@ void yac_point_sphere_part_search_NNN(struct point_sphere_part_search * search,
       curr_tree_depth++;
     } while (1);
 
-    assert(num_points > 0);
+    assert(curr_num_points > 0);
 
     size_t num_results =
       initial_point_bnd_search_NNN(
-        n, points, num_points, curr_coordinates_xyz,
+        n, points, curr_num_points, curr_coordinates_xyz,
         &results, &results_array_size);
 
     // do a detailed search
@@ -1560,14 +1559,14 @@ void yac_point_sphere_part_search_NNN(struct point_sphere_part_search * search,
       result_coordinates_xyz_ = NULL;
     }
 
-    for (size_t i = 0; i < num_results; ++i) {
+    for (size_t j = 0; j < num_results; ++j) {
 
-      local_point_ids_[i] = results[i].idx;
-      if (cos_angles_ != NULL) cos_angles_[i] = results[i].cos_angle;
+      local_point_ids_[j] = results[j].idx;
+      if (cos_angles_ != NULL) cos_angles_[j] = results[j].cos_angle;
       if (result_coordinates_xyz_ != NULL) {
-        result_coordinates_xyz_[i][0] = results[i].coordinates_xyz[0];
-        result_coordinates_xyz_[i][1] = results[i].coordinates_xyz[1];
-        result_coordinates_xyz_[i][2] = results[i].coordinates_xyz[2];
+        result_coordinates_xyz_[j][0] = results[j].coordinates_xyz[0];
+        result_coordinates_xyz_[j][1] = results[j].coordinates_xyz[1];
+        result_coordinates_xyz_[j][2] = results[j].coordinates_xyz[2];
       }
     }
 
@@ -1681,9 +1680,8 @@ static void search_point(struct sphere_part_node * node,
    }
 }
 
-static void sphere_part_do_cell_search(struct grid_search * search,
-                                       struct grid * grid_data,
-                                       struct dep_list * tgt_to_src_cells) {
+static struct dep_list * sphere_part_do_cell_search(struct grid_search * search,
+                                                    struct grid * grid_data) {
 
    struct sphere_part_search * sp_search = (struct sphere_part_search *)search;
    struct sphere_part_node * base_node = &(sp_search->base_node);
@@ -1702,7 +1700,7 @@ static void sphere_part_do_cell_search(struct grid_search * search,
 
    struct overlaps search_interval_tree_buffer = {0, 0, NULL};
 
-   unsigned * num_src_per_tgt_cell =
+   size_t * num_src_per_tgt_cell =
       calloc(num_cells, sizeof(*num_src_per_tgt_cell));
    unsigned * tgt_src_dependencies = NULL;
    size_t tgt_src_dependencies_size = 0;
@@ -1746,9 +1744,9 @@ static void sphere_part_do_cell_search(struct grid_search * search,
 
    tgt_src_dependencies = realloc(tgt_src_dependencies, total_num_dependencies *
                                   sizeof(tgt_src_dependencies));
-   yac_init_dep_list(tgt_to_src_cells);
-   yac_set_dependencies(tgt_to_src_cells, (unsigned)num_cells,
-                        num_src_per_tgt_cell, tgt_src_dependencies);
+   struct dep_list * tgt_to_src_cells =
+      yac_dep_list_new(num_cells, num_src_per_tgt_cell, tgt_src_dependencies);
+   return tgt_to_src_cells;
 }
 
 static void sphere_part_do_cell_search_single(struct grid_search * search,
@@ -1795,9 +1793,8 @@ static void sphere_part_do_cell_search_single(struct grid_search * search,
    free(search_interval_tree_buffer.overlap_iv);
 }
 
-static void sphere_part_do_point_search_c(struct grid_search * search,
-                                          struct grid * grid_data,
-                                          struct dep_list * tgt_to_src_cells) {
+static struct dep_list * sphere_part_do_point_search_c(struct grid_search * search,
+                                                       struct grid * grid_data) {
 
    struct sphere_part_search * sp_search = (struct sphere_part_search *)search;
    struct sphere_part_node * base_node = &(sp_search->base_node);
@@ -1815,7 +1812,7 @@ static void sphere_part_do_point_search_c(struct grid_search * search,
 
    struct overlaps search_interval_tree_buffer = {0, 0, NULL};
 
-   unsigned * num_src_per_tgt_corner =
+   size_t * num_src_per_tgt_corner =
       calloc(num_corners, sizeof(*num_src_per_tgt_corner));
    unsigned * tgt_src_dependencies = NULL;
    size_t tgt_src_dependencies_size = 0;
@@ -1823,8 +1820,9 @@ static void sphere_part_do_point_search_c(struct grid_search * search,
 
    for (size_t i = 0; i < num_corners; ++i) {
       double point_3d[3];
-      LLtoXYZ(yac_get_corner_x_coord(grid_data, i),
-              yac_get_corner_y_coord(grid_data, i), point_3d);
+      double x_coord = yac_get_corner_x_coord(grid_data, i);
+      double y_coord = yac_get_corner_y_coord(grid_data, i);
+      LLtoXYZ(x_coord, y_coord, point_3d);
 
       num_temp_search_results = 0;
 
@@ -1857,19 +1855,19 @@ static void sphere_part_do_point_search_c(struct grid_search * search,
 
    tgt_src_dependencies = realloc(tgt_src_dependencies, total_num_dependencies *
                                   sizeof(tgt_src_dependencies));
-   yac_init_dep_list(tgt_to_src_cells);
-   yac_set_dependencies(tgt_to_src_cells, (unsigned)num_corners,
-                        num_src_per_tgt_corner, tgt_src_dependencies);
+   struct dep_list * tgt_to_src_cells =
+      yac_dep_list_new(
+         num_corners, num_src_per_tgt_corner, tgt_src_dependencies);
+   return tgt_to_src_cells;
 }
 
 static int compare_uint (const void * a, const void * b) {
   return ( *(unsigned*)a - *(unsigned*)b );
 }
 
-static void sphere_part_do_point_search_c2(struct grid_search * search,
-                                           double (*coordinates_xyz)[3],
-                                           size_t num_points,
-                                           struct dep_list * tgt_to_src_cells) {
+static struct dep_list * sphere_part_do_point_search_c2(struct grid_search * search,
+                                                        double (*coordinates_xyz)[3],
+                                                        size_t num_points) {
 
    struct sphere_part_search * sp_search = (struct sphere_part_search *)search;
    struct sphere_part_node * base_node = &(sp_search->base_node);
@@ -1885,7 +1883,7 @@ static void sphere_part_do_point_search_c2(struct grid_search * search,
 
    struct overlaps search_interval_tree_buffer = {0, 0, NULL};
 
-   unsigned * num_src_per_tgt_corner =
+   size_t * num_src_per_tgt_corner =
       calloc(num_points, sizeof(*num_src_per_tgt_corner));
    unsigned * tgt_src_dependencies = NULL;
    size_t tgt_src_dependencies_size = 0;
@@ -1933,54 +1931,53 @@ static void sphere_part_do_point_search_c2(struct grid_search * search,
 
    tgt_src_dependencies = realloc(tgt_src_dependencies, total_num_dependencies *
                                   sizeof(tgt_src_dependencies));
-   yac_init_dep_list(tgt_to_src_cells);
-   yac_set_dependencies(tgt_to_src_cells, (unsigned)num_points,
-                        num_src_per_tgt_corner, tgt_src_dependencies);
+   struct dep_list * tgt_to_src_cells =
+      yac_dep_list_new(
+         num_points, num_src_per_tgt_corner, tgt_src_dependencies);
+   return tgt_to_src_cells;
 }
 
-static void sphere_part_do_point_search_c3(struct grid_search * search,
-                                           double (*coordinates_xyz)[3],
-                                           size_t num_points,
-                                           struct dep_list * tgt_to_src_cells,
-                                           struct points * points) {
+static struct dep_list * sphere_part_do_point_search_c3(struct grid_search * search,
+                                                        double (*coordinates_xyz)[3],
+                                                        size_t num_points,
+                                                        struct points * points) {
 
-  yac_grid_search_utils_do_point_search_c3(
-    search, coordinates_xyz, num_points, tgt_to_src_cells, points);
+  return
+    yac_grid_search_utils_do_point_search_c3(
+       search, coordinates_xyz, num_points, points);
 }
 
-static void sphere_part_do_point_search_p(struct grid_search * search,
-                                          struct grid * grid_data,
-                                          struct dep_list * target_to_src_points) {
+static struct dep_list * sphere_part_do_point_search_p(struct grid_search * search,
+                                                       struct grid * grid_data) {
 
    struct sphere_part_search * sp_search = (struct sphere_part_search *)search;
 
-   yac_grid_search_utils_do_point_search_p(search, sp_search->grid_data, grid_data,
-                                           target_to_src_points);
+   return
+      yac_grid_search_utils_do_point_search_p(
+         search, sp_search->grid_data, grid_data);
 }
 
-static void sphere_part_do_point_search_p2(struct grid_search * search,
-                                           double (*coordinates_xyz)[3],
-                                           size_t num_points,
-                                           struct dep_list * target_to_src_points) {
+static struct dep_list * sphere_part_do_point_search_p2(struct grid_search * search,
+                                                        double (*coordinates_xyz)[3],
+                                                        size_t num_points) {
 
    struct sphere_part_search * sp_search = (struct sphere_part_search *)search;
 
-   yac_grid_search_utils_do_point_search_p2(
-      search, sp_search->grid_data, coordinates_xyz, num_points,
-      target_to_src_points);
+  return
+    yac_grid_search_utils_do_point_search_p2(
+      search, sp_search->grid_data, coordinates_xyz, num_points);
 }
 
-static void sphere_part_do_point_search_p3(struct grid_search * search,
-                                           double (*coordinates_xyz)[3],
-                                           size_t num_points,
-                                           struct dep_list * target_to_src_points,
-                                           struct points * points) {
+static struct dep_list * sphere_part_do_point_search_p3(struct grid_search * search,
+                                                        double (*coordinates_xyz)[3],
+                                                        size_t num_points,
+                                                        struct points * points) {
 
-   yac_grid_search_utils_do_point_search_p3(
-      search, coordinates_xyz, num_points, target_to_src_points, points);
+   return
+      yac_grid_search_utils_do_point_search_p3(
+         search, coordinates_xyz, num_points, points);
 }
 
-
 static void sphere_part_do_point_search_p4 (struct grid_search * search,
                                             double coordinate_xyz[3],
                                             size_t * n_points,
@@ -1994,10 +1991,9 @@ static void sphere_part_do_point_search_p4 (struct grid_search * search,
       points);
 }
 
-static void sphere_part_do_bnd_circle_search (struct grid_search * search,
-                                              struct bounding_circle * bnd_circles,
-                                              size_t num_bnd_circles,
-                                              struct dep_list * bnd_to_cells) {
+static struct dep_list * sphere_part_do_bnd_circle_search (struct grid_search * search,
+                                                           struct bounding_circle * bnd_circles,
+                                                           size_t num_bnd_circles) {
 
    struct sphere_part_search * sp_search = (struct sphere_part_search *)search;
    struct sphere_part_node * base_node = &(sp_search->base_node);
@@ -2012,7 +2008,7 @@ static void sphere_part_do_bnd_circle_search (struct grid_search * search,
 
    struct overlaps search_interval_tree_buffer = {0, 0, NULL};
 
-   unsigned * num_cells_per_bnd =
+   size_t * num_cells_per_bnd =
       calloc(num_bnd_circles, sizeof(*num_cells_per_bnd));
    unsigned * bnd_to_cells_dependencies = NULL;
    size_t bnd_to_cells_dependencies_size = 0;
@@ -2057,9 +2053,10 @@ static void sphere_part_do_bnd_circle_search (struct grid_search * search,
    bnd_to_cells_dependencies =
     realloc(bnd_to_cells_dependencies, total_num_dependencies *
             sizeof(bnd_to_cells_dependencies));
-   yac_init_dep_list(bnd_to_cells);
-   yac_set_dependencies(bnd_to_cells, (unsigned)num_bnd_circles,
-                        num_cells_per_bnd, bnd_to_cells_dependencies);
+   struct dep_list * bnd_to_cells =
+      yac_dep_list_new(
+         num_bnd_circles, num_cells_per_bnd, bnd_to_cells_dependencies);
+   return bnd_to_cells;
 }
 
 static void free_sphere_part_tree (struct sphere_part_node tree) {
@@ -2111,3 +2108,4 @@ void yac_delete_point_sphere_part_search(
    free(search->points);
    free(search);
 }
+
