@@ -194,11 +194,8 @@ sort_add(long num_links, double *restrict add1)
        long num_links; ! num of links for this mapping
        double *add1    ! destination address array [num_links]
   */
-
-  /* Local variables */
-
-  double add1_tmp;     /* temp for addresses during swap     */
-  long lvl, final_lvl; /* level indexes for heap sort levels */
+  double add1_tmp;      // temp for addresses during swap
+  long lvl, final_lvl;  // level indexes for heap sort levels
   long chk_lvl1, chk_lvl2, max_lvl;
   long i;
 
@@ -213,25 +210,18 @@ sort_add(long num_links, double *restrict add1)
       final_lvl = lvl;
       add1_tmp = add1[lvl];
 
-      /* Loop until proper level is found for this link, or reach bottom */
+      // Loop until proper level is found for this link, or reach bottom
 
-      for (i = 0; i < num_links; i++) /* while ( true ) */
+      for (i = 0; i < num_links; i++)  // while ( true )
         {
-          /* Find the largest of the two daughters */
-
+          // Find the largest of the two daughters
           chk_lvl1 = 2 * final_lvl + 1;
           chk_lvl2 = 2 * final_lvl + 2;
           if (chk_lvl1 == num_links - 1) chk_lvl2 = chk_lvl1;
 
-          if (add1[chk_lvl1] > add1[chk_lvl2])
-            max_lvl = chk_lvl1;
-          else
-            max_lvl = chk_lvl2;
+          max_lvl = (add1[chk_lvl1] > add1[chk_lvl2]) ? chk_lvl1 : chk_lvl2;
 
-          /*
-            If the parent is greater than both daughters,
-            the correct level has been found
-          */
+          // If the parent is greater than both daughters, the correct level has been found
           if (add1_tmp > add1[max_lvl])
             {
               add1[final_lvl] = add1_tmp;
@@ -262,27 +252,21 @@ sort_add(long num_links, double *restrict add1)
         }
     }
 
-  /*
-    Now that the heap has been sorted, strip off the top (largest)
-    value and promote the values below
-  */
+  // Now that the heap has been sorted, strip off the top (largest) value and promote the values below
 
   for (lvl = num_links - 1; lvl >= 2; lvl--)
     {
-      /* Move the top value and insert it into the correct place */
-
+      // Move the top value and insert it into the correct place
       add1_tmp = add1[lvl];
       add1[lvl] = add1[0];
 
-      /* As above this loop sifts the tmp values down until proper level is
-       * reached */
+      // As above this loop sifts the tmp values down until proper level is reached
 
       final_lvl = 0;
 
-      for (i = 0; i < num_links; i++) /* while ( true ) */
+      for (i = 0; i < num_links; i++)  // while ( true )
         {
-          /* Find the largest of the two daughters */
-
+          // Find the largest of the two daughters
           chk_lvl1 = 2 * final_lvl + 1;
           chk_lvl2 = 2 * final_lvl + 2;
           if (chk_lvl2 >= lvl) chk_lvl2 = chk_lvl1;
@@ -292,10 +276,7 @@ sort_add(long num_links, double *restrict add1)
           else
             max_lvl = chk_lvl2;
 
-          /*
-            If the parent is greater than both daughters,
-            the correct level has been found
-          */
+          // If the parent is greater than both daughters, the correct level has been found
           if (add1_tmp > add1[max_lvl])
             {
               add1[final_lvl] = add1_tmp;
@@ -304,10 +285,9 @@ sort_add(long num_links, double *restrict add1)
           else
             {
               /*
-                Otherwise, promote the largest daughter and push
-                down one level in the tree.  If haven't reached
-                the end of the tree, repeat the process.  Otherwise
-                store last values and exit the loop
+                Otherwise, promote the largest daughter and push down one level in the tree.
+                If haven't reached the end of the tree, repeat the process.
+                Otherwise store last values and exit the loop
               */
               add1[final_lvl] = add1[max_lvl];
 
@@ -320,16 +300,11 @@ sort_add(long num_links, double *restrict add1)
             }
         }
 
-      if (i == num_links)
-        {
-          cdoAbort("Internal problem; link 2 not found!");
-        }
+      if (i == num_links) cdoAbort("Internal problem; link 2 not found!");
     }
 
-  /* Swap the last two entries */
-  add1_tmp = add1[1];
-  add1[1] = add1[0];
-  add1[0] = add1_tmp;
+  // Swap the last two entries
+  std::swap(add1[0], add1[1]);
 
 } /* sort_add */
 
