@@ -15,16 +15,15 @@
   GNU General Public License for more details.
 */
 
-#include "error.h"
 #include "modules.h"
 #include "util_string.h"
+#include "cdo_output.h"
 
 #include <cstring>
-#include <algorithm> // for std::sort()
-
+#include <algorithm>  // for std::sort()
 
 /* removes '-' from operator string and returns copy of parameter operatorCommand */
-const char *
+static const char *
 getOperatorName(const char *operatorCommand)
 {
   char *operatorName = nullptr;
@@ -33,7 +32,7 @@ getOperatorName(const char *operatorCommand)
     {
       if (operatorCommand[0] == '-') operatorCommand++;
       const char *commapos = strchr(operatorCommand, ',');
-      size_t len = (commapos != nullptr) ? (size_t)(commapos - operatorCommand) : strlen(operatorCommand);
+      const size_t len = (commapos != nullptr) ? (size_t)(commapos - operatorCommand) : strlen(operatorCommand);
 
       operatorName = (char *) malloc(len + 1);
 
@@ -44,7 +43,6 @@ getOperatorName(const char *operatorCommand)
   return operatorName;
 }
 
-
 /**
  * @param a pointer to a string/substring
  * @param b pointer to a string/substring
@@ -96,16 +94,12 @@ similar(std::string original, std::string other)
  * @retval false if not
  */
 static bool
-operator_name_exists(std::string operatorName)
+operator_name_exists(const std::string &operatorName)
 {
-  if (modules_map.find(operatorName) != modules_map.end())
-    {
-      return true;
-    }
-  if (aliases.find(operatorName) != aliases.end())
-    {
-      return true;
-    }
+  if (modules_map.find(operatorName) != modules_map.end()) return true;
+
+  if (aliases.find(operatorName) != aliases.end()) return true;
+
   return false;
 }
 
@@ -124,7 +118,7 @@ module_map_contains(std::string moduleName)
     }
   else
     {
-      Error("Module %s not found", moduleName.c_str());
+      cdoAbort("Module %s not found", moduleName.c_str());
     }
   return false;
 }
@@ -147,7 +141,7 @@ find_similar(std::string operatorName)
       // Searching for simlar operator names in operator to module map
       for (const auto &str : modules_map)
         {
-          if (similar(string2lower(operatorName), str.first))
+          if (similar(stringToLower(operatorName), str.first))
             {
               if (found_similar_operators.size() + str.first.size() > lines * line_length)
                 {
@@ -161,7 +155,7 @@ find_similar(std::string operatorName)
       // Searching for similar operator names in aliases to original map
       for (const auto &str : aliases)
         {
-          if (similar(string2lower(operatorName), str.first))
+          if (similar(stringToLower(operatorName), str.first))
             {
               if (found_similar_operators.size() + str.first.size() > lines * line_length)
                 {
@@ -192,7 +186,7 @@ find_similar(std::string operatorName)
  *  \note If \p operatorName is a file name the program will exit.
  */
 static bool
-check_operator(std::string operatorName)
+check_operator(const std::string &operatorName)
 {
   if (operator_name_exists(operatorName))
     {
@@ -200,7 +194,7 @@ check_operator(std::string operatorName)
     }
   else if (operatorName == "")
     {
-      Error("Operator name missing!");
+      cdoAbort("Operator name missing!");
     }
   else
     {
@@ -209,8 +203,8 @@ check_operator(std::string operatorName)
       if (fp)
         {
           fclose(fp);
-          fprintf(stderr, "Use commandline option -h for help.");
-          Error("Operator missing, %s is a file on disk!", operatorName.c_str());
+          fprintf(stderr, "Use commandline option -h for help.\n");
+          cdoAbort("Operator missing, %s is a file on disk!", operatorName.c_str());
         }
       // Operator is no filename
       // Checking for similar operators
@@ -241,7 +235,7 @@ check_operator(std::string operatorName)
  * @note: if an error happens while adding the new module cdo will exit.
  */
 void
-add_module(std::string module_name, module_t new_module)
+add_module(const std::string &module_name, const module_t new_module)
 {
   if (modules.find(module_name) == modules.end())
     {
@@ -255,13 +249,13 @@ add_module(std::string module_name, module_t new_module)
             }
           else
             {
-              Error("Tried to add operator but the operator name already exists");
+              cdoAbort("Tried to add operator but the operator name already exists");
             }
         }
     }
   else
     {
-      Error("Module %s name already exists", module_name.c_str());
+      cdoAbort("Module %s name already exists", module_name.c_str());
     }
 }
 
@@ -271,26 +265,27 @@ add_module(std::string module_name, module_t new_module)
  * @param original original operator name
  */
 int
-add_alias(std::string alias, std::string original)
+add_alias(const std::string &alias, const std::string &original)
 {
   auto iter_original = modules_map.find(original);
   auto iter_alias = aliases.find(alias);
 
   if (iter_alias != aliases.end())
     {
-      Warning("alias %s could not be added: it already exists", alias.c_str());
+      printf("WARNING: alias %s could not be added: it already exists", alias.c_str());
       return -1;
     }
 
   if (iter_original == modules_map.end())
     {
-      Error("alias %s could not be added: operator %s does not exist", alias.c_str(), original.c_str());
+      printf("ERROR: alias %s could not be added: operator %s does not exist", alias.c_str(), original.c_str());
       return -2;
     }
 
   if (modules_map.find(alias) != modules_map.end())
     {
-      Error("alias %s could not be added: alias name already exists as an operator", alias.c_str());
+      printf("ERROR alias %s could not be added: alias name already exists as an operator", alias.c_str());
+      return -3;
     }
 
   aliases[alias] = original;
@@ -307,7 +302,7 @@ add_alias(std::string alias, std::string original)
  *      std::string -> name of the operators module
  */
 static std::string
-get_module_name_to(std::string operatorName)
+get_module_name_to(const std::string &operatorName)
 {
   // if not true the programm will exit in function check_operator
   if (check_operator(operatorName))
@@ -325,7 +320,7 @@ get_module_name_to(std::string operatorName)
     {
       // TODO: see if depricated since no operator can be added without a
       // module
-      Error("Module for >%s< not installed", operatorName.c_str());
+      cdoAbort("Module for >%s< not installed", operatorName.c_str());
     }
   // Only to quell the warning. Function should never reach this.
   return "";
@@ -338,11 +333,13 @@ get_module_name_to(std::string operatorName)
  * @param operatorName name of the operator
  * @returns the function of the #module_t of \a operatorName
  */
+/* not used
 void *(*operatorModule(const char *operatorName))(void *)
 {
   std::string operator_name = std::string(operatorName);
   return modules[get_module_name_to(operator_name)].func;
 }
+*/
 
 /***
  * returns help for given operator name
@@ -351,10 +348,9 @@ void *(*operatorModule(const char *operatorName))(void *)
  * @return vector of strings containing the help
  */
 std::vector<std::string>
-operatorHelp(std::string operatorName)
+operatorHelp(const std::string &operatorName)
 {
-  std::string operator_name = std::string(operatorName);
-  return modules[get_module_name_to(operator_name)].help;
+  return modules[get_module_name_to(operatorName)].help;
 }
 
 /***
@@ -363,13 +359,14 @@ operatorHelp(std::string operatorName)
  * @param operatorName operator name
  * @retval short
  */
+/* not used
 int
 operatorStreamInCnt(const char *operatorName)
 {
   std::string operator_name = std::string(operatorName);
   return modules[get_module_name_to(operator_name)].streamInCnt;
 }
-
+*/
 /***
  * Returns the number of output streams a given operator has.
  * returns -1 if obase is used
@@ -377,13 +374,14 @@ operatorStreamInCnt(const char *operatorName)
  * @return 1 for CDI_REAL, 2 for CDI_COMP (complex), 3 for CDI_REAL and CDI_COMP
  * @retval short
  */
+/* not used
 int
 operatorStreamOutCnt(const char *operatorName)
 {
   std::string operator_name = std::string(operatorName);
   return modules[get_module_name_to(operator_name)].streamOutCnt;
 }
-
+*/
 /***
  * Returns the number type this operator works with
  * @param operatorName operator name
@@ -551,19 +549,16 @@ close_library_handles()
 #endif
 
 // helper function for setting the spacing in operatorPrintList
-std::string
-get_spacing_for(int p_space, std::string str)
+static std::string
+get_spacing_for(int p_space, const std::string &str)
 {
   std::string spacing = "";
-  for (int i = str.size(); i <= p_space; i++)
-    {
-      spacing += " ";
-    }
+  for (int i = str.size(); i <= p_space; i++) spacing += " ";
   return spacing;
 }
 
-std::string
-get_operator_description(std::string p_current_op_name, std::vector<std::string> help)
+static std::string
+get_operator_description(const std::string &p_current_op_name, const std::vector<std::string> &help)
 {
   std::string description = "";
   unsigned long cur_help_idx;
@@ -674,7 +669,7 @@ operatorPrintList(bool print_no_output)
     }
 }
 
-bool
+static bool
 is_alias(const char *operatorName)
 {
   return (aliases.find(std::string(operatorName)) != aliases.end());
@@ -699,4 +694,3 @@ getModule(const std::string &operator_name)
 {
   return modules[get_module_name_to(operator_name)];
 }
-
