@@ -27,15 +27,30 @@
   */
 typedef void (*dyn_oper_t)(void *arg);
 
+enum ModuleRestrictions
+{
+  NoRestriction = 0,
+  FilesOnly = 1,
+  OnlyFirst = 2
+};
+
 struct module_t
 {
-  void *(*func)(void *);                // Module
-  std::vector<std::string> help;        // Help
-  std::vector<const char *> operators;  // Operator names
-  short mode;                           // Module mode: 0:intern 1:extern
-  short number;                         // Allowed number type
-  short streamInCnt;                    // Number of input streams
-  short streamOutCnt;                   // Number of output streams
+  void *(*func)(void *);               // Module
+  std::vector<std::string> help;       // Help
+  std::vector<std::string> operators;  // Operator names
+  short mode;                          // Module mode: 0:intern 1:extern
+  short number;                        // Allowed number type
+  short streamInCnt;                   // Number of input streams
+  short streamOutCnt;                  // Number of output streams
+  ModuleRestrictions restrictions = NoRestriction;
+  module_t(){};
+  module_t(void *(*p_func)(void *), std::vector<std::string> p_help, std::vector<std::string> p_opers, short p_m, short p_n,
+           short p_siC, short p_soC, ModuleRestrictions p_onlyFirst)
+      : func(p_func), help(p_help), operators(p_opers), mode(p_m), number(p_n), streamInCnt(p_siC), streamOutCnt(p_soC),
+        restrictions(p_onlyFirst)
+  {
+  }
 };
 
 /***
@@ -58,24 +73,22 @@ static std::map<std::string, module_t> modules;
  */
 static std::map<std::string, std::string> aliases;
 
-void *(*operatorModule(const char *operatorName))(void *);
+// void *(*operatorModule(const char *operatorName))(void *);
 
 module_t &getModule(const std::string &operatorName);
 
 void init_modules();
-void init_aliases();
 
-std::vector<std::string> operatorHelp(std::string operatorName);
-int operatorStreamInCnt(const char *operatorName);
-int operatorStreamOutCnt(const char *operatorName);
+std::vector<std::string> operatorHelp(const std::string &operatorName);
+// int operatorStreamInCnt(const char *operatorName);
+// int operatorStreamOutCnt(const char *operatorName);
 int operatorStreamNumber(const char *operatorName);
 
 void operatorPrintAll(void);
 void operatorPrintList(bool print_no_output);
-bool is_alias(const char *operatorName);
 const char *get_original(const char *operatorName);
-void add_module(std::string module_name, module_t new_module);
-int add_alias(std::string alias, std::string original);
+void add_module(const std::string &module_name, const module_t new_module);
+int add_alias(const std::string &alias, const std::string &original);
 
 #ifdef CUSTOM_MODULES
 void load_custom_module(std::string path);
