@@ -16,7 +16,7 @@
 */
 #include <cdi.h>
 
-#include "exception.h"
+#include "cdo_output.h"
 
 void
 nospec(int vlistID)
