@@ -14,10 +14,9 @@
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 */
+
 #include <stdio.h>
-#include <string.h>
 #include <assert.h>
-#include <stdlib.h>
 #include "nth_element.h"
 
 #define SWAP(a, b) \
@@ -33,7 +32,7 @@ nth_element(double *array, int length, int n)
   int i, j, k = 0, l = length - 1, m;
   double a, t;
 
-  assert(array != NULL);
+  assert(array != nullptr);
   assert(l >= 0);
   assert(n >= 0);
   assert(n <= l);
