@@ -24,12 +24,11 @@
 
 #include "cdo_int.h"
 #include "par_io.h"
-#include "pstream_int.h"
 
 void *
 readRecord(void *arg)
 {
-  int streamID;
+  CdoStreamID streamID;
   int *varID, *levelID;
   size_t *nmiss;
   double *array;
@@ -41,17 +40,17 @@ readRecord(void *arg)
   nmiss = read_arg->nmiss;
   array = read_arg->array;
 
-  /* fprintf(stderr, "pstreamInqRecord: streamID = %d\n", streamID); */
-  pstreamInqRecord(streamID, varID, levelID);
-  pstreamReadRecord(streamID, array, nmiss);
-  /* fprintf(stderr, "pstreamReadRecord: varID %d levelID %d\n", *varID,
+  /* fprintf(stderr, "cdoInqRecord: streamID = %d\n", streamID); */
+  cdoInqRecord(streamID, varID, levelID);
+  cdoReadRecord(streamID, array, nmiss);
+  /* fprintf(stderr, "cdoReadRecord: varID %d levelID %d\n", *varID,
    * *levelID); */
 
   return (nullptr);
 }
 
 void
-parReadRecord(int streamID, int *varID, int *levelID, double *array, size_t *nmiss, par_io_t *parIO)
+parReadRecord(CdoStreamID streamID, int *varID, int *levelID, double *array, size_t *nmiss, par_io_t *parIO)
 {
   bool lpario = false;
   int recID = 0, nrecs = 0;
@@ -59,9 +58,7 @@ parReadRecord(int streamID, int *varID, int *levelID, double *array, size_t *nmi
   pthread_t thrID = 0;
   /* pthread_attr_t attr; */
   int rval;
-#endif
 
-#ifdef HAVE_LIBPTHREAD
   if (parIO)
     {
       lpario = true;
