@@ -5,14 +5,13 @@
 #include <cstddef>
 #include <ctype.h>
 
-#include "exception.h"
+#include "cdo_output.h"
 #include "compare.h"
 
-
 const char *
 parameter2word(const char *string)
 {
-  size_t len = strlen(string);
+  const size_t len = strlen(string);
 
   for (size_t i = 0; i < len; ++i)
     {
@@ -28,16 +27,16 @@ parameter2word(const char *string)
 bool
 parameter2bool(const char *str)
 {
-  size_t len = strlen(str);
+  const size_t len = strlen(str);
 
   if (len == 1)
     {
       if (*str == 't' || *str == 'T' || *str == '1') return true;
       if (*str == 'f' || *str == 'F' || *str == '0') return false;
     }
-  else if (len == 4 && (cstrIsEqual(str, "true") || cstrIsEqual(str, "true") || cstrIsEqual(str, "True")))
+  else if (len == 4 && (cstrIsEqual(str, "true") || cstrIsEqual(str, "TRUE") || cstrIsEqual(str, "True")))
     return true;
-  else if (len == 5 && (cstrIsEqual(str, "false") || cstrIsEqual(str, "false") || cstrIsEqual(str, "False")))
+  else if (len == 5 && (cstrIsEqual(str, "false") || cstrIsEqual(str, "FALSE") || cstrIsEqual(str, "False")))
     return false;
 
   cdoAbort("Boolean parameter >%s< contains invalid characters!", str);
@@ -45,14 +44,18 @@ parameter2bool(const char *str)
   return false;
 }
 
+static inline void
+parameterError(const char *name, const char *string, const char *endptr)
+{
+  cdoAbort("%s parameter >%s< contains invalid character at position %d!", name, string, (int) (endptr - string + 1));
+}
+
 double
 parameter2double(const char *string)
 {
   char *endptr = nullptr;
-  double fval = strtod(string, &endptr);
-  if (*endptr != 0)
-    cdoAbort("Float parameter >%s< contains invalid character at position %d!", string, (int) (endptr - string + 1));
-
+  const double fval = strtod(string, &endptr);
+  if (*endptr != 0) parameterError("Float", string, endptr);
   return fval;
 }
 
@@ -60,10 +63,8 @@ int
 parameter2int(const char *string)
 {
   char *endptr = nullptr;
-  int ival = (int) strtol(string, &endptr, 10);
-  if (*endptr != 0)
-    cdoAbort("Integer parameter >%s< contains invalid character at position %d!", string, (int) (endptr - string + 1));
-
+  const int ival = (int) strtol(string, &endptr, 10);
+  if (*endptr != 0) parameterError("Integer", string, endptr);
   return ival;
 }
 
@@ -71,10 +72,8 @@ long
 parameter2long(const char *string)
 {
   char *endptr = nullptr;
-  long ival = strtol(string, &endptr, 10);
-  if (*endptr != 0)
-    cdoAbort("Integer parameter >%s< contains invalid character at position %d!", string, (int) (endptr - string + 1));
-
+  const long ival = strtol(string, &endptr, 10);
+  if (*endptr != 0) parameterError("Integer", string, endptr);
   return ival;
 }
 
@@ -82,10 +81,8 @@ size_t
 parameter2sizet(const char *string)
 {
   char *endptr = nullptr;
-  size_t ival = (size_t) strtoimax(string, &endptr, 10);
-  if (*endptr != 0)
-    cdoAbort("Integer parameter >%s< contains invalid character at position %d!", string, (int) (endptr - string + 1));
-
+  const size_t ival = (size_t) strtoimax(string, &endptr, 10);
+  if (*endptr != 0) parameterError("Integer", string, endptr);
   return ival;
 }
 
@@ -93,11 +90,7 @@ int
 parameter2intlist(const char *string)
 {
   char *endptr = nullptr;
-  int ival = (int) strtol(string, &endptr, 10);
-  if (*endptr != 0 && *endptr != '/' && (endptr - string) == 0)
-    cdoAbort("Integer parameter >%s< contains invalid character at position %d!", string, (int) (endptr - string + 1));
-
+  const int ival = (int) strtol(string, &endptr, 10);
+  if (*endptr != 0 && *endptr != '/' && (endptr - string) == 0) parameterError("Integer", string, endptr);
   return ival;
 }
-
-
