@@ -17,11 +17,11 @@
 
 #include <assert.h>
 #include <stdlib.h>
-#include <math.h>
 
 #include <cdi.h>
 
 #include "cdo_int.h"
+#include "cdo_math.h"
 #include "percentiles.h"
 #include "percentiles_hist.h"
 
@@ -37,57 +37,48 @@ histGetEnvNBins()
 {
   const char *str = getenv("CDO_PCTL_NBINS");
 
-  return str != NULL ? MAX(atoi(str), NBINS_MINIMUM) : NBINS_DEFAULT;
+  return str != nullptr ? cdo::max(atoi(str), NBINS_MINIMUM) : NBINS_DEFAULT;
 }
 
 static void
 histDefBounds(HISTOGRAM *hist, double a, double b)
 {
-  int i;
-
-  assert(hist != NULL);
+  assert(hist != nullptr);
   assert(hist->nbins > 0);
 
-  hist->min = MIN(a, b);
-  hist->max = MAX(a, b);
+  hist->min = cdo::min(a, b);
+  hist->max = cdo::max(a, b);
   hist->step = (hist->max - hist->min) / hist->nbins;
   hist->nsamp = 0;
 
-  for (i = 0; i < hist->nbins; i++) INT_PTR(hist->ptr)[i] = 0;
+  for (int i = 0; i < hist->nbins; i++) INT_PTR(hist->ptr)[i] = 0;
 }
 
 static void
 histBinValue(HISTOGRAM *hist, double value)
 {
-  int bin;
-
   assert(hist->step > 0);
 
-  bin = MIN((int) ((value - hist->min) / hist->step), hist->nbins - 1);
+  const int bin = cdo::min((int) ((value - hist->min) / hist->step), hist->nbins - 1);
   if (bin >= 0 && bin < hist->nbins) INT_PTR(hist->ptr)[bin]++;
 }
 
 static void
 histBin(HISTOGRAM *hist)
 {
-  int i;
-  double *values;
-
   assert(hist->nsamp == DBL_CAPACITY(hist->nbins));
 
-  values = (double *) Malloc(hist->nsamp * sizeof(double));
+  std::vector<double> values(hist->nsamp);
 
-  for (i = 0; i < hist->nsamp; i++) values[i] = DBL_PTR(hist->ptr)[i];
-  for (i = 0; i < hist->nbins; i++) INT_PTR(hist->ptr)[i] = 0;
-  for (i = 0; i < hist->nsamp; i++) histBinValue(hist, values[i]);
-
-  Free(values);
+  for (int i = 0; i < hist->nsamp; i++) values[i] = DBL_PTR(hist->ptr)[i];
+  for (int i = 0; i < hist->nbins; i++) INT_PTR(hist->ptr)[i] = 0;
+  for (int i = 0; i < hist->nsamp; i++) histBinValue(hist, values[i]);
 }
 
 static int
 histAddValue(HISTOGRAM *hist, double value)
 {
-  assert(hist != NULL);
+  assert(hist != nullptr);
   assert(hist->nbins > 0);
 
   /* 2011-08-01 Uwe Schulzweida: added check for rounding errors */
@@ -122,7 +113,7 @@ histGetPercentile(const HISTOGRAM *hist, double p)
 {
   int i = 0, count = 0;
 
-  assert(hist != NULL);
+  assert(hist != nullptr);
   assert(hist->nsamp > 0);
   assert(hist->nbins > 0);
   assert(p >= 0);
@@ -153,25 +144,22 @@ histGetPercentile(const HISTOGRAM *hist, double p)
 HISTOGRAM_SET *
 hsetCreate(int nvars)
 {
-  int varID;
-  HISTOGRAM_SET *hset;
-
   assert(nvars > 0);
 
-  hset = (HISTOGRAM_SET *) Malloc(sizeof(HISTOGRAM_SET));
-  if (hset == NULL) cdoAbort("Not enough memory (%s)", __func__);
+  HISTOGRAM_SET *hset = (HISTOGRAM_SET *) Malloc(sizeof(HISTOGRAM_SET));
+  if (hset == nullptr) cdoAbort("Not enough memory (%s)", __func__);
 
   hset->nvars = nvars;
   hset->nlevels = (int *) Malloc(nvars * sizeof(int));
   hset->grids = (int *) Malloc(nvars * sizeof(int));
   hset->histograms = (HISTOGRAM ***) Malloc(nvars * sizeof(HISTOGRAM **));
-  if (hset->histograms == NULL) cdoAbort("Not enough memory (%s)", __func__);
+  if (hset->histograms == nullptr) cdoAbort("Not enough memory (%s)", __func__);
 
-  for (varID = 0; varID < nvars; varID++)
+  for (int varID = 0; varID < nvars; varID++)
     {
       hset->nlevels[varID] = 0;
       hset->grids[varID] = 0;
-      hset->histograms[varID] = NULL;
+      hset->histograms[varID] = nullptr;
     }
 
   return hset;
@@ -180,22 +168,19 @@ hsetCreate(int nvars)
 void
 hsetCreateVarLevels(HISTOGRAM_SET *hset, int varID, int nlevels, int grid)
 {
-  int nvars, nhists, nbins, levelID, histID;
-  HISTOGRAM *hists;
+  int nbins = histGetEnvNBins();
 
-  nbins = histGetEnvNBins();
-
-  assert(hset != NULL);
+  assert(hset != nullptr);
   assert(nlevels > 0);
   assert(nbins > 0);
 
-  nvars = hset->nvars;
+  int nvars = hset->nvars;
 
   assert(nvars > 0);
 
   if (varID < 0 || varID >= nvars) cdoAbort("Illegal argument: varID %d is undefined (%s)", varID, __func__);
 
-  nhists = gridInqSize(grid);
+  int nhists = gridInqSize(grid);
 
   assert(nhists > 0);
 
@@ -203,14 +188,14 @@ hsetCreateVarLevels(HISTOGRAM_SET *hset, int varID, int nlevels, int grid)
   hset->grids[varID] = grid;
 
   hset->histograms[varID] = (HISTOGRAM **) Malloc(nlevels * sizeof(HISTOGRAM *));
-  if (hset->histograms[varID] == NULL) cdoAbort("Not enough memory (%s)", __func__);
+  if (hset->histograms[varID] == nullptr) cdoAbort("Not enough memory (%s)", __func__);
 
-  for (levelID = 0; levelID < nlevels; levelID++)
+  for (int levelID = 0; levelID < nlevels; levelID++)
     {
-      hists = hset->histograms[varID][levelID] = (HISTOGRAM *) Malloc(nhists * sizeof(HISTOGRAM));
-      if (hists == NULL) cdoAbort("Not enough memory (%s)", __func__);
+      HISTOGRAM *hists = hset->histograms[varID][levelID] = (HISTOGRAM *) Malloc(nhists * sizeof(HISTOGRAM));
+      if (hists == nullptr) cdoAbort("Not enough memory (%s)", __func__);
 
-      for (histID = 0; histID < nhists; histID++)
+      for (int histID = 0; histID < nhists; histID++)
         {
           hists[histID].min = 0.0;
           hists[histID].max = 0.0;
@@ -219,7 +204,7 @@ hsetCreateVarLevels(HISTOGRAM_SET *hset, int varID, int nlevels, int grid)
           hists[histID].nsamp = 0;
 
           hists[histID].ptr = (int *) Malloc(nbins * sizeof(int));
-          if (hists[histID].ptr == NULL) cdoAbort("Not enough memory (%s)", __func__);
+          if (hists[histID].ptr == nullptr) cdoAbort("Not enough memory (%s)", __func__);
         }
     }
 }
@@ -227,16 +212,14 @@ hsetCreateVarLevels(HISTOGRAM_SET *hset, int varID, int nlevels, int grid)
 void
 hsetDestroy(HISTOGRAM_SET *hset)
 {
-  int varID, levelID, histID, nhists;
-
-  if (hset != NULL)
+  if (hset != nullptr)
     {
-      for (varID = hset->nvars; varID-- > 0;)
+      for (int varID = hset->nvars; varID-- > 0;)
         {
-          nhists = gridInqSize(hset->grids[varID]);
-          for (levelID = hset->nlevels[varID]; levelID-- > 0;)
+          int nhists = gridInqSize(hset->grids[varID]);
+          for (int levelID = hset->nlevels[varID]; levelID-- > 0;)
             {
-              for (histID = nhists; histID-- > 0;) Free(hset->histograms[varID][levelID][histID].ptr);
+              for (int histID = nhists; histID-- > 0;) Free(hset->histograms[varID][levelID][histID].ptr);
               Free(hset->histograms[varID][levelID]);
             }
           Free(hset->histograms[varID]);
@@ -247,54 +230,45 @@ hsetDestroy(HISTOGRAM_SET *hset)
       Free(hset->nlevels);
       Free(hset);
 
-      hset = NULL;
+      hset = nullptr;
     }
 }
 
 void
 hsetDefVarLevelBounds(HISTOGRAM_SET *hset, int varID, int levelID, const Field *field1, const Field *field2)
 {
-  const double *array1 = field1->ptr;
-  const double *array2 = field2->ptr;
+  const double *array1 = field1->vec.data();
+  const double *array2 = field2->vec.data();
 
-  int i, nvars, nlevels, nhists, grid;
-  double a, b;
-  HISTOGRAM *hists;
-
-  assert(hset != NULL);
-  assert(field1 != NULL);
-  assert(field2 != NULL);
-  assert(array1 != NULL);
-  assert(array2 != NULL);
-
-  nvars = hset->nvars;
+  assert(hset != nullptr);
+  assert(field1 != nullptr);
+  assert(field2 != nullptr);
+  assert(array1 != nullptr);
+  assert(array2 != nullptr);
 
+  int nvars = hset->nvars;
   assert(nvars > 0);
 
   if (varID < 0 || varID >= nvars) cdoAbort("Illegal argument: varID %d is undefined (%s)", varID, __func__);
 
-  nlevels = hset->nlevels[varID];
-
+  int nlevels = hset->nlevels[varID];
   assert(nlevels > 0);
 
   if (levelID < 0 || levelID >= nlevels) cdoAbort("Illegal argument: levelID %d is undefined (%s)", levelID, __func__);
 
-  grid = hset->grids[varID];
-
+  int grid = hset->grids[varID];
   if (grid != field1->grid || grid != field2->grid) cdoAbort("Grids are different (%s)", __func__);
 
-  hists = hset->histograms[varID][levelID];
-
-  assert(hists != NULL);
-
-  nhists = gridInqSize(grid);
+  HISTOGRAM *hists = hset->histograms[varID][levelID];
+  assert(hists != nullptr);
 
+  int nhists = gridInqSize(grid);
   assert(nhists > 0);
 
-  for (i = 0; i < nhists; i++)
+  for (int i = 0; i < nhists; i++)
     {
-      a = array1[i];
-      b = array2[i];
+      double a = array1[i];
+      double b = array2[i];
 
       if (DBL_IS_EQUAL(a, field1->missval) || DBL_IS_EQUAL(b, field2->missval) || DBL_IS_EQUAL(a, b))
         histDefBounds(&hists[i], 0.0, 0.0);
@@ -306,45 +280,41 @@ hsetDefVarLevelBounds(HISTOGRAM_SET *hset, int varID, int levelID, const Field *
 void
 hsetAddVarLevelValues(HISTOGRAM_SET *hset, int varID, int levelID, const Field *field)
 {
-  const double *array = field->ptr;
-  int i, grid, nvars, nlevels, nhists, nign = 0;
-  HISTOGRAM *hists;
+  const double *array = field->vec.data();
 
-  assert(hset != NULL);
-  assert(field != NULL);
-  assert(array != NULL);
-
-  nvars = hset->nvars;
+  assert(hset != nullptr);
+  assert(field != nullptr);
+  assert(array != nullptr);
 
+  int nvars = hset->nvars;
   assert(nvars > 0);
 
   if (varID < 0 || varID >= nvars) cdoAbort("Illegal argument: varID %d is undefined (%s)", varID, __func__);
 
-  nlevels = hset->nlevels[varID];
-
+  int nlevels = hset->nlevels[varID];
   assert(nlevels > 0);
 
   if (levelID < 0 || levelID >= nlevels) cdoAbort("Illegal argument: levelID %d is undefined (%s)", levelID, __func__);
 
-  grid = hset->grids[varID];
+  int grid = hset->grids[varID];
   if (grid != field->grid) cdoAbort("Grids are different (%s)", __func__);
 
-  hists = hset->histograms[varID][levelID];
-
-  assert(hists != NULL);
-
-  nhists = gridInqSize(grid);
+  HISTOGRAM *hists = hset->histograms[varID][levelID];
+  assert(hists != nullptr);
 
+  int nhists = gridInqSize(grid);
   assert(nhists > 0);
 
+  int nign = 0;
+
   if (field->nmiss)
     {
-      for (i = 0; i < nhists; i++)
+      for (int i = 0; i < nhists; i++)
         if (!DBL_IS_EQUAL(array[i], field->missval)) nign += histAddValue(&hists[i], array[i]);
     }
   else
     {
-      for (i = 0; i < nhists; i++) nign += histAddValue(&hists[i], array[i]);
+      for (int i = 0; i < nhists; i++) nign += histAddValue(&hists[i], array[i]);
     }
 
   if (nign) cdoWarning("%d out of %d grid values are out of bounds and have been ignored (%s)", nign, nhists, __func__);
@@ -353,40 +323,33 @@ hsetAddVarLevelValues(HISTOGRAM_SET *hset, int varID, int levelID, const Field *
 void
 hsetGetVarLevelPercentiles(Field *field, const HISTOGRAM_SET *hset, int varID, int levelID, double p)
 {
-  double *array = field->ptr;
-  int i, nvars, nlevels, nhists, grid;
-  HISTOGRAM *hists;
+  double *array = field->vec.data();
 
-  assert(hset != NULL);
-  assert(field != NULL);
-  assert(array != NULL);
-
-  nvars = hset->nvars;
+  assert(hset != nullptr);
+  assert(field != nullptr);
+  assert(array != nullptr);
 
+  int nvars = hset->nvars;
   assert(nvars > 0);
 
   if (varID < 0 || varID >= nvars) cdoAbort("Illegal argument: varID %d is undefined (%s)", varID, __func__);
 
-  nlevels = hset->nlevels[varID];
-
+  int nlevels = hset->nlevels[varID];
   assert(nlevels > 0);
 
   if (levelID < 0 || levelID >= nlevels) cdoAbort("Illegal argument: levelID %d is undefined (%s)", levelID, __func__);
 
-  grid = hset->grids[varID];
-
+  int grid = hset->grids[varID];
   if (grid != field->grid) cdoAbort("Grids are different (%s)", __func__);
 
-  hists = hset->histograms[varID][levelID];
-
-  assert(hists != NULL);
-
-  nhists = gridInqSize(grid);
+  HISTOGRAM *hists = hset->histograms[varID][levelID];
+  assert(hists != nullptr);
 
+  int nhists = gridInqSize(grid);
   assert(nhists > 0);
 
   field->nmiss = 0;
-  for (i = 0; i < nhists; i++)
+  for (int i = 0; i < nhists; i++)
     {
       if (hists[i].nsamp)
         {
