@@ -17,14 +17,13 @@
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
-#include "cdo_debugOutput.h"
 #endif
 
 #include <stdio.h>
 #include <cdi.h>
 
-#include "error.h"
 #include "pipe.h"
+#include "cdo_output.h"
 
 #ifdef HAVE_LIBPTHREAD
 
@@ -35,13 +34,12 @@ pipe_t::close()
 {
   pthread_mutex_lock(m_mutex);
   EOP = true;
-  if (CdoDebug::PSTREAM)
-    {
-      MESSAGE(name, " write closed");
-    }
+  Debug(PIPE, "%s write closed",name);
   pthread_mutex_unlock(m_mutex);
   pthread_cond_signal(tsDef);
   pthread_cond_signal(tsInq);
+  pthread_cond_signal(recInq); //
+  pthread_cond_signal(isclosed);//
 }
 
 void
@@ -55,7 +53,7 @@ pipe_t::pipe_init()
     */
   /*
 #if defined(_POSIX_THREAD_PROCESS_SHARED)
-  if ( CdoDebug::PIPE )
+  if ( PIPE )
     {
       MESSAGE("setpshared mutexattr to PTHREAD_PROCESS_SHARED");
       MESSAGE("setpshared condattr to PTHREAD_PROCESS_SHARED");
@@ -64,7 +62,7 @@ pipe_t::pipe_init()
   pthread_mutexattr_setpshared(&m_attr, PTHREAD_PROCESS_SHARED);
   pthread_condattr_setpshared(&c_attr, PTHREAD_PROCESS_SHARED);
 
-  if ( CdoDebug::PIPE )
+  if ( PIPE )
     {
       int pshared;
       pthread_mutexattr_getpshared(&m_attr, &pshared);
@@ -80,7 +78,7 @@ pipe_t::pipe_init()
         MESSAGE("getpshared condattr is PTHREAD_PROCESS_PRIVATE");
     }
 #else
-  if ( CdoDebug::PIPE )
+  if ( PIPE )
     MeSSAGE("_POSIX_THREAD_PROCESS_SHARED undefined");
 #endif
   */
@@ -91,7 +89,6 @@ pipe_t::pipe_init()
   tsIDr = -1;
   tsIDw = -1;
 
-  nvals = 0;
   nmiss = 0;
   data = nullptr;
   hasdata = false;
@@ -136,7 +133,7 @@ pipe_t::pipeInqTimestep(int p_tsID)
   if (p_tsID != tsIDr + 1)
     {
       if (!(p_tsID == tsIDr && tsIDr == tsIDw && recIDr == -1))
-        Error("%s unexpected tsID %d %d %d", name.c_str(), p_tsID, tsIDr + 1, tsIDw);
+        cdoAbort("%s unexpected tsID %d %d %d", name.c_str(), p_tsID, tsIDr + 1, tsIDw);
     }
 
   tsIDr = p_tsID;
@@ -144,22 +141,24 @@ pipe_t::pipeInqTimestep(int p_tsID)
     {
       if (EOP)
         {
-          if (CdoDebug::PIPE) MESSAGE(name.c_str(), " EOP");
+          Debug(PIPE, name.c_str(), " EOP");
           break;
         }
       if (hasdata)
         {
-          if (CdoDebug::PIPE) MESSAGE(name.c_str(), " has data");
+          Debug(PIPE, name.c_str(), " has data");
           hasdata = false;
           data = nullptr;
           readCond.notify_all();
         }
-      else if (CdoDebug::PIPE)
-        MESSAGE(name.c_str(), " has no data");
+      else
+        {
+          Debug(PIPE, "%s has no data", name);
+        }
 
       recInq.notify_all(); /* o.k. ??? */
 
-      if (CdoDebug::PIPE) MESSAGE(name.c_str(), " wait of tsDef");
+      Debug(PIPE, name.c_str(), " wait of tsDef");
       tsDef.wait(locked_mutex);
     }
 
@@ -193,6 +192,27 @@ constexpr std::chrono::milliseconds TIMEOUT = std::chrono::milliseconds(1000);
 #define MAX_WAIT_CYCLES 3600
 int processNumsActive(void);
 
+int
+pipe_t::pipeInqVlist(int &p_vlistID)
+{
+  std::chrono::milliseconds time_to_wait(0);
+  int nwaitcycles = 0;
+
+  // LOCK
+  std::unique_lock<std::mutex> locked_mutex(m_mutex);
+  while (p_vlistID == -1 && nwaitcycles < MAX_WAIT_CYCLES)
+    {
+      time_to_wait += TIMEOUT;
+      Debug(PIPE, name.c_str(), " wait of vlistDef");
+      vlistDef.wait_for(locked_mutex, time_to_wait);
+      nwaitcycles++;
+    }
+
+  // UNLOCK
+
+  return p_vlistID;
+}
+/* OLD VERSION FOR SAFETY and comparison (15.5.2019)
 int
 pipe_t::pipeInqVlist(int &p_vlistID)
 {
@@ -207,7 +227,7 @@ pipe_t::pipeInqVlist(int &p_vlistID)
     {
       time_to_wait += TIMEOUT;
       // fprintf(stderr, "tvsec %g\n", (double) time_to_wait.tv_sec);
-      if (CdoDebug::PIPE) MESSAGE(name.c_str(), " wait of vlistDef");
+      Debug(PIPE, name.c_str(), " wait of vlistDef");
       // pthread_cond_wait(pipe->vlistDef, pipe->mutex);
       retcode = vlistDef.wait_for(locked_mutex, time_to_wait);
       // fprintf(stderr, "self %d retcode %d %d %d\n", pstreamptr->self,
@@ -221,14 +241,19 @@ pipe_t::pipeInqVlist(int &p_vlistID)
     }
 
   if (retcode == std::cv_status::timeout)
-    vlistID = p_vlistID;
-  else if (CdoDebug::PIPE)
-    MESSAGE(name.c_str(), " timeout!");
+    {
+      vlistID = p_vlistID;
+    }
+  else
+    {
+      Debug(PIPE, "%s timeout!", name);
+    }
 
   // UNLOCK
 
   return vlistID;
 }
+*/
 
 void
 pipe_t::pipeDefTimestep(int p_vlistID, int p_tsID)
@@ -239,7 +264,7 @@ pipe_t::pipeDefTimestep(int p_vlistID, int p_tsID)
   m_mutex.lock();
   recIDw = -1;
   tsIDw++;
-  if (p_tsID != tsIDw) Error("unexpected p_tsID %d(%d) for %s", p_tsID, tsIDw, name.c_str());
+  if (p_tsID != tsIDw) cdoAbort("unexpected p_tsID %d(%d) for %s", p_tsID, tsIDw, name.c_str());
 
   if (p_tsID == 0)
     numrecs = vlistNrecs(p_vlistID);
@@ -254,11 +279,11 @@ pipe_t::pipeDefTimestep(int p_vlistID, int p_tsID)
               numrecs += zaxisInqSize(vlistInqVarZaxis(vlistID, varID));
             }
         }
-      if (CdoDebug::PIPE) MESSAGE(name.c_str(), "numrecs = ", numrecs, "nvars = ", vlistNvars(vlistID));
+      Debug(PIPE, name.c_str(), "numrecs = ", numrecs, "nvars = ", vlistNvars(vlistID));
     }
 
   nrecs = numrecs;
-  if (CdoDebug::PIPE) MESSAGE(name.c_str(), " numrecs ", numrecs, " p_tsID ", p_tsID, " ", tsIDw, " ", tsIDr);
+  Debug(PIPE, name.c_str(), " numrecs ", numrecs, " p_tsID ", p_tsID, " ", tsIDw, " ", tsIDr);
   if (numrecs == 0) EOP = true;
   m_mutex.unlock();
   // UNLOCK
@@ -272,10 +297,10 @@ pipe_t::pipeDefTimestep(int p_vlistID, int p_tsID)
     {
       if (EOP)
         {
-          if (CdoDebug::PIPE) MESSAGE("EOP");
+          Debug(PIPE, "EOP");
           break;
         }
-      if (CdoDebug::PIPE) MESSAGE(name.c_str(), " wait of tsInq (p_tsID ", p_tsID, " ", tsIDr, ")");
+      Debug(PIPE, name.c_str(), " wait of tsInq (p_tsID ", p_tsID, " ", tsIDr, ")");
       tsInq.wait(locked_mutex);
     }
   // UNLOCK
@@ -286,11 +311,10 @@ pipe_t::pipeInqRecord(int *p_varID, int *p_levelID)
 {
   bool condSignal = false;
 
-  // if (CdoDebug::PIPE)
 
   // LOCK
   m_mutex.lock();
-  if (CdoDebug::PIPE) MESSAGE(name.c_str(), " has no data ", recIDr, " ", recIDw);
+  Debug(PIPE, name.c_str(), " has no data ", recIDr, " ", recIDw);
   if (hasdata || usedata)
     {
       hasdata = false;
@@ -308,16 +332,16 @@ pipe_t::pipeInqRecord(int *p_varID, int *p_levelID)
   usedata = true;
   recIDr++;
 
-  if (CdoDebug::PIPE) MESSAGE(name.c_str(), "recID", recIDr, " ", recIDw);
+  Debug(PIPE, name.c_str(), "recID", recIDr, " ", recIDw);
 
   while (recIDw != recIDr)
     {
       if (EOP)
         {
-          if (CdoDebug::PIPE) MESSAGE("EOP");
+          Debug(PIPE, "EOP");
           break;
         }
-      if (CdoDebug::PIPE) MESSAGE(name.c_str(), "wait for recDef");
+      Debug(PIPE,"%s wait for recDef",name);
       recDef.wait(locked_mutex);
     }
 
@@ -347,7 +371,7 @@ pipe_t::pipeDefRecord(int p_varID, int p_levelID)
 
   // LOCK
   m_mutex.lock();
-  if (CdoDebug::PIPE) MESSAGE(name.c_str(), " has data ", recIDr, " ", recIDw);  //<- TODO: rethink positioning
+  Debug(PIPE, name.c_str(), " has data ", recIDr, " ", recIDw);  //<- TODO: rethink positioning
   if (hasdata)
     {
       hasdata = false;
@@ -365,7 +389,7 @@ pipe_t::pipeDefRecord(int p_varID, int p_levelID)
   recIDw++;
   varID = p_varID;
   levelID = p_levelID;
-  if (CdoDebug::PIPE) MESSAGE(name.c_str(), "recID", recIDr, " ", recIDw);
+  Debug(PIPE, name.c_str(), "recID", recIDr, " ", recIDw);
   m_mutex.unlock();
   // UNLOCK
 
@@ -377,7 +401,7 @@ pipe_t::pipeDefRecord(int p_varID, int p_levelID)
     {
       if (tsIDw != tsIDr) break;
       if (EOP) break;
-      if (CdoDebug::PIPE) MESSAGE(name.c_str(), "wait of recInq ", recIDr);
+      Debug(PIPE, name.c_str(), "wait of recInq ", recIDr);
       recInq.wait(locked_mutex);
     }
   // UNLOCK
@@ -389,16 +413,16 @@ pipe_t::pipeDefRecord(int p_varID, int p_levelID)
  * @param data destination for the record data
  * @param pipe pipe that has the wanted data
  */
-void
+size_t
 pipe_t::pipeReadPipeRecord(double *p_data, int vlistID, size_t *p_nmiss)
 {
-  if (!p_data) Error("No data pointer for %s", name.c_str());
+  if (!p_data) cdoAbort("No data pointer for %s", name.c_str());
 
   size_t datasize = gridInqSize(vlistInqVarGrid(vlistID, varID));
-  nvals += datasize;
   if (vlistNumber(vlistID) != CDI_REAL) datasize *= 2;
   memcpy(p_data, data, datasize * sizeof(double));
   *p_nmiss = nmiss;
+  return datasize;
 }
 
 /*
@@ -410,7 +434,7 @@ pipeGetReadTarget(Pstream *pstreamptr, Pstream *pstreamptr_in)
   pstreamptr = pstreamptr_in;
   while (pstreamptr_in->ispipe)
     {
-      if (CdoDebug::PIPE)
+      if (PIPE)
         fprintf(stderr, "%s: istream %d is pipe\n", __func__,
 pstreamptr_in->self); pstreamptr = pstreamptr_in; pstreamptr_in =
 pstreamptr_in->pipe->pstreamptr_in; if (pstreamptr_in == 0) break;
@@ -418,37 +442,38 @@ pstreamptr_in->pipe->pstreamptr_in; if (pstreamptr_in == 0) break;
 
   if (pstreamptr->pipe->hasdata == false)
     {
-      Error("Internal problem! istream undefined");
+      cdoAbort("Internal problem! istream undefined");
     }
   else if (!pstreamptr->pipe->data)
     {
-      Error("No data pointer for %s", pstreamptr->pipe->name.c_str());
+      cdoAbort("No data pointer for %s", pstreamptr->pipe->name.c_str());
     }
 }
 */
-void
+size_t
 pipe_t::pipeReadRecord(int p_vlistID, double *data, size_t *nmiss)
 {
   *nmiss = 0;
+  size_t nvals = 0 ;
 
   // LOCK
   std::unique_lock<std::mutex> locked_mutex(m_mutex);
   while (!hasdata)
     {
-      if (CdoDebug::PIPE) MESSAGE(name.c_str(), " wait of writeCond");
+      Debug(PIPE, name.c_str(), " wait of writeCond");
       writeCond.wait(locked_mutex);
     }
 
   if (hasdata)
     {
-      pipeReadPipeRecord(data, p_vlistID, nmiss);
+      nvals = pipeReadPipeRecord(data, p_vlistID, nmiss);
     }
   else
     {
-      Error("data type %d not implemented", hasdata);
+      cdoAbort("data type %d not implemented", hasdata);
     }
 
-  if (CdoDebug::PIPE) MESSAGE(name.c_str(), " read record ", recIDr);
+  Debug(PIPE, name.c_str(), " read record ", recIDr);
 
   hasdata = false;
   data = nullptr;
@@ -456,6 +481,7 @@ pipe_t::pipeReadRecord(int p_vlistID, double *data, size_t *nmiss)
   // UNLOCK
 
   readCond.notify_all();
+  return nvals;
 }
 
 void
@@ -474,7 +500,7 @@ pipe_t::pipeWriteRecord(double *p_data, size_t p_nmiss)
 
   writeCond.notify_all();
 
-  if (CdoDebug::PIPE) MESSAGE(name.c_str(), " write record ", recIDw);
+  Debug(PIPE, "%s write record $d", name, recIDw);
 
   // LOCK
   std::unique_lock<std::mutex> locked_mutex(m_mutex);
@@ -489,10 +515,10 @@ pipe_t::pipeWriteRecord(double *p_data, size_t p_nmiss)
 
       if (EOP)
         {
-          if (CdoDebug::PIPE) MESSAGE("EOP");
+          Debug(PIPE, "EOP");
           break;
         }
-      if (CdoDebug::PIPE) MESSAGE(name.c_str(), " wait of readCond");
+      Debug(PIPE, " wait of readCond %s", name);
       readCond.wait(locked_mutex);
     }
   // UNLOCK
