@@ -49,8 +49,8 @@ public:
   int pipeInqRecord(int *varID, int *levelID);
 
   void pipeWriteRecord(double *p_data, size_t p_nmiss);
-  void pipeReadRecord(int p_vlistID, double *data, size_t *nmiss);
-  void pipeReadPipeRecord(double *data, int vlistID, size_t *p_nmiss);
+  size_t pipeReadRecord(int p_vlistID, double *data, size_t *nmiss);
+  size_t pipeReadPipeRecord(double *data, int vlistID, size_t *p_nmiss);
 
   void pipeSetName(int processID, int inputIDX);
   void close();
@@ -63,7 +63,6 @@ public:
   int recIDr, recIDw, tsIDr, tsIDw;
   size_t nmiss;
   double *data;
-  size_t nvals;
 
   std::mutex m_mutex;
   std::condition_variable tsDef, tsInq, vlistDef, isclosed;
