@@ -21,7 +21,7 @@
 
 #include "pmlist.h"
 #include "namelist.h"
-#include "exception.h"
+#include "cdo_output.h"
 
 static void
 keyValuesPrint(const KeyValues &keyValues)
@@ -304,13 +304,9 @@ parseListBuffer(PMList &pmlist, ListBuffer &listBuffer)
           fprintf(stderr, "%s: Invalid character in %s (line=%d character='%c')!\n", errMsg, name, p.lineno,
                   listBuffer.buffer[p.pos]);
           break;
-        case NamelistError::PART:
-          fprintf(stderr, "%s: End of string not found in %s (line=%d)!\n", errMsg, name, p.lineno);
-          break;
+        case NamelistError::PART: fprintf(stderr, "%s: End of string not found in %s (line=%d)!\n", errMsg, name, p.lineno); break;
         case NamelistError::INKEY: fprintf(stderr, "%s: Invalid key word in %s (line=%d)!\n", errMsg, name, p.lineno); break;
-        case NamelistError::INTYP:
-          fprintf(stderr, "%s: Invalid key word type in %s (line=%d)!\n", errMsg, name, p.lineno);
-          break;
+        case NamelistError::INTYP: fprintf(stderr, "%s: Invalid key word type in %s (line=%d)!\n", errMsg, name, p.lineno); break;
         case NamelistError::INOBJ: fprintf(stderr, "%s: Invalid object in %s (line=%d)!\n", errMsg, name, p.lineno); break;
         case NamelistError::EMKEY: fprintf(stderr, "%s: Emtry key name in %s (line=%d)!\n", errMsg, name, p.lineno); break;
         default: fprintf(stderr, "%s in %s (line=%d)!\n", errMsg, name, p.lineno); break;
