@@ -1,9 +1,3 @@
-// This file is used in CDI and CDO !!!
-
-#ifdef HAVE_CONFIG_H
-#include "../src/config.h"
-#endif
-
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
@@ -11,38 +5,17 @@
 #include <cdi.h>
 #include "cdi_uuid.h"
 
-#ifdef CDO
-
-#include "text.h"
+#include "mpmo_color.h"
 #include "datetime.h"
 #include "compare.h"
 
 #include "cdo_options.h"
-#include "pstream_int.h"
+
 #include "process_int.h"  //TODO: this include must not be here.
                           //refactoring needed (11.9.2018);
-#define streamInqFiletype pstreamInqFiletype
-#define streamInqByteorder pstreamInqByteorder
-#define streamInqTimestep cdoStreamInqTimestep
-
-#else
-
-#include "cdi_int.h"
-
-#endif
-
 
 #include "printinfo.h"
 
-static void
-my_reset_text_color(FILE *fp)
-{
-  (void) fp;
-#ifdef CDO
-  reset_text_color(fp);
-#endif
-}
-
 void
 datetime2str(int64_t date, int time, char *datetimestr, int maxlen)
 {
@@ -51,8 +24,7 @@ datetime2str(int64_t date, int time, char *datetimestr, int maxlen)
   int hour, minute, second;
   cdiDecodeTime(time, &hour, &minute, &second);
 
-  const int len = sprintf(datetimestr, DATE_FORMAT "T" TIME_FORMAT, year, month, day, hour, minute, second);
-  if (len > (maxlen - 1)) fprintf(stderr, "Internal problem (%s): sizeof input string is too small!\n", __func__);
+  snprintf(datetimestr, maxlen, DATE_FORMAT "T" TIME_FORMAT, year, month, day, hour, minute, second);
 }
 
 void
@@ -61,8 +33,7 @@ date2str(int64_t date, char *datestr, int maxlen)
   int year, month, day;
   cdiDecodeDate(date, &year, &month, &day);
 
-  const int len = sprintf(datestr, DATE_FORMAT, year, month, day);
-  if (len > (maxlen - 1)) fprintf(stderr, "Internal problem (%s): sizeof input string is too small!\n", __func__);
+  snprintf(datestr, maxlen, DATE_FORMAT, year, month, day);
 }
 
 void
@@ -71,8 +42,7 @@ time2str(int time, char *timestr, int maxlen)
   int hour, minute, second;
   cdiDecodeTime(time, &hour, &minute, &second);
 
-  const int len = sprintf(timestr, TIME_FORMAT, hour, minute, second);
-  if (len > (maxlen - 1)) fprintf(stderr, "Internal problem (%s): sizeof input string is too small!\n", __func__);
+  snprintf(timestr, maxlen, TIME_FORMAT, hour, minute, second);
 }
 
 const char *
@@ -86,9 +56,9 @@ comp_name(int comptype)
 }
 
 void
-printFiletype(int streamID, int vlistID)
+printFiletype(CdoStreamID streamID, int vlistID)
 {
-  const int filetype = streamInqFiletype(streamID);
+  const int filetype = cdoInqFiletype(streamID);
 
   // clang-format off
   switch ( filetype )
@@ -108,11 +78,11 @@ printFiletype(int streamID, int vlistID)
 
   if ( filetype == CDI_FILETYPE_SRV || filetype == CDI_FILETYPE_EXT || filetype == CDI_FILETYPE_IEG )
     {
-      switch ( streamInqByteorder(streamID) )
+      switch ( cdoInqByteorder(streamID) )
 	{
 	case CDI_BIGENDIAN:    printf("  BIGENDIAN");  break;
 	case CDI_LITTLEENDIAN: printf("  LITTLEENDIAN");  break;
-	default: printf("  byteorder: %d undefined", streamInqByteorder(streamID));  break;
+	default: printf("  byteorder: %d undefined", cdoInqByteorder(streamID));  break;
 	}
     }
   // clang-format on
@@ -126,11 +96,7 @@ printFiletype(int streamID, int vlistID)
         const int comptype = vlistInqVarCompType(vlistID, varID);
         if (comptype == comps[k])
           {
-            if (kk++ == 0)
-              printf(" ");
-            else
-              printf("/");
-            printf("%s", comp_name(comptype));
+            printf("%c%s", (kk++ == 0) ? ' ' : '/', comp_name(comptype));
             break;
           }
       }
@@ -142,7 +108,7 @@ static void
 print_xvals(int gridID, int dig)
 {
   const size_t xsize = gridInqXsize(gridID);
-  if (xsize > 0 && gridInqXvals(gridID, NULL))
+  if (xsize && gridInqXvals(gridID, NULL))
     {
       char xname[CDI_MAX_NAME], xunits[CDI_MAX_NAME];
       gridInqXname(gridID, xname);
@@ -167,7 +133,7 @@ static void
 print_yvals(int gridID, int dig)
 {
   const size_t ysize = gridInqYsize(gridID);
-  if (ysize > 0 && gridInqYvals(gridID, NULL))
+  if (ysize && gridInqYvals(gridID, NULL))
     {
       char yname[CDI_MAX_NAME], yunits[CDI_MAX_NAME];
       gridInqYname(gridID, yname);
@@ -201,11 +167,11 @@ print_xyvals2D(int gridID, int dig)
       gridInqYunits(gridID, yunits);
 
       const size_t gridsize = gridInqSize(gridID);
-      double *xvals2D = (double *) malloc(gridsize * sizeof(double));
-      double *yvals2D = (double *) malloc(gridsize * sizeof(double));
+      std::vector<double> xvals2D(gridsize);
+      std::vector<double> yvals2D(gridsize);
 
-      gridInqXvals(gridID, xvals2D);
-      gridInqYvals(gridID, yvals2D);
+      gridInqXvals(gridID, xvals2D.data());
+      gridInqYvals(gridID, yvals2D.data());
 
       double xfirst = xvals2D[0];
       double xlast = xvals2D[0];
@@ -228,16 +194,15 @@ print_xyvals2D(int gridID, int dig)
           const size_t ysize = gridInqYsize(gridID);
           if (xsize > 1)
             {
-              double *xvals = (double *) malloc((size_t) xsize * sizeof(double));
+              std::vector<double> xvals(xsize);
               for (size_t i = 0; i < xsize; ++i) xvals[i] = xvals2D[i];
               xinc = fabs(xvals[xsize - 1] - xvals[0]) / (xsize - 1);
               for (size_t i = 1; i < xsize; i++)
-                if (fabs(fabs(xvals[i - 1] - xvals[i]) - xinc) > 0.01 * xinc)
+                if (fabs(fabs(xvals[i - 1] - xvals[i]) - xinc) > 0.005 * xinc)
                   {
                     xinc = 0;
                     break;
                   }
-              free(xvals);
               if (IS_NOT_EQUAL(xinc, 0))
                 {
                   for (size_t i = 1; i < ysize; i++)
@@ -251,16 +216,15 @@ print_xyvals2D(int gridID, int dig)
             }
           if (ysize > 1)
             {
-              double *yvals = (double *) malloc((size_t) ysize * sizeof(double));
+              std::vector<double> yvals(ysize);
               for (size_t i = 0; i < ysize; ++i) yvals[i] = yvals2D[i * xsize];
               yinc = fabs(yvals[ysize - 1] - yvals[0]) / (ysize - 1);
               for (size_t i = 1; i < ysize; i++)
-                if (fabs(fabs(yvals[i - 1] - yvals[i]) - yinc) > 0.01 * yinc)
+                if (fabs(fabs(yvals[i - 1] - yvals[i]) - yinc) > 0.005 * yinc)
                   {
                     yinc = 0;
                     break;
                   }
-              free(yvals);
               if (IS_NOT_EQUAL(yinc, 0))
                 {
                   for (size_t i = 1; i < xsize; i++)
@@ -285,17 +249,13 @@ print_xyvals2D(int gridID, int dig)
       if (IS_NOT_EQUAL(yinc, 0)) fprintf(stdout, " by %.*g", dig, yinc);
       fprintf(stdout, " %s", yunits);
       fprintf(stdout, "\n");
-
-      free(xvals2D);
-      free(yvals2D);
     }
 }
 
 static void
 printGridNp(int gridtype, int gridID, size_t gridsize, size_t xsize, size_t ysize)
 {
-
-  size_t xysize = xsize * ysize;
+  const size_t xysize = xsize * ysize;
   fprintf(stdout, "points=%zu", gridsize);
   if (gridtype == GRID_GAUSSIAN_REDUCED)
     fprintf(stdout, "  nlat=%zu", ysize);
@@ -303,7 +263,7 @@ printGridNp(int gridtype, int gridID, size_t gridsize, size_t xsize, size_t ysiz
     fprintf(stdout, " (%zux%zu)", xsize, ysize);
 
   if (gridtype == GRID_GAUSSIAN || gridtype == GRID_GAUSSIAN_REDUCED) fprintf(stdout, "  np=%d", gridInqNP(gridID));
-  my_reset_text_color(stdout);
+  reset_text_color(stdout);
 
   fprintf(stdout, "\n");
 }
@@ -311,7 +271,7 @@ printGridNp(int gridtype, int gridID, size_t gridsize, size_t xsize, size_t ysiz
 static void
 printGridInfoKernel(int gridID, int index, int lproj)
 {
-  int gridtype = gridInqType(gridID);
+  const int gridtype = gridInqType(gridID);
 
   if (lproj && gridtype != GRID_PROJECTION)
     fprintf(stderr, "Internal problem (%s): sub grid not equal GRID_PROJECTION!\n", __func__);
@@ -321,21 +281,14 @@ printGridInfoKernel(int gridID, int index, int lproj)
   const size_t xsize = gridInqXsize(gridID);
   const size_t ysize = gridInqYsize(gridID);
 
-  // int prec     = gridInqDatatype(gridID);
-  // int dig = (prec == CDI_DATATYPE_FLT64) ? 15 : 7;
-  int dig = 7;
-#ifdef CDO
-  dig = Options::CDO_flt_digits;
-#endif
+  const int dig = Options::CDO_flt_digits;
 
   if (!lproj)
     {
       fprintf(stdout, "  %4d : ", index + 1);
-#ifdef CDO
-      set_text_color(stdout, RESET, BLUE);
-#endif
+      set_text_color(stdout, BLUE);
       fprintf(stdout, "%-24s", gridNamePtr(gridtype));
-      my_reset_text_color(stdout);
+      reset_text_color(stdout);
       fprintf(stdout, " : ");
     }
 
@@ -345,9 +298,7 @@ printGridInfoKernel(int gridID, int index, int lproj)
       if (!lproj)
 
         {
-#ifdef CDO
-          set_text_color(stdout, RESET, GREEN);
-#endif
+          set_text_color(stdout, GREEN);
           printGridNp(gridtype, gridID, gridsize, xsize, ysize);
         }
 
@@ -357,26 +308,24 @@ printGridInfoKernel(int gridID, int index, int lproj)
       if (gridtype == GRID_PROJECTION || name[0])
         {
           if (name[0] == 0) strcpy(name, "undefined");
-#ifdef CDO
-          set_text_color(stdout, RESET, BLUE);
-#endif
+          set_text_color(stdout, BLUE);
           fprintf(stdout, "         %24s", "mapping");
-          my_reset_text_color(stdout);
+          reset_text_color(stdout);
           fprintf(stdout, " : ");
-#ifdef CDO
-          set_text_color(stdout, RESET, GREEN);
-#endif
+          set_text_color(stdout, GREEN);
           fprintf(stdout, "%s\n", name);
-          my_reset_text_color(stdout);
+          reset_text_color(stdout);
         }
 
-      print_xvals(gridID, dig);
+      if (gridtype != GRID_GAUSSIAN_REDUCED) print_xvals(gridID, dig);
       print_yvals(gridID, dig);
 
       if (gridInqXbounds(gridID, NULL) || gridInqYbounds(gridID, NULL))
         {
           fprintf(stdout, "%33s :", "available");
           if (gridInqXbounds(gridID, NULL) && gridInqYbounds(gridID, NULL)) fprintf(stdout, " cellbounds");
+          else if (gridInqXbounds(gridID, NULL)) fprintf(stdout, " xbounds");
+          else if (gridInqYbounds(gridID, NULL)) fprintf(stdout, " ybounds");
           if (gridHasArea(gridID)) fprintf(stdout, " area");
           if (gridInqMask(gridID, NULL)) fprintf(stdout, " mask");
           fprintf(stdout, "\n");
@@ -384,44 +333,36 @@ printGridInfoKernel(int gridID, int index, int lproj)
     }
   else if (gridtype == GRID_SPECTRAL)
     {
-#ifdef CDO
-      set_text_color(stdout, RESET, GREEN);
-#endif
+      set_text_color(stdout, GREEN);
       fprintf(stdout, "points=%zu  nsp=%zu  truncation=%d", gridsize, gridsize / 2, trunc);
       if (gridInqComplexPacking(gridID)) fprintf(stdout, "  complexPacking");
-      my_reset_text_color(stdout);
+      reset_text_color(stdout);
       fprintf(stdout, "\n");
     }
   else if (gridtype == GRID_FOURIER)
     {
-#ifdef CDO
-      set_text_color(stdout, RESET, GREEN);
-#endif
+      set_text_color(stdout, GREEN);
       fprintf(stdout, "points=%zu  nfc=%zu  truncation=%d\n", gridsize, gridsize / 2, trunc);
-      my_reset_text_color(stdout);
+      reset_text_color(stdout);
     }
   else if (gridtype == GRID_GME)
     {
       int nd, ni, ni2, ni3;
       gridInqParamGME(gridID, &nd, &ni, &ni2, &ni3);
-#ifdef CDO
-      set_text_color(stdout, RESET, GREEN);
-#endif
+      set_text_color(stdout, GREEN);
       fprintf(stdout, "points=%zu  nd=%d  ni=%d\n", gridsize, nd, ni);
-      my_reset_text_color(stdout);
+      reset_text_color(stdout);
     }
   else if (gridtype == GRID_CURVILINEAR || gridtype == GRID_UNSTRUCTURED)
     {
-#ifdef CDO
-      set_text_color(stdout, RESET, GREEN);
-#endif
+      set_text_color(stdout, GREEN);
       if (gridtype == GRID_CURVILINEAR)
         fprintf(stdout, "points=%zu (%zux%zu)", gridsize, xsize, ysize);
       else
         fprintf(stdout, "points=%zu", gridsize);
 
       if (gridtype == GRID_UNSTRUCTURED && gridInqNvertex(gridID) > 0) fprintf(stdout, "  nvertex=%d", gridInqNvertex(gridID));
-      my_reset_text_color(stdout);
+      reset_text_color(stdout);
 
       fprintf(stdout, "\n");
 
@@ -443,14 +384,12 @@ printGridInfoKernel(int gridID, int index, int lproj)
     }
   else /* if ( gridtype == GRID_GENERIC ) */
     {
-#ifdef CDO
-      set_text_color(stdout, RESET, GREEN);
-#endif
+      set_text_color(stdout, GREEN);
       if (ysize == 0)
         fprintf(stdout, "points=%zu\n", gridsize);
       else
         fprintf(stdout, "points=%zu (%zux%zu)\n", gridsize, xsize, ysize);
-      my_reset_text_color(stdout);
+      reset_text_color(stdout);
     }
 
   if (gridtype == GRID_CURVILINEAR || gridtype == GRID_UNSTRUCTURED)
@@ -494,29 +433,54 @@ printGridInfo(int vlistID)
 static void
 printZaxisBoundsInfo(const int zaxisID, const int dig, const int levelsize, const double zinc, const char *zunits)
 {
-  fprintf(stdout, "%33s : ", "bounds");
-
   double level1 = zaxisInqLbound(zaxisID, 0);
   double level2 = zaxisInqUbound(zaxisID, 0);
-  fprintf(stdout, "%.*g-%.*g", dig, level1, dig, level2);
-  if (levelsize > 1)
+  if (!(levelsize == 1 && IS_EQUAL(level1, level2) && fabs(level1) <= 0))
     {
-      level1 = zaxisInqLbound(zaxisID, levelsize - 1);
-      level2 = zaxisInqUbound(zaxisID, levelsize - 1);
-      fprintf(stdout, " to %.*g-%.*g", dig, level1, dig, level2);
-      if (IS_NOT_EQUAL(zinc, 0)) fprintf(stdout, " by %.*g", dig, zinc);
+      fprintf(stdout, "%33s : ", "bounds");
+      fprintf(stdout, "%.*g-%.*g", dig, level1, dig, level2);
+      if (levelsize > 1)
+        {
+          level1 = zaxisInqLbound(zaxisID, levelsize - 1);
+          level2 = zaxisInqUbound(zaxisID, levelsize - 1);
+          fprintf(stdout, " to %.*g-%.*g", dig, level1, dig, level2);
+          if (IS_NOT_EQUAL(zinc, 0)) fprintf(stdout, " by %.*g", dig, zinc);
+        }
+      fprintf(stdout, " %s", zunits);
+      fprintf(stdout, "\n");
     }
-  fprintf(stdout, " %s", zunits);
-  fprintf(stdout, "\n");
+}
+
+static bool
+zaxisTypeIsSingleLayer(int zaxistype)
+{
+  switch (zaxistype)
+    {
+    case ZAXIS_MEANSEA:
+    case ZAXIS_TOA:
+    case ZAXIS_SEA_BOTTOM:
+    case ZAXIS_ATMOSPHERE:
+    case ZAXIS_CLOUD_BASE:
+    case ZAXIS_CLOUD_TOP:
+    case ZAXIS_ISOTHERM_ZERO:
+    case ZAXIS_LAKE_BOTTOM:
+    case ZAXIS_SEDIMENT_BOTTOM:
+    case ZAXIS_SEDIMENT_BOTTOM_TA:
+    case ZAXIS_SEDIMENT_BOTTOM_TW:
+    case ZAXIS_SURFACE:            return true;
+    }
+
+  return false;
 }
 
 static void
-printZaxisLevelInfo(const int levelsize, const int zaxisID, const int zaxistype, double zinc, const int dig, const char *zname, const char *zunits)
+printZaxisLevelInfo(const int levelsize, const int zaxisID, const int zaxistype, double &zinc, const int dig, const char *zname,
+                    const char *zunits)
 {
-  double *levels = (double *) malloc((size_t) levelsize * sizeof(double));
-  zaxisInqLevels(zaxisID, levels);
+  std::vector<double> levels((size_t) levelsize);
+  zaxisInqLevels(zaxisID, levels.data());
 
-  if (!(zaxistype == ZAXIS_SURFACE && levelsize == 1 && fabs(levels[0]) <= 0))
+  if (!(zaxisTypeIsSingleLayer(zaxistype) && levelsize == 1 && fabs(levels[0]) <= 0))
     {
       const double zfirst = levels[0];
       const double zlast = levels[levelsize - 1];
@@ -540,8 +504,6 @@ printZaxisLevelInfo(const int levelsize, const int zaxisID, const int zaxistype,
       fprintf(stdout, " %s", zunits);
       fprintf(stdout, "\n");
     }
-
-  free(levels);
 }
 
 static void
@@ -605,18 +567,12 @@ printZaxisInfo(int vlistID)
   const int nzaxis = vlistNzaxis(vlistID);
   for (int index = 0; index < nzaxis; index++)
     {
-      int dig = 7;
-#ifdef CDO
-      dig = Options::CDO_flt_digits;
-#endif
-      //______________________________________________________--
-      double zinc = 0;
+      const int dig = Options::CDO_flt_digits;
       const int zaxisID = vlistZaxis(vlistID, index);
       const int zaxistype = zaxisInqType(zaxisID);
       const int ltype = zaxisInqLtype(zaxisID);
+      const int ltype2 = zaxisInqLtype2(zaxisID);
       const int levelsize = zaxisInqSize(zaxisID);
-      // int prec      = zaxisInqDatatype(zaxisID);
-      // int dig = (prec == CDI_DATATYPE_FLT64) ? 15 : 7;
 
       zaxisName(zaxistype, zaxisname);
       zaxisInqName(zaxisID, zname);
@@ -624,37 +580,33 @@ printZaxisInfo(int vlistID)
       zunits[12] = 0;
 
       fprintf(stdout, "  %4d : ", vlistZaxisIndex(vlistID, zaxisID) + 1);
-#ifdef CDO
-      set_text_color(stdout, RESET, BLUE);
-#endif
+      set_text_color(stdout, BLUE);
       printZaxisGenericInfo(ltype, zaxistype, zaxisname);
 
-      my_reset_text_color(stdout);
+      reset_text_color(stdout);
 
       fprintf(stdout, " :");
 
-#ifdef CDO
-      set_text_color(stdout, RESET, GREEN);
-#endif
+      set_text_color(stdout, GREEN);
       fprintf(stdout, " levels=%d", levelsize);
       const int zscalar = (levelsize == 1) ? zaxisInqScalar(zaxisID) : false;
       if (zscalar) fprintf(stdout, "  scalar");
-      my_reset_text_color(stdout);
+      reset_text_color(stdout);
       fprintf(stdout, "\n");
 
+      double zinc = 0;
       if (zaxisInqLevels(zaxisID, NULL))
-        {
-          printZaxisLevelInfo(levelsize, zaxisID, zaxistype, zinc, dig, zname, zunits);
-        }
+        printZaxisLevelInfo(levelsize, zaxisID, zaxistype, zinc, dig, zname, zunits);
 
       if (zaxisInqLbounds(zaxisID, NULL) && zaxisInqUbounds(zaxisID, NULL))
-        {
-          printZaxisBoundsInfo(zaxisID, dig, levelsize, zinc, zunits);
-        }
+        printZaxisBoundsInfo(zaxisID, dig, levelsize, zinc, zunits);
 
       if (zaxistype == ZAXIS_HYBRID) printZaxisHybridInfo(zaxisID);
 
       if (zaxistype == ZAXIS_REFERENCE) printZaxisReferenceInfo(zaxisID);
+
+      if (ltype != ltype2 && ltype2 != -1)
+        fprintf(stdout, "%33s : %d\n", "typeOfSecondFixedSurface", ltype2);
     }
 }
 
@@ -676,17 +628,15 @@ printSubtypeInfo(int vlistID)
 static int
 printDateTime(int ntimeout, const int64_t vdate, const int vtime)
 {
-  char vdatestr[32], vtimestr[32];
-
   if (ntimeout == 4)
     {
       ntimeout = 0;
       fprintf(stdout, "\n");
     }
 
+  char vdatestr[32], vtimestr[32];
   date2str(vdate, vdatestr, sizeof(vdatestr));
   time2str(vtime, vtimestr, sizeof(vtimestr));
-
   fprintf(stdout, " %s %s", vdatestr, vtimestr);
 
   return ++ntimeout;
@@ -720,7 +670,7 @@ printDot(int ndotout, int *nfact, int *ncout)
 }
 
 void
-printTimesteps(int streamID, int taxisID, int verbose)
+printTimesteps(CdoStreamID streamID, int taxisID, int verbose)
 {
   int nrecs;
   struct datetime
@@ -742,19 +692,12 @@ printTimesteps(int streamID, int taxisID, int verbose)
   int nfact = 1;
   int tsID = 0;
 
-#ifdef CDO
   DateTimeList dtlist;
-#endif
-  while ((nrecs = streamInqTimestep(streamID, tsID)))
+  while ((nrecs = cdoStreamInqTimestep(streamID, tsID)))
     {
-#ifdef CDO
       dtlist.taxisInqTimestep(taxisID, 0);
       const int64_t vdate = dtlist.getVdate(0);
       const int vtime = dtlist.getVtime(0);
-#else
-      const int64_t vdate = taxisInqVdate(taxisID);
-      const int vtime = taxisInqVtime(taxisID);
-#endif
 
       if (verbose || tsID < NUM_TIMESTEP)
         {
