@@ -1,10 +1,9 @@
-// This file is used in CDI and CDO !!!
-
 #ifndef PRINTINFO_H
 #define PRINTINFO_H
 
 #include <stdio.h>
 #include <inttypes.h>
+#include "process_int.h"
 
 #define DATE_FORMAT "%5.4d-%2.2d-%2.2d"
 #define TIME_FORMAT "%2.2d:%2.2d:%2.2d"
@@ -17,7 +16,7 @@ void time2str(int time, char *timestr, int maxlen);
 
 const char *comp_name(int comptype);
 
-void printFiletype(int streamID, int vlistID);
+void printFiletype(CdoStreamID p_streamID, int vlistID);
 
 void printGridInfo(int vlistID);
 
@@ -25,7 +24,7 @@ void printZaxisInfo(int vlistID);
 
 void printSubtypeInfo(int vlistID);
 
-void printTimesteps(int streamID, int taxisID, int verbose);
+void printTimesteps(CdoStreamID streamID, int taxisID, int verbose);
 
 #endif
 /*
