@@ -29,25 +29,24 @@
 #include <stdio.h>
 #include <iostream>
 #include <string>
+#include <sstream>
 
 #include "process.h"
-#include "exception.h"
+#include "cdo_output.h"
 #include "readline.h"
 
 #include "cdo_wtime.h"
-#include "error.h"
 
 #include "dmemory.h"
 #include "pthread.h"
-#include "cdo_debugOutput.h"
 #include "util_string.h"
 #include "util_files.h"
-#include "text.h"
+#include "mpmo_color.h"
 #include "cdo_options.h"
+#include "fileStream.h"
+#include "pipeStream.h"
 
-
-Process::Process(int p_ID, const char *p_operatorName, const char *operatorCommand)
-    : m_ID(p_ID), operatorName(p_operatorName)
+Process::Process(int p_ID, const char *p_operatorName, const char *operatorCommand) : m_ID(p_ID), operatorName(p_operatorName)
 {
   initProcess();
   setOperatorArgv(operatorCommand);
@@ -65,6 +64,7 @@ Process::setOperatorArgv(const char *operatorArguments)
     {
       char *operatorArg = strdup(operatorArguments);
       // fprintf(stderr, "processDefArgument: %d %s\n", oargc, operatorArg);
+      Debug(PROCESS && strchr(operatorArg, ',') != nullptr, "Setting operator arguments: %s", operatorArguments);
 
       while ((operatorArg = strchr(operatorArg, ',')) != nullptr)
         {
@@ -91,7 +91,6 @@ Process::initProcess()
 
   startTime = cdo_get_wtime();
 
-  m_nvals = 0;
   nvars = 0;
   ntimesteps = 0;
 
@@ -116,25 +115,19 @@ Process::getOutStreamCnt()
   return outputStreams.size();
 }
 
-void
-Process::addNvals(size_t p_nvals)
-{
-  m_nvals += p_nvals;
-}
-
 void
 Process::defPrompt()
 {
   if (m_ID == 0)
-    sprintf(prompt, "%s %s", Cdo::progname, operatorName);
+    snprintf(prompt, sizeof(prompt), "%s    %s", Cdo::progname, operatorName);
   else
-    sprintf(prompt, "%s(%d) %s", Cdo::progname, m_ID + 1, operatorName);
+    snprintf(prompt, sizeof(prompt), "%s(%d) %s", Cdo::progname, m_ID, operatorName);
 }
 
 const char *
 Process::inqPrompt() const
 {
-  return prompt[0] ? prompt : "cdo";
+  return prompt;
 }
 
 void
@@ -144,17 +137,17 @@ Process::handleProcessErr(ProcessStatus p_proErr)
     {
     case ProcessStatus::UnlimitedIOCounts:
       {
-        CdoError::Abort(Cdo::progname, "I/O stream counts unlimited no allowed!");
+        cdoAbort("I/O stream counts unlimited no allowed!");
         break;
       }
     case ProcessStatus::MissInput:
       {
-        CdoError::Abort(Cdo::progname, "Input streams missing!");
+        cdoAbort("Input streams missing!");
         break;
       }
     case ProcessStatus::MissOutput:
       {
-        CdoError::Abort(Cdo::progname, "Output streams missing!");
+        cdoAbort("Output streams missing!");
         break;
       }
     case ProcessStatus::TooManyStreams:
@@ -164,14 +157,19 @@ Process::handleProcessErr(ProcessStatus p_proErr)
         int outCnt = m_module.streamOutCnt;
         const bool lobase = outCnt == -1;
         if (lobase) outCnt = 1;
-        CdoError::Abort(Cdo::progname, "Too ", (p_proErr == ProcessStatus::TooManyStreams) ? "many" : "few",
-                        " streams specified! Operator ", m_operatorCommand, " needs ", inCnt, " input stream",
-                        (inCnt>1) ? "s" : "", " and ", outCnt, " output ", lobase ? "basename" : "stream",
-                        (outCnt>1) ? "s" : "", "!");
+
+        std::string caseCount = ((p_proErr == ProcessStatus::TooManyStreams) ? "many" : "few");
+        std::string pluralIn = ((inCnt > 1) ? "s" : "");
+        std::string pluralOut = ((outCnt > 1) ? "s" : "");
+
+        std::stringstream errMsg;
+        errMsg << "Too " << caseCount << " streams specified! Operator " << m_operatorCommand << " needs " << inCnt
+               << " input stream" << pluralIn << " and " << outCnt << " output " << (lobase ? "basename" : "stream") << pluralOut
+               << "!";
+        cdoAbort(errMsg.str());
         break;
       }
-    case ProcessStatus::Ok:
-      break;
+    case ProcessStatus::Ok: break;
     }
 }
 
@@ -223,7 +221,7 @@ Process::checkStreamCnt()
   if (m_streamCnt > streamCnt) return ProcessStatus::TooManyStreams;
 
   if (m_streamCnt < streamCnt && !obase) return ProcessStatus::TooFewStreams;
-  if (wantedStreamInCnt > (int)inputStreams.size()) return ProcessStatus::TooFewStreams;
+  if (wantedStreamInCnt > (int) inputStreams.size()) return ProcessStatus::TooFewStreams;
 
   if (wantedStreamInCnt == 1 && streamInCnt0 == -1) return ProcessStatus::Ok;
 
@@ -337,7 +335,7 @@ Process::getOperatorID()
         }
       if (operID == m_noper)
         {
-          CdoError::Abort(Cdo::progname, prompt, "Operator not callable by this name! Name is:", operatorName);
+          cdoAbort("%s Operator not callable by this name! Name is: %s", prompt, operatorName);
         }
     }
   else
@@ -351,7 +349,7 @@ Process::getOperatorID()
 void
 Process::addFileInStream(std::string file)
 {
-  inputStreams.push_back(create_pstream(file));
+  inputStreams.push_back(std::make_shared<FileStream>(file));
   m_streamCnt++;
 }
 
@@ -360,9 +358,9 @@ Process::addFileOutStream(std::string file)
 {
   if (file[0] == '-')
     {
-      CdoError::Abort(Cdo::progname, "Missing output file. Found an operator instead of filename: ", file);
+      cdoAbort("Missing output file. Found an operator instead of filename: %s", file);
     }
-  outputStreams.push_back(create_pstream(file));
+  outputStreams.push_back(std::make_shared<FileStream>(file));
   m_streamCnt++;
 }
 
@@ -378,7 +376,7 @@ void
 Process::addPipeInStream()
 {
 #ifdef HAVE_LIBPTHREAD
-  inputStreams.push_back(create_pstream(m_ID, inputStreams.size()));
+  inputStreams.push_back(std::make_shared<PipeStream>(m_ID));
   m_streamCnt++;
 #else
   cdoAbort("Cannot use pipes, pthread support not compiled in!");
@@ -402,26 +400,27 @@ Process::addPipeOutStream()
 pthread_t
 Process::run()
 {
-  Cdo_Debug(CdoDebug::PROCESS, "starting new thread for process ", m_ID);
+  Debug(PROCESS, "starting new thread for process %d", m_ID);
   pthread_attr_t attr;
   int status = pthread_attr_init(&attr);
-  if (status) SysError("pthread_attr_init failed for '%s'", operatorName);
+  if (status) cdoSysError("pthread_attr_init failed for '%s'", operatorName);
   status = pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
-  if (status) SysError("pthread_attr_setdetachstate failed for '%s'", operatorName);
+  if (status) cdoSysError("pthread_attr_setdetachstate failed for '%s'", operatorName);
   /*
     param.sched_priority = 0;
     status = pthread_attr_setschedparam(&attr, &param);
-    if ( status ) SysError("pthread_attr_setschedparam failed for '%s'",
+    if ( status ) cdoSysError("pthread_attr_setschedparam failed for '%s'",
     newarg+1);
   */
   /* status = pthread_attr_setinheritsched(&attr, PTHREAD_EXPLICIT_SCHED); */
-  /* if ( status ) SysError("pthread_attr_setinheritsched failed for '%s'",
+  /* if ( status ) cdoSysError("pthread_attr_setinheritsched failed for '%s'",
    * newarg+1); */
 
+  int pthreadScope;
   pthread_attr_getscope(&attr, &pthreadScope);
 
   /* status = pthread_attr_setscope(&attr, PTHREAD_SCOPE_PROCESS); */
-  /* if ( status ) SysError("pthread_attr_setscope failed for '%s'", newarg+1);
+  /* if ( status ) cdoSysError("pthread_attr_setscope failed for '%s'", newarg+1);
    */
   /* If system scheduling scope is specified, then the thread is scheduled
    * against all threads in the system */
@@ -440,7 +439,7 @@ Process::run()
   if (rval != 0)
     {
       errno = rval;
-      SysError("pthread_create failed for '%s'", operatorName);
+      cdoSysError("pthread_create failed for '%s'", operatorName);
     }
 
   m_isActive = true;
@@ -448,83 +447,44 @@ Process::run()
   return thrID;
 }
 
-void
-Process::query_user_exit(const char *argument)
-{
-/* modified code from NCO */
-#define USR_RPL_MAX_LNG 10 /* Maximum length for user reply */
-#define USR_RPL_MAX_NBR 10 /* Maximum number of chances for user to reply */
-  char usr_rpl[USR_RPL_MAX_LNG];
-  int usr_rpl_int;
-  short nbr_itr = 0;
-  size_t usr_rpl_lng = 0;
-
-  /* Initialize user reply string */
-  usr_rpl[0] = 'z';
-  usr_rpl[1] = '\0';
-
-  while (!(usr_rpl_lng == 1 && (*usr_rpl == 'o' || *usr_rpl == 'O' || *usr_rpl == 'e' || *usr_rpl == 'E')))
-    {
-      if (nbr_itr++ > USR_RPL_MAX_NBR)
-        {
-          (void) fprintf(stdout, "\n%s: ERROR %d failed attempts to obtain valid interactive input.\n", prompt, nbr_itr - 1);
-          exit(EXIT_FAILURE);
-        }
-
-      if (nbr_itr > 1) (void) fprintf(stdout, "%s: ERROR Invalid response.\n", prompt);
-      (void) fprintf(stdout, "%s: %s exists ---`e'xit, or `o'verwrite (delete existing file) (e/o)? ", prompt, argument);
-      (void) fflush(stdout);
-      if (fgets(usr_rpl, USR_RPL_MAX_LNG, stdin) == nullptr) continue;
-
-      /* Ensure last character in input string is \n and replace that with \0 */
-      usr_rpl_lng = strlen(usr_rpl);
-      if (usr_rpl_lng >= 1)
-        if (usr_rpl[usr_rpl_lng - 1] == '\n')
-          {
-            usr_rpl[usr_rpl_lng - 1] = '\0';
-            usr_rpl_lng--;
-          }
-    }
-
-  /* Ensure one case statement for each exit condition in preceding while loop */
-  usr_rpl_int = (int) usr_rpl[0];
-  switch (usr_rpl_int)
-    {
-    case 'E':
-    case 'e': exit(EXIT_SUCCESS); break;
-    case 'O':
-    case 'o': break;
-    default: exit(EXIT_FAILURE); break;
-    } /* end switch */
-}
-
-/** function for operators with obase usage, will be called while operator execution */
-
 void
 Process::printBenchmarks(double p_walltime, const char *p_memstring)
 {
 #ifdef HAVE_SYS_TIMES_H
   if (m_ID == 0)
     {
-      fprintf(stderr, " [%.2fs%s]\n", p_walltime, p_memstring);
+      fprintf(stderr, " [%.2fs%s]", p_walltime, p_memstring);
     }
-  else
-    {
-      fprintf(stderr, "\n");
-    }
-#else
-  fprintf(stderr, "\n");
 #endif
 }
 
+// local helper function
+extern "C" size_t getPeakRSS();
+static void
+getMaxMemString(char *p_memstring, size_t memstringLen)
+{
+  size_t memmax = getPeakRSS();
+  if (memmax)
+    {
+      size_t muindex = 0;
+      const char *mu[] = { "B", "KB", "MB", "GB", "TB", "PB" };
+      const size_t nmu = sizeof(mu) / sizeof(char *);
+      while (memmax > 9999 && muindex < nmu - 1)
+        {
+          memmax /= 1024;
+          muindex++;
+        }
+      snprintf(p_memstring, memstringLen, " %zu%s", memmax, mu[muindex]);
+    }
+}
 void
 Process::printProcessedValues()
 {
-  set_text_color(stderr, RESET, GREEN);
+  set_text_color(stderr, GREEN);
   fprintf(stderr, "%s: ", prompt);
   reset_text_color(stderr);
 
-  const int64_t nvals = m_nvals;
+  const int64_t nvals = inqNvals();
 
   if (nvals > 0)
     {
@@ -545,25 +505,59 @@ Process::printProcessedValues()
 
   if (ntimesteps > 0) fprintf(stderr, " over %d timestep%s", ntimesteps, ADD_PLURAL(ntimesteps));
 
-  //  fprintf(stderr, ".");
+  char memstring[32] = { "" };
+  if (m_ID == 0) getMaxMemString(memstring, sizeof(memstring));
+  if (!Options::silentMode) printBenchmarks(cdo_get_wtime() - startTime, memstring);
+
+  if (nvars > 0 || nvals > 0 || ntimesteps > 0 || m_ID == 0) fprintf(stderr, ".");
+  fprintf(stderr, "\n");
 }
 
 bool
-Process::hasOutStream(const Pstream *p_streamPtr)
+Process::hasOutStream(const CdoStreamID p_streamID)
 {
-  for (const Pstream *streamPtr : outputStreams)
+  for (const CdoStreamID streamID : outputStreams)
     {
-      if (streamPtr == p_streamPtr) return true;
+      if (streamID == p_streamID) return true;
     }
   return false;
 }
 
 bool
-Process::hasInStream(const Pstream *p_streamPtr)
+Process::hasInStream(const CdoStreamID p_streamID)
 {
-  for (const Pstream *streamPtr : inputStreams)
+  for (const CdoStreamID streamID : inputStreams)
     {
-      if (streamPtr == p_streamPtr) return true;
+      if (streamID == p_streamID) return true;
     }
   return false;
 }
+
+const char *
+Process::getCommand()
+{
+  return m_operatorCommand + 1;
+}
+size_t
+Process::inqNvals()
+{
+  size_t nvals = 0;
+  for (size_t i = 0; i < inputStreams.size(); i++)
+    {
+    Debug(PROCESS, "Inquiring nvlas from instream %s", inputStreams[i]->m_name);
+      nvals += inputStreams[i]->getNvals();
+    }
+  return nvals;
+}
+
+bool
+Process::hasNoPipes()
+{
+  if (childProcesses.size() == 0) return true;
+  return false;
+}
+const char *
+Process::getOutStreamName()
+{
+  return outputStreams[0]->m_name.c_str();
+}
