@@ -58,13 +58,12 @@ public:
   short nchild;
   std::vector<Process *> childProcesses;
   std::vector<Process *> parentProcesses;
-  std::vector<Pstream *> inputStreams;
-  std::vector<Pstream *> outputStreams;
+  std::vector<CdoStreamID> inputStreams;
+  std::vector<CdoStreamID> outputStreams;
   int nChildActive = 0;
 
   double startTime;
 
-  size_t m_nvals;
   short nvars;
 
   int ntimesteps;
@@ -96,6 +95,11 @@ public:
    * returns the number of out streams this process currently has.
    */
   int getOutStreamCnt();
+
+  /**
+   * returns full command through which this process was created including operator name and operator arguments
+   */
+  const char *getCommand();
   /**
    * Splits the operator arguments stores them.
    * Operator arguments are stored as char arrays and appended with
@@ -132,8 +136,6 @@ public:
    * Adds and creates a new file pstream to the out streams
    */
   void addPipeOutStream();
-  void addNvals(size_t p_nvals);
-  void query_user_exit(const char *argument);
   void inqUserInputForOpArg(const char *enter);
   /**
    * Adds an operator to the process
@@ -151,8 +153,11 @@ public:
   void validate();
   void handleProcessErr(ProcessStatus p_proErr);
 
-  bool hasOutStream(const Pstream *p_streamPtr);
-  bool hasInStream(const Pstream *p_streamPtr);
+  bool hasOutStream(const CdoStreamID p_streamPtr);
+  bool hasInStream(const CdoStreamID p_streamPtr);
+  bool hasNoPipes();
+  size_t inqNvals();
+  const char * getOutStreamName();
 
 private:
   void initProcess();
