@@ -1,14 +1,16 @@
 #include "processManager.h"
 #include "process.h"
-#include "cdo_debugOutput.h"
+#include "cdo_output.h"
 #include "cdo_options.h"
-#include "text.h"
+#include "mpmo_color.h"
+#include "fileStream.h"
 
 #include <stack>
 
 #ifdef HAVE_LIBPTHREAD
 static pthread_mutex_t processMutex = PTHREAD_MUTEX_INITIALIZER;
 #endif
+static std::string parse_err_msg = "";
 
 void
 ProcessManager::runProcesses()
@@ -20,15 +22,13 @@ ProcessManager::runProcesses()
           /*TEMP*/
           if (Options::silentMode == 0)
             {
-              set_text_color(stderr, RESET, GREEN);
-              fprintf(stderr, "%s: ", idProcessPair.second.prompt);
-              reset_text_color(stderr);
-              std::cerr << "Process started" << std::endl;
+              MpMO::PrintCerr(Green("%s: ") + "Process started", idProcessPair.second.prompt);
             }
           m_threadIDs.push_back(idProcessPair.second.run());
         }
     }
   m_threadIDs.push_back(pthread_self());
+  // MpMO::PrintCerr(Green("%s: ") + "xProcess started", getProcess(0).inqPrompt());
   getProcess(0).m_module.func(&(getProcess(0)));
 }
 
@@ -40,7 +40,7 @@ ProcessManager::killProcesses()
       if (threadID != pthread_self())
         {
           pthread_cancel(threadID);
-          Cdo_Debug(CdoDebug::PROCESS_MANAGER, "process kiled:", threadID);
+          Debug(PROCESS_MANAGER, "process killed: %d", threadID);
         }
     }
 }
@@ -57,7 +57,7 @@ ProcessManager::validateProcesses()
 void
 ProcessManager::clearProcesses()
 {
-  Cdo_Debug(CdoDebug::PROCESS_MANAGER, "Deleting Processes");
+  Debug(PROCESS_MANAGER, "Deleting Processes");
   m_processes.clear();
   m_numProcesses = 0;
   m_numProcessesActive = 0;
@@ -66,20 +66,20 @@ ProcessManager::clearProcesses()
 Process *
 ProcessManager::createProcess(const char *command)
 {
-  Cdo_Debug(CdoDebug::PROCESS, "Creating new process for command: ", command, " with ID: ", m_numProcesses);
+  Debug(PROCESS, "Creating new process for command: %s with ID: %d ", command, m_numProcesses);
   int processID = m_numProcesses++;
 
   const char *operatorName = get_original(command);
   auto success = m_processes.insert(std::make_pair(processID, Process(processID, operatorName, command)));
   if (!success.second)
     {
-      ERROR("Process ", processID, " could not be created");
+      cdoAbort("Process %d could not be created", processID);
     }
 
   m_numProcessesActive++;
-  Cdo_Debug(CdoDebug::PROCESS, "m_numProcessesActive: ", m_numProcessesActive);
+  Debug(PROCESS, "m_numProcessesActive: %d", m_numProcessesActive);
 
-  if (processID >= MAX_PROCESS) ERROR("Limit of ", MAX_PROCESS, " processes reached!");
+  if (processID >= MAX_PROCESS) cdoAbort("Limit of %d processes reached!", MAX_PROCESS);
 
   return &success.first->second;
 }
@@ -124,18 +124,21 @@ ProcessManager::processNumsActive(void)
  * as input and exits with error message if not.
  */
 Process *
-ProcessManager::addProcess(Process *p_parentProces, const char *argvEntry)
+ProcessManager::addProcess(Process *p_parentProcess, const char *argvEntry)
 {
+  if (p_parentProcess->m_module.restrictions == FilesOnly)
+    {
+      cdoAbort("operator -%s can not be used with pipes.", p_parentProcess->operatorName);
+    }
   Process *newProcess = createProcess(argvEntry);
   if (newProcess->m_module.streamOutCnt == 0)
     {
-      CdoError::Abort(Cdo::progname, "operator -", p_parentProces->operatorName, " can not take -", newProcess->operatorName,
-                      "  with 0 outputs as input");
-      exit(EXIT_FAILURE);
+      cdoAbort("operator -%s can not take -%s  with 0 outputs as input", p_parentProcess->operatorName, newProcess->operatorName);
     }
 
-  p_parentProces->addChild(newProcess);
-  newProcess->addParent(p_parentProces);
+  Debug(PROCESS, "Adding new child %s to %s", newProcess->getCommand(), p_parentProcess->getCommand());
+  p_parentProcess->addChild(newProcess);
+  newProcess->addParent(p_parentProcess);
   return newProcess;
 }
 
@@ -144,10 +147,14 @@ ProcessManager::handleFirstOperator(int p_argcStart, int argc, const char **argv
 {
   for (int i = p_argcStart; i < argc; i++)
     {
-      Cdo_Debug(CdoDebug::PROCESS, "Creating new pstream for output file: ", argv[i]);
+      Debug(PROCESS, "Creating new pstream for output file: %s", argv[i]);
       if (strcmp(argv[i], "]") == 0)
         {
-          CdoError::Abort(Cdo::progname, "missing output file");
+          cdoAbort("missing output file");
+        }
+      if (argv[i][0] == '-')
+        {
+          cdoAbort("missing Output file, did you add a '-' to a file?");
         }
       p_rootProcess->addFileOutStream(argv[i]);
     }
@@ -158,7 +165,7 @@ ProcessManager::checkSingleBracketOnly(const char *p_argvEntry, char p_bracketTy
 {
   if (strlen(p_argvEntry) != 1)
     {
-      CdoError::Abort(Cdo::progname, "Only single ", p_bracketType, " allowed");
+      cdoAbort("Only single %s allowed", p_bracketType);
     }
 }
 
@@ -173,6 +180,24 @@ ProcessManager::createProcesses(int argc, const char **argv)
     }
   validateProcesses();
 }
+#include <algorithm>
+std::string
+getStackAsString(std::stack<Process *> p_stack)
+{
+  std::string stackString = "";
+  std::vector<std::string> strVec;
+  while (!p_stack.empty())
+    {
+      strVec.push_back(p_stack.top()->operatorName);
+      p_stack.pop();
+    }
+  std::reverse(strVec.begin(), strVec.end());
+  for (auto strEle : strVec)
+    {
+      stackString += " " + strEle;
+    }
+  return stackString;
+}
 
 /* comment FOR DEVELOPERS ONLY (do not include in docu)
  *
@@ -217,8 +242,8 @@ ProcessManager::createProcesses(int argc, const char **argv)
 ParseStatus
 ProcessManager::createProcessesFromInput(int argc, const char **argv)
 {
-  Cdo_Debug(CdoDebug::PROCESS, "== Process Creation Start ==");
-  Cdo_Debug(CdoDebug::PROCESS, "operators:  ", CdoDebug::argvToString(argc, argv));
+  Debug(PROCESS, "== Process Creation Start ==");
+  Debug(PROCESS, "operators: %s", argvToString(argc, argv));
 
   Process *root_process = createProcess(argv[0]);
   int cntOutFiles = (int) root_process->m_module.streamOutCnt;
@@ -249,17 +274,22 @@ ProcessManager::createProcessesFromInput(int argc, const char **argv)
   processStack.push(root_process);
   while (!processStack.empty() && idx < lastNonOutputIdx)
     {
+      Debug(PROCESS, "%s", getStackAsString(processStack));
       currentProcess = processStack.top();
-      Cdo_Debug(CdoDebug::PROCESS, "iteration ", idx, ", current argv: ", argv[idx],
-                ",  currentProcess: ", currentProcess->m_operatorCommand);
+      Debug(PROCESS, "iteration %d, current argv:%s ,currentProcess: ", idx, argv[idx], currentProcess->m_operatorCommand);
 
       argvEntry = argv[idx];
       //------------------------------------------------------
       // case: operator
       if (argvEntry[0] == '-')
         {
-          Cdo_Debug(CdoDebug::PROCESS, "Found new Operator: ", argvEntry);
+          Debug(PROCESS, "Found new Operator: %s", argvEntry);
           currentProcess = addProcess(currentProcess, argvEntry);
+          if (currentProcess->m_module.restrictions == OnlyFirst)
+            {
+              parse_err_msg += "Operator " + std::string(argvEntry) + " can only be used if it is first in the operator chain";
+              return ParseStatus::OperatorNotFirst;
+            }
           processStack.push(currentProcess);
         }
       // - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -283,7 +313,7 @@ ProcessManager::createProcessesFromInput(int argc, const char **argv)
       // case: file
       else if (currentProcess->m_module.streamInCnt != 0)
         {
-          Cdo_Debug(CdoDebug::PROCESS, "adding in file to ", currentProcess->operatorName);
+          Debug(PROCESS, "adding in file to %s", currentProcess->operatorName);
           currentProcess->addFileInStream(argvEntry);
         }
       // -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
@@ -291,7 +321,7 @@ ProcessManager::createProcessesFromInput(int argc, const char **argv)
       while (!processStack.empty() && processStack.top()->hasAllInputs()
              && bracketOperators.find(processStack.top()) == bracketOperators.end())
         {
-          Cdo_Debug(CdoDebug::PROCESS, "Removing ", processStack.top()->operatorName, " from stack");
+          Debug(PROCESS, "Removing %s from stack", processStack.top()->operatorName);
           processStack.pop();
         }
       //------------------------------------------------------
@@ -315,9 +345,10 @@ ProcessManager::createProcessesFromInput(int argc, const char **argv)
       return ParseStatus::UnprocessedInput;
     }
 
-  Cdo_Debug(CdoDebug::PROCESS, "== Process Creation End ==");
+  Debug(PROCESS, "== Process Creation End ==");
 
   setProcessNum(m_processes.size());
+  FileStream::enableTimers(m_processes.size() == 1 && Threading::ompNumThreads == 1);
   return ParseStatus::Ok;
 }
 
@@ -328,25 +359,32 @@ ProcessManager::handleParseError(ParseStatus p_errCode)
     {
     case ParseStatus::ClosingBracketMissing:
       {
-        CdoError::Abort(Cdo::progname, "Missing ']'.");
+        cdoAbort("Missing ']'.");
         break;
       }
     case ParseStatus::OpenBracketMissing:
       {
-        CdoError::Abort(Cdo::progname, "Missing '['.");
+        cdoAbort("Missing '['.");
         break;
       }
     case ParseStatus::UnprocessedInput:
       {
-        CdoError::Abort(Cdo::progname, "Unprocessed Input, could not process all Operators/Files");
+        cdoAbort("Unprocessed Input, could not process all Operators/Files");
         break;
       }
     case ParseStatus::MissingOutFile:
       {
-        CdoError::Abort(Cdo::progname, "Missing out file for first operator");
+        cdoAbort("Missing out file for first operator");
         break;
       }
-    case ParseStatus::Ok: { return;
+    case ParseStatus::OperatorNotFirst:
+      {
+        cdoAbort("%s", parse_err_msg);
+        break;
+      }
+    case ParseStatus::Ok:
+      {
+        return;
       }
     }
 }
@@ -358,24 +396,8 @@ ProcessManager::getProcess(int p_processID)
   auto process = m_processes.find(p_processID);
   if (process == m_processes.end())
     {
-      ERROR("Process with ID: ", p_processID, " not found");
+      cdoAbort("Process with ID: %d not found", p_processID);
     }
   pthread_mutex_unlock(&processMutex);
   return process->second;
 }
-
-Process &
-ProcessManager::getProcessByThreadID(pthread_t p_threadID)
-{
-  for (auto &id_process_pair : m_processes)
-    {
-      if (id_process_pair.second.m_isActive)
-        {
-          if (pthread_equal(id_process_pair.second.threadID, p_threadID))
-            {
-              return id_process_pair.second;
-            }
-        }
-    }
-  CdoError::Abort(Cdo::progname, "could not find process for thread id: ", p_threadID);
-}
