@@ -24,61 +24,54 @@
 
 #include "varlist.h"
 #include "pipe.h"
+#include "cdoStream.h"
 
 #include <sys/types.h> /* off_t */
 #include <memory>
 #include <vector>
 
-extern int pthreadScope;
 
-class Pstream
+class Pstream : public CdoStream
 {
 public:
-  Pstream(int id);
+  Pstream(int p_processID);
+  Pstream(std::string p_filename);
+
+  int openRead();
+  int openWrite(int p_filetype);
+  int openAppend();
+
   int inqVlist();
-  int inqFileType();
-  int inqTimestep(int tsID);
-  void inqRecord(int *varID, int *levelID);
-  bool isPipe();
-  int inqByteorder();
-  size_t getNvals();
-
-  int pstreamOpenReadPipe();
-  int pstreamOpenWritePipe(const char *filename, int filetype);
-  int pstreamOpenWriteFile(int filetype);
-  void pstreamOpenReadFile(const char *filename);
-  void openAppend(const char *p_filename);
-
-  void writeRecord(double *data, size_t nmiss);
-  void writeRecordF(float *data, size_t nmiss);
-
-  void readRecord(double *data, size_t *nmiss);
-  void readRecordF(float *data, size_t *nmiss);
-  void copyRecord(Pstream *dest);
-
-  void defVarList(int vlistID);
-  void defTimestep(int p_tsID);
   void defVlist(int p_vlistID);
+
+  void inqRecord(int *varID, int *levelID);
   void defRecord(int varID, int levelID);
 
-  void init();
+  void readRecord(double *data, size_t *nmiss);
+  void readRecord(float *data, size_t *nmiss);
+  void writeRecord(double *data, size_t nmiss);
+  void writeRecord(float *data, size_t nmiss);
+  void copyRecord(CdoStreamID dest);
+
+  int inqTimestep(int tsID);
+  void defTimestep(int p_tsID);
+
+  int inqFileType();
+  int inqByteorder();
+
   void close();
+
+  size_t getNvals();
+
+  bool isPipe();
+  void init();
   void waitForPipe();
   void closePipe();
+  int getFileID();
 
-  int self;  // aka the id of the pstream
-  std::pair<int, int> m_id;
   int m_mode;
-  int m_fileID;
-  int m_vlistID;
-  int tsID;
-  int m_filetype;
-  int tsID0;
-  int m_varID; /* next varID defined with streamDefVar */
-  bool ispipe;
   bool isopen;
   std::string m_name;
-  std::vector<std::string> m_mfnames;
   std::vector<varlist_t> m_varlist;
 #ifdef HAVE_LIBPTHREAD
   std::shared_ptr<pipe_t> pipe;
@@ -86,20 +79,19 @@ public:
   pthread_t wthreadID; /* write thread ID */
 #endif
 private:
-  Pstream();
-  void checkDatarange(int varID, double *array, size_t nmiss);
+  int openReadPipe();
+  void openReadFile();
+  int openWritePipe(int filetype);
+  int openWriteFile(int filetype);
+  Pstream(){}
+  ~Pstream() {};
 };
 
-Pstream *pstreamToPointer(int pstreamID);
+CdoStreamID pstreamToPointer(int pstreamID);
+void closeAllStreams();
 
 void cdoVlistCopyFlag(int vlistID2, int vlistID1);
 
-Pstream *create_pstream();
-Pstream *create_pstream(std::vector<std::string> p_filenameList);
-Pstream *create_pstream(std::string p_filename);
-Pstream *create_pstream(int processID, int pstreamIDX);
-
-void pstreamCloseAll();
 void setProcessNum(int p_num);
 
 #endif /* PSTREAM_H */
