@@ -21,10 +21,10 @@
 #include <stdio.h>
 
 #ifdef HAVE_LIBPTHREAD
+
 #include <pthread.h>
 #include <errno.h>
-#include "error.h"
-#include "cdo_debugOutput.h"
+#include "cdo_output.h"
 #include <condition_variable>
 
 #define POUT2(caller, x, a, b) pout2(caller, #x, x, #a, a, #b, b)
@@ -137,21 +137,20 @@ Pthread_create(const char *caller, pthread_t *th, pthread_attr_t *attr, void *(*
 {
   int status;
 
-  if (CdoDebug::PTHREAD) Message("%s", caller);
+  Debug(PTHREAD,"%s", caller);
 
-  if (CdoDebug::PTHREAD)
+  if (PTHREAD)
     {
-      Message("%s attributes:", caller);
+      cdoPrint("%s attributes:", caller);
       if (attr)
         print_pthread_attr(__func__, attr);
       else
-        Message("  default attributes");
+        cdoPrint("  default attributes");
     }
 
   status = pthread_create(th, attr, start_routine, arg);
 
-  // if ( CdoDebug::PTHREAD ) Message("-%s (thID = %ld, status = %d)", caller,
-  // (long) *th, status);
+  // Debug(PTHREAD,"-%s (thID = %ld, status = %d)", caller, (long) *th, status);
 
   return status;
 }
@@ -162,12 +161,11 @@ Pthread_join(const char *caller, pthread_t th, void **thread_return)
   int status;
   (void) caller;
 
-  //  if ( CdoDebug::PTHREAD ) Message("+%s (thID = %ld)", caller, (void *) th);
+  // Debug(PTHREAD,"+%s (thID = %ld)", caller, (void *) th);
 
   status = pthread_join(th, thread_return);
 
-  // if ( CdoDebug::PTHREAD ) Message("-%s (thID = %ld, status = %d)", caller,
-  // (void *) th, status);
+  // Debug(PTHREAD,"-%s (thID = %ld, status = %d)", caller, (void *) th, status);
 
   return status;
 }
@@ -175,16 +173,16 @@ Pthread_join(const char *caller, pthread_t th, void **thread_return)
 void
 Pthread_mutex_lock(const char *caller, pthread_mutex_t *mutex)
 {
-  if (CdoDebug::PTHREAD) Message("%s (mutex = %p)", caller, (void *) mutex);
+  Debug(PTHREAD,"%s (mutex = %p)", caller, (void *) mutex);
 
-  int status = pthread_mutex_lock(mutex);
+  const int status = pthread_mutex_lock(mutex);
   if (status != 0)
     {
       switch (status)
         {
-        case EINVAL: Error("The mutex has not been properly initialized!"); break;
-        case EDEADLK: Error("The mutex is already locked by the calling thread!"); break;
-        default: Error("Status %d unknown!", status, (void *) mutex); break;
+        case EINVAL: cdoAbort("The mutex has not been properly initialized!"); break;
+        case EDEADLK: cdoAbort("The mutex is already locked by the calling thread!"); break;
+        default: cdoAbort("Status %d unknown!", status, (void *) mutex); break;
         }
     }
 }
@@ -204,16 +202,16 @@ Pthread_mutex_lock(const char *caller, std::mutex &p_mutex)
 void
 Pthread_mutex_unlock(const char *caller, pthread_mutex_t *mutex)
 {
-  if (CdoDebug::PTHREAD) Message("%s (mutex = %p)", caller, (void *) mutex);
+  Debug(PTHREAD,"%s (mutex = %p)", caller, (void *) mutex);
 
-  int status = pthread_mutex_unlock(mutex);
+  const int status = pthread_mutex_unlock(mutex);
   if (status != 0)
     {
       switch (status)
         {
-        case EINVAL: Error("The mutex has not been properly initialized!"); break;
-        case EPERM: Error("The calling thread does not own the mutex!"); break;
-        default: Error("Status %d unknown!", status); break;
+        case EINVAL: cdoAbort("The mutex has not been properly initialized!"); break;
+        case EPERM: cdoAbort("The calling thread does not own the mutex!"); break;
+        default: cdoAbort("Status %d unknown!", status); break;
         }
     }
 }
@@ -233,37 +231,38 @@ Pthread_mutex_unlock(const char *caller, std::mutex &p_mutex)
 void
 Pthread_cond_signal(const char *caller, std::condition_variable &p_cond_var)
 {
-  if (CdoDebug::PTHREAD) Message("+%s (cond = %p)", caller, (void *) &p_cond_var);
+  Debug(PTHREAD,"+%s (cond = %p)", caller, (void *) &p_cond_var);
   p_cond_var.notify_all();
-  if (CdoDebug::PTHREAD) Message("-%s (cond = %p)", caller, (void *) &p_cond_var);
+  Debug(PTHREAD,"-%s (cond = %p)", caller, (void *) &p_cond_var);
 }
 void
 Pthread_cond_signal(const char *caller, pthread_cond_t *cond)
 {
-  if (CdoDebug::PTHREAD) Message("+%s (cond = %p)", caller, (void *) cond);
+  Debug(PTHREAD,"+%s (cond = %p)", caller, (void *) cond);
 
   pthread_cond_signal(cond);
 
-  if (CdoDebug::PTHREAD) Message("-%s (cond = %p)", caller, (void *) cond);
+  Debug(PTHREAD,"-%s (cond = %p)", caller, (void *) cond);
 }
 
 void
 Pthread_cond_wait(const char *caller, std::condition_variable &p_cond, std::unique_lock<std::mutex> &p_locked_mutex)
 {
-  if (CdoDebug::PTHREAD) std::cout << caller << "waiting for condition " << &p_cond << std::endl;
+  if (PTHREAD) std::cout << caller << "waiting for condition " << &p_cond << std::endl;
+
   p_cond.wait(p_locked_mutex);
 
-  if (CdoDebug::PTHREAD) std::cout << caller << "finished waiting " << &p_cond << std::endl;
+  if (PTHREAD) std::cout << caller << "finished waiting " << &p_cond << std::endl;
 }
 
 void
 Pthread_cond_wait(const char *caller, pthread_cond_t *cond, pthread_mutex_t *mutex)
 {
-  if (CdoDebug::PTHREAD) Message("+%s (cond = %p, mutex =  %p)", caller, (void *) cond, (void *) mutex);
+  Debug(PTHREAD,"+%s (cond = %p, mutex =  %p)", caller, (void *) cond, (void *) mutex);
 
   pthread_cond_wait(cond, mutex);
 
-  if (CdoDebug::PTHREAD) Message("-%s (cond = %p, mutex = %p)", caller, (void *) cond, (void *) mutex);
+  Debug(PTHREAD,"-%s (cond = %p, mutex = %p)", caller, (void *) cond, (void *) mutex);
 }
 
 #endif
