@@ -20,7 +20,7 @@
 #include <stdint.h>
 #include <cmath>
 #include "remap_vars.h"
-#include "grid_cell_search.h"
+#include "remap_grid_cell_search.h"
 #include "grid_point_search.h"
 
 #ifndef M_PI
@@ -44,7 +44,13 @@ constexpr float PIH_f = PIH;
 
 #define TINY 1.e-14
 
-enum struct RemapGridType {Undefined, Reg2D, Curve2D, Unstruct};
+enum struct RemapGridType
+{
+  Undefined,
+  Reg2D,
+  Curve2D,
+  Unstruct
+};
 
 #define REMAP_GRID_BASIS_SRC 1
 #define REMAP_GRID_BASIS_TGT 2
@@ -151,19 +157,17 @@ void remapDistwgtWeights(size_t numNeighbors, RemapSearch &rsearch, RemapVars &r
 void remapConservWeights(RemapSearch &rsearch, RemapVars &rv);
 void remapConservWeightsScrip(RemapSearch &rsearch, RemapVars &rv);
 
-void remapBilinear(RemapSearch &rsearch, const double * src_array, double * tgt_array, double missval);
-void remapBicubic(RemapSearch &rsearch, const double * src_array, double * tgt_array, double missval);
-void remapDistwgt(size_t numNeighbors, RemapSearch &rsearch, const double * src_array, double * tgt_array,
-                  double missval);
-void remapConserv(RemapSearch &rsearch, const double * src_array, double * tgt_array, double missval);
+void remapBilinear(RemapSearch &rsearch, const double *src_array, double *tgt_array, double missval);
+void remapBicubic(RemapSearch &rsearch, const double *src_array, double *tgt_array, double missval);
+void remapDistwgt(size_t numNeighbors, RemapSearch &rsearch, const double *src_array, double *tgt_array, double missval);
+void remapConserv(RemapSearch &rsearch, const double *src_array, double *tgt_array, double missval);
 
-void remapStat(int remapOrder, RemapGrid &src_grid, RemapGrid &tgt_grid, RemapVars &rv, const double * array1,
-               const double * array2, double missval);
-void remapGradients(RemapGrid &grid, const double * array, RemapGradients &gradients);
+void remapStat(int remapOrder, RemapGrid &src_grid, RemapGrid &tgt_grid, RemapVars &rv, const double *array1, const double *array2,
+               double missval);
+void remapGradients(RemapGrid &grid, const double *array, RemapGradients &gradients);
 
-void sort_add(size_t num_links, size_t num_wts, size_t * add1, size_t * add2, double * weights);
-void sort_iter(size_t num_links, size_t num_wts, size_t * add1, size_t * add2, double * weights,
-               int parent);
+void sort_add(size_t num_links, size_t num_wts, size_t *add1, size_t *add2, double *weights);
+void sort_iter(size_t num_links, size_t num_wts, size_t *add1, size_t *add2, double *weights, int parent);
 
 void remapWriteDataScrip(const char *interp_file, RemapMethod mapType, SubmapType submapType, int numNeighbors, int remapOrder,
                          RemapGrid &src_grid, RemapGrid &tgt_grid, RemapVars &rv);
@@ -174,24 +178,22 @@ void calcLatBins(GridSearchBins &searchBins);
 size_t get_srch_cells(size_t tgt_cell_addr, GridSearchBins &tgtBins, GridSearchBins &srcBins, float *tgt_cell_bound_box,
                       size_t *srch_add);
 
-int gridSearchSquareReg2dNN(size_t nx, size_t ny, size_t * nbr_add, double * nbr_dist, double plat, double plon,
-                            const double * src_center_lat, const double * src_center_lon);
+int gridSearchSquareReg2dNN(size_t nx, size_t ny, size_t *nbr_add, double *nbr_dist, double plat, double plon,
+                            const double *src_center_lat, const double *src_center_lon);
 
-int gridSearchSquareReg2d(RemapGrid *src_grid, size_t * src_add, double * src_lats, double * src_lons,
-                          double plat, double plon);
+int gridSearchSquareReg2d(RemapGrid *src_grid, size_t *src_add, double *src_lats, double *src_lons, double plat, double plon);
 
 bool pointInQuad(bool isCyclic, size_t nx, size_t ny, size_t i, size_t j, size_t adds[4], double lons[4], double lats[4],
-                 double plon, double plat, const double * centerLon, const double * centerLat);
+                 double plon, double plat, const double *centerLon, const double *centerLat);
 
-int gridSearchSquareCurv2dScrip(RemapGrid *src_grid, size_t * src_add, double * src_lats,
-                                double * src_lons, double plat, double plon, GridSearchBins &srcBins);
+int gridSearchSquareCurv2dScrip(RemapGrid *src_grid, size_t *src_add, double *src_lats, double *src_lons, double plat, double plon,
+                                GridSearchBins &srcBins);
 
-bool remapFindWeights(double plon, double plat, double * src_lons, double * src_lats, double *ig, double *jg);
-int rect_grid_search(size_t *ii, size_t *jj, double x, double y, size_t nxm, size_t nym, const double * xm,
-                     const double * ym);
+bool remapFindWeights(double plon, double plat, double *src_lons, double *src_lats, double *ig, double *jg);
+int rect_grid_search(size_t *ii, size_t *jj, double x, double y, size_t nxm, size_t nym, const double *xm, const double *ym);
 
 void remapgrid_get_lonlat(RemapGrid *grid, size_t cell_add, double *plon, double *plat);
 
-void remapCheckArea(size_t grid_size, double * cell_area, const char *name);
+void remapCheckArea(size_t grid_size, double *cell_area, const char *name);
 
 #endif /* REMAP_H */
