@@ -17,7 +17,7 @@
 
 #include "cdo_int.h"
 #include "cdo_wtime.h"
-#include "grid.h"
+#include <mpim_grid.h>
 #include "cdo_options.h"
 #include "remap.h"
 #include "remap_store_link.h"
@@ -118,7 +118,7 @@ remapBicubicWeights(RemapSearch &rsearch, RemapVars &rv)
 
   double findex = 0;
 
-// Loop over destination grid
+  // Loop over destination grid
 
 #ifdef _OPENMP
 #pragma omp parallel for default(none) shared(findex, rsearch, weightLinks, tgt_grid_size, src_grid, tgt_grid, rv)
@@ -225,11 +225,11 @@ remapBicubic(RemapSearch &rsearch, const double *restrict src_array, double *res
 
   double findex = 0;
 
-// Loop over destination grid
+  // Loop over destination grid
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(findex, rsearch, tgt_grid_size, src_grid, tgt_grid, src_array, tgt_array, missval, \
-                                              gradients)
+#pragma omp parallel for default(none) \
+    shared(findex, rsearch, tgt_grid_size, src_grid, tgt_grid, src_array, tgt_array, missval, gradients)
 #endif
   for (size_t tgt_cell_add = 0; tgt_cell_add < tgt_grid_size; ++tgt_cell_add)
     {
