@@ -179,7 +179,7 @@ remapBilinearWeights(RemapSearch &rsearch, RemapVars &rv)
 
   double findex = 0;
 
-// Loop over destination grid
+  // Loop over destination grid
 
 #ifdef _OPENMP
 #pragma omp parallel for default(none) schedule(static) shared(findex, rsearch, weightLinks, tgt_grid_size, src_grid, tgt_grid, rv)
@@ -283,11 +283,11 @@ remapBilinear(RemapSearch &rsearch, const double *restrict src_array, double *re
 
   double findex = 0;
 
-// Loop over destination grid
+  // Loop over destination grid
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) schedule(static) shared(findex, rsearch, tgt_grid_size, src_grid, tgt_grid, src_array, \
-                                                               tgt_array, missval)
+#pragma omp parallel for default(none) schedule(static) \
+    shared(findex, rsearch, tgt_grid_size, src_grid, tgt_grid, src_array, tgt_array, missval)
 #endif
   for (size_t tgt_cell_add = 0; tgt_cell_add < tgt_grid_size; ++tgt_cell_add)
     {
