@@ -16,7 +16,7 @@
 */
 
 #include "cdo_int.h"
-#include "grid.h"
+#include <mpim_grid.h>
 #include "remap.h"
 
 int rect_grid_search2(long *imin, long *imax, double xmin, double xmax, long nxm, const double *restrict xm);
