@@ -22,11 +22,12 @@
 #include "cdo_options.h"
 #include "progress.h"
 #include "cimdOmp.h"
+#include <mpim_grid.h>
 
-extern "C" {
+extern "C"
+{
 #include "lib/yac/clipping.h"
 #include "lib/yac/area.h"
-#include "lib/yac/geometry.h"
 }
 
 struct search_t
@@ -64,7 +65,7 @@ cellSearchRealloc(size_t num_srch_cells, search_t &search)
           search.src_grid_cells[n].edge_type = search.src_edge_type;
           search.src_grid_cells[n].coordinates_x = new double[search.srch_corners];
           search.src_grid_cells[n].coordinates_y = new double[search.srch_corners];
-          search.src_grid_cells[n].coordinates_xyz = new double[3 * search.srch_corners];
+          search.src_grid_cells[n].coordinates_xyz = new double[search.srch_corners][3];
         }
 
       search.max_srch_cells = num_srch_cells;
@@ -182,7 +183,7 @@ cdo_compute_concave_overlap_areas(size_t N, search_t &search, const grid_cell &t
 
   double coordinates_x[3] = { -1, -1, -1 };
   double coordinates_y[3] = { -1, -1, -1 };
-  double coordinates_xyz[9] = { -1, -1, -1 };
+  double coordinates_xyz[3][3] = { {-1, -1, -1}, {-1, -1, -1}, {-1, -1, -1} };
   enum yac_edge_type edge_types[3] = { GREAT_CIRCLE, GREAT_CIRCLE, GREAT_CIRCLE };
 
   grid_cell target_partial_cell;
@@ -200,12 +201,12 @@ cdo_compute_concave_overlap_areas(size_t N, search_t &search, const grid_cell &t
   target_partial_cell.coordinates_x[0] = target_node_x;
   target_partial_cell.coordinates_y[0] = target_node_y;
 
-  LLtoXYZ(target_node_x, target_node_y, target_partial_cell.coordinates_xyz);
+  gcLLtoXYZ(target_node_x, target_node_y, target_partial_cell.coordinates_xyz[0]);
 
   for (unsigned num_corners = 0; num_corners < target_cell.num_corners; ++num_corners)
     {
-      unsigned corner_a = num_corners;
-      unsigned corner_b = (num_corners + 1) % target_cell.num_corners;
+      const unsigned corner_a = num_corners;
+      const unsigned corner_b = (num_corners + 1) % target_cell.num_corners;
 
       // skip clipping and area calculation for degenerated triangles
       //
@@ -220,18 +221,17 @@ cdo_compute_concave_overlap_areas(size_t N, search_t &search, const grid_cell &t
       //
       // and use temp_target_cell for triangulation.
       //
-      // Compared to the if statement below the alternative seems
-      // to be quite costly.
+      // Compared to the if statement below the alternative seems to be quite costly.
 
-      if (((std::fabs(target_cell.coordinates_xyz[0 + 3 * corner_a] - target_cell.coordinates_xyz[0 + 3 * corner_b]) < tol)
-           && (std::fabs(target_cell.coordinates_xyz[1 + 3 * corner_a] - target_cell.coordinates_xyz[1 + 3 * corner_b]) < tol)
-           && (std::fabs(target_cell.coordinates_xyz[2 + 3 * corner_a] - target_cell.coordinates_xyz[2 + 3 * corner_b]) < tol))
-          || ((std::fabs(target_cell.coordinates_xyz[0 + 3 * corner_a] - target_partial_cell.coordinates_xyz[0]) < tol)
-              && (std::fabs(target_cell.coordinates_xyz[1 + 3 * corner_a] - target_partial_cell.coordinates_xyz[1]) < tol)
-              && (std::fabs(target_cell.coordinates_xyz[2 + 3 * corner_a] - target_partial_cell.coordinates_xyz[2]) < tol))
-          || ((std::fabs(target_cell.coordinates_xyz[0 + 3 * corner_b] - target_partial_cell.coordinates_xyz[0]) < tol)
-              && (std::fabs(target_cell.coordinates_xyz[1 + 3 * corner_b] - target_partial_cell.coordinates_xyz[1]) < tol)
-              && (std::fabs(target_cell.coordinates_xyz[2 + 3 * corner_b] - target_partial_cell.coordinates_xyz[2]) < tol)))
+      if (((std::fabs(target_cell.coordinates_xyz[corner_a][0] - target_cell.coordinates_xyz[corner_b][0]) < tol)
+           && (std::fabs(target_cell.coordinates_xyz[corner_a][1] - target_cell.coordinates_xyz[corner_b][1]) < tol)
+           && (std::fabs(target_cell.coordinates_xyz[corner_a][2] - target_cell.coordinates_xyz[corner_b][2]) < tol))
+          || ((std::fabs(target_cell.coordinates_xyz[corner_a][0] - target_partial_cell.coordinates_xyz[0][0]) < tol)
+              && (std::fabs(target_cell.coordinates_xyz[corner_a][1] - target_partial_cell.coordinates_xyz[0][1]) < tol)
+              && (std::fabs(target_cell.coordinates_xyz[corner_a][2] - target_partial_cell.coordinates_xyz[0][2]) < tol))
+          || ((std::fabs(target_cell.coordinates_xyz[corner_b][0] - target_partial_cell.coordinates_xyz[0][0]) < tol)
+              && (std::fabs(target_cell.coordinates_xyz[corner_b][1] - target_partial_cell.coordinates_xyz[0][1]) < tol)
+              && (std::fabs(target_cell.coordinates_xyz[corner_b][2] - target_partial_cell.coordinates_xyz[0][2]) < tol)))
         continue;
 
       target_partial_cell.coordinates_x[1] = target_cell.coordinates_x[corner_a];
@@ -239,12 +239,12 @@ cdo_compute_concave_overlap_areas(size_t N, search_t &search, const grid_cell &t
       target_partial_cell.coordinates_x[2] = target_cell.coordinates_x[corner_b];
       target_partial_cell.coordinates_y[2] = target_cell.coordinates_y[corner_b];
 
-      target_partial_cell.coordinates_xyz[0 + 3 * 1] = target_cell.coordinates_xyz[0 + 3 * corner_a];
-      target_partial_cell.coordinates_xyz[1 + 3 * 1] = target_cell.coordinates_xyz[1 + 3 * corner_a];
-      target_partial_cell.coordinates_xyz[2 + 3 * 1] = target_cell.coordinates_xyz[2 + 3 * corner_a];
-      target_partial_cell.coordinates_xyz[0 + 3 * 2] = target_cell.coordinates_xyz[0 + 3 * corner_b];
-      target_partial_cell.coordinates_xyz[1 + 3 * 2] = target_cell.coordinates_xyz[1 + 3 * corner_b];
-      target_partial_cell.coordinates_xyz[2 + 3 * 2] = target_cell.coordinates_xyz[2 + 3 * corner_b];
+      target_partial_cell.coordinates_xyz[1][0] = target_cell.coordinates_xyz[corner_a][0];
+      target_partial_cell.coordinates_xyz[1][1] = target_cell.coordinates_xyz[corner_a][1];
+      target_partial_cell.coordinates_xyz[1][2] = target_cell.coordinates_xyz[corner_a][2];
+      target_partial_cell.coordinates_xyz[2][0] = target_cell.coordinates_xyz[corner_b][0];
+      target_partial_cell.coordinates_xyz[2][1] = target_cell.coordinates_xyz[corner_b][1];
+      target_partial_cell.coordinates_xyz[2][2] = target_cell.coordinates_xyz[corner_b][2];
 
       yac_cell_clipping(N, source_cell, target_partial_cell, overlap_cells);
 
@@ -355,7 +355,8 @@ remapNormalizeWeights(RemapGrid *tgt_grid, RemapVars &rv)
 }
 
 static void
-setYacCoordinates(RemapGridType remapGridType, const size_t cellAddr, const size_t numCellCorners, RemapGrid *remap_grid, grid_cell &yac_grid_cell)
+setYacCoordinates(RemapGridType remapGridType, const size_t cellAddr, const size_t numCellCorners, RemapGrid *remap_grid,
+                  grid_cell &yac_grid_cell)
 {
   double *restrict x = yac_grid_cell.coordinates_x;
   double *restrict y = yac_grid_cell.coordinates_y;
@@ -387,15 +388,15 @@ setYacCoordinates(RemapGridType remapGridType, const size_t cellAddr, const size
         }
     }
 
-  double *restrict xyz = yac_grid_cell.coordinates_xyz;
-  for (size_t ic = 0; ic < numCellCorners; ++ic) LLtoXYZ(x[ic], y[ic], xyz + ic * 3);
+  double(*restrict xyz)[3] = yac_grid_cell.coordinates_xyz;
+  for (size_t ic = 0; ic < numCellCorners; ++ic) gcLLtoXYZ(x[ic], y[ic], xyz[ic]);
 }
 
 static void
 reg2d_bound_box(RemapGrid *remap_grid, double *grid_bound_box)
 {
-  size_t nx = remap_grid->dims[0];
-  size_t ny = remap_grid->dims[1];
+  const size_t nx = remap_grid->dims[0];
+  const size_t ny = remap_grid->dims[1];
   const double *reg2d_corner_lon = remap_grid->reg2d_corner_lon;
   const double *reg2d_corner_lat = remap_grid->reg2d_corner_lat;
 
@@ -428,7 +429,7 @@ yacGridCellInit(grid_cell &cell, size_t numCellCorners, enum yac_edge_type *edge
   cell.edge_type = edgeType;
   cell.coordinates_x = new double[numCellCorners];
   cell.coordinates_y = new double[numCellCorners];
-  cell.coordinates_xyz = new double[3 * numCellCorners];
+  cell.coordinates_xyz = new double[numCellCorners][3];
 }
 
 static void
@@ -525,12 +526,12 @@ remapConservWeights(RemapSearch &rsearch, RemapVars &rv)
   extern CellSearchMethod cellSearchMethod;
   bool useCellsearch = cellSearchMethod == CellSearchMethod::spherepart || srcGridType == RemapGridType::Reg2D;
 
-// Loop over destination grid
+  // Loop over destination grid
 
 #ifdef _OPENMP
-#pragma omp parallel for schedule(dynamic) default(none)                                                                        \
-    shared(findex, rsearch, srcGridType, tgtGridType, src_grid_bound_box, rv, Options::cdoVerbose, tgt_num_cell_corners, \
-           target_cell_type, weightLinks, srch_corners, src_grid, tgt_grid, tgt_grid_size, src_grid_size, search, srch_add,     \
+#pragma omp parallel for schedule(dynamic) default(none)                                                                    \
+    shared(findex, rsearch, srcGridType, tgtGridType, src_grid_bound_box, rv, Options::cdoVerbose, tgt_num_cell_corners,    \
+           target_cell_type, weightLinks, srch_corners, src_grid, tgt_grid, tgt_grid_size, src_grid_size, search, srch_add, \
            tgt_grid_cell, num_srch_cells_stat, useCellsearch)
 #endif
   for (size_t tgt_cell_add = 0; tgt_cell_add < tgt_grid_size; ++tgt_cell_add)
@@ -559,8 +560,8 @@ remapConservWeights(RemapSearch &rsearch, RemapVars &rv)
         {
           // num_srch_cells = remapSearchCells(rsearch, target_cell_type == LON_LAT_CELL, tgt_grid_cell[ompthID],
           // srch_add[ompthID].data());
-          num_srch_cells = remapSearchCells(rsearch, tgtGridType == RemapGridType::Reg2D, tgt_grid_cell[ompthID],
-                                            srch_add[ompthID].data());
+          num_srch_cells
+              = remapSearchCells(rsearch, tgtGridType == RemapGridType::Reg2D, tgt_grid_cell[ompthID], srch_add[ompthID].data());
         }
       else
         {
