@@ -53,7 +53,8 @@ remapDistwgtWeights(size_t numNeighbors, RemapSearch &rsearch, RemapVars &rv)
   double findex = 0;
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(findex, rsearch, weightLinks, numNeighbors, src_grid, tgt_grid, tgt_grid_size, knnWeights)
+#pragma omp parallel for default(none) \
+    shared(findex, rsearch, weightLinks, numNeighbors, src_grid, tgt_grid, tgt_grid_size, knnWeights)
 #endif
   for (size_t tgt_cell_add = 0; tgt_cell_add < tgt_grid_size; ++tgt_cell_add)
     {
@@ -122,8 +123,8 @@ remapDistwgt(size_t numNeighbors, RemapSearch &rsearch, const double *restrict s
   double findex = 0;
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(findex, rsearch, numNeighbors, src_grid, tgt_grid, tgt_grid_size, src_array, \
-                                              tgt_array, missval, knnWeights)
+#pragma omp parallel for default(none) \
+    shared(findex, rsearch, numNeighbors, src_grid, tgt_grid, tgt_grid_size, src_array, tgt_array, missval, knnWeights)
 #endif
   for (size_t tgt_cell_add = 0; tgt_cell_add < tgt_grid_size; ++tgt_cell_add)
     {
@@ -158,15 +159,15 @@ remapDistwgt(size_t numNeighbors, RemapSearch &rsearch, const double *restrict s
 void remapInit(remapType &remap);
 
 void
-intgriddis(Field *field1, Field *field2, size_t numNeighbors)
+intgriddis(Field &field1, Field &field2, size_t numNeighbors)
 {
   RemapMethod mapType = RemapMethod::DISTWGT;
-  int gridID1 = field1->grid;
-  int gridID2 = field2->grid;
-  double src_missval = field1->missval;
-  double tgt_missval = field2->missval;
-  double *src_array = field1->ptr;
-  double *tgt_array = field2->ptr;
+  int gridID1 = field1.grid;
+  int gridID2 = field2.grid;
+  double src_missval = field1.missval;
+  double tgt_missval = field2.missval;
+  double *src_array = field1.vec.data();
+  double *tgt_array = field2.vec.data();
 
   if (Options::cdoVerbose) cdoPrint("Called %s()", __func__);
 
@@ -227,7 +228,7 @@ intgriddis(Field *field1, Field *field2, size_t numNeighbors)
 
   Progress::update(0, 1, 1);
 
-  field2->nmiss = nmiss;
+  field2.nmiss = nmiss;
 
   remapGridFree(remap.src_grid);
   remapGridFree(remap.tgt_grid);
