@@ -15,9 +15,11 @@
   GNU General Public License for more details.
 */
 
+#include <cdi.h>
 #include "cdo_int.h"
 #include "cdo_math.h"
 #include "remap.h"
+#include <mpim_grid.h>
 
 // This routine finds the closest numNeighbor points to a search point and computes a distance to each of the neighbors
 #define MAX_SEARCH_CELLS 25
@@ -111,7 +113,7 @@ gridSearchPointReg2d(GridPointSearch *gps, double plon, double plat, knnWeightsT
 
       double xyz[3];
       double query_pt[3];
-      cdoLLtoXYZ(plon, plat, query_pt);
+      gcLLtoXYZ(plon, plat, query_pt);
       double sqrSearchRadius = cdo::sqr(gps->searchRadius);
 
       for (size_t na = 0; na < num_add; ++na)
@@ -207,6 +209,52 @@ gridSearchPoint(GridPointSearch *gps, double plon, double plat, knnWeightsType &
   knnWeights.checkDistance();
 }
 
+void
+gridSearchPointSmooth(GridPointSearch *gps, double plon, double plat, knnWeightsType &knnWeights)
+{
+  /*
+    Input variables:
+
+      plat: latitude  of the search point
+      plon: longitude of the search point
+
+    Output variables:
+
+      knnWeights.m_addr[numNeighbors]: address of each of the closest points
+      knnWeights.m_dist[numNeighbors]: distance to each of the closest points
+  */
+  size_t numNeighbors = knnWeights.maxNeighbors();
+
+  size_t ndist = numNeighbors;
+  // check some more points if distance is the same use the smaller index (nadd)
+  if (ndist > 8)
+    ndist += 8;
+  else
+    ndist *= 2;
+  if (ndist > gps->n) ndist = gps->n;
+
+  if (knnWeights.m_tmpaddr.empty()) knnWeights.m_tmpaddr.resize(ndist);
+  if (knnWeights.m_tmpdist.empty()) knnWeights.m_tmpdist.resize(ndist);
+  size_t *adds = knnWeights.m_tmpaddr.data();
+  double *dist = knnWeights.m_tmpdist.data();
+
+  size_t nadds = 0;
+  if (numNeighbors == 1)
+    nadds = gridPointSearchNearest(gps, plon, plat, adds, dist);
+  else
+    nadds = gridPointSearchQnearest(gps, plon, plat, ndist, adds, dist);
+
+  ndist = nadds;
+  if (ndist < numNeighbors) numNeighbors = ndist;
+
+  // Initialize distance and address arrays
+  knnWeights.initAddr(numNeighbors);
+  knnWeights.initDist(numNeighbors);
+  for (size_t i = 0; i < ndist; ++i) knnWeights.storeDistance(adds[i], dist[i], numNeighbors);
+
+  knnWeights.checkDistance();
+}
+
 void
 remapSearchPoints(RemapSearch &rsearch, double plon, double plat, knnWeightsType &knnWeights)
 {
