@@ -28,11 +28,10 @@
 #include "cdo_options.h"
 
 #include "cdo_int.h"
-#include "grid.h"
+#include <mpim_grid.h>
 #include "remap.h"
 #include "commandline.h"
 
-
 #ifdef HAVE_LIBNETCDF
 static void
 nce(int istat)
@@ -60,7 +59,7 @@ writeLinks(int nc_file_id, int nc_add_id, nc_type sizetype, size_t num_links, si
 #endif
 }
 
-void
+static void
 readLinks(int nc_file_id, int nc_add_id, size_t num_links, size_t *cell_add)
 {
   if (num_links < 0x7FFFFC00)  // 2GB
@@ -86,34 +85,34 @@ remapWriteDataScrip(const char *interp_file, RemapMethod mapType, SubmapType sub
 
 #ifdef HAVE_LIBNETCDF
 
-  int nc_file_id = -1;           /* id for NetCDF file                       */
-  int nc_srcgrdsize_id = -1;     /* id for source grid size                  */
-  int nc_dstgrdsize_id = -1;     /* id for destination grid size             */
-  int nc_srcgrdcorn_id = 0;      /* id for number of source grid corners     */
-  int nc_dstgrdcorn_id = 0;      /* id for number of dest grid corners       */
-  int nc_srcgrdrank_id = -1;     /* id for source grid rank                  */
-  int nc_dstgrdrank_id = -1;     /* id for dest grid rank                    */
-  int nc_numlinks_id = -1;       /* id for number of links in mapping        */
-  int nc_numwgts_id = -1;        /* id for number of weights for mapping     */
-  int nc_srcgrddims_id = -1;     /* id for source grid dimensions            */
-  int nc_dstgrddims_id = -1;     /* id for dest grid dimensions              */
-  int nc_srcgrdcntrlat_id = -1;  /* id for source grid center latitude       */
-  int nc_dstgrdcntrlat_id = -1;  /* id for dest grid center latitude         */
-  int nc_srcgrdcntrlon_id = -1;  /* id for source grid center longitude      */
-  int nc_dstgrdcntrlon_id = -1;  /* id for dest grid center longitude        */
-  int nc_srcgrdimask_id = -1;    /* id for source grid mask                  */
-  int nc_dstgrdimask_id = -1;    /* id for dest grid mask                    */
-  int nc_srcgrdcrnrlat_id = -1;  /* id for latitude of source grid corners   */
-  int nc_srcgrdcrnrlon_id = -1;  /* id for longitude of source grid corners  */
-  int nc_dstgrdcrnrlat_id = -1;  /* id for latitude of dest grid corners     */
-  int nc_dstgrdcrnrlon_id = -1;  /* id for longitude of dest grid corners    */
-  int nc_srcgrdarea_id = -1;     /* id for area of source grid cells         */
-  int nc_dstgrdarea_id = -1;     /* id for area of dest grid cells           */
-  int nc_srcgrdfrac_id = -1;     /* id for area fraction on source grid      */
-  int nc_dstgrdfrac_id = -1;     /* id for area fraction on dest grid        */
-  int nc_srcadd_id = -1;         /* id for map source address                */
-  int nc_dstadd_id = -1;         /* id for map destination address           */
-  int nc_rmpmatrix_id = -1;      /* id for remapping matrix                  */
+  int nc_file_id = -1;          /* id for NetCDF file                       */
+  int nc_srcgrdsize_id = -1;    /* id for source grid size                  */
+  int nc_dstgrdsize_id = -1;    /* id for destination grid size             */
+  int nc_srcgrdcorn_id = 0;     /* id for number of source grid corners     */
+  int nc_dstgrdcorn_id = 0;     /* id for number of dest grid corners       */
+  int nc_srcgrdrank_id = -1;    /* id for source grid rank                  */
+  int nc_dstgrdrank_id = -1;    /* id for dest grid rank                    */
+  int nc_numlinks_id = -1;      /* id for number of links in mapping        */
+  int nc_numwgts_id = -1;       /* id for number of weights for mapping     */
+  int nc_srcgrddims_id = -1;    /* id for source grid dimensions            */
+  int nc_dstgrddims_id = -1;    /* id for dest grid dimensions              */
+  int nc_srcgrdcntrlat_id = -1; /* id for source grid center latitude       */
+  int nc_dstgrdcntrlat_id = -1; /* id for dest grid center latitude         */
+  int nc_srcgrdcntrlon_id = -1; /* id for source grid center longitude      */
+  int nc_dstgrdcntrlon_id = -1; /* id for dest grid center longitude        */
+  int nc_srcgrdimask_id = -1;   /* id for source grid mask                  */
+  int nc_dstgrdimask_id = -1;   /* id for dest grid mask                    */
+  int nc_srcgrdcrnrlat_id = -1; /* id for latitude of source grid corners   */
+  int nc_srcgrdcrnrlon_id = -1; /* id for longitude of source grid corners  */
+  int nc_dstgrdcrnrlat_id = -1; /* id for latitude of dest grid corners     */
+  int nc_dstgrdcrnrlon_id = -1; /* id for longitude of dest grid corners    */
+  int nc_srcgrdarea_id = -1;    /* id for area of source grid cells         */
+  int nc_dstgrdarea_id = -1;    /* id for area of dest grid cells           */
+  int nc_srcgrdfrac_id = -1;    /* id for area fraction on source grid      */
+  int nc_dstgrdfrac_id = -1;    /* id for area fraction on dest grid        */
+  int nc_srcadd_id = -1;        /* id for map source address                */
+  int nc_dstadd_id = -1;        /* id for map destination address           */
+  int nc_rmpmatrix_id = -1;     /* id for remapping matrix                  */
 
   int nc_dims2_id[2]; /* NetCDF ids for 2d array dims             */
 
@@ -459,7 +458,7 @@ getMapType(int nc_file_id, SubmapType *submapType, int *numNeighbors, int &remap
         mapType = RemapMethod::CONSERV_SCRIP;
 
       int iatt;
-      int status = nc_get_att_int(nc_file_id, NC_GLOBAL, "remap_order", &iatt);
+      const int status = nc_get_att_int(nc_file_id, NC_GLOBAL, "remap_order", &iatt);
       if (status == NC_NOERR) remapOrder = iatt;
     }
   else if (cmpstr(map_method, "Bilinear") == 0)
@@ -497,7 +496,7 @@ void
 remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMethod *mapType, SubmapType *submapType,
                    int *numNeighbors, int &remapOrder, RemapGrid &src_grid, RemapGrid &tgt_grid, RemapVars &rv)
 {
- // The routine reads a NetCDF file to extract remapping info in SCRIP format
+  // The routine reads a NetCDF file to extract remapping info in SCRIP format
 
 #ifdef HAVE_LIBNETCDF
 
@@ -532,10 +531,10 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
   int nc_rmpmatrix_id;     /* id for remapping matrix                  */
 
   char map_name[1024];
-  char normalize_opt[64];  /* character string for normalization option */
-  char convention[64];     /* character string for output convention    */
-  char src_grid_name[64];  /* grid name for source grid                 */
-  char tgt_grid_name[64];  /* grid name for dest   grid                 */
+  char normalize_opt[64]; /* character string for normalization option */
+  char convention[64];    /* character string for output convention    */
+  char src_grid_name[64]; /* grid name for source grid                 */
+  char tgt_grid_name[64]; /* grid name for dest   grid                 */
   char src_grid_units[64];
   char tgt_grid_units[64];
   size_t attlen, dimlen;
@@ -621,12 +620,11 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
   remapGridInit(tgt_grid);
 
   // Read dimension information
-
   nce(nc_inq_dimid(nc_file_id, "src_grid_size", &nc_srcgrdsize_id));
   nce(nc_inq_dimlen(nc_file_id, nc_srcgrdsize_id, &dimlen));
   src_grid.size = dimlen;
   /*
-  if (  src_grid.size != gridInqSize(gridID1) )
+  if ( src_grid.size != gridInqSize(gridID1) )
     cdoAbort("Source grids have different size!");
   */
   nce(nc_inq_dimid(nc_file_id, "dst_grid_size", &nc_dstgrdsize_id));
@@ -814,7 +812,6 @@ remapReadDataScrip(const char *interp_file, int gridID1, int gridID2, RemapMetho
     }
 
   // Close input file
-
   nce(nc_close(nc_file_id));
 
 #else
