@@ -17,7 +17,7 @@
 #include "cdo_int.h"
 #include "cdo_math.h"
 #include "cdo_options.h"
-#include "grid.h"
+#include <mpim_grid.h>
 #include "remap.h"
 
 static void
