@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -16,6 +16,7 @@
 */
 
 #include "cdo_int.h"
+#include "cdo_options.h"
 #include "remap.h"
 #include "remap_store_link_cnsrv.h"
 
@@ -39,7 +40,7 @@ grid_store_init(grid_store_t *grid_store, long gridsize)
   grid_store->nblocks = grid_store->max_size / grid_store->blk_size;
   if (grid_store->max_size % grid_store->blk_size > 0) grid_store->nblocks++;
 
-  if (cdoVerbose)
+  if (Options::cdoVerbose)
     fprintf(stdout, "blksize = %ld  lastblksize = %ld  max_size = %ld  nblocks = %ld\n", grid_store->blk_size,
             grid_store->max_size % grid_store->blk_size, grid_store->max_size, grid_store->nblocks);
 
@@ -52,7 +53,7 @@ grid_store_init(grid_store_t *grid_store, long gridsize)
     {
       grid_store->blksize[iblk] = grid_store->blk_size;
       grid_store->nlayers[iblk] = 0;
-      grid_store->layers[iblk] = NULL;
+      grid_store->layers[iblk] = nullptr;
     }
   if (grid_store->max_size % grid_store->blk_size > 0)
     grid_store->blksize[grid_store->nblocks - 1] = grid_store->max_size % grid_store->blk_size;
@@ -71,7 +72,7 @@ grid_store_delete(grid_store_t *grid_store)
       long blksize = grid_store->blksize[iblk];
       for (long ilayer = 0; ilayer < nlayers; ++ilayer)
         {
-          if (cdoVerbose)
+          if (Options::cdoVerbose)
             {
               for (long i = 0; i < blksize; ++i)
                 if (grid_layer->grid2_link[i] != -1) j++;
@@ -83,7 +84,7 @@ grid_store_delete(grid_store_t *grid_store)
           Free(grid_layer_f);
         }
       /*
-      if ( cdoVerbose )
+      if ( Options::cdoVerbose )
         {
           fprintf(stderr, "block = %ld nlayers = %ld  allocated = %ld  used =
       %ld\n", iblk+1, nlayers, nlayers*blksize, j);
@@ -110,8 +111,8 @@ store_link_cnsrv(RemapVars &rv, long add1, long add2, long num_wts, double *weig
     double weights[]  ! array of remapping weights for this link
   */
   /* Local variables */
-  long nlink; /* link index */
-  grid_layer_t *grid_layer = NULL;
+  long nlink = 0; /* link index */
+  grid_layer_t *grid_layer = nullptr;
 
   /*  If all weights are ZERO, do not bother storing the link */
   if (num_wts == 3)
@@ -168,7 +169,7 @@ store_link_cnsrv(RemapVars &rv, long add1, long add2, long num_wts, double *weig
   else
     {
       grid_layer = (grid_layer_t *) Malloc(sizeof(grid_layer_t));
-      grid_layer->next = NULL;
+      grid_layer->next = nullptr;
       grid_layer->grid2_link = (long *) Malloc(grid_store->blksize[iblk] * sizeof(long));
 
       long blksize = grid_store->blksize[iblk];
