@@ -68,10 +68,10 @@
 
 #include <cdi.h>
 #include "cdo_options.h"
-
 #include "cdo_int.h"
 #include "cdo_wtime.h"
-#include "grid.h"
+#include <mpim_grid.h>
+#include "gridreference.h"
 #include "remap.h"
 
 #define IS_REG2D_GRID(gridID) (gridInqType(gridID) == GRID_LONLAT || gridInqType(gridID) == GRID_GAUSSIAN)
@@ -132,8 +132,8 @@ boundboxFromCenter(bool lonIsCyclic, size_t size, size_t nx, size_t ny, const do
   float tmp_lats[4], tmp_lons[4];
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(lonIsCyclic, size, nx, ny, center_lon, center_lat, bound_box) private(idx, tmp_lats, \
-                                                                                                                    tmp_lons)
+#pragma omp parallel for default(none) \
+    shared(lonIsCyclic, size, nx, ny, center_lon, center_lat, bound_box) private(idx, tmp_lats, tmp_lons)
 #endif
   for (size_t n = 0; n < size; n++)
     {
@@ -364,7 +364,7 @@ remapDefineGrid(RemapMethod mapType, int gridID, RemapGrid &grid, const char *tx
 
   remapGridAlloc(mapType, grid);
 
-/* Initialize logical mask */
+  /* Initialize logical mask */
 
 #ifdef _OPENMP
 #pragma omp parallel for default(none) shared(gridsize, grid)
@@ -676,8 +676,7 @@ remapInitGrids(RemapMethod mapType, bool lextrapolate, int gridID1, RemapGrid &s
   if (!remap_gen_weights && IS_REG2D_GRID(gridID2) && tgt_grid.type != RemapGridType::Reg2D)
     {
       if (mapType == RemapMethod::DISTWGT) tgt_grid.type = RemapGridType::Reg2D;
-      if (mapType == RemapMethod::BILINEAR && src_grid.type == RemapGridType::Reg2D)
-        tgt_grid.type = RemapGridType::Reg2D;
+      if (mapType == RemapMethod::BILINEAR && src_grid.type == RemapGridType::Reg2D) tgt_grid.type = RemapGridType::Reg2D;
     }
 
   src_grid.lextrapolate = lextrapolate;
@@ -725,11 +724,12 @@ remapInitGrids(RemapMethod mapType, bool lextrapolate, int gridID1, RemapGrid &s
     }
 
   int sgridID = src_grid.gridID;
-  if (gridInqSize(sgridID) > 1 && ((gridInqType(sgridID) == GRID_PROJECTION && gridInqProjType(sgridID) == CDI_PROJ_LCC)
-                                   || (gridInqType(sgridID) == GRID_PROJECTION && gridInqProjType(sgridID) == CDI_PROJ_STERE)
-                                   || (gridInqType(sgridID) == GRID_PROJECTION && gridInqProjType(sgridID) == CDI_PROJ_RLL)
-                                   || (gridInqType(sgridID) == GRID_PROJECTION && gridInqProjType(sgridID) == CDI_PROJ_LAEA)
-                                   || (gridInqType(sgridID) == GRID_PROJECTION && gridInqProjType(sgridID) == CDI_PROJ_SINU)))
+  if (gridInqSize(sgridID) > 1
+      && ((gridInqType(sgridID) == GRID_PROJECTION && gridInqProjType(sgridID) == CDI_PROJ_LCC)
+          || (gridInqType(sgridID) == GRID_PROJECTION && gridInqProjType(sgridID) == CDI_PROJ_STERE)
+          || (gridInqType(sgridID) == GRID_PROJECTION && gridInqProjType(sgridID) == CDI_PROJ_RLL)
+          || (gridInqType(sgridID) == GRID_PROJECTION && gridInqProjType(sgridID) == CDI_PROJ_LAEA)
+          || (gridInqType(sgridID) == GRID_PROJECTION && gridInqProjType(sgridID) == CDI_PROJ_SINU)))
     {
       bool lbounds = true;
       src_grid.gridID = gridID1 = gridToCurvilinear(src_grid.gridID, lbounds);
@@ -759,10 +759,10 @@ remapStat(int remapOrder, RemapGrid &src_grid, RemapGrid &tgt_grid, RemapVars &r
   cdoPrint("----------------------------------------------");
 
   double mean, minval, maxval;
-  arrayMinMaxMeanMV(src_grid.size, array1, missval, &minval, &maxval, &mean);
+  arrayMinMaxMeanMV(src_grid.size, array1, missval, minval, maxval, mean);
   cdoPrint("  Grid1 min,mean,max: %g %g %g", minval, mean, maxval);
 
-  arrayMinMaxMeanMV(tgt_grid.size, array2, missval, &minval, &maxval, &mean);
+  arrayMinMaxMeanMV(tgt_grid.size, array2, missval, minval, maxval, mean);
   cdoPrint("  Grid2 min,mean,max: %g %g %g", minval, mean, maxval);
 
   // Conservation Test
