@@ -128,12 +128,11 @@ remap_heapsort(const size_t num_links, size_t *restrict add1, size_t *restrict a
       add2_tmp = add2[lvl];
       idx_tmp = idx[lvl];
 
-      /* Loop until proper level is found for this link, or reach bottom */
+      // Loop until proper level is found for this link, or reach bottom
 
       for (i = 0; i < num_links; ++i)
         {
-          /* Find the largest of the two daughters */
-
+          // Find the largest of the two daughters
           chk_lvl1 = 2 * final_lvl + 1;
           max_lvl = 2 * final_lvl + 2;
           if (chk_lvl1 == num_links - 1) max_lvl = chk_lvl1;
@@ -141,10 +140,9 @@ remap_heapsort(const size_t num_links, size_t *restrict add1, size_t *restrict a
           if ((add1[chk_lvl1] > add1[max_lvl]) || ((add1[chk_lvl1] == add1[max_lvl]) && (add2[chk_lvl1] > add2[max_lvl])))
             {
               max_lvl = chk_lvl1;
-            } /*
-                If the parent is greater than both daughters,
-                the correct level has been found
-              */
+            }
+
+          // If the parent is greater than both daughters, the correct level has been found
           if ((add1_tmp > add1[max_lvl]) || ((add1_tmp == add1[max_lvl]) && (add2_tmp > add2[max_lvl])))
             {
               add1[final_lvl] = add1_tmp;
@@ -156,10 +154,9 @@ remap_heapsort(const size_t num_links, size_t *restrict add1, size_t *restrict a
           else
             {
               /*
-                Otherwise, promote the largest daughter and push
-                down one level in the tree.  If haven't reached
-                the end of the tree, repeat the process.  Otherwise
-                store last values and exit the loop
+                Otherwise, promote the largest daughter and push down one level in the tree.
+                If haven't reached the end of the tree, repeat the process.
+                Otherwise store last values and exit the loop
               */
               add1[final_lvl] = add1[max_lvl];
               add2[final_lvl] = add2[max_lvl];
@@ -180,14 +177,10 @@ remap_heapsort(const size_t num_links, size_t *restrict add1, size_t *restrict a
       if (i == num_links) cdoAbort("Internal problem, link 1 not found!");
     }
 
-  /*
-    Now that the heap has been sorted, strip off the top (largest)
-    value and promote the values below
-  */
+  // Now that the heap has been sorted, strip off the top (largest) value and promote the values below
   for (lvl = num_links - 1; lvl >= 2; --lvl)
     {
-      /* Move the top value and insert it into the correct place */
-
+      // Move the top value and insert it into the correct place
       add1_tmp = add1[lvl];
       add1[lvl] = add1[0];
 
@@ -197,15 +190,13 @@ remap_heapsort(const size_t num_links, size_t *restrict add1, size_t *restrict a
       idx_tmp = idx[lvl];
       idx[lvl] = idx[0];
 
-      /* As above this loop sifts the tmp values down until proper level is
-       * reached */
+      // As above this loop sifts the tmp values down until proper level is reached
 
       final_lvl = 0;
 
       for (i = 0; i < num_links; ++i)
         {
-          /* Find the largest of the two daughters */
-
+          // Find the largest of the two daughters
           chk_lvl1 = 2 * final_lvl + 1;
           max_lvl = 2 * final_lvl + 2;
           if (max_lvl >= (size_t) lvl) max_lvl = chk_lvl1;
@@ -214,10 +205,8 @@ remap_heapsort(const size_t num_links, size_t *restrict add1, size_t *restrict a
             {
               max_lvl = chk_lvl1;
             }
-          /*
-            If the parent is greater than both daughters,
-            the correct level has been found
-          */
+
+          // If the parent is greater than both daughters, the correct level has been found
           if ((add1_tmp > add1[max_lvl]) || ((add1_tmp == add1[max_lvl]) && (add2_tmp > add2[max_lvl])))
             {
               add1[final_lvl] = add1_tmp;
@@ -229,10 +218,9 @@ remap_heapsort(const size_t num_links, size_t *restrict add1, size_t *restrict a
           else
             {
               /*
-                Otherwise, promote the largest daughter and push
-                down one level in the tree.  If haven't reached
-                the end of the tree, repeat the process.  Otherwise
-                store last values and exit the loop
+                Otherwise, promote the largest daughter and push down one level in the tree.
+                If haven't reached the end of the tree, repeat the process.
+                Otherwise store last values and exit the loop
               */
               add1[final_lvl] = add1[max_lvl];
               add2[final_lvl] = add2[max_lvl];
@@ -253,19 +241,10 @@ remap_heapsort(const size_t num_links, size_t *restrict add1, size_t *restrict a
       if (i == num_links) cdoAbort("Internal problem, link 2 not found!");
     }
 
-  /* Swap the last two entries */
-
-  add1_tmp = add1[1];
-  add1[1] = add1[0];
-  add1[0] = add1_tmp;
-
-  add2_tmp = add2[1];
-  add2[1] = add2[0];
-  add2[0] = add2_tmp;
-
-  idx_tmp = idx[1];
-  idx[1] = idx[0];
-  idx[0] = idx_tmp;
+  // Swap the last two entries
+  std::swap(add1[0], add1[1]);
+  std::swap(add2[0], add2[1]);
+  std::swap(idx[0], idx[1]);
 }
 
 void
@@ -441,7 +420,7 @@ sort_par(size_t num_links, size_t num_wts, size_t *restrict add1, size_t *restri
   add2s[1] = &add2[add_srt[1]];
   nl[0] = num_links / nsplit;
   nl[1] = num_links - nl[0];
-// add_end[0] = nl[0];              add_end[1] = num_links;
+  // add_end[0] = nl[0];              add_end[1] = num_links;
 
 #ifdef _OPENMP
   int depth = (int) (log(parent) / log(2));
@@ -457,12 +436,12 @@ sort_par(size_t num_links, size_t num_wts, size_t *restrict add1, size_t *restri
     }
 #endif
 
-//  printf("I am %i nl[0] %i nl[1] %i\n",parent,nl[0],nl[1]);
-//  printf("add_srt[0] %i add_Srt[1] %i\n",add_srt[0],add_srt[1]);
-//  if ( 1 )
-//      printf("\n\nSplitting thread into %i!! (I AM %i) depth %i
-//      parallel_depth %i add_srt[0]%i add_srt[1] %i\n",
-//	     nsplit,parent,depth,par_depth,add_srt[0],add_srt[1]);
+    //  printf("I am %i nl[0] %i nl[1] %i\n",parent,nl[0],nl[1]);
+    //  printf("add_srt[0] %i add_Srt[1] %i\n",add_srt[0],add_srt[1]);
+    //  if ( 1 )
+    //      printf("\n\nSplitting thread into %i!! (I AM %i) depth %i
+    //      parallel_depth %i add_srt[0]%i add_srt[1] %i\n",
+    //	     nsplit,parent,depth,par_depth,add_srt[0],add_srt[1]);
 
 #ifdef _OPENMP
 #pragma omp parallel for if (depth < par_depth) private(n, m, who_am_i) shared(weights) num_threads(2)
@@ -471,7 +450,7 @@ sort_par(size_t num_links, size_t num_wts, size_t *restrict add1, size_t *restri
     {
 
       who_am_i = nsplit * parent + i;
-//    my_depth = (int) (log(parent)/log(2))+1;
+      //    my_depth = (int) (log(parent)/log(2))+1;
 
 #ifdef _OPENMP
 //      if ( 1 )
