@@ -1327,9 +1327,10 @@ jacobi_1side(double **M, double *A, long n)
 
   if (n_iter == max_jacobi_iter && n_finished < count)
     {
-      fprintf(stderr, "statistics-module (Warning): Eigenvalue computation with one-sided jacobi scheme\n"
-                      "                             did not converge properly. %i of %i pairs of columns did\n"
-                      "                             not achieve requested orthogonality of %10.6g\n",
+      fprintf(stderr,
+              "statistics-module (Warning): Eigenvalue computation with one-sided jacobi scheme\n"
+              "                             did not converge properly. %i of %i pairs of columns did\n"
+              "                             not achieve requested orthogonality of %10.6g\n",
               count - n_finished, count, fnorm_precision);
 
       if (n_finished == 0)
