@@ -84,7 +84,7 @@ expand_filename(const char *string)
   if (find_wildcard(string, strlen(string)))
     {
 #ifdef HAVE_GLOB_H
-      int glob_flags = get_glob_flags();
+      const int glob_flags = get_glob_flags();
       glob_t glob_results;
       glob(string, glob_flags, 0, &glob_results);
       if (glob_results.gl_pathc == 1) filename = strdup(glob_results.gl_pathv[0]);
@@ -108,7 +108,7 @@ expandWildCards(std::vector<std::string> argv)
       // if argv[idx] contains wildcard (* or [?]+), multiple ** are ignored
       if (argv[idx][0] != '-' && argv[idx].find_first_of("*?[ ") != std::string::npos)
         {
-          int status = wordexp(argv[idx].c_str(), &glob_results, flags);
+          const int status = wordexp(argv[idx].c_str(), &glob_results, flags);
           if (status != 0)
             {
               fprintf(stderr, "%s: ", __func__);
