@@ -1,25 +1,22 @@
 #include "bandit/bandit/bandit.h"
 // BANDIT NEEDS TO BE INCLUDED FIRST!!!
 
-#include "../../src/cdo_debugOutput.h"
 #include "../../src/modules.h"
 #include "../../src/operator_help.h"
 #include "../../src/process_int.h"
+#include "test_module_list.h"
 #include <iostream>
 /* clang-format off */
-void * in0_out1(void *test) { return test; }
-void * in1_out1(void *test) { return test; }
-void * in2_out1(void *test) { return test; }
-void * inVariable_out1(void *test) { return test; }
 
 using namespace snowhouse;
 
 go_bandit([]() {
 //==============================================================================
-  add_module("in0_out1", { in0_out1, {}, { "in0_out1" }, 1, 0, 0, 1 });
-  add_module("in1_out1", { in1_out1, {}, { "in1_out1" }, 1, 0, 1, 1 });
-  add_module("in2_out1", { in2_out1, {}, { "in2_out1" }, 1, 0, 2, 1 });
-  add_module("inVariable_out1", { inVariable_out1, {}, { "inVariable_out1" }, 1, 0, -1, 0 });
+
+  add_module("in0_out1", module_in0_out1);
+  add_module("in1_out1", module_in1_out1);
+  add_module("in2_out1", module_in2_out1);
+  add_module("inVariable_out0", module_inVariable_out0);
 
   /* clang-format on */
 
@@ -40,8 +37,8 @@ go_bandit([]() {
     std::vector<const char *> argv_unprocessedInput{
         "-in2_out1", "-in0_out1", "-in0_out1", "-in0_out1", "out" };
     /* clang-format on */
-    parseStatus = g_processManager.createProcessesFromInput(argv_unprocessedInput.size(),
-                                           &argv_unprocessedInput[0]);
+    parseStatus = g_processManager.createProcessesFromInput(
+        argv_unprocessedInput.size(), &argv_unprocessedInput[0]);
 
     result_parse = static_cast<int>(parseStatus);
     expected_parse = static_cast<int>(ParseStatus::UnprocessedInput);
@@ -59,8 +56,8 @@ go_bandit([]() {
         "-in2_out1", "[", "-in0_out1", "-in0_out1", "out"
         };
     /* clang-format on */
-    parseStatus = g_processManager.createProcessesFromInput(argv_missingCloseBracket.size(),
-                                           &argv_missingCloseBracket[0]);
+    parseStatus = g_processManager.createProcessesFromInput(
+        argv_missingCloseBracket.size(), &argv_missingCloseBracket[0]);
 
     result_parse = static_cast<int>(parseStatus);
     expected_parse = static_cast<int>(ParseStatus::ClosingBracketMissing);
@@ -77,8 +74,8 @@ go_bandit([]() {
         "-in2_out1", "-in0_out1", "-in0_out1", "]", "out"
         };
     /* clang-format on */
-    parseStatus = g_processManager.createProcessesFromInput(argv_missingOpenBracket.size(),
-                                           &argv_missingOpenBracket[0]);
+    parseStatus = g_processManager.createProcessesFromInput(
+        argv_missingOpenBracket.size(), &argv_missingOpenBracket[0]);
 
     result_parse = static_cast<int>(parseStatus);
     expected_parse = static_cast<int>(ParseStatus::OpenBracketMissing);
@@ -95,8 +92,8 @@ go_bandit([]() {
         "-in2_out1", "[", "-in0_out1", "-in0_out1", "-in0_out1", "]", "out"
     };
     /* clang-format on */
-    parseStatus = g_processManager.createProcessesFromInput(argv_wrongBracketTooMany.size(),
-                                           &argv_wrongBracketTooMany[0]);
+    parseStatus = g_processManager.createProcessesFromInput(
+        argv_wrongBracketTooMany.size(), &argv_wrongBracketTooMany[0]);
     processStatus = g_processManager.getProcess(0).checkStreamCnt();
 
     result_parse = static_cast<int>(parseStatus);
@@ -118,8 +115,8 @@ go_bandit([]() {
         "-in2_out1", "[", "-in0_out1", "]", "out"
     };
     /* clang-format on */
-    parseStatus = g_processManager.createProcessesFromInput(argv_wrongBracketTooFew.size(),
-                                           &argv_wrongBracketTooFew[0]);
+    parseStatus = g_processManager.createProcessesFromInput(
+        argv_wrongBracketTooFew.size(), &argv_wrongBracketTooFew[0]);
     processStatus = g_processManager.getProcess(0).checkStreamCnt();
 
     bandit::it("this one has too few inputs", [&]() {
@@ -137,8 +134,9 @@ go_bandit([]() {
     std::vector<const char *> argv_missingOutFileOperHasDash{
         "-in2_out1"};
     /* clang-format on */
-    parseStatus = g_processManager.createProcessesFromInput(argv_missingOutFileOperHasDash.size(),
-                                           &argv_missingOutFileOperHasDash[0]);
+    parseStatus = g_processManager.createProcessesFromInput(
+        argv_missingOutFileOperHasDash.size(),
+        &argv_missingOutFileOperHasDash[0]);
 
     result_parse = static_cast<int>(parseStatus);
     expected_parse = static_cast<int>(ParseStatus::MissingOutFile);
@@ -154,8 +152,9 @@ go_bandit([]() {
     std::vector<const char *> argv_missingOutFileOperHasntDash{
         "in2_out1"};
     /* clang-format on */
-    parseStatus = g_processManager.createProcessesFromInput(argv_missingOutFileOperHasntDash.size(),
-                                           &argv_missingOutFileOperHasntDash[0]);
+    parseStatus = g_processManager.createProcessesFromInput(
+        argv_missingOutFileOperHasntDash.size(),
+        &argv_missingOutFileOperHasntDash[0]);
 
     result_parse = static_cast<int>(parseStatus);
     expected_parse = static_cast<int>(ParseStatus::MissingOutFile);
@@ -170,14 +169,7 @@ go_bandit([]() {
 int
 main(int argc, char **argv)
 {
-  CdoDebug::outfile = "createProcessFromInput.debug";
-  CdoDebug::print_to_seperate_file = true;
-
-  CdoDebug::CdoStartMessage();
-  CdoDebug::PROCESS = 1;
-  CdoDebug::PSTREAM = 1;
   int result = bandit::run(argc, argv);
-  CdoDebug::CdoEndMessage();
 
   return result;
 }
